package com.alibaba.fastjson.parser;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.deserializer.CollectionResolveFieldDeserializer;
import com.alibaba.fastjson.parser.deserializer.ExtraProcessor;
import com.alibaba.fastjson.parser.deserializer.ExtraTypeProvider;
import com.alibaba.fastjson.parser.deserializer.FieldDeserializer;
import com.alibaba.fastjson.parser.deserializer.ListResolveFieldDeserializer;
import com.alibaba.fastjson.parser.deserializer.MapResolveFieldDeserializer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.serializer.IntegerCodec;
import com.alibaba.fastjson.serializer.LongCodec;
import com.alibaba.fastjson.serializer.StringCodec;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.Closeable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;
import java.lang.reflect.WildcardType;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import kotlin.text.Typography;
/* loaded from: classes.dex */
public class DefaultJSONParser extends AbstractJSONParser implements Closeable {
    public static final int NONE = 0;
    public static final int NeedToResolve = 1;
    public static final int TypeNameRedirect = 2;
    private static final Set<Class<?>> primitiveClasses;
    protected ParserConfig config;
    protected ParseContext context;
    private ParseContext[] contextArray;
    private int contextArrayIndex;
    private DateFormat dateFormat;
    private String dateFormatPattern;
    private List<ExtraProcessor> extraProcessors;
    private List<ExtraTypeProvider> extraTypeProviders;
    protected final Object input;
    protected final JSONLexer lexer;
    private int resolveStatus;
    private List<ResolveTask> resolveTaskList;
    protected final SymbolTable symbolTable;

    static {
        HashSet hashSet = new HashSet();
        primitiveClasses = hashSet;
        hashSet.add(Boolean.TYPE);
        hashSet.add(Byte.TYPE);
        hashSet.add(Short.TYPE);
        hashSet.add(Integer.TYPE);
        hashSet.add(Long.TYPE);
        hashSet.add(Float.TYPE);
        hashSet.add(Double.TYPE);
        hashSet.add(Boolean.class);
        hashSet.add(Byte.class);
        hashSet.add(Short.class);
        hashSet.add(Integer.class);
        hashSet.add(Long.class);
        hashSet.add(Float.class);
        hashSet.add(Double.class);
        hashSet.add(BigInteger.class);
        hashSet.add(BigDecimal.class);
        hashSet.add(String.class);
    }

    public String getDateFomartPattern() {
        return this.dateFormatPattern;
    }

    public DateFormat getDateFormat() {
        if (this.dateFormat == null) {
            this.dateFormat = new SimpleDateFormat(this.dateFormatPattern);
        }
        return this.dateFormat;
    }

    public void setDateFormat(String dateFormat) {
        this.dateFormatPattern = dateFormat;
        this.dateFormat = null;
    }

    public void setDateFomrat(DateFormat dateFormat) {
        this.dateFormat = dateFormat;
    }

    public DefaultJSONParser(String input) {
        this(input, ParserConfig.getGlobalInstance(), JSON.DEFAULT_PARSER_FEATURE);
    }

    public DefaultJSONParser(String input, ParserConfig config) {
        this(input, new JSONScanner(input, JSON.DEFAULT_PARSER_FEATURE), config);
    }

    public DefaultJSONParser(String input, ParserConfig config, int features) {
        this(input, new JSONScanner(input, features), config);
    }

    public DefaultJSONParser(char[] input, int length, ParserConfig config, int features) {
        this(input, new JSONScanner(input, length, features), config);
    }

    public DefaultJSONParser(JSONLexer lexer) {
        this(lexer, ParserConfig.getGlobalInstance());
    }

    public DefaultJSONParser(JSONLexer lexer, ParserConfig config) {
        this((Object) null, lexer, config);
    }

    public DefaultJSONParser(Object input, JSONLexer lexer, ParserConfig config) {
        this.dateFormatPattern = JSON.DEFFAULT_DATE_FORMAT;
        this.contextArray = new ParseContext[8];
        this.contextArrayIndex = 0;
        this.resolveStatus = 0;
        this.extraTypeProviders = null;
        this.extraProcessors = null;
        this.lexer = lexer;
        this.input = input;
        this.config = config;
        this.symbolTable = config.getSymbolTable();
        lexer.nextToken(12);
    }

    public SymbolTable getSymbolTable() {
        return this.symbolTable;
    }

    public String getInput() {
        Object obj = this.input;
        if (obj instanceof char[]) {
            return new String((char[]) obj);
        }
        return obj.toString();
    }

    /* JADX WARN: Code restructure failed: missing block: B:100:0x021b, code lost:
        if (r17.context == null) goto L212;
     */
    /* JADX WARN: Code restructure failed: missing block: B:102:0x021f, code lost:
        if ((r19 instanceof java.lang.Integer) != false) goto L212;
     */
    /* JADX WARN: Code restructure failed: missing block: B:103:0x0221, code lost:
        popContext();
     */
    /* JADX WARN: Code restructure failed: missing block: B:106:0x0231, code lost:
        return r17.config.getDeserializer(r0).deserialze(r17, r0, r19);
     */
    /* JADX WARN: Code restructure failed: missing block: B:81:0x01d4, code lost:
        r4.nextToken(16);
     */
    /* JADX WARN: Code restructure failed: missing block: B:82:0x01dd, code lost:
        if (r4.token() != 13) goto L207;
     */
    /* JADX WARN: Code restructure failed: missing block: B:83:0x01df, code lost:
        r4.nextToken(16);
     */
    /* JADX WARN: Code restructure failed: missing block: B:84:0x01e2, code lost:
        r0 = null;
     */
    /* JADX WARN: Code restructure failed: missing block: B:85:0x01e3, code lost:
        r11 = r17.config.getDeserializer(r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:86:0x01eb, code lost:
        if ((r11 instanceof com.alibaba.fastjson.parser.deserializer.JavaBeanDeserializer) == false) goto L196;
     */
    /* JADX WARN: Code restructure failed: missing block: B:87:0x01ed, code lost:
        r0 = ((com.alibaba.fastjson.parser.deserializer.JavaBeanDeserializer) r11).createInstance(r17, r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:88:0x01f5, code lost:
        if (r0 != null) goto L202;
     */
    /* JADX WARN: Code restructure failed: missing block: B:90:0x01f9, code lost:
        if (r0 != java.lang.Cloneable.class) goto L200;
     */
    /* JADX WARN: Code restructure failed: missing block: B:91:0x01fb, code lost:
        r0 = new java.util.HashMap();
     */
    /* JADX WARN: Code restructure failed: missing block: B:93:0x0206, code lost:
        r0 = r0.newInstance();
     */
    /* JADX WARN: Code restructure failed: missing block: B:95:0x020b, code lost:
        return r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:96:0x020c, code lost:
        r0 = move-exception;
     */
    /* JADX WARN: Code restructure failed: missing block: B:98:0x0214, code lost:
        throw new com.alibaba.fastjson.JSONException("create instance error", r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:99:0x0215, code lost:
        setResolveStatus(2);
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public final Object parseObject(Map object, Object fieldName) {
        Object key;
        JSONLexer lexer = this.lexer;
        if (lexer.token() == 8) {
            lexer.next();
            return null;
        } else if (lexer.token() != 12 && lexer.token() != 16) {
            throw new JSONException("syntax error, expect {, actual " + lexer.tokenName());
        } else {
            ParseContext context = getContext();
            boolean setContextFlag = false;
            while (true) {
                try {
                    lexer.skipWhitespace();
                    char ch = lexer.getCurrent();
                    if (isEnabled(Feature.AllowArbitraryCommas)) {
                        while (ch == ',') {
                            lexer.next();
                            lexer.skipWhitespace();
                            ch = lexer.getCurrent();
                        }
                    }
                    boolean isObjectKey = false;
                    if (ch == '\"') {
                        String scanSymbol = lexer.scanSymbol(this.symbolTable, Typography.quote);
                        lexer.skipWhitespace();
                        if (lexer.getCurrent() != ':') {
                            throw new JSONException("expect ':' at " + lexer.pos() + ", name " + ((Object) scanSymbol));
                        }
                        key = scanSymbol;
                    } else if (ch == '}') {
                        lexer.next();
                        lexer.resetStringPosition();
                        lexer.nextToken();
                        return object;
                    } else if (ch == '\'') {
                        if (!isEnabled(Feature.AllowSingleQuotes)) {
                            throw new JSONException("syntax error");
                        }
                        String scanSymbol2 = lexer.scanSymbol(this.symbolTable, '\'');
                        lexer.skipWhitespace();
                        if (lexer.getCurrent() != ':') {
                            throw new JSONException("expect ':' at " + lexer.pos());
                        }
                        key = scanSymbol2;
                    } else if (ch == 26) {
                        throw new JSONException("syntax error");
                    } else {
                        if (ch == ',') {
                            throw new JSONException("syntax error");
                        }
                        if ((ch < '0' || ch > '9') && ch != '-') {
                            if (ch != '{' && ch != '[') {
                                if (!isEnabled(Feature.AllowUnQuotedFieldNames)) {
                                    throw new JSONException("syntax error");
                                }
                                String scanSymbolUnQuoted = lexer.scanSymbolUnQuoted(this.symbolTable);
                                lexer.skipWhitespace();
                                char ch2 = lexer.getCurrent();
                                if (ch2 != ':') {
                                    throw new JSONException("expect ':' at " + lexer.pos() + ", actual " + ch2);
                                }
                                key = scanSymbolUnQuoted;
                            }
                            lexer.nextToken();
                            key = parse();
                            isObjectKey = true;
                        } else {
                            lexer.resetStringPosition();
                            lexer.scanNumber();
                            String integerValue = lexer.token() == 2 ? lexer.integerValue() : lexer.decimalValue(true);
                            if (lexer.getCurrent() != ':') {
                                throw new JSONException("expect ':' at " + lexer.pos() + ", name " + ((Object) integerValue));
                            }
                            key = integerValue;
                        }
                    }
                    if (!isObjectKey) {
                        lexer.next();
                        lexer.skipWhitespace();
                    }
                    char ch3 = lexer.getCurrent();
                    lexer.resetStringPosition();
                    if (key == JSON.DEFAULT_TYPE_KEY) {
                        String typeName = lexer.scanSymbol(this.symbolTable, Typography.quote);
                        Class<?> clazz = TypeUtils.loadClass(typeName);
                        if (clazz != null) {
                            break;
                        }
                        object.put(JSON.DEFAULT_TYPE_KEY, typeName);
                    } else if (key == "$ref") {
                        lexer.nextToken(4);
                        if (lexer.token() == 4) {
                            String ref = lexer.stringVal();
                            lexer.nextToken(13);
                            Object refValue = null;
                            if ("@".equals(ref)) {
                                if (getContext() != null) {
                                    ParseContext thisContext = getContext();
                                    Object thisObj = thisContext.getObject();
                                    if (!(thisObj instanceof Object[]) && !(thisObj instanceof Collection)) {
                                        if (thisContext.getParentContext() != null) {
                                            refValue = thisContext.getParentContext().getObject();
                                        }
                                    }
                                    refValue = thisObj;
                                }
                            } else if ("..".equals(ref)) {
                                ParseContext parentContext = context.getParentContext();
                                if (parentContext.getObject() != null) {
                                    refValue = parentContext.getObject();
                                } else {
                                    addResolveTask(new ResolveTask(parentContext, ref));
                                    setResolveStatus(1);
                                }
                            } else if ("$".equals(ref)) {
                                ParseContext rootContext = context;
                                while (rootContext.getParentContext() != null) {
                                    rootContext = rootContext.getParentContext();
                                }
                                if (rootContext.getObject() != null) {
                                    refValue = rootContext.getObject();
                                } else {
                                    addResolveTask(new ResolveTask(rootContext, ref));
                                    setResolveStatus(1);
                                }
                            } else {
                                addResolveTask(new ResolveTask(context, ref));
                                setResolveStatus(1);
                            }
                            if (lexer.token() == 13) {
                                lexer.nextToken(16);
                                return refValue;
                            }
                            throw new JSONException("syntax error");
                        }
                        throw new JSONException("illegal ref, " + JSONToken.name(lexer.token()));
                    } else {
                        if (!setContextFlag) {
                            setContext(object, fieldName);
                            setContextFlag = true;
                        }
                        if (object.getClass() == JSONObject.class) {
                            key = key == null ? "null" : key.toString();
                        }
                        if (ch3 == '\"') {
                            lexer.scanString();
                            String strValue = lexer.stringVal();
                            String str = strValue;
                            if (lexer.isEnabled(Feature.AllowISO8601DateFormat)) {
                                JSONScanner iso8601Lexer = new JSONScanner(strValue);
                                if (iso8601Lexer.scanISO8601DateIfMatch()) {
                                    str = iso8601Lexer.getCalendar().getTime();
                                }
                                iso8601Lexer.close();
                            }
                            object.put(key, str);
                        } else if ((ch3 >= '0' && ch3 <= '9') || ch3 == '-') {
                            lexer.scanNumber();
                            object.put(key, lexer.token() == 2 ? lexer.integerValue() : lexer.decimalValue(isEnabled(Feature.UseBigDecimal)));
                        } else if (ch3 == '[') {
                            lexer.nextToken();
                            JSONArray list = new JSONArray();
                            parseArray(list, key);
                            object.put(key, list);
                            if (lexer.token() == 13) {
                                lexer.nextToken();
                                return object;
                            } else if (lexer.token() != 16) {
                                throw new JSONException("syntax error");
                            }
                        } else if (ch3 == '{') {
                            lexer.nextToken();
                            boolean parentIsArray = fieldName != null && fieldName.getClass() == Integer.class;
                            JSONObject input = new JSONObject();
                            ParseContext ctxLocal = parentIsArray ? null : setContext(context, input, key);
                            Object obj = parseObject(input, key);
                            if (ctxLocal != null && input != obj) {
                                ctxLocal.setObject(object);
                            }
                            checkMapResolve(object, key.toString());
                            if (object.getClass() == JSONObject.class) {
                                object.put(key.toString(), obj);
                            } else {
                                object.put(key, obj);
                            }
                            if (parentIsArray) {
                                setContext(context, obj, key);
                            }
                            if (lexer.token() == 13) {
                                lexer.nextToken();
                                setContext(context);
                                return object;
                            } else if (lexer.token() != 16) {
                                throw new JSONException("syntax error, " + lexer.tokenName());
                            }
                        } else {
                            lexer.nextToken();
                            Object value = parse();
                            if (object.getClass() == JSONObject.class) {
                                key = key.toString();
                            }
                            object.put(key, value);
                            if (lexer.token() == 13) {
                                lexer.nextToken();
                                return object;
                            } else if (lexer.token() != 16) {
                                throw new JSONException("syntax error, position at " + lexer.pos() + ", name " + ((Object) key));
                            }
                        }
                        lexer.skipWhitespace();
                        char ch4 = lexer.getCurrent();
                        if (ch4 != ',') {
                            if (ch4 == '}') {
                                lexer.next();
                                lexer.resetStringPosition();
                                lexer.nextToken();
                                setContext(object, fieldName);
                                return object;
                            }
                            throw new JSONException("syntax error, position at " + lexer.pos() + ", name " + ((Object) key));
                        }
                        lexer.next();
                    }
                } finally {
                    setContext(context);
                }
            }
        }
    }

    public ParserConfig getConfig() {
        return this.config;
    }

    public void setConfig(ParserConfig config) {
        this.config = config;
    }

    public <T> T parseObject(Class<T> clazz) {
        return (T) parseObject((Type) clazz);
    }

    public <T> T parseObject(Type type) {
        if (this.lexer.token() == 8) {
            this.lexer.nextToken();
            return null;
        }
        if (this.lexer.token() == 4) {
            type = TypeUtils.unwrap(type);
            if (type == byte[].class) {
                T t = (T) this.lexer.bytesValue();
                this.lexer.nextToken();
                return t;
            } else if (type == char[].class) {
                String strVal = this.lexer.stringVal();
                this.lexer.nextToken();
                return (T) strVal.toCharArray();
            }
        }
        ObjectDeserializer derializer = this.config.getDeserializer(type);
        try {
            return (T) derializer.deserialze(this, type, null);
        } catch (JSONException e) {
            throw e;
        } catch (Throwable e2) {
            throw new JSONException(e2.getMessage(), e2);
        }
    }

    public <T> List<T> parseArray(Class<T> clazz) {
        List<T> array = new ArrayList<>();
        parseArray((Class<?>) clazz, (Collection) array);
        return array;
    }

    public void parseArray(Class<?> clazz, Collection array) {
        parseArray((Type) clazz, array);
    }

    public void parseArray(Type type, Collection array) {
        parseArray(type, array, null);
    }

    public void parseArray(Type type, Collection array, Object fieldName) {
        ObjectDeserializer deserializer;
        Object val;
        String value;
        if (this.lexer.token() == 21 || this.lexer.token() == 22) {
            this.lexer.nextToken();
        }
        if (this.lexer.token() != 14) {
            throw new JSONException("exepct '[', but " + JSONToken.name(this.lexer.token()));
        }
        if (Integer.TYPE == type) {
            deserializer = IntegerCodec.instance;
            this.lexer.nextToken(2);
        } else if (String.class == type) {
            deserializer = StringCodec.instance;
            this.lexer.nextToken(4);
        } else {
            deserializer = this.config.getDeserializer(type);
            this.lexer.nextToken(deserializer.getFastMatchToken());
        }
        ParseContext context = getContext();
        setContext(array, fieldName);
        int i = 0;
        while (true) {
            try {
                if (isEnabled(Feature.AllowArbitraryCommas)) {
                    while (this.lexer.token() == 16) {
                        this.lexer.nextToken();
                    }
                }
                if (this.lexer.token() != 15) {
                    if (Integer.TYPE == type) {
                        Object val2 = IntegerCodec.instance.deserialze(this, null, null);
                        array.add(val2);
                    } else if (String.class == type) {
                        if (this.lexer.token() == 4) {
                            value = this.lexer.stringVal();
                            this.lexer.nextToken(16);
                        } else {
                            Object obj = parse();
                            if (obj == null) {
                                value = null;
                            } else {
                                String value2 = obj.toString();
                                value = value2;
                            }
                        }
                        array.add(value);
                    } else {
                        if (this.lexer.token() == 8) {
                            this.lexer.nextToken();
                            val = null;
                        } else {
                            Object val3 = Integer.valueOf(i);
                            val = deserializer.deserialze(this, type, val3);
                        }
                        array.add(val);
                        checkListResolve(array);
                    }
                    if (this.lexer.token() == 16) {
                        this.lexer.nextToken(deserializer.getFastMatchToken());
                    }
                    i++;
                } else {
                    setContext(context);
                    this.lexer.nextToken(16);
                    return;
                }
            } catch (Throwable th) {
                setContext(context);
                throw th;
            }
        }
    }

    public Object[] parseArray(Type[] types) {
        ObjectDeserializer objectDeserializer;
        String value;
        ObjectDeserializer derializer = null;
        int i = 8;
        if (this.lexer.token() == 8) {
            this.lexer.nextToken(16);
            return null;
        }
        int i2 = 14;
        if (this.lexer.token() != 14) {
            throw new JSONException("syntax error : " + this.lexer.tokenName());
        }
        Object[] list = new Object[types.length];
        if (types.length == 0) {
            this.lexer.nextToken(15);
            if (this.lexer.token() != 15) {
                throw new JSONException("syntax error");
            }
            this.lexer.nextToken(16);
            return new Object[0];
        }
        this.lexer.nextToken(2);
        int i3 = 0;
        while (i3 < types.length) {
            if (this.lexer.token() == i) {
                value = null;
                this.lexer.nextToken(16);
                objectDeserializer = derializer;
            } else {
                Type type = types[i3];
                if (type == Integer.TYPE) {
                    objectDeserializer = derializer;
                } else if (type == Integer.class) {
                    objectDeserializer = derializer;
                } else if (type == String.class) {
                    if (this.lexer.token() == 4) {
                        String stringVal = this.lexer.stringVal();
                        this.lexer.nextToken(16);
                        objectDeserializer = derializer;
                        value = stringVal;
                    } else {
                        Object value2 = parse();
                        objectDeserializer = derializer;
                        value = TypeUtils.cast(value2, type, this.config);
                    }
                } else {
                    boolean isArray = false;
                    Class<?> componentType = null;
                    if (i3 == types.length - 1 && (type instanceof Class)) {
                        Class<?> clazz = (Class) type;
                        isArray = clazz.isArray();
                        componentType = clazz.getComponentType();
                    }
                    if (isArray && this.lexer.token() != i2) {
                        List<Object> varList = new ArrayList<>();
                        ObjectDeserializer derializer2 = this.config.getDeserializer(componentType);
                        int fastMatch = derializer2.getFastMatchToken();
                        if (this.lexer.token() != 15) {
                            while (true) {
                                Object item = derializer2.deserialze(this, type, derializer);
                                varList.add(item);
                                if (this.lexer.token() != 16) {
                                    break;
                                }
                                this.lexer.nextToken(fastMatch);
                                derializer = null;
                            }
                            if (this.lexer.token() != 15) {
                                throw new JSONException("syntax error :" + JSONToken.name(this.lexer.token()));
                            }
                        }
                        Object value3 = TypeUtils.cast(varList, type, this.config);
                        value = value3;
                        objectDeserializer = null;
                    } else {
                        ObjectDeserializer derializer3 = this.config.getDeserializer(type);
                        objectDeserializer = null;
                        value = derializer3.deserialze(this, type, null);
                    }
                }
                if (this.lexer.token() == 2) {
                    Integer valueOf = Integer.valueOf(this.lexer.intValue());
                    this.lexer.nextToken(16);
                    value = valueOf;
                } else {
                    Object value4 = parse();
                    value = TypeUtils.cast(value4, type, this.config);
                }
            }
            list[i3] = value;
            if (this.lexer.token() == 15) {
                break;
            } else if (this.lexer.token() != 16) {
                throw new JSONException("syntax error :" + JSONToken.name(this.lexer.token()));
            } else {
                if (i3 == types.length - 1) {
                    this.lexer.nextToken(15);
                } else {
                    this.lexer.nextToken(2);
                }
                i3++;
                derializer = objectDeserializer;
                i = 8;
                i2 = 14;
            }
        }
        if (this.lexer.token() != 15) {
            throw new JSONException("syntax error");
        }
        this.lexer.nextToken(16);
        return list;
    }

    public void parseObject(Object object) {
        Object fieldValue;
        Class<?> clazz = object.getClass();
        Map<String, FieldDeserializer> setters = this.config.getFieldDeserializers(clazz);
        if (this.lexer.token() != 12 && this.lexer.token() != 16) {
            throw new JSONException("syntax error, expect {, actual " + this.lexer.tokenName());
        }
        while (true) {
            String key = this.lexer.scanSymbol(this.symbolTable);
            if (key == null) {
                if (this.lexer.token() == 13) {
                    this.lexer.nextToken(16);
                    return;
                } else if (this.lexer.token() == 16 && isEnabled(Feature.AllowArbitraryCommas)) {
                }
            }
            FieldDeserializer fieldDeser = setters.get(key);
            if (fieldDeser == null && key != null) {
                Iterator i$ = setters.entrySet().iterator();
                while (true) {
                    if (!i$.hasNext()) {
                        break;
                    }
                    Map.Entry<String, FieldDeserializer> entry = i$.next();
                    if (key.equalsIgnoreCase(entry.getKey())) {
                        fieldDeser = entry.getValue();
                        break;
                    }
                }
            }
            if (fieldDeser == null) {
                if (!isEnabled(Feature.IgnoreNotMatch)) {
                    throw new JSONException("setter not found, class " + clazz.getName() + ", property " + key);
                }
                this.lexer.nextTokenWithColon();
                parse();
                if (this.lexer.token() == 13) {
                    this.lexer.nextToken();
                    return;
                }
            } else {
                Class<?> fieldClass = fieldDeser.getFieldClass();
                Type fieldType = fieldDeser.getFieldType();
                if (fieldClass == Integer.TYPE) {
                    this.lexer.nextTokenWithColon(2);
                    fieldValue = IntegerCodec.instance.deserialze(this, fieldType, null);
                } else if (fieldClass == String.class) {
                    this.lexer.nextTokenWithColon(4);
                    fieldValue = StringCodec.deserialze(this);
                } else {
                    Object fieldValue2 = Long.TYPE;
                    if (fieldClass == fieldValue2) {
                        this.lexer.nextTokenWithColon(2);
                        fieldValue = LongCodec.instance.deserialze(this, fieldType, null);
                    } else {
                        ObjectDeserializer fieldValueDeserializer = this.config.getDeserializer(fieldClass, fieldType);
                        this.lexer.nextTokenWithColon(fieldValueDeserializer.getFastMatchToken());
                        fieldValue = fieldValueDeserializer.deserialze(this, fieldType, null);
                    }
                }
                fieldDeser.setValue(object, fieldValue);
                if (this.lexer.token() != 16 && this.lexer.token() == 13) {
                    this.lexer.nextToken(16);
                    return;
                }
            }
        }
    }

    public Object parseArrayWithType(Type collectionType) {
        if (this.lexer.token() == 8) {
            this.lexer.nextToken();
            return null;
        }
        Type[] actualTypes = ((ParameterizedType) collectionType).getActualTypeArguments();
        if (actualTypes.length != 1) {
            throw new JSONException("not support type " + collectionType);
        }
        Type actualTypeArgument = actualTypes[0];
        if (actualTypeArgument instanceof Class) {
            List<Object> array = new ArrayList<>();
            parseArray((Class) actualTypeArgument, (Collection) array);
            return array;
        } else if (actualTypeArgument instanceof WildcardType) {
            WildcardType wildcardType = (WildcardType) actualTypeArgument;
            Type upperBoundType = wildcardType.getUpperBounds()[0];
            if (Object.class.equals(upperBoundType)) {
                if (wildcardType.getLowerBounds().length == 0) {
                    return parse();
                }
                throw new JSONException("not support type : " + collectionType);
            }
            List<Object> array2 = new ArrayList<>();
            parseArray((Class) upperBoundType, (Collection) array2);
            return array2;
        } else {
            if (actualTypeArgument instanceof TypeVariable) {
                TypeVariable<?> typeVariable = (TypeVariable) actualTypeArgument;
                Type[] bounds = typeVariable.getBounds();
                if (bounds.length != 1) {
                    throw new JSONException("not support : " + typeVariable);
                }
                Type boundType = bounds[0];
                if (boundType instanceof Class) {
                    List<Object> array3 = new ArrayList<>();
                    parseArray((Class) boundType, (Collection) array3);
                    return array3;
                }
            }
            if (actualTypeArgument instanceof ParameterizedType) {
                ParameterizedType parameterizedType = (ParameterizedType) actualTypeArgument;
                List<Object> array4 = new ArrayList<>();
                parseArray(parameterizedType, array4);
                return array4;
            }
            throw new JSONException("TODO : " + collectionType);
        }
    }

    public int getResolveStatus() {
        return this.resolveStatus;
    }

    public void setResolveStatus(int resolveStatus) {
        this.resolveStatus = resolveStatus;
    }

    public Object getObject(String path) {
        for (int i = 0; i < this.contextArrayIndex; i++) {
            if (path.equals(this.contextArray[i].getPath())) {
                return this.contextArray[i].getObject();
            }
        }
        return null;
    }

    public void checkListResolve(Collection array) {
        if (this.resolveStatus == 1) {
            if (array instanceof List) {
                int index = array.size() - 1;
                List list = (List) array;
                ResolveTask task = getLastResolveTask();
                task.setFieldDeserializer(new ListResolveFieldDeserializer(this, list, index));
                task.setOwnerContext(this.context);
                setResolveStatus(0);
                return;
            }
            ResolveTask task2 = getLastResolveTask();
            task2.setFieldDeserializer(new CollectionResolveFieldDeserializer(this, array));
            task2.setOwnerContext(this.context);
            setResolveStatus(0);
        }
    }

    public void checkMapResolve(Map object, String fieldName) {
        if (this.resolveStatus == 1) {
            MapResolveFieldDeserializer fieldResolver = new MapResolveFieldDeserializer(object, fieldName);
            ResolveTask task = getLastResolveTask();
            task.setFieldDeserializer(fieldResolver);
            task.setOwnerContext(this.context);
            setResolveStatus(0);
        }
    }

    public Object parseObject(Map object) {
        return parseObject(object, null);
    }

    public JSONObject parseObject() {
        JSONObject object = new JSONObject();
        parseObject((Map) object);
        return object;
    }

    public final void parseArray(Collection array) {
        parseArray(array, (Object) null);
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r5v4, types: [com.alibaba.fastjson.parser.JSONScanner] */
    /* JADX WARN: Type inference failed for: r8v13, types: [java.lang.Boolean] */
    /* JADX WARN: Type inference failed for: r8v14, types: [java.lang.Boolean] */
    public final void parseArray(Collection array, Object fieldName) {
        JSONObject object;
        JSONLexer lexer = getLexer();
        if (lexer.token() == 21 || lexer.token() == 22) {
            lexer.nextToken();
        }
        if (lexer.token() != 14) {
            throw new JSONException("syntax error, expect [, actual " + JSONToken.name(lexer.token()) + ", pos " + lexer.pos());
        }
        lexer.nextToken(4);
        ParseContext context = getContext();
        setContext(array, fieldName);
        int i = 0;
        Number number = null;
        JSONObject jSONObject = null;
        Collection items = null;
        String stringLiteral = null;
        while (true) {
            try {
                if (isEnabled(Feature.AllowArbitraryCommas)) {
                    while (lexer.token() == 16) {
                        lexer.nextToken();
                    }
                }
                switch (lexer.token()) {
                    case 2:
                        number = lexer.integerValue();
                        lexer.nextToken(16);
                        object = jSONObject;
                        break;
                    case 3:
                        number = lexer.isEnabled(Feature.UseBigDecimal) ? lexer.decimalValue(true) : lexer.decimalValue(false);
                        lexer.nextToken(16);
                        object = jSONObject;
                        break;
                    case 4:
                        stringLiteral = lexer.stringVal();
                        lexer.nextToken(16);
                        if (!lexer.isEnabled(Feature.AllowISO8601DateFormat)) {
                            number = stringLiteral;
                            object = jSONObject;
                            break;
                        } else {
                            ?? jSONScanner = new JSONScanner(stringLiteral);
                            number = jSONScanner.scanISO8601DateIfMatch() ? jSONScanner.getCalendar().getTime() : stringLiteral;
                            jSONScanner.close();
                            object = jSONScanner;
                            break;
                        }
                    case 6:
                        number = Boolean.TRUE;
                        lexer.nextToken(16);
                        object = jSONObject;
                        break;
                    case 7:
                        number = Boolean.FALSE;
                        lexer.nextToken(16);
                        object = jSONObject;
                        break;
                    case 8:
                        number = null;
                        lexer.nextToken(4);
                        object = jSONObject;
                        break;
                    case 12:
                        String stringLiteral2 = stringLiteral;
                        JSONObject object2 = new JSONObject();
                        number = parseObject(object2, Integer.valueOf(i));
                        stringLiteral = stringLiteral2;
                        object = object2;
                        break;
                    case 14:
                        String stringLiteral3 = stringLiteral;
                        Collection items2 = new JSONArray();
                        parseArray(items2, Integer.valueOf(i));
                        number = items2;
                        stringLiteral = stringLiteral3;
                        items = items2;
                        object = jSONObject;
                        break;
                    case 15:
                        lexer.nextToken(16);
                        return;
                    case 20:
                        throw new JSONException("unclosed jsonArray");
                    case 23:
                        number = null;
                        lexer.nextToken(4);
                        object = jSONObject;
                        break;
                    default:
                        number = parse();
                        object = jSONObject;
                        break;
                }
                array.add(number);
                checkListResolve(array);
                if (lexer.token() == 16) {
                    lexer.nextToken(4);
                }
                i++;
                jSONObject = object;
            } finally {
                setContext(context);
            }
        }
    }

    public ParseContext getContext() {
        return this.context;
    }

    public void addResolveTask(ResolveTask task) {
        if (this.resolveTaskList == null) {
            this.resolveTaskList = new ArrayList(2);
        }
        this.resolveTaskList.add(task);
    }

    public ResolveTask getLastResolveTask() {
        List<ResolveTask> list = this.resolveTaskList;
        return list.get(list.size() - 1);
    }

    public List<ExtraProcessor> getExtraProcessors() {
        if (this.extraProcessors == null) {
            this.extraProcessors = new ArrayList(2);
        }
        return this.extraProcessors;
    }

    public List<ExtraProcessor> getExtraProcessorsDirect() {
        return this.extraProcessors;
    }

    public List<ExtraTypeProvider> getExtraTypeProviders() {
        if (this.extraTypeProviders == null) {
            this.extraTypeProviders = new ArrayList(2);
        }
        return this.extraTypeProviders;
    }

    public List<ExtraTypeProvider> getExtraTypeProvidersDirect() {
        return this.extraTypeProviders;
    }

    public void setContext(ParseContext context) {
        if (isEnabled(Feature.DisableCircularReferenceDetect)) {
            return;
        }
        this.context = context;
    }

    public void popContext() {
        if (isEnabled(Feature.DisableCircularReferenceDetect)) {
            return;
        }
        this.context = this.context.getParentContext();
        ParseContext[] parseContextArr = this.contextArray;
        int i = this.contextArrayIndex;
        parseContextArr[i - 1] = null;
        this.contextArrayIndex = i - 1;
    }

    public ParseContext setContext(Object object, Object fieldName) {
        if (isEnabled(Feature.DisableCircularReferenceDetect)) {
            return null;
        }
        return setContext(this.context, object, fieldName);
    }

    public ParseContext setContext(ParseContext parent, Object object, Object fieldName) {
        if (isEnabled(Feature.DisableCircularReferenceDetect)) {
            return null;
        }
        ParseContext parseContext = new ParseContext(parent, object, fieldName);
        this.context = parseContext;
        addContext(parseContext);
        return this.context;
    }

    private void addContext(ParseContext context) {
        int i = this.contextArrayIndex;
        this.contextArrayIndex = i + 1;
        ParseContext[] parseContextArr = this.contextArray;
        if (i >= parseContextArr.length) {
            int newLen = (parseContextArr.length * 3) / 2;
            ParseContext[] newArray = new ParseContext[newLen];
            System.arraycopy(parseContextArr, 0, newArray, 0, parseContextArr.length);
            this.contextArray = newArray;
        }
        this.contextArray[i] = context;
    }

    public Object parse() {
        return parse(null);
    }

    public Object parseKey() {
        if (this.lexer.token() == 18) {
            String value = this.lexer.stringVal();
            this.lexer.nextToken(16);
            return value;
        }
        return parse(null);
    }

    public Object parse(Object fieldName) {
        JSONLexer lexer = getLexer();
        switch (lexer.token()) {
            case 2:
                Number intValue = lexer.integerValue();
                lexer.nextToken();
                return intValue;
            case 3:
                Object value = lexer.decimalValue(isEnabled(Feature.UseBigDecimal));
                lexer.nextToken();
                return value;
            case 4:
                String stringLiteral = lexer.stringVal();
                lexer.nextToken(16);
                if (lexer.isEnabled(Feature.AllowISO8601DateFormat)) {
                    JSONScanner iso8601Lexer = new JSONScanner(stringLiteral);
                    try {
                        if (iso8601Lexer.scanISO8601DateIfMatch()) {
                            return iso8601Lexer.getCalendar().getTime();
                        }
                    } finally {
                        iso8601Lexer.close();
                    }
                }
                return stringLiteral;
            case 5:
            case 10:
            case 11:
            case 13:
            case 15:
            case 16:
            case 17:
            case 18:
            case 19:
            default:
                throw new JSONException("syntax error, pos " + lexer.getBufferPosition());
            case 6:
                lexer.nextToken();
                return Boolean.TRUE;
            case 7:
                lexer.nextToken();
                return Boolean.FALSE;
            case 8:
                lexer.nextToken();
                return null;
            case 9:
                lexer.nextToken(18);
                if (lexer.token() == 18) {
                    lexer.nextToken(10);
                    accept(10);
                    long time = lexer.integerValue().longValue();
                    accept(2);
                    accept(11);
                    return new Date(time);
                }
                throw new JSONException("syntax error");
            case 12:
                JSONObject object = new JSONObject();
                return parseObject(object, fieldName);
            case 14:
                JSONArray array = new JSONArray();
                parseArray(array, fieldName);
                return array;
            case 20:
                if (lexer.isBlankInput()) {
                    return null;
                }
                throw new JSONException("unterminated json string, pos " + lexer.getBufferPosition());
            case 21:
                lexer.nextToken();
                HashSet<Object> set = new HashSet<>();
                parseArray(set, fieldName);
                return set;
            case 22:
                lexer.nextToken();
                TreeSet<Object> treeSet = new TreeSet<>();
                parseArray(treeSet, fieldName);
                return treeSet;
            case 23:
                lexer.nextToken();
                return null;
        }
    }

    public void config(Feature feature, boolean state) {
        getLexer().config(feature, state);
    }

    public boolean isEnabled(Feature feature) {
        return getLexer().isEnabled(feature);
    }

    public JSONLexer getLexer() {
        return this.lexer;
    }

    public final void accept(int token) {
        JSONLexer lexer = getLexer();
        if (lexer.token() == token) {
            lexer.nextToken();
            return;
        }
        throw new JSONException("syntax error, expect " + JSONToken.name(token) + ", actual " + JSONToken.name(lexer.token()));
    }

    public final void accept(int token, int nextExpectToken) {
        JSONLexer lexer = getLexer();
        if (lexer.token() == token) {
            lexer.nextToken(nextExpectToken);
            return;
        }
        throw new JSONException("syntax error, expect " + JSONToken.name(token) + ", actual " + JSONToken.name(lexer.token()));
    }

    @Override // java.io.Closeable, java.lang.AutoCloseable
    public void close() {
        JSONLexer lexer = getLexer();
        try {
            if (isEnabled(Feature.AutoCloseSource) && lexer.token() != 20) {
                throw new JSONException("not close json text, token : " + JSONToken.name(lexer.token()));
            }
        } finally {
            lexer.close();
        }
    }

    public void handleResovleTask(Object value) {
        Object refValue;
        List<ResolveTask> list = this.resolveTaskList;
        if (list == null) {
            return;
        }
        int size = list.size();
        for (int i = 0; i < size; i++) {
            ResolveTask task = this.resolveTaskList.get(i);
            FieldDeserializer fieldDeser = task.getFieldDeserializer();
            if (fieldDeser != null) {
                Object object = null;
                if (task.getOwnerContext() != null) {
                    object = task.getOwnerContext().getObject();
                }
                String ref = task.getReferenceValue();
                if (ref.startsWith("$")) {
                    refValue = getObject(ref);
                } else {
                    refValue = task.getContext().getObject();
                }
                fieldDeser.setValue(object, refValue);
            }
        }
    }

    /* loaded from: classes.dex */
    public static class ResolveTask {
        private final ParseContext context;
        private FieldDeserializer fieldDeserializer;
        private ParseContext ownerContext;
        private final String referenceValue;

        public ResolveTask(ParseContext context, String referenceValue) {
            this.context = context;
            this.referenceValue = referenceValue;
        }

        public ParseContext getContext() {
            return this.context;
        }

        public String getReferenceValue() {
            return this.referenceValue;
        }

        public FieldDeserializer getFieldDeserializer() {
            return this.fieldDeserializer;
        }

        public void setFieldDeserializer(FieldDeserializer fieldDeserializer) {
            this.fieldDeserializer = fieldDeserializer;
        }

        public ParseContext getOwnerContext() {
            return this.ownerContext;
        }

        public void setOwnerContext(ParseContext ownerContext) {
            this.ownerContext = ownerContext;
        }
    }
}
