package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.util.FieldInfo;
import com.alibaba.fastjson.util.TypeUtils;
import java.lang.reflect.Type;
import java.util.Map;
/* loaded from: classes.dex */
public class BooleanFieldDeserializer extends FieldDeserializer {
    public BooleanFieldDeserializer(ParserConfig mapping, Class<?> clazz, FieldInfo fieldInfo) {
        super(clazz, fieldInfo);
    }

    @Override // com.alibaba.fastjson.parser.deserializer.FieldDeserializer
    public void parseField(DefaultJSONParser parser, Object object, Type objectType, Map<String, Object> fieldValues) {
        JSONLexer lexer = parser.getLexer();
        if (lexer.token() == 6) {
            lexer.nextToken(16);
            if (object == null) {
                fieldValues.put(this.fieldInfo.getName(), Boolean.TRUE);
            } else {
                setValue(object, true);
            }
        } else if (lexer.token() == 2) {
            int val = lexer.intValue();
            lexer.nextToken(16);
            boolean booleanValue = val == 1;
            if (object == null) {
                fieldValues.put(this.fieldInfo.getName(), Boolean.valueOf(booleanValue));
            } else {
                setValue(object, booleanValue);
            }
        } else {
            int val2 = lexer.token();
            if (val2 == 8) {
                lexer.nextToken(16);
                if (getFieldClass() != Boolean.TYPE && object != null) {
                    setValue(object, (String) null);
                }
            } else if (lexer.token() == 7) {
                lexer.nextToken(16);
                if (object == null) {
                    fieldValues.put(this.fieldInfo.getName(), Boolean.FALSE);
                } else {
                    setValue(object, false);
                }
            } else {
                Object obj = parser.parse();
                Boolean value = TypeUtils.castToBoolean(obj);
                if (value == null && getFieldClass() == Boolean.TYPE) {
                    return;
                }
                if (object == null) {
                    fieldValues.put(this.fieldInfo.getName(), value);
                } else {
                    setValue(object, value);
                }
            }
        }
    }

    @Override // com.alibaba.fastjson.parser.deserializer.FieldDeserializer
    public int getFastMatchToken() {
        return 6;
    }
}
