package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.JSONToken;
import com.alibaba.fastjson.parser.ParseContext;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.serializer.FilterUtils;
import com.alibaba.fastjson.util.DeserializeBeanInfo;
import com.alibaba.fastjson.util.FieldInfo;
import java.lang.reflect.Constructor;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
/* loaded from: classes.dex */
public class JavaBeanDeserializer implements ObjectDeserializer {
    private DeserializeBeanInfo beanInfo;
    private final Class<?> clazz;
    private final Map<String, FieldDeserializer> feildDeserializerMap;
    private final List<FieldDeserializer> fieldDeserializers;
    private final List<FieldDeserializer> sortedFieldDeserializers;

    public JavaBeanDeserializer(ParserConfig config, Class<?> clazz) {
        this(config, clazz, clazz);
    }

    public JavaBeanDeserializer(ParserConfig config, Class<?> clazz, Type type) {
        this.feildDeserializerMap = new IdentityHashMap();
        this.fieldDeserializers = new ArrayList();
        this.sortedFieldDeserializers = new ArrayList();
        this.clazz = clazz;
        DeserializeBeanInfo computeSetters = DeserializeBeanInfo.computeSetters(clazz, type);
        this.beanInfo = computeSetters;
        for (FieldInfo fieldInfo : computeSetters.getFieldList()) {
            addFieldDeserializer(config, clazz, fieldInfo);
        }
        for (FieldInfo fieldInfo2 : this.beanInfo.getSortedFieldList()) {
            FieldDeserializer fieldDeserializer = this.feildDeserializerMap.get(fieldInfo2.getName().intern());
            this.sortedFieldDeserializers.add(fieldDeserializer);
        }
    }

    public Map<String, FieldDeserializer> getFieldDeserializerMap() {
        return this.feildDeserializerMap;
    }

    private void addFieldDeserializer(ParserConfig mapping, Class<?> clazz, FieldInfo fieldInfo) {
        String interName = fieldInfo.getName().intern();
        FieldDeserializer fieldDeserializer = createFieldDeserializer(mapping, clazz, fieldInfo);
        this.feildDeserializerMap.put(interName, fieldDeserializer);
        this.fieldDeserializers.add(fieldDeserializer);
    }

    public FieldDeserializer createFieldDeserializer(ParserConfig mapping, Class<?> clazz, FieldInfo fieldInfo) {
        return mapping.createFieldDeserializer(mapping, clazz, fieldInfo);
    }

    public Object createInstance(DefaultJSONParser parser, Type type) {
        if ((type instanceof Class) && this.clazz.isInterface()) {
            Class<?> clazz = (Class) type;
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            JSONObject obj = new JSONObject();
            Object proxy = Proxy.newProxyInstance(loader, new Class[]{clazz}, obj);
            return proxy;
        } else if (this.beanInfo.getDefaultConstructor() == null) {
            return null;
        } else {
            try {
                Constructor<?> constructor = this.beanInfo.getDefaultConstructor();
                Object object = constructor.getParameterTypes().length == 0 ? constructor.newInstance(new Object[0]) : constructor.newInstance(parser.getContext().getObject());
                if (parser.isEnabled(Feature.InitStringFieldAsEmpty)) {
                    for (FieldInfo fieldInfo : this.beanInfo.getFieldList()) {
                        if (fieldInfo.getFieldClass() == String.class) {
                            try {
                                fieldInfo.set(object, "");
                            } catch (Exception e) {
                                throw new JSONException("create instance error, class " + this.clazz.getName(), e);
                            }
                        }
                    }
                }
                return object;
            } catch (Exception e2) {
                throw new JSONException("create instance error, class " + this.clazz.getName(), e2);
            }
        }
    }

    @Override // com.alibaba.fastjson.parser.deserializer.ObjectDeserializer
    public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName) {
        return (T) deserialze(parser, type, fieldName, null);
    }

    public <T> T deserialzeArrayMapping(DefaultJSONParser parser, Type type, Object fieldName, Object object) {
        JSONLexer lexer = parser.getLexer();
        if (lexer.token() == 14) {
            T t = (T) createInstance(parser, type);
            int size = this.sortedFieldDeserializers.size();
            int i = 0;
            while (i < size) {
                char seperator = i == size + (-1) ? ']' : ',';
                FieldDeserializer fieldDeser = this.sortedFieldDeserializers.get(i);
                Class<?> fieldClass = fieldDeser.getFieldClass();
                if (fieldClass == Integer.TYPE) {
                    int value = lexer.scanInt(seperator);
                    fieldDeser.setValue((Object) t, value);
                } else if (fieldClass == String.class) {
                    String value2 = lexer.scanString(seperator);
                    fieldDeser.setValue((Object) t, value2);
                } else if (fieldClass == Long.TYPE) {
                    long value3 = lexer.scanLong(seperator);
                    fieldDeser.setValue(t, value3);
                } else if (fieldClass.isEnum()) {
                    Enum<?> value4 = lexer.scanEnum(fieldClass, parser.getSymbolTable(), seperator);
                    fieldDeser.setValue(t, value4);
                } else {
                    lexer.nextToken(14);
                    Object value5 = parser.parseObject(fieldDeser.getFieldType());
                    fieldDeser.setValue(t, value5);
                    if (seperator == ']') {
                        if (lexer.token() != 15) {
                            throw new JSONException("syntax error");
                        }
                        lexer.nextToken(16);
                    } else if (seperator == ',' && lexer.token() != 16) {
                        throw new JSONException("syntax error");
                    }
                }
                i++;
            }
            lexer.nextToken(16);
            return t;
        }
        throw new JSONException("error");
    }

    /* JADX WARN: Code restructure failed: missing block: B:107:0x01d4, code lost:
        r16 = r1;
        r1 = r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:108:0x01d9, code lost:
        r4 = com.alibaba.fastjson.util.TypeUtils.loadClass(r3);
        r5 = r20.getConfig().getDeserializer(r4);
        r12 = (T) r5.deserialze(r20, r4, r22);
     */
    /* JADX WARN: Code restructure failed: missing block: B:109:0x01e9, code lost:
        if (r2 == null) goto L166;
     */
    /* JADX WARN: Code restructure failed: missing block: B:110:0x01eb, code lost:
        r2.setObject(r1);
     */
    /* JADX WARN: Code restructure failed: missing block: B:111:0x01ee, code lost:
        r20.setContext(r14);
     */
    /* JADX WARN: Code restructure failed: missing block: B:112:0x01f1, code lost:
        return r12;
     */
    /* JADX WARN: Code restructure failed: missing block: B:135:0x024d, code lost:
        r1 = r0;
        r2 = r17;
     */
    /* JADX WARN: Code restructure failed: missing block: B:136:0x0250, code lost:
        if (r16 != null) goto L83;
     */
    /* JADX WARN: Code restructure failed: missing block: B:137:0x0252, code lost:
        if (r1 != null) goto L107;
     */
    /* JADX WARN: Code restructure failed: missing block: B:138:0x0254, code lost:
        r0 = (T) createInstance(r20, r21);
     */
    /* JADX WARN: Code restructure failed: missing block: B:140:0x0259, code lost:
        if (r2 != null) goto L93;
     */
    /* JADX WARN: Code restructure failed: missing block: B:142:0x025f, code lost:
        r2 = r20.setContext(r14, r0, r22);
     */
    /* JADX WARN: Code restructure failed: missing block: B:143:0x0261, code lost:
        r0 = th;
     */
    /* JADX WARN: Code restructure failed: missing block: B:144:0x0262, code lost:
        r1 = r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:146:0x0266, code lost:
        if (r2 == null) goto L95;
     */
    /* JADX WARN: Code restructure failed: missing block: B:147:0x0268, code lost:
        r2.setObject(r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:148:0x026b, code lost:
        r20.setContext(r14);
     */
    /* JADX WARN: Code restructure failed: missing block: B:149:0x026e, code lost:
        return r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:150:0x026f, code lost:
        r0 = r19.beanInfo.getFieldList();
        r0 = r0.size();
        r0 = new java.lang.Object[r0];
        r0 = 0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:151:0x027f, code lost:
        if (r0 >= r0) goto L110;
     */
    /* JADX WARN: Code restructure failed: missing block: B:152:0x0281, code lost:
        r6 = r0.get(r0);
        r0[r0] = r1.get(r6.getName());
        r0 = r0 + 1;
     */
    /* JADX WARN: Code restructure failed: missing block: B:154:0x029a, code lost:
        if (r19.beanInfo.getCreatorConstructor() == null) goto L113;
     */
    /* JADX WARN: Code restructure failed: missing block: B:156:0x02a6, code lost:
        r0 = (T) r19.beanInfo.getCreatorConstructor().newInstance(r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:157:0x02ab, code lost:
        r0 = move-exception;
     */
    /* JADX WARN: Code restructure failed: missing block: B:159:0x02ce, code lost:
        throw new com.alibaba.fastjson.JSONException("create instance error, " + r19.beanInfo.getCreatorConstructor().toGenericString(), r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:161:0x02d5, code lost:
        if (r19.beanInfo.getFactoryMethod() == null) goto L83;
     */
    /* JADX WARN: Code restructure failed: missing block: B:163:0x02e1, code lost:
        r0 = (T) r19.beanInfo.getFactoryMethod().invoke(null, r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:164:0x02e6, code lost:
        r0 = move-exception;
     */
    /* JADX WARN: Code restructure failed: missing block: B:166:0x0309, code lost:
        throw new com.alibaba.fastjson.JSONException("create factory method error, " + r19.beanInfo.getFactoryMethod().toString(), r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:167:0x030a, code lost:
        r0 = th;
     */
    /* JADX WARN: Code restructure failed: missing block: B:168:0x030b, code lost:
        r1 = (T) r16;
     */
    /* JADX WARN: Code restructure failed: missing block: B:169:0x030e, code lost:
        r0 = (T) r16;
     */
    /* JADX WARN: Code restructure failed: missing block: B:171:0x0311, code lost:
        if (r2 == null) goto L86;
     */
    /* JADX WARN: Code restructure failed: missing block: B:172:0x0313, code lost:
        r2.setObject(r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:173:0x0316, code lost:
        r20.setContext(r14);
     */
    /* JADX WARN: Code restructure failed: missing block: B:174:0x0319, code lost:
        return (T) r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:181:0x0350, code lost:
        throw new com.alibaba.fastjson.JSONException("syntax error, unexpect token " + com.alibaba.fastjson.parser.JSONToken.name(r11.token()));
     */
    /* JADX WARN: Removed duplicated region for block: B:188:0x035c  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName, Object object) {
        Object object2;
        ParseContext childContext;
        Object object3 = (T) object;
        if (type == JSON.class || type == JSONObject.class) {
            return (T) parser.parse();
        }
        JSONLexer lexer = parser.getLexer();
        if (lexer.token() == 8) {
            lexer.nextToken(16);
            return null;
        }
        ParseContext context = parser.getContext();
        ParseContext context2 = (object3 == null || context == null) ? context : context.getParentContext();
        ParseContext childContext2 = null;
        Map<String, Object> fieldValues = null;
        try {
            if (lexer.token() == 13) {
                lexer.nextToken(16);
                if (object3 == null) {
                    object3 = (T) createInstance(parser, type);
                }
                if (0 != 0) {
                    childContext2.setObject(object3);
                }
                parser.setContext(context2);
                return (T) object3;
            } else if (lexer.token() == 14 && isSupportArrayToBean(lexer)) {
                T t = (T) deserialzeArrayMapping(parser, type, fieldName, object);
                if (0 != 0) {
                    childContext2.setObject(object3);
                }
                parser.setContext(context2);
                return t;
            } else if (lexer.token() != 12 && lexer.token() != 16) {
                StringBuffer buf = new StringBuffer().append("syntax error, expect {, actual ").append(lexer.tokenName()).append(", pos ").append(lexer.pos());
                if (fieldName instanceof String) {
                    buf.append(", fieldName ").append(fieldName);
                }
                throw new JSONException(buf.toString());
            } else {
                if (parser.getResolveStatus() == 2) {
                    parser.setResolveStatus(0);
                }
                while (true) {
                    try {
                        String key = lexer.scanSymbol(parser.getSymbolTable());
                        if (key == null) {
                            if (lexer.token() == 13) {
                                lexer.nextToken(16);
                                break;
                            } else if (lexer.token() == 16 && parser.isEnabled(Feature.AllowArbitraryCommas)) {
                            }
                        }
                        if ("$ref" == key) {
                            lexer.nextTokenWithColon(4);
                            if (lexer.token() == 4) {
                                String ref = lexer.stringVal();
                                if ("@".equals(ref)) {
                                    object3 = (T) context2.getObject();
                                } else if ("..".equals(ref)) {
                                    ParseContext parentContext = context2.getParentContext();
                                    if (parentContext.getObject() != null) {
                                        object3 = (T) parentContext.getObject();
                                    } else {
                                        parser.addResolveTask(new DefaultJSONParser.ResolveTask(parentContext, ref));
                                        parser.setResolveStatus(1);
                                    }
                                } else if ("$".equals(ref)) {
                                    ParseContext rootContext = context2;
                                    while (rootContext.getParentContext() != null) {
                                        rootContext = rootContext.getParentContext();
                                    }
                                    if (rootContext.getObject() != null) {
                                        object3 = (T) rootContext.getObject();
                                    } else {
                                        parser.addResolveTask(new DefaultJSONParser.ResolveTask(rootContext, ref));
                                        parser.setResolveStatus(1);
                                    }
                                } else {
                                    parser.addResolveTask(new DefaultJSONParser.ResolveTask(context2, ref));
                                    parser.setResolveStatus(1);
                                }
                                lexer.nextToken(13);
                                if (lexer.token() == 13) {
                                    lexer.nextToken(16);
                                    parser.setContext(context2, object3, fieldName);
                                    if (childContext2 != null) {
                                        childContext2.setObject(object3);
                                    }
                                    parser.setContext(context2);
                                    return (T) object3;
                                }
                                throw new JSONException("illegal ref");
                            }
                            throw new JSONException("illegal ref, " + JSONToken.name(lexer.token()));
                        } else if (JSON.DEFAULT_TYPE_KEY == key) {
                            lexer.nextTokenWithColon(4);
                            if (lexer.token() != 4) {
                                throw new JSONException("syntax error");
                            }
                            String typeName = lexer.stringVal();
                            lexer.nextToken(16);
                            if (!(type instanceof Class) || !typeName.equals(((Class) type).getName())) {
                                break;
                            } else if (lexer.token() == 13) {
                                lexer.nextToken();
                                break;
                            }
                        } else {
                            if (object3 == null && fieldValues == null) {
                                Object object4 = createInstance(parser, type);
                                if (object4 == null) {
                                    fieldValues = new HashMap<>(this.fieldDeserializers.size());
                                }
                                object2 = object4;
                                childContext = parser.setContext(context2, object4, fieldName);
                            } else {
                                object2 = object3;
                                childContext = childContext2;
                            }
                            try {
                                boolean match = parseField(parser, key, object2, type, fieldValues);
                                if (match) {
                                    if (lexer.token() != 16) {
                                        if (lexer.token() != 13) {
                                            if (lexer.token() == 18 || lexer.token() == 1) {
                                                break;
                                            }
                                        } else {
                                            lexer.nextToken(16);
                                            break;
                                        }
                                    }
                                    object3 = (T) object2;
                                    childContext2 = childContext;
                                } else {
                                    if (lexer.token() == 13) {
                                        lexer.nextToken();
                                        break;
                                    }
                                    object3 = (T) object2;
                                    childContext2 = childContext;
                                }
                            } catch (Throwable th) {
                                e = th;
                                object3 = (T) object2;
                                childContext2 = childContext;
                                if (childContext2 != null) {
                                    childContext2.setObject(object3);
                                }
                                parser.setContext(context2);
                                throw e;
                            }
                        }
                    } catch (Throwable th2) {
                        e = th2;
                    }
                }
            }
        } catch (Throwable th3) {
            e = th3;
        }
    }

    public boolean parseField(DefaultJSONParser parser, String key, Object object, Type objectType, Map<String, Object> fieldValues) {
        JSONLexer lexer = parser.getLexer();
        FieldDeserializer fieldDeserializer = this.feildDeserializerMap.get(key);
        if (fieldDeserializer == null) {
            Iterator i$ = this.feildDeserializerMap.entrySet().iterator();
            while (true) {
                if (!i$.hasNext()) {
                    break;
                }
                Map.Entry<String, FieldDeserializer> entry = i$.next();
                if (entry.getKey().equalsIgnoreCase(key)) {
                    fieldDeserializer = entry.getValue();
                    break;
                }
            }
        }
        if (fieldDeserializer == null) {
            parseExtra(parser, object, key);
            return false;
        }
        lexer.nextTokenWithColon(fieldDeserializer.getFastMatchToken());
        fieldDeserializer.parseField(parser, object, objectType, fieldValues);
        return true;
    }

    void parseExtra(DefaultJSONParser parser, Object object, String key) {
        Object value;
        JSONLexer lexer = parser.getLexer();
        if (!lexer.isEnabled(Feature.IgnoreNotMatch)) {
            throw new JSONException("setter not found, class " + this.clazz.getName() + ", property " + key);
        }
        lexer.nextTokenWithColon();
        Type type = FilterUtils.getExtratype(parser, object, key);
        if (type == null) {
            value = parser.parse();
        } else {
            value = parser.parseObject(type);
        }
        FilterUtils.processExtra(parser, object, key, value);
    }

    @Override // com.alibaba.fastjson.parser.deserializer.ObjectDeserializer
    public int getFastMatchToken() {
        return 12;
    }

    public final boolean isSupportArrayToBean(JSONLexer lexer) {
        return Feature.isEnabled(this.beanInfo.getParserFeatures(), Feature.SupportArrayToBean) || lexer.isEnabled(Feature.SupportArrayToBean);
    }
}
