package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.util.TypeUtils;
import java.lang.reflect.Type;
/* loaded from: classes.dex */
public class NumberDeserializer implements ObjectDeserializer {
    public static final NumberDeserializer instance = new NumberDeserializer();

    /* JADX WARN: Type inference failed for: r1v6, types: [java.math.BigDecimal, T] */
    @Override // com.alibaba.fastjson.parser.deserializer.ObjectDeserializer
    public <T> T deserialze(DefaultJSONParser parser, Type clazz, Object fieldName) {
        JSONLexer lexer = parser.getLexer();
        if (lexer.token() == 2) {
            if (clazz == Double.TYPE || clazz == Double.class) {
                String val = lexer.numberString();
                lexer.nextToken(16);
                return (T) Double.valueOf(Double.parseDouble(val));
            }
            long val2 = lexer.longValue();
            lexer.nextToken(16);
            if (clazz == Short.TYPE || clazz == Short.class) {
                return (T) Short.valueOf((short) val2);
            }
            if (clazz == Byte.TYPE || clazz == Byte.class) {
                return (T) Byte.valueOf((byte) val2);
            }
            if (val2 >= -2147483648L && val2 <= 2147483647L) {
                return (T) Integer.valueOf((int) val2);
            }
            return (T) Long.valueOf(val2);
        } else if (lexer.token() == 3) {
            if (clazz == Double.TYPE || clazz == Double.class) {
                String val3 = lexer.numberString();
                lexer.nextToken(16);
                return (T) Double.valueOf(Double.parseDouble(val3));
            }
            ?? r1 = (T) lexer.decimalValue();
            lexer.nextToken(16);
            if (clazz == Short.TYPE || clazz == Short.class) {
                return (T) Short.valueOf(r1.shortValue());
            }
            if (clazz == Byte.TYPE || clazz == Byte.class) {
                return (T) Byte.valueOf(r1.byteValue());
            }
            return r1;
        } else {
            Object value = parser.parse();
            if (value == null) {
                return null;
            }
            if (clazz == Double.TYPE || clazz == Double.class) {
                return (T) TypeUtils.castToDouble(value);
            }
            if (clazz == Short.TYPE || clazz == Short.class) {
                return (T) TypeUtils.castToShort(value);
            }
            if (clazz == Byte.TYPE || clazz == Byte.class) {
                return (T) TypeUtils.castToByte(value);
            }
            return (T) TypeUtils.castToBigDecimal(value);
        }
    }

    @Override // com.alibaba.fastjson.parser.deserializer.ObjectDeserializer
    public int getFastMatchToken() {
        return 2;
    }
}
