package com.alibaba.fastjson.util;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.annotation.JSONCreator;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.serializer.SerializerFeature;
import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
/* loaded from: classes.dex */
public class DeserializeBeanInfo {
    private Constructor<?> creatorConstructor;
    private Constructor<?> defaultConstructor;
    private Method factoryMethod;
    private int parserFeatures;
    private final List<FieldInfo> fieldList = new ArrayList();
    private final List<FieldInfo> sortedFieldList = new ArrayList();

    public DeserializeBeanInfo(Class<?> clazz) {
        this.parserFeatures = 0;
        this.parserFeatures = TypeUtils.getParserFeatures(clazz);
    }

    public Constructor<?> getDefaultConstructor() {
        return this.defaultConstructor;
    }

    public void setDefaultConstructor(Constructor<?> defaultConstructor) {
        this.defaultConstructor = defaultConstructor;
    }

    public Constructor<?> getCreatorConstructor() {
        return this.creatorConstructor;
    }

    public void setCreatorConstructor(Constructor<?> createConstructor) {
        this.creatorConstructor = createConstructor;
    }

    public Method getFactoryMethod() {
        return this.factoryMethod;
    }

    public void setFactoryMethod(Method factoryMethod) {
        this.factoryMethod = factoryMethod;
    }

    public List<FieldInfo> getFieldList() {
        return this.fieldList;
    }

    public List<FieldInfo> getSortedFieldList() {
        return this.sortedFieldList;
    }

    public boolean add(FieldInfo field) {
        for (FieldInfo item : this.fieldList) {
            if (item.getName().equals(field.getName()) && (!item.isGetOnly() || field.isGetOnly())) {
                return false;
            }
        }
        this.fieldList.add(field);
        this.sortedFieldList.add(field);
        Collections.sort(this.sortedFieldList);
        return true;
    }

    /* JADX WARN: Incorrect condition in loop: B:122:0x0325 */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static DeserializeBeanInfo computeSetters(Class<?> clazz, Type type) {
        JSONField fieldAnnotation;
        JSONField fieldAnnotation2;
        String propertyName;
        int ordinal;
        int serialzeFeatures;
        String propertyName2;
        Field[] arr$;
        JSONField annotation;
        String methodName;
        Method method;
        int i;
        String propertyName3;
        Field field;
        int ordinal2;
        int serialzeFeatures2;
        JSONField fieldAnnotation3;
        Class<?> cls = clazz;
        DeserializeBeanInfo beanInfo = new DeserializeBeanInfo(cls);
        Constructor<?> defaultConstructor = getDefaultConstructor(clazz);
        if (defaultConstructor != null) {
            TypeUtils.setAccessible(defaultConstructor);
            beanInfo.setDefaultConstructor(defaultConstructor);
        } else if (defaultConstructor == null && !clazz.isInterface() && !Modifier.isAbstract(clazz.getModifiers())) {
            Constructor<?> creatorConstructor = getCreatorConstructor(clazz);
            if (creatorConstructor != null) {
                TypeUtils.setAccessible(creatorConstructor);
                beanInfo.setCreatorConstructor(creatorConstructor);
                for (int i2 = 0; i2 < creatorConstructor.getParameterTypes().length; i2++) {
                    Annotation[] paramAnnotations = creatorConstructor.getParameterAnnotations()[i2];
                    int len$ = paramAnnotations.length;
                    int i$ = 0;
                    while (true) {
                        if (i$ >= len$) {
                            fieldAnnotation2 = null;
                            break;
                        }
                        Annotation paramAnnotation = paramAnnotations[i$];
                        if (!(paramAnnotation instanceof JSONField)) {
                            i$++;
                        } else {
                            JSONField fieldAnnotation4 = (JSONField) paramAnnotation;
                            fieldAnnotation2 = fieldAnnotation4;
                            break;
                        }
                    }
                    if (fieldAnnotation2 == null) {
                        throw new JSONException("illegal json creator");
                    }
                    Class<?> fieldClass = creatorConstructor.getParameterTypes()[i2];
                    Type fieldType = creatorConstructor.getGenericParameterTypes()[i2];
                    Field field2 = TypeUtils.getField(cls, fieldAnnotation2.name());
                    int ordinal3 = fieldAnnotation2.ordinal();
                    int serialzeFeatures3 = SerializerFeature.of(fieldAnnotation2.serialzeFeatures());
                    FieldInfo fieldInfo = new FieldInfo(fieldAnnotation2.name(), clazz, fieldClass, fieldType, field2, ordinal3, serialzeFeatures3);
                    beanInfo.add(fieldInfo);
                }
                return beanInfo;
            }
            Method factoryMethod = getFactoryMethod(clazz);
            if (factoryMethod != null) {
                TypeUtils.setAccessible(factoryMethod);
                beanInfo.setFactoryMethod(factoryMethod);
                for (int i3 = 0; i3 < factoryMethod.getParameterTypes().length; i3++) {
                    Annotation[] paramAnnotations2 = factoryMethod.getParameterAnnotations()[i3];
                    int len$2 = paramAnnotations2.length;
                    int i$2 = 0;
                    while (true) {
                        if (i$2 >= len$2) {
                            fieldAnnotation = null;
                            break;
                        }
                        Annotation paramAnnotation2 = paramAnnotations2[i$2];
                        if (!(paramAnnotation2 instanceof JSONField)) {
                            i$2++;
                        } else {
                            JSONField fieldAnnotation5 = (JSONField) paramAnnotation2;
                            fieldAnnotation = fieldAnnotation5;
                            break;
                        }
                    }
                    if (fieldAnnotation == null) {
                        throw new JSONException("illegal json creator");
                    }
                    Class<?> fieldClass2 = factoryMethod.getParameterTypes()[i3];
                    Type fieldType2 = factoryMethod.getGenericParameterTypes()[i3];
                    Field field3 = TypeUtils.getField(cls, fieldAnnotation.name());
                    int ordinal4 = fieldAnnotation.ordinal();
                    int serialzeFeatures4 = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                    FieldInfo fieldInfo2 = new FieldInfo(fieldAnnotation.name(), clazz, fieldClass2, fieldType2, field3, ordinal4, serialzeFeatures4);
                    beanInfo.add(fieldInfo2);
                }
                return beanInfo;
            }
            throw new JSONException("default constructor not found. " + cls);
        }
        Method[] arr$2 = clazz.getMethods();
        int len$3 = arr$2.length;
        int i$3 = 0;
        while (i$3 < len$3) {
            Method method2 = arr$2[i$3];
            int ordinal5 = 0;
            int serialzeFeatures5 = 0;
            String methodName2 = method2.getName();
            if (methodName2.length() >= 4 && !Modifier.isStatic(method2.getModifiers()) && ((method2.getReturnType().equals(Void.TYPE) || method2.getReturnType().equals(cls)) && method2.getParameterTypes().length == 1)) {
                JSONField annotation2 = (JSONField) method2.getAnnotation(JSONField.class);
                if (annotation2 != null) {
                    annotation = annotation2;
                } else {
                    annotation = TypeUtils.getSupperMethodAnnotation(cls, method2);
                }
                if (annotation == null) {
                    methodName = methodName2;
                    method = method2;
                    i = 4;
                } else if (annotation.deserialize()) {
                    int ordinal6 = annotation.ordinal();
                    int serialzeFeatures6 = SerializerFeature.of(annotation.serialzeFeatures());
                    if (annotation.name().length() == 0) {
                        methodName = methodName2;
                        method = method2;
                        i = 4;
                        ordinal5 = ordinal6;
                        serialzeFeatures5 = serialzeFeatures6;
                    } else {
                        String propertyName4 = annotation.name();
                        beanInfo.add(new FieldInfo(propertyName4, method2, (Field) null, clazz, type, ordinal6, serialzeFeatures6));
                        TypeUtils.setAccessible(method2);
                    }
                }
                if (methodName.startsWith("set")) {
                    char c3 = methodName.charAt(3);
                    if (Character.isUpperCase(c3)) {
                        if (TypeUtils.compatibleWithJavaBean) {
                            propertyName3 = TypeUtils.decapitalize(methodName.substring(3));
                        } else {
                            propertyName3 = Character.toLowerCase(methodName.charAt(3)) + methodName.substring(i);
                        }
                    } else if (c3 == '_') {
                        propertyName3 = methodName.substring(i);
                    } else if (c3 == 'f') {
                        propertyName3 = methodName.substring(3);
                    } else if (methodName.length() >= 5 && Character.isUpperCase(methodName.charAt(i))) {
                        propertyName3 = TypeUtils.decapitalize(methodName.substring(3));
                    }
                    Field field4 = TypeUtils.getField(cls, propertyName3);
                    if (field4 == null && method.getParameterTypes()[0] == Boolean.TYPE) {
                        String isFieldName = "is" + Character.toUpperCase(propertyName3.charAt(0)) + propertyName3.substring(1);
                        Field field5 = TypeUtils.getField(cls, isFieldName);
                        field = field5;
                    } else {
                        field = field4;
                    }
                    if (field != null && (fieldAnnotation3 = (JSONField) field.getAnnotation(JSONField.class)) != null) {
                        ordinal2 = fieldAnnotation3.ordinal();
                        serialzeFeatures2 = SerializerFeature.of(fieldAnnotation3.serialzeFeatures());
                        if (fieldAnnotation3.name().length() != 0) {
                            String propertyName5 = fieldAnnotation3.name();
                            beanInfo.add(new FieldInfo(propertyName5, method, field, clazz, type, ordinal2, serialzeFeatures2));
                        } else {
                            beanInfo.add(new FieldInfo(propertyName3, method, (Field) null, clazz, type, ordinal2, serialzeFeatures2));
                            TypeUtils.setAccessible(method);
                        }
                    }
                    ordinal2 = ordinal5;
                    serialzeFeatures2 = serialzeFeatures5;
                    beanInfo.add(new FieldInfo(propertyName3, method, (Field) null, clazz, type, ordinal2, serialzeFeatures2));
                    TypeUtils.setAccessible(method);
                }
            }
            i$3++;
            cls = clazz;
        }
        Field[] arr$3 = clazz.getFields();
        int len$4 = arr$3.length;
        int i$4 = 0;
        while (i$4 < len$4) {
            Field field6 = arr$3[i$4];
            if (Modifier.isStatic(field6.getModifiers())) {
                arr$ = arr$3;
            } else {
                Iterator i$5 = beanInfo.getFieldList().iterator();
                boolean contains = false;
                while (contains) {
                    FieldInfo item = i$5.next();
                    if (item.getName().equals(field6.getName())) {
                        contains = true;
                    }
                }
                if (contains) {
                    arr$ = arr$3;
                } else {
                    String propertyName6 = field6.getName();
                    JSONField fieldAnnotation6 = (JSONField) field6.getAnnotation(JSONField.class);
                    if (fieldAnnotation6 == null) {
                        ordinal = 0;
                        serialzeFeatures = 0;
                        propertyName2 = propertyName6;
                    } else {
                        int ordinal7 = fieldAnnotation6.ordinal();
                        int serialzeFeatures7 = SerializerFeature.of(fieldAnnotation6.serialzeFeatures());
                        if (fieldAnnotation6.name().length() == 0) {
                            ordinal = ordinal7;
                            serialzeFeatures = serialzeFeatures7;
                            propertyName2 = propertyName6;
                        } else {
                            String propertyName7 = fieldAnnotation6.name();
                            ordinal = ordinal7;
                            serialzeFeatures = serialzeFeatures7;
                            propertyName2 = propertyName7;
                        }
                    }
                    arr$ = arr$3;
                    beanInfo.add(new FieldInfo(propertyName2, (Method) null, field6, clazz, type, ordinal, serialzeFeatures));
                }
            }
            i$4++;
            arr$3 = arr$;
        }
        int i4 = 4;
        Method[] arr$4 = clazz.getMethods();
        int len$5 = arr$4.length;
        int i$6 = 0;
        while (i$6 < len$5) {
            Method method3 = arr$4[i$6];
            String methodName3 = method3.getName();
            if (methodName3.length() >= i4 && !Modifier.isStatic(method3.getModifiers())) {
                if (methodName3.startsWith("get") && Character.isUpperCase(methodName3.charAt(3))) {
                    if (method3.getParameterTypes().length == 0 && (Collection.class.isAssignableFrom(method3.getReturnType()) || Map.class.isAssignableFrom(method3.getReturnType()))) {
                        JSONField annotation3 = (JSONField) method3.getAnnotation(JSONField.class);
                        if (annotation3 != null && annotation3.name().length() > 0) {
                            propertyName = annotation3.name();
                        } else {
                            propertyName = Character.toLowerCase(methodName3.charAt(3)) + methodName3.substring(i4);
                        }
                        beanInfo.add(new FieldInfo(propertyName, method3, (Field) null, clazz, type));
                        TypeUtils.setAccessible(method3);
                    }
                }
            }
            i$6++;
            i4 = 4;
        }
        return beanInfo;
    }

    public static Constructor<?> getDefaultConstructor(Class<?> clazz) {
        if (Modifier.isAbstract(clazz.getModifiers())) {
            return null;
        }
        Constructor<?> defaultConstructor = null;
        Constructor<?>[] arr$ = clazz.getDeclaredConstructors();
        int len$ = arr$.length;
        int i$ = 0;
        while (true) {
            if (i$ >= len$) {
                break;
            }
            Constructor<?> constructor = arr$[i$];
            if (constructor.getParameterTypes().length != 0) {
                i$++;
            } else {
                defaultConstructor = constructor;
                break;
            }
        }
        if (defaultConstructor == null && clazz.isMemberClass() && !Modifier.isStatic(clazz.getModifiers())) {
            Constructor<?>[] arr$2 = clazz.getDeclaredConstructors();
            for (Constructor<?> constructor2 : arr$2) {
                if (constructor2.getParameterTypes().length == 1 && constructor2.getParameterTypes()[0].equals(clazz.getDeclaringClass())) {
                    return constructor2;
                }
            }
            return defaultConstructor;
        }
        return defaultConstructor;
    }

    public static Constructor<?> getCreatorConstructor(Class<?> clazz) {
        Constructor<?>[] arr$ = clazz.getDeclaredConstructors();
        for (Constructor<?> constructor : arr$) {
            JSONCreator annotation = (JSONCreator) constructor.getAnnotation(JSONCreator.class);
            if (annotation != null) {
                if (0 != 0) {
                    throw new JSONException("multi-json creator");
                } else {
                    return constructor;
                }
            }
        }
        return null;
    }

    public static Method getFactoryMethod(Class<?> clazz) {
        Method[] arr$ = clazz.getDeclaredMethods();
        for (Method method : arr$) {
            if (Modifier.isStatic(method.getModifiers()) && clazz.isAssignableFrom(method.getReturnType())) {
                JSONCreator annotation = (JSONCreator) method.getAnnotation(JSONCreator.class);
                if (annotation != null) {
                    if (0 != 0) {
                        throw new JSONException("multi-json creator");
                    }
                    return method;
                }
            }
        }
        return null;
    }

    public int getParserFeatures() {
        return this.parserFeatures;
    }
}
