package com.alivc.rtc;

import android.content.Context;
import android.os.Build;
import android.os.CpuUsageInfo;
import android.os.HardwarePropertiesManager;
import android.os.Process;
import java.io.Serializable;
import java.lang.reflect.Method;
/* loaded from: classes.dex */
public class ProcessCpuTracker implements Serializable {
    private static final int PROCESS_STAT_STIME = 3;
    private static final int PROCESS_STAT_UTIME = 2;
    private static final int PROC_COMBINE = 256;
    private static final int PROC_OUT_LONG = 8192;
    private static final int PROC_PARENS = 512;
    private static final int PROC_SPACE_TERM = 32;
    private long mBaseIdleTime;
    private long mBaseIoWaitTime;
    private long mBaseIrqTime;
    private long mBaseSoftIrqTime;
    private long mBaseSystemTime;
    private long mBaseUserTime;
    private int mMyPidPercent;
    private String mPidStatFile;
    private long mProcessBaseSystemTime;
    private long mProcessBaseUserTime;
    private int mProcessRelSystemTime;
    private int mProcessRelUserTime;
    private long mProcessSystemTime;
    private long mProcessUserTime;
    private Method mReadProcFile;
    private int mRelIdleTime;
    private int mRelIoWaitTime;
    private int mRelIrqTime;
    private int mRelSoftIrqTime;
    private int mRelSystemTime;
    private int mRelUserTime;
    private int mTotalSysPercent;
    static final int[] PROCESS_STATS_FORMAT = {32, 544, 32, 32, 32, 32, 32, 32, 32, 8224, 32, 8224, 32, 8224, 8224, 32, 32, 32, 32, 8224, 32, 8224, 32};
    static final int[] SYSTEM_CPU_FORMAT = {288, 8224, 8224, 8224, 8224, 8224, 8224, 8224};
    private final long[] mSysCpu = new long[7];
    private final long[] mStatsData = new long[6];

    /* JADX INFO: Access modifiers changed from: package-private */
    public ProcessCpuTracker() {
        try {
            this.mPidStatFile = "/proc/" + Process.myPid() + "/stat";
            Method method = Process.class.getMethod("readProcFile", String.class, int[].class, String[].class, long[].class, float[].class);
            this.mReadProcFile = method;
            method.setAccessible(true);
        } catch (Exception e) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:20:0x0096 A[Catch: Exception -> 0x016a, TryCatch #0 {Exception -> 0x016a, blocks: (B:7:0x000e, B:10:0x0032, B:12:0x0054, B:14:0x0065, B:16:0x006b, B:18:0x0078, B:20:0x0096, B:22:0x009e, B:24:0x00a6, B:26:0x00b0, B:28:0x00bc, B:30:0x00c4, B:32:0x00ee, B:34:0x0112, B:35:0x0114, B:37:0x0118, B:45:0x0149, B:44:0x013c, B:17:0x0074), top: B:62:0x000e }] */
    /* JADX WARN: Removed duplicated region for block: B:43:0x0132  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void updateCpuUsages(Context context) {
        long usertime;
        long j;
        long usertime2;
        long idletime;
        long iowaittime;
        long irqtime;
        long irqtime2;
        Method method = this.mReadProcFile;
        if (method != null && this.mPidStatFile != null) {
            try {
                boolean invokeRet = ((Boolean) method.invoke(null, "/proc/stat", SYSTEM_CPU_FORMAT, null, this.mSysCpu, null)).booleanValue();
                if (!invokeRet) {
                    return;
                }
                boolean invokeRet2 = ((Boolean) this.mReadProcFile.invoke(null, this.mPidStatFile, PROCESS_STATS_FORMAT, null, this.mStatsData, null)).booleanValue();
                if (invokeRet2) {
                    long[] jArr = this.mStatsData;
                    long j2 = jArr[2];
                    this.mProcessUserTime = j2;
                    long j3 = jArr[3];
                    this.mProcessSystemTime = j3;
                    long j4 = this.mProcessBaseUserTime;
                    if (j2 >= j4) {
                        long j5 = this.mProcessBaseSystemTime;
                        if (j3 >= j5) {
                            this.mProcessRelUserTime = (int) (j2 - j4);
                            this.mProcessRelSystemTime = (int) (j3 - j5);
                            long[] jArr2 = this.mSysCpu;
                            usertime = jArr2[0] + jArr2[1];
                            long systemtime = jArr2[2];
                            long idletime2 = jArr2[3];
                            long iowaittime2 = jArr2[4];
                            long irqtime3 = jArr2[5];
                            long softirqtime = jArr2[6];
                            j = this.mBaseUserTime;
                            if (usertime < j) {
                                usertime2 = usertime;
                                long j6 = this.mBaseSystemTime;
                                if (systemtime >= j6) {
                                    idletime = idletime2;
                                    long j7 = this.mBaseIoWaitTime;
                                    if (iowaittime2 >= j7) {
                                        iowaittime = iowaittime2;
                                        long j8 = this.mBaseIrqTime;
                                        if (irqtime3 >= j8) {
                                            long j9 = this.mBaseSoftIrqTime;
                                            irqtime = irqtime3;
                                            irqtime2 = softirqtime;
                                            if (irqtime2 >= j9) {
                                                long j10 = this.mBaseIdleTime;
                                                if (idletime >= j10) {
                                                    int i = (int) (usertime2 - j);
                                                    this.mRelUserTime = i;
                                                    int i2 = (int) (systemtime - j6);
                                                    this.mRelSystemTime = i2;
                                                    int i3 = (int) (iowaittime - j7);
                                                    this.mRelIoWaitTime = i3;
                                                    int i4 = (int) (irqtime - j8);
                                                    this.mRelIrqTime = i4;
                                                    int i5 = (int) (irqtime2 - j9);
                                                    this.mRelSoftIrqTime = i5;
                                                    int i6 = (int) (idletime - j10);
                                                    this.mRelIdleTime = i6;
                                                    long totalTime = i + i2 + i3 + i4 + i5 + i6;
                                                    if (totalTime > 1) {
                                                        int percent = Math.abs((int) (((this.mProcessRelUserTime + this.mProcessRelSystemTime) * 100) / totalTime));
                                                        int abs = Math.abs((int) (((this.mRelUserTime + this.mRelSystemTime) * 100) / totalTime));
                                                        this.mTotalSysPercent = abs;
                                                        this.mMyPidPercent = percent;
                                                        if (abs > 100) {
                                                            this.mTotalSysPercent = 100;
                                                        }
                                                        int i7 = this.mTotalSysPercent;
                                                        if (percent > i7) {
                                                            this.mMyPidPercent = i7;
                                                        }
                                                    }
                                                    this.mProcessBaseUserTime = this.mProcessUserTime;
                                                    this.mProcessBaseSystemTime = this.mProcessSystemTime;
                                                    this.mBaseUserTime = usertime2;
                                                    this.mBaseSystemTime = systemtime;
                                                    this.mBaseIoWaitTime = iowaittime;
                                                    this.mBaseIrqTime = irqtime;
                                                    this.mBaseSoftIrqTime = irqtime2;
                                                    this.mBaseIdleTime = idletime;
                                                }
                                            }
                                        } else {
                                            irqtime = irqtime3;
                                            irqtime2 = softirqtime;
                                        }
                                    } else {
                                        iowaittime = iowaittime2;
                                        irqtime = irqtime3;
                                        irqtime2 = softirqtime;
                                    }
                                } else {
                                    idletime = idletime2;
                                    iowaittime = iowaittime2;
                                    irqtime = irqtime3;
                                    irqtime2 = softirqtime;
                                }
                            } else {
                                usertime2 = usertime;
                                idletime = idletime2;
                                iowaittime = iowaittime2;
                                irqtime = irqtime3;
                                irqtime2 = softirqtime;
                            }
                            this.mRelUserTime = 0;
                            this.mRelSystemTime = 0;
                            this.mRelIoWaitTime = 0;
                            this.mRelIrqTime = 0;
                            this.mRelSoftIrqTime = 0;
                            this.mRelIdleTime = 0;
                            this.mProcessBaseUserTime = this.mProcessUserTime;
                            this.mProcessBaseSystemTime = this.mProcessSystemTime;
                            this.mBaseUserTime = usertime2;
                            this.mBaseSystemTime = systemtime;
                            this.mBaseIoWaitTime = iowaittime;
                            this.mBaseIrqTime = irqtime;
                            this.mBaseSoftIrqTime = irqtime2;
                            this.mBaseIdleTime = idletime;
                        }
                    }
                    this.mProcessRelUserTime = 0;
                    this.mProcessRelSystemTime = 0;
                    long[] jArr22 = this.mSysCpu;
                    usertime = jArr22[0] + jArr22[1];
                    long systemtime2 = jArr22[2];
                    long idletime22 = jArr22[3];
                    long iowaittime22 = jArr22[4];
                    long irqtime32 = jArr22[5];
                    long softirqtime2 = jArr22[6];
                    j = this.mBaseUserTime;
                    if (usertime < j) {
                    }
                    this.mRelUserTime = 0;
                    this.mRelSystemTime = 0;
                    this.mRelIoWaitTime = 0;
                    this.mRelIrqTime = 0;
                    this.mRelSoftIrqTime = 0;
                    this.mRelIdleTime = 0;
                    this.mProcessBaseUserTime = this.mProcessUserTime;
                    this.mProcessBaseSystemTime = this.mProcessSystemTime;
                    this.mBaseUserTime = usertime2;
                    this.mBaseSystemTime = systemtime2;
                    this.mBaseIoWaitTime = iowaittime;
                    this.mBaseIrqTime = irqtime;
                    this.mBaseSoftIrqTime = irqtime2;
                    this.mBaseIdleTime = idletime;
                }
            } catch (Exception e) {
            }
        }
        if (Build.VERSION.SDK_INT >= 24 && context != null) {
            try {
                HardwarePropertiesManager manager = (HardwarePropertiesManager) context.getSystemService("hardware_properties");
                if (manager != null) {
                    CpuUsageInfo[] cpuUsages = manager.getCpuUsages();
                    int cpuPercent = 0;
                    for (CpuUsageInfo usageInfo : cpuUsages) {
                        cpuPercent = (int) (cpuPercent + usageInfo.getActive());
                    }
                    this.mMyPidPercent = cpuPercent;
                }
            } catch (Exception e2) {
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getMyPicCpuPercent() {
        return this.mMyPidPercent;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getTotalSysCpuPercent() {
        return this.mTotalSysPercent;
    }
}
