package com.alivc.rtc.device.core.persistent;

import com.alivc.rtc.device.core.persistent.MySharedPreferences;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;
import org.xmlpull.v1.XmlPullParserException;
/* loaded from: classes.dex */
public class TransactionXMLFile {
    private static final Object GLOBAL_COMMIT_LOCK = new Object();
    public static final int MODE_PRIVATE = 0;
    private File mPreferencesDir;
    private final Object mSync = new Object();
    private HashMap<File, MySharedPreferencesImpl> sSharedPrefs = new HashMap<>();

    public TransactionXMLFile(String dir) {
        if (dir != null && dir.length() > 0) {
            this.mPreferencesDir = new File(dir);
            return;
        }
        throw new RuntimeException("Directory can not be empty");
    }

    private File makeFilename(File base, String name) {
        if (name.indexOf(File.separatorChar) < 0) {
            return new File(base, name);
        }
        throw new IllegalArgumentException("File " + name + " contains a path separator");
    }

    private File getPreferencesDir() {
        File file;
        synchronized (this.mSync) {
            file = this.mPreferencesDir;
        }
        return file;
    }

    private File getSharedPrefsFile(String name) {
        return makeFilename(getPreferencesDir(), name + ".xml");
    }

    /* JADX WARN: Removed duplicated region for block: B:77:0x008a A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:79:0x0095 A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /* JADX WARN: Unsupported multi-entry loop pattern (BACK_EDGE: B:35:0x006d -> B:73:0x0088). Please submit an issue!!! */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public MySharedPreferences getMySharedPreferences(String name, int mode) {
        File f = getSharedPrefsFile(name);
        synchronized (GLOBAL_COMMIT_LOCK) {
            MySharedPreferencesImpl sp = this.sSharedPrefs.get(f);
            if (sp == null || sp.hasFileChanged()) {
                FileInputStream str = null;
                File backup = makeBackupFile(f);
                if (backup.exists()) {
                    f.delete();
                    backup.renameTo(f);
                }
                Map map = null;
                if (f.exists()) {
                    try {
                        if (f.canRead()) {
                            try {
                                str = new FileInputStream(f);
                                map = XmlUtils.readMapXml(str);
                                str.close();
                                try {
                                    str.close();
                                } catch (Throwable th) {
                                }
                            } catch (XmlPullParserException e) {
                                try {
                                    try {
                                        str = new FileInputStream(f);
                                        byte[] buf = new byte[str.available()];
                                        str.read(buf);
                                        str.close();
                                    } catch (Throwable th2) {
                                    }
                                } catch (Exception e2) {
                                    if (str != null) {
                                        str.close();
                                    }
                                    if (str != null) {
                                        try {
                                            str.close();
                                        } catch (Throwable th3) {
                                        }
                                    }
                                    Map map2 = map;
                                    synchronized (GLOBAL_COMMIT_LOCK) {
                                    }
                                }
                                if (str != null) {
                                }
                            } catch (Exception e3) {
                                if (str != null) {
                                    try {
                                        str.close();
                                    } catch (Throwable th4) {
                                    }
                                }
                            }
                        }
                    } catch (Throwable th5) {
                        if (str != null) {
                            try {
                                str.close();
                            } catch (Throwable th6) {
                            }
                        }
                        throw th5;
                    }
                }
                Map map22 = map;
                synchronized (GLOBAL_COMMIT_LOCK) {
                    if (sp != null) {
                        sp.replace(map22);
                    } else {
                        sp = this.sSharedPrefs.get(f);
                        if (sp == null) {
                            sp = new MySharedPreferencesImpl(f, mode, map22);
                            this.sSharedPrefs.put(f, sp);
                        }
                    }
                }
                return sp;
            }
            return sp;
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public static File makeBackupFile(File prefsFile) {
        return new File(prefsFile.getPath() + ".bak");
    }

    /* loaded from: classes.dex */
    private static final class MySharedPreferencesImpl implements MySharedPreferences {
        private static final Object mContent = new Object();
        private boolean hasChange = false;
        private final File mBackupFile;
        private final File mFile;
        private WeakHashMap<MySharedPreferences.OnSharedPreferenceChangeListener, Object> mListeners;
        private Map mMap;
        private final int mMode;

        MySharedPreferencesImpl(File file, int mode, Map initialContents) {
            this.mFile = file;
            this.mBackupFile = TransactionXMLFile.makeBackupFile(file);
            this.mMode = mode;
            this.mMap = initialContents != null ? initialContents : new HashMap();
            this.mListeners = new WeakHashMap<>();
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public boolean checkFile() {
            if (this.mFile != null) {
                File fTmp = new File(this.mFile.getAbsolutePath());
                if (fTmp.exists()) {
                    return true;
                }
                return false;
            }
            return false;
        }

        public void setHasChange(boolean hasChange) {
            synchronized (this) {
                this.hasChange = hasChange;
            }
        }

        public boolean hasFileChanged() {
            boolean z;
            synchronized (this) {
                z = this.hasChange;
            }
            return z;
        }

        public void replace(Map newContents) {
            if (newContents != null) {
                synchronized (this) {
                    this.mMap = newContents;
                }
            }
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public void registerOnSharedPreferenceChangeListener(MySharedPreferences.OnSharedPreferenceChangeListener listener) {
            synchronized (this) {
                this.mListeners.put(listener, mContent);
            }
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public void unregisterOnSharedPreferenceChangeListener(MySharedPreferences.OnSharedPreferenceChangeListener listener) {
            synchronized (this) {
                this.mListeners.remove(listener);
            }
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public Map<String, ?> getAll() {
            HashMap hashMap;
            synchronized (this) {
                hashMap = new HashMap(this.mMap);
            }
            return hashMap;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public String getString(String key, String defValue) {
            String str;
            synchronized (this) {
                String v = (String) this.mMap.get(key);
                str = v != null ? v : defValue;
            }
            return str;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public int getInt(String key, int defValue) {
            int intValue;
            synchronized (this) {
                Integer v = (Integer) this.mMap.get(key);
                intValue = v != null ? v.intValue() : defValue;
            }
            return intValue;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public long getLong(String key, long defValue) {
            long longValue;
            synchronized (this) {
                Long v = (Long) this.mMap.get(key);
                longValue = v != null ? v.longValue() : defValue;
            }
            return longValue;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public float getFloat(String key, float defValue) {
            float floatValue;
            synchronized (this) {
                Float v = (Float) this.mMap.get(key);
                floatValue = v != null ? v.floatValue() : defValue;
            }
            return floatValue;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public boolean getBoolean(String key, boolean defValue) {
            boolean booleanValue;
            synchronized (this) {
                Boolean v = (Boolean) this.mMap.get(key);
                booleanValue = v != null ? v.booleanValue() : defValue;
            }
            return booleanValue;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public boolean contains(String key) {
            boolean containsKey;
            synchronized (this) {
                containsKey = this.mMap.containsKey(key);
            }
            return containsKey;
        }

        @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences
        public MySharedPreferences.MyEditor edit() {
            return new EditorImpl();
        }

        private FileOutputStream createFileOutputStream(File file) {
            try {
                FileOutputStream str = new FileOutputStream(file);
                return str;
            } catch (FileNotFoundException e) {
                File parent = file.getParentFile();
                if (!parent.mkdir()) {
                    return null;
                }
                try {
                    FileOutputStream str2 = new FileOutputStream(file);
                    return str2;
                } catch (FileNotFoundException e2) {
                    return null;
                }
            }
        }

        /* JADX INFO: Access modifiers changed from: private */
        public boolean writeFileLocked() {
            if (this.mFile.exists()) {
                if (!this.mBackupFile.exists()) {
                    if (!this.mFile.renameTo(this.mBackupFile)) {
                        return false;
                    }
                } else {
                    this.mFile.delete();
                }
            }
            try {
                FileOutputStream str = createFileOutputStream(this.mFile);
                if (str == null) {
                    return false;
                }
                XmlUtils.writeMapXml(this.mMap, str);
                str.close();
                this.mBackupFile.delete();
                return true;
            } catch (Exception e) {
                if (this.mFile.exists()) {
                    this.mFile.delete();
                }
                return false;
            }
        }

        /* loaded from: classes.dex */
        public final class EditorImpl implements MySharedPreferences.MyEditor {
            private final Map<String, Object> mModified = new HashMap();
            private boolean mClear = false;

            public EditorImpl() {
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor putString(String key, String value) {
                synchronized (this) {
                    this.mModified.put(key, value);
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor putInt(String key, int value) {
                synchronized (this) {
                    this.mModified.put(key, Integer.valueOf(value));
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor putLong(String key, long value) {
                synchronized (this) {
                    this.mModified.put(key, Long.valueOf(value));
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor putFloat(String key, float value) {
                synchronized (this) {
                    this.mModified.put(key, Float.valueOf(value));
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor putBoolean(String key, boolean value) {
                synchronized (this) {
                    this.mModified.put(key, Boolean.valueOf(value));
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor remove(String key) {
                synchronized (this) {
                    this.mModified.put(key, this);
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public MySharedPreferences.MyEditor clear() {
                synchronized (this) {
                    this.mClear = true;
                }
                return this;
            }

            @Override // com.alivc.rtc.device.core.persistent.MySharedPreferences.MyEditor
            public boolean commit() {
                boolean hasListeners;
                boolean returnValue;
                List<String> keysModified = null;
                Set<MySharedPreferences.OnSharedPreferenceChangeListener> listeners = null;
                synchronized (TransactionXMLFile.GLOBAL_COMMIT_LOCK) {
                    hasListeners = MySharedPreferencesImpl.this.mListeners.size() > 0;
                    if (hasListeners) {
                        keysModified = new ArrayList<>();
                        listeners = new HashSet<>(MySharedPreferencesImpl.this.mListeners.keySet());
                    }
                    synchronized (this) {
                        if (this.mClear) {
                            MySharedPreferencesImpl.this.mMap.clear();
                            this.mClear = false;
                        }
                        for (Map.Entry<String, Object> e : this.mModified.entrySet()) {
                            String k = e.getKey();
                            Object v = e.getValue();
                            if (v == this) {
                                MySharedPreferencesImpl.this.mMap.remove(k);
                            } else {
                                MySharedPreferencesImpl.this.mMap.put(k, v);
                            }
                            if (hasListeners) {
                                keysModified.add(k);
                            }
                        }
                        this.mModified.clear();
                    }
                    returnValue = MySharedPreferencesImpl.this.writeFileLocked();
                    if (returnValue) {
                        MySharedPreferencesImpl.this.setHasChange(true);
                    }
                }
                if (hasListeners) {
                    for (int i = keysModified.size() - 1; i >= 0; i--) {
                        String key = keysModified.get(i);
                        for (MySharedPreferences.OnSharedPreferenceChangeListener listener : listeners) {
                            if (listener != null) {
                                listener.onSharedPreferenceChanged(MySharedPreferencesImpl.this, key);
                            }
                        }
                    }
                }
                return returnValue;
            }
        }
    }
}
