package com.alivc.rtc.device.utils;
/* loaded from: classes.dex */
public class RC4 {
    private static final String RC4_PK = "QrMgt8GGYI6T52ZY5AnhtxkLzb8egpFn3j5JELI8H6wtACbUnZ5cc3aYTsTRbmkAkRJeYbtx92LPBWm7nBO9UIl7y5i5MQNmUZNf5QENurR5tGyo7yJ2G0MBjWvy6iAtlAbacKP0SwOUeUWx5dsBdyhxa7Id1APtybSdDgicBDuNjI0mlZFUzZSS9dmN8lBD0WTVOMz0pRZbR3cysomRXOO1ghqjJdTcyDIxzpNAEszN8RMGjrzyU7Hjbmwi6YNK";

    public static byte[] rc4(byte[] pData) {
        RC4Key lRC4Key;
        if (pData != null && (lRC4Key = prepareKey(RC4_PK)) != null) {
            return rc4(pData, lRC4Key);
        }
        return null;
    }

    private static RC4Key prepareKey(String pPrivateKey) {
        if (pPrivateKey == null) {
            return null;
        }
        RC4Key lRC4Key = new RC4Key();
        for (int counter = 0; counter < 256; counter++) {
            lRC4Key.state[counter] = counter;
        }
        lRC4Key.x = 0;
        lRC4Key.y = 0;
        int index1 = 0;
        int index2 = 0;
        for (int counter2 = 0; counter2 < 256; counter2++) {
            try {
                index2 = ((pPrivateKey.charAt(index1) + lRC4Key.state[counter2]) + index2) % 256;
                int swapInt = lRC4Key.state[counter2];
                lRC4Key.state[counter2] = lRC4Key.state[index2];
                lRC4Key.state[index2] = swapInt;
                index1 = (index1 + 1) % pPrivateKey.length();
            } catch (Exception e) {
                return null;
            }
        }
        return lRC4Key;
    }

    private static byte[] rc4(byte[] pData, RC4Key pKey) {
        if (pData != null && pKey != null) {
            int x = pKey.x;
            int y = pKey.y;
            for (int counter = 0; counter < pData.length; counter++) {
                x = (x + 1) % 256;
                y = (pKey.state[x] + y) % 256;
                int lSwapInt = pKey.state[x];
                pKey.state[x] = pKey.state[y];
                pKey.state[y] = lSwapInt;
                int xorIndex = (pKey.state[x] + pKey.state[y]) % 256;
                pData[counter] = (byte) (pData[counter] ^ pKey.state[xorIndex]);
            }
            pKey.x = x;
            pKey.y = y;
            return pData;
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes.dex */
    public static class RC4Key {
        public int[] state;
        public int x;
        public int y;

        private RC4Key() {
            this.state = new int[256];
        }
    }
}
