package com.bjz.comm.net.utils;

import android.text.TextUtils;
import android.util.Log;
import com.bjz.comm.net.BuildVars;
import com.bjz.comm.net.bean.BResponse;
import com.bjz.comm.net.bean.BResponseNoData;
import com.bjz.comm.net.exception.ApiException;
import com.bjz.comm.net.exception.KeyNotValidThrowable;
import com.bjz.comm.net.utils.RxHelper;
import com.google.gson.JsonSyntaxException;
import io.reactivex.Observable;
import io.reactivex.ObservableSource;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Action;
import io.reactivex.functions.Consumer;
import io.reactivex.functions.Function;
import io.reactivex.schedulers.Schedulers;
import java.io.IOException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.concurrent.Callable;
/* loaded from: classes2.dex */
public class RxHelper {
    private static String TAG = RxHelper.class.getSimpleName();
    private HashMap<String, CompositeDisposable> mTaskDisposable;

    /* synthetic */ RxHelper(AnonymousClass1 x0) {
        this();
    }

    /* loaded from: classes2.dex */
    private static class RxHelperHolder {
        private static RxHelper instance = new RxHelper(null);

        private RxHelperHolder() {
        }
    }

    private RxHelper() {
        this.mTaskDisposable = new HashMap<>();
    }

    public static RxHelper getInstance() {
        return RxHelperHolder.instance;
    }

    private void addTaskDisposable(String tag, Disposable disposable) {
        if (this.mTaskDisposable.get(tag) != null) {
            this.mTaskDisposable.get(tag).add(disposable);
            return;
        }
        CompositeDisposable compositeDisposable = new CompositeDisposable();
        compositeDisposable.add(disposable);
        this.mTaskDisposable.put(tag, compositeDisposable);
    }

    /* JADX WARN: Multi-variable type inference failed */
    public <T> void sendSimpleRequest(final String tag, Observable<T> observable, Consumer<T> onNext, Consumer<Throwable> onError) {
        addTaskDisposable(tag, observable.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).doOnComplete(new Action() { // from class: com.bjz.comm.net.utils.RxHelper$$ExternalSyntheticLambda5
            @Override // io.reactivex.functions.Action
            public final void run() {
                RxHelper.this.m20lambda$sendSimpleRequest$0$combjzcommnetutilsRxHelper(tag);
            }
        }).subscribe(onNext, onError));
    }

    public <R> void sendRequest(final String tag, Observable<BResponse<R>> observable, Consumer<BResponse<R>> onNext, Consumer<Throwable> onError) {
        Disposable disposable = send(observable).subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).doOnComplete(new Action() { // from class: com.bjz.comm.net.utils.RxHelper$$ExternalSyntheticLambda2
            @Override // io.reactivex.functions.Action
            public final void run() {
                RxHelper.this.m18lambda$sendRequest$1$combjzcommnetutilsRxHelper(tag);
            }
        }).subscribe(onNext, onError);
        addTaskDisposable(tag, disposable);
    }

    private <R> Observable send(final Observable<R> observable) {
        return Observable.defer(new Callable() { // from class: com.bjz.comm.net.utils.RxHelper$$ExternalSyntheticLambda0
            @Override // java.util.concurrent.Callable
            public final Object call() {
                ObservableSource just;
                just = Observable.just(HttpUtils.getInstance().getAuthorization());
                return just;
            }
        }).flatMap(new Function() { // from class: com.bjz.comm.net.utils.RxHelper$$ExternalSyntheticLambda1
            @Override // io.reactivex.functions.Function
            public final Object apply(Object obj) {
                return RxHelper.lambda$send$3(Observable.this, (String) obj);
            }
        }).retryWhen(new AnonymousClass1());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ ObservableSource lambda$send$3(Observable observable, String key) throws Exception {
        if (TextUtils.isEmpty(key)) {
            if (BuildVars.DEBUG_VERSION) {
                Log.e(TAG, "TokenRequest = null");
            }
            return Observable.error(new KeyNotValidThrowable());
        }
        if (BuildVars.DEBUG_VERSION) {
            Log.e(TAG, "TokenRequest = " + key);
        }
        return observable;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: com.bjz.comm.net.utils.RxHelper$1  reason: invalid class name */
    /* loaded from: classes2.dex */
    public class AnonymousClass1 implements Function<Observable<? extends Throwable>, ObservableSource<?>> {
        private int retryCount = 0;

        AnonymousClass1() {
        }

        @Override // io.reactivex.functions.Function
        public ObservableSource<?> apply(Observable<? extends Throwable> throwableObservable) throws Exception {
            return throwableObservable.flatMap(new Function() { // from class: com.bjz.comm.net.utils.RxHelper$1$$ExternalSyntheticLambda0
                @Override // io.reactivex.functions.Function
                public final Object apply(Object obj) {
                    return RxHelper.AnonymousClass1.this.m21lambda$apply$0$combjzcommnetutilsRxHelper$1((Throwable) obj);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$apply$0$com-bjz-comm-net-utils-RxHelper$1  reason: not valid java name */
        public /* synthetic */ ObservableSource m21lambda$apply$0$combjzcommnetutilsRxHelper$1(Throwable throwable) throws Exception {
            if (BuildVars.DEBUG_VERSION) {
                Log.e(RxHelper.TAG, "TokenRequest retryCount = " + this.retryCount);
            }
            if (throwable instanceof KeyNotValidThrowable) {
                if (BuildVars.DEBUG_VERSION) {
                    Log.e(RxHelper.TAG, "TokenRequest KeyNotValidThrowable");
                }
                int i = this.retryCount;
                if (i > 0) {
                    return Observable.error(throwable);
                }
                this.retryCount = i + 1;
                return TokenLoader.getInstance().getNetTokenLocked();
            }
            if (throwable instanceof ApiException) {
                if (BuildVars.DEBUG_VERSION) {
                    Log.e(RxHelper.TAG, "TokenRequest ApiException");
                }
                ApiException apiException = (ApiException) throwable;
                if (apiException.getCode() == 400) {
                    int i2 = this.retryCount;
                    if (i2 > 0) {
                        return Observable.error(throwable);
                    }
                    this.retryCount = i2 + 1;
                    return TokenLoader.getInstance().getNetTokenLocked();
                }
            }
            return Observable.error(throwable);
        }
    }

    public void sendRequestNoData(final String tag, Observable<BResponseNoData> observable, Consumer<BResponseNoData> onNext, Consumer<Throwable> onError) {
        Disposable disposable = send(observable).subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).doOnComplete(new Action() { // from class: com.bjz.comm.net.utils.RxHelper$$ExternalSyntheticLambda4
            @Override // io.reactivex.functions.Action
            public final void run() {
                RxHelper.this.m19lambda$sendRequestNoData$4$combjzcommnetutilsRxHelper(tag);
            }
        }).subscribe(onNext, onError);
        addTaskDisposable(tag, disposable);
    }

    public <R> void sendCommRequest(final String tag, Observable<R> observable, Consumer<R> onNext, Consumer<Throwable> onError) {
        Disposable disposable = send(observable).subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).doOnComplete(new Action() { // from class: com.bjz.comm.net.utils.RxHelper$$ExternalSyntheticLambda3
            @Override // io.reactivex.functions.Action
            public final void run() {
                RxHelper.this.m17lambda$sendCommRequest$5$combjzcommnetutilsRxHelper(tag);
            }
        }).subscribe(onNext, onError);
        addTaskDisposable(tag, disposable);
    }

    /* renamed from: unSubscribeTask */
    public void m20lambda$sendSimpleRequest$0$combjzcommnetutilsRxHelper(String tag) {
        Disposable dis = this.mTaskDisposable.get(tag);
        if (dis != null) {
            if (!dis.isDisposed()) {
                dis.dispose();
            }
            this.mTaskDisposable.remove(tag);
        }
    }

    public String getErrorInfo(Throwable throwable) {
        if (BuildVars.DEBUG_VERSION) {
            Log.e("TAG", "" + throwable.getMessage());
        }
        if (throwable instanceof SocketTimeoutException) {
            return "请求超时";
        }
        if (throwable instanceof UnknownHostException) {
            return "网络异常";
        }
        if (throwable instanceof IOException) {
            return "服务器异常";
        }
        if (throwable instanceof JsonSyntaxException) {
            return "返回数据异常";
        }
        return "请求失败";
    }
}
