package com.bjz.comm.net.utils;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
/* loaded from: classes2.dex */
public class StringEscapeUtils {
    public static String unescapeJavaScript(String str) {
        return unescapeJava(str);
    }

    public static void unescapeJavaScript(Writer out, String str) throws IOException {
        unescapeJava(out, str);
    }

    public static String unescapeJava(String str) {
        if (str == null) {
            return null;
        }
        try {
            StringWriter writer = new StringWriter(str.length());
            unescapeJava(writer, str);
            return writer.toString();
        } catch (IOException ioe) {
            throw new UnhandledException(ioe);
        }
    }

    public static void unescapeJava(Writer out, String str) throws IOException {
        if (out == null) {
            throw new IllegalArgumentException("The Writer must not be null");
        }
        if (str == null) {
            return;
        }
        int sz = str.length();
        StringBuffer unicode = new StringBuffer(4);
        boolean hadSlash = false;
        boolean inUnicode = false;
        for (int i = 0; i < sz; i++) {
            char ch = str.charAt(i);
            if (inUnicode) {
                unicode.append(ch);
                if (unicode.length() == 4) {
                    try {
                        int value = Integer.parseInt(unicode.toString(), 16);
                        out.write((char) value);
                        unicode.setLength(0);
                        inUnicode = false;
                        hadSlash = false;
                    } catch (NumberFormatException nfe) {
                        throw new NestableRuntimeException("Unable to parse unicode value: " + ((Object) unicode), nfe);
                    }
                } else {
                    continue;
                }
            } else if (hadSlash) {
                hadSlash = false;
                switch (ch) {
                    case '\"':
                        out.write(34);
                        continue;
                    case '\'':
                        out.write(39);
                        continue;
                    case '\\':
                        out.write(92);
                        continue;
                    case 'b':
                        out.write(8);
                        continue;
                    case 'f':
                        out.write(12);
                        continue;
                    case 'n':
                        out.write(10);
                        continue;
                    case 'r':
                        out.write(13);
                        continue;
                    case 't':
                        out.write(9);
                        continue;
                    case 'u':
                        inUnicode = true;
                        continue;
                    default:
                        out.write(ch);
                        continue;
                }
            } else if (ch == '\\') {
                hadSlash = true;
            } else {
                out.write(ch);
            }
        }
        if (hadSlash) {
            out.write(92);
        }
    }
}
