package com.bjz.comm.net.utils;

import android.util.Log;
import com.bjz.comm.net.BuildVars;
import com.bjz.comm.net.listener.GetHttpTokenCallBack;
import io.reactivex.Observable;
import io.reactivex.ObservableEmitter;
import io.reactivex.ObservableOnSubscribe;
import io.reactivex.functions.Consumer;
import io.reactivex.schedulers.Schedulers;
import io.reactivex.subjects.PublishSubject;
import java.util.concurrent.atomic.AtomicBoolean;
/* loaded from: classes2.dex */
public class TokenLoader {
    public static final String TAG = TokenLoader.class.getSimpleName();
    private GetHttpTokenCallBack callBack;
    private PublishSubject<String> mPublishSubject;
    private AtomicBoolean mRefreshing;
    private Observable<String> mTokenObservable;

    private TokenLoader() {
        this.mRefreshing = new AtomicBoolean(false);
        this.mTokenObservable = Observable.create(new ObservableOnSubscribe() { // from class: com.bjz.comm.net.utils.TokenLoader$$ExternalSyntheticLambda0
            @Override // io.reactivex.ObservableOnSubscribe
            public final void subscribe(ObservableEmitter observableEmitter) {
                TokenLoader.this.m22lambda$new$0$combjzcommnetutilsTokenLoader(observableEmitter);
            }
        }).doOnNext(new Consumer() { // from class: com.bjz.comm.net.utils.TokenLoader$$ExternalSyntheticLambda1
            @Override // io.reactivex.functions.Consumer
            public final void accept(Object obj) {
                TokenLoader.this.m23lambda$new$1$combjzcommnetutilsTokenLoader((String) obj);
            }
        }).doOnError(new Consumer() { // from class: com.bjz.comm.net.utils.TokenLoader$$ExternalSyntheticLambda2
            @Override // io.reactivex.functions.Consumer
            public final void accept(Object obj) {
                TokenLoader.this.m24lambda$new$2$combjzcommnetutilsTokenLoader((Throwable) obj);
            }
        }).subscribeOn(Schedulers.io());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$new$0$com-bjz-comm-net-utils-TokenLoader  reason: not valid java name */
    public /* synthetic */ void m22lambda$new$0$combjzcommnetutilsTokenLoader(ObservableEmitter e) throws Exception {
        GetHttpTokenCallBack getHttpTokenCallBack = this.callBack;
        if (getHttpTokenCallBack != null && e != null) {
            getHttpTokenCallBack.requestToken(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$new$1$com-bjz-comm-net-utils-TokenLoader  reason: not valid java name */
    public /* synthetic */ void m23lambda$new$1$combjzcommnetutilsTokenLoader(String token) throws Exception {
        if (BuildVars.DEBUG_VERSION) {
            Log.d(TAG, "存储Token=" + token);
        }
        setCacheToken(token);
        this.mRefreshing.set(false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$new$2$com-bjz-comm-net-utils-TokenLoader  reason: not valid java name */
    public /* synthetic */ void m24lambda$new$2$combjzcommnetutilsTokenLoader(Throwable throwable) throws Exception {
        this.mRefreshing.set(false);
    }

    public static TokenLoader getInstance() {
        return Holder.INSTANCE;
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes2.dex */
    public static class Holder {
        private static final TokenLoader INSTANCE = new TokenLoader();

        private Holder() {
        }
    }

    public String getCacheToken() {
        return HttpUtils.getInstance().getAuthorization();
    }

    public void setCacheToken(String key) {
        HttpUtils.getInstance().setAuthorization(key);
    }

    public Observable<String> getNetTokenLocked() {
        if (this.mRefreshing.compareAndSet(false, true)) {
            if (BuildVars.DEBUG_VERSION) {
                Log.e(TAG, "没有请求，发起一次新的Token请求");
            }
            startTokenRequest();
        } else if (BuildVars.DEBUG_VERSION) {
            Log.e(TAG, "已经有请求，直接返回等待");
        }
        return this.mPublishSubject;
    }

    private void startTokenRequest() {
        PublishSubject<String> create = PublishSubject.create();
        this.mPublishSubject = create;
        this.mTokenObservable.subscribe(create);
    }

    public void setCallBack(GetHttpTokenCallBack callBack) {
        this.callBack = callBack;
    }
}
