package com.blankj.utilcode.util;

import android.util.Log;
import com.litesuits.orm.db.assit.SQLBuilder;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.CopyOnWriteArraySet;
/* loaded from: classes.dex */
public final class BusUtils {
    private static final Object NULL = "nULl";
    private static final String TAG = "BusUtils";
    private final Map<String, Set<Object>> mClassName_BusesMap;
    private final Map<String, Map<String, Object>> mClassName_Tag_Arg4StickyMap;
    private final Map<String, List<String>> mClassName_TagsMap;
    private final Map<String, List<BusInfo>> mTag_BusInfoListMap;

    @Target({ElementType.METHOD})
    @Retention(RetentionPolicy.CLASS)
    /* loaded from: classes.dex */
    public @interface Bus {
        int priority() default 0;

        boolean sticky() default false;

        String tag();

        ThreadMode threadMode() default ThreadMode.POSTING;
    }

    /* loaded from: classes.dex */
    public enum ThreadMode {
        MAIN,
        IO,
        CPU,
        CACHED,
        SINGLE,
        POSTING
    }

    private BusUtils() {
        this.mTag_BusInfoListMap = new HashMap();
        this.mClassName_BusesMap = new ConcurrentHashMap();
        this.mClassName_TagsMap = new HashMap();
        this.mClassName_Tag_Arg4StickyMap = new ConcurrentHashMap();
        init();
    }

    private void init() {
    }

    private void registerBus(String tag, String className, String funName, String paramType, String paramName, boolean sticky, String threadMode) {
        registerBus(tag, className, funName, paramType, paramName, sticky, threadMode, 0);
    }

    private void registerBus(String tag, String className, String funName, String paramType, String paramName, boolean sticky, String threadMode, int priority) {
        List<BusInfo> busInfoList = this.mTag_BusInfoListMap.get(tag);
        if (busInfoList == null) {
            busInfoList = new ArrayList();
            this.mTag_BusInfoListMap.put(tag, busInfoList);
        }
        busInfoList.add(new BusInfo(className, funName, paramType, paramName, sticky, threadMode, priority));
    }

    public static void register(Object bus) {
        getInstance().registerInner(bus);
    }

    public static void unregister(Object bus) {
        getInstance().unregisterInner(bus);
    }

    public static void post(String tag) {
        post(tag, NULL);
    }

    public static void post(String tag, Object arg) {
        getInstance().postInner(tag, arg);
    }

    public static void postSticky(String tag) {
        postSticky(tag, NULL);
    }

    public static void postSticky(String tag, Object arg) {
        getInstance().postStickyInner(tag, arg);
    }

    public static void removeSticky(String tag) {
        getInstance().removeStickyInner(tag);
    }

    public static String toString_() {
        return getInstance().toString();
    }

    public String toString() {
        return "BusUtils: " + this.mTag_BusInfoListMap;
    }

    private static BusUtils getInstance() {
        return LazyHolder.INSTANCE;
    }

    private void registerInner(Object bus) {
        if (bus == null) {
            return;
        }
        Class aClass = bus.getClass();
        String className = aClass.getName();
        synchronized (this.mClassName_BusesMap) {
            Set<Object> buses = this.mClassName_BusesMap.get(className);
            if (buses == null) {
                buses = new CopyOnWriteArraySet();
                this.mClassName_BusesMap.put(className, buses);
            }
            buses.add(bus);
        }
        List<String> tags = this.mClassName_TagsMap.get(className);
        if (tags == null) {
            synchronized (this.mClassName_TagsMap) {
                List<String> tags2 = this.mClassName_TagsMap.get(className);
                if (tags2 == null) {
                    List<String> tags3 = new ArrayList<>();
                    for (Map.Entry<String, List<BusInfo>> entry : this.mTag_BusInfoListMap.entrySet()) {
                        for (BusInfo busInfo : entry.getValue()) {
                            try {
                                if (Class.forName(busInfo.className).isAssignableFrom(aClass)) {
                                    tags3.add(entry.getKey());
                                    busInfo.classNames.add(className);
                                }
                            } catch (ClassNotFoundException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    this.mClassName_TagsMap.put(className, tags3);
                }
            }
        }
        processSticky(bus);
    }

    private void processSticky(Object bus) {
        Map<String, Object> tagArgMap = this.mClassName_Tag_Arg4StickyMap.get(bus.getClass().getName());
        if (tagArgMap == null) {
            return;
        }
        synchronized (this.mClassName_Tag_Arg4StickyMap) {
            for (Map.Entry<String, Object> tagArgEntry : tagArgMap.entrySet()) {
                postInner(tagArgEntry.getKey(), tagArgEntry.getValue());
            }
        }
    }

    private void unregisterInner(Object bus) {
        if (bus == null) {
            return;
        }
        String className = bus.getClass().getName();
        synchronized (this.mClassName_BusesMap) {
            Set<Object> buses = this.mClassName_BusesMap.get(className);
            if (buses != null && buses.contains(bus)) {
                buses.remove(bus);
                return;
            }
            Log.e(TAG, "The bus of <" + bus + "> was not registered before.");
        }
    }

    private void postInner(String tag, Object arg) {
        postInner(tag, arg, false);
    }

    private void postInner(String tag, Object arg, boolean sticky) {
        List<BusInfo> busInfoList = this.mTag_BusInfoListMap.get(tag);
        if (busInfoList == null) {
            Log.e(TAG, "The bus of tag <" + tag + "> is not exists.");
            return;
        }
        for (BusInfo busInfo : busInfoList) {
            if (busInfo.method == null) {
                Method method = getMethodByBusInfo(busInfo);
                if (method == null) {
                    return;
                }
                busInfo.method = method;
            }
            invokeMethod(tag, arg, busInfo, sticky);
        }
    }

    private Method getMethodByBusInfo(BusInfo busInfo) {
        try {
            return "".equals(busInfo.paramType) ? Class.forName(busInfo.className).getDeclaredMethod(busInfo.funName, new Class[0]) : Class.forName(busInfo.className).getDeclaredMethod(busInfo.funName, getClassName(busInfo.paramType));
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            return null;
        } catch (NoSuchMethodException e2) {
            e2.printStackTrace();
            return null;
        }
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    private Class getClassName(String paramType) throws ClassNotFoundException {
        char c;
        switch (paramType.hashCode()) {
            case -1325958191:
                if (paramType.equals("double")) {
                    c = 5;
                    break;
                }
                c = 65535;
                break;
            case 104431:
                if (paramType.equals("int")) {
                    c = 1;
                    break;
                }
                c = 65535;
                break;
            case 3039496:
                if (paramType.equals("byte")) {
                    c = 4;
                    break;
                }
                c = 65535;
                break;
            case 3052374:
                if (paramType.equals("char")) {
                    c = 7;
                    break;
                }
                c = 65535;
                break;
            case 3327612:
                if (paramType.equals("long")) {
                    c = 2;
                    break;
                }
                c = 65535;
                break;
            case 64711720:
                if (paramType.equals("boolean")) {
                    c = 0;
                    break;
                }
                c = 65535;
                break;
            case 97526364:
                if (paramType.equals("float")) {
                    c = 6;
                    break;
                }
                c = 65535;
                break;
            case 109413500:
                if (paramType.equals("short")) {
                    c = 3;
                    break;
                }
                c = 65535;
                break;
            default:
                c = 65535;
                break;
        }
        switch (c) {
            case 0:
                return Boolean.TYPE;
            case 1:
                return Integer.TYPE;
            case 2:
                return Long.TYPE;
            case 3:
                return Short.TYPE;
            case 4:
                return Byte.TYPE;
            case 5:
                return Double.TYPE;
            case 6:
                return Float.TYPE;
            case 7:
                return Character.TYPE;
            default:
                return Class.forName(paramType);
        }
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    private void invokeMethod(final String tag, final Object arg, final BusInfo busInfo, final boolean sticky) {
        char c;
        Runnable runnable = new Runnable() { // from class: com.blankj.utilcode.util.BusUtils.1
            @Override // java.lang.Runnable
            public void run() {
                BusUtils.this.realInvokeMethod(tag, arg, busInfo, sticky);
            }
        };
        String str = busInfo.threadMode;
        switch (str.hashCode()) {
            case -1848936376:
                if (str.equals("SINGLE")) {
                    c = 4;
                    break;
                }
                c = 65535;
                break;
            case 2342:
                if (str.equals("IO")) {
                    c = 1;
                    break;
                }
                c = 65535;
                break;
            case 66952:
                if (str.equals("CPU")) {
                    c = 2;
                    break;
                }
                c = 65535;
                break;
            case 2358713:
                if (str.equals("MAIN")) {
                    c = 0;
                    break;
                }
                c = 65535;
                break;
            case 1980249378:
                if (str.equals("CACHED")) {
                    c = 3;
                    break;
                }
                c = 65535;
                break;
            default:
                c = 65535;
                break;
        }
        switch (c) {
            case 0:
                Utils.runOnUiThread(runnable);
                return;
            case 1:
                ThreadUtils.getIoPool().execute(runnable);
                return;
            case 2:
                ThreadUtils.getCpuPool().execute(runnable);
                return;
            case 3:
                ThreadUtils.getCachedPool().execute(runnable);
                return;
            case 4:
                ThreadUtils.getSinglePool().execute(runnable);
                return;
            default:
                runnable.run();
                return;
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void realInvokeMethod(String tag, Object arg, BusInfo busInfo, boolean sticky) {
        Set<Object> buses = new HashSet<>();
        for (String className : busInfo.classNames) {
            Set<Object> subBuses = this.mClassName_BusesMap.get(className);
            if (subBuses != null && !subBuses.isEmpty()) {
                buses.addAll(subBuses);
            }
        }
        if (buses.size() == 0) {
            if (!sticky) {
                Log.e(TAG, "The bus of tag <" + tag + "> was not registered before.");
                return;
            }
            return;
        }
        try {
            if (arg == NULL) {
                for (Object bus : buses) {
                    busInfo.method.invoke(bus, new Object[0]);
                }
                return;
            }
            for (Object bus2 : buses) {
                busInfo.method.invoke(bus2, arg);
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e2) {
            e2.printStackTrace();
        }
    }

    private void postStickyInner(String tag, Object arg) {
        List<BusInfo> busInfoList = this.mTag_BusInfoListMap.get(tag);
        if (busInfoList == null) {
            Log.e(TAG, "The bus of tag <" + tag + "> is not exists.");
            return;
        }
        for (BusInfo busInfo : busInfoList) {
            if (!busInfo.sticky) {
                postInner(tag, arg);
                return;
            }
            synchronized (this.mClassName_Tag_Arg4StickyMap) {
                Map<String, Object> tagArgMap = this.mClassName_Tag_Arg4StickyMap.get(busInfo.className);
                if (tagArgMap == null) {
                    tagArgMap = new HashMap();
                    this.mClassName_Tag_Arg4StickyMap.put(busInfo.className, tagArgMap);
                }
                tagArgMap.put(tag, arg);
            }
            postInner(tag, arg, true);
        }
    }

    private void removeStickyInner(String tag) {
        List<BusInfo> busInfoList = this.mTag_BusInfoListMap.get(tag);
        if (busInfoList == null) {
            Log.e(TAG, "The bus of tag <" + tag + "> is not exists.");
            return;
        }
        for (BusInfo busInfo : busInfoList) {
            if (!busInfo.sticky) {
                Log.e(TAG, "The bus of tag <" + tag + "> is not sticky.");
                return;
            }
            synchronized (this.mClassName_Tag_Arg4StickyMap) {
                Map<String, Object> tagArgMap = this.mClassName_Tag_Arg4StickyMap.get(busInfo.className);
                if (tagArgMap != null && tagArgMap.containsKey(tag)) {
                    tagArgMap.remove(tag);
                }
                Log.e(TAG, "The sticky bus of tag <" + tag + "> didn't post.");
                return;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes.dex */
    public static final class BusInfo {
        String className;
        List<String> classNames = new CopyOnWriteArrayList();
        String funName;
        Method method;
        String paramName;
        String paramType;
        int priority;
        boolean sticky;
        String threadMode;

        BusInfo(String className, String funName, String paramType, String paramName, boolean sticky, String threadMode, int priority) {
            this.className = className;
            this.funName = funName;
            this.paramType = paramType;
            this.paramName = paramName;
            this.sticky = sticky;
            this.threadMode = threadMode;
            this.priority = priority;
        }

        public String toString() {
            return "BusInfo { desc: " + this.className + "#" + this.funName + ("".equals(this.paramType) ? "()" : SQLBuilder.PARENTHESES_LEFT + this.paramType + " " + this.paramName + SQLBuilder.PARENTHESES_RIGHT) + ", sticky: " + this.sticky + ", threadMode: " + this.threadMode + ", method: " + this.method + ", priority: " + this.priority + " }";
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes.dex */
    public static class LazyHolder {
        private static final BusUtils INSTANCE = new BusUtils();

        private LazyHolder() {
        }
    }
}
