package com.blankj.utilcode.util;

import com.blankj.utilcode.util.Utils;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
/* loaded from: classes.dex */
public final class ShellUtils {
    private static final String LINE_SEP = System.getProperty("line.separator");

    private ShellUtils() {
        throw new UnsupportedOperationException("u can't instantiate me...");
    }

    public static Utils.Task<CommandResult> execCmdAsync(String command, boolean isRooted, Utils.Callback<CommandResult> callback) {
        return execCmdAsync(new String[]{command}, isRooted, true, callback);
    }

    public static Utils.Task<CommandResult> execCmdAsync(List<String> commands, boolean isRooted, Utils.Callback<CommandResult> callback) {
        return execCmdAsync(commands == null ? null : (String[]) commands.toArray(new String[0]), isRooted, true, callback);
    }

    public static Utils.Task<CommandResult> execCmdAsync(String[] commands, boolean isRooted, Utils.Callback<CommandResult> callback) {
        return execCmdAsync(commands, isRooted, true, callback);
    }

    public static Utils.Task<CommandResult> execCmdAsync(String command, boolean isRooted, boolean isNeedResultMsg, Utils.Callback<CommandResult> callback) {
        return execCmdAsync(new String[]{command}, isRooted, isNeedResultMsg, callback);
    }

    public static Utils.Task<CommandResult> execCmdAsync(List<String> commands, boolean isRooted, boolean isNeedResultMsg, Utils.Callback<CommandResult> callback) {
        return execCmdAsync(commands == null ? null : (String[]) commands.toArray(new String[0]), isRooted, isNeedResultMsg, callback);
    }

    public static Utils.Task<CommandResult> execCmdAsync(final String[] commands, final boolean isRooted, final boolean isNeedResultMsg, Utils.Callback<CommandResult> callback) {
        if (callback == null) {
            throw new NullPointerException("Argument 'callback' of type Utils.Callback<CommandResult> (#3 out of 4, zero-based) is marked by @android.support.annotation.NonNull but got null for it");
        }
        return Utils.doAsync(new Utils.Task<CommandResult>(callback) { // from class: com.blankj.utilcode.util.ShellUtils.1
            /* JADX WARN: Can't rename method to resolve collision */
            @Override // com.blankj.utilcode.util.Utils.Task
            public CommandResult doInBackground() {
                return ShellUtils.execCmd(commands, isRooted, isNeedResultMsg);
            }
        });
    }

    public static CommandResult execCmd(String command, boolean isRooted) {
        return execCmd(new String[]{command}, isRooted, true);
    }

    public static CommandResult execCmd(List<String> commands, boolean isRooted) {
        return execCmd(commands == null ? null : (String[]) commands.toArray(new String[0]), isRooted, true);
    }

    public static CommandResult execCmd(String[] commands, boolean isRooted) {
        return execCmd(commands, isRooted, true);
    }

    public static CommandResult execCmd(String command, boolean isRooted, boolean isNeedResultMsg) {
        return execCmd(new String[]{command}, isRooted, isNeedResultMsg);
    }

    public static CommandResult execCmd(List<String> commands, boolean isRooted, boolean isNeedResultMsg) {
        return execCmd(commands == null ? null : (String[]) commands.toArray(new String[0]), isRooted, isNeedResultMsg);
    }

    /* JADX WARN: Can't wrap try/catch for region: R(12:6|7|(1:9)(1:64)|10|(3:12|(2:14|15)(2:17|18)|16)|19|20|(4:22|(2:24|(2:25|(1:27)(1:28)))(0)|29|(5:31|(2:32|(1:34)(0))|37|(2:56|57)|(2:51|52))(0))(0)|36|37|(0)|(0)) */
    /* JADX WARN: Code restructure failed: missing block: B:37:0x00ce, code lost:
        r0 = move-exception;
     */
    /* JADX WARN: Code restructure failed: missing block: B:38:0x00cf, code lost:
        r0.printStackTrace();
     */
    /* JADX WARN: Code restructure failed: missing block: B:51:0x00ea, code lost:
        if (r3 != null) goto L41;
     */
    /* JADX WARN: Code restructure failed: missing block: B:52:0x00ec, code lost:
        r3.destroy();
     */
    /* JADX WARN: Code restructure failed: missing block: B:75:0x011a, code lost:
        if (r3 == null) goto L42;
     */
    /* JADX WARN: Code restructure failed: missing block: B:78:0x011f, code lost:
        if (r6 != null) goto L50;
     */
    /* JADX WARN: Code restructure failed: missing block: B:79:0x0121, code lost:
        r9 = "";
     */
    /* JADX WARN: Code restructure failed: missing block: B:80:0x0123, code lost:
        r9 = r6.toString();
     */
    /* JADX WARN: Code restructure failed: missing block: B:81:0x0127, code lost:
        if (r7 != null) goto L46;
     */
    /* JADX WARN: Code restructure failed: missing block: B:83:0x012a, code lost:
        r2 = r7.toString();
     */
    /* JADX WARN: Code restructure failed: missing block: B:85:0x0131, code lost:
        return new com.blankj.utilcode.util.ShellUtils.CommandResult(r1, r9, r2);
     */
    /* JADX WARN: Removed duplicated region for block: B:114:0x00d4 A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:120:0x00e0 A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static CommandResult execCmd(String[] commands, boolean isRooted, boolean isNeedResultMsg) {
        int result = -1;
        String str = "";
        if (commands == null || commands.length == 0) {
            return new CommandResult(-1, "", "");
        }
        Process process = null;
        BufferedReader successResult = null;
        BufferedReader errorResult = null;
        StringBuilder successMsg = null;
        StringBuilder errorMsg = null;
        DataOutputStream os = null;
        try {
            try {
                process = Runtime.getRuntime().exec(isRooted ? "su" : "sh");
                os = new DataOutputStream(process.getOutputStream());
                for (String command : commands) {
                    if (command != null) {
                        os.write(command.getBytes());
                        os.writeBytes(LINE_SEP);
                        os.flush();
                    }
                }
                os.writeBytes("exit" + LINE_SEP);
                os.flush();
                result = process.waitFor();
                if (isNeedResultMsg) {
                    successMsg = new StringBuilder();
                    errorMsg = new StringBuilder();
                    successResult = new BufferedReader(new InputStreamReader(process.getInputStream(), "UTF-8"));
                    errorResult = new BufferedReader(new InputStreamReader(process.getErrorStream(), "UTF-8"));
                    String line = successResult.readLine();
                    if (line != null) {
                        successMsg.append(line);
                        while (true) {
                            String line2 = successResult.readLine();
                            if (line2 == null) {
                                break;
                            }
                            successMsg.append(LINE_SEP).append(line2);
                        }
                    }
                    String line3 = errorResult.readLine();
                    if (line3 != null) {
                        errorMsg.append(line3);
                        while (true) {
                            String line4 = errorResult.readLine();
                            if (line4 != null) {
                                errorMsg.append(LINE_SEP).append(line4);
                            }
                        }
                        os.close();
                        if (successResult != null) {
                            try {
                                successResult.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                        if (errorResult != null) {
                            try {
                                errorResult.close();
                            } catch (IOException e2) {
                                e2.printStackTrace();
                            }
                        }
                    }
                }
                os.close();
                if (successResult != null) {
                }
                if (errorResult != null) {
                }
            } catch (Throwable th) {
                if (os != null) {
                    try {
                        os.close();
                    } catch (IOException e3) {
                        e3.printStackTrace();
                    }
                }
                if (successResult != null) {
                    try {
                        successResult.close();
                    } catch (IOException e4) {
                        e4.printStackTrace();
                    }
                }
                if (errorResult != null) {
                    try {
                        errorResult.close();
                    } catch (IOException e5) {
                        e5.printStackTrace();
                    }
                }
                if (process != null) {
                    process.destroy();
                }
                throw th;
            }
        } catch (Exception e6) {
            e6.printStackTrace();
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e7) {
                    e7.printStackTrace();
                }
            }
            if (successResult != null) {
                try {
                    successResult.close();
                } catch (IOException e8) {
                    e8.printStackTrace();
                }
            }
            if (errorResult != null) {
                try {
                    errorResult.close();
                } catch (IOException e9) {
                    e9.printStackTrace();
                }
            }
        }
    }

    /* loaded from: classes.dex */
    public static class CommandResult {
        public String errorMsg;
        public int result;
        public String successMsg;

        public CommandResult(int result, String successMsg, String errorMsg) {
            this.result = result;
            this.successMsg = successMsg;
            this.errorMsg = errorMsg;
        }

        public String toString() {
            return "result: " + this.result + "\nsuccessMsg: " + this.successMsg + "\nerrorMsg: " + this.errorMsg;
        }
    }
}
