package com.blankj.utilcode.util;

import android.content.ContentUris;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.storage.StorageManager;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.text.TextUtils;
import android.util.Log;
import androidx.core.content.FileProvider;
import com.google.android.exoplayer2.text.ttml.TtmlNode;
import com.google.android.exoplayer2.util.MimeTypes;
import java.io.File;
import java.lang.reflect.Array;
import java.lang.reflect.Method;
/* loaded from: classes.dex */
public final class UriUtils {
    private UriUtils() {
        throw new UnsupportedOperationException("u can't instantiate me...");
    }

    public static Uri file2Uri(File file) {
        if (file == null) {
            throw new NullPointerException("Argument 'file' of type File (#0 out of 1, zero-based) is marked by @android.support.annotation.NonNull but got null for it");
        }
        if (Build.VERSION.SDK_INT >= 24) {
            String authority = Utils.getApp().getPackageName() + ".utilcode.provider";
            return FileProvider.getUriForFile(Utils.getApp(), authority, file);
        }
        return Uri.fromFile(file);
    }

    /* JADX WARN: Removed duplicated region for block: B:50:0x01a3  */
    /* JADX WARN: Removed duplicated region for block: B:51:0x01a6 A[Catch: Exception -> 0x0213, TryCatch #4 {Exception -> 0x0213, blocks: (B:41:0x017c, B:43:0x018a, B:51:0x01a6, B:53:0x01b5, B:56:0x01c7, B:58:0x01d2, B:60:0x01d8), top: B:126:0x017c }] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static File uri2File(Uri uri) {
        Uri contentUri;
        Object result;
        boolean mounted;
        Method isEmulated;
        if (uri == null) {
            throw new NullPointerException("Argument 'uri' of type Uri (#0 out of 1, zero-based) is marked by @android.support.annotation.NonNull but got null for it");
        }
        Log.d("UriUtils", uri.toString());
        String authority = uri.getAuthority();
        String scheme = uri.getScheme();
        String path = uri.getPath();
        if (Build.VERSION.SDK_INT >= 24 && path != null) {
            String[] externals = {"/external", "/external_path"};
            for (String external : externals) {
                if (path.startsWith(external + "/")) {
                    File file = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + path.replace(external, ""));
                    if (file.exists()) {
                        Log.d("UriUtils", uri.toString() + " -> " + external);
                        return file;
                    }
                }
            }
        }
        if (!"file".equals(scheme)) {
            if (DocumentsContract.isDocumentUri(Utils.getApp(), uri)) {
                if (!"com.android.externalstorage.documents".equals(authority)) {
                    if ("com.android.providers.downloads.documents".equals(authority)) {
                        String id = DocumentsContract.getDocumentId(uri);
                        if (!TextUtils.isEmpty(id)) {
                            try {
                                Uri contentUri2 = ContentUris.withAppendedId(Uri.parse("content://downloads/public_downloads"), Long.valueOf(id).longValue());
                                return getFileFromUri(contentUri2, "1_1");
                            } catch (NumberFormatException e) {
                                if (id.startsWith("raw:")) {
                                    return new File(id.substring(4));
                                }
                            }
                        }
                        Log.d("UriUtils", uri.toString() + " parse failed. -> 1_1");
                        return null;
                    } else if ("com.android.providers.media.documents".equals(authority)) {
                        String docId = DocumentsContract.getDocumentId(uri);
                        String[] split = docId.split(com.king.zxing.util.LogUtils.COLON);
                        String type = split[0];
                        if (TtmlNode.TAG_IMAGE.equals(type)) {
                            contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
                        } else if (MimeTypes.BASE_TYPE_VIDEO.equals(type)) {
                            contentUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
                        } else if (MimeTypes.BASE_TYPE_AUDIO.equals(type)) {
                            contentUri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
                        } else {
                            Log.d("UriUtils", uri.toString() + " parse failed. -> 1_2");
                            return null;
                        }
                        String[] selectionArgs = {split[1]};
                        return getFileFromUri(contentUri, "_id=?", selectionArgs, "1_2");
                    } else if ("content".equals(scheme)) {
                        return getFileFromUri(uri, "1_3");
                    } else {
                        Log.d("UriUtils", uri.toString() + " parse failed. -> 1_4");
                        return null;
                    }
                }
                String docId2 = DocumentsContract.getDocumentId(uri);
                String[] split2 = docId2.split(com.king.zxing.util.LogUtils.COLON);
                String type2 = split2[0];
                if ("primary".equalsIgnoreCase(type2)) {
                    return new File(Environment.getExternalStorageDirectory() + "/" + split2[1]);
                }
                StorageManager mStorageManager = (StorageManager) Utils.getApp().getSystemService("storage");
                try {
                    Class<?> storageVolumeClazz = Class.forName("android.os.storage.StorageVolume");
                    Method getVolumeList = mStorageManager.getClass().getMethod("getVolumeList", new Class[0]);
                    Method getUuid = storageVolumeClazz.getMethod("getUuid", new Class[0]);
                    Method getState = storageVolumeClazz.getMethod("getState", new Class[0]);
                    try {
                        Method getPath = storageVolumeClazz.getMethod("getPath", new Class[0]);
                        try {
                            Method isPrimary = storageVolumeClazz.getMethod("isPrimary", new Class[0]);
                            Method isEmulated2 = storageVolumeClazz.getMethod("isEmulated", new Class[0]);
                            Object result2 = getVolumeList.invoke(mStorageManager, new Object[0]);
                            int length = Array.getLength(result2);
                            int i = 0;
                            while (true) {
                                Class<?> storageVolumeClazz2 = storageVolumeClazz;
                                int length2 = length;
                                if (i >= length2) {
                                    break;
                                }
                                Object storageVolumeElement = Array.get(result2, i);
                                length = length2;
                                Method getVolumeList2 = getVolumeList;
                                StorageManager mStorageManager2 = mStorageManager;
                                try {
                                    if ("mounted".equals(getState.invoke(storageVolumeElement, new Object[0]))) {
                                        result = result2;
                                    } else {
                                        result = result2;
                                        if (!"mounted_ro".equals(getState.invoke(storageVolumeElement, new Object[0]))) {
                                            mounted = false;
                                            if (mounted) {
                                                isEmulated = isEmulated2;
                                            } else if (((Boolean) isPrimary.invoke(storageVolumeElement, new Object[0])).booleanValue() && ((Boolean) isEmulated2.invoke(storageVolumeElement, new Object[0])).booleanValue()) {
                                                isEmulated = isEmulated2;
                                            } else {
                                                String uuid = (String) getUuid.invoke(storageVolumeElement, new Object[0]);
                                                if (uuid != null && uuid.equals(type2)) {
                                                    return new File(getPath.invoke(storageVolumeElement, new Object[0]) + "/" + split2[1]);
                                                }
                                                isEmulated = isEmulated2;
                                            }
                                            i++;
                                            storageVolumeClazz = storageVolumeClazz2;
                                            result2 = result;
                                            getVolumeList = getVolumeList2;
                                            mStorageManager = mStorageManager2;
                                            isEmulated2 = isEmulated;
                                        }
                                    }
                                    mounted = true;
                                    if (mounted) {
                                    }
                                    i++;
                                    storageVolumeClazz = storageVolumeClazz2;
                                    result2 = result;
                                    getVolumeList = getVolumeList2;
                                    mStorageManager = mStorageManager2;
                                    isEmulated2 = isEmulated;
                                } catch (Exception e2) {
                                    ex = e2;
                                    Log.d("UriUtils", uri.toString() + " parse failed. " + ex.toString() + " -> 1_0");
                                    Log.d("UriUtils", uri.toString() + " parse failed. -> 1_0");
                                    return null;
                                }
                            }
                        } catch (Exception e3) {
                            ex = e3;
                        }
                    } catch (Exception e4) {
                        ex = e4;
                    }
                } catch (Exception e5) {
                    ex = e5;
                }
                Log.d("UriUtils", uri.toString() + " parse failed. -> 1_0");
                return null;
            } else if ("content".equals(scheme)) {
                return getFileFromUri(uri, "2");
            } else {
                Log.d("UriUtils", uri.toString() + " parse failed. -> 3");
                return null;
            }
        } else if (path != null) {
            return new File(path);
        } else {
            Log.d("UriUtils", uri.toString() + " parse failed. -> 0");
            return null;
        }
    }

    private static File getFileFromUri(Uri uri, String code) {
        return getFileFromUri(uri, null, null, code);
    }

    private static File getFileFromUri(Uri uri, String selection, String[] selectionArgs, String code) {
        Cursor cursor = Utils.getApp().getContentResolver().query(uri, new String[]{"_data"}, selection, selectionArgs, null);
        try {
            if (cursor == null) {
                Log.d("UriUtils", uri.toString() + " parse failed(cursor is null). -> " + code);
                return null;
            } else if (!cursor.moveToFirst()) {
                Log.d("UriUtils", uri.toString() + " parse failed(moveToFirst return false). -> " + code);
                return null;
            } else {
                int columnIndex = cursor.getColumnIndex("_data");
                if (columnIndex > -1) {
                    return new File(cursor.getString(columnIndex));
                }
                Log.d("UriUtils", uri.toString() + " parse failed(columnIndex: " + columnIndex + " is wrong). -> " + code);
                return null;
            }
        } catch (Exception e) {
            Log.d("UriUtils", uri.toString() + " parse failed. -> " + code);
            return null;
        } finally {
            cursor.close();
        }
    }
}
