package com.google.android.exoplayer2.extractor.mp4;

import com.coremedia.iso.boxes.sampleentry.VisualSampleEntry;
import com.google.android.exoplayer2.extractor.ExtractorInput;
import com.google.android.exoplayer2.util.ParsableByteArray;
import com.google.android.exoplayer2.util.Util;
import java.io.IOException;
/* loaded from: classes17.dex */
final class Sniffer {
    private static final int[] COMPATIBLE_BRANDS = {Util.getIntegerCodeForString("isom"), Util.getIntegerCodeForString("iso2"), Util.getIntegerCodeForString("iso3"), Util.getIntegerCodeForString("iso4"), Util.getIntegerCodeForString("iso5"), Util.getIntegerCodeForString("iso6"), Util.getIntegerCodeForString(VisualSampleEntry.TYPE3), Util.getIntegerCodeForString(VisualSampleEntry.TYPE6), Util.getIntegerCodeForString(VisualSampleEntry.TYPE7), Util.getIntegerCodeForString("mp41"), Util.getIntegerCodeForString("mp42"), Util.getIntegerCodeForString("3g2a"), Util.getIntegerCodeForString("3g2b"), Util.getIntegerCodeForString("3gr6"), Util.getIntegerCodeForString("3gs6"), Util.getIntegerCodeForString("3ge6"), Util.getIntegerCodeForString("3gg6"), Util.getIntegerCodeForString("M4V "), Util.getIntegerCodeForString("M4A "), Util.getIntegerCodeForString("f4v "), Util.getIntegerCodeForString("kddi"), Util.getIntegerCodeForString("M4VP"), Util.getIntegerCodeForString("qt  "), Util.getIntegerCodeForString("MSNV")};
    private static final int SEARCH_LENGTH = 4096;

    public static boolean sniffFragmented(ExtractorInput input) throws IOException, InterruptedException {
        return sniffInternal(input, true);
    }

    public static boolean sniffUnfragmented(ExtractorInput input) throws IOException, InterruptedException {
        return sniffInternal(input, false);
    }

    /* JADX WARN: Code restructure failed: missing block: B:67:0x00e7, code lost:
        r9 = true;
     */
    /* JADX WARN: Code restructure failed: missing block: B:69:0x00ea, code lost:
        if (r8 == false) goto L64;
     */
    /* JADX WARN: Code restructure failed: missing block: B:71:0x00ee, code lost:
        if (r23 != r9) goto L63;
     */
    /* JADX WARN: Code restructure failed: missing block: B:72:0x00f0, code lost:
        return true;
     */
    /* JADX WARN: Code restructure failed: missing block: B:74:0x00f4, code lost:
        return r10;
     */
    /* JADX WARN: Code restructure failed: missing block: B:96:?, code lost:
        return r10;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private static boolean sniffInternal(ExtractorInput input, boolean fragmented) throws IOException, InterruptedException {
        boolean z;
        long inputLength = input.getLength();
        long j = 4096;
        long j2 = -1;
        if (inputLength != -1 && inputLength <= 4096) {
            j = inputLength;
        }
        int bytesToSearch = (int) j;
        ParsableByteArray buffer = new ParsableByteArray(64);
        int bytesSearched = 0;
        boolean foundGoodFileType = false;
        boolean isFragmented = false;
        while (true) {
            if (bytesSearched >= bytesToSearch) {
                z = false;
                break;
            }
            int headerSize = 8;
            buffer.reset(8);
            input.peekFully(buffer.data, 0, 8);
            long atomSize = buffer.readUnsignedInt();
            int atomType = buffer.readInt();
            if (atomSize == 1) {
                headerSize = 16;
                input.peekFully(buffer.data, 8, 8);
                buffer.setLimit(16);
                atomSize = buffer.readLong();
            } else if (atomSize == 0) {
                long fileEndPosition = input.getLength();
                if (fileEndPosition != j2) {
                    atomSize = (fileEndPosition - input.getPeekPosition()) + 8;
                }
            }
            if ((inputLength != j2 && bytesSearched + atomSize > inputLength) || atomSize < headerSize) {
                return false;
            }
            bytesSearched += headerSize;
            if (atomType != Atom.TYPE_moov) {
                if (atomType == Atom.TYPE_moof) {
                    z = false;
                    break;
                } else if (atomType == Atom.TYPE_mvex) {
                    z = false;
                    break;
                } else if ((bytesSearched + atomSize) - headerSize >= bytesToSearch) {
                    z = false;
                    break;
                } else {
                    int atomDataSize = (int) (atomSize - headerSize);
                    bytesSearched += atomDataSize;
                    if (atomType == Atom.TYPE_ftyp) {
                        if (atomDataSize >= 8) {
                            buffer.reset(atomDataSize);
                            input.peekFully(buffer.data, 0, atomDataSize);
                            int brandsCount = atomDataSize / 4;
                            int i = 0;
                            while (true) {
                                if (i >= brandsCount) {
                                    break;
                                }
                                if (i == 1) {
                                    buffer.skipBytes(4);
                                } else if (isCompatibleBrand(buffer.readInt())) {
                                    foundGoodFileType = true;
                                    break;
                                }
                                i++;
                            }
                            if (!foundGoodFileType) {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    } else if (atomDataSize != 0) {
                        input.advancePeekPosition(atomDataSize);
                    }
                    j2 = -1;
                }
            } else {
                bytesToSearch += (int) atomSize;
                j2 = -1;
                if (inputLength != -1 && bytesToSearch > inputLength) {
                    bytesToSearch = (int) inputLength;
                }
            }
        }
    }

    private static boolean isCompatibleBrand(int brand) {
        int[] iArr;
        if ((brand >>> 8) == Util.getIntegerCodeForString("3gp")) {
            return true;
        }
        for (int compatibleBrand : COMPATIBLE_BRANDS) {
            if (compatibleBrand == brand) {
                return true;
            }
        }
        return false;
    }

    private Sniffer() {
    }
}
