package com.google.android.exoplayer2.source.dash.manifest;

import android.net.Uri;
import android.text.TextUtils;
import android.util.Base64;
import android.util.Pair;
import android.util.Xml;
import androidx.exifinterface.media.ExifInterface;
import com.google.android.exoplayer2.C;
import com.google.android.exoplayer2.Format;
import com.google.android.exoplayer2.ParserException;
import com.google.android.exoplayer2.drm.DrmInitData;
import com.google.android.exoplayer2.extractor.mp4.PsshAtomUtil;
import com.google.android.exoplayer2.metadata.emsg.EventMessage;
import com.google.android.exoplayer2.source.dash.manifest.SegmentBase;
import com.google.android.exoplayer2.text.ttml.TtmlNode;
import com.google.android.exoplayer2.upstream.ParsingLoadable;
import com.google.android.exoplayer2.util.Assertions;
import com.google.android.exoplayer2.util.Log;
import com.google.android.exoplayer2.util.MimeTypes;
import com.google.android.exoplayer2.util.UriUtil;
import com.google.android.exoplayer2.util.Util;
import com.google.android.exoplayer2.util.XmlPullParserUtil;
import com.mp4parser.iso14496.part30.WebVTTSampleEntry;
import com.mp4parser.iso14496.part30.XMLSubtitleSampleEntry;
import com.mp4parser.iso23001.part7.ProtectionSystemSpecificHeaderBox;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.aspectj.lang.JoinPoint;
import org.xml.sax.helpers.DefaultHandler;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;
import org.xmlpull.v1.XmlSerializer;
/* loaded from: classes14.dex */
public class DashManifestParser extends DefaultHandler implements ParsingLoadable.Parser<DashManifest> {
    private static final String TAG = "MpdParser";
    private final XmlPullParserFactory xmlParserFactory;
    private static final Pattern FRAME_RATE_PATTERN = Pattern.compile("(\\d+)(?:/(\\d+))?");
    private static final Pattern CEA_608_ACCESSIBILITY_PATTERN = Pattern.compile("CC([1-4])=.*");
    private static final Pattern CEA_708_ACCESSIBILITY_PATTERN = Pattern.compile("([1-9]|[1-5][0-9]|6[0-3])=.*");

    public DashManifestParser() {
        try {
            this.xmlParserFactory = XmlPullParserFactory.newInstance();
        } catch (XmlPullParserException e) {
            throw new RuntimeException("Couldn't create XmlPullParserFactory instance", e);
        }
    }

    /* JADX WARN: Can't rename method to resolve collision */
    @Override // com.google.android.exoplayer2.upstream.ParsingLoadable.Parser
    public DashManifest parse(Uri uri, InputStream inputStream) throws IOException {
        try {
            XmlPullParser xpp = this.xmlParserFactory.newPullParser();
            xpp.setInput(inputStream, null);
            int eventType = xpp.next();
            if (eventType != 2 || !"MPD".equals(xpp.getName())) {
                throw new ParserException("inputStream does not contain a valid media presentation description");
            }
            return parseMediaPresentationDescription(xpp, uri.toString());
        } catch (XmlPullParserException e) {
            throw new ParserException(e);
        }
    }

    /* JADX WARN: Removed duplicated region for block: B:69:0x01ac  */
    /* JADX WARN: Removed duplicated region for block: B:71:0x01c7  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    protected DashManifest parseMediaPresentationDescription(XmlPullParser xpp, String baseUrl) throws XmlPullParserException, IOException {
        long minUpdateTimeMs;
        long nextPeriodStartMs;
        String baseUrl2;
        ProgramInformation programInformation;
        String baseUrl3;
        ProgramInformation programInformation2;
        UtcTimingElement utcTiming;
        Uri location;
        boolean seenEarlyAccessPeriod;
        boolean seenFirstBaseUrl;
        String baseUrl4;
        long durationMs;
        long availabilityStartTime = parseDateTime(xpp, "availabilityStartTime", C.TIME_UNSET);
        long durationMs2 = parseDuration(xpp, "mediaPresentationDuration", C.TIME_UNSET);
        long minBufferTimeMs = parseDuration(xpp, "minBufferTime", C.TIME_UNSET);
        String baseUrl5 = xpp.getAttributeValue(null, "type");
        boolean dynamic = baseUrl5 != null && "dynamic".equals(baseUrl5);
        if (dynamic) {
            minUpdateTimeMs = parseDuration(xpp, "minimumUpdatePeriod", C.TIME_UNSET);
        } else {
            minUpdateTimeMs = -9223372036854775807L;
        }
        long timeShiftBufferDepthMs = dynamic ? parseDuration(xpp, "timeShiftBufferDepth", C.TIME_UNSET) : -9223372036854775807L;
        long suggestedPresentationDelayMs = dynamic ? parseDuration(xpp, "suggestedPresentationDelay", C.TIME_UNSET) : -9223372036854775807L;
        long publishTimeMs = parseDateTime(xpp, "publishTime", C.TIME_UNSET);
        List<Period> periods = new ArrayList<>();
        long nextPeriodStartMs2 = dynamic ? -9223372036854775807L : 0L;
        long nextPeriodStartMs3 = nextPeriodStartMs2;
        boolean seenEarlyAccessPeriod2 = false;
        boolean seenFirstBaseUrl2 = false;
        Uri location2 = null;
        UtcTimingElement utcTiming2 = null;
        ProgramInformation programInformation3 = null;
        String baseUrl6 = baseUrl;
        while (true) {
            xpp.next();
            String typeString = baseUrl5;
            if (!XmlPullParserUtil.isStartTag(xpp, "BaseURL")) {
                if (!XmlPullParserUtil.isStartTag(xpp, "ProgramInformation")) {
                    if (!XmlPullParserUtil.isStartTag(xpp, "UTCTiming")) {
                        if (!XmlPullParserUtil.isStartTag(xpp, "Location")) {
                            if (XmlPullParserUtil.isStartTag(xpp, "Period") && !seenEarlyAccessPeriod2) {
                                ProgramInformation programInformation4 = programInformation3;
                                Pair<Period, Long> periodWithDurationMs = parsePeriod(xpp, baseUrl6, nextPeriodStartMs3);
                                Period period = (Period) periodWithDurationMs.first;
                                long nextPeriodStartMs4 = nextPeriodStartMs3;
                                long nextPeriodStartMs5 = period.startMs;
                                if (nextPeriodStartMs5 == C.TIME_UNSET) {
                                    if (dynamic) {
                                        seenEarlyAccessPeriod2 = true;
                                        baseUrl3 = baseUrl6;
                                        nextPeriodStartMs3 = nextPeriodStartMs4;
                                    } else {
                                        throw new ParserException("Unable to determine start of period " + periods.size());
                                    }
                                } else {
                                    baseUrl3 = baseUrl6;
                                    long periodDurationMs = ((Long) periodWithDurationMs.second).longValue();
                                    long nextPeriodStartMs6 = periodDurationMs == C.TIME_UNSET ? C.TIME_UNSET : period.startMs + periodDurationMs;
                                    periods.add(period);
                                    nextPeriodStartMs3 = nextPeriodStartMs6;
                                }
                                programInformation2 = programInformation4;
                                utcTiming = utcTiming2;
                                location = location2;
                                seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
                                seenFirstBaseUrl = seenFirstBaseUrl2;
                                baseUrl4 = baseUrl3;
                            } else {
                                nextPeriodStartMs = nextPeriodStartMs3;
                                baseUrl2 = baseUrl6;
                                programInformation = programInformation3;
                                maybeSkipTag(xpp);
                                programInformation2 = programInformation;
                                utcTiming = utcTiming2;
                                location = location2;
                                seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
                                seenFirstBaseUrl = seenFirstBaseUrl2;
                                nextPeriodStartMs3 = nextPeriodStartMs;
                                baseUrl4 = baseUrl2;
                            }
                        } else {
                            location = Uri.parse(xpp.nextText());
                            baseUrl4 = baseUrl6;
                            programInformation2 = programInformation3;
                            utcTiming = utcTiming2;
                            seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
                            seenFirstBaseUrl = seenFirstBaseUrl2;
                        }
                    } else {
                        utcTiming = parseUtcTiming(xpp);
                        baseUrl4 = baseUrl6;
                        programInformation2 = programInformation3;
                        location = location2;
                        seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
                        seenFirstBaseUrl = seenFirstBaseUrl2;
                    }
                } else {
                    programInformation2 = parseProgramInformation(xpp);
                    baseUrl4 = baseUrl6;
                    utcTiming = utcTiming2;
                    location = location2;
                    seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
                    seenFirstBaseUrl = seenFirstBaseUrl2;
                }
            } else if (seenFirstBaseUrl2) {
                nextPeriodStartMs = nextPeriodStartMs3;
                baseUrl2 = baseUrl6;
                programInformation = programInformation3;
                programInformation2 = programInformation;
                utcTiming = utcTiming2;
                location = location2;
                seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
                nextPeriodStartMs3 = nextPeriodStartMs;
                baseUrl4 = baseUrl2;
            } else {
                baseUrl4 = parseBaseUrl(xpp, baseUrl6);
                seenFirstBaseUrl = true;
                programInformation2 = programInformation3;
                utcTiming = utcTiming2;
                location = location2;
                seenEarlyAccessPeriod = seenEarlyAccessPeriod2;
            }
            if (XmlPullParserUtil.isEndTag(xpp, "MPD")) {
                if (durationMs2 == C.TIME_UNSET) {
                    if (nextPeriodStartMs3 != C.TIME_UNSET) {
                        durationMs = nextPeriodStartMs3;
                        if (!periods.isEmpty()) {
                            throw new ParserException("No periods found.");
                        }
                        return buildMediaPresentationDescription(availabilityStartTime, durationMs, minBufferTimeMs, dynamic, minUpdateTimeMs, timeShiftBufferDepthMs, suggestedPresentationDelayMs, publishTimeMs, programInformation2, utcTiming, location, periods);
                    } else if (!dynamic) {
                        throw new ParserException("Unable to determine duration of static manifest.");
                    }
                }
                durationMs = durationMs2;
                if (!periods.isEmpty()) {
                }
            } else {
                baseUrl6 = baseUrl4;
                baseUrl5 = typeString;
                programInformation3 = programInformation2;
                utcTiming2 = utcTiming;
                location2 = location;
                seenEarlyAccessPeriod2 = seenEarlyAccessPeriod;
                seenFirstBaseUrl2 = seenFirstBaseUrl;
            }
        }
    }

    protected DashManifest buildMediaPresentationDescription(long availabilityStartTime, long durationMs, long minBufferTimeMs, boolean dynamic, long minUpdateTimeMs, long timeShiftBufferDepthMs, long suggestedPresentationDelayMs, long publishTimeMs, ProgramInformation programInformation, UtcTimingElement utcTiming, Uri location, List<Period> periods) {
        return new DashManifest(availabilityStartTime, durationMs, minBufferTimeMs, dynamic, minUpdateTimeMs, timeShiftBufferDepthMs, suggestedPresentationDelayMs, publishTimeMs, programInformation, utcTiming, location, periods);
    }

    protected UtcTimingElement parseUtcTiming(XmlPullParser xpp) {
        String schemeIdUri = xpp.getAttributeValue(null, "schemeIdUri");
        String value = xpp.getAttributeValue(null, "value");
        return buildUtcTimingElement(schemeIdUri, value);
    }

    protected UtcTimingElement buildUtcTimingElement(String schemeIdUri, String value) {
        return new UtcTimingElement(schemeIdUri, value);
    }

    protected Pair<Period, Long> parsePeriod(XmlPullParser xpp, String baseUrl, long defaultStartMs) throws XmlPullParserException, IOException {
        String baseUrl2;
        SegmentBase segmentBase;
        boolean seenFirstBaseUrl;
        String id = xpp.getAttributeValue(null, TtmlNode.ATTR_ID);
        long startMs = parseDuration(xpp, TtmlNode.START, defaultStartMs);
        long durationMs = parseDuration(xpp, "duration", C.TIME_UNSET);
        List<AdaptationSet> adaptationSets = new ArrayList<>();
        List<EventStream> eventStreams = new ArrayList<>();
        boolean seenFirstBaseUrl2 = false;
        SegmentBase segmentBase2 = null;
        String baseUrl3 = baseUrl;
        while (true) {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, "BaseURL")) {
                if (!seenFirstBaseUrl2) {
                    baseUrl2 = parseBaseUrl(xpp, baseUrl3);
                    segmentBase = segmentBase2;
                    seenFirstBaseUrl = true;
                }
                baseUrl2 = baseUrl3;
                segmentBase = segmentBase2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            } else {
                if (XmlPullParserUtil.isStartTag(xpp, "AdaptationSet")) {
                    adaptationSets.add(parseAdaptationSet(xpp, baseUrl3, segmentBase2));
                } else if (XmlPullParserUtil.isStartTag(xpp, "EventStream")) {
                    eventStreams.add(parseEventStream(xpp));
                } else if (XmlPullParserUtil.isStartTag(xpp, "SegmentBase")) {
                    SegmentBase segmentBase3 = parseSegmentBase(xpp, null);
                    baseUrl2 = baseUrl3;
                    segmentBase = segmentBase3;
                    seenFirstBaseUrl = seenFirstBaseUrl2;
                } else if (XmlPullParserUtil.isStartTag(xpp, "SegmentList")) {
                    SegmentBase segmentBase4 = parseSegmentList(xpp, null);
                    baseUrl2 = baseUrl3;
                    segmentBase = segmentBase4;
                    seenFirstBaseUrl = seenFirstBaseUrl2;
                } else if (XmlPullParserUtil.isStartTag(xpp, "SegmentTemplate")) {
                    SegmentBase segmentBase5 = parseSegmentTemplate(xpp, null);
                    baseUrl2 = baseUrl3;
                    segmentBase = segmentBase5;
                    seenFirstBaseUrl = seenFirstBaseUrl2;
                } else {
                    maybeSkipTag(xpp);
                }
                baseUrl2 = baseUrl3;
                segmentBase = segmentBase2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            }
            if (XmlPullParserUtil.isEndTag(xpp, "Period")) {
                return Pair.create(buildPeriod(id, startMs, adaptationSets, eventStreams), Long.valueOf(durationMs));
            }
            baseUrl3 = baseUrl2;
            segmentBase2 = segmentBase;
            seenFirstBaseUrl2 = seenFirstBaseUrl;
        }
    }

    protected Period buildPeriod(String id, long startMs, List<AdaptationSet> adaptationSets, List<EventStream> eventStreams) {
        return new Period(id, startMs, adaptationSets, eventStreams);
    }

    protected AdaptationSet parseAdaptationSet(XmlPullParser xpp, String baseUrl, SegmentBase segmentBase) throws XmlPullParserException, IOException {
        String language;
        int contentType;
        String baseUrl2;
        List<RepresentationInfo> representationInfos;
        ArrayList<Descriptor> supplementalProperties;
        ArrayList<Descriptor> accessibilityDescriptors;
        ArrayList<DrmInitData.SchemeData> drmSchemeDatas;
        String str;
        String str2;
        XmlPullParser xmlPullParser;
        ArrayList<Descriptor> supplementalProperties2;
        int contentType2;
        String drmSchemeType;
        XmlPullParser xmlPullParser2 = xpp;
        int id = parseInt(xmlPullParser2, TtmlNode.ATTR_ID, -1);
        int contentType3 = parseContentType(xpp);
        String str3 = null;
        String mimeType = xmlPullParser2.getAttributeValue(null, "mimeType");
        String codecs = xmlPullParser2.getAttributeValue(null, "codecs");
        int width = parseInt(xmlPullParser2, "width", -1);
        int height = parseInt(xmlPullParser2, "height", -1);
        float frameRate = parseFrameRate(xmlPullParser2, -1.0f);
        int audioSamplingRate = parseInt(xmlPullParser2, "audioSamplingRate", -1);
        String str4 = "lang";
        String language2 = xmlPullParser2.getAttributeValue(null, "lang");
        String label = xmlPullParser2.getAttributeValue(null, "label");
        ArrayList<DrmInitData.SchemeData> drmSchemeDatas2 = new ArrayList<>();
        ArrayList<Descriptor> inbandEventStreams = new ArrayList<>();
        ArrayList<Descriptor> accessibilityDescriptors2 = new ArrayList<>();
        ArrayList<Descriptor> inbandEventStreams2 = new ArrayList<>();
        List<RepresentationInfo> representationInfos2 = new ArrayList<>();
        String baseUrl3 = baseUrl;
        SegmentBase segmentBase2 = segmentBase;
        int audioChannels = -1;
        String drmSchemeType2 = null;
        int selectionFlags = 0;
        boolean seenFirstBaseUrl = false;
        int contentType4 = contentType3;
        String language3 = language2;
        while (true) {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xmlPullParser2, "BaseURL")) {
                if (seenFirstBaseUrl) {
                    language = language3;
                    contentType = contentType4;
                    baseUrl2 = baseUrl3;
                    representationInfos = representationInfos2;
                    supplementalProperties = inbandEventStreams2;
                    accessibilityDescriptors = accessibilityDescriptors2;
                    supplementalProperties2 = inbandEventStreams;
                    drmSchemeDatas = drmSchemeDatas2;
                    str = str4;
                    str2 = str3;
                    xmlPullParser = xmlPullParser2;
                    contentType2 = contentType;
                    drmSchemeType = drmSchemeType2;
                } else {
                    baseUrl2 = parseBaseUrl(xmlPullParser2, baseUrl3);
                    seenFirstBaseUrl = true;
                    language = language3;
                    representationInfos = representationInfos2;
                    supplementalProperties = inbandEventStreams2;
                    accessibilityDescriptors = accessibilityDescriptors2;
                    supplementalProperties2 = inbandEventStreams;
                    drmSchemeDatas = drmSchemeDatas2;
                    str = str4;
                    str2 = str3;
                    xmlPullParser = xmlPullParser2;
                    drmSchemeType = drmSchemeType2;
                    contentType2 = contentType4;
                }
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser2, "ContentProtection")) {
                Pair<String, DrmInitData.SchemeData> contentProtection = parseContentProtection(xpp);
                if (contentProtection.first != null) {
                    drmSchemeType2 = (String) contentProtection.first;
                }
                if (contentProtection.second != null) {
                    drmSchemeDatas2.add((DrmInitData.SchemeData) contentProtection.second);
                }
                language = language3;
                baseUrl2 = baseUrl3;
                representationInfos = representationInfos2;
                supplementalProperties = inbandEventStreams2;
                accessibilityDescriptors = accessibilityDescriptors2;
                supplementalProperties2 = inbandEventStreams;
                drmSchemeDatas = drmSchemeDatas2;
                str = str4;
                str2 = str3;
                xmlPullParser = xmlPullParser2;
                drmSchemeType = drmSchemeType2;
                contentType2 = contentType4;
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser2, "ContentComponent")) {
                language = checkLanguageConsistency(language3, xmlPullParser2.getAttributeValue(str3, str4));
                baseUrl2 = baseUrl3;
                representationInfos = representationInfos2;
                supplementalProperties = inbandEventStreams2;
                accessibilityDescriptors = accessibilityDescriptors2;
                supplementalProperties2 = inbandEventStreams;
                drmSchemeDatas = drmSchemeDatas2;
                str = str4;
                str2 = str3;
                xmlPullParser = xmlPullParser2;
                drmSchemeType = drmSchemeType2;
                contentType2 = checkContentTypeConsistency(contentType4, parseContentType(xpp));
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser2, "Role")) {
                selectionFlags |= parseRole(xpp);
                language = language3;
                baseUrl2 = baseUrl3;
                representationInfos = representationInfos2;
                supplementalProperties = inbandEventStreams2;
                accessibilityDescriptors = accessibilityDescriptors2;
                supplementalProperties2 = inbandEventStreams;
                drmSchemeDatas = drmSchemeDatas2;
                str = str4;
                str2 = str3;
                xmlPullParser = xmlPullParser2;
                drmSchemeType = drmSchemeType2;
                contentType2 = contentType4;
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser2, "AudioChannelConfiguration")) {
                audioChannels = parseAudioChannelConfiguration(xpp);
                language = language3;
                baseUrl2 = baseUrl3;
                representationInfos = representationInfos2;
                supplementalProperties = inbandEventStreams2;
                accessibilityDescriptors = accessibilityDescriptors2;
                supplementalProperties2 = inbandEventStreams;
                drmSchemeDatas = drmSchemeDatas2;
                str = str4;
                str2 = str3;
                xmlPullParser = xmlPullParser2;
                drmSchemeType = drmSchemeType2;
                contentType2 = contentType4;
            } else {
                if (XmlPullParserUtil.isStartTag(xmlPullParser2, "Accessibility")) {
                    accessibilityDescriptors2.add(parseDescriptor(xmlPullParser2, "Accessibility"));
                    language = language3;
                    contentType = contentType4;
                    baseUrl2 = baseUrl3;
                    representationInfos = representationInfos2;
                    supplementalProperties = inbandEventStreams2;
                    accessibilityDescriptors = accessibilityDescriptors2;
                    supplementalProperties2 = inbandEventStreams;
                    drmSchemeDatas = drmSchemeDatas2;
                    str = str4;
                    str2 = str3;
                    xmlPullParser = xmlPullParser2;
                } else if (XmlPullParserUtil.isStartTag(xmlPullParser2, "SupplementalProperty")) {
                    inbandEventStreams2.add(parseDescriptor(xmlPullParser2, "SupplementalProperty"));
                    language = language3;
                    contentType = contentType4;
                    baseUrl2 = baseUrl3;
                    representationInfos = representationInfos2;
                    supplementalProperties = inbandEventStreams2;
                    accessibilityDescriptors = accessibilityDescriptors2;
                    supplementalProperties2 = inbandEventStreams;
                    drmSchemeDatas = drmSchemeDatas2;
                    str = str4;
                    str2 = str3;
                    xmlPullParser = xmlPullParser2;
                } else if (XmlPullParserUtil.isStartTag(xmlPullParser2, "Representation")) {
                    language = language3;
                    baseUrl2 = baseUrl3;
                    supplementalProperties = inbandEventStreams2;
                    accessibilityDescriptors = accessibilityDescriptors2;
                    ArrayList<Descriptor> inbandEventStreams3 = inbandEventStreams;
                    drmSchemeDatas = drmSchemeDatas2;
                    str = str4;
                    str2 = str3;
                    RepresentationInfo representationInfo = parseRepresentation(xpp, baseUrl3, label, mimeType, codecs, width, height, frameRate, audioChannels, audioSamplingRate, language, selectionFlags, accessibilityDescriptors, segmentBase2);
                    int contentType5 = checkContentTypeConsistency(contentType4, getContentType(representationInfo.format));
                    representationInfos = representationInfos2;
                    representationInfos.add(representationInfo);
                    xmlPullParser = xpp;
                    contentType2 = contentType5;
                    drmSchemeType = drmSchemeType2;
                    supplementalProperties2 = inbandEventStreams3;
                } else {
                    language = language3;
                    contentType = contentType4;
                    baseUrl2 = baseUrl3;
                    representationInfos = representationInfos2;
                    supplementalProperties = inbandEventStreams2;
                    accessibilityDescriptors = accessibilityDescriptors2;
                    ArrayList<Descriptor> inbandEventStreams4 = inbandEventStreams;
                    drmSchemeDatas = drmSchemeDatas2;
                    str = str4;
                    str2 = str3;
                    xmlPullParser = xpp;
                    if (XmlPullParserUtil.isStartTag(xmlPullParser, "SegmentBase")) {
                        segmentBase2 = parseSegmentBase(xmlPullParser, (SegmentBase.SingleSegmentBase) segmentBase2);
                        contentType2 = contentType;
                        drmSchemeType = drmSchemeType2;
                        supplementalProperties2 = inbandEventStreams4;
                    } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "SegmentList")) {
                        segmentBase2 = parseSegmentList(xmlPullParser, (SegmentBase.SegmentList) segmentBase2);
                        contentType2 = contentType;
                        drmSchemeType = drmSchemeType2;
                        supplementalProperties2 = inbandEventStreams4;
                    } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "SegmentTemplate")) {
                        segmentBase2 = parseSegmentTemplate(xmlPullParser, (SegmentBase.SegmentTemplate) segmentBase2);
                        contentType2 = contentType;
                        drmSchemeType = drmSchemeType2;
                        supplementalProperties2 = inbandEventStreams4;
                    } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "InbandEventStream")) {
                        supplementalProperties2 = inbandEventStreams4;
                        supplementalProperties2.add(parseDescriptor(xmlPullParser, "InbandEventStream"));
                    } else {
                        supplementalProperties2 = inbandEventStreams4;
                        if (XmlPullParserUtil.isStartTag(xpp)) {
                            parseAdaptationSetChild(xpp);
                        }
                    }
                }
                contentType2 = contentType;
                drmSchemeType = drmSchemeType2;
            }
            if (XmlPullParserUtil.isEndTag(xmlPullParser, "AdaptationSet")) {
                break;
            }
            xmlPullParser2 = xmlPullParser;
            contentType4 = contentType2;
            drmSchemeType2 = drmSchemeType;
            drmSchemeDatas2 = drmSchemeDatas;
            language3 = language;
            accessibilityDescriptors2 = accessibilityDescriptors;
            str4 = str;
            str3 = str2;
            representationInfos2 = representationInfos;
            inbandEventStreams = supplementalProperties2;
            baseUrl3 = baseUrl2;
            inbandEventStreams2 = supplementalProperties;
        }
        List<Representation> representations = new ArrayList<>(representationInfos.size());
        for (int i = 0; i < representationInfos.size(); i++) {
            representations.add(buildRepresentation(representationInfos.get(i), drmSchemeType, drmSchemeDatas, supplementalProperties2));
        }
        return buildAdaptationSet(id, contentType2, representations, accessibilityDescriptors, supplementalProperties);
    }

    protected AdaptationSet buildAdaptationSet(int id, int contentType, List<Representation> representations, List<Descriptor> accessibilityDescriptors, List<Descriptor> supplementalProperties) {
        return new AdaptationSet(id, contentType, representations, accessibilityDescriptors, supplementalProperties);
    }

    protected int parseContentType(XmlPullParser xpp) {
        String contentType = xpp.getAttributeValue(null, "contentType");
        if (TextUtils.isEmpty(contentType)) {
            return -1;
        }
        if (MimeTypes.BASE_TYPE_AUDIO.equals(contentType)) {
            return 1;
        }
        if (MimeTypes.BASE_TYPE_VIDEO.equals(contentType)) {
            return 2;
        }
        return "text".equals(contentType) ? 3 : -1;
    }

    protected int getContentType(Format format) {
        String sampleMimeType = format.sampleMimeType;
        if (TextUtils.isEmpty(sampleMimeType)) {
            return -1;
        }
        if (MimeTypes.isVideo(sampleMimeType)) {
            return 2;
        }
        if (MimeTypes.isAudio(sampleMimeType)) {
            return 1;
        }
        return mimeTypeIsRawText(sampleMimeType) ? 3 : -1;
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    protected Pair<String, DrmInitData.SchemeData> parseContentProtection(XmlPullParser xpp) throws XmlPullParserException, IOException {
        char c;
        String schemeType = null;
        String licenseServerUrl = null;
        byte[] data = null;
        UUID uuid = null;
        boolean requiresSecureDecoder = false;
        String schemeIdUri = xpp.getAttributeValue(null, "schemeIdUri");
        if (schemeIdUri != null) {
            String lowerInvariant = Util.toLowerInvariant(schemeIdUri);
            switch (lowerInvariant.hashCode()) {
                case 489446379:
                    if (lowerInvariant.equals("urn:uuid:9a04f079-9840-4286-ab92-e65be0885f95")) {
                        c = 1;
                        break;
                    }
                    c = 65535;
                    break;
                case 755418770:
                    if (lowerInvariant.equals("urn:uuid:edef8ba9-79d6-4ace-a3c8-27dcd51d21ed")) {
                        c = 2;
                        break;
                    }
                    c = 65535;
                    break;
                case 1812765994:
                    if (lowerInvariant.equals("urn:mpeg:dash:mp4protection:2011")) {
                        c = 0;
                        break;
                    }
                    c = 65535;
                    break;
                default:
                    c = 65535;
                    break;
            }
            switch (c) {
                case 0:
                    schemeType = xpp.getAttributeValue(null, "value");
                    String defaultKid = XmlPullParserUtil.getAttributeValueIgnorePrefix(xpp, "default_KID");
                    if (!TextUtils.isEmpty(defaultKid) && !"00000000-0000-0000-0000-000000000000".equals(defaultKid)) {
                        String[] defaultKidStrings = defaultKid.split("\\s+");
                        UUID[] defaultKids = new UUID[defaultKidStrings.length];
                        for (int i = 0; i < defaultKidStrings.length; i++) {
                            defaultKids[i] = UUID.fromString(defaultKidStrings[i]);
                        }
                        data = PsshAtomUtil.buildPsshAtom(C.COMMON_PSSH_UUID, defaultKids, null);
                        uuid = C.COMMON_PSSH_UUID;
                        break;
                    }
                    break;
                case 1:
                    uuid = C.PLAYREADY_UUID;
                    break;
                case 2:
                    uuid = C.WIDEVINE_UUID;
                    break;
            }
        }
        do {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, "ms:laurl")) {
                licenseServerUrl = xpp.getAttributeValue(null, "licenseUrl");
            } else if (XmlPullParserUtil.isStartTag(xpp, "widevine:license")) {
                String robustnessLevel = xpp.getAttributeValue(null, "robustness_level");
                requiresSecureDecoder = robustnessLevel != null && robustnessLevel.startsWith("HW");
            } else if (data == null && XmlPullParserUtil.isStartTagIgnorePrefix(xpp, ProtectionSystemSpecificHeaderBox.TYPE) && xpp.next() == 4) {
                data = Base64.decode(xpp.getText(), 0);
                uuid = PsshAtomUtil.parseUuid(data);
                if (uuid == null) {
                    Log.w(TAG, "Skipping malformed cenc:pssh data");
                    data = null;
                }
            } else if (data == null && C.PLAYREADY_UUID.equals(uuid) && XmlPullParserUtil.isStartTag(xpp, "mspr:pro") && xpp.next() == 4) {
                data = PsshAtomUtil.buildPsshAtom(C.PLAYREADY_UUID, Base64.decode(xpp.getText(), 0));
            } else {
                maybeSkipTag(xpp);
            }
        } while (!XmlPullParserUtil.isEndTag(xpp, "ContentProtection"));
        DrmInitData.SchemeData schemeData = uuid != null ? new DrmInitData.SchemeData(uuid, licenseServerUrl, MimeTypes.VIDEO_MP4, data, requiresSecureDecoder) : null;
        return Pair.create(schemeType, schemeData);
    }

    protected int parseRole(XmlPullParser xpp) throws XmlPullParserException, IOException {
        String schemeIdUri = parseString(xpp, "schemeIdUri", null);
        String value = parseString(xpp, "value", null);
        do {
            xpp.next();
        } while (!XmlPullParserUtil.isEndTag(xpp, "Role"));
        return ("urn:mpeg:dash:role:2011".equals(schemeIdUri) && "main".equals(value)) ? 1 : 0;
    }

    protected void parseAdaptationSetChild(XmlPullParser xpp) throws XmlPullParserException, IOException {
        maybeSkipTag(xpp);
    }

    protected RepresentationInfo parseRepresentation(XmlPullParser xpp, String baseUrl, String label, String adaptationSetMimeType, String adaptationSetCodecs, int adaptationSetWidth, int adaptationSetHeight, float adaptationSetFrameRate, int adaptationSetAudioChannels, int adaptationSetAudioSamplingRate, String adaptationSetLanguage, int adaptationSetSelectionFlags, List<Descriptor> adaptationSetAccessibilityDescriptors, SegmentBase segmentBase) throws XmlPullParserException, IOException {
        String baseUrl2;
        int audioChannels;
        String baseUrl3;
        SegmentBase segmentBase2;
        String drmSchemeType;
        boolean seenFirstBaseUrl;
        XmlPullParser xmlPullParser = xpp;
        String id = xmlPullParser.getAttributeValue(null, TtmlNode.ATTR_ID);
        int bandwidth = parseInt(xmlPullParser, "bandwidth", -1);
        String mimeType = parseString(xmlPullParser, "mimeType", adaptationSetMimeType);
        String codecs = parseString(xmlPullParser, "codecs", adaptationSetCodecs);
        int width = parseInt(xmlPullParser, "width", adaptationSetWidth);
        int height = parseInt(xmlPullParser, "height", adaptationSetHeight);
        float frameRate = parseFrameRate(xmlPullParser, adaptationSetFrameRate);
        int audioSamplingRate = parseInt(xmlPullParser, "audioSamplingRate", adaptationSetAudioSamplingRate);
        ArrayList<DrmInitData.SchemeData> drmSchemeDatas = new ArrayList<>();
        ArrayList<Descriptor> inbandEventStreams = new ArrayList<>();
        ArrayList<Descriptor> supplementalProperties = new ArrayList<>();
        String drmSchemeType2 = null;
        boolean seenFirstBaseUrl2 = false;
        SegmentBase segmentBase3 = segmentBase;
        int audioChannels2 = adaptationSetAudioChannels;
        String baseUrl4 = baseUrl;
        while (true) {
            xpp.next();
            int audioChannels3 = audioChannels2;
            if (XmlPullParserUtil.isStartTag(xmlPullParser, "BaseURL")) {
                if (seenFirstBaseUrl2) {
                    baseUrl2 = baseUrl4;
                    audioChannels = audioChannels3;
                    baseUrl3 = baseUrl2;
                    segmentBase2 = segmentBase3;
                    drmSchemeType = drmSchemeType2;
                    seenFirstBaseUrl = seenFirstBaseUrl2;
                } else {
                    audioChannels = audioChannels3;
                    baseUrl3 = parseBaseUrl(xmlPullParser, baseUrl4);
                    segmentBase2 = segmentBase3;
                    seenFirstBaseUrl = true;
                    drmSchemeType = drmSchemeType2;
                }
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "AudioChannelConfiguration")) {
                baseUrl3 = baseUrl4;
                segmentBase2 = segmentBase3;
                audioChannels = parseAudioChannelConfiguration(xpp);
                drmSchemeType = drmSchemeType2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "SegmentBase")) {
                audioChannels = audioChannels3;
                baseUrl3 = baseUrl4;
                segmentBase2 = parseSegmentBase(xmlPullParser, (SegmentBase.SingleSegmentBase) segmentBase3);
                drmSchemeType = drmSchemeType2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "SegmentList")) {
                audioChannels = audioChannels3;
                baseUrl3 = baseUrl4;
                segmentBase2 = parseSegmentList(xmlPullParser, (SegmentBase.SegmentList) segmentBase3);
                drmSchemeType = drmSchemeType2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "SegmentTemplate")) {
                audioChannels = audioChannels3;
                baseUrl3 = baseUrl4;
                segmentBase2 = parseSegmentTemplate(xmlPullParser, (SegmentBase.SegmentTemplate) segmentBase3);
                drmSchemeType = drmSchemeType2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "ContentProtection")) {
                Pair<String, DrmInitData.SchemeData> contentProtection = parseContentProtection(xpp);
                String baseUrl5 = baseUrl4;
                if (contentProtection.first != null) {
                    drmSchemeType2 = (String) contentProtection.first;
                }
                if (contentProtection.second != null) {
                    drmSchemeDatas.add((DrmInitData.SchemeData) contentProtection.second);
                }
                audioChannels = audioChannels3;
                baseUrl3 = baseUrl5;
                segmentBase2 = segmentBase3;
                drmSchemeType = drmSchemeType2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            } else {
                baseUrl2 = baseUrl4;
                if (XmlPullParserUtil.isStartTag(xmlPullParser, "InbandEventStream")) {
                    inbandEventStreams.add(parseDescriptor(xmlPullParser, "InbandEventStream"));
                } else if (XmlPullParserUtil.isStartTag(xmlPullParser, "SupplementalProperty")) {
                    supplementalProperties.add(parseDescriptor(xmlPullParser, "SupplementalProperty"));
                } else {
                    maybeSkipTag(xpp);
                }
                audioChannels = audioChannels3;
                baseUrl3 = baseUrl2;
                segmentBase2 = segmentBase3;
                drmSchemeType = drmSchemeType2;
                seenFirstBaseUrl = seenFirstBaseUrl2;
            }
            if (XmlPullParserUtil.isEndTag(xmlPullParser, "Representation")) {
                break;
            }
            xmlPullParser = xpp;
            baseUrl4 = baseUrl3;
            segmentBase3 = segmentBase2;
            audioChannels2 = audioChannels;
            drmSchemeType2 = drmSchemeType;
            seenFirstBaseUrl2 = seenFirstBaseUrl;
        }
        Format format = buildFormat(id, label, mimeType, width, height, frameRate, audioChannels, audioSamplingRate, bandwidth, adaptationSetLanguage, adaptationSetSelectionFlags, adaptationSetAccessibilityDescriptors, codecs, supplementalProperties);
        return new RepresentationInfo(format, baseUrl3, segmentBase2 != null ? segmentBase2 : new SegmentBase.SingleSegmentBase(), drmSchemeType, drmSchemeDatas, inbandEventStreams, -1L);
    }

    protected Format buildFormat(String id, String label, String containerMimeType, int width, int height, float frameRate, int audioChannels, int audioSamplingRate, int bitrate, String language, int selectionFlags, List<Descriptor> accessibilityDescriptors, String codecs, List<Descriptor> supplementalProperties) {
        String sampleMimeType;
        int accessibilityChannel;
        String sampleMimeType2 = getSampleMimeType(containerMimeType, codecs);
        if (sampleMimeType2 == null) {
            sampleMimeType = sampleMimeType2;
        } else {
            if (!MimeTypes.AUDIO_E_AC3.equals(sampleMimeType2)) {
                sampleMimeType = sampleMimeType2;
            } else {
                sampleMimeType = parseEac3SupplementalProperties(supplementalProperties);
            }
            if (MimeTypes.isVideo(sampleMimeType)) {
                return Format.createVideoContainerFormat(id, label, containerMimeType, sampleMimeType, codecs, bitrate, width, height, frameRate, null, selectionFlags);
            }
            if (MimeTypes.isAudio(sampleMimeType)) {
                return Format.createAudioContainerFormat(id, label, containerMimeType, sampleMimeType, codecs, bitrate, audioChannels, audioSamplingRate, null, selectionFlags, language);
            }
            if (mimeTypeIsRawText(sampleMimeType)) {
                if (MimeTypes.APPLICATION_CEA608.equals(sampleMimeType)) {
                    accessibilityChannel = parseCea608AccessibilityChannel(accessibilityDescriptors);
                } else if (MimeTypes.APPLICATION_CEA708.equals(sampleMimeType)) {
                    accessibilityChannel = parseCea708AccessibilityChannel(accessibilityDescriptors);
                } else {
                    accessibilityChannel = -1;
                }
                return Format.createTextContainerFormat(id, label, containerMimeType, sampleMimeType, codecs, bitrate, selectionFlags, language, accessibilityChannel);
            }
        }
        return Format.createContainerFormat(id, label, containerMimeType, sampleMimeType, codecs, bitrate, selectionFlags, language);
    }

    protected Representation buildRepresentation(RepresentationInfo representationInfo, String extraDrmSchemeType, ArrayList<DrmInitData.SchemeData> extraDrmSchemeDatas, ArrayList<Descriptor> extraInbandEventStreams) {
        Format format = representationInfo.format;
        String drmSchemeType = representationInfo.drmSchemeType != null ? representationInfo.drmSchemeType : extraDrmSchemeType;
        ArrayList<DrmInitData.SchemeData> drmSchemeDatas = representationInfo.drmSchemeDatas;
        drmSchemeDatas.addAll(extraDrmSchemeDatas);
        if (!drmSchemeDatas.isEmpty()) {
            filterRedundantIncompleteSchemeDatas(drmSchemeDatas);
            DrmInitData drmInitData = new DrmInitData(drmSchemeType, drmSchemeDatas);
            format = format.copyWithDrmInitData(drmInitData);
        }
        ArrayList<Descriptor> inbandEventStreams = representationInfo.inbandEventStreams;
        inbandEventStreams.addAll(extraInbandEventStreams);
        return Representation.newInstance(representationInfo.revisionId, format, representationInfo.baseUrl, representationInfo.segmentBase, inbandEventStreams);
    }

    protected SegmentBase.SingleSegmentBase parseSegmentBase(XmlPullParser xpp, SegmentBase.SingleSegmentBase parent) throws XmlPullParserException, IOException {
        long indexLength;
        long indexStart;
        long timescale = parseLong(xpp, "timescale", parent != null ? parent.timescale : 1L);
        long presentationTimeOffset = parseLong(xpp, "presentationTimeOffset", parent != null ? parent.presentationTimeOffset : 0L);
        long indexStart2 = parent != null ? parent.indexStart : 0L;
        long indexLength2 = parent != null ? parent.indexLength : 0L;
        String indexRangeText = xpp.getAttributeValue(null, "indexRange");
        if (indexRangeText == null) {
            indexLength = indexLength2;
            indexStart = indexStart2;
        } else {
            String[] indexRange = indexRangeText.split("-");
            long indexStart3 = Long.parseLong(indexRange[0]);
            long indexLength3 = (Long.parseLong(indexRange[1]) - indexStart3) + 1;
            indexLength = indexLength3;
            indexStart = indexStart3;
        }
        RangedUri initialization = parent != null ? parent.initialization : null;
        while (true) {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, "Initialization")) {
                initialization = parseInitialization(xpp);
            } else {
                maybeSkipTag(xpp);
            }
            if (XmlPullParserUtil.isEndTag(xpp, "SegmentBase")) {
                return buildSingleSegmentBase(initialization, timescale, presentationTimeOffset, indexStart, indexLength);
            }
        }
    }

    protected SegmentBase.SingleSegmentBase buildSingleSegmentBase(RangedUri initialization, long timescale, long presentationTimeOffset, long indexStart, long indexLength) {
        return new SegmentBase.SingleSegmentBase(initialization, timescale, presentationTimeOffset, indexStart, indexLength);
    }

    protected SegmentBase.SegmentList parseSegmentList(XmlPullParser xpp, SegmentBase.SegmentList parent) throws XmlPullParserException, IOException {
        List<SegmentBase.SegmentTimelineElement> timeline;
        List<RangedUri> segments;
        long timescale = parseLong(xpp, "timescale", parent != null ? parent.timescale : 1L);
        long presentationTimeOffset = parseLong(xpp, "presentationTimeOffset", parent != null ? parent.presentationTimeOffset : 0L);
        long duration = parseLong(xpp, "duration", parent != null ? parent.duration : C.TIME_UNSET);
        long startNumber = parseLong(xpp, "startNumber", parent != null ? parent.startNumber : 1L);
        RangedUri initialization = null;
        List<SegmentBase.SegmentTimelineElement> timeline2 = null;
        List<RangedUri> segments2 = null;
        do {
            xpp.next();
            if (!XmlPullParserUtil.isStartTag(xpp, "Initialization")) {
                if (!XmlPullParserUtil.isStartTag(xpp, "SegmentTimeline")) {
                    if (XmlPullParserUtil.isStartTag(xpp, "SegmentURL")) {
                        if (segments2 == null) {
                            segments2 = new ArrayList<>();
                        }
                        segments2.add(parseSegmentUrl(xpp));
                    } else {
                        maybeSkipTag(xpp);
                    }
                } else {
                    timeline2 = parseSegmentTimeline(xpp);
                }
            } else {
                initialization = parseInitialization(xpp);
            }
        } while (!XmlPullParserUtil.isEndTag(xpp, "SegmentList"));
        if (parent == null) {
            timeline = timeline2;
            segments = segments2;
        } else {
            initialization = initialization != null ? initialization : parent.initialization;
            timeline = timeline2 != null ? timeline2 : parent.segmentTimeline;
            segments = segments2 != null ? segments2 : parent.mediaSegments;
        }
        return buildSegmentList(initialization, timescale, presentationTimeOffset, startNumber, duration, timeline, segments);
    }

    protected SegmentBase.SegmentList buildSegmentList(RangedUri initialization, long timescale, long presentationTimeOffset, long startNumber, long duration, List<SegmentBase.SegmentTimelineElement> timeline, List<RangedUri> segments) {
        return new SegmentBase.SegmentList(initialization, timescale, presentationTimeOffset, startNumber, duration, timeline, segments);
    }

    protected SegmentBase.SegmentTemplate parseSegmentTemplate(XmlPullParser xpp, SegmentBase.SegmentTemplate parent) throws XmlPullParserException, IOException {
        RangedUri initialization;
        List<SegmentBase.SegmentTimelineElement> timeline;
        long timescale = parseLong(xpp, "timescale", parent != null ? parent.timescale : 1L);
        long presentationTimeOffset = parseLong(xpp, "presentationTimeOffset", parent != null ? parent.presentationTimeOffset : 0L);
        long duration = parseLong(xpp, "duration", parent != null ? parent.duration : C.TIME_UNSET);
        long startNumber = parseLong(xpp, "startNumber", parent != null ? parent.startNumber : 1L);
        UrlTemplate mediaTemplate = parseUrlTemplate(xpp, "media", parent != null ? parent.mediaTemplate : null);
        UrlTemplate initializationTemplate = parseUrlTemplate(xpp, JoinPoint.INITIALIZATION, parent != null ? parent.initializationTemplate : null);
        RangedUri initialization2 = null;
        List<SegmentBase.SegmentTimelineElement> timeline2 = null;
        do {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, "Initialization")) {
                initialization2 = parseInitialization(xpp);
            } else if (XmlPullParserUtil.isStartTag(xpp, "SegmentTimeline")) {
                timeline2 = parseSegmentTimeline(xpp);
            } else {
                maybeSkipTag(xpp);
            }
        } while (!XmlPullParserUtil.isEndTag(xpp, "SegmentTemplate"));
        if (parent == null) {
            initialization = initialization2;
            timeline = timeline2;
        } else {
            initialization = initialization2 != null ? initialization2 : parent.initialization;
            timeline = timeline2 != null ? timeline2 : parent.segmentTimeline;
        }
        return buildSegmentTemplate(initialization, timescale, presentationTimeOffset, startNumber, duration, timeline, initializationTemplate, mediaTemplate);
    }

    protected SegmentBase.SegmentTemplate buildSegmentTemplate(RangedUri initialization, long timescale, long presentationTimeOffset, long startNumber, long duration, List<SegmentBase.SegmentTimelineElement> timeline, UrlTemplate initializationTemplate, UrlTemplate mediaTemplate) {
        return new SegmentBase.SegmentTemplate(initialization, timescale, presentationTimeOffset, startNumber, duration, timeline, initializationTemplate, mediaTemplate);
    }

    protected EventStream parseEventStream(XmlPullParser xpp) throws XmlPullParserException, IOException {
        String schemeIdUri = parseString(xpp, "schemeIdUri", "");
        String value = parseString(xpp, "value", "");
        long timescale = parseLong(xpp, "timescale", 1L);
        List<EventMessage> eventMessages = new ArrayList<>();
        ByteArrayOutputStream scratchOutputStream = new ByteArrayOutputStream(512);
        while (true) {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, "Event")) {
                eventMessages.add(parseEvent(xpp, schemeIdUri, value, timescale, scratchOutputStream));
            } else {
                maybeSkipTag(xpp);
            }
            if (XmlPullParserUtil.isEndTag(xpp, "EventStream")) {
                break;
            }
        }
        long[] presentationTimesUs = new long[eventMessages.size()];
        EventMessage[] events = new EventMessage[eventMessages.size()];
        for (int i = 0; i < eventMessages.size(); i++) {
            EventMessage event = eventMessages.get(i);
            presentationTimesUs[i] = event.presentationTimeUs;
            events[i] = event;
        }
        return buildEventStream(schemeIdUri, value, timescale, presentationTimesUs, events);
    }

    protected EventStream buildEventStream(String schemeIdUri, String value, long timescale, long[] presentationTimesUs, EventMessage[] events) {
        return new EventStream(schemeIdUri, value, timescale, presentationTimesUs, events);
    }

    protected EventMessage parseEvent(XmlPullParser xpp, String schemeIdUri, String value, long timescale, ByteArrayOutputStream scratchOutputStream) throws IOException, XmlPullParserException {
        long id = parseLong(xpp, TtmlNode.ATTR_ID, 0L);
        long duration = parseLong(xpp, "duration", C.TIME_UNSET);
        long presentationTime = parseLong(xpp, "presentationTime", 0L);
        long durationMs = Util.scaleLargeTimestamp(duration, 1000L, timescale);
        long presentationTimesUs = Util.scaleLargeTimestamp(presentationTime, 1000000L, timescale);
        String messageData = parseString(xpp, "messageData", null);
        byte[] eventObject = parseEventObject(xpp, scratchOutputStream);
        return buildEvent(schemeIdUri, value, id, durationMs, messageData == null ? eventObject : Util.getUtf8Bytes(messageData), presentationTimesUs);
    }

    protected byte[] parseEventObject(XmlPullParser xpp, ByteArrayOutputStream scratchOutputStream) throws XmlPullParserException, IOException {
        scratchOutputStream.reset();
        XmlSerializer xmlSerializer = Xml.newSerializer();
        xmlSerializer.setOutput(scratchOutputStream, "UTF-8");
        xpp.nextToken();
        while (!XmlPullParserUtil.isEndTag(xpp, "Event")) {
            switch (xpp.getEventType()) {
                case 0:
                    xmlSerializer.startDocument(null, false);
                    break;
                case 1:
                    xmlSerializer.endDocument();
                    break;
                case 2:
                    xmlSerializer.startTag(xpp.getNamespace(), xpp.getName());
                    for (int i = 0; i < xpp.getAttributeCount(); i++) {
                        xmlSerializer.attribute(xpp.getAttributeNamespace(i), xpp.getAttributeName(i), xpp.getAttributeValue(i));
                    }
                    break;
                case 3:
                    xmlSerializer.endTag(xpp.getNamespace(), xpp.getName());
                    break;
                case 4:
                    xmlSerializer.text(xpp.getText());
                    break;
                case 5:
                    xmlSerializer.cdsect(xpp.getText());
                    break;
                case 6:
                    xmlSerializer.entityRef(xpp.getText());
                    break;
                case 7:
                    xmlSerializer.ignorableWhitespace(xpp.getText());
                    break;
                case 8:
                    xmlSerializer.processingInstruction(xpp.getText());
                    break;
                case 9:
                    xmlSerializer.comment(xpp.getText());
                    break;
                case 10:
                    xmlSerializer.docdecl(xpp.getText());
                    break;
            }
            xpp.nextToken();
        }
        xmlSerializer.flush();
        return scratchOutputStream.toByteArray();
    }

    protected EventMessage buildEvent(String schemeIdUri, String value, long id, long durationMs, byte[] messageData, long presentationTimeUs) {
        return new EventMessage(schemeIdUri, value, durationMs, id, messageData, presentationTimeUs);
    }

    protected List<SegmentBase.SegmentTimelineElement> parseSegmentTimeline(XmlPullParser xpp) throws XmlPullParserException, IOException {
        List<SegmentBase.SegmentTimelineElement> segmentTimeline = new ArrayList<>();
        long elapsedTime = 0;
        do {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, ExifInterface.LATITUDE_SOUTH)) {
                elapsedTime = parseLong(xpp, "t", elapsedTime);
                long duration = parseLong(xpp, "d", C.TIME_UNSET);
                int count = parseInt(xpp, "r", 0) + 1;
                for (int i = 0; i < count; i++) {
                    segmentTimeline.add(buildSegmentTimelineElement(elapsedTime, duration));
                    elapsedTime += duration;
                }
            } else {
                maybeSkipTag(xpp);
            }
        } while (!XmlPullParserUtil.isEndTag(xpp, "SegmentTimeline"));
        return segmentTimeline;
    }

    protected SegmentBase.SegmentTimelineElement buildSegmentTimelineElement(long elapsedTime, long duration) {
        return new SegmentBase.SegmentTimelineElement(elapsedTime, duration);
    }

    protected UrlTemplate parseUrlTemplate(XmlPullParser xpp, String name, UrlTemplate defaultValue) {
        String valueString = xpp.getAttributeValue(null, name);
        if (valueString != null) {
            return UrlTemplate.compile(valueString);
        }
        return defaultValue;
    }

    protected RangedUri parseInitialization(XmlPullParser xpp) {
        return parseRangedUrl(xpp, "sourceURL", "range");
    }

    protected RangedUri parseSegmentUrl(XmlPullParser xpp) {
        return parseRangedUrl(xpp, "media", "mediaRange");
    }

    protected RangedUri parseRangedUrl(XmlPullParser xpp, String urlAttribute, String rangeAttribute) {
        long rangeStart;
        long rangeLength;
        String urlText = xpp.getAttributeValue(null, urlAttribute);
        String rangeText = xpp.getAttributeValue(null, rangeAttribute);
        if (rangeText == null) {
            rangeStart = 0;
            rangeLength = -1;
        } else {
            String[] rangeTextArray = rangeText.split("-");
            long rangeStart2 = Long.parseLong(rangeTextArray[0]);
            if (rangeTextArray.length != 2) {
                rangeStart = rangeStart2;
                rangeLength = -1;
            } else {
                long rangeLength2 = (Long.parseLong(rangeTextArray[1]) - rangeStart2) + 1;
                rangeStart = rangeStart2;
                rangeLength = rangeLength2;
            }
        }
        return buildRangedUri(urlText, rangeStart, rangeLength);
    }

    protected RangedUri buildRangedUri(String urlText, long rangeStart, long rangeLength) {
        return new RangedUri(urlText, rangeStart, rangeLength);
    }

    protected ProgramInformation parseProgramInformation(XmlPullParser xpp) throws IOException, XmlPullParserException {
        String title = null;
        String source = null;
        String copyright = null;
        String moreInformationURL = parseString(xpp, "moreInformationURL", null);
        String lang = parseString(xpp, "lang", null);
        do {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp, "Title")) {
                title = xpp.nextText();
            } else if (XmlPullParserUtil.isStartTag(xpp, "Source")) {
                source = xpp.nextText();
            } else if (XmlPullParserUtil.isStartTag(xpp, ExifInterface.TAG_COPYRIGHT)) {
                copyright = xpp.nextText();
            } else {
                maybeSkipTag(xpp);
            }
        } while (!XmlPullParserUtil.isEndTag(xpp, "ProgramInformation"));
        return new ProgramInformation(title, source, copyright, moreInformationURL, lang);
    }

    protected int parseAudioChannelConfiguration(XmlPullParser xpp) throws XmlPullParserException, IOException {
        String schemeIdUri = parseString(xpp, "schemeIdUri", null);
        int i = -1;
        if ("urn:mpeg:dash:23003:3:audio_channel_configuration:2011".equals(schemeIdUri)) {
            i = parseInt(xpp, "value", -1);
        } else if ("tag:dolby.com,2014:dash:audio_channel_configuration:2011".equals(schemeIdUri)) {
            i = parseDolbyChannelConfiguration(xpp);
        }
        int audioChannels = i;
        do {
            xpp.next();
        } while (!XmlPullParserUtil.isEndTag(xpp, "AudioChannelConfiguration"));
        return audioChannels;
    }

    public static void maybeSkipTag(XmlPullParser xpp) throws IOException, XmlPullParserException {
        if (!XmlPullParserUtil.isStartTag(xpp)) {
            return;
        }
        int depth = 1;
        while (depth != 0) {
            xpp.next();
            if (XmlPullParserUtil.isStartTag(xpp)) {
                depth++;
            } else if (XmlPullParserUtil.isEndTag(xpp)) {
                depth--;
            }
        }
    }

    private static void filterRedundantIncompleteSchemeDatas(ArrayList<DrmInitData.SchemeData> schemeDatas) {
        for (int i = schemeDatas.size() - 1; i >= 0; i--) {
            DrmInitData.SchemeData schemeData = schemeDatas.get(i);
            if (!schemeData.hasData()) {
                int j = 0;
                while (true) {
                    if (j < schemeDatas.size()) {
                        if (!schemeDatas.get(j).canReplace(schemeData)) {
                            j++;
                        } else {
                            schemeDatas.remove(i);
                            break;
                        }
                    } else {
                        break;
                    }
                }
            }
        }
    }

    private static String getSampleMimeType(String containerMimeType, String codecs) {
        if (MimeTypes.isAudio(containerMimeType)) {
            return MimeTypes.getAudioMediaMimeType(codecs);
        }
        if (MimeTypes.isVideo(containerMimeType)) {
            return MimeTypes.getVideoMediaMimeType(codecs);
        }
        if (mimeTypeIsRawText(containerMimeType)) {
            return containerMimeType;
        }
        if (MimeTypes.APPLICATION_MP4.equals(containerMimeType)) {
            if (codecs != null) {
                if (codecs.startsWith(XMLSubtitleSampleEntry.TYPE)) {
                    return MimeTypes.APPLICATION_TTML;
                }
                if (codecs.startsWith(WebVTTSampleEntry.TYPE)) {
                    return MimeTypes.APPLICATION_MP4VTT;
                }
            }
        } else {
            if (MimeTypes.APPLICATION_RAWCC.equals(containerMimeType) && codecs != null) {
                if (codecs.contains("cea708")) {
                    return MimeTypes.APPLICATION_CEA708;
                }
                if (codecs.contains("eia608") || codecs.contains("cea608")) {
                    return MimeTypes.APPLICATION_CEA608;
                }
            }
            return null;
        }
        return null;
    }

    private static boolean mimeTypeIsRawText(String mimeType) {
        return MimeTypes.isText(mimeType) || MimeTypes.APPLICATION_TTML.equals(mimeType) || MimeTypes.APPLICATION_MP4VTT.equals(mimeType) || MimeTypes.APPLICATION_CEA708.equals(mimeType) || MimeTypes.APPLICATION_CEA608.equals(mimeType);
    }

    private static String checkLanguageConsistency(String firstLanguage, String secondLanguage) {
        if (firstLanguage == null) {
            return secondLanguage;
        }
        if (secondLanguage == null) {
            return firstLanguage;
        }
        Assertions.checkState(firstLanguage.equals(secondLanguage));
        return firstLanguage;
    }

    private static int checkContentTypeConsistency(int firstType, int secondType) {
        if (firstType == -1) {
            return secondType;
        }
        if (secondType == -1) {
            return firstType;
        }
        Assertions.checkState(firstType == secondType);
        return firstType;
    }

    protected static Descriptor parseDescriptor(XmlPullParser xpp, String tag) throws XmlPullParserException, IOException {
        String schemeIdUri = parseString(xpp, "schemeIdUri", "");
        String value = parseString(xpp, "value", null);
        String id = parseString(xpp, TtmlNode.ATTR_ID, null);
        do {
            xpp.next();
        } while (!XmlPullParserUtil.isEndTag(xpp, tag));
        return new Descriptor(schemeIdUri, value, id);
    }

    protected static int parseCea608AccessibilityChannel(List<Descriptor> accessibilityDescriptors) {
        for (int i = 0; i < accessibilityDescriptors.size(); i++) {
            Descriptor descriptor = accessibilityDescriptors.get(i);
            if ("urn:scte:dash:cc:cea-608:2015".equals(descriptor.schemeIdUri) && descriptor.value != null) {
                Matcher accessibilityValueMatcher = CEA_608_ACCESSIBILITY_PATTERN.matcher(descriptor.value);
                if (accessibilityValueMatcher.matches()) {
                    return Integer.parseInt(accessibilityValueMatcher.group(1));
                }
                Log.w(TAG, "Unable to parse CEA-608 channel number from: " + descriptor.value);
            }
        }
        return -1;
    }

    protected static int parseCea708AccessibilityChannel(List<Descriptor> accessibilityDescriptors) {
        for (int i = 0; i < accessibilityDescriptors.size(); i++) {
            Descriptor descriptor = accessibilityDescriptors.get(i);
            if ("urn:scte:dash:cc:cea-708:2015".equals(descriptor.schemeIdUri) && descriptor.value != null) {
                Matcher accessibilityValueMatcher = CEA_708_ACCESSIBILITY_PATTERN.matcher(descriptor.value);
                if (accessibilityValueMatcher.matches()) {
                    return Integer.parseInt(accessibilityValueMatcher.group(1));
                }
                Log.w(TAG, "Unable to parse CEA-708 service block number from: " + descriptor.value);
            }
        }
        return -1;
    }

    protected static String parseEac3SupplementalProperties(List<Descriptor> supplementalProperties) {
        for (int i = 0; i < supplementalProperties.size(); i++) {
            Descriptor descriptor = supplementalProperties.get(i);
            String schemeIdUri = descriptor.schemeIdUri;
            if ("tag:dolby.com,2014:dash:DolbyDigitalPlusExtensionType:2014".equals(schemeIdUri) && "ec+3".equals(descriptor.value)) {
                return MimeTypes.AUDIO_E_AC3_JOC;
            }
        }
        return MimeTypes.AUDIO_E_AC3;
    }

    protected static float parseFrameRate(XmlPullParser xpp, float defaultValue) {
        String frameRateAttribute = xpp.getAttributeValue(null, "frameRate");
        if (frameRateAttribute == null) {
            return defaultValue;
        }
        Matcher frameRateMatcher = FRAME_RATE_PATTERN.matcher(frameRateAttribute);
        if (!frameRateMatcher.matches()) {
            return defaultValue;
        }
        int numerator = Integer.parseInt(frameRateMatcher.group(1));
        String denominatorString = frameRateMatcher.group(2);
        if (!TextUtils.isEmpty(denominatorString)) {
            float frameRate = numerator / Integer.parseInt(denominatorString);
            return frameRate;
        }
        float frameRate2 = numerator;
        return frameRate2;
    }

    protected static long parseDuration(XmlPullParser xpp, String name, long defaultValue) {
        String value = xpp.getAttributeValue(null, name);
        if (value == null) {
            return defaultValue;
        }
        return Util.parseXsDuration(value);
    }

    protected static long parseDateTime(XmlPullParser xpp, String name, long defaultValue) throws ParserException {
        String value = xpp.getAttributeValue(null, name);
        if (value == null) {
            return defaultValue;
        }
        return Util.parseXsDateTime(value);
    }

    protected static String parseBaseUrl(XmlPullParser xpp, String parentBaseUrl) throws XmlPullParserException, IOException {
        xpp.next();
        return UriUtil.resolve(parentBaseUrl, xpp.getText());
    }

    protected static int parseInt(XmlPullParser xpp, String name, int defaultValue) {
        String value = xpp.getAttributeValue(null, name);
        return value == null ? defaultValue : Integer.parseInt(value);
    }

    protected static long parseLong(XmlPullParser xpp, String name, long defaultValue) {
        String value = xpp.getAttributeValue(null, name);
        return value == null ? defaultValue : Long.parseLong(value);
    }

    protected static String parseString(XmlPullParser xpp, String name, String defaultValue) {
        String value = xpp.getAttributeValue(null, name);
        return value == null ? defaultValue : value;
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    protected static int parseDolbyChannelConfiguration(XmlPullParser xpp) {
        char c;
        String value = Util.toLowerInvariant(xpp.getAttributeValue(null, "value"));
        if (value == null) {
            return -1;
        }
        switch (value.hashCode()) {
            case 1596796:
                if (value.equals("4000")) {
                    c = 0;
                    break;
                }
                c = 65535;
                break;
            case 2937391:
                if (value.equals("a000")) {
                    c = 1;
                    break;
                }
                c = 65535;
                break;
            case 3094035:
                if (value.equals("f801")) {
                    c = 2;
                    break;
                }
                c = 65535;
                break;
            case 3133436:
                if (value.equals("fa01")) {
                    c = 3;
                    break;
                }
                c = 65535;
                break;
            default:
                c = 65535;
                break;
        }
        switch (c) {
            case 0:
                return 1;
            case 1:
                return 2;
            case 2:
                return 6;
            case 3:
                return 8;
            default:
                return -1;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* loaded from: classes14.dex */
    public static final class RepresentationInfo {
        public final String baseUrl;
        public final ArrayList<DrmInitData.SchemeData> drmSchemeDatas;
        public final String drmSchemeType;
        public final Format format;
        public final ArrayList<Descriptor> inbandEventStreams;
        public final long revisionId;
        public final SegmentBase segmentBase;

        public RepresentationInfo(Format format, String baseUrl, SegmentBase segmentBase, String drmSchemeType, ArrayList<DrmInitData.SchemeData> drmSchemeDatas, ArrayList<Descriptor> inbandEventStreams, long revisionId) {
            this.format = format;
            this.baseUrl = baseUrl;
            this.segmentBase = segmentBase;
            this.drmSchemeType = drmSchemeType;
            this.drmSchemeDatas = drmSchemeDatas;
            this.inbandEventStreams = inbandEventStreams;
            this.revisionId = revisionId;
        }
    }
}
