package com.google.android.gms.vision.text;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Rect;
import android.graphics.YuvImage;
import android.util.SparseArray;
import com.google.android.gms.internal.vision.zzad;
import com.google.android.gms.internal.vision.zzae;
import com.google.android.gms.internal.vision.zzm;
import com.google.android.gms.internal.vision.zzo;
import com.google.android.gms.internal.vision.zzx;
import com.google.android.gms.internal.vision.zzz;
import com.google.android.gms.vision.Detector;
import com.google.android.gms.vision.Frame;
import java.io.ByteArrayOutputStream;
import java.nio.ByteBuffer;
/* loaded from: classes.dex */
public final class TextRecognizer extends Detector<TextBlock> {
    private final zzad zzdf;

    /* loaded from: classes.dex */
    public static class Builder {
        private zzae zzdg = new zzae();
        private Context zze;

        public Builder(Context context) {
            this.zze = context;
        }

        public TextRecognizer build() {
            return new TextRecognizer(new zzad(this.zze, this.zzdg));
        }
    }

    private TextRecognizer() {
        throw new IllegalStateException("Default constructor called");
    }

    private TextRecognizer(zzad zzadVar) {
        this.zzdf = zzadVar;
    }

    @Override // com.google.android.gms.vision.Detector
    public final SparseArray<TextBlock> detect(Frame frame) {
        byte[] bArr;
        Bitmap decodeByteArray;
        zzz zzzVar = new zzz(new Rect());
        if (frame != null) {
            zzm zzc = zzm.zzc(frame);
            if (frame.getBitmap() != null) {
                decodeByteArray = frame.getBitmap();
            } else {
                Frame.Metadata metadata = frame.getMetadata();
                ByteBuffer grayscaleImageData = frame.getGrayscaleImageData();
                int format = metadata.getFormat();
                int i = zzc.width;
                int i2 = zzc.height;
                if (grayscaleImageData.hasArray() && grayscaleImageData.arrayOffset() == 0) {
                    bArr = grayscaleImageData.array();
                } else {
                    byte[] bArr2 = new byte[grayscaleImageData.capacity()];
                    grayscaleImageData.get(bArr2);
                    bArr = bArr2;
                }
                ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                new YuvImage(bArr, format, i, i2, null).compressToJpeg(new Rect(0, 0, i, i2), 100, byteArrayOutputStream);
                byte[] byteArray = byteArrayOutputStream.toByteArray();
                decodeByteArray = BitmapFactory.decodeByteArray(byteArray, 0, byteArray.length);
            }
            Bitmap zzb = zzo.zzb(decodeByteArray, zzc);
            if (!zzzVar.zzdr.isEmpty()) {
                Rect rect = zzzVar.zzdr;
                int width = frame.getMetadata().getWidth();
                int height = frame.getMetadata().getHeight();
                switch (zzc.rotation) {
                    case 1:
                        rect = new Rect(height - rect.bottom, rect.left, height - rect.top, rect.right);
                        break;
                    case 2:
                        rect = new Rect(width - rect.right, height - rect.bottom, width - rect.left, height - rect.top);
                        break;
                    case 3:
                        rect = new Rect(rect.top, width - rect.right, rect.bottom, width - rect.left);
                        break;
                }
                zzzVar.zzdr.set(rect);
            }
            zzc.rotation = 0;
            zzx[] zza = this.zzdf.zza(zzb, zzc, zzzVar);
            SparseArray sparseArray = new SparseArray();
            for (zzx zzxVar : zza) {
                SparseArray sparseArray2 = (SparseArray) sparseArray.get(zzxVar.zzdp);
                if (sparseArray2 == null) {
                    sparseArray2 = new SparseArray();
                    sparseArray.append(zzxVar.zzdp, sparseArray2);
                }
                sparseArray2.append(zzxVar.zzdq, zzxVar);
            }
            SparseArray<TextBlock> sparseArray3 = new SparseArray<>(sparseArray.size());
            for (int i3 = 0; i3 < sparseArray.size(); i3++) {
                sparseArray3.append(sparseArray.keyAt(i3), new TextBlock((SparseArray) sparseArray.valueAt(i3)));
            }
            return sparseArray3;
        }
        throw new IllegalArgumentException("No frame supplied.");
    }

    @Override // com.google.android.gms.vision.Detector
    public final boolean isOperational() {
        return this.zzdf.isOperational();
    }

    @Override // com.google.android.gms.vision.Detector
    public final void release() {
        super.release();
        this.zzdf.zzo();
    }
}
