package com.google.firebase.messaging;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.graphics.Color;
import android.graphics.drawable.AdaptiveIconDrawable;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Log;
import androidx.core.app.NotificationCompat;
import androidx.core.content.ContextCompat;
import com.google.android.exoplayer2.C;
import java.util.concurrent.atomic.AtomicInteger;
/* compiled from: com.google.firebase:firebase-messaging@@20.1.0 */
/* loaded from: classes19.dex */
public final class zzb {
    private static final AtomicInteger zza = new AtomicInteger((int) SystemClock.elapsedRealtime());

    /* JADX INFO: Access modifiers changed from: package-private */
    public static zza zza(Context context, zzn zznVar) {
        Uri defaultUri;
        Intent intent;
        PendingIntent activity;
        PendingIntent zza2;
        Bundle zza3 = zza(context.getPackageManager(), context.getPackageName());
        String packageName = context.getPackageName();
        String zzb = zzb(context, zznVar.zza("gcm.n.android_channel_id"), zza3);
        Resources resources = context.getResources();
        PackageManager packageManager = context.getPackageManager();
        NotificationCompat.Builder builder = new NotificationCompat.Builder(context, zzb);
        builder.setContentTitle(zza(packageName, zznVar, packageManager, resources));
        String zza4 = zznVar.zza(resources, packageName, "gcm.n.body");
        if (!TextUtils.isEmpty(zza4)) {
            builder.setContentText(zza4);
            builder.setStyle(new NotificationCompat.BigTextStyle().bigText(zza4));
        }
        builder.setSmallIcon(zza(packageManager, resources, packageName, zznVar.zza("gcm.n.icon"), zza3));
        String zzb2 = zznVar.zzb();
        Integer num = null;
        if (TextUtils.isEmpty(zzb2)) {
            defaultUri = null;
        } else if (!"default".equals(zzb2) && resources.getIdentifier(zzb2, "raw", packageName) != 0) {
            defaultUri = Uri.parse(new StringBuilder(String.valueOf(packageName).length() + 24 + String.valueOf(zzb2).length()).append("android.resource://").append(packageName).append("/raw/").append(zzb2).toString());
        } else {
            defaultUri = RingtoneManager.getDefaultUri(2);
        }
        if (defaultUri != null) {
            builder.setSound(defaultUri);
        }
        String zza5 = zznVar.zza("gcm.n.click_action");
        if (!TextUtils.isEmpty(zza5)) {
            intent = new Intent(zza5);
            intent.setPackage(packageName);
            intent.setFlags(C.ENCODING_PCM_MU_LAW);
        } else {
            Uri zza6 = zznVar.zza();
            if (zza6 != null) {
                intent = new Intent("android.intent.action.VIEW");
                intent.setPackage(packageName);
                intent.setData(zza6);
            } else {
                Intent launchIntentForPackage = packageManager.getLaunchIntentForPackage(packageName);
                if (launchIntentForPackage == null) {
                    Log.w("FirebaseMessaging", "No activity found to launch app");
                }
                intent = launchIntentForPackage;
            }
        }
        if (intent == null) {
            activity = null;
        } else {
            intent.addFlags(67108864);
            intent.putExtras(zznVar.zze());
            activity = PendingIntent.getActivity(context, zza.incrementAndGet(), intent, 1073741824);
            if (zznVar.zzb("google.c.a.e")) {
                activity = zza(context, new Intent("com.google.firebase.messaging.NOTIFICATION_OPEN").putExtras(zznVar.zzf()).putExtra("pending_intent", activity));
            }
        }
        builder.setContentIntent(activity);
        if (!zznVar.zzb("google.c.a.e")) {
            zza2 = null;
        } else {
            zza2 = zza(context, new Intent("com.google.firebase.messaging.NOTIFICATION_DISMISS").putExtras(zznVar.zzf()));
        }
        if (zza2 != null) {
            builder.setDeleteIntent(zza2);
        }
        Integer zza7 = zza(context, zznVar.zza("gcm.n.color"), zza3);
        if (zza7 != null) {
            builder.setColor(zza7.intValue());
        }
        int i = 1;
        builder.setAutoCancel(!zznVar.zzb("gcm.n.sticky"));
        builder.setLocalOnly(zznVar.zzb("gcm.n.local_only"));
        String zza8 = zznVar.zza("gcm.n.ticker");
        if (zza8 != null) {
            builder.setTicker(zza8);
        }
        Integer zzc = zznVar.zzc("gcm.n.notification_priority");
        if (zzc == null) {
            zzc = null;
        } else if (zzc.intValue() < -2 || zzc.intValue() > 2) {
            String valueOf = String.valueOf(zzc);
            Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(valueOf).length() + 72).append("notificationPriority is invalid ").append(valueOf).append(". Skipping setting notificationPriority.").toString());
            zzc = null;
        }
        if (zzc != null) {
            builder.setPriority(zzc.intValue());
        }
        Integer zzc2 = zznVar.zzc("gcm.n.visibility");
        if (zzc2 == null) {
            zzc2 = null;
        } else if (zzc2.intValue() < -1 || zzc2.intValue() > 1) {
            String valueOf2 = String.valueOf(zzc2);
            Log.w("NotificationParams", new StringBuilder(String.valueOf(valueOf2).length() + 53).append("visibility is invalid: ").append(valueOf2).append(". Skipping setting visibility.").toString());
            zzc2 = null;
        }
        if (zzc2 != null) {
            builder.setVisibility(zzc2.intValue());
        }
        Integer zzc3 = zznVar.zzc("gcm.n.notification_count");
        if (zzc3 != null) {
            if (zzc3.intValue() < 0) {
                String valueOf3 = String.valueOf(zzc3);
                Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(valueOf3).length() + 67).append("notificationCount is invalid: ").append(valueOf3).append(". Skipping setting notificationCount.").toString());
            } else {
                num = zzc3;
            }
        }
        if (num != null) {
            builder.setNumber(num.intValue());
        }
        Long zzd = zznVar.zzd("gcm.n.event_time");
        if (zzd != null) {
            builder.setShowWhen(true);
            builder.setWhen(zzd.longValue());
        }
        long[] zzc4 = zznVar.zzc();
        if (zzc4 != null) {
            builder.setVibrate(zzc4);
        }
        int[] zzd2 = zznVar.zzd();
        if (zzd2 != null) {
            builder.setLights(zzd2[0], zzd2[1], zzd2[2]);
        }
        if (!zznVar.zzb("gcm.n.default_sound")) {
            i = 0;
        }
        if (zznVar.zzb("gcm.n.default_vibrate_timings")) {
            i |= 2;
        }
        if (zznVar.zzb("gcm.n.default_light_settings")) {
            i |= 4;
        }
        builder.setDefaults(i);
        String zza9 = zznVar.zza("gcm.n.tag");
        if (TextUtils.isEmpty(zza9)) {
            zza9 = new StringBuilder(37).append("FCM-Notification:").append(SystemClock.uptimeMillis()).toString();
        }
        return new zza(builder, zza9, 0);
    }

    private static CharSequence zza(String str, zzn zznVar, PackageManager packageManager, Resources resources) {
        String zza2 = zznVar.zza(resources, str, "gcm.n.title");
        if (!TextUtils.isEmpty(zza2)) {
            return zza2;
        }
        try {
            return packageManager.getApplicationInfo(str, 0).loadLabel(packageManager);
        } catch (PackageManager.NameNotFoundException e) {
            String valueOf = String.valueOf(e);
            Log.e("FirebaseMessaging", new StringBuilder(String.valueOf(valueOf).length() + 35).append("Couldn't get own application info: ").append(valueOf).toString());
            return "";
        }
    }

    private static boolean zza(Resources resources, int i) {
        if (Build.VERSION.SDK_INT != 26) {
            return true;
        }
        try {
            if (resources.getDrawable(i, null) instanceof AdaptiveIconDrawable) {
                Log.e("FirebaseMessaging", new StringBuilder(77).append("Adaptive icons cannot be used in notifications. Ignoring icon id: ").append(i).toString());
                return false;
            }
            return true;
        } catch (Resources.NotFoundException e) {
            Log.e("FirebaseMessaging", new StringBuilder(66).append("Couldn't find resource ").append(i).append(", treating it as an invalid icon").toString());
            return false;
        }
    }

    private static int zza(PackageManager packageManager, Resources resources, String str, String str2, Bundle bundle) {
        if (!TextUtils.isEmpty(str2)) {
            int identifier = resources.getIdentifier(str2, "drawable", str);
            if (identifier != 0 && zza(resources, identifier)) {
                return identifier;
            }
            int identifier2 = resources.getIdentifier(str2, "mipmap", str);
            if (identifier2 != 0 && zza(resources, identifier2)) {
                return identifier2;
            }
            Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(str2).length() + 61).append("Icon resource ").append(str2).append(" not found. Notification will use default icon.").toString());
        }
        int i = bundle.getInt("com.google.firebase.messaging.default_notification_icon", 0);
        if (i == 0 || !zza(resources, i)) {
            try {
                i = packageManager.getApplicationInfo(str, 0).icon;
            } catch (PackageManager.NameNotFoundException e) {
                String valueOf = String.valueOf(e);
                Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(valueOf).length() + 35).append("Couldn't get own application info: ").append(valueOf).toString());
            }
        }
        if (i == 0 || !zza(resources, i)) {
            return 17301651;
        }
        return i;
    }

    private static Integer zza(Context context, String str, Bundle bundle) {
        if (!TextUtils.isEmpty(str)) {
            try {
                return Integer.valueOf(Color.parseColor(str));
            } catch (IllegalArgumentException e) {
                Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(str).length() + 56).append("Color is invalid: ").append(str).append(". Notification will use default color.").toString());
            }
        }
        int i = bundle.getInt("com.google.firebase.messaging.default_notification_color", 0);
        if (i != 0) {
            try {
                return Integer.valueOf(ContextCompat.getColor(context, i));
            } catch (Resources.NotFoundException e2) {
                Log.w("FirebaseMessaging", "Cannot find the color resource referenced in AndroidManifest.");
                return null;
            }
        }
        return null;
    }

    private static Bundle zza(PackageManager packageManager, String str) {
        try {
            ApplicationInfo applicationInfo = packageManager.getApplicationInfo(str, 128);
            if (applicationInfo != null && applicationInfo.metaData != null) {
                return applicationInfo.metaData;
            }
        } catch (PackageManager.NameNotFoundException e) {
            String valueOf = String.valueOf(e);
            Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(valueOf).length() + 35).append("Couldn't get own application info: ").append(valueOf).toString());
        }
        return Bundle.EMPTY;
    }

    private static String zzb(Context context, String str, Bundle bundle) {
        if (Build.VERSION.SDK_INT < 26) {
            return null;
        }
        try {
            if (context.getPackageManager().getApplicationInfo(context.getPackageName(), 0).targetSdkVersion < 26) {
                return null;
            }
            NotificationManager notificationManager = (NotificationManager) context.getSystemService(NotificationManager.class);
            if (!TextUtils.isEmpty(str)) {
                if (notificationManager.getNotificationChannel(str) != null) {
                    return str;
                }
                Log.w("FirebaseMessaging", new StringBuilder(String.valueOf(str).length() + 122).append("Notification Channel requested (").append(str).append(") has not been created by the app. Manifest configuration, or default, value will be used.").toString());
            }
            String string = bundle.getString("com.google.firebase.messaging.default_notification_channel_id");
            if (!TextUtils.isEmpty(string)) {
                if (notificationManager.getNotificationChannel(string) != null) {
                    return string;
                }
                Log.w("FirebaseMessaging", "Notification Channel set in AndroidManifest.xml has not been created by the app. Default value will be used.");
            } else {
                Log.w("FirebaseMessaging", "Missing Default Notification Channel metadata in AndroidManifest. Default value will be used.");
            }
            if (notificationManager.getNotificationChannel("fcm_fallback_notification_channel") == null) {
                notificationManager.createNotificationChannel(new NotificationChannel("fcm_fallback_notification_channel", context.getString(context.getResources().getIdentifier("fcm_fallback_notification_channel_label", "string", context.getPackageName())), 3));
            }
            return "fcm_fallback_notification_channel";
        } catch (PackageManager.NameNotFoundException e) {
            return null;
        }
    }

    private static PendingIntent zza(Context context, Intent intent) {
        return PendingIntent.getBroadcast(context, zza.incrementAndGet(), new Intent("com.google.firebase.MESSAGING_EVENT").setComponent(new ComponentName(context, "com.google.firebase.iid.FirebaseInstanceIdReceiver")).putExtra("wrapped_intent", intent), 1073741824);
    }
}
