package com.google.firebase.messaging;

import android.content.res.Resources;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import com.litesuits.orm.db.assit.SQLBuilder;
import java.util.Arrays;
import java.util.MissingFormatArgumentException;
import org.json.JSONArray;
import org.json.JSONException;
/* compiled from: com.google.firebase:firebase-messaging@@20.1.0 */
/* loaded from: classes19.dex */
public final class zzn {
    private final Bundle zza;

    public zzn(Bundle bundle) {
        if (bundle == null) {
            throw new NullPointerException("data");
        }
        this.zza = new Bundle(bundle);
    }

    public final String zza(String str) {
        Bundle bundle = this.zza;
        if (!bundle.containsKey(str) && str.startsWith("gcm.n.")) {
            String zzi = zzi(str);
            if (this.zza.containsKey(zzi)) {
                str = zzi;
            }
        }
        return bundle.getString(str);
    }

    public final boolean zzb(String str) {
        String zza = zza(str);
        return "1".equals(zza) || Boolean.parseBoolean(zza);
    }

    public final Integer zzc(String str) {
        String zza = zza(str);
        if (!TextUtils.isEmpty(zza)) {
            try {
                return Integer.valueOf(Integer.parseInt(zza));
            } catch (NumberFormatException e) {
                String zzh = zzh(str);
                Log.w("NotificationParams", new StringBuilder(String.valueOf(zzh).length() + 38 + String.valueOf(zza).length()).append("Couldn't parse value of ").append(zzh).append(SQLBuilder.PARENTHESES_LEFT).append(zza).append(") into an int").toString());
                return null;
            }
        }
        return null;
    }

    public final Long zzd(String str) {
        String zza = zza(str);
        if (!TextUtils.isEmpty(zza)) {
            try {
                return Long.valueOf(Long.parseLong(zza));
            } catch (NumberFormatException e) {
                String zzh = zzh(str);
                Log.w("NotificationParams", new StringBuilder(String.valueOf(zzh).length() + 38 + String.valueOf(zza).length()).append("Couldn't parse value of ").append(zzh).append(SQLBuilder.PARENTHESES_LEFT).append(zza).append(") into a long").toString());
                return null;
            }
        }
        return null;
    }

    public final String zze(String str) {
        String valueOf = String.valueOf(str);
        String valueOf2 = String.valueOf("_loc_key");
        return zza(valueOf2.length() != 0 ? valueOf.concat(valueOf2) : new String(valueOf));
    }

    public final Object[] zzf(String str) {
        String valueOf = String.valueOf(str);
        String valueOf2 = String.valueOf("_loc_args");
        JSONArray zzg = zzg(valueOf2.length() != 0 ? valueOf.concat(valueOf2) : new String(valueOf));
        if (zzg == null) {
            return null;
        }
        int length = zzg.length();
        String[] strArr = new String[length];
        for (int i = 0; i < length; i++) {
            strArr[i] = zzg.optString(i);
        }
        return strArr;
    }

    private final JSONArray zzg(String str) {
        String zza = zza(str);
        if (!TextUtils.isEmpty(zza)) {
            try {
                return new JSONArray(zza);
            } catch (JSONException e) {
                String zzh = zzh(str);
                Log.w("NotificationParams", new StringBuilder(String.valueOf(zzh).length() + 50 + String.valueOf(zza).length()).append("Malformed JSON for key ").append(zzh).append(": ").append(zza).append(", falling back to default").toString());
                return null;
            }
        }
        return null;
    }

    private static String zzh(String str) {
        if (str.startsWith("gcm.n.")) {
            return str.substring(6);
        }
        return str;
    }

    public final Uri zza() {
        String zza = zza("gcm.n.link_android");
        if (TextUtils.isEmpty(zza)) {
            zza = zza("gcm.n.link");
        }
        if (!TextUtils.isEmpty(zza)) {
            return Uri.parse(zza);
        }
        return null;
    }

    public final String zzb() {
        String zza = zza("gcm.n.sound2");
        if (TextUtils.isEmpty(zza)) {
            return zza("gcm.n.sound");
        }
        return zza;
    }

    public final long[] zzc() {
        JSONArray zzg = zzg("gcm.n.vibrate_timings");
        if (zzg == null) {
            return null;
        }
        try {
            if (zzg.length() <= 1) {
                throw new JSONException("vibrateTimings have invalid length");
            }
            int length = zzg.length();
            long[] jArr = new long[length];
            for (int i = 0; i < length; i++) {
                jArr[i] = zzg.optLong(i);
            }
            return jArr;
        } catch (NumberFormatException | JSONException e) {
            String valueOf = String.valueOf(zzg);
            Log.w("NotificationParams", new StringBuilder(String.valueOf(valueOf).length() + 74).append("User defined vibrateTimings is invalid: ").append(valueOf).append(". Skipping setting vibrateTimings.").toString());
            return null;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final int[] zzd() {
        JSONArray zzg = zzg("gcm.n.light_settings");
        if (zzg == null) {
            return null;
        }
        int[] iArr = new int[3];
        try {
            if (zzg.length() != 3) {
                throw new JSONException("lightSettings don't have all three fields");
            }
            int parseColor = Color.parseColor(zzg.optString(0));
            if (parseColor == -16777216) {
                throw new IllegalArgumentException("Transparent color is invalid");
            }
            iArr[0] = parseColor;
            iArr[1] = zzg.optInt(1);
            iArr[2] = zzg.optInt(2);
            return iArr;
        } catch (IllegalArgumentException e) {
            String valueOf = String.valueOf(zzg);
            String message = e.getMessage();
            Log.w("NotificationParams", new StringBuilder(String.valueOf(valueOf).length() + 60 + String.valueOf(message).length()).append("LightSettings is invalid: ").append(valueOf).append(". ").append(message).append(". Skipping setting LightSettings").toString());
            return null;
        } catch (JSONException e2) {
            String valueOf2 = String.valueOf(zzg);
            Log.w("NotificationParams", new StringBuilder(String.valueOf(valueOf2).length() + 58).append("LightSettings is invalid: ").append(valueOf2).append(". Skipping setting LightSettings").toString());
            return null;
        }
    }

    public final Bundle zze() {
        Bundle bundle = new Bundle(this.zza);
        for (String str : this.zza.keySet()) {
            if (str.startsWith("google.c.") || str.startsWith("gcm.n.") || str.startsWith("gcm.notification.")) {
                bundle.remove(str);
            }
        }
        return bundle;
    }

    public final Bundle zzf() {
        Bundle bundle = new Bundle(this.zza);
        for (String str : this.zza.keySet()) {
            if (!(str.startsWith("google.c.a.") || str.equals("from"))) {
                bundle.remove(str);
            }
        }
        return bundle;
    }

    private final String zzb(Resources resources, String str, String str2) {
        String zze = zze(str2);
        if (TextUtils.isEmpty(zze)) {
            return null;
        }
        int identifier = resources.getIdentifier(zze, "string", str);
        if (identifier == 0) {
            String valueOf = String.valueOf(str2);
            String valueOf2 = String.valueOf("_loc_key");
            String zzh = zzh(valueOf2.length() != 0 ? valueOf.concat(valueOf2) : new String(valueOf));
            Log.w("NotificationParams", new StringBuilder(String.valueOf(zzh).length() + 49 + String.valueOf(str2).length()).append(zzh).append(" resource not found: ").append(str2).append(" Default value will be used.").toString());
            return null;
        }
        Object[] zzf = zzf(str2);
        if (zzf == null) {
            return resources.getString(identifier);
        }
        try {
            return resources.getString(identifier, zzf);
        } catch (MissingFormatArgumentException e) {
            String zzh2 = zzh(str2);
            String arrays = Arrays.toString(zzf);
            Log.w("NotificationParams", new StringBuilder(String.valueOf(zzh2).length() + 58 + String.valueOf(arrays).length()).append("Missing format argument for ").append(zzh2).append(": ").append(arrays).append(" Default value will be used.").toString(), e);
            return null;
        }
    }

    public final String zza(Resources resources, String str, String str2) {
        String zza = zza(str2);
        if (!TextUtils.isEmpty(zza)) {
            return zza;
        }
        return zzb(resources, str, str2);
    }

    public static boolean zza(Bundle bundle) {
        if ("1".equals(bundle.getString("gcm.n.e")) || "1".equals(bundle.getString(zzi("gcm.n.e"))) || bundle.getString("gcm.n.icon") != null || bundle.getString(zzi("gcm.n.icon")) != null) {
            return true;
        }
        return false;
    }

    private static String zzi(String str) {
        if (str.startsWith("gcm.n.")) {
            return str.replace("gcm.n.", "gcm.notification.");
        }
        return str;
    }
}
