package com.library;

import android.graphics.Canvas;
import android.graphics.Paint;
import android.text.TextPaint;
import android.view.View;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.library.listener.GroupListener;
import com.library.listener.OnGroupClickListener;
/* loaded from: classes11.dex */
public class StickyDecoration extends BaseDecoration {
    private GroupListener mGroupListener;
    private int mGroupTextColor;
    private Paint mGroutPaint;
    private int mSideMargin;
    private TextPaint mTextPaint;
    private int mTextSize;

    private StickyDecoration(GroupListener groupListener) {
        this.mGroupTextColor = -1;
        this.mSideMargin = 0;
        this.mTextSize = 50;
        this.mGroupListener = groupListener;
        Paint paint = new Paint();
        this.mGroutPaint = paint;
        paint.setColor(this.mGroupBackground);
        TextPaint textPaint = new TextPaint();
        this.mTextPaint = textPaint;
        textPaint.setAntiAlias(true);
        this.mTextPaint.setTextSize(this.mTextSize);
        this.mTextPaint.setColor(this.mGroupTextColor);
        this.mTextPaint.setTextAlign(Paint.Align.LEFT);
    }

    /* JADX WARN: Removed duplicated region for block: B:20:0x0083  */
    /* JADX WARN: Removed duplicated region for block: B:27:0x0092 A[SYNTHETIC] */
    @Override // com.library.BaseDecoration, androidx.recyclerview.widget.RecyclerView.ItemDecoration
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void onDrawOver(Canvas c, RecyclerView parent, RecyclerView.State state) {
        int childCount;
        int childCount2;
        int bottom;
        super.onDrawOver(c, parent, state);
        int itemCount = state.getItemCount();
        int childCount3 = parent.getChildCount();
        int left = parent.getPaddingLeft();
        int right = parent.getWidth() - parent.getPaddingRight();
        int i = 0;
        while (i < childCount3) {
            View childView = parent.getChildAt(i);
            int position = parent.getChildAdapterPosition(childView);
            int realPosition = getRealPosition(position);
            if (!isFirstInGroup(realPosition)) {
                if (isFirstInRecyclerView(realPosition, i)) {
                    childCount = childCount3;
                    childCount2 = realPosition;
                } else {
                    childCount = childCount3;
                    drawDivide(c, parent, childView, realPosition, left, right);
                    i++;
                    childCount3 = childCount;
                }
            } else {
                childCount = childCount3;
                childCount2 = realPosition;
            }
            int bottom2 = Math.max(this.mGroupHeight, childView.getTop() + parent.getPaddingTop());
            if (position + 1 < itemCount) {
                int viewBottom = childView.getBottom();
                if (isLastLineInGroup(parent, childCount2) && viewBottom < bottom2) {
                    bottom = viewBottom;
                    drawDecoration(c, childCount2, left, right, bottom);
                    if (this.mOnGroupClickListener == null) {
                        this.stickyHeaderPosArray.put(Integer.valueOf(childCount2), new ClickInfo(bottom));
                    }
                    i++;
                    childCount3 = childCount;
                }
            }
            bottom = bottom2;
            drawDecoration(c, childCount2, left, right, bottom);
            if (this.mOnGroupClickListener == null) {
            }
            i++;
            childCount3 = childCount;
        }
    }

    private void drawDecoration(Canvas c, int realPosition, int left, int right, int bottom) {
        int firstPositionInGroup = getFirstInGroupWithCash(realPosition);
        String curGroupName = getGroupName(firstPositionInGroup);
        c.drawRect(left, bottom - this.mGroupHeight, right, bottom, this.mGroutPaint);
        if (curGroupName == null) {
            return;
        }
        Paint.FontMetrics fm = this.mTextPaint.getFontMetrics();
        float baseLine = (bottom - ((this.mGroupHeight - (fm.bottom - fm.top)) / 2.0f)) - fm.bottom;
        int abs = Math.abs(this.mSideMargin);
        this.mSideMargin = abs;
        c.drawText(curGroupName, abs + left, baseLine, this.mTextPaint);
    }

    @Override // com.library.BaseDecoration
    String getGroupName(int realPosition) {
        GroupListener groupListener = this.mGroupListener;
        if (groupListener != null) {
            return groupListener.getGroupName(realPosition);
        }
        return null;
    }

    /* loaded from: classes11.dex */
    public static class Builder {
        private StickyDecoration mDecoration;

        private Builder(GroupListener groupListener) {
            this.mDecoration = new StickyDecoration(groupListener);
        }

        public static Builder init(GroupListener groupListener) {
            return new Builder(groupListener);
        }

        public Builder setGroupBackground(int background) {
            this.mDecoration.mGroupBackground = background;
            this.mDecoration.mGroutPaint.setColor(this.mDecoration.mGroupBackground);
            return this;
        }

        public Builder setGroupTextSize(int textSize) {
            this.mDecoration.mTextSize = textSize;
            this.mDecoration.mTextPaint.setTextSize(this.mDecoration.mTextSize);
            return this;
        }

        public Builder setGroupHeight(int groutHeight) {
            this.mDecoration.mGroupHeight = groutHeight;
            return this;
        }

        public Builder setGroupTextColor(int color) {
            this.mDecoration.mGroupTextColor = color;
            this.mDecoration.mTextPaint.setColor(this.mDecoration.mGroupTextColor);
            return this;
        }

        public Builder setTextSideMargin(int leftMargin) {
            this.mDecoration.mSideMargin = leftMargin;
            return this;
        }

        public Builder setDivideHeight(int height) {
            this.mDecoration.mDivideHeight = height;
            return this;
        }

        public Builder setDivideColor(int color) {
            this.mDecoration.mDivideColor = color;
            this.mDecoration.mDividePaint.setColor(color);
            return this;
        }

        public Builder setOnClickListener(OnGroupClickListener listener) {
            this.mDecoration.setOnGroupClickListener(listener);
            return this;
        }

        public Builder resetSpan(RecyclerView recyclerView, GridLayoutManager gridLayoutManager) {
            this.mDecoration.resetSpan(recyclerView, gridLayoutManager);
            return this;
        }

        public Builder setHeaderCount(int headerCount) {
            if (headerCount >= 0) {
                this.mDecoration.mHeaderCount = headerCount;
            }
            return this;
        }

        public StickyDecoration build() {
            return this.mDecoration;
        }
    }
}
