package com.litesuits.orm;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteClosable;
import android.database.sqlite.SQLiteDatabase;
import com.litesuits.orm.db.DataBase;
import com.litesuits.orm.db.DataBaseConfig;
import com.litesuits.orm.db.TableManager;
import com.litesuits.orm.db.assit.Checker;
import com.litesuits.orm.db.assit.CollSpliter;
import com.litesuits.orm.db.assit.Querier;
import com.litesuits.orm.db.assit.QueryBuilder;
import com.litesuits.orm.db.assit.SQLBuilder;
import com.litesuits.orm.db.assit.SQLStatement;
import com.litesuits.orm.db.assit.SQLiteHelper;
import com.litesuits.orm.db.assit.WhereBuilder;
import com.litesuits.orm.db.impl.CascadeSQLiteImpl;
import com.litesuits.orm.db.impl.SingleSQLiteImpl;
import com.litesuits.orm.db.model.ColumnsValue;
import com.litesuits.orm.db.model.ConflictAlgorithm;
import com.litesuits.orm.db.model.EntityTable;
import com.litesuits.orm.db.model.MapProperty;
import com.litesuits.orm.db.model.RelationKey;
import com.litesuits.orm.db.utils.ClassUtil;
import com.litesuits.orm.db.utils.DataUtil;
import com.litesuits.orm.db.utils.FieldUtil;
import com.litesuits.orm.log.OrmLog;
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
/* loaded from: classes19.dex */
public abstract class LiteOrm extends SQLiteClosable implements DataBase {
    public static final String TAG = LiteOrm.class.getSimpleName();
    protected DataBaseConfig mConfig;
    protected SQLiteHelper mHelper;
    protected TableManager mTableManager;
    protected LiteOrm otherDatabase;

    public abstract LiteOrm cascade();

    public abstract LiteOrm single();

    /* JADX INFO: Access modifiers changed from: protected */
    public LiteOrm(LiteOrm dataBase) {
        this.mHelper = dataBase.mHelper;
        this.mConfig = dataBase.mConfig;
        this.mTableManager = dataBase.mTableManager;
        this.otherDatabase = dataBase;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public LiteOrm(DataBaseConfig config) {
        config.context = config.context.getApplicationContext();
        if (config.dbName == null) {
            config.dbName = DataBaseConfig.DEFAULT_DB_NAME;
        }
        if (config.dbVersion <= 0) {
            config.dbVersion = 1;
        }
        this.mConfig = config;
        setDebugged(config.debugged);
        openOrCreateDatabase();
    }

    @Override // com.litesuits.orm.db.DataBase
    public SQLiteDatabase openOrCreateDatabase() {
        initDatabasePath(this.mConfig.dbName);
        if (this.mHelper != null) {
            justRelease();
        }
        this.mHelper = new SQLiteHelper(this.mConfig.context.getApplicationContext(), this.mConfig.dbName, null, this.mConfig.dbVersion, this.mConfig.onUpdateListener);
        this.mTableManager = new TableManager(this.mConfig.dbName, this.mHelper.getReadableDatabase());
        return this.mHelper.getWritableDatabase();
    }

    private void initDatabasePath(String path) {
        String str = TAG;
        OrmLog.i(str, "create  database path: " + path);
        String path2 = this.mConfig.context.getDatabasePath(this.mConfig.dbName).getPath();
        OrmLog.i(str, "context database path: " + path2);
        File dbp = new File(path2).getParentFile();
        if (dbp != null && !dbp.exists()) {
            boolean mks = dbp.mkdirs();
            OrmLog.i(str, "create database, parent file mkdirs: " + mks + "  path:" + dbp.getAbsolutePath());
        }
    }

    public static LiteOrm newSingleInstance(Context context, String dbName) {
        return newSingleInstance(new DataBaseConfig(context, dbName));
    }

    public static synchronized LiteOrm newSingleInstance(DataBaseConfig config) {
        LiteOrm newInstance;
        synchronized (LiteOrm.class) {
            newInstance = SingleSQLiteImpl.newInstance(config);
        }
        return newInstance;
    }

    public static LiteOrm newCascadeInstance(Context context, String dbName) {
        return newCascadeInstance(new DataBaseConfig(context, dbName));
    }

    public static synchronized LiteOrm newCascadeInstance(DataBaseConfig config) {
        LiteOrm newInstance;
        synchronized (LiteOrm.class) {
            newInstance = CascadeSQLiteImpl.newInstance(config);
        }
        return newInstance;
    }

    public void setDebugged(boolean debugged) {
        this.mConfig.debugged = debugged;
        OrmLog.isPrint = debugged;
    }

    @Override // com.litesuits.orm.db.DataBase
    public ArrayList<RelationKey> queryRelation(final Class class1, final Class class2, final List<String> key1List) {
        acquireReference();
        final ArrayList<RelationKey> rList = new ArrayList<>();
        try {
            try {
                final EntityTable table1 = TableManager.getTable((Class<?>) class1);
                final EntityTable table2 = TableManager.getTable((Class<?>) class2);
                if (this.mTableManager.isSQLMapTableCreated(table1.name, table2.name)) {
                    CollSpliter.split(key1List, 999, new CollSpliter.Spliter<String>() { // from class: com.litesuits.orm.LiteOrm.1
                        @Override // com.litesuits.orm.db.assit.CollSpliter.Spliter
                        public int oneSplit(ArrayList<String> list) throws Exception {
                            SQLStatement stmt = SQLBuilder.buildQueryRelationSql(class1, class2, key1List);
                            Querier.doQuery(LiteOrm.this.mHelper.getReadableDatabase(), stmt, new Querier.CursorParser() { // from class: com.litesuits.orm.LiteOrm.1.1
                                @Override // com.litesuits.orm.db.assit.Querier.CursorParser
                                public void parseEachCursor(SQLiteDatabase db, Cursor c) throws Exception {
                                    RelationKey relation = new RelationKey();
                                    relation.key1 = c.getString(c.getColumnIndex(table1.name));
                                    relation.key2 = c.getString(c.getColumnIndex(table2.name));
                                    rList.add(relation);
                                }
                            });
                            return 0;
                        }
                    });
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            return rList;
        } finally {
            releaseReference();
        }
    }

    @Override // com.litesuits.orm.db.DataBase
    public <E, T> boolean mapping(Collection<E> col1, Collection<T> col2) {
        if (Checker.isEmpty((Collection<?>) col1) || Checker.isEmpty((Collection<?>) col2)) {
            return false;
        }
        acquireReference();
        try {
            return keepMapping(col1, col2) | keepMapping(col2, col1);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            releaseReference();
        }
    }

    @Override // com.litesuits.orm.db.DataBase
    public SQLStatement createSQLStatement(String sql, Object[] bindArgs) {
        return new SQLStatement(sql, bindArgs);
    }

    @Override // com.litesuits.orm.db.DataBase
    public boolean execute(SQLiteDatabase db, SQLStatement statement) {
        acquireReference();
        if (statement != null) {
            try {
                try {
                    return statement.execute(db);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } finally {
                releaseReference();
            }
        }
        releaseReference();
        return false;
    }

    @Override // com.litesuits.orm.db.DataBase
    @Deprecated
    public boolean dropTable(Object entity) {
        return dropTable(entity.getClass());
    }

    @Override // com.litesuits.orm.db.DataBase
    public boolean dropTable(Class<?> claxx) {
        return dropTable(TableManager.getTable(claxx, false).name);
    }

    @Override // com.litesuits.orm.db.DataBase
    public boolean dropTable(String tableName) {
        acquireReference();
        try {
            try {
                return SQLBuilder.buildDropTable(tableName).execute(this.mHelper.getWritableDatabase());
            } catch (Exception e) {
                e.printStackTrace();
                releaseReference();
                return false;
            }
        } finally {
            releaseReference();
        }
    }

    @Override // com.litesuits.orm.db.DataBase
    public <T> long queryCount(Class<T> claxx) {
        return queryCount(new QueryBuilder(claxx));
    }

    @Override // com.litesuits.orm.db.DataBase
    public long queryCount(QueryBuilder qb) {
        acquireReference();
        try {
            try {
                if (!this.mTableManager.isSQLTableCreated(qb.getTableName())) {
                    releaseReference();
                    return 0L;
                }
                SQLiteDatabase db = this.mHelper.getReadableDatabase();
                SQLStatement stmt = qb.createStatementForCount();
                return stmt.queryForLong(db);
            } catch (Exception e) {
                e.printStackTrace();
                releaseReference();
                return -1L;
            }
        } finally {
            releaseReference();
        }
    }

    @Override // com.litesuits.orm.db.DataBase
    public int update(WhereBuilder where, ColumnsValue cvs, ConflictAlgorithm conflictAlgorithm) {
        acquireReference();
        try {
            try {
                SQLiteDatabase db = this.mHelper.getWritableDatabase();
                SQLStatement stmt = SQLBuilder.buildUpdateSql(where, cvs, conflictAlgorithm);
                return stmt.execUpdate(db);
            } catch (Exception e) {
                e.printStackTrace();
                releaseReference();
                return -1;
            }
        } finally {
            releaseReference();
        }
    }

    @Override // com.litesuits.orm.db.DataBase
    public synchronized SQLiteDatabase getReadableDatabase() {
        return this.mHelper.getReadableDatabase();
    }

    @Override // com.litesuits.orm.db.DataBase
    public synchronized SQLiteDatabase getWritableDatabase() {
        return this.mHelper.getWritableDatabase();
    }

    @Override // com.litesuits.orm.db.DataBase
    public TableManager getTableManager() {
        return this.mTableManager;
    }

    @Override // com.litesuits.orm.db.DataBase
    public SQLiteHelper getSQLiteHelper() {
        return this.mHelper;
    }

    @Override // com.litesuits.orm.db.DataBase
    public DataBaseConfig getDataBaseConfig() {
        return this.mConfig;
    }

    @Override // com.litesuits.orm.db.DataBase
    public SQLiteDatabase openOrCreateDatabase(String path, SQLiteDatabase.CursorFactory factory) {
        String path2 = this.mConfig.context.getDatabasePath(this.mConfig.dbName).getPath();
        return SQLiteDatabase.openOrCreateDatabase(path2, factory);
    }

    @Override // com.litesuits.orm.db.DataBase
    public boolean deleteDatabase() {
        String path = this.mHelper.getWritableDatabase().getPath();
        justRelease();
        OrmLog.i(TAG, "data has cleared. delete Database path: " + path);
        return deleteDatabase(new File(path));
    }

    @Override // com.litesuits.orm.db.DataBase
    public boolean deleteDatabase(File file) {
        acquireReference();
        try {
            try {
                if (file == null) {
                    throw new IllegalArgumentException("file must not be null");
                }
                boolean deleted = file.delete() | new File(file.getPath() + "-journal").delete() | new File(file.getPath() + "-shm").delete() | new File(file.getPath() + "-wal").delete();
                File dir = file.getParentFile();
                if (dir != null) {
                    final String prefix = file.getName() + "-mj";
                    FileFilter filter = new FileFilter() { // from class: com.litesuits.orm.LiteOrm.2
                        @Override // java.io.FileFilter
                        public boolean accept(File candidate) {
                            return candidate.getName().startsWith(prefix);
                        }
                    };
                    File[] arr$ = dir.listFiles(filter);
                    for (File masterJournal : arr$) {
                        deleted |= masterJournal.delete();
                    }
                }
                return deleted;
            } catch (Exception e) {
                e.printStackTrace();
                releaseReference();
                return false;
            }
        } finally {
            releaseReference();
        }
    }

    @Override // android.database.sqlite.SQLiteClosable, java.io.Closeable, java.lang.AutoCloseable, com.litesuits.orm.db.DataBase
    public synchronized void close() {
        releaseReference();
    }

    @Override // android.database.sqlite.SQLiteClosable
    protected void onAllReferencesReleased() {
        justRelease();
    }

    protected void justRelease() {
        SQLiteHelper sQLiteHelper = this.mHelper;
        if (sQLiteHelper != null) {
            sQLiteHelper.getWritableDatabase().close();
            this.mHelper.close();
            this.mHelper = null;
        }
        TableManager tableManager = this.mTableManager;
        if (tableManager != null) {
            tableManager.release();
            this.mTableManager = null;
        }
    }

    public static int releaseMemory() {
        return SQLiteDatabase.releaseMemory();
    }

    /* JADX WARN: Multi-variable type inference failed */
    private <E, T> boolean keepMapping(Collection<E> col1, Collection<T> col2) throws IllegalAccessException, InstantiationException {
        Class itemClass;
        HashMap<Object, ArrayList> collMap;
        Iterator i$;
        EntityTable table1;
        ArrayList col;
        Class claxx1;
        Object key1;
        Class claxx12 = col1.iterator().next().getClass();
        Class claxx2 = col2.iterator().next().getClass();
        EntityTable table12 = TableManager.getTable((Class<?>) claxx12);
        EntityTable table2 = TableManager.getTable((Class<?>) claxx2);
        if (table12.mappingList != null) {
            Iterator i$2 = table12.mappingList.iterator();
            while (i$2.hasNext()) {
                MapProperty mp = i$2.next();
                Class fieldClass = mp.field.getType();
                if (mp.isToMany()) {
                    if (ClassUtil.isCollection(fieldClass)) {
                        itemClass = FieldUtil.getGenericType(mp.field);
                    } else if (fieldClass.isArray()) {
                        itemClass = FieldUtil.getComponentType(mp.field);
                    } else {
                        throw new RuntimeException("OneToMany and ManyToMany Relation, Must use collection or array object");
                    }
                } else {
                    itemClass = fieldClass;
                }
                if (itemClass == claxx2) {
                    ArrayList<String> key1List = new ArrayList<>();
                    HashMap<String, Object> map1 = new HashMap<>();
                    for (Object o1 : col1) {
                        if (o1 != null && (key1 = FieldUtil.get(table12.key.field, o1)) != null) {
                            key1List.add(key1.toString());
                            map1.put(key1.toString(), o1);
                        }
                    }
                    ArrayList<RelationKey> relationKeys = queryRelation(claxx12, claxx2, key1List);
                    if (!Checker.isEmpty(relationKeys)) {
                        HashMap<String, Object> map2 = new HashMap<>();
                        for (Object o2 : col2) {
                            if (o2 == null) {
                                claxx1 = claxx12;
                            } else {
                                Object key2 = FieldUtil.get(table2.key.field, o2);
                                if (key2 == null) {
                                    claxx1 = claxx12;
                                } else {
                                    claxx1 = claxx12;
                                    map2.put(key2.toString(), o2);
                                }
                            }
                            claxx12 = claxx1;
                        }
                        HashMap<Object, ArrayList> collMap2 = new HashMap<>();
                        Iterator i$3 = relationKeys.iterator();
                        while (i$3.hasNext()) {
                            RelationKey m = i$3.next();
                            Object obj1 = map1.get(m.key1);
                            Class claxx22 = claxx2;
                            Object obj2 = map2.get(m.key2);
                            if (obj1 == null || obj2 == null) {
                                table1 = table12;
                            } else if (mp.isToMany()) {
                                ArrayList col3 = collMap2.get(obj1);
                                if (col3 != null) {
                                    table1 = table12;
                                    col = col3;
                                } else {
                                    table1 = table12;
                                    col = new ArrayList();
                                    collMap2.put(obj1, col);
                                }
                                col.add(obj2);
                            } else {
                                table1 = table12;
                                FieldUtil.set(mp.field, obj1, obj2);
                            }
                            claxx2 = claxx22;
                            table12 = table1;
                        }
                        if (!Checker.isEmpty(collMap2)) {
                            Iterator i$4 = collMap2.entrySet().iterator();
                            while (i$4.hasNext()) {
                                Map.Entry<Object, ArrayList> entry = i$4.next();
                                Object obj12 = entry.getKey();
                                ArrayList value = entry.getValue();
                                if (ClassUtil.isCollection(itemClass)) {
                                    Collection col4 = (Collection) FieldUtil.get(mp.field, obj12);
                                    if (col4 == null) {
                                        collMap = collMap2;
                                        FieldUtil.set(mp.field, obj12, value);
                                    } else {
                                        collMap = collMap2;
                                        col4.addAll(value);
                                    }
                                    i$ = i$4;
                                } else {
                                    collMap = collMap2;
                                    if (!ClassUtil.isArray(itemClass)) {
                                        i$ = i$4;
                                    } else {
                                        Object[] tempArray = (Object[]) ClassUtil.newArray(itemClass, value.size());
                                        value.toArray(tempArray);
                                        Object[] array = (Object[]) FieldUtil.get(mp.field, obj12);
                                        if (array == null) {
                                            i$ = i$4;
                                            FieldUtil.set(mp.field, obj12, tempArray);
                                        } else {
                                            i$ = i$4;
                                            Object[] newArray = DataUtil.concat(array, tempArray);
                                            FieldUtil.set(mp.field, obj12, newArray);
                                        }
                                    }
                                }
                                collMap2 = collMap;
                                i$4 = i$;
                            }
                            return true;
                        }
                        return true;
                    }
                }
                claxx12 = claxx12;
                claxx2 = claxx2;
                table12 = table12;
            }
            return false;
        }
        return false;
    }
}
