package com.qiniu.android.dns;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.Uri;
import android.text.TextUtils;
import com.qiniu.android.dns.NetworkInfo;
import java.util.Locale;
/* loaded from: classes2.dex */
public final class NetworkReceiver extends BroadcastReceiver {
    private static final Uri PREFERRED_APN_URI = Uri.parse("content://telephony/carriers/preferapn");
    private static DnsManager mdnsManager;

    public static NetworkInfo createNetInfo(android.net.NetworkInfo info, Context context) {
        NetworkInfo.NetSatus net;
        String netMode;
        if (info == null) {
            return NetworkInfo.noNetwork;
        }
        int provider = 0;
        int main = info.getType();
        if (main == 1) {
            net = NetworkInfo.NetSatus.WIFI;
            provider = 0;
        } else {
            NetworkInfo.NetSatus net2 = NetworkInfo.NetSatus.MOBILE;
            Cursor c = context.getContentResolver().query(PREFERRED_APN_URI, null, null, null, null);
            if (c != null) {
                c.moveToFirst();
                String user = c.getString(c.getColumnIndex("user"));
                if (!TextUtils.isEmpty(user) && (user.startsWith("ctwap") || user.startsWith("ctnet"))) {
                    provider = 1;
                }
            }
            c.close();
            if (provider != 1 && (netMode = info.getExtraInfo()) != null) {
                String netMode2 = netMode.toLowerCase(Locale.getDefault());
                if (netMode2.equals("cmwap") || netMode2.equals("cmnet")) {
                    provider = 3;
                    net = net2;
                } else if (netMode2.equals("3gnet") || netMode2.equals("uninet") || netMode2.equals("3gwap") || netMode2.equals("uniwap")) {
                    provider = 2;
                    net = net2;
                }
            }
            net = net2;
        }
        return new NetworkInfo(net, provider);
    }

    public static void setDnsManager(DnsManager dnsManager) {
        mdnsManager = dnsManager;
    }

    @Override // android.content.BroadcastReceiver
    public void onReceive(Context context, Intent intent) {
        if (mdnsManager == null) {
            return;
        }
        ConnectivityManager manager = (ConnectivityManager) context.getSystemService("connectivity");
        android.net.NetworkInfo activeInfo = manager.getActiveNetworkInfo();
        NetworkInfo info = createNetInfo(activeInfo, context);
        mdnsManager.onNetworkChange(info);
    }
}
