package com.qiniu.android.dns.dns;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.IDN;
/* loaded from: classes2.dex */
class DnsRequest extends DnsMessage {
    private final String host;
    private final int recordType;

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getRecordType() {
        return this.recordType;
    }

    String getHost() {
        return this.host;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public DnsRequest(short messageId, int recordType, String host) {
        this(messageId, 0, 1, recordType, host);
    }

    DnsRequest(short messageId, int opCode, int rd, int recordType, String host) {
        this.messageId = messageId;
        this.opCode = opCode;
        this.rd = rd;
        this.recordType = recordType;
        this.host = host;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public byte[] toDnsQuestionData() throws IOException {
        String[] split;
        String str = this.host;
        if (str == null || str.length() == 0) {
            throw new IOException("host can not empty");
        }
        if (this.opCode != 0 && this.opCode != 1 && this.opCode != 2 && this.opCode != 5) {
            throw new IOException("opCode is not valid");
        }
        if (this.rd != 0 && this.rd != 1) {
            throw new IOException("rd is not valid");
        }
        int i = this.recordType;
        if (i != 1 && i != 28 && i != 5 && i != 16) {
            throw new IOException("recordType is not valid");
        }
        ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
        DataOutputStream dos = new DataOutputStream(baos);
        dos.writeShort(this.messageId);
        dos.writeByte((this.opCode << 3) + this.rd);
        dos.writeByte(0);
        dos.writeByte(0);
        dos.writeByte(1);
        dos.writeByte(0);
        dos.writeByte(0);
        dos.writeByte(0);
        dos.writeByte(0);
        dos.writeByte(0);
        dos.writeByte(0);
        for (String s : this.host.split("[.。．｡]")) {
            if (s.length() > 63) {
                throw new IOException("host part is too long");
            }
            byte[] buffer = IDN.toASCII(s).getBytes();
            dos.write(buffer.length);
            dos.write(buffer, 0, buffer.length);
        }
        dos.writeByte(0);
        dos.writeByte(0);
        dos.writeByte(this.recordType);
        dos.writeByte(0);
        dos.writeByte(1);
        return baos.toByteArray();
    }
}
