package com.qiniu.android.dns.dns;

import com.qiniu.android.dns.dns.DnsResolver;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.concurrent.ExecutorService;
import javax.net.ssl.HttpsURLConnection;
/* loaded from: classes2.dex */
public class DohResolver extends DnsResolver {
    public DohResolver(String server) {
        super(server);
    }

    public DohResolver(String server, int timeout) {
        super(server, timeout);
    }

    public DohResolver(String server, int recordType, int timeout) {
        super(server, recordType, timeout);
    }

    public DohResolver(String[] servers, int recordType, int timeout) {
        super(servers, recordType, timeout);
    }

    public DohResolver(String[] servers, int recordType, int timeout, ExecutorService executorService) {
        super(servers, recordType, timeout, executorService);
    }

    @Override // com.qiniu.android.dns.dns.DnsResolver
    DnsResponse request(DnsResolver.RequestCanceller canceller, String server, String host, int recordType) throws IOException {
        int length;
        double d = Math.random();
        short messageId = (short) (65535.0d * d);
        DnsRequest request = new DnsRequest(messageId, recordType, host);
        byte[] requestData = request.toDnsQuestionData();
        final HttpsURLConnection httpConn = (HttpsURLConnection) new URL(server).openConnection();
        httpConn.setConnectTimeout(3000);
        httpConn.setReadTimeout(this.timeout * 1000);
        httpConn.setDoOutput(true);
        httpConn.setRequestMethod("POST");
        httpConn.setRequestProperty("Content-Type", "application/dns-message");
        httpConn.setRequestProperty("Accept", "application/dns-message");
        httpConn.setRequestProperty("Accept-Encoding", "");
        httpConn.setRequestProperty("User-Agent", "Dalvik/2.1.0");
        final DataOutputStream bodyStream = new DataOutputStream(httpConn.getOutputStream());
        canceller.addCancelAction(new Runnable() { // from class: com.qiniu.android.dns.dns.DohResolver.1
            @Override // java.lang.Runnable
            public void run() {
                try {
                    httpConn.disconnect();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                try {
                    bodyStream.close();
                } catch (IOException e2) {
                    e2.printStackTrace();
                }
            }
        });
        bodyStream.write(requestData);
        bodyStream.close();
        int responseCode = httpConn.getResponseCode();
        if (responseCode == 200 && (length = httpConn.getContentLength()) > 0 && length <= 1048576) {
            InputStream is = httpConn.getInputStream();
            byte[] responseData = new byte[length];
            int read = is.read(responseData);
            is.close();
            if (read <= 0) {
                return null;
            }
            return new DnsResponse(server, 5, request, responseData);
        }
        return null;
    }
}
