package com.qiniu.android.dns.local;

import com.qiniu.android.dns.Domain;
import com.qiniu.android.dns.IResolver;
import com.qiniu.android.dns.NetworkInfo;
import com.qiniu.android.dns.Record;
import com.qiniu.android.dns.dns.DnsUdpResolver;
import java.io.IOException;
/* loaded from: classes2.dex */
public final class HijackingDetectWrapper implements IResolver {
    private final DnsUdpResolver resolver;

    public HijackingDetectWrapper(DnsUdpResolver r) {
        this.resolver = r;
    }

    @Override // com.qiniu.android.dns.IResolver
    public Record[] resolve(Domain domain, NetworkInfo info) throws IOException {
        Record[] records = this.resolver.resolve(domain, info);
        if (domain.hasCname) {
            boolean cname = false;
            String server = null;
            int length = records.length;
            int i = 0;
            while (true) {
                if (i >= length) {
                    break;
                }
                Record r = records[i];
                if (!r.isCname()) {
                    i++;
                } else {
                    cname = true;
                    server = r.server;
                    break;
                }
            }
            if (!cname) {
                throw new DnshijackingException(domain.domain, server);
            }
        }
        if (domain.maxTtl != 0) {
            for (Record r2 : records) {
                if (!r2.isCname() && r2.ttl > domain.maxTtl) {
                    throw new DnshijackingException(domain.domain, r2.server, r2.ttl);
                }
            }
        }
        return records;
    }
}
