package com.qiniu.android.dns.local;

import com.qiniu.android.dns.Domain;
import com.qiniu.android.dns.NetworkInfo;
import com.qiniu.android.dns.Record;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
/* loaded from: classes2.dex */
public final class Hosts {
    private final Map<String, LinkedList<Value>> hosts = new ConcurrentHashMap();

    public synchronized Record[] query(Domain domain, NetworkInfo info) {
        LinkedList<Value> values = this.hosts.get(domain.domain);
        if (values != null && !values.isEmpty()) {
            if (values.size() > 1) {
                Value first = values.get(0);
                values.remove(0);
                values.add(first);
            }
            return toRecords(filter(values, info));
        }
        return null;
    }

    private LinkedList<Value> filter(LinkedList<Value> origin, NetworkInfo info) {
        if (origin == null) {
            return null;
        }
        LinkedList<Value> normal = new LinkedList<>();
        LinkedList<Value> special = new LinkedList<>();
        Iterator<Value> it = origin.iterator();
        while (it.hasNext()) {
            Value v = it.next();
            if (v.provider == 0) {
                normal.add(v);
            } else if (v.provider == info.provider) {
                special.add(v);
            }
        }
        if (special.size() != 0) {
            return special;
        }
        return normal;
    }

    private Record[] toRecords(LinkedList<Value> vals) {
        if (vals == null) {
            return null;
        }
        int size = vals.size();
        List<Record> records = new ArrayList<>();
        for (int i = 0; i < size; i++) {
            Value v = vals.get(i);
            if (v != null && v.record != null && v.record.value != null) {
                records.add(v.record);
            }
        }
        int i2 = records.size();
        return (Record[]) records.toArray(new Record[i2]);
    }

    public synchronized Hosts put(String domain, Value val) {
        LinkedList<Value> vals = this.hosts.get(domain);
        if (vals == null) {
            vals = new LinkedList<>();
        }
        vals.add(val);
        this.hosts.put(domain, vals);
        return this;
    }

    public Hosts put(String domain, Record record) {
        put(domain, new Value(record));
        return this;
    }

    /* loaded from: classes2.dex */
    public static class Value {
        public final int provider;
        public final Record record;

        public Value(Record record, int provider) {
            this.record = record;
            this.provider = provider;
        }

        public Value(Record record) {
            this(record, 0);
        }

        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (o instanceof Value) {
                Value another = (Value) o;
                Record record = this.record;
                Record record2 = another.record;
                if (record == record2) {
                    return true;
                }
                return record != null && record2 != null && record.value.equals(another.record.value) && this.provider == another.provider;
            }
            return false;
        }
    }
}
