package com.serenegiant.usb;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.util.SparseArray;
import com.google.android.exoplayer2.trackselection.AdaptiveTrackSelection;
import com.king.zxing.util.LogUtils;
import com.serenegiant.common.BaseBuildCheck;
import com.serenegiant.common.BaseHandler;
import java.io.UnsupportedEncodingException;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import kotlin.UByte;
/* loaded from: classes19.dex */
public final class USBMonitor {
    public static final String ACTION_USB_DEVICE_ATTACHED = "android.hardware.usb.action.USB_DEVICE_ATTACHED";
    private static final String ACTION_USB_PERMISSION_BASE = "com.serenegiant.USB_PERMISSION.";
    private static final boolean DEBUG = false;
    private static final String TAG = "USBMonitor";
    private static final int USB_DIR_IN = 128;
    private static final int USB_DIR_OUT = 0;
    private static final int USB_DT_BOS = 15;
    private static final int USB_DT_CONFIG = 2;
    private static final int USB_DT_CS_CONFIG = 34;
    private static final int USB_DT_CS_DEVICE = 33;
    private static final int USB_DT_CS_ENDPOINT = 37;
    private static final int USB_DT_CS_INTERFACE = 36;
    private static final int USB_DT_CS_RADIO_CONTROL = 35;
    private static final int USB_DT_CS_STRING = 35;
    private static final int USB_DT_DEBUG = 10;
    private static final int USB_DT_DEVICE = 1;
    private static final int USB_DT_DEVICE_CAPABILITY = 16;
    private static final int USB_DT_DEVICE_QUALIFIER = 6;
    private static final int USB_DT_DEVICE_SIZE = 18;
    private static final int USB_DT_ENCRYPTION_TYPE = 14;
    private static final int USB_DT_ENDPOINT = 5;
    private static final int USB_DT_INTERFACE = 4;
    private static final int USB_DT_INTERFACE_ASSOCIATION = 11;
    private static final int USB_DT_INTERFACE_POWER = 8;
    private static final int USB_DT_KEY = 13;
    private static final int USB_DT_OTG = 9;
    private static final int USB_DT_OTHER_SPEED_CONFIG = 7;
    private static final int USB_DT_PIPE_USAGE = 36;
    private static final int USB_DT_RPIPE = 34;
    private static final int USB_DT_SECURITY = 12;
    private static final int USB_DT_SS_ENDPOINT_COMP = 48;
    private static final int USB_DT_STRING = 3;
    private static final int USB_DT_WIRELESS_ENDPOINT_COMP = 17;
    private static final int USB_DT_WIRE_ADAPTER = 33;
    private static final int USB_RECIP_DEVICE = 0;
    private static final int USB_RECIP_ENDPOINT = 2;
    private static final int USB_RECIP_INTERFACE = 1;
    private static final int USB_RECIP_MASK = 31;
    private static final int USB_RECIP_OTHER = 3;
    private static final int USB_RECIP_PORT = 4;
    private static final int USB_RECIP_RPIPE = 5;
    private static final int USB_REQ_CLEAR_FEATURE = 1;
    private static final int USB_REQ_CS_DEVICE_GET = 160;
    private static final int USB_REQ_CS_DEVICE_SET = 32;
    private static final int USB_REQ_CS_ENDPOINT_GET = 162;
    private static final int USB_REQ_CS_ENDPOINT_SET = 34;
    private static final int USB_REQ_CS_INTERFACE_GET = 161;
    private static final int USB_REQ_CS_INTERFACE_SET = 33;
    private static final int USB_REQ_GET_CONFIGURATION = 8;
    private static final int USB_REQ_GET_DESCRIPTOR = 6;
    private static final int USB_REQ_GET_ENCRYPTION = 14;
    private static final int USB_REQ_GET_HANDSHAKE = 16;
    private static final int USB_REQ_GET_INTERFACE = 10;
    private static final int USB_REQ_GET_SECURITY_DATA = 19;
    private static final int USB_REQ_GET_STATUS = 0;
    private static final int USB_REQ_LOOPBACK_DATA_READ = 22;
    private static final int USB_REQ_LOOPBACK_DATA_WRITE = 21;
    private static final int USB_REQ_RPIPE_ABORT = 14;
    private static final int USB_REQ_RPIPE_RESET = 15;
    private static final int USB_REQ_SET_ADDRESS = 5;
    private static final int USB_REQ_SET_CONFIGURATION = 9;
    private static final int USB_REQ_SET_CONNECTION = 17;
    private static final int USB_REQ_SET_DESCRIPTOR = 7;
    private static final int USB_REQ_SET_ENCRYPTION = 13;
    private static final int USB_REQ_SET_FEATURE = 3;
    private static final int USB_REQ_SET_HANDSHAKE = 15;
    private static final int USB_REQ_SET_INTERFACE = 11;
    private static final int USB_REQ_SET_INTERFACE_DS = 23;
    private static final int USB_REQ_SET_ISOCH_DELAY = 49;
    private static final int USB_REQ_SET_SECURITY_DATA = 18;
    private static final int USB_REQ_SET_SEL = 48;
    private static final int USB_REQ_SET_WUSB_DATA = 20;
    private static final int USB_REQ_STANDARD_DEVICE_GET = 128;
    private static final int USB_REQ_STANDARD_DEVICE_SET = 0;
    private static final int USB_REQ_STANDARD_ENDPOINT_GET = 130;
    private static final int USB_REQ_STANDARD_ENDPOINT_SET = 2;
    private static final int USB_REQ_STANDARD_INTERFACE_GET = 129;
    private static final int USB_REQ_STANDARD_INTERFACE_SET = 1;
    private static final int USB_REQ_SYNCH_FRAME = 12;
    private static final int USB_REQ_VENDER_DEVICE_GET = 160;
    private static final int USB_REQ_VENDER_DEVICE_SET = 32;
    private static final int USB_REQ_VENDER_ENDPOINT_GET = 162;
    private static final int USB_REQ_VENDER_ENDPOINT_SET = 34;
    private static final int USB_REQ_VENDER_INTERFACE_GET = 161;
    private static final int USB_REQ_VENDER_INTERFACE_SET = 33;
    private static final int USB_TYPE_CLASS = 32;
    private static final int USB_TYPE_MASK = 96;
    private static final int USB_TYPE_RESERVED = 96;
    private static final int USB_TYPE_STANDARD = 0;
    private static final int USB_TYPE_VENDOR = 64;
    private volatile boolean destroyed;
    private final Handler mAsyncHandler;
    private final OnDeviceConnectListener mOnDeviceConnectListener;
    private final UsbManager mUsbManager;
    private final WeakReference<Context> mWeakContext;
    private final String ACTION_USB_PERMISSION = ACTION_USB_PERMISSION_BASE + hashCode();
    private final ConcurrentHashMap<UsbDevice, UsbControlBlock> mCtrlBlocks = new ConcurrentHashMap<>();
    private final SparseArray<WeakReference<UsbDevice>> mHasPermissions = new SparseArray<>();
    private PendingIntent mPermissionIntent = null;
    private List<DeviceFilter> mDeviceFilters = new ArrayList();
    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() { // from class: com.serenegiant.usb.USBMonitor.1
        @Override // android.content.BroadcastReceiver
        public void onReceive(Context context, Intent intent) {
            UsbDevice device;
            if (USBMonitor.this.destroyed) {
                return;
            }
            String action = intent.getAction();
            if (USBMonitor.this.ACTION_USB_PERMISSION.equals(action)) {
                synchronized (USBMonitor.this) {
                    UsbDevice device2 = (UsbDevice) intent.getParcelableExtra("device");
                    if (!intent.getBooleanExtra("permission", false)) {
                        USBMonitor.this.processCancel(device2);
                    } else if (device2 != null) {
                        USBMonitor.this.processConnect(device2);
                    }
                }
            } else if (USBMonitor.ACTION_USB_DEVICE_ATTACHED.equals(action)) {
                UsbDevice device3 = (UsbDevice) intent.getParcelableExtra("device");
                USBMonitor uSBMonitor = USBMonitor.this;
                uSBMonitor.updatePermission(device3, uSBMonitor.hasPermission(device3));
                USBMonitor.this.processAttach(device3);
            } else if ("android.hardware.usb.action.USB_DEVICE_DETACHED".equals(action) && (device = (UsbDevice) intent.getParcelableExtra("device")) != null) {
                UsbControlBlock ctrlBlock = (UsbControlBlock) USBMonitor.this.mCtrlBlocks.remove(device);
                if (ctrlBlock != null) {
                    ctrlBlock.close();
                }
                USBMonitor.this.mDeviceCounts = 0;
                USBMonitor.this.processDettach(device);
            }
        }
    };
    private volatile int mDeviceCounts = 0;
    private final Runnable mDeviceCheckRunnable = new Runnable() { // from class: com.serenegiant.usb.USBMonitor.2
        @Override // java.lang.Runnable
        public void run() {
            int hasPermissionCounts;
            int m;
            if (USBMonitor.this.destroyed) {
                return;
            }
            List<UsbDevice> devices = USBMonitor.this.getDeviceList();
            int n = devices.size();
            synchronized (USBMonitor.this.mHasPermissions) {
                hasPermissionCounts = USBMonitor.this.mHasPermissions.size();
                USBMonitor.this.mHasPermissions.clear();
                for (UsbDevice device : devices) {
                    USBMonitor.this.hasPermission(device);
                }
                m = USBMonitor.this.mHasPermissions.size();
            }
            if (n > USBMonitor.this.mDeviceCounts || m > hasPermissionCounts) {
                USBMonitor.this.mDeviceCounts = n;
                if (USBMonitor.this.mOnDeviceConnectListener != null) {
                    for (int i = 0; i < n; i++) {
                        final UsbDevice device2 = devices.get(i);
                        USBMonitor.this.mAsyncHandler.post(new Runnable() { // from class: com.serenegiant.usb.USBMonitor.2.1
                            @Override // java.lang.Runnable
                            public void run() {
                                USBMonitor.this.mOnDeviceConnectListener.onAttach(device2);
                            }
                        });
                    }
                }
            }
            USBMonitor.this.mAsyncHandler.postDelayed(this, AdaptiveTrackSelection.DEFAULT_MIN_TIME_BETWEEN_BUFFER_REEVALUTATION_MS);
        }
    };

    /* loaded from: classes19.dex */
    public interface OnDeviceConnectListener {
        void onAttach(UsbDevice usbDevice);

        void onCancel(UsbDevice usbDevice);

        void onConnect(UsbDevice usbDevice, UsbControlBlock usbControlBlock, boolean z);

        void onDettach(UsbDevice usbDevice);

        void onDisconnect(UsbDevice usbDevice, UsbControlBlock usbControlBlock);
    }

    public USBMonitor(Context context, OnDeviceConnectListener listener) {
        if (listener == null) {
            throw new IllegalArgumentException("OnDeviceConnectListener should not null.");
        }
        this.mWeakContext = new WeakReference<>(context);
        this.mUsbManager = (UsbManager) context.getSystemService("usb");
        this.mOnDeviceConnectListener = listener;
        this.mAsyncHandler = BaseHandler.createHandler(TAG);
        this.destroyed = false;
    }

    public void destroy() {
        unregister();
        if (!this.destroyed) {
            this.destroyed = true;
            Set<UsbDevice> keys = this.mCtrlBlocks.keySet();
            if (keys != null) {
                try {
                    for (UsbDevice key : keys) {
                        UsbControlBlock ctrlBlock = this.mCtrlBlocks.remove(key);
                        if (ctrlBlock != null) {
                            ctrlBlock.close();
                        }
                    }
                } catch (Exception e) {
                    Log.e(TAG, "destroy:", e);
                }
            }
            this.mCtrlBlocks.clear();
            try {
                this.mAsyncHandler.getLooper().quit();
            } catch (Exception e2) {
                Log.e(TAG, "destroy:", e2);
            }
        }
    }

    public synchronized void register() throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        if (this.mPermissionIntent == null) {
            Context context = this.mWeakContext.get();
            if (context != null) {
                this.mPermissionIntent = PendingIntent.getBroadcast(context, 0, new Intent(this.ACTION_USB_PERMISSION), 0);
                IntentFilter filter = new IntentFilter(this.ACTION_USB_PERMISSION);
                filter.addAction("android.hardware.usb.action.USB_DEVICE_DETACHED");
                context.registerReceiver(this.mUsbReceiver, filter);
            }
            this.mDeviceCounts = 0;
            this.mAsyncHandler.postDelayed(this.mDeviceCheckRunnable, 1000L);
        }
    }

    public synchronized void unregister() throws IllegalStateException {
        this.mDeviceCounts = 0;
        if (!this.destroyed) {
            this.mAsyncHandler.removeCallbacks(this.mDeviceCheckRunnable);
        }
        if (this.mPermissionIntent != null) {
            Context context = this.mWeakContext.get();
            if (context != null) {
                try {
                    context.unregisterReceiver(this.mUsbReceiver);
                } catch (Exception e) {
                    Log.w(TAG, e);
                }
            }
            this.mPermissionIntent = null;
        }
    }

    public synchronized boolean isRegistered() {
        boolean z;
        if (!this.destroyed) {
            z = this.mPermissionIntent != null;
        }
        return z;
    }

    public void setDeviceFilter(DeviceFilter filter) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        this.mDeviceFilters.clear();
        this.mDeviceFilters.add(filter);
    }

    public void addDeviceFilter(DeviceFilter filter) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        this.mDeviceFilters.add(filter);
    }

    public void removeDeviceFilter(DeviceFilter filter) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        this.mDeviceFilters.remove(filter);
    }

    public void setDeviceFilter(List<DeviceFilter> filters) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        this.mDeviceFilters.clear();
        this.mDeviceFilters.addAll(filters);
    }

    public void addDeviceFilter(List<DeviceFilter> filters) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        this.mDeviceFilters.addAll(filters);
    }

    public void removeDeviceFilter(List<DeviceFilter> filters) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        this.mDeviceFilters.removeAll(filters);
    }

    public int getDeviceCount() throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        return getDeviceList().size();
    }

    public List<UsbDevice> getDeviceList() throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        return getDeviceList(this.mDeviceFilters);
    }

    public List<UsbDevice> getDeviceList(List<DeviceFilter> filters) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        HashMap<String, UsbDevice> deviceList = this.mUsbManager.getDeviceList();
        List<UsbDevice> result = new ArrayList<>();
        if (deviceList != null) {
            if (filters == null || filters.isEmpty()) {
                result.addAll(deviceList.values());
            } else {
                for (UsbDevice device : deviceList.values()) {
                    Iterator<DeviceFilter> it = filters.iterator();
                    while (true) {
                        if (!it.hasNext()) {
                            break;
                        }
                        DeviceFilter filter = it.next();
                        if (filter != null && filter.matches(device)) {
                            if (!filter.isExclude) {
                                result.add(device);
                            }
                        }
                    }
                }
            }
        }
        return result;
    }

    public List<UsbDevice> getDeviceList(DeviceFilter filter) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        HashMap<String, UsbDevice> deviceList = this.mUsbManager.getDeviceList();
        List<UsbDevice> result = new ArrayList<>();
        if (deviceList != null) {
            for (UsbDevice device : deviceList.values()) {
                if (filter == null || (filter.matches(device) && !filter.isExclude)) {
                    result.add(device);
                }
            }
        }
        return result;
    }

    public Iterator<UsbDevice> getDevices() throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        HashMap<String, UsbDevice> list = this.mUsbManager.getDeviceList();
        if (list == null) {
            return null;
        }
        Iterator<UsbDevice> iterator = list.values().iterator();
        return iterator;
    }

    public final void dumpDevices() {
        HashMap<String, UsbDevice> list = this.mUsbManager.getDeviceList();
        if (list != null) {
            Set<String> keys = list.keySet();
            if (keys != null && keys.size() > 0) {
                StringBuilder sb = new StringBuilder();
                for (String key : keys) {
                    UsbDevice device = list.get(key);
                    int num_interface = device != null ? device.getInterfaceCount() : 0;
                    sb.setLength(0);
                    for (int i = 0; i < num_interface; i++) {
                        sb.append(String.format(Locale.US, "interface%d:%s", Integer.valueOf(i), device.getInterface(i).toString()));
                    }
                    Log.i(TAG, "key=" + key + LogUtils.COLON + device + LogUtils.COLON + sb.toString());
                }
                return;
            }
            Log.i(TAG, "no device");
            return;
        }
        Log.i(TAG, "no device");
    }

    public final boolean hasPermission(UsbDevice device) throws IllegalStateException {
        if (this.destroyed) {
            throw new IllegalStateException("already destroyed");
        }
        return updatePermission(device, device != null && this.mUsbManager.hasPermission(device));
    }

    /* JADX INFO: Access modifiers changed from: private */
    public boolean updatePermission(UsbDevice device, boolean hasPermission) {
        int deviceKey = getDeviceKey(device, true);
        synchronized (this.mHasPermissions) {
            if (hasPermission) {
                if (this.mHasPermissions.get(deviceKey) == null) {
                    this.mHasPermissions.put(deviceKey, new WeakReference<>(device));
                }
            } else {
                this.mHasPermissions.remove(deviceKey);
            }
        }
        return hasPermission;
    }

    public synchronized boolean requestPermission(UsbDevice device) {
        boolean result;
        result = false;
        if (isRegistered()) {
            if (device != null) {
                if (this.mUsbManager.hasPermission(device)) {
                    processConnect(device);
                } else {
                    try {
                        this.mUsbManager.requestPermission(device, this.mPermissionIntent);
                    } catch (Exception e) {
                        Log.w(TAG, e);
                        processCancel(device);
                        result = true;
                    }
                }
            } else {
                processCancel(device);
                result = true;
            }
        } else {
            processCancel(device);
            result = true;
        }
        return result;
    }

    public UsbControlBlock openDevice(UsbDevice device) throws SecurityException {
        if (hasPermission(device)) {
            UsbControlBlock result = this.mCtrlBlocks.get(device);
            if (result == null) {
                UsbControlBlock result2 = new UsbControlBlock(device);
                this.mCtrlBlocks.put(device, result2);
                return result2;
            }
            return result;
        }
        throw new SecurityException("has no permission");
    }

    /* JADX INFO: Access modifiers changed from: private */
    public final void processConnect(final UsbDevice device) {
        if (this.destroyed) {
            return;
        }
        updatePermission(device, true);
        this.mAsyncHandler.post(new Runnable() { // from class: com.serenegiant.usb.USBMonitor.3
            @Override // java.lang.Runnable
            public void run() {
                boolean createNew;
                UsbControlBlock ctrlBlock = (UsbControlBlock) USBMonitor.this.mCtrlBlocks.get(device);
                if (ctrlBlock == null) {
                    ctrlBlock = new UsbControlBlock(device);
                    USBMonitor.this.mCtrlBlocks.put(device, ctrlBlock);
                    createNew = true;
                } else {
                    createNew = false;
                }
                if (USBMonitor.this.mOnDeviceConnectListener != null) {
                    USBMonitor.this.mOnDeviceConnectListener.onConnect(device, ctrlBlock, createNew);
                }
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public final void processCancel(final UsbDevice device) {
        if (this.destroyed) {
            return;
        }
        updatePermission(device, false);
        if (this.mOnDeviceConnectListener != null) {
            this.mAsyncHandler.post(new Runnable() { // from class: com.serenegiant.usb.USBMonitor.4
                @Override // java.lang.Runnable
                public void run() {
                    USBMonitor.this.mOnDeviceConnectListener.onCancel(device);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public final void processAttach(final UsbDevice device) {
        if (!this.destroyed && this.mOnDeviceConnectListener != null) {
            this.mAsyncHandler.post(new Runnable() { // from class: com.serenegiant.usb.USBMonitor.5
                @Override // java.lang.Runnable
                public void run() {
                    USBMonitor.this.mOnDeviceConnectListener.onAttach(device);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public final void processDettach(final UsbDevice device) {
        if (!this.destroyed && this.mOnDeviceConnectListener != null) {
            this.mAsyncHandler.post(new Runnable() { // from class: com.serenegiant.usb.USBMonitor.6
                @Override // java.lang.Runnable
                public void run() {
                    USBMonitor.this.mOnDeviceConnectListener.onDettach(device);
                }
            });
        }
    }

    public static final String getDeviceKeyName(UsbDevice device) {
        return getDeviceKeyName(device, null, false);
    }

    public static final String getDeviceKeyName(UsbDevice device, boolean useNewAPI) {
        return getDeviceKeyName(device, null, useNewAPI);
    }

    public static final String getDeviceKeyName(UsbDevice device, String serial, boolean useNewAPI) {
        if (device == null) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        sb.append(device.getVendorId());
        sb.append("#");
        sb.append(device.getProductId());
        sb.append("#");
        sb.append(device.getDeviceClass());
        sb.append("#");
        sb.append(device.getDeviceSubclass());
        sb.append("#");
        sb.append(device.getDeviceProtocol());
        if (!TextUtils.isEmpty(serial)) {
            sb.append("#");
            sb.append(serial);
        }
        if (useNewAPI && BaseBuildCheck.isAndroid5()) {
            sb.append("#");
            if (TextUtils.isEmpty(serial)) {
                sb.append(device.getSerialNumber());
                sb.append("#");
            }
            sb.append(device.getManufacturerName());
            sb.append("#");
            sb.append(device.getConfigurationCount());
            sb.append("#");
            if (BaseBuildCheck.isMarshmallow()) {
                sb.append(device.getVersion());
                sb.append("#");
            }
        }
        return sb.toString();
    }

    public static final int getDeviceKey(UsbDevice device) {
        if (device != null) {
            return getDeviceKeyName(device, null, false).hashCode();
        }
        return 0;
    }

    public static final int getDeviceKey(UsbDevice device, boolean useNewAPI) {
        if (device != null) {
            return getDeviceKeyName(device, null, useNewAPI).hashCode();
        }
        return 0;
    }

    public static final int getDeviceKey(UsbDevice device, String serial, boolean useNewAPI) {
        if (device != null) {
            return getDeviceKeyName(device, serial, useNewAPI).hashCode();
        }
        return 0;
    }

    /* loaded from: classes19.dex */
    public static class UsbDeviceInfo {
        public String manufacturer;
        public String product;
        public String serial;
        public String usb_version;
        public String version;

        /* JADX INFO: Access modifiers changed from: private */
        public void clear() {
            this.serial = null;
            this.version = null;
            this.product = null;
            this.manufacturer = null;
            this.usb_version = null;
        }

        public String toString() {
            Object[] objArr = new Object[5];
            String str = this.usb_version;
            if (str == null) {
                str = "";
            }
            objArr[0] = str;
            String str2 = this.manufacturer;
            if (str2 == null) {
                str2 = "";
            }
            objArr[1] = str2;
            String str3 = this.product;
            if (str3 == null) {
                str3 = "";
            }
            objArr[2] = str3;
            String str4 = this.version;
            if (str4 == null) {
                str4 = "";
            }
            objArr[3] = str4;
            String str5 = this.serial;
            objArr[4] = str5 != null ? str5 : "";
            return String.format("UsbDevice:usb_version=%s,manufacturer=%s,product=%s,version=%s,serial=%s", objArr);
        }
    }

    private static String getString(UsbDeviceConnection connection, int id, int languageCount, byte[] languages) {
        byte[] work = new byte[256];
        String result = null;
        for (int i = 1; i <= languageCount; i++) {
            int ret = connection.controlTransfer(128, 6, id | 768, languages[i], work, 256, 0);
            if (ret > 2 && work[0] == ret && work[1] == 3) {
                try {
                    result = new String(work, 2, ret - 2, "UTF-16LE");
                    if (!"Љ".equals(result)) {
                        break;
                    }
                    result = null;
                } catch (UnsupportedEncodingException e) {
                }
            }
        }
        return result;
    }

    public UsbDeviceInfo getDeviceInfo(UsbDevice device) {
        return updateDeviceInfo(this.mUsbManager, device, null);
    }

    public static UsbDeviceInfo getDeviceInfo(Context context, UsbDevice device) {
        return updateDeviceInfo((UsbManager) context.getSystemService("usb"), device, new UsbDeviceInfo());
    }

    public static UsbDeviceInfo updateDeviceInfo(UsbManager manager, UsbDevice device, UsbDeviceInfo _info, UsbDeviceConnection connection) {
        byte[] desc;
        UsbDeviceInfo info = _info != null ? _info : new UsbDeviceInfo();
        info.clear();
        if (device != null) {
            if (BaseBuildCheck.isLollipop()) {
                info.manufacturer = device.getManufacturerName();
                info.product = device.getProductName();
                info.serial = device.getSerialNumber();
            }
            if (BaseBuildCheck.isMarshmallow()) {
                info.usb_version = device.getVersion();
            }
            if (manager != null && manager.hasPermission(device) && connection != null && (desc = connection.getRawDescriptors()) != null && desc.length >= 17) {
                if (TextUtils.isEmpty(info.usb_version)) {
                    info.usb_version = String.format("%x.%02x", Integer.valueOf(desc[3] & UByte.MAX_VALUE), Integer.valueOf(desc[2] & UByte.MAX_VALUE));
                }
                if (TextUtils.isEmpty(info.version)) {
                    info.version = String.format("%x.%02x", Integer.valueOf(desc[13] & UByte.MAX_VALUE), Integer.valueOf(desc[12] & UByte.MAX_VALUE));
                }
                if (TextUtils.isEmpty(info.serial)) {
                    info.serial = connection.getSerial();
                }
                byte[] languages = new byte[256];
                int languageCount = 0;
                try {
                    int result = connection.controlTransfer(128, 6, 768, 0, languages, 256, 0);
                    if (result > 0) {
                        languageCount = (result - 2) / 2;
                    }
                    if (languageCount > 0) {
                        if (TextUtils.isEmpty(info.manufacturer)) {
                            info.manufacturer = getString(connection, desc[14], languageCount, languages);
                        }
                        if (TextUtils.isEmpty(info.product)) {
                            info.product = getString(connection, desc[15], languageCount, languages);
                        }
                        if (TextUtils.isEmpty(info.serial)) {
                            info.serial = getString(connection, desc[16], languageCount, languages);
                        }
                    }
                } finally {
                    connection.close();
                }
            }
            if (TextUtils.isEmpty(info.manufacturer)) {
                info.manufacturer = USBVendorId.vendorName(device.getVendorId());
            }
            if (TextUtils.isEmpty(info.manufacturer)) {
                info.manufacturer = String.format("%04x", Integer.valueOf(device.getVendorId()));
            }
            if (TextUtils.isEmpty(info.product)) {
                info.product = String.format("%04x", Integer.valueOf(device.getProductId()));
            }
        }
        return info;
    }

    public static UsbDeviceInfo updateDeviceInfo(UsbManager manager, UsbDevice device, UsbDeviceInfo _info) {
        UsbDeviceInfo info = _info != null ? _info : new UsbDeviceInfo();
        info.clear();
        if (device != null) {
            if (BaseBuildCheck.isLollipop()) {
                info.manufacturer = device.getManufacturerName();
                info.product = device.getProductName();
                info.serial = device.getSerialNumber();
            }
            if (BaseBuildCheck.isMarshmallow()) {
                info.usb_version = device.getVersion();
            }
            if (manager != null && manager.hasPermission(device)) {
                UsbDeviceConnection connection = manager.openDevice(device);
                if (connection == null) {
                    return null;
                }
                byte[] desc = connection.getRawDescriptors();
                if (desc != null && desc.length >= 17) {
                    if (TextUtils.isEmpty(info.usb_version)) {
                        info.usb_version = String.format("%x.%02x", Integer.valueOf(desc[3] & UByte.MAX_VALUE), Integer.valueOf(desc[2] & UByte.MAX_VALUE));
                    }
                    if (TextUtils.isEmpty(info.version)) {
                        info.version = String.format("%x.%02x", Integer.valueOf(desc[13] & UByte.MAX_VALUE), Integer.valueOf(desc[12] & UByte.MAX_VALUE));
                    }
                    if (TextUtils.isEmpty(info.serial)) {
                        info.serial = connection.getSerial();
                    }
                    byte[] languages = new byte[256];
                    int languageCount = 0;
                    try {
                        int result = connection.controlTransfer(128, 6, 768, 0, languages, 256, 0);
                        if (result > 0) {
                            languageCount = (result - 2) / 2;
                        }
                        if (languageCount > 0) {
                            if (TextUtils.isEmpty(info.manufacturer)) {
                                info.manufacturer = getString(connection, desc[14], languageCount, languages);
                            }
                            if (TextUtils.isEmpty(info.product)) {
                                info.product = getString(connection, desc[15], languageCount, languages);
                            }
                            if (TextUtils.isEmpty(info.serial)) {
                                info.serial = getString(connection, desc[16], languageCount, languages);
                            }
                        }
                    } finally {
                        connection.close();
                    }
                }
            }
            if (TextUtils.isEmpty(info.manufacturer)) {
                info.manufacturer = USBVendorId.vendorName(device.getVendorId());
            }
            if (TextUtils.isEmpty(info.manufacturer)) {
                info.manufacturer = String.format("%04x", Integer.valueOf(device.getVendorId()));
            }
            if (TextUtils.isEmpty(info.product)) {
                info.product = String.format("%04x", Integer.valueOf(device.getProductId()));
            }
        }
        return info;
    }

    /* loaded from: classes19.dex */
    public static final class UsbControlBlock implements Cloneable {
        private final int mBusNum;
        protected UsbDeviceConnection mConnection;
        private final int mDevNum;
        protected final UsbDeviceInfo mInfo;
        private final SparseArray<SparseArray<UsbInterface>> mInterfaces;
        private final WeakReference<UsbDevice> mWeakDevice;
        private final WeakReference<USBMonitor> mWeakMonitor;

        private UsbControlBlock(USBMonitor monitor, UsbDevice device) {
            this.mInterfaces = new SparseArray<>();
            this.mWeakMonitor = new WeakReference<>(monitor);
            this.mWeakDevice = new WeakReference<>(device);
            this.mConnection = monitor.mUsbManager.openDevice(device);
            Log.e(USBMonitor.TAG, "openDevice");
            this.mInfo = USBMonitor.updateDeviceInfo(monitor.mUsbManager, device, null, this.mConnection);
            String name = device.getDeviceName();
            String[] v = TextUtils.isEmpty(name) ? null : name.split("/");
            int busnum = 0;
            int devnum = 0;
            if (v != null) {
                busnum = Integer.parseInt(v[v.length - 2]);
                devnum = Integer.parseInt(v[v.length - 1]);
            }
            this.mBusNum = busnum;
            this.mDevNum = devnum;
            UsbDeviceConnection usbDeviceConnection = this.mConnection;
            if (usbDeviceConnection != null) {
                int desc = usbDeviceConnection.getFileDescriptor();
                byte[] rawDesc = this.mConnection.getRawDescriptors();
                Log.i(USBMonitor.TAG, String.format(Locale.US, "name=%s,desc=%d,busnum=%d,devnum=%d,rawDesc=", name, Integer.valueOf(desc), Integer.valueOf(busnum), Integer.valueOf(devnum)) + rawDesc);
                return;
            }
            Log.e(USBMonitor.TAG, "could not connect to device " + name);
        }

        private UsbControlBlock(UsbControlBlock src) throws IllegalStateException {
            this.mInterfaces = new SparseArray<>();
            USBMonitor monitor = src.getUSBMonitor();
            UsbDevice device = src.getDevice();
            if (device != null) {
                UsbDeviceConnection openDevice = monitor.mUsbManager.openDevice(device);
                this.mConnection = openDevice;
                if (openDevice != null) {
                    this.mInfo = USBMonitor.updateDeviceInfo(monitor.mUsbManager, device, null);
                    this.mWeakMonitor = new WeakReference<>(monitor);
                    this.mWeakDevice = new WeakReference<>(device);
                    this.mBusNum = src.mBusNum;
                    this.mDevNum = src.mDevNum;
                    return;
                }
                throw new IllegalStateException("device may already be removed or have no permission");
            }
            throw new IllegalStateException("device may already be removed");
        }

        /* renamed from: clone */
        public UsbControlBlock m92clone() throws CloneNotSupportedException {
            try {
                UsbControlBlock ctrlblock = new UsbControlBlock(this);
                return ctrlblock;
            } catch (IllegalStateException e) {
                throw new CloneNotSupportedException(e.getMessage());
            }
        }

        public USBMonitor getUSBMonitor() {
            return this.mWeakMonitor.get();
        }

        public final UsbDevice getDevice() {
            return this.mWeakDevice.get();
        }

        public String getDeviceName() {
            UsbDevice device = this.mWeakDevice.get();
            return device != null ? device.getDeviceName() : "";
        }

        public int getDeviceId() {
            UsbDevice device = this.mWeakDevice.get();
            if (device != null) {
                return device.getDeviceId();
            }
            return 0;
        }

        public String getDeviceKeyName() {
            return USBMonitor.getDeviceKeyName(this.mWeakDevice.get());
        }

        public String getDeviceKeyName(boolean useNewAPI) throws IllegalStateException {
            if (useNewAPI) {
                checkConnection();
            }
            return USBMonitor.getDeviceKeyName(this.mWeakDevice.get(), this.mInfo.serial, useNewAPI);
        }

        public int getDeviceKey() throws IllegalStateException {
            checkConnection();
            return USBMonitor.getDeviceKey(this.mWeakDevice.get());
        }

        public int getDeviceKey(boolean useNewAPI) throws IllegalStateException {
            if (useNewAPI) {
                checkConnection();
            }
            return USBMonitor.getDeviceKey(this.mWeakDevice.get(), this.mInfo.serial, useNewAPI);
        }

        public String getDeviceKeyNameWithSerial() {
            return USBMonitor.getDeviceKeyName(this.mWeakDevice.get(), this.mInfo.serial, false);
        }

        public int getDeviceKeyWithSerial() {
            return getDeviceKeyNameWithSerial().hashCode();
        }

        public synchronized UsbDeviceConnection getConnection() {
            return this.mConnection;
        }

        public synchronized int getFileDescriptor() throws IllegalStateException {
            checkConnection();
            return this.mConnection.getFileDescriptor();
        }

        public synchronized byte[] getRawDescriptors() throws IllegalStateException {
            checkConnection();
            return this.mConnection.getRawDescriptors();
        }

        public int getVenderId() {
            UsbDevice device = this.mWeakDevice.get();
            if (device != null) {
                return device.getVendorId();
            }
            return 0;
        }

        public int getProductId() {
            UsbDevice device = this.mWeakDevice.get();
            if (device != null) {
                return device.getProductId();
            }
            return 0;
        }

        public String getUsbVersion() {
            return this.mInfo.usb_version;
        }

        public String getManufacture() {
            return this.mInfo.manufacturer;
        }

        public String getProductName() {
            return this.mInfo.product;
        }

        public String getVersion() {
            return this.mInfo.version;
        }

        public String getSerial() {
            return this.mInfo.serial;
        }

        public int getBusNum() {
            return this.mBusNum;
        }

        public int getDevNum() {
            return this.mDevNum;
        }

        public synchronized UsbInterface getInterface(int interface_id) throws IllegalStateException {
            return getInterface(interface_id, 0);
        }

        public synchronized UsbInterface getInterface(int interface_id, int altsetting) throws IllegalStateException {
            UsbInterface intf;
            checkConnection();
            SparseArray<UsbInterface> intfs = this.mInterfaces.get(interface_id);
            if (intfs == null) {
                intfs = new SparseArray<>();
                this.mInterfaces.put(interface_id, intfs);
            }
            intf = intfs.get(altsetting);
            if (intf == null) {
                UsbDevice device = this.mWeakDevice.get();
                int n = device.getInterfaceCount();
                int i = 0;
                while (true) {
                    if (i >= n) {
                        break;
                    }
                    UsbInterface temp = device.getInterface(i);
                    if (temp.getId() != interface_id || temp.getAlternateSetting() != altsetting) {
                        i++;
                    } else {
                        intf = temp;
                        break;
                    }
                }
                if (intf != null) {
                    intfs.append(altsetting, intf);
                }
            }
            return intf;
        }

        public synchronized void claimInterface(UsbInterface intf) {
            claimInterface(intf, true);
        }

        public synchronized void claimInterface(UsbInterface intf, boolean force) {
            checkConnection();
            this.mConnection.claimInterface(intf, force);
        }

        public synchronized void releaseInterface(UsbInterface intf) throws IllegalStateException {
            checkConnection();
            SparseArray<UsbInterface> intfs = this.mInterfaces.get(intf.getId());
            if (intfs != null) {
                int index = intfs.indexOfValue(intf);
                intfs.removeAt(index);
                if (intfs.size() == 0) {
                    this.mInterfaces.remove(intf.getId());
                }
            }
            this.mConnection.releaseInterface(intf);
        }

        public synchronized void close() {
            if (this.mConnection != null) {
                int n = this.mInterfaces.size();
                for (int i = 0; i < n; i++) {
                    SparseArray<UsbInterface> intfs = this.mInterfaces.valueAt(i);
                    if (intfs != null) {
                        int m = intfs.size();
                        for (int j = 0; j < m; j++) {
                            UsbInterface intf = intfs.valueAt(j);
                            this.mConnection.releaseInterface(intf);
                        }
                        intfs.clear();
                    }
                }
                this.mInterfaces.clear();
                this.mConnection.close();
                this.mConnection = null;
                USBMonitor monitor = this.mWeakMonitor.get();
                if (monitor != null) {
                    if (monitor.mOnDeviceConnectListener != null) {
                        monitor.mOnDeviceConnectListener.onDisconnect(this.mWeakDevice.get(), this);
                    }
                    monitor.mCtrlBlocks.remove(getDevice());
                }
            }
        }

        public boolean equals(Object o) {
            if (o == null) {
                return false;
            }
            if (o instanceof UsbControlBlock) {
                UsbDevice device = ((UsbControlBlock) o).getDevice();
                if (device == null) {
                    return this.mWeakDevice.get() == null;
                }
                return device.equals(this.mWeakDevice.get());
            } else if (o instanceof UsbDevice) {
                return o.equals(this.mWeakDevice.get());
            } else {
                return super.equals(o);
            }
        }

        private synchronized void checkConnection() throws IllegalStateException {
            if (this.mConnection == null) {
                throw new IllegalStateException("already closed");
            }
        }
    }
}
