package im.uecahxyvjd.messenger;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.util.LongSparseArray;
import android.util.SparseArray;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.SendMessagesHelper;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
/* loaded from: classes16.dex */
public class DownloadController extends BaseController implements NotificationCenter.NotificationCenterDelegate {
    public static final int AUTODOWNLOAD_TYPE_AUDIO = 2;
    public static final int AUTODOWNLOAD_TYPE_DOCUMENT = 8;
    public static final int AUTODOWNLOAD_TYPE_PHOTO = 1;
    public static final int AUTODOWNLOAD_TYPE_VIDEO = 4;
    private static volatile DownloadController[] Instance = new DownloadController[3];
    public static final int PRESET_NUM_CHANNEL = 3;
    public static final int PRESET_NUM_CONTACT = 0;
    public static final int PRESET_NUM_GROUP = 2;
    public static final int PRESET_NUM_PM = 1;
    public static final int PRESET_SIZE_NUM_AUDIO = 3;
    public static final int PRESET_SIZE_NUM_DOCUMENT = 2;
    public static final int PRESET_SIZE_NUM_PHOTO = 0;
    public static final int PRESET_SIZE_NUM_VIDEO = 1;
    private HashMap<String, FileDownloadProgressListener> addLaterArray;
    private ArrayList<DownloadObject> audioDownloadQueue;
    public int currentMobilePreset;
    public int currentRoamingPreset;
    public int currentWifiPreset;
    private ArrayList<FileDownloadProgressListener> deleteLaterArray;
    private ArrayList<DownloadObject> documentDownloadQueue;
    private HashMap<String, DownloadObject> downloadQueueKeys;
    public Preset highPreset;
    private int lastCheckMask;
    private int lastTag;
    private boolean listenerInProgress;
    private boolean loadingAutoDownloadConfig;
    private HashMap<String, ArrayList<MessageObject>> loadingFileMessagesObservers;
    private HashMap<String, ArrayList<WeakReference<FileDownloadProgressListener>>> loadingFileObservers;
    public Preset lowPreset;
    public Preset mediumPreset;
    public Preset mobilePreset;
    private SparseArray<String> observersByTag;
    private ArrayList<DownloadObject> photoDownloadQueue;
    public Preset roamingPreset;
    private LongSparseArray<Long> typingTimes;
    private ArrayList<DownloadObject> videoDownloadQueue;
    public Preset wifiPreset;

    /* loaded from: classes16.dex */
    public interface FileDownloadProgressListener {
        int getObserverTag();

        void onFailedDownload(String str, boolean z);

        void onProgressDownload(String str, float f);

        void onProgressUpload(String str, float f, boolean z);

        void onSuccessDownload(String str);
    }

    /* loaded from: classes16.dex */
    public static class Preset {
        public boolean enabled;
        public boolean lessCallData;
        public int[] mask;
        public boolean preloadMusic;
        public boolean preloadVideo;
        public int[] sizes;

        public Preset(int[] m, int p, int v, int f, boolean pv, boolean pm, boolean e, boolean l) {
            int[] iArr = new int[4];
            this.mask = iArr;
            this.sizes = new int[4];
            System.arraycopy(m, 0, iArr, 0, iArr.length);
            int[] iArr2 = this.sizes;
            iArr2[0] = p;
            iArr2[1] = v;
            iArr2[2] = f;
            iArr2[3] = 524288;
            this.preloadVideo = pv;
            this.preloadMusic = pm;
            this.lessCallData = l;
            this.enabled = e;
        }

        public Preset(String str) {
            this.mask = new int[4];
            this.sizes = new int[4];
            String[] args = str.split("_");
            if (args.length >= 11) {
                this.mask[0] = Utilities.parseInt(args[0]).intValue();
                this.mask[1] = Utilities.parseInt(args[1]).intValue();
                this.mask[2] = Utilities.parseInt(args[2]).intValue();
                this.mask[3] = Utilities.parseInt(args[3]).intValue();
                this.sizes[0] = Utilities.parseInt(args[4]).intValue();
                this.sizes[1] = Utilities.parseInt(args[5]).intValue();
                this.sizes[2] = Utilities.parseInt(args[6]).intValue();
                this.sizes[3] = Utilities.parseInt(args[7]).intValue();
                this.preloadVideo = Utilities.parseInt(args[8]).intValue() == 1;
                this.preloadMusic = Utilities.parseInt(args[9]).intValue() == 1;
                this.enabled = Utilities.parseInt(args[10]).intValue() == 1;
                if (args.length >= 12) {
                    this.lessCallData = Utilities.parseInt(args[11]).intValue() == 1;
                }
            }
        }

        public void set(Preset preset) {
            int[] iArr = preset.mask;
            int[] iArr2 = this.mask;
            System.arraycopy(iArr, 0, iArr2, 0, iArr2.length);
            int[] iArr3 = preset.sizes;
            int[] iArr4 = this.sizes;
            System.arraycopy(iArr3, 0, iArr4, 0, iArr4.length);
            this.preloadVideo = preset.preloadVideo;
            this.preloadMusic = preset.preloadMusic;
            this.lessCallData = preset.lessCallData;
        }

        public void set(TLRPC.TL_autoDownloadSettings settings) {
            this.preloadMusic = settings.audio_preload_next;
            this.preloadVideo = settings.video_preload_large;
            this.lessCallData = settings.phonecalls_less_data;
            this.sizes[0] = Math.max(512000, settings.photo_size_max);
            this.sizes[1] = Math.max(512000, settings.video_size_max);
            this.sizes[2] = Math.max(512000, settings.file_size_max);
            for (int a2 = 0; a2 < this.mask.length; a2++) {
                if (settings.photo_size_max != 0 && !settings.disabled) {
                    int[] iArr = this.mask;
                    iArr[a2] = iArr[a2] | 1;
                } else {
                    int[] iArr2 = this.mask;
                    iArr2[a2] = iArr2[a2] & (-2);
                }
                if (settings.video_size_max != 0 && !settings.disabled) {
                    int[] iArr3 = this.mask;
                    iArr3[a2] = iArr3[a2] | 4;
                } else {
                    int[] iArr4 = this.mask;
                    iArr4[a2] = iArr4[a2] & (-5);
                }
                if (settings.file_size_max != 0 && !settings.disabled) {
                    int[] iArr5 = this.mask;
                    iArr5[a2] = iArr5[a2] | 8;
                } else {
                    int[] iArr6 = this.mask;
                    iArr6[a2] = iArr6[a2] & (-9);
                }
            }
        }

        public String toString() {
            return this.mask[0] + "_" + this.mask[1] + "_" + this.mask[2] + "_" + this.mask[3] + "_" + this.sizes[0] + "_" + this.sizes[1] + "_" + this.sizes[2] + "_" + this.sizes[3] + "_" + (this.preloadVideo ? 1 : 0) + "_" + (this.preloadMusic ? 1 : 0) + "_" + (this.enabled ? 1 : 0) + "_" + (this.lessCallData ? 1 : 0);
        }

        public boolean equals(Preset obj) {
            int[] iArr = this.mask;
            int i = iArr[0];
            int[] iArr2 = obj.mask;
            if (i == iArr2[0] && iArr[1] == iArr2[1] && iArr[2] == iArr2[2] && iArr[3] == iArr2[3]) {
                int[] iArr3 = this.sizes;
                int i2 = iArr3[0];
                int[] iArr4 = obj.sizes;
                return i2 == iArr4[0] && iArr3[1] == iArr4[1] && iArr3[2] == iArr4[2] && iArr3[3] == iArr4[3] && this.preloadVideo == obj.preloadVideo && this.preloadMusic == obj.preloadMusic;
            }
            return false;
        }

        public boolean isEnabled() {
            int a2 = 0;
            while (true) {
                int[] iArr = this.mask;
                if (a2 < iArr.length) {
                    if (iArr[a2] == 0) {
                        a2++;
                    } else {
                        return true;
                    }
                } else {
                    return false;
                }
            }
        }
    }

    public static DownloadController getInstance(int num) {
        DownloadController localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (DownloadController.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    DownloadController[] downloadControllerArr = Instance;
                    DownloadController downloadController = new DownloadController(num);
                    localInstance = downloadController;
                    downloadControllerArr[num] = downloadController;
                }
            }
        }
        return localInstance;
    }

    /* JADX WARN: Removed duplicated region for block: B:37:0x02da  */
    /* JADX WARN: Removed duplicated region for block: B:42:? A[RETURN, SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public DownloadController(int instance) {
        super(instance);
        boolean newConfig;
        String str;
        String str2;
        Integer num;
        this.lastCheckMask = 0;
        this.photoDownloadQueue = new ArrayList<>();
        this.audioDownloadQueue = new ArrayList<>();
        this.documentDownloadQueue = new ArrayList<>();
        this.videoDownloadQueue = new ArrayList<>();
        this.downloadQueueKeys = new HashMap<>();
        this.loadingFileObservers = new HashMap<>();
        this.loadingFileMessagesObservers = new HashMap<>();
        this.observersByTag = new SparseArray<>();
        this.listenerInProgress = false;
        this.addLaterArray = new HashMap<>();
        this.deleteLaterArray = new ArrayList<>();
        this.lastTag = 0;
        this.typingTimes = new LongSparseArray<>();
        SharedPreferences preferences = MessagesController.getMainSettings(this.currentAccount);
        this.lowPreset = new Preset(preferences.getString("preset0", "1_1_1_1_1048576_512000_512000_524288_0_0_1_1"));
        this.mediumPreset = new Preset(preferences.getString("preset1", "13_13_13_13_1048576_10485760_1048576_524288_1_1_1_0"));
        this.highPreset = new Preset(preferences.getString("preset2", "13_13_13_13_1048576_15728640_3145728_524288_1_1_1_0"));
        boolean newConfig2 = preferences.contains("newConfig");
        String str3 = "currentRoamingPreset";
        String str4 = "currentWifiPreset";
        if (newConfig2) {
            newConfig = newConfig2;
            str = "currentRoamingPreset";
        } else if (getUserConfig().isClientActivated()) {
            int[] mobileDataDownloadMask = new int[4];
            int[] wifiDownloadMask = new int[4];
            int[] roamingDownloadMask = new int[4];
            int[] mobileMaxFileSize = new int[7];
            int[] wifiMaxFileSize = new int[7];
            int[] roamingMaxFileSize = new int[7];
            int a2 = 0;
            while (true) {
                str2 = str3;
                if (a2 >= 4) {
                    break;
                }
                String str5 = str4;
                StringBuilder append = new StringBuilder().append("mobileDataDownloadMask");
                Object obj = "";
                if (a2 == 0) {
                    num = "";
                } else {
                    num = "";
                    obj = Integer.valueOf(a2);
                }
                String key = append.append(obj).toString();
                if (a2 == 0 || preferences.contains(key)) {
                    mobileDataDownloadMask[a2] = preferences.getInt(key, 13);
                    wifiDownloadMask[a2] = preferences.getInt("wifiDownloadMask" + (a2 == 0 ? num : Integer.valueOf(a2)), 13);
                    roamingDownloadMask[a2] = preferences.getInt("roamingDownloadMask" + (a2 == 0 ? num : Integer.valueOf(a2)), 1);
                } else {
                    mobileDataDownloadMask[a2] = mobileDataDownloadMask[0];
                    wifiDownloadMask[a2] = wifiDownloadMask[0];
                    roamingDownloadMask[a2] = roamingDownloadMask[0];
                }
                a2++;
                str3 = str2;
                str4 = str5;
            }
            mobileMaxFileSize[2] = preferences.getInt("mobileMaxDownloadSize2", this.mediumPreset.sizes[1]);
            mobileMaxFileSize[3] = preferences.getInt("mobileMaxDownloadSize3", this.mediumPreset.sizes[2]);
            wifiMaxFileSize[2] = preferences.getInt("wifiMaxDownloadSize2", this.highPreset.sizes[1]);
            wifiMaxFileSize[3] = preferences.getInt("wifiMaxDownloadSize3", this.highPreset.sizes[2]);
            roamingMaxFileSize[2] = preferences.getInt("roamingMaxDownloadSize2", this.lowPreset.sizes[1]);
            roamingMaxFileSize[3] = preferences.getInt("roamingMaxDownloadSize3", this.lowPreset.sizes[2]);
            boolean globalAutodownloadEnabled = preferences.getBoolean("globalAutodownloadEnabled", true);
            this.mobilePreset = new Preset(mobileDataDownloadMask, this.mediumPreset.sizes[0], mobileMaxFileSize[2], mobileMaxFileSize[3], true, true, globalAutodownloadEnabled, false);
            this.wifiPreset = new Preset(wifiDownloadMask, this.highPreset.sizes[0], wifiMaxFileSize[2], wifiMaxFileSize[3], true, true, globalAutodownloadEnabled, false);
            this.roamingPreset = new Preset(roamingDownloadMask, this.lowPreset.sizes[0], roamingMaxFileSize[2], roamingMaxFileSize[3], false, false, globalAutodownloadEnabled, true);
            SharedPreferences.Editor editor = preferences.edit();
            editor.putBoolean("newConfig", true);
            editor.putString("mobilePreset", this.mobilePreset.toString());
            editor.putString("wifiPreset", this.wifiPreset.toString());
            editor.putString("roamingPreset", this.roamingPreset.toString());
            this.currentMobilePreset = 3;
            editor.putInt("currentMobilePreset", 3);
            this.currentWifiPreset = 3;
            editor.putInt(str4, 3);
            this.currentRoamingPreset = 3;
            editor.putInt(str2, 3);
            editor.commit();
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.DownloadController$$ExternalSyntheticLambda1
                @Override // java.lang.Runnable
                public final void run() {
                    DownloadController.this.m175lambda$new$0$imuecahxyvjdmessengerDownloadController();
                }
            });
            BroadcastReceiver networkStateReceiver = new BroadcastReceiver() { // from class: im.uecahxyvjd.messenger.DownloadController.1
                @Override // android.content.BroadcastReceiver
                public void onReceive(Context context, Intent intent) {
                    DownloadController.this.checkAutodownloadSettings();
                }
            };
            IntentFilter filter = new IntentFilter("android.net.conn.CONNECTIVITY_CHANGE");
            ApplicationLoader.applicationContext.registerReceiver(networkStateReceiver, filter);
            if (!getUserConfig().isClientActivated()) {
                checkAutodownloadSettings();
                return;
            }
            return;
        } else {
            newConfig = newConfig2;
            str = "currentRoamingPreset";
        }
        this.mobilePreset = new Preset(preferences.getString("mobilePreset", this.mediumPreset.toString()));
        this.wifiPreset = new Preset(preferences.getString("wifiPreset", this.highPreset.toString()));
        this.roamingPreset = new Preset(preferences.getString("roamingPreset", this.lowPreset.toString()));
        this.currentMobilePreset = preferences.getInt("currentMobilePreset", 3);
        this.currentWifiPreset = preferences.getInt("currentWifiPreset", 3);
        this.currentRoamingPreset = preferences.getInt(str, 3);
        if (!newConfig) {
            preferences.edit().putBoolean("newConfig", true).commit();
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.DownloadController$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                DownloadController.this.m175lambda$new$0$imuecahxyvjdmessengerDownloadController();
            }
        });
        BroadcastReceiver networkStateReceiver2 = new BroadcastReceiver() { // from class: im.uecahxyvjd.messenger.DownloadController.1
            @Override // android.content.BroadcastReceiver
            public void onReceive(Context context, Intent intent) {
                DownloadController.this.checkAutodownloadSettings();
            }
        };
        IntentFilter filter2 = new IntentFilter("android.net.conn.CONNECTIVITY_CHANGE");
        ApplicationLoader.applicationContext.registerReceiver(networkStateReceiver2, filter2);
        if (!getUserConfig().isClientActivated()) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$new$0$im-uecahxyvjd-messenger-DownloadController  reason: not valid java name */
    public /* synthetic */ void m175lambda$new$0$imuecahxyvjdmessengerDownloadController() {
        getNotificationCenter().addObserver(this, NotificationCenter.fileDidFailToLoad);
        getNotificationCenter().addObserver(this, NotificationCenter.fileDidLoad);
        getNotificationCenter().addObserver(this, NotificationCenter.FileLoadProgressChanged);
        getNotificationCenter().addObserver(this, NotificationCenter.FileUploadProgressChanged);
        getNotificationCenter().addObserver(this, NotificationCenter.httpFileDidLoad);
        getNotificationCenter().addObserver(this, NotificationCenter.httpFileDidFailedLoad);
        loadAutoDownloadConfig(false);
    }

    public void loadAutoDownloadConfig(boolean force) {
        if (this.loadingAutoDownloadConfig) {
            return;
        }
        if (!force && Math.abs(System.currentTimeMillis() - getUserConfig().autoDownloadConfigLoadTime) < 86400000) {
            return;
        }
        this.loadingAutoDownloadConfig = true;
        TLRPC.TL_account_getAutoDownloadSettings req = new TLRPC.TL_account_getAutoDownloadSettings();
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.DownloadController$$ExternalSyntheticLambda3
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                DownloadController.this.m174xe78fbcc7(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadAutoDownloadConfig$2$im-uecahxyvjd-messenger-DownloadController  reason: not valid java name */
    public /* synthetic */ void m174xe78fbcc7(final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.DownloadController$$ExternalSyntheticLambda0
            @Override // java.lang.Runnable
            public final void run() {
                DownloadController.this.m173x24a35368(response);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadAutoDownloadConfig$1$im-uecahxyvjd-messenger-DownloadController  reason: not valid java name */
    public /* synthetic */ void m173x24a35368(TLObject response) {
        Preset preset;
        this.loadingAutoDownloadConfig = false;
        getUserConfig().autoDownloadConfigLoadTime = System.currentTimeMillis();
        getUserConfig().saveConfig(false);
        if (response != null) {
            TLRPC.TL_account_autoDownloadSettings res = (TLRPC.TL_account_autoDownloadSettings) response;
            this.lowPreset.set(res.low);
            this.mediumPreset.set(res.medium);
            this.highPreset.set(res.high);
            for (int a2 = 0; a2 < 3; a2++) {
                if (a2 == 0) {
                    preset = this.mobilePreset;
                } else if (a2 == 1) {
                    preset = this.wifiPreset;
                } else {
                    preset = this.roamingPreset;
                }
                if (preset.equals(this.lowPreset)) {
                    preset.set(res.low);
                } else if (preset.equals(this.mediumPreset)) {
                    preset.set(res.medium);
                } else if (preset.equals(this.highPreset)) {
                    preset.set(res.high);
                }
            }
            int a3 = this.currentAccount;
            SharedPreferences.Editor editor = MessagesController.getMainSettings(a3).edit();
            editor.putString("mobilePreset", this.mobilePreset.toString());
            editor.putString("wifiPreset", this.wifiPreset.toString());
            editor.putString("roamingPreset", this.roamingPreset.toString());
            editor.putString("preset0", this.lowPreset.toString());
            editor.putString("preset1", this.mediumPreset.toString());
            editor.putString("preset2", this.highPreset.toString());
            editor.commit();
            this.lowPreset.toString();
            this.mediumPreset.toString();
            this.highPreset.toString();
            checkAutodownloadSettings();
        }
    }

    public Preset getCurrentMobilePreset() {
        int i = this.currentMobilePreset;
        if (i == 0) {
            return this.lowPreset;
        }
        if (i == 1) {
            return this.mediumPreset;
        }
        if (i == 2) {
            return this.highPreset;
        }
        return this.mobilePreset;
    }

    public Preset getCurrentWiFiPreset() {
        int i = this.currentWifiPreset;
        if (i == 0) {
            return this.lowPreset;
        }
        if (i == 1) {
            return this.mediumPreset;
        }
        if (i == 2) {
            return this.highPreset;
        }
        return this.wifiPreset;
    }

    public Preset getCurrentRoamingPreset() {
        int i = this.currentRoamingPreset;
        if (i == 0) {
            return this.lowPreset;
        }
        if (i == 1) {
            return this.mediumPreset;
        }
        if (i == 2) {
            return this.highPreset;
        }
        return this.roamingPreset;
    }

    public static int typeToIndex(int type) {
        if (type == 1) {
            return 0;
        }
        if (type == 2) {
            return 3;
        }
        if (type == 4) {
            return 1;
        }
        if (type != 8) {
            return 0;
        }
        return 2;
    }

    public void cleanup() {
        this.photoDownloadQueue.clear();
        this.audioDownloadQueue.clear();
        this.documentDownloadQueue.clear();
        this.videoDownloadQueue.clear();
        this.downloadQueueKeys.clear();
        this.typingTimes.clear();
    }

    public int getAutodownloadMask() {
        int[] masksArray;
        int result = 0;
        if (ApplicationLoader.isConnectedToWiFi()) {
            if (!this.wifiPreset.enabled) {
                return 0;
            }
            masksArray = getCurrentWiFiPreset().mask;
        } else if (ApplicationLoader.isRoaming()) {
            if (!this.roamingPreset.enabled) {
                return 0;
            }
            masksArray = getCurrentRoamingPreset().mask;
        } else if (!this.mobilePreset.enabled) {
            return 0;
        } else {
            masksArray = getCurrentMobilePreset().mask;
        }
        for (int a2 = 0; a2 < masksArray.length; a2++) {
            int mask = 0;
            if ((masksArray[a2] & 1) != 0) {
                mask = 0 | 1;
            }
            if ((masksArray[a2] & 2) != 0) {
                mask |= 2;
            }
            if ((masksArray[a2] & 4) != 0) {
                mask |= 4;
            }
            if ((masksArray[a2] & 8) != 0) {
                mask |= 8;
            }
            result |= mask << (a2 * 8);
        }
        return result;
    }

    protected int getAutodownloadMaskAll() {
        if (!this.mobilePreset.enabled && !this.roamingPreset.enabled && !this.wifiPreset.enabled) {
            return 0;
        }
        int mask = 0;
        for (int a2 = 0; a2 < 4; a2++) {
            if ((getCurrentMobilePreset().mask[a2] & 1) != 0 || (getCurrentWiFiPreset().mask[a2] & 1) != 0 || (getCurrentRoamingPreset().mask[a2] & 1) != 0) {
                mask |= 1;
            }
            if ((getCurrentMobilePreset().mask[a2] & 2) != 0 || (getCurrentWiFiPreset().mask[a2] & 2) != 0 || (getCurrentRoamingPreset().mask[a2] & 2) != 0) {
                mask |= 2;
            }
            if ((getCurrentMobilePreset().mask[a2] & 4) != 0 || (getCurrentWiFiPreset().mask[a2] & 4) != 0 || (4 & getCurrentRoamingPreset().mask[a2]) != 0) {
                mask |= 4;
            }
            if ((getCurrentMobilePreset().mask[a2] & 8) != 0 || (getCurrentWiFiPreset().mask[a2] & 8) != 0 || (getCurrentRoamingPreset().mask[a2] & 8) != 0) {
                mask |= 8;
            }
        }
        return mask;
    }

    public void checkAutodownloadSettings() {
        int currentMask = getCurrentDownloadMask();
        if (currentMask == this.lastCheckMask) {
            return;
        }
        this.lastCheckMask = currentMask;
        if ((currentMask & 1) != 0) {
            if (this.photoDownloadQueue.isEmpty()) {
                newDownloadObjectsAvailable(1);
            }
        } else {
            for (int a2 = 0; a2 < this.photoDownloadQueue.size(); a2++) {
                DownloadObject downloadObject = this.photoDownloadQueue.get(a2);
                if (downloadObject.object instanceof TLRPC.Photo) {
                    TLRPC.Photo photo = (TLRPC.Photo) downloadObject.object;
                    TLRPC.PhotoSize photoSize = FileLoader.getClosestPhotoSizeWithSize(photo.sizes, AndroidUtilities.getPhotoSize());
                    getFileLoader().cancelLoadFile(photoSize);
                } else if (downloadObject.object instanceof TLRPC.Document) {
                    getFileLoader().cancelLoadFile((TLRPC.Document) downloadObject.object);
                }
            }
            this.photoDownloadQueue.clear();
        }
        if ((currentMask & 2) != 0) {
            if (this.audioDownloadQueue.isEmpty()) {
                newDownloadObjectsAvailable(2);
            }
        } else {
            for (int a3 = 0; a3 < this.audioDownloadQueue.size(); a3++) {
                getFileLoader().cancelLoadFile((TLRPC.Document) this.audioDownloadQueue.get(a3).object);
            }
            this.audioDownloadQueue.clear();
        }
        if ((currentMask & 8) != 0) {
            if (this.documentDownloadQueue.isEmpty()) {
                newDownloadObjectsAvailable(8);
            }
        } else {
            for (int a4 = 0; a4 < this.documentDownloadQueue.size(); a4++) {
                TLRPC.Document document = (TLRPC.Document) this.documentDownloadQueue.get(a4).object;
                getFileLoader().cancelLoadFile(document);
            }
            this.documentDownloadQueue.clear();
        }
        if ((currentMask & 4) != 0) {
            if (this.videoDownloadQueue.isEmpty()) {
                newDownloadObjectsAvailable(4);
            }
        } else {
            for (int a5 = 0; a5 < this.videoDownloadQueue.size(); a5++) {
                getFileLoader().cancelLoadFile((TLRPC.Document) this.videoDownloadQueue.get(a5).object);
            }
            this.videoDownloadQueue.clear();
        }
        int mask = getAutodownloadMaskAll();
        if (mask == 0) {
            getMessagesStorage().clearDownloadQueue(0);
            return;
        }
        if ((mask & 1) == 0) {
            getMessagesStorage().clearDownloadQueue(1);
        }
        if ((mask & 2) == 0) {
            getMessagesStorage().clearDownloadQueue(2);
        }
        if ((mask & 4) == 0) {
            getMessagesStorage().clearDownloadQueue(4);
        }
        if ((mask & 8) == 0) {
            getMessagesStorage().clearDownloadQueue(8);
        }
    }

    public boolean canDownloadMedia(MessageObject messageObject) {
        return canDownloadMedia(messageObject.messageOwner) == 1;
    }

    public boolean canDownloadMedia(int type, int size) {
        Preset preset;
        if (ApplicationLoader.isConnectedToWiFi()) {
            if (!this.wifiPreset.enabled) {
                return false;
            }
            preset = getCurrentWiFiPreset();
        } else if (ApplicationLoader.isRoaming()) {
            if (!this.roamingPreset.enabled) {
                return false;
            }
            preset = getCurrentRoamingPreset();
        } else {
            Preset preset2 = this.mobilePreset;
            if (!preset2.enabled) {
                return false;
            }
            preset = getCurrentMobilePreset();
        }
        int mask = preset.mask[1];
        int maxSize = preset.sizes[typeToIndex(type)];
        if (type == 1 || (size != 0 && size <= maxSize)) {
            return type == 2 || (mask & type) != 0;
        }
        return false;
    }

    public int canDownloadMedia(TLRPC.Message message) {
        int type;
        int index;
        Preset preset;
        if (message == null) {
            return 0;
        }
        boolean isVideo = MessageObject.isVideoMessage(message);
        if (isVideo || MessageObject.isGifMessage(message) || MessageObject.isRoundVideoMessage(message) || MessageObject.isGameMessage(message)) {
            type = 4;
        } else if (MessageObject.isVoiceMessage(message)) {
            type = 2;
        } else if (MessageObject.isPhoto(message) || MessageObject.isStickerMessage(message) || MessageObject.isAnimatedStickerMessage(message)) {
            type = 1;
        } else if (MessageObject.getDocument(message) == null) {
            return 0;
        } else {
            type = 8;
        }
        TLRPC.Peer peer = message.to_id;
        if (peer != null) {
            if (peer.user_id != 0) {
                if (getContactsController().contactsDict.containsKey(Integer.valueOf(peer.user_id))) {
                    index = 0;
                } else {
                    index = 1;
                }
            } else {
                int index2 = peer.chat_id;
                if (index2 != 0) {
                    if (message.from_id != 0 && getContactsController().contactsDict.containsKey(Integer.valueOf(message.from_id))) {
                        index = 0;
                    } else {
                        index = 2;
                    }
                } else if (MessageObject.isMegagroup(message)) {
                    if (message.from_id != 0 && getContactsController().contactsDict.containsKey(Integer.valueOf(message.from_id))) {
                        index = 0;
                    } else {
                        index = 2;
                    }
                } else {
                    index = 3;
                }
            }
        } else {
            index = 1;
        }
        if (ApplicationLoader.isConnectedToWiFi()) {
            if (!this.wifiPreset.enabled) {
                return 0;
            }
            preset = getCurrentWiFiPreset();
        } else if (ApplicationLoader.isRoaming()) {
            if (!this.roamingPreset.enabled) {
                return 0;
            }
            preset = getCurrentRoamingPreset();
        } else {
            Preset preset2 = this.mobilePreset;
            if (!preset2.enabled) {
                return 0;
            }
            preset = getCurrentMobilePreset();
        }
        int mask = preset.mask[index];
        int maxSize = preset.sizes[typeToIndex(type)];
        int size = MessageObject.getMessageSize(message);
        if (isVideo && preset.preloadVideo && size > maxSize && maxSize > 2097152) {
            return (mask & type) != 0 ? 2 : 0;
        } else if (type == 1 || (size != 0 && size <= maxSize)) {
            return (type == 2 || (mask & type) != 0) ? 1 : 0;
        } else {
            return 0;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean canDownloadNextTrack() {
        return ApplicationLoader.isConnectedToWiFi() ? this.wifiPreset.enabled && getCurrentWiFiPreset().preloadMusic : ApplicationLoader.isRoaming() ? this.roamingPreset.enabled && getCurrentRoamingPreset().preloadMusic : this.mobilePreset.enabled && getCurrentMobilePreset().preloadMusic;
    }

    public int getCurrentDownloadMask() {
        if (ApplicationLoader.isConnectedToWiFi()) {
            if (this.wifiPreset.enabled) {
                int mask = 0;
                for (int a2 = 0; a2 < 4; a2++) {
                    mask |= getCurrentWiFiPreset().mask[a2];
                }
                return mask;
            }
            return 0;
        } else if (ApplicationLoader.isRoaming()) {
            if (this.roamingPreset.enabled) {
                int mask2 = 0;
                for (int a3 = 0; a3 < 4; a3++) {
                    mask2 |= getCurrentRoamingPreset().mask[a3];
                }
                return mask2;
            }
            return 0;
        } else if (this.mobilePreset.enabled) {
            int mask3 = 0;
            for (int a4 = 0; a4 < 4; a4++) {
                mask3 |= getCurrentMobilePreset().mask[a4];
            }
            return mask3;
        } else {
            return 0;
        }
    }

    public void savePresetToServer(int type) {
        Preset preset;
        boolean enabled;
        TLRPC.TL_account_saveAutoDownloadSettings req = new TLRPC.TL_account_saveAutoDownloadSettings();
        if (type == 0) {
            preset = getCurrentMobilePreset();
            enabled = this.mobilePreset.enabled;
        } else if (type == 1) {
            preset = getCurrentWiFiPreset();
            enabled = this.wifiPreset.enabled;
        } else {
            preset = getCurrentRoamingPreset();
            enabled = this.roamingPreset.enabled;
        }
        req.settings = new TLRPC.TL_autoDownloadSettings();
        req.settings.audio_preload_next = preset.preloadMusic;
        req.settings.video_preload_large = preset.preloadVideo;
        req.settings.phonecalls_less_data = preset.lessCallData;
        req.settings.disabled = !enabled;
        boolean photo = false;
        boolean video = false;
        boolean document = false;
        for (int a2 = 0; a2 < preset.mask.length; a2++) {
            if ((preset.mask[a2] & 1) != 0) {
                photo = true;
            }
            if ((preset.mask[a2] & 4) != 0) {
                video = true;
            }
            if ((preset.mask[a2] & 8) != 0) {
                document = true;
            }
            if (photo && video && document) {
                break;
            }
        }
        req.settings.photo_size_max = photo ? preset.sizes[0] : 0;
        req.settings.video_size_max = video ? preset.sizes[1] : 0;
        req.settings.file_size_max = document ? preset.sizes[2] : 0;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.DownloadController$$ExternalSyntheticLambda2
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                DownloadController.lambda$savePresetToServer$3(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$savePresetToServer$3(TLObject response, TLRPC.TL_error error) {
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void processDownloadObjects(int type, ArrayList<DownloadObject> objects) {
        String path;
        int cacheType;
        if (objects.isEmpty()) {
            return;
        }
        ArrayList<DownloadObject> queue = null;
        if (type == 1) {
            queue = this.photoDownloadQueue;
        } else if (type == 2) {
            queue = this.audioDownloadQueue;
        } else if (type == 4) {
            queue = this.videoDownloadQueue;
        } else if (type == 8) {
            queue = this.documentDownloadQueue;
        }
        for (int a2 = 0; a2 < objects.size(); a2++) {
            DownloadObject downloadObject = objects.get(a2);
            TLRPC.PhotoSize photoSize = null;
            if (downloadObject.object instanceof TLRPC.Document) {
                TLRPC.Document document = (TLRPC.Document) downloadObject.object;
                path = FileLoader.getAttachFileName(document);
            } else if (downloadObject.object instanceof TLRPC.Photo) {
                path = FileLoader.getAttachFileName(downloadObject.object);
                TLRPC.Photo photo = (TLRPC.Photo) downloadObject.object;
                photoSize = FileLoader.getClosestPhotoSizeWithSize(photo.sizes, AndroidUtilities.getPhotoSize());
            } else {
                path = null;
            }
            if (path != null && !this.downloadQueueKeys.containsKey(path)) {
                boolean added = true;
                if (photoSize != null) {
                    TLRPC.Photo photo2 = (TLRPC.Photo) downloadObject.object;
                    if (downloadObject.secret) {
                        cacheType = 2;
                    } else if (downloadObject.forceCache) {
                        cacheType = 1;
                    } else {
                        cacheType = 0;
                    }
                    getFileLoader().loadFile(ImageLocation.getForPhoto(photoSize, photo2), downloadObject.parent, null, 0, cacheType);
                } else if (downloadObject.object instanceof TLRPC.Document) {
                    TLRPC.Document document2 = (TLRPC.Document) downloadObject.object;
                    getFileLoader().loadFile(document2, downloadObject.parent, 0, downloadObject.secret ? 2 : 0);
                } else {
                    added = false;
                }
                if (added) {
                    queue.add(downloadObject);
                    this.downloadQueueKeys.put(path, downloadObject);
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void newDownloadObjectsAvailable(int downloadMask) {
        int mask = getCurrentDownloadMask();
        if ((mask & 1) != 0 && (downloadMask & 1) != 0 && this.photoDownloadQueue.isEmpty()) {
            getMessagesStorage().getDownloadQueue(1);
        }
        if ((mask & 2) != 0 && (downloadMask & 2) != 0 && this.audioDownloadQueue.isEmpty()) {
            getMessagesStorage().getDownloadQueue(2);
        }
        if ((mask & 4) != 0 && (downloadMask & 4) != 0 && this.videoDownloadQueue.isEmpty()) {
            getMessagesStorage().getDownloadQueue(4);
        }
        if ((mask & 8) != 0 && (downloadMask & 8) != 0 && this.documentDownloadQueue.isEmpty()) {
            getMessagesStorage().getDownloadQueue(8);
        }
    }

    private void checkDownloadFinished(String fileName, int state) {
        DownloadObject downloadObject = this.downloadQueueKeys.get(fileName);
        if (downloadObject != null) {
            this.downloadQueueKeys.remove(fileName);
            if (state == 0 || state == 2) {
                getMessagesStorage().removeFromDownloadQueue(downloadObject.id, downloadObject.type, false);
            }
            if (downloadObject.type != 1) {
                if (downloadObject.type == 2) {
                    this.audioDownloadQueue.remove(downloadObject);
                    if (this.audioDownloadQueue.isEmpty()) {
                        newDownloadObjectsAvailable(2);
                        return;
                    }
                    return;
                } else if (downloadObject.type == 4) {
                    this.videoDownloadQueue.remove(downloadObject);
                    if (this.videoDownloadQueue.isEmpty()) {
                        newDownloadObjectsAvailable(4);
                        return;
                    }
                    return;
                } else if (downloadObject.type == 8) {
                    this.documentDownloadQueue.remove(downloadObject);
                    if (this.documentDownloadQueue.isEmpty()) {
                        newDownloadObjectsAvailable(8);
                        return;
                    }
                    return;
                } else {
                    return;
                }
            }
            this.photoDownloadQueue.remove(downloadObject);
            if (this.photoDownloadQueue.isEmpty()) {
                newDownloadObjectsAvailable(1);
            }
        }
    }

    public int generateObserverTag() {
        int i = this.lastTag;
        this.lastTag = i + 1;
        return i;
    }

    public void addLoadingFileObserver(String fileName, FileDownloadProgressListener observer) {
        addLoadingFileObserver(fileName, null, observer);
    }

    public void addLoadingFileObserver(String fileName, MessageObject messageObject, FileDownloadProgressListener observer) {
        if (this.listenerInProgress) {
            this.addLaterArray.put(fileName, observer);
            return;
        }
        removeLoadingFileObserver(observer);
        ArrayList<WeakReference<FileDownloadProgressListener>> arrayList = this.loadingFileObservers.get(fileName);
        if (arrayList == null) {
            arrayList = new ArrayList<>();
            this.loadingFileObservers.put(fileName, arrayList);
        }
        arrayList.add(new WeakReference<>(observer));
        if (messageObject != null) {
            ArrayList<MessageObject> messageObjects = this.loadingFileMessagesObservers.get(fileName);
            if (messageObjects == null) {
                messageObjects = new ArrayList<>();
                this.loadingFileMessagesObservers.put(fileName, messageObjects);
            }
            messageObjects.add(messageObject);
        }
        this.observersByTag.put(observer.getObserverTag(), fileName);
    }

    public void removeLoadingFileObserver(FileDownloadProgressListener observer) {
        if (this.listenerInProgress) {
            this.deleteLaterArray.add(observer);
            return;
        }
        String fileName = this.observersByTag.get(observer.getObserverTag());
        if (fileName != null) {
            ArrayList<WeakReference<FileDownloadProgressListener>> arrayList = this.loadingFileObservers.get(fileName);
            if (arrayList != null) {
                int a2 = 0;
                while (a2 < arrayList.size()) {
                    WeakReference<FileDownloadProgressListener> reference = arrayList.get(a2);
                    if (reference.get() == null || reference.get() == observer) {
                        arrayList.remove(a2);
                        a2--;
                    }
                    a2++;
                }
                if (arrayList.isEmpty()) {
                    this.loadingFileObservers.remove(fileName);
                }
            }
            this.observersByTag.remove(observer.getObserverTag());
        }
    }

    private void processLaterArrays() {
        for (Map.Entry<String, FileDownloadProgressListener> listener : this.addLaterArray.entrySet()) {
            addLoadingFileObserver(listener.getKey(), listener.getValue());
        }
        this.addLaterArray.clear();
        Iterator<FileDownloadProgressListener> it = this.deleteLaterArray.iterator();
        while (it.hasNext()) {
            removeLoadingFileObserver(it.next());
        }
        this.deleteLaterArray.clear();
    }

    @Override // im.uecahxyvjd.messenger.NotificationCenter.NotificationCenterDelegate
    public void didReceivedNotification(int id, int account, Object... args) {
        if (id == NotificationCenter.fileDidFailToLoad || id == NotificationCenter.httpFileDidFailedLoad) {
            String fileName = (String) args[0];
            Integer canceled = (Integer) args[1];
            this.listenerInProgress = true;
            ArrayList<WeakReference<FileDownloadProgressListener>> arrayList = this.loadingFileObservers.get(fileName);
            if (arrayList != null) {
                int size = arrayList.size();
                for (int a2 = 0; a2 < size; a2++) {
                    WeakReference<FileDownloadProgressListener> reference = arrayList.get(a2);
                    if (reference.get() != null) {
                        reference.get().onFailedDownload(fileName, canceled.intValue() == 1);
                        if (canceled.intValue() != 1) {
                            this.observersByTag.remove(reference.get().getObserverTag());
                        }
                    }
                }
                int a3 = canceled.intValue();
                if (a3 != 1) {
                    this.loadingFileObservers.remove(fileName);
                }
            }
            this.listenerInProgress = false;
            processLaterArrays();
            checkDownloadFinished(fileName, canceled.intValue());
        } else if (id == NotificationCenter.fileDidLoad || id == NotificationCenter.httpFileDidLoad) {
            this.listenerInProgress = true;
            String fileName2 = (String) args[0];
            ArrayList<MessageObject> messageObjects = this.loadingFileMessagesObservers.get(fileName2);
            if (messageObjects != null) {
                int size2 = messageObjects.size();
                for (int a4 = 0; a4 < size2; a4++) {
                    messageObjects.get(a4).mediaExists = true;
                }
                this.loadingFileMessagesObservers.remove(fileName2);
            }
            ArrayList<WeakReference<FileDownloadProgressListener>> arrayList2 = this.loadingFileObservers.get(fileName2);
            if (arrayList2 != null) {
                int size3 = arrayList2.size();
                for (int a5 = 0; a5 < size3; a5++) {
                    WeakReference<FileDownloadProgressListener> reference2 = arrayList2.get(a5);
                    if (reference2.get() != null) {
                        reference2.get().onSuccessDownload(fileName2);
                        this.observersByTag.remove(reference2.get().getObserverTag());
                    }
                }
                this.loadingFileObservers.remove(fileName2);
            }
            this.listenerInProgress = false;
            processLaterArrays();
            checkDownloadFinished(fileName2, 0);
        } else if (id == NotificationCenter.FileLoadProgressChanged) {
            this.listenerInProgress = true;
            String fileName3 = (String) args[0];
            ArrayList<WeakReference<FileDownloadProgressListener>> arrayList3 = this.loadingFileObservers.get(fileName3);
            if (arrayList3 != null) {
                Float progress = (Float) args[1];
                int size4 = arrayList3.size();
                for (int a6 = 0; a6 < size4; a6++) {
                    WeakReference<FileDownloadProgressListener> reference3 = arrayList3.get(a6);
                    if (reference3.get() != null) {
                        reference3.get().onProgressDownload(fileName3, progress.floatValue());
                    }
                }
            }
            this.listenerInProgress = false;
            processLaterArrays();
        } else if (id == NotificationCenter.FileUploadProgressChanged) {
            this.listenerInProgress = true;
            String fileName4 = (String) args[0];
            ArrayList<WeakReference<FileDownloadProgressListener>> arrayList4 = this.loadingFileObservers.get(fileName4);
            if (arrayList4 != null) {
                Float progress2 = (Float) args[1];
                Boolean enc = (Boolean) args[2];
                int size5 = arrayList4.size();
                for (int a7 = 0; a7 < size5; a7++) {
                    WeakReference<FileDownloadProgressListener> reference4 = arrayList4.get(a7);
                    if (reference4.get() != null) {
                        reference4.get().onProgressUpload(fileName4, progress2.floatValue(), enc.booleanValue());
                    }
                }
            }
            this.listenerInProgress = false;
            processLaterArrays();
            try {
                ArrayList<SendMessagesHelper.DelayedMessage> delayedMessages = getSendMessagesHelper().getDelayedMessages(fileName4);
                if (delayedMessages != null) {
                    for (int a8 = 0; a8 < delayedMessages.size(); a8++) {
                        SendMessagesHelper.DelayedMessage delayedMessage = delayedMessages.get(a8);
                        if (delayedMessage.encryptedChat == null) {
                            long dialog_id = delayedMessage.peer;
                            if (delayedMessage.type == 4) {
                                Long lastTime = this.typingTimes.get(dialog_id);
                                if (lastTime == null || lastTime.longValue() + 4000 < System.currentTimeMillis()) {
                                    MessageObject messageObject = (MessageObject) delayedMessage.extraHashMap.get(fileName4 + "_i");
                                    if (messageObject != null && messageObject.isVideo()) {
                                        getMessagesController().sendTyping(dialog_id, 5, 0);
                                    } else {
                                        getMessagesController().sendTyping(dialog_id, 4, 0);
                                    }
                                    this.typingTimes.put(dialog_id, Long.valueOf(System.currentTimeMillis()));
                                }
                            } else {
                                Long lastTime2 = this.typingTimes.get(dialog_id);
                                delayedMessage.obj.getDocument();
                                if (lastTime2 == null || lastTime2.longValue() + 4000 < System.currentTimeMillis()) {
                                    if (delayedMessage.obj.isRoundVideo()) {
                                        getMessagesController().sendTyping(dialog_id, 8, 0);
                                    } else if (delayedMessage.obj.isVideo()) {
                                        getMessagesController().sendTyping(dialog_id, 5, 0);
                                    } else if (delayedMessage.obj.isVoice()) {
                                        getMessagesController().sendTyping(dialog_id, 9, 0);
                                    } else if (delayedMessage.obj.getDocument() != null) {
                                        getMessagesController().sendTyping(dialog_id, 3, 0);
                                    } else if (delayedMessage.photoSize != null) {
                                        getMessagesController().sendTyping(dialog_id, 4, 0);
                                    }
                                    this.typingTimes.put(dialog_id, Long.valueOf(System.currentTimeMillis()));
                                }
                            }
                        }
                    }
                }
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
    }
}
