package im.uecahxyvjd.messenger;

import android.util.SparseArray;
import android.util.SparseIntArray;
import im.uecahxyvjd.messenger.FileLoadOperation;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.NativeByteBuffer;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPCContacts;
import java.io.File;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Scanner;
import java.util.concurrent.CountDownLatch;
import java.util.zip.GZIPInputStream;
import java.util.zip.ZipException;
import kotlin.UByte;
/* loaded from: classes16.dex */
public class FileLoadOperation {
    private static final int bigFileSizeFrom = 1048576;
    private static final int cdnChunkCheckSize = 131072;
    private static final int downloadChunkSize = 32768;
    private static final int downloadChunkSizeBig = 131072;
    private static final int maxCdnParts = 12288;
    private static final int maxDownloadRequests = 4;
    private static final int maxDownloadRequestsBig = 4;
    private static final int preloadMaxBytes = 2097152;
    private static final int stateDownloading = 1;
    private static final int stateFailed = 2;
    private static final int stateFinished = 3;
    private static final int stateIdle = 0;
    private boolean allowDisordererFileSave;
    private int bytesCountPadding;
    private File cacheFileFinal;
    private File cacheFileGzipTemp;
    private File cacheFileParts;
    private File cacheFilePreload;
    private File cacheFileTemp;
    private File cacheIvTemp;
    private byte[] cdnCheckBytes;
    private int cdnDatacenterId;
    private SparseArray<TLRPC.TL_fileHash> cdnHashes;
    private byte[] cdnIv;
    private byte[] cdnKey;
    private byte[] cdnToken;
    private int currentAccount;
    private int currentDownloadChunkSize;
    private int currentMaxDownloadRequests;
    private int currentType;
    private int datacenterId;
    private ArrayList<RequestInfo> delayedRequestInfos;
    private FileLoadOperationDelegate delegate;
    private int downloadedBytes;
    private boolean encryptFile;
    private byte[] encryptIv;
    private byte[] encryptKey;
    private String ext;
    private RandomAccessFile fileOutputStream;
    private RandomAccessFile filePartsStream;
    private RandomAccessFile fileReadStream;
    private RandomAccessFile fiv;
    private int foundMoovSize;
    private int initialDatacenterId;
    private boolean isCdn;
    private boolean isForceRequest;
    private boolean isPreloadVideoOperation;
    private byte[] iv;
    private byte[] key;
    protected TLRPC.InputFileLocation location;
    private int moovFound;
    private int nextAtomOffset;
    private boolean nextPartWasPreloaded;
    private int nextPreloadDownloadOffset;
    private ArrayList<Range> notCheckedCdnRanges;
    private ArrayList<Range> notLoadedBytesRanges;
    private volatile ArrayList<Range> notLoadedBytesRangesCopy;
    private ArrayList<Range> notRequestedBytesRanges;
    private Object parentObject;
    private volatile boolean paused;
    private boolean preloadFinished;
    private int preloadNotRequestedBytesCount;
    private RandomAccessFile preloadStream;
    private int preloadStreamFileOffset;
    private byte[] preloadTempBuffer;
    private int preloadTempBufferCount;
    private SparseArray<PreloadRange> preloadedBytesRanges;
    private int priority;
    private RequestInfo priorityRequestInfo;
    private int renameRetryCount;
    private ArrayList<RequestInfo> requestInfos;
    private int requestedBytesCount;
    private SparseIntArray requestedPreloadedBytesRanges;
    private boolean requestingCdnOffsets;
    protected boolean requestingReference;
    private int requestsCount;
    private boolean reuploadingCdn;
    private boolean started;
    private volatile int state;
    private File storePath;
    private ArrayList<FileLoadOperationStream> streamListeners;
    private int streamPriorityStartOffset;
    private int streamStartOffset;
    private boolean supportsPreloading;
    private File tempPath;
    private int totalBytesCount;
    private int totalPreloadedBytes;
    private boolean ungzip;
    private WebFile webFile;
    private TLRPC.InputWebFileLocation webLocation;

    /* loaded from: classes16.dex */
    public interface FileLoadOperationDelegate {
        void didChangedLoadProgress(FileLoadOperation fileLoadOperation, float f);

        void didFailedLoadingFile(FileLoadOperation fileLoadOperation, int i);

        void didFinishLoadingFile(FileLoadOperation fileLoadOperation, File file);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* loaded from: classes16.dex */
    public static class RequestInfo {
        private int offset;
        private int requestToken;
        private TLRPC.TL_upload_file response;
        private TLRPC.TL_upload_cdnFile responseCdn;
        private TLRPC.TL_upload_webFile responseWeb;

        protected RequestInfo() {
        }
    }

    /* loaded from: classes16.dex */
    public static class Range {
        private int end;
        private int start;

        private Range(int s, int e) {
            this.start = s;
            this.end = e;
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes16.dex */
    public static class PreloadRange {
        private int fileOffset;
        private int length;
        private int start;

        private PreloadRange(int o, int s, int l) {
            this.fileOffset = o;
            this.start = s;
            this.length = l;
        }
    }

    public FileLoadOperation(ImageLocation imageLocation, Object parent, String extension, int size) {
        this.preloadTempBuffer = new byte[16];
        this.state = 0;
        this.parentObject = parent;
        if (imageLocation.isEncrypted()) {
            TLRPC.TL_inputEncryptedFileLocation tL_inputEncryptedFileLocation = new TLRPC.TL_inputEncryptedFileLocation();
            this.location = tL_inputEncryptedFileLocation;
            tL_inputEncryptedFileLocation.id = imageLocation.location.volume_id;
            this.location.volume_id = imageLocation.location.volume_id;
            this.location.local_id = imageLocation.location.local_id;
            this.location.access_hash = imageLocation.access_hash;
            this.iv = new byte[32];
            byte[] bArr = imageLocation.iv;
            byte[] bArr2 = this.iv;
            System.arraycopy(bArr, 0, bArr2, 0, bArr2.length);
            this.key = imageLocation.key;
        } else if (imageLocation.photoPeer != null) {
            TLRPC.TL_inputPeerPhotoFileLocation tL_inputPeerPhotoFileLocation = new TLRPC.TL_inputPeerPhotoFileLocation();
            this.location = tL_inputPeerPhotoFileLocation;
            tL_inputPeerPhotoFileLocation.id = imageLocation.location.volume_id;
            this.location.volume_id = imageLocation.location.volume_id;
            this.location.local_id = imageLocation.location.local_id;
            this.location.big = imageLocation.photoPeerBig;
            this.location.peer = imageLocation.photoPeer;
        } else if (imageLocation.stickerSet != null) {
            TLRPC.TL_inputStickerSetThumb tL_inputStickerSetThumb = new TLRPC.TL_inputStickerSetThumb();
            this.location = tL_inputStickerSetThumb;
            tL_inputStickerSetThumb.id = imageLocation.location.volume_id;
            this.location.volume_id = imageLocation.location.volume_id;
            this.location.local_id = imageLocation.location.local_id;
            this.location.stickerset = imageLocation.stickerSet;
        } else if (imageLocation.thumbSize != null) {
            if (imageLocation.photoId != 0) {
                TLRPC.TL_inputPhotoFileLocation tL_inputPhotoFileLocation = new TLRPC.TL_inputPhotoFileLocation();
                this.location = tL_inputPhotoFileLocation;
                tL_inputPhotoFileLocation.id = imageLocation.photoId;
                this.location.volume_id = imageLocation.location.volume_id;
                this.location.local_id = imageLocation.location.local_id;
                this.location.access_hash = imageLocation.access_hash;
                this.location.file_reference = imageLocation.file_reference;
                this.location.thumb_size = imageLocation.thumbSize;
            } else {
                TLRPC.TL_inputDocumentFileLocation tL_inputDocumentFileLocation = new TLRPC.TL_inputDocumentFileLocation();
                this.location = tL_inputDocumentFileLocation;
                tL_inputDocumentFileLocation.id = imageLocation.documentId;
                this.location.volume_id = imageLocation.location.volume_id;
                this.location.local_id = imageLocation.location.local_id;
                this.location.access_hash = imageLocation.access_hash;
                this.location.file_reference = imageLocation.file_reference;
                this.location.thumb_size = imageLocation.thumbSize;
            }
            if (this.location.file_reference == null) {
                this.location.file_reference = new byte[0];
            }
        } else {
            TLRPC.TL_inputFileLocation tL_inputFileLocation = new TLRPC.TL_inputFileLocation();
            this.location = tL_inputFileLocation;
            tL_inputFileLocation.volume_id = imageLocation.location.volume_id;
            this.location.local_id = imageLocation.location.local_id;
            this.location.secret = imageLocation.access_hash;
            this.location.file_reference = imageLocation.file_reference;
            if (this.location.file_reference == null) {
                this.location.file_reference = new byte[0];
            }
            this.allowDisordererFileSave = true;
        }
        this.ungzip = imageLocation.lottieAnimation;
        int i = imageLocation.dc_id;
        this.datacenterId = i;
        this.initialDatacenterId = i;
        this.currentType = 16777216;
        this.totalBytesCount = size;
        this.ext = extension != null ? extension : "jpg";
    }

    public FileLoadOperation(SecureDocument secureDocument) {
        this.preloadTempBuffer = new byte[16];
        this.state = 0;
        TLRPC.TL_inputSecureFileLocation tL_inputSecureFileLocation = new TLRPC.TL_inputSecureFileLocation();
        this.location = tL_inputSecureFileLocation;
        tL_inputSecureFileLocation.id = secureDocument.secureFile.id;
        this.location.access_hash = secureDocument.secureFile.access_hash;
        this.datacenterId = secureDocument.secureFile.dc_id;
        this.totalBytesCount = secureDocument.secureFile.size;
        this.allowDisordererFileSave = true;
        this.currentType = 67108864;
        this.ext = ".jpg";
    }

    public FileLoadOperation(int instance, WebFile webDocument) {
        this.preloadTempBuffer = new byte[16];
        this.state = 0;
        this.currentAccount = instance;
        this.webFile = webDocument;
        this.webLocation = webDocument.location;
        this.totalBytesCount = webDocument.size;
        int i = MessagesController.getInstance(this.currentAccount).webFileDatacenterId;
        this.datacenterId = i;
        this.initialDatacenterId = i;
        String defaultExt = FileLoader.getMimeTypePart(webDocument.mime_type);
        if (webDocument.mime_type.startsWith("image/")) {
            this.currentType = 16777216;
        } else if (webDocument.mime_type.equals("audio/ogg")) {
            this.currentType = ConnectionsManager.FileTypeAudio;
        } else if (webDocument.mime_type.startsWith("video/")) {
            this.currentType = 33554432;
        } else {
            this.currentType = 67108864;
        }
        this.allowDisordererFileSave = true;
        this.ext = ImageLoader.getHttpUrlExtension(webDocument.url, defaultExt);
    }

    /* JADX WARN: Removed duplicated region for block: B:32:0x00d2 A[Catch: Exception -> 0x00f9, TryCatch #0 {Exception -> 0x00f9, blocks: (B:3:0x000d, B:6:0x0015, B:18:0x008b, B:20:0x009d, B:22:0x00a2, B:23:0x00aa, B:25:0x00b2, B:28:0x00bd, B:30:0x00c8, B:32:0x00d2, B:37:0x00e8, B:39:0x00f0, B:33:0x00d7, B:35:0x00df, B:36:0x00e4, B:29:0x00c6, B:7:0x003f, B:9:0x0043, B:11:0x0064, B:12:0x006a, B:14:0x007b, B:16:0x0085, B:17:0x0088), top: B:44:0x000d }] */
    /* JADX WARN: Removed duplicated region for block: B:33:0x00d7 A[Catch: Exception -> 0x00f9, TryCatch #0 {Exception -> 0x00f9, blocks: (B:3:0x000d, B:6:0x0015, B:18:0x008b, B:20:0x009d, B:22:0x00a2, B:23:0x00aa, B:25:0x00b2, B:28:0x00bd, B:30:0x00c8, B:32:0x00d2, B:37:0x00e8, B:39:0x00f0, B:33:0x00d7, B:35:0x00df, B:36:0x00e4, B:29:0x00c6, B:7:0x003f, B:9:0x0043, B:11:0x0064, B:12:0x006a, B:14:0x007b, B:16:0x0085, B:17:0x0088), top: B:44:0x000d }] */
    /* JADX WARN: Removed duplicated region for block: B:39:0x00f0 A[Catch: Exception -> 0x00f9, TRY_LEAVE, TryCatch #0 {Exception -> 0x00f9, blocks: (B:3:0x000d, B:6:0x0015, B:18:0x008b, B:20:0x009d, B:22:0x00a2, B:23:0x00aa, B:25:0x00b2, B:28:0x00bd, B:30:0x00c8, B:32:0x00d2, B:37:0x00e8, B:39:0x00f0, B:33:0x00d7, B:35:0x00df, B:36:0x00e4, B:29:0x00c6, B:7:0x003f, B:9:0x0043, B:11:0x0064, B:12:0x006a, B:14:0x007b, B:16:0x0085, B:17:0x0088), top: B:44:0x000d }] */
    /* JADX WARN: Removed duplicated region for block: B:49:? A[RETURN, SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public FileLoadOperation(TLRPC.Document documentLocation, Object parent) {
        int idx;
        this.preloadTempBuffer = new byte[16];
        this.state = 0;
        try {
            this.parentObject = parent;
            if (documentLocation instanceof TLRPC.TL_documentEncrypted) {
                TLRPC.TL_inputEncryptedFileLocation tL_inputEncryptedFileLocation = new TLRPC.TL_inputEncryptedFileLocation();
                this.location = tL_inputEncryptedFileLocation;
                tL_inputEncryptedFileLocation.id = documentLocation.id;
                this.location.access_hash = documentLocation.access_hash;
                int i = documentLocation.dc_id;
                this.datacenterId = i;
                this.initialDatacenterId = i;
                this.iv = new byte[32];
                byte[] bArr = documentLocation.iv;
                byte[] bArr2 = this.iv;
                System.arraycopy(bArr, 0, bArr2, 0, bArr2.length);
                this.key = documentLocation.key;
            } else if (documentLocation instanceof TLRPC.TL_document) {
                TLRPC.TL_inputDocumentFileLocation tL_inputDocumentFileLocation = new TLRPC.TL_inputDocumentFileLocation();
                this.location = tL_inputDocumentFileLocation;
                tL_inputDocumentFileLocation.id = documentLocation.id;
                this.location.access_hash = documentLocation.access_hash;
                this.location.file_reference = documentLocation.file_reference;
                this.location.thumb_size = "";
                if (this.location.file_reference == null) {
                    this.location.file_reference = new byte[0];
                }
                int i2 = documentLocation.dc_id;
                this.datacenterId = i2;
                this.initialDatacenterId = i2;
                this.allowDisordererFileSave = true;
                int a2 = 0;
                int N = documentLocation.attributes.size();
                while (true) {
                    if (a2 >= N) {
                        break;
                    } else if (!(documentLocation.attributes.get(a2) instanceof TLRPC.TL_documentAttributeVideo)) {
                        a2++;
                    } else {
                        this.supportsPreloading = true;
                        break;
                    }
                }
            }
            this.ungzip = "application/x-tgsticker".equals(documentLocation.mime_type);
            int i3 = documentLocation.size;
            this.totalBytesCount = i3;
            if (this.key != null && i3 % 16 != 0) {
                int i4 = 16 - (i3 % 16);
                this.bytesCountPadding = i4;
                this.totalBytesCount = i3 + i4;
            }
            String documentFileName = FileLoader.getDocumentFileName(documentLocation);
            this.ext = documentFileName;
            if (documentFileName != null && (idx = documentFileName.lastIndexOf(46)) != -1) {
                this.ext = this.ext.substring(idx);
                if (!"audio/ogg".equals(documentLocation.mime_type)) {
                    this.currentType = ConnectionsManager.FileTypeAudio;
                } else if (FileLoader.isVideoMimeType(documentLocation.mime_type)) {
                    this.currentType = 33554432;
                } else {
                    this.currentType = 67108864;
                }
                if (this.ext.length() > 1) {
                    this.ext = FileLoader.getExtensionByMimeType(documentLocation.mime_type);
                    return;
                }
                return;
            }
            this.ext = "";
            if (!"audio/ogg".equals(documentLocation.mime_type)) {
            }
            if (this.ext.length() > 1) {
            }
        } catch (Exception e) {
            FileLog.e(e);
            onFail(true, 0);
        }
    }

    public void setEncryptFile(boolean value) {
        this.encryptFile = value;
        if (value) {
            this.allowDisordererFileSave = false;
        }
    }

    public int getDatacenterId() {
        return this.initialDatacenterId;
    }

    public void setForceRequest(boolean forceRequest) {
        this.isForceRequest = forceRequest;
    }

    public boolean isForceRequest() {
        return this.isForceRequest;
    }

    public void setPriority(int value) {
        this.priority = value;
    }

    public int getPriority() {
        return this.priority;
    }

    public void setPaths(int instance, File store, File temp) {
        this.storePath = store;
        this.tempPath = temp;
        this.currentAccount = instance;
    }

    public boolean wasStarted() {
        return this.started && !this.paused;
    }

    public int getCurrentType() {
        return this.currentType;
    }

    private void removePart(ArrayList<Range> ranges, int start, int end) {
        if (ranges == null || end < start) {
            return;
        }
        int count = ranges.size();
        boolean modified = false;
        int a2 = 0;
        while (true) {
            if (a2 >= count) {
                break;
            }
            Range range = ranges.get(a2);
            if (start == range.end) {
                range.end = end;
                modified = true;
                break;
            } else if (end == range.start) {
                range.start = start;
                modified = true;
                break;
            } else {
                a2++;
            }
        }
        Collections.sort(ranges, new Comparator() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda11
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return FileLoadOperation.lambda$removePart$0((FileLoadOperation.Range) obj, (FileLoadOperation.Range) obj2);
            }
        });
        int a3 = 0;
        while (a3 < ranges.size() - 1) {
            Range r1 = ranges.get(a3);
            Range r2 = ranges.get(a3 + 1);
            if (r1.end == r2.start) {
                r1.end = r2.end;
                ranges.remove(a3 + 1);
                a3--;
            }
            a3++;
        }
        if (!modified) {
            ranges.add(new Range(start, end));
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$removePart$0(Range o1, Range o2) {
        if (o1.start <= o2.start) {
            if (o1.start < o2.start) {
                return -1;
            }
            return 0;
        }
        return 1;
    }

    private void addPart(ArrayList<Range> ranges, int start, int end, boolean save) {
        if (ranges == null || end < start) {
            return;
        }
        boolean modified = false;
        int count = ranges.size();
        int a2 = 0;
        while (true) {
            if (a2 >= count) {
                break;
            }
            Range range = ranges.get(a2);
            if (start <= range.start) {
                if (end < range.end) {
                    if (end > range.start) {
                        range.start = end;
                        modified = true;
                        break;
                    }
                    a2++;
                } else {
                    ranges.remove(a2);
                    modified = true;
                    break;
                }
            } else if (end >= range.end) {
                if (start < range.end) {
                    range.end = start;
                    modified = true;
                    break;
                }
                a2++;
            } else {
                Range newRange = new Range(range.start, start);
                ranges.add(0, newRange);
                modified = true;
                range.start = end;
                break;
            }
        }
        if (save) {
            if (modified) {
                try {
                    this.filePartsStream.seek(0L);
                    int count2 = ranges.size();
                    this.filePartsStream.writeInt(count2);
                    for (int a3 = 0; a3 < count2; a3++) {
                        Range range2 = ranges.get(a3);
                        this.filePartsStream.writeInt(range2.start);
                        this.filePartsStream.writeInt(range2.end);
                    }
                } catch (Exception e) {
                    FileLog.e(e);
                }
                ArrayList<FileLoadOperationStream> arrayList = this.streamListeners;
                if (arrayList != null) {
                    int count3 = arrayList.size();
                    for (int a4 = 0; a4 < count3; a4++) {
                        this.streamListeners.get(a4).newDataAvailable();
                    }
                }
            } else if (BuildVars.LOGS_ENABLED) {
                FileLog.e(this.cacheFileFinal + " downloaded duplicate file part " + start + " - " + end);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public File getCurrentFile() {
        final CountDownLatch countDownLatch = new CountDownLatch(1);
        final File[] result = new File[1];
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda12
            @Override // java.lang.Runnable
            public final void run() {
                FileLoadOperation.this.m178x3e2282c1(result, countDownLatch);
            }
        });
        try {
            countDownLatch.await();
        } catch (Exception e) {
            FileLog.e(e);
        }
        return result[0];
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getCurrentFile$1$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m178x3e2282c1(File[] result, CountDownLatch countDownLatch) {
        if (this.state == 3) {
            result[0] = this.cacheFileFinal;
        } else {
            result[0] = this.cacheFileTemp;
        }
        countDownLatch.countDown();
    }

    private int getDownloadedLengthFromOffsetInternal(ArrayList<Range> ranges, int offset, int length) {
        if (ranges == null || this.state == 3 || ranges.isEmpty()) {
            int count = this.downloadedBytes;
            if (count != 0) {
                return Math.min(length, Math.max(count - offset, 0));
            }
            return length;
        }
        int count2 = ranges.size();
        Range minRange = null;
        int availableLength = length;
        int a2 = 0;
        while (true) {
            if (a2 >= count2) {
                break;
            }
            Range range = ranges.get(a2);
            if (offset <= range.start && (minRange == null || range.start < minRange.start)) {
                minRange = range;
            }
            if (range.start > offset || range.end <= offset) {
                a2++;
            } else {
                availableLength = 0;
                break;
            }
        }
        if (availableLength == 0) {
            return 0;
        }
        if (minRange != null) {
            return Math.min(length, minRange.start - offset);
        }
        return Math.min(length, Math.max(this.totalBytesCount - offset, 0));
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public float getDownloadedLengthFromOffset(float progress) {
        ArrayList<Range> ranges = this.notLoadedBytesRangesCopy;
        int i = this.totalBytesCount;
        if (i == 0 || ranges == null) {
            return 0.0f;
        }
        return (getDownloadedLengthFromOffsetInternal(ranges, (int) (i * progress), i) / this.totalBytesCount) + progress;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public int getDownloadedLengthFromOffset(final int offset, final int length) {
        final CountDownLatch countDownLatch = new CountDownLatch(1);
        final int[] result = new int[1];
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda4
            @Override // java.lang.Runnable
            public final void run() {
                FileLoadOperation.this.m179xf70118a9(result, offset, length, countDownLatch);
            }
        });
        try {
            countDownLatch.await();
        } catch (Exception e) {
        }
        return result[0];
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getDownloadedLengthFromOffset$2$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m179xf70118a9(int[] result, int offset, int length, CountDownLatch countDownLatch) {
        result[0] = getDownloadedLengthFromOffsetInternal(this.notLoadedBytesRanges, offset, length);
        countDownLatch.countDown();
    }

    public String getFileName() {
        if (this.location != null) {
            return this.location.volume_id + "_" + this.location.local_id + "." + this.ext;
        }
        return Utilities.MD5(this.webFile.url) + "." + this.ext;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void removeStreamListener(final FileLoadOperationStream operation) {
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda13
            @Override // java.lang.Runnable
            public final void run() {
                FileLoadOperation.this.m182x149f299c(operation);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeStreamListener$3$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m182x149f299c(FileLoadOperationStream operation) {
        ArrayList<FileLoadOperationStream> arrayList = this.streamListeners;
        if (arrayList == null) {
            return;
        }
        arrayList.remove(operation);
    }

    private void copyNotLoadedRanges() {
        if (this.notLoadedBytesRanges == null) {
            return;
        }
        this.notLoadedBytesRangesCopy = new ArrayList<>(this.notLoadedBytesRanges);
    }

    public void pause() {
        if (this.state != 1) {
            return;
        }
        this.paused = true;
    }

    public boolean start() {
        return start(null, 0, false);
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Removed duplicated region for block: B:185:0x0619  */
    /* JADX WARN: Removed duplicated region for block: B:200:0x0645  */
    /* JADX WARN: Removed duplicated region for block: B:220:0x06b6  */
    /* JADX WARN: Removed duplicated region for block: B:223:0x06c0  */
    /* JADX WARN: Removed duplicated region for block: B:237:0x071a  */
    /* JADX WARN: Removed duplicated region for block: B:244:0x0742  */
    /* JADX WARN: Removed duplicated region for block: B:250:0x076e  */
    /* JADX WARN: Removed duplicated region for block: B:255:0x07b1  */
    /* JADX WARN: Removed duplicated region for block: B:270:0x07fb  */
    /* JADX WARN: Removed duplicated region for block: B:277:0x0825 A[Catch: Exception -> 0x082a, TRY_LEAVE, TryCatch #4 {Exception -> 0x082a, blocks: (B:275:0x0818, B:277:0x0825), top: B:304:0x0818 }] */
    /* JADX WARN: Removed duplicated region for block: B:283:0x0832  */
    /* JADX WARN: Removed duplicated region for block: B:285:0x0838  */
    /* JADX WARN: Removed duplicated region for block: B:302:0x0623 A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /* JADX WARN: Type inference failed for: r4v1 */
    /* JADX WARN: Type inference failed for: r4v2, types: [boolean, int] */
    /* JADX WARN: Type inference failed for: r4v9 */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean start(final FileLoadOperationStream stream, final int streamOffset, final boolean steamPriority) {
        ?? r4;
        boolean z;
        String fileNameFinal;
        String md5;
        String fileNameTemp;
        boolean finalFileExist;
        boolean newKeyGenerated;
        boolean[] preloaded;
        ArrayList<Range> arrayList;
        int i;
        RandomAccessFile randomAccessFile;
        int i2;
        int i3;
        if (this.currentDownloadChunkSize == 0) {
            this.currentDownloadChunkSize = this.totalBytesCount >= 1048576 ? 131072 : 32768;
            this.currentMaxDownloadRequests = 4;
        }
        final boolean alreadyStarted = this.state != 0;
        boolean wasPaused = this.paused;
        this.paused = false;
        if (stream != null) {
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda6
                @Override // java.lang.Runnable
                public final void run() {
                    FileLoadOperation.this.m185lambda$start$4$imuecahxyvjdmessengerFileLoadOperation(steamPriority, streamOffset, stream, alreadyStarted);
                }
            });
        } else if (wasPaused && alreadyStarted) {
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda7
                @Override // java.lang.Runnable
                public final void run() {
                    FileLoadOperation.this.startDownloadRequest();
                }
            });
        }
        if (alreadyStarted) {
            return wasPaused;
        }
        if (this.location == null && this.webLocation == null) {
            onFail(true, 0);
            return false;
        }
        int i4 = this.currentDownloadChunkSize;
        this.streamStartOffset = (streamOffset / i4) * i4;
        if (this.allowDisordererFileSave && (i3 = this.totalBytesCount) > 0 && i3 > i4) {
            this.notLoadedBytesRanges = new ArrayList<>();
            this.notRequestedBytesRanges = new ArrayList<>();
        }
        String fileNamePreload = null;
        String fileNameIv = null;
        if (this.webLocation != null) {
            String md52 = Utilities.MD5(this.webFile.url);
            if (this.encryptFile) {
                fileNameTemp = md52 + ".temp.enc";
                fileNameFinal = md52 + "." + this.ext + ".enc";
                if (this.key != null) {
                    fileNameIv = md52 + ".iv.enc";
                }
            } else {
                String fileNameTemp2 = md52 + ".temp";
                fileNameFinal = md52 + "." + this.ext;
                if (this.key == null) {
                    fileNameTemp = fileNameTemp2;
                } else {
                    fileNameIv = md52 + ".iv";
                    fileNameTemp = fileNameTemp2;
                }
            }
            md5 = null;
        } else if (this.location.volume_id != 0 && this.location.local_id != 0) {
            if (this.datacenterId == Integer.MIN_VALUE || this.location.volume_id == -2147483648L || this.datacenterId == 0) {
                onFail(true, 0);
                return false;
            } else if (this.encryptFile) {
                fileNameTemp = this.location.volume_id + "_" + this.location.local_id + ".temp.enc";
                fileNameFinal = this.location.volume_id + "_" + this.location.local_id + "." + this.ext + ".enc";
                if (this.key != null) {
                    fileNameIv = this.location.volume_id + "_" + this.location.local_id + ".iv.enc";
                    md5 = null;
                } else {
                    md5 = null;
                }
            } else {
                String fileNameTemp3 = this.location.volume_id + "_" + this.location.local_id + ".temp";
                fileNameFinal = this.location.volume_id + "_" + this.location.local_id + "." + this.ext;
                if (this.key != null) {
                    fileNameIv = this.location.volume_id + "_" + this.location.local_id + ".iv";
                }
                if (this.notLoadedBytesRanges != null) {
                    md5 = this.location.volume_id + "_" + this.location.local_id + ".pt";
                } else {
                    md5 = null;
                }
                fileNamePreload = this.location.volume_id + "_" + this.location.local_id + ".preload";
                fileNameTemp = fileNameTemp3;
            }
        } else {
            if (this.datacenterId != 0) {
                if (this.location.id == 0) {
                    r4 = 0;
                    z = true;
                } else if (this.encryptFile) {
                    fileNameTemp = this.datacenterId + "_" + this.location.id + ".temp.enc";
                    fileNameFinal = this.datacenterId + "_" + this.location.id + this.ext + ".enc";
                    if (this.key != null) {
                        fileNameIv = this.datacenterId + "_" + this.location.id + ".iv.enc";
                        md5 = null;
                    } else {
                        md5 = null;
                    }
                } else {
                    String fileNameTemp4 = this.datacenterId + "_" + this.location.id + ".temp";
                    fileNameFinal = this.datacenterId + "_" + this.location.id + this.ext;
                    if (this.key != null) {
                        fileNameIv = this.datacenterId + "_" + this.location.id + ".iv";
                    }
                    if (this.notLoadedBytesRanges != null) {
                        md5 = this.datacenterId + "_" + this.location.id + ".pt";
                    } else {
                        md5 = null;
                    }
                    fileNamePreload = this.datacenterId + "_" + this.location.id + ".preload";
                    fileNameTemp = fileNameTemp4;
                }
            } else {
                r4 = 0;
                z = true;
            }
            onFail(z, r4);
            return r4;
        }
        this.requestInfos = new ArrayList<>(this.currentMaxDownloadRequests);
        this.delayedRequestInfos = new ArrayList<>(this.currentMaxDownloadRequests - 1);
        this.state = 1;
        Object obj = this.parentObject;
        if (!(obj instanceof TLRPC.TL_theme)) {
            this.cacheFileFinal = new File(this.storePath, fileNameFinal);
        } else {
            TLRPC.TL_theme theme = (TLRPC.TL_theme) obj;
            this.cacheFileFinal = new File(ApplicationLoader.getFilesDirFixed(), "remote" + theme.id + ".attheme");
        }
        boolean finalFileExist2 = this.cacheFileFinal.exists();
        if (finalFileExist2 && (i2 = this.totalBytesCount) != 0 && i2 != this.cacheFileFinal.length()) {
            this.cacheFileFinal.delete();
            finalFileExist = false;
        } else {
            finalFileExist = finalFileExist2;
        }
        if (!finalFileExist) {
            this.cacheFileTemp = new File(this.tempPath, fileNameTemp);
            if (this.ungzip) {
                this.cacheFileGzipTemp = new File(this.tempPath, fileNameTemp + ".gz");
            }
            boolean newKeyGenerated2 = false;
            if (this.encryptFile) {
                File keyFile = new File(FileLoader.getInternalCacheDir(), fileNameFinal + ".key");
                try {
                    RandomAccessFile file = new RandomAccessFile(keyFile, "rws");
                    long len = keyFile.length();
                    byte[] bArr = new byte[32];
                    this.encryptKey = bArr;
                    this.encryptIv = new byte[16];
                    if (len <= 0 || len % 48 != 0) {
                        Utilities.random.nextBytes(this.encryptKey);
                        Utilities.random.nextBytes(this.encryptIv);
                        file.write(this.encryptKey);
                        file.write(this.encryptIv);
                        newKeyGenerated2 = true;
                    } else {
                        file.read(bArr, 0, 32);
                        file.read(this.encryptIv, 0, 16);
                    }
                    try {
                        file.getChannel().close();
                    } catch (Exception e) {
                        FileLog.e(e);
                    }
                    file.close();
                } catch (Exception e2) {
                    FileLog.e(e2);
                }
            }
            boolean[] preloaded2 = {false};
            if (!this.supportsPreloading || fileNamePreload == null) {
                newKeyGenerated = newKeyGenerated2;
                preloaded = preloaded2;
            } else {
                this.cacheFilePreload = new File(this.tempPath, fileNamePreload);
                try {
                    RandomAccessFile randomAccessFile2 = new RandomAccessFile(this.cacheFilePreload, "rws");
                    this.preloadStream = randomAccessFile2;
                    long len2 = randomAccessFile2.length();
                    this.preloadStreamFileOffset = 1;
                    if (len2 - 0 > 1) {
                        preloaded2[0] = this.preloadStream.readByte() != 0;
                        int readOffset = 0 + 1;
                        while (true) {
                            if (readOffset >= len2) {
                                newKeyGenerated = newKeyGenerated2;
                                preloaded = preloaded2;
                                break;
                            } else if (len2 - readOffset < 4) {
                                newKeyGenerated = newKeyGenerated2;
                                preloaded = preloaded2;
                                break;
                            } else {
                                int offset = this.preloadStream.readInt();
                                int readOffset2 = readOffset + 4;
                                newKeyGenerated = newKeyGenerated2;
                                if (len2 - readOffset2 < 4 || offset < 0) {
                                    break;
                                }
                                try {
                                    if (offset > this.totalBytesCount) {
                                        preloaded = preloaded2;
                                        break;
                                    }
                                    int size = this.preloadStream.readInt();
                                    int readOffset3 = readOffset2 + 4;
                                    boolean finalFileExist3 = finalFileExist;
                                    String fileNameTemp5 = fileNameTemp;
                                    preloaded = preloaded2;
                                    if (len2 - readOffset3 < size) {
                                        break;
                                    }
                                    try {
                                        if (size > this.currentDownloadChunkSize) {
                                            break;
                                        }
                                        PreloadRange range = new PreloadRange(readOffset3, offset, size);
                                        int readOffset4 = readOffset3 + size;
                                        this.preloadStream.seek(readOffset4);
                                        if (len2 - readOffset4 < 12) {
                                            break;
                                        }
                                        int readInt = this.preloadStream.readInt();
                                        this.foundMoovSize = readInt;
                                        if (readInt != 0) {
                                            this.moovFound = this.nextPreloadDownloadOffset > this.totalBytesCount / 2 ? 2 : 1;
                                            this.preloadNotRequestedBytesCount = readInt;
                                        }
                                        this.nextPreloadDownloadOffset = this.preloadStream.readInt();
                                        this.nextAtomOffset = this.preloadStream.readInt();
                                        readOffset = readOffset4 + 12;
                                        if (this.preloadedBytesRanges == null) {
                                            this.preloadedBytesRanges = new SparseArray<>();
                                        }
                                        if (this.requestedPreloadedBytesRanges == null) {
                                            this.requestedPreloadedBytesRanges = new SparseIntArray();
                                        }
                                        this.preloadedBytesRanges.put(offset, range);
                                        this.requestedPreloadedBytesRanges.put(offset, 1);
                                        this.totalPreloadedBytes += size;
                                        this.preloadStreamFileOffset += size + 20;
                                        newKeyGenerated2 = newKeyGenerated;
                                        finalFileExist = finalFileExist3;
                                        preloaded2 = preloaded;
                                        fileNameTemp = fileNameTemp5;
                                    } catch (Exception e3) {
                                        e = e3;
                                        FileLog.e(e);
                                        if (!this.isPreloadVideoOperation) {
                                            this.cacheFilePreload = null;
                                            try {
                                                randomAccessFile = this.preloadStream;
                                                if (randomAccessFile != null) {
                                                }
                                            } catch (Exception e4) {
                                                FileLog.e(e4);
                                            }
                                        }
                                        if (md5 != null) {
                                        }
                                        if (!this.cacheFileTemp.exists()) {
                                        }
                                        arrayList = this.notLoadedBytesRanges;
                                        if (arrayList != null) {
                                        }
                                        if (BuildVars.LOGS_ENABLED) {
                                        }
                                        if (fileNameIv != null) {
                                        }
                                        if (!this.isPreloadVideoOperation) {
                                            copyNotLoadedRanges();
                                            this.delegate.didChangedLoadProgress(this, Math.min(1.0f, this.downloadedBytes / this.totalBytesCount));
                                        }
                                        RandomAccessFile randomAccessFile3 = new RandomAccessFile(this.cacheFileTemp, "rws");
                                        this.fileOutputStream = randomAccessFile3;
                                        i = this.downloadedBytes;
                                        if (i != 0) {
                                        }
                                        if (this.fileOutputStream != null) {
                                        }
                                    }
                                } catch (Exception e5) {
                                    e = e5;
                                    preloaded = preloaded2;
                                    FileLog.e(e);
                                    if (!this.isPreloadVideoOperation) {
                                    }
                                    if (md5 != null) {
                                    }
                                    if (!this.cacheFileTemp.exists()) {
                                    }
                                    arrayList = this.notLoadedBytesRanges;
                                    if (arrayList != null) {
                                    }
                                    if (BuildVars.LOGS_ENABLED) {
                                    }
                                    if (fileNameIv != null) {
                                    }
                                    if (!this.isPreloadVideoOperation) {
                                    }
                                    RandomAccessFile randomAccessFile32 = new RandomAccessFile(this.cacheFileTemp, "rws");
                                    this.fileOutputStream = randomAccessFile32;
                                    i = this.downloadedBytes;
                                    if (i != 0) {
                                    }
                                    if (this.fileOutputStream != null) {
                                    }
                                }
                            }
                        }
                        preloaded = preloaded2;
                    } else {
                        newKeyGenerated = newKeyGenerated2;
                        preloaded = preloaded2;
                    }
                    this.preloadStream.seek(this.preloadStreamFileOffset);
                } catch (Exception e6) {
                    e = e6;
                    newKeyGenerated = newKeyGenerated2;
                    preloaded = preloaded2;
                }
                if (!this.isPreloadVideoOperation && this.preloadedBytesRanges == null) {
                    this.cacheFilePreload = null;
                    randomAccessFile = this.preloadStream;
                    if (randomAccessFile != null) {
                        try {
                            randomAccessFile.getChannel().close();
                        } catch (Exception e7) {
                            FileLog.e(e7);
                        }
                        this.preloadStream.close();
                        this.preloadStream = null;
                    }
                }
            }
            if (md5 != null) {
                this.cacheFileParts = new File(this.tempPath, md5);
                try {
                    RandomAccessFile randomAccessFile4 = new RandomAccessFile(this.cacheFileParts, "rws");
                    this.filePartsStream = randomAccessFile4;
                    long len3 = randomAccessFile4.length();
                    if (len3 % 8 == 4) {
                        int count = this.filePartsStream.readInt();
                        if (count <= (len3 - 4) / 2) {
                            int a2 = 0;
                            while (a2 < count) {
                                int start = this.filePartsStream.readInt();
                                int end = this.filePartsStream.readInt();
                                String fileNamePreload2 = fileNamePreload;
                                try {
                                    this.notLoadedBytesRanges.add(new Range(start, end));
                                    this.notRequestedBytesRanges.add(new Range(start, end));
                                    a2++;
                                    fileNamePreload = fileNamePreload2;
                                } catch (Exception e8) {
                                    e = e8;
                                    FileLog.e(e);
                                    if (!this.cacheFileTemp.exists()) {
                                    }
                                    arrayList = this.notLoadedBytesRanges;
                                    if (arrayList != null) {
                                    }
                                    if (BuildVars.LOGS_ENABLED) {
                                    }
                                    if (fileNameIv != null) {
                                    }
                                    if (!this.isPreloadVideoOperation) {
                                    }
                                    RandomAccessFile randomAccessFile322 = new RandomAccessFile(this.cacheFileTemp, "rws");
                                    this.fileOutputStream = randomAccessFile322;
                                    i = this.downloadedBytes;
                                    if (i != 0) {
                                    }
                                    if (this.fileOutputStream != null) {
                                    }
                                }
                            }
                        }
                    }
                } catch (Exception e9) {
                    e = e9;
                }
            }
            if (!this.cacheFileTemp.exists()) {
                if (newKeyGenerated) {
                    this.cacheFileTemp.delete();
                } else {
                    long totalDownloadedLen = this.cacheFileTemp.length();
                    if (fileNameIv != null && totalDownloadedLen % this.currentDownloadChunkSize != 0) {
                        this.downloadedBytes = 0;
                        this.requestedBytesCount = 0;
                    } else {
                        int length = (int) this.cacheFileTemp.length();
                        int i5 = this.currentDownloadChunkSize;
                        int i6 = (length / i5) * i5;
                        this.downloadedBytes = i6;
                        this.requestedBytesCount = i6;
                    }
                    ArrayList<Range> arrayList2 = this.notLoadedBytesRanges;
                    if (arrayList2 != null && arrayList2.isEmpty()) {
                        this.notLoadedBytesRanges.add(new Range(this.downloadedBytes, this.totalBytesCount));
                        this.notRequestedBytesRanges.add(new Range(this.downloadedBytes, this.totalBytesCount));
                    }
                }
            } else {
                ArrayList<Range> arrayList3 = this.notLoadedBytesRanges;
                if (arrayList3 != null && arrayList3.isEmpty()) {
                    this.notLoadedBytesRanges.add(new Range(0, this.totalBytesCount));
                    this.notRequestedBytesRanges.add(new Range(0, this.totalBytesCount));
                }
            }
            arrayList = this.notLoadedBytesRanges;
            if (arrayList != null) {
                this.downloadedBytes = this.totalBytesCount;
                int size2 = arrayList.size();
                for (int a3 = 0; a3 < size2; a3++) {
                    Range range2 = this.notLoadedBytesRanges.get(a3);
                    this.downloadedBytes -= range2.end - range2.start;
                }
                int a4 = this.downloadedBytes;
                this.requestedBytesCount = a4;
            }
            if (BuildVars.LOGS_ENABLED) {
                if (this.isPreloadVideoOperation) {
                    FileLog.d("start preloading file to temp = " + this.cacheFileTemp);
                } else {
                    FileLog.d("start loading file to temp = " + this.cacheFileTemp + " final = " + this.cacheFileFinal);
                }
            }
            if (fileNameIv != null) {
                this.cacheIvTemp = new File(this.tempPath, fileNameIv);
                try {
                    this.fiv = new RandomAccessFile(this.cacheIvTemp, "rws");
                    if (this.downloadedBytes != 0 && !newKeyGenerated) {
                        long len4 = this.cacheIvTemp.length();
                        if (len4 <= 0 || len4 % 32 != 0) {
                            this.downloadedBytes = 0;
                            this.requestedBytesCount = 0;
                        } else {
                            this.fiv.read(this.iv, 0, 32);
                        }
                    }
                } catch (Exception e10) {
                    FileLog.e(e10);
                    this.downloadedBytes = 0;
                    this.requestedBytesCount = 0;
                }
            }
            if (!this.isPreloadVideoOperation && this.downloadedBytes != 0 && this.totalBytesCount > 0) {
                copyNotLoadedRanges();
                this.delegate.didChangedLoadProgress(this, Math.min(1.0f, this.downloadedBytes / this.totalBytesCount));
            }
            try {
                RandomAccessFile randomAccessFile3222 = new RandomAccessFile(this.cacheFileTemp, "rws");
                this.fileOutputStream = randomAccessFile3222;
                i = this.downloadedBytes;
                if (i != 0) {
                    randomAccessFile3222.seek(i);
                }
            } catch (Exception e11) {
                FileLog.e(e11);
            }
            if (this.fileOutputStream != null) {
                onFail(true, 0);
                return false;
            }
            this.started = true;
            final boolean[] preloaded3 = preloaded;
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda8
                @Override // java.lang.Runnable
                public final void run() {
                    FileLoadOperation.this.m186lambda$start$5$imuecahxyvjdmessengerFileLoadOperation(preloaded3);
                }
            });
            return true;
        }
        this.started = true;
        try {
            onFinishLoadingFile(false);
            return true;
        } catch (Exception e12) {
            onFail(true, 0);
            return true;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$start$4$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m185lambda$start$4$imuecahxyvjdmessengerFileLoadOperation(boolean steamPriority, int streamOffset, FileLoadOperationStream stream, boolean alreadyStarted) {
        if (this.streamListeners == null) {
            this.streamListeners = new ArrayList<>();
        }
        if (steamPriority) {
            int i = this.currentDownloadChunkSize;
            int offset = (streamOffset / i) * i;
            RequestInfo requestInfo = this.priorityRequestInfo;
            if (requestInfo != null && requestInfo.offset != offset) {
                this.requestInfos.remove(this.priorityRequestInfo);
                this.requestedBytesCount -= this.currentDownloadChunkSize;
                removePart(this.notRequestedBytesRanges, this.priorityRequestInfo.offset, this.priorityRequestInfo.offset + this.currentDownloadChunkSize);
                if (this.priorityRequestInfo.requestToken != 0) {
                    ConnectionsManager.getInstance(this.currentAccount).cancelRequest(this.priorityRequestInfo.requestToken, true);
                    this.requestsCount--;
                }
                if (BuildVars.DEBUG_VERSION) {
                    FileLog.d("frame get cancel request at offset " + this.priorityRequestInfo.offset);
                }
                this.priorityRequestInfo = null;
            }
            if (this.priorityRequestInfo == null) {
                this.streamPriorityStartOffset = offset;
            }
        } else {
            int i2 = this.currentDownloadChunkSize;
            this.streamStartOffset = (streamOffset / i2) * i2;
        }
        this.streamListeners.add(stream);
        if (alreadyStarted) {
            if (this.preloadedBytesRanges != null && getDownloadedLengthFromOffsetInternal(this.notLoadedBytesRanges, this.streamStartOffset, 1) == 0 && this.preloadedBytesRanges.get(this.streamStartOffset) != null) {
                this.nextPartWasPreloaded = true;
            }
            startDownloadRequest();
            this.nextPartWasPreloaded = false;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$start$5$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m186lambda$start$5$imuecahxyvjdmessengerFileLoadOperation(boolean[] preloaded) {
        int i = this.totalBytesCount;
        if (i != 0 && ((this.isPreloadVideoOperation && preloaded[0]) || this.downloadedBytes == i)) {
            try {
                onFinishLoadingFile(false);
                return;
            } catch (Exception e) {
                onFail(true, 0);
                return;
            }
        }
        startDownloadRequest();
    }

    public boolean isPaused() {
        return this.paused;
    }

    public void setIsPreloadVideoOperation(final boolean value) {
        boolean z = this.isPreloadVideoOperation;
        if (z != value) {
            if (value && this.totalBytesCount <= 2097152) {
                return;
            }
            if (!value && z) {
                if (this.state == 3) {
                    this.isPreloadVideoOperation = value;
                    this.state = 0;
                    this.preloadFinished = false;
                    start();
                    return;
                } else if (this.state == 1) {
                    Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda2
                        @Override // java.lang.Runnable
                        public final void run() {
                            FileLoadOperation.this.m184xba061870(value);
                        }
                    });
                    return;
                } else {
                    this.isPreloadVideoOperation = value;
                    return;
                }
            }
            this.isPreloadVideoOperation = value;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$setIsPreloadVideoOperation$6$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m184xba061870(boolean value) {
        this.requestedBytesCount = 0;
        clearOperaion(null, true);
        this.isPreloadVideoOperation = value;
        startDownloadRequest();
    }

    public boolean isPreloadVideoOperation() {
        return this.isPreloadVideoOperation;
    }

    public boolean isPreloadFinished() {
        return this.preloadFinished;
    }

    public void cancel() {
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                FileLoadOperation.this.m177lambda$cancel$7$imuecahxyvjdmessengerFileLoadOperation();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$cancel$7$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m177lambda$cancel$7$imuecahxyvjdmessengerFileLoadOperation() {
        if (this.state == 3 || this.state == 2) {
            return;
        }
        if (this.requestInfos != null) {
            for (int a2 = 0; a2 < this.requestInfos.size(); a2++) {
                RequestInfo requestInfo = this.requestInfos.get(a2);
                if (requestInfo.requestToken != 0) {
                    ConnectionsManager.getInstance(this.currentAccount).cancelRequest(requestInfo.requestToken, true);
                }
            }
        }
        onFail(false, 1);
    }

    private void cleanup() {
        try {
            RandomAccessFile randomAccessFile = this.fileOutputStream;
            if (randomAccessFile != null) {
                try {
                    randomAccessFile.getChannel().close();
                } catch (Exception e) {
                    FileLog.e(e);
                }
                this.fileOutputStream.close();
                this.fileOutputStream = null;
            }
        } catch (Exception e2) {
            FileLog.e(e2);
        }
        try {
            RandomAccessFile randomAccessFile2 = this.preloadStream;
            if (randomAccessFile2 != null) {
                try {
                    randomAccessFile2.getChannel().close();
                } catch (Exception e3) {
                    FileLog.e(e3);
                }
                this.preloadStream.close();
                this.preloadStream = null;
            }
        } catch (Exception e4) {
            FileLog.e(e4);
        }
        try {
            RandomAccessFile randomAccessFile3 = this.fileReadStream;
            if (randomAccessFile3 != null) {
                try {
                    randomAccessFile3.getChannel().close();
                } catch (Exception e5) {
                    FileLog.e(e5);
                }
                this.fileReadStream.close();
                this.fileReadStream = null;
            }
        } catch (Exception e6) {
            FileLog.e(e6);
        }
        try {
            RandomAccessFile randomAccessFile4 = this.filePartsStream;
            if (randomAccessFile4 != null) {
                try {
                    randomAccessFile4.getChannel().close();
                } catch (Exception e7) {
                    FileLog.e(e7);
                }
                this.filePartsStream.close();
                this.filePartsStream = null;
            }
        } catch (Exception e8) {
            FileLog.e(e8);
        }
        try {
            RandomAccessFile randomAccessFile5 = this.fiv;
            if (randomAccessFile5 != null) {
                randomAccessFile5.close();
                this.fiv = null;
            }
        } catch (Exception e9) {
            FileLog.e(e9);
        }
        if (this.delayedRequestInfos != null) {
            for (int a2 = 0; a2 < this.delayedRequestInfos.size(); a2++) {
                RequestInfo requestInfo = this.delayedRequestInfos.get(a2);
                if (requestInfo.response != null) {
                    requestInfo.response.disableFree = false;
                    requestInfo.response.freeResources();
                } else if (requestInfo.responseWeb != null) {
                    requestInfo.responseWeb.disableFree = false;
                    requestInfo.responseWeb.freeResources();
                } else if (requestInfo.responseCdn != null) {
                    requestInfo.responseCdn.disableFree = false;
                    requestInfo.responseCdn.freeResources();
                }
            }
            this.delayedRequestInfos.clear();
        }
    }

    private void onFinishLoadingFile(final boolean increment) {
        boolean renameResult;
        if (this.state != 1) {
            return;
        }
        this.state = 3;
        cleanup();
        if (this.isPreloadVideoOperation) {
            this.preloadFinished = true;
            if (BuildVars.DEBUG_VERSION) {
                FileLog.d("finished preloading file to " + this.cacheFileTemp + " loaded " + this.totalPreloadedBytes + " of " + this.totalBytesCount);
            }
        } else {
            File file = this.cacheIvTemp;
            if (file != null) {
                file.delete();
                this.cacheIvTemp = null;
            }
            File file2 = this.cacheFileParts;
            if (file2 != null) {
                file2.delete();
                this.cacheFileParts = null;
            }
            File file3 = this.cacheFilePreload;
            if (file3 != null) {
                file3.delete();
                this.cacheFilePreload = null;
            }
            if (this.cacheFileTemp != null) {
                if (this.ungzip) {
                    try {
                        GZIPInputStream gzipInputStream = new GZIPInputStream(new FileInputStream(this.cacheFileTemp));
                        FileLoader.copyFile(gzipInputStream, this.cacheFileGzipTemp, 2097152);
                        gzipInputStream.close();
                        this.cacheFileTemp.delete();
                        this.cacheFileTemp = this.cacheFileGzipTemp;
                        this.ungzip = false;
                    } catch (ZipException e) {
                        this.ungzip = false;
                    } catch (Throwable e2) {
                        FileLog.e(e2);
                        if (BuildVars.LOGS_ENABLED) {
                            FileLog.e("unable to ungzip temp = " + this.cacheFileTemp + " to final = " + this.cacheFileFinal);
                        }
                    }
                }
                if (!this.ungzip) {
                    if (this.parentObject instanceof TLRPC.TL_theme) {
                        try {
                            renameResult = AndroidUtilities.copyFile(this.cacheFileTemp, this.cacheFileFinal);
                        } catch (Exception e3) {
                            FileLog.e(e3);
                            renameResult = false;
                        }
                    } else {
                        renameResult = this.cacheFileTemp.renameTo(this.cacheFileFinal);
                    }
                    if (!renameResult) {
                        if (BuildVars.LOGS_ENABLED) {
                            FileLog.e("unable to rename temp = " + this.cacheFileTemp + " to final = " + this.cacheFileFinal + " retry = " + this.renameRetryCount);
                        }
                        int i = this.renameRetryCount + 1;
                        this.renameRetryCount = i;
                        if (i < 3) {
                            this.state = 1;
                            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda1
                                @Override // java.lang.Runnable
                                public final void run() {
                                    FileLoadOperation.this.m181xd9b4d475(increment);
                                }
                            }, 200L);
                            return;
                        }
                        this.cacheFileFinal = this.cacheFileTemp;
                    }
                } else {
                    onFail(false, 0);
                    return;
                }
            }
            if (BuildVars.LOGS_ENABLED) {
                FileLog.d("finished downloading file to " + this.cacheFileFinal);
            }
            if (increment) {
                int i2 = this.currentType;
                if (i2 == 50331648) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedItemsCount(ApplicationLoader.getCurrentNetworkType(), 3, 1);
                } else if (i2 == 33554432) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedItemsCount(ApplicationLoader.getCurrentNetworkType(), 2, 1);
                } else if (i2 == 16777216) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedItemsCount(ApplicationLoader.getCurrentNetworkType(), 4, 1);
                } else if (i2 == 67108864) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedItemsCount(ApplicationLoader.getCurrentNetworkType(), 5, 1);
                }
            }
        }
        this.delegate.didFinishLoadingFile(this, this.cacheFileFinal);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onFinishLoadingFile$8$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m181xd9b4d475(boolean increment) {
        try {
            onFinishLoadingFile(increment);
        } catch (Exception e) {
            onFail(false, 0);
        }
    }

    private void delayRequestInfo(RequestInfo requestInfo) {
        this.delayedRequestInfos.add(requestInfo);
        if (requestInfo.response != null) {
            requestInfo.response.disableFree = true;
        } else if (requestInfo.responseWeb != null) {
            requestInfo.responseWeb.disableFree = true;
        } else if (requestInfo.responseCdn != null) {
            requestInfo.responseCdn.disableFree = true;
        }
    }

    /* JADX WARN: Code restructure failed: missing block: B:40:0x00c0, code lost:
        return 0;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private int findNextPreloadDownloadOffset(int atomOffset, int partOffset, NativeByteBuffer partBuffer) {
        int partSize = partBuffer.limit();
        while (true) {
            if (atomOffset < partOffset - (this.preloadTempBuffer != null ? 16 : 0) || atomOffset >= partOffset + partSize) {
                break;
            } else if (atomOffset >= (partOffset + partSize) - 16) {
                this.preloadTempBufferCount = (partOffset + partSize) - atomOffset;
                partBuffer.position(partBuffer.limit() - this.preloadTempBufferCount);
                partBuffer.readBytes(this.preloadTempBuffer, 0, this.preloadTempBufferCount, false);
                return partOffset + partSize;
            } else {
                if (this.preloadTempBufferCount != 0) {
                    partBuffer.position(0);
                    byte[] bArr = this.preloadTempBuffer;
                    int i = this.preloadTempBufferCount;
                    partBuffer.readBytes(bArr, i, 16 - i, false);
                    this.preloadTempBufferCount = 0;
                } else {
                    partBuffer.position(atomOffset - partOffset);
                    partBuffer.readBytes(this.preloadTempBuffer, 0, 16, false);
                }
                byte[] bArr2 = this.preloadTempBuffer;
                int atomSize = ((bArr2[0] & UByte.MAX_VALUE) << 24) + ((bArr2[1] & UByte.MAX_VALUE) << 16) + ((bArr2[2] & UByte.MAX_VALUE) << 8) + (bArr2[3] & UByte.MAX_VALUE);
                if (atomSize == 0) {
                    return 0;
                }
                if (atomSize == 1) {
                    atomSize = ((bArr2[12] & UByte.MAX_VALUE) << 24) + ((bArr2[13] & UByte.MAX_VALUE) << 16) + ((bArr2[14] & UByte.MAX_VALUE) << 8) + (bArr2[15] & UByte.MAX_VALUE);
                }
                if (bArr2[4] == 109 && bArr2[5] == 111 && bArr2[6] == 111 && bArr2[7] == 118) {
                    return -atomSize;
                }
                if (atomSize + atomOffset >= partOffset + partSize) {
                    return atomSize + atomOffset;
                }
                atomOffset += atomSize;
            }
        }
    }

    private void requestFileOffsets(int offset) {
        if (this.requestingCdnOffsets) {
            return;
        }
        this.requestingCdnOffsets = true;
        TLRPC.TL_upload_getCdnFileHashes req = new TLRPC.TL_upload_getCdnFileHashes();
        req.file_token = this.cdnToken;
        req.offset = offset;
        ConnectionsManager.getInstance(this.currentAccount).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda5
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                FileLoadOperation.this.m183x4c13b2ff(tLObject, tL_error);
            }
        }, null, null, 0, this.datacenterId, 1, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestFileOffsets$9$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m183x4c13b2ff(TLObject response, TLRPC.TL_error error) {
        if (error != null) {
            onFail(false, 0);
            return;
        }
        this.requestingCdnOffsets = false;
        TLRPC.Vector vector = (TLRPC.Vector) response;
        if (!vector.objects.isEmpty()) {
            if (this.cdnHashes == null) {
                this.cdnHashes = new SparseArray<>();
            }
            for (int a2 = 0; a2 < vector.objects.size(); a2++) {
                TLRPC.TL_fileHash hash = (TLRPC.TL_fileHash) vector.objects.get(a2);
                this.cdnHashes.put(hash.offset, hash);
            }
        }
        for (int a3 = 0; a3 < this.delayedRequestInfos.size(); a3++) {
            RequestInfo delayedRequestInfo = this.delayedRequestInfos.get(a3);
            if (this.notLoadedBytesRanges != null || this.downloadedBytes == delayedRequestInfo.offset) {
                this.delayedRequestInfos.remove(a3);
                if (!processRequestResult(delayedRequestInfo, null)) {
                    if (delayedRequestInfo.response == null) {
                        if (delayedRequestInfo.responseWeb == null) {
                            if (delayedRequestInfo.responseCdn != null) {
                                delayedRequestInfo.responseCdn.disableFree = false;
                                delayedRequestInfo.responseCdn.freeResources();
                                return;
                            }
                            return;
                        }
                        delayedRequestInfo.responseWeb.disableFree = false;
                        delayedRequestInfo.responseWeb.freeResources();
                        return;
                    }
                    delayedRequestInfo.response.disableFree = false;
                    delayedRequestInfo.response.freeResources();
                    return;
                }
                return;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* JADX WARN: Removed duplicated region for block: B:187:0x04cf A[Catch: Exception -> 0x04df, TryCatch #0 {Exception -> 0x04df, blocks: (B:10:0x0048, B:12:0x004c, B:14:0x0054, B:16:0x0058, B:18:0x005e, B:27:0x0082, B:190:0x04d9, B:30:0x008b, B:32:0x0095, B:34:0x00a0, B:37:0x00aa, B:39:0x00b1, B:41:0x00bf, B:42:0x00ef, B:44:0x00f3, B:46:0x0116, B:47:0x0144, B:49:0x0148, B:50:0x014f, B:52:0x0172, B:54:0x017e, B:56:0x018d, B:58:0x01a1, B:60:0x01ad, B:57:0x0198, B:59:0x01a5, B:61:0x01af, B:63:0x01ce, B:65:0x01d2, B:67:0x01d6, B:69:0x01dc, B:75:0x01e6, B:167:0x0463, B:169:0x046b, B:171:0x0477, B:174:0x0480, B:175:0x0483, B:177:0x048f, B:179:0x0495, B:180:0x04a4, B:182:0x04aa, B:183:0x04b9, B:185:0x04bf, B:187:0x04cf, B:188:0x04d4, B:76:0x01f4, B:78:0x01f8, B:80:0x0203, B:96:0x0224, B:98:0x0228, B:100:0x0243, B:102:0x0247, B:103:0x0251, B:105:0x0255, B:106:0x028b, B:108:0x028f, B:110:0x029d, B:111:0x02c1, B:113:0x02df, B:115:0x02f0, B:117:0x0300, B:123:0x0311, B:126:0x031f, B:128:0x0323, B:157:0x0432, B:159:0x0436, B:160:0x0442, B:162:0x0446, B:164:0x044b, B:137:0x0339, B:139:0x0345, B:141:0x035f, B:143:0x037d, B:145:0x0381, B:147:0x0385, B:148:0x03da, B:150:0x03de, B:151:0x0402, B:153:0x040c, B:120:0x0308, B:86:0x0212, B:89:0x0218, B:19:0x0065, B:21:0x006b, B:22:0x0072, B:24:0x0078), top: B:230:0x0048 }] */
    /* JADX WARN: Removed duplicated region for block: B:188:0x04d4 A[Catch: Exception -> 0x04df, TryCatch #0 {Exception -> 0x04df, blocks: (B:10:0x0048, B:12:0x004c, B:14:0x0054, B:16:0x0058, B:18:0x005e, B:27:0x0082, B:190:0x04d9, B:30:0x008b, B:32:0x0095, B:34:0x00a0, B:37:0x00aa, B:39:0x00b1, B:41:0x00bf, B:42:0x00ef, B:44:0x00f3, B:46:0x0116, B:47:0x0144, B:49:0x0148, B:50:0x014f, B:52:0x0172, B:54:0x017e, B:56:0x018d, B:58:0x01a1, B:60:0x01ad, B:57:0x0198, B:59:0x01a5, B:61:0x01af, B:63:0x01ce, B:65:0x01d2, B:67:0x01d6, B:69:0x01dc, B:75:0x01e6, B:167:0x0463, B:169:0x046b, B:171:0x0477, B:174:0x0480, B:175:0x0483, B:177:0x048f, B:179:0x0495, B:180:0x04a4, B:182:0x04aa, B:183:0x04b9, B:185:0x04bf, B:187:0x04cf, B:188:0x04d4, B:76:0x01f4, B:78:0x01f8, B:80:0x0203, B:96:0x0224, B:98:0x0228, B:100:0x0243, B:102:0x0247, B:103:0x0251, B:105:0x0255, B:106:0x028b, B:108:0x028f, B:110:0x029d, B:111:0x02c1, B:113:0x02df, B:115:0x02f0, B:117:0x0300, B:123:0x0311, B:126:0x031f, B:128:0x0323, B:157:0x0432, B:159:0x0436, B:160:0x0442, B:162:0x0446, B:164:0x044b, B:137:0x0339, B:139:0x0345, B:141:0x035f, B:143:0x037d, B:145:0x0381, B:147:0x0385, B:148:0x03da, B:150:0x03de, B:151:0x0402, B:153:0x040c, B:120:0x0308, B:86:0x0212, B:89:0x0218, B:19:0x0065, B:21:0x006b, B:22:0x0072, B:24:0x0078), top: B:230:0x0048 }] */
    /* JADX WARN: Removed duplicated region for block: B:75:0x01e6 A[Catch: Exception -> 0x04df, TryCatch #0 {Exception -> 0x04df, blocks: (B:10:0x0048, B:12:0x004c, B:14:0x0054, B:16:0x0058, B:18:0x005e, B:27:0x0082, B:190:0x04d9, B:30:0x008b, B:32:0x0095, B:34:0x00a0, B:37:0x00aa, B:39:0x00b1, B:41:0x00bf, B:42:0x00ef, B:44:0x00f3, B:46:0x0116, B:47:0x0144, B:49:0x0148, B:50:0x014f, B:52:0x0172, B:54:0x017e, B:56:0x018d, B:58:0x01a1, B:60:0x01ad, B:57:0x0198, B:59:0x01a5, B:61:0x01af, B:63:0x01ce, B:65:0x01d2, B:67:0x01d6, B:69:0x01dc, B:75:0x01e6, B:167:0x0463, B:169:0x046b, B:171:0x0477, B:174:0x0480, B:175:0x0483, B:177:0x048f, B:179:0x0495, B:180:0x04a4, B:182:0x04aa, B:183:0x04b9, B:185:0x04bf, B:187:0x04cf, B:188:0x04d4, B:76:0x01f4, B:78:0x01f8, B:80:0x0203, B:96:0x0224, B:98:0x0228, B:100:0x0243, B:102:0x0247, B:103:0x0251, B:105:0x0255, B:106:0x028b, B:108:0x028f, B:110:0x029d, B:111:0x02c1, B:113:0x02df, B:115:0x02f0, B:117:0x0300, B:123:0x0311, B:126:0x031f, B:128:0x0323, B:157:0x0432, B:159:0x0436, B:160:0x0442, B:162:0x0446, B:164:0x044b, B:137:0x0339, B:139:0x0345, B:141:0x035f, B:143:0x037d, B:145:0x0381, B:147:0x0385, B:148:0x03da, B:150:0x03de, B:151:0x0402, B:153:0x040c, B:120:0x0308, B:86:0x0212, B:89:0x0218, B:19:0x0065, B:21:0x006b, B:22:0x0072, B:24:0x0078), top: B:230:0x0048 }] */
    /* JADX WARN: Removed duplicated region for block: B:76:0x01f4 A[Catch: Exception -> 0x04df, TryCatch #0 {Exception -> 0x04df, blocks: (B:10:0x0048, B:12:0x004c, B:14:0x0054, B:16:0x0058, B:18:0x005e, B:27:0x0082, B:190:0x04d9, B:30:0x008b, B:32:0x0095, B:34:0x00a0, B:37:0x00aa, B:39:0x00b1, B:41:0x00bf, B:42:0x00ef, B:44:0x00f3, B:46:0x0116, B:47:0x0144, B:49:0x0148, B:50:0x014f, B:52:0x0172, B:54:0x017e, B:56:0x018d, B:58:0x01a1, B:60:0x01ad, B:57:0x0198, B:59:0x01a5, B:61:0x01af, B:63:0x01ce, B:65:0x01d2, B:67:0x01d6, B:69:0x01dc, B:75:0x01e6, B:167:0x0463, B:169:0x046b, B:171:0x0477, B:174:0x0480, B:175:0x0483, B:177:0x048f, B:179:0x0495, B:180:0x04a4, B:182:0x04aa, B:183:0x04b9, B:185:0x04bf, B:187:0x04cf, B:188:0x04d4, B:76:0x01f4, B:78:0x01f8, B:80:0x0203, B:96:0x0224, B:98:0x0228, B:100:0x0243, B:102:0x0247, B:103:0x0251, B:105:0x0255, B:106:0x028b, B:108:0x028f, B:110:0x029d, B:111:0x02c1, B:113:0x02df, B:115:0x02f0, B:117:0x0300, B:123:0x0311, B:126:0x031f, B:128:0x0323, B:157:0x0432, B:159:0x0436, B:160:0x0442, B:162:0x0446, B:164:0x044b, B:137:0x0339, B:139:0x0345, B:141:0x035f, B:143:0x037d, B:145:0x0381, B:147:0x0385, B:148:0x03da, B:150:0x03de, B:151:0x0402, B:153:0x040c, B:120:0x0308, B:86:0x0212, B:89:0x0218, B:19:0x0065, B:21:0x006b, B:22:0x0072, B:24:0x0078), top: B:230:0x0048 }] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean processRequestResult(RequestInfo requestInfo, TLRPC.TL_error error) {
        NativeByteBuffer bytes;
        boolean finishedDownloading;
        boolean finishedDownloading2;
        boolean finishedDownloading3;
        int a2;
        Integer val;
        if (this.state != 1) {
            if (BuildVars.DEBUG_VERSION) {
                FileLog.d("trying to write to finished file " + this.cacheFileFinal + " offset " + requestInfo.offset);
            }
            return false;
        }
        this.requestInfos.remove(requestInfo);
        if (error == null) {
            try {
            } catch (Exception e) {
                onFail(false, 0);
                FileLog.e(e);
            }
            if (this.notLoadedBytesRanges != null || this.downloadedBytes == requestInfo.offset) {
                if (requestInfo.response != null) {
                    bytes = requestInfo.response.bytes;
                } else {
                    bytes = requestInfo.responseWeb != null ? requestInfo.responseWeb.bytes : requestInfo.responseCdn != null ? requestInfo.responseCdn.bytes : null;
                }
                if (bytes != null && bytes.limit() != 0) {
                    int currentBytesSize = bytes.limit();
                    if (this.isCdn) {
                        int fileOffset = (requestInfo.offset / 131072) * 131072;
                        SparseArray<TLRPC.TL_fileHash> sparseArray = this.cdnHashes;
                        TLRPC.TL_fileHash hash = sparseArray != null ? sparseArray.get(fileOffset) : null;
                        if (hash == null) {
                            delayRequestInfo(requestInfo);
                            requestFileOffsets(fileOffset);
                            return true;
                        }
                    }
                    if (requestInfo.responseCdn != null) {
                        int offset = requestInfo.offset / 16;
                        byte[] bArr = this.cdnIv;
                        bArr[15] = (byte) (offset & 255);
                        bArr[14] = (byte) ((offset >> 8) & 255);
                        bArr[13] = (byte) ((offset >> 16) & 255);
                        bArr[12] = (byte) ((offset >> 24) & 255);
                        Utilities.aesCtrDecryption(bytes.buffer, this.cdnKey, this.cdnIv, 0, bytes.limit());
                    }
                    if (this.isPreloadVideoOperation) {
                        this.preloadStream.writeInt(requestInfo.offset);
                        this.preloadStream.writeInt(currentBytesSize);
                        this.preloadStreamFileOffset += 8;
                        this.preloadStream.getChannel().write(bytes.buffer);
                        if (BuildVars.DEBUG_VERSION) {
                            FileLog.d("save preload file part " + this.cacheFilePreload + " offset " + requestInfo.offset + " size " + currentBytesSize);
                        }
                        if (this.preloadedBytesRanges == null) {
                            this.preloadedBytesRanges = new SparseArray<>();
                        }
                        this.preloadedBytesRanges.put(requestInfo.offset, new PreloadRange(this.preloadStreamFileOffset, requestInfo.offset, currentBytesSize));
                        this.totalPreloadedBytes += currentBytesSize;
                        this.preloadStreamFileOffset += currentBytesSize;
                        if (this.moovFound == 0) {
                            int offset2 = findNextPreloadDownloadOffset(this.nextAtomOffset, requestInfo.offset, bytes);
                            if (offset2 < 0) {
                                offset2 *= -1;
                                int i = this.nextPreloadDownloadOffset + this.currentDownloadChunkSize;
                                this.nextPreloadDownloadOffset = i;
                                if (i >= this.totalBytesCount / 2) {
                                    this.foundMoovSize = 2097152;
                                    this.preloadNotRequestedBytesCount = 2097152;
                                    this.moovFound = 2;
                                } else {
                                    int i2 = 1048576 + offset2;
                                    this.foundMoovSize = i2;
                                    this.preloadNotRequestedBytesCount = i2;
                                    this.moovFound = 1;
                                }
                                this.nextPreloadDownloadOffset = -1;
                            } else {
                                int i3 = this.currentDownloadChunkSize;
                                this.nextPreloadDownloadOffset = (offset2 / i3) * i3;
                            }
                            this.nextAtomOffset = offset2;
                        }
                        this.preloadStream.writeInt(this.foundMoovSize);
                        this.preloadStream.writeInt(this.nextPreloadDownloadOffset);
                        this.preloadStream.writeInt(this.nextAtomOffset);
                        this.preloadStreamFileOffset += 12;
                        int i4 = this.nextPreloadDownloadOffset;
                        if (i4 != 0 && ((this.moovFound == 0 || this.foundMoovSize >= 0) && this.totalPreloadedBytes <= 2097152 && i4 < this.totalBytesCount)) {
                            finishedDownloading3 = false;
                            if (!finishedDownloading3) {
                                this.preloadStream.seek(0L);
                                this.preloadStream.write(1);
                            } else if (this.moovFound != 0) {
                                this.foundMoovSize -= this.currentDownloadChunkSize;
                            }
                        }
                        finishedDownloading3 = true;
                        if (!finishedDownloading3) {
                        }
                    } else {
                        int i5 = this.downloadedBytes + currentBytesSize;
                        this.downloadedBytes = i5;
                        int i6 = this.totalBytesCount;
                        if (i6 > 0) {
                            finishedDownloading = i5 >= i6;
                        } else {
                            int i7 = this.currentDownloadChunkSize;
                            if (currentBytesSize == i7 && ((i6 != i5 && i5 % i7 == 0) || (i6 > 0 && i6 > i5))) {
                                finishedDownloading = false;
                            }
                            finishedDownloading = true;
                        }
                        if (this.key != null) {
                            Utilities.aesIgeEncryption(bytes.buffer, this.key, this.iv, false, true, 0, bytes.limit());
                            if (finishedDownloading && this.bytesCountPadding != 0) {
                                bytes.limit(bytes.limit() - this.bytesCountPadding);
                            }
                        }
                        if (this.encryptFile) {
                            int offset3 = requestInfo.offset / 16;
                            byte[] bArr2 = this.encryptIv;
                            bArr2[15] = (byte) (offset3 & 255);
                            bArr2[14] = (byte) ((offset3 >> 8) & 255);
                            bArr2[13] = (byte) ((offset3 >> 16) & 255);
                            bArr2[12] = (byte) ((offset3 >> 24) & 255);
                            Utilities.aesCtrDecryption(bytes.buffer, this.encryptKey, this.encryptIv, 0, bytes.limit());
                        }
                        if (this.notLoadedBytesRanges != null) {
                            this.fileOutputStream.seek(requestInfo.offset);
                            if (BuildVars.DEBUG_VERSION) {
                                FileLog.d("save file part " + this.cacheFileFinal + " offset " + requestInfo.offset);
                            }
                        }
                        FileChannel channel = this.fileOutputStream.getChannel();
                        channel.write(bytes.buffer);
                        addPart(this.notLoadedBytesRanges, requestInfo.offset, requestInfo.offset + currentBytesSize, true);
                        if (this.isCdn) {
                            int cdnCheckPart = requestInfo.offset / 131072;
                            int size = this.notCheckedCdnRanges.size();
                            boolean checked = true;
                            int a3 = 0;
                            while (true) {
                                if (a3 >= size) {
                                    break;
                                }
                                Range range = this.notCheckedCdnRanges.get(a3);
                                FileChannel channel2 = channel;
                                if (range.start > cdnCheckPart || cdnCheckPart > range.end) {
                                    a3++;
                                    channel = channel2;
                                } else {
                                    checked = false;
                                    break;
                                }
                            }
                            if (checked) {
                                finishedDownloading2 = finishedDownloading;
                            } else {
                                int fileOffset2 = cdnCheckPart * 131072;
                                int availableSize = getDownloadedLengthFromOffsetInternal(this.notLoadedBytesRanges, fileOffset2, 131072);
                                if (availableSize != 0) {
                                    if (availableSize != 131072) {
                                        int i8 = this.totalBytesCount;
                                        if (i8 > 0) {
                                            int size2 = i8 - fileOffset2;
                                            if (availableSize != size2) {
                                            }
                                        }
                                        if (i8 > 0 || !finishedDownloading) {
                                            finishedDownloading2 = finishedDownloading;
                                        }
                                    }
                                    TLRPC.TL_fileHash hash2 = this.cdnHashes.get(fileOffset2);
                                    if (this.fileReadStream == null) {
                                        this.cdnCheckBytes = new byte[131072];
                                        finishedDownloading2 = finishedDownloading;
                                        this.fileReadStream = new RandomAccessFile(this.cacheFileTemp, "r");
                                    } else {
                                        finishedDownloading2 = finishedDownloading;
                                    }
                                    this.fileReadStream.seek(fileOffset2);
                                    this.fileReadStream.readFully(this.cdnCheckBytes, 0, availableSize);
                                    byte[] sha256 = Utilities.computeSHA256(this.cdnCheckBytes, 0, availableSize);
                                    if (!Arrays.equals(sha256, hash2.hash)) {
                                        if (BuildVars.LOGS_ENABLED) {
                                            if (this.location != null) {
                                                FileLog.e("invalid cdn hash " + this.location + " id = " + this.location.id + " local_id = " + this.location.local_id + " access_hash = " + this.location.access_hash + " volume_id = " + this.location.volume_id + " secret = " + this.location.secret);
                                            } else if (this.webLocation != null) {
                                                FileLog.e("invalid cdn hash  " + this.webLocation + " id = " + getFileName());
                                            }
                                        }
                                        onFail(false, 0);
                                        this.cacheFileTemp.delete();
                                        return false;
                                    }
                                    this.cdnHashes.remove(fileOffset2);
                                    addPart(this.notCheckedCdnRanges, cdnCheckPart, cdnCheckPart + 1, false);
                                } else {
                                    finishedDownloading2 = finishedDownloading;
                                }
                            }
                        } else {
                            finishedDownloading2 = finishedDownloading;
                        }
                        RandomAccessFile randomAccessFile = this.fiv;
                        if (randomAccessFile != null) {
                            randomAccessFile.seek(0L);
                            this.fiv.write(this.iv);
                        }
                        if (this.totalBytesCount > 0 && this.state == 1) {
                            copyNotLoadedRanges();
                            this.delegate.didChangedLoadProgress(this, Math.min(1.0f, this.downloadedBytes / this.totalBytesCount));
                        }
                        finishedDownloading3 = finishedDownloading2;
                    }
                    while (a2 < this.delayedRequestInfos.size()) {
                        RequestInfo delayedRequestInfo = this.delayedRequestInfos.get(a2);
                        a2 = (this.notLoadedBytesRanges == null && this.downloadedBytes != delayedRequestInfo.offset) ? a2 + 1 : 0;
                        this.delayedRequestInfos.remove(a2);
                        if (!processRequestResult(delayedRequestInfo, null)) {
                            if (delayedRequestInfo.response != null) {
                                delayedRequestInfo.response.disableFree = false;
                                delayedRequestInfo.response.freeResources();
                            } else if (delayedRequestInfo.responseWeb != null) {
                                delayedRequestInfo.responseWeb.disableFree = false;
                                delayedRequestInfo.responseWeb.freeResources();
                            } else if (delayedRequestInfo.responseCdn != null) {
                                delayedRequestInfo.responseCdn.disableFree = false;
                                delayedRequestInfo.responseCdn.freeResources();
                            }
                        }
                        if (!finishedDownloading3) {
                            onFinishLoadingFile(true);
                        } else {
                            startDownloadRequest();
                        }
                        return false;
                    }
                    if (!finishedDownloading3) {
                    }
                    return false;
                }
                onFinishLoadingFile(true);
                return false;
            }
            delayRequestInfo(requestInfo);
            return false;
        } else if (error.text.contains("FILE_MIGRATE_")) {
            String errorMsg = error.text.replace("FILE_MIGRATE_", "");
            Scanner scanner = new Scanner(errorMsg);
            scanner.useDelimiter("");
            try {
                val = Integer.valueOf(scanner.nextInt());
            } catch (Exception e2) {
                val = null;
            }
            if (val == null) {
                onFail(false, 0);
            } else {
                this.datacenterId = val.intValue();
                this.downloadedBytes = 0;
                this.requestedBytesCount = 0;
                startDownloadRequest();
            }
            return false;
        } else if (error.text.contains("OFFSET_INVALID")) {
            if (this.downloadedBytes % this.currentDownloadChunkSize == 0) {
                try {
                    onFinishLoadingFile(true);
                    return false;
                } catch (Exception e3) {
                    FileLog.e(e3);
                    onFail(false, 0);
                    return false;
                }
            }
            onFail(false, 0);
            return false;
        } else if (error.text.contains("RETRY_LIMIT")) {
            onFail(false, 2);
            return false;
        } else {
            if (BuildVars.LOGS_ENABLED) {
                if (this.location != null) {
                    FileLog.e(error.text + " " + this.location + " id = " + this.location.id + " local_id = " + this.location.local_id + " access_hash = " + this.location.access_hash + " volume_id = " + this.location.volume_id + " secret = " + this.location.secret);
                } else if (this.webLocation != null) {
                    FileLog.e(error.text + " " + this.webLocation + " id = " + getFileName());
                }
            }
            onFail(false, 0);
            return false;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void onFail(boolean thread, final int reason) {
        cleanup();
        this.state = 2;
        if (thread) {
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    FileLoadOperation.this.m180lambda$onFail$10$imuecahxyvjdmessengerFileLoadOperation(reason);
                }
            });
        } else {
            this.delegate.didFailedLoadingFile(this, reason);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onFail$10$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m180lambda$onFail$10$imuecahxyvjdmessengerFileLoadOperation(int reason) {
        this.delegate.didFailedLoadingFile(this, reason);
    }

    private void clearOperaion(RequestInfo currentInfo, boolean preloadChanged) {
        int minOffset = Integer.MAX_VALUE;
        for (int a2 = 0; a2 < this.requestInfos.size(); a2++) {
            RequestInfo info = this.requestInfos.get(a2);
            minOffset = Math.min(info.offset, minOffset);
            if (this.isPreloadVideoOperation) {
                this.requestedPreloadedBytesRanges.delete(info.offset);
            } else {
                removePart(this.notRequestedBytesRanges, info.offset, info.offset + this.currentDownloadChunkSize);
            }
            if (currentInfo != info && info.requestToken != 0) {
                ConnectionsManager.getInstance(this.currentAccount).cancelRequest(info.requestToken, true);
            }
        }
        this.requestInfos.clear();
        for (int a3 = 0; a3 < this.delayedRequestInfos.size(); a3++) {
            RequestInfo info2 = this.delayedRequestInfos.get(a3);
            if (this.isPreloadVideoOperation) {
                this.requestedPreloadedBytesRanges.delete(info2.offset);
            } else {
                removePart(this.notRequestedBytesRanges, info2.offset, info2.offset + this.currentDownloadChunkSize);
            }
            if (info2.response != null) {
                info2.response.disableFree = false;
                info2.response.freeResources();
            } else if (info2.responseWeb != null) {
                info2.responseWeb.disableFree = false;
                info2.responseWeb.freeResources();
            } else if (info2.responseCdn != null) {
                info2.responseCdn.disableFree = false;
                info2.responseCdn.freeResources();
            }
            minOffset = Math.min(info2.offset, minOffset);
        }
        this.delayedRequestInfos.clear();
        this.requestsCount = 0;
        if (!preloadChanged && this.isPreloadVideoOperation) {
            this.requestedBytesCount = this.totalPreloadedBytes;
        } else if (this.notLoadedBytesRanges == null) {
            this.downloadedBytes = minOffset;
            this.requestedBytesCount = minOffset;
        }
    }

    private void requestReference(RequestInfo requestInfo) {
        if (this.requestingReference) {
            return;
        }
        clearOperaion(requestInfo, false);
        this.requestingReference = true;
        Object obj = this.parentObject;
        if (obj instanceof MessageObject) {
            MessageObject messageObject = (MessageObject) obj;
            if (messageObject.getId() < 0 && messageObject.messageOwner.media.webpage != null) {
                this.parentObject = messageObject.messageOwner.media.webpage;
            }
        }
        FileRefController.getInstance(this.currentAccount).requestReference(this.parentObject, this.location, this, requestInfo);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* JADX WARN: Multi-variable type inference failed */
    public void startDownloadRequest() {
        int count;
        int downloadOffset;
        int size;
        int flags;
        final TLRPC.TL_upload_getFile tL_upload_getFile;
        SparseArray<PreloadRange> sparseArray;
        PreloadRange range;
        ArrayList<Range> arrayList;
        int downloadOffset2;
        if (!this.paused && this.state == 1) {
            int i = 2097152;
            if (this.streamPriorityStartOffset == 0) {
                if (this.nextPartWasPreloaded || this.requestInfos.size() + this.delayedRequestInfos.size() < this.currentMaxDownloadRequests) {
                    if (this.isPreloadVideoOperation) {
                        if (this.requestedBytesCount <= 2097152) {
                            if (this.moovFound != 0 && this.requestInfos.size() > 0) {
                                return;
                            }
                        } else {
                            return;
                        }
                    }
                } else {
                    return;
                }
            }
            if (this.streamPriorityStartOffset == 0 && !this.nextPartWasPreloaded && ((!this.isPreloadVideoOperation || this.moovFound != 0) && this.totalBytesCount > 0)) {
                int count2 = Math.max(0, this.currentMaxDownloadRequests - this.requestInfos.size());
                count = count2;
            } else {
                count = 1;
            }
            int a2 = 0;
            while (a2 < count) {
                if (this.isPreloadVideoOperation) {
                    if (this.moovFound != 0 && this.preloadNotRequestedBytesCount <= 0) {
                        return;
                    }
                    if (this.nextPreloadDownloadOffset == -1) {
                        downloadOffset2 = 0;
                        boolean found = false;
                        int tries = (i / this.currentDownloadChunkSize) + 2;
                        while (true) {
                            if (tries == 0) {
                                break;
                            } else if (this.requestedPreloadedBytesRanges.get(downloadOffset2, 0) == 0) {
                                found = true;
                                break;
                            } else {
                                int i2 = this.currentDownloadChunkSize;
                                downloadOffset2 += i2;
                                int i3 = this.totalBytesCount;
                                if (downloadOffset2 > i3) {
                                    break;
                                }
                                if (this.moovFound == 2 && downloadOffset2 == i2 * 8) {
                                    downloadOffset2 = ((i3 - 1048576) / i2) * i2;
                                }
                                tries--;
                            }
                        }
                        if (!found && this.requestInfos.isEmpty()) {
                            onFinishLoadingFile(false);
                        }
                    } else {
                        downloadOffset2 = this.nextPreloadDownloadOffset;
                    }
                    if (this.requestedPreloadedBytesRanges == null) {
                        this.requestedPreloadedBytesRanges = new SparseIntArray();
                    }
                    this.requestedPreloadedBytesRanges.put(downloadOffset2, 1);
                    if (BuildVars.DEBUG_VERSION) {
                        FileLog.d("start next preload from " + downloadOffset2 + " size " + this.totalBytesCount + " for " + this.cacheFilePreload);
                    }
                    this.preloadNotRequestedBytesCount -= this.currentDownloadChunkSize;
                    downloadOffset = downloadOffset2;
                } else {
                    ArrayList<Range> arrayList2 = this.notRequestedBytesRanges;
                    if (arrayList2 != null) {
                        int sreamOffset = this.streamPriorityStartOffset;
                        if (sreamOffset == 0) {
                            sreamOffset = this.streamStartOffset;
                        }
                        int size2 = arrayList2.size();
                        int minStart = Integer.MAX_VALUE;
                        int minStreamStart = Integer.MAX_VALUE;
                        int b = 0;
                        while (true) {
                            if (b >= size2) {
                                break;
                            }
                            Range range2 = this.notRequestedBytesRanges.get(b);
                            if (sreamOffset != 0) {
                                if (range2.start > sreamOffset || range2.end <= sreamOffset) {
                                    if (sreamOffset < range2.start && range2.start < minStreamStart) {
                                        minStreamStart = range2.start;
                                    }
                                } else {
                                    minStreamStart = sreamOffset;
                                    minStart = Integer.MAX_VALUE;
                                    break;
                                }
                            }
                            minStart = Math.min(minStart, range2.start);
                            b++;
                        }
                        if (minStreamStart != Integer.MAX_VALUE) {
                            size = minStreamStart;
                        } else if (minStart != Integer.MAX_VALUE) {
                            size = minStart;
                        } else {
                            return;
                        }
                        downloadOffset = size;
                    } else {
                        int downloadOffset3 = this.requestedBytesCount;
                        downloadOffset = downloadOffset3;
                    }
                }
                if (!this.isPreloadVideoOperation && (arrayList = this.notRequestedBytesRanges) != null) {
                    addPart(arrayList, downloadOffset, this.currentDownloadChunkSize + downloadOffset, false);
                }
                int i4 = this.totalBytesCount;
                if (i4 <= 0 || downloadOffset < i4) {
                    boolean isLast = i4 <= 0 || a2 == count + (-1) || (i4 > 0 && this.currentDownloadChunkSize + downloadOffset >= i4);
                    int connectionType = this.requestsCount % 2 == 0 ? 2 : ConnectionsManager.ConnectionTypeDownload2;
                    int flags2 = this.isForceRequest ? 32 : 0;
                    TLRPC.InputWebFileLocation inputWebFileLocation = this.webLocation;
                    if (!(inputWebFileLocation instanceof TLRPC.TL_inputWebFileGeoPointLocation)) {
                        flags2 |= 2;
                    }
                    if (this.isCdn) {
                        TLRPC.TL_upload_getCdnFile req = new TLRPC.TL_upload_getCdnFile();
                        req.file_token = this.cdnToken;
                        req.offset = downloadOffset;
                        req.limit = this.currentDownloadChunkSize;
                        tL_upload_getFile = req;
                        flags = flags2 | 1;
                    } else if (inputWebFileLocation != null) {
                        TLRPC.TL_upload_getWebFile req2 = new TLRPC.TL_upload_getWebFile();
                        req2.location = this.webLocation;
                        req2.offset = downloadOffset;
                        req2.limit = this.currentDownloadChunkSize;
                        tL_upload_getFile = req2;
                        flags = flags2;
                    } else {
                        TLRPC.TL_upload_getFile req3 = new TLRPC.TL_upload_getFile();
                        TLRPC.UserFull userFull = MessagesController.getInstance(this.currentAccount).getUserFull(UserConfig.getInstance(this.currentAccount).getClientUserId());
                        if (userFull instanceof TLRPCContacts.CL_userFull_v1) {
                            req3.isCdnVip = ((TLRPCContacts.CL_userFull_v1) userFull).getExtendBean().vip;
                        }
                        req3.location = this.location;
                        req3.offset = downloadOffset;
                        req3.limit = this.currentDownloadChunkSize;
                        flags = flags2;
                        tL_upload_getFile = req3;
                    }
                    this.requestedBytesCount += this.currentDownloadChunkSize;
                    final RequestInfo requestInfo = new RequestInfo();
                    this.requestInfos.add(requestInfo);
                    requestInfo.offset = downloadOffset;
                    if (!this.isPreloadVideoOperation && this.supportsPreloading && this.preloadStream != null && (sparseArray = this.preloadedBytesRanges) != null && (range = sparseArray.get(requestInfo.offset)) != null) {
                        requestInfo.response = new TLRPC.TL_upload_file();
                        try {
                            NativeByteBuffer buffer = new NativeByteBuffer(range.length);
                            this.preloadStream.seek(range.fileOffset);
                            this.preloadStream.getChannel().read(buffer.buffer);
                            buffer.buffer.position(0);
                            requestInfo.response.bytes = buffer;
                            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda9
                                @Override // java.lang.Runnable
                                public final void run() {
                                    FileLoadOperation.this.m187xf5ddb73a(requestInfo);
                                }
                            });
                        } catch (Exception e) {
                        }
                        a2++;
                        i = 2097152;
                    }
                    if (this.streamPriorityStartOffset != 0) {
                        if (BuildVars.DEBUG_VERSION) {
                            FileLog.d("frame get offset = " + this.streamPriorityStartOffset);
                        }
                        this.streamPriorityStartOffset = 0;
                        this.priorityRequestInfo = requestInfo;
                    }
                    requestInfo.requestToken = ConnectionsManager.getInstance(this.currentAccount).sendRequest(tL_upload_getFile, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda10
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            FileLoadOperation.this.m189xc06094bc(requestInfo, tL_upload_getFile, tLObject, tL_error);
                        }
                    }, null, null, flags, this.isCdn ? this.cdnDatacenterId : this.datacenterId, connectionType, isLast);
                    this.requestsCount++;
                    a2++;
                    i = 2097152;
                } else {
                    return;
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startDownloadRequest$11$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m187xf5ddb73a(RequestInfo requestInfo) {
        processRequestResult(requestInfo, null);
        requestInfo.response.freeResources();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startDownloadRequest$13$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m189xc06094bc(final RequestInfo requestInfo, TLObject request, TLObject response, TLRPC.TL_error error) {
        if (!this.requestInfos.contains(requestInfo)) {
            return;
        }
        if (requestInfo == this.priorityRequestInfo) {
            if (BuildVars.DEBUG_VERSION) {
                FileLog.d("frame get request completed " + this.priorityRequestInfo.offset);
            }
            this.priorityRequestInfo = null;
        }
        if (error != null) {
            if (FileRefController.isFileRefError(error.text)) {
                requestReference(requestInfo);
                return;
            } else if ((request instanceof TLRPC.TL_upload_getCdnFile) && error.text.equals("FILE_TOKEN_INVALID")) {
                this.isCdn = false;
                clearOperaion(requestInfo, false);
                startDownloadRequest();
                return;
            }
        }
        if (response instanceof TLRPC.TL_upload_fileCdnRedirect) {
            TLRPC.TL_upload_fileCdnRedirect res = (TLRPC.TL_upload_fileCdnRedirect) response;
            if (!res.file_hashes.isEmpty()) {
                if (this.cdnHashes == null) {
                    this.cdnHashes = new SparseArray<>();
                }
                for (int a1 = 0; a1 < res.file_hashes.size(); a1++) {
                    TLRPC.TL_fileHash hash = res.file_hashes.get(a1);
                    this.cdnHashes.put(hash.offset, hash);
                }
            }
            if (res.encryption_iv == null || res.encryption_key == null || res.encryption_iv.length != 16 || res.encryption_key.length != 32) {
                TLRPC.TL_error error2 = new TLRPC.TL_error();
                error2.text = "bad redirect response";
                error2.code = 400;
                processRequestResult(requestInfo, error2);
                return;
            }
            this.isCdn = true;
            if (this.notCheckedCdnRanges == null) {
                ArrayList<Range> arrayList = new ArrayList<>();
                this.notCheckedCdnRanges = arrayList;
                arrayList.add(new Range(0, maxCdnParts));
            }
            this.cdnDatacenterId = res.dc_id;
            this.cdnIv = res.encryption_iv;
            this.cdnKey = res.encryption_key;
            this.cdnToken = res.file_token;
            clearOperaion(requestInfo, false);
            startDownloadRequest();
        } else if (response instanceof TLRPC.TL_upload_cdnFileReuploadNeeded) {
            if (!this.reuploadingCdn) {
                clearOperaion(requestInfo, false);
                this.reuploadingCdn = true;
                TLRPC.TL_upload_reuploadCdnFile req = new TLRPC.TL_upload_reuploadCdnFile();
                req.file_token = this.cdnToken;
                req.request_token = ((TLRPC.TL_upload_cdnFileReuploadNeeded) response).request_token;
                ConnectionsManager.getInstance(this.currentAccount).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileLoadOperation$$ExternalSyntheticLambda14
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileLoadOperation.this.m188xdb1f25fb(requestInfo, tLObject, tL_error);
                    }
                }, null, null, 0, this.datacenterId, 1, true);
            }
        } else {
            if (response instanceof TLRPC.TL_upload_file) {
                requestInfo.response = (TLRPC.TL_upload_file) response;
            } else if (response instanceof TLRPC.TL_upload_webFile) {
                requestInfo.responseWeb = (TLRPC.TL_upload_webFile) response;
                if (this.totalBytesCount == 0 && requestInfo.responseWeb.size != 0) {
                    this.totalBytesCount = requestInfo.responseWeb.size;
                }
            } else {
                requestInfo.responseCdn = (TLRPC.TL_upload_cdnFile) response;
            }
            if (response != null) {
                int i = this.currentType;
                if (i == 50331648) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedBytesCount(response.networkType, 3, response.getObjectSize() + 4);
                } else if (i == 33554432) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedBytesCount(response.networkType, 2, response.getObjectSize() + 4);
                } else if (i == 16777216) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedBytesCount(response.networkType, 4, response.getObjectSize() + 4);
                } else if (i == 67108864) {
                    StatsController.getInstance(this.currentAccount).incrementReceivedBytesCount(response.networkType, 5, response.getObjectSize() + 4);
                }
            }
            processRequestResult(requestInfo, error);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startDownloadRequest$12$im-uecahxyvjd-messenger-FileLoadOperation  reason: not valid java name */
    public /* synthetic */ void m188xdb1f25fb(RequestInfo requestInfo, TLObject response1, TLRPC.TL_error error1) {
        this.reuploadingCdn = false;
        if (error1 == null) {
            TLRPC.Vector vector = (TLRPC.Vector) response1;
            if (!vector.objects.isEmpty()) {
                if (this.cdnHashes == null) {
                    this.cdnHashes = new SparseArray<>();
                }
                for (int a1 = 0; a1 < vector.objects.size(); a1++) {
                    TLRPC.TL_fileHash hash = (TLRPC.TL_fileHash) vector.objects.get(a1);
                    this.cdnHashes.put(hash.offset, hash);
                }
            }
            startDownloadRequest();
        } else if (error1.text.equals("FILE_TOKEN_INVALID") || error1.text.equals("REQUEST_TOKEN_INVALID")) {
            this.isCdn = false;
            clearOperaion(requestInfo, false);
            startDownloadRequest();
        } else {
            onFail(false, 0);
        }
    }

    public void setDelegate(FileLoadOperationDelegate delegate) {
        this.delegate = delegate;
    }
}
