package im.uecahxyvjd.messenger;

import android.os.SystemClock;
import im.uecahxyvjd.messenger.FileLoadOperation;
import im.uecahxyvjd.messenger.SendMessagesHelper;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.Theme;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
/* loaded from: classes16.dex */
public class FileRefController extends BaseController {
    private static volatile FileRefController[] Instance = new FileRefController[3];
    private long lastCleanupTime;
    private HashMap<String, ArrayList<Requester>> locationRequester;
    private HashMap<TLRPC.TL_messages_sendMultiMedia, Object[]> multiMediaCache;
    private HashMap<String, ArrayList<Requester>> parentRequester;
    private HashMap<String, CachedResult> responseCache;

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes16.dex */
    public class Requester {
        private Object[] args;
        private boolean completed;
        private TLRPC.InputFileLocation location;
        private String locationKey;

        private Requester() {
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes16.dex */
    public class CachedResult {
        private long firstQueryTime;
        private long lastQueryTime;
        private TLObject response;

        private CachedResult() {
        }
    }

    public static FileRefController getInstance(int num) {
        FileRefController localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (FileRefController.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    FileRefController[] fileRefControllerArr = Instance;
                    FileRefController fileRefController = new FileRefController(num);
                    localInstance = fileRefController;
                    fileRefControllerArr[num] = fileRefController;
                }
            }
        }
        return localInstance;
    }

    public FileRefController(int instance) {
        super(instance);
        this.locationRequester = new HashMap<>();
        this.parentRequester = new HashMap<>();
        this.responseCache = new HashMap<>();
        this.multiMediaCache = new HashMap<>();
        this.lastCleanupTime = SystemClock.uptimeMillis();
    }

    public static String getKeyForParentObject(Object parentObject) {
        if (parentObject instanceof MessageObject) {
            MessageObject messageObject = (MessageObject) parentObject;
            int channelId = messageObject.getChannelId();
            return "message" + messageObject.getRealId() + "_" + channelId + "_" + messageObject.scheduled;
        } else if (parentObject instanceof TLRPC.Message) {
            TLRPC.Message message = (TLRPC.Message) parentObject;
            int channelId2 = message.to_id != null ? message.to_id.channel_id : 0;
            return "message" + message.id + "_" + channelId2;
        } else if (parentObject instanceof TLRPC.WebPage) {
            TLRPC.WebPage webPage = (TLRPC.WebPage) parentObject;
            return "webpage" + webPage.id;
        } else if (parentObject instanceof TLRPC.User) {
            TLRPC.User user = (TLRPC.User) parentObject;
            return "user" + user.id;
        } else if (parentObject instanceof TLRPC.Chat) {
            TLRPC.Chat chat = (TLRPC.Chat) parentObject;
            return "chat" + chat.id;
        } else if (parentObject instanceof String) {
            String string = (String) parentObject;
            return "str" + string;
        } else if (parentObject instanceof TLRPC.TL_messages_stickerSet) {
            TLRPC.TL_messages_stickerSet stickerSet = (TLRPC.TL_messages_stickerSet) parentObject;
            return "set" + stickerSet.set.id;
        } else if (parentObject instanceof TLRPC.StickerSetCovered) {
            TLRPC.StickerSetCovered stickerSet2 = (TLRPC.StickerSetCovered) parentObject;
            return "set" + stickerSet2.set.id;
        } else if (parentObject instanceof TLRPC.InputStickerSet) {
            TLRPC.InputStickerSet inputStickerSet = (TLRPC.InputStickerSet) parentObject;
            return "set" + inputStickerSet.id;
        } else if (parentObject instanceof TLRPC.TL_wallPaper) {
            TLRPC.TL_wallPaper wallPaper = (TLRPC.TL_wallPaper) parentObject;
            return "wallpaper" + wallPaper.id;
        } else if (parentObject instanceof TLRPC.TL_theme) {
            TLRPC.TL_theme theme = (TLRPC.TL_theme) parentObject;
            return "theme" + theme.id;
        } else if (parentObject != null) {
            return "" + parentObject;
        } else {
            return null;
        }
    }

    public void requestReference(Object parentObject, Object... args) {
        TLRPC.InputFileLocation location;
        String locationKey;
        if (BuildVars.LOGS_ENABLED) {
            FileLog.d("start loading request reference for parent = " + parentObject + " args = " + args[0]);
        }
        if (args[0] instanceof TLRPC.TL_inputSingleMedia) {
            TLRPC.TL_inputSingleMedia req = (TLRPC.TL_inputSingleMedia) args[0];
            if (req.media instanceof TLRPC.TL_inputMediaDocument) {
                TLRPC.TL_inputMediaDocument mediaDocument = (TLRPC.TL_inputMediaDocument) req.media;
                locationKey = "file_" + mediaDocument.id.id;
                location = new TLRPC.TL_inputDocumentFileLocation();
                location.id = mediaDocument.id.id;
            } else if (req.media instanceof TLRPC.TL_inputMediaPhoto) {
                TLRPC.TL_inputMediaPhoto mediaPhoto = (TLRPC.TL_inputMediaPhoto) req.media;
                String locationKey2 = "photo_" + mediaPhoto.id.id;
                location = new TLRPC.TL_inputPhotoFileLocation();
                location.id = mediaPhoto.id.id;
                locationKey = locationKey2;
            } else {
                sendErrorToObject(args, 0);
                return;
            }
        } else if (args[0] instanceof TLRPC.TL_messages_sendMultiMedia) {
            TLRPC.TL_messages_sendMultiMedia req2 = (TLRPC.TL_messages_sendMultiMedia) args[0];
            ArrayList<Object> parentObjects = (ArrayList) parentObject;
            this.multiMediaCache.put(req2, args);
            int size = req2.multi_media.size();
            for (int a2 = 0; a2 < size; a2++) {
                TLRPC.TL_inputSingleMedia media = req2.multi_media.get(a2);
                Object parentObject2 = parentObjects.get(a2);
                if (parentObject2 != null) {
                    requestReference(parentObject2, media, req2);
                }
            }
            return;
        } else if (args[0] instanceof TLRPC.TL_messages_sendMedia) {
            TLRPC.TL_messages_sendMedia req3 = (TLRPC.TL_messages_sendMedia) args[0];
            if (req3.media instanceof TLRPC.TL_inputMediaDocument) {
                TLRPC.TL_inputMediaDocument mediaDocument2 = (TLRPC.TL_inputMediaDocument) req3.media;
                locationKey = "file_" + mediaDocument2.id.id;
                location = new TLRPC.TL_inputDocumentFileLocation();
                location.id = mediaDocument2.id.id;
            } else if (req3.media instanceof TLRPC.TL_inputMediaPhoto) {
                TLRPC.TL_inputMediaPhoto mediaPhoto2 = (TLRPC.TL_inputMediaPhoto) req3.media;
                String locationKey3 = "photo_" + mediaPhoto2.id.id;
                location = new TLRPC.TL_inputPhotoFileLocation();
                location.id = mediaPhoto2.id.id;
                locationKey = locationKey3;
            } else {
                sendErrorToObject(args, 0);
                return;
            }
        } else if (args[0] instanceof TLRPC.TL_messages_editMessage) {
            TLRPC.TL_messages_editMessage req4 = (TLRPC.TL_messages_editMessage) args[0];
            if (req4.media instanceof TLRPC.TL_inputMediaDocument) {
                TLRPC.TL_inputMediaDocument mediaDocument3 = (TLRPC.TL_inputMediaDocument) req4.media;
                locationKey = "file_" + mediaDocument3.id.id;
                location = new TLRPC.TL_inputDocumentFileLocation();
                location.id = mediaDocument3.id.id;
            } else if (req4.media instanceof TLRPC.TL_inputMediaPhoto) {
                TLRPC.TL_inputMediaPhoto mediaPhoto3 = (TLRPC.TL_inputMediaPhoto) req4.media;
                String locationKey4 = "photo_" + mediaPhoto3.id.id;
                location = new TLRPC.TL_inputPhotoFileLocation();
                location.id = mediaPhoto3.id.id;
                locationKey = locationKey4;
            } else {
                sendErrorToObject(args, 0);
                return;
            }
        } else if (args[0] instanceof TLRPC.TL_messages_saveGif) {
            TLRPC.TL_messages_saveGif req5 = (TLRPC.TL_messages_saveGif) args[0];
            locationKey = "file_" + req5.id.id;
            location = new TLRPC.TL_inputDocumentFileLocation();
            location.id = req5.id.id;
        } else if (args[0] instanceof TLRPC.TL_messages_saveRecentSticker) {
            TLRPC.TL_messages_saveRecentSticker req6 = (TLRPC.TL_messages_saveRecentSticker) args[0];
            locationKey = "file_" + req6.id.id;
            location = new TLRPC.TL_inputDocumentFileLocation();
            location.id = req6.id.id;
        } else if (args[0] instanceof TLRPC.TL_messages_faveSticker) {
            TLRPC.TL_messages_faveSticker req7 = (TLRPC.TL_messages_faveSticker) args[0];
            locationKey = "file_" + req7.id.id;
            location = new TLRPC.TL_inputDocumentFileLocation();
            location.id = req7.id.id;
        } else if (args[0] instanceof TLRPC.TL_messages_getAttachedStickers) {
            TLRPC.TL_messages_getAttachedStickers req8 = (TLRPC.TL_messages_getAttachedStickers) args[0];
            if (req8.media instanceof TLRPC.TL_inputStickeredMediaDocument) {
                TLRPC.TL_inputStickeredMediaDocument mediaDocument4 = (TLRPC.TL_inputStickeredMediaDocument) req8.media;
                locationKey = "file_" + mediaDocument4.id.id;
                location = new TLRPC.TL_inputDocumentFileLocation();
                location.id = mediaDocument4.id.id;
            } else if (req8.media instanceof TLRPC.TL_inputStickeredMediaPhoto) {
                TLRPC.TL_inputStickeredMediaPhoto mediaPhoto4 = (TLRPC.TL_inputStickeredMediaPhoto) req8.media;
                String locationKey5 = "photo_" + mediaPhoto4.id.id;
                location = new TLRPC.TL_inputPhotoFileLocation();
                location.id = mediaPhoto4.id.id;
                locationKey = locationKey5;
            } else {
                sendErrorToObject(args, 0);
                return;
            }
        } else if (args[0] instanceof TLRPC.TL_inputFileLocation) {
            location = (TLRPC.TL_inputFileLocation) args[0];
            locationKey = "loc_" + location.local_id + "_" + location.volume_id;
        } else if (args[0] instanceof TLRPC.TL_inputDocumentFileLocation) {
            location = (TLRPC.TL_inputDocumentFileLocation) args[0];
            locationKey = "file_" + location.id;
        } else if (args[0] instanceof TLRPC.TL_inputPhotoFileLocation) {
            location = (TLRPC.TL_inputPhotoFileLocation) args[0];
            locationKey = "photo_" + location.id;
        } else {
            sendErrorToObject(args, 0);
            return;
        }
        if (parentObject instanceof MessageObject) {
            MessageObject messageObject = (MessageObject) parentObject;
            if (messageObject.getRealId() < 0 && messageObject.messageOwner.media.webpage != null) {
                parentObject = messageObject.messageOwner.media.webpage;
            }
        }
        String parentKey = getKeyForParentObject(parentObject);
        if (parentKey == null) {
            sendErrorToObject(args, 0);
            return;
        }
        Requester requester = new Requester();
        requester.args = args;
        requester.location = location;
        requester.locationKey = locationKey;
        int added = 0;
        ArrayList<Requester> arrayList = this.locationRequester.get(locationKey);
        if (arrayList == null) {
            arrayList = new ArrayList<>();
            this.locationRequester.put(locationKey, arrayList);
            added = 0 + 1;
        }
        arrayList.add(requester);
        ArrayList<Requester> arrayList2 = this.parentRequester.get(parentKey);
        if (arrayList2 == null) {
            arrayList2 = new ArrayList<>();
            this.parentRequester.put(parentKey, arrayList2);
            added++;
        }
        arrayList2.add(requester);
        if (added != 2) {
            return;
        }
        cleanupCache();
        CachedResult cachedResult = getCachedResponse(locationKey);
        if (cachedResult != null) {
            if (!onRequestComplete(locationKey, parentKey, cachedResult.response, false)) {
                this.responseCache.remove(locationKey);
            } else {
                return;
            }
        } else {
            CachedResult cachedResult2 = getCachedResponse(parentKey);
            if (cachedResult2 != null) {
                if (!onRequestComplete(locationKey, parentKey, cachedResult2.response, false)) {
                    this.responseCache.remove(parentKey);
                } else {
                    return;
                }
            }
        }
        requestReferenceFromServer(parentObject, locationKey, parentKey, args);
    }

    private void requestReferenceFromServer(Object parentObject, final String locationKey, final String parentKey, Object[] args) {
        if (parentObject instanceof MessageObject) {
            MessageObject messageObject = (MessageObject) parentObject;
            int channelId = messageObject.getChannelId();
            if (messageObject.scheduled) {
                TLRPC.TL_messages_getScheduledMessages req = new TLRPC.TL_messages_getScheduledMessages();
                req.peer = getMessagesController().getInputPeer((int) messageObject.getDialogId());
                req.id.add(Integer.valueOf(messageObject.getRealId()));
                getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda29
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m209x315b93b8(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if (channelId != 0) {
                TLRPC.TL_channels_getMessages req2 = new TLRPC.TL_channels_getMessages();
                req2.channel = getMessagesController().getInputChannel(channelId);
                req2.id.add(Integer.valueOf(messageObject.getRealId()));
                getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda8
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m210x169d0279(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else {
                TLRPC.TL_messages_getMessages req3 = new TLRPC.TL_messages_getMessages();
                req3.id.add(Integer.valueOf(messageObject.getRealId()));
                getConnectionsManager().sendRequest(req3, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda9
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m221xfbde713a(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            }
        } else if (parentObject instanceof TLRPC.TL_wallPaper) {
            TLRPC.TL_wallPaper wallPaper = (TLRPC.TL_wallPaper) parentObject;
            TLRPC.TL_account_getWallPaper req4 = new TLRPC.TL_account_getWallPaper();
            TLRPC.TL_inputWallPaper inputWallPaper = new TLRPC.TL_inputWallPaper();
            inputWallPaper.id = wallPaper.id;
            inputWallPaper.access_hash = wallPaper.access_hash;
            req4.wallpaper = inputWallPaper;
            getConnectionsManager().sendRequest(req4, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda10
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.this.m222xe11fdffb(locationKey, parentKey, tLObject, tL_error);
                }
            });
        } else if (parentObject instanceof TLRPC.TL_theme) {
            TLRPC.TL_theme theme = (TLRPC.TL_theme) parentObject;
            TLRPC.TL_account_getTheme req5 = new TLRPC.TL_account_getTheme();
            TLRPC.TL_inputTheme inputTheme = new TLRPC.TL_inputTheme();
            inputTheme.id = theme.id;
            inputTheme.access_hash = theme.access_hash;
            req5.theme = inputTheme;
            req5.format = "android";
            getConnectionsManager().sendRequest(req5, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda12
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.this.m223xc6614ebc(locationKey, parentKey, tLObject, tL_error);
                }
            });
        } else if (parentObject instanceof TLRPC.WebPage) {
            TLRPC.WebPage webPage = (TLRPC.WebPage) parentObject;
            TLRPC.TL_messages_getWebPage req6 = new TLRPC.TL_messages_getWebPage();
            req6.url = webPage.url;
            req6.hash = 0;
            getConnectionsManager().sendRequest(req6, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda13
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.this.m224xaba2bd7d(locationKey, parentKey, tLObject, tL_error);
                }
            });
        } else if (parentObject instanceof TLRPC.User) {
            TLRPC.User user = (TLRPC.User) parentObject;
            TLRPC.TL_users_getUsers req7 = new TLRPC.TL_users_getUsers();
            req7.id.add(getMessagesController().getInputUser(user));
            getConnectionsManager().sendRequest(req7, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda14
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.this.m225x90e42c3e(locationKey, parentKey, tLObject, tL_error);
                }
            });
        } else if (parentObject instanceof TLRPC.Chat) {
            TLRPC.Chat chat = (TLRPC.Chat) parentObject;
            if (chat instanceof TLRPC.TL_chat) {
                TLRPC.TL_messages_getChats req8 = new TLRPC.TL_messages_getChats();
                req8.id.add(Integer.valueOf(chat.id));
                getConnectionsManager().sendRequest(req8, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda15
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m226x76259aff(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if (chat instanceof TLRPC.TL_channel) {
                TLRPC.TL_channels_getChannels req9 = new TLRPC.TL_channels_getChannels();
                req9.id.add(MessagesController.getInputChannel(chat));
                getConnectionsManager().sendRequest(req9, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda16
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m227x5b6709c0(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            }
        } else if (!(parentObject instanceof String)) {
            if (parentObject instanceof TLRPC.TL_messages_stickerSet) {
                TLRPC.TL_messages_stickerSet stickerSet = (TLRPC.TL_messages_stickerSet) parentObject;
                TLRPC.TL_messages_getStickerSet req10 = new TLRPC.TL_messages_getStickerSet();
                req10.stickerset = new TLRPC.TL_inputStickerSetID();
                req10.stickerset.id = stickerSet.set.id;
                req10.stickerset.access_hash = stickerSet.set.access_hash;
                getConnectionsManager().sendRequest(req10, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda5
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m218xf9129b6a(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if (!(parentObject instanceof TLRPC.StickerSetCovered)) {
                if (parentObject instanceof TLRPC.InputStickerSet) {
                    TLRPC.TL_messages_getStickerSet req11 = new TLRPC.TL_messages_getStickerSet();
                    req11.stickerset = (TLRPC.InputStickerSet) parentObject;
                    getConnectionsManager().sendRequest(req11, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda7
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            FileRefController.this.m220xc39578ec(locationKey, parentKey, tLObject, tL_error);
                        }
                    });
                    return;
                }
                sendErrorToObject(args, 0);
            } else {
                TLRPC.StickerSetCovered stickerSet2 = (TLRPC.StickerSetCovered) parentObject;
                TLRPC.TL_messages_getStickerSet req12 = new TLRPC.TL_messages_getStickerSet();
                req12.stickerset = new TLRPC.TL_inputStickerSetID();
                req12.stickerset.id = stickerSet2.set.id;
                req12.stickerset.access_hash = stickerSet2.set.access_hash;
                getConnectionsManager().sendRequest(req12, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda6
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m219xde540a2b(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            }
        } else {
            String string = (String) parentObject;
            if ("wallpaper".equals(string)) {
                getConnectionsManager().sendRequest(new TLRPC.TL_account_getWallPapers(), new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda17
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m228x40a87881(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if (string.startsWith("gif")) {
                getConnectionsManager().sendRequest(new TLRPC.TL_messages_getSavedGifs(), new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda30
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m211xb4489423(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if ("recent".equals(string)) {
                getConnectionsManager().sendRequest(new TLRPC.TL_messages_getRecentStickers(), new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda31
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m212x998a02e4(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if ("fav".equals(string)) {
                getConnectionsManager().sendRequest(new TLRPC.TL_messages_getFavedStickers(), new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda32
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m213x7ecb71a5(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if (string.startsWith("avatar_")) {
                int id = Utilities.parseInt(string).intValue();
                if (id > 0) {
                    TLRPC.TL_photos_getUserPhotos req13 = new TLRPC.TL_photos_getUserPhotos();
                    req13.limit = 80;
                    req13.offset = 0;
                    req13.max_id = 0L;
                    req13.user_id = getMessagesController().getInputUser(id);
                    getConnectionsManager().sendRequest(req13, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda1
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            FileRefController.this.m214x640ce066(locationKey, parentKey, tLObject, tL_error);
                        }
                    });
                    return;
                }
                TLRPC.TL_messages_search req14 = new TLRPC.TL_messages_search();
                req14.filter = new TLRPC.TL_inputMessagesFilterChatPhotos();
                req14.limit = 80;
                req14.offset_id = 0;
                req14.q = "";
                req14.peer = getMessagesController().getInputPeer(id);
                getConnectionsManager().sendRequest(req14, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda2
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        FileRefController.this.m215x494e4f27(locationKey, parentKey, tLObject, tL_error);
                    }
                });
            } else if (string.startsWith("sent_")) {
                String[] params = string.split("_");
                if (params.length == 3) {
                    int channelId2 = Utilities.parseInt(params[1]).intValue();
                    if (channelId2 != 0) {
                        TLRPC.TL_channels_getMessages req15 = new TLRPC.TL_channels_getMessages();
                        req15.channel = getMessagesController().getInputChannel(channelId2);
                        req15.id.add(Utilities.parseInt(params[2]));
                        getConnectionsManager().sendRequest(req15, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda3
                            @Override // im.uecahxyvjd.tgnet.RequestDelegate
                            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                                FileRefController.this.m216x2e8fbde8(locationKey, parentKey, tLObject, tL_error);
                            }
                        });
                        return;
                    }
                    TLRPC.TL_messages_getMessages req16 = new TLRPC.TL_messages_getMessages();
                    req16.id.add(Utilities.parseInt(params[2]));
                    getConnectionsManager().sendRequest(req16, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda4
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            FileRefController.this.m217x13d12ca9(locationKey, parentKey, tLObject, tL_error);
                        }
                    });
                    return;
                }
                sendErrorToObject(args, 0);
            } else {
                sendErrorToObject(args, 0);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$0$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m209x315b93b8(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$1$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m210x169d0279(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$2$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m221xfbde713a(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$3$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m222xe11fdffb(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$4$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m223xc6614ebc(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$5$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m224xaba2bd7d(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$6$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m225x90e42c3e(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$7$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m226x76259aff(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$8$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m227x5b6709c0(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$9$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m228x40a87881(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$10$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m211xb4489423(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$11$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m212x998a02e4(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$12$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m213x7ecb71a5(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$13$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m214x640ce066(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$14$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m215x494e4f27(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$15$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m216x2e8fbde8(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$16$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m217x13d12ca9(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$17$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m218xf9129b6a(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$18$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m219xde540a2b(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$requestReferenceFromServer$19$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m220xc39578ec(String locationKey, String parentKey, TLObject response, TLRPC.TL_error error) {
        onRequestComplete(locationKey, parentKey, response, true);
    }

    private void onUpdateObjectReference(final Requester requester, byte[] file_reference, TLRPC.InputFileLocation locationReplacement) {
        if (BuildVars.DEBUG_VERSION) {
            FileLog.d("fileref updated for " + requester.args[0] + " " + requester.locationKey);
        }
        if (requester.args[0] instanceof TLRPC.TL_inputSingleMedia) {
            final TLRPC.TL_messages_sendMultiMedia multiMedia = (TLRPC.TL_messages_sendMultiMedia) requester.args[1];
            final Object[] objects = this.multiMediaCache.get(multiMedia);
            if (objects != null) {
                TLRPC.TL_inputSingleMedia req = (TLRPC.TL_inputSingleMedia) requester.args[0];
                if (req.media instanceof TLRPC.TL_inputMediaDocument) {
                    TLRPC.TL_inputMediaDocument mediaDocument = (TLRPC.TL_inputMediaDocument) req.media;
                    mediaDocument.id.file_reference = file_reference;
                } else if (req.media instanceof TLRPC.TL_inputMediaPhoto) {
                    TLRPC.TL_inputMediaPhoto mediaPhoto = (TLRPC.TL_inputMediaPhoto) req.media;
                    mediaPhoto.id.file_reference = file_reference;
                }
                int index = multiMedia.multi_media.indexOf(req);
                if (index < 0) {
                    return;
                }
                ArrayList<Object> parentObjects = (ArrayList) objects[3];
                parentObjects.set(index, null);
                boolean done = true;
                for (int a2 = 0; a2 < parentObjects.size(); a2++) {
                    if (parentObjects.get(a2) != null) {
                        done = false;
                    }
                }
                if (done) {
                    this.multiMediaCache.remove(multiMedia);
                    AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda0
                        @Override // java.lang.Runnable
                        public final void run() {
                            FileRefController.this.m206x1dd47c01(multiMedia, objects);
                        }
                    });
                }
            }
        } else if (requester.args[0] instanceof TLRPC.TL_messages_sendMedia) {
            TLRPC.TL_messages_sendMedia req2 = (TLRPC.TL_messages_sendMedia) requester.args[0];
            if (req2.media instanceof TLRPC.TL_inputMediaDocument) {
                TLRPC.TL_inputMediaDocument mediaDocument2 = (TLRPC.TL_inputMediaDocument) req2.media;
                mediaDocument2.id.file_reference = file_reference;
            } else if (req2.media instanceof TLRPC.TL_inputMediaPhoto) {
                TLRPC.TL_inputMediaPhoto mediaPhoto2 = (TLRPC.TL_inputMediaPhoto) req2.media;
                mediaPhoto2.id.file_reference = file_reference;
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda11
                @Override // java.lang.Runnable
                public final void run() {
                    FileRefController.this.m207x315eac2(requester);
                }
            });
        } else if (requester.args[0] instanceof TLRPC.TL_messages_editMessage) {
            TLRPC.TL_messages_editMessage req3 = (TLRPC.TL_messages_editMessage) requester.args[0];
            if (req3.media instanceof TLRPC.TL_inputMediaDocument) {
                TLRPC.TL_inputMediaDocument mediaDocument3 = (TLRPC.TL_inputMediaDocument) req3.media;
                mediaDocument3.id.file_reference = file_reference;
            } else if (req3.media instanceof TLRPC.TL_inputMediaPhoto) {
                TLRPC.TL_inputMediaPhoto mediaPhoto3 = (TLRPC.TL_inputMediaPhoto) req3.media;
                mediaPhoto3.id.file_reference = file_reference;
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda22
                @Override // java.lang.Runnable
                public final void run() {
                    FileRefController.this.m208xe8575983(requester);
                }
            });
        } else if (requester.args[0] instanceof TLRPC.TL_messages_saveGif) {
            TLRPC.TL_messages_saveGif req4 = (TLRPC.TL_messages_saveGif) requester.args[0];
            req4.id.file_reference = file_reference;
            getConnectionsManager().sendRequest(req4, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda26
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.lambda$onUpdateObjectReference$23(tLObject, tL_error);
                }
            });
        } else if (requester.args[0] instanceof TLRPC.TL_messages_saveRecentSticker) {
            TLRPC.TL_messages_saveRecentSticker req5 = (TLRPC.TL_messages_saveRecentSticker) requester.args[0];
            req5.id.file_reference = file_reference;
            getConnectionsManager().sendRequest(req5, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda27
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.lambda$onUpdateObjectReference$24(tLObject, tL_error);
                }
            });
        } else if (requester.args[0] instanceof TLRPC.TL_messages_faveSticker) {
            TLRPC.TL_messages_faveSticker req6 = (TLRPC.TL_messages_faveSticker) requester.args[0];
            req6.id.file_reference = file_reference;
            getConnectionsManager().sendRequest(req6, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda28
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    FileRefController.lambda$onUpdateObjectReference$25(tLObject, tL_error);
                }
            });
        } else if (requester.args[0] instanceof TLRPC.TL_messages_getAttachedStickers) {
            TLRPC.TL_messages_getAttachedStickers req7 = (TLRPC.TL_messages_getAttachedStickers) requester.args[0];
            if (req7.media instanceof TLRPC.TL_inputStickeredMediaDocument) {
                TLRPC.TL_inputStickeredMediaDocument mediaDocument4 = (TLRPC.TL_inputStickeredMediaDocument) req7.media;
                mediaDocument4.id.file_reference = file_reference;
            } else if (req7.media instanceof TLRPC.TL_inputStickeredMediaPhoto) {
                TLRPC.TL_inputStickeredMediaPhoto mediaPhoto4 = (TLRPC.TL_inputStickeredMediaPhoto) req7.media;
                mediaPhoto4.id.file_reference = file_reference;
            }
            getConnectionsManager().sendRequest(req7, (RequestDelegate) requester.args[1]);
        } else if (requester.args[1] instanceof FileLoadOperation) {
            FileLoadOperation fileLoadOperation = (FileLoadOperation) requester.args[1];
            if (locationReplacement == null) {
                requester.location.file_reference = file_reference;
            } else {
                fileLoadOperation.location = locationReplacement;
            }
            fileLoadOperation.requestingReference = false;
            fileLoadOperation.startDownloadRequest();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onUpdateObjectReference$20$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m206x1dd47c01(TLRPC.TL_messages_sendMultiMedia multiMedia, Object[] objects) {
        getSendMessagesHelper().performSendMessageRequestMulti(multiMedia, (ArrayList) objects[1], (ArrayList) objects[2], null, (SendMessagesHelper.DelayedMessage) objects[4], ((Boolean) objects[5]).booleanValue());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onUpdateObjectReference$21$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m207x315eac2(Requester requester) {
        getSendMessagesHelper().performSendMessageRequest((TLObject) requester.args[0], (MessageObject) requester.args[1], (String) requester.args[2], (SendMessagesHelper.DelayedMessage) requester.args[3], ((Boolean) requester.args[4]).booleanValue(), (SendMessagesHelper.DelayedMessage) requester.args[5], null, ((Boolean) requester.args[6]).booleanValue());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onUpdateObjectReference$22$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m208xe8575983(Requester requester) {
        getSendMessagesHelper().performSendMessageRequest((TLObject) requester.args[0], (MessageObject) requester.args[1], (String) requester.args[2], (SendMessagesHelper.DelayedMessage) requester.args[3], ((Boolean) requester.args[4]).booleanValue(), (SendMessagesHelper.DelayedMessage) requester.args[5], null, ((Boolean) requester.args[6]).booleanValue());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$onUpdateObjectReference$23(TLObject response, TLRPC.TL_error error) {
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$onUpdateObjectReference$24(TLObject response, TLRPC.TL_error error) {
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$onUpdateObjectReference$25(TLObject response, TLRPC.TL_error error) {
    }

    private void sendErrorToObject(final Object[] args, int reason) {
        if (!(args[0] instanceof TLRPC.TL_inputSingleMedia)) {
            if ((args[0] instanceof TLRPC.TL_messages_sendMedia) || (args[0] instanceof TLRPC.TL_messages_editMessage)) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda25
                    @Override // java.lang.Runnable
                    public final void run() {
                        FileRefController.this.m230xd570e5d2(args);
                    }
                });
                return;
            } else if (!(args[0] instanceof TLRPC.TL_messages_saveGif)) {
                if (!(args[0] instanceof TLRPC.TL_messages_saveRecentSticker)) {
                    if (!(args[0] instanceof TLRPC.TL_messages_faveSticker)) {
                        if (args[0] instanceof TLRPC.TL_messages_getAttachedStickers) {
                            getConnectionsManager().sendRequest((TLRPC.TL_messages_getAttachedStickers) args[0], (RequestDelegate) args[1]);
                            return;
                        } else if (reason == 0) {
                            TLRPC.TL_error error = new TLRPC.TL_error();
                            error.text = "not found parent object to request reference";
                            error.code = 400;
                            if (args[1] instanceof FileLoadOperation) {
                                FileLoadOperation fileLoadOperation = (FileLoadOperation) args[1];
                                fileLoadOperation.requestingReference = false;
                                fileLoadOperation.processRequestResult((FileLoadOperation.RequestInfo) args[2], error);
                                return;
                            }
                            return;
                        } else if (reason == 1 && (args[1] instanceof FileLoadOperation)) {
                            FileLoadOperation fileLoadOperation2 = (FileLoadOperation) args[1];
                            fileLoadOperation2.requestingReference = false;
                            fileLoadOperation2.onFail(false, 0);
                            return;
                        } else {
                            return;
                        }
                    }
                    TLRPC.TL_messages_faveSticker tL_messages_faveSticker = (TLRPC.TL_messages_faveSticker) args[0];
                    return;
                }
                TLRPC.TL_messages_saveRecentSticker tL_messages_saveRecentSticker = (TLRPC.TL_messages_saveRecentSticker) args[0];
                return;
            } else {
                TLRPC.TL_messages_saveGif tL_messages_saveGif = (TLRPC.TL_messages_saveGif) args[0];
                return;
            }
        }
        final TLRPC.TL_messages_sendMultiMedia req = (TLRPC.TL_messages_sendMultiMedia) args[1];
        final Object[] objects = this.multiMediaCache.get(req);
        if (objects != null) {
            this.multiMediaCache.remove(req);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda24
                @Override // java.lang.Runnable
                public final void run() {
                    FileRefController.this.m229xf02f7711(req, objects);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendErrorToObject$26$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m229xf02f7711(TLRPC.TL_messages_sendMultiMedia req, Object[] objects) {
        getSendMessagesHelper().performSendMessageRequestMulti(req, (ArrayList) objects[1], (ArrayList) objects[2], null, (SendMessagesHelper.DelayedMessage) objects[4], ((Boolean) objects[5]).booleanValue());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendErrorToObject$27$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m230xd570e5d2(Object[] args) {
        getSendMessagesHelper().performSendMessageRequest((TLObject) args[0], (MessageObject) args[1], (String) args[2], (SendMessagesHelper.DelayedMessage) args[3], ((Boolean) args[4]).booleanValue(), (SendMessagesHelper.DelayedMessage) args[5], null, ((Boolean) args[6]).booleanValue());
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r7v0 */
    /* JADX WARN: Type inference failed for: r7v23 */
    /* JADX WARN: Type inference failed for: r7v24 */
    /* JADX WARN: Type inference failed for: r7v45 */
    /* JADX WARN: Type inference failed for: r7v46 */
    private boolean onRequestComplete(String locationKey, String parentKey, TLObject response, boolean cache) {
        ArrayList<Requester> arrayList;
        TLRPC.InputFileLocation inputFileLocation;
        TLRPC.Vector vector;
        int size10;
        byte[] result;
        byte[] result2;
        int size3;
        byte[] result3;
        ArrayList<Requester> arrayList2;
        boolean found = false;
        TLRPC.InputFileLocation inputFileLocation2 = null;
        int i = 1;
        if (parentKey != null && (arrayList2 = this.parentRequester.get(parentKey)) != null) {
            int N = arrayList2.size();
            for (int q = 0; q < N; q++) {
                Requester requester = arrayList2.get(q);
                if (!requester.completed) {
                    if (onRequestComplete(requester.locationKey, null, response, cache && !found)) {
                        found = true;
                    }
                }
            }
            if (found) {
                putReponseToCache(parentKey, response);
            }
            this.parentRequester.remove(parentKey);
        }
        byte[] result4 = null;
        TLRPC.InputFileLocation[] locationReplacement = null;
        boolean[] needReplacement = null;
        ArrayList<Requester> arrayList3 = this.locationRequester.get(locationKey);
        if (arrayList3 == null) {
            return found;
        }
        int q2 = 0;
        int N2 = arrayList3.size();
        while (q2 < N2) {
            Requester requester2 = arrayList3.get(q2);
            if (!requester2.completed) {
                if (requester2.location instanceof TLRPC.TL_inputFileLocation) {
                    locationReplacement = new TLRPC.InputFileLocation[i];
                    needReplacement = new boolean[i];
                }
                requester2.completed = i;
                if (response instanceof TLRPC.messages_Messages) {
                    TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
                    if (res.messages.isEmpty()) {
                        arrayList = arrayList3;
                    } else {
                        int i2 = 0;
                        int size32 = res.messages.size();
                        while (true) {
                            if (i2 >= size32) {
                                arrayList = arrayList3;
                                break;
                            }
                            TLRPC.Message message = res.messages.get(i2);
                            if (message.media != null) {
                                if (message.media.document != null) {
                                    size3 = size32;
                                    result4 = getFileReference(message.media.document, requester2.location, needReplacement, locationReplacement);
                                } else {
                                    size3 = size32;
                                    if (message.media.game != null) {
                                        byte[] result5 = getFileReference(message.media.game.document, requester2.location, needReplacement, locationReplacement);
                                        if (result5 == null) {
                                            byte[] result6 = getFileReference(message.media.game.photo, requester2.location, needReplacement, locationReplacement);
                                            result4 = result6;
                                        } else {
                                            result4 = result5;
                                        }
                                    } else if (message.media.photo != null) {
                                        result4 = getFileReference(message.media.photo, requester2.location, needReplacement, locationReplacement);
                                    } else if (message.media.webpage != null) {
                                        result4 = getFileReference(message.media.webpage, requester2.location, needReplacement, locationReplacement);
                                    }
                                }
                            } else {
                                size3 = size32;
                                if (message.action instanceof TLRPC.TL_messageActionChatEditPhoto) {
                                    result4 = getFileReference(message.action.photo, requester2.location, needReplacement, locationReplacement);
                                }
                            }
                            if (result4 == null) {
                                i2++;
                                size32 = size3;
                            } else {
                                if (!cache) {
                                    result3 = result4;
                                    arrayList = arrayList3;
                                } else {
                                    if (message.to_id == null || message.to_id.channel_id == 0) {
                                        result3 = result4;
                                        arrayList = arrayList3;
                                    } else {
                                        int a2 = 0;
                                        int N22 = res.chats.size();
                                        while (true) {
                                            if (a2 >= N22) {
                                                result3 = result4;
                                                arrayList = arrayList3;
                                                break;
                                            }
                                            int N23 = N22;
                                            TLRPC.Chat chat = res.chats.get(a2);
                                            result3 = result4;
                                            arrayList = arrayList3;
                                            if (chat.id != message.to_id.channel_id) {
                                                a2++;
                                                N22 = N23;
                                                result4 = result3;
                                                arrayList3 = arrayList;
                                            } else if (chat.megagroup) {
                                                message.flags |= Integer.MIN_VALUE;
                                            }
                                        }
                                    }
                                    getMessagesStorage().replaceMessageIfExists(message, this.currentAccount, res.users, res.chats, false);
                                }
                                result4 = result3;
                            }
                        }
                        if (result4 == null) {
                            getMessagesStorage().replaceMessageIfExists(res.messages.get(0), this.currentAccount, res.users, res.chats, true);
                            if (BuildVars.DEBUG_VERSION) {
                                FileLog.d("file ref not found in messages, replacing message");
                            }
                        }
                    }
                    inputFileLocation = null;
                } else {
                    arrayList = arrayList3;
                    if (response instanceof TLRPC.WebPage) {
                        result4 = getFileReference((TLRPC.WebPage) response, requester2.location, needReplacement, locationReplacement);
                        inputFileLocation = null;
                    } else if (response instanceof TLRPC.TL_account_wallPapers) {
                        TLRPC.TL_account_wallPapers accountWallPapers = (TLRPC.TL_account_wallPapers) response;
                        int size102 = accountWallPapers.wallpapers.size();
                        for (int i3 = 0; i3 < size102; i3++) {
                            result4 = getFileReference(((TLRPC.TL_wallPaper) accountWallPapers.wallpapers.get(i3)).document, requester2.location, needReplacement, locationReplacement);
                            if (result4 != null) {
                                break;
                            }
                        }
                        if (result4 != null && cache) {
                            getMessagesStorage().putWallpapers(accountWallPapers.wallpapers, 1);
                        }
                        inputFileLocation = null;
                    } else if (response instanceof TLRPC.TL_wallPaper) {
                        TLRPC.TL_wallPaper wallPaper = (TLRPC.TL_wallPaper) response;
                        result4 = getFileReference(wallPaper.document, requester2.location, needReplacement, locationReplacement);
                        if (result4 != null && cache) {
                            ArrayList<TLRPC.WallPaper> wallpapers = new ArrayList<>();
                            wallpapers.add(wallPaper);
                            getMessagesStorage().putWallpapers(wallpapers, 0);
                        }
                        inputFileLocation = null;
                    } else if (response instanceof TLRPC.TL_theme) {
                        final TLRPC.TL_theme theme = (TLRPC.TL_theme) response;
                        result4 = getFileReference(theme.document, requester2.location, needReplacement, locationReplacement);
                        if (result4 != null && cache) {
                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda18
                                @Override // java.lang.Runnable
                                public final void run() {
                                    Theme.setThemeFileReference(TLRPC.TL_theme.this);
                                }
                            });
                        }
                        inputFileLocation = null;
                    } else if (response instanceof TLRPC.Vector) {
                        TLRPC.Vector vector2 = (TLRPC.Vector) response;
                        if (!vector2.objects.isEmpty()) {
                            int i4 = 0;
                            int size103 = vector2.objects.size();
                            while (i4 < size103) {
                                Object object = vector2.objects.get(i4);
                                if (object instanceof TLRPC.User) {
                                    final TLRPC.User user = (TLRPC.User) object;
                                    byte[] result7 = getFileReference(user, requester2.location, needReplacement, locationReplacement);
                                    if (!cache || result7 == null) {
                                        vector = vector2;
                                        size10 = size103;
                                        result2 = result7;
                                    } else {
                                        ArrayList<TLRPC.User> arrayList1 = new ArrayList<>();
                                        arrayList1.add(user);
                                        vector = vector2;
                                        size10 = size103;
                                        result2 = result7;
                                        getMessagesStorage().putUsersAndChats(arrayList1, null, true, true);
                                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda19
                                            @Override // java.lang.Runnable
                                            public final void run() {
                                                FileRefController.this.m202x6787a3c5(user);
                                            }
                                        });
                                    }
                                    result4 = result2;
                                } else {
                                    vector = vector2;
                                    size10 = size103;
                                    if (object instanceof TLRPC.Chat) {
                                        final TLRPC.Chat chat2 = (TLRPC.Chat) object;
                                        byte[] result8 = getFileReference(chat2, requester2.location, needReplacement, locationReplacement);
                                        if (!cache || result8 == null) {
                                            result = result8;
                                        } else {
                                            ArrayList<TLRPC.Chat> arrayList12 = new ArrayList<>();
                                            arrayList12.add(chat2);
                                            result = result8;
                                            getMessagesStorage().putUsersAndChats(null, arrayList12, true, true);
                                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda20
                                                @Override // java.lang.Runnable
                                                public final void run() {
                                                    FileRefController.this.m203x1b27285b(chat2);
                                                }
                                            });
                                        }
                                        result4 = result;
                                    }
                                }
                                if (result4 != null) {
                                    break;
                                }
                                i4++;
                                vector2 = vector;
                                size103 = size10;
                            }
                        }
                        inputFileLocation = null;
                    } else if (response instanceof TLRPC.TL_messages_chats) {
                        TLRPC.TL_messages_chats res2 = (TLRPC.TL_messages_chats) response;
                        if (res2.chats.isEmpty()) {
                            inputFileLocation = null;
                        } else {
                            int i5 = 0;
                            int size104 = res2.chats.size();
                            while (true) {
                                if (i5 >= size104) {
                                    inputFileLocation = null;
                                    break;
                                }
                                final TLRPC.Chat chat3 = res2.chats.get(i5);
                                result4 = getFileReference(chat3, requester2.location, needReplacement, locationReplacement);
                                if (result4 == null) {
                                    i5++;
                                    res2 = res2;
                                } else if (cache) {
                                    ArrayList<TLRPC.Chat> arrayList13 = new ArrayList<>();
                                    arrayList13.add(chat3);
                                    inputFileLocation = null;
                                    getMessagesStorage().putUsersAndChats(null, arrayList13, true, true);
                                    AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda21
                                        @Override // java.lang.Runnable
                                        public final void run() {
                                            FileRefController.this.m204x68971c(chat3);
                                        }
                                    });
                                } else {
                                    inputFileLocation = null;
                                }
                            }
                        }
                    } else {
                        inputFileLocation = null;
                        if (response instanceof TLRPC.TL_messages_savedGifs) {
                            TLRPC.TL_messages_savedGifs savedGifs = (TLRPC.TL_messages_savedGifs) response;
                            int size2 = savedGifs.gifs.size();
                            for (int b = 0; b < size2; b++) {
                                result4 = getFileReference(savedGifs.gifs.get(b), requester2.location, needReplacement, locationReplacement);
                                if (result4 != null) {
                                    break;
                                }
                            }
                            if (cache) {
                                getMediaDataController().processLoadedRecentDocuments(0, savedGifs.gifs, true, 0, true);
                            }
                        } else if (response instanceof TLRPC.TL_messages_stickerSet) {
                            final TLRPC.TL_messages_stickerSet stickerSet = (TLRPC.TL_messages_stickerSet) response;
                            if (result4 == null) {
                                int size22 = stickerSet.documents.size();
                                for (int b2 = 0; b2 < size22; b2++) {
                                    result4 = getFileReference(stickerSet.documents.get(b2), requester2.location, needReplacement, locationReplacement);
                                    if (result4 != null) {
                                        break;
                                    }
                                }
                            }
                            if (cache) {
                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.FileRefController$$ExternalSyntheticLambda23
                                    @Override // java.lang.Runnable
                                    public final void run() {
                                        FileRefController.this.m205xe5aa05dd(stickerSet);
                                    }
                                });
                            }
                        } else if (response instanceof TLRPC.TL_messages_recentStickers) {
                            TLRPC.TL_messages_recentStickers recentStickers = (TLRPC.TL_messages_recentStickers) response;
                            int size23 = recentStickers.stickers.size();
                            for (int b3 = 0; b3 < size23; b3++) {
                                result4 = getFileReference(recentStickers.stickers.get(b3), requester2.location, needReplacement, locationReplacement);
                                if (result4 != null) {
                                    break;
                                }
                            }
                            if (cache) {
                                getMediaDataController().processLoadedRecentDocuments(0, recentStickers.stickers, false, 0, true);
                            }
                        } else if (response instanceof TLRPC.TL_messages_favedStickers) {
                            TLRPC.TL_messages_favedStickers favedStickers = (TLRPC.TL_messages_favedStickers) response;
                            int size24 = favedStickers.stickers.size();
                            for (int b4 = 0; b4 < size24; b4++) {
                                result4 = getFileReference(favedStickers.stickers.get(b4), requester2.location, needReplacement, locationReplacement);
                                if (result4 != null) {
                                    break;
                                }
                            }
                            if (cache) {
                                getMediaDataController().processLoadedRecentDocuments(2, favedStickers.stickers, false, 0, true);
                            }
                        } else if (response instanceof TLRPC.photos_Photos) {
                            TLRPC.photos_Photos res3 = (TLRPC.photos_Photos) response;
                            int size = res3.photos.size();
                            for (int b5 = 0; b5 < size; b5++) {
                                result4 = getFileReference(res3.photos.get(b5), requester2.location, needReplacement, locationReplacement);
                                if (result4 != null) {
                                    break;
                                }
                            }
                        }
                    }
                }
                if (result4 != null) {
                    onUpdateObjectReference(requester2, result4, locationReplacement != null ? locationReplacement[0] : inputFileLocation);
                    found = true;
                    i = 1;
                } else {
                    i = 1;
                    sendErrorToObject(requester2.args, 1);
                }
            } else {
                inputFileLocation = inputFileLocation2;
                arrayList = arrayList3;
            }
            q2++;
            inputFileLocation2 = inputFileLocation;
            arrayList3 = arrayList;
            i = i;
        }
        this.locationRequester.remove(locationKey);
        if (found) {
            putReponseToCache(locationKey, response);
        }
        return found;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onRequestComplete$29$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m202x6787a3c5(TLRPC.User user) {
        getMessagesController().putUser(user, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onRequestComplete$30$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m203x1b27285b(TLRPC.Chat chat) {
        getMessagesController().putChat(chat, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onRequestComplete$31$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m204x68971c(TLRPC.Chat chat) {
        getMessagesController().putChat(chat, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onRequestComplete$32$im-uecahxyvjd-messenger-FileRefController  reason: not valid java name */
    public /* synthetic */ void m205xe5aa05dd(TLRPC.TL_messages_stickerSet stickerSet) {
        getMediaDataController().replaceStickerSet(stickerSet);
    }

    private void cleanupCache() {
        if (Math.abs(SystemClock.uptimeMillis() - this.lastCleanupTime) < 600000) {
            return;
        }
        this.lastCleanupTime = SystemClock.uptimeMillis();
        ArrayList<String> keysToDelete = null;
        for (Map.Entry<String, CachedResult> entry : this.responseCache.entrySet()) {
            CachedResult cachedResult = entry.getValue();
            if (Math.abs(SystemClock.uptimeMillis() - cachedResult.firstQueryTime) >= 600000) {
                if (keysToDelete == null) {
                    keysToDelete = new ArrayList<>();
                }
                keysToDelete.add(entry.getKey());
            }
        }
        if (keysToDelete != null) {
            int size = keysToDelete.size();
            for (int a2 = 0; a2 < size; a2++) {
                this.responseCache.remove(keysToDelete.get(a2));
            }
        }
    }

    private CachedResult getCachedResponse(String key) {
        CachedResult cachedResult = this.responseCache.get(key);
        if (cachedResult != null && Math.abs(SystemClock.uptimeMillis() - cachedResult.firstQueryTime) >= 600000) {
            this.responseCache.remove(key);
            return null;
        }
        return cachedResult;
    }

    private void putReponseToCache(String key, TLObject response) {
        CachedResult cachedResult = this.responseCache.get(key);
        if (cachedResult == null) {
            cachedResult = new CachedResult();
            cachedResult.response = response;
            cachedResult.firstQueryTime = SystemClock.uptimeMillis();
            this.responseCache.put(key, cachedResult);
        }
        cachedResult.lastQueryTime = SystemClock.uptimeMillis();
    }

    private byte[] getFileReference(TLRPC.Document document, TLRPC.InputFileLocation location, boolean[] needReplacement, TLRPC.InputFileLocation[] replacement) {
        if (document == null || location == null) {
            return null;
        }
        if (location instanceof TLRPC.TL_inputDocumentFileLocation) {
            if (document.id == location.id) {
                return document.file_reference;
            }
        } else {
            int size = document.thumbs.size();
            for (int a2 = 0; a2 < size; a2++) {
                TLRPC.PhotoSize photoSize = document.thumbs.get(a2);
                byte[] result = getFileReference(photoSize, location, needReplacement);
                if (needReplacement != null && needReplacement[0]) {
                    replacement[0] = new TLRPC.TL_inputDocumentFileLocation();
                    replacement[0].id = document.id;
                    replacement[0].volume_id = location.volume_id;
                    replacement[0].local_id = location.local_id;
                    replacement[0].access_hash = document.access_hash;
                    replacement[0].file_reference = document.file_reference;
                    replacement[0].thumb_size = photoSize.type;
                    return document.file_reference;
                } else if (result != null) {
                    return result;
                }
            }
        }
        return null;
    }

    /* JADX WARN: Multi-variable type inference failed */
    private boolean getPeerReferenceReplacement(TLRPC.User user, TLRPC.Chat chat, boolean big, TLRPC.InputFileLocation location, TLRPC.InputFileLocation[] replacement, boolean[] needReplacement) {
        TLRPC.InputPeer peer;
        if (needReplacement == null || !needReplacement[0]) {
            return false;
        }
        replacement[0] = new TLRPC.TL_inputPeerPhotoFileLocation();
        replacement[0].id = location.volume_id;
        replacement[0].volume_id = location.volume_id;
        replacement[0].local_id = location.local_id;
        replacement[0].big = big;
        if (user != null) {
            TLRPC.TL_inputPeerUser inputPeerUser = new TLRPC.TL_inputPeerUser();
            inputPeerUser.user_id = user.id;
            inputPeerUser.access_hash = user.access_hash;
            peer = inputPeerUser;
        } else if (ChatObject.isChannel(chat)) {
            TLRPC.TL_inputPeerChat inputPeerChat = new TLRPC.TL_inputPeerChat();
            inputPeerChat.chat_id = chat.id;
            peer = inputPeerChat;
        } else {
            TLRPC.TL_inputPeerChannel inputPeerChannel = new TLRPC.TL_inputPeerChannel();
            inputPeerChannel.channel_id = chat.id;
            inputPeerChannel.access_hash = chat.access_hash;
            peer = inputPeerChannel;
        }
        replacement[0].peer = peer;
        return true;
    }

    private byte[] getFileReference(TLRPC.User user, TLRPC.InputFileLocation location, boolean[] needReplacement, TLRPC.InputFileLocation[] replacement) {
        if (user == null || user.photo == null || !(location instanceof TLRPC.TL_inputFileLocation)) {
            return null;
        }
        byte[] result = getFileReference(user.photo.photo_small, location, needReplacement);
        if (getPeerReferenceReplacement(user, null, false, location, replacement, needReplacement)) {
            return new byte[0];
        }
        if (result == null) {
            result = getFileReference(user.photo.photo_big, location, needReplacement);
            if (getPeerReferenceReplacement(user, null, true, location, replacement, needReplacement)) {
                return new byte[0];
            }
        }
        return result;
    }

    private byte[] getFileReference(TLRPC.Chat chat, TLRPC.InputFileLocation location, boolean[] needReplacement, TLRPC.InputFileLocation[] replacement) {
        if (chat == null || chat.photo == null || !(location instanceof TLRPC.TL_inputFileLocation)) {
            return null;
        }
        byte[] result = getFileReference(chat.photo.photo_small, location, needReplacement);
        if (getPeerReferenceReplacement(null, chat, false, location, replacement, needReplacement)) {
            return new byte[0];
        }
        if (result == null) {
            result = getFileReference(chat.photo.photo_big, location, needReplacement);
            if (getPeerReferenceReplacement(null, chat, true, location, replacement, needReplacement)) {
                return new byte[0];
            }
        }
        return result;
    }

    private byte[] getFileReference(TLRPC.Photo photo, TLRPC.InputFileLocation location, boolean[] needReplacement, TLRPC.InputFileLocation[] replacement) {
        if (photo == null) {
            return null;
        }
        if (location instanceof TLRPC.TL_inputPhotoFileLocation) {
            if (photo.id == location.id) {
                return photo.file_reference;
            }
            return null;
        }
        if (location instanceof TLRPC.TL_inputFileLocation) {
            int size = photo.sizes.size();
            for (int a2 = 0; a2 < size; a2++) {
                TLRPC.PhotoSize photoSize = photo.sizes.get(a2);
                byte[] result = getFileReference(photoSize, location, needReplacement);
                if (needReplacement != null && needReplacement[0]) {
                    replacement[0] = new TLRPC.TL_inputPhotoFileLocation();
                    replacement[0].id = photo.id;
                    replacement[0].volume_id = location.volume_id;
                    replacement[0].local_id = location.local_id;
                    replacement[0].access_hash = photo.access_hash;
                    replacement[0].file_reference = photo.file_reference;
                    replacement[0].thumb_size = photoSize.type;
                    return photo.file_reference;
                } else if (result != null) {
                    return result;
                }
            }
        }
        return null;
    }

    private byte[] getFileReference(TLRPC.PhotoSize photoSize, TLRPC.InputFileLocation location, boolean[] needReplacement) {
        if (photoSize == null || !(location instanceof TLRPC.TL_inputFileLocation)) {
            return null;
        }
        return getFileReference(photoSize.location, location, needReplacement);
    }

    private byte[] getFileReference(TLRPC.FileLocation fileLocation, TLRPC.InputFileLocation location, boolean[] needReplacement) {
        if (fileLocation == null || !(location instanceof TLRPC.TL_inputFileLocation) || fileLocation.local_id != location.local_id || fileLocation.volume_id != location.volume_id) {
            return null;
        }
        if (fileLocation.file_reference == null && needReplacement != null) {
            needReplacement[0] = true;
        }
        return fileLocation.file_reference;
    }

    private byte[] getFileReference(TLRPC.WebPage webpage, TLRPC.InputFileLocation location, boolean[] needReplacement, TLRPC.InputFileLocation[] replacement) {
        byte[] result = getFileReference(webpage.document, location, needReplacement, replacement);
        if (result != null) {
            return result;
        }
        byte[] result2 = getFileReference(webpage.photo, location, needReplacement, replacement);
        if (result2 != null) {
            return result2;
        }
        if (result2 == null && webpage.cached_page != null) {
            int size2 = webpage.cached_page.documents.size();
            for (int b = 0; b < size2; b++) {
                byte[] result3 = getFileReference(webpage.cached_page.documents.get(b), location, needReplacement, replacement);
                if (result3 != null) {
                    return result3;
                }
            }
            int size22 = webpage.cached_page.photos.size();
            for (int b2 = 0; b2 < size22; b2++) {
                byte[] result4 = getFileReference(webpage.cached_page.photos.get(b2), location, needReplacement, replacement);
                if (result4 != null) {
                    return result4;
                }
            }
            return null;
        }
        return null;
    }

    public static boolean isFileRefError(String error) {
        return "FILEREF_EXPIRED".equals(error) || "FILE_REFERENCE_EXPIRED".equals(error) || "FILE_REFERENCE_EMPTY".equals(error) || (error != null && error.startsWith("FILE_REFERENCE_"));
    }
}
