package im.uecahxyvjd.messenger;

import android.content.Intent;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.LongSparseArray;
import android.util.SparseIntArray;
import com.baidu.location.BDAbstractLocationListener;
import com.baidu.location.BDLocation;
import com.baidu.location.LocationClient;
import com.baidu.location.LocationClientOption;
import com.baidu.mapapi.model.LatLng;
import com.baidu.mapapi.utils.DistanceUtil;
import com.baidu.mapsdkplatform.comapi.location.CoordinateType;
import com.google.android.exoplayer2.DefaultRenderersFactory;
import com.google.android.gms.location.LocationRequest;
import com.zhy.http.okhttp.OkHttpUtils;
import im.uecahxyvjd.messenger.LocationController;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.sqlite.SQLiteCursor;
import im.uecahxyvjd.sqlite.SQLitePreparedStatement;
import im.uecahxyvjd.tgnet.NativeByteBuffer;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
/* loaded from: classes16.dex */
public class LocationController extends BaseController implements NotificationCenter.NotificationCenterDelegate {
    private static final int BACKGROUD_UPDATE_TIME = 30000;
    private static final long FASTEST_INTERVAL = 1000;
    private static final int FOREGROUND_UPDATE_TIME = 20000;
    private static final int LOCATION_ACQUIRE_TIME = 10000;
    private static final int PLAY_SERVICES_RESOLUTION_REQUEST = 9000;
    private static final long UPDATE_INTERVAL = 1000;
    private BDLocationListener bdLocationListener;
    private LongSparseArray<Boolean> cacheRequests;
    private ArrayList<TLRPC.TL_peerLocated> cachedNearbyChats;
    private ArrayList<TLRPC.TL_peerLocated> cachedNearbyUsers;
    private BDLocation lastKnownLocation;
    private boolean lastLocationByBaiduMaps;
    private long lastLocationSendTime;
    private long lastLocationStartTime;
    private LocationRequest locationRequest;
    private boolean locationSentSinceLastBaiduMapUpdate;
    public LongSparseArray<ArrayList<TLRPC.Message>> locationsCache;
    private boolean lookingForPeopleNearby;
    private LocationClient mLocClient;
    private SparseIntArray requests;
    private ArrayList<SharingLocationInfo> sharingLocations;
    private LongSparseArray<SharingLocationInfo> sharingLocationsMap;
    private LongSparseArray<SharingLocationInfo> sharingLocationsMapUI;
    public ArrayList<SharingLocationInfo> sharingLocationsUI;
    private boolean started;
    private static volatile LocationController[] Instance = new LocationController[3];
    private static HashMap<LocationFetchCallback, Runnable> callbacks = new HashMap<>();

    /* loaded from: classes16.dex */
    public interface LocationFetchCallback {
        void onLocationAddressAvailable(String str, String str2, BDLocation bDLocation);
    }

    /* loaded from: classes16.dex */
    public static class SharingLocationInfo {
        public long did;
        public MessageObject messageObject;
        public int mid;
        public int period;
        public int stopTime;
    }

    public static LocationController getInstance(int num) {
        LocationController localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (LocationController.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    LocationController[] locationControllerArr = Instance;
                    LocationController locationController = new LocationController(num);
                    localInstance = locationController;
                    locationControllerArr[num] = locationController;
                }
            }
        }
        return localInstance;
    }

    /* loaded from: classes16.dex */
    public class BDLocationListener extends BDAbstractLocationListener {
        public BDLocationListener() {
        }

        @Override // com.baidu.location.BDAbstractLocationListener
        public void onReceiveLocation(BDLocation bdLocation) {
            if (bdLocation != null) {
                if (LocationController.this.lastKnownLocation == null) {
                    LocationController.this.setLastKnownLocation(bdLocation);
                    return;
                }
                LatLng newLatLng = new LatLng(bdLocation.getLatitude(), bdLocation.getLongitude());
                LatLng lastLatLng = new LatLng(LocationController.this.lastKnownLocation.getLatitude(), LocationController.this.lastKnownLocation.getLongitude());
                if (!LocationController.this.started && DistanceUtil.getDistance(newLatLng, lastLatLng) > 20.0d) {
                    LocationController.this.setLastKnownLocation(bdLocation);
                    LocationController.this.lastLocationSendTime = (SystemClock.uptimeMillis() - 30000) + DefaultRenderersFactory.DEFAULT_ALLOWED_VIDEO_JOINING_TIME_MS;
                }
            }
        }
    }

    public LocationController(int instance) {
        super(instance);
        this.sharingLocationsMap = new LongSparseArray<>();
        this.sharingLocations = new ArrayList<>();
        this.locationsCache = new LongSparseArray<>();
        this.locationSentSinceLastBaiduMapUpdate = true;
        this.requests = new SparseIntArray();
        this.cacheRequests = new LongSparseArray<>();
        this.sharingLocationsUI = new ArrayList<>();
        this.sharingLocationsMapUI = new LongSparseArray<>();
        this.cachedNearbyUsers = new ArrayList<>();
        this.cachedNearbyChats = new ArrayList<>();
        initLocationClient();
        LocationRequest locationRequest = new LocationRequest();
        this.locationRequest = locationRequest;
        locationRequest.setPriority(100);
        this.locationRequest.setInterval(1000L);
        this.locationRequest.setFastestInterval(1000L);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda13
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m298lambda$new$0$imuecahxyvjdmessengerLocationController();
            }
        });
        loadSharingLocations();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$new$0$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m298lambda$new$0$imuecahxyvjdmessengerLocationController() {
        LocationController locationController = getAccountInstance().getLocationController();
        getNotificationCenter().addObserver(locationController, NotificationCenter.didReceiveNewMessages);
        getNotificationCenter().addObserver(locationController, NotificationCenter.messagesDeleted);
        getNotificationCenter().addObserver(locationController, NotificationCenter.replaceMessagesObjects);
    }

    private void initLocationClient() {
        LocationClient locationClient = new LocationClient(ApplicationLoader.applicationContext);
        this.mLocClient = locationClient;
        BDLocationListener bDLocationListener = new BDLocationListener();
        this.bdLocationListener = bDLocationListener;
        locationClient.registerLocationListener(bDLocationListener);
        LocationClientOption option = new LocationClientOption();
        option.setOpenGps(true);
        option.setCoorType(CoordinateType.GCJ02);
        option.setScanSpan(1000);
        option.setIgnoreKillProcess(true);
        option.setIsNeedAddress(true);
        this.mLocClient.setLocOption(option);
    }

    @Override // im.uecahxyvjd.messenger.NotificationCenter.NotificationCenterDelegate
    public void didReceivedNotification(int id, int account, Object... args) {
        ArrayList<TLRPC.Message> messages;
        ArrayList<TLRPC.Message> messages2;
        if (id == NotificationCenter.didReceiveNewMessages) {
            boolean scheduled = ((Boolean) args[2]).booleanValue();
            if (scheduled) {
                return;
            }
            long did = ((Long) args[0]).longValue();
            if (!isSharingLocation(did) || (messages2 = this.locationsCache.get(did)) == null) {
                return;
            }
            ArrayList<MessageObject> arr = (ArrayList) args[1];
            boolean added = false;
            for (int a2 = 0; a2 < arr.size(); a2++) {
                MessageObject messageObject = arr.get(a2);
                if (messageObject.isLiveLocation()) {
                    added = true;
                    boolean replaced = false;
                    int b = 0;
                    while (true) {
                        if (b < messages2.size()) {
                            if (messages2.get(b).from_id != messageObject.messageOwner.from_id) {
                                b++;
                            } else {
                                replaced = true;
                                messages2.set(b, messageObject.messageOwner);
                                break;
                            }
                        } else {
                            break;
                        }
                    }
                    if (!replaced) {
                        messages2.add(messageObject.messageOwner);
                    }
                }
            }
            if (added) {
                NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsCacheChanged, Long.valueOf(did), Integer.valueOf(this.currentAccount));
            }
        } else if (id == NotificationCenter.messagesDeleted) {
            boolean scheduled2 = ((Boolean) args[2]).booleanValue();
            if (!scheduled2 && !this.sharingLocationsUI.isEmpty()) {
                ArrayList<Integer> markAsDeletedMessages = (ArrayList) args[0];
                int channelId = ((Integer) args[1]).intValue();
                ArrayList<Long> toRemove = null;
                for (int a3 = 0; a3 < this.sharingLocationsUI.size(); a3++) {
                    SharingLocationInfo info = this.sharingLocationsUI.get(a3);
                    int messageChannelId = info.messageObject != null ? info.messageObject.getChannelId() : 0;
                    if (channelId == messageChannelId && markAsDeletedMessages.contains(Integer.valueOf(info.mid))) {
                        if (toRemove == null) {
                            toRemove = new ArrayList<>();
                        }
                        toRemove.add(Long.valueOf(info.did));
                    }
                }
                if (toRemove != null) {
                    for (int a4 = 0; a4 < toRemove.size(); a4++) {
                        removeSharingLocation(toRemove.get(a4).longValue());
                    }
                }
            }
        } else if (id == NotificationCenter.replaceMessagesObjects) {
            long did2 = ((Long) args[0]).longValue();
            if (!isSharingLocation(did2) || (messages = this.locationsCache.get(did2)) == null) {
                return;
            }
            boolean updated = false;
            ArrayList<MessageObject> messageObjects = (ArrayList) args[1];
            for (int a5 = 0; a5 < messageObjects.size(); a5++) {
                MessageObject messageObject2 = messageObjects.get(a5);
                int b2 = 0;
                while (true) {
                    if (b2 >= messages.size()) {
                        break;
                    } else if (messages.get(b2).from_id != messageObject2.messageOwner.from_id) {
                        b2++;
                    } else {
                        if (!messageObject2.isLiveLocation()) {
                            messages.remove(b2);
                        } else {
                            messages.set(b2, messageObject2.messageOwner);
                        }
                        updated = true;
                    }
                }
            }
            if (updated) {
                NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsCacheChanged, Long.valueOf(did2), Integer.valueOf(this.currentAccount));
            }
        }
    }

    public void startFusedLocationRequest(final boolean permissionsGranted) {
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda5
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m306x38977675(permissionsGranted);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startFusedLocationRequest$1$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m306x38977675(boolean permissionsGranted) {
        if (this.lookingForPeopleNearby || !this.sharingLocations.isEmpty()) {
            if (permissionsGranted) {
                setLastKnownLocation(this.mLocClient.getLastKnownLocation());
            } else {
                start();
            }
        }
    }

    private void broadcastLastKnownLocation() {
        int a2;
        if (this.lastKnownLocation == null) {
            return;
        }
        if (this.requests.size() != 0) {
            for (int a3 = 0; a3 < this.requests.size(); a3++) {
                getConnectionsManager().cancelRequest(this.requests.keyAt(a3), false);
            }
            this.requests.clear();
        }
        int date = getConnectionsManager().getCurrentTime();
        float[] result = new float[1];
        for (a2 = 0; a2 < this.sharingLocations.size(); a2 = a2 + 1) {
            final SharingLocationInfo info = this.sharingLocations.get(a2);
            if (info.messageObject.messageOwner.media != null && info.messageObject.messageOwner.media.geo != null) {
                int messageDate = info.messageObject.messageOwner.edit_date != 0 ? info.messageObject.messageOwner.edit_date : info.messageObject.messageOwner.date;
                TLRPC.GeoPoint point = info.messageObject.messageOwner.media.geo;
                if (Math.abs(date - messageDate) < 10) {
                    Location.distanceBetween(point.lat, point._long, this.lastKnownLocation.getLatitude(), this.lastKnownLocation.getLongitude(), result);
                    a2 = result[0] < 1.0f ? a2 + 1 : 0;
                }
            }
            TLRPC.TL_messages_editMessage req = new TLRPC.TL_messages_editMessage();
            req.peer = getMessagesController().getInputPeer((int) info.did);
            req.id = info.mid;
            req.flags |= 16384;
            req.media = new TLRPC.TL_inputMediaGeoLive();
            req.media.stopped = false;
            req.media.geo_point = new TLRPC.TL_inputGeoPoint();
            req.media.geo_point.lat = AndroidUtilities.fixLocationCoord(this.lastKnownLocation.getLatitude());
            req.media.geo_point._long = AndroidUtilities.fixLocationCoord(this.lastKnownLocation.getLongitude());
            final int[] reqId = {getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda0
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    LocationController.this.m290x1465a171(info, reqId, tLObject, tL_error);
                }
            })};
            this.requests.put(reqId[0], 0);
        }
        getConnectionsManager().resumeNetworkMaybe();
        stop();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$broadcastLastKnownLocation$3$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m290x1465a171(final SharingLocationInfo info, int[] reqId, TLObject response, TLRPC.TL_error error) {
        if (error != null) {
            if (error.text.equals("MESSAGE_ID_INVALID")) {
                this.sharingLocations.remove(info);
                this.sharingLocationsMap.remove(info.did);
                saveSharingLocation(info, 1);
                this.requests.delete(reqId[0]);
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda10
                    @Override // java.lang.Runnable
                    public final void run() {
                        LocationController.this.m289x51793812(info);
                    }
                });
                return;
            }
            return;
        }
        TLRPC.Updates updates = (TLRPC.Updates) response;
        boolean updated = false;
        for (int a1 = 0; a1 < updates.updates.size(); a1++) {
            TLRPC.Update update = updates.updates.get(a1);
            if (update instanceof TLRPC.TL_updateEditMessage) {
                updated = true;
                info.messageObject.messageOwner = ((TLRPC.TL_updateEditMessage) update).message;
            } else if (update instanceof TLRPC.TL_updateEditChannelMessage) {
                updated = true;
                info.messageObject.messageOwner = ((TLRPC.TL_updateEditChannelMessage) update).message;
            }
        }
        if (updated) {
            saveSharingLocation(info, 0);
        }
        getMessagesController().processUpdates(updates, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$broadcastLastKnownLocation$2$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m289x51793812(SharingLocationInfo info) {
        this.sharingLocationsUI.remove(info);
        this.sharingLocationsMapUI.remove(info.did);
        if (this.sharingLocationsUI.isEmpty()) {
            stopService();
        }
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsChanged, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void update() {
        if (this.sharingLocations.isEmpty()) {
            return;
        }
        int a2 = 0;
        while (a2 < this.sharingLocations.size()) {
            final SharingLocationInfo info = this.sharingLocations.get(a2);
            int currentTime = getConnectionsManager().getCurrentTime();
            if (info.stopTime <= currentTime) {
                this.sharingLocations.remove(a2);
                this.sharingLocationsMap.remove(info.did);
                saveSharingLocation(info, 1);
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda23
                    @Override // java.lang.Runnable
                    public final void run() {
                        LocationController.this.m308lambda$update$4$imuecahxyvjdmessengerLocationController(info);
                    }
                });
                a2--;
            }
            a2++;
        }
        if (!this.started) {
            if (Math.abs(this.lastLocationSendTime - SystemClock.uptimeMillis()) > 30000) {
                this.lastLocationStartTime = SystemClock.uptimeMillis();
                start();
            }
        } else if (Math.abs(this.lastLocationStartTime - SystemClock.uptimeMillis()) > OkHttpUtils.DEFAULT_MILLISECONDS) {
            this.lastLocationSendTime = SystemClock.uptimeMillis();
            this.locationSentSinceLastBaiduMapUpdate = true;
            broadcastLastKnownLocation();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$update$4$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m308lambda$update$4$imuecahxyvjdmessengerLocationController(SharingLocationInfo info) {
        this.sharingLocationsUI.remove(info);
        this.sharingLocationsMapUI.remove(info.did);
        if (this.sharingLocationsUI.isEmpty()) {
            stopService();
        }
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsChanged, new Object[0]);
    }

    public void cleanup() {
        this.sharingLocationsUI.clear();
        this.sharingLocationsMapUI.clear();
        this.locationsCache.clear();
        this.cacheRequests.clear();
        this.cachedNearbyUsers.clear();
        this.cachedNearbyChats.clear();
        stopService();
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m291lambda$cleanup$5$imuecahxyvjdmessengerLocationController();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$cleanup$5$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m291lambda$cleanup$5$imuecahxyvjdmessengerLocationController() {
        this.requests.clear();
        this.sharingLocationsMap.clear();
        this.sharingLocations.clear();
        setLastKnownLocation(null);
        stop();
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void setLastKnownLocation(BDLocation location) {
        this.lastKnownLocation = location;
        if (location != null) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda17
                @Override // java.lang.Runnable
                public final void run() {
                    NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.newLocationAvailable, new Object[0]);
                }
            });
        }
    }

    public void setCachedNearbyUsersAndChats(ArrayList<TLRPC.TL_peerLocated> u, ArrayList<TLRPC.TL_peerLocated> c) {
        this.cachedNearbyUsers = new ArrayList<>(u);
        this.cachedNearbyChats = new ArrayList<>(c);
    }

    public ArrayList<TLRPC.TL_peerLocated> getCachedNearbyUsers() {
        return this.cachedNearbyUsers;
    }

    public ArrayList<TLRPC.TL_peerLocated> getCachedNearbyChats() {
        return this.cachedNearbyChats;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void addSharingLocation(long did, int mid, int period, TLRPC.Message message) {
        final SharingLocationInfo info = new SharingLocationInfo();
        info.did = did;
        info.mid = mid;
        info.period = period;
        info.messageObject = new MessageObject(this.currentAccount, message, false);
        info.stopTime = getConnectionsManager().getCurrentTime() + period;
        final SharingLocationInfo old = this.sharingLocationsMap.get(did);
        this.sharingLocationsMap.put(did, info);
        if (old != null) {
            this.sharingLocations.remove(old);
        }
        this.sharingLocations.add(info);
        saveSharingLocation(info, 0);
        this.lastLocationSendTime = (SystemClock.uptimeMillis() - 30000) + DefaultRenderersFactory.DEFAULT_ALLOWED_VIDEO_JOINING_TIME_MS;
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda4
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m288xc954a6de(old, info);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$addSharingLocation$7$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m288xc954a6de(SharingLocationInfo old, SharingLocationInfo info) {
        if (old != null) {
            this.sharingLocationsUI.remove(old);
        }
        this.sharingLocationsUI.add(info);
        this.sharingLocationsMapUI.put(info.did, info);
        startService();
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsChanged, new Object[0]);
    }

    public boolean isSharingLocation(long did) {
        return this.sharingLocationsMapUI.indexOfKey(did) >= 0;
    }

    public SharingLocationInfo getSharingLocationInfo(long did) {
        return this.sharingLocationsMapUI.get(did);
    }

    private void loadSharingLocations() {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda22
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m295x7dbd091f();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadSharingLocations$11$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m295x7dbd091f() {
        final ArrayList<SharingLocationInfo> result = new ArrayList<>();
        final ArrayList<TLRPC.User> users = new ArrayList<>();
        final ArrayList<TLRPC.Chat> chats = new ArrayList<>();
        try {
            ArrayList<Integer> usersToLoad = new ArrayList<>();
            ArrayList<Integer> chatsToLoad = new ArrayList<>();
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT uid, mid, date, period, message FROM sharing_locations WHERE 1", new Object[0]);
            while (cursor.next()) {
                SharingLocationInfo info = new SharingLocationInfo();
                info.did = cursor.longValue(0);
                info.mid = cursor.intValue(1);
                info.stopTime = cursor.intValue(2);
                info.period = cursor.intValue(3);
                NativeByteBuffer data = cursor.byteBufferValue(4);
                if (data != null) {
                    info.messageObject = new MessageObject(this.currentAccount, TLRPC.Message.TLdeserialize(data, data.readInt32(false), false), false);
                    MessagesStorage.addUsersAndChatsFromMessage(info.messageObject.messageOwner, usersToLoad, chatsToLoad);
                    data.reuse();
                }
                result.add(info);
                int lower_id = (int) info.did;
                int i = (int) (info.did >> 32);
                if (lower_id != 0) {
                    if (lower_id < 0) {
                        if (!chatsToLoad.contains(Integer.valueOf(-lower_id))) {
                            chatsToLoad.add(Integer.valueOf(-lower_id));
                        }
                    } else if (!usersToLoad.contains(Integer.valueOf(lower_id))) {
                        usersToLoad.add(Integer.valueOf(lower_id));
                    }
                }
            }
            cursor.dispose();
            if (!chatsToLoad.isEmpty()) {
                getMessagesStorage().getChatsInternal(TextUtils.join(",", chatsToLoad), chats);
            }
            if (!usersToLoad.isEmpty()) {
                getMessagesStorage().getUsersInternal(TextUtils.join(",", usersToLoad), users);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        if (!result.isEmpty()) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda11
                @Override // java.lang.Runnable
                public final void run() {
                    LocationController.this.m294xbad09fc0(users, chats, result);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadSharingLocations$10$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m294xbad09fc0(ArrayList users, ArrayList chats, final ArrayList result) {
        getMessagesController().putUsers(users, true);
        getMessagesController().putChats(chats, true);
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda16
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m297xea960ac4(result);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadSharingLocations$9$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m297xea960ac4(final ArrayList result) {
        this.sharingLocations.addAll(result);
        for (int a2 = 0; a2 < this.sharingLocations.size(); a2++) {
            SharingLocationInfo info = this.sharingLocations.get(a2);
            this.sharingLocationsMap.put(info.did, info);
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda20
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m296x27a9a165(result);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadSharingLocations$8$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m296x27a9a165(ArrayList result) {
        this.sharingLocationsUI.addAll(result);
        for (int a2 = 0; a2 < result.size(); a2++) {
            SharingLocationInfo info = (SharingLocationInfo) result.get(a2);
            this.sharingLocationsMapUI.put(info.did, info);
        }
        startService();
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsChanged, new Object[0]);
    }

    private void saveSharingLocation(final SharingLocationInfo info, final int remove) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m305xfa56b070(remove, info);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$saveSharingLocation$12$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m305xfa56b070(int remove, SharingLocationInfo info) {
        try {
            if (remove == 2) {
                getMessagesStorage().getDatabase().executeFast("DELETE FROM sharing_locations WHERE 1").stepThis().dispose();
            } else if (remove == 1) {
                if (info == null) {
                    return;
                }
                getMessagesStorage().getDatabase().executeFast("DELETE FROM sharing_locations WHERE uid = " + info.did).stepThis().dispose();
            } else if (info == null) {
            } else {
                SQLitePreparedStatement state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO sharing_locations VALUES(?, ?, ?, ?, ?)");
                state.requery();
                NativeByteBuffer data = new NativeByteBuffer(info.messageObject.messageOwner.getObjectSize());
                info.messageObject.messageOwner.serializeToStream(data);
                state.bindLong(1, info.did);
                state.bindInteger(2, info.mid);
                state.bindInteger(3, info.stopTime);
                state.bindInteger(4, info.period);
                state.bindByteBuffer(5, data);
                state.step();
                state.dispose();
                data.reuse();
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public void removeSharingLocation(final long did) {
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda15
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m304xd3c61626(did);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeSharingLocation$15$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m304xd3c61626(long did) {
        final SharingLocationInfo info = this.sharingLocationsMap.get(did);
        this.sharingLocationsMap.remove(did);
        if (info != null) {
            TLRPC.TL_messages_editMessage req = new TLRPC.TL_messages_editMessage();
            req.peer = getMessagesController().getInputPeer((int) info.did);
            req.id = info.mid;
            req.flags |= 16384;
            req.media = new TLRPC.TL_inputMediaGeoLive();
            req.media.stopped = true;
            req.media.geo_point = new TLRPC.TL_inputGeoPointEmpty();
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda18
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    LocationController.this.m302x4ded4368(tLObject, tL_error);
                }
            });
            this.sharingLocations.remove(info);
            saveSharingLocation(info, 1);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda19
                @Override // java.lang.Runnable
                public final void run() {
                    LocationController.this.m303x10d9acc7(info);
                }
            });
            if (this.sharingLocations.isEmpty()) {
                stop();
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeSharingLocation$13$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m302x4ded4368(TLObject response, TLRPC.TL_error error) {
        if (error != null) {
            return;
        }
        getMessagesController().processUpdates((TLRPC.Updates) response, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeSharingLocation$14$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m303x10d9acc7(SharingLocationInfo info) {
        this.sharingLocationsUI.remove(info);
        this.sharingLocationsMapUI.remove(info.did);
        if (this.sharingLocationsUI.isEmpty()) {
            stopService();
        }
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsChanged, new Object[0]);
    }

    private void startService() {
        try {
            ApplicationLoader.applicationContext.startService(new Intent(ApplicationLoader.applicationContext, LocationSharingService.class));
        } catch (Throwable e) {
            FileLog.e(e);
        }
    }

    private void stopService() {
        ApplicationLoader.applicationContext.stopService(new Intent(ApplicationLoader.applicationContext, LocationSharingService.class));
    }

    public void removeAllLocationSharings() {
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda9
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m301xa848cebf();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeAllLocationSharings$18$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m301xa848cebf() {
        for (int a2 = 0; a2 < this.sharingLocations.size(); a2++) {
            SharingLocationInfo info = this.sharingLocations.get(a2);
            TLRPC.TL_messages_editMessage req = new TLRPC.TL_messages_editMessage();
            req.peer = getMessagesController().getInputPeer((int) info.did);
            req.id = info.mid;
            req.flags |= 16384;
            req.media = new TLRPC.TL_inputMediaGeoLive();
            req.media.stopped = true;
            req.media.geo_point = new TLRPC.TL_inputGeoPointEmpty();
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda6
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    LocationController.this.m299x226ffc01(tLObject, tL_error);
                }
            });
        }
        this.sharingLocations.clear();
        this.sharingLocationsMap.clear();
        saveSharingLocation(null, 2);
        stop();
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda7
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m300xe55c6560();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeAllLocationSharings$16$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m299x226ffc01(TLObject response, TLRPC.TL_error error) {
        if (error != null) {
            return;
        }
        getMessagesController().processUpdates((TLRPC.Updates) response, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeAllLocationSharings$17$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m300xe55c6560() {
        this.sharingLocationsUI.clear();
        this.sharingLocationsMapUI.clear();
        stopService();
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsChanged, new Object[0]);
    }

    public void setBaiduMapLocation(BDLocation location, boolean first) {
        if (location == null) {
            return;
        }
        this.lastLocationByBaiduMaps = true;
        LatLng newLatLng = new LatLng(location.getLatitude(), location.getLongitude());
        LatLng lastLatLng = new LatLng(this.lastKnownLocation.getLatitude(), this.lastKnownLocation.getLongitude());
        if (first || (this.lastKnownLocation != null && DistanceUtil.getDistance(newLatLng, lastLatLng) >= 20.0d)) {
            this.lastLocationSendTime = SystemClock.uptimeMillis() - 30000;
            this.locationSentSinceLastBaiduMapUpdate = false;
        } else if (this.locationSentSinceLastBaiduMapUpdate) {
            this.lastLocationSendTime = (SystemClock.uptimeMillis() - 30000) + 20000;
            this.locationSentSinceLastBaiduMapUpdate = false;
        }
        setLastKnownLocation(location);
    }

    private void start() {
        if (this.started) {
            return;
        }
        this.lastLocationStartTime = SystemClock.uptimeMillis();
        this.started = true;
        this.mLocClient.start();
    }

    private void stop() {
        if (this.lookingForPeopleNearby) {
            return;
        }
        this.started = false;
        this.mLocClient.stop();
    }

    public void startLocationLookupForPeopleNearby(final boolean stop) {
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda8
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m307x825d2075(stop);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startLocationLookupForPeopleNearby$19$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m307x825d2075(boolean stop) {
        boolean z = !stop;
        this.lookingForPeopleNearby = z;
        if (z) {
            start();
        } else if (this.sharingLocations.isEmpty()) {
            stop();
        }
    }

    public BDLocation getLastKnownLocation() {
        return this.lastKnownLocation;
    }

    public void loadLiveLocations(final long did) {
        if (this.cacheRequests.indexOfKey(did) >= 0) {
            return;
        }
        this.cacheRequests.put(did, true);
        TLRPC.TL_messages_getRecentLocations req = new TLRPC.TL_messages_getRecentLocations();
        req.peer = getMessagesController().getInputPeer((int) did);
        req.limit = 100;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda12
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                LocationController.this.m293xc68a376a(did, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadLiveLocations$21$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m293xc68a376a(final long did, final TLObject response, TLRPC.TL_error error) {
        if (error != null) {
            return;
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda21
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.this.m292x39dce0b(did, response);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadLiveLocations$20$im-uecahxyvjd-messenger-LocationController  reason: not valid java name */
    public /* synthetic */ void m292x39dce0b(long did, TLObject response) {
        this.cacheRequests.delete(did);
        TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
        int a2 = 0;
        while (a2 < res.messages.size()) {
            if (!(res.messages.get(a2).media instanceof TLRPC.TL_messageMediaGeoLive)) {
                res.messages.remove(a2);
                a2--;
            }
            a2++;
        }
        getMessagesStorage().putUsersAndChats(res.users, res.chats, true, true);
        getMessagesController().putUsers(res.users, false);
        getMessagesController().putChats(res.chats, false);
        this.locationsCache.put(did, res.messages);
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.liveLocationsCacheChanged, Long.valueOf(did), Integer.valueOf(this.currentAccount));
    }

    public static int getLocationsCount() {
        int count = 0;
        for (int a2 = 0; a2 < 3; a2++) {
            count += getInstance(a2).sharingLocationsUI.size();
        }
        return count;
    }

    public static void fetchLocationAddress(final BDLocation location, final LocationFetchCallback callback) {
        if (callback == null) {
            return;
        }
        Runnable fetchLocationRunnable = callbacks.get(callback);
        if (fetchLocationRunnable != null) {
            Utilities.globalQueue.cancelRunnable(fetchLocationRunnable);
            callbacks.remove(callback);
        }
        if (location == null) {
            if (callback != null) {
                callback.onLocationAddressAvailable(null, null, null);
                return;
            }
            return;
        }
        DispatchQueue dispatchQueue = Utilities.globalQueue;
        Runnable fetchLocationRunnable2 = new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda14
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.lambda$fetchLocationAddress$23(BDLocation.this, callback);
            }
        };
        dispatchQueue.postRunnable(fetchLocationRunnable2, 300L);
        callbacks.put(callback, fetchLocationRunnable2);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$fetchLocationAddress$23(final BDLocation location, final LocationFetchCallback callback) {
        String displayName;
        String name;
        try {
            Geocoder gcd = new Geocoder(ApplicationLoader.applicationContext, LocaleController.getInstance().getSystemDefaultLocale());
            List<Address> addresses = gcd.getFromLocation(location.getLatitude(), location.getLongitude(), 1);
            if (addresses.size() <= 0) {
                displayName = String.format(Locale.US, "Unknown address (%f,%f)", Double.valueOf(location.getLatitude()), Double.valueOf(location.getLongitude()));
                name = displayName;
            } else {
                Address address = addresses.get(0);
                boolean hasAny = false;
                StringBuilder nameBuilder = new StringBuilder();
                StringBuilder displayNameBuilder = new StringBuilder();
                String arg = address.getSubThoroughfare();
                if (!TextUtils.isEmpty(arg)) {
                    nameBuilder.append(arg);
                    hasAny = true;
                }
                String arg2 = address.getThoroughfare();
                if (!TextUtils.isEmpty(arg2)) {
                    if (nameBuilder.length() > 0) {
                        nameBuilder.append(", ");
                    }
                    nameBuilder.append(arg2);
                    hasAny = true;
                }
                if (!hasAny) {
                    String arg3 = address.getAdminArea();
                    if (!TextUtils.isEmpty(arg3)) {
                        if (nameBuilder.length() > 0) {
                            nameBuilder.append(", ");
                        }
                        nameBuilder.append(arg3);
                    }
                    String arg4 = address.getSubAdminArea();
                    if (!TextUtils.isEmpty(arg4)) {
                        if (nameBuilder.length() > 0) {
                            nameBuilder.append(", ");
                        }
                        nameBuilder.append(arg4);
                    }
                }
                String arg5 = address.getLocality();
                if (!TextUtils.isEmpty(arg5)) {
                    if (nameBuilder.length() > 0) {
                        nameBuilder.append(", ");
                    }
                    nameBuilder.append(arg5);
                }
                String arg6 = address.getCountryName();
                if (!TextUtils.isEmpty(arg6)) {
                    if (nameBuilder.length() > 0) {
                        nameBuilder.append(", ");
                    }
                    nameBuilder.append(arg6);
                }
                String arg7 = address.getCountryName();
                if (!TextUtils.isEmpty(arg7)) {
                    if (displayNameBuilder.length() > 0) {
                        displayNameBuilder.append(", ");
                    }
                    displayNameBuilder.append(arg7);
                }
                String arg8 = address.getLocality();
                if (!TextUtils.isEmpty(arg8)) {
                    if (displayNameBuilder.length() > 0) {
                        displayNameBuilder.append(", ");
                    }
                    displayNameBuilder.append(arg8);
                }
                if (!hasAny) {
                    String arg9 = address.getAdminArea();
                    if (!TextUtils.isEmpty(arg9)) {
                        if (displayNameBuilder.length() > 0) {
                            displayNameBuilder.append(", ");
                        }
                        displayNameBuilder.append(arg9);
                    }
                    String arg10 = address.getSubAdminArea();
                    if (!TextUtils.isEmpty(arg10)) {
                        if (displayNameBuilder.length() > 0) {
                            displayNameBuilder.append(", ");
                        }
                        displayNameBuilder.append(arg10);
                    }
                }
                name = nameBuilder.toString();
                displayName = displayNameBuilder.toString();
            }
        } catch (Exception e) {
            displayName = String.format(Locale.US, "Unknown address (%f,%f)", Double.valueOf(location.getLatitude()), Double.valueOf(location.getLongitude()));
            name = displayName;
        }
        final String displayName2 = name;
        final String displayNameFinal = displayName;
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.LocationController$$ExternalSyntheticLambda2
            @Override // java.lang.Runnable
            public final void run() {
                LocationController.lambda$fetchLocationAddress$22(LocationController.LocationFetchCallback.this, displayName2, displayNameFinal, location);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$fetchLocationAddress$22(LocationFetchCallback callback, String nameFinal, String displayNameFinal, BDLocation location) {
        callbacks.remove(callback);
        if (callback != null) {
            callback.onLocationAddressAvailable(nameFinal, displayNameFinal, location);
        }
    }
}
