package im.uecahxyvjd.messenger;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ShortcutInfo;
import android.content.pm.ShortcutManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.Shader;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.Icon;
import android.os.Build;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.SpannedString;
import android.text.TextUtils;
import android.text.style.CharacterStyle;
import android.util.LongSparseArray;
import android.util.SparseArray;
import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
import im.uecahxyvjd.messenger.MediaDataController;
import im.uecahxyvjd.messenger.support.SparseLongArray;
import im.uecahxyvjd.sqlite.SQLiteCursor;
import im.uecahxyvjd.sqlite.SQLiteDatabase;
import im.uecahxyvjd.sqlite.SQLitePreparedStatement;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.NativeByteBuffer;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.SerializedData;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.LaunchActivity;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.components.AvatarDrawable;
import im.uecahxyvjd.ui.components.TextStyleSpan;
import im.uecahxyvjd.ui.components.URLSpanReplacement;
import im.uecahxyvjd.ui.components.URLSpanUserMention;
import im.uecahxyvjd.ui.components.toast.ToastUtils;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
/* loaded from: classes16.dex */
public class MediaDataController extends BaseController {
    public static final int MEDIA_AUDIO = 2;
    public static final int MEDIA_FILE = 1;
    public static final int MEDIA_MUSIC = 4;
    public static final int MEDIA_PHOTOVIDEO = 0;
    public static final int MEDIA_TYPES_COUNT = 5;
    public static final int MEDIA_URL = 3;
    public static final int TYPE_EMOJI = 4;
    public static final int TYPE_FAVE = 2;
    public static final int TYPE_FEATURED = 3;
    public static final int TYPE_IMAGE = 0;
    public static final int TYPE_MASK = 1;
    private static RectF bitmapRect;
    private static Paint erasePaint;
    private static Paint roundPaint;
    private static Path roundPath;
    private HashMap<String, ArrayList<TLRPC.Document>> allStickers;
    private HashMap<String, ArrayList<TLRPC.Document>> allStickersFeatured;
    private int[] archivedStickersCount;
    private SparseArray<TLRPC.BotInfo> botInfos;
    private LongSparseArray<TLRPC.Message> botKeyboards;
    private SparseLongArray botKeyboardsByMids;
    private HashMap<String, Boolean> currentFetchingEmoji;
    private LongSparseArray<TLRPC.Message> draftMessages;
    private LongSparseArray<TLRPC.DraftMessage> drafts;
    private ArrayList<TLRPC.StickerSetCovered> featuredStickerSets;
    private LongSparseArray<TLRPC.StickerSetCovered> featuredStickerSetsById;
    private boolean featuredStickersLoaded;
    private LongSparseArray<TLRPC.TL_messages_stickerSet> groupStickerSets;
    public ArrayList<TLRPC.TL_topPeer> hints;
    private boolean inTransaction;
    public ArrayList<TLRPC.TL_topPeer> inlineBots;
    private LongSparseArray<TLRPC.TL_messages_stickerSet> installedStickerSetsById;
    private long lastMergeDialogId;
    private int lastReqId;
    private int lastReturnedNum;
    private String lastSearchQuery;
    private int[] loadDate;
    private int loadFeaturedDate;
    private int loadFeaturedHash;
    private int[] loadHash;
    boolean loaded;
    boolean loading;
    private boolean loadingDrafts;
    private boolean loadingFeaturedStickers;
    private boolean loadingRecentGifs;
    private boolean[] loadingRecentStickers;
    private boolean[] loadingStickers;
    private int mergeReqId;
    private int[] messagesSearchCount;
    private boolean[] messagesSearchEndReached;
    private SharedPreferences preferences;
    private ArrayList<Long> readingStickerSets;
    private ArrayList<TLRPC.Document> recentGifs;
    private boolean recentGifsLoaded;
    private ArrayList<TLRPC.Document>[] recentStickers;
    private boolean[] recentStickersLoaded;
    private int reqId;
    private ArrayList<MessageObject> searchResultMessages;
    private SparseArray<MessageObject>[] searchResultMessagesMap;
    private ArrayList<TLRPC.TL_messages_stickerSet>[] stickerSets;
    private LongSparseArray<TLRPC.TL_messages_stickerSet> stickerSetsById;
    private HashMap<String, TLRPC.TL_messages_stickerSet> stickerSetsByName;
    private LongSparseArray<String> stickersByEmoji;
    private LongSparseArray<TLRPC.Document>[] stickersByIds;
    private boolean[] stickersLoaded;
    private ArrayList<Long> unreadStickerSets;
    private static volatile MediaDataController[] Instance = new MediaDataController[3];
    public static long installingStickerSetId = -1;
    private static Comparator<TLRPC.MessageEntity> entityComparator = new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda38
        @Override // java.util.Comparator
        public final int compare(Object obj, Object obj2) {
            return MediaDataController.lambda$static$86((TLRPC.MessageEntity) obj, (TLRPC.MessageEntity) obj2);
        }
    };

    /* loaded from: classes16.dex */
    public static class KeywordResult {
        public String emoji;
        public String keyword;
    }

    /* loaded from: classes16.dex */
    public interface KeywordResultCallback {
        void run(ArrayList<KeywordResult> arrayList, String str);
    }

    public static MediaDataController getInstance(int num) {
        MediaDataController localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (MediaDataController.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    MediaDataController[] mediaDataControllerArr = Instance;
                    MediaDataController mediaDataController = new MediaDataController(num);
                    localInstance = mediaDataController;
                    mediaDataControllerArr[num] = mediaDataController;
                }
            }
        }
        return localInstance;
    }

    public MediaDataController(int num) {
        super(num);
        this.stickerSets = new ArrayList[]{new ArrayList<>(), new ArrayList<>(), new ArrayList<>(0), new ArrayList<>(), new ArrayList<>()};
        this.stickersByIds = new LongSparseArray[]{new LongSparseArray<>(), new LongSparseArray<>(), new LongSparseArray<>(), new LongSparseArray<>(), new LongSparseArray<>()};
        this.stickerSetsById = new LongSparseArray<>();
        this.installedStickerSetsById = new LongSparseArray<>();
        this.groupStickerSets = new LongSparseArray<>();
        this.stickerSetsByName = new HashMap<>();
        this.loadingStickers = new boolean[5];
        this.stickersLoaded = new boolean[5];
        this.loadHash = new int[5];
        this.loadDate = new int[5];
        this.archivedStickersCount = new int[2];
        this.stickersByEmoji = new LongSparseArray<>();
        this.allStickers = new HashMap<>();
        this.allStickersFeatured = new HashMap<>();
        this.recentStickers = new ArrayList[]{new ArrayList<>(), new ArrayList<>(), new ArrayList<>()};
        this.loadingRecentStickers = new boolean[3];
        this.recentStickersLoaded = new boolean[3];
        this.recentGifs = new ArrayList<>();
        this.featuredStickerSets = new ArrayList<>();
        this.featuredStickerSetsById = new LongSparseArray<>();
        this.unreadStickerSets = new ArrayList<>();
        this.readingStickerSets = new ArrayList<>();
        this.messagesSearchCount = new int[]{0, 0};
        this.messagesSearchEndReached = new boolean[]{false, false};
        this.searchResultMessages = new ArrayList<>();
        this.searchResultMessagesMap = new SparseArray[]{new SparseArray<>(), new SparseArray<>()};
        this.hints = new ArrayList<>();
        this.inlineBots = new ArrayList<>();
        this.drafts = new LongSparseArray<>();
        this.draftMessages = new LongSparseArray<>();
        this.botInfos = new SparseArray<>();
        this.botKeyboards = new LongSparseArray<>();
        this.botKeyboardsByMids = new SparseLongArray();
        this.currentFetchingEmoji = new HashMap<>();
        if (this.currentAccount == 0) {
            this.preferences = ApplicationLoader.applicationContext.getSharedPreferences("drafts", 0);
        } else {
            this.preferences = ApplicationLoader.applicationContext.getSharedPreferences("drafts" + this.currentAccount, 0);
        }
        Map<String, ?> values = this.preferences.getAll();
        for (Map.Entry<String, ?> entry : values.entrySet()) {
            try {
                String key = entry.getKey();
                long did = Utilities.parseLong(key).longValue();
                byte[] bytes = Utilities.hexToBytes((String) entry.getValue());
                SerializedData serializedData = new SerializedData(bytes);
                if (key.startsWith("r_")) {
                    TLRPC.Message message = TLRPC.Message.TLdeserialize(serializedData, serializedData.readInt32(true), true);
                    message.readAttachPath(serializedData, getUserConfig().clientUserId);
                    if (message != null) {
                        this.draftMessages.put(did, message);
                    }
                } else {
                    TLRPC.DraftMessage draftMessage = TLRPC.DraftMessage.TLdeserialize(serializedData, serializedData.readInt32(true), true);
                    if (draftMessage != null) {
                        this.drafts.put(did, draftMessage);
                    }
                }
                serializedData.cleanup();
            } catch (Exception e) {
            }
        }
    }

    public void cleanup() {
        for (int a2 = 0; a2 < 3; a2++) {
            this.recentStickers[a2].clear();
            this.loadingRecentStickers[a2] = false;
            this.recentStickersLoaded[a2] = false;
        }
        for (int a3 = 0; a3 < 4; a3++) {
            this.loadHash[a3] = 0;
            this.loadDate[a3] = 0;
            this.stickerSets[a3].clear();
            this.loadingStickers[a3] = false;
            this.stickersLoaded[a3] = false;
        }
        this.featuredStickerSets.clear();
        this.loadFeaturedDate = 0;
        this.loadFeaturedHash = 0;
        this.allStickers.clear();
        this.allStickersFeatured.clear();
        this.stickersByEmoji.clear();
        this.featuredStickerSetsById.clear();
        this.featuredStickerSets.clear();
        this.unreadStickerSets.clear();
        this.recentGifs.clear();
        this.stickerSetsById.clear();
        this.installedStickerSetsById.clear();
        this.stickerSetsByName.clear();
        this.loadingFeaturedStickers = false;
        this.featuredStickersLoaded = false;
        this.loadingRecentGifs = false;
        this.recentGifsLoaded = false;
        this.currentFetchingEmoji.clear();
        if (Build.VERSION.SDK_INT >= 25) {
            Utilities.globalQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda124
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.lambda$cleanup$0();
                }
            });
        }
        this.loading = false;
        this.loaded = false;
        this.hints.clear();
        this.inlineBots.clear();
        getNotificationCenter().postNotificationName(NotificationCenter.reloadHints, new Object[0]);
        getNotificationCenter().postNotificationName(NotificationCenter.reloadInlineHints, new Object[0]);
        this.drafts.clear();
        this.draftMessages.clear();
        this.preferences.edit().clear().commit();
        this.botInfos.clear();
        this.botKeyboards.clear();
        this.botKeyboardsByMids.clear();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$cleanup$0() {
        try {
            ShortcutManager shortcutManager = (ShortcutManager) ApplicationLoader.applicationContext.getSystemService(ShortcutManager.class);
            shortcutManager.removeAllDynamicShortcuts();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public void checkStickers(int type) {
        if (this.loadingStickers[type]) {
            return;
        }
        if (!this.stickersLoaded[type] || Math.abs((System.currentTimeMillis() / 1000) - this.loadDate[type]) >= 3600) {
            loadStickers(type, true, false);
        }
    }

    public void checkFeaturedStickers() {
        if (this.loadingFeaturedStickers) {
            return;
        }
        if (!this.featuredStickersLoaded || Math.abs((System.currentTimeMillis() / 1000) - this.loadFeaturedDate) >= 3600) {
            loadFeaturedStickers(true, false);
        }
    }

    public ArrayList<TLRPC.Document> getRecentStickers(int type) {
        ArrayList<TLRPC.Document> arrayList = this.recentStickers[type];
        return new ArrayList<>(arrayList.subList(0, Math.min(arrayList.size(), 20)));
    }

    public ArrayList<TLRPC.Document> getRecentStickersNoCopy(int type) {
        return this.recentStickers[type];
    }

    public boolean isStickerInFavorites(TLRPC.Document document) {
        if (document == null) {
            return false;
        }
        for (int a2 = 0; a2 < this.recentStickers[2].size(); a2++) {
            TLRPC.Document d = this.recentStickers[2].get(a2);
            if (d.id == document.id && d.dc_id == document.dc_id) {
                return true;
            }
        }
        return false;
    }

    public void addRecentSticker(final int type, final Object parentObject, TLRPC.Document document, int date, boolean remove) {
        boolean found;
        int maxCount;
        final TLRPC.Document old;
        if (!MessageObject.isStickerDocument(document)) {
            return;
        }
        int a2 = 0;
        while (true) {
            if (a2 >= this.recentStickers[type].size()) {
                found = false;
                break;
            }
            TLRPC.Document image = this.recentStickers[type].get(a2);
            if (image.id != document.id) {
                a2++;
            } else {
                this.recentStickers[type].remove(a2);
                if (!remove) {
                    this.recentStickers[type].add(0, image);
                }
                found = true;
            }
        }
        if (!found && !remove) {
            this.recentStickers[type].add(0, document);
        }
        if (type == 2) {
            if (remove) {
                ToastUtils.show((int) R.string.RemovedFromFavorites);
            } else {
                ToastUtils.show((int) R.string.AddedToFavorites);
            }
            final TLRPC.TL_messages_faveSticker req = new TLRPC.TL_messages_faveSticker();
            req.id = new TLRPC.TL_inputDocument();
            req.id.id = document.id;
            req.id.access_hash = document.access_hash;
            req.id.file_reference = document.file_reference;
            if (req.id.file_reference == null) {
                req.id.file_reference = new byte[0];
            }
            req.unfave = remove;
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda99
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.this.m341xca04ffc8(parentObject, req, tLObject, tL_error);
                }
            });
            int maxCount2 = getMessagesController().maxFaveStickersCount;
            maxCount = maxCount2;
        } else {
            maxCount = getMessagesController().maxRecentStickersCount;
        }
        if (this.recentStickers[type].size() > maxCount || remove) {
            if (remove) {
                old = document;
            } else {
                ArrayList<TLRPC.Document> arrayList = this.recentStickers[type];
                old = arrayList.remove(arrayList.size() - 1);
            }
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda100
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m342x64a5c249(type, old);
                }
            });
        }
        if (!remove) {
            ArrayList<TLRPC.Document> arrayList2 = new ArrayList<>();
            arrayList2.add(document);
            processLoadedRecentDocuments(type, arrayList2, false, date, false);
        }
        if (type == 2) {
            getNotificationCenter().postNotificationName(NotificationCenter.recentDocumentsDidLoad, false, Integer.valueOf(type));
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$addRecentSticker$1$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m341xca04ffc8(Object parentObject, TLRPC.TL_messages_faveSticker req, TLObject response, TLRPC.TL_error error) {
        if (error != null && FileRefController.isFileRefError(error.text) && parentObject != null) {
            getFileRefController().requestReference(parentObject, req);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$addRecentSticker$2$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m342x64a5c249(int type, TLRPC.Document old) {
        int cacheType;
        if (type == 0) {
            cacheType = 3;
        } else if (type == 1) {
            cacheType = 4;
        } else {
            cacheType = 5;
        }
        try {
            getMessagesStorage().getDatabase().executeFast("DELETE FROM web_recent_v3 WHERE id = '" + old.id + "' AND type = " + cacheType).stepThis().dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public ArrayList<TLRPC.Document> getRecentGifs() {
        return new ArrayList<>(this.recentGifs);
    }

    public void removeRecentGifById(TLRPC.Document document) {
        if (document == null) {
            return;
        }
        for (int i = 0; i < this.recentGifs.size(); i++) {
            TLRPC.Document realDocument = this.recentGifs.get(i);
            if (realDocument.id == document.id) {
                removeRecentGif(realDocument);
                return;
            }
        }
    }

    public void removeRecentGif(final TLRPC.Document document) {
        this.recentGifs.remove(document);
        final TLRPC.TL_messages_saveGif req = new TLRPC.TL_messages_saveGif();
        req.id = new TLRPC.TL_inputDocument();
        req.id.id = document.id;
        req.id.access_hash = document.access_hash;
        req.id.file_reference = document.file_reference;
        if (req.id.file_reference == null) {
            req.id.file_reference = new byte[0];
        }
        req.unsave = true;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda39
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m437xf49ca134(req, tLObject, tL_error);
            }
        });
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda40
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m438x8f3d63b5(document);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeRecentGif$3$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m437xf49ca134(TLRPC.TL_messages_saveGif req, TLObject response, TLRPC.TL_error error) {
        if (error != null && FileRefController.isFileRefError(error.text)) {
            getFileRefController().requestReference("gif", req);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeRecentGif$4$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m438x8f3d63b5(TLRPC.Document document) {
        try {
            getMessagesStorage().getDatabase().executeFast("DELETE FROM web_recent_v3 WHERE id = '" + document.id + "' AND type = 2").stepThis().dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public boolean hasRecentGif(TLRPC.Document document) {
        for (int a2 = 0; a2 < this.recentGifs.size(); a2++) {
            TLRPC.Document image = this.recentGifs.get(a2);
            if (image.id == document.id) {
                this.recentGifs.remove(a2);
                this.recentGifs.add(0, image);
                return true;
            }
        }
        return false;
    }

    public boolean hasRecentGifNoChangeINdex(TLRPC.Document document) {
        for (int a2 = 0; a2 < this.recentGifs.size(); a2++) {
            if (this.recentGifs.get(a2).id == document.id) {
                return true;
            }
        }
        return false;
    }

    public void addRecentGif(TLRPC.Document document, int date) {
        boolean found = false;
        int a2 = 0;
        while (true) {
            if (a2 >= this.recentGifs.size()) {
                break;
            }
            TLRPC.Document image = this.recentGifs.get(a2);
            if (image.id != document.id) {
                a2++;
            } else {
                this.recentGifs.remove(a2);
                this.recentGifs.add(0, image);
                found = true;
                break;
            }
        }
        if (!found) {
            this.recentGifs.add(0, document);
        }
        if (this.recentGifs.size() > getMessagesController().maxRecentGifsCount) {
            ArrayList<TLRPC.Document> arrayList = this.recentGifs;
            final TLRPC.Document old = arrayList.remove(arrayList.size() - 1);
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda75
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m340x28302493(old);
                }
            });
        }
        ArrayList<TLRPC.Document> arrayList2 = new ArrayList<>();
        arrayList2.add(document);
        processLoadedRecentDocuments(0, arrayList2, true, date, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$addRecentGif$5$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m340x28302493(TLRPC.Document old) {
        try {
            getMessagesStorage().getDatabase().executeFast("DELETE FROM web_recent_v3 WHERE id = '" + old.id + "' AND type = 2").stepThis().dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public boolean isLoadingStickers(int type) {
        return this.loadingStickers[type];
    }

    public void replaceStickerSet(final TLRPC.TL_messages_stickerSet set) {
        TLRPC.TL_messages_stickerSet existingSet = this.stickerSetsById.get(set.set.id);
        boolean isGroupSet = false;
        if (existingSet == null) {
            existingSet = this.stickerSetsByName.get(set.set.short_name);
        }
        if (existingSet == null && (existingSet = this.groupStickerSets.get(set.set.id)) != null) {
            isGroupSet = true;
        }
        if (existingSet == null) {
            return;
        }
        boolean changed = false;
        if ("AnimatedEmojies".equals(set.set.short_name)) {
            changed = true;
            existingSet.documents = set.documents;
            existingSet.packs = set.packs;
            existingSet.set = set.set;
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda29
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m441xcc4ef96b(set);
                }
            });
        } else {
            LongSparseArray<TLRPC.Document> documents = new LongSparseArray<>();
            int size = set.documents.size();
            for (int a2 = 0; a2 < size; a2++) {
                TLRPC.Document document = set.documents.get(a2);
                documents.put(document.id, document);
            }
            int size2 = existingSet.documents.size();
            for (int a3 = 0; a3 < size2; a3++) {
                TLRPC.Document newDocument = documents.get(set.documents.get(a3).id);
                if (newDocument != null) {
                    existingSet.documents.set(a3, newDocument);
                    changed = true;
                }
            }
        }
        if (changed) {
            if (isGroupSet) {
                putSetToCache(existingSet);
                return;
            }
            boolean z = set.set.masks;
            ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr = this.stickerSets;
            int type = z ? 1 : 0;
            ArrayList<TLRPC.TL_messages_stickerSet> arrayList = arrayListArr[type];
            int[] iArr = this.loadDate;
            int type2 = z ? 1 : 0;
            int i = iArr[type2];
            int[] iArr2 = this.loadHash;
            int type3 = z ? 1 : 0;
            int i2 = iArr2[type3];
            int type4 = z ? 1 : 0;
            putStickersToCache(type4, arrayList, i, i2);
            if ("AnimatedEmojies".equals(set.set.short_name)) {
                putStickersToCache(4, this.stickerSets[4], this.loadDate[4], this.loadHash[4]);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$replaceStickerSet$6$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m441xcc4ef96b(TLRPC.TL_messages_stickerSet set) {
        LongSparseArray<TLRPC.Document> stickersById = getStickerByIds(4);
        for (int b = 0; b < set.documents.size(); b++) {
            TLRPC.Document document = set.documents.get(b);
            stickersById.put(document.id, document);
        }
    }

    public TLRPC.TL_messages_stickerSet getStickerSetByName(String name) {
        return this.stickerSetsByName.get(name);
    }

    public TLRPC.TL_messages_stickerSet getStickerSetById(long id) {
        return this.stickerSetsById.get(id);
    }

    public TLRPC.TL_messages_stickerSet getGroupStickerSetById(TLRPC.StickerSet stickerSet) {
        TLRPC.TL_messages_stickerSet set = this.stickerSetsById.get(stickerSet.id);
        if (set == null) {
            set = this.groupStickerSets.get(stickerSet.id);
            if (set == null || set.set == null) {
                loadGroupStickerSet(stickerSet, true);
            } else if (set.set.hash != stickerSet.hash) {
                loadGroupStickerSet(stickerSet, false);
            }
        }
        return set;
    }

    public void putGroupStickerSet(TLRPC.TL_messages_stickerSet stickerSet) {
        this.groupStickerSets.put(stickerSet.set.id, stickerSet);
    }

    private void loadGroupStickerSet(final TLRPC.StickerSet stickerSet, boolean cache) {
        if (cache) {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda18
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m382xa36d9d32(stickerSet);
                }
            });
            return;
        }
        TLRPC.TL_messages_getStickerSet req = new TLRPC.TL_messages_getStickerSet();
        req.stickerset = new TLRPC.TL_inputStickerSetID();
        req.stickerset.id = stickerSet.id;
        req.stickerset.access_hash = stickerSet.access_hash;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda19
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m380xf6d15bbf(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadGroupStickerSet$8$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m382xa36d9d32(TLRPC.StickerSet stickerSet) {
        final TLRPC.TL_messages_stickerSet set;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT document FROM web_recent_v3 WHERE id = 's_" + stickerSet.id + "'", new Object[0]);
            if (cursor.next() && !cursor.isNull(0)) {
                NativeByteBuffer data = cursor.byteBufferValue(0);
                if (data != null) {
                    set = TLRPC.TL_messages_stickerSet.TLdeserialize(data, data.readInt32(false), false);
                    data.reuse();
                } else {
                    set = null;
                }
            } else {
                set = null;
            }
            cursor.dispose();
            if (set == null || set.set == null || set.set.hash != stickerSet.hash) {
                loadGroupStickerSet(stickerSet, false);
            }
            if (set != null && set.set != null) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda51
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m381x8ccdab1(set);
                    }
                });
            }
        } catch (Throwable e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadGroupStickerSet$7$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m381x8ccdab1(TLRPC.TL_messages_stickerSet set) {
        this.groupStickerSets.put(set.set.id, set);
        getNotificationCenter().postNotificationName(NotificationCenter.groupStickersDidLoad, Long.valueOf(set.set.id));
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadGroupStickerSet$10$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m380xf6d15bbf(TLObject response, TLRPC.TL_error error) {
        if (response != null) {
            final TLRPC.TL_messages_stickerSet set = (TLRPC.TL_messages_stickerSet) response;
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda120
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m383x3e0e5fb3(set);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadGroupStickerSet$9$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m383x3e0e5fb3(TLRPC.TL_messages_stickerSet set) {
        this.groupStickerSets.put(set.set.id, set);
        getNotificationCenter().postNotificationName(NotificationCenter.groupStickersDidLoad, Long.valueOf(set.set.id));
    }

    private void putSetToCache(final TLRPC.TL_messages_stickerSet set) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda93
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m435x4fa525ca(set);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putSetToCache$11$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m435x4fa525ca(TLRPC.TL_messages_stickerSet set) {
        try {
            SQLiteDatabase database = getMessagesStorage().getDatabase();
            SQLitePreparedStatement state = database.executeFast("REPLACE INTO web_recent_v3 VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
            state.requery();
            state.bindString(1, "s_" + set.set.id);
            state.bindInteger(2, 6);
            state.bindString(3, "");
            state.bindString(4, "");
            state.bindString(5, "");
            state.bindInteger(6, 0);
            state.bindInteger(7, 0);
            state.bindInteger(8, 0);
            state.bindInteger(9, 0);
            NativeByteBuffer data = new NativeByteBuffer(set.getObjectSize());
            set.serializeToStream(data);
            state.bindByteBuffer(10, data);
            state.step();
            data.reuse();
            state.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public HashMap<String, ArrayList<TLRPC.Document>> getAllStickers() {
        return this.allStickers;
    }

    public HashMap<String, ArrayList<TLRPC.Document>> getAllStickersFeatured() {
        return this.allStickersFeatured;
    }

    public TLRPC.Document getEmojiAnimatedSticker(CharSequence message) {
        String emoji = message.toString().replace("️", "");
        ArrayList<TLRPC.TL_messages_stickerSet> arrayList = getStickerSets(4);
        int N = arrayList.size();
        for (int a2 = 0; a2 < N; a2++) {
            TLRPC.TL_messages_stickerSet set = arrayList.get(a2);
            int N2 = set.packs.size();
            for (int b = 0; b < N2; b++) {
                TLRPC.TL_stickerPack pack = set.packs.get(b);
                if (!pack.documents.isEmpty() && TextUtils.equals(pack.emoticon, emoji)) {
                    LongSparseArray<TLRPC.Document> stickerByIds = getStickerByIds(4);
                    return stickerByIds.get(pack.documents.get(0).longValue());
                }
            }
        }
        return null;
    }

    public boolean canAddStickerToFavorites() {
        return (this.stickersLoaded[0] && this.stickerSets[0].size() < 5 && this.recentStickers[2].isEmpty()) ? false : true;
    }

    public ArrayList<TLRPC.TL_messages_stickerSet> getStickerSets(int type) {
        if (type == 3) {
            return this.stickerSets[2];
        }
        return this.stickerSets[type];
    }

    public LongSparseArray<TLRPC.Document> getStickerByIds(int type) {
        return this.stickersByIds[type];
    }

    public ArrayList<TLRPC.StickerSetCovered> getFeaturedStickerSets() {
        return this.featuredStickerSets;
    }

    public ArrayList<Long> getUnreadStickerSets() {
        return this.unreadStickerSets;
    }

    public boolean areAllTrendingStickerSetsUnread() {
        int N = this.featuredStickerSets.size();
        for (int a2 = 0; a2 < N; a2++) {
            TLRPC.StickerSetCovered pack = this.featuredStickerSets.get(a2);
            if (!isStickerPackInstalled(pack.set.id) && ((!pack.covers.isEmpty() || pack.cover != null) && !this.unreadStickerSets.contains(Long.valueOf(pack.set.id)))) {
                return false;
            }
        }
        return true;
    }

    public boolean isStickerPackInstalled(long id) {
        return this.installedStickerSetsById.indexOfKey(id) >= 0;
    }

    public boolean isStickerPackUnread(long id) {
        return this.unreadStickerSets.contains(Long.valueOf(id));
    }

    public boolean isStickerPackInstalled(String name) {
        return this.stickerSetsByName.containsKey(name);
    }

    public String getEmojiForSticker(long id) {
        String value = this.stickersByEmoji.get(id);
        return value != null ? value : "";
    }

    private static int calcDocumentsHash(ArrayList<TLRPC.Document> arrayList) {
        if (arrayList == null) {
            return 0;
        }
        long acc = 0;
        for (int a2 = 0; a2 < Math.min(200, arrayList.size()); a2++) {
            TLRPC.Document document = arrayList.get(a2);
            if (document != null) {
                int high_id = (int) (document.id >> 32);
                int lower_id = (int) document.id;
                acc = (((20261 * ((((acc * 20261) + 2147483648L) + high_id) % 2147483648L)) + 2147483648L) + lower_id) % 2147483648L;
            }
        }
        int a3 = (int) acc;
        return a3;
    }

    /* JADX WARN: Multi-variable type inference failed */
    public void loadRecents(final int type, final boolean gif, boolean cache, boolean force) {
        TLObject request;
        long lastLoadTime;
        if (gif) {
            if (this.loadingRecentGifs) {
                return;
            }
            this.loadingRecentGifs = true;
            if (this.recentGifsLoaded) {
                cache = false;
            }
        } else {
            boolean[] zArr = this.loadingRecentStickers;
            if (zArr[type]) {
                return;
            }
            zArr[type] = true;
            if (this.recentStickersLoaded[type]) {
                cache = false;
            }
        }
        if (cache) {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda5
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m399x9650980e(gif, type);
                }
            });
            return;
        }
        SharedPreferences preferences = MessagesController.getEmojiSettings(this.currentAccount);
        if (!force) {
            if (gif) {
                lastLoadTime = preferences.getLong("lastGifLoadTime", 0L);
            } else {
                lastLoadTime = type == 0 ? preferences.getLong("lastStickersLoadTime", 0L) : type == 1 ? preferences.getLong("lastStickersLoadTimeMask", 0L) : preferences.getLong("lastStickersLoadTimeFavs", 0L);
            }
            if (Math.abs(System.currentTimeMillis() - lastLoadTime) < 3600000) {
                if (!gif) {
                    this.loadingRecentStickers[type] = false;
                    return;
                } else {
                    this.loadingRecentGifs = false;
                    return;
                }
            }
        }
        if (gif) {
            TLRPC.TL_messages_getSavedGifs req = new TLRPC.TL_messages_getSavedGifs();
            req.hash = calcDocumentsHash(this.recentGifs);
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda6
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.this.m400x30f15a8f(type, gif, tLObject, tL_error);
                }
            });
            return;
        }
        if (type == 2) {
            TLRPC.TL_messages_getFavedStickers req2 = new TLRPC.TL_messages_getFavedStickers();
            req2.hash = calcDocumentsHash(this.recentStickers[type]);
            request = req2;
        } else {
            TLRPC.TL_messages_getRecentStickers req3 = new TLRPC.TL_messages_getRecentStickers();
            req3.hash = calcDocumentsHash(this.recentStickers[type]);
            req3.attached = type == 1;
            request = req3;
        }
        getConnectionsManager().sendRequest(request, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda7
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m401xcb921d10(type, gif, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadRecents$13$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m399x9650980e(final boolean gif, final int type) {
        int cacheType;
        if (gif) {
            cacheType = 2;
        } else if (type == 0) {
            cacheType = 3;
        } else if (type == 1) {
            cacheType = 4;
        } else {
            cacheType = 5;
        }
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT document FROM web_recent_v3 WHERE type = " + cacheType + " ORDER BY date DESC", new Object[0]);
            final ArrayList<TLRPC.Document> arrayList = new ArrayList<>();
            while (cursor.next()) {
                if (!cursor.isNull(0)) {
                    NativeByteBuffer data = cursor.byteBufferValue(0);
                    if (data != null) {
                        TLRPC.Document document = TLRPC.Document.TLdeserialize(data, data.readInt32(false), false);
                        if (document != null) {
                            arrayList.add(document);
                        }
                        data.reuse();
                    }
                }
            }
            cursor.dispose();
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda56
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m398xfbafd58d(gif, arrayList, type);
                }
            });
        } catch (Throwable e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$loadRecents$12$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m398xfbafd58d(boolean gif, ArrayList arrayList, int type) {
        if (gif) {
            this.recentGifs = arrayList;
            this.loadingRecentGifs = false;
            this.recentGifsLoaded = true;
        } else {
            this.recentStickers[type] = arrayList;
            this.loadingRecentStickers[type] = false;
            this.recentStickersLoaded[type] = true;
        }
        getNotificationCenter().postNotificationName(NotificationCenter.recentDocumentsDidLoad, Boolean.valueOf(gif), Integer.valueOf(type));
        loadRecents(type, gif, false, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadRecents$14$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m400x30f15a8f(int type, boolean gif, TLObject response, TLRPC.TL_error error) {
        ArrayList<TLRPC.Document> arrayList = null;
        if (response instanceof TLRPC.TL_messages_savedGifs) {
            TLRPC.TL_messages_savedGifs res = (TLRPC.TL_messages_savedGifs) response;
            arrayList = res.gifs;
        }
        processLoadedRecentDocuments(type, arrayList, gif, 0, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadRecents$15$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m401xcb921d10(int type, boolean gif, TLObject response, TLRPC.TL_error error) {
        ArrayList<TLRPC.Document> arrayList = null;
        if (type == 2) {
            if (response instanceof TLRPC.TL_messages_favedStickers) {
                TLRPC.TL_messages_favedStickers res = (TLRPC.TL_messages_favedStickers) response;
                arrayList = res.stickers;
            }
        } else if (response instanceof TLRPC.TL_messages_recentStickers) {
            TLRPC.TL_messages_recentStickers res2 = (TLRPC.TL_messages_recentStickers) response;
            arrayList = res2.stickers;
        }
        processLoadedRecentDocuments(type, arrayList, gif, 0, true);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void processLoadedRecentDocuments(final int type, final ArrayList<TLRPC.Document> documents, final boolean gif, final int date, final boolean replace) {
        if (documents != null) {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda90
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m421xf082f760(gif, type, documents, replace, date);
                }
            });
        }
        if (date == 0) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda91
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m422x8b23b9e1(gif, type, documents);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedRecentDocuments$16$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m421xf082f760(boolean gif, int type, ArrayList documents, boolean replace, int date) {
        SQLiteDatabase database = getMessagesStorage().getDatabase();
        if (database == null) {
            return;
        }
        int maxCount = gif ? getMessagesController().maxRecentGifsCount : type == 2 ? getMessagesController().maxFaveStickersCount : getMessagesController().maxRecentStickersCount;
        try {
            database.beginTransaction();
        } catch (Exception e) {
            FileLog.e("processLoadedRecentDocuments ---> exception 1 ", e);
        }
        NativeByteBuffer data = null;
        SQLitePreparedStatement state = null;
        try {
            try {
                SQLitePreparedStatement state2 = database.executeFast("REPLACE INTO web_recent_v3 VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
                int count = documents.size();
                int cacheType = gif ? 2 : type == 0 ? 3 : type == 1 ? 4 : 5;
                if (replace) {
                    database.executeFast("DELETE FROM web_recent_v3 WHERE type = " + cacheType).stepThis().dispose();
                }
                for (int a2 = 0; a2 < count && a2 != maxCount; a2++) {
                    TLRPC.Document document = (TLRPC.Document) documents.get(a2);
                    state2.requery();
                    state2.bindString(1, "" + document.id);
                    state2.bindInteger(2, cacheType);
                    state2.bindString(3, "");
                    state2.bindString(4, "");
                    state2.bindString(5, "");
                    state2.bindInteger(6, 0);
                    state2.bindInteger(7, 0);
                    state2.bindInteger(8, 0);
                    state2.bindInteger(9, date != 0 ? date : count - a2);
                    NativeByteBuffer data2 = new NativeByteBuffer(document.getObjectSize());
                    document.serializeToStream(data2);
                    state2.bindByteBuffer(10, data2);
                    state2.step();
                    data2.reuse();
                    data = null;
                }
                state2.dispose();
                state = null;
                database.commitTransaction();
                if (documents.size() >= maxCount) {
                    try {
                        database.beginTransaction();
                    } catch (Exception e2) {
                        FileLog.e("processLoadedRecentDocuments ---> exception 2 ", e2);
                    }
                    for (int a3 = maxCount; a3 < documents.size(); a3++) {
                        database.executeFast("DELETE FROM web_recent_v3 WHERE id = '" + ((TLRPC.Document) documents.get(a3)).id + "' AND type = " + cacheType).stepThis().dispose();
                    }
                    database.commitTransaction();
                }
                if (data != null) {
                    data.reuse();
                }
                if (0 == 0) {
                    return;
                }
            } catch (Exception e3) {
                FileLog.e("processLoadedRecentDocuments ---> exception 3 ", e3);
                if (data != null) {
                    data.reuse();
                }
                if (state == null) {
                    return;
                }
            }
            state.dispose();
        } catch (Throwable th) {
            if (data != null) {
                data.reuse();
            }
            if (state != null) {
                state.dispose();
            }
            throw th;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$processLoadedRecentDocuments$17$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m422x8b23b9e1(boolean gif, int type, ArrayList documents) {
        SharedPreferences.Editor editor = MessagesController.getEmojiSettings(this.currentAccount).edit();
        if (!gif) {
            this.loadingRecentStickers[type] = false;
            this.recentStickersLoaded[type] = true;
            if (type == 0) {
                editor.putLong("lastStickersLoadTime", System.currentTimeMillis()).commit();
            } else if (type == 1) {
                editor.putLong("lastStickersLoadTimeMask", System.currentTimeMillis()).commit();
            } else {
                editor.putLong("lastStickersLoadTimeFavs", System.currentTimeMillis()).commit();
            }
        } else {
            this.loadingRecentGifs = false;
            this.recentGifsLoaded = true;
            editor.putLong("lastGifLoadTime", System.currentTimeMillis()).commit();
        }
        if (documents != null) {
            if (gif) {
                this.recentGifs = documents;
            } else {
                this.recentStickers[type] = documents;
            }
            getNotificationCenter().postNotificationName(NotificationCenter.recentDocumentsDidLoad, Boolean.valueOf(gif), Integer.valueOf(type));
        }
    }

    public void reorderStickers(int type, final ArrayList<Long> order) {
        Collections.sort(this.stickerSets[type], new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda34
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return MediaDataController.lambda$reorderStickers$18(order, (TLRPC.TL_messages_stickerSet) obj, (TLRPC.TL_messages_stickerSet) obj2);
            }
        });
        this.loadHash[type] = calcStickersHash(this.stickerSets[type]);
        getNotificationCenter().postNotificationName(NotificationCenter.stickersDidLoad, Integer.valueOf(type));
        loadStickers(type, false, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$reorderStickers$18(ArrayList order, TLRPC.TL_messages_stickerSet lhs, TLRPC.TL_messages_stickerSet rhs) {
        int index1 = order.indexOf(Long.valueOf(lhs.set.id));
        int index2 = order.indexOf(Long.valueOf(rhs.set.id));
        if (index1 > index2) {
            return 1;
        }
        if (index1 < index2) {
            return -1;
        }
        return 0;
    }

    public void calcNewHash(int type) {
        this.loadHash[type] = calcStickersHash(this.stickerSets[type]);
    }

    public void addNewStickerSet(TLRPC.TL_messages_stickerSet set) {
        if (this.stickerSetsById.indexOfKey(set.set.id) >= 0 || this.stickerSetsByName.containsKey(set.set.short_name)) {
            return;
        }
        boolean z = set.set.masks;
        ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr = this.stickerSets;
        int type = z ? 1 : 0;
        arrayListArr[type].add(0, set);
        this.stickerSetsById.put(set.set.id, set);
        this.installedStickerSetsById.put(set.set.id, set);
        this.stickerSetsByName.put(set.set.short_name, set);
        LongSparseArray<TLRPC.Document> stickersById = new LongSparseArray<>();
        for (int a2 = 0; a2 < set.documents.size(); a2++) {
            TLRPC.Document document = set.documents.get(a2);
            stickersById.put(document.id, document);
        }
        for (int a3 = 0; a3 < set.packs.size(); a3++) {
            TLRPC.TL_stickerPack stickerPack = set.packs.get(a3);
            stickerPack.emoticon = stickerPack.emoticon.replace("️", "");
            ArrayList<TLRPC.Document> arrayList = this.allStickers.get(stickerPack.emoticon);
            if (arrayList == null) {
                arrayList = new ArrayList<>();
                this.allStickers.put(stickerPack.emoticon, arrayList);
            }
            for (int c = 0; c < stickerPack.documents.size(); c++) {
                Long id = stickerPack.documents.get(c);
                if (this.stickersByEmoji.indexOfKey(id.longValue()) < 0) {
                    this.stickersByEmoji.put(id.longValue(), stickerPack.emoticon);
                }
                TLRPC.Document sticker = stickersById.get(id.longValue());
                if (sticker != null) {
                    arrayList.add(sticker);
                }
            }
        }
        int[] iArr = this.loadHash;
        ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr2 = this.stickerSets;
        int type2 = z ? 1 : 0;
        int calcStickersHash = calcStickersHash(arrayListArr2[type2]);
        int type3 = z ? 1 : 0;
        iArr[type3] = calcStickersHash;
        NotificationCenter notificationCenter = getNotificationCenter();
        int i = NotificationCenter.stickersDidLoad;
        int type4 = z ? 1 : 0;
        notificationCenter.postNotificationName(i, Integer.valueOf(type4));
        int type5 = z ? 1 : 0;
        loadStickers(type5, false, true);
    }

    public void loadFeaturedStickers(boolean cache, boolean force) {
        if (this.loadingFeaturedStickers) {
            return;
        }
        this.loadingFeaturedStickers = true;
        if (cache) {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda31
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m377xd1d90a42();
                }
            });
            return;
        }
        final TLRPC.TL_messages_getFeaturedStickers req = new TLRPC.TL_messages_getFeaturedStickers();
        req.hash = force ? 0 : this.loadFeaturedHash;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda32
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m379xb64a83d9(req, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Code restructure failed: missing block: B:19:0x0071, code lost:
        if (r4 != null) goto L26;
     */
    /* JADX WARN: Code restructure failed: missing block: B:25:0x007d, code lost:
        r9 = r3;
        processLoadedFeaturedStickers(r0, r1, true, r2, r9);
     */
    /* JADX WARN: Code restructure failed: missing block: B:26:0x0089, code lost:
        return;
     */
    /* renamed from: lambda$loadFeaturedStickers$19$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m377xd1d90a42() {
        ArrayList<TLRPC.StickerSetCovered> newStickerArray = null;
        ArrayList<Long> unread = new ArrayList<>();
        int date = 0;
        int hash = 0;
        SQLiteCursor cursor = null;
        try {
            cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT data, unread, date, hash FROM stickers_featured WHERE 1", new Object[0]);
            if (cursor.next()) {
                NativeByteBuffer data = cursor.byteBufferValue(0);
                if (data != null) {
                    newStickerArray = new ArrayList<>();
                    int count = data.readInt32(false);
                    for (int a2 = 0; a2 < count; a2++) {
                        TLRPC.StickerSetCovered stickerSet = TLRPC.StickerSetCovered.TLdeserialize(data, data.readInt32(false), false);
                        newStickerArray.add(stickerSet);
                    }
                    data.reuse();
                }
                NativeByteBuffer data2 = cursor.byteBufferValue(1);
                if (data2 != null) {
                    int count2 = data2.readInt32(false);
                    for (int a3 = 0; a3 < count2; a3++) {
                        unread.add(Long.valueOf(data2.readInt64(false)));
                    }
                    data2.reuse();
                }
                date = cursor.intValue(2);
                hash = calcFeaturedStickersHash(newStickerArray);
            }
        } catch (Throwable e) {
            try {
                FileLog.e(e);
            } finally {
                if (cursor != null) {
                    cursor.dispose();
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadFeaturedStickers$21$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m379xb64a83d9(final TLRPC.TL_messages_getFeaturedStickers req, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda30
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m378x1ba9c158(response, req);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadFeaturedStickers$20$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m378x1ba9c158(TLObject response, TLRPC.TL_messages_getFeaturedStickers req) {
        if (response instanceof TLRPC.TL_messages_featuredStickers) {
            TLRPC.TL_messages_featuredStickers res = (TLRPC.TL_messages_featuredStickers) response;
            processLoadedFeaturedStickers(res.sets, res.unread, false, (int) (System.currentTimeMillis() / 1000), res.hash);
            return;
        }
        processLoadedFeaturedStickers(null, null, false, (int) (System.currentTimeMillis() / 1000), req.hash);
    }

    private void processLoadedFeaturedStickers(final ArrayList<TLRPC.StickerSetCovered> res, final ArrayList<Long> unreadStickers, final boolean cache, final int date, final int hash) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda65
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m414xf9bc7646();
            }
        });
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda66
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m418x643f804a(cache, res, date, hash, unreadStickers);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedFeaturedStickers$22$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m414xf9bc7646() {
        this.loadingFeaturedStickers = false;
        this.featuredStickersLoaded = true;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedFeaturedStickers$26$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m418x643f804a(boolean cache, final ArrayList res, final int date, final int hash, final ArrayList unreadStickers) {
        long j = 1000;
        if ((cache && (res == null || Math.abs((System.currentTimeMillis() / 1000) - date) >= 3600)) || (!cache && res == null && hash == 0)) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda41
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m415x945d38c7(res, hash);
                }
            }, (res != null || cache) ? 0L : 0L);
            if (res == null) {
                return;
            }
        }
        if (res != null) {
            try {
                final ArrayList<TLRPC.StickerSetCovered> stickerSetsNew = new ArrayList<>();
                final LongSparseArray<TLRPC.StickerSetCovered> stickerSetsByIdNew = new LongSparseArray<>();
                for (int a2 = 0; a2 < res.size(); a2++) {
                    TLRPC.StickerSetCovered stickerSet = (TLRPC.StickerSetCovered) res.get(a2);
                    stickerSetsNew.add(stickerSet);
                    stickerSetsByIdNew.put(stickerSet.set.id, stickerSet);
                }
                if (!cache) {
                    putFeaturedStickersToCache(stickerSetsNew, unreadStickers, date, hash);
                }
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda42
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m416x2efdfb48(unreadStickers, stickerSetsByIdNew, stickerSetsNew, hash, date);
                    }
                });
            } catch (Throwable e) {
                FileLog.e(e);
            }
        } else if (!cache) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda43
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m417xc99ebdc9(date);
                }
            });
            putFeaturedStickersToCache(null, null, date, 0);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedFeaturedStickers$23$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m415x945d38c7(ArrayList res, int hash) {
        if (res != null && hash != 0) {
            this.loadFeaturedHash = hash;
        }
        loadFeaturedStickers(false, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedFeaturedStickers$24$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m416x2efdfb48(ArrayList unreadStickers, LongSparseArray stickerSetsByIdNew, ArrayList stickerSetsNew, int hash, int date) {
        this.unreadStickerSets = unreadStickers;
        this.featuredStickerSetsById = stickerSetsByIdNew;
        this.featuredStickerSets = stickerSetsNew;
        this.loadFeaturedHash = hash;
        this.loadFeaturedDate = date;
        loadStickers(3, true, false);
        getNotificationCenter().postNotificationName(NotificationCenter.featuredStickersDidLoad, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedFeaturedStickers$25$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m417xc99ebdc9(int date) {
        this.loadFeaturedDate = date;
    }

    private void putFeaturedStickersToCache(ArrayList<TLRPC.StickerSetCovered> stickers, final ArrayList<Long> unreadStickers, final int date, final int hash) {
        final ArrayList<TLRPC.StickerSetCovered> stickersFinal = stickers != null ? new ArrayList<>(stickers) : null;
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda89
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m432x408d6595(stickersFinal, unreadStickers, date, hash);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putFeaturedStickersToCache$27$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m432x408d6595(ArrayList stickersFinal, ArrayList unreadStickers, int date, int hash) {
        try {
            if (stickersFinal != null) {
                SQLitePreparedStatement state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO stickers_featured VALUES(?, ?, ?, ?, ?)");
                state.requery();
                int size = 4;
                for (int a2 = 0; a2 < stickersFinal.size(); a2++) {
                    size += ((TLRPC.StickerSetCovered) stickersFinal.get(a2)).getObjectSize();
                }
                NativeByteBuffer data = new NativeByteBuffer(size);
                NativeByteBuffer data2 = new NativeByteBuffer((unreadStickers.size() * 8) + 4);
                data.writeInt32(stickersFinal.size());
                for (int a3 = 0; a3 < stickersFinal.size(); a3++) {
                    ((TLRPC.StickerSetCovered) stickersFinal.get(a3)).serializeToStream(data);
                }
                int a4 = unreadStickers.size();
                data2.writeInt32(a4);
                for (int a5 = 0; a5 < unreadStickers.size(); a5++) {
                    data2.writeInt64(((Long) unreadStickers.get(a5)).longValue());
                }
                state.bindInteger(1, 1);
                state.bindByteBuffer(2, data);
                state.bindByteBuffer(3, data2);
                state.bindInteger(4, date);
                state.bindInteger(5, hash);
                state.step();
                data.reuse();
                data2.reuse();
                state.dispose();
                return;
            }
            SQLitePreparedStatement state2 = getMessagesStorage().getDatabase().executeFast("UPDATE stickers_featured SET date = ?");
            state2.requery();
            state2.bindInteger(1, date);
            state2.step();
            state2.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    private int calcFeaturedStickersHash(ArrayList<TLRPC.StickerSetCovered> sets) {
        long acc = 0;
        for (int a2 = 0; a2 < sets.size(); a2++) {
            TLRPC.StickerSet set = sets.get(a2).set;
            if (!set.archived) {
                int high_id = (int) (set.id >> 32);
                int lower_id = (int) set.id;
                acc = (((((((acc * 20261) + 2147483648L) + high_id) % 2147483648L) * 20261) + 2147483648L) + lower_id) % 2147483648L;
                if (this.unreadStickerSets.contains(Long.valueOf(set.id))) {
                    acc = (((20261 * acc) + 2147483648L) + 1) % 2147483648L;
                }
            }
        }
        int a3 = (int) acc;
        return a3;
    }

    public void markFaturedStickersAsRead(boolean query) {
        if (this.unreadStickerSets.isEmpty()) {
            return;
        }
        this.unreadStickerSets.clear();
        this.loadFeaturedHash = calcFeaturedStickersHash(this.featuredStickerSets);
        getNotificationCenter().postNotificationName(NotificationCenter.featuredStickersDidLoad, new Object[0]);
        putFeaturedStickersToCache(this.featuredStickerSets, this.unreadStickerSets, this.loadFeaturedDate, this.loadFeaturedHash);
        if (query) {
            TLRPC.TL_messages_readFeaturedStickers req = new TLRPC.TL_messages_readFeaturedStickers();
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda81
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.lambda$markFaturedStickersAsRead$28(tLObject, tL_error);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$markFaturedStickersAsRead$28(TLObject response, TLRPC.TL_error error) {
    }

    public int getFeaturesStickersHashWithoutUnread() {
        long acc = 0;
        for (int a2 = 0; a2 < this.featuredStickerSets.size(); a2++) {
            TLRPC.StickerSet set = this.featuredStickerSets.get(a2).set;
            if (!set.archived) {
                int high_id = (int) (set.id >> 32);
                int lower_id = (int) set.id;
                acc = (((20261 * ((((acc * 20261) + 2147483648L) + high_id) % 2147483648L)) + 2147483648L) + lower_id) % 2147483648L;
            }
        }
        int a3 = (int) acc;
        return a3;
    }

    public void markFaturedStickersByIdAsRead(final long id) {
        if (!this.unreadStickerSets.contains(Long.valueOf(id)) || this.readingStickerSets.contains(Long.valueOf(id))) {
            return;
        }
        this.readingStickerSets.add(Long.valueOf(id));
        TLRPC.TL_messages_readFeaturedStickers req = new TLRPC.TL_messages_readFeaturedStickers();
        req.id.add(Long.valueOf(id));
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda105
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.lambda$markFaturedStickersByIdAsRead$29(tLObject, tL_error);
            }
        });
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda106
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m411x9fd0af83(id);
            }
        }, 1000L);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$markFaturedStickersByIdAsRead$29(TLObject response, TLRPC.TL_error error) {
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$markFaturedStickersByIdAsRead$30$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m411x9fd0af83(long id) {
        this.unreadStickerSets.remove(Long.valueOf(id));
        this.readingStickerSets.remove(Long.valueOf(id));
        this.loadFeaturedHash = calcFeaturedStickersHash(this.featuredStickerSets);
        getNotificationCenter().postNotificationName(NotificationCenter.featuredStickersDidLoad, new Object[0]);
        putFeaturedStickersToCache(this.featuredStickerSets, this.unreadStickerSets, this.loadFeaturedDate, this.loadFeaturedHash);
    }

    public int getArchivedStickersCount(int type) {
        return this.archivedStickersCount[type];
    }

    public void loadArchivedStickersCount(final int type, boolean cache) {
        if (cache) {
            SharedPreferences preferences = MessagesController.getNotificationsSettings(this.currentAccount);
            int count = preferences.getInt("archivedStickersCount" + type, -1);
            if (count == -1) {
                loadArchivedStickersCount(type, false);
                return;
            }
            this.archivedStickersCount[type] = count;
            getNotificationCenter().postNotificationName(NotificationCenter.archivedStickersCountDidLoad, Integer.valueOf(type));
            return;
        }
        TLRPC.TL_messages_getArchivedStickers req = new TLRPC.TL_messages_getArchivedStickers();
        req.limit = 0;
        req.masks = type == 1;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda17
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m370x27da900c(type, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadArchivedStickersCount$32$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m370x27da900c(final int type, final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda16
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m369x8d39cd8b(error, response, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadArchivedStickersCount$31$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m369x8d39cd8b(TLRPC.TL_error error, TLObject response, int type) {
        if (error == null) {
            TLRPC.TL_messages_archivedStickers res = (TLRPC.TL_messages_archivedStickers) response;
            this.archivedStickersCount[type] = res.count;
            SharedPreferences preferences = MessagesController.getNotificationsSettings(this.currentAccount);
            preferences.edit().putInt("archivedStickersCount" + type, res.count).commit();
            getNotificationCenter().postNotificationName(NotificationCenter.archivedStickersCountDidLoad, Integer.valueOf(type));
        }
    }

    private void processLoadStickersResponse(final int type, final TLRPC.TL_messages_allStickers res) {
        TLRPC.TL_messages_allStickers tL_messages_allStickers = res;
        final ArrayList<TLRPC.TL_messages_stickerSet> newStickerArray = new ArrayList<>();
        long j = 1000;
        if (tL_messages_allStickers.sets.isEmpty()) {
            processLoadedStickers(type, newStickerArray, false, (int) (System.currentTimeMillis() / 1000), tL_messages_allStickers.hash);
            return;
        }
        final LongSparseArray<TLRPC.TL_messages_stickerSet> newStickerSets = new LongSparseArray<>();
        int a2 = 0;
        while (a2 < tL_messages_allStickers.sets.size()) {
            final TLRPC.StickerSet stickerSet = tL_messages_allStickers.sets.get(a2);
            TLRPC.TL_messages_stickerSet oldSet = this.stickerSetsById.get(stickerSet.id);
            if (oldSet != null && oldSet.set.hash == stickerSet.hash) {
                oldSet.set.archived = stickerSet.archived;
                oldSet.set.installed = stickerSet.installed;
                oldSet.set.official = stickerSet.official;
                newStickerSets.put(oldSet.set.id, oldSet);
                newStickerArray.add(oldSet);
                if (newStickerSets.size() == tL_messages_allStickers.sets.size()) {
                    processLoadedStickers(type, newStickerArray, false, (int) (System.currentTimeMillis() / j), tL_messages_allStickers.hash);
                }
            } else {
                newStickerArray.add(null);
                final int index = a2;
                TLRPC.TL_messages_getStickerSet req = new TLRPC.TL_messages_getStickerSet();
                req.stickerset = new TLRPC.TL_inputStickerSetID();
                req.stickerset.id = stickerSet.id;
                req.stickerset.access_hash = stickerSet.access_hash;
                getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda37
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.this.m413x474a2f3(newStickerArray, index, newStickerSets, stickerSet, res, type, tLObject, tL_error);
                    }
                });
            }
            a2++;
            tL_messages_allStickers = res;
            j = 1000;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadStickersResponse$34$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m413x474a2f3(final ArrayList newStickerArray, final int index, final LongSparseArray newStickerSets, final TLRPC.StickerSet stickerSet, final TLRPC.TL_messages_allStickers res, final int type, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda92
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m412x69d3e072(response, newStickerArray, index, newStickerSets, stickerSet, res, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadStickersResponse$33$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m412x69d3e072(TLObject response, ArrayList newStickerArray, int index, LongSparseArray newStickerSets, TLRPC.StickerSet stickerSet, TLRPC.TL_messages_allStickers res, int type) {
        TLRPC.TL_messages_stickerSet res1 = (TLRPC.TL_messages_stickerSet) response;
        newStickerArray.set(index, res1);
        newStickerSets.put(stickerSet.id, res1);
        if (newStickerSets.size() == res.sets.size()) {
            Iterator<TLRPC.TL_messages_stickerSet> iterator = newStickerArray.iterator();
            while (iterator.hasNext()) {
                TLRPC.TL_messages_stickerSet set = iterator.next();
                if (set == null) {
                    iterator.remove();
                } else if (!isStickerPackInstalled(set.set.id) && set.set.id != installingStickerSetId) {
                    iterator.remove();
                }
            }
            processLoadedStickers(type, newStickerArray, false, (int) (System.currentTimeMillis() / 1000), res.hash);
        }
    }

    public void installStickerSet(Context context, final int type, TLRPC.StickerSetCovered stickerSet) {
        if (stickerSet.set == null || !isStickerPackInstalled(stickerSet.set.id)) {
            installingStickerSetId = stickerSet.set.id;
            TLRPC.TL_messages_installStickerSet req = new TLRPC.TL_messages_installStickerSet();
            TLRPC.InputStickerSet inputStickerSet = new TLRPC.TL_inputStickerSetID();
            inputStickerSet.id = stickerSet.set.id;
            inputStickerSet.access_hash = stickerSet.set.access_hash;
            req.stickerset = inputStickerSet;
            ConnectionsManager.getInstance(this.currentAccount).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda67
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.this.m368x8fa8f721(type, tLObject, tL_error);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$installStickerSet$36$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m368x8fa8f721(final int type, final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda104
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m367xf50834a0(error, response, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$installStickerSet$35$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m367xf50834a0(TLRPC.TL_error error, TLObject response, int type) {
        try {
            if (error == null) {
                if (response instanceof TLRPC.TL_messages_stickerSetInstallResultArchive) {
                    NotificationCenter.getInstance(this.currentAccount).postNotificationName(NotificationCenter.needReloadArchivedStickers, new Object[0]);
                }
            } else {
                ToastUtils.show((int) R.string.ErrorOccurred);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        loadStickers(type, false, true);
    }

    public void loadStickers(final int type, boolean cache, boolean force) {
        TLObject req;
        final int hash;
        if (this.loadingStickers[type]) {
            return;
        }
        if (type == 3) {
            if (this.featuredStickerSets.isEmpty() || !getMessagesController().preloadFeaturedStickers) {
                return;
            }
        } else if (type != 4) {
            loadArchivedStickersCount(type, cache);
        }
        this.loadingStickers[type] = true;
        if (cache) {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda95
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m407x3ccff58c(type);
                }
            });
        } else if (type != 3) {
            if (type == 4) {
                TLRPC.TL_messages_getStickerSet req2 = new TLRPC.TL_messages_getStickerSet();
                req2.stickerset = new TLRPC.TL_inputStickerSetAnimatedEmoji();
                getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda96
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.this.m408xd770b80d(type, tLObject, tL_error);
                    }
                });
                return;
            }
            if (type == 0) {
                req = new TLRPC.TL_messages_getAllStickers();
                TLRPC.TL_messages_getAllStickers tL_messages_getAllStickers = (TLRPC.TL_messages_getAllStickers) req;
                hash = force ? 0 : this.loadHash[type];
                tL_messages_getAllStickers.hash = hash;
            } else {
                req = new TLRPC.TL_messages_getMaskStickers();
                TLRPC.TL_messages_getMaskStickers tL_messages_getMaskStickers = (TLRPC.TL_messages_getMaskStickers) req;
                hash = force ? 0 : this.loadHash[type];
                tL_messages_getMaskStickers.hash = hash;
            }
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda97
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.this.m410xbbe231a4(type, hash, tLObject, tL_error);
                }
            });
        } else {
            TLRPC.TL_messages_allStickers response = new TLRPC.TL_messages_allStickers();
            response.hash = this.loadFeaturedHash;
            int size = this.featuredStickerSets.size();
            for (int a2 = 0; a2 < size; a2++) {
                response.sets.add(this.featuredStickerSets.get(a2).set);
            }
            processLoadStickersResponse(type, response);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Code restructure failed: missing block: B:17:0x0069, code lost:
        if (r3 == null) goto L17;
     */
    /* JADX WARN: Code restructure failed: missing block: B:19:0x006c, code lost:
        processLoadedStickers(r12, r0, true, r1, r2);
     */
    /* JADX WARN: Code restructure failed: missing block: B:20:0x0075, code lost:
        return;
     */
    /* renamed from: lambda$loadStickers$37$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m407x3ccff58c(int type) {
        ArrayList<TLRPC.TL_messages_stickerSet> newStickerArray = null;
        int date = 0;
        int hash = 0;
        SQLiteCursor cursor = null;
        try {
            cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT data, date, hash FROM stickers_v2 WHERE id = " + (type + 1), new Object[0]);
            if (cursor.next()) {
                NativeByteBuffer data = cursor.byteBufferValue(0);
                if (data != null) {
                    newStickerArray = new ArrayList<>();
                    int count = data.readInt32(false);
                    for (int a2 = 0; a2 < count; a2++) {
                        TLRPC.TL_messages_stickerSet stickerSet = TLRPC.TL_messages_stickerSet.TLdeserialize(data, data.readInt32(false), false);
                        newStickerArray.add(stickerSet);
                    }
                    data.reuse();
                }
                date = cursor.intValue(1);
                hash = calcStickersHash(newStickerArray);
            }
        } catch (Throwable e) {
            try {
                FileLog.e(e);
            } finally {
                if (cursor != null) {
                    cursor.dispose();
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadStickers$38$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m408xd770b80d(int type, TLObject response, TLRPC.TL_error error) {
        if (response instanceof TLRPC.TL_messages_stickerSet) {
            ArrayList<TLRPC.TL_messages_stickerSet> newStickerArray = new ArrayList<>();
            newStickerArray.add((TLRPC.TL_messages_stickerSet) response);
            processLoadedStickers(type, newStickerArray, false, (int) (System.currentTimeMillis() / 1000), calcStickersHash(newStickerArray));
            return;
        }
        processLoadedStickers(type, null, false, (int) (System.currentTimeMillis() / 1000), 0);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadStickers$40$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m410xbbe231a4(final int type, final int hash, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda22
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m409x72117a8e(response, type, hash);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadStickers$39$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m409x72117a8e(TLObject response, int type, int hash) {
        if (response instanceof TLRPC.TL_messages_allStickers) {
            processLoadStickersResponse(type, (TLRPC.TL_messages_allStickers) response);
        } else {
            processLoadedStickers(type, null, false, (int) (System.currentTimeMillis() / 1000), hash);
        }
    }

    private void putStickersToCache(final int type, ArrayList<TLRPC.TL_messages_stickerSet> stickers, final int date, final int hash) {
        final ArrayList<TLRPC.TL_messages_stickerSet> stickersFinal = stickers != null ? new ArrayList<>(stickers) : null;
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda47
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m436x6893afff(stickersFinal, type, date, hash);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putStickersToCache$41$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m436x6893afff(ArrayList stickersFinal, int type, int date, int hash) {
        try {
            if (stickersFinal != null) {
                SQLitePreparedStatement state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO stickers_v2 VALUES(?, ?, ?, ?)");
                state.requery();
                int size = 4;
                for (int a2 = 0; a2 < stickersFinal.size(); a2++) {
                    size += ((TLRPC.TL_messages_stickerSet) stickersFinal.get(a2)).getObjectSize();
                }
                NativeByteBuffer data = new NativeByteBuffer(size);
                data.writeInt32(stickersFinal.size());
                for (int a3 = 0; a3 < stickersFinal.size(); a3++) {
                    ((TLRPC.TL_messages_stickerSet) stickersFinal.get(a3)).serializeToStream(data);
                }
                int a4 = type + 1;
                state.bindInteger(1, a4);
                state.bindByteBuffer(2, data);
                state.bindInteger(3, date);
                state.bindInteger(4, hash);
                state.step();
                data.reuse();
                state.dispose();
                return;
            }
            SQLitePreparedStatement state2 = getMessagesStorage().getDatabase().executeFast("UPDATE stickers_v2 SET date = ?");
            state2.requery();
            state2.bindInteger(1, date);
            state2.step();
            state2.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public String getStickerSetName(long setId) {
        TLRPC.TL_messages_stickerSet stickerSet = this.stickerSetsById.get(setId);
        if (stickerSet != null) {
            return stickerSet.set.short_name;
        }
        TLRPC.StickerSetCovered stickerSetCovered = this.featuredStickerSetsById.get(setId);
        if (stickerSetCovered != null) {
            return stickerSetCovered.set.short_name;
        }
        return null;
    }

    public static long getStickerSetId(TLRPC.Document document) {
        for (int a2 = 0; a2 < document.attributes.size(); a2++) {
            TLRPC.DocumentAttribute attribute = document.attributes.get(a2);
            if (attribute instanceof TLRPC.TL_documentAttributeSticker) {
                if (attribute.stickerset instanceof TLRPC.TL_inputStickerSetID) {
                    return attribute.stickerset.id;
                } else {
                    return -1L;
                }
            }
        }
        return -1L;
    }

    public static TLRPC.InputStickerSet getInputStickerSet(TLRPC.Document document) {
        for (int a2 = 0; a2 < document.attributes.size(); a2++) {
            TLRPC.DocumentAttribute attribute = document.attributes.get(a2);
            if (attribute instanceof TLRPC.TL_documentAttributeSticker) {
                if (attribute.stickerset instanceof TLRPC.TL_inputStickerSetEmpty) {
                    return null;
                } else {
                    return attribute.stickerset;
                }
            }
        }
        return null;
    }

    private static int calcStickersHash(ArrayList<TLRPC.TL_messages_stickerSet> sets) {
        long acc = 0;
        for (int a2 = 0; a2 < sets.size(); a2++) {
            TLRPC.StickerSet set = sets.get(a2).set;
            if (!set.archived) {
                acc = (((20261 * acc) + 2147483648L) + set.hash) % 2147483648L;
            }
        }
        int a3 = (int) acc;
        return a3;
    }

    private void processLoadedStickers(final int type, final ArrayList<TLRPC.TL_messages_stickerSet> res, final boolean cache, final int date, final int hash) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda21
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m423xf4e99292(type);
            }
        });
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda23
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m427x5f6c9c96(cache, res, date, hash, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedStickers$42$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m423xf4e99292(int type) {
        this.loadingStickers[type] = false;
        this.stickersLoaded[type] = true;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedStickers$46$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m427x5f6c9c96(boolean cache, final ArrayList res, final int date, final int hash, final int type) {
        HashMap<String, TLRPC.TL_messages_stickerSet> stickerSetsByNameNew;
        LongSparseArray<TLRPC.TL_messages_stickerSet> stickerSetsByIdNew;
        TLRPC.TL_messages_stickerSet stickerSet;
        TLRPC.TL_messages_stickerSet stickerSet2;
        HashMap<String, TLRPC.TL_messages_stickerSet> stickerSetsByNameNew2;
        LongSparseArray<TLRPC.TL_messages_stickerSet> stickerSetsByIdNew2;
        final int i = type;
        long j = 1000;
        if ((cache && (res == null || Math.abs((System.currentTimeMillis() / 1000) - date) >= 3600)) || (!cache && res == null && hash == 0)) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda72
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m424x8f8a5513(res, hash, i);
                }
            }, (res != null || cache) ? 0L : 0L);
            if (res == null) {
                return;
            }
        }
        if (res == null) {
            if (!cache) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda74
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m426xc4cbda15(i, date);
                    }
                });
                putStickersToCache(i, null, date, 0);
                return;
            }
            return;
        }
        try {
            final ArrayList<TLRPC.TL_messages_stickerSet> stickerSetsNew = new ArrayList<>();
            LongSparseArray<TLRPC.TL_messages_stickerSet> stickerSetsByIdNew3 = new LongSparseArray<>();
            HashMap<String, TLRPC.TL_messages_stickerSet> stickerSetsByNameNew3 = new HashMap<>();
            final LongSparseArray<String> stickersByEmojiNew = new LongSparseArray<>();
            final LongSparseArray<TLRPC.Document> stickersByIdNew = new LongSparseArray<>();
            final HashMap<String, ArrayList<TLRPC.Document>> allStickersNew = new HashMap<>();
            int a2 = 0;
            while (a2 < res.size()) {
                try {
                    TLRPC.TL_messages_stickerSet stickerSet3 = (TLRPC.TL_messages_stickerSet) res.get(a2);
                    if (stickerSet3 == null) {
                        stickerSetsByNameNew = stickerSetsByNameNew3;
                        stickerSetsByIdNew = stickerSetsByIdNew3;
                    } else {
                        stickerSetsNew.add(stickerSet3);
                        stickerSetsByIdNew3.put(stickerSet3.set.id, stickerSet3);
                        stickerSetsByNameNew3.put(stickerSet3.set.short_name, stickerSet3);
                        int b = 0;
                        while (b < stickerSet3.documents.size()) {
                            TLRPC.Document document = stickerSet3.documents.get(b);
                            if (document == null) {
                                stickerSetsByNameNew2 = stickerSetsByNameNew3;
                                stickerSetsByIdNew2 = stickerSetsByIdNew3;
                            } else if (document instanceof TLRPC.TL_documentEmpty) {
                                stickerSetsByNameNew2 = stickerSetsByNameNew3;
                                stickerSetsByIdNew2 = stickerSetsByIdNew3;
                            } else {
                                stickerSetsByNameNew2 = stickerSetsByNameNew3;
                                stickerSetsByIdNew2 = stickerSetsByIdNew3;
                                stickersByIdNew.put(document.id, document);
                            }
                            b++;
                            stickerSetsByIdNew3 = stickerSetsByIdNew2;
                            stickerSetsByNameNew3 = stickerSetsByNameNew2;
                        }
                        stickerSetsByNameNew = stickerSetsByNameNew3;
                        stickerSetsByIdNew = stickerSetsByIdNew3;
                        if (!stickerSet3.set.archived) {
                            int b2 = 0;
                            while (b2 < stickerSet3.packs.size()) {
                                TLRPC.TL_stickerPack stickerPack = stickerSet3.packs.get(b2);
                                if (stickerPack == null) {
                                    stickerSet = stickerSet3;
                                } else if (stickerPack.emoticon == null) {
                                    stickerSet = stickerSet3;
                                } else {
                                    stickerPack.emoticon = stickerPack.emoticon.replace("️", "");
                                    ArrayList<TLRPC.Document> arrayList = allStickersNew.get(stickerPack.emoticon);
                                    if (arrayList == null) {
                                        arrayList = new ArrayList<>();
                                        allStickersNew.put(stickerPack.emoticon, arrayList);
                                    }
                                    int c = 0;
                                    while (c < stickerPack.documents.size()) {
                                        Long id = stickerPack.documents.get(c);
                                        if (stickersByEmojiNew.indexOfKey(id.longValue()) >= 0) {
                                            stickerSet2 = stickerSet3;
                                        } else {
                                            stickerSet2 = stickerSet3;
                                            stickersByEmojiNew.put(id.longValue(), stickerPack.emoticon);
                                        }
                                        TLRPC.Document sticker = stickersByIdNew.get(id.longValue());
                                        if (sticker != null) {
                                            arrayList.add(sticker);
                                        }
                                        c++;
                                        stickerSet3 = stickerSet2;
                                    }
                                    stickerSet = stickerSet3;
                                }
                                b2++;
                                stickerSet3 = stickerSet;
                            }
                        }
                    }
                    a2++;
                    stickerSetsByIdNew3 = stickerSetsByIdNew;
                    stickerSetsByNameNew3 = stickerSetsByNameNew;
                } catch (Throwable th) {
                    e = th;
                    FileLog.e(e);
                }
            }
            final HashMap<String, TLRPC.TL_messages_stickerSet> stickerSetsByNameNew4 = stickerSetsByNameNew3;
            final LongSparseArray<TLRPC.TL_messages_stickerSet> stickerSetsByIdNew4 = stickerSetsByIdNew3;
            if (cache) {
                i = type;
            } else {
                i = type;
                putStickersToCache(i, stickerSetsNew, date, hash);
            }
            try {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda73
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m425x2a2b1794(type, stickerSetsByIdNew4, stickerSetsByNameNew4, stickerSetsNew, hash, date, stickersByIdNew, allStickersNew, stickersByEmojiNew);
                    }
                });
            } catch (Throwable th2) {
                e = th2;
                FileLog.e(e);
            }
        } catch (Throwable th3) {
            e = th3;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedStickers$43$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m424x8f8a5513(ArrayList res, int hash, int type) {
        if (res != null && hash != 0) {
            this.loadHash[type] = hash;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$processLoadedStickers$44$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m425x2a2b1794(int type, LongSparseArray stickerSetsByIdNew, HashMap stickerSetsByNameNew, ArrayList stickerSetsNew, int hash, int date, LongSparseArray stickersByIdNew, HashMap allStickersNew, LongSparseArray stickersByEmojiNew) {
        for (int a2 = 0; a2 < this.stickerSets[type].size(); a2++) {
            TLRPC.StickerSet set = this.stickerSets[type].get(a2).set;
            this.stickerSetsById.remove(set.id);
            this.stickerSetsByName.remove(set.short_name);
            if (type != 3 && type != 4) {
                this.installedStickerSetsById.remove(set.id);
            }
        }
        for (int a3 = 0; a3 < stickerSetsByIdNew.size(); a3++) {
            this.stickerSetsById.put(stickerSetsByIdNew.keyAt(a3), (TLRPC.TL_messages_stickerSet) stickerSetsByIdNew.valueAt(a3));
            if (type != 3 && type != 4) {
                this.installedStickerSetsById.put(stickerSetsByIdNew.keyAt(a3), (TLRPC.TL_messages_stickerSet) stickerSetsByIdNew.valueAt(a3));
            }
        }
        this.stickerSetsByName.putAll(stickerSetsByNameNew);
        this.stickerSets[type] = stickerSetsNew;
        this.loadHash[type] = hash;
        this.loadDate[type] = date;
        this.stickersByIds[type] = stickersByIdNew;
        if (type == 0) {
            this.allStickers = allStickersNew;
            this.stickersByEmoji = stickersByEmojiNew;
        } else if (type == 3) {
            this.allStickersFeatured = allStickersNew;
        }
        getNotificationCenter().postNotificationName(NotificationCenter.stickersDidLoad, Integer.valueOf(type));
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedStickers$45$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m426xc4cbda15(int type, int date) {
        this.loadDate[type] = date;
    }

    public void removeStickersSet(Context context, final TLRPC.StickerSet stickerSet, int hide, BaseFragment baseFragment, boolean showSettings) {
        boolean z = stickerSet.masks;
        TLRPC.TL_inputStickerSetID stickerSetID = new TLRPC.TL_inputStickerSetID();
        stickerSetID.access_hash = stickerSet.access_hash;
        stickerSetID.id = stickerSet.id;
        if (hide != 0) {
            stickerSet.archived = hide == 1;
            int a2 = 0;
            while (true) {
                ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr = this.stickerSets;
                int type = z ? 1 : 0;
                if (a2 >= arrayListArr[type].size()) {
                    break;
                }
                ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr2 = this.stickerSets;
                int type2 = z ? 1 : 0;
                TLRPC.TL_messages_stickerSet set = arrayListArr2[type2].get(a2);
                if (set.set.id != stickerSet.id) {
                    a2++;
                } else {
                    ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr3 = this.stickerSets;
                    int type3 = z ? 1 : 0;
                    arrayListArr3[type3].remove(a2);
                    if (hide == 2) {
                        ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr4 = this.stickerSets;
                        int type4 = z ? 1 : 0;
                        arrayListArr4[type4].add(0, set);
                    } else {
                        this.stickerSetsById.remove(set.set.id);
                        this.installedStickerSetsById.remove(set.set.id);
                        this.stickerSetsByName.remove(set.set.short_name);
                    }
                }
            }
            int[] iArr = this.loadHash;
            ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr5 = this.stickerSets;
            int type5 = z ? 1 : 0;
            int calcStickersHash = calcStickersHash(arrayListArr5[type5]);
            int type6 = z ? 1 : 0;
            iArr[type6] = calcStickersHash;
            ArrayList<TLRPC.TL_messages_stickerSet>[] arrayListArr6 = this.stickerSets;
            int type7 = z ? 1 : 0;
            ArrayList<TLRPC.TL_messages_stickerSet> arrayList = arrayListArr6[type7];
            int[] iArr2 = this.loadDate;
            int type8 = z ? 1 : 0;
            int i = iArr2[type8];
            int[] iArr3 = this.loadHash;
            int type9 = z ? 1 : 0;
            int i2 = iArr3[type9];
            int type10 = z ? 1 : 0;
            putStickersToCache(type10, arrayList, i, i2);
            NotificationCenter notificationCenter = getNotificationCenter();
            int i3 = NotificationCenter.stickersDidLoad;
            int type11 = z ? 1 : 0;
            notificationCenter.postNotificationName(i3, Integer.valueOf(type11));
            return;
        }
        TLRPC.TL_messages_uninstallStickerSet req = new TLRPC.TL_messages_uninstallStickerSet();
        req.stickerset = stickerSetID;
        ConnectionsManager connectionsManager = getConnectionsManager();
        final int type12 = z ? 1 : 0;
        connectionsManager.sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda88
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m440x73f8f41a(stickerSet, type12, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeStickersSet$48$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m440x73f8f41a(final TLRPC.StickerSet stickerSet, final int type, TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda69
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m439xd9583199(error, stickerSet, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$removeStickersSet$47$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m439xd9583199(TLRPC.TL_error error, TLRPC.StickerSet stickerSet, int type) {
        try {
            if (error == null) {
                if (stickerSet.masks) {
                    ToastUtils.show((int) R.string.MasksRemoved);
                } else {
                    ToastUtils.show((int) R.string.StickersRemoved);
                }
            } else {
                ToastUtils.show((int) R.string.ErrorOccurred);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        loadStickers(type, false, true);
    }

    /* JADX WARN: Code restructure failed: missing block: B:7:0x0016, code lost:
        if (r1[1] != false) goto L7;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private int getMask() {
        int mask = 0;
        if (this.lastReturnedNum >= this.searchResultMessages.size() - 1) {
            boolean[] zArr = this.messagesSearchEndReached;
            if (zArr[0]) {
            }
        }
        mask = 0 | 1;
        if (this.lastReturnedNum > 0) {
            return mask | 2;
        }
        return mask;
    }

    public boolean isMessageFound(int messageId, boolean mergeDialog) {
        return this.searchResultMessagesMap[mergeDialog ? 1 : 0].indexOfKey(messageId) >= 0;
    }

    public void searchMessagesInChat(String query, long dialog_id, long mergeDialogId, int guid, int direction, TLRPC.User user) {
        searchMessagesInChat(query, dialog_id, mergeDialogId, guid, direction, false, user);
    }

    private void searchMessagesInChat(String query, final long dialog_id, final long mergeDialogId, final int guid, final int direction, boolean internal, final TLRPC.User user) {
        String query2;
        int max_id;
        boolean firstQuery;
        long queryWithDialog;
        long queryWithDialog2;
        int max_id2;
        String query3;
        int max_id3;
        long queryWithDialog3 = dialog_id;
        boolean firstQuery2 = !internal;
        if (this.reqId != 0) {
            getConnectionsManager().cancelRequest(this.reqId, true);
            this.reqId = 0;
        }
        if (this.mergeReqId != 0) {
            getConnectionsManager().cancelRequest(this.mergeReqId, true);
            this.mergeReqId = 0;
        }
        if (query == null) {
            if (this.searchResultMessages.isEmpty()) {
                return;
            }
            if (direction == 1) {
                int i = this.lastReturnedNum + 1;
                this.lastReturnedNum = i;
                if (i < this.searchResultMessages.size()) {
                    MessageObject messageObject = this.searchResultMessages.get(this.lastReturnedNum);
                    NotificationCenter notificationCenter = getNotificationCenter();
                    int i2 = NotificationCenter.chatSearchResultsAvailable;
                    int[] iArr = this.messagesSearchCount;
                    notificationCenter.postNotificationName(i2, Integer.valueOf(guid), Integer.valueOf(messageObject.getId()), Integer.valueOf(getMask()), Long.valueOf(messageObject.getDialogId()), Integer.valueOf(this.lastReturnedNum), Integer.valueOf(iArr[0] + iArr[1]));
                    return;
                }
                boolean[] zArr = this.messagesSearchEndReached;
                if (zArr[0] && mergeDialogId == 0 && zArr[1]) {
                    this.lastReturnedNum--;
                    return;
                }
                String query4 = this.lastSearchQuery;
                ArrayList<MessageObject> arrayList = this.searchResultMessages;
                MessageObject messageObject2 = arrayList.get(arrayList.size() - 1);
                if (messageObject2.getDialogId() == dialog_id && !this.messagesSearchEndReached[0]) {
                    max_id3 = messageObject2.getId();
                    queryWithDialog3 = dialog_id;
                } else {
                    max_id3 = messageObject2.getDialogId() == mergeDialogId ? messageObject2.getId() : 0;
                    queryWithDialog3 = mergeDialogId;
                    this.messagesSearchEndReached[1] = false;
                }
                max_id = max_id3;
                firstQuery = false;
                query2 = query4;
            } else if (direction == 2) {
                int i3 = this.lastReturnedNum - 1;
                this.lastReturnedNum = i3;
                if (i3 < 0) {
                    this.lastReturnedNum = 0;
                    return;
                }
                if (i3 >= this.searchResultMessages.size()) {
                    this.lastReturnedNum = this.searchResultMessages.size() - 1;
                }
                MessageObject messageObject3 = this.searchResultMessages.get(this.lastReturnedNum);
                NotificationCenter notificationCenter2 = getNotificationCenter();
                int i4 = NotificationCenter.chatSearchResultsAvailable;
                int[] iArr2 = this.messagesSearchCount;
                notificationCenter2.postNotificationName(i4, Integer.valueOf(guid), Integer.valueOf(messageObject3.getId()), Integer.valueOf(getMask()), Long.valueOf(messageObject3.getDialogId()), Integer.valueOf(this.lastReturnedNum), Integer.valueOf(iArr2[0] + iArr2[1]));
                return;
            } else {
                return;
            }
        } else {
            if (firstQuery2) {
                getNotificationCenter().postNotificationName(NotificationCenter.chatSearchResultsLoading, Integer.valueOf(guid));
                boolean[] zArr2 = this.messagesSearchEndReached;
                zArr2[1] = false;
                zArr2[0] = false;
                int[] iArr3 = this.messagesSearchCount;
                iArr3[1] = 0;
                iArr3[0] = 0;
                this.searchResultMessages.clear();
                this.searchResultMessagesMap[0].clear();
                this.searchResultMessagesMap[1].clear();
            }
            query2 = query;
            max_id = 0;
            firstQuery = firstQuery2;
        }
        boolean[] zArr3 = this.messagesSearchEndReached;
        if (zArr3[0] && !zArr3[1] && mergeDialogId != 0) {
            queryWithDialog = mergeDialogId;
        } else {
            queryWithDialog = queryWithDialog3;
        }
        if (queryWithDialog != dialog_id || !firstQuery) {
            queryWithDialog2 = queryWithDialog;
            max_id2 = max_id;
            query3 = query2;
        } else if (mergeDialogId != 0) {
            TLRPC.InputPeer inputPeer = getMessagesController().getInputPeer((int) mergeDialogId);
            if (inputPeer == null) {
                return;
            }
            final TLRPC.TL_messages_search req = new TLRPC.TL_messages_search();
            req.peer = inputPeer;
            this.lastMergeDialogId = mergeDialogId;
            req.limit = 1;
            req.q = query2 != null ? query2 : "";
            if (user != null) {
                req.from_id = getMessagesController().getInputUser(user);
                req.flags |= 1;
            }
            req.filter = new TLRPC.TL_inputMessagesFilterEmpty();
            this.mergeReqId = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda113
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.this.m450xf6e87f2e(mergeDialogId, req, dialog_id, guid, direction, user, tLObject, tL_error);
                }
            }, 2);
            return;
        } else {
            queryWithDialog2 = queryWithDialog;
            max_id2 = max_id;
            query3 = query2;
            this.lastMergeDialogId = 0L;
            zArr3[1] = true;
            this.messagesSearchCount[1] = 0;
        }
        final TLRPC.TL_messages_search req2 = new TLRPC.TL_messages_search();
        final long queryWithDialog4 = queryWithDialog2;
        req2.peer = getMessagesController().getInputPeer((int) queryWithDialog4);
        if (req2.peer == null) {
            return;
        }
        req2.limit = 21;
        String query5 = query3;
        req2.q = query5 != null ? query5 : "";
        req2.offset_id = max_id2;
        if (user != null) {
            req2.from_id = getMessagesController().getInputUser(user);
            req2.flags |= 1;
        }
        req2.filter = new TLRPC.TL_inputMessagesFilterEmpty();
        final int currentReqId = this.lastReqId + 1;
        this.lastReqId = currentReqId;
        this.lastSearchQuery = query5;
        this.reqId = getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda115
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m452x2c2a0430(currentReqId, req2, queryWithDialog4, dialog_id, guid, mergeDialogId, user, tLObject, tL_error);
            }
        }, 2);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$searchMessagesInChat$50$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m450xf6e87f2e(final long mergeDialogId, final TLRPC.TL_messages_search req, final long dialog_id, final int guid, final int direction, final TLRPC.User user, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda109
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m449xad17c818(mergeDialogId, response, req, dialog_id, guid, direction, user);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$searchMessagesInChat$49$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m449xad17c818(long mergeDialogId, TLObject response, TLRPC.TL_messages_search req, long dialog_id, int guid, int direction, TLRPC.User user) {
        if (this.lastMergeDialogId == mergeDialogId) {
            this.mergeReqId = 0;
            if (response != null) {
                TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
                this.messagesSearchEndReached[1] = res.messages.isEmpty();
                this.messagesSearchCount[1] = res instanceof TLRPC.TL_messages_messagesSlice ? res.count : res.messages.size();
                searchMessagesInChat(req.q, dialog_id, mergeDialogId, guid, direction, true, user);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$searchMessagesInChat$52$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m452x2c2a0430(final int currentReqId, final TLRPC.TL_messages_search req, final long queryWithDialogFinal, final long dialog_id, final int guid, final long mergeDialogId, final TLRPC.User user, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda63
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m451x918941af(currentReqId, response, req, queryWithDialogFinal, dialog_id, guid, mergeDialogId, user);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$searchMessagesInChat$51$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m451x918941af(int currentReqId, TLObject response, TLRPC.TL_messages_search req, long queryWithDialogFinal, long dialog_id, int guid, long mergeDialogId, TLRPC.User user) {
        if (currentReqId == this.lastReqId) {
            this.reqId = 0;
            if (response != null) {
                TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
                int a2 = 0;
                while (a2 < res.messages.size()) {
                    TLRPC.Message message = res.messages.get(a2);
                    if ((message instanceof TLRPC.TL_messageEmpty) || (message.action instanceof TLRPC.TL_messageActionHistoryClear)) {
                        res.messages.remove(a2);
                        a2--;
                    }
                    a2++;
                }
                getMessagesStorage().putUsersAndChats(res.users, res.chats, true, true);
                getMessagesController().putUsers(res.users, false);
                getMessagesController().putChats(res.chats, false);
                if (req.offset_id == 0 && queryWithDialogFinal == dialog_id) {
                    this.lastReturnedNum = 0;
                    this.searchResultMessages.clear();
                    this.searchResultMessagesMap[0].clear();
                    this.searchResultMessagesMap[1].clear();
                    this.messagesSearchCount[0] = 0;
                }
                boolean added = false;
                for (int a3 = 0; a3 < Math.min(res.messages.size(), 20); a3++) {
                    added = true;
                    MessageObject messageObject = new MessageObject(this.currentAccount, res.messages.get(a3), false);
                    this.searchResultMessages.add(messageObject);
                    this.searchResultMessagesMap[queryWithDialogFinal == dialog_id ? (char) 0 : (char) 1].put(messageObject.getId(), messageObject);
                }
                this.messagesSearchEndReached[queryWithDialogFinal == dialog_id ? (char) 0 : (char) 1] = res.messages.size() != 21;
                this.messagesSearchCount[queryWithDialogFinal == dialog_id ? (char) 0 : (char) 1] = ((res instanceof TLRPC.TL_messages_messagesSlice) || (res instanceof TLRPC.TL_messages_channelMessages)) ? res.count : res.messages.size();
                if (this.searchResultMessages.isEmpty()) {
                    getNotificationCenter().postNotificationName(NotificationCenter.chatSearchResultsAvailable, Integer.valueOf(guid), 0, Integer.valueOf(getMask()), 0L, 0, 0);
                } else if (added) {
                    if (this.lastReturnedNum >= this.searchResultMessages.size()) {
                        this.lastReturnedNum = this.searchResultMessages.size() - 1;
                    }
                    MessageObject messageObject2 = this.searchResultMessages.get(this.lastReturnedNum);
                    NotificationCenter notificationCenter = getNotificationCenter();
                    int i = NotificationCenter.chatSearchResultsAvailable;
                    int[] iArr = this.messagesSearchCount;
                    notificationCenter.postNotificationName(i, Integer.valueOf(guid), Integer.valueOf(messageObject2.getId()), Integer.valueOf(getMask()), Long.valueOf(messageObject2.getDialogId()), Integer.valueOf(this.lastReturnedNum), Integer.valueOf(iArr[0] + iArr[1]));
                }
                if (queryWithDialogFinal == dialog_id) {
                    boolean[] zArr = this.messagesSearchEndReached;
                    if (zArr[0] && mergeDialogId != 0 && !zArr[1]) {
                        searchMessagesInChat(this.lastSearchQuery, dialog_id, mergeDialogId, guid, 0, true, user);
                    }
                }
            }
        }
    }

    public String getLastSearchQuery() {
        return this.lastSearchQuery;
    }

    /* JADX WARN: Removed duplicated region for block: B:11:0x001d  */
    /* JADX WARN: Removed duplicated region for block: B:31:0x00a4  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void loadMedia(final long uid, final int count, final int max_id, final int type, int fromCache, final int classGuid) {
        final boolean isChannel;
        int lower_part;
        if (((int) uid) < 0 && ChatObject.isChannel(-((int) uid), this.currentAccount)) {
            isChannel = true;
            int lower_part2 = (int) uid;
            if (fromCache == 0) {
                lower_part = lower_part2;
            } else if (lower_part2 != 0) {
                TLRPC.TL_messages_search req = new TLRPC.TL_messages_search();
                req.limit = count;
                req.offset_id = max_id;
                if (type == 0) {
                    req.filter = new TLRPC.TL_inputMessagesFilterPhotoVideo();
                } else if (type == 1) {
                    req.filter = new TLRPC.TL_inputMessagesFilterDocument();
                } else if (type == 2) {
                    req.filter = new TLRPC.TL_inputMessagesFilterRoundVoice();
                } else if (type == 3) {
                    req.filter = new TLRPC.TL_inputMessagesFilterUrl();
                } else if (type == 4) {
                    req.filter = new TLRPC.TL_inputMessagesFilterMusic();
                }
                req.q = "";
                req.peer = getMessagesController().getInputPeer(lower_part2);
                if (req.peer == null) {
                    return;
                }
                int reqId = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda86
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.this.m391lambda$loadMedia$53$imuecahxyvjdmessengerMediaDataController(uid, count, max_id, type, classGuid, isChannel, tLObject, tL_error);
                    }
                });
                getConnectionsManager().bindRequestToGuid(reqId, classGuid);
                return;
            } else {
                lower_part = lower_part2;
            }
            loadMediaDatabase(uid, count, max_id, type, classGuid, isChannel, fromCache);
        }
        isChannel = false;
        int lower_part22 = (int) uid;
        if (fromCache == 0) {
        }
        loadMediaDatabase(uid, count, max_id, type, classGuid, isChannel, fromCache);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadMedia$53$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m391lambda$loadMedia$53$imuecahxyvjdmessengerMediaDataController(long uid, int count, int max_id, int type, int classGuid, boolean isChannel, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
            getMessagesController().removeDeletedMessagesFromArray(uid, res.messages);
            processLoadedMedia(res, uid, count, max_id, type, 0, classGuid, isChannel, res.messages.size() == 0);
        }
    }

    public void getMediaCounts(final long uid, final int classGuid) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda0
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m364x8c98495(uid, classGuid);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCounts$58$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m364x8c98495(final long uid, int classGuid) {
        int i;
        SQLiteCursor cursor;
        int a2;
        int lower_part;
        int i2 = -1;
        try {
            final int[] counts = {-1, -1, -1, -1, -1};
            final int[] countsFinal = {-1, -1, -1, -1, -1};
            int i3 = 0;
            int[] old = {0, 0, 0, 0, 0};
            int i4 = 1;
            SQLiteCursor cursor2 = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT type, count, old FROM media_counts_v2 WHERE uid = %d", Long.valueOf(uid)), new Object[0]);
            while (true) {
                i = 2;
                if (!cursor2.next()) {
                    break;
                }
                int type = cursor2.intValue(0);
                if (type >= 0 && type < 5) {
                    int intValue = cursor2.intValue(1);
                    counts[type] = intValue;
                    countsFinal[type] = intValue;
                    old[type] = cursor2.intValue(2);
                }
            }
            cursor2.dispose();
            int lower_part2 = (int) uid;
            if (lower_part2 == 0) {
                int a3 = 0;
                while (a3 < counts.length) {
                    if (counts[a3] == i2) {
                        SQLiteCursor cursor3 = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT COUNT(mid) FROM media_v2 WHERE uid = %d AND type = %d LIMIT 1", Long.valueOf(uid), Integer.valueOf(a3)), new Object[0]);
                        if (cursor3.next()) {
                            counts[a3] = cursor3.intValue(0);
                        } else {
                            counts[a3] = 0;
                        }
                        cursor3.dispose();
                        putMediaCountDatabase(uid, a3, counts[a3]);
                    }
                    a3++;
                    i2 = -1;
                }
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda78
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m360x9e467a91(uid, counts);
                    }
                });
                return;
            }
            boolean missing = false;
            int a4 = 0;
            while (a4 < counts.length) {
                if (counts[a4] != -1 && old[a4] != i4) {
                    a2 = a4;
                    lower_part = lower_part2;
                    cursor = cursor2;
                    int reqId = a2 + 1;
                    a4 = reqId;
                    cursor2 = cursor;
                    lower_part2 = lower_part;
                    i = 2;
                    i3 = 0;
                    i4 = 1;
                }
                final int type2 = a4;
                TLRPC.TL_messages_search req = new TLRPC.TL_messages_search();
                req.limit = i4;
                req.offset_id = i3;
                if (a4 == 0) {
                    req.filter = new TLRPC.TL_inputMessagesFilterPhotoVideo();
                } else if (a4 == i4) {
                    req.filter = new TLRPC.TL_inputMessagesFilterDocument();
                } else if (a4 == i) {
                    req.filter = new TLRPC.TL_inputMessagesFilterRoundVoice();
                } else if (a4 == 3) {
                    req.filter = new TLRPC.TL_inputMessagesFilterUrl();
                } else if (a4 == 4) {
                    req.filter = new TLRPC.TL_inputMessagesFilterMusic();
                }
                req.q = "";
                req.peer = getMessagesController().getInputPeer(lower_part2);
                if (req.peer == null) {
                    counts[a4] = i3;
                    a2 = a4;
                    lower_part = lower_part2;
                    cursor = cursor2;
                } else {
                    cursor = cursor2;
                    a2 = a4;
                    lower_part = lower_part2;
                    int reqId2 = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda79
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            MediaDataController.this.m362xd387ff93(counts, type2, uid, tLObject, tL_error);
                        }
                    });
                    try {
                        getConnectionsManager().bindRequestToGuid(reqId2, classGuid);
                        if (counts[a2] == -1) {
                            missing = true;
                        } else if (old[a2] == 1) {
                            counts[a2] = -1;
                        }
                    } catch (Exception e) {
                        e = e;
                        FileLog.e(e);
                        return;
                    }
                }
                int reqId3 = a2 + 1;
                a4 = reqId3;
                cursor2 = cursor;
                lower_part2 = lower_part;
                i = 2;
                i3 = 0;
                i4 = 1;
            }
            if (!missing) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda80
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m363x6e28c214(uid, countsFinal);
                    }
                });
            }
        } catch (Exception e2) {
            e = e2;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCounts$54$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m360x9e467a91(long uid, int[] counts) {
        getNotificationCenter().postNotificationName(NotificationCenter.mediaCountsDidLoad, Long.valueOf(uid), counts);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCounts$56$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m362xd387ff93(final int[] counts, int type, final long uid, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
            if (res instanceof TLRPC.TL_messages_messages) {
                counts[type] = res.messages.size();
            } else {
                counts[type] = res.count;
            }
            putMediaCountDatabase(uid, type, counts[type]);
        } else {
            counts[type] = 0;
        }
        boolean finished = true;
        int b = 0;
        while (true) {
            if (b >= counts.length) {
                break;
            } else if (counts[b] != -1) {
                b++;
            } else {
                finished = false;
                break;
            }
        }
        if (finished) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda54
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m361x38e73d12(uid, counts);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCounts$55$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m361x38e73d12(long uid, int[] counts) {
        getNotificationCenter().postNotificationName(NotificationCenter.mediaCountsDidLoad, Long.valueOf(uid), counts);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCounts$57$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m363x6e28c214(long uid, int[] countsFinal) {
        getNotificationCenter().postNotificationName(NotificationCenter.mediaCountsDidLoad, Long.valueOf(uid), countsFinal);
    }

    public void getMediaCount(final long uid, final int type, final int classGuid, boolean fromCache) {
        int lower_part = (int) uid;
        if (fromCache || lower_part == 0) {
            getMediaCountDatabase(uid, type, classGuid);
            return;
        }
        TLRPC.TL_messages_search req = new TLRPC.TL_messages_search();
        req.limit = 1;
        req.offset_id = 0;
        if (type == 0) {
            req.filter = new TLRPC.TL_inputMessagesFilterPhotoVideo();
        } else if (type == 1) {
            req.filter = new TLRPC.TL_inputMessagesFilterDocument();
        } else if (type == 2) {
            req.filter = new TLRPC.TL_inputMessagesFilterRoundVoice();
        } else if (type == 3) {
            req.filter = new TLRPC.TL_inputMessagesFilterUrl();
        } else if (type == 4) {
            req.filter = new TLRPC.TL_inputMessagesFilterMusic();
        }
        req.q = "";
        req.peer = getMessagesController().getInputPeer(lower_part);
        if (req.peer == null) {
            return;
        }
        int reqId = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda4
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m358x16cbbfb7(uid, type, classGuid, tLObject, tL_error);
            }
        });
        getConnectionsManager().bindRequestToGuid(reqId, classGuid);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCount$60$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m358x16cbbfb7(long uid, int type, int classGuid, TLObject response, TLRPC.TL_error error) {
        int count;
        if (error == null) {
            final TLRPC.messages_Messages res = (TLRPC.messages_Messages) response;
            getMessagesStorage().putUsersAndChats(res.users, res.chats, true, true);
            if (res instanceof TLRPC.TL_messages_messages) {
                count = res.messages.size();
            } else {
                count = res.count;
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda15
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m357xccfb08a1(res);
                }
            });
            processLoadedMediaCount(count, uid, type, classGuid, false, 0);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCount$59$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m357xccfb08a1(TLRPC.messages_Messages res) {
        getMessagesController().putUsers(res.users, false);
        getMessagesController().putChats(res.chats, false);
    }

    public static int getMediaType(TLRPC.Message message) {
        if (message == null) {
            return -1;
        }
        if (message.media instanceof TLRPC.TL_messageMediaPhoto) {
            return 0;
        }
        if (message.media instanceof TLRPC.TL_messageMediaDocument) {
            if (MessageObject.isVoiceMessage(message) || MessageObject.isRoundVideoMessage(message)) {
                return 2;
            }
            if (MessageObject.isVideoMessage(message)) {
                return 0;
            }
            if (MessageObject.isStickerMessage(message) || MessageObject.isAnimatedStickerMessage(message)) {
                return -1;
            }
            if (MessageObject.isMusicMessage(message)) {
                return 4;
            }
            return 1;
        }
        if (!message.entities.isEmpty()) {
            for (int a2 = 0; a2 < message.entities.size(); a2++) {
                TLRPC.MessageEntity entity = message.entities.get(a2);
                if ((entity instanceof TLRPC.TL_messageEntityUrl) || (entity instanceof TLRPC.TL_messageEntityTextUrl) || (entity instanceof TLRPC.TL_messageEntityEmail)) {
                    return 3;
                }
            }
        }
        return -1;
    }

    public static boolean canAddMessageToMedia(TLRPC.Message message) {
        if (!(message instanceof TLRPC.TL_message_secret) || (!((message.media instanceof TLRPC.TL_messageMediaPhoto) || MessageObject.isVideoMessage(message) || MessageObject.isGifMessage(message)) || message.media.ttl_seconds == 0 || message.media.ttl_seconds > 60)) {
            if ((message instanceof TLRPC.TL_message_secret) || !(message instanceof TLRPC.TL_message) || (!((message.media instanceof TLRPC.TL_messageMediaPhoto) || (message.media instanceof TLRPC.TL_messageMediaDocument)) || message.media.ttl_seconds == 0)) {
                if ((message.media instanceof TLRPC.TL_messageMediaPhoto) || ((message.media instanceof TLRPC.TL_messageMediaDocument) && !MessageObject.isGifDocument(message.media.document))) {
                    return true;
                }
                if (!message.entities.isEmpty()) {
                    for (int a2 = 0; a2 < message.entities.size(); a2++) {
                        TLRPC.MessageEntity entity = message.entities.get(a2);
                        if ((entity instanceof TLRPC.TL_messageEntityUrl) || (entity instanceof TLRPC.TL_messageEntityTextUrl) || (entity instanceof TLRPC.TL_messageEntityEmail)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            return false;
        }
        return false;
    }

    private void processLoadedMedia(final TLRPC.messages_Messages res, final long uid, int count, int max_id, final int type, final int fromCache, final int classGuid, boolean isChannel, final boolean topReached) {
        int lower_part = (int) uid;
        if (fromCache != 0 && res.messages.isEmpty() && lower_part != 0) {
            if (fromCache == 2) {
                return;
            }
            loadMedia(uid, count, max_id, type, 0, classGuid);
            return;
        }
        if (fromCache == 0) {
            ImageLoader.saveMessagesThumbs(res.messages);
            getMessagesStorage().putUsersAndChats(res.users, res.chats, true, true);
            putMediaDatabase(uid, type, res.messages, max_id, topReached);
        }
        SparseArray<TLRPC.User> usersDict = new SparseArray<>();
        for (int a2 = 0; a2 < res.users.size(); a2++) {
            TLRPC.User u = res.users.get(a2);
            usersDict.put(u.id, u);
        }
        final ArrayList<MessageObject> objects = new ArrayList<>();
        for (int a3 = 0; a3 < res.messages.size(); a3++) {
            TLRPC.Message message = res.messages.get(a3);
            objects.add(new MessageObject(this.currentAccount, message, usersDict, true));
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda62
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m419xb096878f(res, fromCache, uid, objects, classGuid, type, topReached);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processLoadedMedia$61$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m419xb096878f(TLRPC.messages_Messages res, int fromCache, long uid, ArrayList objects, int classGuid, int type, boolean topReached) {
        int totalCount = res.count;
        getMessagesController().putUsers(res.users, fromCache != 0);
        getMessagesController().putChats(res.chats, fromCache != 0);
        getNotificationCenter().postNotificationName(NotificationCenter.mediaDidLoad, Long.valueOf(uid), Integer.valueOf(totalCount), objects, Integer.valueOf(classGuid), Integer.valueOf(type), Boolean.valueOf(topReached));
    }

    private void processLoadedMediaCount(final int count, final long uid, final int type, final int classGuid, final boolean fromCache, final int old) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda94
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m420x49ff23b(uid, fromCache, count, type, old, classGuid);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Code restructure failed: missing block: B:14:0x001f, code lost:
        if (r10 == 0) goto L14;
     */
    /* JADX WARN: Removed duplicated region for block: B:19:0x0035  */
    /* JADX WARN: Removed duplicated region for block: B:28:0x006b  */
    /* renamed from: lambda$processLoadedMediaCount$62$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m420x49ff23b(long uid, boolean fromCache, int count, int type, int old, int classGuid) {
        int lower_part = (int) uid;
        int i = 0;
        boolean reload = fromCache && (count == -1 || (count == 0 && type == 2)) && lower_part != 0;
        if (!reload) {
            if (old == 1) {
            }
            if (reload) {
                if (!fromCache) {
                    putMediaCountDatabase(uid, type, count);
                }
                NotificationCenter notificationCenter = getNotificationCenter();
                int i2 = NotificationCenter.mediaCountDidLoad;
                Object[] objArr = new Object[4];
                objArr[0] = Long.valueOf(uid);
                if (!fromCache || count != -1) {
                    i = count;
                }
                objArr[1] = Integer.valueOf(i);
                objArr[2] = Boolean.valueOf(fromCache);
                objArr[3] = Integer.valueOf(type);
                notificationCenter.postNotificationName(i2, objArr);
                return;
            }
            return;
        }
        getMediaCount(uid, type, classGuid, false);
        if (reload) {
        }
    }

    private void putMediaCountDatabase(final long uid, final int type, final int count) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda60
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m433xe269b3a6(uid, type, count);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putMediaCountDatabase$63$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m433xe269b3a6(long uid, int type, int count) {
        try {
            SQLitePreparedStatement state2 = getMessagesStorage().getDatabase().executeFast("REPLACE INTO media_counts_v2 VALUES(?, ?, ?, ?)");
            state2.requery();
            state2.bindLong(1, uid);
            state2.bindInteger(2, type);
            state2.bindInteger(3, count);
            state2.bindInteger(4, 0);
            state2.step();
            state2.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    private void getMediaCountDatabase(final long uid, final int type, final int classGuid) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda76
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m359x4114dac0(uid, type, classGuid);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getMediaCountDatabase$64$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m359x4114dac0(long uid, int type, int classGuid) {
        int old;
        int count = -1;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT count, old FROM media_counts_v2 WHERE uid = %d AND type = %d LIMIT 1", Long.valueOf(uid), Integer.valueOf(type)), new Object[0]);
            if (!cursor.next()) {
                old = 0;
            } else {
                count = cursor.intValue(0);
                int old2 = cursor.intValue(1);
                old = old2;
            }
            cursor.dispose();
            int lower_part = (int) uid;
            if (count == -1 && lower_part == 0) {
                cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT COUNT(mid) FROM media_v2 WHERE uid = %d AND type = %d LIMIT 1", Long.valueOf(uid), Integer.valueOf(type)), new Object[0]);
                if (cursor.next()) {
                    count = cursor.intValue(0);
                }
                cursor.dispose();
                if (count != -1) {
                    try {
                        putMediaCountDatabase(uid, type, count);
                    } catch (Exception e) {
                        e = e;
                        FileLog.e(e);
                        return;
                    }
                }
            }
            processLoadedMediaCount(count, uid, type, classGuid, true, old);
        } catch (Exception e2) {
            e = e2;
        }
    }

    private void loadMediaDatabase(final long uid, final int count, final int max_id, final int type, final int classGuid, final boolean isChannel, final int fromCache) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda25
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m392xf2e9a404(count, uid, max_id, isChannel, type, fromCache, classGuid);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadMediaDatabase$65$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m392xf2e9a404(int count, long uid, int max_id, boolean isChannel, int type, int fromCache, int classGuid) {
        boolean topReached;
        TLRPC.TL_messages_messages res;
        TLRPC.TL_messages_messages res2;
        ArrayList<Integer> usersToLoad;
        ArrayList<Integer> chatsToLoad;
        boolean isEnd;
        SQLiteCursor cursor;
        TLRPC.TL_messages_messages res3;
        boolean topReached2;
        ArrayList<Integer> usersToLoad2;
        boolean isEnd2;
        long holeMessageId;
        TLRPC.TL_messages_messages res4 = new TLRPC.TL_messages_messages();
        try {
            ArrayList<Integer> usersToLoad3 = new ArrayList<>();
            ArrayList<Integer> chatsToLoad2 = new ArrayList<>();
            int countToLoad = count + 1;
            SQLiteDatabase database = getMessagesStorage().getDatabase();
            try {
                if (((int) uid) != 0) {
                    res2 = res4;
                    long messageMaxId = max_id;
                    int channelId = isChannel ? -((int) uid) : 0;
                    if (messageMaxId == 0 || channelId == 0) {
                        usersToLoad = usersToLoad3;
                        topReached = false;
                    } else {
                        usersToLoad = usersToLoad3;
                        topReached = false;
                        messageMaxId |= channelId << 32;
                    }
                    SQLiteCursor cursor2 = database.queryFinalized(String.format(Locale.US, "SELECT start FROM media_holes_v2 WHERE uid = %d AND type = %d AND start IN (0, 1)", Long.valueOf(uid), Integer.valueOf(type)), new Object[0]);
                    if (cursor2.next()) {
                        isEnd = cursor2.intValue(0) == 1;
                        cursor2.dispose();
                    } else {
                        cursor2.dispose();
                        cursor2 = database.queryFinalized(String.format(Locale.US, "SELECT min(mid) FROM media_v2 WHERE uid = %d AND type = %d AND mid > 0", Long.valueOf(uid), Integer.valueOf(type)), new Object[0]);
                        if (cursor2.next()) {
                            try {
                                int mid = cursor2.intValue(0);
                                if (mid != 0) {
                                    SQLitePreparedStatement state = database.executeFast("REPLACE INTO media_holes_v2 VALUES(?, ?, ?, ?)");
                                    state.requery();
                                    state.bindLong(1, uid);
                                    state.bindInteger(2, type);
                                    isEnd2 = false;
                                    state.bindInteger(3, 0);
                                    state.bindInteger(4, mid);
                                    state.step();
                                    state.dispose();
                                } else {
                                    isEnd2 = false;
                                }
                            } catch (Exception e) {
                                e = e;
                                res = res2;
                                try {
                                    res.messages.clear();
                                    res.chats.clear();
                                    res.users.clear();
                                    FileLog.e(e);
                                    processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                                    return;
                                } catch (Throwable th) {
                                    th = th;
                                    processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                                    throw th;
                                }
                            } catch (Throwable th2) {
                                th = th2;
                                res = res2;
                                processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                                throw th;
                            }
                        } else {
                            isEnd2 = false;
                        }
                        cursor2.dispose();
                        isEnd = isEnd2;
                    }
                    if (messageMaxId != 0) {
                        long holeMessageId2 = 0;
                        SQLiteCursor cursor3 = database.queryFinalized(String.format(Locale.US, "SELECT end FROM media_holes_v2 WHERE uid = %d AND type = %d AND end <= %d ORDER BY end DESC LIMIT 1", Long.valueOf(uid), Integer.valueOf(type), Integer.valueOf(max_id)), new Object[0]);
                        if (cursor3.next()) {
                            long holeMessageId3 = cursor3.intValue(0);
                            if (channelId != 0) {
                                chatsToLoad = chatsToLoad2;
                                holeMessageId2 = holeMessageId3 | (channelId << 32);
                            } else {
                                chatsToLoad = chatsToLoad2;
                                holeMessageId2 = holeMessageId3;
                            }
                        } else {
                            chatsToLoad = chatsToLoad2;
                        }
                        cursor3.dispose();
                        cursor = holeMessageId2 > 1 ? database.queryFinalized(String.format(Locale.US, "SELECT data, mid FROM media_v2 WHERE uid = %d AND mid > 0 AND mid < %d AND mid >= %d AND type = %d ORDER BY date DESC, mid DESC LIMIT %d", Long.valueOf(uid), Long.valueOf(messageMaxId), Long.valueOf(holeMessageId2), Integer.valueOf(type), Integer.valueOf(countToLoad)), new Object[0]) : database.queryFinalized(String.format(Locale.US, "SELECT data, mid FROM media_v2 WHERE uid = %d AND mid > 0 AND mid < %d AND type = %d ORDER BY date DESC, mid DESC LIMIT %d", Long.valueOf(uid), Long.valueOf(messageMaxId), Integer.valueOf(type), Integer.valueOf(countToLoad)), new Object[0]);
                    } else {
                        chatsToLoad = chatsToLoad2;
                        SQLiteCursor cursor4 = database.queryFinalized(String.format(Locale.US, "SELECT max(end) FROM media_holes_v2 WHERE uid = %d AND type = %d", Long.valueOf(uid), Integer.valueOf(type)), new Object[0]);
                        if (cursor4.next()) {
                            holeMessageId = cursor4.intValue(0);
                            if (channelId != 0) {
                                holeMessageId |= channelId << 32;
                            }
                        } else {
                            holeMessageId = 0;
                        }
                        cursor4.dispose();
                        cursor = holeMessageId > 1 ? database.queryFinalized(String.format(Locale.US, "SELECT data, mid FROM media_v2 WHERE uid = %d AND mid >= %d AND type = %d ORDER BY date DESC, mid DESC LIMIT %d", Long.valueOf(uid), Long.valueOf(holeMessageId), Integer.valueOf(type), Integer.valueOf(countToLoad)), new Object[0]) : database.queryFinalized(String.format(Locale.US, "SELECT data, mid FROM media_v2 WHERE uid = %d AND mid > 0 AND type = %d ORDER BY date DESC, mid DESC LIMIT %d", Long.valueOf(uid), Integer.valueOf(type), Integer.valueOf(countToLoad)), new Object[0]);
                    }
                } else {
                    usersToLoad = usersToLoad3;
                    topReached = false;
                    chatsToLoad = chatsToLoad2;
                    res2 = res4;
                    isEnd = true;
                    if (max_id != 0) {
                        cursor = database.queryFinalized(String.format(Locale.US, "SELECT m.data, m.mid, r.random_id FROM media_v2 as m LEFT JOIN randoms as r ON r.mid = m.mid WHERE m.uid = %d AND m.mid > %d AND type = %d ORDER BY m.mid ASC LIMIT %d", Long.valueOf(uid), Integer.valueOf(max_id), Integer.valueOf(type), Integer.valueOf(countToLoad)), new Object[0]);
                    } else {
                        try {
                            cursor = database.queryFinalized(String.format(Locale.US, "SELECT m.data, m.mid, r.random_id FROM media_v2 as m LEFT JOIN randoms as r ON r.mid = m.mid WHERE m.uid = %d AND type = %d ORDER BY m.mid ASC LIMIT %d", Long.valueOf(uid), Integer.valueOf(type), Integer.valueOf(countToLoad)), new Object[0]);
                        } catch (Exception e2) {
                            e = e2;
                            res = res2;
                            res.messages.clear();
                            res.chats.clear();
                            res.users.clear();
                            FileLog.e(e);
                            processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                            return;
                        } catch (Throwable th3) {
                            th = th3;
                            res = res2;
                            processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                            throw th;
                        }
                    }
                }
                while (cursor.next()) {
                    try {
                        NativeByteBuffer data = cursor.byteBufferValue(0);
                        if (data != null) {
                            TLRPC.Message message = TLRPC.Message.TLdeserialize(data, data.readInt32(false), false);
                            message.readAttachPath(data, getUserConfig().clientUserId);
                            data.reuse();
                            message.id = cursor.intValue(1);
                            message.dialog_id = uid;
                            if (((int) uid) == 0) {
                                message.random_id = cursor.longValue(2);
                            }
                            res3 = res2;
                            try {
                                res3.messages.add(message);
                                usersToLoad2 = usersToLoad;
                                MessagesStorage.addUsersAndChatsFromMessage(message, usersToLoad2, chatsToLoad);
                            } catch (Exception e3) {
                                e = e3;
                                res = res3;
                                res.messages.clear();
                                res.chats.clear();
                                res.users.clear();
                                FileLog.e(e);
                                processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                                return;
                            } catch (Throwable th4) {
                                th = th4;
                                res = res3;
                                processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                                throw th;
                            }
                        } else {
                            res3 = res2;
                            usersToLoad2 = usersToLoad;
                        }
                        usersToLoad = usersToLoad2;
                        res2 = res3;
                    } catch (Exception e4) {
                        e = e4;
                        res = res2;
                        res.messages.clear();
                        res.chats.clear();
                        res.users.clear();
                        FileLog.e(e);
                        processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                        return;
                    } catch (Throwable th5) {
                        th = th5;
                        res = res2;
                        processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                        throw th;
                    }
                }
                res3 = res2;
                ArrayList<Integer> usersToLoad4 = usersToLoad;
                try {
                    cursor.dispose();
                    if (!usersToLoad4.isEmpty()) {
                        getMessagesStorage().getUsersInternal(TextUtils.join(",", usersToLoad4), res3.users);
                    }
                    if (!chatsToLoad.isEmpty()) {
                        getMessagesStorage().getChatsInternal(TextUtils.join(",", chatsToLoad), res3.chats);
                    }
                    if (res3.messages.size() > count) {
                        try {
                            res3.messages.remove(res3.messages.size() - 1);
                            topReached2 = false;
                        } catch (Exception e5) {
                            e = e5;
                            topReached = false;
                            res = res3;
                            res.messages.clear();
                            res.chats.clear();
                            res.users.clear();
                            FileLog.e(e);
                            processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                            return;
                        } catch (Throwable th6) {
                            th = th6;
                            topReached = false;
                            res = res3;
                            processLoadedMedia(res, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached);
                            throw th;
                        }
                    } else {
                        boolean topReached3 = isEnd;
                        topReached2 = topReached3;
                    }
                    processLoadedMedia(res3, uid, count, max_id, type, fromCache, classGuid, isChannel, topReached2);
                } catch (Exception e6) {
                    e = e6;
                    res = res3;
                } catch (Throwable th7) {
                    th = th7;
                    res = res3;
                }
            } catch (Exception e7) {
                e = e7;
            } catch (Throwable th8) {
                th = th8;
            }
        } catch (Exception e8) {
            e = e8;
            topReached = false;
            res = res4;
        } catch (Throwable th9) {
            th = th9;
            topReached = false;
            res = res4;
        }
    }

    private void putMediaDatabase(final long uid, final int type, final ArrayList<TLRPC.Message> messages, final int max_id, final boolean topReached) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda122
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m434x43d27314(messages, topReached, uid, max_id, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:62:0x0117  */
    /* JADX WARN: Removed duplicated region for block: B:64:0x011c  */
    /* JADX WARN: Removed duplicated region for block: B:68:0x0125  */
    /* JADX WARN: Removed duplicated region for block: B:70:0x012a  */
    /* renamed from: lambda$putMediaDatabase$66$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m434x43d27314(ArrayList messages, boolean topReached, long uid, int max_id, int type) {
        int minId;
        NativeByteBuffer data = null;
        SQLitePreparedStatement state = null;
        try {
            try {
                if (messages.isEmpty() || topReached) {
                    getMessagesStorage().doneHolesInMedia(uid, max_id, type);
                    if (messages.isEmpty()) {
                        if (0 != 0) {
                            data.reuse();
                        }
                        if (0 != 0) {
                            state.dispose();
                            return;
                        }
                        return;
                    }
                }
                try {
                    getMessagesStorage().getDatabase().beginTransaction();
                } catch (Exception e) {
                    FileLog.e("putMediaDatabase ---> exception 1 ", e);
                }
                state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO media_v2 VALUES(?, ?, ?, ?, ?)");
                Iterator it = messages.iterator();
                NativeByteBuffer data2 = null;
                while (it.hasNext()) {
                    try {
                        TLRPC.Message message = (TLRPC.Message) it.next();
                        if (canAddMessageToMedia(message)) {
                            long messageId = message.id;
                            if (message.to_id.channel_id != 0) {
                                messageId |= message.to_id.channel_id << 32;
                            }
                            state.requery();
                            NativeByteBuffer data3 = new NativeByteBuffer(message.getObjectSize());
                            try {
                                message.serializeToStream(data3);
                                state.bindLong(1, messageId);
                                state.bindLong(2, uid);
                                state.bindInteger(3, message.date);
                                state.bindInteger(4, type);
                                state.bindByteBuffer(5, data3);
                                state.step();
                                data3.reuse();
                                data2 = null;
                            } catch (Exception e2) {
                                e = e2;
                                data = data3;
                                FileLog.e("putMediaDatabase ---> exception 2 ", e);
                                if (data != null) {
                                    data.reuse();
                                }
                                if (state != null) {
                                    state.dispose();
                                }
                                return;
                            } catch (Throwable th) {
                                th = th;
                                data = data3;
                                if (data != null) {
                                    data.reuse();
                                }
                                if (state != null) {
                                    state.dispose();
                                }
                                throw th;
                            }
                        }
                    } catch (Exception e3) {
                        e = e3;
                        data = data2;
                    } catch (Throwable th2) {
                        th = th2;
                        data = data2;
                    }
                }
                state.dispose();
                SQLitePreparedStatement state2 = null;
                if (!topReached || max_id != 0) {
                    if (topReached) {
                        minId = 1;
                    } else {
                        try {
                            minId = ((TLRPC.Message) messages.get(messages.size() - 1)).id;
                        } catch (Exception e4) {
                            e = e4;
                            data = data2;
                            state = null;
                            FileLog.e("putMediaDatabase ---> exception 2 ", e);
                            if (data != null) {
                            }
                            if (state != null) {
                            }
                            return;
                        } catch (Throwable th3) {
                            th = th3;
                            data = data2;
                            state = null;
                            if (data != null) {
                            }
                            if (state != null) {
                            }
                            throw th;
                        }
                    }
                    if (max_id != 0) {
                        getMessagesStorage().closeHolesInMedia(uid, minId, max_id, type);
                    } else {
                        getMessagesStorage().closeHolesInMedia(uid, minId, Integer.MAX_VALUE, type);
                    }
                }
                getMessagesStorage().getDatabase().commitTransaction();
                if (data2 != null) {
                    data2.reuse();
                }
                if (0 != 0) {
                    state2.dispose();
                }
            } catch (Exception e5) {
                e = e5;
            }
        } catch (Throwable th4) {
            th = th4;
        }
    }

    public void loadMusic(final long uid, final long max_id) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda14
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m394lambda$loadMusic$68$imuecahxyvjdmessengerMediaDataController(uid, max_id);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadMusic$68$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m394lambda$loadMusic$68$imuecahxyvjdmessengerMediaDataController(final long uid, long max_id) {
        SQLiteCursor cursor;
        final ArrayList<MessageObject> arrayList = new ArrayList<>();
        int lower_id = (int) uid;
        try {
            if (lower_id != 0) {
                cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT data, mid FROM media_v2 WHERE uid = %d AND mid < %d AND type = %d ORDER BY date DESC, mid DESC LIMIT 1000", Long.valueOf(uid), Long.valueOf(max_id), 4), new Object[0]);
            } else {
                cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT data, mid FROM media_v2 WHERE uid = %d AND mid > %d AND type = %d ORDER BY date DESC, mid DESC LIMIT 1000", Long.valueOf(uid), Long.valueOf(max_id), 4), new Object[0]);
            }
            while (cursor.next()) {
                NativeByteBuffer data = cursor.byteBufferValue(0);
                if (data != null) {
                    TLRPC.Message message = TLRPC.Message.TLdeserialize(data, data.readInt32(false), false);
                    message.readAttachPath(data, getUserConfig().clientUserId);
                    data.reuse();
                    if (MessageObject.isMusicMessage(message)) {
                        message.id = cursor.intValue(1);
                        message.dialog_id = uid;
                        arrayList.add(0, new MessageObject(this.currentAccount, message, false));
                    }
                }
            }
            cursor.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda36
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m393lambda$loadMusic$67$imuecahxyvjdmessengerMediaDataController(uid, arrayList);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadMusic$67$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m393lambda$loadMusic$67$imuecahxyvjdmessengerMediaDataController(long uid, ArrayList arrayList) {
        getNotificationCenter().postNotificationName(NotificationCenter.musicDidLoad, Long.valueOf(uid), arrayList);
    }

    public void buildShortcuts() {
        if (Build.VERSION.SDK_INT < 25) {
            return;
        }
        final ArrayList<TLRPC.TL_topPeer> hintsFinal = new ArrayList<>();
        for (int a2 = 0; a2 < this.hints.size(); a2++) {
            hintsFinal.add(this.hints.get(a2));
            if (hintsFinal.size() == 3) {
                break;
            }
        }
        Utilities.globalQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m345x2a928408(hintsFinal);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:101:0x02dc  */
    /* JADX WARN: Removed duplicated region for block: B:102:0x02e1  */
    /* JADX WARN: Removed duplicated region for block: B:105:0x02f8 A[Catch: all -> 0x033a, TryCatch #3 {all -> 0x033a, blocks: (B:3:0x0004, B:6:0x0027, B:8:0x002d, B:9:0x0031, B:11:0x0037, B:13:0x0043, B:17:0x005b, B:14:0x0049, B:16:0x0055, B:19:0x0074, B:21:0x007a, B:23:0x008a, B:24:0x008d, B:25:0x0094, B:27:0x009a, B:30:0x00a1, B:32:0x00ec, B:34:0x00f3, B:36:0x00fc, B:38:0x0101, B:40:0x0107, B:42:0x0120, B:48:0x0167, B:54:0x017c, B:56:0x018c, B:62:0x01ad, B:99:0x02c5, B:103:0x02e3, B:105:0x02f8, B:107:0x030c, B:109:0x0319, B:111:0x0320, B:110:0x031d, B:106:0x0300, B:97:0x02b9, B:58:0x0198, B:60:0x01a2, B:43:0x0141, B:45:0x0147, B:46:0x014c, B:33:0x00f0), top: B:122:0x0004 }] */
    /* JADX WARN: Removed duplicated region for block: B:106:0x0300 A[Catch: all -> 0x033a, TryCatch #3 {all -> 0x033a, blocks: (B:3:0x0004, B:6:0x0027, B:8:0x002d, B:9:0x0031, B:11:0x0037, B:13:0x0043, B:17:0x005b, B:14:0x0049, B:16:0x0055, B:19:0x0074, B:21:0x007a, B:23:0x008a, B:24:0x008d, B:25:0x0094, B:27:0x009a, B:30:0x00a1, B:32:0x00ec, B:34:0x00f3, B:36:0x00fc, B:38:0x0101, B:40:0x0107, B:42:0x0120, B:48:0x0167, B:54:0x017c, B:56:0x018c, B:62:0x01ad, B:99:0x02c5, B:103:0x02e3, B:105:0x02f8, B:107:0x030c, B:109:0x0319, B:111:0x0320, B:110:0x031d, B:106:0x0300, B:97:0x02b9, B:58:0x0198, B:60:0x01a2, B:43:0x0141, B:45:0x0147, B:46:0x014c, B:33:0x00f0), top: B:122:0x0004 }] */
    /* JADX WARN: Removed duplicated region for block: B:109:0x0319 A[Catch: all -> 0x033a, TryCatch #3 {all -> 0x033a, blocks: (B:3:0x0004, B:6:0x0027, B:8:0x002d, B:9:0x0031, B:11:0x0037, B:13:0x0043, B:17:0x005b, B:14:0x0049, B:16:0x0055, B:19:0x0074, B:21:0x007a, B:23:0x008a, B:24:0x008d, B:25:0x0094, B:27:0x009a, B:30:0x00a1, B:32:0x00ec, B:34:0x00f3, B:36:0x00fc, B:38:0x0101, B:40:0x0107, B:42:0x0120, B:48:0x0167, B:54:0x017c, B:56:0x018c, B:62:0x01ad, B:99:0x02c5, B:103:0x02e3, B:105:0x02f8, B:107:0x030c, B:109:0x0319, B:111:0x0320, B:110:0x031d, B:106:0x0300, B:97:0x02b9, B:58:0x0198, B:60:0x01a2, B:43:0x0141, B:45:0x0147, B:46:0x014c, B:33:0x00f0), top: B:122:0x0004 }] */
    /* JADX WARN: Removed duplicated region for block: B:110:0x031d A[Catch: all -> 0x033a, TryCatch #3 {all -> 0x033a, blocks: (B:3:0x0004, B:6:0x0027, B:8:0x002d, B:9:0x0031, B:11:0x0037, B:13:0x0043, B:17:0x005b, B:14:0x0049, B:16:0x0055, B:19:0x0074, B:21:0x007a, B:23:0x008a, B:24:0x008d, B:25:0x0094, B:27:0x009a, B:30:0x00a1, B:32:0x00ec, B:34:0x00f3, B:36:0x00fc, B:38:0x0101, B:40:0x0107, B:42:0x0120, B:48:0x0167, B:54:0x017c, B:56:0x018c, B:62:0x01ad, B:99:0x02c5, B:103:0x02e3, B:105:0x02f8, B:107:0x030c, B:109:0x0319, B:111:0x0320, B:110:0x031d, B:106:0x0300, B:97:0x02b9, B:58:0x0198, B:60:0x01a2, B:43:0x0141, B:45:0x0147, B:46:0x014c, B:33:0x00f0), top: B:122:0x0004 }] */
    /* renamed from: lambda$buildShortcuts$69$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m345x2a928408(ArrayList hintsFinal) {
        long did;
        TLRPC.User user;
        List<ShortcutInfo> currentShortcuts;
        ArrayList<String> newShortcutsIds;
        ArrayList<String> shortcutsToDelete;
        Intent intent;
        TLRPC.FileLocation photo;
        String name;
        String id;
        String name2;
        Path path;
        long did2;
        ArrayList arrayList = hintsFinal;
        try {
            ShortcutManager shortcutManager = (ShortcutManager) ApplicationLoader.applicationContext.getSystemService(ShortcutManager.class);
            List<ShortcutInfo> currentShortcuts2 = shortcutManager.getDynamicShortcuts();
            ArrayList<String> shortcutsToUpdate = new ArrayList<>();
            ArrayList<String> newShortcutsIds2 = new ArrayList<>();
            ArrayList<String> shortcutsToDelete2 = new ArrayList<>();
            if (currentShortcuts2 != null && !currentShortcuts2.isEmpty()) {
                newShortcutsIds2.add("compose");
                for (int a2 = 0; a2 < hintsFinal.size(); a2++) {
                    TLRPC.TL_topPeer hint = (TLRPC.TL_topPeer) arrayList.get(a2);
                    if (hint.peer.user_id != 0) {
                        did2 = hint.peer.user_id;
                    } else {
                        did2 = -hint.peer.chat_id;
                        if (did2 == 0) {
                            did2 = -hint.peer.channel_id;
                        }
                    }
                    newShortcutsIds2.add("did" + did2);
                }
                for (int a3 = 0; a3 < currentShortcuts2.size(); a3++) {
                    String id2 = currentShortcuts2.get(a3).getId();
                    if (!newShortcutsIds2.remove(id2)) {
                        shortcutsToDelete2.add(id2);
                    }
                    shortcutsToUpdate.add(id2);
                }
                if (newShortcutsIds2.isEmpty() && shortcutsToDelete2.isEmpty()) {
                    return;
                }
            }
            Intent intent2 = new Intent(ApplicationLoader.applicationContext, LaunchActivity.class);
            intent2.setAction("new_dialog");
            ArrayList<ShortcutInfo> arrayList2 = new ArrayList<>();
            arrayList2.add(new ShortcutInfo.Builder(ApplicationLoader.applicationContext, "compose").setShortLabel(LocaleController.getString("NewConversationShortcut", R.string.NewConversationShortcut)).setLongLabel(LocaleController.getString("NewConversationShortcut", R.string.NewConversationShortcut)).setIcon(Icon.createWithResource(ApplicationLoader.applicationContext, (int) R.drawable.shortcut_compose)).setIntent(intent2).build());
            if (shortcutsToUpdate.contains("compose")) {
                shortcutManager.updateShortcuts(arrayList2);
            } else {
                shortcutManager.addDynamicShortcuts(arrayList2);
            }
            arrayList2.clear();
            if (!shortcutsToDelete2.isEmpty()) {
                shortcutManager.removeDynamicShortcuts(shortcutsToDelete2);
            }
            int a4 = 0;
            while (a4 < hintsFinal.size()) {
                Intent shortcutIntent = new Intent(ApplicationLoader.applicationContext, OpenChatReceiver.class);
                TLRPC.TL_topPeer hint2 = (TLRPC.TL_topPeer) arrayList.get(a4);
                TLRPC.Chat chat = null;
                if (hint2.peer.user_id != 0) {
                    shortcutIntent.putExtra("userId", hint2.peer.user_id);
                    TLRPC.User user2 = getMessagesController().getUser(Integer.valueOf(hint2.peer.user_id));
                    did = hint2.peer.user_id;
                    user = user2;
                } else {
                    int chat_id = hint2.peer.chat_id;
                    if (chat_id == 0) {
                        chat_id = hint2.peer.channel_id;
                    }
                    chat = getMessagesController().getChat(Integer.valueOf(chat_id));
                    shortcutIntent.putExtra("chatId", chat_id);
                    did = -chat_id;
                    user = null;
                }
                if ((user == null || UserObject.isDeleted(user)) && chat == null) {
                    currentShortcuts = currentShortcuts2;
                    newShortcutsIds = newShortcutsIds2;
                    shortcutsToDelete = shortcutsToDelete2;
                    intent = intent2;
                } else {
                    if (user != null) {
                        currentShortcuts = currentShortcuts2;
                        String name3 = ContactsController.formatName(user.first_name, user.last_name);
                        if (user.photo == null) {
                            photo = null;
                            name = name3;
                        } else {
                            photo = user.photo.photo_small;
                            name = name3;
                        }
                    } else {
                        currentShortcuts = currentShortcuts2;
                        String name4 = chat.title;
                        if (chat.photo == null) {
                            photo = null;
                            name = name4;
                        } else {
                            photo = chat.photo.photo_small;
                            name = name4;
                        }
                    }
                    newShortcutsIds = newShortcutsIds2;
                    shortcutIntent.putExtra("currentAccount", this.currentAccount);
                    shortcutIntent.setAction("com.tmessages.openchat" + did);
                    shortcutIntent.addFlags(67108864);
                    Bitmap bitmap = null;
                    if (photo != null) {
                        try {
                            File path2 = FileLoader.getPathToAttach(photo, true);
                            bitmap = BitmapFactory.decodeFile(path2.toString());
                            if (bitmap != null) {
                                int size = AndroidUtilities.dp(48.0f);
                                Bitmap result = Bitmap.createBitmap(size, size, Bitmap.Config.ARGB_8888);
                                try {
                                    Canvas canvas = new Canvas(result);
                                    if (roundPaint == null) {
                                        shortcutsToDelete = shortcutsToDelete2;
                                        try {
                                            roundPaint = new Paint(3);
                                            bitmapRect = new RectF();
                                            Paint paint = new Paint(1);
                                            erasePaint = paint;
                                            intent = intent2;
                                            try {
                                                paint.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.CLEAR));
                                                path = new Path();
                                                roundPath = path;
                                            } catch (Throwable th) {
                                                e = th;
                                                FileLog.e(e);
                                                id = "did" + did;
                                                if (!TextUtils.isEmpty(name)) {
                                                }
                                                ShortcutInfo.Builder builder = new ShortcutInfo.Builder(ApplicationLoader.applicationContext, id).setShortLabel(name2).setLongLabel(name2).setIntent(shortcutIntent);
                                                if (bitmap != null) {
                                                }
                                                arrayList2.add(builder.build());
                                                if (shortcutsToUpdate.contains(id)) {
                                                }
                                                arrayList2.clear();
                                                a4++;
                                                arrayList = hintsFinal;
                                                currentShortcuts2 = currentShortcuts;
                                                newShortcutsIds2 = newShortcutsIds;
                                                intent2 = intent;
                                                shortcutsToDelete2 = shortcutsToDelete;
                                            }
                                        } catch (Throwable th2) {
                                            e = th2;
                                            intent = intent2;
                                        }
                                        try {
                                            path.addCircle(size / 2, size / 2, (size / 2) - AndroidUtilities.dp(2.0f), Path.Direction.CW);
                                            roundPath.toggleInverseFillType();
                                        } catch (Throwable th3) {
                                            e = th3;
                                            FileLog.e(e);
                                            id = "did" + did;
                                            if (!TextUtils.isEmpty(name)) {
                                            }
                                            ShortcutInfo.Builder builder2 = new ShortcutInfo.Builder(ApplicationLoader.applicationContext, id).setShortLabel(name2).setLongLabel(name2).setIntent(shortcutIntent);
                                            if (bitmap != null) {
                                            }
                                            arrayList2.add(builder2.build());
                                            if (shortcutsToUpdate.contains(id)) {
                                            }
                                            arrayList2.clear();
                                            a4++;
                                            arrayList = hintsFinal;
                                            currentShortcuts2 = currentShortcuts;
                                            newShortcutsIds2 = newShortcutsIds;
                                            intent2 = intent;
                                            shortcutsToDelete2 = shortcutsToDelete;
                                        }
                                    } else {
                                        shortcutsToDelete = shortcutsToDelete2;
                                        intent = intent2;
                                    }
                                    bitmapRect.set(AndroidUtilities.dp(2.0f), AndroidUtilities.dp(2.0f), AndroidUtilities.dp(46.0f), AndroidUtilities.dp(46.0f));
                                    canvas.drawBitmap(bitmap, (Rect) null, bitmapRect, roundPaint);
                                    canvas.drawPath(roundPath, erasePaint);
                                    try {
                                        canvas.setBitmap(null);
                                    } catch (Exception e) {
                                    }
                                    bitmap = result;
                                } catch (Throwable th4) {
                                    e = th4;
                                    shortcutsToDelete = shortcutsToDelete2;
                                    intent = intent2;
                                }
                            } else {
                                shortcutsToDelete = shortcutsToDelete2;
                                intent = intent2;
                            }
                        } catch (Throwable th5) {
                            e = th5;
                            shortcutsToDelete = shortcutsToDelete2;
                            intent = intent2;
                        }
                    } else {
                        shortcutsToDelete = shortcutsToDelete2;
                        intent = intent2;
                    }
                    id = "did" + did;
                    if (!TextUtils.isEmpty(name)) {
                        name2 = name;
                    } else {
                        name2 = " ";
                    }
                    ShortcutInfo.Builder builder22 = new ShortcutInfo.Builder(ApplicationLoader.applicationContext, id).setShortLabel(name2).setLongLabel(name2).setIntent(shortcutIntent);
                    if (bitmap != null) {
                        builder22.setIcon(Icon.createWithBitmap(bitmap));
                    } else {
                        builder22.setIcon(Icon.createWithResource(ApplicationLoader.applicationContext, (int) R.drawable.shortcut_user));
                    }
                    arrayList2.add(builder22.build());
                    if (shortcutsToUpdate.contains(id)) {
                        shortcutManager.updateShortcuts(arrayList2);
                    } else {
                        shortcutManager.addDynamicShortcuts(arrayList2);
                    }
                    arrayList2.clear();
                }
                a4++;
                arrayList = hintsFinal;
                currentShortcuts2 = currentShortcuts;
                newShortcutsIds2 = newShortcutsIds;
                intent2 = intent;
                shortcutsToDelete2 = shortcutsToDelete;
            }
        } catch (Throwable th6) {
        }
    }

    public void loadHints(boolean cache) {
        if (this.loading || !getUserConfig().suggestContacts) {
            return;
        }
        if (cache) {
            if (this.loaded) {
                return;
            }
            this.loading = true;
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda110
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m385lambda$loadHints$71$imuecahxyvjdmessengerMediaDataController();
                }
            });
            this.loaded = true;
            return;
        }
        this.loading = true;
        TLRPC.TL_contacts_getTopPeers req = new TLRPC.TL_contacts_getTopPeers();
        req.hash = 0;
        req.bots_pm = false;
        req.correspondents = true;
        req.groups = false;
        req.channels = false;
        req.bots_inline = true;
        req.offset = 0;
        req.limit = 20;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda111
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m390lambda$loadHints$76$imuecahxyvjdmessengerMediaDataController(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$71$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m385lambda$loadHints$71$imuecahxyvjdmessengerMediaDataController() {
        final ArrayList<TLRPC.TL_topPeer> hintsNew = new ArrayList<>();
        final ArrayList<TLRPC.TL_topPeer> inlineBotsNew = new ArrayList<>();
        final ArrayList<TLRPC.User> users = new ArrayList<>();
        final ArrayList<TLRPC.Chat> chats = new ArrayList<>();
        int selfUserId = getUserConfig().getClientUserId();
        try {
            ArrayList<Integer> usersToLoad = new ArrayList<>();
            ArrayList<Integer> chatsToLoad = new ArrayList<>();
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT did, type, rating FROM chat_hints WHERE 1 ORDER BY rating DESC", new Object[0]);
            while (cursor.next()) {
                int did = cursor.intValue(0);
                if (did != selfUserId) {
                    int type = cursor.intValue(1);
                    TLRPC.TL_topPeer peer = new TLRPC.TL_topPeer();
                    peer.rating = cursor.doubleValue(2);
                    if (did > 0) {
                        peer.peer = new TLRPC.TL_peerUser();
                        peer.peer.user_id = did;
                        usersToLoad.add(Integer.valueOf(did));
                    } else {
                        peer.peer = new TLRPC.TL_peerChat();
                        peer.peer.chat_id = -did;
                        chatsToLoad.add(Integer.valueOf(-did));
                    }
                    if (type == 0) {
                        hintsNew.add(peer);
                    } else if (type == 1) {
                        inlineBotsNew.add(peer);
                    }
                }
            }
            cursor.dispose();
            if (!usersToLoad.isEmpty()) {
                getMessagesStorage().getUsersInternal(TextUtils.join(",", usersToLoad), users);
            }
            if (!chatsToLoad.isEmpty()) {
                getMessagesStorage().getChatsInternal(TextUtils.join(",", chatsToLoad), chats);
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda35
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m384lambda$loadHints$70$imuecahxyvjdmessengerMediaDataController(users, chats, hintsNew, inlineBotsNew);
                }
            });
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$70$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m384lambda$loadHints$70$imuecahxyvjdmessengerMediaDataController(ArrayList users, ArrayList chats, ArrayList hintsNew, ArrayList inlineBotsNew) {
        getMessagesController().putUsers(users, true);
        getMessagesController().putChats(chats, true);
        this.loading = false;
        this.loaded = true;
        this.hints = hintsNew;
        this.inlineBots = inlineBotsNew;
        buildShortcuts();
        getNotificationCenter().postNotificationName(NotificationCenter.reloadHints, new Object[0]);
        getNotificationCenter().postNotificationName(NotificationCenter.reloadInlineHints, new Object[0]);
        if (Math.abs(getUserConfig().lastHintsSyncTime - ((int) (System.currentTimeMillis() / 1000))) >= 86400) {
            loadHints(false);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$76$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m390lambda$loadHints$76$imuecahxyvjdmessengerMediaDataController(final TLObject response, TLRPC.TL_error error) {
        if (response instanceof TLRPC.TL_contacts_topPeers) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda84
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m388lambda$loadHints$74$imuecahxyvjdmessengerMediaDataController(response);
                }
            });
        } else if (response instanceof TLRPC.TL_contacts_topPeersDisabled) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda85
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m389lambda$loadHints$75$imuecahxyvjdmessengerMediaDataController();
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$74$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m388lambda$loadHints$74$imuecahxyvjdmessengerMediaDataController(TLObject response) {
        final TLRPC.TL_contacts_topPeers topPeers = (TLRPC.TL_contacts_topPeers) response;
        getMessagesController().putUsers(topPeers.users, false);
        getMessagesController().putChats(topPeers.chats, false);
        for (int a2 = 0; a2 < topPeers.categories.size(); a2++) {
            TLRPC.TL_topPeerCategoryPeers category = topPeers.categories.get(a2);
            if (category.category instanceof TLRPC.TL_topPeerCategoryBotsInline) {
                this.inlineBots = category.peers;
                getUserConfig().botRatingLoadTime = (int) (System.currentTimeMillis() / 1000);
            } else {
                this.hints = category.peers;
                int selfUserId = getUserConfig().getClientUserId();
                int b = 0;
                while (true) {
                    if (b >= this.hints.size()) {
                        break;
                    }
                    TLRPC.TL_topPeer topPeer = this.hints.get(b);
                    if (topPeer.peer.user_id != selfUserId) {
                        b++;
                    } else {
                        this.hints.remove(b);
                        break;
                    }
                }
                getUserConfig().ratingLoadTime = (int) (System.currentTimeMillis() / 1000);
            }
        }
        getUserConfig().saveConfig(false);
        buildShortcuts();
        getNotificationCenter().postNotificationName(NotificationCenter.reloadHints, new Object[0]);
        getNotificationCenter().postNotificationName(NotificationCenter.reloadInlineHints, new Object[0]);
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda87
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m387lambda$loadHints$73$imuecahxyvjdmessengerMediaDataController(topPeers);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$73$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m387lambda$loadHints$73$imuecahxyvjdmessengerMediaDataController(TLRPC.TL_contacts_topPeers topPeers) {
        SQLitePreparedStatement state = null;
        try {
            try {
                getMessagesStorage().getDatabase().executeFast("DELETE FROM chat_hints WHERE 1").stepThis().dispose();
                try {
                    getMessagesStorage().getDatabase().beginTransaction();
                } catch (Exception e) {
                    FileLog.e("loadHints ---> exception 1 ", e);
                }
                getMessagesStorage().putUsersAndChats(topPeers.users, topPeers.chats, false, true);
                SQLitePreparedStatement state2 = getMessagesStorage().getDatabase().executeFast("REPLACE INTO chat_hints VALUES(?, ?, ?, ?)");
                for (int a2 = 0; a2 < topPeers.categories.size(); a2++) {
                    TLRPC.TL_topPeerCategoryPeers category = topPeers.categories.get(a2);
                    int type = category.category instanceof TLRPC.TL_topPeerCategoryBotsInline ? 1 : 0;
                    for (int b = 0; b < category.peers.size(); b++) {
                        TLRPC.TL_topPeer peer = category.peers.get(b);
                        int did = peer.peer instanceof TLRPC.TL_peerUser ? peer.peer.user_id : peer.peer instanceof TLRPC.TL_peerChat ? -peer.peer.chat_id : -peer.peer.channel_id;
                        state2.requery();
                        state2.bindInteger(1, did);
                        state2.bindInteger(2, type);
                        state2.bindDouble(3, peer.rating);
                        state2.bindInteger(4, 0);
                        state2.step();
                    }
                }
                state2.dispose();
                state = null;
                getMessagesStorage().getDatabase().commitTransaction();
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda71
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m386lambda$loadHints$72$imuecahxyvjdmessengerMediaDataController();
                    }
                });
                if (0 == 0) {
                    return;
                }
            } catch (Exception e2) {
                FileLog.e("loadHints ---> exception 2 ", e2);
                if (state == null) {
                    return;
                }
            }
            state.dispose();
        } catch (Throwable th) {
            if (state != null) {
                state.dispose();
            }
            throw th;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$72$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m386lambda$loadHints$72$imuecahxyvjdmessengerMediaDataController() {
        getUserConfig().suggestContacts = true;
        getUserConfig().lastHintsSyncTime = (int) (System.currentTimeMillis() / 1000);
        getUserConfig().saveConfig(false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadHints$75$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m389lambda$loadHints$75$imuecahxyvjdmessengerMediaDataController() {
        getUserConfig().suggestContacts = false;
        getUserConfig().lastHintsSyncTime = (int) (System.currentTimeMillis() / 1000);
        getUserConfig().saveConfig(false);
        clearTopPeers();
    }

    public void clearTopPeers() {
        this.hints.clear();
        this.inlineBots.clear();
        getNotificationCenter().postNotificationName(NotificationCenter.reloadHints, new Object[0]);
        getNotificationCenter().postNotificationName(NotificationCenter.reloadInlineHints, new Object[0]);
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda49
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m347xcd22b4f5();
            }
        });
        buildShortcuts();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$clearTopPeers$77$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m347xcd22b4f5() {
        try {
            getMessagesStorage().getDatabase().executeFast("DELETE FROM chat_hints WHERE 1").stepThis().dispose();
        } catch (Exception e) {
        }
    }

    public void increaseInlineRaiting(int uid) {
        int dt;
        if (!getUserConfig().suggestContacts) {
            return;
        }
        if (getUserConfig().botRatingLoadTime != 0) {
            dt = Math.max(1, ((int) (System.currentTimeMillis() / 1000)) - getUserConfig().botRatingLoadTime);
        } else {
            dt = 60;
        }
        TLRPC.TL_topPeer peer = null;
        int a2 = 0;
        while (true) {
            if (a2 >= this.inlineBots.size()) {
                break;
            }
            TLRPC.TL_topPeer p = this.inlineBots.get(a2);
            if (p.peer.user_id != uid) {
                a2++;
            } else {
                peer = p;
                break;
            }
        }
        if (peer == null) {
            peer = new TLRPC.TL_topPeer();
            peer.peer = new TLRPC.TL_peerUser();
            peer.peer.user_id = uid;
            this.inlineBots.add(peer);
        }
        peer.rating += Math.exp(dt / getMessagesController().ratingDecay);
        Collections.sort(this.inlineBots, new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda68
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return MediaDataController.lambda$increaseInlineRaiting$78((TLRPC.TL_topPeer) obj, (TLRPC.TL_topPeer) obj2);
            }
        });
        if (this.inlineBots.size() > 20) {
            ArrayList<TLRPC.TL_topPeer> arrayList = this.inlineBots;
            arrayList.remove(arrayList.size() - 1);
        }
        savePeer(uid, 1, peer.rating);
        getNotificationCenter().postNotificationName(NotificationCenter.reloadInlineHints, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$increaseInlineRaiting$78(TLRPC.TL_topPeer lhs, TLRPC.TL_topPeer rhs) {
        if (lhs.rating > rhs.rating) {
            return -1;
        }
        if (lhs.rating < rhs.rating) {
            return 1;
        }
        return 0;
    }

    public void removeInline(int uid) {
        for (int a2 = 0; a2 < this.inlineBots.size(); a2++) {
            if (this.inlineBots.get(a2).peer.user_id == uid) {
                this.inlineBots.remove(a2);
                TLRPC.TL_contacts_resetTopPeerRating req = new TLRPC.TL_contacts_resetTopPeerRating();
                req.category = new TLRPC.TL_topPeerCategoryBotsInline();
                req.peer = getMessagesController().getInputPeer(uid);
                getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda10
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.lambda$removeInline$79(tLObject, tL_error);
                    }
                });
                deletePeer(uid, 1);
                getNotificationCenter().postNotificationName(NotificationCenter.reloadInlineHints, new Object[0]);
                return;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$removeInline$79(TLObject response, TLRPC.TL_error error) {
    }

    public void removePeer(int uid) {
        for (int a2 = 0; a2 < this.hints.size(); a2++) {
            if (this.hints.get(a2).peer.user_id == uid) {
                this.hints.remove(a2);
                getNotificationCenter().postNotificationName(NotificationCenter.reloadHints, new Object[0]);
                TLRPC.TL_contacts_resetTopPeerRating req = new TLRPC.TL_contacts_resetTopPeerRating();
                req.category = new TLRPC.TL_topPeerCategoryCorrespondents();
                req.peer = getMessagesController().getInputPeer(uid);
                deletePeer(uid, 0);
                getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda103
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.lambda$removePeer$80(tLObject, tL_error);
                    }
                });
                return;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$removePeer$80(TLObject response, TLRPC.TL_error error) {
    }

    public void increasePeerRaiting(final long did) {
        final int lower_id;
        if (!getUserConfig().suggestContacts || (lower_id = (int) did) <= 0) {
            return;
        }
        TLRPC.User user = lower_id > 0 ? getMessagesController().getUser(Integer.valueOf(lower_id)) : null;
        if (user == null || user.bot || user.self) {
            return;
        }
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda9
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m366xdd822b35(did, lower_id);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$increasePeerRaiting$83$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m366xdd822b35(final long did, final int lower_id) {
        double dt = FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE;
        int lastTime = 0;
        int lastMid = 0;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT MAX(mid), MAX(date) FROM messages WHERE uid = %d AND out = 1", Long.valueOf(did)), new Object[0]);
            if (cursor.next()) {
                lastMid = cursor.intValue(0);
                lastTime = cursor.intValue(1);
            }
            cursor.dispose();
            if (lastMid > 0 && getUserConfig().ratingLoadTime != 0) {
                dt = lastTime - getUserConfig().ratingLoadTime;
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        final double dtFinal = dt;
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda55
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m365x42e168b4(lower_id, dtFinal, did);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$increasePeerRaiting$82$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m365x42e168b4(int lower_id, double dtFinal, long did) {
        TLRPC.TL_topPeer peer = null;
        for (int a2 = 0; a2 < this.hints.size(); a2++) {
            TLRPC.TL_topPeer p = this.hints.get(a2);
            if ((lower_id < 0 && (p.peer.chat_id == (-lower_id) || p.peer.channel_id == (-lower_id))) || (lower_id > 0 && p.peer.user_id == lower_id)) {
                peer = p;
                break;
            }
        }
        if (peer == null) {
            peer = new TLRPC.TL_topPeer();
            if (lower_id > 0) {
                peer.peer = new TLRPC.TL_peerUser();
                peer.peer.user_id = lower_id;
            } else {
                peer.peer = new TLRPC.TL_peerChat();
                peer.peer.chat_id = -lower_id;
            }
            this.hints.add(peer);
        }
        peer.rating += Math.exp(dtFinal / getMessagesController().ratingDecay);
        Collections.sort(this.hints, new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda82
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return MediaDataController.lambda$increasePeerRaiting$81((TLRPC.TL_topPeer) obj, (TLRPC.TL_topPeer) obj2);
            }
        });
        savePeer((int) did, 0, peer.rating);
        getNotificationCenter().postNotificationName(NotificationCenter.reloadHints, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$increasePeerRaiting$81(TLRPC.TL_topPeer lhs, TLRPC.TL_topPeer rhs) {
        if (lhs.rating > rhs.rating) {
            return -1;
        }
        if (lhs.rating < rhs.rating) {
            return 1;
        }
        return 0;
    }

    private void savePeer(final int did, final int type, final double rating) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda101
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m446lambda$savePeer$84$imuecahxyvjdmessengerMediaDataController(did, type, rating);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$savePeer$84$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m446lambda$savePeer$84$imuecahxyvjdmessengerMediaDataController(int did, int type, double rating) {
        try {
            SQLitePreparedStatement state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO chat_hints VALUES(?, ?, ?, ?)");
            state.requery();
            state.bindInteger(1, did);
            state.bindInteger(2, type);
            state.bindDouble(3, rating);
            state.bindInteger(4, ((int) System.currentTimeMillis()) / 1000);
            state.step();
            state.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    private void deletePeer(final int did, final int type) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda123
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m348lambda$deletePeer$85$imuecahxyvjdmessengerMediaDataController(did, type);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$deletePeer$85$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m348lambda$deletePeer$85$imuecahxyvjdmessengerMediaDataController(int did, int type) {
        try {
            getMessagesStorage().getDatabase().executeFast(String.format(Locale.US, "DELETE FROM chat_hints WHERE did = %d AND type = %d", Integer.valueOf(did), Integer.valueOf(type))).stepThis().dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    private Intent createIntrnalShortcutIntent(long did) {
        Intent shortcutIntent = new Intent(ApplicationLoader.applicationContext, OpenChatReceiver.class);
        int lower_id = (int) did;
        int high_id = (int) (did >> 32);
        if (lower_id == 0) {
            shortcutIntent.putExtra("encId", high_id);
            TLRPC.EncryptedChat encryptedChat = getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
            if (encryptedChat == null) {
                return null;
            }
        } else if (lower_id > 0) {
            shortcutIntent.putExtra("userId", lower_id);
        } else if (lower_id >= 0) {
            return null;
        } else {
            shortcutIntent.putExtra("chatId", -lower_id);
        }
        shortcutIntent.putExtra("currentAccount", this.currentAccount);
        shortcutIntent.setAction("com.tmessages.openchat" + did);
        shortcutIntent.addFlags(67108864);
        return shortcutIntent;
    }

    /* JADX WARN: Removed duplicated region for block: B:100:0x025a A[Catch: Exception -> 0x02be, TryCatch #1 {Exception -> 0x02be, blocks: (B:3:0x0002, B:5:0x0011, B:8:0x0020, B:18:0x005e, B:20:0x0064, B:82:0x01cf, B:84:0x01e1, B:86:0x0205, B:99:0x0246, B:88:0x020f, B:90:0x0213, B:91:0x021d, B:93:0x0229, B:95:0x022f, B:97:0x0233, B:98:0x023d, B:100:0x025a, B:102:0x0261, B:117:0x02a2, B:106:0x026b, B:108:0x026f, B:109:0x0279, B:111:0x0285, B:113:0x028b, B:115:0x028f, B:116:0x0299, B:81:0x01cc, B:21:0x0072, B:23:0x007e, B:25:0x008b, B:27:0x0091, B:10:0x0034, B:12:0x0046), top: B:125:0x0002 }] */
    /* JADX WARN: Removed duplicated region for block: B:84:0x01e1 A[Catch: Exception -> 0x02be, TryCatch #1 {Exception -> 0x02be, blocks: (B:3:0x0002, B:5:0x0011, B:8:0x0020, B:18:0x005e, B:20:0x0064, B:82:0x01cf, B:84:0x01e1, B:86:0x0205, B:99:0x0246, B:88:0x020f, B:90:0x0213, B:91:0x021d, B:93:0x0229, B:95:0x022f, B:97:0x0233, B:98:0x023d, B:100:0x025a, B:102:0x0261, B:117:0x02a2, B:106:0x026b, B:108:0x026f, B:109:0x0279, B:111:0x0285, B:113:0x028b, B:115:0x028f, B:116:0x0299, B:81:0x01cc, B:21:0x0072, B:23:0x007e, B:25:0x008b, B:27:0x0091, B:10:0x0034, B:12:0x0046), top: B:125:0x0002 }] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void installShortcut(long did) {
        TLRPC.Chat chat;
        TLRPC.User user;
        String name;
        boolean selfUser;
        TLRPC.FileLocation photo;
        try {
            Intent shortcutIntent = createIntrnalShortcutIntent(did);
            int lower_id = (int) did;
            int high_id = (int) (did >> 32);
            if (lower_id == 0) {
                TLRPC.EncryptedChat encryptedChat = getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
                if (encryptedChat == null) {
                    return;
                }
                TLRPC.User user2 = getMessagesController().getUser(Integer.valueOf(encryptedChat.user_id));
                chat = null;
                user = user2;
            } else if (lower_id > 0) {
                TLRPC.User user3 = getMessagesController().getUser(Integer.valueOf(lower_id));
                chat = null;
                user = user3;
            } else if (lower_id < 0) {
                chat = getMessagesController().getChat(Integer.valueOf(-lower_id));
                user = null;
            } else {
                return;
            }
            if (user == null && chat == null) {
                return;
            }
            if (user != null) {
                if (UserObject.isUserSelf(user)) {
                    name = LocaleController.getString("SavedMessages", R.string.SavedMessages);
                    selfUser = true;
                    photo = null;
                } else {
                    String name2 = ContactsController.formatName(user.first_name, user.last_name);
                    if (user.photo == null) {
                        name = name2;
                        selfUser = false;
                        photo = null;
                    } else {
                        TLRPC.FileLocation photo2 = user.photo.photo_small;
                        name = name2;
                        selfUser = false;
                        photo = photo2;
                    }
                }
            } else {
                String name3 = chat.title;
                if (chat.photo == null) {
                    name = name3;
                    selfUser = false;
                    photo = null;
                } else {
                    TLRPC.FileLocation photo3 = chat.photo.photo_small;
                    name = name3;
                    selfUser = false;
                    photo = photo3;
                }
            }
            Bitmap bitmap = null;
            if (selfUser || photo != null) {
                if (!selfUser) {
                    try {
                        File path = FileLoader.getPathToAttach(photo, true);
                        bitmap = BitmapFactory.decodeFile(path.toString());
                    } catch (Throwable th) {
                        e = th;
                        FileLog.e(e);
                        if (Build.VERSION.SDK_INT < 26) {
                        }
                    }
                }
                if (selfUser || bitmap != null) {
                    try {
                        int size = AndroidUtilities.dp(58.0f);
                        Bitmap result = Bitmap.createBitmap(size, size, Bitmap.Config.ARGB_8888);
                        result.eraseColor(0);
                        Canvas canvas = new Canvas(result);
                        try {
                            if (selfUser) {
                                try {
                                    AvatarDrawable avatarDrawable = new AvatarDrawable(user);
                                    avatarDrawable.setAvatarType(1);
                                    avatarDrawable.setBounds(0, 0, size, size);
                                    avatarDrawable.draw(canvas);
                                } catch (Throwable th2) {
                                    e = th2;
                                    FileLog.e(e);
                                    if (Build.VERSION.SDK_INT < 26) {
                                    }
                                }
                            } else {
                                try {
                                    BitmapShader shader = new BitmapShader(bitmap, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP);
                                    if (roundPaint == null) {
                                        roundPaint = new Paint(1);
                                        bitmapRect = new RectF();
                                    }
                                    float scale = size / bitmap.getWidth();
                                    canvas.save();
                                    canvas.scale(scale, scale);
                                    roundPaint.setShader(shader);
                                    try {
                                        bitmapRect.set(0.0f, 0.0f, bitmap.getWidth(), bitmap.getHeight());
                                        canvas.drawRoundRect(bitmapRect, bitmap.getWidth(), bitmap.getHeight(), roundPaint);
                                        canvas.restore();
                                    } catch (Throwable th3) {
                                        e = th3;
                                        FileLog.e(e);
                                        if (Build.VERSION.SDK_INT < 26) {
                                        }
                                    }
                                } catch (Throwable th4) {
                                    e = th4;
                                    FileLog.e(e);
                                    if (Build.VERSION.SDK_INT < 26) {
                                    }
                                }
                            }
                            Drawable drawable = ApplicationLoader.applicationContext.getResources().getDrawable(R.mipmap.ic_logo);
                            int w = AndroidUtilities.dp(15.0f);
                            int left = (size - w) - AndroidUtilities.dp(2.0f);
                            int top = (size - w) - AndroidUtilities.dp(2.0f);
                            try {
                                drawable.setBounds(left, top, left + w, top + w);
                                drawable.draw(canvas);
                                try {
                                    canvas.setBitmap(null);
                                } catch (Exception e) {
                                }
                                bitmap = result;
                            } catch (Throwable th5) {
                                e = th5;
                                FileLog.e(e);
                                if (Build.VERSION.SDK_INT < 26) {
                                }
                            }
                        } catch (Throwable th6) {
                            e = th6;
                        }
                    } catch (Throwable th7) {
                        e = th7;
                    }
                }
            }
            if (Build.VERSION.SDK_INT < 26) {
                ShortcutInfo.Builder pinShortcutInfo = new ShortcutInfo.Builder(ApplicationLoader.applicationContext, "sdid_" + did).setShortLabel(name).setIntent(shortcutIntent);
                if (bitmap != null) {
                    pinShortcutInfo.setIcon(Icon.createWithBitmap(bitmap));
                } else if (user != null) {
                    if (user.bot) {
                        pinShortcutInfo.setIcon(Icon.createWithResource(ApplicationLoader.applicationContext, (int) R.drawable.book_bot));
                    } else {
                        pinShortcutInfo.setIcon(Icon.createWithResource(ApplicationLoader.applicationContext, (int) R.drawable.book_user));
                    }
                } else if (chat != null) {
                    if (ChatObject.isChannel(chat) && !chat.megagroup) {
                        pinShortcutInfo.setIcon(Icon.createWithResource(ApplicationLoader.applicationContext, (int) R.drawable.book_channel));
                    } else {
                        pinShortcutInfo.setIcon(Icon.createWithResource(ApplicationLoader.applicationContext, (int) R.drawable.book_group));
                    }
                }
                ShortcutManager shortcutManager = (ShortcutManager) ApplicationLoader.applicationContext.getSystemService(ShortcutManager.class);
                shortcutManager.requestPinShortcut(pinShortcutInfo.build(), null);
                return;
            }
            Intent addIntent = new Intent();
            if (bitmap != null) {
                addIntent.putExtra("android.intent.extra.shortcut.ICON", bitmap);
            } else if (user != null) {
                if (user.bot) {
                    addIntent.putExtra("android.intent.extra.shortcut.ICON_RESOURCE", Intent.ShortcutIconResource.fromContext(ApplicationLoader.applicationContext, R.drawable.book_bot));
                } else {
                    addIntent.putExtra("android.intent.extra.shortcut.ICON_RESOURCE", Intent.ShortcutIconResource.fromContext(ApplicationLoader.applicationContext, R.drawable.book_user));
                }
            } else if (chat != null) {
                if (!ChatObject.isChannel(chat) || chat.megagroup) {
                    addIntent.putExtra("android.intent.extra.shortcut.ICON_RESOURCE", Intent.ShortcutIconResource.fromContext(ApplicationLoader.applicationContext, R.drawable.book_group));
                } else {
                    addIntent.putExtra("android.intent.extra.shortcut.ICON_RESOURCE", Intent.ShortcutIconResource.fromContext(ApplicationLoader.applicationContext, R.drawable.book_channel));
                }
            }
            addIntent.putExtra("android.intent.extra.shortcut.INTENT", shortcutIntent);
            addIntent.putExtra("android.intent.extra.shortcut.NAME", name);
            addIntent.putExtra("duplicate", false);
            addIntent.setAction("com.android.launcher.action.INSTALL_SHORTCUT");
            ApplicationLoader.applicationContext.sendBroadcast(addIntent);
        } catch (Exception e2) {
            FileLog.e(e2);
        }
    }

    public void uninstallShortcut(long did) {
        String name;
        try {
            if (Build.VERSION.SDK_INT >= 26) {
                ShortcutManager shortcutManager = (ShortcutManager) ApplicationLoader.applicationContext.getSystemService(ShortcutManager.class);
                ArrayList<String> arrayList = new ArrayList<>();
                arrayList.add("sdid_" + did);
                shortcutManager.removeDynamicShortcuts(arrayList);
                return;
            }
            int lower_id = (int) did;
            int high_id = (int) (did >> 32);
            TLRPC.User user = null;
            TLRPC.Chat chat = null;
            if (lower_id == 0) {
                TLRPC.EncryptedChat encryptedChat = getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
                if (encryptedChat == null) {
                    return;
                }
                user = getMessagesController().getUser(Integer.valueOf(encryptedChat.user_id));
            } else if (lower_id > 0) {
                user = getMessagesController().getUser(Integer.valueOf(lower_id));
            } else if (lower_id < 0) {
                chat = getMessagesController().getChat(Integer.valueOf(-lower_id));
            } else {
                return;
            }
            if (user == null && chat == null) {
                return;
            }
            if (user != null) {
                name = ContactsController.formatName(user.first_name, user.last_name);
            } else {
                name = chat.title;
            }
            Intent addIntent = new Intent();
            addIntent.putExtra("android.intent.extra.shortcut.INTENT", createIntrnalShortcutIntent(did));
            addIntent.putExtra("android.intent.extra.shortcut.NAME", name);
            addIntent.putExtra("duplicate", false);
            addIntent.setAction("com.android.launcher.action.UNINSTALL_SHORTCUT");
            ApplicationLoader.applicationContext.sendBroadcast(addIntent);
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$static$86(TLRPC.MessageEntity entity1, TLRPC.MessageEntity entity2) {
        if (entity1.offset > entity2.offset) {
            return 1;
        }
        if (entity1.offset < entity2.offset) {
            return -1;
        }
        return 0;
    }

    public MessageObject loadPinnedMessage(final long dialogId, final int channelId, final int mid, boolean useQueue) {
        if (useQueue) {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda50
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m395xc6f632b4(dialogId, channelId, mid);
                }
            });
            return null;
        }
        return loadPinnedMessageInternal(dialogId, channelId, mid, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadPinnedMessage$87$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m395xc6f632b4(long dialogId, int channelId, int mid) {
        loadPinnedMessageInternal(dialogId, channelId, mid, false);
    }

    private MessageObject loadPinnedMessageInternal(long dialogId, final int channelId, int mid, boolean returnValue) {
        long messageId;
        TLRPC.Message result;
        TLRPC.Message result2;
        ArrayList<Integer> usersToLoad;
        ArrayList<Integer> chatsToLoad;
        TLRPC.Message result3;
        NativeByteBuffer data;
        NativeByteBuffer data2;
        if (channelId != 0) {
            messageId = mid | (channelId << 32);
        } else {
            long messageId2 = mid;
            messageId = messageId2;
        }
        try {
            ArrayList<TLRPC.User> users = new ArrayList<>();
            ArrayList<TLRPC.Chat> chats = new ArrayList<>();
            ArrayList<Integer> usersToLoad2 = new ArrayList<>();
            ArrayList<Integer> chatsToLoad2 = new ArrayList<>();
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT data, mid, date FROM messages WHERE mid = %d", Long.valueOf(messageId)), new Object[0]);
            if (cursor.next() && (data2 = cursor.byteBufferValue(0)) != null) {
                result = TLRPC.Message.TLdeserialize(data2, data2.readInt32(false), false);
                result.readAttachPath(data2, getUserConfig().clientUserId);
                data2.reuse();
                if (result.action instanceof TLRPC.TL_messageActionHistoryClear) {
                    result = null;
                } else {
                    result.id = cursor.intValue(1);
                    result.date = cursor.intValue(2);
                    result.dialog_id = dialogId;
                    MessagesStorage.addUsersAndChatsFromMessage(result, usersToLoad2, chatsToLoad2);
                }
            } else {
                result = null;
            }
            cursor.dispose();
            if (result == null) {
                TLRPC.Message result4 = result;
                SQLiteCursor cursor2 = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT data FROM chat_pinned WHERE uid = %d", Long.valueOf(dialogId)), new Object[0]);
                if (cursor2.next() && (data = cursor2.byteBufferValue(0)) != null) {
                    TLRPC.Message result5 = TLRPC.Message.TLdeserialize(data, data.readInt32(false), false);
                    result5.readAttachPath(data, getUserConfig().clientUserId);
                    data.reuse();
                    if (result5.id == mid && !(result5.action instanceof TLRPC.TL_messageActionHistoryClear)) {
                        result5.dialog_id = dialogId;
                        MessagesStorage.addUsersAndChatsFromMessage(result5, usersToLoad2, chatsToLoad2);
                        result3 = result5;
                    }
                    result3 = null;
                } else {
                    result3 = result4;
                }
                cursor2.dispose();
                result2 = result3;
            } else {
                result2 = result;
            }
            if (result2 == null) {
                if (channelId != 0) {
                    TLRPC.TL_channels_getMessages req = new TLRPC.TL_channels_getMessages();
                    req.channel = getMessagesController().getInputChannel(channelId);
                    req.id.add(Integer.valueOf(mid));
                    getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda27
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            MediaDataController.this.m396xe989b4b8(channelId, tLObject, tL_error);
                        }
                    });
                    return null;
                }
                TLRPC.TL_messages_getMessages req2 = new TLRPC.TL_messages_getMessages();
                req2.id.add(Integer.valueOf(mid));
                getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda28
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.this.m397x842a7739(channelId, tLObject, tL_error);
                    }
                });
                return null;
            } else if (returnValue) {
                return broadcastPinnedMessage(result2, users, chats, true, returnValue);
            } else {
                if (usersToLoad2.isEmpty()) {
                    usersToLoad = usersToLoad2;
                } else {
                    usersToLoad = usersToLoad2;
                    getMessagesStorage().getUsersInternal(TextUtils.join(",", usersToLoad), users);
                }
                if (!chatsToLoad2.isEmpty()) {
                    chatsToLoad = chatsToLoad2;
                    getMessagesStorage().getChatsInternal(TextUtils.join(",", chatsToLoad), chats);
                } else {
                    chatsToLoad = chatsToLoad2;
                }
                broadcastPinnedMessage(result2, users, chats, true, false);
                return null;
            }
        } catch (Exception e) {
            FileLog.e(e);
            return null;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadPinnedMessageInternal$88$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m396xe989b4b8(int channelId, TLObject response, TLRPC.TL_error error) {
        boolean ok = false;
        if (error == null) {
            TLRPC.messages_Messages messagesRes = (TLRPC.messages_Messages) response;
            removeEmptyMessages(messagesRes.messages);
            if (!messagesRes.messages.isEmpty()) {
                ImageLoader.saveMessagesThumbs(messagesRes.messages);
                broadcastPinnedMessage(messagesRes.messages.get(0), messagesRes.users, messagesRes.chats, false, false);
                getMessagesStorage().putUsersAndChats(messagesRes.users, messagesRes.chats, true, true);
                savePinnedMessage(messagesRes.messages.get(0));
                ok = true;
            }
        }
        if (!ok) {
            getMessagesStorage().updateChatPinnedMessage(channelId, 0);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadPinnedMessageInternal$89$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m397x842a7739(int channelId, TLObject response, TLRPC.TL_error error) {
        boolean ok = false;
        if (error == null) {
            TLRPC.messages_Messages messagesRes = (TLRPC.messages_Messages) response;
            removeEmptyMessages(messagesRes.messages);
            if (!messagesRes.messages.isEmpty()) {
                ImageLoader.saveMessagesThumbs(messagesRes.messages);
                broadcastPinnedMessage(messagesRes.messages.get(0), messagesRes.users, messagesRes.chats, false, false);
                getMessagesStorage().putUsersAndChats(messagesRes.users, messagesRes.chats, true, true);
                savePinnedMessage(messagesRes.messages.get(0));
                ok = true;
            }
        }
        if (!ok) {
            getMessagesStorage().updateChatPinnedMessage(channelId, 0);
        }
    }

    private void savePinnedMessage(final TLRPC.Message result) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda98
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m447x4c6b16e3(result);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$savePinnedMessage$90$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m447x4c6b16e3(TLRPC.Message result) {
        long dialogId;
        NativeByteBuffer data = null;
        SQLitePreparedStatement state = null;
        try {
            try {
                if (result.to_id.channel_id != 0) {
                    dialogId = -result.to_id.channel_id;
                } else if (result.to_id.chat_id != 0) {
                    dialogId = -result.to_id.chat_id;
                } else if (result.to_id.user_id == 0) {
                    if (0 != 0) {
                        data.reuse();
                    }
                    if (0 != 0) {
                        state.dispose();
                        return;
                    }
                    return;
                } else {
                    dialogId = result.to_id.user_id;
                }
                try {
                    getMessagesStorage().getDatabase().beginTransaction();
                } catch (Exception e) {
                    FileLog.e("savePinnedMessage ---> exception 1 ", e);
                }
                SQLitePreparedStatement state2 = getMessagesStorage().getDatabase().executeFast("REPLACE INTO chat_pinned VALUES(?, ?, ?)");
                NativeByteBuffer data2 = new NativeByteBuffer(result.getObjectSize());
                result.serializeToStream(data2);
                state2.requery();
                state2.bindLong(1, dialogId);
                state2.bindInteger(2, result.id);
                state2.bindByteBuffer(3, data2);
                state2.step();
                data2.reuse();
                data = null;
                state2.dispose();
                state = null;
                getMessagesStorage().getDatabase().commitTransaction();
                if (0 != 0) {
                    data.reuse();
                }
                if (0 == 0) {
                    return;
                }
            } catch (Exception e2) {
                FileLog.e("savePinnedMessage ---> exception 2 ", e2);
                if (data != null) {
                    data.reuse();
                }
                if (state == null) {
                    return;
                }
            }
            state.dispose();
        } catch (Throwable th) {
            if (data != null) {
                data.reuse();
            }
            if (state != null) {
                state.dispose();
            }
            throw th;
        }
    }

    private MessageObject broadcastPinnedMessage(final TLRPC.Message result, final ArrayList<TLRPC.User> users, final ArrayList<TLRPC.Chat> chats, final boolean isCache, boolean returnValue) {
        final SparseArray<TLRPC.User> usersDict = new SparseArray<>();
        for (int a2 = 0; a2 < users.size(); a2++) {
            TLRPC.User user = users.get(a2);
            usersDict.put(user.id, user);
        }
        final SparseArray<TLRPC.Chat> chatsDict = new SparseArray<>();
        for (int a3 = 0; a3 < chats.size(); a3++) {
            TLRPC.Chat chat = chats.get(a3);
            chatsDict.put(chat.id, chat);
        }
        if (returnValue) {
            return new MessageObject(this.currentAccount, result, usersDict, chatsDict, false);
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda8
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m343xfaf6664e(users, isCache, chats, result, usersDict, chatsDict);
            }
        });
        return null;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$broadcastPinnedMessage$91$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m343xfaf6664e(ArrayList users, boolean isCache, ArrayList chats, TLRPC.Message result, SparseArray usersDict, SparseArray chatsDict) {
        getMessagesController().putUsers(users, isCache);
        getMessagesController().putChats(chats, isCache);
        getNotificationCenter().postNotificationName(NotificationCenter.pinnedMessageDidLoad, new MessageObject(this.currentAccount, result, (SparseArray<TLRPC.User>) usersDict, (SparseArray<TLRPC.Chat>) chatsDict, false));
    }

    private static void removeEmptyMessages(ArrayList<TLRPC.Message> messages) {
        int a2 = 0;
        while (a2 < messages.size()) {
            TLRPC.Message message = messages.get(a2);
            if (message == null || (message instanceof TLRPC.TL_messageEmpty) || (message.action instanceof TLRPC.TL_messageActionHistoryClear)) {
                messages.remove(a2);
                a2--;
            }
            a2++;
        }
    }

    /* JADX WARN: Incorrect condition in loop: B:24:0x008f */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void loadReplyMessagesForMessages(ArrayList<MessageObject> messages, final long dialogId, final boolean scheduled) {
        if (((int) dialogId) == 0) {
            final ArrayList<Long> replyMessages = new ArrayList<>();
            final LongSparseArray<ArrayList<MessageObject>> replyMessageRandomOwners = new LongSparseArray<>();
            for (int a2 = 0; a2 < messages.size(); a2++) {
                MessageObject messageObject = messages.get(a2);
                if (messageObject.isReply() && messageObject.replyMessageObject == null) {
                    long id = messageObject.messageOwner.reply_to_random_id;
                    ArrayList<MessageObject> messageObjects = replyMessageRandomOwners.get(id);
                    if (messageObjects == null) {
                        messageObjects = new ArrayList<>();
                        replyMessageRandomOwners.put(id, messageObjects);
                    }
                    messageObjects.add(messageObject);
                    if (!replyMessages.contains(Long.valueOf(id))) {
                        replyMessages.add(Long.valueOf(id));
                    }
                }
            }
            if (replyMessages.isEmpty()) {
                return;
            }
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda12
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m403x93ad0df9(replyMessages, dialogId, replyMessageRandomOwners);
                }
            });
            return;
        }
        final ArrayList<Integer> replyMessages2 = new ArrayList<>();
        final SparseArray<ArrayList<MessageObject>> replyMessageOwners = new SparseArray<>();
        final StringBuilder stringBuilder = new StringBuilder();
        int channelId = 0;
        for (int a3 = 0; a3 < channelId; a3++) {
            MessageObject messageObject2 = messages.get(a3);
            if (messageObject2.getId() > 0 && messageObject2.isReply() && messageObject2.replyMessageObject == null) {
                int id2 = messageObject2.messageOwner.reply_to_msg_id;
                long messageId = id2;
                if (messageObject2.messageOwner.to_id.channel_id != 0) {
                    messageId |= messageObject2.messageOwner.to_id.channel_id << 32;
                    channelId = messageObject2.messageOwner.to_id.channel_id;
                }
                if (stringBuilder.length() > 0) {
                    stringBuilder.append(',');
                }
                stringBuilder.append(messageId);
                ArrayList<MessageObject> messageObjects2 = replyMessageOwners.get(id2);
                if (messageObjects2 == null) {
                    messageObjects2 = new ArrayList<>();
                    replyMessageOwners.put(id2, messageObjects2);
                }
                messageObjects2.add(messageObject2);
                if (!replyMessages2.contains(Integer.valueOf(id2))) {
                    replyMessages2.add(Integer.valueOf(id2));
                }
            }
        }
        if (replyMessages2.isEmpty()) {
            return;
        }
        final int channelIdFinal = channelId;
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda13
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m406x638f557c(stringBuilder, dialogId, replyMessages2, replyMessageOwners, channelIdFinal, scheduled);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Type inference failed for: r11v0 */
    /* JADX WARN: Type inference failed for: r11v1, types: [boolean, int] */
    /* JADX WARN: Type inference failed for: r11v3 */
    /* renamed from: lambda$loadReplyMessagesForMessages$93$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m403x93ad0df9(ArrayList replyMessages, final long dialogId, LongSparseArray replyMessageRandomOwners) {
        SQLiteDatabase database;
        Locale locale;
        int i;
        Object[] objArr;
        try {
            database = getMessagesStorage().getDatabase();
            locale = Locale.US;
            i = 1;
            objArr = new Object[1];
        } catch (Exception e) {
            e = e;
        }
        try {
            ?? r11 = 0;
            objArr[0] = TextUtils.join(",", replyMessages);
            SQLiteCursor cursor = database.queryFinalized(String.format(locale, "SELECT m.data, m.mid, m.date, r.random_id FROM randoms as r INNER JOIN messages as m ON r.mid = m.mid WHERE r.random_id IN(%s)", objArr), new Object[0]);
            while (cursor.next()) {
                NativeByteBuffer data = cursor.byteBufferValue(r11);
                if (data != null) {
                    TLRPC.Message message = TLRPC.Message.TLdeserialize(data, data.readInt32(r11), r11);
                    message.readAttachPath(data, getUserConfig().clientUserId);
                    data.reuse();
                    message.id = cursor.intValue(i);
                    message.date = cursor.intValue(2);
                    message.dialog_id = dialogId;
                    long value = cursor.longValue(3);
                    ArrayList<MessageObject> arrayList = (ArrayList) replyMessageRandomOwners.get(value);
                    replyMessageRandomOwners.remove(value);
                    if (arrayList != null) {
                        MessageObject messageObject = new MessageObject(this.currentAccount, message, r11);
                        for (int b = 0; b < arrayList.size(); b++) {
                            MessageObject object = arrayList.get(b);
                            object.replyMessageObject = messageObject;
                            object.messageOwner.reply_to_msg_id = messageObject.getId();
                            if (object.isMegagroup()) {
                                object.replyMessageObject.messageOwner.flags |= Integer.MIN_VALUE;
                            }
                        }
                    }
                }
                i = 1;
                r11 = 0;
            }
            cursor.dispose();
            if (replyMessageRandomOwners.size() != 0) {
                for (int b2 = 0; b2 < replyMessageRandomOwners.size(); b2++) {
                    ArrayList<MessageObject> arrayList2 = (ArrayList) replyMessageRandomOwners.valueAt(b2);
                    for (int a2 = 0; a2 < arrayList2.size(); a2++) {
                        arrayList2.get(a2).messageOwner.reply_to_random_id = 0L;
                    }
                }
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda26
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m402xf90c4b78(dialogId);
                }
            });
        } catch (Exception e2) {
            e = e2;
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadReplyMessagesForMessages$92$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m402xf90c4b78(long dialogId) {
        getNotificationCenter().postNotificationName(NotificationCenter.replyMessagesDidLoad, Long.valueOf(dialogId));
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadReplyMessagesForMessages$96$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m406x638f557c(StringBuilder stringBuilder, final long dialogId, ArrayList replyMessages, final SparseArray replyMessageOwners, int channelIdFinal, final boolean scheduled) {
        try {
            ArrayList<TLRPC.Message> result = new ArrayList<>();
            ArrayList<TLRPC.User> users = new ArrayList<>();
            ArrayList<TLRPC.Chat> chats = new ArrayList<>();
            ArrayList<Integer> usersToLoad = new ArrayList<>();
            ArrayList<Integer> chatsToLoad = new ArrayList<>();
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT data, mid, date FROM messages WHERE mid IN(%s)", stringBuilder.toString()), new Object[0]);
            while (cursor.next()) {
                try {
                    NativeByteBuffer data = cursor.byteBufferValue(0);
                    if (data != null) {
                        TLRPC.Message message = TLRPC.Message.TLdeserialize(data, data.readInt32(false), false);
                        message.readAttachPath(data, getUserConfig().clientUserId);
                        data.reuse();
                        message.id = cursor.intValue(1);
                        message.date = cursor.intValue(2);
                        message.dialog_id = dialogId;
                        MessagesStorage.addUsersAndChatsFromMessage(message, usersToLoad, chatsToLoad);
                        result.add(message);
                        replyMessages.remove(Integer.valueOf(message.id));
                    }
                } catch (Exception e) {
                    e = e;
                    FileLog.e(e);
                    return;
                }
            }
            cursor.dispose();
            if (!usersToLoad.isEmpty()) {
                getMessagesStorage().getUsersInternal(TextUtils.join(",", usersToLoad), users);
            }
            if (!chatsToLoad.isEmpty()) {
                getMessagesStorage().getChatsInternal(TextUtils.join(",", chatsToLoad), chats);
            }
            broadcastReplyMessages(result, replyMessageOwners, users, chats, dialogId, true);
            if (!replyMessages.isEmpty()) {
                if (channelIdFinal != 0) {
                    TLRPC.TL_channels_getMessages req = new TLRPC.TL_channels_getMessages();
                    req.channel = getMessagesController().getInputChannel(channelIdFinal);
                    req.id = replyMessages;
                    getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda48
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            MediaDataController.this.m404x2e4dd07a(replyMessageOwners, dialogId, scheduled, tLObject, tL_error);
                        }
                    });
                    return;
                }
                TLRPC.TL_messages_getMessages req2 = new TLRPC.TL_messages_getMessages();
                req2.id = replyMessages;
                getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda59
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.this.m405xc8ee92fb(replyMessageOwners, dialogId, scheduled, tLObject, tL_error);
                    }
                });
            }
        } catch (Exception e2) {
            e = e2;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadReplyMessagesForMessages$94$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m404x2e4dd07a(SparseArray replyMessageOwners, long dialogId, boolean scheduled, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_Messages messagesRes = (TLRPC.messages_Messages) response;
            removeEmptyMessages(messagesRes.messages);
            ImageLoader.saveMessagesThumbs(messagesRes.messages);
            broadcastReplyMessages(messagesRes.messages, replyMessageOwners, messagesRes.users, messagesRes.chats, dialogId, false);
            getMessagesStorage().putUsersAndChats(messagesRes.users, messagesRes.chats, true, true);
            saveReplyMessages(replyMessageOwners, messagesRes.messages, scheduled);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadReplyMessagesForMessages$95$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m405xc8ee92fb(SparseArray replyMessageOwners, long dialogId, boolean scheduled, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_Messages messagesRes = (TLRPC.messages_Messages) response;
            removeEmptyMessages(messagesRes.messages);
            ImageLoader.saveMessagesThumbs(messagesRes.messages);
            broadcastReplyMessages(messagesRes.messages, replyMessageOwners, messagesRes.users, messagesRes.chats, dialogId, false);
            getMessagesStorage().putUsersAndChats(messagesRes.users, messagesRes.chats, true, true);
            saveReplyMessages(replyMessageOwners, messagesRes.messages, scheduled);
        }
    }

    private void saveReplyMessages(final SparseArray<ArrayList<MessageObject>> replyMessageOwners, final ArrayList<TLRPC.Message> result, final boolean scheduled) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda112
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m448xc05ac043(scheduled, result, replyMessageOwners);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$saveReplyMessages$97$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m448xc05ac043(boolean scheduled, ArrayList result, SparseArray replyMessageOwners) {
        NativeByteBuffer data = null;
        SQLitePreparedStatement state = null;
        try {
            try {
                getMessagesStorage().getDatabase().beginTransaction();
            } catch (Exception e) {
                try {
                    FileLog.e("saveReplyMessages ---> exception 1 ", e);
                } catch (Exception e2) {
                    FileLog.e("saveReplyMessages ---> exception 2 ", e2);
                    if (data != null) {
                        data.reuse();
                    }
                    if (state == null) {
                        return;
                    }
                }
            }
            SQLitePreparedStatement state2 = scheduled ? getMessagesStorage().getDatabase().executeFast("UPDATE scheduled_messages SET replydata = ? WHERE mid = ?") : getMessagesStorage().getDatabase().executeFast("UPDATE messages SET replydata = ? WHERE mid = ?");
            for (int a2 = 0; a2 < result.size(); a2++) {
                TLRPC.Message message = (TLRPC.Message) result.get(a2);
                ArrayList<MessageObject> messageObjects = (ArrayList) replyMessageOwners.get(message.id);
                if (messageObjects != null) {
                    NativeByteBuffer data2 = new NativeByteBuffer(message.getObjectSize());
                    message.serializeToStream(data2);
                    for (int b = 0; b < messageObjects.size(); b++) {
                        MessageObject messageObject = messageObjects.get(b);
                        state2.requery();
                        long messageId = messageObject.getId();
                        if (messageObject.messageOwner.to_id.channel_id != 0) {
                            messageId |= messageObject.messageOwner.to_id.channel_id << 32;
                        }
                        state2.bindByteBuffer(1, data2);
                        state2.bindLong(2, messageId);
                        state2.step();
                    }
                    data2.reuse();
                    data = null;
                }
            }
            state2.dispose();
            state = null;
            getMessagesStorage().getDatabase().commitTransaction();
            if (data != null) {
                data.reuse();
            }
            if (0 == 0) {
                return;
            }
            state.dispose();
        } catch (Throwable th) {
            if (data != null) {
                data.reuse();
            }
            if (state != null) {
                state.dispose();
            }
            throw th;
        }
    }

    private void broadcastReplyMessages(final ArrayList<TLRPC.Message> result, final SparseArray<ArrayList<MessageObject>> replyMessageOwners, final ArrayList<TLRPC.User> users, final ArrayList<TLRPC.Chat> chats, final long dialog_id, final boolean isCache) {
        final SparseArray<TLRPC.User> usersDict = new SparseArray<>();
        for (int a2 = 0; a2 < users.size(); a2++) {
            TLRPC.User user = users.get(a2);
            usersDict.put(user.id, user);
        }
        final SparseArray<TLRPC.Chat> chatsDict = new SparseArray<>();
        for (int a3 = 0; a3 < chats.size(); a3++) {
            TLRPC.Chat chat = chats.get(a3);
            chatsDict.put(chat.id, chat);
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda108
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m344x6ee60fae(users, isCache, chats, result, replyMessageOwners, usersDict, chatsDict, dialog_id);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$broadcastReplyMessages$98$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m344x6ee60fae(ArrayList users, boolean isCache, ArrayList chats, ArrayList result, SparseArray replyMessageOwners, SparseArray usersDict, SparseArray chatsDict, long dialog_id) {
        getMessagesController().putUsers(users, isCache);
        getMessagesController().putChats(chats, isCache);
        boolean changed = false;
        for (int a2 = 0; a2 < result.size(); a2++) {
            TLRPC.Message message = (TLRPC.Message) result.get(a2);
            ArrayList<MessageObject> arrayList = (ArrayList) replyMessageOwners.get(message.id);
            if (arrayList != null) {
                MessageObject messageObject = new MessageObject(this.currentAccount, message, (SparseArray<TLRPC.User>) usersDict, (SparseArray<TLRPC.Chat>) chatsDict, false);
                for (int b = 0; b < arrayList.size(); b++) {
                    MessageObject m = arrayList.get(b);
                    m.replyMessageObject = messageObject;
                    if (m.messageOwner.action instanceof TLRPC.TL_messageActionPinMessage) {
                        m.generatePinMessageText(null, null);
                    } else if (m.messageOwner.action instanceof TLRPC.TL_messageActionGameScore) {
                        m.generateGameMessageText(null);
                    } else if (m.messageOwner.action instanceof TLRPC.TL_messageActionPaymentSent) {
                        m.generatePaymentSentMessageText(null);
                    }
                    if (m.isMegagroup()) {
                        m.replyMessageObject.messageOwner.flags |= Integer.MIN_VALUE;
                    }
                }
                changed = true;
            }
        }
        if (changed) {
            getNotificationCenter().postNotificationName(NotificationCenter.replyMessagesDidLoad, Long.valueOf(dialog_id));
        }
    }

    public static void sortEntities(ArrayList<TLRPC.MessageEntity> entities) {
        Collections.sort(entities, entityComparator);
    }

    private static boolean checkInclusion(int index, ArrayList<TLRPC.MessageEntity> entities) {
        if (entities == null || entities.isEmpty()) {
            return false;
        }
        int count = entities.size();
        for (int a2 = 0; a2 < count; a2++) {
            TLRPC.MessageEntity entity = entities.get(a2);
            if (entity.offset <= index && entity.offset + entity.length > index) {
                return true;
            }
        }
        return false;
    }

    private static boolean checkIntersection(int start, int end, ArrayList<TLRPC.MessageEntity> entities) {
        if (entities == null || entities.isEmpty()) {
            return false;
        }
        int count = entities.size();
        for (int a2 = 0; a2 < count; a2++) {
            TLRPC.MessageEntity entity = entities.get(a2);
            if (entity.offset > start && entity.offset + entity.length <= end) {
                return true;
            }
        }
        return false;
    }

    private static void removeOffsetAfter(int start, int countToRemove, ArrayList<TLRPC.MessageEntity> entities) {
        int count = entities.size();
        for (int a2 = 0; a2 < count; a2++) {
            TLRPC.MessageEntity entity = entities.get(a2);
            if (entity.offset > start) {
                entity.offset -= countToRemove;
            }
        }
    }

    public CharSequence substring(CharSequence source, int start, int end) {
        if (source instanceof SpannableStringBuilder) {
            return source.subSequence(start, end);
        }
        if (source instanceof SpannedString) {
            return source.subSequence(start, end);
        }
        return TextUtils.substring(source, start, end);
    }

    private static CharacterStyle createNewSpan(CharacterStyle baseSpan, TextStyleSpan.TextStyleRun textStyleRun, TextStyleSpan.TextStyleRun newStyleRun, boolean allowIntersection) {
        TextStyleSpan.TextStyleRun run = new TextStyleSpan.TextStyleRun(textStyleRun);
        if (newStyleRun != null) {
            if (allowIntersection) {
                run.merge(newStyleRun);
            } else {
                run.replace(newStyleRun);
            }
        }
        if (baseSpan instanceof TextStyleSpan) {
            return new TextStyleSpan(run);
        }
        if (baseSpan instanceof URLSpanReplacement) {
            URLSpanReplacement span = (URLSpanReplacement) baseSpan;
            return new URLSpanReplacement(span.getURL(), run);
        }
        return null;
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:60:0x00c4
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:81)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:47)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:39)
        */
    public static void addStyleToText(im.uecahxyvjd.ui.components.TextStyleSpan r16, int r17, int r18, android.text.Spannable r19, boolean r20) {
        /*
            r1 = r16
            r2 = r19
            r3 = r20
            java.lang.Class<android.text.style.CharacterStyle> r0 = android.text.style.CharacterStyle.class
            r4 = r17
            r5 = r18
            java.lang.Object[] r0 = r2.getSpans(r4, r5, r0)     // Catch: java.lang.Exception -> Lc2
            android.text.style.CharacterStyle[] r0 = (android.text.style.CharacterStyle[]) r0     // Catch: java.lang.Exception -> Lc2
            r6 = 33
            if (r0 == 0) goto Lb7
            int r7 = r0.length     // Catch: java.lang.Exception -> Lc2
            if (r7 <= 0) goto Lb7
            r7 = 0
        L1a:
            int r8 = r0.length     // Catch: java.lang.Exception -> Lbf
            if (r7 >= r8) goto Lb7
            r8 = r0[r7]     // Catch: java.lang.Exception -> Lbf
            if (r1 == 0) goto L26
            im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun r9 = r16.getTextStyleRun()     // Catch: java.lang.Exception -> Lbf
            goto L2b
        L26:
            im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun r9 = new im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun     // Catch: java.lang.Exception -> Lbf
            r9.<init>()     // Catch: java.lang.Exception -> Lbf
        L2b:
            boolean r10 = r8 instanceof im.uecahxyvjd.ui.components.TextStyleSpan     // Catch: java.lang.Exception -> Lbf
            if (r10 == 0) goto L38
            r10 = r8
            im.uecahxyvjd.ui.components.TextStyleSpan r10 = (im.uecahxyvjd.ui.components.TextStyleSpan) r10     // Catch: java.lang.Exception -> Lbf
            im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun r11 = r10.getTextStyleRun()     // Catch: java.lang.Exception -> Lbf
            r10 = r11
            goto L4e
        L38:
            boolean r10 = r8 instanceof im.uecahxyvjd.ui.components.URLSpanReplacement     // Catch: java.lang.Exception -> Lbf
            if (r10 == 0) goto Lb3
            r10 = r8
            im.uecahxyvjd.ui.components.URLSpanReplacement r10 = (im.uecahxyvjd.ui.components.URLSpanReplacement) r10     // Catch: java.lang.Exception -> Lbf
            im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun r11 = r10.getTextStyleRun()     // Catch: java.lang.Exception -> Lbf
            if (r11 != 0) goto L4d
            im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun r12 = new im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun     // Catch: java.lang.Exception -> Lbf
            r12.<init>()     // Catch: java.lang.Exception -> Lbf
            r11 = r12
            r10 = r11
            goto L4e
        L4d:
            r10 = r11
        L4e:
            if (r10 != 0) goto L51
            goto Lb3
        L51:
            int r11 = r2.getSpanStart(r8)     // Catch: java.lang.Exception -> Lbf
            int r12 = r2.getSpanEnd(r8)     // Catch: java.lang.Exception -> Lbf
            r2.removeSpan(r8)     // Catch: java.lang.Exception -> Lbf
            if (r11 <= r4) goto L78
            if (r5 <= r12) goto L78
            android.text.style.CharacterStyle r13 = createNewSpan(r8, r10, r9, r3)     // Catch: java.lang.Exception -> Lbf
            r2.setSpan(r13, r11, r12, r6)     // Catch: java.lang.Exception -> Lbf
            if (r1 == 0) goto L76
            im.uecahxyvjd.ui.components.TextStyleSpan r13 = new im.uecahxyvjd.ui.components.TextStyleSpan     // Catch: java.lang.Exception -> Lbf
            im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun r14 = new im.uecahxyvjd.ui.components.TextStyleSpan$TextStyleRun     // Catch: java.lang.Exception -> Lbf
            r14.<init>(r9)     // Catch: java.lang.Exception -> Lbf
            r13.<init>(r14)     // Catch: java.lang.Exception -> Lbf
            r2.setSpan(r13, r12, r5, r6)     // Catch: java.lang.Exception -> Lbf
        L76:
            r5 = r11
            goto Lb3
        L78:
            r13 = r4
            r14 = 0
            if (r11 > r4) goto L95
            if (r11 == r4) goto L85
            android.text.style.CharacterStyle r15 = createNewSpan(r8, r10, r14, r3)     // Catch: java.lang.Exception -> Lbf
            r2.setSpan(r15, r11, r4, r6)     // Catch: java.lang.Exception -> Lbf
        L85:
            if (r12 <= r4) goto L95
            if (r1 == 0) goto L94
            android.text.style.CharacterStyle r15 = createNewSpan(r8, r10, r9, r3)     // Catch: java.lang.Exception -> Lbf
            int r14 = java.lang.Math.min(r12, r5)     // Catch: java.lang.Exception -> Lbf
            r2.setSpan(r15, r4, r14, r6)     // Catch: java.lang.Exception -> Lbf
        L94:
            r4 = r12
        L95:
            if (r12 < r5) goto Lb3
            if (r12 == r5) goto La1
            r14 = 0
            android.text.style.CharacterStyle r14 = createNewSpan(r8, r10, r14, r3)     // Catch: java.lang.Exception -> Lbf
            r2.setSpan(r14, r5, r12, r6)     // Catch: java.lang.Exception -> Lbf
        La1:
            if (r5 <= r11) goto Lb3
            if (r12 > r13) goto Lb3
            if (r1 == 0) goto Lb2
            android.text.style.CharacterStyle r14 = createNewSpan(r8, r10, r9, r3)     // Catch: java.lang.Exception -> Lbf
            int r15 = java.lang.Math.min(r12, r5)     // Catch: java.lang.Exception -> Lbf
            r2.setSpan(r14, r11, r15, r6)     // Catch: java.lang.Exception -> Lbf
        Lb2:
            r5 = r11
        Lb3:
            int r7 = r7 + 1
            goto L1a
        Lb7:
            if (r1 == 0) goto Lc1
            if (r4 >= r5) goto Lc1
            r2.setSpan(r1, r4, r5, r6)     // Catch: java.lang.Exception -> Lbf
            goto Lc1
        Lbf:
            r0 = move-exception
            goto Lc9
        Lc1:
            goto Lcc
        Lc2:
            r0 = move-exception
            goto Lc9
        Lc4:
            r0 = move-exception
            r4 = r17
            r5 = r18
        Lc9:
            im.uecahxyvjd.messenger.FileLog.e(r0)
        Lcc:
            return
        */
        throw new UnsupportedOperationException("Method not decompiled: im.uecahxyvjd.messenger.MediaDataController.addStyleToText(im.uecahxyvjd.ui.components.TextStyleSpan, int, int, android.text.Spannable, boolean):void");
    }

    public static ArrayList<TextStyleSpan.TextStyleRun> getTextStyleRuns(ArrayList<TLRPC.MessageEntity> entities, CharSequence text) {
        ArrayList<TextStyleSpan.TextStyleRun> runs = new ArrayList<>();
        ArrayList<TLRPC.MessageEntity> entitiesCopy = new ArrayList<>(entities);
        Collections.sort(entitiesCopy, new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda77
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return MediaDataController.lambda$getTextStyleRuns$99((TLRPC.MessageEntity) obj, (TLRPC.MessageEntity) obj2);
            }
        });
        int N = entitiesCopy.size();
        for (int a2 = 0; a2 < N; a2++) {
            TLRPC.MessageEntity entity = entitiesCopy.get(a2);
            if (entity.length > 0 && entity.offset >= 0 && entity.offset < text.length()) {
                if (entity.offset + entity.length > text.length()) {
                    entity.length = text.length() - entity.offset;
                }
                TextStyleSpan.TextStyleRun newRun = new TextStyleSpan.TextStyleRun();
                newRun.start = entity.offset;
                newRun.end = newRun.start + entity.length;
                if (entity instanceof TLRPC.TL_messageEntityStrike) {
                    newRun.flags = 8;
                } else if (entity instanceof TLRPC.TL_messageEntityUnderline) {
                    newRun.flags = 16;
                } else if (entity instanceof TLRPC.TL_messageEntityBlockquote) {
                    newRun.flags = 32;
                } else if (entity instanceof TLRPC.TL_messageEntityBold) {
                    newRun.flags = 1;
                } else if (entity instanceof TLRPC.TL_messageEntityItalic) {
                    newRun.flags = 2;
                } else if ((entity instanceof TLRPC.TL_messageEntityCode) || (entity instanceof TLRPC.TL_messageEntityPre)) {
                    newRun.flags = 4;
                } else if (entity instanceof TLRPC.TL_messageEntityMentionName) {
                    newRun.flags = 64;
                    newRun.urlEntity = entity;
                } else if (entity instanceof TLRPC.TL_inputMessageEntityMentionName) {
                    newRun.flags = 64;
                    newRun.urlEntity = entity;
                } else {
                    newRun.flags = 128;
                    newRun.urlEntity = entity;
                }
                int b = 0;
                int N2 = runs.size();
                while (b < N2) {
                    TextStyleSpan.TextStyleRun run = runs.get(b);
                    if (newRun.start > run.start) {
                        if (newRun.start < run.end) {
                            if (newRun.end < run.end) {
                                TextStyleSpan.TextStyleRun r = new TextStyleSpan.TextStyleRun(newRun);
                                r.merge(run);
                                int b2 = b + 1;
                                runs.add(b2, r);
                                TextStyleSpan.TextStyleRun r2 = new TextStyleSpan.TextStyleRun(run);
                                r2.start = newRun.end;
                                b = b2 + 1;
                                N2 = N2 + 1 + 1;
                                runs.add(b, r2);
                            } else if (newRun.end >= run.end) {
                                TextStyleSpan.TextStyleRun r3 = new TextStyleSpan.TextStyleRun(newRun);
                                r3.merge(run);
                                r3.end = run.end;
                                b++;
                                N2++;
                                runs.add(b, r3);
                            }
                            int temp = newRun.start;
                            newRun.start = run.end;
                            run.end = temp;
                        }
                    } else if (run.start < newRun.end) {
                        int temp2 = run.start;
                        if (newRun.end == run.end) {
                            run.merge(newRun);
                        } else if (newRun.end < run.end) {
                            TextStyleSpan.TextStyleRun r4 = new TextStyleSpan.TextStyleRun(run);
                            r4.merge(newRun);
                            r4.end = newRun.end;
                            b++;
                            N2++;
                            runs.add(b, r4);
                            run.start = newRun.end;
                        } else {
                            TextStyleSpan.TextStyleRun r5 = new TextStyleSpan.TextStyleRun(newRun);
                            r5.start = run.end;
                            b++;
                            N2++;
                            runs.add(b, r5);
                            run.merge(newRun);
                        }
                        newRun.end = temp2;
                    }
                    b++;
                }
                int b3 = newRun.start;
                if (b3 < newRun.end) {
                    runs.add(newRun);
                }
            }
        }
        return runs;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$getTextStyleRuns$99(TLRPC.MessageEntity o1, TLRPC.MessageEntity o2) {
        if (o1.offset > o2.offset) {
            return 1;
        }
        if (o1.offset < o2.offset) {
            return -1;
        }
        return 0;
    }

    /* JADX WARN: Code restructure failed: missing block: B:190:0x03c9, code lost:
        if (r10 == '\n') goto L240;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public ArrayList<TLRPC.MessageEntity> getEntities(CharSequence[] message) {
        ArrayList<TLRPC.MessageEntity> entities;
        TLRPC.MessageEntity entity;
        TextStyleSpan[] spans;
        String mono;
        String pre;
        char c;
        int firstChar;
        char c2;
        char c3;
        if (message != null) {
            int i = 0;
            if (message[0] == null) {
                return null;
            }
            ArrayList<TLRPC.MessageEntity> entities2 = null;
            int start = -1;
            int lastIndex = 0;
            boolean isPre = false;
            String mono2 = "`";
            String pre2 = "```";
            while (true) {
                int indexOf = TextUtils.indexOf(message[i], !isPre ? "`" : "```", lastIndex);
                int index = indexOf;
                if (indexOf == -1) {
                    break;
                } else if (start == -1) {
                    isPre = message[i].length() - index > 2 && message[i].charAt(index + 1) == '`' && message[i].charAt(index + 2) == '`';
                    start = index;
                    lastIndex = index + (isPre ? 3 : 1);
                } else {
                    if (entities2 == null) {
                        entities2 = new ArrayList<>();
                    }
                    for (int a2 = (isPre ? 3 : 1) + index; a2 < message[i].length() && message[i].charAt(a2) == '`'; a2++) {
                        index++;
                    }
                    int lastIndex2 = (isPre ? 3 : 1) + index;
                    if (isPre) {
                        int firstChar2 = start > 0 ? message[i].charAt(start - 1) : 0;
                        boolean replacedFirst = firstChar2 == 32 || firstChar2 == 10;
                        CharSequence startMessage = substring(message[i], i, start - (replacedFirst ? 1 : 0));
                        CharSequence content = substring(message[i], start + 3, index);
                        if (index + 3 < message[i].length()) {
                            c = 0;
                            firstChar = message[0].charAt(index + 3);
                        } else {
                            c = 0;
                            firstChar = 0;
                        }
                        mono = mono2;
                        CharSequence endMessage = substring(message[c], index + 3 + ((firstChar == 32 || firstChar == '\n') ? 1 : 0), message[0].length());
                        if (startMessage.length() != 0) {
                            startMessage = AndroidUtilities.concat(startMessage, "\n");
                        } else {
                            replacedFirst = true;
                        }
                        if (endMessage.length() != 0) {
                            c2 = 0;
                            c3 = 1;
                            endMessage = AndroidUtilities.concat("\n", endMessage);
                        } else {
                            c2 = 0;
                            c3 = 1;
                        }
                        if (TextUtils.isEmpty(content)) {
                            pre = pre2;
                        } else {
                            pre = pre2;
                            CharSequence[] charSequenceArr = new CharSequence[3];
                            charSequenceArr[c2] = startMessage;
                            charSequenceArr[c3] = content;
                            charSequenceArr[2] = endMessage;
                            message[c2] = AndroidUtilities.concat(charSequenceArr);
                            TLRPC.TL_messageEntityPre entity2 = new TLRPC.TL_messageEntityPre();
                            entity2.offset = (replacedFirst ? 0 : 1) + start;
                            entity2.length = ((index - start) - 3) + (replacedFirst ? 0 : 1);
                            entity2.language = "";
                            entities2.add(entity2);
                            lastIndex2 -= 6;
                        }
                    } else {
                        mono = mono2;
                        pre = pre2;
                        if (start + 1 != index) {
                            message[0] = AndroidUtilities.concat(substring(message[0], 0, start), substring(message[0], start + 1, index), substring(message[0], index + 1, message[0].length()));
                            TLRPC.TL_messageEntityCode entity3 = new TLRPC.TL_messageEntityCode();
                            entity3.offset = start;
                            entity3.length = (index - start) - 1;
                            entities2.add(entity3);
                            lastIndex2 -= 2;
                        }
                    }
                    lastIndex = lastIndex2;
                    start = -1;
                    isPre = false;
                    mono2 = mono;
                    pre2 = pre;
                    i = 0;
                }
            }
            if (start != -1 && isPre) {
                message[0] = AndroidUtilities.concat(substring(message[0], 0, start), substring(message[0], start + 2, message[0].length()));
                if (entities2 == null) {
                    entities2 = new ArrayList<>();
                }
                TLRPC.TL_messageEntityCode entity4 = new TLRPC.TL_messageEntityCode();
                entity4.offset = start;
                entity4.length = 1;
                entities2.add(entity4);
            }
            if (message[0] instanceof Spanned) {
                Spanned spannable = (Spanned) message[0];
                TextStyleSpan[] spans2 = (TextStyleSpan[]) spannable.getSpans(0, message[0].length(), TextStyleSpan.class);
                if (spans2 != null && spans2.length > 0) {
                    int a3 = 0;
                    while (a3 < spans2.length) {
                        TextStyleSpan span = spans2[a3];
                        int spanStart = spannable.getSpanStart(span);
                        int spanEnd = spannable.getSpanEnd(span);
                        if (checkInclusion(spanStart, entities2) || checkInclusion(spanEnd, entities2)) {
                            spans = spans2;
                        } else if (checkIntersection(spanStart, spanEnd, entities2)) {
                            spans = spans2;
                        } else {
                            if (entities2 == null) {
                                entities2 = new ArrayList<>();
                            }
                            int flags = span.getStyleFlags();
                            if ((flags & 1) == 0) {
                                spans = spans2;
                            } else {
                                TLRPC.MessageEntity entity5 = new TLRPC.TL_messageEntityBold();
                                entity5.offset = spanStart;
                                spans = spans2;
                                entity5.length = spanEnd - spanStart;
                                entities2.add(entity5);
                            }
                            if ((flags & 2) != 0) {
                                TLRPC.MessageEntity entity6 = new TLRPC.TL_messageEntityItalic();
                                entity6.offset = spanStart;
                                entity6.length = spanEnd - spanStart;
                                entities2.add(entity6);
                            }
                            if ((flags & 4) != 0) {
                                TLRPC.MessageEntity entity7 = new TLRPC.TL_messageEntityCode();
                                entity7.offset = spanStart;
                                entity7.length = spanEnd - spanStart;
                                entities2.add(entity7);
                            }
                            if ((flags & 8) != 0) {
                                TLRPC.MessageEntity entity8 = new TLRPC.TL_messageEntityStrike();
                                entity8.offset = spanStart;
                                entity8.length = spanEnd - spanStart;
                                entities2.add(entity8);
                            }
                            if ((flags & 16) != 0) {
                                TLRPC.MessageEntity entity9 = new TLRPC.TL_messageEntityUnderline();
                                entity9.offset = spanStart;
                                entity9.length = spanEnd - spanStart;
                                entities2.add(entity9);
                            }
                            if ((flags & 32) != 0) {
                                TLRPC.MessageEntity entity10 = new TLRPC.TL_messageEntityBlockquote();
                                entity10.offset = spanStart;
                                entity10.length = spanEnd - spanStart;
                                entities2.add(entity10);
                            }
                        }
                        a3++;
                        spans2 = spans;
                    }
                }
                URLSpanUserMention[] spansMentions = (URLSpanUserMention[]) spannable.getSpans(0, message[0].length(), URLSpanUserMention.class);
                if (spansMentions != null && spansMentions.length > 0) {
                    if (entities2 == null) {
                        entities2 = new ArrayList<>();
                    }
                    for (int b = 0; b < spansMentions.length; b++) {
                        TLRPC.TL_inputMessageEntityMentionName entity11 = new TLRPC.TL_inputMessageEntityMentionName();
                        entity11.user_id = getMessagesController().getInputUser(Utilities.parseInt(spansMentions[b].getURL()).intValue());
                        if (entity11.user_id != null) {
                            entity11.offset = spannable.getSpanStart(spansMentions[b]);
                            entity11.length = Math.min(spannable.getSpanEnd(spansMentions[b]), message[0].length()) - entity11.offset;
                            if (message[0].charAt((entity11.offset + entity11.length) - 1) == ' ') {
                                entity11.length--;
                            }
                            entities2.add(entity11);
                        }
                    }
                }
                URLSpanReplacement[] spansUrlReplacement = (URLSpanReplacement[]) spannable.getSpans(0, message[0].length(), URLSpanReplacement.class);
                if (spansUrlReplacement != null && spansUrlReplacement.length > 0) {
                    if (entities2 == null) {
                        entities2 = new ArrayList<>();
                    }
                    for (int b2 = 0; b2 < spansUrlReplacement.length; b2++) {
                        TLRPC.TL_messageEntityTextUrl entity12 = new TLRPC.TL_messageEntityTextUrl();
                        entity12.offset = spannable.getSpanStart(spansUrlReplacement[b2]);
                        entity12.length = Math.min(spannable.getSpanEnd(spansUrlReplacement[b2]), message[0].length()) - entity12.offset;
                        entity12.url = spansUrlReplacement[b2].getURL();
                        entities2.add(entity12);
                    }
                }
            }
            int c4 = 0;
            while (c4 < 2) {
                int lastIndex3 = 0;
                int start2 = -1;
                String checkString = c4 == 0 ? "**" : "__";
                char checkChar = c4 == 0 ? '*' : '_';
                while (true) {
                    int indexOf2 = TextUtils.indexOf(message[0], checkString, lastIndex3);
                    int index2 = indexOf2;
                    if (indexOf2 != -1) {
                        if (start2 == -1) {
                            char prevChar = index2 == 0 ? ' ' : message[0].charAt(index2 - 1);
                            if (!checkInclusion(index2, entities2)) {
                                if (prevChar != ' ') {
                                }
                                start2 = index2;
                            }
                            lastIndex3 = index2 + 2;
                        } else {
                            for (int a4 = index2 + 2; a4 < message[0].length() && message[0].charAt(a4) == checkChar; a4++) {
                                index2++;
                            }
                            lastIndex3 = index2 + 2;
                            if (checkInclusion(index2, entities2) || checkIntersection(start2, index2, entities2)) {
                                start2 = -1;
                            } else {
                                if (start2 + 2 != index2) {
                                    if (entities2 != null) {
                                        entities = entities2;
                                    } else {
                                        entities = new ArrayList<>();
                                    }
                                    try {
                                        message[0] = AndroidUtilities.concat(substring(message[0], 0, start2), substring(message[0], start2 + 2, index2), substring(message[0], index2 + 2, message[0].length()));
                                    } catch (Exception e) {
                                        message[0] = substring(message[0], 0, start2).toString() + substring(message[0], start2 + 2, index2).toString() + substring(message[0], index2 + 2, message[0].length()).toString();
                                    }
                                    if (c4 == 0) {
                                        entity = new TLRPC.TL_messageEntityBold();
                                    } else {
                                        entity = new TLRPC.TL_messageEntityItalic();
                                    }
                                    entity.offset = start2;
                                    entity.length = (index2 - start2) - 2;
                                    removeOffsetAfter(entity.offset + entity.length, 4, entities);
                                    entities.add(entity);
                                    lastIndex3 -= 4;
                                    entities2 = entities;
                                }
                                start2 = -1;
                            }
                        }
                    }
                }
                c4++;
            }
            return entities2;
        }
        return null;
    }

    public void loadDrafts() {
        if (getUserConfig().draftsLoaded || this.loadingDrafts) {
            return;
        }
        this.loadingDrafts = true;
        TLRPC.TL_messages_getAllDrafts req = new TLRPC.TL_messages_getAllDrafts();
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda114
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m376x1a07cd80(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadDrafts$101$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m376x1a07cd80(TLObject response, TLRPC.TL_error error) {
        if (error != null) {
            return;
        }
        getMessagesController().processUpdates((TLRPC.Updates) response, false);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda70
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m375x7f670aff();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadDrafts$100$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m375x7f670aff() {
        getUserConfig().draftsLoaded = true;
        this.loadingDrafts = false;
        getUserConfig().saveConfig(false);
    }

    public TLRPC.DraftMessage getDraft(long did) {
        return this.drafts.get(did);
    }

    public TLRPC.Message getDraftMessage(long did) {
        return this.draftMessages.get(did);
    }

    public void saveDraft(long did, CharSequence message, ArrayList<TLRPC.MessageEntity> entities, TLRPC.Message replyToMessage, boolean noWebpage) {
        saveDraft(did, message, entities, replyToMessage, noWebpage, false);
    }

    public void saveDraft(long did, CharSequence message, ArrayList<TLRPC.MessageEntity> entities, TLRPC.Message replyToMessage, boolean noWebpage, boolean clean) {
        TLRPC.DraftMessage draftMessage;
        if (!TextUtils.isEmpty(message) || replyToMessage != null) {
            draftMessage = new TLRPC.TL_draftMessage();
        } else {
            draftMessage = new TLRPC.TL_draftMessageEmpty();
        }
        draftMessage.date = (int) (System.currentTimeMillis() / 1000);
        draftMessage.message = message == null ? "" : message.toString();
        draftMessage.no_webpage = noWebpage;
        if (replyToMessage != null) {
            draftMessage.reply_to_msg_id = replyToMessage.id;
            draftMessage.flags |= 1;
        }
        if (entities != null && !entities.isEmpty()) {
            draftMessage.entities = entities;
            draftMessage.flags |= 8;
        }
        TLRPC.DraftMessage currentDraft = this.drafts.get(did);
        if (!clean) {
            if (currentDraft == null || !currentDraft.message.equals(draftMessage.message) || currentDraft.reply_to_msg_id != draftMessage.reply_to_msg_id || currentDraft.no_webpage != draftMessage.no_webpage) {
                if (currentDraft == null && TextUtils.isEmpty(draftMessage.message) && draftMessage.reply_to_msg_id == 0) {
                    return;
                }
            } else {
                return;
            }
        }
        saveDraft(did, draftMessage, replyToMessage, false);
        int lower_id = (int) did;
        if (lower_id != 0) {
            TLRPC.TL_messages_saveDraft req = new TLRPC.TL_messages_saveDraft();
            req.peer = getMessagesController().getInputPeer(lower_id);
            if (req.peer == null) {
                return;
            }
            req.message = draftMessage.message;
            req.no_webpage = draftMessage.no_webpage;
            req.reply_to_msg_id = draftMessage.reply_to_msg_id;
            req.entities = draftMessage.entities;
            req.flags = draftMessage.flags;
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda20
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    MediaDataController.lambda$saveDraft$102(tLObject, tL_error);
                }
            });
        }
        getMessagesController().sortDialogs(null);
        getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$saveDraft$102(TLObject response, TLRPC.TL_error error) {
    }

    public void saveDraft(final long did, TLRPC.DraftMessage draft, TLRPC.Message replyToMessage, boolean fromServer) {
        TLRPC.User user;
        TLRPC.Chat chat;
        long messageId;
        int channelIdFinal;
        SharedPreferences.Editor editor = this.preferences.edit();
        if (draft == null || (draft instanceof TLRPC.TL_draftMessageEmpty)) {
            this.drafts.remove(did);
            this.draftMessages.remove(did);
            this.preferences.edit().remove("" + did).remove("r_" + did).commit();
        } else {
            this.drafts.put(did, draft);
            try {
                SerializedData serializedData = new SerializedData(draft.getObjectSize());
                draft.serializeToStream(serializedData);
                editor.putString("" + did, Utilities.bytesToHex(serializedData.toByteArray()));
                serializedData.cleanup();
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
        if (replyToMessage == null) {
            this.draftMessages.remove(did);
            editor.remove("r_" + did);
        } else {
            this.draftMessages.put(did, replyToMessage);
            SerializedData serializedData2 = new SerializedData(replyToMessage.getObjectSize());
            replyToMessage.serializeToStream(serializedData2);
            editor.putString("r_" + did, Utilities.bytesToHex(serializedData2.toByteArray()));
            serializedData2.cleanup();
        }
        editor.commit();
        if (fromServer) {
            if (draft.reply_to_msg_id != 0 && replyToMessage == null) {
                int lower_id = (int) did;
                if (lower_id > 0) {
                    TLRPC.User user2 = getMessagesController().getUser(Integer.valueOf(lower_id));
                    user = user2;
                    chat = null;
                } else {
                    TLRPC.Chat chat2 = getMessagesController().getChat(Integer.valueOf(-lower_id));
                    user = null;
                    chat = chat2;
                }
                if (user != null || chat != null) {
                    long messageId2 = draft.reply_to_msg_id;
                    if (ChatObject.isChannel(chat)) {
                        messageId = messageId2 | (chat.id << 32);
                        channelIdFinal = chat.id;
                    } else {
                        messageId = messageId2;
                        channelIdFinal = 0;
                    }
                    final long messageIdFinal = messageId;
                    final int i = channelIdFinal;
                    getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda44
                        @Override // java.lang.Runnable
                        public final void run() {
                            MediaDataController.this.m444lambda$saveDraft$105$imuecahxyvjdmessengerMediaDataController(messageIdFinal, i, did);
                        }
                    });
                }
            }
            getNotificationCenter().postNotificationName(NotificationCenter.newDraftReceived, Long.valueOf(did));
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$saveDraft$105$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m444lambda$saveDraft$105$imuecahxyvjdmessengerMediaDataController(long messageIdFinal, int channelIdFinal, final long did) {
        NativeByteBuffer data;
        TLRPC.Message message = null;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT data FROM messages WHERE mid = %d", Long.valueOf(messageIdFinal)), new Object[0]);
            if (cursor.next() && (data = cursor.byteBufferValue(0)) != null) {
                message = TLRPC.Message.TLdeserialize(data, data.readInt32(false), false);
                message.readAttachPath(data, getUserConfig().clientUserId);
                data.reuse();
            }
            cursor.dispose();
            if (message == null) {
                if (channelIdFinal != 0) {
                    TLRPC.TL_channels_getMessages req = new TLRPC.TL_channels_getMessages();
                    req.channel = getMessagesController().getInputChannel(channelIdFinal);
                    req.id.add(Integer.valueOf((int) messageIdFinal));
                    getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda45
                        @Override // im.uecahxyvjd.tgnet.RequestDelegate
                        public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                            MediaDataController.this.m442lambda$saveDraft$103$imuecahxyvjdmessengerMediaDataController(did, tLObject, tL_error);
                        }
                    });
                    return;
                }
                TLRPC.TL_messages_getMessages req2 = new TLRPC.TL_messages_getMessages();
                req2.id.add(Integer.valueOf((int) messageIdFinal));
                getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda46
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        MediaDataController.this.m443lambda$saveDraft$104$imuecahxyvjdmessengerMediaDataController(did, tLObject, tL_error);
                    }
                });
                return;
            }
            saveDraftReplyMessage(did, message);
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$saveDraft$103$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m442lambda$saveDraft$103$imuecahxyvjdmessengerMediaDataController(long did, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_Messages messagesRes = (TLRPC.messages_Messages) response;
            if (!messagesRes.messages.isEmpty()) {
                saveDraftReplyMessage(did, messagesRes.messages.get(0));
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$saveDraft$104$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m443lambda$saveDraft$104$imuecahxyvjdmessengerMediaDataController(long did, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_Messages messagesRes = (TLRPC.messages_Messages) response;
            if (!messagesRes.messages.isEmpty()) {
                saveDraftReplyMessage(did, messagesRes.messages.get(0));
            }
        }
    }

    private void saveDraftReplyMessage(final long did, final TLRPC.Message message) {
        if (message == null) {
            return;
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda24
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m445x4cd66474(did, message);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$saveDraftReplyMessage$106$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m445x4cd66474(long did, TLRPC.Message message) {
        TLRPC.DraftMessage draftMessage = this.drafts.get(did);
        if (draftMessage != null && draftMessage.reply_to_msg_id == message.id) {
            this.draftMessages.put(did, message);
            SerializedData serializedData = new SerializedData(message.getObjectSize());
            message.serializeToStream(serializedData);
            this.preferences.edit().putString("r_" + did, Utilities.bytesToHex(serializedData.toByteArray())).commit();
            getNotificationCenter().postNotificationName(NotificationCenter.newDraftReceived, Long.valueOf(did));
            serializedData.cleanup();
        }
    }

    public void clearAllDrafts() {
        this.drafts.clear();
        this.draftMessages.clear();
        this.preferences.edit().clear().commit();
        getMessagesController().sortDialogs(null);
        getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
    }

    public void cleanDraft(long did, boolean replyOnly) {
        TLRPC.DraftMessage draftMessage = this.drafts.get(did);
        if (draftMessage == null) {
            return;
        }
        if (!replyOnly) {
            this.drafts.remove(did);
            this.draftMessages.remove(did);
            this.preferences.edit().remove("" + did).remove("r_" + did).commit();
            getMessagesController().sortDialogs(null);
            getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
        } else if (draftMessage.reply_to_msg_id != 0) {
            draftMessage.reply_to_msg_id = 0;
            draftMessage.flags &= -2;
            saveDraft(did, draftMessage.message, draftMessage.entities, null, draftMessage.no_webpage, true);
        }
    }

    public void beginTransaction() {
        this.inTransaction = true;
    }

    public void endTransaction() {
        this.inTransaction = false;
    }

    public void clearBotKeyboard(final long did, final ArrayList<Integer> messages) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda83
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m346x40063b8f(messages, did);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$clearBotKeyboard$107$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m346x40063b8f(ArrayList messages, long did) {
        if (messages != null) {
            for (int a2 = 0; a2 < messages.size(); a2++) {
                long did1 = this.botKeyboardsByMids.get(((Integer) messages.get(a2)).intValue());
                if (did1 != 0) {
                    this.botKeyboards.remove(did1);
                    this.botKeyboardsByMids.delete(((Integer) messages.get(a2)).intValue());
                    getNotificationCenter().postNotificationName(NotificationCenter.botKeyboardDidLoad, null, Long.valueOf(did1));
                }
            }
            return;
        }
        this.botKeyboards.remove(did);
        getNotificationCenter().postNotificationName(NotificationCenter.botKeyboardDidLoad, null, Long.valueOf(did));
    }

    public void loadBotKeyboard(final long did) {
        TLRPC.Message keyboard = this.botKeyboards.get(did);
        if (keyboard != null) {
            getNotificationCenter().postNotificationName(NotificationCenter.botKeyboardDidLoad, keyboard, Long.valueOf(did));
        } else {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda53
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m374xe369e83e(did);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadBotKeyboard$109$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m374xe369e83e(final long did) {
        NativeByteBuffer data;
        TLRPC.Message botKeyboard = null;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT info FROM bot_keyboard WHERE uid = %d", Long.valueOf(did)), new Object[0]);
            if (cursor.next() && !cursor.isNull(0) && (data = cursor.byteBufferValue(0)) != null) {
                botKeyboard = TLRPC.Message.TLdeserialize(data, data.readInt32(false), false);
                data.reuse();
            }
            cursor.dispose();
            if (botKeyboard != null) {
                final TLRPC.Message botKeyboardFinal = botKeyboard;
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda33
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m373x48c925bd(botKeyboardFinal, did);
                    }
                });
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadBotKeyboard$108$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m373x48c925bd(TLRPC.Message botKeyboardFinal, long did) {
        getNotificationCenter().postNotificationName(NotificationCenter.botKeyboardDidLoad, botKeyboardFinal, Long.valueOf(did));
    }

    public void loadBotInfo(final int uid, boolean cache, final int classGuid) {
        TLRPC.BotInfo botInfo;
        if (cache && (botInfo = this.botInfos.get(uid)) != null) {
            getNotificationCenter().postNotificationName(NotificationCenter.botInfoDidLoad, botInfo, Integer.valueOf(classGuid));
        } else {
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda121
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m372x31d94a9c(uid, classGuid);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadBotInfo$111$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m372x31d94a9c(int uid, final int classGuid) {
        NativeByteBuffer data;
        TLRPC.BotInfo botInfo = null;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT info FROM bot_info WHERE uid = %d", Integer.valueOf(uid)), new Object[0]);
            if (cursor.next() && !cursor.isNull(0) && (data = cursor.byteBufferValue(0)) != null) {
                botInfo = TLRPC.BotInfo.TLdeserialize(data, data.readInt32(false), false);
                data.reuse();
            }
            cursor.dispose();
            if (botInfo != null) {
                final TLRPC.BotInfo botInfoFinal = botInfo;
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda116
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m371x9738881b(botInfoFinal, classGuid);
                    }
                });
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadBotInfo$110$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m371x9738881b(TLRPC.BotInfo botInfoFinal, int classGuid) {
        getNotificationCenter().postNotificationName(NotificationCenter.botInfoDidLoad, botInfoFinal, Integer.valueOf(classGuid));
    }

    public void putBotKeyboard(final long did, final TLRPC.Message message) {
        if (message == null) {
            return;
        }
        int mid = 0;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT mid FROM bot_keyboard WHERE uid = %d", Long.valueOf(did)), new Object[0]);
            if (cursor.next()) {
                mid = cursor.intValue(0);
            }
            cursor.dispose();
            if (mid >= message.id) {
                return;
            }
            SQLitePreparedStatement state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO bot_keyboard VALUES(?, ?, ?)");
            state.requery();
            NativeByteBuffer data = new NativeByteBuffer(message.getObjectSize());
            message.serializeToStream(data);
            state.bindLong(1, did);
            state.bindInteger(2, message.id);
            state.bindByteBuffer(3, data);
            state.step();
            data.reuse();
            state.dispose();
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda11
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m429x15c8c9e7(did, message);
                }
            });
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putBotKeyboard$112$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m429x15c8c9e7(long did, TLRPC.Message message) {
        TLRPC.Message old = this.botKeyboards.get(did);
        this.botKeyboards.put(did, message);
        if (old != null) {
            this.botKeyboardsByMids.delete(old.id);
        }
        this.botKeyboardsByMids.put(message.id, did);
        getNotificationCenter().postNotificationName(NotificationCenter.botKeyboardDidLoad, message, Long.valueOf(did));
    }

    public void putBotInfo(final TLRPC.BotInfo botInfo) {
        if (botInfo == null) {
            return;
        }
        this.botInfos.put(botInfo.user_id, botInfo);
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda125
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m428xcf72e5af(botInfo);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putBotInfo$113$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m428xcf72e5af(TLRPC.BotInfo botInfo) {
        try {
            SQLitePreparedStatement state = getMessagesStorage().getDatabase().executeFast("REPLACE INTO bot_info(uid, info) VALUES(?, ?)");
            state.requery();
            NativeByteBuffer data = new NativeByteBuffer(botInfo.getObjectSize());
            botInfo.serializeToStream(data);
            state.bindInteger(1, botInfo.user_id);
            state.bindByteBuffer(2, data);
            state.step();
            data.reuse();
            state.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public void fetchNewEmojiKeywords(String[] langCodes) {
        if (langCodes == null) {
            return;
        }
        for (final String langCode : langCodes) {
            if (TextUtils.isEmpty(langCode) || this.currentFetchingEmoji.get(langCode) != null) {
                return;
            }
            this.currentFetchingEmoji.put(langCode, true);
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda107
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m354x6ac4ccbf(langCode);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$fetchNewEmojiKeywords$119$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m354x6ac4ccbf(final String langCode) {
        TLObject request;
        int version = -1;
        String alias = null;
        long date = 0;
        try {
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized("SELECT alias, version, date FROM emoji_keywords_info_v2 WHERE lang = ?", langCode);
            if (cursor.next()) {
                alias = cursor.stringValue(0);
                version = cursor.intValue(1);
                date = cursor.longValue(2);
            }
            cursor.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
        if (!BuildVars.DEBUG_VERSION && Math.abs(System.currentTimeMillis() - date) < 3600000) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda57
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m349x65a1003a(langCode);
                }
            });
            return;
        }
        if (version == -1) {
            TLRPC.TL_messages_getEmojiKeywords req = new TLRPC.TL_messages_getEmojiKeywords();
            req.lang_code = langCode;
            request = req;
        } else {
            TLRPC.TL_messages_getEmojiKeywordsDifference req2 = new TLRPC.TL_messages_getEmojiKeywordsDifference();
            req2.lang_code = langCode;
            req2.from_version = version;
            request = req2;
        }
        final String aliasFinal = alias;
        final int versionFinal = version;
        getConnectionsManager().sendRequest(request, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda58
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                MediaDataController.this.m353xd0240a3e(versionFinal, aliasFinal, langCode, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$fetchNewEmojiKeywords$114$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m349x65a1003a(String langCode) {
        this.currentFetchingEmoji.remove(langCode);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$fetchNewEmojiKeywords$118$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m353xd0240a3e(int versionFinal, String aliasFinal, final String langCode, TLObject response, TLRPC.TL_error error) {
        if (response != null) {
            TLRPC.TL_emojiKeywordsDifference res = (TLRPC.TL_emojiKeywordsDifference) response;
            if (versionFinal != -1 && !res.lang_code.equals(aliasFinal)) {
                getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda1
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m351x9ae2853c(langCode);
                    }
                });
                return;
            } else {
                putEmojiKeywords(langCode, res);
                return;
            }
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda2
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m352x358347bd(langCode);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$fetchNewEmojiKeywords$116$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m351x9ae2853c(final String langCode) {
        try {
            SQLitePreparedStatement deleteState = getMessagesStorage().getDatabase().executeFast("DELETE FROM emoji_keywords_info_v2 WHERE lang = ?");
            deleteState.bindString(1, langCode);
            deleteState.step();
            deleteState.dispose();
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda102
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m350x41c2bb(langCode);
                }
            });
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$fetchNewEmojiKeywords$115$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m350x41c2bb(String langCode) {
        this.currentFetchingEmoji.remove(langCode);
        fetchNewEmojiKeywords(new String[]{langCode});
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$fetchNewEmojiKeywords$117$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m352x358347bd(String langCode) {
        this.currentFetchingEmoji.remove(langCode);
    }

    private void putEmojiKeywords(final String lang, final TLRPC.TL_emojiKeywordsDifference res) {
        if (res == null) {
            return;
        }
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda52
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m431xa0af44c7(res, lang);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:52:0x0133  */
    /* JADX WARN: Removed duplicated region for block: B:54:0x0138  */
    /* JADX WARN: Removed duplicated region for block: B:56:0x013d  */
    /* renamed from: lambda$putEmojiKeywords$121$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m431xa0af44c7(TLRPC.TL_emojiKeywordsDifference res, final String lang) {
        SQLitePreparedStatement insertState = null;
        SQLitePreparedStatement deleteState = null;
        SQLitePreparedStatement infoState = null;
        try {
            try {
                try {
                    if (!res.keywords.isEmpty()) {
                        SQLitePreparedStatement insertState2 = getMessagesStorage().getDatabase().executeFast("REPLACE INTO emoji_keywords_v2 VALUES(?, ?, ?)");
                        SQLitePreparedStatement deleteState2 = getMessagesStorage().getDatabase().executeFast("DELETE FROM emoji_keywords_v2 WHERE lang = ? AND keyword = ? AND emoji = ?");
                        try {
                            getMessagesStorage().getDatabase().beginTransaction();
                        } catch (Exception e) {
                            FileLog.e("putEmojiKeywords ---> exception 1 ", e);
                        }
                        int N = res.keywords.size();
                        for (int a2 = 0; a2 < N; a2++) {
                            TLRPC.EmojiKeyword keyword = res.keywords.get(a2);
                            if (keyword instanceof TLRPC.TL_emojiKeyword) {
                                TLRPC.TL_emojiKeyword emojiKeyword = (TLRPC.TL_emojiKeyword) keyword;
                                String key = emojiKeyword.keyword.toLowerCase();
                                int N2 = emojiKeyword.emoticons.size();
                                for (int b = 0; b < N2; b++) {
                                    insertState2.requery();
                                    insertState2.bindString(1, res.lang_code);
                                    insertState2.bindString(2, key);
                                    insertState2.bindString(3, emojiKeyword.emoticons.get(b));
                                    insertState2.step();
                                }
                            } else if (keyword instanceof TLRPC.TL_emojiKeywordDeleted) {
                                TLRPC.TL_emojiKeywordDeleted keywordDeleted = (TLRPC.TL_emojiKeywordDeleted) keyword;
                                String key2 = keywordDeleted.keyword.toLowerCase();
                                int N22 = keywordDeleted.emoticons.size();
                                for (int b2 = 0; b2 < N22; b2++) {
                                    deleteState2.requery();
                                    deleteState2.bindString(1, res.lang_code);
                                    deleteState2.bindString(2, key2);
                                    deleteState2.bindString(3, keywordDeleted.emoticons.get(b2));
                                    deleteState2.step();
                                }
                            }
                        }
                        getMessagesStorage().getDatabase().commitTransaction();
                        insertState2.dispose();
                        insertState = null;
                        deleteState2.dispose();
                        deleteState = null;
                    }
                    SQLitePreparedStatement infoState2 = getMessagesStorage().getDatabase().executeFast("REPLACE INTO emoji_keywords_info_v2 VALUES(?, ?, ?, ?)");
                    infoState2.bindString(1, lang);
                    infoState2.bindString(2, res.lang_code);
                    infoState2.bindInteger(3, res.version);
                    infoState2.bindLong(4, System.currentTimeMillis());
                    infoState2.step();
                    infoState2.dispose();
                    infoState = null;
                } catch (Throwable th) {
                    th = th;
                    if (insertState != null) {
                        insertState.dispose();
                    }
                    if (deleteState != null) {
                        deleteState.dispose();
                    }
                    if (infoState != null) {
                        infoState.dispose();
                    }
                    throw th;
                }
            } catch (Exception e2) {
                e = e2;
            }
            try {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda64
                    @Override // java.lang.Runnable
                    public final void run() {
                        MediaDataController.this.m430x60e8246(lang);
                    }
                });
                if (insertState != null) {
                    insertState.dispose();
                }
                if (deleteState != null) {
                    deleteState.dispose();
                }
                if (0 == 0) {
                    return;
                }
            } catch (Exception e3) {
                e = e3;
                FileLog.e("putEmojiKeywords ---> exception 2 ", e);
                if (insertState != null) {
                    insertState.dispose();
                }
                if (deleteState != null) {
                    deleteState.dispose();
                }
                if (infoState == null) {
                    return;
                }
                infoState.dispose();
            }
            infoState.dispose();
        } catch (Throwable th2) {
            th = th2;
            if (insertState != null) {
            }
            if (deleteState != null) {
            }
            if (infoState != null) {
            }
            throw th;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putEmojiKeywords$120$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m430x60e8246(String lang) {
        this.currentFetchingEmoji.remove(lang);
        getNotificationCenter().postNotificationName(NotificationCenter.newEmojiSuggestionsAvailable, lang);
    }

    public void getEmojiSuggestions(String[] langCodes, String keyword, boolean fullMatch, KeywordResultCallback callback) {
        getEmojiSuggestions(langCodes, keyword, fullMatch, callback, null);
    }

    public void getEmojiSuggestions(final String[] langCodes, final String keyword, final boolean fullMatch, final KeywordResultCallback callback, final CountDownLatch sync) {
        if (callback == null) {
            return;
        }
        if (TextUtils.isEmpty(keyword) || langCodes == null) {
            callback.run(new ArrayList<>(), null);
            return;
        }
        final ArrayList<String> recentEmoji = new ArrayList<>(Emoji.recentEmoji);
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda61
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.this.m356xb7196e2f(langCodes, callback, keyword, fullMatch, recentEmoji, sync);
            }
        });
        if (sync != null) {
            try {
                sync.await();
            } catch (Throwable th) {
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:53:0x0150  */
    /* JADX WARN: Removed duplicated region for block: B:54:0x0157  */
    /* renamed from: lambda$getEmojiSuggestions$125$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m356xb7196e2f(final String[] langCodes, final KeywordResultCallback callback, String keyword, boolean fullMatch, final ArrayList recentEmoji, CountDownLatch sync) {
        boolean z;
        int i;
        SQLiteCursor cursor;
        boolean hasAny;
        boolean hasAny2;
        final ArrayList<KeywordResult> result = new ArrayList<>();
        HashMap<String, Boolean> resultMap = new HashMap<>();
        int a2 = 0;
        String alias = null;
        boolean hasAny3 = false;
        while (true) {
            try {
                z = false;
                i = 1;
                if (a2 < langCodes.length) {
                    SQLiteCursor cursor2 = getMessagesStorage().getDatabase().queryFinalized("SELECT alias FROM emoji_keywords_info_v2 WHERE lang = ?", langCodes[a2]);
                    if (cursor2.next()) {
                        alias = cursor2.stringValue(0);
                    }
                    cursor2.dispose();
                    if (alias != null) {
                        hasAny3 = true;
                    }
                    a2++;
                } else {
                    try {
                        break;
                    } catch (Exception e) {
                        e = e;
                        FileLog.e(e);
                        Collections.sort(result, new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda118
                            @Override // java.util.Comparator
                            public final int compare(Object obj, Object obj2) {
                                return MediaDataController.lambda$getEmojiSuggestions$123(recentEmoji, (MediaDataController.KeywordResult) obj, (MediaDataController.KeywordResult) obj2);
                            }
                        });
                        final String aliasFinal = alias;
                        if (sync == null) {
                        }
                    }
                }
            } catch (Exception e2) {
                e = e2;
            }
        }
        if (!hasAny3) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda117
                @Override // java.lang.Runnable
                public final void run() {
                    MediaDataController.this.m355xe73726ac(langCodes, callback, result);
                }
            });
            return;
        }
        String key = keyword.toLowerCase();
        int a3 = 0;
        while (a3 < 2) {
            if (a3 == i) {
                String translitKey = LocaleController.getInstance().getTranslitString(key, z, z);
                if (translitKey.equals(key)) {
                    hasAny = hasAny3;
                    a3++;
                    hasAny3 = hasAny;
                    z = false;
                    i = 1;
                } else {
                    key = translitKey;
                }
            }
            String key2 = null;
            StringBuilder nextKey = new StringBuilder(key);
            int pos = nextKey.length();
            while (true) {
                if (pos <= 0) {
                    break;
                }
                pos--;
                char value = (char) (nextKey.charAt(pos) + 1);
                nextKey.setCharAt(pos, value);
                if (value != 0) {
                    key2 = nextKey.toString();
                    break;
                }
            }
            if (fullMatch) {
                SQLiteDatabase database = getMessagesStorage().getDatabase();
                Object[] objArr = new Object[i];
                objArr[0] = key;
                cursor = database.queryFinalized("SELECT emoji, keyword FROM emoji_keywords_v2 WHERE keyword = ?", objArr);
            } else if (key2 != null) {
                SQLiteDatabase database2 = getMessagesStorage().getDatabase();
                Object[] objArr2 = new Object[2];
                objArr2[0] = key;
                objArr2[i] = key2;
                cursor = database2.queryFinalized("SELECT emoji, keyword FROM emoji_keywords_v2 WHERE keyword >= ? AND keyword < ?", objArr2);
            } else {
                key = key + "%";
                SQLiteDatabase database3 = getMessagesStorage().getDatabase();
                Object[] objArr3 = new Object[i];
                objArr3[0] = key;
                cursor = database3.queryFinalized("SELECT emoji, keyword FROM emoji_keywords_v2 WHERE keyword LIKE ?", objArr3);
            }
            while (cursor.next()) {
                String value2 = cursor.stringValue(0).replace("️", "");
                if (resultMap.get(value2) != null) {
                    hasAny2 = hasAny3;
                } else {
                    resultMap.put(value2, true);
                    KeywordResult keywordResult = new KeywordResult();
                    keywordResult.emoji = value2;
                    hasAny2 = hasAny3;
                    keywordResult.keyword = cursor.stringValue(1);
                    result.add(keywordResult);
                }
                hasAny3 = hasAny2;
            }
            hasAny = hasAny3;
            cursor.dispose();
            a3++;
            hasAny3 = hasAny;
            z = false;
            i = 1;
        }
        Collections.sort(result, new Comparator() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda118
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return MediaDataController.lambda$getEmojiSuggestions$123(recentEmoji, (MediaDataController.KeywordResult) obj, (MediaDataController.KeywordResult) obj2);
            }
        });
        final String aliasFinal2 = alias;
        if (sync == null) {
            callback.run(result, aliasFinal2);
            sync.countDown();
            return;
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.MediaDataController$$ExternalSyntheticLambda119
            @Override // java.lang.Runnable
            public final void run() {
                MediaDataController.KeywordResultCallback.this.run(result, aliasFinal2);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getEmojiSuggestions$122$im-uecahxyvjd-messenger-MediaDataController  reason: not valid java name */
    public /* synthetic */ void m355xe73726ac(String[] langCodes, KeywordResultCallback callback, ArrayList result) {
        for (String str : langCodes) {
            if (this.currentFetchingEmoji.get(str) != null) {
                return;
            }
        }
        callback.run(result, null);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$getEmojiSuggestions$123(ArrayList recentEmoji, KeywordResult o1, KeywordResult o2) {
        int idx1 = recentEmoji.indexOf(o1.emoji);
        if (idx1 < 0) {
            idx1 = Integer.MAX_VALUE;
        }
        int idx2 = recentEmoji.indexOf(o2.emoji);
        if (idx2 < 0) {
            idx2 = Integer.MAX_VALUE;
        }
        if (idx1 < idx2) {
            return -1;
        }
        if (idx1 > idx2) {
            return 1;
        }
        int len1 = o1.keyword.length();
        int len2 = o2.keyword.length();
        if (len1 < len2) {
            return -1;
        }
        if (len1 > len2) {
            return 1;
        }
        return 0;
    }
}
