package im.uecahxyvjd.messenger;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.os.Build;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
/* loaded from: classes16.dex */
public class NativeLoader {
    private static final String LIB_NAME = "tmessages.31";
    private static final String LIB_SO_NAME = "libtmessages.31.so";
    private static final int LIB_VERSION = 31;
    private static final String LOCALE_LIB_SO_NAME = "libtmessages.31loc.so";
    private static volatile boolean nativeLoaded = false;
    private String crashPath = "";

    private static native void init(String str, boolean z);

    private static File getNativeLibraryDir(Context context) {
        File f = null;
        if (context != null) {
            try {
                f = new File((String) ApplicationInfo.class.getField("nativeLibraryDir").get(context.getApplicationInfo()));
            } catch (Throwable th) {
                th.printStackTrace();
            }
        }
        if (f == null) {
            f = new File(context.getApplicationInfo().dataDir, "lib");
        }
        if (f.isDirectory()) {
            return f;
        }
        return null;
    }

    private static boolean loadFromZip(Context context, File destDir, File destLocalFile, String folder) {
        File[] listFiles;
        try {
            for (File file : destDir.listFiles()) {
                file.delete();
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        ZipFile zipFile = null;
        InputStream stream = null;
        try {
            try {
                ZipFile zipFile2 = new ZipFile(context.getApplicationInfo().sourceDir);
                ZipEntry entry = zipFile2.getEntry("lib/" + folder + "/" + LIB_SO_NAME);
                if (entry != null) {
                    InputStream stream2 = zipFile2.getInputStream(entry);
                    OutputStream out = new FileOutputStream(destLocalFile);
                    byte[] buf = new byte[4096];
                    while (true) {
                        int len = stream2.read(buf);
                        if (len <= 0) {
                            break;
                        }
                        Thread.yield();
                        out.write(buf, 0, len);
                    }
                    out.close();
                    destLocalFile.setReadable(true, false);
                    destLocalFile.setExecutable(true, false);
                    destLocalFile.setWritable(true);
                    try {
                        System.load(destLocalFile.getAbsolutePath());
                        nativeLoaded = true;
                    } catch (Error e2) {
                        FileLog.e(e2);
                    }
                    if (stream2 != null) {
                        try {
                            stream2.close();
                        } catch (Exception e3) {
                            FileLog.e(e3);
                        }
                    }
                    try {
                        zipFile2.close();
                    } catch (Exception e4) {
                        FileLog.e(e4);
                    }
                    return true;
                }
                throw new Exception("Unable to find file in apk:lib/" + folder + "/" + LIB_NAME);
            } catch (Exception e5) {
                FileLog.e(e5);
                if (0 != 0) {
                    try {
                        stream.close();
                    } catch (Exception e6) {
                        FileLog.e(e6);
                    }
                }
                if (0 != 0) {
                    try {
                        zipFile.close();
                    } catch (Exception e7) {
                        FileLog.e(e7);
                    }
                }
                return false;
            }
        } catch (Throwable th) {
            if (0 != 0) {
                try {
                    stream.close();
                } catch (Exception e8) {
                    FileLog.e(e8);
                }
            }
            if (0 != 0) {
                try {
                    zipFile.close();
                } catch (Exception e9) {
                    FileLog.e(e9);
                }
            }
            throw th;
        }
    }

    /* JADX WARN: Removed duplicated region for block: B:59:0x00e5 A[Catch: all -> 0x001c, TryCatch #2 {, blocks: (B:4:0x0003, B:66:0x0107, B:69:0x0110, B:9:0x000a, B:11:0x0015, B:17:0x0020, B:18:0x0023, B:42:0x0098, B:44:0x00a0, B:47:0x00ab, B:49:0x00c6, B:51:0x00ca, B:52:0x00cf, B:57:0x00e1, B:59:0x00e5, B:60:0x00fb, B:56:0x00db, B:21:0x0032, B:24:0x003f, B:27:0x004c, B:30:0x0059, B:33:0x0066, B:36:0x0073, B:38:0x0079, B:41:0x0093), top: B:77:0x0003, inners: #0 }] */
    /* JADX WARN: Removed duplicated region for block: B:63:0x0102 A[RETURN] */
    /* JADX WARN: Removed duplicated region for block: B:75:0x0107 A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static synchronized void initNativeLibs(Context context) {
        String folder;
        synchronized (NativeLoader.class) {
            if (nativeLoaded) {
                return;
            }
            try {
                System.loadLibrary(LIB_NAME);
                nativeLoaded = true;
                if (BuildVars.LOGS_ENABLED) {
                    FileLog.d("loaded normal lib");
                }
            } catch (Error e) {
                FileLog.e(e);
                try {
                    String str = Build.CPU_ABI;
                    if (Build.CPU_ABI.equalsIgnoreCase("x86_64")) {
                        folder = "x86_64";
                    } else {
                        String folder2 = Build.CPU_ABI;
                        if (folder2.equalsIgnoreCase("arm64-v8a")) {
                            folder = "arm64-v8a";
                        } else {
                            String folder3 = Build.CPU_ABI;
                            if (folder3.equalsIgnoreCase("armeabi-v7a")) {
                                folder = "armeabi-v7a";
                            } else {
                                String folder4 = Build.CPU_ABI;
                                if (folder4.equalsIgnoreCase("armeabi")) {
                                    folder = "armeabi";
                                } else {
                                    String folder5 = Build.CPU_ABI;
                                    if (folder5.equalsIgnoreCase("x86")) {
                                        folder = "x86";
                                    } else {
                                        String folder6 = Build.CPU_ABI;
                                        if (folder6.equalsIgnoreCase("mips")) {
                                            folder = "mips";
                                        } else {
                                            folder = "armeabi";
                                            if (BuildVars.LOGS_ENABLED) {
                                                FileLog.e("Unsupported arch: " + Build.CPU_ABI);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                } catch (Exception e2) {
                    FileLog.e(e2);
                    folder = "armeabi";
                }
                String javaArch = System.getProperty("os.arch");
                if (javaArch != null && javaArch.contains("686")) {
                    folder = "x86";
                }
                File destDir = new File(context.getFilesDir(), "lib");
                destDir.mkdirs();
                File destLocalFile = new File(destDir, LOCALE_LIB_SO_NAME);
                if (destLocalFile.exists()) {
                    try {
                        if (BuildVars.LOGS_ENABLED) {
                            FileLog.d("Load local lib");
                        }
                        System.load(destLocalFile.getAbsolutePath());
                        nativeLoaded = true;
                        return;
                    } catch (Error e3) {
                        FileLog.e(e3);
                        destLocalFile.delete();
                        if (BuildVars.LOGS_ENABLED) {
                            FileLog.e("Library not found, arch = " + folder);
                        }
                        if (loadFromZip(context, destDir, destLocalFile, folder)) {
                            try {
                                System.loadLibrary(LIB_NAME);
                                nativeLoaded = true;
                            } catch (Error e4) {
                                FileLog.e(e4);
                            }
                            return;
                        }
                        return;
                    }
                }
                if (BuildVars.LOGS_ENABLED) {
                }
                if (loadFromZip(context, destDir, destLocalFile, folder)) {
                }
            }
        }
    }
}
