package im.uecahxyvjd.messenger;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.util.LongSparseArray;
import android.util.SparseArray;
import com.google.android.exoplayer2.util.MimeTypes;
import im.uecahxyvjd.messenger.SecretChatHelper;
import im.uecahxyvjd.messenger.SendMessagesHelper;
import im.uecahxyvjd.sqlite.SQLiteCursor;
import im.uecahxyvjd.tgnet.AbstractSerializedData;
import im.uecahxyvjd.tgnet.NativeByteBuffer;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLClassStore;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.AlertDialog;
import java.io.File;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Locale;
import java.util.concurrent.ConcurrentHashMap;
/* loaded from: classes16.dex */
public class SecretChatHelper extends BaseController {
    public static final int CURRENT_SECRET_CHAT_LAYER = 101;
    private static volatile SecretChatHelper[] Instance = new SecretChatHelper[3];
    private SparseArray<TLRPC.EncryptedChat> acceptingChats;
    public ArrayList<TLRPC.Update> delayedEncryptedChatUpdates;
    private ArrayList<Long> pendingEncMessagesToDelete;
    private SparseArray<ArrayList<TL_decryptedMessageHolder>> secretHolesQueue;
    private ArrayList<Integer> sendingNotifyLayer;
    private boolean startingSecretChat;

    /* loaded from: classes16.dex */
    public static class TL_decryptedMessageHolder extends TLObject {
        public static int constructor = 1431655929;
        public int date;
        public int decryptedWithVersion;
        public TLRPC.EncryptedFile file;
        public TLRPC.TL_decryptedMessageLayer layer;
        public boolean new_key_used;

        @Override // im.uecahxyvjd.tgnet.TLObject
        public void readParams(AbstractSerializedData stream, boolean exception) {
            stream.readInt64(exception);
            this.date = stream.readInt32(exception);
            this.layer = TLRPC.TL_decryptedMessageLayer.TLdeserialize(stream, stream.readInt32(exception), exception);
            if (stream.readBool(exception)) {
                this.file = TLRPC.EncryptedFile.TLdeserialize(stream, stream.readInt32(exception), exception);
            }
            this.new_key_used = stream.readBool(exception);
        }

        @Override // im.uecahxyvjd.tgnet.TLObject
        public void serializeToStream(AbstractSerializedData stream) {
            stream.writeInt32(constructor);
            stream.writeInt64(0L);
            stream.writeInt32(this.date);
            this.layer.serializeToStream(stream);
            stream.writeBool(this.file != null);
            TLRPC.EncryptedFile encryptedFile = this.file;
            if (encryptedFile != null) {
                encryptedFile.serializeToStream(stream);
            }
            stream.writeBool(this.new_key_used);
        }
    }

    public static SecretChatHelper getInstance(int num) {
        SecretChatHelper localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (SecretChatHelper.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    SecretChatHelper[] secretChatHelperArr = Instance;
                    SecretChatHelper secretChatHelper = new SecretChatHelper(num);
                    localInstance = secretChatHelper;
                    secretChatHelperArr[num] = secretChatHelper;
                }
            }
        }
        return localInstance;
    }

    public SecretChatHelper(int instance) {
        super(instance);
        this.sendingNotifyLayer = new ArrayList<>();
        this.secretHolesQueue = new SparseArray<>();
        this.acceptingChats = new SparseArray<>();
        this.delayedEncryptedChatUpdates = new ArrayList<>();
        this.pendingEncMessagesToDelete = new ArrayList<>();
        this.startingSecretChat = false;
    }

    public void cleanup() {
        this.sendingNotifyLayer.clear();
        this.acceptingChats.clear();
        this.secretHolesQueue.clear();
        this.delayedEncryptedChatUpdates.clear();
        this.pendingEncMessagesToDelete.clear();
        this.startingSecretChat = false;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void processPendingEncMessages() {
        if (!this.pendingEncMessagesToDelete.isEmpty()) {
            final ArrayList<Long> pendingEncMessagesToDeleteCopy = new ArrayList<>(this.pendingEncMessagesToDelete);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda7
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m939x9db1edf6(pendingEncMessagesToDeleteCopy);
                }
            });
            ArrayList<Long> arr = new ArrayList<>(this.pendingEncMessagesToDelete);
            getMessagesStorage().markMessagesAsDeletedByRandoms(arr);
            this.pendingEncMessagesToDelete.clear();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processPendingEncMessages$0$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m939x9db1edf6(ArrayList pendingEncMessagesToDeleteCopy) {
        for (int a2 = 0; a2 < pendingEncMessagesToDeleteCopy.size(); a2++) {
            MessageObject messageObject = getMessagesController().dialogMessagesByRandomIds.get(((Long) pendingEncMessagesToDeleteCopy.get(a2)).longValue());
            if (messageObject != null) {
                messageObject.deleted = true;
            }
        }
    }

    private TLRPC.TL_messageService createServiceSecretMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.DecryptedMessageAction decryptedMessage) {
        TLRPC.TL_messageService newMsg = new TLRPC.TL_messageService();
        newMsg.action = new TLRPC.TL_messageEncryptedAction();
        newMsg.action.encryptedAction = decryptedMessage;
        int newMessageId = getUserConfig().getNewMessageId();
        newMsg.id = newMessageId;
        newMsg.local_id = newMessageId;
        newMsg.from_id = getUserConfig().getClientUserId();
        newMsg.unread = true;
        newMsg.out = true;
        newMsg.flags = 256;
        newMsg.dialog_id = encryptedChat.id << 32;
        newMsg.to_id = new TLRPC.TL_peerUser();
        newMsg.send_state = 1;
        if (encryptedChat.participant_id == getUserConfig().getClientUserId()) {
            newMsg.to_id.user_id = encryptedChat.admin_id;
        } else {
            newMsg.to_id.user_id = encryptedChat.participant_id;
        }
        if ((decryptedMessage instanceof TLRPC.TL_decryptedMessageActionScreenshotMessages) || (decryptedMessage instanceof TLRPC.TL_decryptedMessageActionSetMessageTTL)) {
            newMsg.date = getConnectionsManager().getCurrentTime();
        } else {
            newMsg.date = 0;
        }
        newMsg.random_id = getSendMessagesHelper().getNextRandomId();
        getUserConfig().saveConfig(false);
        ArrayList<TLRPC.Message> arr = new ArrayList<>();
        arr.add(newMsg);
        getMessagesStorage().putMessages(arr, false, true, true, 0, false);
        return newMsg;
    }

    public void sendMessagesReadMessage(TLRPC.EncryptedChat encryptedChat, ArrayList<Long> random_ids, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionReadMessages();
            reqSend.action.random_ids = random_ids;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void processUpdateEncryption(TLRPC.TL_updateEncryption update, ConcurrentHashMap<Integer, TLRPC.User> usersDict) {
        final TLRPC.EncryptedChat newChat = update.chat;
        long dialog_id = newChat.id << 32;
        final TLRPC.EncryptedChat existingChat = getMessagesController().getEncryptedChatDB(newChat.id, false);
        if ((newChat instanceof TLRPC.TL_encryptedChatRequested) && existingChat == null) {
            int user_id = newChat.participant_id;
            if (user_id == getUserConfig().getClientUserId()) {
                user_id = newChat.admin_id;
            }
            TLRPC.User user = getMessagesController().getUser(Integer.valueOf(user_id));
            if (user == null) {
                user = usersDict.get(Integer.valueOf(user_id));
            }
            newChat.user_id = user_id;
            final TLRPC.Dialog dialog = new TLRPC.TL_dialog();
            dialog.id = dialog_id;
            dialog.unread_count = 0;
            dialog.top_message = 0;
            dialog.last_message_date = update.date;
            getMessagesController().putEncryptedChat(newChat, false);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda30
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m940x864d378(dialog);
                }
            });
            getMessagesStorage().putEncryptedChat(newChat, user, dialog);
            acceptSecretChat(newChat);
        } else if (newChat instanceof TLRPC.TL_encryptedChat) {
            if ((existingChat instanceof TLRPC.TL_encryptedChatWaiting) && (existingChat.auth_key == null || existingChat.auth_key.length == 1)) {
                newChat.a_or_b = existingChat.a_or_b;
                newChat.user_id = existingChat.user_id;
                processAcceptedSecretChat(newChat);
            } else if (existingChat == null && this.startingSecretChat) {
                this.delayedEncryptedChatUpdates.add(update);
            }
        } else {
            if (existingChat != null) {
                newChat.user_id = existingChat.user_id;
                newChat.auth_key = existingChat.auth_key;
                newChat.key_create_date = existingChat.key_create_date;
                newChat.key_use_count_in = existingChat.key_use_count_in;
                newChat.key_use_count_out = existingChat.key_use_count_out;
                newChat.ttl = existingChat.ttl;
                newChat.seq_in = existingChat.seq_in;
                newChat.seq_out = existingChat.seq_out;
                newChat.admin_id = existingChat.admin_id;
                newChat.mtproto_seq = existingChat.mtproto_seq;
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda1
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m941xfca0897(existingChat, newChat);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processUpdateEncryption$1$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m940x864d378(TLRPC.Dialog dialog) {
        getMessagesController().dialogs_dict.put(dialog.id, dialog);
        getMessagesController().allDialogs.add(dialog);
        getMessagesController().sortDialogs(null);
        getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processUpdateEncryption$2$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m941xfca0897(TLRPC.EncryptedChat exist, TLRPC.EncryptedChat newChat) {
        if (exist != null) {
            getMessagesController().putEncryptedChat(newChat, false);
        }
        getMessagesStorage().updateEncryptedChat(newChat);
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatUpdated, newChat);
    }

    public void sendMessagesDeleteMessage(TLRPC.EncryptedChat encryptedChat, ArrayList<Long> random_ids, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionDeleteMessages();
            reqSend.action.random_ids = random_ids;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendClearHistoryMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionFlushHistory();
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendNotifyLayerMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat) || this.sendingNotifyLayer.contains(Integer.valueOf(encryptedChat.id))) {
            return;
        }
        this.sendingNotifyLayer.add(Integer.valueOf(encryptedChat.id));
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionNotifyLayer();
            reqSend.action.layer = 101;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendRequestKeyMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionRequestKey();
            reqSend.action.exchange_id = encryptedChat.exchange_id;
            reqSend.action.g_a = encryptedChat.g_a;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendAcceptKeyMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionAcceptKey();
            reqSend.action.exchange_id = encryptedChat.exchange_id;
            reqSend.action.key_fingerprint = encryptedChat.future_key_fingerprint;
            reqSend.action.g_b = encryptedChat.g_a_or_b;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendCommitKeyMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionCommitKey();
            reqSend.action.exchange_id = encryptedChat.exchange_id;
            reqSend.action.key_fingerprint = encryptedChat.future_key_fingerprint;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendAbortKeyMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage, long excange_id) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionAbortKey();
            reqSend.action.exchange_id = excange_id;
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendNoopMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionNoop();
            message = createServiceSecretMessage(encryptedChat, reqSend.action);
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendTTLMessage(TLRPC.EncryptedChat encryptedChat, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionSetMessageTTL();
            reqSend.action.ttl_seconds = encryptedChat.ttl;
            TLRPC.Message message2 = createServiceSecretMessage(encryptedChat, reqSend.action);
            MessageObject newMsgObj = new MessageObject(this.currentAccount, message2, false);
            newMsgObj.messageOwner.send_state = 1;
            ArrayList<MessageObject> objArr = new ArrayList<>();
            objArr.add(newMsgObj);
            getMessagesController().updateInterfaceWithMessages(message2.dialog_id, objArr, false);
            getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
            message = message2;
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    public void sendScreenshotMessage(TLRPC.EncryptedChat encryptedChat, ArrayList<Long> random_ids, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (!(encryptedChat instanceof TLRPC.TL_encryptedChat)) {
            return;
        }
        TLRPC.TL_decryptedMessageService reqSend = new TLRPC.TL_decryptedMessageService();
        if (resendMessage != null) {
            reqSend.action = resendMessage.action.encryptedAction;
            message = resendMessage;
        } else {
            reqSend.action = new TLRPC.TL_decryptedMessageActionScreenshotMessages();
            reqSend.action.random_ids = random_ids;
            TLRPC.Message message2 = createServiceSecretMessage(encryptedChat, reqSend.action);
            MessageObject newMsgObj = new MessageObject(this.currentAccount, message2, false);
            newMsgObj.messageOwner.send_state = 1;
            ArrayList<MessageObject> objArr = new ArrayList<>();
            objArr.add(newMsgObj);
            getMessagesController().updateInterfaceWithMessages(message2.dialog_id, objArr, false);
            getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
            message = message2;
        }
        reqSend.random_id = message.random_id;
        performSendEncryptedRequest(reqSend, message, encryptedChat, null, null, null);
    }

    private void updateMediaPaths(MessageObject newMsgObj, TLRPC.EncryptedFile file, TLRPC.DecryptedMessage decryptedMessage, String originalPath) {
        TLRPC.Message newMsg = newMsgObj.messageOwner;
        if (file != null) {
            if ((newMsg.media instanceof TLRPC.TL_messageMediaPhoto) && newMsg.media.photo != null) {
                TLRPC.PhotoSize size = newMsg.media.photo.sizes.get(newMsg.media.photo.sizes.size() - 1);
                String fileName = size.location.volume_id + "_" + size.location.local_id;
                size.location = new TLRPC.TL_fileEncryptedLocation();
                size.location.key = decryptedMessage.media.key;
                size.location.iv = decryptedMessage.media.iv;
                size.location.dc_id = file.dc_id;
                size.location.volume_id = file.id;
                size.location.secret = file.access_hash;
                size.location.local_id = file.key_fingerprint;
                String fileName2 = size.location.volume_id + "_" + size.location.local_id;
                File cacheFile = new File(FileLoader.getDirectory(4), fileName + ".jpg");
                File cacheFile2 = FileLoader.getPathToAttach(size);
                cacheFile.renameTo(cacheFile2);
                ImageLoader.getInstance().replaceImageInCache(fileName, fileName2, ImageLocation.getForPhoto(size, newMsg.media.photo), true);
                ArrayList<TLRPC.Message> arr = new ArrayList<>();
                arr.add(newMsg);
                getMessagesStorage().putMessages(arr, false, true, false, 0, false);
            } else if ((newMsg.media instanceof TLRPC.TL_messageMediaDocument) && newMsg.media.document != null) {
                TLRPC.Document document = newMsg.media.document;
                newMsg.media.document = new TLRPC.TL_documentEncrypted();
                newMsg.media.document.id = file.id;
                newMsg.media.document.access_hash = file.access_hash;
                newMsg.media.document.date = document.date;
                newMsg.media.document.attributes = document.attributes;
                newMsg.media.document.mime_type = document.mime_type;
                newMsg.media.document.size = file.size;
                newMsg.media.document.key = decryptedMessage.media.key;
                newMsg.media.document.iv = decryptedMessage.media.iv;
                newMsg.media.document.thumbs = document.thumbs;
                newMsg.media.document.dc_id = file.dc_id;
                if (newMsg.media.document.thumbs.isEmpty()) {
                    TLRPC.PhotoSize thumb = new TLRPC.TL_photoSizeEmpty();
                    thumb.type = "s";
                    newMsg.media.document.thumbs.add(thumb);
                }
                if (newMsg.attachPath != null && newMsg.attachPath.startsWith(FileLoader.getDirectory(4).getAbsolutePath())) {
                    File cacheFile3 = new File(newMsg.attachPath);
                    File cacheFile22 = FileLoader.getPathToAttach(newMsg.media.document);
                    if (cacheFile3.renameTo(cacheFile22)) {
                        newMsgObj.mediaExists = newMsgObj.attachPathExists;
                        newMsgObj.attachPathExists = false;
                        newMsg.attachPath = "";
                    }
                }
                ArrayList<TLRPC.Message> arr2 = new ArrayList<>();
                arr2.add(newMsg);
                getMessagesStorage().putMessages(arr2, false, true, false, 0, false);
            }
        }
    }

    public static boolean isSecretVisibleMessage(TLRPC.Message message) {
        return (message.action instanceof TLRPC.TL_messageEncryptedAction) && ((message.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionScreenshotMessages) || (message.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionSetMessageTTL));
    }

    public static boolean isSecretInvisibleMessage(TLRPC.Message message) {
        return (!(message.action instanceof TLRPC.TL_messageEncryptedAction) || (message.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionScreenshotMessages) || (message.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionSetMessageTTL)) ? false : true;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void performSendEncryptedRequest(TLRPC.TL_messages_sendEncryptedMultiMedia req, SendMessagesHelper.DelayedMessage message) {
        for (int a2 = 0; a2 < req.files.size(); a2++) {
            performSendEncryptedRequest(req.messages.get(a2), message.messages.get(a2), message.encryptedChat, req.files.get(a2), message.originalPaths.get(a2), message.messageObjects.get(a2));
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void performSendEncryptedRequest(final TLRPC.DecryptedMessage req, final TLRPC.Message newMsgObj, final TLRPC.EncryptedChat chat, final TLRPC.InputEncryptedFile encryptedFile, final String originalPath, final MessageObject newMsg) {
        if (req != null && chat.auth_key != null && !(chat instanceof TLRPC.TL_encryptedChatRequested)) {
            if (chat instanceof TLRPC.TL_encryptedChatWaiting) {
                return;
            }
            getSendMessagesHelper().putToSendingMessages(newMsgObj, false);
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m933x176ba6f9(chat, req, newMsgObj, encryptedFile, newMsg, originalPath);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$performSendEncryptedRequest$7$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m933x176ba6f9(final TLRPC.EncryptedChat chat, final TLRPC.DecryptedMessage req, final TLRPC.Message newMsgObj, TLRPC.InputEncryptedFile encryptedFile, final MessageObject newMsg, final String originalPath) {
        int extraLen;
        byte[] messageKeyFull;
        TLObject reqToSend;
        try {
            TLRPC.TL_decryptedMessageLayer layer = new TLRPC.TL_decryptedMessageLayer();
            int myLayer = Math.max(46, AndroidUtilities.getMyLayerVersion(chat.layer));
            layer.layer = Math.min(myLayer, Math.max(46, AndroidUtilities.getPeerLayerVersion(chat.layer)));
            layer.message = req;
            layer.random_bytes = new byte[15];
            Utilities.random.nextBytes(layer.random_bytes);
            int mtprotoVersion = AndroidUtilities.getPeerLayerVersion(chat.layer) >= 73 ? 2 : 1;
            if (chat.seq_in == 0 && chat.seq_out == 0) {
                if (chat.admin_id != getUserConfig().getClientUserId()) {
                    chat.seq_in = -1;
                } else {
                    chat.seq_out = 1;
                    chat.seq_in = -2;
                }
            }
            if (newMsgObj.seq_in == 0 && newMsgObj.seq_out == 0) {
                layer.in_seq_no = chat.seq_in > 0 ? chat.seq_in : chat.seq_in + 2;
                layer.out_seq_no = chat.seq_out;
                chat.seq_out += 2;
                if (AndroidUtilities.getPeerLayerVersion(chat.layer) >= 20) {
                    if (chat.key_create_date == 0) {
                        chat.key_create_date = getConnectionsManager().getCurrentTime();
                    }
                    chat.key_use_count_out = (short) (chat.key_use_count_out + 1);
                    if ((chat.key_use_count_out >= 100 || chat.key_create_date < getConnectionsManager().getCurrentTime() - 604800) && chat.exchange_id == 0 && chat.future_key_fingerprint == 0) {
                        requestNewSecretChatKey(chat);
                    }
                }
                getMessagesStorage().updateEncryptedChatSeq(chat, false);
                if (newMsgObj != null) {
                    newMsgObj.seq_in = layer.in_seq_no;
                    newMsgObj.seq_out = layer.out_seq_no;
                    getMessagesStorage().setMessageSeq(newMsgObj.id, newMsgObj.seq_in, newMsgObj.seq_out);
                }
            } else {
                layer.in_seq_no = newMsgObj.seq_in;
                layer.out_seq_no = newMsgObj.seq_out;
            }
            if (BuildVars.LOGS_ENABLED) {
                FileLog.d(req + " send message with in_seq = " + layer.in_seq_no + " out_seq = " + layer.out_seq_no);
            }
            int len = layer.getObjectSize();
            NativeByteBuffer toEncrypt = new NativeByteBuffer(len + 4);
            toEncrypt.writeInt32(len);
            layer.serializeToStream(toEncrypt);
            int len2 = toEncrypt.length();
            int extraLen2 = len2 % 16 != 0 ? 16 - (len2 % 16) : 0;
            if (mtprotoVersion != 2) {
                extraLen = extraLen2;
            } else {
                extraLen = extraLen2 + ((Utilities.random.nextInt(3) + 2) * 16);
            }
            NativeByteBuffer dataForEncryption = new NativeByteBuffer(len2 + extraLen);
            toEncrypt.position(0);
            dataForEncryption.writeBytes(toEncrypt);
            if (extraLen != 0) {
                byte[] b = new byte[extraLen];
                Utilities.random.nextBytes(b);
                dataForEncryption.writeBytes(b);
            }
            byte[] b2 = new byte[16];
            boolean incoming = mtprotoVersion == 2 && chat.admin_id != getUserConfig().getClientUserId();
            if (mtprotoVersion == 2) {
                byte[] messageKeyFull2 = Utilities.computeSHA256(chat.auth_key, 88 + (incoming ? 8 : 0), 32, dataForEncryption.buffer, 0, dataForEncryption.buffer.limit());
                System.arraycopy(messageKeyFull2, 8, b2, 0, 16);
                messageKeyFull = messageKeyFull2;
            } else {
                byte[] messageKeyFull3 = Utilities.computeSHA1(toEncrypt.buffer);
                System.arraycopy(messageKeyFull3, messageKeyFull3.length - 16, b2, 0, 16);
                messageKeyFull = messageKeyFull3;
            }
            toEncrypt.reuse();
            MessageKeyData keyData = MessageKeyData.generateMessageKeyData(chat.auth_key, b2, incoming, mtprotoVersion);
            Utilities.aesIgeEncryption(dataForEncryption.buffer, keyData.aesKey, keyData.aesIv, true, false, 0, dataForEncryption.limit());
            NativeByteBuffer data = new NativeByteBuffer(b2.length + 8 + dataForEncryption.length());
            dataForEncryption.position(0);
            data.writeInt64(chat.key_fingerprint);
            data.writeBytes(b2);
            data.writeBytes(dataForEncryption);
            dataForEncryption.reuse();
            data.position(0);
            if (encryptedFile == null) {
                if (req instanceof TLRPC.TL_decryptedMessageService) {
                    TLRPC.TL_messages_sendEncryptedService req2 = new TLRPC.TL_messages_sendEncryptedService();
                    req2.data = data;
                    req2.random_id = req.random_id;
                    req2.peer = new TLRPC.TL_inputEncryptedChat();
                    req2.peer.chat_id = chat.id;
                    req2.peer.access_hash = chat.access_hash;
                    reqToSend = req2;
                } else {
                    TLRPC.TL_messages_sendEncrypted req22 = new TLRPC.TL_messages_sendEncrypted();
                    req22.data = data;
                    req22.random_id = req.random_id;
                    req22.peer = new TLRPC.TL_inputEncryptedChat();
                    req22.peer.chat_id = chat.id;
                    req22.peer.access_hash = chat.access_hash;
                    reqToSend = req22;
                }
            } else {
                TLRPC.TL_messages_sendEncryptedFile req23 = new TLRPC.TL_messages_sendEncryptedFile();
                req23.data = data;
                req23.random_id = req.random_id;
                req23.peer = new TLRPC.TL_inputEncryptedChat();
                req23.peer.chat_id = chat.id;
                req23.peer.access_hash = chat.access_hash;
                req23.file = encryptedFile;
                reqToSend = req23;
            }
            getConnectionsManager().sendRequest(reqToSend, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda17
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    SecretChatHelper.this.m932x100671da(req, chat, newMsgObj, newMsg, originalPath, tLObject, tL_error);
                }
            }, 64);
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendEncryptedRequest$6$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m932x100671da(TLRPC.DecryptedMessage req, TLRPC.EncryptedChat chat, final TLRPC.Message newMsgObj, MessageObject newMsg, String originalPath, TLObject response, TLRPC.TL_error error) {
        int existFlags;
        TLRPC.EncryptedChat currentChat;
        if (error == null && (req.action instanceof TLRPC.TL_decryptedMessageActionNotifyLayer)) {
            TLRPC.EncryptedChat currentChat2 = getMessagesController().getEncryptedChat(Integer.valueOf(chat.id));
            if (currentChat2 != null) {
                currentChat = currentChat2;
            } else {
                currentChat = chat;
            }
            if (currentChat.key_hash == null) {
                currentChat.key_hash = AndroidUtilities.calcAuthKeyHash(currentChat.auth_key);
            }
            if (AndroidUtilities.getPeerLayerVersion(currentChat.layer) >= 46 && currentChat.key_hash.length == 16) {
                try {
                    byte[] sha256 = Utilities.computeSHA256(chat.auth_key, 0, chat.auth_key.length);
                    byte[] key_hash = new byte[36];
                    System.arraycopy(chat.key_hash, 0, key_hash, 0, 16);
                    System.arraycopy(sha256, 0, key_hash, 16, 20);
                    currentChat.key_hash = key_hash;
                    getMessagesStorage().updateEncryptedChat(currentChat);
                } catch (Throwable e) {
                    FileLog.e(e);
                }
            }
            this.sendingNotifyLayer.remove(Integer.valueOf(currentChat.id));
            currentChat.layer = AndroidUtilities.setMyLayerVersion(currentChat.layer, 101);
            getMessagesStorage().updateEncryptedChatLayer(currentChat);
        }
        if (newMsgObj != null) {
            if (error != null) {
                getMessagesStorage().markMessageAsSendError(newMsgObj, false);
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda13
                    @Override // java.lang.Runnable
                    public final void run() {
                        SecretChatHelper.this.m931x8a13cbb(newMsgObj);
                    }
                });
                return;
            }
            final String attachPath = newMsgObj.attachPath;
            final TLRPC.messages_SentEncryptedMessage res = (TLRPC.messages_SentEncryptedMessage) response;
            if (isSecretVisibleMessage(newMsgObj)) {
                newMsgObj.date = res.date;
            }
            if (newMsg != null && (res.file instanceof TLRPC.TL_encryptedFile)) {
                updateMediaPaths(newMsg, res.file, req, originalPath);
                existFlags = newMsg.getMediaExistanceFlags();
            } else {
                existFlags = 0;
            }
            final int i = existFlags;
            getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda12
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m930x13c079c(newMsgObj, res, i, attachPath);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendEncryptedRequest$4$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m930x13c079c(final TLRPC.Message newMsgObj, TLRPC.messages_SentEncryptedMessage res, final int existFlags, final String attachPath) {
        if (isSecretInvisibleMessage(newMsgObj)) {
            res.date = 0;
        }
        getMessagesStorage().updateMessageStateAndId(newMsgObj.random_id, Integer.valueOf(newMsgObj.id), newMsgObj.id, res.date, false, 0, 0);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda23
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m929xf9d6d27d(newMsgObj, existFlags, attachPath);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendEncryptedRequest$3$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m929xf9d6d27d(TLRPC.Message newMsgObj, int existFlags, String attachPath) {
        newMsgObj.send_state = 0;
        getNotificationCenter().postNotificationName(NotificationCenter.messageReceivedByServer, Integer.valueOf(newMsgObj.id), Integer.valueOf(newMsgObj.id), newMsgObj, Long.valueOf(newMsgObj.dialog_id), 0L, Integer.valueOf(existFlags), false);
        getSendMessagesHelper().processSentMessage(newMsgObj.id);
        if (MessageObject.isVideoMessage(newMsgObj) || MessageObject.isNewGifMessage(newMsgObj) || MessageObject.isRoundVideoMessage(newMsgObj)) {
            getSendMessagesHelper().stopVideoService(attachPath);
        }
        getSendMessagesHelper().removeFromSendingMessages(newMsgObj.id, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendEncryptedRequest$5$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m931x8a13cbb(TLRPC.Message newMsgObj) {
        newMsgObj.send_state = 2;
        getNotificationCenter().postNotificationName(NotificationCenter.messageSendError, Integer.valueOf(newMsgObj.id));
        getSendMessagesHelper().processSentMessage(newMsgObj.id);
        if (MessageObject.isVideoMessage(newMsgObj) || MessageObject.isNewGifMessage(newMsgObj) || MessageObject.isRoundVideoMessage(newMsgObj)) {
            getSendMessagesHelper().stopVideoService(newMsgObj.attachPath);
        }
        getSendMessagesHelper().removeFromSendingMessages(newMsgObj.id, false);
    }

    private void applyPeerLayer(final TLRPC.EncryptedChat chat, int newPeerLayer) {
        int currentPeerLayer = AndroidUtilities.getPeerLayerVersion(chat.layer);
        if (newPeerLayer <= currentPeerLayer) {
            return;
        }
        if (chat.key_hash.length == 16 && currentPeerLayer >= 46) {
            try {
                byte[] sha256 = Utilities.computeSHA256(chat.auth_key, 0, chat.auth_key.length);
                byte[] key_hash = new byte[36];
                System.arraycopy(chat.key_hash, 0, key_hash, 0, 16);
                System.arraycopy(sha256, 0, key_hash, 16, 20);
                chat.key_hash = key_hash;
                getMessagesStorage().updateEncryptedChat(chat);
            } catch (Throwable e) {
                FileLog.e(e);
            }
        }
        chat.layer = AndroidUtilities.setPeerLayerVersion(chat.layer, newPeerLayer);
        getMessagesStorage().updateEncryptedChatLayer(chat);
        if (currentPeerLayer < 101) {
            sendNotifyLayerMessage(chat, null);
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda25
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m927lambda$applyPeerLayer$8$imuecahxyvjdmessengerSecretChatHelper(chat);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$applyPeerLayer$8$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m927lambda$applyPeerLayer$8$imuecahxyvjdmessengerSecretChatHelper(TLRPC.EncryptedChat chat) {
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatUpdated, chat);
    }

    public TLRPC.Message processDecryptedObject(TLRPC.EncryptedChat chat, TLRPC.EncryptedFile file, int date, TLObject object, boolean new_key_used) {
        int i;
        TLRPC.TL_message newMessage;
        TLRPC.PhotoSize photoSize;
        TLRPC.PhotoSize photoSize2;
        if (object != null) {
            int from_id = chat.admin_id;
            if (from_id == getUserConfig().getClientUserId()) {
                from_id = chat.participant_id;
            }
            if (AndroidUtilities.getPeerLayerVersion(chat.layer) >= 20 && chat.exchange_id == 0 && chat.future_key_fingerprint == 0 && chat.key_use_count_in >= 120) {
                requestNewSecretChatKey(chat);
            }
            if (chat.exchange_id == 0 && chat.future_key_fingerprint != 0 && !new_key_used) {
                chat.future_auth_key = new byte[256];
                chat.future_key_fingerprint = 0L;
                getMessagesStorage().updateEncryptedChat(chat);
            } else if (chat.exchange_id != 0 && new_key_used) {
                chat.key_fingerprint = chat.future_key_fingerprint;
                chat.auth_key = chat.future_auth_key;
                chat.key_create_date = getConnectionsManager().getCurrentTime();
                chat.future_auth_key = new byte[256];
                chat.future_key_fingerprint = 0L;
                chat.key_use_count_in = (short) 0;
                chat.key_use_count_out = (short) 0;
                chat.exchange_id = 0L;
                getMessagesStorage().updateEncryptedChat(chat);
            }
            if (object instanceof TLRPC.TL_decryptedMessage) {
                TLRPC.TL_decryptedMessage decryptedMessage = (TLRPC.TL_decryptedMessage) object;
                if (AndroidUtilities.getPeerLayerVersion(chat.layer) >= 17) {
                    newMessage = new TLRPC.TL_message_secret();
                    newMessage.ttl = decryptedMessage.ttl;
                    newMessage.entities = decryptedMessage.entities;
                } else {
                    newMessage = new TLRPC.TL_message();
                    newMessage.ttl = chat.ttl;
                }
                newMessage.message = decryptedMessage.message;
                newMessage.date = date;
                int newMessageId = getUserConfig().getNewMessageId();
                newMessage.id = newMessageId;
                newMessage.local_id = newMessageId;
                getUserConfig().saveConfig(false);
                newMessage.from_id = from_id;
                newMessage.to_id = new TLRPC.TL_peerUser();
                newMessage.random_id = decryptedMessage.random_id;
                newMessage.to_id.user_id = getUserConfig().getClientUserId();
                newMessage.unread = true;
                newMessage.flags = 768;
                if (decryptedMessage.via_bot_name != null && decryptedMessage.via_bot_name.length() > 0) {
                    newMessage.via_bot_name = decryptedMessage.via_bot_name;
                    newMessage.flags |= 2048;
                }
                if (decryptedMessage.grouped_id != 0) {
                    newMessage.grouped_id = decryptedMessage.grouped_id;
                    newMessage.flags |= 131072;
                }
                newMessage.dialog_id = chat.id << 32;
                if (decryptedMessage.reply_to_random_id != 0) {
                    newMessage.reply_to_random_id = decryptedMessage.reply_to_random_id;
                    newMessage.flags |= 8;
                }
                if (decryptedMessage.media == null || (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaEmpty)) {
                    newMessage.media = new TLRPC.TL_messageMediaEmpty();
                } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaWebPage) {
                    newMessage.media = new TLRPC.TL_messageMediaWebPage();
                    newMessage.media.webpage = new TLRPC.TL_webPageUrlPending();
                    newMessage.media.webpage.url = decryptedMessage.media.url;
                } else {
                    if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaContact) {
                        newMessage.media = new TLRPC.TL_messageMediaContact();
                        newMessage.media.last_name = decryptedMessage.media.last_name;
                        newMessage.media.first_name = decryptedMessage.media.first_name;
                        newMessage.media.phone_number = decryptedMessage.media.phone_number;
                        newMessage.media.user_id = decryptedMessage.media.user_id;
                        newMessage.media.vcard = "";
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaGeoPoint) {
                        newMessage.media = new TLRPC.TL_messageMediaGeo();
                        newMessage.media.geo = new TLRPC.TL_geoPoint();
                        newMessage.media.geo.lat = decryptedMessage.media.lat;
                        newMessage.media.geo._long = decryptedMessage.media._long;
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaPhoto) {
                        if (decryptedMessage.media.key == null || decryptedMessage.media.key.length != 32 || decryptedMessage.media.iv == null || decryptedMessage.media.iv.length != 32) {
                            return null;
                        }
                        newMessage.media = new TLRPC.TL_messageMediaPhoto();
                        newMessage.media.flags |= 3;
                        newMessage.message = decryptedMessage.media.caption != null ? decryptedMessage.media.caption : "";
                        newMessage.media.photo = new TLRPC.TL_photo();
                        newMessage.media.photo.file_reference = new byte[0];
                        newMessage.media.photo.date = newMessage.date;
                        byte[] thumb = ((TLRPC.TL_decryptedMessageMediaPhoto) decryptedMessage.media).thumb;
                        if (thumb != null && thumb.length != 0 && thumb.length <= 6000 && decryptedMessage.media.thumb_w <= 100 && decryptedMessage.media.thumb_h <= 100) {
                            TLRPC.TL_photoCachedSize small = new TLRPC.TL_photoCachedSize();
                            small.w = decryptedMessage.media.thumb_w;
                            small.h = decryptedMessage.media.thumb_h;
                            small.bytes = thumb;
                            small.type = "s";
                            small.location = new TLRPC.TL_fileLocationUnavailable();
                            newMessage.media.photo.sizes.add(small);
                        }
                        if (newMessage.ttl != 0) {
                            newMessage.media.ttl_seconds = newMessage.ttl;
                            newMessage.media.flags |= 4;
                        }
                        TLRPC.TL_photoSize big = new TLRPC.TL_photoSize();
                        big.w = decryptedMessage.media.w;
                        big.h = decryptedMessage.media.h;
                        big.type = "x";
                        big.size = file.size;
                        big.location = new TLRPC.TL_fileEncryptedLocation();
                        big.location.key = decryptedMessage.media.key;
                        big.location.iv = decryptedMessage.media.iv;
                        big.location.dc_id = file.dc_id;
                        big.location.volume_id = file.id;
                        big.location.secret = file.access_hash;
                        big.location.local_id = file.key_fingerprint;
                        newMessage.media.photo.sizes.add(big);
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaVideo) {
                        if (decryptedMessage.media.key == null || decryptedMessage.media.key.length != 32 || decryptedMessage.media.iv == null || decryptedMessage.media.iv.length != 32) {
                            return null;
                        }
                        newMessage.media = new TLRPC.TL_messageMediaDocument();
                        newMessage.media.flags |= 3;
                        newMessage.media.document = new TLRPC.TL_documentEncrypted();
                        newMessage.media.document.key = decryptedMessage.media.key;
                        newMessage.media.document.iv = decryptedMessage.media.iv;
                        newMessage.media.document.dc_id = file.dc_id;
                        newMessage.message = decryptedMessage.media.caption != null ? decryptedMessage.media.caption : "";
                        newMessage.media.document.date = date;
                        newMessage.media.document.size = file.size;
                        newMessage.media.document.id = file.id;
                        newMessage.media.document.access_hash = file.access_hash;
                        newMessage.media.document.mime_type = decryptedMessage.media.mime_type;
                        if (newMessage.media.document.mime_type == null) {
                            newMessage.media.document.mime_type = MimeTypes.VIDEO_MP4;
                        }
                        byte[] thumb2 = ((TLRPC.TL_decryptedMessageMediaVideo) decryptedMessage.media).thumb;
                        if (thumb2 != null && thumb2.length != 0 && thumb2.length <= 6000 && decryptedMessage.media.thumb_w <= 100 && decryptedMessage.media.thumb_h <= 100) {
                            photoSize2 = new TLRPC.TL_photoCachedSize();
                            photoSize2.bytes = thumb2;
                            photoSize2.w = decryptedMessage.media.thumb_w;
                            photoSize2.h = decryptedMessage.media.thumb_h;
                            photoSize2.type = "s";
                            photoSize2.location = new TLRPC.TL_fileLocationUnavailable();
                        } else {
                            photoSize2 = new TLRPC.TL_photoSizeEmpty();
                            photoSize2.type = "s";
                        }
                        newMessage.media.document.thumbs.add(photoSize2);
                        newMessage.media.document.flags |= 1;
                        TLRPC.TL_documentAttributeVideo attributeVideo = new TLRPC.TL_documentAttributeVideo();
                        attributeVideo.w = decryptedMessage.media.w;
                        attributeVideo.h = decryptedMessage.media.h;
                        attributeVideo.duration = decryptedMessage.media.duration;
                        attributeVideo.supports_streaming = false;
                        newMessage.media.document.attributes.add(attributeVideo);
                        if (newMessage.ttl != 0) {
                            newMessage.media.ttl_seconds = newMessage.ttl;
                            newMessage.media.flags |= 4;
                        }
                        if (newMessage.ttl != 0) {
                            newMessage.ttl = Math.max(decryptedMessage.media.duration + 1, newMessage.ttl);
                        }
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaDocument) {
                        if (decryptedMessage.media.key == null || decryptedMessage.media.key.length != 32 || decryptedMessage.media.iv == null || decryptedMessage.media.iv.length != 32) {
                            return null;
                        }
                        newMessage.media = new TLRPC.TL_messageMediaDocument();
                        newMessage.media.flags |= 3;
                        newMessage.message = decryptedMessage.media.caption != null ? decryptedMessage.media.caption : "";
                        newMessage.media.document = new TLRPC.TL_documentEncrypted();
                        newMessage.media.document.id = file.id;
                        newMessage.media.document.access_hash = file.access_hash;
                        newMessage.media.document.date = date;
                        if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaDocument_layer8) {
                            TLRPC.TL_documentAttributeFilename fileName = new TLRPC.TL_documentAttributeFilename();
                            fileName.file_name = decryptedMessage.media.file_name;
                            newMessage.media.document.attributes.add(fileName);
                        } else {
                            newMessage.media.document.attributes = decryptedMessage.media.attributes;
                        }
                        newMessage.media.document.mime_type = decryptedMessage.media.mime_type;
                        newMessage.media.document.size = decryptedMessage.media.size != 0 ? Math.min(decryptedMessage.media.size, file.size) : file.size;
                        newMessage.media.document.key = decryptedMessage.media.key;
                        newMessage.media.document.iv = decryptedMessage.media.iv;
                        if (newMessage.media.document.mime_type == null) {
                            newMessage.media.document.mime_type = "";
                        }
                        byte[] thumb3 = ((TLRPC.TL_decryptedMessageMediaDocument) decryptedMessage.media).thumb;
                        if (thumb3 != null && thumb3.length != 0 && thumb3.length <= 6000 && decryptedMessage.media.thumb_w <= 100 && decryptedMessage.media.thumb_h <= 100) {
                            photoSize = new TLRPC.TL_photoCachedSize();
                            photoSize.bytes = thumb3;
                            photoSize.w = decryptedMessage.media.thumb_w;
                            photoSize.h = decryptedMessage.media.thumb_h;
                            photoSize.type = "s";
                            photoSize.location = new TLRPC.TL_fileLocationUnavailable();
                        } else {
                            photoSize = new TLRPC.TL_photoSizeEmpty();
                            photoSize.type = "s";
                        }
                        newMessage.media.document.thumbs.add(photoSize);
                        newMessage.media.document.flags |= 1;
                        newMessage.media.document.dc_id = file.dc_id;
                        if (MessageObject.isVoiceMessage(newMessage) || MessageObject.isRoundVideoMessage(newMessage)) {
                            newMessage.media_unread = true;
                        }
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaExternalDocument) {
                        newMessage.media = new TLRPC.TL_messageMediaDocument();
                        newMessage.media.flags |= 3;
                        newMessage.message = "";
                        newMessage.media.document = new TLRPC.TL_document();
                        newMessage.media.document.id = decryptedMessage.media.id;
                        newMessage.media.document.access_hash = decryptedMessage.media.access_hash;
                        newMessage.media.document.file_reference = new byte[0];
                        newMessage.media.document.date = decryptedMessage.media.date;
                        newMessage.media.document.attributes = decryptedMessage.media.attributes;
                        newMessage.media.document.mime_type = decryptedMessage.media.mime_type;
                        newMessage.media.document.dc_id = decryptedMessage.media.dc_id;
                        newMessage.media.document.size = decryptedMessage.media.size;
                        newMessage.media.document.thumbs.add(((TLRPC.TL_decryptedMessageMediaExternalDocument) decryptedMessage.media).thumb);
                        newMessage.media.document.flags |= 1;
                        if (newMessage.media.document.mime_type == null) {
                            newMessage.media.document.mime_type = "";
                        }
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaAudio) {
                        if (decryptedMessage.media.key == null || decryptedMessage.media.key.length != 32 || decryptedMessage.media.iv == null || decryptedMessage.media.iv.length != 32) {
                            return null;
                        }
                        newMessage.media = new TLRPC.TL_messageMediaDocument();
                        newMessage.media.flags |= 3;
                        newMessage.media.document = new TLRPC.TL_documentEncrypted();
                        newMessage.media.document.key = decryptedMessage.media.key;
                        newMessage.media.document.iv = decryptedMessage.media.iv;
                        newMessage.media.document.id = file.id;
                        newMessage.media.document.access_hash = file.access_hash;
                        newMessage.media.document.date = date;
                        newMessage.media.document.size = file.size;
                        newMessage.media.document.dc_id = file.dc_id;
                        newMessage.media.document.mime_type = decryptedMessage.media.mime_type;
                        newMessage.message = decryptedMessage.media.caption != null ? decryptedMessage.media.caption : "";
                        if (newMessage.media.document.mime_type == null) {
                            newMessage.media.document.mime_type = "audio/ogg";
                        }
                        TLRPC.TL_documentAttributeAudio attributeAudio = new TLRPC.TL_documentAttributeAudio();
                        attributeAudio.duration = decryptedMessage.media.duration;
                        attributeAudio.voice = true;
                        newMessage.media.document.attributes.add(attributeAudio);
                        if (newMessage.ttl != 0) {
                            newMessage.ttl = Math.max(decryptedMessage.media.duration + 1, newMessage.ttl);
                        }
                        if (newMessage.media.document.thumbs.isEmpty()) {
                            TLRPC.PhotoSize thumb4 = new TLRPC.TL_photoSizeEmpty();
                            thumb4.type = "s";
                            newMessage.media.document.thumbs.add(thumb4);
                        }
                    } else if (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaVenue) {
                        newMessage.media = new TLRPC.TL_messageMediaVenue();
                        newMessage.media.geo = new TLRPC.TL_geoPoint();
                        newMessage.media.geo.lat = decryptedMessage.media.lat;
                        newMessage.media.geo._long = decryptedMessage.media._long;
                        newMessage.media.title = decryptedMessage.media.title;
                        newMessage.media.address = decryptedMessage.media.address;
                        newMessage.media.provider = decryptedMessage.media.provider;
                        newMessage.media.venue_id = decryptedMessage.media.venue_id;
                        newMessage.media.venue_type = "";
                    } else {
                        return null;
                    }
                }
                if (newMessage.ttl != 0 && newMessage.media.ttl_seconds == 0) {
                    newMessage.media.ttl_seconds = newMessage.ttl;
                    newMessage.media.flags |= 4;
                }
                if (newMessage.message != null) {
                    newMessage.message = newMessage.message.replace((char) 8238, ' ');
                }
                return newMessage;
            }
            int from_id2 = from_id;
            if (!(object instanceof TLRPC.TL_decryptedMessageService)) {
                if (BuildVars.LOGS_ENABLED) {
                    FileLog.e("unknown message " + object);
                    return null;
                }
                return null;
            }
            TLRPC.TL_decryptedMessageService serviceMessage = (TLRPC.TL_decryptedMessageService) object;
            if ((serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionSetMessageTTL) || (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionScreenshotMessages)) {
                TLRPC.TL_messageService newMessage2 = new TLRPC.TL_messageService();
                if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionSetMessageTTL) {
                    newMessage2.action = new TLRPC.TL_messageEncryptedAction();
                    if (serviceMessage.action.ttl_seconds < 0 || serviceMessage.action.ttl_seconds > 31536000) {
                        serviceMessage.action.ttl_seconds = 31536000;
                    }
                    chat.ttl = serviceMessage.action.ttl_seconds;
                    newMessage2.action.encryptedAction = serviceMessage.action;
                    getMessagesStorage().updateEncryptedChatTTL(chat);
                } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionScreenshotMessages) {
                    newMessage2.action = new TLRPC.TL_messageEncryptedAction();
                    newMessage2.action.encryptedAction = serviceMessage.action;
                }
                int newMessageId2 = getUserConfig().getNewMessageId();
                newMessage2.id = newMessageId2;
                newMessage2.local_id = newMessageId2;
                getUserConfig().saveConfig(false);
                newMessage2.unread = true;
                newMessage2.flags = 256;
                newMessage2.date = date;
                newMessage2.from_id = from_id2;
                newMessage2.to_id = new TLRPC.TL_peerUser();
                newMessage2.to_id.user_id = getUserConfig().getClientUserId();
                newMessage2.dialog_id = chat.id << 32;
                return newMessage2;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionFlushHistory) {
                final long did = chat.id << 32;
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda27
                    @Override // java.lang.Runnable
                    public final void run() {
                        SecretChatHelper.this.m937xb8e2618(did);
                    }
                });
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionDeleteMessages) {
                if (!serviceMessage.action.random_ids.isEmpty()) {
                    this.pendingEncMessagesToDelete.addAll(serviceMessage.action.random_ids);
                    return null;
                }
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionReadMessages) {
                if (!serviceMessage.action.random_ids.isEmpty()) {
                    int time = getConnectionsManager().getCurrentTime();
                    getMessagesStorage().createTaskForSecretChat(chat.id, time, time, 1, serviceMessage.action.random_ids);
                    return null;
                }
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionNotifyLayer) {
                applyPeerLayer(chat, serviceMessage.action.layer);
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionRequestKey) {
                if (chat.exchange_id != 0) {
                    if (chat.exchange_id <= serviceMessage.action.exchange_id) {
                        sendAbortKeyMessage(chat, null, chat.exchange_id);
                    } else if (BuildVars.LOGS_ENABLED) {
                        FileLog.d("we already have request key with higher exchange_id");
                        return null;
                    } else {
                        return null;
                    }
                }
                byte[] salt = new byte[256];
                Utilities.random.nextBytes(salt);
                BigInteger p = new BigInteger(1, getMessagesStorage().getSecretPBytes());
                BigInteger g_b = BigInteger.valueOf(getMessagesStorage().getSecretG());
                BigInteger g_b2 = g_b.modPow(new BigInteger(1, salt), p);
                BigInteger g_a = new BigInteger(1, serviceMessage.action.g_a);
                if (!Utilities.isGoodGaAndGb(g_a, p)) {
                    sendAbortKeyMessage(chat, null, serviceMessage.action.exchange_id);
                    return null;
                }
                byte[] g_b_bytes = g_b2.toByteArray();
                if (g_b_bytes.length <= 256) {
                    i = 1;
                } else {
                    byte[] correctedAuth = new byte[256];
                    i = 1;
                    System.arraycopy(g_b_bytes, 1, correctedAuth, 0, 256);
                    g_b_bytes = correctedAuth;
                }
                byte[] authKey = g_a.modPow(new BigInteger(i, salt), p).toByteArray();
                if (authKey.length > 256) {
                    byte[] correctedAuth2 = new byte[256];
                    System.arraycopy(authKey, authKey.length - 256, correctedAuth2, 0, 256);
                    authKey = correctedAuth2;
                } else if (authKey.length < 256) {
                    byte[] correctedAuth3 = new byte[256];
                    System.arraycopy(authKey, 0, correctedAuth3, 256 - authKey.length, authKey.length);
                    for (int a2 = 0; a2 < 256 - authKey.length; a2++) {
                        correctedAuth3[a2] = 0;
                    }
                    authKey = correctedAuth3;
                }
                byte[] authKeyHash = Utilities.computeSHA1(authKey);
                byte[] authKeyId = new byte[8];
                System.arraycopy(authKeyHash, authKeyHash.length - 8, authKeyId, 0, 8);
                chat.exchange_id = serviceMessage.action.exchange_id;
                chat.future_auth_key = authKey;
                chat.future_key_fingerprint = Utilities.bytesToLong(authKeyId);
                chat.g_a_or_b = g_b_bytes;
                getMessagesStorage().updateEncryptedChat(chat);
                sendAcceptKeyMessage(chat, null);
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionAcceptKey) {
                if (chat.exchange_id == serviceMessage.action.exchange_id) {
                    BigInteger p2 = new BigInteger(1, getMessagesStorage().getSecretPBytes());
                    BigInteger i_authKey = new BigInteger(1, serviceMessage.action.g_b);
                    if (!Utilities.isGoodGaAndGb(i_authKey, p2)) {
                        chat.future_auth_key = new byte[256];
                        chat.future_key_fingerprint = 0L;
                        chat.exchange_id = 0L;
                        getMessagesStorage().updateEncryptedChat(chat);
                        sendAbortKeyMessage(chat, null, serviceMessage.action.exchange_id);
                        return null;
                    }
                    byte[] authKey2 = i_authKey.modPow(new BigInteger(1, chat.a_or_b), p2).toByteArray();
                    if (authKey2.length > 256) {
                        byte[] correctedAuth4 = new byte[256];
                        System.arraycopy(authKey2, authKey2.length - 256, correctedAuth4, 0, 256);
                        authKey2 = correctedAuth4;
                    } else if (authKey2.length < 256) {
                        byte[] correctedAuth5 = new byte[256];
                        byte b = 0;
                        System.arraycopy(authKey2, 0, correctedAuth5, 256 - authKey2.length, authKey2.length);
                        int a3 = 0;
                        while (a3 < 256 - authKey2.length) {
                            correctedAuth5[a3] = b;
                            a3++;
                            b = 0;
                        }
                        authKey2 = correctedAuth5;
                    }
                    byte[] authKeyHash2 = Utilities.computeSHA1(authKey2);
                    byte[] authKeyId2 = new byte[8];
                    System.arraycopy(authKeyHash2, authKeyHash2.length - 8, authKeyId2, 0, 8);
                    long fingerprint = Utilities.bytesToLong(authKeyId2);
                    if (serviceMessage.action.key_fingerprint == fingerprint) {
                        chat.future_auth_key = authKey2;
                        chat.future_key_fingerprint = fingerprint;
                        getMessagesStorage().updateEncryptedChat(chat);
                        sendCommitKeyMessage(chat, null);
                        return null;
                    }
                    chat.future_auth_key = new byte[256];
                    chat.future_key_fingerprint = 0L;
                    chat.exchange_id = 0L;
                    getMessagesStorage().updateEncryptedChat(chat);
                    sendAbortKeyMessage(chat, null, serviceMessage.action.exchange_id);
                    return null;
                }
                chat.future_auth_key = new byte[256];
                chat.future_key_fingerprint = 0L;
                chat.exchange_id = 0L;
                getMessagesStorage().updateEncryptedChat(chat);
                sendAbortKeyMessage(chat, null, serviceMessage.action.exchange_id);
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionCommitKey) {
                if (chat.exchange_id == serviceMessage.action.exchange_id && chat.future_key_fingerprint == serviceMessage.action.key_fingerprint) {
                    long old_fingerpring = chat.key_fingerprint;
                    byte[] old_key = chat.auth_key;
                    chat.key_fingerprint = chat.future_key_fingerprint;
                    chat.auth_key = chat.future_auth_key;
                    chat.key_create_date = getConnectionsManager().getCurrentTime();
                    chat.future_auth_key = old_key;
                    chat.future_key_fingerprint = old_fingerpring;
                    chat.key_use_count_in = (short) 0;
                    chat.key_use_count_out = (short) 0;
                    chat.exchange_id = 0L;
                    getMessagesStorage().updateEncryptedChat(chat);
                    sendNoopMessage(chat, null);
                    return null;
                }
                chat.future_auth_key = new byte[256];
                chat.future_key_fingerprint = 0L;
                chat.exchange_id = 0L;
                getMessagesStorage().updateEncryptedChat(chat);
                sendAbortKeyMessage(chat, null, serviceMessage.action.exchange_id);
                return null;
            } else if (serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionAbortKey) {
                if (chat.exchange_id == serviceMessage.action.exchange_id) {
                    chat.future_auth_key = new byte[256];
                    chat.future_key_fingerprint = 0L;
                    chat.exchange_id = 0L;
                    getMessagesStorage().updateEncryptedChat(chat);
                    return null;
                }
                return null;
            } else if ((serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionNoop) || !(serviceMessage.action instanceof TLRPC.TL_decryptedMessageActionResend) || serviceMessage.action.end_seq_no < chat.in_seq_no || serviceMessage.action.end_seq_no < serviceMessage.action.start_seq_no) {
                return null;
            } else {
                if (serviceMessage.action.start_seq_no < chat.in_seq_no) {
                    serviceMessage.action.start_seq_no = chat.in_seq_no;
                }
                resendMessages(serviceMessage.action.start_seq_no, serviceMessage.action.end_seq_no, chat);
                return null;
            }
        } else if (BuildVars.LOGS_ENABLED) {
            FileLog.e("unknown TLObject");
            return null;
        } else {
            return null;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processDecryptedObject$11$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m937xb8e2618(final long did) {
        TLRPC.Dialog dialog = getMessagesController().dialogs_dict.get(did);
        if (dialog != null) {
            dialog.unread_count = 0;
            getMessagesController().dialogMessage.remove(dialog.id);
        }
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda14
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m936x428f0f9(did);
            }
        });
        getMessagesStorage().deleteDialog(did, 1);
        getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
        getNotificationCenter().postNotificationName(NotificationCenter.removeAllMessagesFromDialog, Long.valueOf(did), false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processDecryptedObject$10$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m936x428f0f9(final long did) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda24
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m938xdfee2915(did);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processDecryptedObject$9$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m938xdfee2915(long did) {
        getNotificationsController().processReadMessages(null, did, 0, Integer.MAX_VALUE, false);
        LongSparseArray<Integer> dialogsToUpdate = new LongSparseArray<>(1);
        dialogsToUpdate.put(did, 0);
        getNotificationsController().processDialogsUpdateRead(dialogsToUpdate);
    }

    private TLRPC.Message createDeleteMessage(int mid, int seq_out, int seq_in, long random_id, TLRPC.EncryptedChat encryptedChat) {
        TLRPC.TL_messageService newMsg = new TLRPC.TL_messageService();
        newMsg.action = new TLRPC.TL_messageEncryptedAction();
        newMsg.action.encryptedAction = new TLRPC.TL_decryptedMessageActionDeleteMessages();
        newMsg.action.encryptedAction.random_ids.add(Long.valueOf(random_id));
        newMsg.id = mid;
        newMsg.local_id = mid;
        newMsg.from_id = getUserConfig().getClientUserId();
        newMsg.unread = true;
        newMsg.out = true;
        newMsg.flags = 256;
        newMsg.dialog_id = encryptedChat.id << 32;
        newMsg.to_id = new TLRPC.TL_peerUser();
        newMsg.send_state = 1;
        newMsg.seq_in = seq_in;
        newMsg.seq_out = seq_out;
        if (encryptedChat.participant_id == getUserConfig().getClientUserId()) {
            newMsg.to_id.user_id = encryptedChat.admin_id;
        } else {
            newMsg.to_id.user_id = encryptedChat.participant_id;
        }
        newMsg.date = 0;
        newMsg.random_id = random_id;
        return newMsg;
    }

    private void resendMessages(final int startSeq, final int endSeq, final TLRPC.EncryptedChat encryptedChat) {
        if (encryptedChat == null || endSeq - startSeq < 0) {
            return;
        }
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda15
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m943xd8e061c0(startSeq, encryptedChat, endSeq);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Type inference failed for: r10v0 */
    /* JADX WARN: Type inference failed for: r10v1, types: [boolean, int] */
    /* JADX WARN: Type inference failed for: r10v10 */
    /* renamed from: lambda$resendMessages$14$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m943xd8e061c0(int startSeq, TLRPC.EncryptedChat encryptedChat, int endSeq) {
        boolean exists;
        int seq_out;
        ArrayList<TLRPC.Message> messages;
        long dialog_id;
        SparseArray<TLRPC.Message> messagesToResend;
        TLRPC.Message message;
        int sSeq = startSeq;
        try {
            if (encryptedChat.admin_id == getUserConfig().getClientUserId() && sSeq % 2 == 0) {
                sSeq++;
            }
            ?? r10 = 0;
            int i = 1;
            int i2 = 3;
            SQLiteCursor cursor = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT uid FROM requested_holes WHERE uid = %d AND ((seq_out_start >= %d AND %d <= seq_out_end) OR (seq_out_start >= %d AND %d <= seq_out_end))", Integer.valueOf(encryptedChat.id), Integer.valueOf(sSeq), Integer.valueOf(sSeq), Integer.valueOf(endSeq), Integer.valueOf(endSeq)), new Object[0]);
            boolean exists2 = cursor.next();
            cursor.dispose();
            if (exists2) {
                return;
            }
            long dialog_id2 = encryptedChat.id << 32;
            SparseArray<TLRPC.Message> messagesToResend2 = new SparseArray<>();
            ArrayList<TLRPC.Message> messages2 = new ArrayList<>();
            for (int a2 = sSeq; a2 < endSeq; a2 += 2) {
                messagesToResend2.put(a2, null);
            }
            SQLiteCursor cursor2 = getMessagesStorage().getDatabase().queryFinalized(String.format(Locale.US, "SELECT m.data, r.random_id, s.seq_in, s.seq_out, m.ttl, s.mid FROM messages_seq as s LEFT JOIN randoms as r ON r.mid = s.mid LEFT JOIN messages as m ON m.mid = s.mid WHERE m.uid = %d AND m.out = 1 AND s.seq_out >= %d AND s.seq_out <= %d ORDER BY seq_out ASC", Long.valueOf(dialog_id2), Integer.valueOf(sSeq), Integer.valueOf(endSeq)), new Object[0]);
            while (cursor2.next()) {
                long random_id = cursor2.longValue(i);
                if (random_id == 0) {
                    random_id = Utilities.random.nextLong();
                }
                int seq_in = cursor2.intValue(2);
                int seq_out2 = cursor2.intValue(i2);
                int mid = cursor2.intValue(5);
                long random_id2 = random_id;
                NativeByteBuffer data = cursor2.byteBufferValue(r10);
                if (data != null) {
                    TLRPC.Message message2 = TLRPC.Message.TLdeserialize(data, data.readInt32(r10), r10);
                    message2.readAttachPath(data, getUserConfig().clientUserId);
                    data.reuse();
                    message2.random_id = random_id2;
                    message2.dialog_id = dialog_id2;
                    message2.seq_in = seq_in;
                    seq_out = seq_out2;
                    message2.seq_out = seq_out;
                    exists = exists2;
                    message2.ttl = cursor2.intValue(4);
                    messages = messages2;
                    dialog_id = dialog_id2;
                    message = message2;
                    messagesToResend = messagesToResend2;
                } else {
                    exists = exists2;
                    seq_out = seq_out2;
                    messages = messages2;
                    dialog_id = dialog_id2;
                    messagesToResend = messagesToResend2;
                    message = createDeleteMessage(mid, seq_out, seq_in, random_id2, encryptedChat);
                }
                messages.add(message);
                messagesToResend.remove(seq_out);
                messagesToResend2 = messagesToResend;
                messages2 = messages;
                exists2 = exists;
                dialog_id2 = dialog_id;
                r10 = 0;
                i = 1;
                i2 = 3;
            }
            SparseArray<TLRPC.Message> messagesToResend3 = messagesToResend2;
            final ArrayList<TLRPC.Message> messages3 = messages2;
            cursor2.dispose();
            if (messagesToResend3.size() != 0) {
                for (int a3 = 0; a3 < messagesToResend3.size(); a3++) {
                    messages3.add(createDeleteMessage(getUserConfig().getNewMessageId(), messagesToResend3.keyAt(a3), 0, Utilities.random.nextLong(), encryptedChat));
                }
                getUserConfig().saveConfig(false);
            }
            Collections.sort(messages3, new Comparator() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda11
                @Override // java.util.Comparator
                public final int compare(Object obj, Object obj2) {
                    int compare;
                    compare = AndroidUtilities.compare(((TLRPC.Message) obj).seq_out, ((TLRPC.Message) obj2).seq_out);
                    return compare;
                }
            });
            ArrayList<TLRPC.EncryptedChat> encryptedChats = new ArrayList<>();
            encryptedChats.add(encryptedChat);
            try {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda22
                    @Override // java.lang.Runnable
                    public final void run() {
                        SecretChatHelper.this.m942xd17b2ca1(messages3);
                    }
                });
                getSendMessagesHelper().processUnsentMessages(messages3, null, new ArrayList<>(), new ArrayList<>(), encryptedChats);
                getMessagesStorage().getDatabase().executeFast(String.format(Locale.US, "REPLACE INTO requested_holes VALUES(%d, %d, %d)", Integer.valueOf(encryptedChat.id), Integer.valueOf(sSeq), Integer.valueOf(endSeq))).stepThis().dispose();
            } catch (Exception e) {
                e = e;
                FileLog.e(e);
            }
        } catch (Exception e2) {
            e = e2;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$resendMessages$13$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m942xd17b2ca1(ArrayList messages) {
        for (int a2 = 0; a2 < messages.size(); a2++) {
            TLRPC.Message message = (TLRPC.Message) messages.get(a2);
            MessageObject messageObject = new MessageObject(this.currentAccount, message, false);
            messageObject.resendAsIs = true;
            getSendMessagesHelper().retrySendMessage(messageObject, true);
        }
    }

    public void checkSecretHoles(TLRPC.EncryptedChat chat, ArrayList<TLRPC.Message> messages) {
        ArrayList<TL_decryptedMessageHolder> holes = this.secretHolesQueue.get(chat.id);
        if (holes == null) {
            return;
        }
        Collections.sort(holes, new Comparator() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda6
            @Override // java.util.Comparator
            public final int compare(Object obj, Object obj2) {
                return SecretChatHelper.lambda$checkSecretHoles$15((SecretChatHelper.TL_decryptedMessageHolder) obj, (SecretChatHelper.TL_decryptedMessageHolder) obj2);
            }
        });
        boolean update = false;
        int a2 = 0;
        while (a2 < holes.size()) {
            TL_decryptedMessageHolder holder = holes.get(a2);
            if (holder.layer.out_seq_no != chat.seq_in && chat.seq_in != holder.layer.out_seq_no - 2) {
                break;
            }
            applyPeerLayer(chat, holder.layer.layer);
            chat.seq_in = holder.layer.out_seq_no;
            chat.in_seq_no = holder.layer.in_seq_no;
            holes.remove(a2);
            int a3 = a2 - 1;
            update = true;
            if (holder.decryptedWithVersion == 2) {
                chat.mtproto_seq = Math.min(chat.mtproto_seq, chat.seq_in);
            }
            TLRPC.Message message = processDecryptedObject(chat, holder.file, holder.date, holder.layer.message, holder.new_key_used);
            if (message != null) {
                messages.add(message);
            }
            a2 = a3 + 1;
        }
        if (holes.isEmpty()) {
            this.secretHolesQueue.remove(chat.id);
        }
        if (update) {
            getMessagesStorage().updateEncryptedChatSeq(chat, true);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$checkSecretHoles$15(TL_decryptedMessageHolder lhs, TL_decryptedMessageHolder rhs) {
        if (lhs.layer.out_seq_no > rhs.layer.out_seq_no) {
            return 1;
        }
        if (lhs.layer.out_seq_no < rhs.layer.out_seq_no) {
            return -1;
        }
        return 0;
    }

    private boolean decryptWithMtProtoVersion(NativeByteBuffer is, byte[] keyToDecrypt, byte[] messageKey, int version, boolean incoming, boolean encryptOnError) {
        boolean incoming2;
        if (version != 1) {
            incoming2 = incoming;
        } else {
            incoming2 = false;
        }
        MessageKeyData keyData = MessageKeyData.generateMessageKeyData(keyToDecrypt, messageKey, incoming2, version);
        Utilities.aesIgeEncryption(is.buffer, keyData.aesKey, keyData.aesIv, false, false, 24, is.limit() - 24);
        int len = is.readInt32(false);
        if (version == 2) {
            if (!Utilities.arraysEquals(messageKey, 0, Utilities.computeSHA256(keyToDecrypt, (incoming2 ? 8 : 0) + 88, 32, is.buffer, 24, is.buffer.limit()), 8)) {
                if (encryptOnError) {
                    Utilities.aesIgeEncryption(is.buffer, keyData.aesKey, keyData.aesIv, true, false, 24, is.limit() - 24);
                    is.position(24);
                }
                return false;
            }
        } else {
            int l = len + 28;
            if (l < is.buffer.limit() - 15 || l > is.buffer.limit()) {
                l = is.buffer.limit();
            }
            byte[] messageKeyFull = Utilities.computeSHA1(is.buffer, 24, l);
            if (!Utilities.arraysEquals(messageKey, 0, messageKeyFull, messageKeyFull.length - 16)) {
                if (encryptOnError) {
                    Utilities.aesIgeEncryption(is.buffer, keyData.aesKey, keyData.aesIv, true, false, 24, is.limit() - 24);
                    is.position(24);
                }
                return false;
            }
        }
        if (len <= 0 || len > is.limit() - 28) {
            return false;
        }
        int padding = (is.limit() - 28) - len;
        if ((version == 2 && (padding < 12 || padding > 1024)) || (version == 1 && padding > 15)) {
            return false;
        }
        return true;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* JADX WARN: Removed duplicated region for block: B:106:0x0251 A[Catch: Exception -> 0x029e, TryCatch #1 {Exception -> 0x029e, blocks: (B:45:0x00bf, B:55:0x00f5, B:57:0x0109, B:59:0x0113, B:60:0x0119, B:62:0x011d, B:64:0x0124, B:66:0x0128, B:68:0x0134, B:69:0x013a, B:70:0x013d, B:72:0x0144, B:74:0x0148, B:76:0x014e, B:78:0x0152, B:79:0x0196, B:83:0x019f, B:85:0x01a3, B:88:0x01aa, B:90:0x01b2, B:92:0x01b6, B:93:0x01bb, B:95:0x01c7, B:96:0x01d4, B:98:0x01db, B:100:0x0215, B:104:0x022f, B:105:0x0239, B:112:0x0260, B:114:0x0274, B:115:0x0277, B:106:0x0251, B:108:0x0255, B:51:0x00d2, B:118:0x027c, B:120:0x028c), top: B:132:0x0070 }] */
    /* JADX WARN: Removed duplicated region for block: B:114:0x0274 A[Catch: Exception -> 0x029e, TryCatch #1 {Exception -> 0x029e, blocks: (B:45:0x00bf, B:55:0x00f5, B:57:0x0109, B:59:0x0113, B:60:0x0119, B:62:0x011d, B:64:0x0124, B:66:0x0128, B:68:0x0134, B:69:0x013a, B:70:0x013d, B:72:0x0144, B:74:0x0148, B:76:0x014e, B:78:0x0152, B:79:0x0196, B:83:0x019f, B:85:0x01a3, B:88:0x01aa, B:90:0x01b2, B:92:0x01b6, B:93:0x01bb, B:95:0x01c7, B:96:0x01d4, B:98:0x01db, B:100:0x0215, B:104:0x022f, B:105:0x0239, B:112:0x0260, B:114:0x0274, B:115:0x0277, B:106:0x0251, B:108:0x0255, B:51:0x00d2, B:118:0x027c, B:120:0x028c), top: B:132:0x0070 }] */
    /* JADX WARN: Removed duplicated region for block: B:41:0x00a5  */
    /* JADX WARN: Removed duplicated region for block: B:54:0x00ec  */
    /* JADX WARN: Removed duplicated region for block: B:62:0x011d A[Catch: Exception -> 0x029e, TryCatch #1 {Exception -> 0x029e, blocks: (B:45:0x00bf, B:55:0x00f5, B:57:0x0109, B:59:0x0113, B:60:0x0119, B:62:0x011d, B:64:0x0124, B:66:0x0128, B:68:0x0134, B:69:0x013a, B:70:0x013d, B:72:0x0144, B:74:0x0148, B:76:0x014e, B:78:0x0152, B:79:0x0196, B:83:0x019f, B:85:0x01a3, B:88:0x01aa, B:90:0x01b2, B:92:0x01b6, B:93:0x01bb, B:95:0x01c7, B:96:0x01d4, B:98:0x01db, B:100:0x0215, B:104:0x022f, B:105:0x0239, B:112:0x0260, B:114:0x0274, B:115:0x0277, B:106:0x0251, B:108:0x0255, B:51:0x00d2, B:118:0x027c, B:120:0x028c), top: B:132:0x0070 }] */
    /* JADX WARN: Type inference failed for: r5v16 */
    /* JADX WARN: Type inference failed for: r5v18 */
    /* JADX WARN: Type inference failed for: r5v6 */
    /* JADX WARN: Type inference failed for: r5v7, types: [boolean, int] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public ArrayList<TLRPC.Message> decryptMessage(TLRPC.EncryptedMessage message) {
        NativeByteBuffer is;
        long fingerprint;
        byte[] keyToDecrypt;
        boolean new_key_used;
        int mtprotoVersion;
        boolean tryAnotherDecrypt;
        boolean new_key_used2;
        NativeByteBuffer is2;
        TLRPC.EncryptedChat chat;
        ?? r5;
        int decryptedWithVersion;
        TLObject object;
        TLObject object2;
        TLRPC.Message decryptedMessage;
        TLRPC.EncryptedChat chat2 = getMessagesController().getEncryptedChatDB(message.chat_id, true);
        if (chat2 != null && !(chat2 instanceof TLRPC.TL_encryptedChatDiscarded)) {
            try {
                is = new NativeByteBuffer(message.bytes.length);
                is.writeBytes(message.bytes);
                is.position(0);
                fingerprint = is.readInt64(false);
                try {
                    if (chat2.key_fingerprint == fingerprint) {
                        byte[] keyToDecrypt2 = chat2.auth_key;
                        keyToDecrypt = keyToDecrypt2;
                        new_key_used = false;
                    } else if (chat2.future_key_fingerprint != 0 && chat2.future_key_fingerprint == fingerprint) {
                        byte[] keyToDecrypt3 = chat2.future_auth_key;
                        keyToDecrypt = keyToDecrypt3;
                        new_key_used = true;
                    } else {
                        keyToDecrypt = null;
                        new_key_used = false;
                    }
                    mtprotoVersion = AndroidUtilities.getPeerLayerVersion(chat2.layer) >= 73 ? 2 : 1;
                    try {
                    } catch (Exception e) {
                        e = e;
                        FileLog.e(e);
                        return null;
                    }
                } catch (Exception e2) {
                    e = e2;
                }
            } catch (Exception e3) {
                e = e3;
            }
            if (keyToDecrypt != null) {
                byte[] messageKey = is.readData(16, false);
                boolean incoming = chat2.admin_id == getUserConfig().getClientUserId();
                if (mtprotoVersion == 2) {
                    if (chat2.mtproto_seq != 0) {
                        tryAnotherDecrypt = false;
                        if (!decryptWithMtProtoVersion(is, keyToDecrypt, messageKey, mtprotoVersion, incoming, tryAnotherDecrypt)) {
                            new_key_used2 = new_key_used;
                            is2 = is;
                            chat = chat2;
                            r5 = 1;
                            decryptedWithVersion = mtprotoVersion;
                        } else if (mtprotoVersion == 2) {
                            if (tryAnotherDecrypt) {
                                new_key_used2 = new_key_used;
                                is2 = is;
                                chat = chat2;
                                r5 = 1;
                                if (decryptWithMtProtoVersion(is, keyToDecrypt, messageKey, 1, incoming, false)) {
                                    decryptedWithVersion = 1;
                                }
                            }
                            return null;
                        } else {
                            new_key_used2 = new_key_used;
                            is2 = is;
                            chat = chat2;
                            r5 = 1;
                            decryptedWithVersion = 2;
                            if (!decryptWithMtProtoVersion(is2, keyToDecrypt, messageKey, 2, incoming, tryAnotherDecrypt)) {
                                return null;
                            }
                        }
                        NativeByteBuffer is3 = is2;
                        object = TLClassStore.Instance().TLdeserialize(is3, is3.readInt32(false), false);
                        is3.reuse();
                        if (!new_key_used2 && AndroidUtilities.getPeerLayerVersion(chat.layer) >= 20) {
                            chat.key_use_count_in = (short) (chat.key_use_count_in + r5);
                        }
                        if (object instanceof TLRPC.TL_decryptedMessageLayer) {
                            if ((object instanceof TLRPC.TL_decryptedMessageService) && (((TLRPC.TL_decryptedMessageService) object).action instanceof TLRPC.TL_decryptedMessageActionNotifyLayer)) {
                                object2 = object;
                            }
                            return null;
                        }
                        TLRPC.TL_decryptedMessageLayer layer = (TLRPC.TL_decryptedMessageLayer) object;
                        if (chat.seq_in == 0 && chat.seq_out == 0) {
                            if (chat.admin_id == getUserConfig().getClientUserId()) {
                                chat.seq_out = r5;
                                chat.seq_in = -2;
                            } else {
                                chat.seq_in = -1;
                            }
                        }
                        if (layer.random_bytes.length < 15) {
                            if (BuildVars.LOGS_ENABLED) {
                                FileLog.e("got random bytes less than needed");
                            }
                            return null;
                        }
                        if (BuildVars.LOGS_ENABLED) {
                            FileLog.d("current chat in_seq = " + chat.seq_in + " out_seq = " + chat.seq_out);
                            FileLog.d("got message with in_seq = " + layer.in_seq_no + " out_seq = " + layer.out_seq_no);
                        }
                        if (layer.out_seq_no <= chat.seq_in) {
                            return null;
                        }
                        if (decryptedWithVersion == r5 && chat.mtproto_seq != 0 && layer.out_seq_no >= chat.mtproto_seq) {
                            return null;
                        }
                        if (chat.seq_in != layer.out_seq_no - 2) {
                            if (BuildVars.LOGS_ENABLED) {
                                FileLog.e("got hole");
                            }
                            ArrayList<TL_decryptedMessageHolder> arr = this.secretHolesQueue.get(chat.id);
                            if (arr == null) {
                                arr = new ArrayList<>();
                                this.secretHolesQueue.put(chat.id, arr);
                            }
                            if (arr.size() >= 4) {
                                this.secretHolesQueue.remove(chat.id);
                                final TLRPC.TL_encryptedChatDiscarded newChat = new TLRPC.TL_encryptedChatDiscarded();
                                newChat.id = chat.id;
                                newChat.user_id = chat.user_id;
                                newChat.auth_key = chat.auth_key;
                                newChat.key_create_date = chat.key_create_date;
                                newChat.key_use_count_in = chat.key_use_count_in;
                                newChat.key_use_count_out = chat.key_use_count_out;
                                newChat.seq_in = chat.seq_in;
                                newChat.seq_out = chat.seq_out;
                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda26
                                    @Override // java.lang.Runnable
                                    public final void run() {
                                        SecretChatHelper.this.m928xb3ba0a91(newChat);
                                    }
                                });
                                declineSecretChat(chat.id);
                                return null;
                            }
                            TL_decryptedMessageHolder holder = new TL_decryptedMessageHolder();
                            holder.layer = layer;
                            holder.file = message.file;
                            holder.date = message.date;
                            holder.new_key_used = new_key_used2;
                            holder.decryptedWithVersion = decryptedWithVersion;
                            arr.add(holder);
                            return null;
                        }
                        if (decryptedWithVersion == 2) {
                            chat.mtproto_seq = Math.min(chat.mtproto_seq, chat.seq_in);
                        }
                        applyPeerLayer(chat, layer.layer);
                        chat.seq_in = layer.out_seq_no;
                        chat.in_seq_no = layer.in_seq_no;
                        getMessagesStorage().updateEncryptedChatSeq(chat, r5);
                        object2 = layer.message;
                        ArrayList<TLRPC.Message> messages = new ArrayList<>();
                        decryptedMessage = processDecryptedObject(chat, message.file, message.date, object2, new_key_used2);
                        if (decryptedMessage != null) {
                            messages.add(decryptedMessage);
                        }
                        checkSecretHoles(chat, messages);
                        return messages;
                    }
                }
                tryAnotherDecrypt = true;
                if (!decryptWithMtProtoVersion(is, keyToDecrypt, messageKey, mtprotoVersion, incoming, tryAnotherDecrypt)) {
                }
                NativeByteBuffer is32 = is2;
                object = TLClassStore.Instance().TLdeserialize(is32, is32.readInt32(false), false);
                is32.reuse();
                if (!new_key_used2) {
                    chat.key_use_count_in = (short) (chat.key_use_count_in + r5);
                }
                if (object instanceof TLRPC.TL_decryptedMessageLayer) {
                }
                ArrayList<TLRPC.Message> messages2 = new ArrayList<>();
                decryptedMessage = processDecryptedObject(chat, message.file, message.date, object2, new_key_used2);
                if (decryptedMessage != null) {
                }
                checkSecretHoles(chat, messages2);
                return messages2;
            }
            is.reuse();
            if (BuildVars.LOGS_ENABLED) {
                FileLog.e(String.format("fingerprint mismatch %x", Long.valueOf(fingerprint)));
            }
            return null;
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$decryptMessage$16$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m928xb3ba0a91(TLRPC.TL_encryptedChatDiscarded newChat) {
        getMessagesController().putEncryptedChat(newChat, false);
        getMessagesStorage().updateEncryptedChat(newChat);
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatUpdated, newChat);
    }

    public void requestNewSecretChatKey(TLRPC.EncryptedChat encryptedChat) {
        if (AndroidUtilities.getPeerLayerVersion(encryptedChat.layer) < 20) {
            return;
        }
        byte[] salt = new byte[256];
        Utilities.random.nextBytes(salt);
        BigInteger i_g_a = BigInteger.valueOf(getMessagesStorage().getSecretG());
        byte[] g_a = i_g_a.modPow(new BigInteger(1, salt), new BigInteger(1, getMessagesStorage().getSecretPBytes())).toByteArray();
        if (g_a.length > 256) {
            byte[] correctedAuth = new byte[256];
            System.arraycopy(g_a, 1, correctedAuth, 0, 256);
            g_a = correctedAuth;
        }
        encryptedChat.exchange_id = getSendMessagesHelper().getNextRandomId();
        encryptedChat.a_or_b = salt;
        encryptedChat.g_a = g_a;
        getMessagesStorage().updateEncryptedChat(encryptedChat);
        sendRequestKeyMessage(encryptedChat, null);
    }

    public void processAcceptedSecretChat(final TLRPC.EncryptedChat encryptedChat) {
        BigInteger p = new BigInteger(1, getMessagesStorage().getSecretPBytes());
        BigInteger i_authKey = new BigInteger(1, encryptedChat.g_a_or_b);
        if (!Utilities.isGoodGaAndGb(i_authKey, p)) {
            declineSecretChat(encryptedChat.id);
            return;
        }
        byte[] authKey = i_authKey.modPow(new BigInteger(1, encryptedChat.a_or_b), p).toByteArray();
        if (authKey.length > 256) {
            byte[] correctedAuth = new byte[256];
            System.arraycopy(authKey, authKey.length - 256, correctedAuth, 0, 256);
            authKey = correctedAuth;
        } else if (authKey.length < 256) {
            byte[] correctedAuth2 = new byte[256];
            System.arraycopy(authKey, 0, correctedAuth2, 256 - authKey.length, authKey.length);
            for (int a2 = 0; a2 < 256 - authKey.length; a2++) {
                correctedAuth2[a2] = 0;
            }
            authKey = correctedAuth2;
        }
        byte[] authKeyHash = Utilities.computeSHA1(authKey);
        byte[] authKeyId = new byte[8];
        System.arraycopy(authKeyHash, authKeyHash.length - 8, authKeyId, 0, 8);
        long fingerprint = Utilities.bytesToLong(authKeyId);
        if (encryptedChat.key_fingerprint == fingerprint) {
            encryptedChat.auth_key = authKey;
            encryptedChat.key_create_date = getConnectionsManager().getCurrentTime();
            encryptedChat.seq_in = -2;
            encryptedChat.seq_out = 1;
            getMessagesStorage().updateEncryptedChat(encryptedChat);
            getMessagesController().putEncryptedChat(encryptedChat, false);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda2
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m934x8ba1f7da(encryptedChat);
                }
            });
            return;
        }
        final TLRPC.TL_encryptedChatDiscarded newChat = new TLRPC.TL_encryptedChatDiscarded();
        newChat.id = encryptedChat.id;
        newChat.user_id = encryptedChat.user_id;
        newChat.auth_key = encryptedChat.auth_key;
        newChat.key_create_date = encryptedChat.key_create_date;
        newChat.key_use_count_in = encryptedChat.key_use_count_in;
        newChat.key_use_count_out = encryptedChat.key_use_count_out;
        newChat.seq_in = encryptedChat.seq_in;
        newChat.seq_out = encryptedChat.seq_out;
        newChat.admin_id = encryptedChat.admin_id;
        newChat.mtproto_seq = encryptedChat.mtproto_seq;
        getMessagesStorage().updateEncryptedChat(newChat);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m935x93072cf9(newChat);
            }
        });
        declineSecretChat(encryptedChat.id);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processAcceptedSecretChat$17$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m934x8ba1f7da(TLRPC.EncryptedChat encryptedChat) {
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatUpdated, encryptedChat);
        sendNotifyLayerMessage(encryptedChat, null);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processAcceptedSecretChat$18$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m935x93072cf9(TLRPC.TL_encryptedChatDiscarded newChat) {
        getMessagesController().putEncryptedChat(newChat, false);
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatUpdated, newChat);
    }

    public void declineSecretChat(int chat_id) {
        TLRPC.TL_messages_discardEncryption req = new TLRPC.TL_messages_discardEncryption();
        req.chat_id = chat_id;
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda21
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SecretChatHelper.lambda$declineSecretChat$19(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$declineSecretChat$19(TLObject response, TLRPC.TL_error error) {
    }

    public void acceptSecretChat(final TLRPC.EncryptedChat encryptedChat) {
        if (this.acceptingChats.get(encryptedChat.id) != null) {
            return;
        }
        this.acceptingChats.put(encryptedChat.id, encryptedChat);
        TLRPC.TL_messages_getDhConfig req = new TLRPC.TL_messages_getDhConfig();
        req.random_length = 256;
        req.version = getMessagesStorage().getLastSecretVersion();
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda19
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SecretChatHelper.this.m926x4651e02c(encryptedChat, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$acceptSecretChat$22$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m926x4651e02c(final TLRPC.EncryptedChat encryptedChat, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            TLRPC.messages_DhConfig res = (TLRPC.messages_DhConfig) response;
            if (response instanceof TLRPC.TL_messages_dhConfig) {
                if (!Utilities.isGoodPrime(res.p, res.g)) {
                    this.acceptingChats.remove(encryptedChat.id);
                    declineSecretChat(encryptedChat.id);
                    return;
                }
                getMessagesStorage().setSecretPBytes(res.p);
                getMessagesStorage().setSecretG(res.g);
                getMessagesStorage().setLastSecretVersion(res.version);
                getMessagesStorage().saveSecretParams(getMessagesStorage().getLastSecretVersion(), getMessagesStorage().getSecretG(), getMessagesStorage().getSecretPBytes());
            }
            byte[] salt = new byte[256];
            for (int a2 = 0; a2 < 256; a2++) {
                salt[a2] = (byte) (((byte) (Utilities.random.nextDouble() * 256.0d)) ^ res.random[a2]);
            }
            encryptedChat.a_or_b = salt;
            encryptedChat.seq_in = -1;
            encryptedChat.seq_out = 0;
            BigInteger p = new BigInteger(1, getMessagesStorage().getSecretPBytes());
            BigInteger g_b = BigInteger.valueOf(getMessagesStorage().getSecretG());
            BigInteger g_b2 = g_b.modPow(new BigInteger(1, salt), p);
            BigInteger g_a = new BigInteger(1, encryptedChat.g_a);
            if (!Utilities.isGoodGaAndGb(g_a, p)) {
                this.acceptingChats.remove(encryptedChat.id);
                declineSecretChat(encryptedChat.id);
                return;
            }
            byte[] g_b_bytes = g_b2.toByteArray();
            if (g_b_bytes.length > 256) {
                byte[] correctedAuth = new byte[256];
                System.arraycopy(g_b_bytes, 1, correctedAuth, 0, 256);
                g_b_bytes = correctedAuth;
            }
            byte[] authKey = g_a.modPow(new BigInteger(1, salt), p).toByteArray();
            if (authKey.length <= 256) {
                if (authKey.length < 256) {
                    byte[] correctedAuth2 = new byte[256];
                    System.arraycopy(authKey, 0, correctedAuth2, 256 - authKey.length, authKey.length);
                    for (int a3 = 0; a3 < 256 - authKey.length; a3++) {
                        correctedAuth2[a3] = 0;
                    }
                    authKey = correctedAuth2;
                }
            } else {
                byte[] correctedAuth3 = new byte[256];
                System.arraycopy(authKey, authKey.length - 256, correctedAuth3, 0, 256);
                authKey = correctedAuth3;
            }
            byte[] authKeyHash = Utilities.computeSHA1(authKey);
            byte[] authKeyId = new byte[8];
            System.arraycopy(authKeyHash, authKeyHash.length - 8, authKeyId, 0, 8);
            encryptedChat.auth_key = authKey;
            encryptedChat.key_create_date = getConnectionsManager().getCurrentTime();
            TLRPC.TL_messages_acceptEncryption req2 = new TLRPC.TL_messages_acceptEncryption();
            req2.g_b = g_b_bytes;
            req2.peer = new TLRPC.TL_inputEncryptedChat();
            req2.peer.chat_id = encryptedChat.id;
            req2.peer.access_hash = encryptedChat.access_hash;
            req2.key_fingerprint = Utilities.bytesToLong(authKeyId);
            FileLog.e("J----------------------> TL_messages_acceptEncryption req");
            getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda20
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    SecretChatHelper.this.m925x3eecab0d(encryptedChat, tLObject, tL_error);
                }
            });
            return;
        }
        this.acceptingChats.remove(encryptedChat.id);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$acceptSecretChat$21$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m925x3eecab0d(TLRPC.EncryptedChat encryptedChat, TLObject response1, TLRPC.TL_error error1) {
        this.acceptingChats.remove(encryptedChat.id);
        FileLog.e("J----------------------> TL_messages_acceptEncryption res");
        if (error1 == null) {
            final TLRPC.EncryptedChat newChat = (TLRPC.EncryptedChat) response1;
            newChat.auth_key = encryptedChat.auth_key;
            newChat.user_id = encryptedChat.user_id;
            newChat.seq_in = encryptedChat.seq_in;
            newChat.seq_out = encryptedChat.seq_out;
            newChat.key_create_date = encryptedChat.key_create_date;
            newChat.key_use_count_in = encryptedChat.key_use_count_in;
            newChat.key_use_count_out = encryptedChat.key_use_count_out;
            getMessagesStorage().updateEncryptedChat(newChat);
            getMessagesController().putEncryptedChat(newChat, false);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda16
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m924x378775ee(newChat);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$acceptSecretChat$20$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m924x378775ee(TLRPC.EncryptedChat newChat) {
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatUpdated, newChat);
        sendNotifyLayerMessage(newChat, null);
    }

    public void startSecretChat(final Context context, final TLRPC.User user) {
        if (user == null || context == null) {
            return;
        }
        this.startingSecretChat = true;
        final AlertDialog progressDialog = new AlertDialog(context, 3);
        TLRPC.TL_messages_getDhConfig req = new TLRPC.TL_messages_getDhConfig();
        req.random_length = 256;
        req.version = getMessagesStorage().getLastSecretVersion();
        final int reqId = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda4
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SecretChatHelper.this.m949xc5a668c5(context, progressDialog, user, tLObject, tL_error);
            }
        }, 2);
        progressDialog.setOnCancelListener(new DialogInterface.OnCancelListener() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda5
            @Override // android.content.DialogInterface.OnCancelListener
            public final void onCancel(DialogInterface dialogInterface) {
                SecretChatHelper.this.m950x6858f96f(reqId, dialogInterface);
            }
        });
        try {
            progressDialog.show();
        } catch (Exception e) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$29$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m949xc5a668c5(final Context context, final AlertDialog progressDialog, final TLRPC.User user, TLObject response, TLRPC.TL_error error) {
        byte[] g_a;
        if (error != null) {
            this.delayedEncryptedChatUpdates.clear();
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda10
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m948xbe4133a6(context, progressDialog);
                }
            });
            return;
        }
        TLRPC.messages_DhConfig res = (TLRPC.messages_DhConfig) response;
        if (response instanceof TLRPC.TL_messages_dhConfig) {
            if (!Utilities.isGoodPrime(res.p, res.g)) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda8
                    @Override // java.lang.Runnable
                    public final void run() {
                        SecretChatHelper.lambda$startSecretChat$23(context, progressDialog);
                    }
                });
                return;
            }
            getMessagesStorage().setSecretPBytes(res.p);
            getMessagesStorage().setSecretG(res.g);
            getMessagesStorage().setLastSecretVersion(res.version);
            getMessagesStorage().saveSecretParams(getMessagesStorage().getLastSecretVersion(), getMessagesStorage().getSecretG(), getMessagesStorage().getSecretPBytes());
        }
        final byte[] salt = new byte[256];
        for (int a2 = 0; a2 < 256; a2++) {
            salt[a2] = (byte) (((byte) (Utilities.random.nextDouble() * 256.0d)) ^ res.random[a2]);
        }
        BigInteger i_g_a = BigInteger.valueOf(getMessagesStorage().getSecretG());
        byte[] g_a2 = i_g_a.modPow(new BigInteger(1, salt), new BigInteger(1, getMessagesStorage().getSecretPBytes())).toByteArray();
        if (g_a2.length <= 256) {
            g_a = g_a2;
        } else {
            byte[] correctedAuth = new byte[256];
            System.arraycopy(g_a2, 1, correctedAuth, 0, 256);
            g_a = correctedAuth;
        }
        TLRPC.TL_messages_requestEncryption req2 = new TLRPC.TL_messages_requestEncryption();
        req2.g_a = g_a;
        req2.user_id = getMessagesController().getInputUser(user);
        req2.random_id = Utilities.random.nextInt();
        getConnectionsManager().sendRequest(req2, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda9
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SecretChatHelper.this.m947xb6dbfe87(context, progressDialog, salt, user, tLObject, tL_error);
            }
        }, 2);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$startSecretChat$23(Context context, AlertDialog progressDialog) {
        try {
            if (!((Activity) context).isFinishing()) {
                progressDialog.dismiss();
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$27$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m947xb6dbfe87(final Context context, final AlertDialog progressDialog, final byte[] salt, final TLRPC.User user, final TLObject response1, TLRPC.TL_error error1) {
        if (error1 == null) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda28
                @Override // java.lang.Runnable
                public final void run() {
                    SecretChatHelper.this.m945xa8119449(context, progressDialog, response1, salt, user);
                }
            });
            return;
        }
        this.delayedEncryptedChatUpdates.clear();
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda29
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m946xaf76c968(context, progressDialog);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$25$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m945xa8119449(Context context, AlertDialog progressDialog, TLObject response1, byte[] salt, TLRPC.User user) {
        this.startingSecretChat = false;
        if (!((Activity) context).isFinishing()) {
            try {
                progressDialog.dismiss();
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
        TLRPC.EncryptedChat chat = (TLRPC.EncryptedChat) response1;
        chat.user_id = chat.participant_id;
        chat.seq_in = -2;
        chat.seq_out = 1;
        chat.a_or_b = salt;
        getMessagesController().putEncryptedChat(chat, false);
        TLRPC.Dialog dialog = new TLRPC.TL_dialog();
        dialog.id = DialogObject.makeSecretDialogId(chat.id);
        dialog.unread_count = 0;
        dialog.top_message = 0;
        dialog.last_message_date = getConnectionsManager().getCurrentTime();
        getMessagesController().dialogs_dict.put(dialog.id, dialog);
        getMessagesController().allDialogs.add(dialog);
        getMessagesController().sortDialogs(null);
        getMessagesStorage().putEncryptedChat(chat, user, dialog);
        getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
        getNotificationCenter().postNotificationName(NotificationCenter.encryptedChatCreated, chat);
        Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SecretChatHelper$$ExternalSyntheticLambda18
            @Override // java.lang.Runnable
            public final void run() {
                SecretChatHelper.this.m944xa0ac5f2a();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$24$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m944xa0ac5f2a() {
        if (!this.delayedEncryptedChatUpdates.isEmpty()) {
            getMessagesController().processUpdateArray(this.delayedEncryptedChatUpdates, null, null, false, 0);
            this.delayedEncryptedChatUpdates.clear();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$26$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m946xaf76c968(Context context, AlertDialog progressDialog) {
        if (!((Activity) context).isFinishing()) {
            this.startingSecretChat = false;
            try {
                progressDialog.dismiss();
            } catch (Exception e) {
                FileLog.e(e);
            }
            AlertDialog.Builder builder = new AlertDialog.Builder(context);
            builder.setTitle(LocaleController.getString("AppName", R.string.AppName));
            builder.setMessage(LocaleController.getString("CreateEncryptedChatError", R.string.CreateEncryptedChatError));
            builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
            builder.show().setCanceledOnTouchOutside(true);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$28$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m948xbe4133a6(Context context, AlertDialog progressDialog) {
        this.startingSecretChat = false;
        if (!((Activity) context).isFinishing()) {
            try {
                progressDialog.dismiss();
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$startSecretChat$30$im-uecahxyvjd-messenger-SecretChatHelper  reason: not valid java name */
    public /* synthetic */ void m950x6858f96f(int reqId, DialogInterface dialog) {
        getConnectionsManager().cancelRequest(reqId, true);
    }
}
