package im.uecahxyvjd.messenger;

import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.media.MediaMetadataRetriever;
import android.media.MediaPlayer;
import android.media.ThumbnailUtils;
import android.net.Uri;
import android.os.Bundle;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Base64;
import android.util.LongSparseArray;
import android.util.SparseArray;
import android.webkit.MimeTypeMap;
import androidx.core.view.inputmethod.InputContentInfoCompat;
import com.google.android.exoplayer2.DefaultRenderersFactory;
import com.google.android.exoplayer2.util.MimeTypes;
import im.uecahxyvjd.messenger.MediaController;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.SendMessagesHelper;
import im.uecahxyvjd.messenger.audioinfo.AudioInfo;
import im.uecahxyvjd.messenger.support.SparseLongArray;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.NativeByteBuffer;
import im.uecahxyvjd.tgnet.QuickAckDelegate;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.SerializedData;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPCContacts;
import im.uecahxyvjd.ui.ChatActivity;
import im.uecahxyvjd.ui.actionbar.AlertDialog;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.components.AlertsCreator;
import im.uecahxyvjd.ui.components.AnimatedFileDrawable;
import im.uecahxyvjd.ui.components.toast.ToastUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
/* loaded from: classes16.dex */
public class SendMessagesHelper extends BaseController implements NotificationCenter.NotificationCenterDelegate {
    private static volatile SendMessagesHelper[] Instance;
    private static DispatchQueue mediaSendQueue = new DispatchQueue("mediaSendQueue");
    private static ThreadPoolExecutor mediaSendThreadPool;
    private HashMap<String, ArrayList<DelayedMessage>> delayedMessages;
    private SparseArray<TLRPC.Message> editingMessages;
    private LocationProvider locationProvider;
    private SparseArray<TLRPC.Message> sendingMessages;
    private LongSparseArray<Integer> sendingMessagesIdDialogs;
    private SparseArray<MessageObject> unsentMessages;
    private SparseArray<TLRPC.Message> uploadMessages;
    private LongSparseArray<Integer> uploadingMessagesIdDialogs;
    private LongSparseArray<Long> voteSendTime;
    private HashMap<String, Boolean> waitingForCallback;
    private HashMap<String, MessageObject> waitingForLocation;
    private HashMap<String, byte[]> waitingForVote;

    /* loaded from: classes16.dex */
    public static class SendingMediaInfo {
        public boolean canDeleteAfter;
        public String caption;
        public ArrayList<TLRPC.MessageEntity> entities;
        public TLRPC.BotInlineResult inlineResult;
        public boolean isVideo;
        public ArrayList<TLRPC.InputDocument> masks;
        public HashMap<String, String> params;
        public String path;
        public MediaController.SearchImage searchImage;
        public int ttl;
        public Uri uri;
        public VideoEditedInfo videoEditedInfo;
    }

    static {
        int cores = Runtime.getRuntime().availableProcessors();
        mediaSendThreadPool = new ThreadPoolExecutor(cores, cores, 60L, TimeUnit.SECONDS, new LinkedBlockingQueue());
        Instance = new SendMessagesHelper[3];
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes16.dex */
    public static class MediaSendPrepareWorker {
        public volatile String parentObject;
        public volatile TLRPC.TL_photo photo;
        public CountDownLatch sync;

        private MediaSendPrepareWorker() {
        }
    }

    /* loaded from: classes16.dex */
    public static class LocationProvider {
        private LocationProviderDelegate delegate;
        private Location lastKnownLocation;
        private LocationManager locationManager;
        private Runnable locationQueryCancelRunnable;
        private GpsLocationListener gpsLocationListener = new GpsLocationListener();
        private GpsLocationListener networkLocationListener = new GpsLocationListener();

        /* loaded from: classes16.dex */
        public interface LocationProviderDelegate {
            void onLocationAcquired(Location location);

            void onUnableLocationAcquire();
        }

        /* JADX INFO: Access modifiers changed from: private */
        /* loaded from: classes16.dex */
        public class GpsLocationListener implements LocationListener {
            private GpsLocationListener() {
            }

            @Override // android.location.LocationListener
            public void onLocationChanged(Location location) {
                if (location == null || LocationProvider.this.locationQueryCancelRunnable == null) {
                    return;
                }
                if (BuildVars.LOGS_ENABLED) {
                    FileLog.d("found location " + location);
                }
                LocationProvider.this.lastKnownLocation = location;
                if (location.getAccuracy() < 100.0f) {
                    if (LocationProvider.this.delegate != null) {
                        LocationProvider.this.delegate.onLocationAcquired(location);
                    }
                    if (LocationProvider.this.locationQueryCancelRunnable != null) {
                        AndroidUtilities.cancelRunOnUIThread(LocationProvider.this.locationQueryCancelRunnable);
                    }
                    LocationProvider.this.cleanup();
                }
            }

            @Override // android.location.LocationListener
            public void onStatusChanged(String provider, int status, Bundle extras) {
            }

            @Override // android.location.LocationListener
            public void onProviderEnabled(String provider) {
            }

            @Override // android.location.LocationListener
            public void onProviderDisabled(String provider) {
            }
        }

        public LocationProvider() {
        }

        public LocationProvider(LocationProviderDelegate locationProviderDelegate) {
            this.delegate = locationProviderDelegate;
        }

        public void setDelegate(LocationProviderDelegate locationProviderDelegate) {
            this.delegate = locationProviderDelegate;
        }

        /* JADX INFO: Access modifiers changed from: private */
        public void cleanup() {
            this.locationManager.removeUpdates(this.gpsLocationListener);
            this.locationManager.removeUpdates(this.networkLocationListener);
            this.lastKnownLocation = null;
            this.locationQueryCancelRunnable = null;
        }

        public void start() {
            if (this.locationManager == null) {
                this.locationManager = (LocationManager) ApplicationLoader.applicationContext.getSystemService("location");
            }
            try {
                this.locationManager.requestLocationUpdates("gps", 1L, 0.0f, this.gpsLocationListener);
            } catch (Exception e) {
                FileLog.e(e);
            }
            try {
                this.locationManager.requestLocationUpdates("network", 1L, 0.0f, this.networkLocationListener);
            } catch (Exception e2) {
                FileLog.e(e2);
            }
            try {
                Location lastKnownLocation = this.locationManager.getLastKnownLocation("gps");
                this.lastKnownLocation = lastKnownLocation;
                if (lastKnownLocation == null) {
                    this.lastKnownLocation = this.locationManager.getLastKnownLocation("network");
                }
            } catch (Exception e3) {
                FileLog.e(e3);
            }
            Runnable runnable = this.locationQueryCancelRunnable;
            if (runnable != null) {
                AndroidUtilities.cancelRunOnUIThread(runnable);
            }
            Runnable runnable2 = new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper.LocationProvider.1
                @Override // java.lang.Runnable
                public void run() {
                    if (LocationProvider.this.locationQueryCancelRunnable == this) {
                        if (LocationProvider.this.delegate != null) {
                            if (LocationProvider.this.lastKnownLocation != null) {
                                LocationProvider.this.delegate.onLocationAcquired(LocationProvider.this.lastKnownLocation);
                            } else {
                                LocationProvider.this.delegate.onUnableLocationAcquire();
                            }
                        }
                        LocationProvider.this.cleanup();
                    }
                }
            };
            this.locationQueryCancelRunnable = runnable2;
            AndroidUtilities.runOnUIThread(runnable2, DefaultRenderersFactory.DEFAULT_ALLOWED_VIDEO_JOINING_TIME_MS);
        }

        public void stop() {
            if (this.locationManager == null) {
                return;
            }
            Runnable runnable = this.locationQueryCancelRunnable;
            if (runnable != null) {
                AndroidUtilities.cancelRunOnUIThread(runnable);
            }
            cleanup();
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* loaded from: classes16.dex */
    public class DelayedMessageSendAfterRequest {
        public DelayedMessage delayedMessage;
        public MessageObject msgObj;
        public ArrayList<MessageObject> msgObjs;
        public String originalPath;
        public ArrayList<String> originalPaths;
        public Object parentObject;
        public ArrayList<Object> parentObjects;
        public TLObject request;
        public boolean scheduled;

        protected DelayedMessageSendAfterRequest() {
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* loaded from: classes16.dex */
    public class DelayedMessage {
        public TLRPC.EncryptedChat encryptedChat;
        public HashMap<Object, Object> extraHashMap;
        public int finalGroupMessage;
        public long groupId;
        public String httpLocation;
        public ArrayList<String> httpLocations;
        public ArrayList<TLRPC.InputMedia> inputMedias;
        public TLRPC.InputMedia inputUploadMedia;
        public TLObject locationParent;
        public ArrayList<TLRPC.PhotoSize> locations;
        public ArrayList<MessageObject> messageObjects;
        public ArrayList<TLRPC.Message> messages;
        public MessageObject obj;
        public String originalPath;
        public ArrayList<String> originalPaths;
        public Object parentObject;
        public ArrayList<Object> parentObjects;
        public long peer;
        public boolean performMediaUpload;
        public TLRPC.PhotoSize photoSize;
        ArrayList<DelayedMessageSendAfterRequest> requests;
        public boolean scheduled;
        public TLObject sendEncryptedRequest;
        public TLObject sendRequest;
        public int type;
        public VideoEditedInfo videoEditedInfo;
        public ArrayList<VideoEditedInfo> videoEditedInfos;

        public DelayedMessage(long peer) {
            this.peer = peer;
        }

        public void initForGroup(long id) {
            this.type = 4;
            this.groupId = id;
            this.messageObjects = new ArrayList<>();
            this.messages = new ArrayList<>();
            this.inputMedias = new ArrayList<>();
            this.originalPaths = new ArrayList<>();
            this.parentObjects = new ArrayList<>();
            this.extraHashMap = new HashMap<>();
            this.locations = new ArrayList<>();
            this.httpLocations = new ArrayList<>();
            this.videoEditedInfos = new ArrayList<>();
        }

        public void addDelayedRequest(TLObject req, MessageObject msgObj, String originalPath, Object parentObject, DelayedMessage delayedMessage, boolean scheduled) {
            DelayedMessageSendAfterRequest request = new DelayedMessageSendAfterRequest();
            request.request = req;
            request.msgObj = msgObj;
            request.originalPath = originalPath;
            request.delayedMessage = delayedMessage;
            request.parentObject = parentObject;
            request.scheduled = scheduled;
            if (this.requests == null) {
                this.requests = new ArrayList<>();
            }
            this.requests.add(request);
        }

        public void addDelayedRequest(TLObject req, ArrayList<MessageObject> msgObjs, ArrayList<String> originalPaths, ArrayList<Object> parentObjects, DelayedMessage delayedMessage, boolean scheduled) {
            DelayedMessageSendAfterRequest request = new DelayedMessageSendAfterRequest();
            request.request = req;
            request.msgObjs = msgObjs;
            request.originalPaths = originalPaths;
            request.delayedMessage = delayedMessage;
            request.parentObjects = parentObjects;
            request.scheduled = scheduled;
            if (this.requests == null) {
                this.requests = new ArrayList<>();
            }
            this.requests.add(request);
        }

        public void sendDelayedRequests() {
            ArrayList<DelayedMessageSendAfterRequest> arrayList = this.requests;
            if (arrayList != null) {
                int i = this.type;
                if (i != 4 && i != 0) {
                    return;
                }
                int size = arrayList.size();
                for (int a2 = 0; a2 < size; a2++) {
                    DelayedMessageSendAfterRequest request = this.requests.get(a2);
                    if (request.request instanceof TLRPC.TL_messages_sendEncryptedMultiMedia) {
                        SendMessagesHelper.this.getSecretChatHelper().performSendEncryptedRequest((TLRPC.TL_messages_sendEncryptedMultiMedia) request.request, this);
                    } else if (!(request.request instanceof TLRPC.TL_messages_sendMultiMedia)) {
                        SendMessagesHelper.this.performSendMessageRequest(request.request, request.msgObj, request.originalPath, request.delayedMessage, request.parentObject, request.scheduled);
                    } else {
                        SendMessagesHelper.this.performSendMessageRequestMulti((TLRPC.TL_messages_sendMultiMedia) request.request, request.msgObjs, request.originalPaths, request.parentObjects, request.delayedMessage, request.scheduled);
                    }
                }
                this.requests = null;
            }
        }

        public void markAsError() {
            if (this.type == 4) {
                for (int a2 = 0; a2 < this.messageObjects.size(); a2++) {
                    MessageObject obj = this.messageObjects.get(a2);
                    SendMessagesHelper.this.getMessagesStorage().markMessageAsSendError(obj.messageOwner, obj.scheduled);
                    obj.messageOwner.send_state = 2;
                    SendMessagesHelper.this.getNotificationCenter().postNotificationName(NotificationCenter.messageSendError, Integer.valueOf(obj.getId()));
                    SendMessagesHelper.this.processSentMessage(obj.getId());
                    SendMessagesHelper.this.removeFromUploadingMessages(obj.getId(), this.scheduled);
                }
                SendMessagesHelper.this.delayedMessages.remove("group_" + this.groupId);
            } else {
                SendMessagesHelper.this.getMessagesStorage().markMessageAsSendError(this.obj.messageOwner, this.obj.scheduled);
                this.obj.messageOwner.send_state = 2;
                SendMessagesHelper.this.getNotificationCenter().postNotificationName(NotificationCenter.messageSendError, Integer.valueOf(this.obj.getId()));
                SendMessagesHelper.this.processSentMessage(this.obj.getId());
                SendMessagesHelper.this.removeFromUploadingMessages(this.obj.getId(), this.scheduled);
            }
            sendDelayedRequests();
        }
    }

    public static SendMessagesHelper getInstance(int num) {
        SendMessagesHelper localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (SendMessagesHelper.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    SendMessagesHelper[] sendMessagesHelperArr = Instance;
                    SendMessagesHelper sendMessagesHelper = new SendMessagesHelper(num);
                    localInstance = sendMessagesHelper;
                    sendMessagesHelperArr[num] = sendMessagesHelper;
                }
            }
        }
        return localInstance;
    }

    public SendMessagesHelper(int instance) {
        super(instance);
        this.delayedMessages = new HashMap<>();
        this.unsentMessages = new SparseArray<>();
        this.sendingMessages = new SparseArray<>();
        this.editingMessages = new SparseArray<>();
        this.uploadMessages = new SparseArray<>();
        this.sendingMessagesIdDialogs = new LongSparseArray<>();
        this.uploadingMessagesIdDialogs = new LongSparseArray<>();
        this.waitingForLocation = new HashMap<>();
        this.waitingForCallback = new HashMap<>();
        this.waitingForVote = new HashMap<>();
        this.voteSendTime = new LongSparseArray<>();
        this.locationProvider = new LocationProvider(new LocationProvider.LocationProviderDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper.1
            @Override // im.uecahxyvjd.messenger.SendMessagesHelper.LocationProvider.LocationProviderDelegate
            public void onLocationAcquired(Location location) {
                SendMessagesHelper.this.sendLocation(location);
                SendMessagesHelper.this.waitingForLocation.clear();
            }

            @Override // im.uecahxyvjd.messenger.SendMessagesHelper.LocationProvider.LocationProviderDelegate
            public void onUnableLocationAcquire() {
                HashMap<String, MessageObject> waitingForLocationCopy = new HashMap<>(SendMessagesHelper.this.waitingForLocation);
                SendMessagesHelper.this.getNotificationCenter().postNotificationName(NotificationCenter.wasUnableToFindCurrentLocation, waitingForLocationCopy);
                SendMessagesHelper.this.waitingForLocation.clear();
            }
        });
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m957lambda$new$0$imuecahxyvjdmessengerSendMessagesHelper();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$new$0$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m957lambda$new$0$imuecahxyvjdmessengerSendMessagesHelper() {
        getNotificationCenter().addObserver(this, NotificationCenter.FileDidUpload);
        getNotificationCenter().addObserver(this, NotificationCenter.FileDidFailUpload);
        getNotificationCenter().addObserver(this, NotificationCenter.filePreparingStarted);
        getNotificationCenter().addObserver(this, NotificationCenter.fileNewChunkAvailable);
        getNotificationCenter().addObserver(this, NotificationCenter.filePreparingFailed);
        getNotificationCenter().addObserver(this, NotificationCenter.httpFileDidFailedLoad);
        getNotificationCenter().addObserver(this, NotificationCenter.httpFileDidLoad);
        getNotificationCenter().addObserver(this, NotificationCenter.fileDidLoad);
        getNotificationCenter().addObserver(this, NotificationCenter.fileDidFailToLoad);
    }

    public void cleanup() {
        this.delayedMessages.clear();
        this.unsentMessages.clear();
        this.sendingMessages.clear();
        this.editingMessages.clear();
        this.sendingMessagesIdDialogs.clear();
        this.uploadMessages.clear();
        this.uploadingMessagesIdDialogs.clear();
        this.waitingForLocation.clear();
        this.waitingForCallback.clear();
        this.waitingForVote.clear();
        this.locationProvider.stop();
    }

    @Override // im.uecahxyvjd.messenger.NotificationCenter.NotificationCenterDelegate
    public void didReceivedNotification(int id, int account, Object... args) {
        String path;
        ArrayList<DelayedMessage> arr;
        int fileType;
        final MessageObject messageObject;
        ArrayList<DelayedMessage> arr2;
        ArrayList<DelayedMessage> arr3;
        TLRPC.InputMedia media;
        ArrayList<DelayedMessage> arr4;
        String location;
        TLRPC.InputFile file;
        String str;
        TLRPC.InputEncryptedFile encryptedFile;
        int a2;
        int a3;
        TLRPC.InputEncryptedFile encryptedFile2;
        String str2 = "_t";
        if (id == NotificationCenter.FileDidUpload) {
            String location2 = (String) args[0];
            TLRPC.InputFile file2 = (TLRPC.InputFile) args[1];
            TLRPC.InputEncryptedFile encryptedFile3 = (TLRPC.InputEncryptedFile) args[2];
            ArrayList<DelayedMessage> arr5 = this.delayedMessages.get(location2);
            if (arr5 != null) {
                int a4 = 0;
                while (true) {
                    int a5 = a4;
                    if (a5 >= arr5.size()) {
                        break;
                    }
                    DelayedMessage message = arr5.get(a5);
                    if (message.sendRequest instanceof TLRPC.TL_messages_sendMedia) {
                        TLRPC.InputMedia media2 = ((TLRPC.TL_messages_sendMedia) message.sendRequest).media;
                        media = media2;
                    } else if (message.sendRequest instanceof TLRPC.TL_messages_editMessage) {
                        TLRPC.InputMedia media3 = ((TLRPC.TL_messages_editMessage) message.sendRequest).media;
                        media = media3;
                    } else if (!(message.sendRequest instanceof TLRPC.TL_messages_sendMultiMedia)) {
                        media = null;
                    } else {
                        TLRPC.InputMedia media4 = (TLRPC.InputMedia) message.extraHashMap.get(location2);
                        media = media4;
                    }
                    if (file2 != null && media != null) {
                        if (message.type == 0) {
                            media.file = file2;
                            a3 = a5;
                            arr4 = arr5;
                            String location3 = location2;
                            encryptedFile2 = encryptedFile3;
                            performSendMessageRequest(message.sendRequest, message.obj, message.originalPath, message, true, null, message.parentObject, message.scheduled);
                            file = file2;
                            location = location3;
                        } else {
                            TLRPC.InputMedia media5 = media;
                            a3 = a5;
                            arr4 = arr5;
                            encryptedFile2 = encryptedFile3;
                            TLRPC.InputFile file3 = file2;
                            String location4 = location2;
                            if (message.type == 1) {
                                if (media5.file == null) {
                                    file = file3;
                                    media5.file = file;
                                    if (media5.thumb == null && message.photoSize != null && message.photoSize.location != null) {
                                        performSendDelayedMessage(message);
                                        location = location4;
                                    } else {
                                        performSendMessageRequest(message.sendRequest, message.obj, message.originalPath, null, message.parentObject, message.scheduled);
                                        location = location4;
                                    }
                                } else {
                                    file = file3;
                                    media5.thumb = file;
                                    media5.flags |= 4;
                                    performSendMessageRequest(message.sendRequest, message.obj, message.originalPath, null, message.parentObject, message.scheduled);
                                    location = location4;
                                }
                            } else {
                                file = file3;
                                if (message.type == 2) {
                                    if (media5.file == null) {
                                        media5.file = file;
                                        if (media5.thumb == null && message.photoSize != null && message.photoSize.location != null) {
                                            performSendDelayedMessage(message);
                                            location = location4;
                                        } else {
                                            performSendMessageRequest(message.sendRequest, message.obj, message.originalPath, null, message.parentObject, message.scheduled);
                                            location = location4;
                                        }
                                    } else {
                                        media5.thumb = file;
                                        media5.flags |= 4;
                                        performSendMessageRequest(message.sendRequest, message.obj, message.originalPath, null, message.parentObject, message.scheduled);
                                        location = location4;
                                    }
                                } else if (message.type == 3) {
                                    media5.file = file;
                                    performSendMessageRequest(message.sendRequest, message.obj, message.originalPath, null, message.parentObject, message.scheduled);
                                    location = location4;
                                } else if (message.type != 4) {
                                    location = location4;
                                } else if (media5 instanceof TLRPC.TL_inputMediaUploadedDocument) {
                                    if (media5.file == null) {
                                        media5.file = file;
                                        location = location4;
                                        MessageObject messageObject2 = (MessageObject) message.extraHashMap.get(location + "_i");
                                        int index = message.messageObjects.indexOf(messageObject2);
                                        message.photoSize = (TLRPC.PhotoSize) message.extraHashMap.get(location + str2);
                                        stopVideoService(message.messageObjects.get(index).messageOwner.attachPath);
                                        if (media5.thumb != null || message.photoSize == null) {
                                            uploadMultiMedia(message, media5, null, location);
                                        } else {
                                            message.performMediaUpload = true;
                                            performSendDelayedMessage(message, index);
                                        }
                                    } else {
                                        location = location4;
                                        media5.thumb = file;
                                        media5.flags |= 4;
                                        uploadMultiMedia(message, media5, null, (String) message.extraHashMap.get(location + "_o"));
                                    }
                                } else {
                                    location = location4;
                                    media5.file = file;
                                    uploadMultiMedia(message, media5, null, location);
                                }
                            }
                        }
                        int a6 = a3;
                        arr4.remove(a6);
                        a2 = a6 - 1;
                        str = str2;
                        encryptedFile = encryptedFile2;
                    } else {
                        arr4 = arr5;
                        TLRPC.InputEncryptedFile encryptedFile4 = encryptedFile3;
                        location = location2;
                        file = file2;
                        if (encryptedFile4 == null || message.sendEncryptedRequest == null) {
                            str = str2;
                            encryptedFile = encryptedFile4;
                            a2 = a5;
                        } else {
                            TLRPC.TL_decryptedMessage decryptedMessage = null;
                            if (message.type == 4) {
                                TLRPC.TL_messages_sendEncryptedMultiMedia req = (TLRPC.TL_messages_sendEncryptedMultiMedia) message.sendEncryptedRequest;
                                TLRPC.InputEncryptedFile inputEncryptedFile = (TLRPC.InputEncryptedFile) message.extraHashMap.get(location);
                                int index2 = req.files.indexOf(inputEncryptedFile);
                                if (index2 < 0) {
                                    str = str2;
                                    encryptedFile = encryptedFile4;
                                } else {
                                    encryptedFile = encryptedFile4;
                                    req.files.set(index2, encryptedFile);
                                    str = str2;
                                    if (inputEncryptedFile.id == 1) {
                                        MessageObject messageObject3 = (MessageObject) message.extraHashMap.get(location + "_i");
                                        message.photoSize = (TLRPC.PhotoSize) message.extraHashMap.get(location + str);
                                        stopVideoService(message.messageObjects.get(index2).messageOwner.attachPath);
                                    }
                                    TLRPC.TL_decryptedMessage decryptedMessage2 = req.messages.get(index2);
                                    decryptedMessage = decryptedMessage2;
                                }
                            } else {
                                str = str2;
                                encryptedFile = encryptedFile4;
                                decryptedMessage = (TLRPC.TL_decryptedMessage) message.sendEncryptedRequest;
                            }
                            if (decryptedMessage != null) {
                                if ((decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaVideo) || (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaPhoto) || (decryptedMessage.media instanceof TLRPC.TL_decryptedMessageMediaDocument)) {
                                    long size = ((Long) args[5]).longValue();
                                    decryptedMessage.media.size = (int) size;
                                }
                                decryptedMessage.media.key = (byte[]) args[3];
                                decryptedMessage.media.iv = (byte[]) args[4];
                                if (message.type == 4) {
                                    uploadMultiMedia(message, null, encryptedFile, location);
                                } else {
                                    getSecretChatHelper().performSendEncryptedRequest(decryptedMessage, message.obj.messageOwner, message.encryptedChat, encryptedFile, message.originalPath, message.obj);
                                }
                            }
                            arr4.remove(a5);
                            a2 = a5 - 1;
                        }
                    }
                    a4 = a2 + 1;
                    file2 = file;
                    str2 = str;
                    encryptedFile3 = encryptedFile;
                    arr5 = arr4;
                    location2 = location;
                }
                String location5 = location2;
                if (arr5.isEmpty()) {
                    this.delayedMessages.remove(location5);
                }
            }
        } else if (id == NotificationCenter.FileDidFailUpload) {
            String location6 = (String) args[0];
            boolean enc = ((Boolean) args[1]).booleanValue();
            ArrayList<DelayedMessage> arr6 = this.delayedMessages.get(location6);
            if (arr6 != null) {
                int a7 = 0;
                while (a7 < arr6.size()) {
                    DelayedMessage obj = arr6.get(a7);
                    if ((enc && obj.sendEncryptedRequest != null) || (!enc && obj.sendRequest != null)) {
                        obj.markAsError();
                        arr6.remove(a7);
                        a7--;
                    }
                    a7++;
                }
                if (arr6.isEmpty()) {
                    this.delayedMessages.remove(location6);
                }
            }
        } else if (id == NotificationCenter.filePreparingStarted) {
            MessageObject messageObject4 = (MessageObject) args[0];
            if (messageObject4.getId() == 0) {
                return;
            }
            String str3 = (String) args[1];
            ArrayList<DelayedMessage> arr7 = this.delayedMessages.get(messageObject4.messageOwner.attachPath);
            if (arr7 != null) {
                int a8 = 0;
                while (true) {
                    if (a8 >= arr7.size()) {
                        break;
                    }
                    DelayedMessage message2 = arr7.get(a8);
                    if (message2.type == 4) {
                        int index3 = message2.messageObjects.indexOf(messageObject4);
                        message2.photoSize = (TLRPC.PhotoSize) message2.extraHashMap.get(messageObject4.messageOwner.attachPath + "_t");
                        message2.performMediaUpload = true;
                        performSendDelayedMessage(message2, index3);
                        arr7.remove(a8);
                        break;
                    } else if (message2.obj != messageObject4) {
                        a8++;
                    } else {
                        message2.videoEditedInfo = null;
                        performSendDelayedMessage(message2);
                        arr7.remove(a8);
                        break;
                    }
                }
                if (arr7.isEmpty()) {
                    this.delayedMessages.remove(messageObject4.messageOwner.attachPath);
                }
            }
        } else if (id == NotificationCenter.fileNewChunkAvailable) {
            MessageObject messageObject5 = (MessageObject) args[0];
            if (messageObject5.getId() == 0) {
                return;
            }
            String finalPath = (String) args[1];
            long availableSize = ((Long) args[2]).longValue();
            long finalSize = ((Long) args[3]).longValue();
            boolean isEncrypted = ((int) messageObject5.getDialogId()) == 0;
            getFileLoader().checkUploadNewDataAvailable(finalPath, isEncrypted, availableSize, finalSize);
            if (finalSize != 0) {
                stopVideoService(messageObject5.messageOwner.attachPath);
                ArrayList<DelayedMessage> arr8 = this.delayedMessages.get(messageObject5.messageOwner.attachPath);
                if (arr8 != null) {
                    int a9 = 0;
                    while (a9 < arr8.size()) {
                        DelayedMessage message3 = arr8.get(a9);
                        if (message3.type == 4) {
                            int b = 0;
                            while (true) {
                                if (b >= message3.messageObjects.size()) {
                                    arr3 = arr8;
                                    break;
                                }
                                MessageObject obj2 = message3.messageObjects.get(b);
                                if (obj2 != messageObject5) {
                                    b++;
                                } else {
                                    obj2.videoEditedInfo = null;
                                    obj2.messageOwner.params.remove("ve");
                                    obj2.messageOwner.media.document.size = (int) finalSize;
                                    ArrayList<TLRPC.Message> messages = new ArrayList<>();
                                    messages.add(obj2.messageOwner);
                                    arr3 = arr8;
                                    getMessagesStorage().putMessages(messages, false, true, false, 0, obj2.scheduled);
                                    break;
                                }
                            }
                        } else {
                            arr3 = arr8;
                            if (message3.obj == messageObject5) {
                                message3.obj.videoEditedInfo = null;
                                message3.obj.messageOwner.params.remove("ve");
                                message3.obj.messageOwner.media.document.size = (int) finalSize;
                                ArrayList<TLRPC.Message> messages2 = new ArrayList<>();
                                messages2.add(message3.obj.messageOwner);
                                getMessagesStorage().putMessages(messages2, false, true, false, 0, message3.obj.scheduled);
                                return;
                            }
                        }
                        a9++;
                        arr8 = arr3;
                    }
                }
            }
        } else if (id == NotificationCenter.filePreparingFailed) {
            MessageObject messageObject6 = (MessageObject) args[0];
            if (messageObject6.getId() == 0) {
                return;
            }
            String finalPath2 = (String) args[1];
            stopVideoService(messageObject6.messageOwner.attachPath);
            ArrayList<DelayedMessage> arr9 = this.delayedMessages.get(finalPath2);
            if (arr9 != null) {
                int a10 = 0;
                while (a10 < arr9.size()) {
                    DelayedMessage message4 = arr9.get(a10);
                    if (message4.type == 4) {
                        int b2 = 0;
                        while (true) {
                            if (b2 < message4.messages.size()) {
                                if (message4.messageObjects.get(b2) != messageObject6) {
                                    b2++;
                                } else {
                                    message4.markAsError();
                                    arr9.remove(a10);
                                    a10--;
                                    break;
                                }
                            } else {
                                break;
                            }
                        }
                    } else if (message4.obj == messageObject6) {
                        message4.markAsError();
                        arr9.remove(a10);
                        a10--;
                    }
                    a10++;
                }
                if (arr9.isEmpty()) {
                    this.delayedMessages.remove(finalPath2);
                }
            }
        } else if (id == NotificationCenter.httpFileDidLoad) {
            final String path2 = (String) args[0];
            ArrayList<DelayedMessage> arr10 = this.delayedMessages.get(path2);
            if (arr10 != null) {
                int a11 = 0;
                while (a11 < arr10.size()) {
                    final DelayedMessage message5 = arr10.get(a11);
                    if (message5.type == 0) {
                        fileType = 0;
                        messageObject = message5.obj;
                    } else if (message5.type == 2) {
                        fileType = 1;
                        messageObject = message5.obj;
                    } else if (message5.type != 4) {
                        fileType = -1;
                        messageObject = null;
                    } else {
                        MessageObject messageObject7 = (MessageObject) message5.extraHashMap.get(path2);
                        if (messageObject7.getDocument() != null) {
                            fileType = 1;
                            messageObject = messageObject7;
                        } else {
                            fileType = 0;
                            messageObject = messageObject7;
                        }
                    }
                    if (fileType == 0) {
                        String md5 = Utilities.MD5(path2) + "." + ImageLoader.getHttpUrlExtension(path2, "file");
                        final File cacheFile = new File(FileLoader.getDirectory(4), md5);
                        final MessageObject messageObject8 = messageObject;
                        arr2 = arr10;
                        Utilities.globalQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda35
                            @Override // java.lang.Runnable
                            public final void run() {
                                SendMessagesHelper.this.m952x90e33d53(cacheFile, messageObject8, message5, path2);
                            }
                        });
                    } else {
                        arr2 = arr10;
                        if (fileType == 1) {
                            String md52 = Utilities.MD5(path2) + ".gif";
                            final File cacheFile2 = new File(FileLoader.getDirectory(4), md52);
                            Utilities.globalQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda36
                                @Override // java.lang.Runnable
                                public final void run() {
                                    SendMessagesHelper.this.m954x16bc1011(message5, cacheFile2, messageObject);
                                }
                            });
                        }
                    }
                    a11++;
                    arr10 = arr2;
                }
                this.delayedMessages.remove(path2);
            }
        } else if (id == NotificationCenter.fileDidLoad) {
            String path3 = (String) args[0];
            ArrayList<DelayedMessage> arr11 = this.delayedMessages.get(path3);
            if (arr11 != null) {
                for (int a12 = 0; a12 < arr11.size(); a12++) {
                    performSendDelayedMessage(arr11.get(a12));
                }
                this.delayedMessages.remove(path3);
            }
        } else if ((id == NotificationCenter.httpFileDidFailedLoad || id == NotificationCenter.fileDidFailToLoad) && (arr = this.delayedMessages.get((path = (String) args[0]))) != null) {
            for (int a13 = 0; a13 < arr.size(); a13++) {
                arr.get(a13).markAsError();
            }
            this.delayedMessages.remove(path);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$didReceivedNotification$2$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m952x90e33d53(final File cacheFile, final MessageObject messageObject, final DelayedMessage message, final String path) {
        final TLRPC.TL_photo photo = generatePhotoSizes(cacheFile.toString(), null, false);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda25
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m951xcdf6d3f4(photo, messageObject, cacheFile, message, path);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$didReceivedNotification$1$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m951xcdf6d3f4(TLRPC.TL_photo photo, MessageObject messageObject, File cacheFile, DelayedMessage message, String path) {
        if (photo != null) {
            messageObject.messageOwner.media.photo = photo;
            messageObject.messageOwner.attachPath = cacheFile.toString();
            ArrayList<TLRPC.Message> messages = new ArrayList<>();
            messages.add(messageObject.messageOwner);
            getMessagesStorage().putMessages(messages, false, true, false, 0, messageObject.scheduled);
            getNotificationCenter().postNotificationName(NotificationCenter.updateMessageMedia, messageObject.messageOwner);
            message.photoSize = photo.sizes.get(photo.sizes.size() - 1);
            message.locationParent = photo;
            message.httpLocation = null;
            if (message.type == 4) {
                message.performMediaUpload = true;
                performSendDelayedMessage(message, message.messageObjects.indexOf(messageObject));
                return;
            }
            performSendDelayedMessage(message);
            return;
        }
        if (BuildVars.LOGS_ENABLED) {
            FileLog.e("can't load image " + path + " to file " + cacheFile.toString());
        }
        message.markAsError();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$didReceivedNotification$4$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m954x16bc1011(final DelayedMessage message, final File cacheFile, final MessageObject messageObject) {
        final TLRPC.Document document = message.obj.getDocument();
        if (document.thumbs.isEmpty() || (document.thumbs.get(0).location instanceof TLRPC.TL_fileLocationUnavailable)) {
            try {
                Bitmap bitmap = ImageLoader.loadBitmap(cacheFile.getAbsolutePath(), null, 90.0f, 90.0f, true);
                if (bitmap != null) {
                    document.thumbs.clear();
                    document.thumbs.add(ImageLoader.scaleAndSaveImage(bitmap, 90.0f, 90.0f, 55, message.sendEncryptedRequest != null));
                    bitmap.recycle();
                }
            } catch (Exception e) {
                document.thumbs.clear();
                FileLog.e(e);
            }
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda4
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m953x53cfa6b2(message, cacheFile, document, messageObject);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$didReceivedNotification$3$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m953x53cfa6b2(DelayedMessage message, File cacheFile, TLRPC.Document document, MessageObject messageObject) {
        message.httpLocation = null;
        message.obj.messageOwner.attachPath = cacheFile.toString();
        if (!document.thumbs.isEmpty()) {
            message.photoSize = document.thumbs.get(0);
            message.locationParent = document;
        }
        ArrayList<TLRPC.Message> messages = new ArrayList<>();
        messages.add(messageObject.messageOwner);
        getMessagesStorage().putMessages(messages, false, true, false, 0, messageObject.scheduled);
        message.performMediaUpload = true;
        performSendDelayedMessage(message);
        getNotificationCenter().postNotificationName(NotificationCenter.updateMessageMedia, message.obj.messageOwner);
    }

    private void revertEditingMessageObject(MessageObject object) {
        object.cancelEditing = true;
        object.messageOwner.media = object.previousMedia;
        object.messageOwner.message = object.previousCaption;
        object.messageOwner.entities = object.previousCaptionEntities;
        object.messageOwner.attachPath = object.previousAttachPath;
        object.messageOwner.send_state = 0;
        object.previousMedia = null;
        object.previousCaption = null;
        object.previousCaptionEntities = null;
        object.previousAttachPath = null;
        object.videoEditedInfo = null;
        object.type = -1;
        object.setType();
        object.caption = null;
        object.generateCaption();
        ArrayList<TLRPC.Message> arr = new ArrayList<>();
        arr.add(object.messageOwner);
        getMessagesStorage().putMessages(arr, false, true, false, 0, object.scheduled);
        ArrayList<MessageObject> arrayList = new ArrayList<>();
        arrayList.add(object);
        getNotificationCenter().postNotificationName(NotificationCenter.replaceMessagesObjects, Long.valueOf(object.getDialogId()), arrayList);
    }

    public void cancelSendingMessage(MessageObject object) {
        ArrayList<MessageObject> arrayList = new ArrayList<>();
        arrayList.add(object);
        cancelSendingMessage(arrayList);
    }

    public void cancelSendingMessage(ArrayList<MessageObject> objects) {
        boolean scheduled;
        long scheduledDialogId;
        TLRPC.Message sendingMessage;
        Iterator<Map.Entry<String, ArrayList<DelayedMessage>>> it;
        boolean enc;
        int channelId;
        int b;
        MessageObject messageObject;
        ArrayList<String> keysToRemove = new ArrayList<>();
        ArrayList<DelayedMessage> checkReadyToSendGroups = new ArrayList<>();
        ArrayList<Integer> messageIds = new ArrayList<>();
        int c = 0;
        boolean enc2 = false;
        int channelId2 = 0;
        boolean scheduled2 = false;
        long scheduledDialogId2 = 0;
        while (c < objects.size()) {
            MessageObject object = objects.get(c);
            if (!object.scheduled) {
                scheduled = scheduled2;
                scheduledDialogId = scheduledDialogId2;
            } else {
                scheduled = true;
                scheduledDialogId = object.getDialogId();
            }
            messageIds.add(Integer.valueOf(object.getId()));
            channelId2 = object.messageOwner.to_id.channel_id;
            TLRPC.Message sendingMessage2 = removeFromSendingMessages(object.getId(), object.scheduled);
            if (sendingMessage2 != null) {
                getConnectionsManager().cancelRequest(sendingMessage2.reqId, true);
            }
            Iterator<Map.Entry<String, ArrayList<DelayedMessage>>> it2 = this.delayedMessages.entrySet().iterator();
            while (it2.hasNext()) {
                Map.Entry<String, ArrayList<DelayedMessage>> entry = it2.next();
                ArrayList<DelayedMessage> messages = entry.getValue();
                long scheduledDialogId3 = scheduledDialogId;
                int a2 = 0;
                while (true) {
                    if (a2 >= messages.size()) {
                        sendingMessage = sendingMessage2;
                        it = it2;
                        enc = enc2;
                        channelId = channelId2;
                        break;
                    }
                    DelayedMessage message = messages.get(a2);
                    sendingMessage = sendingMessage2;
                    it = it2;
                    if (message.type == 4) {
                        MessageObject messageObject2 = null;
                        int index = 0;
                        while (true) {
                            MessageObject messageObject3 = messageObject2;
                            if (index >= message.messageObjects.size()) {
                                enc = enc2;
                                b = -1;
                                messageObject = messageObject3;
                                break;
                            }
                            MessageObject messageObject4 = message.messageObjects.get(index);
                            enc = enc2;
                            if (messageObject4.getId() == object.getId()) {
                                removeFromUploadingMessages(object.getId(), object.scheduled);
                                b = index;
                                messageObject = messageObject4;
                                break;
                            }
                            index++;
                            messageObject2 = messageObject4;
                            enc2 = enc;
                        }
                        if (b >= 0) {
                            message.messageObjects.remove(b);
                            message.messages.remove(b);
                            message.originalPaths.remove(b);
                            if (message.sendRequest != null) {
                                channelId = channelId2;
                                ((TLRPC.TL_messages_sendMultiMedia) message.sendRequest).multi_media.remove(b);
                            } else {
                                channelId = channelId2;
                                TLRPC.TL_messages_sendEncryptedMultiMedia request = (TLRPC.TL_messages_sendEncryptedMultiMedia) message.sendEncryptedRequest;
                                request.messages.remove(b);
                                request.files.remove(b);
                            }
                            MediaController.getInstance().cancelVideoConvert(object);
                            String keyToRemove = (String) message.extraHashMap.get(messageObject);
                            if (keyToRemove != null) {
                                keysToRemove.add(keyToRemove);
                            }
                            if (message.messageObjects.isEmpty()) {
                                message.sendDelayedRequests();
                            } else {
                                int i = message.finalGroupMessage;
                                int index2 = object.getId();
                                if (i == index2) {
                                    MessageObject prevMessage = message.messageObjects.get(message.messageObjects.size() - 1);
                                    message.finalGroupMessage = prevMessage.getId();
                                    prevMessage.messageOwner.params.put("final", "1");
                                    TLRPC.TL_messages_messages messagesRes = new TLRPC.TL_messages_messages();
                                    messagesRes.messages.add(prevMessage.messageOwner);
                                    getMessagesStorage().putMessages((TLRPC.messages_Messages) messagesRes, message.peer, -2, 0, false, scheduled);
                                }
                                if (!checkReadyToSendGroups.contains(message)) {
                                    checkReadyToSendGroups.add(message);
                                }
                            }
                        } else {
                            channelId = channelId2;
                        }
                    } else {
                        enc = enc2;
                        channelId = channelId2;
                        if (message.obj.getId() == object.getId()) {
                            removeFromUploadingMessages(object.getId(), object.scheduled);
                            messages.remove(a2);
                            message.sendDelayedRequests();
                            MediaController.getInstance().cancelVideoConvert(message.obj);
                            if (messages.size() == 0) {
                                keysToRemove.add(entry.getKey());
                                if (message.sendEncryptedRequest != null) {
                                    enc2 = true;
                                }
                            }
                        } else {
                            a2++;
                            sendingMessage2 = sendingMessage;
                            it2 = it;
                            enc2 = enc;
                            channelId2 = channelId;
                        }
                    }
                }
                enc2 = enc;
                scheduledDialogId = scheduledDialogId3;
                sendingMessage2 = sendingMessage;
                it2 = it;
                channelId2 = channelId;
            }
            c++;
            scheduled2 = scheduled;
            scheduledDialogId2 = scheduledDialogId;
        }
        for (int a3 = 0; a3 < keysToRemove.size(); a3++) {
            String key = keysToRemove.get(a3);
            if (key.startsWith("http")) {
                ImageLoader.getInstance().cancelLoadHttpFile(key);
            } else {
                getFileLoader().cancelUploadFile(key, enc2);
            }
            stopVideoService(key);
            this.delayedMessages.remove(key);
        }
        int N = checkReadyToSendGroups.size();
        for (int a4 = 0; a4 < N; a4++) {
            sendReadyToSendGroup(checkReadyToSendGroups.get(a4), false, true);
        }
        int a5 = objects.size();
        if (a5 == 1 && objects.get(0).isEditing() && objects.get(0).previousMedia != null) {
            revertEditingMessageObject(objects.get(0));
        } else {
            getMessagesController().deleteMessages(messageIds, null, null, scheduledDialogId2, channelId2, false, scheduled2);
        }
    }

    public boolean retrySendMessage(MessageObject messageObject, boolean unsent) {
        if (messageObject.getId() >= 0) {
            if (messageObject.isEditing()) {
                editMessageMedia(messageObject, null, null, null, null, null, true, messageObject);
            }
            return false;
        } else if (messageObject.messageOwner.action instanceof TLRPC.TL_messageEncryptedAction) {
            int enc_id = (int) (messageObject.getDialogId() >> 32);
            TLRPC.EncryptedChat encryptedChat = getMessagesController().getEncryptedChat(Integer.valueOf(enc_id));
            if (encryptedChat == null) {
                getMessagesStorage().markMessageAsSendError(messageObject.messageOwner, messageObject.scheduled);
                messageObject.messageOwner.send_state = 2;
                getNotificationCenter().postNotificationName(NotificationCenter.messageSendError, Integer.valueOf(messageObject.getId()));
                processSentMessage(messageObject.getId());
                return false;
            }
            if (messageObject.messageOwner.random_id == 0) {
                messageObject.messageOwner.random_id = getNextRandomId();
            }
            if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionSetMessageTTL) {
                getSecretChatHelper().sendTTLMessage(encryptedChat, messageObject.messageOwner);
            } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionDeleteMessages) {
                getSecretChatHelper().sendMessagesDeleteMessage(encryptedChat, null, messageObject.messageOwner);
            } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionFlushHistory) {
                getSecretChatHelper().sendClearHistoryMessage(encryptedChat, messageObject.messageOwner);
            } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionNotifyLayer) {
                getSecretChatHelper().sendNotifyLayerMessage(encryptedChat, messageObject.messageOwner);
            } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionReadMessages) {
                getSecretChatHelper().sendMessagesReadMessage(encryptedChat, null, messageObject.messageOwner);
            } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionScreenshotMessages) {
                getSecretChatHelper().sendScreenshotMessage(encryptedChat, null, messageObject.messageOwner);
            } else if (!(messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionTyping) && !(messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionResend)) {
                if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionCommitKey) {
                    getSecretChatHelper().sendCommitKeyMessage(encryptedChat, messageObject.messageOwner);
                } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionAbortKey) {
                    getSecretChatHelper().sendAbortKeyMessage(encryptedChat, messageObject.messageOwner, 0L);
                } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionRequestKey) {
                    getSecretChatHelper().sendRequestKeyMessage(encryptedChat, messageObject.messageOwner);
                } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionAcceptKey) {
                    getSecretChatHelper().sendAcceptKeyMessage(encryptedChat, messageObject.messageOwner);
                } else if (messageObject.messageOwner.action.encryptedAction instanceof TLRPC.TL_decryptedMessageActionNoop) {
                    getSecretChatHelper().sendNoopMessage(encryptedChat, messageObject.messageOwner);
                }
            }
            return true;
        } else {
            if (messageObject.messageOwner.action instanceof TLRPC.TL_messageActionScreenshotTaken) {
                TLRPC.User user = getMessagesController().getUser(Integer.valueOf((int) messageObject.getDialogId()));
                sendScreenshotMessage(user, messageObject.messageOwner.reply_to_msg_id, messageObject.messageOwner);
            }
            if (unsent) {
                this.unsentMessages.put(messageObject.getId(), messageObject);
            }
            sendMessage(messageObject);
            return true;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void processSentMessage(int id) {
        int prevSize = this.unsentMessages.size();
        this.unsentMessages.remove(id);
        if (prevSize != 0 && this.unsentMessages.size() == 0) {
            checkUnsentMessages();
        }
    }

    public void processForwardFromMyName(MessageObject messageObject, long did) {
        TLRPC.WebPage webPage;
        ArrayList<TLRPC.MessageEntity> entities;
        HashMap<String, String> params;
        if (messageObject == null) {
            return;
        }
        if (messageObject.messageOwner.media != null && !(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaEmpty) && !(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaWebPage) && !(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaGame) && !(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaInvoice)) {
            if (((int) did) == 0 && messageObject.messageOwner.to_id != null && ((messageObject.messageOwner.media.photo instanceof TLRPC.TL_photo) || (messageObject.messageOwner.media.document instanceof TLRPC.TL_document))) {
                HashMap<String, String> params2 = new HashMap<>();
                params2.put("parentObject", "sent_" + messageObject.messageOwner.to_id.channel_id + "_" + messageObject.getId());
                params = params2;
            } else {
                params = null;
            }
            if (messageObject.messageOwner.media.photo instanceof TLRPC.TL_photo) {
                sendMessage((TLRPC.TL_photo) messageObject.messageOwner.media.photo, null, did, messageObject.replyMessageObject, messageObject.messageOwner.message, messageObject.messageOwner.entities, null, params, true, 0, messageObject.messageOwner.media.ttl_seconds, messageObject);
            } else if (messageObject.messageOwner.media.document instanceof TLRPC.TL_document) {
                sendMessage((TLRPC.TL_document) messageObject.messageOwner.media.document, null, messageObject.messageOwner.attachPath, did, messageObject.replyMessageObject, messageObject.messageOwner.message, messageObject.messageOwner.entities, null, params, true, 0, messageObject.messageOwner.media.ttl_seconds, messageObject);
            } else {
                if (!(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaVenue) && !(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaGeo)) {
                    if (messageObject.messageOwner.media.phone_number == null) {
                        if (((int) did) != 0) {
                            ArrayList<MessageObject> arrayList = new ArrayList<>();
                            arrayList.add(messageObject);
                            sendMessage(arrayList, did, true, 0);
                            return;
                        }
                        return;
                    }
                    TLRPC.User user = new TLRPC.TL_userContact_old2();
                    user.phone = messageObject.messageOwner.media.phone_number;
                    user.first_name = messageObject.messageOwner.media.first_name;
                    user.last_name = messageObject.messageOwner.media.last_name;
                    user.id = messageObject.messageOwner.media.user_id;
                    sendMessage(user, did, messageObject.replyMessageObject, (TLRPC.ReplyMarkup) null, (HashMap<String, String>) null, true, 0);
                    return;
                }
                sendMessage(messageObject.messageOwner.media, did, messageObject.replyMessageObject, (TLRPC.ReplyMarkup) null, (HashMap<String, String>) null, true, 0);
            }
        } else if (messageObject.messageOwner.message != null) {
            if (!(messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaWebPage)) {
                webPage = null;
            } else {
                TLRPC.WebPage webPage2 = messageObject.messageOwner.media.webpage;
                webPage = webPage2;
            }
            if (messageObject.messageOwner.entities != null && !messageObject.messageOwner.entities.isEmpty()) {
                ArrayList<TLRPC.MessageEntity> entities2 = new ArrayList<>();
                for (int a2 = 0; a2 < messageObject.messageOwner.entities.size(); a2++) {
                    TLRPC.MessageEntity entity = messageObject.messageOwner.entities.get(a2);
                    if ((entity instanceof TLRPC.TL_messageEntityBold) || (entity instanceof TLRPC.TL_messageEntityItalic) || (entity instanceof TLRPC.TL_messageEntityPre) || (entity instanceof TLRPC.TL_messageEntityCode) || (entity instanceof TLRPC.TL_messageEntityTextUrl)) {
                        entities2.add(entity);
                    }
                }
                entities = entities2;
            } else {
                entities = null;
            }
            sendMessage(messageObject.messageOwner.message, did, messageObject.replyMessageObject, webPage, true, entities, null, null, true, 0);
        } else if (((int) did) != 0) {
            ArrayList<MessageObject> arrayList2 = new ArrayList<>();
            arrayList2.add(messageObject);
            sendMessage(arrayList2, did, true, 0);
        }
    }

    public void sendScreenshotMessage(TLRPC.User user, int messageId, TLRPC.Message resendMessage) {
        TLRPC.Message message;
        if (user != null && messageId != 0) {
            if (user.id == getUserConfig().getClientUserId()) {
                return;
            }
            TLRPC.TL_messages_sendScreenshotNotification req = new TLRPC.TL_messages_sendScreenshotNotification();
            req.peer = new TLRPC.TL_inputPeerUser();
            req.peer.access_hash = user.access_hash;
            req.peer.user_id = user.id;
            if (resendMessage != null) {
                req.reply_to_msg_id = messageId;
                req.random_id = resendMessage.random_id;
                message = resendMessage;
            } else {
                TLRPC.Message message2 = new TLRPC.TL_messageService();
                message2.random_id = getNextRandomId();
                message2.dialog_id = user.id;
                message2.unread = true;
                message2.out = true;
                int newMessageId = getUserConfig().getNewMessageId();
                message2.id = newMessageId;
                message2.local_id = newMessageId;
                message2.from_id = getUserConfig().getClientUserId();
                message2.flags |= 256;
                message2.flags |= 8;
                message2.reply_to_msg_id = messageId;
                message2.to_id = new TLRPC.TL_peerUser();
                message2.to_id.user_id = user.id;
                message2.date = getConnectionsManager().getCurrentTime();
                message2.action = new TLRPC.TL_messageActionScreenshotTaken();
                getUserConfig().saveConfig(false);
                message = message2;
            }
            req.random_id = message.random_id;
            MessageObject newMsgObj = new MessageObject(this.currentAccount, message, false);
            newMsgObj.messageOwner.send_state = 1;
            ArrayList<MessageObject> objArr = new ArrayList<>();
            objArr.add(newMsgObj);
            getMessagesController().updateInterfaceWithMessages(message.dialog_id, objArr, false);
            getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
            ArrayList<TLRPC.Message> arr = new ArrayList<>();
            arr.add(message);
            getMessagesStorage().putMessages(arr, false, true, false, 0, false);
            performSendMessageRequest(req, newMsgObj, null, null, null, false);
        }
    }

    public void sendSticker(TLRPC.Document document, long peer, MessageObject replyingMessageObject, Object parentObject, boolean notify, int scheduleDate) {
        TLRPC.Document document2;
        if (document == null) {
            return;
        }
        if (((int) peer) != 0) {
            document2 = document;
        } else {
            int high_id = (int) (peer >> 32);
            TLRPC.EncryptedChat encryptedChat = getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
            if (encryptedChat == null) {
                return;
            }
            TLRPC.TL_document_layer82 newDocument = new TLRPC.TL_document_layer82();
            newDocument.id = document.id;
            newDocument.access_hash = document.access_hash;
            newDocument.date = document.date;
            newDocument.mime_type = document.mime_type;
            newDocument.file_reference = document.file_reference;
            if (newDocument.file_reference == null) {
                newDocument.file_reference = new byte[0];
            }
            newDocument.size = document.size;
            newDocument.dc_id = document.dc_id;
            newDocument.attributes = new ArrayList<>(document.attributes);
            if (newDocument.mime_type == null) {
                newDocument.mime_type = "";
            }
            TLRPC.PhotoSize thumb = FileLoader.getClosestPhotoSizeWithSize(document.thumbs, 90);
            if (thumb instanceof TLRPC.TL_photoSize) {
                File file = FileLoader.getPathToAttach(thumb, true);
                if (file.exists()) {
                    try {
                        int length = (int) file.length();
                        byte[] arr = new byte[(int) file.length()];
                        RandomAccessFile reader = new RandomAccessFile(file, "r");
                        reader.readFully(arr);
                        TLRPC.PhotoSize newThumb = new TLRPC.TL_photoCachedSize();
                        TLRPC.TL_fileLocation_layer82 fileLocation = new TLRPC.TL_fileLocation_layer82();
                        fileLocation.dc_id = thumb.location.dc_id;
                        fileLocation.volume_id = thumb.location.volume_id;
                        fileLocation.local_id = thumb.location.local_id;
                        fileLocation.secret = thumb.location.secret;
                        newThumb.location = fileLocation;
                        newThumb.size = thumb.size;
                        newThumb.w = thumb.w;
                        newThumb.h = thumb.h;
                        newThumb.type = thumb.type;
                        newThumb.bytes = arr;
                        newDocument.thumbs.add(newThumb);
                        newDocument.flags = 1 | newDocument.flags;
                    } catch (Exception e) {
                        FileLog.e(e);
                    }
                }
            }
            if (newDocument.thumbs.isEmpty()) {
                TLRPC.PhotoSize thumb2 = new TLRPC.TL_photoSizeEmpty();
                thumb2.type = "s";
                newDocument.thumbs.add(thumb2);
            }
            document2 = newDocument;
        }
        if (document2 instanceof TLRPC.TL_document) {
            sendMessage((TLRPC.TL_document) document2, null, null, peer, replyingMessageObject, null, null, null, null, notify, scheduleDate, 0, parentObject);
        }
    }

    /* JADX WARN: Removed duplicated region for block: B:270:0x07ea  */
    /* JADX WARN: Removed duplicated region for block: B:271:0x0814  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public int sendMessage(ArrayList<MessageObject> messages, final long peer, boolean notify, final int scheduleDate) {
        boolean isMegagroup;
        boolean isSignature;
        boolean canSendStickers;
        boolean canSendMedia;
        boolean canSendPolls;
        boolean canSendPreview;
        TLRPC.Chat chat;
        int a2;
        TLRPC.InputPeer inputPeer;
        int myId;
        MessageObject msgObj;
        ArrayList<MessageObject> objArr;
        ArrayList<Long> randomIds;
        ArrayList<TLRPC.Message> arr;
        TLRPC.Chat chat2;
        int lower_id;
        ArrayList<Integer> ids;
        LongSparseArray<TLRPC.Message> messagesByRandomIds;
        final TLRPC.Peer to_id;
        LongSparseArray<Long> groupsMap;
        int a3;
        ArrayList<MessageObject> arrayList;
        long j;
        SendMessagesHelper sendMessagesHelper;
        TLRPC.Message newMsg;
        boolean groupedIdChanged;
        ArrayList<MessageObject> objArr2;
        int myId2;
        LongSparseArray<Long> groupsMap2;
        TLRPC.Message newMsg2;
        ArrayList<Long> randomIds2;
        MessageObject msgObj2;
        boolean z;
        SendMessagesHelper sendMessagesHelper2 = this;
        ArrayList<MessageObject> arrayList2 = messages;
        long j2 = peer;
        int i = scheduleDate;
        if (arrayList2 != null && !messages.isEmpty()) {
            int lower_id2 = (int) j2;
            if (lower_id2 == 0) {
                for (int a4 = 0; a4 < messages.size(); a4++) {
                    sendMessagesHelper2.processForwardFromMyName(arrayList2.get(a4), j2);
                }
                return 0;
            }
            TLRPC.Peer to_id2 = getMessagesController().getPeer((int) j2);
            boolean isMegagroup2 = false;
            boolean isSignature2 = false;
            if (lower_id2 > 0) {
                TLRPC.User sendToUser = getMessagesController().getUser(Integer.valueOf(lower_id2));
                if (sendToUser == null) {
                    return 0;
                }
                isMegagroup = false;
                isSignature = false;
                canSendStickers = true;
                canSendMedia = true;
                canSendPolls = true;
                canSendPreview = true;
                chat = null;
            } else {
                TLRPC.Chat chat3 = getMessagesController().getChat(Integer.valueOf(-lower_id2));
                if (ChatObject.isChannel(chat3)) {
                    isMegagroup2 = chat3.megagroup;
                    isSignature2 = chat3.signatures;
                }
                boolean canSendStickers2 = ChatObject.canSendStickers(chat3);
                boolean canSendMedia2 = ChatObject.canSendMedia(chat3);
                boolean canSendPreview2 = ChatObject.canSendEmbed(chat3);
                boolean canSendPolls2 = ChatObject.canSendPolls(chat3);
                isMegagroup = isMegagroup2;
                isSignature = isSignature2;
                canSendStickers = canSendStickers2;
                canSendMedia = canSendMedia2;
                canSendPolls = canSendPolls2;
                canSendPreview = canSendPreview2;
                chat = chat3;
            }
            LongSparseArray<Long> groupsMap3 = new LongSparseArray<>();
            ArrayList<MessageObject> objArr3 = new ArrayList<>();
            ArrayList<TLRPC.Message> arr2 = new ArrayList<>();
            ArrayList<Long> randomIds3 = new ArrayList<>();
            ArrayList<Integer> ids2 = new ArrayList<>();
            LongSparseArray<TLRPC.Message> messagesByRandomIds2 = new LongSparseArray<>();
            TLRPC.InputPeer inputPeer2 = getMessagesController().getInputPeer(lower_id2);
            int lower_id3 = lower_id2;
            int myId3 = getUserConfig().getClientUserId();
            TLRPC.InputPeer inputPeer3 = inputPeer2;
            final boolean toMyself = j2 == ((long) myId3);
            int a5 = 0;
            ArrayList<Integer> ids3 = ids2;
            LongSparseArray<TLRPC.Message> messagesByRandomIds3 = messagesByRandomIds2;
            ArrayList<TLRPC.Message> arr3 = arr2;
            ArrayList<Long> randomIds4 = randomIds3;
            ArrayList<MessageObject> objArr4 = objArr3;
            int sendResult = 0;
            while (a5 < messages.size()) {
                MessageObject msgObj3 = arrayList2.get(a5);
                if (msgObj3.getId() <= 0) {
                    a2 = a5;
                    inputPeer = inputPeer3;
                    myId = myId3;
                    msgObj = msgObj3;
                    objArr = objArr4;
                    randomIds = randomIds4;
                    arr = arr3;
                    chat2 = chat;
                    lower_id = lower_id3;
                    ids = ids3;
                    messagesByRandomIds = messagesByRandomIds3;
                    to_id = to_id2;
                    groupsMap = groupsMap3;
                } else if (msgObj3.needDrawBluredPreview()) {
                    a2 = a5;
                    inputPeer = inputPeer3;
                    myId = myId3;
                    msgObj = msgObj3;
                    objArr = objArr4;
                    randomIds = randomIds4;
                    arr = arr3;
                    chat2 = chat;
                    lower_id = lower_id3;
                    ids = ids3;
                    messagesByRandomIds = messagesByRandomIds3;
                    to_id = to_id2;
                    groupsMap = groupsMap3;
                } else {
                    boolean toMyself2 = toMyself;
                    if (!canSendStickers && (msgObj3.isSticker() || msgObj3.isAnimatedSticker() || msgObj3.isGif() || msgObj3.isGame())) {
                        if (sendResult != 0) {
                            a3 = a5;
                            inputPeer = inputPeer3;
                            myId = myId3;
                            objArr = objArr4;
                            randomIds = randomIds4;
                            arr = arr3;
                            chat2 = chat;
                            arrayList = arrayList2;
                            sendMessagesHelper = sendMessagesHelper2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            ids = ids3;
                            messagesByRandomIds = messagesByRandomIds3;
                            to_id = to_id2;
                            groupsMap = groupsMap3;
                            j = j2;
                            randomIds4 = randomIds;
                            arr3 = arr;
                            objArr4 = objArr;
                            i = scheduleDate;
                            sendMessagesHelper2 = sendMessagesHelper;
                            a5 = a3 + 1;
                            j2 = j;
                            arrayList2 = arrayList;
                            to_id2 = to_id;
                            groupsMap3 = groupsMap;
                            lower_id3 = lower_id;
                            chat = chat2;
                            inputPeer3 = inputPeer;
                            myId3 = myId;
                            ids3 = ids;
                            messagesByRandomIds3 = messagesByRandomIds;
                        } else {
                            sendResult = ChatObject.isActionBannedByDefault(chat, 8) ? 4 : 1;
                            a3 = a5;
                            inputPeer = inputPeer3;
                            myId = myId3;
                            chat2 = chat;
                            arrayList = arrayList2;
                            sendMessagesHelper = sendMessagesHelper2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            ids = ids3;
                            messagesByRandomIds = messagesByRandomIds3;
                            to_id = to_id2;
                            groupsMap = groupsMap3;
                            j = j2;
                            i = scheduleDate;
                            sendMessagesHelper2 = sendMessagesHelper;
                            a5 = a3 + 1;
                            j2 = j;
                            arrayList2 = arrayList;
                            to_id2 = to_id;
                            groupsMap3 = groupsMap;
                            lower_id3 = lower_id;
                            chat = chat2;
                            inputPeer3 = inputPeer;
                            myId3 = myId;
                            ids3 = ids;
                            messagesByRandomIds3 = messagesByRandomIds;
                        }
                    } else if (!canSendMedia && ((msgObj3.messageOwner.media instanceof TLRPC.TL_messageMediaPhoto) || (msgObj3.messageOwner.media instanceof TLRPC.TL_messageMediaDocument))) {
                        if (sendResult != 0) {
                            a3 = a5;
                            inputPeer = inputPeer3;
                            myId = myId3;
                            objArr = objArr4;
                            randomIds = randomIds4;
                            arr = arr3;
                            chat2 = chat;
                            arrayList = arrayList2;
                            sendMessagesHelper = sendMessagesHelper2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            ids = ids3;
                            messagesByRandomIds = messagesByRandomIds3;
                            to_id = to_id2;
                            groupsMap = groupsMap3;
                            j = j2;
                            randomIds4 = randomIds;
                            arr3 = arr;
                            objArr4 = objArr;
                            i = scheduleDate;
                            sendMessagesHelper2 = sendMessagesHelper;
                            a5 = a3 + 1;
                            j2 = j;
                            arrayList2 = arrayList;
                            to_id2 = to_id;
                            groupsMap3 = groupsMap;
                            lower_id3 = lower_id;
                            chat = chat2;
                            inputPeer3 = inputPeer;
                            myId3 = myId;
                            ids3 = ids;
                            messagesByRandomIds3 = messagesByRandomIds;
                        } else {
                            sendResult = ChatObject.isActionBannedByDefault(chat, 7) ? 5 : 2;
                            a3 = a5;
                            inputPeer = inputPeer3;
                            myId = myId3;
                            chat2 = chat;
                            arrayList = arrayList2;
                            sendMessagesHelper = sendMessagesHelper2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            ids = ids3;
                            messagesByRandomIds = messagesByRandomIds3;
                            to_id = to_id2;
                            groupsMap = groupsMap3;
                            j = j2;
                            i = scheduleDate;
                            sendMessagesHelper2 = sendMessagesHelper;
                            a5 = a3 + 1;
                            j2 = j;
                            arrayList2 = arrayList;
                            to_id2 = to_id;
                            groupsMap3 = groupsMap;
                            lower_id3 = lower_id;
                            chat = chat2;
                            inputPeer3 = inputPeer;
                            myId3 = myId;
                            ids3 = ids;
                            messagesByRandomIds3 = messagesByRandomIds;
                        }
                    } else if (!canSendPolls && (msgObj3.messageOwner.media instanceof TLRPC.TL_messageMediaPoll)) {
                        if (sendResult != 0) {
                            a3 = a5;
                            inputPeer = inputPeer3;
                            myId = myId3;
                            objArr = objArr4;
                            randomIds = randomIds4;
                            arr = arr3;
                            chat2 = chat;
                            arrayList = arrayList2;
                            sendMessagesHelper = sendMessagesHelper2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            ids = ids3;
                            messagesByRandomIds = messagesByRandomIds3;
                            to_id = to_id2;
                            groupsMap = groupsMap3;
                            j = j2;
                            randomIds4 = randomIds;
                            arr3 = arr;
                            objArr4 = objArr;
                            i = scheduleDate;
                            sendMessagesHelper2 = sendMessagesHelper;
                            a5 = a3 + 1;
                            j2 = j;
                            arrayList2 = arrayList;
                            to_id2 = to_id;
                            groupsMap3 = groupsMap;
                            lower_id3 = lower_id;
                            chat = chat2;
                            inputPeer3 = inputPeer;
                            myId3 = myId;
                            ids3 = ids;
                            messagesByRandomIds3 = messagesByRandomIds;
                        } else {
                            sendResult = ChatObject.isActionBannedByDefault(chat, 10) ? 6 : 3;
                            a3 = a5;
                            inputPeer = inputPeer3;
                            myId = myId3;
                            chat2 = chat;
                            arrayList = arrayList2;
                            sendMessagesHelper = sendMessagesHelper2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            ids = ids3;
                            messagesByRandomIds = messagesByRandomIds3;
                            to_id = to_id2;
                            groupsMap = groupsMap3;
                            j = j2;
                            i = scheduleDate;
                            sendMessagesHelper2 = sendMessagesHelper;
                            a5 = a3 + 1;
                            j2 = j;
                            arrayList2 = arrayList;
                            to_id2 = to_id;
                            groupsMap3 = groupsMap;
                            lower_id3 = lower_id;
                            chat = chat2;
                            inputPeer3 = inputPeer;
                            myId3 = myId;
                            ids3 = ids;
                            messagesByRandomIds3 = messagesByRandomIds;
                        }
                    } else {
                        TLRPC.Message newMsg3 = new TLRPC.TL_message();
                        final ArrayList<TLRPC.Message> arr4 = arr3;
                        chat2 = chat;
                        boolean forwardFromSaved = msgObj3.getDialogId() == ((long) myId3) && msgObj3.messageOwner.from_id == getUserConfig().getClientUserId();
                        if (msgObj3.isForwarded()) {
                            newMsg = newMsg3;
                            newMsg.fwd_from = new TLRPC.TL_messageFwdHeader();
                            groupedIdChanged = false;
                            newMsg.fwd_from.flags = msgObj3.messageOwner.fwd_from.flags;
                            newMsg.fwd_from.from_id = msgObj3.messageOwner.fwd_from.from_id;
                            newMsg.fwd_from.date = msgObj3.messageOwner.fwd_from.date;
                            newMsg.fwd_from.channel_id = msgObj3.messageOwner.fwd_from.channel_id;
                            newMsg.fwd_from.channel_post = msgObj3.messageOwner.fwd_from.channel_post;
                            newMsg.fwd_from.post_author = msgObj3.messageOwner.fwd_from.post_author;
                            newMsg.fwd_from.from_name = msgObj3.messageOwner.fwd_from.from_name;
                            newMsg.flags = 4;
                            objArr2 = objArr4;
                        } else {
                            newMsg = newMsg3;
                            groupedIdChanged = false;
                            if (forwardFromSaved) {
                                objArr2 = objArr4;
                            } else {
                                newMsg.fwd_from = new TLRPC.TL_messageFwdHeader();
                                newMsg.fwd_from.channel_post = msgObj3.getId();
                                newMsg.fwd_from.flags |= 4;
                                if (msgObj3.isFromUser()) {
                                    newMsg.fwd_from.from_id = msgObj3.messageOwner.from_id;
                                    newMsg.fwd_from.flags |= 1;
                                } else {
                                    newMsg.fwd_from.channel_id = msgObj3.messageOwner.to_id.channel_id;
                                    newMsg.fwd_from.flags |= 2;
                                    if (msgObj3.messageOwner.post && msgObj3.messageOwner.from_id > 0) {
                                        newMsg.fwd_from.from_id = msgObj3.messageOwner.from_id;
                                        newMsg.fwd_from.flags |= 1;
                                    }
                                }
                                if (msgObj3.messageOwner.post_author != null) {
                                    newMsg.fwd_from.post_author = msgObj3.messageOwner.post_author;
                                    newMsg.fwd_from.flags |= 8;
                                    objArr2 = objArr4;
                                } else if (msgObj3.isOutOwner() || msgObj3.messageOwner.from_id <= 0 || !msgObj3.messageOwner.post) {
                                    objArr2 = objArr4;
                                } else {
                                    TLRPC.User signUser = getMessagesController().getUser(Integer.valueOf(msgObj3.messageOwner.from_id));
                                    if (signUser == null) {
                                        objArr2 = objArr4;
                                    } else {
                                        objArr2 = objArr4;
                                        newMsg.fwd_from.post_author = ContactsController.formatName(signUser.first_name, signUser.last_name);
                                        newMsg.fwd_from.flags |= 8;
                                    }
                                }
                                newMsg.date = msgObj3.messageOwner.date;
                                newMsg.flags = 4;
                            }
                        }
                        if (j2 == myId3 && newMsg.fwd_from != null) {
                            newMsg.fwd_from.flags |= 16;
                            newMsg.fwd_from.saved_from_msg_id = msgObj3.getId();
                            newMsg.fwd_from.saved_from_peer = msgObj3.messageOwner.to_id;
                        }
                        if (!canSendPreview && (msgObj3.messageOwner.media instanceof TLRPC.TL_messageMediaWebPage)) {
                            newMsg.media = new TLRPC.TL_messageMediaEmpty();
                        } else {
                            newMsg.media = msgObj3.messageOwner.media;
                        }
                        if (newMsg.media != null) {
                            newMsg.flags |= 512;
                        }
                        if (isMegagroup) {
                            newMsg.flags |= Integer.MIN_VALUE;
                        }
                        if (msgObj3.messageOwner.via_bot_id != 0) {
                            newMsg.via_bot_id = msgObj3.messageOwner.via_bot_id;
                            newMsg.flags |= 2048;
                        }
                        newMsg.message = msgObj3.messageOwner.message;
                        newMsg.fwd_msg_id = msgObj3.getId();
                        newMsg.attachPath = msgObj3.messageOwner.attachPath;
                        newMsg.entities = msgObj3.messageOwner.entities;
                        if (!(msgObj3.messageOwner.reply_markup instanceof TLRPC.TL_replyInlineMarkup)) {
                            myId2 = myId3;
                        } else {
                            newMsg.flags |= 64;
                            newMsg.reply_markup = new TLRPC.TL_replyInlineMarkup();
                            int b = 0;
                            int N = msgObj3.messageOwner.reply_markup.rows.size();
                            while (b < N) {
                                TLRPC.TL_keyboardButtonRow oldRow = msgObj3.messageOwner.reply_markup.rows.get(b);
                                TLRPC.TL_keyboardButtonRow newRow = null;
                                int myId4 = myId3;
                                int N2 = oldRow.buttons.size();
                                int c = N;
                                int N3 = 0;
                                while (N3 < N2) {
                                    int N22 = N2;
                                    TLRPC.KeyboardButton button = oldRow.buttons.get(N3);
                                    TLRPC.TL_keyboardButtonRow oldRow2 = oldRow;
                                    if ((button instanceof TLRPC.TL_keyboardButtonUrlAuth) || (button instanceof TLRPC.TL_keyboardButtonUrl) || (button instanceof TLRPC.TL_keyboardButtonSwitchInline)) {
                                        if (button instanceof TLRPC.TL_keyboardButtonUrlAuth) {
                                            TLRPC.TL_keyboardButtonUrlAuth auth = new TLRPC.TL_keyboardButtonUrlAuth();
                                            auth.flags = button.flags;
                                            if (button.fwd_text != null) {
                                                String str = button.fwd_text;
                                                auth.fwd_text = str;
                                                auth.text = str;
                                            } else {
                                                auth.text = button.text;
                                            }
                                            auth.url = button.url;
                                            auth.button_id = button.button_id;
                                            button = auth;
                                        }
                                        if (newRow == null) {
                                            newRow = new TLRPC.TL_keyboardButtonRow();
                                            newMsg.reply_markup.rows.add(newRow);
                                        }
                                        newRow.buttons.add(button);
                                    }
                                    N3++;
                                    N2 = N22;
                                    oldRow = oldRow2;
                                }
                                b++;
                                N = c;
                                myId3 = myId4;
                            }
                            myId2 = myId3;
                        }
                        if (!newMsg.entities.isEmpty()) {
                            newMsg.flags |= 128;
                        }
                        if (newMsg.attachPath == null) {
                            newMsg.attachPath = "";
                        }
                        int newMessageId = getUserConfig().getNewMessageId();
                        newMsg.id = newMessageId;
                        newMsg.local_id = newMessageId;
                        newMsg.out = true;
                        long lastGroupedId = msgObj3.messageOwner.grouped_id;
                        if (lastGroupedId != 0) {
                            Long gId = groupsMap3.get(msgObj3.messageOwner.grouped_id);
                            if (gId == null) {
                                gId = Long.valueOf(Utilities.random.nextLong());
                                groupsMap3.put(msgObj3.messageOwner.grouped_id, gId);
                            }
                            newMsg.grouped_id = gId.longValue();
                            newMsg.flags |= 131072;
                        }
                        if (a5 == messages.size() - 1) {
                            groupsMap2 = groupsMap3;
                        } else {
                            MessageObject next = arrayList2.get(a5 + 1);
                            groupsMap2 = groupsMap3;
                            if (next.messageOwner.grouped_id != msgObj3.messageOwner.grouped_id) {
                                groupedIdChanged = true;
                            }
                        }
                        if (to_id2.channel_id != 0 && !isMegagroup) {
                            newMsg.from_id = isSignature ? getUserConfig().getClientUserId() : -to_id2.channel_id;
                            newMsg.post = true;
                        } else {
                            newMsg.from_id = getUserConfig().getClientUserId();
                            newMsg.flags |= 256;
                        }
                        if (newMsg.random_id == 0) {
                            newMsg.random_id = getNextRandomId();
                        }
                        randomIds4.add(Long.valueOf(newMsg.random_id));
                        final LongSparseArray<TLRPC.Message> messagesByRandomIds4 = messagesByRandomIds3;
                        messagesByRandomIds4.put(newMsg.random_id, newMsg);
                        ArrayList<Integer> ids4 = ids3;
                        ids4.add(Integer.valueOf(newMsg.fwd_msg_id));
                        newMsg.date = i != 0 ? i : getConnectionsManager().getCurrentTime();
                        if ((inputPeer3 instanceof TLRPC.TL_inputPeerChannel) && !isMegagroup) {
                            if (i == 0) {
                                newMsg.views = 1;
                                newMsg.flags |= 1024;
                            }
                        } else {
                            if ((msgObj3.messageOwner.flags & 1024) != 0 && i == 0) {
                                newMsg.views = msgObj3.messageOwner.views;
                                newMsg.flags |= 1024;
                            }
                            newMsg.unread = true;
                        }
                        newMsg.dialog_id = peer;
                        newMsg.to_id = to_id2;
                        if (MessageObject.isVoiceMessage(newMsg) || MessageObject.isRoundVideoMessage(newMsg)) {
                            if ((inputPeer3 instanceof TLRPC.TL_inputPeerChannel) && msgObj3.getChannelId() != 0) {
                                newMsg.media_unread = msgObj3.isContentUnread();
                            } else {
                                newMsg.media_unread = true;
                            }
                        }
                        if (msgObj3.messageOwner.to_id instanceof TLRPC.TL_peerChannel) {
                            newMsg.ttl = -msgObj3.messageOwner.to_id.channel_id;
                        }
                        to_id = to_id2;
                        groupsMap = groupsMap2;
                        MessageObject newMsgObj = new MessageObject(this.currentAccount, newMsg, true);
                        newMsgObj.scheduled = i != 0;
                        newMsgObj.messageOwner.send_state = 1;
                        final ArrayList<MessageObject> objArr5 = objArr2;
                        objArr5.add(newMsgObj);
                        arr4.add(newMsg);
                        putToSendingMessages(newMsg, i != 0);
                        if (!BuildVars.LOGS_ENABLED) {
                            newMsg2 = newMsg;
                            randomIds2 = randomIds4;
                        } else {
                            newMsg2 = newMsg;
                            randomIds2 = randomIds4;
                            FileLog.d("forward message user_id = " + inputPeer3.user_id + " chat_id = " + inputPeer3.chat_id + " channel_id = " + inputPeer3.channel_id + " access_hash = " + inputPeer3.access_hash);
                        }
                        if ((!groupedIdChanged || arr4.size() <= 0) && arr4.size() != 100 && a5 != messages.size() - 1 && (a5 == messages.size() - 1 || arrayList2.get(a5 + 1).getDialogId() == msgObj3.getDialogId())) {
                            a3 = a5;
                            inputPeer = inputPeer3;
                            objArr = objArr5;
                            arr = arr4;
                            ids = ids4;
                            j = peer;
                            arrayList = arrayList2;
                            randomIds = randomIds2;
                            toMyself = toMyself2;
                            myId = myId2;
                            sendMessagesHelper = this;
                            messagesByRandomIds = messagesByRandomIds4;
                            lower_id = lower_id3;
                        } else {
                            getMessagesStorage().putMessages(new ArrayList<>(arr4), false, true, false, 0, i != 0);
                            getMessagesController().updateInterfaceWithMessages(peer, objArr5, i != 0);
                            int a6 = a5;
                            getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
                            getUserConfig().saveConfig(false);
                            final TLRPC.TL_messages_forwardMessages req = new TLRPC.TL_messages_forwardMessages();
                            req.to_peer = inputPeer3;
                            req.grouped = lastGroupedId != 0;
                            req.silent = !notify || MessagesController.getNotificationsSettings(this.currentAccount).getBoolean(new StringBuilder().append("silent_").append(peer).toString(), false);
                            if (i != 0) {
                                req.schedule_date = i;
                                req.flags |= 1024;
                            }
                            if (msgObj3.messageOwner.to_id instanceof TLRPC.TL_peerChannel) {
                                TLRPC.Chat channel = getMessagesController().getChat(Integer.valueOf(msgObj3.messageOwner.to_id.channel_id));
                                req.from_peer = new TLRPC.TL_inputPeerChannel();
                                req.from_peer.channel_id = msgObj3.messageOwner.to_id.channel_id;
                                if (channel == null) {
                                    msgObj2 = msgObj3;
                                } else {
                                    msgObj2 = msgObj3;
                                    req.from_peer.access_hash = channel.access_hash;
                                }
                            } else {
                                msgObj2 = msgObj3;
                                req.from_peer = new TLRPC.TL_inputPeerEmpty();
                            }
                            ArrayList<Long> randomIds5 = randomIds2;
                            req.random_id = randomIds5;
                            req.id = ids4;
                            if (messages.size() == 1 && arrayList2.get(0).messageOwner.with_my_score) {
                                z = true;
                                req.with_my_score = z;
                                randomIds = randomIds5;
                                arr = arr4;
                                objArr = objArr5;
                                final boolean isMegagroupFinal = isMegagroup;
                                inputPeer = inputPeer3;
                                ids = ids4;
                                myId = myId2;
                                lower_id = lower_id3;
                                toMyself = toMyself2;
                                messagesByRandomIds = messagesByRandomIds4;
                                getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda47
                                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                                        SendMessagesHelper.this.m989lambda$sendMessage$9$imuecahxyvjdmessengerSendMessagesHelper(peer, scheduleDate, isMegagroupFinal, toMyself, messagesByRandomIds4, arr4, objArr5, to_id, req, tLObject, tL_error);
                                    }
                                }, 68);
                                if (a6 != messages.size() - 1) {
                                    sendMessagesHelper = this;
                                    arrayList = messages;
                                    j = peer;
                                    a3 = a6;
                                } else {
                                    ArrayList<MessageObject> objArr6 = new ArrayList<>();
                                    ArrayList<TLRPC.Message> arr5 = new ArrayList<>();
                                    ArrayList<Long> randomIds6 = new ArrayList<>();
                                    ArrayList<Integer> ids5 = new ArrayList<>();
                                    arrayList = messages;
                                    objArr4 = objArr6;
                                    arr3 = arr5;
                                    randomIds4 = randomIds6;
                                    ids = ids5;
                                    messagesByRandomIds = new LongSparseArray<>();
                                    a3 = a6;
                                    sendMessagesHelper = this;
                                    j = peer;
                                    i = scheduleDate;
                                    sendMessagesHelper2 = sendMessagesHelper;
                                    a5 = a3 + 1;
                                    j2 = j;
                                    arrayList2 = arrayList;
                                    to_id2 = to_id;
                                    groupsMap3 = groupsMap;
                                    lower_id3 = lower_id;
                                    chat = chat2;
                                    inputPeer3 = inputPeer;
                                    myId3 = myId;
                                    ids3 = ids;
                                    messagesByRandomIds3 = messagesByRandomIds;
                                }
                            }
                            z = false;
                            req.with_my_score = z;
                            randomIds = randomIds5;
                            arr = arr4;
                            objArr = objArr5;
                            final boolean isMegagroupFinal2 = isMegagroup;
                            inputPeer = inputPeer3;
                            ids = ids4;
                            myId = myId2;
                            lower_id = lower_id3;
                            toMyself = toMyself2;
                            messagesByRandomIds = messagesByRandomIds4;
                            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda47
                                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                                    SendMessagesHelper.this.m989lambda$sendMessage$9$imuecahxyvjdmessengerSendMessagesHelper(peer, scheduleDate, isMegagroupFinal2, toMyself, messagesByRandomIds4, arr4, objArr5, to_id, req, tLObject, tL_error);
                                }
                            }, 68);
                            if (a6 != messages.size() - 1) {
                            }
                        }
                        randomIds4 = randomIds;
                        arr3 = arr;
                        objArr4 = objArr;
                        i = scheduleDate;
                        sendMessagesHelper2 = sendMessagesHelper;
                        a5 = a3 + 1;
                        j2 = j;
                        arrayList2 = arrayList;
                        to_id2 = to_id;
                        groupsMap3 = groupsMap;
                        lower_id3 = lower_id;
                        chat = chat2;
                        inputPeer3 = inputPeer;
                        myId3 = myId;
                        ids3 = ids;
                        messagesByRandomIds3 = messagesByRandomIds;
                    }
                }
                MessageObject msgObj4 = msgObj;
                if (msgObj4.type != 0 || TextUtils.isEmpty(msgObj4.messageText)) {
                    arrayList = messages;
                    j = peer;
                    a3 = a2;
                    sendMessagesHelper = this;
                } else {
                    TLRPC.WebPage webPage = msgObj4.messageOwner.media != null ? msgObj4.messageOwner.media.webpage : null;
                    String charSequence = msgObj4.messageText.toString();
                    j = peer;
                    arrayList = messages;
                    a3 = a2;
                    sendMessagesHelper = this;
                    sendMessage(charSequence, peer, null, webPage, webPage != null, msgObj4.messageOwner.entities, null, null, notify, scheduleDate);
                }
                randomIds4 = randomIds;
                arr3 = arr;
                objArr4 = objArr;
                i = scheduleDate;
                sendMessagesHelper2 = sendMessagesHelper;
                a5 = a3 + 1;
                j2 = j;
                arrayList2 = arrayList;
                to_id2 = to_id;
                groupsMap3 = groupsMap;
                lower_id3 = lower_id;
                chat = chat2;
                inputPeer3 = inputPeer;
                myId3 = myId;
                ids3 = ids;
                messagesByRandomIds3 = messagesByRandomIds;
            }
            return sendResult;
        }
        return 0;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendMessage$9$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m989lambda$sendMessage$9$imuecahxyvjdmessengerSendMessagesHelper(final long peer, final int scheduleDate, boolean isMegagroupFinal, boolean toMyself, LongSparseArray messagesByRandomIdsFinal, ArrayList newMsgObjArr, ArrayList newMsgArr, final TLRPC.Peer to_id, final TLRPC.TL_messages_forwardMessages req, TLObject response, final TLRPC.TL_error error) {
        final SendMessagesHelper sendMessagesHelper;
        Integer value;
        TLRPC.Message message;
        int sentCount;
        SparseLongArray newMessagesByIds;
        TLRPC.Updates updates;
        int i;
        int i2 = scheduleDate;
        ArrayList arrayList = newMsgObjArr;
        ArrayList arrayList2 = newMsgArr;
        if (error == null) {
            SparseLongArray newMessagesByIds2 = new SparseLongArray();
            TLRPC.Updates updates2 = (TLRPC.Updates) response;
            int a1 = 0;
            while (a1 < updates2.updates.size()) {
                TLRPC.Update update = updates2.updates.get(a1);
                if (update instanceof TLRPC.TL_updateMessageID) {
                    TLRPC.TL_updateMessageID updateMessageID = (TLRPC.TL_updateMessageID) update;
                    newMessagesByIds2.put(updateMessageID.id, updateMessageID.random_id);
                    updates2.updates.remove(a1);
                    a1--;
                }
                a1++;
            }
            Integer value2 = getMessagesController().dialogs_read_outbox_max.get(Long.valueOf(peer));
            if (i2 != 0) {
                value = 0;
            } else if (value2 == null) {
                Integer value3 = Integer.valueOf(getMessagesStorage().getDialogReadMax(true, peer));
                getMessagesController().dialogs_read_outbox_max.put(Long.valueOf(peer), value3);
                value = value3;
            } else {
                value = value2;
            }
            int a12 = 0;
            int sentCount2 = 0;
            while (a12 < updates2.updates.size()) {
                TLRPC.Update update2 = updates2.updates.get(a12);
                if ((update2 instanceof TLRPC.TL_updateNewMessage) || (update2 instanceof TLRPC.TL_updateNewChannelMessage) || (update2 instanceof TLRPC.TL_updateNewScheduledMessage)) {
                    updates2.updates.remove(a12);
                    int a13 = a12 - 1;
                    if (update2 instanceof TLRPC.TL_updateNewMessage) {
                        TLRPC.TL_updateNewMessage updateNewMessage = (TLRPC.TL_updateNewMessage) update2;
                        TLRPC.Message message2 = updateNewMessage.message;
                        getMessagesController().processNewDifferenceParams(-1, updateNewMessage.pts, -1, updateNewMessage.pts_count);
                        message = message2;
                    } else if (update2 instanceof TLRPC.TL_updateNewScheduledMessage) {
                        message = ((TLRPC.TL_updateNewScheduledMessage) update2).message;
                    } else {
                        TLRPC.TL_updateNewChannelMessage updateNewChannelMessage = (TLRPC.TL_updateNewChannelMessage) update2;
                        TLRPC.Message message3 = updateNewChannelMessage.message;
                        getMessagesController().processNewChannelDifferenceParams(updateNewChannelMessage.pts, updateNewChannelMessage.pts_count, message3.to_id.channel_id);
                        if (isMegagroupFinal) {
                            message3.flags |= Integer.MIN_VALUE;
                        }
                        message = message3;
                    }
                    ImageLoader.saveMessageThumbs(message);
                    if (i2 == 0) {
                        message.unread = value.intValue() < message.id;
                    }
                    if (toMyself) {
                        message.out = true;
                        message.unread = false;
                        message.media_unread = false;
                    }
                    long random_id = newMessagesByIds2.get(message.id);
                    if (random_id != 0) {
                        final TLRPC.Message newMsgObj1 = (TLRPC.Message) messagesByRandomIdsFinal.get(random_id);
                        if (newMsgObj1 == null) {
                            sentCount = sentCount2;
                            newMessagesByIds = newMessagesByIds2;
                            updates = updates2;
                            i = 1;
                        } else {
                            int index = arrayList.indexOf(newMsgObj1);
                            if (index == -1) {
                                sentCount = sentCount2;
                                newMessagesByIds = newMessagesByIds2;
                                updates = updates2;
                                i = 1;
                            } else {
                                MessageObject msgObj1 = (MessageObject) arrayList2.get(index);
                                arrayList.remove(index);
                                arrayList2.remove(index);
                                final int oldId = newMsgObj1.id;
                                final ArrayList<TLRPC.Message> sentMessages = new ArrayList<>();
                                sentMessages.add(message);
                                int sentCount3 = message.id;
                                updateMediaPaths(msgObj1, message, sentCount3, null, true);
                                final int existFlags = msgObj1.getMediaExistanceFlags();
                                newMsgObj1.id = message.id;
                                newMessagesByIds = newMessagesByIds2;
                                updates = updates2;
                                i = 1;
                                final TLRPC.Message message4 = message;
                                getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda18
                                    @Override // java.lang.Runnable
                                    public final void run() {
                                        SendMessagesHelper.this.m986lambda$sendMessage$6$imuecahxyvjdmessengerSendMessagesHelper(newMsgObj1, oldId, to_id, scheduleDate, sentMessages, peer, message4, existFlags);
                                    }
                                });
                                a12 = a13;
                                sentCount2++;
                            }
                        }
                    } else {
                        sentCount = sentCount2;
                        newMessagesByIds = newMessagesByIds2;
                        updates = updates2;
                        i = 1;
                    }
                    sentCount2 = sentCount;
                    a12 = a13;
                } else {
                    newMessagesByIds = newMessagesByIds2;
                    updates = updates2;
                    i = 1;
                }
                a12 += i;
                arrayList = newMsgObjArr;
                arrayList2 = newMsgArr;
                updates2 = updates;
                newMessagesByIds2 = newMessagesByIds;
                i2 = scheduleDate;
            }
            int sentCount4 = sentCount2;
            TLRPC.Updates updates3 = updates2;
            if (!updates3.updates.isEmpty()) {
                getMessagesController().processUpdates(updates3, false);
            }
            getStatsController().incrementSentItemsCount(ApplicationLoader.getCurrentNetworkType(), 1, sentCount4);
            sendMessagesHelper = this;
        } else {
            sendMessagesHelper = this;
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda19
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m987lambda$sendMessage$7$imuecahxyvjdmessengerSendMessagesHelper(error, req);
                }
            });
        }
        for (int a14 = 0; a14 < newMsgObjArr.size(); a14++) {
            final TLRPC.Message newMsgObj12 = (TLRPC.Message) newMsgObjArr.get(a14);
            getMessagesStorage().markMessageAsSendError(newMsgObj12, scheduleDate != 0);
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda20
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m988lambda$sendMessage$8$imuecahxyvjdmessengerSendMessagesHelper(newMsgObj12, scheduleDate);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendMessage$6$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m986lambda$sendMessage$6$imuecahxyvjdmessengerSendMessagesHelper(final TLRPC.Message newMsgObj1, final int oldId, TLRPC.Peer to_id, final int scheduleDate, ArrayList sentMessages, final long peer, final TLRPC.Message message, final int existFlags) {
        getMessagesStorage().updateMessageStateAndId(newMsgObj1.random_id, Integer.valueOf(oldId), newMsgObj1.id, 0, false, to_id.channel_id, scheduleDate != 0 ? 1 : 0);
        getMessagesStorage().putMessages((ArrayList<TLRPC.Message>) sentMessages, true, true, false, 0, scheduleDate != 0);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda52
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m985lambda$sendMessage$5$imuecahxyvjdmessengerSendMessagesHelper(newMsgObj1, peer, oldId, message, existFlags, scheduleDate);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendMessage$5$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m985lambda$sendMessage$5$imuecahxyvjdmessengerSendMessagesHelper(TLRPC.Message newMsgObj1, long peer, int oldId, TLRPC.Message message, int existFlags, int scheduleDate) {
        newMsgObj1.send_state = 0;
        getMediaDataController().increasePeerRaiting(peer);
        NotificationCenter notificationCenter = getNotificationCenter();
        int i = NotificationCenter.messageReceivedByServer;
        Object[] objArr = new Object[7];
        objArr[0] = Integer.valueOf(oldId);
        objArr[1] = Integer.valueOf(message.id);
        objArr[2] = message;
        objArr[3] = Long.valueOf(peer);
        objArr[4] = 0L;
        objArr[5] = Integer.valueOf(existFlags);
        objArr[6] = Boolean.valueOf(scheduleDate != 0);
        notificationCenter.postNotificationName(i, objArr);
        processSentMessage(oldId);
        removeFromSendingMessages(oldId, scheduleDate != 0);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendMessage$7$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m987lambda$sendMessage$7$imuecahxyvjdmessengerSendMessagesHelper(TLRPC.TL_error error, TLRPC.TL_messages_forwardMessages req) {
        AlertsCreator.processError(this.currentAccount, error, null, req, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendMessage$8$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m988lambda$sendMessage$8$imuecahxyvjdmessengerSendMessagesHelper(TLRPC.Message newMsgObj1, int scheduleDate) {
        newMsgObj1.send_state = 2;
        getNotificationCenter().postNotificationName(NotificationCenter.messageSendError, Integer.valueOf(newMsgObj1.id));
        processSentMessage(newMsgObj1.id);
        removeFromSendingMessages(newMsgObj1.id, scheduleDate != 0);
    }

    private void writePreviousMessageData(TLRPC.Message message, SerializedData data) {
        message.media.serializeToStream(data);
        data.writeString(message.message != null ? message.message : "");
        data.writeString(message.attachPath != null ? message.attachPath : "");
        int count = message.entities.size();
        data.writeInt32(count);
        for (int a2 = 0; a2 < count; a2++) {
            message.entities.get(a2).serializeToStream(data);
        }
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Removed duplicated region for block: B:58:0x0153 A[Catch: Exception -> 0x0122, TryCatch #10 {Exception -> 0x0122, blocks: (B:40:0x00dd, B:42:0x00f3, B:44:0x00f9, B:46:0x00ff, B:47:0x0102, B:51:0x0130, B:58:0x0153, B:59:0x015c), top: B:316:0x00db }] */
    /* JADX WARN: Type inference failed for: r18v1 */
    /* JADX WARN: Type inference failed for: r18v11 */
    /* JADX WARN: Type inference failed for: r18v12 */
    /* JADX WARN: Type inference failed for: r18v7 */
    /* JADX WARN: Type inference failed for: r18v9 */
    /* JADX WARN: Type inference failed for: r1v19, types: [im.uecahxyvjd.tgnet.TLRPC$TL_inputMediaPhoto] */
    /* JADX WARN: Type inference failed for: r1v35 */
    /* JADX WARN: Type inference failed for: r1v36 */
    /* JADX WARN: Type inference failed for: r1v38 */
    /* JADX WARN: Type inference failed for: r1v41 */
    /* JADX WARN: Type inference failed for: r1v43 */
    /* JADX WARN: Type inference failed for: r1v45 */
    /* JADX WARN: Type inference failed for: r1v63, types: [im.uecahxyvjd.tgnet.TLRPC$InputMedia, im.uecahxyvjd.tgnet.TLRPC$TL_inputMediaUploadedDocument] */
    /* JADX WARN: Type inference failed for: r1v71 */
    /* JADX WARN: Type inference failed for: r1v72 */
    /* JADX WARN: Type inference failed for: r1v73 */
    /* JADX WARN: Type inference failed for: r1v74 */
    /* JADX WARN: Type inference failed for: r1v75 */
    /* JADX WARN: Type inference failed for: r1v76 */
    /* JADX WARN: Type inference failed for: r1v77 */
    /* JADX WARN: Type inference failed for: r22v10 */
    /* JADX WARN: Type inference failed for: r22v11 */
    /* JADX WARN: Type inference failed for: r22v4 */
    /* JADX WARN: Type inference failed for: r22v6 */
    /* JADX WARN: Type inference failed for: r22v8 */
    /* JADX WARN: Type inference failed for: r2v44, types: [im.uecahxyvjd.tgnet.TLRPC$TL_inputMediaDocument] */
    /* JADX WARN: Type inference failed for: r2v51, types: [im.uecahxyvjd.tgnet.TLRPC$InputMedia, im.uecahxyvjd.tgnet.TLRPC$TL_inputMediaUploadedDocument] */
    /* JADX WARN: Type inference failed for: r30v0, types: [im.uecahxyvjd.messenger.SendMessagesHelper] */
    /* JADX WARN: Type inference failed for: r3v38, types: [im.uecahxyvjd.tgnet.TLRPC$TL_inputMediaDocument] */
    /* JADX WARN: Type inference failed for: r7v3, types: [java.lang.String] */
    /* JADX WARN: Type inference failed for: r8v10 */
    /* JADX WARN: Type inference failed for: r8v63 */
    /* JADX WARN: Type inference failed for: r8v9 */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private void editMessageMedia(MessageObject messageObject, TLRPC.TL_photo photo, VideoEditedInfo videoEditedInfo, TLRPC.TL_document document, String path, HashMap<String, String> params, boolean retry, Object parentObject) {
        VideoEditedInfo videoEditedInfo2;
        HashMap<String, String> params2;
        TLRPC.TL_document document2;
        DelayedMessage delayedMessage;
        int i;
        VideoEditedInfo videoEditedInfo3;
        TLRPC.TL_document document3;
        HashMap<String, String> params3;
        TLRPC.TL_photo photo2;
        String path2;
        int type;
        String path3;
        TLRPC.TL_document document4;
        VideoEditedInfo videoEditedInfo4;
        HashMap<String, String> params4;
        ?? r8;
        TLRPC.TL_photo photo3;
        TLRPC.TL_document document5;
        boolean performMediaUpload;
        int type2;
        TLRPC.TL_inputMediaUploadedPhoto tL_inputMediaUploadedPhoto;
        boolean performMediaUpload2;
        boolean performMediaUpload3;
        DelayedMessage delayedMessage2;
        TLRPC.TL_photo photo4;
        TLRPC.TL_document document6;
        TLRPC.TL_inputMediaUploadedPhoto tL_inputMediaUploadedPhoto2;
        String path4;
        ?? tL_inputMediaUploadedDocument;
        TLRPC.TL_document document7;
        VideoEditedInfo videoEditedInfo5;
        boolean performMediaUpload4;
        TLRPC.TL_inputMediaUploadedPhoto tL_inputMediaUploadedPhoto3;
        boolean performMediaUpload5;
        TLRPC.TL_inputMediaUploadedPhoto tL_inputMediaUploadedPhoto4;
        int type3;
        TLRPC.TL_photo photo5 = photo;
        TLRPC.TL_document document8 = document;
        if (messageObject == null) {
            return;
        }
        TLRPC.Message newMsg = messageObject.messageOwner;
        messageObject.cancelEditing = false;
        try {
            long peer = messageObject.getDialogId();
            ?? r7 = "http";
            try {
                if (retry) {
                    if (messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaPhoto) {
                        photo5 = (TLRPC.TL_photo) messageObject.messageOwner.media.photo;
                        type3 = 2;
                        videoEditedInfo3 = videoEditedInfo;
                    } else {
                        document8 = (TLRPC.TL_document) messageObject.messageOwner.media.document;
                        try {
                            if (!MessageObject.isVideoDocument(document8) && videoEditedInfo == null) {
                                type3 = 7;
                                videoEditedInfo3 = messageObject.videoEditedInfo;
                            }
                            type3 = 3;
                            videoEditedInfo3 = messageObject.videoEditedInfo;
                        } catch (Exception e) {
                            e = e;
                            videoEditedInfo2 = videoEditedInfo;
                            params2 = params;
                            document2 = document8;
                            FileLog.e(e);
                            revertEditingMessageObject(messageObject);
                        }
                    }
                    try {
                        params3 = newMsg.params;
                    } catch (Exception e2) {
                        e = e2;
                        params2 = params;
                        document2 = document8;
                        videoEditedInfo2 = videoEditedInfo3;
                    }
                    try {
                        messageObject.editingMessage = newMsg.message;
                        messageObject.editingMessageEntities = newMsg.entities;
                        photo2 = photo5;
                        delayedMessage = null;
                        document3 = document8;
                        int i2 = type3;
                        path2 = newMsg.attachPath;
                        type = i2;
                    } catch (Exception e3) {
                        e = e3;
                        document2 = document8;
                        params2 = params3;
                        videoEditedInfo2 = videoEditedInfo3;
                        FileLog.e(e);
                        revertEditingMessageObject(messageObject);
                    }
                } else {
                    messageObject.previousMedia = newMsg.media;
                    messageObject.previousCaption = newMsg.message;
                    messageObject.previousCaptionEntities = newMsg.entities;
                    messageObject.previousAttachPath = newMsg.attachPath;
                    SerializedData serializedDataCalc = new SerializedData(true);
                    writePreviousMessageData(newMsg, serializedDataCalc);
                    SerializedData prevMessageData = new SerializedData(serializedDataCalc.length());
                    writePreviousMessageData(newMsg, prevMessageData);
                    HashMap<String, String> params5 = params == null ? new HashMap<>() : params;
                    try {
                        delayedMessage = null;
                        params5.put("prevMedia", Base64.encodeToString(prevMessageData.toByteArray(), 0));
                        prevMessageData.cleanup();
                        try {
                            if (photo5 != null) {
                                newMsg.media = new TLRPC.TL_messageMediaPhoto();
                                newMsg.media.flags |= 3;
                                newMsg.media.photo = photo5;
                                i = 2;
                                if (path == null || path.length() <= 0 || !path.startsWith("http")) {
                                    path3 = 1;
                                    TLRPC.FileLocation location1 = photo5.sizes.get(photo5.sizes.size() - 1).location;
                                    newMsg.attachPath = FileLoader.getPathToAttach(location1, true).toString();
                                } else {
                                    newMsg.attachPath = path;
                                }
                            } else if (document8 != null) {
                                newMsg.media = new TLRPC.TL_messageMediaDocument();
                                newMsg.media.flags |= 3;
                                newMsg.media.document = document8;
                                if (!MessageObject.isVideoDocument(document) && videoEditedInfo == null) {
                                    i = 7;
                                    if (videoEditedInfo != null) {
                                        String ve = videoEditedInfo.getString();
                                        params5.put("ve", ve);
                                    }
                                    newMsg.attachPath = path;
                                }
                                i = 3;
                                if (videoEditedInfo != null) {
                                }
                                newMsg.attachPath = path;
                            } else {
                                i = 65535;
                            }
                            newMsg.params = params5;
                            newMsg.send_state = 3;
                            videoEditedInfo3 = videoEditedInfo;
                            document3 = document8;
                            params3 = params5;
                            photo2 = photo5;
                            int i3 = i;
                            path2 = path;
                            type = i3;
                        } catch (Exception e4) {
                            e = e4;
                            videoEditedInfo2 = videoEditedInfo;
                            document2 = document8;
                            params2 = params5;
                            FileLog.e(e);
                            revertEditingMessageObject(messageObject);
                        }
                    } catch (Exception e5) {
                        e = e5;
                        videoEditedInfo2 = videoEditedInfo;
                        document2 = document8;
                        params2 = params5;
                    }
                }
                try {
                    if (newMsg.attachPath == null) {
                        try {
                            newMsg.attachPath = "";
                        } catch (Exception e6) {
                            e = e6;
                            params2 = params3;
                            document2 = document3;
                            videoEditedInfo2 = videoEditedInfo3;
                            FileLog.e(e);
                            revertEditingMessageObject(messageObject);
                        }
                    }
                    newMsg.local_id = 0;
                    if ((messageObject.type == 3 || videoEditedInfo3 != null || messageObject.type == 2) && !TextUtils.isEmpty(newMsg.attachPath)) {
                        messageObject.attachPathExists = true;
                    }
                    videoEditedInfo3 = videoEditedInfo3;
                    videoEditedInfo3 = videoEditedInfo3;
                    if (messageObject.videoEditedInfo != null && videoEditedInfo3 == null) {
                        videoEditedInfo3 = messageObject.videoEditedInfo;
                    }
                    if (retry) {
                        document4 = document3;
                        videoEditedInfo4 = videoEditedInfo3;
                        r8 = videoEditedInfo3;
                    } else {
                        try {
                            if (messageObject.editingMessage != null) {
                                newMsg.message = messageObject.editingMessage.toString();
                                if (messageObject.editingMessageEntities != null) {
                                    newMsg.entities = messageObject.editingMessageEntities;
                                } else {
                                    CharSequence[] message = {messageObject.editingMessage};
                                    ArrayList<TLRPC.MessageEntity> entities = getMediaDataController().getEntities(message);
                                    if (entities != null && !entities.isEmpty()) {
                                        newMsg.entities = entities;
                                    }
                                }
                                messageObject.caption = null;
                                messageObject.generateCaption();
                            }
                            ArrayList<TLRPC.Message> arr = new ArrayList<>();
                            arr.add(newMsg);
                            getMessagesStorage().putMessages(arr, false, true, false, 0, messageObject.scheduled);
                            messageObject.type = -1;
                            messageObject.setType();
                            messageObject.createMessageSendInfo();
                            ArrayList<MessageObject> arrayList = new ArrayList<>();
                            arrayList.add(messageObject);
                            try {
                                document4 = document3;
                                videoEditedInfo4 = videoEditedInfo3;
                            } catch (Exception e7) {
                                e = e7;
                                params2 = params3;
                                document2 = document3;
                                videoEditedInfo2 = videoEditedInfo3;
                                FileLog.e(e);
                                revertEditingMessageObject(messageObject);
                            }
                            try {
                                params4 = null;
                                r8 = 1;
                                getNotificationCenter().postNotificationName(NotificationCenter.replaceMessagesObjects, Long.valueOf(peer), arrayList);
                            } catch (Exception e8) {
                                e = e8;
                                params2 = params3;
                                document2 = document4;
                                videoEditedInfo2 = videoEditedInfo4;
                                FileLog.e(e);
                                revertEditingMessageObject(messageObject);
                            }
                        } catch (Exception e9) {
                            e = e9;
                            params2 = params3;
                            document2 = document3;
                            videoEditedInfo2 = videoEditedInfo3;
                        }
                    }
                    String originalPath = (params3 == null || !params3.containsKey("originalPath")) ? null : params3.get("originalPath");
                    if ((type < 1 || type > 3) && (type < 5 || type > 8)) {
                        params4 = params3;
                        path3 = path2;
                        photo3 = photo2;
                        document5 = document4;
                        videoEditedInfo2 = videoEditedInfo4;
                    } else {
                        document5 = 0;
                        if (type == 2) {
                            try {
                                TLRPC.TL_inputMediaUploadedPhoto uploadedPhoto = new TLRPC.TL_inputMediaUploadedPhoto();
                                if (params3 != null) {
                                    String masks = params3.get("masks");
                                    if (masks != null) {
                                        performMediaUpload = false;
                                        SerializedData serializedData = new SerializedData(Utilities.hexToBytes(masks));
                                        int count = serializedData.readInt32(false);
                                        int a2 = 0;
                                        while (a2 < count) {
                                            params2 = params3;
                                            try {
                                                ArrayList<TLRPC.InputDocument> arrayList2 = uploadedPhoto.stickers;
                                                int count2 = count;
                                                int count3 = serializedData.readInt32(false);
                                                arrayList2.add(TLRPC.InputDocument.TLdeserialize(serializedData, count3, false));
                                                a2++;
                                                params3 = params2;
                                                masks = masks;
                                                count = count2;
                                            } catch (Exception e10) {
                                                e = e10;
                                                document2 = document4;
                                                videoEditedInfo2 = videoEditedInfo4;
                                                FileLog.e(e);
                                                revertEditingMessageObject(messageObject);
                                            }
                                        }
                                        params2 = params3;
                                        int a3 = uploadedPhoto.flags;
                                        uploadedPhoto.flags = a3 | 1;
                                        serializedData.cleanup();
                                    } else {
                                        performMediaUpload = false;
                                        params2 = params3;
                                    }
                                } else {
                                    performMediaUpload = false;
                                    params2 = params3;
                                }
                                if (photo2.access_hash == 0) {
                                    type2 = type;
                                    tL_inputMediaUploadedPhoto = uploadedPhoto;
                                    performMediaUpload2 = true;
                                } else {
                                    ?? tL_inputMediaPhoto = new TLRPC.TL_inputMediaPhoto();
                                    tL_inputMediaPhoto.id = new TLRPC.TL_inputPhoto();
                                    type2 = type;
                                    tL_inputMediaPhoto.id.id = photo2.id;
                                    tL_inputMediaPhoto.id.access_hash = photo2.access_hash;
                                    tL_inputMediaPhoto.id.file_reference = photo2.file_reference;
                                    if (tL_inputMediaPhoto.id.file_reference == null) {
                                        tL_inputMediaPhoto.id.file_reference = new byte[0];
                                    }
                                    tL_inputMediaUploadedPhoto = tL_inputMediaPhoto;
                                    performMediaUpload2 = performMediaUpload;
                                }
                                DelayedMessage delayedMessage3 = new DelayedMessage(peer);
                                delayedMessage3.type = 0;
                                delayedMessage3.obj = messageObject;
                                delayedMessage3.originalPath = originalPath;
                                delayedMessage3.parentObject = parentObject;
                                delayedMessage3.inputUploadMedia = uploadedPhoto;
                                delayedMessage3.performMediaUpload = performMediaUpload2;
                                if (path2 == null || path2.length() <= 0 || !path2.startsWith("http")) {
                                    delayedMessage3.photoSize = photo2.sizes.get(photo2.sizes.size() - 1);
                                    delayedMessage3.locationParent = photo2;
                                } else {
                                    delayedMessage3.httpLocation = path2;
                                }
                                performMediaUpload3 = performMediaUpload2;
                                delayedMessage2 = delayedMessage3;
                                photo4 = photo2;
                                document6 = document4;
                                videoEditedInfo2 = videoEditedInfo4;
                                tL_inputMediaUploadedPhoto2 = tL_inputMediaUploadedPhoto;
                                path4 = path2;
                            } catch (Exception e11) {
                                e = e11;
                                params2 = params3;
                                document2 = document4;
                                videoEditedInfo2 = videoEditedInfo4;
                            }
                        } else {
                            performMediaUpload3 = false;
                            type2 = type;
                            params2 = params3;
                            try {
                                if (type2 == 3) {
                                    try {
                                        tL_inputMediaUploadedDocument = new TLRPC.TL_inputMediaUploadedDocument();
                                        document7 = document4;
                                        try {
                                            tL_inputMediaUploadedDocument.mime_type = document7.mime_type;
                                            tL_inputMediaUploadedDocument.attributes = document7.attributes;
                                            if (messageObject.isGif()) {
                                                videoEditedInfo5 = videoEditedInfo4;
                                            } else {
                                                if (videoEditedInfo4 != null) {
                                                    videoEditedInfo5 = videoEditedInfo4;
                                                    try {
                                                        if (!videoEditedInfo5.muted) {
                                                        }
                                                    } catch (Exception e12) {
                                                        e = e12;
                                                        videoEditedInfo2 = videoEditedInfo5;
                                                        document2 = document7;
                                                        FileLog.e(e);
                                                        revertEditingMessageObject(messageObject);
                                                    }
                                                } else {
                                                    videoEditedInfo5 = videoEditedInfo4;
                                                }
                                                tL_inputMediaUploadedDocument.nosound_video = true;
                                                if (BuildVars.DEBUG_VERSION) {
                                                    FileLog.d("nosound_video = true");
                                                }
                                            }
                                        } catch (Exception e13) {
                                            e = e13;
                                            videoEditedInfo2 = videoEditedInfo4;
                                            document2 = document7;
                                        }
                                    } catch (Exception e14) {
                                        e = e14;
                                        document2 = document4;
                                        videoEditedInfo2 = videoEditedInfo4;
                                    }
                                    try {
                                        if (document7.access_hash == 0) {
                                            tL_inputMediaUploadedPhoto3 = tL_inputMediaUploadedDocument;
                                            performMediaUpload4 = true;
                                            path4 = path2;
                                        } else {
                                            ?? tL_inputMediaDocument = new TLRPC.TL_inputMediaDocument();
                                            tL_inputMediaDocument.id = new TLRPC.TL_inputDocument();
                                            path4 = path2;
                                            tL_inputMediaDocument.id.id = document7.id;
                                            tL_inputMediaDocument.id.access_hash = document7.access_hash;
                                            tL_inputMediaDocument.id.file_reference = document7.file_reference;
                                            if (tL_inputMediaDocument.id.file_reference == null) {
                                                tL_inputMediaDocument.id.file_reference = new byte[0];
                                            }
                                            performMediaUpload4 = false;
                                            tL_inputMediaUploadedPhoto3 = tL_inputMediaDocument;
                                        }
                                        DelayedMessage delayedMessage4 = new DelayedMessage(peer);
                                        delayedMessage4.type = 1;
                                        delayedMessage4.obj = messageObject;
                                        delayedMessage4.originalPath = originalPath;
                                        delayedMessage4.parentObject = parentObject;
                                        delayedMessage4.inputUploadMedia = tL_inputMediaUploadedDocument;
                                        delayedMessage4.performMediaUpload = performMediaUpload4;
                                        if (!document7.thumbs.isEmpty()) {
                                            delayedMessage4.photoSize = document7.thumbs.get(0);
                                            delayedMessage4.locationParent = document7;
                                        }
                                        delayedMessage4.videoEditedInfo = videoEditedInfo5;
                                        performMediaUpload3 = performMediaUpload4;
                                        videoEditedInfo2 = videoEditedInfo5;
                                        photo4 = photo2;
                                        tL_inputMediaUploadedPhoto2 = tL_inputMediaUploadedPhoto3;
                                        delayedMessage2 = delayedMessage4;
                                        document6 = document7;
                                    } catch (Exception e15) {
                                        e = e15;
                                        videoEditedInfo2 = videoEditedInfo5;
                                        document2 = document7;
                                        FileLog.e(e);
                                        revertEditingMessageObject(messageObject);
                                    }
                                } else {
                                    VideoEditedInfo videoEditedInfo6 = videoEditedInfo4;
                                    path4 = path2;
                                    document6 = document4;
                                    if (type2 == 7) {
                                        try {
                                            ?? tL_inputMediaUploadedDocument2 = new TLRPC.TL_inputMediaUploadedDocument();
                                            tL_inputMediaUploadedDocument2.mime_type = document6.mime_type;
                                            tL_inputMediaUploadedDocument2.attributes = document6.attributes;
                                            if (document6.access_hash == 0) {
                                                tL_inputMediaUploadedPhoto4 = tL_inputMediaUploadedDocument2;
                                                performMediaUpload5 = tL_inputMediaUploadedDocument2 instanceof TLRPC.TL_inputMediaUploadedDocument;
                                                videoEditedInfo2 = videoEditedInfo6;
                                                photo4 = photo2;
                                            } else {
                                                ?? tL_inputMediaDocument2 = new TLRPC.TL_inputMediaDocument();
                                                tL_inputMediaDocument2.id = new TLRPC.TL_inputDocument();
                                                videoEditedInfo2 = videoEditedInfo6;
                                                photo4 = photo2;
                                                try {
                                                    tL_inputMediaDocument2.id.id = document6.id;
                                                    tL_inputMediaDocument2.id.access_hash = document6.access_hash;
                                                    tL_inputMediaDocument2.id.file_reference = document6.file_reference;
                                                    if (tL_inputMediaDocument2.id.file_reference == null) {
                                                        tL_inputMediaDocument2.id.file_reference = new byte[0];
                                                    }
                                                    performMediaUpload5 = false;
                                                    tL_inputMediaUploadedPhoto4 = tL_inputMediaDocument2;
                                                } catch (Exception e16) {
                                                    e = e16;
                                                    document2 = document6;
                                                    FileLog.e(e);
                                                    revertEditingMessageObject(messageObject);
                                                }
                                            }
                                            if (0 == 0) {
                                                DelayedMessage delayedMessage5 = new DelayedMessage(peer);
                                                delayedMessage5.originalPath = originalPath;
                                                delayedMessage5.type = 2;
                                                delayedMessage5.obj = messageObject;
                                                if (!document6.thumbs.isEmpty()) {
                                                    delayedMessage5.photoSize = document6.thumbs.get(0);
                                                    delayedMessage5.locationParent = document6;
                                                }
                                                delayedMessage5.parentObject = parentObject;
                                                delayedMessage5.inputUploadMedia = tL_inputMediaUploadedDocument2;
                                                delayedMessage5.performMediaUpload = performMediaUpload5;
                                                tL_inputMediaUploadedPhoto2 = tL_inputMediaUploadedPhoto4;
                                                performMediaUpload3 = performMediaUpload5;
                                                delayedMessage2 = delayedMessage5;
                                                document6 = document6;
                                            } else {
                                                tL_inputMediaUploadedPhoto2 = tL_inputMediaUploadedPhoto4;
                                                performMediaUpload3 = performMediaUpload5;
                                                delayedMessage2 = delayedMessage;
                                                document6 = document6;
                                            }
                                        } catch (Exception e17) {
                                            e = e17;
                                            videoEditedInfo2 = videoEditedInfo6;
                                            document2 = document6;
                                            FileLog.e(e);
                                            revertEditingMessageObject(messageObject);
                                        }
                                    } else {
                                        videoEditedInfo2 = videoEditedInfo6;
                                        photo4 = photo2;
                                        delayedMessage2 = delayedMessage;
                                        tL_inputMediaUploadedPhoto2 = null;
                                        document6 = document6;
                                    }
                                }
                            } catch (Exception e18) {
                                e = e18;
                                videoEditedInfo2 = r8;
                                document2 = r7;
                            }
                        }
                        try {
                            TLRPC.TL_messages_editMessage request = new TLRPC.TL_messages_editMessage();
                            request.id = messageObject.getId();
                            request.peer = getMessagesController().getInputPeer((int) peer);
                            request.flags |= 16384;
                            request.media = tL_inputMediaUploadedPhoto2;
                            if (messageObject.scheduled) {
                                request.schedule_date = messageObject.messageOwner.date;
                                request.flags |= 32768;
                            }
                            if (messageObject.editingMessage != null) {
                                request.message = messageObject.editingMessage.toString();
                                request.flags |= 2048;
                                if (messageObject.editingMessageEntities != null) {
                                    request.entities = messageObject.editingMessageEntities;
                                    request.flags |= 8;
                                } else {
                                    CharSequence[] message2 = {messageObject.editingMessage};
                                    ArrayList<TLRPC.MessageEntity> entities2 = getMediaDataController().getEntities(message2);
                                    if (entities2 != null && !entities2.isEmpty()) {
                                        request.entities = entities2;
                                        request.flags |= 8;
                                    }
                                }
                                messageObject.editingMessage = null;
                                messageObject.editingMessageEntities = null;
                            }
                            if (delayedMessage2 != null) {
                                delayedMessage2.sendRequest = request;
                            }
                            photo3 = 2;
                            try {
                                if (type2 == 1) {
                                    try {
                                        params4 = params2;
                                        path3 = path4;
                                        document5 = document6;
                                        performSendMessageRequest(request, messageObject, null, delayedMessage2, parentObject, messageObject.scheduled);
                                        photo3 = photo4;
                                    } catch (Exception e19) {
                                        e = e19;
                                        document2 = document6;
                                        FileLog.e(e);
                                        revertEditingMessageObject(messageObject);
                                    }
                                } else {
                                    path3 = path4;
                                    int type4 = type2;
                                    document5 = document6;
                                    params4 = params2;
                                    try {
                                        if (type4 != 2) {
                                            DelayedMessage delayedMessage6 = delayedMessage2;
                                            photo3 = photo4;
                                            if (type4 == 3) {
                                                if (performMediaUpload3) {
                                                    performSendDelayedMessage(delayedMessage6);
                                                    photo3 = photo3;
                                                } else {
                                                    performSendMessageRequest(request, messageObject, originalPath, delayedMessage6, parentObject, messageObject.scheduled);
                                                    photo3 = photo3;
                                                }
                                            } else if (type4 == 6) {
                                                performSendMessageRequest(request, messageObject, originalPath, delayedMessage6, parentObject, messageObject.scheduled);
                                                photo3 = photo3;
                                            } else if (type4 == 7) {
                                                if (performMediaUpload3) {
                                                    performSendDelayedMessage(delayedMessage6);
                                                    photo3 = photo3;
                                                } else {
                                                    performSendMessageRequest(request, messageObject, originalPath, delayedMessage6, parentObject, messageObject.scheduled);
                                                    photo3 = photo3;
                                                }
                                            } else if (type4 == 8) {
                                                if (performMediaUpload3) {
                                                    performSendDelayedMessage(delayedMessage6);
                                                    photo3 = photo3;
                                                } else {
                                                    performSendMessageRequest(request, messageObject, originalPath, delayedMessage6, parentObject, messageObject.scheduled);
                                                    photo3 = photo3;
                                                }
                                            }
                                        } else if (performMediaUpload3) {
                                            performSendDelayedMessage(delayedMessage2);
                                            photo3 = photo4;
                                            path3 = path3;
                                            document5 = document5;
                                        } else {
                                            try {
                                                photo3 = photo4;
                                                performSendMessageRequest(request, messageObject, originalPath, null, true, delayedMessage2, parentObject, messageObject.scheduled);
                                            } catch (Exception e20) {
                                                e = e20;
                                                params2 = params4;
                                                document2 = document5;
                                                FileLog.e(e);
                                                revertEditingMessageObject(messageObject);
                                            }
                                        }
                                    } catch (Exception e21) {
                                        e = e21;
                                        params2 = params4;
                                        document2 = document5;
                                    }
                                }
                            } catch (Exception e22) {
                                e = e22;
                                params2 = params4;
                                document2 = document5;
                            }
                        } catch (Exception e23) {
                            e = e23;
                            document2 = document6;
                        }
                    }
                } catch (Exception e24) {
                    e = e24;
                    videoEditedInfo2 = videoEditedInfo3;
                    params2 = params3;
                    document2 = document3;
                }
            } catch (Exception e25) {
                e = e25;
                videoEditedInfo2 = videoEditedInfo;
                params2 = params;
                document2 = document8;
            }
        } catch (Exception e26) {
            e = e26;
            videoEditedInfo2 = videoEditedInfo;
            params2 = params;
            document2 = document8;
        }
    }

    public int editMessage(MessageObject messageObject, String message, boolean searchLinks, final BaseFragment fragment, ArrayList<TLRPC.MessageEntity> entities, int scheduleDate, final Runnable callback) {
        if (fragment == null || fragment.getParentActivity() == null) {
            return 0;
        }
        final TLRPC.TL_messages_editMessage req = new TLRPC.TL_messages_editMessage();
        req.peer = getMessagesController().getInputPeer((int) messageObject.getDialogId());
        if (message != null) {
            req.message = message;
            req.flags |= 2048;
            req.no_webpage = !searchLinks;
        }
        req.id = messageObject.getId();
        if (entities != null) {
            req.entities = entities;
            req.flags |= 8;
        }
        if (scheduleDate != 0) {
            req.schedule_date = scheduleDate;
            req.flags |= 32768;
        }
        return getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda37
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m956lambda$editMessage$11$imuecahxyvjdmessengerSendMessagesHelper(fragment, req, callback, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$editMessage$11$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m956lambda$editMessage$11$imuecahxyvjdmessengerSendMessagesHelper(final BaseFragment fragment, final TLRPC.TL_messages_editMessage req, Runnable callback, TLObject response, final TLRPC.TL_error error) {
        if (error == null) {
            getMessagesController().processUpdates((TLRPC.Updates) response, false);
        } else {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda40
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m955lambda$editMessage$10$imuecahxyvjdmessengerSendMessagesHelper(error, fragment, req);
                }
            });
        }
        if (callback != null) {
            AndroidUtilities.runOnUIThread(callback);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$editMessage$10$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m955lambda$editMessage$10$imuecahxyvjdmessengerSendMessagesHelper(TLRPC.TL_error error, BaseFragment fragment, TLRPC.TL_messages_editMessage req) {
        AlertsCreator.processError(this.currentAccount, error, fragment, req, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void sendLocation(Location location) {
        TLRPC.TL_messageMediaGeo mediaGeo = new TLRPC.TL_messageMediaGeo();
        mediaGeo.geo = new TLRPC.TL_geoPoint();
        mediaGeo.geo.lat = AndroidUtilities.fixLocationCoord(location.getLatitude());
        mediaGeo.geo._long = AndroidUtilities.fixLocationCoord(location.getLongitude());
        for (Map.Entry<String, MessageObject> entry : this.waitingForLocation.entrySet()) {
            MessageObject messageObject = entry.getValue();
            sendMessage((TLRPC.MessageMedia) mediaGeo, messageObject.getDialogId(), messageObject, (TLRPC.ReplyMarkup) null, (HashMap<String, String>) null, true, 0);
        }
    }

    public void sendCurrentLocation(MessageObject messageObject, TLRPC.KeyboardButton button) {
        if (messageObject == null || button == null) {
            return;
        }
        String key = messageObject.getDialogId() + "_" + messageObject.getId() + "_" + Utilities.bytesToHex(button.data) + "_" + (button instanceof TLRPC.TL_keyboardButtonGame ? "1" : "0");
        this.waitingForLocation.put(key, messageObject);
        this.locationProvider.start();
    }

    public boolean isSendingCurrentLocation(MessageObject messageObject, TLRPC.KeyboardButton button) {
        if (messageObject == null || button == null) {
            return false;
        }
        String key = messageObject.getDialogId() + "_" + messageObject.getId() + "_" + Utilities.bytesToHex(button.data) + "_" + (button instanceof TLRPC.TL_keyboardButtonGame ? "1" : "0");
        return this.waitingForLocation.containsKey(key);
    }

    public void sendNotificationCallback(final long dialogId, final int msgId, final byte[] data) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda8
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m992x57f2ea5d(dialogId, msgId, data);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendNotificationCallback$14$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m992x57f2ea5d(long dialogId, int msgId, byte[] data) {
        TLRPC.Chat chat;
        TLRPC.User user;
        int lowerId = (int) dialogId;
        final String key = dialogId + "_" + msgId + "_" + Utilities.bytesToHex(data) + "_0";
        this.waitingForCallback.put(key, true);
        if (lowerId > 0) {
            if (getMessagesController().getUser(Integer.valueOf(lowerId)) == null && (user = getMessagesStorage().getUserSync(lowerId)) != null) {
                getMessagesController().putUser(user, true);
            }
        } else if (getMessagesController().getChat(Integer.valueOf(-lowerId)) == null && (chat = getMessagesStorage().getChatSync(-lowerId)) != null) {
            getMessagesController().putChat(chat, true);
        }
        TLRPC.TL_messages_getBotCallbackAnswer req = new TLRPC.TL_messages_getBotCallbackAnswer();
        req.peer = getMessagesController().getInputPeer(lowerId);
        req.msg_id = msgId;
        req.game = false;
        if (data != null) {
            req.flags |= 1;
            req.data = data;
        }
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda11
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m991x950680fe(key, tLObject, tL_error);
            }
        }, 2);
        getMessagesController().markDialogAsRead(dialogId, msgId, msgId, 0, false, 0, true, 0);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendNotificationCallback$12$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m990xd21a179f(String key) {
        this.waitingForCallback.remove(key);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendNotificationCallback$13$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m991x950680fe(final String key, TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda17
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m990xd21a179f(key);
            }
        });
    }

    public byte[] isSendingVote(MessageObject messageObject) {
        if (messageObject == null) {
            return null;
        }
        String key = "poll_" + messageObject.getPollId();
        return this.waitingForVote.get(key);
    }

    public int sendVote(final MessageObject messageObject, TLRPC.TL_pollAnswer answer, final Runnable finishRunnable) {
        if (messageObject == null) {
            return 0;
        }
        final String key = "poll_" + messageObject.getPollId();
        if (this.waitingForCallback.containsKey(key)) {
            return 0;
        }
        this.waitingForVote.put(key, answer != null ? answer.option : new byte[0]);
        TLRPC.TL_messages_sendVote req = new TLRPC.TL_messages_sendVote();
        req.msg_id = messageObject.getId();
        req.peer = getMessagesController().getInputPeer((int) messageObject.getDialogId());
        if (answer != null) {
            req.options.add(answer.option);
        }
        return getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda22
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m994lambda$sendVote$15$imuecahxyvjdmessengerSendMessagesHelper(messageObject, key, finishRunnable, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendVote$15$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m994lambda$sendVote$15$imuecahxyvjdmessengerSendMessagesHelper(MessageObject messageObject, final String key, final Runnable finishRunnable, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            this.voteSendTime.put(messageObject.getPollId(), 0L);
            getMessagesController().processUpdates((TLRPC.Updates) response, false);
            this.voteSendTime.put(messageObject.getPollId(), Long.valueOf(SystemClock.uptimeMillis()));
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper.2
            @Override // java.lang.Runnable
            public void run() {
                SendMessagesHelper.this.waitingForVote.remove(key);
                Runnable runnable = finishRunnable;
                if (runnable != null) {
                    runnable.run();
                }
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public long getVoteSendTime(long pollId) {
        return this.voteSendTime.get(pollId, 0L).longValue();
    }

    public void sendReaction(MessageObject messageObject, CharSequence reaction, ChatActivity parentFragment) {
        if (messageObject == null || parentFragment == null) {
            return;
        }
        TLRPC.TL_messages_sendReaction req = new TLRPC.TL_messages_sendReaction();
        req.peer = getMessagesController().getInputPeer((int) messageObject.getDialogId());
        req.msg_id = messageObject.getId();
        if (reaction != null) {
            req.reaction = reaction.toString();
            req.flags |= 1;
        }
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda5
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m993x5904a0b4(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendReaction$16$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m993x5904a0b4(TLObject response, TLRPC.TL_error error) {
        if (response != null) {
            getMessagesController().processUpdates((TLRPC.Updates) response, false);
        }
    }

    public void sendCallback(boolean cache, final MessageObject messageObject, final TLRPC.KeyboardButton button, final ChatActivity parentFragment) {
        boolean cacheFinal;
        int type;
        if (messageObject == null || button == null) {
            return;
        }
        if (parentFragment == null) {
            return;
        }
        if (button instanceof TLRPC.TL_keyboardButtonUrlAuth) {
            cacheFinal = false;
            type = 3;
        } else {
            boolean cacheFinal2 = button instanceof TLRPC.TL_keyboardButtonGame;
            if (cacheFinal2) {
                cacheFinal = false;
                type = 1;
            } else if (button instanceof TLRPC.TL_keyboardButtonBuy) {
                cacheFinal = cache;
                type = 2;
            } else {
                cacheFinal = cache;
                type = 0;
            }
        }
        final String key = messageObject.getDialogId() + "_" + messageObject.getId() + "_" + Utilities.bytesToHex(button.data) + "_" + type;
        this.waitingForCallback.put(key, true);
        final TLObject[] request = new TLObject[1];
        final boolean z = cacheFinal;
        RequestDelegate requestDelegate = new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda27
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m982x6a60688e(key, z, messageObject, button, parentFragment, request, tLObject, tL_error);
            }
        };
        if (cacheFinal) {
            getMessagesStorage().getBotCache(key, requestDelegate);
        } else if (button instanceof TLRPC.TL_keyboardButtonUrlAuth) {
            TLRPC.TL_messages_requestUrlAuth req = new TLRPC.TL_messages_requestUrlAuth();
            req.peer = getMessagesController().getInputPeer((int) messageObject.getDialogId());
            req.msg_id = messageObject.getId();
            req.button_id = button.button_id;
            request[0] = req;
            getConnectionsManager().sendRequest(req, requestDelegate, 2);
        } else if (button instanceof TLRPC.TL_keyboardButtonBuy) {
            if ((messageObject.messageOwner.media.flags & 4) == 0) {
                TLRPC.TL_payments_getPaymentForm req2 = new TLRPC.TL_payments_getPaymentForm();
                req2.msg_id = messageObject.getId();
                getConnectionsManager().sendRequest(req2, requestDelegate, 2);
                return;
            }
            TLRPC.TL_payments_getPaymentReceipt req3 = new TLRPC.TL_payments_getPaymentReceipt();
            req3.msg_id = messageObject.messageOwner.media.receipt_msg_id;
            getConnectionsManager().sendRequest(req3, requestDelegate, 2);
        } else {
            TLRPC.TL_messages_getBotCallbackAnswer req4 = new TLRPC.TL_messages_getBotCallbackAnswer();
            req4.peer = getMessagesController().getInputPeer((int) messageObject.getDialogId());
            req4.msg_id = messageObject.getId();
            req4.game = button instanceof TLRPC.TL_keyboardButtonGame;
            if (button.data != null) {
                req4.flags |= 1;
                req4.data = button.data;
            }
            getConnectionsManager().sendRequest(req4, requestDelegate, 2);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendCallback$18$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m982x6a60688e(final String key, final boolean cacheFinal, final MessageObject messageObject, final TLRPC.KeyboardButton button, final ChatActivity parentFragment, final TLObject[] request, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda46
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m981xa773ff2f(key, cacheFinal, response, messageObject, button, parentFragment, request);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendCallback$17$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m981xa773ff2f(String key, boolean cacheFinal, TLObject response, MessageObject messageObject, TLRPC.KeyboardButton button, ChatActivity parentFragment, TLObject[] request) {
        int uid;
        this.waitingForCallback.remove(key);
        if (cacheFinal && response == null) {
            sendCallback(false, messageObject, button, parentFragment);
        } else if (response != null) {
            if (button instanceof TLRPC.TL_keyboardButtonUrlAuth) {
                if (response instanceof TLRPC.TL_urlAuthResultRequest) {
                    parentFragment.showRequestUrlAlert((TLRPC.TL_urlAuthResultRequest) response, (TLRPC.TL_messages_requestUrlAuth) request[0], button.url);
                } else if (response instanceof TLRPC.TL_urlAuthResultAccepted) {
                    parentFragment.showOpenUrlAlert(((TLRPC.TL_urlAuthResultAccepted) response).url, false);
                } else if (response instanceof TLRPC.TL_urlAuthResultDefault) {
                    TLRPC.TL_urlAuthResultDefault tL_urlAuthResultDefault = (TLRPC.TL_urlAuthResultDefault) response;
                    parentFragment.showOpenUrlAlert(button.url, true);
                }
            } else if (button instanceof TLRPC.TL_keyboardButtonBuy) {
                if (response instanceof TLRPC.TL_payments_paymentForm) {
                    TLRPC.TL_payments_paymentForm form = (TLRPC.TL_payments_paymentForm) response;
                    getMessagesController().putUsers(form.users, false);
                    return;
                }
                boolean z = response instanceof TLRPC.TL_payments_paymentReceipt;
            } else {
                TLRPC.TL_messages_botCallbackAnswer res = (TLRPC.TL_messages_botCallbackAnswer) response;
                if (!cacheFinal && res.cache_time != 0) {
                    getMessagesStorage().saveBotCache(key, res);
                }
                if (res.message != null) {
                    int uid2 = messageObject.messageOwner.from_id;
                    if (messageObject.messageOwner.via_bot_id != 0) {
                        uid2 = messageObject.messageOwner.via_bot_id;
                    }
                    String name = null;
                    if (uid2 > 0) {
                        TLRPC.User user = getMessagesController().getUser(Integer.valueOf(uid2));
                        if (user != null) {
                            name = ContactsController.formatName(user.first_name, user.last_name);
                        }
                    } else {
                        TLRPC.Chat chat = getMessagesController().getChat(Integer.valueOf(-uid2));
                        if (chat != null) {
                            name = chat.title;
                        }
                    }
                    if (name == null) {
                        name = "bot";
                    }
                    if (res.alert) {
                        if (parentFragment.getParentActivity() == null) {
                            return;
                        }
                        AlertDialog.Builder builder = new AlertDialog.Builder(parentFragment.getParentActivity());
                        builder.setTitle(name);
                        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
                        builder.setMessage(res.message);
                        parentFragment.showDialog(builder.create());
                        return;
                    }
                    parentFragment.showAlert(name, res.message);
                } else if (res.url == null || parentFragment.getParentActivity() == null) {
                } else {
                    int uid3 = messageObject.messageOwner.from_id;
                    if (messageObject.messageOwner.via_bot_id == 0) {
                        uid = uid3;
                    } else {
                        int uid4 = messageObject.messageOwner.via_bot_id;
                        uid = uid4;
                    }
                    TLRPC.User user2 = getMessagesController().getUser(Integer.valueOf(uid));
                    boolean verified = user2 != null && user2.verified;
                    if (!(button instanceof TLRPC.TL_keyboardButtonGame)) {
                        parentFragment.showOpenUrlAlert(res.url, false);
                        return;
                    }
                    TLRPC.TL_game game = messageObject.messageOwner.media instanceof TLRPC.TL_messageMediaGame ? messageObject.messageOwner.media.game : null;
                    if (game != null) {
                        parentFragment.showOpenGameAlert(game, messageObject, res.url, !verified && MessagesController.getNotificationsSettings(this.currentAccount).getBoolean(new StringBuilder().append("askgame_").append(uid).toString(), true), uid);
                    }
                }
            }
        }
    }

    public boolean isSendingCallback(MessageObject messageObject, TLRPC.KeyboardButton button) {
        int type;
        if (messageObject == null || button == null) {
            return false;
        }
        if (button instanceof TLRPC.TL_keyboardButtonUrlAuth) {
            type = 3;
        } else if (button instanceof TLRPC.TL_keyboardButtonGame) {
            type = 1;
        } else if (button instanceof TLRPC.TL_keyboardButtonBuy) {
            type = 2;
        } else {
            type = 0;
        }
        String key = messageObject.getDialogId() + "_" + messageObject.getId() + "_" + Utilities.bytesToHex(button.data) + "_" + type;
        return this.waitingForCallback.containsKey(key);
    }

    public void sendEditMessageMedia(TLRPC.InputPeer peer, int id, TLRPC.InputMedia media) {
        if (peer == null) {
            return;
        }
        TLRPCContacts.TL_EditMessageMedia request = new TLRPCContacts.TL_EditMessageMedia();
        request.peer = peer;
        request.id = id;
        request.media = media;
        getConnectionsManager().sendRequest(request, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda59
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m983x8ef1b86f(tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendEditMessageMedia$19$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m983x8ef1b86f(TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            getMessagesController().processUpdates((TLRPC.Updates) response, false);
        }
    }

    public void sendGame(TLRPC.InputPeer peer, TLRPC.TL_inputMediaGame game, long random_id, long taskId) {
        final long newTaskId;
        if (peer == null || game == null) {
            return;
        }
        TLRPC.TL_messages_sendMedia request = new TLRPC.TL_messages_sendMedia();
        request.peer = peer;
        if (request.peer instanceof TLRPC.TL_inputPeerChannel) {
            request.silent = MessagesController.getNotificationsSettings(this.currentAccount).getBoolean("silent_" + (-peer.channel_id), false);
        } else if (request.peer instanceof TLRPC.TL_inputPeerChat) {
            request.silent = MessagesController.getNotificationsSettings(this.currentAccount).getBoolean("silent_" + (-peer.chat_id), false);
        } else {
            request.silent = MessagesController.getNotificationsSettings(this.currentAccount).getBoolean("silent_" + peer.user_id, false);
        }
        request.random_id = random_id != 0 ? random_id : getNextRandomId();
        request.message = "";
        request.media = game;
        if (taskId == 0) {
            NativeByteBuffer data = null;
            try {
                data = new NativeByteBuffer(peer.getObjectSize() + game.getObjectSize() + 4 + 8);
                data.writeInt32(3);
                data.writeInt64(random_id);
                peer.serializeToStream(data);
                game.serializeToStream(data);
            } catch (Exception e) {
                FileLog.e(e);
            }
            newTaskId = getMessagesStorage().createPendingTask(data);
        } else {
            newTaskId = taskId;
        }
        getConnectionsManager().sendRequest(request, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda39
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m984lambda$sendGame$20$imuecahxyvjdmessengerSendMessagesHelper(newTaskId, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$sendGame$20$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m984lambda$sendGame$20$imuecahxyvjdmessengerSendMessagesHelper(long newTaskId, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            getMessagesController().processUpdates((TLRPC.Updates) response, false);
        }
        if (newTaskId != 0) {
            getMessagesStorage().removePendingTask(newTaskId);
        }
    }

    public void sendMessage(MessageObject retryMessageObject) {
        sendMessage(null, null, null, null, null, null, null, null, null, retryMessageObject.getDialogId(), retryMessageObject.messageOwner.attachPath, null, null, true, retryMessageObject, null, retryMessageObject.messageOwner.reply_markup, retryMessageObject.messageOwner.params, !retryMessageObject.messageOwner.silent, retryMessageObject.scheduled ? retryMessageObject.messageOwner.date : 0, 0, null);
    }

    public void sendMessage(TLRPC.User user, long peer, MessageObject reply_to_msg, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate) {
        sendMessage(null, null, null, null, null, user, null, null, null, peer, null, reply_to_msg, null, true, null, null, replyMarkup, params, notify, scheduleDate, 0, null);
    }

    public void sendMessage(TLRPC.TL_document document, VideoEditedInfo videoEditedInfo, String path, long peer, MessageObject reply_to_msg, String caption, ArrayList<TLRPC.MessageEntity> entities, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate, int ttl, Object parentObject) {
        sendMessage(null, caption, null, null, videoEditedInfo, null, document, null, null, peer, path, reply_to_msg, null, true, null, entities, replyMarkup, params, notify, scheduleDate, ttl, parentObject);
    }

    public void sendMessage(String message, long peer, MessageObject reply_to_msg, TLRPC.WebPage webPage, boolean searchLinks, ArrayList<TLRPC.MessageEntity> entities, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate) {
        sendMessage(message, null, null, null, null, null, null, null, null, peer, null, reply_to_msg, webPage, searchLinks, null, entities, replyMarkup, params, notify, scheduleDate, 0, null);
    }

    public void sendMessage(TLRPC.MessageMedia location, long peer, MessageObject reply_to_msg, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate) {
        sendMessage(null, null, location, null, null, null, null, null, null, peer, null, reply_to_msg, null, true, null, null, replyMarkup, params, notify, scheduleDate, 0, null);
    }

    public void sendMessage(TLRPC.TL_messageMediaPoll poll, long peer, MessageObject reply_to_msg, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate) {
        sendMessage(null, null, null, null, null, null, null, null, poll, peer, null, reply_to_msg, null, true, null, null, replyMarkup, params, notify, scheduleDate, 0, null);
    }

    public void sendMessage(TLRPC.TL_game game, long peer, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate) {
        sendMessage(null, null, null, null, null, null, null, game, null, peer, null, null, null, true, null, null, replyMarkup, params, notify, scheduleDate, 0, null);
    }

    public void sendMessage(TLRPC.TL_photo photo, String path, long peer, MessageObject reply_to_msg, String caption, ArrayList<TLRPC.MessageEntity> entities, TLRPC.ReplyMarkup replyMarkup, HashMap<String, String> params, boolean notify, int scheduleDate, int ttl, Object parentObject) {
        sendMessage(null, caption, null, photo, null, null, null, null, null, peer, path, reply_to_msg, null, true, null, entities, replyMarkup, params, notify, scheduleDate, ttl, parentObject);
    }

    public void sendRedpaketTransfer(TLRPC.User user, long peer, String message, String caption) {
        TLRPC.EncryptedChat encryptedChat;
        if ((user != null && user.phone == null) || peer == 0) {
            return;
        }
        int lower_id = (int) peer;
        int high_id = (int) (peer >> 32);
        TLRPC.InputPeer sendToPeer = lower_id != 0 ? getMessagesController().getInputPeer(lower_id) : null;
        if (lower_id == 0) {
            TLRPC.EncryptedChat encryptedChat2 = getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
            if (encryptedChat2 != null) {
                encryptedChat = encryptedChat2;
            } else {
                return;
            }
        } else if (!(sendToPeer instanceof TLRPC.TL_inputPeerChannel)) {
            encryptedChat = null;
        } else {
            TLRPC.Chat chat = getMessagesController().getChat(Integer.valueOf(sendToPeer.channel_id));
            boolean z = (chat == null || chat.megagroup) ? false : true;
            encryptedChat = null;
        }
        if (message != null) {
            try {
                if (encryptedChat != null) {
                    new TLRPC.TL_message_secret();
                } else {
                    new TLRPC.TL_message();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:965:0x1aa4
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:81)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:47)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:39)
        */
    private void sendMessage(java.lang.String r56, java.lang.String r57, im.uecahxyvjd.tgnet.TLRPC.MessageMedia r58, im.uecahxyvjd.tgnet.TLRPC.TL_photo r59, im.uecahxyvjd.messenger.VideoEditedInfo r60, im.uecahxyvjd.tgnet.TLRPC.User r61, im.uecahxyvjd.tgnet.TLRPC.TL_document r62, im.uecahxyvjd.tgnet.TLRPC.TL_game r63, im.uecahxyvjd.tgnet.TLRPC.TL_messageMediaPoll r64, long r65, java.lang.String r67, im.uecahxyvjd.messenger.MessageObject r68, im.uecahxyvjd.tgnet.TLRPC.WebPage r69, boolean r70, im.uecahxyvjd.messenger.MessageObject r71, java.util.ArrayList<im.uecahxyvjd.tgnet.TLRPC.MessageEntity> r72, im.uecahxyvjd.tgnet.TLRPC.ReplyMarkup r73, java.util.HashMap<java.lang.String, java.lang.String> r74, boolean r75, int r76, int r77, java.lang.Object r78) {
        /*
            Method dump skipped, instructions count: 13218
            To view this dump add '--comments-level debug' option
        */
        throw new UnsupportedOperationException("Method not decompiled: im.uecahxyvjd.messenger.SendMessagesHelper.sendMessage(java.lang.String, java.lang.String, im.uecahxyvjd.tgnet.TLRPC$MessageMedia, im.uecahxyvjd.tgnet.TLRPC$TL_photo, im.uecahxyvjd.messenger.VideoEditedInfo, im.uecahxyvjd.tgnet.TLRPC$User, im.uecahxyvjd.tgnet.TLRPC$TL_document, im.uecahxyvjd.tgnet.TLRPC$TL_game, im.uecahxyvjd.tgnet.TLRPC$TL_messageMediaPoll, long, java.lang.String, im.uecahxyvjd.messenger.MessageObject, im.uecahxyvjd.tgnet.TLRPC$WebPage, boolean, im.uecahxyvjd.messenger.MessageObject, java.util.ArrayList, im.uecahxyvjd.tgnet.TLRPC$ReplyMarkup, java.util.HashMap, boolean, int, int, java.lang.Object):void");
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void performSendDelayedMessage(DelayedMessage message) {
        performSendDelayedMessage(message, -1);
    }

    private TLRPC.PhotoSize getThumbForSecretChat(ArrayList<TLRPC.PhotoSize> arrayList) {
        if (arrayList == null || arrayList.isEmpty()) {
            return null;
        }
        int N = arrayList.size();
        for (int a2 = 0; a2 < N; a2++) {
            TLRPC.PhotoSize size = arrayList.get(a2);
            if (size != null && !(size instanceof TLRPC.TL_photoStrippedSize) && !(size instanceof TLRPC.TL_photoSizeEmpty) && size.location != null) {
                TLRPC.TL_photoSize photoSize = new TLRPC.TL_photoSize();
                photoSize.type = size.type;
                photoSize.w = size.w;
                photoSize.h = size.h;
                photoSize.size = size.size;
                photoSize.bytes = size.bytes;
                if (photoSize.bytes == null) {
                    photoSize.bytes = new byte[0];
                }
                photoSize.location = new TLRPC.TL_fileLocation_layer82();
                photoSize.location.dc_id = size.location.dc_id;
                photoSize.location.volume_id = size.location.volume_id;
                photoSize.location.local_id = size.location.local_id;
                photoSize.location.secret = size.location.secret;
                return photoSize;
            }
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void performSendDelayedMessage(DelayedMessage message, int index) {
        int index2;
        TLObject inputMedia;
        MessageObject messageObject;
        TLRPC.InputMedia media;
        TLRPC.InputMedia media2;
        TLRPC.InputMedia media3;
        if (message.type == 0) {
            if (message.httpLocation != null) {
                putToDelayedMessages(message.httpLocation, message);
                ImageLoader.getInstance().loadHttpFile(message.httpLocation, "file", this.currentAccount);
            } else if (message.sendRequest != null) {
                String location = FileLoader.getPathToAttach(message.photoSize).toString();
                putToDelayedMessages(location, message);
                getFileLoader().uploadFile(location, false, true, 16777216);
                putToUploadingMessages(message.obj);
            } else {
                String location2 = FileLoader.getPathToAttach(message.photoSize).toString();
                if (message.sendEncryptedRequest != null && message.photoSize.location.dc_id != 0) {
                    File file = new File(location2);
                    if (!file.exists()) {
                        location2 = FileLoader.getPathToAttach(message.photoSize, true).toString();
                        file = new File(location2);
                    }
                    if (!file.exists()) {
                        putToDelayedMessages(FileLoader.getAttachFileName(message.photoSize), message);
                        getFileLoader().loadFile(ImageLocation.getForObject(message.photoSize, message.locationParent), message.parentObject, "jpg", 2, 0);
                        return;
                    }
                }
                putToDelayedMessages(location2, message);
                getFileLoader().uploadFile(location2, true, true, 16777216);
                putToUploadingMessages(message.obj);
            }
        } else if (message.type == 1) {
            if (message.videoEditedInfo != null && message.videoEditedInfo.needConvert()) {
                String location3 = message.obj.messageOwner.attachPath;
                TLRPC.Document document = message.obj.getDocument();
                if (location3 == null) {
                    location3 = FileLoader.getDirectory(4) + "/" + document.id + ".mp4";
                }
                putToDelayedMessages(location3, message);
                MediaController.getInstance().scheduleVideoConvert(message.obj);
                putToUploadingMessages(message.obj);
            } else {
                if (message.videoEditedInfo != null) {
                    if (message.videoEditedInfo.file != null) {
                        if (message.sendRequest instanceof TLRPC.TL_messages_sendMedia) {
                            media3 = ((TLRPC.TL_messages_sendMedia) message.sendRequest).media;
                        } else {
                            media3 = ((TLRPC.TL_messages_editMessage) message.sendRequest).media;
                        }
                        media3.file = message.videoEditedInfo.file;
                        message.videoEditedInfo.file = null;
                    } else if (message.videoEditedInfo.encryptedFile != null) {
                        TLRPC.TL_decryptedMessage decryptedMessage = (TLRPC.TL_decryptedMessage) message.sendEncryptedRequest;
                        decryptedMessage.media.size = (int) message.videoEditedInfo.estimatedSize;
                        decryptedMessage.media.key = message.videoEditedInfo.key;
                        decryptedMessage.media.iv = message.videoEditedInfo.iv;
                        getSecretChatHelper().performSendEncryptedRequest(decryptedMessage, message.obj.messageOwner, message.encryptedChat, message.videoEditedInfo.encryptedFile, message.originalPath, message.obj);
                        message.videoEditedInfo.encryptedFile = null;
                        return;
                    }
                }
                if (message.sendRequest != null) {
                    if (message.sendRequest instanceof TLRPC.TL_messages_sendMedia) {
                        media2 = ((TLRPC.TL_messages_sendMedia) message.sendRequest).media;
                    } else {
                        media2 = ((TLRPC.TL_messages_editMessage) message.sendRequest).media;
                    }
                    if (media2.file == null) {
                        String location4 = message.obj.messageOwner.attachPath;
                        TLRPC.Document document2 = message.obj.getDocument();
                        if (location4 == null) {
                            location4 = FileLoader.getDirectory(4) + "/" + document2.id + ".mp4";
                        }
                        putToDelayedMessages(location4, message);
                        if (message.obj.videoEditedInfo != null && message.obj.videoEditedInfo.needConvert()) {
                            getFileLoader().uploadFile(location4, false, false, document2.size, 33554432, true);
                        } else {
                            getFileLoader().uploadFile(location4, false, false, 33554432);
                        }
                        putToUploadingMessages(message.obj);
                    } else {
                        String location5 = FileLoader.getDirectory(4) + "/" + message.photoSize.location.volume_id + "_" + message.photoSize.location.local_id + ".jpg";
                        putToDelayedMessages(location5, message);
                        getFileLoader().uploadFile(location5, false, true, 16777216);
                        putToUploadingMessages(message.obj);
                    }
                } else {
                    String location6 = message.obj.messageOwner.attachPath;
                    TLRPC.Document document3 = message.obj.getDocument();
                    if (location6 == null) {
                        location6 = FileLoader.getDirectory(4) + "/" + document3.id + ".mp4";
                    }
                    if (message.sendEncryptedRequest != null && document3.dc_id != 0) {
                        File file2 = new File(location6);
                        if (!file2.exists()) {
                            putToDelayedMessages(FileLoader.getAttachFileName(document3), message);
                            getFileLoader().loadFile(document3, message.parentObject, 2, 0);
                            return;
                        }
                    }
                    putToDelayedMessages(location6, message);
                    if (message.obj.videoEditedInfo != null && message.obj.videoEditedInfo.needConvert()) {
                        getFileLoader().uploadFile(location6, true, false, document3.size, 33554432, true);
                    } else {
                        getFileLoader().uploadFile(location6, true, false, 33554432);
                    }
                    putToUploadingMessages(message.obj);
                }
            }
        } else if (message.type == 2) {
            if (message.httpLocation != null) {
                putToDelayedMessages(message.httpLocation, message);
                ImageLoader.getInstance().loadHttpFile(message.httpLocation, "gif", this.currentAccount);
            } else if (message.sendRequest != null) {
                if (message.sendRequest instanceof TLRPC.TL_messages_sendMedia) {
                    media = ((TLRPC.TL_messages_sendMedia) message.sendRequest).media;
                } else {
                    media = ((TLRPC.TL_messages_editMessage) message.sendRequest).media;
                }
                if (media.file == null) {
                    String location7 = message.obj.messageOwner.attachPath;
                    putToDelayedMessages(location7, message);
                    getFileLoader().uploadFile(location7, message.sendRequest == null, false, 67108864);
                    putToUploadingMessages(message.obj);
                } else if (media.thumb == null && message.photoSize != null) {
                    String location8 = FileLoader.getDirectory(4) + "/" + message.photoSize.location.volume_id + "_" + message.photoSize.location.local_id + ".jpg";
                    putToDelayedMessages(location8, message);
                    getFileLoader().uploadFile(location8, false, true, 16777216);
                    putToUploadingMessages(message.obj);
                }
            } else {
                String location9 = message.obj.messageOwner.attachPath;
                TLRPC.Document document4 = message.obj.getDocument();
                if (message.sendEncryptedRequest != null && document4.dc_id != 0) {
                    File file3 = new File(location9);
                    if (!file3.exists()) {
                        putToDelayedMessages(FileLoader.getAttachFileName(document4), message);
                        getFileLoader().loadFile(document4, message.parentObject, 2, 0);
                        return;
                    }
                }
                putToDelayedMessages(location9, message);
                getFileLoader().uploadFile(location9, true, false, 67108864);
                putToUploadingMessages(message.obj);
            }
        } else if (message.type == 3) {
            String location10 = message.obj.messageOwner.attachPath;
            putToDelayedMessages(location10, message);
            getFileLoader().uploadFile(location10, message.sendRequest == null, true, ConnectionsManager.FileTypeAudio);
            putToUploadingMessages(message.obj);
        } else if (message.type == 4) {
            boolean add = index < 0;
            if (message.performMediaUpload) {
                if (index >= 0) {
                    index2 = index;
                } else {
                    index2 = message.messageObjects.size() - 1;
                }
                MessageObject messageObject2 = message.messageObjects.get(index2);
                if (messageObject2.getDocument() != null) {
                    if (message.videoEditedInfo != null) {
                        String location11 = messageObject2.messageOwner.attachPath;
                        TLRPC.Document document5 = messageObject2.getDocument();
                        if (location11 == null) {
                            location11 = FileLoader.getDirectory(4) + "/" + document5.id + ".mp4";
                        }
                        putToDelayedMessages(location11, message);
                        message.extraHashMap.put(messageObject2, location11);
                        message.extraHashMap.put(location11 + "_i", messageObject2);
                        if (message.photoSize != null) {
                            message.extraHashMap.put(location11 + "_t", message.photoSize);
                        }
                        MediaController.getInstance().scheduleVideoConvert(messageObject2);
                        message.obj = messageObject2;
                        putToUploadingMessages(messageObject2);
                    } else {
                        TLRPC.Document document6 = messageObject2.getDocument();
                        String documentLocation = messageObject2.messageOwner.attachPath;
                        if (documentLocation != null) {
                            messageObject = messageObject2;
                        } else {
                            messageObject = messageObject2;
                            documentLocation = FileLoader.getDirectory(4) + "/" + document6.id + ".mp4";
                        }
                        if (message.sendRequest != null) {
                            TLRPC.TL_messages_sendMultiMedia request = (TLRPC.TL_messages_sendMultiMedia) message.sendRequest;
                            TLRPC.InputMedia media4 = request.multi_media.get(index2).media;
                            if (media4.file == null) {
                                putToDelayedMessages(documentLocation, message);
                                MessageObject messageObject3 = messageObject;
                                message.extraHashMap.put(messageObject3, documentLocation);
                                message.extraHashMap.put(documentLocation, media4);
                                message.extraHashMap.put(documentLocation + "_i", messageObject3);
                                if (message.photoSize != null) {
                                    message.extraHashMap.put(documentLocation + "_t", message.photoSize);
                                }
                                if (messageObject3.videoEditedInfo != null && messageObject3.videoEditedInfo.needConvert()) {
                                    getFileLoader().uploadFile(documentLocation, false, false, document6.size, 33554432, true);
                                } else {
                                    getFileLoader().uploadFile(documentLocation, false, false, 33554432);
                                }
                                putToUploadingMessages(messageObject3);
                            } else {
                                MessageObject messageObject4 = messageObject;
                                String location12 = FileLoader.getDirectory(4) + "/" + message.photoSize.location.volume_id + "_" + message.photoSize.location.local_id + ".jpg";
                                putToDelayedMessages(location12, message);
                                message.extraHashMap.put(location12 + "_o", documentLocation);
                                message.extraHashMap.put(messageObject4, location12);
                                message.extraHashMap.put(location12, media4);
                                getFileLoader().uploadFile(location12, false, true, 16777216);
                                putToUploadingMessages(messageObject4);
                            }
                        } else {
                            MessageObject messageObject5 = messageObject;
                            TLRPC.TL_messages_sendEncryptedMultiMedia request2 = (TLRPC.TL_messages_sendEncryptedMultiMedia) message.sendEncryptedRequest;
                            putToDelayedMessages(documentLocation, message);
                            message.extraHashMap.put(messageObject5, documentLocation);
                            message.extraHashMap.put(documentLocation, request2.files.get(index2));
                            message.extraHashMap.put(documentLocation + "_i", messageObject5);
                            if (message.photoSize != null) {
                                message.extraHashMap.put(documentLocation + "_t", message.photoSize);
                            }
                            if (messageObject5.videoEditedInfo != null && messageObject5.videoEditedInfo.needConvert()) {
                                getFileLoader().uploadFile(documentLocation, true, false, document6.size, 33554432, true);
                            } else {
                                getFileLoader().uploadFile(documentLocation, true, false, 33554432);
                            }
                            putToUploadingMessages(messageObject5);
                        }
                    }
                    message.videoEditedInfo = null;
                    message.photoSize = null;
                } else if (message.httpLocation != null) {
                    putToDelayedMessages(message.httpLocation, message);
                    message.extraHashMap.put(messageObject2, message.httpLocation);
                    message.extraHashMap.put(message.httpLocation, messageObject2);
                    ImageLoader.getInstance().loadHttpFile(message.httpLocation, "file", this.currentAccount);
                    message.httpLocation = null;
                } else {
                    if (message.sendRequest != null) {
                        TLRPC.TL_messages_sendMultiMedia request3 = (TLRPC.TL_messages_sendMultiMedia) message.sendRequest;
                        inputMedia = request3.multi_media.get(index2).media;
                    } else {
                        TLObject inputMedia2 = message.sendEncryptedRequest;
                        TLRPC.TL_messages_sendEncryptedMultiMedia request4 = (TLRPC.TL_messages_sendEncryptedMultiMedia) inputMedia2;
                        inputMedia = request4.files.get(index2);
                    }
                    String location13 = FileLoader.getPathToAttach(message.photoSize).toString();
                    putToDelayedMessages(location13, message);
                    message.extraHashMap.put(location13, inputMedia);
                    message.extraHashMap.put(messageObject2, location13);
                    getFileLoader().uploadFile(location13, message.sendEncryptedRequest != null, true, 16777216);
                    putToUploadingMessages(messageObject2);
                    message.photoSize = null;
                }
                message.performMediaUpload = false;
            } else if (!message.messageObjects.isEmpty()) {
                putToSendingMessages(message.messageObjects.get(message.messageObjects.size() - 1).messageOwner, message.finalGroupMessage != 0);
            }
            sendReadyToSendGroup(message, add, true);
        }
    }

    private void uploadMultiMedia(final DelayedMessage message, final TLRPC.InputMedia inputMedia, TLRPC.InputEncryptedFile inputEncryptedFile, String key) {
        Float valueOf = Float.valueOf(1.0f);
        if (inputMedia != null) {
            TLRPC.TL_messages_sendMultiMedia multiMedia = (TLRPC.TL_messages_sendMultiMedia) message.sendRequest;
            int a2 = 0;
            while (true) {
                if (a2 >= multiMedia.multi_media.size()) {
                    break;
                } else if (multiMedia.multi_media.get(a2).media != inputMedia) {
                    a2++;
                } else {
                    putToSendingMessages(message.messages.get(a2), message.scheduled);
                    getNotificationCenter().postNotificationName(NotificationCenter.FileUploadProgressChanged, key, valueOf, false);
                    break;
                }
            }
            TLRPC.TL_messages_uploadMedia req = new TLRPC.TL_messages_uploadMedia();
            req.media = inputMedia;
            req.peer = ((TLRPC.TL_messages_sendMultiMedia) message.sendRequest).peer;
            getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda50
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    SendMessagesHelper.this.m998xd5172534(inputMedia, message, tLObject, tL_error);
                }
            });
        } else if (inputEncryptedFile != null) {
            TLRPC.TL_messages_sendEncryptedMultiMedia multiMedia2 = (TLRPC.TL_messages_sendEncryptedMultiMedia) message.sendEncryptedRequest;
            int a3 = 0;
            while (true) {
                if (a3 >= multiMedia2.files.size()) {
                    break;
                } else if (multiMedia2.files.get(a3) != inputEncryptedFile) {
                    a3++;
                } else {
                    putToSendingMessages(message.messages.get(a3), message.scheduled);
                    getNotificationCenter().postNotificationName(NotificationCenter.FileUploadProgressChanged, key, valueOf, false);
                    break;
                }
            }
            sendReadyToSendGroup(message, false, true);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$uploadMultiMedia$22$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m998xd5172534(final TLRPC.InputMedia inputMedia, final DelayedMessage message, final TLObject response, TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda7
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m997x122abbd5(response, inputMedia, message);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$uploadMultiMedia$21$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m997x122abbd5(TLObject response, TLRPC.InputMedia inputMedia, DelayedMessage message) {
        TLRPC.InputMedia newInputMedia = null;
        if (response != null) {
            TLRPC.MessageMedia messageMedia = (TLRPC.MessageMedia) response;
            if ((inputMedia instanceof TLRPC.TL_inputMediaUploadedPhoto) && (messageMedia instanceof TLRPC.TL_messageMediaPhoto)) {
                TLRPC.TL_inputMediaPhoto inputMediaPhoto = new TLRPC.TL_inputMediaPhoto();
                inputMediaPhoto.id = new TLRPC.TL_inputPhoto();
                inputMediaPhoto.id.id = messageMedia.photo.id;
                inputMediaPhoto.id.access_hash = messageMedia.photo.access_hash;
                inputMediaPhoto.id.file_reference = messageMedia.photo.file_reference;
                newInputMedia = inputMediaPhoto;
            } else if ((inputMedia instanceof TLRPC.TL_inputMediaUploadedDocument) && (messageMedia instanceof TLRPC.TL_messageMediaDocument)) {
                TLRPC.TL_inputMediaDocument inputMediaDocument = new TLRPC.TL_inputMediaDocument();
                inputMediaDocument.id = new TLRPC.TL_inputDocument();
                inputMediaDocument.id.id = messageMedia.document.id;
                inputMediaDocument.id.access_hash = messageMedia.document.access_hash;
                inputMediaDocument.id.file_reference = messageMedia.document.file_reference;
                newInputMedia = inputMediaDocument;
            }
        }
        if (newInputMedia != null) {
            if (inputMedia.ttl_seconds != 0) {
                newInputMedia.ttl_seconds = inputMedia.ttl_seconds;
                newInputMedia.flags |= 1;
            }
            TLRPC.TL_messages_sendMultiMedia req1 = (TLRPC.TL_messages_sendMultiMedia) message.sendRequest;
            int a2 = 0;
            while (true) {
                if (a2 >= req1.multi_media.size()) {
                    break;
                } else if (req1.multi_media.get(a2).media != inputMedia) {
                    a2++;
                } else {
                    req1.multi_media.get(a2).media = newInputMedia;
                    break;
                }
            }
            sendReadyToSendGroup(message, false, true);
            return;
        }
        message.markAsError();
    }

    private void sendReadyToSendGroup(DelayedMessage message, boolean add, boolean check) {
        DelayedMessage maxDelayedMessage;
        if (message.messageObjects.isEmpty()) {
            message.markAsError();
            return;
        }
        String key = "group_" + message.groupId;
        if (message.finalGroupMessage != message.messageObjects.get(message.messageObjects.size() - 1).getId()) {
            if (add) {
                putToDelayedMessages(key, message);
                return;
            }
            return;
        }
        if (add) {
            this.delayedMessages.remove(key);
            getMessagesStorage().putMessages(message.messages, false, true, false, 0, message.scheduled);
            getMessagesController().updateInterfaceWithMessages(message.peer, message.messageObjects, message.scheduled);
            if (!message.scheduled) {
                getNotificationCenter().postNotificationName(NotificationCenter.dialogsNeedReload, new Object[0]);
            }
        }
        if (message.sendRequest instanceof TLRPC.TL_messages_sendMultiMedia) {
            TLRPC.TL_messages_sendMultiMedia request = (TLRPC.TL_messages_sendMultiMedia) message.sendRequest;
            for (int a2 = 0; a2 < request.multi_media.size(); a2++) {
                TLRPC.InputMedia inputMedia = request.multi_media.get(a2).media;
                if ((inputMedia instanceof TLRPC.TL_inputMediaUploadedPhoto) || (inputMedia instanceof TLRPC.TL_inputMediaUploadedDocument)) {
                    return;
                }
            }
            if (check && (maxDelayedMessage = findMaxDelayedMessageForMessageId(message.finalGroupMessage, message.peer)) != null) {
                maxDelayedMessage.addDelayedRequest(message.sendRequest, message.messageObjects, message.originalPaths, message.parentObjects, message, message.scheduled);
                if (message.requests != null) {
                    maxDelayedMessage.requests.addAll(message.requests);
                    return;
                }
                return;
            }
        } else {
            TLRPC.TL_messages_sendEncryptedMultiMedia request2 = (TLRPC.TL_messages_sendEncryptedMultiMedia) message.sendEncryptedRequest;
            for (int a3 = 0; a3 < request2.files.size(); a3++) {
                if (request2.files.get(a3) instanceof TLRPC.TL_inputEncryptedFile) {
                    return;
                }
            }
        }
        if (message.sendRequest instanceof TLRPC.TL_messages_sendMultiMedia) {
            performSendMessageRequestMulti((TLRPC.TL_messages_sendMultiMedia) message.sendRequest, message.messageObjects, message.originalPaths, message.parentObjects, message, message.scheduled);
        } else {
            getSecretChatHelper().performSendEncryptedRequest((TLRPC.TL_messages_sendEncryptedMultiMedia) message.sendEncryptedRequest, message);
        }
        message.sendDelayedRequests();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$stopVideoService$23$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m995x5b060263(String path) {
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.stopEncodingService, path, Integer.valueOf(this.currentAccount));
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$stopVideoService$24$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m996x1df26bc2(final String path) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda60
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m995x5b060263(path);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void stopVideoService(final String path) {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda21
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m996x1df26bc2(path);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void putToSendingMessages(final TLRPC.Message message, final boolean scheduled) {
        if (Thread.currentThread() != ApplicationLoader.applicationHandler.getLooper().getThread()) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda14
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m980x2d93cb41(message, scheduled);
                }
            });
        } else {
            putToSendingMessages(message, scheduled, true);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putToSendingMessages$25$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m980x2d93cb41(TLRPC.Message message, boolean scheduled) {
        putToSendingMessages(message, scheduled, true);
    }

    protected void putToSendingMessages(TLRPC.Message message, boolean scheduled, boolean notify) {
        if (message == null) {
            return;
        }
        if (message.id > 0) {
            this.editingMessages.put(message.id, message);
            return;
        }
        boolean contains = this.sendingMessages.indexOfKey(message.id) >= 0;
        removeFromUploadingMessages(message.id, scheduled);
        this.sendingMessages.put(message.id, message);
        if (!scheduled && !contains) {
            long did = MessageObject.getDialogId(message);
            LongSparseArray<Integer> longSparseArray = this.sendingMessagesIdDialogs;
            longSparseArray.put(did, Integer.valueOf(longSparseArray.get(did, 0).intValue() + 1));
            if (notify) {
                getNotificationCenter().postNotificationName(NotificationCenter.sendingMessagesChanged, new Object[0]);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public TLRPC.Message removeFromSendingMessages(int mid, boolean scheduled) {
        TLRPC.Message message;
        long did;
        Integer currentCount;
        if (mid > 0) {
            message = this.editingMessages.get(mid);
            if (message != null) {
                this.editingMessages.remove(mid);
            }
        } else {
            message = this.sendingMessages.get(mid);
            if (message != null) {
                this.sendingMessages.remove(mid);
                if (!scheduled && (currentCount = this.sendingMessagesIdDialogs.get((did = MessageObject.getDialogId(message)))) != null) {
                    int count = currentCount.intValue() - 1;
                    if (count <= 0) {
                        this.sendingMessagesIdDialogs.remove(did);
                    } else {
                        this.sendingMessagesIdDialogs.put(did, Integer.valueOf(count));
                    }
                    getNotificationCenter().postNotificationName(NotificationCenter.sendingMessagesChanged, new Object[0]);
                }
            }
        }
        return message;
    }

    public int getSendingMessageId(long did) {
        for (int a2 = 0; a2 < this.sendingMessages.size(); a2++) {
            TLRPC.Message message = this.sendingMessages.valueAt(a2);
            if (message.dialog_id == did) {
                return message.id;
            }
        }
        for (int a3 = 0; a3 < this.uploadMessages.size(); a3++) {
            TLRPC.Message message2 = this.uploadMessages.valueAt(a3);
            if (message2.dialog_id == did) {
                return message2.id;
            }
        }
        return 0;
    }

    protected void putToUploadingMessages(MessageObject obj) {
        if (obj == null || obj.getId() > 0 || obj.scheduled) {
            return;
        }
        TLRPC.Message message = obj.messageOwner;
        boolean contains = this.uploadMessages.indexOfKey(message.id) >= 0;
        this.uploadMessages.put(message.id, message);
        if (!contains) {
            long did = MessageObject.getDialogId(message);
            LongSparseArray<Integer> longSparseArray = this.uploadingMessagesIdDialogs;
            longSparseArray.put(did, Integer.valueOf(longSparseArray.get(did, 0).intValue() + 1));
            getNotificationCenter().postNotificationName(NotificationCenter.sendingMessagesChanged, new Object[0]);
        }
    }

    protected void removeFromUploadingMessages(int mid, boolean scheduled) {
        TLRPC.Message message;
        if (mid <= 0 && !scheduled && (message = this.uploadMessages.get(mid)) != null) {
            this.uploadMessages.remove(mid);
            long did = MessageObject.getDialogId(message);
            Integer currentCount = this.uploadingMessagesIdDialogs.get(did);
            if (currentCount != null) {
                int count = currentCount.intValue() - 1;
                if (count <= 0) {
                    this.uploadingMessagesIdDialogs.remove(did);
                } else {
                    this.uploadingMessagesIdDialogs.put(did, Integer.valueOf(count));
                }
                getNotificationCenter().postNotificationName(NotificationCenter.sendingMessagesChanged, new Object[0]);
            }
        }
    }

    public boolean isSendingMessage(int mid) {
        return this.sendingMessages.indexOfKey(mid) >= 0 || this.editingMessages.indexOfKey(mid) >= 0;
    }

    public boolean isSendingMessageIdDialog(long did) {
        return this.sendingMessagesIdDialogs.get(did, 0).intValue() > 0;
    }

    public boolean isUploadingMessageIdDialog(long did) {
        return this.uploadingMessagesIdDialogs.get(did, 0).intValue() > 0;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void performSendMessageRequestMulti(final TLRPC.TL_messages_sendMultiMedia req, final ArrayList<MessageObject> msgObjs, final ArrayList<String> originalPaths, final ArrayList<Object> parentObjects, final DelayedMessage delayedMessage, final boolean scheduled) {
        int size = msgObjs.size();
        for (int a2 = 0; a2 < size; a2++) {
            putToSendingMessages(msgObjs.get(a2).messageOwner, scheduled);
        }
        getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda23
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m978xd7d64131(parentObjects, req, msgObjs, originalPaths, delayedMessage, scheduled, tLObject, tL_error);
            }
        }, (QuickAckDelegate) null, 68);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$32$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m978xd7d64131(final ArrayList parentObjects, final TLRPC.TL_messages_sendMultiMedia req, final ArrayList msgObjs, final ArrayList originalPaths, final DelayedMessage delayedMessage, final boolean scheduled, final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda15
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m977x14e9d7d2(error, parentObjects, req, msgObjs, originalPaths, delayedMessage, scheduled, response);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$31$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m977x14e9d7d2(TLRPC.TL_error error, ArrayList parentObjects, final TLRPC.TL_messages_sendMultiMedia req, final ArrayList msgObjs, ArrayList originalPaths, final DelayedMessage delayedMessage, final boolean scheduled, TLObject response) {
        char c;
        TLRPC.Updates updates;
        int i;
        ArrayList arrayList = originalPaths;
        if (error != null && FileRefController.isFileRefError(error.text)) {
            if (parentObjects != null) {
                ArrayList<Object> arrayList2 = new ArrayList<>(parentObjects);
                getFileRefController().requestReference(arrayList2, req, msgObjs, arrayList, arrayList2, delayedMessage, Boolean.valueOf(scheduled));
                return;
            } else if (delayedMessage != null) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper.3
                    @Override // java.lang.Runnable
                    public void run() {
                        int size = req.multi_media.size();
                        for (int a2 = 0; a2 < size; a2++) {
                            if (delayedMessage.parentObjects.get(a2) != null) {
                                SendMessagesHelper.this.removeFromSendingMessages(((MessageObject) msgObjs.get(a2)).getId(), scheduled);
                                TLRPC.TL_inputSingleMedia request = req.multi_media.get(a2);
                                if (request.media instanceof TLRPC.TL_inputMediaPhoto) {
                                    request.media = delayedMessage.inputMedias.get(a2);
                                } else if (request.media instanceof TLRPC.TL_inputMediaDocument) {
                                    request.media = delayedMessage.inputMedias.get(a2);
                                }
                                DelayedMessage delayedMessage2 = delayedMessage;
                                delayedMessage2.videoEditedInfo = delayedMessage2.videoEditedInfos.get(a2);
                                DelayedMessage delayedMessage3 = delayedMessage;
                                delayedMessage3.httpLocation = delayedMessage3.httpLocations.get(a2);
                                DelayedMessage delayedMessage4 = delayedMessage;
                                delayedMessage4.photoSize = delayedMessage4.locations.get(a2);
                                delayedMessage.performMediaUpload = true;
                                SendMessagesHelper.this.performSendDelayedMessage(delayedMessage, a2);
                            }
                        }
                    }
                });
                return;
            }
        }
        boolean isSentError = false;
        if (error == null) {
            SparseArray<TLRPC.Message> newMessages = new SparseArray<>();
            LongSparseArray<Integer> newIds = new LongSparseArray<>();
            TLRPC.Updates updates2 = (TLRPC.Updates) response;
            ArrayList<TLRPC.Update> updatesArr = ((TLRPC.Updates) response).updates;
            int a2 = 0;
            while (a2 < updatesArr.size()) {
                TLRPC.Update update = updatesArr.get(a2);
                if (update instanceof TLRPC.TL_updateMessageID) {
                    TLRPC.TL_updateMessageID updateMessageID = (TLRPC.TL_updateMessageID) update;
                    newIds.put(updateMessageID.random_id, Integer.valueOf(updateMessageID.id));
                    updatesArr.remove(a2);
                    a2--;
                } else if (update instanceof TLRPC.TL_updateNewMessage) {
                    final TLRPC.TL_updateNewMessage newMessage = (TLRPC.TL_updateNewMessage) update;
                    newMessages.put(newMessage.message.id, newMessage.message);
                    Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda53
                        @Override // java.lang.Runnable
                        public final void run() {
                            SendMessagesHelper.this.m972x48e7242c(newMessage);
                        }
                    });
                    updatesArr.remove(a2);
                    a2--;
                } else if (update instanceof TLRPC.TL_updateNewChannelMessage) {
                    final TLRPC.TL_updateNewChannelMessage newMessage2 = (TLRPC.TL_updateNewChannelMessage) update;
                    newMessages.put(newMessage2.message.id, newMessage2.message);
                    Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda54
                        @Override // java.lang.Runnable
                        public final void run() {
                            SendMessagesHelper.this.m973xbd38d8b(newMessage2);
                        }
                    });
                    updatesArr.remove(a2);
                    a2--;
                } else if (update instanceof TLRPC.TL_updateNewScheduledMessage) {
                    TLRPC.TL_updateNewScheduledMessage newMessage3 = (TLRPC.TL_updateNewScheduledMessage) update;
                    newMessages.put(newMessage3.message.id, newMessage3.message);
                    updatesArr.remove(a2);
                    a2--;
                }
                a2++;
            }
            int i2 = 0;
            while (true) {
                if (i2 >= msgObjs.size()) {
                    updates = updates2;
                    c = 0;
                    break;
                }
                MessageObject msgObj = (MessageObject) msgObjs.get(i2);
                String originalPath = (String) arrayList.get(i2);
                final TLRPC.Message newMsgObj = msgObj.messageOwner;
                final int oldId = newMsgObj.id;
                final ArrayList<TLRPC.Message> sentMessages = new ArrayList<>();
                String str = newMsgObj.attachPath;
                ArrayList<TLRPC.Update> updatesArr2 = updatesArr;
                Integer id = newIds.get(newMsgObj.random_id);
                if (id != null) {
                    TLRPC.Message message = newMessages.get(id.intValue());
                    if (message == null) {
                        updates = updates2;
                        c = 0;
                        isSentError = true;
                        break;
                    }
                    sentMessages.add(message);
                    LongSparseArray<Integer> newIds2 = newIds;
                    TLRPC.Updates updates3 = updates2;
                    SparseArray<TLRPC.Message> newMessages2 = newMessages;
                    updateMediaPaths(msgObj, message, message.id, originalPath, false);
                    final int existFlags = msgObj.getMediaExistanceFlags();
                    newMsgObj.id = message.id;
                    if ((newMsgObj.flags & Integer.MIN_VALUE) != 0) {
                        message.flags |= Integer.MIN_VALUE;
                    }
                    final long grouped_id = message.grouped_id;
                    if (!scheduled) {
                        Integer value = getMessagesController().dialogs_read_outbox_max.get(Long.valueOf(message.dialog_id));
                        if (value == null) {
                            value = Integer.valueOf(getMessagesStorage().getDialogReadMax(message.out, message.dialog_id));
                            getMessagesController().dialogs_read_outbox_max.put(Long.valueOf(message.dialog_id), value);
                        }
                        message.unread = value.intValue() < message.id;
                    }
                    if (0 == 0) {
                        getStatsController().incrementSentItemsCount(ApplicationLoader.getCurrentNetworkType(), 1, 1);
                        newMsgObj.send_state = 0;
                        getNotificationCenter().postNotificationName(NotificationCenter.messageReceivedByServer, Integer.valueOf(oldId), Integer.valueOf(newMsgObj.id), newMsgObj, Long.valueOf(newMsgObj.dialog_id), Long.valueOf(grouped_id), Integer.valueOf(existFlags), Boolean.valueOf(scheduled));
                        i = i2;
                        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda56
                            @Override // java.lang.Runnable
                            public final void run() {
                                SendMessagesHelper.this.m975x91ac6049(newMsgObj, oldId, scheduled, sentMessages, grouped_id, existFlags);
                            }
                        });
                    } else {
                        i = i2;
                    }
                    i2 = i + 1;
                    arrayList = originalPaths;
                    newIds = newIds2;
                    updatesArr = updatesArr2;
                    updates2 = updates3;
                    newMessages = newMessages2;
                } else {
                    updates = updates2;
                    c = 0;
                    isSentError = true;
                    break;
                }
            }
            final TLRPC.Updates updates4 = updates;
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda57
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m976x51fd6e73(updates4);
                }
            });
        } else {
            c = 0;
            AlertsCreator.processError(this.currentAccount, error, null, req, new Object[0]);
            isSentError = true;
        }
        if (isSentError) {
            for (int i3 = 0; i3 < msgObjs.size(); i3++) {
                TLRPC.Message newMsgObj2 = ((MessageObject) msgObjs.get(i3)).messageOwner;
                getMessagesStorage().markMessageAsSendError(newMsgObj2, scheduled);
                newMsgObj2.send_state = 2;
                NotificationCenter notificationCenter = getNotificationCenter();
                int i4 = NotificationCenter.messageSendError;
                Object[] objArr = new Object[1];
                objArr[c] = Integer.valueOf(newMsgObj2.id);
                notificationCenter.postNotificationName(i4, objArr);
                processSentMessage(newMsgObj2.id);
                removeFromSendingMessages(newMsgObj2.id, scheduled);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$26$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m972x48e7242c(TLRPC.TL_updateNewMessage newMessage) {
        getMessagesController().processNewDifferenceParams(-1, newMessage.pts, -1, newMessage.pts_count);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$27$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m973xbd38d8b(TLRPC.TL_updateNewChannelMessage newMessage) {
        getMessagesController().processNewChannelDifferenceParams(newMessage.pts, newMessage.pts_count, newMessage.message.to_id.channel_id);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$29$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m975x91ac6049(final TLRPC.Message newMsgObj, final int oldId, final boolean scheduled, ArrayList sentMessages, final long grouped_id, final int existFlags) {
        getMessagesStorage().updateMessageStateAndId(newMsgObj.random_id, Integer.valueOf(oldId), newMsgObj.id, 0, false, newMsgObj.to_id.channel_id, scheduled ? 1 : 0);
        getMessagesStorage().putMessages((ArrayList<TLRPC.Message>) sentMessages, true, true, false, 0, scheduled);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda41
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m974xcebff6ea(newMsgObj, oldId, grouped_id, existFlags, scheduled);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$28$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m974xcebff6ea(TLRPC.Message newMsgObj, int oldId, long grouped_id, int existFlags, boolean scheduled) {
        getMediaDataController().increasePeerRaiting(newMsgObj.dialog_id);
        getNotificationCenter().postNotificationName(NotificationCenter.messageReceivedByServer, Integer.valueOf(oldId), Integer.valueOf(newMsgObj.id), newMsgObj, Long.valueOf(newMsgObj.dialog_id), Long.valueOf(grouped_id), Integer.valueOf(existFlags), Boolean.valueOf(scheduled));
        processSentMessage(oldId);
        removeFromSendingMessages(oldId, scheduled);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequestMulti$30$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m976x51fd6e73(TLRPC.Updates updates) {
        getMessagesController().processUpdates(updates, false);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void performSendMessageRequest(TLObject req, MessageObject msgObj, String originalPath, DelayedMessage delayedMessage, Object parentObject, boolean scheduled) {
        performSendMessageRequest(req, msgObj, originalPath, null, false, delayedMessage, parentObject, scheduled);
    }

    private DelayedMessage findMaxDelayedMessageForMessageId(int messageId, long dialogId) {
        DelayedMessage maxDelayedMessage = null;
        int maxDalyedMessageId = Integer.MIN_VALUE;
        for (Map.Entry<String, ArrayList<DelayedMessage>> entry : this.delayedMessages.entrySet()) {
            ArrayList<DelayedMessage> messages = entry.getValue();
            int size = messages.size();
            for (int a2 = 0; a2 < size; a2++) {
                DelayedMessage delayedMessage = messages.get(a2);
                if ((delayedMessage.type == 4 || delayedMessage.type == 0) && delayedMessage.peer == dialogId) {
                    int mid = 0;
                    if (delayedMessage.obj != null) {
                        mid = delayedMessage.obj.getId();
                    } else if (delayedMessage.messageObjects != null && !delayedMessage.messageObjects.isEmpty()) {
                        mid = delayedMessage.messageObjects.get(delayedMessage.messageObjects.size() - 1).getId();
                    }
                    if (mid != 0 && mid > messageId && maxDelayedMessage == null && maxDalyedMessageId < mid) {
                        maxDelayedMessage = delayedMessage;
                        maxDalyedMessageId = mid;
                    }
                }
            }
        }
        return maxDelayedMessage;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void performSendMessageRequest(final TLObject req, final MessageObject msgObj, final String originalPath, final DelayedMessage parentMessage, final boolean check, final DelayedMessage delayedMessage, final Object parentObject, final boolean scheduled) {
        DelayedMessage maxDelayedMessage;
        if (!(req instanceof TLRPC.TL_messages_editMessage) && check && (maxDelayedMessage = findMaxDelayedMessageForMessageId(msgObj.getId(), msgObj.getDialogId())) != null) {
            maxDelayedMessage.addDelayedRequest(req, msgObj, originalPath, parentObject, delayedMessage, parentMessage != null ? parentMessage.scheduled : false);
            if (parentMessage != null && parentMessage.requests != null) {
                maxDelayedMessage.requests.addAll(parentMessage.requests);
                return;
            }
            return;
        }
        final TLRPC.Message newMsgObj = msgObj.messageOwner;
        putToSendingMessages(newMsgObj, scheduled);
        newMsgObj.reqId = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda10
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                SendMessagesHelper.this.m969x20ad5c3f(req, parentObject, msgObj, originalPath, parentMessage, check, delayedMessage, scheduled, newMsgObj, tLObject, tL_error);
            }
        }, new QuickAckDelegate() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda12
            @Override // im.uecahxyvjd.tgnet.QuickAckDelegate
            public final void run() {
                SendMessagesHelper.this.m971xa6862efd(newMsgObj);
            }
        }, (req instanceof TLRPC.TL_messages_sendMessage ? 128 : 0) | 68);
        if (parentMessage != null) {
            parentMessage.sendDelayedRequests();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$44$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m969x20ad5c3f(final TLObject req, Object parentObject, final MessageObject msgObj, final String originalPath, DelayedMessage parentMessage, boolean check, final DelayedMessage delayedMessage, final boolean scheduled, final TLRPC.Message newMsgObj, final TLObject response, final TLRPC.TL_error error) {
        if (error != null && (((req instanceof TLRPC.TL_messages_sendMedia) || (req instanceof TLRPC.TL_messages_editMessage)) && FileRefController.isFileRefError(error.text))) {
            if (parentObject != null) {
                getFileRefController().requestReference(parentObject, req, msgObj, originalPath, parentMessage, Boolean.valueOf(check), delayedMessage, Boolean.valueOf(scheduled));
                return;
            } else if (delayedMessage != null) {
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper.4
                    @Override // java.lang.Runnable
                    public void run() {
                        SendMessagesHelper.this.removeFromSendingMessages(newMsgObj.id, scheduled);
                        TLObject tLObject = req;
                        if (tLObject instanceof TLRPC.TL_messages_sendMedia) {
                            TLRPC.TL_messages_sendMedia request = (TLRPC.TL_messages_sendMedia) tLObject;
                            if (request.media instanceof TLRPC.TL_inputMediaPhoto) {
                                request.media = delayedMessage.inputUploadMedia;
                            } else if (request.media instanceof TLRPC.TL_inputMediaDocument) {
                                request.media = delayedMessage.inputUploadMedia;
                            }
                        } else if (tLObject instanceof TLRPC.TL_messages_editMessage) {
                            TLRPC.TL_messages_editMessage request2 = (TLRPC.TL_messages_editMessage) tLObject;
                            if (request2.media instanceof TLRPC.TL_inputMediaPhoto) {
                                request2.media = delayedMessage.inputUploadMedia;
                            } else if (request2.media instanceof TLRPC.TL_inputMediaDocument) {
                                request2.media = delayedMessage.inputUploadMedia;
                            }
                        }
                        delayedMessage.performMediaUpload = true;
                        SendMessagesHelper.this.performSendDelayedMessage(delayedMessage);
                    }
                });
                return;
            }
        }
        if (req instanceof TLRPC.TL_messages_editMessage) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda48
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m960x48f9031d(error, newMsgObj, response, msgObj, originalPath, scheduled, req);
                }
            });
        } else {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda49
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m968x5dc0f2e0(error, newMsgObj, response, msgObj, scheduled, originalPath, req);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$35$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m960x48f9031d(TLRPC.TL_error error, final TLRPC.Message newMsgObj, TLObject response, MessageObject msgObj, String originalPath, final boolean scheduled, TLObject req) {
        TLRPC.Message message;
        if (error == null) {
            String attachPath = newMsgObj.attachPath;
            final TLRPC.Updates updates = (TLRPC.Updates) response;
            ArrayList<TLRPC.Update> updatesArr = ((TLRPC.Updates) response).updates;
            int a2 = 0;
            while (true) {
                if (a2 >= updatesArr.size()) {
                    message = null;
                    break;
                }
                TLRPC.Update update = updatesArr.get(a2);
                if (update instanceof TLRPC.TL_updateEditMessage) {
                    TLRPC.TL_updateEditMessage newMessage = (TLRPC.TL_updateEditMessage) update;
                    TLRPC.Message message2 = newMessage.message;
                    message = message2;
                    break;
                } else if (update instanceof TLRPC.TL_updateEditChannelMessage) {
                    TLRPC.TL_updateEditChannelMessage newMessage2 = (TLRPC.TL_updateEditChannelMessage) update;
                    TLRPC.Message message3 = newMessage2.message;
                    message = message3;
                    break;
                } else if (!(update instanceof TLRPC.TL_updateNewScheduledMessage)) {
                    a2++;
                } else {
                    TLRPC.TL_updateNewScheduledMessage newMessage3 = (TLRPC.TL_updateNewScheduledMessage) update;
                    TLRPC.Message message4 = newMessage3.message;
                    message = message4;
                    break;
                }
            }
            if (message != null) {
                ImageLoader.saveMessageThumbs(message);
                updateMediaPaths(msgObj, message, message.id, originalPath, false);
            }
            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda6
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.this.m959x860c99be(updates, newMsgObj, scheduled);
                }
            });
            if (MessageObject.isVideoMessage(newMsgObj) || MessageObject.isRoundVideoMessage(newMsgObj) || MessageObject.isNewGifMessage(newMsgObj)) {
                stopVideoService(attachPath);
            }
            return;
        }
        AlertsCreator.processError(this.currentAccount, error, null, req, new Object[0]);
        if (MessageObject.isVideoMessage(newMsgObj) || MessageObject.isRoundVideoMessage(newMsgObj) || MessageObject.isNewGifMessage(newMsgObj)) {
            stopVideoService(newMsgObj.attachPath);
        }
        removeFromSendingMessages(newMsgObj.id, scheduled);
        revertEditingMessageObject(msgObj);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$34$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m959x860c99be(TLRPC.Updates updates, final TLRPC.Message newMsgObj, final boolean scheduled) {
        getMessagesController().processUpdates(updates, false);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda45
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m958xc320305f(newMsgObj, scheduled);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$33$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m958xc320305f(TLRPC.Message newMsgObj, boolean scheduled) {
        processSentMessage(newMsgObj.id);
        removeFromSendingMessages(newMsgObj.id, scheduled);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$43$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m968x5dc0f2e0(TLRPC.TL_error error, final TLRPC.Message newMsgObj, TLObject response, MessageObject msgObj, final boolean scheduled, String originalPath, TLObject req) {
        boolean isSentError;
        String attachPath;
        ArrayList<TLRPC.Message> sentMessages;
        int existFlags;
        TLRPC.Message message;
        int existFlags2;
        TLRPC.Message message2;
        TLRPC.Message message3;
        boolean isSentError2 = false;
        if (error == null) {
            final int oldId = newMsgObj.id;
            ArrayList<TLRPC.Message> sentMessages2 = new ArrayList<>();
            String attachPath2 = newMsgObj.attachPath;
            if (response instanceof TLRPC.TL_updateShortSentMessage) {
                final TLRPC.TL_updateShortSentMessage res = (TLRPC.TL_updateShortSentMessage) response;
                attachPath = attachPath2;
                sentMessages = sentMessages2;
                updateMediaPaths(msgObj, null, res.id, null, false);
                int existFlags3 = msgObj.getMediaExistanceFlags();
                int i = res.id;
                newMsgObj.id = i;
                newMsgObj.local_id = i;
                newMsgObj.date = res.date;
                newMsgObj.entities = res.entities;
                newMsgObj.out = res.out;
                if (res.media != null) {
                    newMsgObj.media = res.media;
                    newMsgObj.flags |= 512;
                    ImageLoader.saveMessageThumbs(newMsgObj);
                }
                if ((res.media instanceof TLRPC.TL_messageMediaGame) && !TextUtils.isEmpty(res.message)) {
                    newMsgObj.message = res.message;
                }
                if (!newMsgObj.entities.isEmpty()) {
                    newMsgObj.flags |= 128;
                }
                Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda28
                    @Override // java.lang.Runnable
                    public final void run() {
                        SendMessagesHelper.this.m961xbe56c7c(res);
                    }
                });
                sentMessages.add(newMsgObj);
                existFlags = existFlags3;
                isSentError = false;
            } else {
                attachPath = attachPath2;
                sentMessages = sentMessages2;
                if (response instanceof TLRPC.Updates) {
                    final TLRPC.Updates updates = (TLRPC.Updates) response;
                    ArrayList<TLRPC.Update> updatesArr = ((TLRPC.Updates) response).updates;
                    int a2 = 0;
                    while (true) {
                        if (a2 >= updatesArr.size()) {
                            message = null;
                            break;
                        }
                        TLRPC.Update update = updatesArr.get(a2);
                        if (update instanceof TLRPC.TL_updateNewMessage) {
                            final TLRPC.TL_updateNewMessage newMessage = (TLRPC.TL_updateNewMessage) update;
                            TLRPC.Message message4 = newMessage.message;
                            sentMessages.add(message4);
                            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda29
                                @Override // java.lang.Runnable
                                public final void run() {
                                    SendMessagesHelper.this.m962xced1d5db(newMessage);
                                }
                            });
                            updatesArr.remove(a2);
                            message = message4;
                            break;
                        } else if (update instanceof TLRPC.TL_updateNewChannelMessage) {
                            final TLRPC.TL_updateNewChannelMessage newMessage2 = (TLRPC.TL_updateNewChannelMessage) update;
                            TLRPC.Message message5 = newMessage2.message;
                            sentMessages.add(message5);
                            if ((newMsgObj.flags & Integer.MIN_VALUE) == 0) {
                                message2 = message5;
                            } else {
                                message2 = message5;
                                newMessage2.message.flags |= Integer.MIN_VALUE;
                            }
                            Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda30
                                @Override // java.lang.Runnable
                                public final void run() {
                                    SendMessagesHelper.this.m963x91be3f3a(newMessage2);
                                }
                            });
                            updatesArr.remove(a2);
                            message = message2;
                        } else if (!(update instanceof TLRPC.TL_updateNewScheduledMessage)) {
                            a2++;
                        } else {
                            TLRPC.TL_updateNewScheduledMessage newMessage3 = (TLRPC.TL_updateNewScheduledMessage) update;
                            TLRPC.Message message6 = newMessage3.message;
                            sentMessages.add(message6);
                            if ((newMsgObj.flags & Integer.MIN_VALUE) == 0) {
                                message3 = message6;
                            } else {
                                message3 = message6;
                                newMessage3.message.flags |= Integer.MIN_VALUE;
                            }
                            updatesArr.remove(a2);
                            message = message3;
                        }
                    }
                    if (message != null) {
                        ImageLoader.saveMessageThumbs(message);
                        if (!scheduled) {
                            Integer value = getMessagesController().dialogs_read_outbox_max.get(Long.valueOf(message.dialog_id));
                            if (value == null) {
                                value = Integer.valueOf(getMessagesStorage().getDialogReadMax(message.out, message.dialog_id));
                                getMessagesController().dialogs_read_outbox_max.put(Long.valueOf(message.dialog_id), value);
                            }
                            message.unread = value.intValue() < message.id;
                        }
                        updateMediaPaths(msgObj, message, message.id, originalPath, false);
                        existFlags2 = msgObj.getMediaExistanceFlags();
                        newMsgObj.id = message.id;
                        if (newMsgObj.message != null && message.message != null) {
                            newMsgObj.message = message.message;
                            final TLRPC.Message finalMessage = message;
                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda31
                                @Override // java.lang.Runnable
                                public final void run() {
                                    SendMessagesHelper.this.m964x54aaa899(finalMessage);
                                }
                            });
                        }
                    } else {
                        isSentError2 = true;
                        existFlags2 = 0;
                    }
                    Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda32
                        @Override // java.lang.Runnable
                        public final void run() {
                            SendMessagesHelper.this.m965x14fbb6c3(updates);
                        }
                    });
                    existFlags = existFlags2;
                    isSentError = isSentError2;
                } else {
                    existFlags = 0;
                    isSentError = false;
                }
            }
            if (MessageObject.isLiveLocationMessage(newMsgObj)) {
                getLocationController().addSharingLocation(newMsgObj.dialog_id, newMsgObj.id, newMsgObj.media.period, newMsgObj);
            }
            if (!isSentError) {
                getStatsController().incrementSentItemsCount(ApplicationLoader.getCurrentNetworkType(), 1, 1);
                newMsgObj.send_state = 0;
                getNotificationCenter().postNotificationName(NotificationCenter.messageReceivedByServer, Integer.valueOf(oldId), Integer.valueOf(newMsgObj.id), newMsgObj, Long.valueOf(newMsgObj.dialog_id), 0L, Integer.valueOf(existFlags), Boolean.valueOf(scheduled));
                final ArrayList<TLRPC.Message> arrayList = sentMessages;
                final int i2 = existFlags;
                final String str = attachPath;
                getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda34
                    @Override // java.lang.Runnable
                    public final void run() {
                        SendMessagesHelper.this.m967x9ad48981(newMsgObj, oldId, scheduled, arrayList, i2, str);
                    }
                });
            }
        } else {
            AlertsCreator.processError(this.currentAccount, error, null, req, new Object[0]);
            isSentError = true;
        }
        if (isSentError) {
            getMessagesStorage().markMessageAsSendError(newMsgObj, scheduled);
            newMsgObj.send_state = 2;
            getNotificationCenter().postNotificationName(NotificationCenter.messageSendError, Integer.valueOf(newMsgObj.id));
            processSentMessage(newMsgObj.id);
            if (MessageObject.isVideoMessage(newMsgObj) || MessageObject.isRoundVideoMessage(newMsgObj) || MessageObject.isNewGifMessage(newMsgObj)) {
                stopVideoService(newMsgObj.attachPath);
            }
            removeFromSendingMessages(newMsgObj.id, scheduled);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$36$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m961xbe56c7c(TLRPC.TL_updateShortSentMessage res) {
        getMessagesController().processNewDifferenceParams(-1, res.pts, res.date, res.pts_count);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$37$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m962xced1d5db(TLRPC.TL_updateNewMessage newMessage) {
        getMessagesController().processNewDifferenceParams(-1, newMessage.pts, -1, newMessage.pts_count);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$38$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m963x91be3f3a(TLRPC.TL_updateNewChannelMessage newMessage) {
        getMessagesController().processNewChannelDifferenceParams(newMessage.pts, newMessage.pts_count, newMessage.message.to_id.channel_id);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$39$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m964x54aaa899(TLRPC.Message finalMessage) {
        getNotificationCenter().postNotificationName(NotificationCenter.updateChatNewmsgMentionText, finalMessage);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$40$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m965x14fbb6c3(TLRPC.Updates updates) {
        getMessagesController().processUpdates(updates, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$42$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m967x9ad48981(final TLRPC.Message newMsgObj, final int oldId, final boolean scheduled, ArrayList sentMessages, final int existFlags, String attachPath) {
        getMessagesStorage().updateMessageStateAndId(newMsgObj.random_id, Integer.valueOf(oldId), newMsgObj.id, 0, false, newMsgObj.to_id.channel_id, scheduled ? 1 : 0);
        getMessagesStorage().putMessages((ArrayList<TLRPC.Message>) sentMessages, true, true, false, 0, scheduled);
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda16
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m966xd7e82022(newMsgObj, oldId, existFlags, scheduled);
            }
        });
        if (MessageObject.isVideoMessage(newMsgObj) || MessageObject.isRoundVideoMessage(newMsgObj) || MessageObject.isNewGifMessage(newMsgObj)) {
            stopVideoService(attachPath);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$41$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m966xd7e82022(TLRPC.Message newMsgObj, int oldId, int existFlags, boolean scheduled) {
        getMediaDataController().increasePeerRaiting(newMsgObj.dialog_id);
        getNotificationCenter().postNotificationName(NotificationCenter.messageReceivedByServer, Integer.valueOf(oldId), Integer.valueOf(newMsgObj.id), newMsgObj, Long.valueOf(newMsgObj.dialog_id), 0L, Integer.valueOf(existFlags), Boolean.valueOf(scheduled));
        processSentMessage(oldId);
        removeFromSendingMessages(oldId, scheduled);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$46$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m971xa6862efd(final TLRPC.Message newMsgObj) {
        final int msg_id = newMsgObj.id;
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda13
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m970xe399c59e(newMsgObj, msg_id);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$performSendMessageRequest$45$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m970xe399c59e(TLRPC.Message newMsgObj, int msg_id) {
        newMsgObj.send_state = 0;
        getNotificationCenter().postNotificationName(NotificationCenter.messageReceivedByAck, Integer.valueOf(msg_id));
    }

    private void updateMediaPaths(MessageObject newMsgObj, TLRPC.Message sentMessage, int newMsgId, String originalPath, boolean post) {
        String str;
        byte[] oldWaveform;
        File cacheFile2;
        String newKey;
        TLRPC.Message newMsg = newMsgObj.messageOwner;
        if (newMsg.media != null) {
            TLRPC.PhotoSize strippedOld = null;
            TLRPC.PhotoSize strippedNew = null;
            TLObject photoObject = null;
            if (newMsg.media.photo != null) {
                strippedOld = FileLoader.getClosestPhotoSizeWithSize(newMsg.media.photo.sizes, 40);
                if (sentMessage != null && sentMessage.media != null && sentMessage.media.photo != null) {
                    strippedNew = FileLoader.getClosestPhotoSizeWithSize(sentMessage.media.photo.sizes, 40);
                } else {
                    strippedNew = strippedOld;
                }
                photoObject = newMsg.media.photo;
            } else if (newMsg.media.document != null) {
                strippedOld = FileLoader.getClosestPhotoSizeWithSize(newMsg.media.document.thumbs, 40);
                if (sentMessage != null && sentMessage.media != null && sentMessage.media.document != null) {
                    strippedNew = FileLoader.getClosestPhotoSizeWithSize(sentMessage.media.document.thumbs, 40);
                } else {
                    strippedNew = strippedOld;
                }
                photoObject = newMsg.media.document;
            } else if (newMsg.media.webpage != null) {
                if (newMsg.media.webpage.photo != null) {
                    strippedOld = FileLoader.getClosestPhotoSizeWithSize(newMsg.media.webpage.photo.sizes, 40);
                    if (sentMessage != null && sentMessage.media != null && sentMessage.media.webpage != null && sentMessage.media.webpage.photo != null) {
                        strippedNew = FileLoader.getClosestPhotoSizeWithSize(sentMessage.media.webpage.photo.sizes, 40);
                    } else {
                        strippedNew = strippedOld;
                    }
                    photoObject = newMsg.media.webpage.photo;
                } else if (newMsg.media.webpage.document != null) {
                    strippedOld = FileLoader.getClosestPhotoSizeWithSize(newMsg.media.webpage.document.thumbs, 40);
                    if (sentMessage != null && sentMessage.media != null && sentMessage.media.webpage != null && sentMessage.media.webpage.document != null) {
                        strippedNew = FileLoader.getClosestPhotoSizeWithSize(sentMessage.media.webpage.document.thumbs, 40);
                    } else {
                        strippedNew = strippedOld;
                    }
                    photoObject = newMsg.media.webpage.document;
                }
            }
            if ((strippedNew instanceof TLRPC.TL_photoStrippedSize) && (strippedOld instanceof TLRPC.TL_photoStrippedSize)) {
                String oldKey = "stripped" + FileRefController.getKeyForParentObject(newMsgObj);
                if (sentMessage != null) {
                    newKey = "stripped" + FileRefController.getKeyForParentObject(sentMessage);
                } else {
                    newKey = "strippedmessage" + newMsgId + "_" + newMsgObj.getChannelId();
                }
                ImageLoader.getInstance().replaceImageInCache(oldKey, newKey, ImageLocation.getForObject(strippedNew, photoObject), post);
            }
        }
        if (sentMessage == null) {
            return;
        }
        long j = -2147483648L;
        if ((sentMessage.media instanceof TLRPC.TL_messageMediaPhoto) && sentMessage.media.photo != null && (newMsg.media instanceof TLRPC.TL_messageMediaPhoto) && newMsg.media.photo != null) {
            if (sentMessage.media.ttl_seconds == 0 && !newMsgObj.scheduled) {
                getMessagesStorage().putSentFile(originalPath, sentMessage.media.photo, 0, "sent_" + sentMessage.to_id.channel_id + "_" + sentMessage.id);
            }
            if (newMsg.media.photo.sizes.size() == 1 && (newMsg.media.photo.sizes.get(0).location instanceof TLRPC.TL_fileLocationUnavailable)) {
                newMsg.media.photo.sizes = sentMessage.media.photo.sizes;
            } else {
                int a2 = 0;
                while (a2 < sentMessage.media.photo.sizes.size()) {
                    TLRPC.PhotoSize size = sentMessage.media.photo.sizes.get(a2);
                    if (size != null && size.location != null && !(size instanceof TLRPC.TL_photoSizeEmpty) && size.type != null) {
                        int b = 0;
                        while (b < newMsg.media.photo.sizes.size()) {
                            TLRPC.PhotoSize size2 = newMsg.media.photo.sizes.get(b);
                            if (size2 == null || size2.location == null || size2.type == null || ((size2.location.volume_id != j || !size.type.equals(size2.type)) && (size.w != size2.w || size.h != size2.h))) {
                                b++;
                                j = -2147483648L;
                            } else {
                                String fileName = size2.location.volume_id + "_" + size2.location.local_id;
                                String fileName2 = size.location.volume_id + "_" + size.location.local_id;
                                if (!fileName.equals(fileName2)) {
                                    File cacheFile = new File(FileLoader.getDirectory(4), fileName + ".jpg");
                                    if (sentMessage.media.ttl_seconds == 0 && (sentMessage.media.photo.sizes.size() == 1 || size.w > 90 || size.h > 90)) {
                                        cacheFile2 = FileLoader.getPathToAttach(size);
                                    } else {
                                        cacheFile2 = new File(FileLoader.getDirectory(4), fileName2 + ".jpg");
                                    }
                                    cacheFile.renameTo(cacheFile2);
                                    ImageLoader.getInstance().replaceImageInCache(fileName, fileName2, ImageLocation.getForPhoto(size, sentMessage.media.photo), post);
                                    size2.location = size.location;
                                    size2.size = size.size;
                                }
                            }
                        }
                    }
                    a2++;
                    j = -2147483648L;
                }
            }
            sentMessage.message = newMsg.message;
            sentMessage.attachPath = newMsg.attachPath;
            newMsg.media.photo.id = sentMessage.media.photo.id;
            newMsg.media.photo.access_hash = sentMessage.media.photo.access_hash;
        } else if ((sentMessage.media instanceof TLRPC.TL_messageMediaDocument) && sentMessage.media.document != null && (newMsg.media instanceof TLRPC.TL_messageMediaDocument) && newMsg.media.document != null) {
            if (sentMessage.media.ttl_seconds != 0) {
                str = originalPath;
            } else {
                boolean isVideo = MessageObject.isVideoMessage(sentMessage);
                if (!isVideo && !MessageObject.isGifMessage(sentMessage)) {
                    str = originalPath;
                } else if (MessageObject.isGifDocument(sentMessage.media.document) != MessageObject.isGifDocument(newMsg.media.document)) {
                    str = originalPath;
                } else {
                    if (newMsgObj.scheduled) {
                        str = originalPath;
                    } else {
                        str = originalPath;
                        getMessagesStorage().putSentFile(str, sentMessage.media.document, 2, "sent_" + sentMessage.to_id.channel_id + "_" + sentMessage.id);
                    }
                    if (isVideo) {
                        sentMessage.attachPath = newMsg.attachPath;
                    }
                }
                if (!MessageObject.isVoiceMessage(sentMessage) && !MessageObject.isRoundVideoMessage(sentMessage) && !newMsgObj.scheduled) {
                    getMessagesStorage().putSentFile(str, sentMessage.media.document, 1, "sent_" + sentMessage.to_id.channel_id + "_" + sentMessage.id);
                }
            }
            TLRPC.PhotoSize size22 = FileLoader.getClosestPhotoSizeWithSize(newMsg.media.document.thumbs, 320);
            TLRPC.PhotoSize size3 = FileLoader.getClosestPhotoSizeWithSize(sentMessage.media.document.thumbs, 320);
            if (size22 != null && size22.location != null && size22.location.volume_id == -2147483648L && size3 != null && size3.location != null && !(size3 instanceof TLRPC.TL_photoSizeEmpty) && !(size22 instanceof TLRPC.TL_photoSizeEmpty)) {
                String fileName3 = size22.location.volume_id + "_" + size22.location.local_id;
                String fileName22 = size3.location.volume_id + "_" + size3.location.local_id;
                if (!fileName3.equals(fileName22)) {
                    new File(FileLoader.getDirectory(4), fileName3 + ".jpg").renameTo(new File(FileLoader.getDirectory(4), fileName22 + ".jpg"));
                    ImageLoader.getInstance().replaceImageInCache(fileName3, fileName22, ImageLocation.getForDocument(size3, sentMessage.media.document), post);
                    size22.location = size3.location;
                    size22.size = size3.size;
                }
            } else if (size22 != null && MessageObject.isStickerMessage(sentMessage) && size22.location != null) {
                size3.location = size22.location;
            } else if (size22 == null || ((size22 != null && (size22.location instanceof TLRPC.TL_fileLocationUnavailable)) || (size22 instanceof TLRPC.TL_photoSizeEmpty))) {
                newMsg.media.document.thumbs = sentMessage.media.document.thumbs;
            }
            newMsg.media.document.dc_id = sentMessage.media.document.dc_id;
            newMsg.media.document.id = sentMessage.media.document.id;
            newMsg.media.document.access_hash = sentMessage.media.document.access_hash;
            int a3 = 0;
            while (true) {
                if (a3 >= newMsg.media.document.attributes.size()) {
                    oldWaveform = null;
                    break;
                }
                TLRPC.DocumentAttribute attribute = newMsg.media.document.attributes.get(a3);
                if (!(attribute instanceof TLRPC.TL_documentAttributeAudio)) {
                    a3++;
                } else {
                    byte[] oldWaveform2 = attribute.waveform;
                    oldWaveform = oldWaveform2;
                    break;
                }
            }
            newMsg.media.document.attributes = sentMessage.media.document.attributes;
            if (oldWaveform != null) {
                for (int a4 = 0; a4 < newMsg.media.document.attributes.size(); a4++) {
                    TLRPC.DocumentAttribute attribute2 = newMsg.media.document.attributes.get(a4);
                    if (attribute2 instanceof TLRPC.TL_documentAttributeAudio) {
                        attribute2.waveform = oldWaveform;
                        attribute2.flags |= 4;
                    }
                }
            }
            newMsg.media.document.size = sentMessage.media.document.size;
            newMsg.media.document.mime_type = sentMessage.media.document.mime_type;
            if ((sentMessage.flags & 4) == 0 && MessageObject.isOut(sentMessage)) {
                if (MessageObject.isNewGifDocument(sentMessage.media.document)) {
                    getMediaDataController().addRecentGif(sentMessage.media.document, sentMessage.date);
                } else if (MessageObject.isStickerDocument(sentMessage.media.document) || MessageObject.isAnimatedStickerDocument(sentMessage.media.document)) {
                    getMediaDataController().addRecentSticker(0, sentMessage, sentMessage.media.document, sentMessage.date, false);
                }
            }
            if (newMsg.attachPath != null && newMsg.attachPath.startsWith(FileLoader.getDirectory(4).getAbsolutePath())) {
                File cacheFile3 = new File(newMsg.attachPath);
                File cacheFile22 = FileLoader.getPathToAttach(sentMessage.media.document, sentMessage.media.ttl_seconds != 0);
                if (!cacheFile3.renameTo(cacheFile22)) {
                    if (cacheFile3.exists()) {
                        sentMessage.attachPath = newMsg.attachPath;
                    } else {
                        newMsgObj.attachPathExists = false;
                    }
                    newMsgObj.mediaExists = cacheFile22.exists();
                    sentMessage.message = newMsg.message;
                    return;
                } else if (MessageObject.isVideoMessage(sentMessage)) {
                    newMsgObj.attachPathExists = true;
                    return;
                } else {
                    newMsgObj.mediaExists = newMsgObj.attachPathExists;
                    newMsgObj.attachPathExists = false;
                    newMsg.attachPath = "";
                    if (str != null && str.startsWith("http")) {
                        getMessagesStorage().addRecentLocalFile(str, cacheFile22.toString(), newMsg.media.document);
                        return;
                    }
                    return;
                }
            }
            sentMessage.attachPath = newMsg.attachPath;
            sentMessage.message = newMsg.message;
        } else if ((sentMessage.media instanceof TLRPC.TL_messageMediaContact) && (newMsg.media instanceof TLRPC.TL_messageMediaContact)) {
            newMsg.media = sentMessage.media;
        } else if (sentMessage.media instanceof TLRPC.TL_messageMediaWebPage) {
            newMsg.media = sentMessage.media;
        } else if (sentMessage.media instanceof TLRPC.TL_messageMediaGeo) {
            sentMessage.media.geo.lat = newMsg.media.geo.lat;
            sentMessage.media.geo._long = newMsg.media.geo._long;
        } else if (sentMessage.media instanceof TLRPC.TL_messageMediaGame) {
            newMsg.media = sentMessage.media;
            if ((newMsg.media instanceof TLRPC.TL_messageMediaGame) && !TextUtils.isEmpty(sentMessage.message)) {
                newMsg.entities = sentMessage.entities;
                newMsg.message = sentMessage.message;
            }
        } else if (sentMessage.media instanceof TLRPC.TL_messageMediaPoll) {
            newMsg.media = sentMessage.media;
        }
    }

    private void putToDelayedMessages(String location, DelayedMessage message) {
        ArrayList<DelayedMessage> arrayList = this.delayedMessages.get(location);
        if (arrayList == null) {
            arrayList = new ArrayList<>();
            this.delayedMessages.put(location, arrayList);
        }
        arrayList.add(message);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public ArrayList<DelayedMessage> getDelayedMessages(String location) {
        return this.delayedMessages.get(location);
    }

    public long getNextRandomId() {
        long val = 0;
        while (val == 0) {
            val = Utilities.random.nextLong();
        }
        return val;
    }

    public void checkUnsentMessages() {
        getMessagesStorage().getUnsentMessages(1000);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void processUnsentMessages(final ArrayList<TLRPC.Message> messages, final ArrayList<TLRPC.Message> scheduledMessages, final ArrayList<TLRPC.User> users, final ArrayList<TLRPC.Chat> chats, final ArrayList<TLRPC.EncryptedChat> encryptedChats) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.this.m979x3ec22d97(users, chats, encryptedChats, messages, scheduledMessages);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processUnsentMessages$47$im-uecahxyvjd-messenger-SendMessagesHelper  reason: not valid java name */
    public /* synthetic */ void m979x3ec22d97(ArrayList users, ArrayList chats, ArrayList encryptedChats, ArrayList messages, ArrayList scheduledMessages) {
        getMessagesController().putUsers(users, true);
        getMessagesController().putChats(chats, true);
        getMessagesController().putEncryptedChats(encryptedChats, true);
        for (int a2 = 0; a2 < messages.size(); a2++) {
            retrySendMessage(new MessageObject(this.currentAccount, (TLRPC.Message) messages.get(a2), false), true);
        }
        if (scheduledMessages != null) {
            for (int a3 = 0; a3 < scheduledMessages.size(); a3++) {
                MessageObject messageObject = new MessageObject(this.currentAccount, (TLRPC.Message) scheduledMessages.get(a3), false);
                messageObject.scheduled = true;
                retrySendMessage(messageObject, true);
            }
        }
    }

    public TLRPC.TL_photo generatePhotoSizes(String path, Uri imageUri, boolean blnOriginalImg) {
        return generatePhotoSizes(null, path, imageUri, blnOriginalImg);
    }

    public TLRPC.TL_photo generatePhotoSizes(TLRPC.TL_photo photo, String path, Uri imageUri, boolean blnOriginalImg) {
        Bitmap bitmap;
        boolean isPng;
        TLRPC.PhotoSize size;
        TLRPC.TL_photo photo2;
        Bitmap bitmap2 = ImageLoader.loadBitmap(path, imageUri, AndroidUtilities.getPhotoSize(), AndroidUtilities.getPhotoSize(), true);
        if (bitmap2 == null) {
            bitmap = ImageLoader.loadBitmap(path, imageUri, 800.0f, 800.0f, true);
        } else {
            bitmap = bitmap2;
        }
        if (path != null) {
            boolean isPng2 = path.endsWith(".png");
            isPng = isPng2;
        } else {
            isPng = false;
        }
        ArrayList<TLRPC.PhotoSize> sizes = new ArrayList<>();
        TLRPC.PhotoSize size2 = ImageLoader.scaleAndSaveImage(bitmap, 90.0f, 90.0f, 55, true, isPng);
        if (size2 != null) {
            sizes.add(size2);
        }
        if (blnOriginalImg) {
            try {
                size = ImageLoader.SaveImageWithOriginalInternal(null, path, false);
            } catch (Throwable e) {
                FileLog.e(e);
                ImageLoader.getInstance().clearMemory();
                System.gc();
                try {
                    size = ImageLoader.SaveImageWithOriginalInternal(null, path, false);
                } catch (Throwable e2) {
                    FileLog.e(e2);
                    size = null;
                }
            }
        } else {
            size = ImageLoader.scaleAndSaveImage(bitmap, AndroidUtilities.getPhotoSize(), AndroidUtilities.getPhotoSize(), 80, false, 101, 101, isPng);
        }
        if (size != null) {
            sizes.add(size);
        }
        if (bitmap != null) {
            bitmap.recycle();
        }
        if (sizes.isEmpty()) {
            return null;
        }
        getUserConfig().saveConfig(false);
        if (photo != null) {
            photo2 = photo;
        } else {
            photo2 = new TLRPC.TL_photo();
        }
        photo2.date = getConnectionsManager().getCurrentTime();
        photo2.sizes = sizes;
        photo2.file_reference = new byte[0];
        return photo2;
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    /* JADX WARN: Removed duplicated region for block: B:126:0x01ba  */
    /* JADX WARN: Removed duplicated region for block: B:137:0x01ee  */
    /* JADX WARN: Removed duplicated region for block: B:140:0x01f6  */
    /* JADX WARN: Removed duplicated region for block: B:146:0x0250  */
    /* JADX WARN: Removed duplicated region for block: B:173:0x0316  */
    /* JADX WARN: Removed duplicated region for block: B:290:0x0516  */
    /* JADX WARN: Removed duplicated region for block: B:292:0x0521  */
    /* JADX WARN: Removed duplicated region for block: B:293:0x0527  */
    /* JADX WARN: Removed duplicated region for block: B:296:0x0531  */
    /* JADX WARN: Removed duplicated region for block: B:297:0x0539  */
    /* JADX WARN: Removed duplicated region for block: B:299:0x053d  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private static boolean prepareSendingDocumentInternal(final AccountInstance accountInstance, String path, String originalPath, Uri uri, String mime, final long dialog_id, final MessageObject reply_to_msg, CharSequence caption, final ArrayList<TLRPC.MessageEntity> entities, final MessageObject editingMessageObject, boolean forceDocument, final boolean notify, final int scheduleDate) {
        String extension;
        String extension2;
        String ext;
        long j;
        String permormer;
        String title;
        boolean isVoice;
        int duration;
        TLRPC.TL_documentAttributeAudio attributeAudio;
        String name;
        Object obj;
        boolean sendNew;
        TLRPC.TL_document document;
        String originalPath2;
        String str;
        String title2;
        String originalPath3;
        boolean isEncrypted;
        MimeTypeMap myMime;
        String name2;
        long j2;
        Object obj2;
        String parentObject;
        String parentObject2;
        String name3;
        String str2;
        MimeTypeMap myMime2;
        boolean isEncrypted2;
        String originalPath4;
        char c;
        String str3;
        Throwable th;
        String originalPath5 = originalPath;
        if ((path == null || path.length() == 0) && uri == null) {
            return false;
        }
        if (uri == null || !AndroidUtilities.isInternalUri(uri)) {
            if (path == null || !AndroidUtilities.isInternalUri(Uri.fromFile(new File(path)))) {
                MimeTypeMap myMime3 = MimeTypeMap.getSingleton();
                if (uri != null) {
                    boolean hasExt = false;
                    String extension3 = mime != null ? myMime3.getExtensionFromMimeType(mime) : null;
                    if (extension3 == null) {
                        extension3 = "txt";
                    } else {
                        hasExt = true;
                    }
                    String path2 = MediaController.copyFileToCache(uri, extension3);
                    if (path2 == null) {
                        return false;
                    }
                    if (hasExt) {
                        extension = extension3;
                        extension2 = path2;
                    } else {
                        extension = null;
                        extension2 = path2;
                    }
                } else {
                    extension = null;
                    extension2 = path;
                }
                File f = new File(extension2);
                if (f.exists() && f.length() != 0) {
                    boolean isEncrypted3 = ((int) dialog_id) == 0;
                    boolean allowSticker = !isEncrypted3;
                    String name4 = f.getName();
                    if (extension != null) {
                        String ext2 = extension;
                        ext = ext2;
                    } else {
                        int idx = extension2.lastIndexOf(46);
                        if (idx != -1) {
                            String ext3 = extension2.substring(idx + 1);
                            ext = ext3;
                        } else {
                            ext = "";
                        }
                    }
                    String extL = ext.toLowerCase();
                    String permormer2 = null;
                    String title3 = null;
                    boolean isVoice2 = false;
                    int duration2 = 0;
                    if (extL.equals("mp3") || extL.equals("m4a")) {
                        AudioInfo audioInfo = AudioInfo.getAudioInfo(f);
                        if (audioInfo != null) {
                            j = 0;
                            if (audioInfo.getDuration() != 0) {
                                permormer2 = audioInfo.getArtist();
                                title3 = audioInfo.getTitle();
                            }
                        } else {
                            j = 0;
                        }
                        permormer = permormer2;
                        title = title3;
                        isVoice = false;
                        duration = 0;
                    } else if (extL.equals("opus") || extL.equals("ogg") || extL.equals("flac")) {
                        MediaMetadataRetriever mediaMetadataRetriever = null;
                        try {
                            try {
                                mediaMetadataRetriever = new MediaMetadataRetriever();
                                try {
                                    try {
                                        mediaMetadataRetriever.setDataSource(f.getAbsolutePath());
                                        String d = mediaMetadataRetriever.extractMetadata(9);
                                        if (d != null) {
                                            int duration3 = (int) Math.ceil(((float) Long.parseLong(d)) / 1000.0f);
                                            try {
                                                title3 = mediaMetadataRetriever.extractMetadata(7);
                                                duration2 = duration3;
                                                permormer2 = mediaMetadataRetriever.extractMetadata(2);
                                            } catch (Exception e) {
                                                e = e;
                                                duration2 = duration3;
                                                mediaMetadataRetriever = mediaMetadataRetriever;
                                                FileLog.e(e);
                                                if (mediaMetadataRetriever != null) {
                                                    try {
                                                        mediaMetadataRetriever.release();
                                                    } catch (Exception e2) {
                                                        FileLog.e(e2);
                                                    }
                                                }
                                                permormer = permormer2;
                                                title = title3;
                                                isVoice = isVoice2;
                                                duration = duration2;
                                                j = 0;
                                                if (duration == 0) {
                                                }
                                                if (originalPath5 != null) {
                                                }
                                                document = null;
                                                String parentObject3 = null;
                                                if (sendNew) {
                                                }
                                                originalPath2 = originalPath5;
                                                str = "";
                                                title2 = "opus";
                                                originalPath3 = "m4a";
                                                isEncrypted = isEncrypted3;
                                                myMime = myMime3;
                                                name2 = name;
                                                j2 = j;
                                                obj2 = obj;
                                                parentObject = "mp3";
                                                parentObject2 = null;
                                                if (document != null) {
                                                }
                                                if (caption == null) {
                                                }
                                                final HashMap<String, String> params = new HashMap<>();
                                                if (originalPath2 == null) {
                                                }
                                                if (forceDocument) {
                                                }
                                                final TLRPC.TL_document documentFinal = document;
                                                final String pathFinal = extension2;
                                                final String parentFinal = parentObject2;
                                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda0
                                                    @Override // java.lang.Runnable
                                                    public final void run() {
                                                        SendMessagesHelper.lambda$prepareSendingDocumentInternal$48(MessageObject.this, accountInstance, documentFinal, pathFinal, params, parentFinal, dialog_id, reply_to_msg, captionFinal, entities, notify, scheduleDate);
                                                    }
                                                });
                                                return true;
                                            } catch (Throwable th2) {
                                                mediaMetadataRetriever = mediaMetadataRetriever;
                                                th = th2;
                                                if (mediaMetadataRetriever != null) {
                                                    try {
                                                        mediaMetadataRetriever.release();
                                                    } catch (Exception e3) {
                                                        FileLog.e(e3);
                                                    }
                                                }
                                                throw th;
                                            }
                                        }
                                        if (editingMessageObject == null && extL.equals("ogg")) {
                                            if (MediaController.isOpusFile(f.getAbsolutePath()) == 1) {
                                                isVoice2 = true;
                                            }
                                        }
                                        try {
                                            mediaMetadataRetriever.release();
                                        } catch (Exception e4) {
                                            FileLog.e(e4);
                                        }
                                    } catch (Exception e5) {
                                        e = e5;
                                        mediaMetadataRetriever = mediaMetadataRetriever;
                                    } catch (Throwable th3) {
                                        th = th3;
                                        mediaMetadataRetriever = mediaMetadataRetriever;
                                    }
                                } catch (Exception e6) {
                                    e = e6;
                                } catch (Throwable th4) {
                                    th = th4;
                                }
                            } catch (Throwable th5) {
                                th = th5;
                            }
                        } catch (Exception e7) {
                            e = e7;
                        }
                        permormer = permormer2;
                        title = title3;
                        isVoice = isVoice2;
                        duration = duration2;
                        j = 0;
                    } else {
                        permormer = null;
                        title = null;
                        isVoice = false;
                        duration = 0;
                        j = 0;
                    }
                    if (duration == 0) {
                        TLRPC.TL_documentAttributeAudio attributeAudio2 = new TLRPC.TL_documentAttributeAudio();
                        attributeAudio2.duration = duration;
                        attributeAudio2.title = title;
                        attributeAudio2.performer = permormer;
                        if (attributeAudio2.title == null) {
                            attributeAudio2.title = "";
                        }
                        attributeAudio2.flags |= 1;
                        if (attributeAudio2.performer == null) {
                            attributeAudio2.performer = "";
                        }
                        attributeAudio2.flags |= 2;
                        if (isVoice) {
                            attributeAudio2.voice = true;
                        }
                        attributeAudio = attributeAudio2;
                    } else {
                        attributeAudio = null;
                    }
                    if (originalPath5 != null) {
                        name = name4;
                        obj = "flac";
                        sendNew = false;
                    } else if (originalPath5.endsWith("attheme")) {
                        sendNew = true;
                        name = name4;
                        obj = "flac";
                    } else if (attributeAudio != null) {
                        name = name4;
                        obj = "flac";
                        originalPath5 = originalPath5 + MimeTypes.BASE_TYPE_AUDIO + f.length();
                        sendNew = false;
                    } else {
                        name = name4;
                        obj = "flac";
                        originalPath5 = originalPath5 + "" + f.length();
                        sendNew = false;
                    }
                    document = null;
                    String parentObject32 = null;
                    if (!sendNew || isEncrypted3) {
                        originalPath2 = originalPath5;
                        str = "";
                        title2 = "opus";
                        originalPath3 = "m4a";
                        isEncrypted = isEncrypted3;
                        myMime = myMime3;
                        name2 = name;
                        j2 = j;
                        obj2 = obj;
                        parentObject = "mp3";
                        parentObject2 = null;
                    } else {
                        Object[] sentData = accountInstance.getMessagesStorage().getSentFile(originalPath5, !isEncrypted3 ? 1 : 4);
                        if (sentData != null) {
                            document = (TLRPC.TL_document) sentData[0];
                            parentObject32 = (String) sentData[1];
                        }
                        if (document != null || extension2.equals(originalPath5) || isEncrypted3) {
                            str3 = "";
                            document = document;
                        } else {
                            TLRPC.TL_document document2 = document;
                            str3 = "";
                            Object[] sentData2 = accountInstance.getMessagesStorage().getSentFile(extension2 + f.length(), !isEncrypted3 ? 1 : 4);
                            if (sentData2 != null) {
                                document = (TLRPC.TL_document) sentData2[0];
                                parentObject32 = (String) sentData2[1];
                            } else {
                                document = document2;
                            }
                        }
                        name2 = name;
                        str = str3;
                        obj2 = obj;
                        String parentObject4 = parentObject32;
                        title2 = "opus";
                        parentObject = "mp3";
                        originalPath2 = originalPath5;
                        originalPath3 = "m4a";
                        isEncrypted = isEncrypted3;
                        myMime = myMime3;
                        j2 = j;
                        ensureMediaThumbExists(isEncrypted3, document, extension2, null, 0L);
                        parentObject2 = parentObject4;
                    }
                    if (document != null) {
                        TLRPC.TL_document document3 = new TLRPC.TL_document();
                        document3.id = j2;
                        document3.date = accountInstance.getConnectionsManager().getCurrentTime();
                        TLRPC.TL_documentAttributeFilename fileName = new TLRPC.TL_documentAttributeFilename();
                        name3 = name2;
                        fileName.file_name = name3;
                        document3.file_reference = new byte[0];
                        document3.attributes.add(fileName);
                        document3.size = (int) f.length();
                        document3.dc_id = 0;
                        if (attributeAudio != null) {
                            document3.attributes.add(attributeAudio);
                        }
                        if (ext.length() != 0) {
                            switch (extL.hashCode()) {
                                case 106458:
                                    if (extL.equals(originalPath3)) {
                                        c = 3;
                                        break;
                                    }
                                    c = 65535;
                                    break;
                                case 108272:
                                    if (extL.equals(parentObject)) {
                                        c = 2;
                                        break;
                                    }
                                    c = 65535;
                                    break;
                                case 109967:
                                    if (extL.equals("ogg")) {
                                        c = 4;
                                        break;
                                    }
                                    c = 65535;
                                    break;
                                case 3145576:
                                    if (extL.equals(obj2)) {
                                        c = 5;
                                        break;
                                    }
                                    c = 65535;
                                    break;
                                case 3418175:
                                    if (extL.equals(title2)) {
                                        c = 1;
                                        break;
                                    }
                                    c = 65535;
                                    break;
                                case 3645340:
                                    if (extL.equals("webp")) {
                                        c = 0;
                                        break;
                                    }
                                    c = 65535;
                                    break;
                                default:
                                    c = 65535;
                                    break;
                            }
                            switch (c) {
                                case 0:
                                    document3.mime_type = "image/webp";
                                    myMime2 = myMime;
                                    break;
                                case 1:
                                    document3.mime_type = MimeTypes.AUDIO_OPUS;
                                    myMime2 = myMime;
                                    break;
                                case 2:
                                    document3.mime_type = MimeTypes.AUDIO_MPEG;
                                    myMime2 = myMime;
                                    break;
                                case 3:
                                    document3.mime_type = "audio/m4a";
                                    myMime2 = myMime;
                                    break;
                                case 4:
                                    document3.mime_type = "audio/ogg";
                                    myMime2 = myMime;
                                    break;
                                case 5:
                                    document3.mime_type = MimeTypes.AUDIO_FLAC;
                                    myMime2 = myMime;
                                    break;
                                default:
                                    myMime2 = myMime;
                                    String mimeType = myMime2.getMimeTypeFromExtension(extL);
                                    if (mimeType == null) {
                                        document3.mime_type = "application/octet-stream";
                                        break;
                                    } else {
                                        document3.mime_type = mimeType;
                                        break;
                                    }
                            }
                        } else {
                            myMime2 = myMime;
                            document3.mime_type = "application/octet-stream";
                        }
                        if (!document3.mime_type.equals("image/gif")) {
                            isEncrypted2 = isEncrypted;
                        } else if (editingMessageObject == null || editingMessageObject.getGroupIdForUse() == j2) {
                            try {
                                Bitmap bitmap = ImageLoader.loadBitmap(f.getAbsolutePath(), null, 90.0f, 90.0f, true);
                                if (bitmap != null) {
                                    fileName.file_name = "animation.gif";
                                    document3.attributes.add(new TLRPC.TL_documentAttributeAnimated());
                                    isEncrypted2 = isEncrypted;
                                    try {
                                        TLRPC.PhotoSize thumb = ImageLoader.scaleAndSaveImage(bitmap, 90.0f, 90.0f, 55, isEncrypted2);
                                        if (thumb != null) {
                                            document3.thumbs.add(thumb);
                                            document3.flags |= 1;
                                        }
                                        bitmap.recycle();
                                    } catch (Exception e8) {
                                        e = e8;
                                        FileLog.e(e);
                                        if (!document3.mime_type.equals("image/webp")) {
                                        }
                                        str2 = str;
                                        if (!document3.mime_type.startsWith("image/")) {
                                        }
                                        document = document3;
                                        if (caption == null) {
                                        }
                                        final HashMap params2 = new HashMap<>();
                                        if (originalPath2 == null) {
                                        }
                                        if (forceDocument) {
                                        }
                                        final TLRPC.TL_document documentFinal2 = document;
                                        final String pathFinal2 = extension2;
                                        final String parentFinal2 = parentObject2;
                                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda0
                                            @Override // java.lang.Runnable
                                            public final void run() {
                                                SendMessagesHelper.lambda$prepareSendingDocumentInternal$48(MessageObject.this, accountInstance, documentFinal2, pathFinal2, params2, parentFinal2, dialog_id, reply_to_msg, captionFinal, entities, notify, scheduleDate);
                                            }
                                        });
                                        return true;
                                    }
                                } else {
                                    isEncrypted2 = isEncrypted;
                                }
                            } catch (Exception e9) {
                                e = e9;
                                isEncrypted2 = isEncrypted;
                            }
                        } else {
                            isEncrypted2 = isEncrypted;
                        }
                        if (!document3.mime_type.equals("image/webp") && allowSticker && editingMessageObject == null) {
                            BitmapFactory.Options bmOptions = new BitmapFactory.Options();
                            try {
                                bmOptions.inJustDecodeBounds = true;
                                RandomAccessFile file = new RandomAccessFile(extension2, "r");
                                ByteBuffer buffer = file.getChannel().map(FileChannel.MapMode.READ_ONLY, 0L, extension2.length());
                                Utilities.loadWebpImage(null, buffer, buffer.limit(), bmOptions, true);
                                file.close();
                            } catch (Exception e10) {
                                FileLog.e(e10);
                            }
                            if (bmOptions.outWidth == 0 || bmOptions.outHeight == 0 || bmOptions.outWidth > 800 || bmOptions.outHeight > 800) {
                                str2 = str;
                            } else {
                                TLRPC.TL_documentAttributeSticker attributeSticker = new TLRPC.TL_documentAttributeSticker();
                                str2 = str;
                                attributeSticker.alt = str2;
                                attributeSticker.stickerset = new TLRPC.TL_inputStickerSetEmpty();
                                document3.attributes.add(attributeSticker);
                                TLRPC.TL_documentAttributeImageSize attributeImageSize = new TLRPC.TL_documentAttributeImageSize();
                                attributeImageSize.w = bmOptions.outWidth;
                                attributeImageSize.h = bmOptions.outHeight;
                                document3.attributes.add(attributeImageSize);
                            }
                        } else {
                            str2 = str;
                        }
                        if (!document3.mime_type.startsWith("image/") && !document3.mime_type.equals("image/gif") && !document3.mime_type.equals("image/webp")) {
                            if (editingMessageObject == null || editingMessageObject.getGroupIdForUse() == 0) {
                                try {
                                } catch (Exception e11) {
                                    e = e11;
                                }
                                try {
                                    Bitmap bitmap2 = ImageLoader.loadBitmap(f.getAbsolutePath(), null, 90.0f, 90.0f, true);
                                    if (bitmap2 != null) {
                                        TLRPC.PhotoSize thumb2 = ImageLoader.scaleAndSaveImage(bitmap2, 90.0f, 90.0f, 55, isEncrypted2, document3.mime_type.equals("image/png"));
                                        if (thumb2 != null) {
                                            document3.thumbs.add(thumb2);
                                            document3.flags |= 1;
                                        }
                                        bitmap2.recycle();
                                    }
                                } catch (Exception e12) {
                                    e = e12;
                                    FileLog.e(e);
                                    document = document3;
                                    if (caption == null) {
                                    }
                                    final HashMap params22 = new HashMap<>();
                                    if (originalPath2 == null) {
                                    }
                                    if (forceDocument) {
                                    }
                                    final TLRPC.TL_document documentFinal22 = document;
                                    final String pathFinal22 = extension2;
                                    final String parentFinal22 = parentObject2;
                                    AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda0
                                        @Override // java.lang.Runnable
                                        public final void run() {
                                            SendMessagesHelper.lambda$prepareSendingDocumentInternal$48(MessageObject.this, accountInstance, documentFinal22, pathFinal22, params22, parentFinal22, dialog_id, reply_to_msg, captionFinal, entities, notify, scheduleDate);
                                        }
                                    });
                                    return true;
                                }
                            }
                        }
                        document = document3;
                    } else {
                        name3 = name2;
                        str2 = str;
                        myMime2 = myMime;
                        isEncrypted2 = isEncrypted;
                    }
                    final String captionFinal = caption == null ? caption.toString() : str2;
                    final HashMap params222 = new HashMap<>();
                    if (originalPath2 == null) {
                        originalPath4 = originalPath2;
                        params222.put("originalPath", originalPath4);
                    } else {
                        originalPath4 = originalPath2;
                    }
                    if (forceDocument) {
                        params222.put("forceDocument", "1");
                    }
                    final TLRPC.TL_document documentFinal222 = document;
                    final String pathFinal222 = extension2;
                    final String parentFinal222 = parentObject2;
                    AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda0
                        @Override // java.lang.Runnable
                        public final void run() {
                            SendMessagesHelper.lambda$prepareSendingDocumentInternal$48(MessageObject.this, accountInstance, documentFinal222, pathFinal222, params222, parentFinal222, dialog_id, reply_to_msg, captionFinal, entities, notify, scheduleDate);
                        }
                    });
                    return true;
                }
                return false;
            }
            return false;
        }
        return false;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingDocumentInternal$48(MessageObject editingMessageObject, AccountInstance accountInstance, TLRPC.TL_document documentFinal, String pathFinal, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, String captionFinal, ArrayList entities, boolean notify, int scheduleDate) {
        if (editingMessageObject != null) {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, null, null, documentFinal, pathFinal, params, false, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().sendMessage(documentFinal, null, pathFinal, dialog_id, reply_to_msg, captionFinal, entities, null, params, notify, scheduleDate, 0, parentFinal);
        }
    }

    public static void prepareSendingDocument(AccountInstance accountInstance, String path, String originalPath, Uri uri, String caption, String mine, long dialog_id, MessageObject reply_to_msg, InputContentInfoCompat inputContent, MessageObject editingMessageObject, boolean notify, int scheduleDate) {
        ArrayList<Uri> uris;
        if ((path == null || originalPath == null) && uri == null) {
            return;
        }
        ArrayList<String> paths = new ArrayList<>();
        ArrayList<String> originalPaths = new ArrayList<>();
        if (uri == null) {
            uris = null;
        } else {
            ArrayList<Uri> uris2 = new ArrayList<>();
            uris2.add(uri);
            uris = uris2;
        }
        if (path != null) {
            paths.add(path);
            originalPaths.add(originalPath);
        }
        prepareSendingDocuments(accountInstance, paths, originalPaths, uris, caption, mine, dialog_id, reply_to_msg, inputContent, editingMessageObject, notify, scheduleDate);
    }

    public static void prepareSendingAudioDocuments(final AccountInstance accountInstance, final ArrayList<MessageObject> messageObjects, final long dialog_id, final MessageObject reply_to_msg, final MessageObject editingMessageObject, final boolean notify, final int scheduleDate) {
        new Thread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda65
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.lambda$prepareSendingAudioDocuments$50(messageObjects, dialog_id, accountInstance, editingMessageObject, reply_to_msg, notify, scheduleDate);
            }
        }).start();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:23:0x0075  */
    /* JADX WARN: Removed duplicated region for block: B:24:0x0081  */
    /* JADX WARN: Removed duplicated region for block: B:26:0x0085  */
    /* JADX WARN: Removed duplicated region for block: B:31:0x00a1  */
    /* JADX WARN: Removed duplicated region for block: B:37:0x00a6 A[SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static /* synthetic */ void lambda$prepareSendingAudioDocuments$50(ArrayList messageObjects, final long dialog_id, final AccountInstance accountInstance, final MessageObject editingMessageObject, final MessageObject reply_to_msg, final boolean notify, final int scheduleDate) {
        String originalPath;
        String parentObject;
        TLRPC.TL_document document;
        int size = messageObjects.size();
        for (int a2 = 0; a2 < size; a2++) {
            final MessageObject messageObject = (MessageObject) messageObjects.get(a2);
            String originalPath2 = messageObject.messageOwner.attachPath;
            File f = new File(originalPath2);
            boolean isEncrypted = ((int) dialog_id) == 0;
            if (originalPath2 == null) {
                originalPath = originalPath2;
            } else {
                originalPath = originalPath2 + MimeTypes.BASE_TYPE_AUDIO + f.length();
            }
            TLRPC.TL_document document2 = null;
            if (!isEncrypted) {
                Object[] sentData = accountInstance.getMessagesStorage().getSentFile(originalPath, !isEncrypted ? 1 : 4);
                if (sentData != null) {
                    document2 = (TLRPC.TL_document) sentData[0];
                    String parentObject2 = (String) sentData[1];
                    ensureMediaThumbExists(isEncrypted, document2, originalPath, null, 0L);
                    parentObject = parentObject2;
                    if (document2 == null) {
                        document = document2;
                    } else {
                        TLRPC.TL_document document3 = (TLRPC.TL_document) messageObject.messageOwner.media.document;
                        document = document3;
                    }
                    if (isEncrypted) {
                        int high_id = (int) (dialog_id >> 32);
                        TLRPC.EncryptedChat encryptedChat = accountInstance.getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
                        if (encryptedChat == null) {
                            return;
                        }
                    }
                    final HashMap<String, String> params = new HashMap<>();
                    if (originalPath == null) {
                        params.put("originalPath", originalPath);
                    }
                    final TLRPC.TL_document documentFinal = document;
                    final String parentFinal = parentObject;
                    AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda61
                        @Override // java.lang.Runnable
                        public final void run() {
                            SendMessagesHelper.lambda$prepareSendingAudioDocuments$49(MessageObject.this, accountInstance, documentFinal, messageObject, params, parentFinal, dialog_id, reply_to_msg, notify, scheduleDate);
                        }
                    });
                }
            }
            parentObject = null;
            if (document2 == null) {
            }
            if (isEncrypted) {
            }
            final HashMap params2 = new HashMap<>();
            if (originalPath == null) {
            }
            final TLRPC.TL_document documentFinal2 = document;
            final String parentFinal2 = parentObject;
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda61
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.lambda$prepareSendingAudioDocuments$49(MessageObject.this, accountInstance, documentFinal2, messageObject, params2, parentFinal2, dialog_id, reply_to_msg, notify, scheduleDate);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingAudioDocuments$49(MessageObject editingMessageObject, AccountInstance accountInstance, TLRPC.TL_document documentFinal, MessageObject messageObject, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, boolean notify, int scheduleDate) {
        if (editingMessageObject != null) {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, null, null, documentFinal, messageObject.messageOwner.attachPath, params, false, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().sendMessage(documentFinal, null, messageObject.messageOwner.attachPath, dialog_id, reply_to_msg, null, null, null, params, notify, scheduleDate, 0, parentFinal);
        }
    }

    public static void prepareSendingDocuments(final AccountInstance accountInstance, final ArrayList<String> paths, final ArrayList<String> originalPaths, final ArrayList<Uri> uris, final String caption, final String mime, final long dialog_id, final MessageObject reply_to_msg, final InputContentInfoCompat inputContent, final MessageObject editingMessageObject, final boolean notify, final int scheduleDate) {
        if (paths == null && originalPaths == null && uris == null) {
            return;
        }
        if (paths != null && originalPaths != null && paths.size() != originalPaths.size()) {
            return;
        }
        new Thread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda51
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.lambda$prepareSendingDocuments$51(paths, accountInstance, originalPaths, mime, dialog_id, reply_to_msg, caption, editingMessageObject, notify, scheduleDate, uris, inputContent);
            }
        }).start();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingDocuments$51(ArrayList paths, AccountInstance accountInstance, ArrayList originalPaths, String mime, long dialog_id, MessageObject reply_to_msg, String caption, MessageObject editingMessageObject, boolean notify, int scheduleDate, ArrayList uris, InputContentInfoCompat inputContent) {
        boolean error = false;
        if (paths != null) {
            for (int a2 = 0; a2 < paths.size(); a2++) {
                if (!prepareSendingDocumentInternal(accountInstance, (String) paths.get(a2), (String) originalPaths.get(a2), null, mime, dialog_id, reply_to_msg, caption, null, editingMessageObject, false, notify, scheduleDate)) {
                    error = true;
                }
            }
        }
        if (uris != null) {
            for (int a3 = 0; a3 < uris.size(); a3++) {
                if (!prepareSendingDocumentInternal(accountInstance, null, null, (Uri) uris.get(a3), mime, dialog_id, reply_to_msg, caption, null, editingMessageObject, false, notify, scheduleDate)) {
                    error = true;
                }
            }
        }
        if (inputContent != null) {
            inputContent.releasePermission();
        }
        if (error) {
            ToastUtils.show((int) R.string.UnsupportedAttachment);
        }
    }

    public static void prepareSendingPhoto(AccountInstance accountInstance, String imageFilePath, Uri imageUri, long dialog_id, MessageObject reply_to_msg, CharSequence caption, ArrayList<TLRPC.MessageEntity> entities, ArrayList<TLRPC.InputDocument> stickers, InputContentInfoCompat inputContent, int ttl, MessageObject editingMessageObject, boolean notify, int scheduleDate) {
        SendingMediaInfo info = new SendingMediaInfo();
        info.path = imageFilePath;
        info.uri = imageUri;
        if (caption != null) {
            info.caption = caption.toString();
        }
        info.entities = entities;
        info.ttl = ttl;
        if (stickers != null && !stickers.isEmpty()) {
            info.masks = new ArrayList<>(stickers);
        }
        ArrayList<SendingMediaInfo> infos = new ArrayList<>();
        infos.add(info);
        prepareSendingMedia(accountInstance, infos, dialog_id, reply_to_msg, inputContent, false, false, editingMessageObject, notify, scheduleDate, false);
    }

    public static void prepareSendingBotContextResult(final AccountInstance accountInstance, final TLRPC.BotInlineResult result, final HashMap<String, String> params, final long dialog_id, final MessageObject reply_to_msg, final boolean notify, final int scheduleDate) {
        if (result == null) {
            return;
        }
        if (result.send_message instanceof TLRPC.TL_botInlineMessageMediaAuto) {
            new Thread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda26
                @Override // java.lang.Runnable
                public final void run() {
                    SendMessagesHelper.lambda$prepareSendingBotContextResult$53(dialog_id, result, accountInstance, params, reply_to_msg, notify, scheduleDate);
                }
            }).run();
        } else if (!(result.send_message instanceof TLRPC.TL_botInlineMessageText)) {
            if (!(result.send_message instanceof TLRPC.TL_botInlineMessageMediaVenue)) {
                if (result.send_message instanceof TLRPC.TL_botInlineMessageMediaGeo) {
                    if (result.send_message.period != 0) {
                        TLRPC.TL_messageMediaGeoLive location = new TLRPC.TL_messageMediaGeoLive();
                        location.period = result.send_message.period;
                        location.geo = result.send_message.geo;
                        accountInstance.getSendMessagesHelper().sendMessage(location, dialog_id, reply_to_msg, result.send_message.reply_markup, params, notify, scheduleDate);
                        return;
                    }
                    TLRPC.TL_messageMediaGeo location2 = new TLRPC.TL_messageMediaGeo();
                    location2.geo = result.send_message.geo;
                    accountInstance.getSendMessagesHelper().sendMessage(location2, dialog_id, reply_to_msg, result.send_message.reply_markup, params, notify, scheduleDate);
                    return;
                } else if (result.send_message instanceof TLRPC.TL_botInlineMessageMediaContact) {
                    TLRPC.User user = new TLRPC.TL_user();
                    user.phone = result.send_message.phone_number;
                    user.first_name = result.send_message.first_name;
                    user.last_name = result.send_message.last_name;
                    TLRPC.TL_restrictionReason reason = new TLRPC.TL_restrictionReason();
                    reason.text = result.send_message.vcard;
                    reason.platform = "";
                    reason.reason = "";
                    user.restriction_reason.add(reason);
                    accountInstance.getSendMessagesHelper().sendMessage(user, dialog_id, reply_to_msg, result.send_message.reply_markup, params, notify, scheduleDate);
                    return;
                } else {
                    return;
                }
            }
            TLRPC.TL_messageMediaVenue venue = new TLRPC.TL_messageMediaVenue();
            venue.geo = result.send_message.geo;
            venue.address = result.send_message.address;
            venue.title = result.send_message.title;
            venue.provider = result.send_message.provider;
            venue.venue_id = result.send_message.venue_id;
            String str = result.send_message.venue_type;
            venue.venue_id = str;
            venue.venue_type = str;
            if (venue.venue_type == null) {
                venue.venue_type = "";
            }
            accountInstance.getSendMessagesHelper().sendMessage(venue, dialog_id, reply_to_msg, result.send_message.reply_markup, params, notify, scheduleDate);
        } else {
            TLRPC.WebPage webPage = null;
            if (((int) dialog_id) == 0) {
                int a2 = 0;
                while (true) {
                    if (a2 >= result.send_message.entities.size()) {
                        break;
                    }
                    TLRPC.MessageEntity entity = result.send_message.entities.get(a2);
                    if (!(entity instanceof TLRPC.TL_messageEntityUrl)) {
                        a2++;
                    } else {
                        webPage = new TLRPC.TL_webPagePending();
                        webPage.url = result.send_message.message.substring(entity.offset, entity.offset + entity.length);
                        break;
                    }
                }
            }
            accountInstance.getSendMessagesHelper().sendMessage(result.send_message.message, dialog_id, reply_to_msg, webPage, !result.send_message.no_webpage, result.send_message.entities, result.send_message.reply_markup, params, notify, scheduleDate);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    /* JADX WARN: Code restructure failed: missing block: B:93:0x0211, code lost:
        if (r6.equals("sticker") != false) goto L64;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static /* synthetic */ void lambda$prepareSendingBotContextResult$53(final long dialog_id, final TLRPC.BotInlineResult result, final AccountInstance accountInstance, final HashMap params, final MessageObject reply_to_msg, final boolean notify, final int scheduleDate) {
        TLRPC.TL_document document;
        TLRPC.TL_photo photo;
        TLRPC.TL_game game;
        String finalPath;
        char c;
        Bitmap bitmap;
        TLRPC.TL_photo photo2;
        boolean isEncrypted = ((int) dialog_id) == 0;
        String finalPath2 = null;
        if ("game".equals(result.type)) {
            if (((int) dialog_id) == 0) {
                return;
            }
            TLRPC.TL_game game2 = new TLRPC.TL_game();
            game2.title = result.title;
            game2.description = result.description;
            game2.short_name = result.id;
            game2.photo = result.photo;
            if (game2.photo == null) {
                game2.photo = new TLRPC.TL_photoEmpty();
            }
            if (result.document instanceof TLRPC.TL_document) {
                game2.document = result.document;
                game2.flags |= 1;
            }
            document = null;
            photo = null;
            game = game2;
        } else if (result instanceof TLRPC.TL_botInlineMediaResult) {
            if (result.document != null) {
                if (!(result.document instanceof TLRPC.TL_document)) {
                    document = null;
                    photo = null;
                    game = null;
                } else {
                    TLRPC.TL_document document2 = (TLRPC.TL_document) result.document;
                    document = document2;
                    photo = null;
                    game = null;
                }
            } else if (result.photo == null) {
                document = null;
                photo = null;
                game = null;
            } else if (!(result.photo instanceof TLRPC.TL_photo)) {
                document = null;
                photo = null;
                game = null;
            } else {
                TLRPC.TL_photo photo3 = (TLRPC.TL_photo) result.photo;
                document = null;
                photo = photo3;
                game = null;
            }
        } else if (result.content == null) {
            document = null;
            photo = null;
            game = null;
        } else {
            File f = new File(FileLoader.getDirectory(4), Utilities.MD5(result.content.url) + "." + ImageLoader.getHttpUrlExtension(result.content.url, "file"));
            if (f.exists()) {
                finalPath = f.getAbsolutePath();
            } else {
                finalPath = result.content.url;
            }
            String finalPath3 = result.type;
            char c2 = 5;
            document = null;
            photo = null;
            game = null;
            switch (finalPath3.hashCode()) {
                case -1890252483:
                    if (finalPath3.equals("sticker")) {
                        c = 4;
                        break;
                    }
                    c = 65535;
                    break;
                case 102340:
                    if (finalPath3.equals("gif")) {
                        c = 5;
                        break;
                    }
                    c = 65535;
                    break;
                case 3143036:
                    if (finalPath3.equals("file")) {
                        c = 2;
                        break;
                    }
                    c = 65535;
                    break;
                case 93166550:
                    if (finalPath3.equals(MimeTypes.BASE_TYPE_AUDIO)) {
                        c = 0;
                        break;
                    }
                    c = 65535;
                    break;
                case 106642994:
                    if (finalPath3.equals("photo")) {
                        c = 6;
                        break;
                    }
                    c = 65535;
                    break;
                case 112202875:
                    if (finalPath3.equals(MimeTypes.BASE_TYPE_VIDEO)) {
                        c = 3;
                        break;
                    }
                    c = 65535;
                    break;
                case 112386354:
                    if (finalPath3.equals("voice")) {
                        c = 1;
                        break;
                    }
                    c = 65535;
                    break;
                default:
                    c = 65535;
                    break;
            }
            switch (c) {
                case 0:
                case 1:
                case 2:
                case 3:
                case 4:
                case 5:
                    TLRPC.TL_document document3 = new TLRPC.TL_document();
                    document3.id = 0L;
                    document3.size = 0;
                    document3.dc_id = 0;
                    document3.mime_type = result.content.mime_type;
                    document3.file_reference = new byte[0];
                    document3.date = accountInstance.getConnectionsManager().getCurrentTime();
                    TLRPC.TL_documentAttributeFilename fileName = new TLRPC.TL_documentAttributeFilename();
                    document3.attributes.add(fileName);
                    String str = result.type;
                    switch (str.hashCode()) {
                        case -1890252483:
                            break;
                        case 102340:
                            if (str.equals("gif")) {
                                c2 = 0;
                                break;
                            }
                            c2 = 65535;
                            break;
                        case 3143036:
                            if (str.equals("file")) {
                                c2 = 3;
                                break;
                            }
                            c2 = 65535;
                            break;
                        case 93166550:
                            if (str.equals(MimeTypes.BASE_TYPE_AUDIO)) {
                                c2 = 2;
                                break;
                            }
                            c2 = 65535;
                            break;
                        case 112202875:
                            if (str.equals(MimeTypes.BASE_TYPE_VIDEO)) {
                                c2 = 4;
                                break;
                            }
                            c2 = 65535;
                            break;
                        case 112386354:
                            if (str.equals("voice")) {
                                c2 = 1;
                                break;
                            }
                            c2 = 65535;
                            break;
                        default:
                            c2 = 65535;
                            break;
                    }
                    switch (c2) {
                        case 0:
                            fileName.file_name = "animation.gif";
                            if (finalPath.endsWith("mp4")) {
                                document3.mime_type = MimeTypes.VIDEO_MP4;
                                document3.attributes.add(new TLRPC.TL_documentAttributeAnimated());
                            } else {
                                document3.mime_type = "image/gif";
                            }
                            int side = isEncrypted ? 90 : 320;
                            try {
                                if (finalPath.endsWith("mp4")) {
                                    bitmap = ThumbnailUtils.createVideoThumbnail(finalPath, 1);
                                } else {
                                    bitmap = ImageLoader.loadBitmap(finalPath, null, side, side, true);
                                }
                                if (bitmap != null) {
                                    TLRPC.PhotoSize thumb = ImageLoader.scaleAndSaveImage(bitmap, side, side, side > 90 ? 80 : 55, false);
                                    if (thumb != null) {
                                        document3.thumbs.add(thumb);
                                        document3.flags |= 1;
                                    }
                                    bitmap.recycle();
                                    break;
                                }
                            } catch (Throwable e) {
                                FileLog.e(e);
                                break;
                            }
                            break;
                        case 1:
                            TLRPC.TL_documentAttributeAudio audio = new TLRPC.TL_documentAttributeAudio();
                            audio.duration = MessageObject.getInlineResultDuration(result);
                            audio.voice = true;
                            fileName.file_name = "audio.ogg";
                            document3.attributes.add(audio);
                            break;
                        case 2:
                            TLRPC.TL_documentAttributeAudio audio2 = new TLRPC.TL_documentAttributeAudio();
                            audio2.duration = MessageObject.getInlineResultDuration(result);
                            audio2.title = result.title;
                            audio2.flags |= 1;
                            if (result.description != null) {
                                audio2.performer = result.description;
                                audio2.flags |= 2;
                            }
                            fileName.file_name = "audio.mp3";
                            document3.attributes.add(audio2);
                            break;
                        case 3:
                            int idx = result.content.mime_type.lastIndexOf(47);
                            if (idx != -1) {
                                fileName.file_name = "file." + result.content.mime_type.substring(idx + 1);
                                break;
                            } else {
                                fileName.file_name = "file";
                                break;
                            }
                        case 4:
                            fileName.file_name = "video.mp4";
                            TLRPC.TL_documentAttributeVideo attributeVideo = new TLRPC.TL_documentAttributeVideo();
                            int[] wh = MessageObject.getInlineResultWidthAndHeight(result);
                            attributeVideo.w = wh[0];
                            attributeVideo.h = wh[1];
                            attributeVideo.duration = MessageObject.getInlineResultDuration(result);
                            attributeVideo.supports_streaming = true;
                            document3.attributes.add(attributeVideo);
                            try {
                                if (result.thumb != null) {
                                    String thumbPath = new File(FileLoader.getDirectory(4), Utilities.MD5(result.thumb.url) + "." + ImageLoader.getHttpUrlExtension(result.thumb.url, "jpg")).getAbsolutePath();
                                    Bitmap bitmap2 = ImageLoader.loadBitmap(thumbPath, null, 90.0f, 90.0f, true);
                                    if (bitmap2 != null) {
                                        TLRPC.PhotoSize thumb2 = ImageLoader.scaleAndSaveImage(bitmap2, 90.0f, 90.0f, 55, false);
                                        if (thumb2 != null) {
                                            document3.thumbs.add(thumb2);
                                            document3.flags |= 1;
                                        }
                                        bitmap2.recycle();
                                        break;
                                    }
                                }
                            } catch (Throwable e2) {
                                FileLog.e(e2);
                                break;
                            }
                            break;
                        case 5:
                            TLRPC.TL_documentAttributeSticker attributeSticker = new TLRPC.TL_documentAttributeSticker();
                            attributeSticker.alt = "";
                            attributeSticker.stickerset = new TLRPC.TL_inputStickerSetEmpty();
                            document3.attributes.add(attributeSticker);
                            TLRPC.TL_documentAttributeImageSize attributeImageSize = new TLRPC.TL_documentAttributeImageSize();
                            int[] wh2 = MessageObject.getInlineResultWidthAndHeight(result);
                            attributeImageSize.w = wh2[0];
                            attributeImageSize.h = wh2[1];
                            document3.attributes.add(attributeImageSize);
                            fileName.file_name = "sticker.webp";
                            try {
                                if (result.thumb != null) {
                                    String thumbPath2 = new File(FileLoader.getDirectory(4), Utilities.MD5(result.thumb.url) + "." + ImageLoader.getHttpUrlExtension(result.thumb.url, "webp")).getAbsolutePath();
                                    Bitmap bitmap3 = ImageLoader.loadBitmap(thumbPath2, null, 90.0f, 90.0f, true);
                                    if (bitmap3 != null) {
                                        TLRPC.PhotoSize thumb3 = ImageLoader.scaleAndSaveImage(bitmap3, 90.0f, 90.0f, 55, false);
                                        if (thumb3 != null) {
                                            document3.thumbs.add(thumb3);
                                            document3.flags |= 1;
                                        }
                                        bitmap3.recycle();
                                        break;
                                    }
                                }
                            } catch (Throwable e3) {
                                FileLog.e(e3);
                                break;
                            }
                            break;
                    }
                    if (fileName.file_name == null) {
                        fileName.file_name = "file";
                    }
                    if (document3.mime_type == null) {
                        document3.mime_type = "application/octet-stream";
                    }
                    if (document3.thumbs.isEmpty()) {
                        TLRPC.PhotoSize thumb4 = new TLRPC.TL_photoSize();
                        int[] wh3 = MessageObject.getInlineResultWidthAndHeight(result);
                        thumb4.w = wh3[0];
                        thumb4.h = wh3[1];
                        thumb4.size = 0;
                        thumb4.location = new TLRPC.TL_fileLocationUnavailable();
                        thumb4.type = "x";
                        document3.thumbs.add(thumb4);
                        document3.flags |= 1;
                    }
                    finalPath2 = finalPath;
                    document = document3;
                    break;
                case 6:
                    if (!f.exists()) {
                        photo2 = null;
                    } else {
                        photo2 = accountInstance.getSendMessagesHelper().generatePhotoSizes(finalPath, null, false);
                    }
                    if (photo2 != null) {
                        photo = photo2;
                        finalPath2 = finalPath;
                        break;
                    } else {
                        TLRPC.TL_photo photo4 = new TLRPC.TL_photo();
                        photo4.date = accountInstance.getConnectionsManager().getCurrentTime();
                        photo4.file_reference = new byte[0];
                        TLRPC.TL_photoSize photoSize = new TLRPC.TL_photoSize();
                        int[] wh4 = MessageObject.getInlineResultWidthAndHeight(result);
                        photoSize.w = wh4[0];
                        photoSize.h = wh4[1];
                        photoSize.size = 1;
                        photoSize.location = new TLRPC.TL_fileLocationUnavailable();
                        photoSize.type = "x";
                        photo4.sizes.add(photoSize);
                        photo = photo4;
                        finalPath2 = finalPath;
                        break;
                    }
                default:
                    finalPath2 = finalPath;
                    break;
            }
        }
        final String finalPathFinal = finalPath2;
        final TLRPC.TL_document finalDocument = document;
        final TLRPC.TL_photo finalPhoto = photo;
        final TLRPC.TL_game finalGame = game;
        if (params != null && result.content != null) {
            params.put("originalPath", result.content.url);
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda38
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.lambda$prepareSendingBotContextResult$52(TLRPC.TL_document.this, accountInstance, finalPathFinal, dialog_id, reply_to_msg, result, params, notify, scheduleDate, finalPhoto, finalGame);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingBotContextResult$52(TLRPC.TL_document finalDocument, AccountInstance accountInstance, String finalPathFinal, long dialog_id, MessageObject reply_to_msg, TLRPC.BotInlineResult result, HashMap params, boolean notify, int scheduleDate, TLRPC.TL_photo finalPhoto, TLRPC.TL_game finalGame) {
        if (finalDocument != null) {
            accountInstance.getSendMessagesHelper().sendMessage(finalDocument, null, finalPathFinal, dialog_id, reply_to_msg, result.send_message.message, result.send_message.entities, result.send_message.reply_markup, params, notify, scheduleDate, 0, result);
        } else if (finalPhoto != null) {
            accountInstance.getSendMessagesHelper().sendMessage(finalPhoto, result.content != null ? result.content.url : null, dialog_id, reply_to_msg, result.send_message.message, result.send_message.entities, result.send_message.reply_markup, params, notify, scheduleDate, 0, result);
        } else if (finalGame != null) {
            accountInstance.getSendMessagesHelper().sendMessage(finalGame, dialog_id, result.send_message.reply_markup, params, notify, scheduleDate);
        }
    }

    private static String getTrimmedString(String src) {
        String result = src.trim();
        if (result.length() == 0) {
            return result;
        }
        while (src.startsWith("\n")) {
            src = src.substring(1);
        }
        while (src.endsWith("\n")) {
            src = src.substring(0, src.length() - 1);
        }
        return src;
    }

    public static void prepareSendingText(final AccountInstance accountInstance, final String text, final long dialog_id, final boolean notify, final int scheduleDate) {
        accountInstance.getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda43
            @Override // java.lang.Runnable
            public final void run() {
                Utilities.stageQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda58
                    @Override // java.lang.Runnable
                    public final void run() {
                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda42
                            @Override // java.lang.Runnable
                            public final void run() {
                                SendMessagesHelper.lambda$prepareSendingText$54(r1, r2, r3, r5, r6);
                            }
                        });
                    }
                });
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingText$54(String text, AccountInstance accountInstance, long dialog_id, boolean notify, int scheduleDate) {
        String textFinal = getTrimmedString(text);
        if (textFinal.length() != 0) {
            int count = (int) Math.ceil(textFinal.length() / 4096.0f);
            for (int a2 = 0; a2 < count; a2++) {
                String mess = textFinal.substring(a2 * 4096, Math.min((a2 + 1) * 4096, textFinal.length()));
                accountInstance.getSendMessagesHelper().sendMessage(mess, dialog_id, null, null, true, null, null, null, notify, scheduleDate);
            }
        }
    }

    /* JADX WARN: Code restructure failed: missing block: B:25:0x0092, code lost:
        r6.recycle();
     */
    /* JADX WARN: Code restructure failed: missing block: B:57:?, code lost:
        return;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static void ensureMediaThumbExists(boolean isEncrypted, TLObject object, String path, Uri uri, long startTime) {
        Bitmap thumb;
        boolean smallExists;
        Bitmap bitmap;
        TLRPC.PhotoSize size;
        if (object instanceof TLRPC.TL_photo) {
            TLRPC.TL_photo photo = (TLRPC.TL_photo) object;
            TLRPC.PhotoSize smallSize = FileLoader.getClosestPhotoSizeWithSize(photo.sizes, 90);
            if (smallSize instanceof TLRPC.TL_photoStrippedSize) {
                smallExists = true;
            } else {
                File smallFile = FileLoader.getPathToAttach(smallSize, true);
                smallExists = smallFile.exists();
            }
            TLRPC.PhotoSize bigSize = FileLoader.getClosestPhotoSizeWithSize(photo.sizes, AndroidUtilities.getPhotoSize());
            File bigFile = FileLoader.getPathToAttach(bigSize, false);
            boolean bigExists = bigFile.exists();
            if (!smallExists || !bigExists) {
                Bitmap bitmap2 = ImageLoader.loadBitmap(path, uri, AndroidUtilities.getPhotoSize(), AndroidUtilities.getPhotoSize(), true);
                if (bitmap2 != null) {
                    bitmap = bitmap2;
                } else {
                    bitmap = ImageLoader.loadBitmap(path, uri, 800.0f, 800.0f, true);
                }
                if (!bigExists) {
                    TLRPC.PhotoSize size2 = ImageLoader.scaleAndSaveImage(bigSize, bitmap, AndroidUtilities.getPhotoSize(), AndroidUtilities.getPhotoSize(), 80, false, 101, 101, false);
                    if (size2 != bigSize) {
                        photo.sizes.add(0, size2);
                    }
                }
                if (!smallExists && (size = ImageLoader.scaleAndSaveImage(smallSize, bitmap, 90.0f, 90.0f, 55, true)) != smallSize) {
                    photo.sizes.add(0, size);
                }
            }
        } else if (object instanceof TLRPC.TL_document) {
            TLRPC.TL_document document = (TLRPC.TL_document) object;
            if ((MessageObject.isVideoDocument(document) || MessageObject.isNewGifDocument(document)) && MessageObject.isDocumentHasThumb(document)) {
                TLRPC.PhotoSize photoSize = FileLoader.getClosestPhotoSizeWithSize(document.thumbs, 320);
                if (photoSize instanceof TLRPC.TL_photoStrippedSize) {
                    return;
                }
                File smallFile2 = FileLoader.getPathToAttach(photoSize, true);
                if (!smallFile2.exists()) {
                    Bitmap thumb2 = createVideoThumbnail(path, startTime);
                    if (thumb2 != null) {
                        thumb = thumb2;
                    } else {
                        thumb = ThumbnailUtils.createVideoThumbnail(path, 1);
                    }
                    int side = isEncrypted ? 90 : 320;
                    document.thumbs.set(0, ImageLoader.scaleAndSaveImage(photoSize, thumb, side, side, side > 90 ? 80 : 55, false));
                }
            }
        }
    }

    private static String getKeyForPhotoSize(TLRPC.PhotoSize photoSize, Bitmap[] bitmap, boolean blur) {
        int photoWidth;
        if (photoSize == null) {
            return null;
        }
        if (AndroidUtilities.isTablet()) {
            photoWidth = (int) (AndroidUtilities.getMinTabletSide() * 0.7f);
        } else {
            int maxPhotoWidth = photoSize.w;
            if (maxPhotoWidth >= photoSize.h) {
                photoWidth = Math.min(AndroidUtilities.displaySize.x, AndroidUtilities.displaySize.y) - AndroidUtilities.dp(64.0f);
            } else {
                photoWidth = (int) (Math.min(AndroidUtilities.displaySize.x, AndroidUtilities.displaySize.y) * 0.7f);
            }
        }
        int photoHeight = AndroidUtilities.dp(100.0f) + photoWidth;
        if (photoWidth > AndroidUtilities.getPhotoSize()) {
            photoWidth = AndroidUtilities.getPhotoSize();
        }
        if (photoHeight > AndroidUtilities.getPhotoSize()) {
            photoHeight = AndroidUtilities.getPhotoSize();
        }
        float scale = photoSize.w / photoWidth;
        int w = (int) (photoSize.w / scale);
        int h = (int) (photoSize.h / scale);
        if (w == 0) {
            w = AndroidUtilities.dp(150.0f);
        }
        if (h == 0) {
            h = AndroidUtilities.dp(150.0f);
        }
        if (h > photoHeight) {
            float scale2 = h;
            h = photoHeight;
            w = (int) (w / (scale2 / h));
        } else if (h < AndroidUtilities.dp(120.0f)) {
            h = AndroidUtilities.dp(120.0f);
            float hScale = photoSize.h / h;
            if (photoSize.w / hScale < photoWidth) {
                w = (int) (photoSize.w / hScale);
            }
        }
        if (bitmap != null) {
            try {
                BitmapFactory.Options opts = new BitmapFactory.Options();
                opts.inJustDecodeBounds = true;
                File file = FileLoader.getPathToAttach(photoSize);
                FileInputStream is = new FileInputStream(file);
                BitmapFactory.decodeStream(is, null, opts);
                is.close();
                float photoW = opts.outWidth;
                float photoH = opts.outHeight;
                float scaleFactor = Math.max(photoW / w, photoH / h);
                if (scaleFactor < 1.0f) {
                    scaleFactor = 1.0f;
                }
                opts.inJustDecodeBounds = false;
                opts.inSampleSize = (int) scaleFactor;
                opts.inPreferredConfig = Bitmap.Config.RGB_565;
                FileInputStream is2 = new FileInputStream(file);
                bitmap[0] = BitmapFactory.decodeStream(is2, null, opts);
                is2.close();
            } catch (Throwable th) {
            }
        }
        return String.format(Locale.US, blur ? "%d_%d@%d_%d_b" : "%d_%d@%d_%d", Long.valueOf(photoSize.location.volume_id), Integer.valueOf(photoSize.location.local_id), Integer.valueOf((int) (w / AndroidUtilities.density)), Integer.valueOf((int) (h / AndroidUtilities.density)));
    }

    public static void prepareSendingMedia(final AccountInstance accountInstance, final ArrayList<SendingMediaInfo> media, final long dialog_id, final MessageObject reply_to_msg, final InputContentInfoCompat inputContent, final boolean forceDocument, final boolean groupPhotos, final MessageObject editingMessageObject, final boolean notify, final int scheduleDate, final boolean blnOriginalImg) {
        if (media.isEmpty()) {
            return;
        }
        mediaSendQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda2
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.lambda$prepareSendingMedia$63(media, dialog_id, accountInstance, forceDocument, groupPhotos, blnOriginalImg, editingMessageObject, reply_to_msg, notify, scheduleDate, inputContent);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Can't wrap try/catch for region: R(12:313|(1:362)(1:317)|318|(3:320|(2:321|(1:323)(1:324))|325)(1:361)|(1:327)|(4:353|354|355|(4:357|(2:337|(1:344)(1:341))(1:345)|342|343))(1:329)|330|331|(3:333|334|335)(1:349)|(0)(0)|342|343) */
    /* JADX WARN: Code restructure failed: missing block: B:369:0x0ad5, code lost:
        if (r3.endsWith(r12) != false) goto L426;
     */
    /* JADX WARN: Code restructure failed: missing block: B:486:0x0d97, code lost:
        r0 = e;
     */
    /* JADX WARN: Code restructure failed: missing block: B:98:0x027e, code lost:
        if (r8 >= 73) goto L21;
     */
    /* JADX WARN: Removed duplicated region for block: B:109:0x02b6  */
    /* JADX WARN: Removed duplicated region for block: B:141:0x03d3 A[Catch: Exception -> 0x03b9, TRY_ENTER, TRY_LEAVE, TryCatch #1 {Exception -> 0x03b9, blocks: (B:132:0x03b1, B:141:0x03d3), top: B:526:0x03b1 }] */
    /* JADX WARN: Removed duplicated region for block: B:143:0x03de A[Catch: Exception -> 0x0442, TRY_ENTER, TRY_LEAVE, TryCatch #7 {Exception -> 0x0442, blocks: (B:139:0x03c9, B:152:0x03ff, B:143:0x03de), top: B:538:0x03c9 }] */
    /* JADX WARN: Removed duplicated region for block: B:161:0x042a  */
    /* JADX WARN: Removed duplicated region for block: B:172:0x0465  */
    /* JADX WARN: Removed duplicated region for block: B:173:0x0492  */
    /* JADX WARN: Removed duplicated region for block: B:178:0x04b8  */
    /* JADX WARN: Removed duplicated region for block: B:179:0x04bd  */
    /* JADX WARN: Removed duplicated region for block: B:182:0x04c8  */
    /* JADX WARN: Removed duplicated region for block: B:224:0x06ce  */
    /* JADX WARN: Removed duplicated region for block: B:271:0x0827  */
    /* JADX WARN: Removed duplicated region for block: B:286:0x0863  */
    /* JADX WARN: Removed duplicated region for block: B:287:0x0866  */
    /* JADX WARN: Removed duplicated region for block: B:321:0x092b  */
    /* JADX WARN: Removed duplicated region for block: B:327:0x0946  */
    /* JADX WARN: Removed duplicated region for block: B:332:0x0958  */
    /* JADX WARN: Removed duplicated region for block: B:339:0x09b6  */
    /* JADX WARN: Removed duplicated region for block: B:33:0x008e  */
    /* JADX WARN: Removed duplicated region for block: B:395:0x0b50  */
    /* JADX WARN: Removed duplicated region for block: B:400:0x0b9a  */
    /* JADX WARN: Removed duplicated region for block: B:41:0x00c2  */
    /* JADX WARN: Removed duplicated region for block: B:42:0x00cf  */
    /* JADX WARN: Removed duplicated region for block: B:490:0x0d9e  */
    /* JADX WARN: Removed duplicated region for block: B:497:0x0dcd  */
    /* JADX WARN: Removed duplicated region for block: B:507:0x0e9e  */
    /* JADX WARN: Removed duplicated region for block: B:508:0x0ead  */
    /* JADX WARN: Removed duplicated region for block: B:510:0x0eb3  */
    /* JADX WARN: Removed duplicated region for block: B:512:0x0eb8  */
    /* JADX WARN: Removed duplicated region for block: B:522:0x0f27  */
    /* JADX WARN: Removed duplicated region for block: B:534:0x03ec A[EXC_TOP_SPLITTER, SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:547:0x0956 A[EDGE_INSN: B:547:0x0956->B:331:0x0956 ?: BREAK  , SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:560:? A[RETURN, SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:94:0x0271  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static /* synthetic */ void lambda$prepareSendingMedia$63(ArrayList media, final long dialog_id, final AccountInstance accountInstance, boolean forceDocument, boolean groupPhotos, final boolean blnOriginalImg, final MessageObject editingMessageObject, final MessageObject reply_to_msg, final boolean notify, final int scheduleDate, InputContentInfoCompat inputContent) {
        int enryptedLayer;
        String str;
        String str2;
        String str3;
        String str4;
        HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers;
        HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers2;
        String str5;
        String str6;
        String str7;
        String str8;
        String originalPath;
        String originalPath2;
        String originalPath3;
        TLRPC.TL_photo photo;
        String parentObject;
        String str9;
        String parentObject2;
        String originalPath4;
        long lastGroupId;
        int a2;
        ArrayList<String> sendAsDocuments;
        int photosCount;
        long groupId;
        boolean isEncrypted;
        int enryptedLayer2;
        Object obj;
        String str10;
        String str11;
        boolean isDocument;
        String tempPath;
        String tempPath2;
        String str12;
        String str13;
        int a3;
        String str14;
        String originalPath5;
        HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers3;
        String str15;
        String str16;
        String str17;
        String str18;
        boolean z;
        int a4;
        TLRPC.TL_photo photo2;
        String parentObject3;
        String parentObject4;
        long groupId2;
        int a5;
        String str19;
        String str20;
        ArrayList<String> sendAsDocuments2;
        TLRPC.TL_photo photoFinal;
        TLRPC.TL_photo photo3;
        long groupId3;
        String originalPath6;
        ArrayList<String> sendAsDocumentsOriginal;
        VideoEditedInfo videoEditedInfo;
        String str21;
        HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers4;
        int a6;
        long groupId4;
        SendingMediaInfo info;
        String path;
        boolean muted;
        TLRPC.TL_document document;
        String str22;
        long startTime;
        String str23;
        String path2;
        String path3;
        TLRPC.TL_document document2;
        String parentObject5;
        boolean isEncrypted2;
        int enryptedLayer3;
        Bitmap thumb;
        TLRPC.TL_document document3;
        Bitmap thumbFinal;
        String thumbKeyFinal;
        Bitmap thumbFinal2;
        String thumbKeyFinal2;
        int a7;
        long groupId5;
        int photosCount2;
        boolean found;
        int b;
        int N;
        TLRPC.TL_documentAttributeVideo attributeVideo;
        Bitmap thumb2;
        int side;
        int i;
        boolean z2;
        boolean isEncrypted3;
        TLRPC.TL_photo photo4;
        long groupId6;
        long groupId7;
        int a8;
        Uri uri;
        File cacheFile;
        ArrayList<String> sendAsDocumentsOriginal2;
        String str24;
        HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers5;
        int a9;
        ArrayList<String> arrayList;
        SendingMediaInfo info2;
        File cacheFile2;
        File cacheFile3;
        File thumbFile;
        int side2;
        Object obj2;
        Bitmap bitmap;
        ArrayList arrayList2 = media;
        boolean z3 = blnOriginalImg;
        long beginTime = System.currentTimeMillis();
        int count = media.size();
        final boolean isEncrypted4 = ((int) dialog_id) == 0;
        if (isEncrypted4) {
            int high_id = (int) (dialog_id >> 32);
            TLRPC.EncryptedChat encryptedChat = accountInstance.getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
            if (encryptedChat != null) {
                int enryptedLayer4 = AndroidUtilities.getPeerLayerVersion(encryptedChat.layer);
                enryptedLayer = enryptedLayer4;
                String str25 = "_ori";
                String str26 = ".webp";
                String str27 = ".gif";
                String str28 = "_";
                if (!isEncrypted4 && enryptedLayer < 73) {
                    str = "_";
                    str2 = ".gif";
                    str3 = "_ori";
                    str4 = ".webp";
                } else if (!forceDocument || !groupPhotos) {
                    str = "_";
                    str2 = ".gif";
                    str3 = "_ori";
                    str4 = ".webp";
                } else {
                    HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers6 = new HashMap<>();
                    int a10 = 0;
                    while (a10 < count) {
                        final SendingMediaInfo info3 = (SendingMediaInfo) arrayList2.get(a10);
                        if (info3.searchImage != null || info3.isVideo) {
                            workers2 = workers6;
                            str5 = str28;
                            str6 = str27;
                            str7 = str25;
                            str8 = str26;
                        } else {
                            String originalPath7 = info3.path;
                            String tempPath3 = info3.path;
                            if (tempPath3 == null) {
                                originalPath = originalPath7;
                                if (info3.uri != null) {
                                    tempPath3 = AndroidUtilities.getPath(info3.uri);
                                    originalPath2 = info3.uri.toString();
                                    if (tempPath3 != null) {
                                        if (tempPath3.endsWith(str27)) {
                                            workers2 = workers6;
                                            str5 = str28;
                                            str6 = str27;
                                            str7 = str25;
                                            str8 = str26;
                                        } else if (tempPath3.endsWith(str26)) {
                                            workers2 = workers6;
                                            str5 = str28;
                                            str6 = str27;
                                            str7 = str25;
                                            str8 = str26;
                                        }
                                    }
                                    String str29 = str27;
                                    String str30 = str26;
                                    if (!ImageLoader.shouldSendImageAsDocument(info3.path, info3.uri)) {
                                        str5 = str28;
                                        str7 = str25;
                                        str6 = str29;
                                        str8 = str30;
                                        workers2 = workers6;
                                    } else {
                                        if (tempPath3 == null && info3.uri != null) {
                                            if (MediaController.isGif(info3.uri)) {
                                                str5 = str28;
                                                str7 = str25;
                                                str6 = str29;
                                                str8 = str30;
                                                workers2 = workers6;
                                            } else if (MediaController.isWebp(info3.uri)) {
                                                str5 = str28;
                                                str7 = str25;
                                                str6 = str29;
                                                str8 = str30;
                                                workers2 = workers6;
                                            }
                                        }
                                        if (tempPath3 != null) {
                                            File temp = new File(tempPath3);
                                            if (!z3) {
                                                originalPath4 = originalPath2 + temp.length() + str28 + temp.lastModified();
                                            } else {
                                                originalPath4 = originalPath2 + temp.length() + str28 + temp.lastModified() + str25;
                                            }
                                            originalPath3 = originalPath4;
                                        } else {
                                            originalPath3 = null;
                                        }
                                        TLRPC.TL_photo photo5 = null;
                                        String parentObject6 = null;
                                        if (isEncrypted4 || info3.ttl != 0) {
                                            str5 = str28;
                                            str7 = str25;
                                            str6 = str29;
                                            str8 = str30;
                                            photo = null;
                                            parentObject = null;
                                        } else {
                                            Object[] sentData = accountInstance.getMessagesStorage().getSentFile(originalPath3, !isEncrypted4 ? 0 : 3);
                                            if (sentData != null) {
                                                photo5 = (TLRPC.TL_photo) sentData[0];
                                                parentObject6 = (String) sentData[1];
                                            }
                                            if (photo5 != null || info3.uri == null) {
                                                str9 = str28;
                                                photo = photo5;
                                                parentObject2 = parentObject6;
                                            } else {
                                                str9 = str28;
                                                Object[] sentData2 = accountInstance.getMessagesStorage().getSentFile(AndroidUtilities.getPath(info3.uri), !isEncrypted4 ? 0 : 3);
                                                if (sentData2 == null) {
                                                    photo = photo5;
                                                    parentObject2 = parentObject6;
                                                } else {
                                                    TLRPC.TL_photo photo6 = (TLRPC.TL_photo) sentData2[0];
                                                    photo = photo6;
                                                    parentObject2 = (String) sentData2[1];
                                                }
                                            }
                                            String parentObject7 = info3.path;
                                            str5 = str9;
                                            str6 = str29;
                                            str7 = str25;
                                            str8 = str30;
                                            ensureMediaThumbExists(isEncrypted4, photo, parentObject7, info3.uri, 0L);
                                            parentObject = parentObject2;
                                        }
                                        final MediaSendPrepareWorker worker = new MediaSendPrepareWorker();
                                        workers6.put(info3, worker);
                                        if (photo != null) {
                                            worker.parentObject = parentObject;
                                            worker.photo = photo;
                                            workers2 = workers6;
                                        } else {
                                            worker.sync = new CountDownLatch(1);
                                            workers2 = workers6;
                                            mediaSendThreadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda33
                                                @Override // java.lang.Runnable
                                                public final void run() {
                                                    SendMessagesHelper.lambda$prepareSendingMedia$57(SendMessagesHelper.MediaSendPrepareWorker.this, accountInstance, info3, blnOriginalImg, isEncrypted4);
                                                }
                                            });
                                        }
                                    }
                                }
                            } else {
                                originalPath = originalPath7;
                            }
                            originalPath2 = originalPath;
                            if (tempPath3 != null) {
                            }
                            String str292 = str27;
                            String str302 = str26;
                            if (!ImageLoader.shouldSendImageAsDocument(info3.path, info3.uri)) {
                            }
                        }
                        a10++;
                        z3 = blnOriginalImg;
                        workers6 = workers2;
                        str28 = str5;
                        str27 = str6;
                        str25 = str7;
                        str26 = str8;
                    }
                    str = str28;
                    str2 = str27;
                    str3 = str25;
                    str4 = str26;
                    workers = workers6;
                    long groupId8 = 0;
                    lastGroupId = 0;
                    String extension = null;
                    ArrayList<String> sendAsDocumentsOriginal3 = null;
                    a2 = 0;
                    ArrayList<String> sendAsDocuments3 = null;
                    int photosCount3 = 0;
                    ArrayList<ArrayList<TLRPC.MessageEntity>> sendAsDocumentsEntities = null;
                    ArrayList<String> sendAsDocumentsCaptions = null;
                    while (a2 < count) {
                        final SendingMediaInfo info4 = (SendingMediaInfo) arrayList2.get(a2);
                        if (groupPhotos) {
                            if (isEncrypted4) {
                            }
                            if (count > 1 && photosCount3 % 10 == 0) {
                                long groupId9 = Utilities.random.nextLong();
                                lastGroupId = groupId9;
                                photosCount = 0;
                                groupId = groupId9;
                                int count2 = count;
                                ArrayList<String> sendAsDocuments4 = sendAsDocuments3;
                                if (info4.searchImage == null) {
                                    int a11 = a2;
                                    ArrayList<String> arrayList3 = sendAsDocumentsCaptions;
                                    int enryptedLayer5 = enryptedLayer;
                                    if (info4.searchImage.type == 1) {
                                        final HashMap<String, String> params = new HashMap<>();
                                        TLRPC.TL_document document4 = null;
                                        if (info4.searchImage.document instanceof TLRPC.TL_document) {
                                            document4 = (TLRPC.TL_document) info4.searchImage.document;
                                            cacheFile = FileLoader.getPathToAttach(document4, true);
                                        } else {
                                            String md5 = Utilities.MD5(info4.searchImage.imageUrl) + "." + ImageLoader.getHttpUrlExtension(info4.searchImage.imageUrl, "jpg");
                                            cacheFile = new File(FileLoader.getDirectory(4), md5);
                                        }
                                        if (document4 != null) {
                                            sendAsDocumentsOriginal2 = sendAsDocumentsOriginal3;
                                            str24 = "originalPath";
                                            workers5 = workers;
                                            a9 = a11;
                                            arrayList = arrayList3;
                                            info2 = info4;
                                            cacheFile2 = cacheFile;
                                        } else {
                                            File thumbFile2 = null;
                                            TLRPC.TL_document document5 = new TLRPC.TL_document();
                                            sendAsDocumentsOriginal2 = sendAsDocumentsOriginal3;
                                            str24 = "originalPath";
                                            document5.id = 0L;
                                            document5.file_reference = new byte[0];
                                            document5.date = accountInstance.getConnectionsManager().getCurrentTime();
                                            TLRPC.TL_documentAttributeFilename fileName = new TLRPC.TL_documentAttributeFilename();
                                            fileName.file_name = "animation.gif";
                                            document5.attributes.add(fileName);
                                            document5.size = info4.searchImage.size;
                                            document5.dc_id = 0;
                                            if (cacheFile.toString().endsWith("mp4")) {
                                                document5.mime_type = MimeTypes.VIDEO_MP4;
                                                document5.attributes.add(new TLRPC.TL_documentAttributeAnimated());
                                            } else {
                                                document5.mime_type = "image/gif";
                                            }
                                            if (cacheFile.exists()) {
                                                thumbFile2 = cacheFile;
                                                cacheFile3 = cacheFile;
                                            } else {
                                                cacheFile3 = null;
                                            }
                                            if (thumbFile2 != null) {
                                                thumbFile = thumbFile2;
                                            } else {
                                                String thumb3 = Utilities.MD5(info4.searchImage.thumbUrl) + "." + ImageLoader.getHttpUrlExtension(info4.searchImage.thumbUrl, "jpg");
                                                File thumbFile3 = new File(FileLoader.getDirectory(4), thumb3);
                                                if (thumbFile3.exists()) {
                                                    thumbFile = thumbFile3;
                                                } else {
                                                    thumbFile = null;
                                                }
                                            }
                                            if (thumbFile == null) {
                                                workers5 = workers;
                                                a9 = a11;
                                                arrayList = arrayList3;
                                                info2 = info4;
                                            } else {
                                                try {
                                                    if (!isEncrypted4) {
                                                        try {
                                                            if (info4.ttl == 0) {
                                                                side2 = 320;
                                                                if (!thumbFile.getAbsolutePath().endsWith("mp4")) {
                                                                    bitmap = ThumbnailUtils.createVideoThumbnail(thumbFile.getAbsolutePath(), 1);
                                                                    obj2 = null;
                                                                } else {
                                                                    obj2 = null;
                                                                    bitmap = ImageLoader.loadBitmap(thumbFile.getAbsolutePath(), null, side2, side2, true);
                                                                }
                                                                if (bitmap == null) {
                                                                    try {
                                                                        workers5 = workers;
                                                                        info2 = info4;
                                                                        a9 = a11;
                                                                        arrayList = arrayList3;
                                                                        try {
                                                                            TLRPC.PhotoSize thumb4 = ImageLoader.scaleAndSaveImage(bitmap, side2, side2, side2 > 90 ? 80 : 55, isEncrypted4, thumbFile.getAbsolutePath().endsWith(".png"));
                                                                            if (thumb4 != null) {
                                                                                document5.thumbs.add(thumb4);
                                                                                document5.flags |= 1;
                                                                            }
                                                                            bitmap.recycle();
                                                                        } catch (Exception e) {
                                                                            e = e;
                                                                            FileLog.e(e);
                                                                            if (!document5.thumbs.isEmpty()) {
                                                                            }
                                                                            document4 = document5;
                                                                            cacheFile2 = cacheFile3;
                                                                            final TLRPC.TL_document documentFinal = document4;
                                                                            String str31 = info2.searchImage.imageUrl;
                                                                            final String pathFinal = cacheFile2 != null ? info2.searchImage.imageUrl : cacheFile2.toString();
                                                                            if (info2.searchImage.imageUrl != null) {
                                                                            }
                                                                            ArrayList<String> sendAsDocumentsOriginal4 = sendAsDocumentsOriginal2;
                                                                            final SendingMediaInfo sendingMediaInfo = info2;
                                                                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda44
                                                                                @Override // java.lang.Runnable
                                                                                public final void run() {
                                                                                    SendMessagesHelper.lambda$prepareSendingMedia$58(MessageObject.this, accountInstance, documentFinal, pathFinal, params, r6, dialog_id, reply_to_msg, sendingMediaInfo, notify, scheduleDate);
                                                                                }
                                                                            });
                                                                            isEncrypted = isEncrypted4;
                                                                            photosCount3 = photosCount;
                                                                            str15 = str;
                                                                            groupId2 = groupId;
                                                                            workers3 = workers5;
                                                                            a5 = a9;
                                                                            sendAsDocumentsCaptions = arrayList;
                                                                            sendAsDocumentsOriginal3 = sendAsDocumentsOriginal4;
                                                                            sendAsDocumentsEntities = sendAsDocumentsEntities;
                                                                            sendAsDocuments3 = sendAsDocuments4;
                                                                            enryptedLayer2 = enryptedLayer5;
                                                                            obj = null;
                                                                            str19 = str4;
                                                                            str20 = str3;
                                                                            str17 = str2;
                                                                            a2 = a5 + 1;
                                                                            arrayList2 = media;
                                                                            count = count2;
                                                                            str2 = str17;
                                                                            str3 = str20;
                                                                            workers = workers3;
                                                                            str4 = str19;
                                                                            enryptedLayer = enryptedLayer2;
                                                                            isEncrypted4 = isEncrypted;
                                                                            str = str15;
                                                                            groupId8 = groupId2;
                                                                        }
                                                                    } catch (Exception e2) {
                                                                        e = e2;
                                                                        workers5 = workers;
                                                                        a9 = a11;
                                                                        arrayList = arrayList3;
                                                                        info2 = info4;
                                                                        FileLog.e(e);
                                                                        if (!document5.thumbs.isEmpty()) {
                                                                        }
                                                                        document4 = document5;
                                                                        cacheFile2 = cacheFile3;
                                                                        final TLRPC.TL_document documentFinal2 = document4;
                                                                        String str312 = info2.searchImage.imageUrl;
                                                                        final String pathFinal2 = cacheFile2 != null ? info2.searchImage.imageUrl : cacheFile2.toString();
                                                                        if (info2.searchImage.imageUrl != null) {
                                                                        }
                                                                        ArrayList<String> sendAsDocumentsOriginal42 = sendAsDocumentsOriginal2;
                                                                        final SendingMediaInfo sendingMediaInfo2 = info2;
                                                                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda44
                                                                            @Override // java.lang.Runnable
                                                                            public final void run() {
                                                                                SendMessagesHelper.lambda$prepareSendingMedia$58(MessageObject.this, accountInstance, documentFinal2, pathFinal2, params, r6, dialog_id, reply_to_msg, sendingMediaInfo2, notify, scheduleDate);
                                                                            }
                                                                        });
                                                                        isEncrypted = isEncrypted4;
                                                                        photosCount3 = photosCount;
                                                                        str15 = str;
                                                                        groupId2 = groupId;
                                                                        workers3 = workers5;
                                                                        a5 = a9;
                                                                        sendAsDocumentsCaptions = arrayList;
                                                                        sendAsDocumentsOriginal3 = sendAsDocumentsOriginal42;
                                                                        sendAsDocumentsEntities = sendAsDocumentsEntities;
                                                                        sendAsDocuments3 = sendAsDocuments4;
                                                                        enryptedLayer2 = enryptedLayer5;
                                                                        obj = null;
                                                                        str19 = str4;
                                                                        str20 = str3;
                                                                        str17 = str2;
                                                                        a2 = a5 + 1;
                                                                        arrayList2 = media;
                                                                        count = count2;
                                                                        str2 = str17;
                                                                        str3 = str20;
                                                                        workers = workers3;
                                                                        str4 = str19;
                                                                        enryptedLayer = enryptedLayer2;
                                                                        isEncrypted4 = isEncrypted;
                                                                        str = str15;
                                                                        groupId8 = groupId2;
                                                                    }
                                                                } else {
                                                                    workers5 = workers;
                                                                    a9 = a11;
                                                                    arrayList = arrayList3;
                                                                    info2 = info4;
                                                                }
                                                            }
                                                        } catch (Exception e3) {
                                                            e = e3;
                                                            workers5 = workers;
                                                            a9 = a11;
                                                            arrayList = arrayList3;
                                                            info2 = info4;
                                                            FileLog.e(e);
                                                            if (!document5.thumbs.isEmpty()) {
                                                            }
                                                            document4 = document5;
                                                            cacheFile2 = cacheFile3;
                                                            final TLRPC.TL_document documentFinal22 = document4;
                                                            String str3122 = info2.searchImage.imageUrl;
                                                            final String pathFinal22 = cacheFile2 != null ? info2.searchImage.imageUrl : cacheFile2.toString();
                                                            if (info2.searchImage.imageUrl != null) {
                                                            }
                                                            ArrayList<String> sendAsDocumentsOriginal422 = sendAsDocumentsOriginal2;
                                                            final SendingMediaInfo sendingMediaInfo22 = info2;
                                                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda44
                                                                @Override // java.lang.Runnable
                                                                public final void run() {
                                                                    SendMessagesHelper.lambda$prepareSendingMedia$58(MessageObject.this, accountInstance, documentFinal22, pathFinal22, params, r6, dialog_id, reply_to_msg, sendingMediaInfo22, notify, scheduleDate);
                                                                }
                                                            });
                                                            isEncrypted = isEncrypted4;
                                                            photosCount3 = photosCount;
                                                            str15 = str;
                                                            groupId2 = groupId;
                                                            workers3 = workers5;
                                                            a5 = a9;
                                                            sendAsDocumentsCaptions = arrayList;
                                                            sendAsDocumentsOriginal3 = sendAsDocumentsOriginal422;
                                                            sendAsDocumentsEntities = sendAsDocumentsEntities;
                                                            sendAsDocuments3 = sendAsDocuments4;
                                                            enryptedLayer2 = enryptedLayer5;
                                                            obj = null;
                                                            str19 = str4;
                                                            str20 = str3;
                                                            str17 = str2;
                                                            a2 = a5 + 1;
                                                            arrayList2 = media;
                                                            count = count2;
                                                            str2 = str17;
                                                            str3 = str20;
                                                            workers = workers3;
                                                            str4 = str19;
                                                            enryptedLayer = enryptedLayer2;
                                                            isEncrypted4 = isEncrypted;
                                                            str = str15;
                                                            groupId8 = groupId2;
                                                        }
                                                    }
                                                    if (!thumbFile.getAbsolutePath().endsWith("mp4")) {
                                                    }
                                                    if (bitmap == null) {
                                                    }
                                                } catch (Exception e4) {
                                                    e = e4;
                                                    workers5 = workers;
                                                    a9 = a11;
                                                    arrayList = arrayList3;
                                                }
                                                side2 = 90;
                                            }
                                            if (!document5.thumbs.isEmpty()) {
                                                TLRPC.TL_photoSize thumb5 = new TLRPC.TL_photoSize();
                                                thumb5.w = info2.searchImage.width;
                                                thumb5.h = info2.searchImage.height;
                                                thumb5.size = 0;
                                                thumb5.location = new TLRPC.TL_fileLocationUnavailable();
                                                thumb5.type = "x";
                                                document5.thumbs.add(thumb5);
                                                document5.flags |= 1;
                                            }
                                            document4 = document5;
                                            cacheFile2 = cacheFile3;
                                        }
                                        final TLRPC.TL_document documentFinal222 = document4;
                                        String str31222 = info2.searchImage.imageUrl;
                                        final String pathFinal222 = cacheFile2 != null ? info2.searchImage.imageUrl : cacheFile2.toString();
                                        if (info2.searchImage.imageUrl != null) {
                                            params.put(str24, info2.searchImage.imageUrl);
                                        }
                                        ArrayList<String> sendAsDocumentsOriginal4222 = sendAsDocumentsOriginal2;
                                        final SendingMediaInfo sendingMediaInfo222 = info2;
                                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda44
                                            @Override // java.lang.Runnable
                                            public final void run() {
                                                SendMessagesHelper.lambda$prepareSendingMedia$58(MessageObject.this, accountInstance, documentFinal222, pathFinal222, params, r6, dialog_id, reply_to_msg, sendingMediaInfo222, notify, scheduleDate);
                                            }
                                        });
                                        isEncrypted = isEncrypted4;
                                        photosCount3 = photosCount;
                                        str15 = str;
                                        groupId2 = groupId;
                                        workers3 = workers5;
                                        a5 = a9;
                                        sendAsDocumentsCaptions = arrayList;
                                        sendAsDocumentsOriginal3 = sendAsDocumentsOriginal4222;
                                        sendAsDocumentsEntities = sendAsDocumentsEntities;
                                        sendAsDocuments3 = sendAsDocuments4;
                                        enryptedLayer2 = enryptedLayer5;
                                        obj = null;
                                        str19 = str4;
                                        str20 = str3;
                                        str17 = str2;
                                    } else {
                                        ArrayList<ArrayList<TLRPC.MessageEntity>> arrayList4 = sendAsDocumentsEntities;
                                        boolean isEncrypted5 = isEncrypted4;
                                        ArrayList<String> sendAsDocumentsOriginal5 = sendAsDocumentsOriginal3;
                                        HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers7 = workers;
                                        int a12 = a11;
                                        boolean needDownloadHttp = true;
                                        TLRPC.TL_photo photo7 = null;
                                        if (info4.searchImage.photo instanceof TLRPC.TL_photo) {
                                            photo7 = (TLRPC.TL_photo) info4.searchImage.photo;
                                        } else if (!isEncrypted5) {
                                            int i2 = info4.ttl;
                                        }
                                        if (photo7 != null) {
                                            z2 = blnOriginalImg;
                                            isEncrypted3 = isEncrypted5;
                                            photo4 = photo7;
                                        } else {
                                            String md52 = Utilities.MD5(info4.searchImage.imageUrl) + "." + ImageLoader.getHttpUrlExtension(info4.searchImage.imageUrl, "jpg");
                                            File cacheFile4 = new File(FileLoader.getDirectory(4), md52);
                                            if (!cacheFile4.exists() || cacheFile4.length() == 0) {
                                                z2 = blnOriginalImg;
                                                isEncrypted3 = isEncrypted5;
                                                uri = null;
                                            } else {
                                                z2 = blnOriginalImg;
                                                isEncrypted3 = isEncrypted5;
                                                uri = null;
                                                photo7 = accountInstance.getSendMessagesHelper().generatePhotoSizes(cacheFile4.toString(), null, z2);
                                                if (photo7 != null) {
                                                    needDownloadHttp = false;
                                                }
                                            }
                                            if (photo7 == null) {
                                                String md53 = Utilities.MD5(info4.searchImage.thumbUrl) + "." + ImageLoader.getHttpUrlExtension(info4.searchImage.thumbUrl, "jpg");
                                                File cacheFile5 = new File(FileLoader.getDirectory(4), md53);
                                                if (cacheFile5.exists()) {
                                                    photo7 = accountInstance.getSendMessagesHelper().generatePhotoSizes(cacheFile5.toString(), uri, z2);
                                                }
                                                if (photo7 == null) {
                                                    TLRPC.TL_photo photo8 = new TLRPC.TL_photo();
                                                    photo8.date = accountInstance.getConnectionsManager().getCurrentTime();
                                                    photo8.file_reference = new byte[0];
                                                    TLRPC.TL_photoSize photoSize = new TLRPC.TL_photoSize();
                                                    photoSize.w = info4.searchImage.width;
                                                    photoSize.h = info4.searchImage.height;
                                                    photoSize.size = 0;
                                                    photoSize.location = new TLRPC.TL_fileLocationUnavailable();
                                                    photoSize.type = "x";
                                                    photo8.sizes.add(photoSize);
                                                    photo4 = photo8;
                                                } else {
                                                    photo4 = photo7;
                                                }
                                            } else {
                                                photo4 = photo7;
                                            }
                                        }
                                        if (photo4 == null) {
                                            groupId6 = groupId;
                                        } else {
                                            final TLRPC.TL_photo photoFinal2 = photo4;
                                            final boolean needDownloadHttpFinal = needDownloadHttp;
                                            final HashMap<String, String> params2 = new HashMap<>();
                                            if (info4.searchImage.imageUrl != null) {
                                                params2.put("originalPath", info4.searchImage.imageUrl);
                                            }
                                            if (!groupPhotos) {
                                                groupId7 = groupId;
                                                a8 = a12;
                                            } else {
                                                int photosCount4 = photosCount + 1;
                                                groupId7 = groupId;
                                                params2.put("groupId", "" + groupId7);
                                                if (photosCount4 != 10) {
                                                    a8 = a12;
                                                    if (a8 != count2 - 1) {
                                                        photosCount = photosCount4;
                                                    }
                                                } else {
                                                    a8 = a12;
                                                }
                                                params2.put("final", "1");
                                                lastGroupId = 0;
                                                photosCount = photosCount4;
                                            }
                                            groupId6 = groupId7;
                                            a12 = a8;
                                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda55
                                                @Override // java.lang.Runnable
                                                public final void run() {
                                                    SendMessagesHelper.lambda$prepareSendingMedia$59(MessageObject.this, accountInstance, photoFinal2, needDownloadHttpFinal, info4, params2, r7, dialog_id, reply_to_msg, notify, scheduleDate);
                                                }
                                            });
                                        }
                                        photosCount3 = photosCount;
                                        isEncrypted = isEncrypted3;
                                        str15 = str;
                                        str19 = str4;
                                        workers3 = workers7;
                                        a5 = a12;
                                        sendAsDocumentsCaptions = arrayList3;
                                        sendAsDocumentsOriginal3 = sendAsDocumentsOriginal5;
                                        sendAsDocumentsEntities = arrayList4;
                                        sendAsDocuments3 = sendAsDocuments4;
                                        enryptedLayer2 = enryptedLayer5;
                                        groupId2 = groupId6;
                                        obj = null;
                                        str20 = str3;
                                        str17 = str2;
                                    }
                                } else {
                                    int a13 = a2;
                                    ArrayList<String> arrayList5 = sendAsDocumentsCaptions;
                                    int enryptedLayer6 = enryptedLayer;
                                    ArrayList<ArrayList<TLRPC.MessageEntity>> arrayList6 = sendAsDocumentsEntities;
                                    boolean isEncrypted6 = isEncrypted4;
                                    ArrayList<String> sendAsDocumentsOriginal6 = sendAsDocumentsOriginal3;
                                    HashMap<SendingMediaInfo, MediaSendPrepareWorker> workers8 = workers;
                                    long groupId10 = groupId;
                                    if (info4.isVideo) {
                                        String thumbKey = null;
                                        if (forceDocument) {
                                            videoEditedInfo = null;
                                        } else {
                                            VideoEditedInfo videoEditedInfo2 = info4.videoEditedInfo;
                                            videoEditedInfo = videoEditedInfo2 != null ? info4.videoEditedInfo : createCompressionSettings(info4.path);
                                        }
                                        if (!forceDocument) {
                                            if (videoEditedInfo != null || info4.path.endsWith("mp4")) {
                                                String path4 = info4.path;
                                                String originalPath8 = info4.path;
                                                File temp2 = new File(originalPath8);
                                                long startTime2 = 0;
                                                String finalPath = str;
                                                String originalPath9 = originalPath8 + temp2.length() + finalPath + temp2.lastModified();
                                                if (videoEditedInfo == null) {
                                                    path = path4;
                                                    muted = false;
                                                } else {
                                                    boolean muted2 = videoEditedInfo.muted;
                                                    path = path4;
                                                    String originalPath10 = originalPath9 + videoEditedInfo.estimatedDuration + finalPath + videoEditedInfo.startTime + finalPath + videoEditedInfo.endTime + (videoEditedInfo.muted ? "_m" : "");
                                                    if (videoEditedInfo.resultWidth == videoEditedInfo.originalWidth) {
                                                        originalPath9 = originalPath10;
                                                    } else {
                                                        originalPath9 = originalPath10 + finalPath + videoEditedInfo.resultWidth;
                                                    }
                                                    startTime2 = videoEditedInfo.startTime >= 0 ? videoEditedInfo.startTime : 0L;
                                                    muted = muted2;
                                                }
                                                if (isEncrypted6 || info4.ttl != 0) {
                                                    document = null;
                                                    str22 = MimeTypes.VIDEO_MP4;
                                                    startTime = startTime2;
                                                    str23 = "groupId";
                                                    path2 = path;
                                                    path3 = "final";
                                                } else {
                                                    MessagesStorage messagesStorage = accountInstance.getMessagesStorage();
                                                    if (!isEncrypted6) {
                                                        document = null;
                                                        i = 2;
                                                    } else {
                                                        document = null;
                                                        i = 5;
                                                    }
                                                    Object[] sentData3 = messagesStorage.getSentFile(originalPath9, i);
                                                    if (sentData3 == null) {
                                                        startTime = startTime2;
                                                        str23 = "groupId";
                                                        str22 = MimeTypes.VIDEO_MP4;
                                                        path2 = path;
                                                        path3 = "final";
                                                    } else {
                                                        TLRPC.TL_document document6 = (TLRPC.TL_document) sentData3[0];
                                                        parentObject5 = (String) sentData3[1];
                                                        String parentObject8 = info4.path;
                                                        str23 = "groupId";
                                                        str22 = MimeTypes.VIDEO_MP4;
                                                        startTime = startTime2;
                                                        path2 = path;
                                                        path3 = "final";
                                                        ensureMediaThumbExists(isEncrypted6, document6, parentObject8, null, startTime);
                                                        document2 = document6;
                                                        if (document2 == null) {
                                                            isEncrypted2 = isEncrypted6;
                                                            enryptedLayer3 = enryptedLayer6;
                                                            thumb = null;
                                                            document3 = document2;
                                                        } else {
                                                            long startTime3 = startTime;
                                                            Bitmap thumb6 = createVideoThumbnail(info4.path, startTime3);
                                                            if (thumb6 == null) {
                                                                thumb6 = ThumbnailUtils.createVideoThumbnail(info4.path, 1);
                                                            }
                                                            TLRPC.PhotoSize size = null;
                                                            if (thumb6 != null) {
                                                                if (!isEncrypted6 && info4.ttl == 0) {
                                                                    side = Math.max(thumb6.getWidth(), thumb6.getHeight());
                                                                    startTime = startTime3;
                                                                    isEncrypted2 = isEncrypted6;
                                                                    size = ImageLoader.scaleAndSaveImage(thumb6, side, side, side <= 90 ? 80 : 55, isEncrypted2);
                                                                    thumbKey = getKeyForPhotoSize(size, null, true);
                                                                }
                                                                side = 90;
                                                                startTime = startTime3;
                                                                isEncrypted2 = isEncrypted6;
                                                                size = ImageLoader.scaleAndSaveImage(thumb6, side, side, side <= 90 ? 80 : 55, isEncrypted2);
                                                                thumbKey = getKeyForPhotoSize(size, null, true);
                                                            } else {
                                                                startTime = startTime3;
                                                                isEncrypted2 = isEncrypted6;
                                                            }
                                                            TLRPC.TL_document document7 = new TLRPC.TL_document();
                                                            document7.file_reference = new byte[0];
                                                            if (size != null) {
                                                                document7.thumbs.add(size);
                                                                document7.flags |= 1;
                                                            }
                                                            document7.mime_type = str22;
                                                            accountInstance.getUserConfig().saveConfig(false);
                                                            if (isEncrypted2) {
                                                                enryptedLayer3 = enryptedLayer6;
                                                                if (enryptedLayer3 >= 66) {
                                                                    attributeVideo = new TLRPC.TL_documentAttributeVideo();
                                                                } else {
                                                                    attributeVideo = new TLRPC.TL_documentAttributeVideo_layer65();
                                                                }
                                                            } else {
                                                                enryptedLayer3 = enryptedLayer6;
                                                                attributeVideo = new TLRPC.TL_documentAttributeVideo();
                                                                attributeVideo.supports_streaming = true;
                                                            }
                                                            document7.attributes.add(attributeVideo);
                                                            if (videoEditedInfo == null || !videoEditedInfo.needConvert()) {
                                                                thumb2 = thumb6;
                                                                if (temp2.exists()) {
                                                                    document7.size = (int) temp2.length();
                                                                }
                                                                fillVideoAttribute(info4.path, attributeVideo, null);
                                                            } else {
                                                                if (videoEditedInfo.muted) {
                                                                    fillVideoAttribute(info4.path, attributeVideo, videoEditedInfo);
                                                                    videoEditedInfo.originalWidth = attributeVideo.w;
                                                                    videoEditedInfo.originalHeight = attributeVideo.h;
                                                                    thumb2 = thumb6;
                                                                } else {
                                                                    thumb2 = thumb6;
                                                                    attributeVideo.duration = (int) (videoEditedInfo.estimatedDuration / 1000);
                                                                }
                                                                if (videoEditedInfo.rotationValue == 90 || videoEditedInfo.rotationValue == 270) {
                                                                    attributeVideo.w = videoEditedInfo.resultHeight;
                                                                    attributeVideo.h = videoEditedInfo.resultWidth;
                                                                } else {
                                                                    attributeVideo.w = videoEditedInfo.resultWidth;
                                                                    attributeVideo.h = videoEditedInfo.resultHeight;
                                                                }
                                                                document7.size = (int) videoEditedInfo.estimatedSize;
                                                            }
                                                            document3 = document7;
                                                            thumb = thumb2;
                                                        }
                                                        if (videoEditedInfo != null && videoEditedInfo.muted) {
                                                            found = false;
                                                            b = 0;
                                                            N = document3.attributes.size();
                                                            while (true) {
                                                                if (b >= N) {
                                                                    break;
                                                                } else if (!(document3.attributes.get(b) instanceof TLRPC.TL_documentAttributeAnimated)) {
                                                                    b++;
                                                                } else {
                                                                    found = true;
                                                                    break;
                                                                }
                                                            }
                                                            if (!found) {
                                                                document3.attributes.add(new TLRPC.TL_documentAttributeAnimated());
                                                            }
                                                        }
                                                        if (videoEditedInfo != null && videoEditedInfo.needConvert()) {
                                                            File cacheFile6 = new File(FileLoader.getDirectory(4), "-2147483648_" + SharedConfig.getLastLocalId() + ".mp4");
                                                            SharedConfig.saveConfig();
                                                            path2 = cacheFile6.getAbsolutePath();
                                                        }
                                                        final TLRPC.TL_document videoFinal = document3;
                                                        final String parentFinal = parentObject5;
                                                        final String finalPath2 = path2;
                                                        final HashMap<String, String> params3 = new HashMap<>();
                                                        thumbFinal = thumb;
                                                        str21 = finalPath;
                                                        obj = null;
                                                        thumbKeyFinal = thumbKey;
                                                        if (originalPath9 != null) {
                                                            params3.put("originalPath", originalPath9);
                                                        }
                                                        if (muted && groupPhotos) {
                                                            int photosCount5 = photosCount + 1;
                                                            thumbFinal2 = thumbFinal;
                                                            thumbKeyFinal2 = thumbKeyFinal;
                                                            groupId5 = groupId10;
                                                            params3.put(str23, "" + groupId5);
                                                            if (photosCount5 != 10) {
                                                                a7 = a13;
                                                                if (a7 != count2 - 1) {
                                                                    photosCount2 = photosCount5;
                                                                }
                                                            } else {
                                                                a7 = a13;
                                                            }
                                                            params3.put(path3, "1");
                                                            lastGroupId = 0;
                                                            photosCount2 = photosCount5;
                                                        } else {
                                                            thumbFinal2 = thumbFinal;
                                                            thumbKeyFinal2 = thumbKeyFinal;
                                                            a7 = a13;
                                                            groupId5 = groupId10;
                                                            photosCount2 = photosCount;
                                                        }
                                                        enryptedLayer2 = enryptedLayer3;
                                                        isEncrypted = isEncrypted2;
                                                        final VideoEditedInfo videoEditedInfo3 = videoEditedInfo;
                                                        a6 = a7;
                                                        groupId4 = groupId5;
                                                        final Bitmap thumbFinal3 = thumbFinal2;
                                                        final String thumbKeyFinal3 = thumbKeyFinal2;
                                                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda62
                                                            @Override // java.lang.Runnable
                                                            public final void run() {
                                                                SendMessagesHelper.lambda$prepareSendingMedia$60(thumbFinal3, thumbKeyFinal3, editingMessageObject, accountInstance, videoEditedInfo3, videoFinal, finalPath2, params3, parentFinal, dialog_id, reply_to_msg, info4, notify, scheduleDate);
                                                            }
                                                        });
                                                        photosCount = photosCount2;
                                                        photosCount3 = photosCount;
                                                        str19 = str4;
                                                        a5 = a6;
                                                        workers3 = workers8;
                                                        sendAsDocumentsCaptions = arrayList5;
                                                        sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                        sendAsDocumentsEntities = arrayList6;
                                                        sendAsDocuments3 = sendAsDocuments4;
                                                        str15 = str21;
                                                        groupId2 = groupId4;
                                                        str20 = str3;
                                                        str17 = str2;
                                                    }
                                                }
                                                document2 = document;
                                                parentObject5 = null;
                                                if (document2 == null) {
                                                }
                                                if (videoEditedInfo != null) {
                                                    found = false;
                                                    b = 0;
                                                    N = document3.attributes.size();
                                                    while (true) {
                                                        if (b >= N) {
                                                        }
                                                        b++;
                                                    }
                                                    if (!found) {
                                                    }
                                                }
                                                if (videoEditedInfo != null) {
                                                    File cacheFile62 = new File(FileLoader.getDirectory(4), "-2147483648_" + SharedConfig.getLastLocalId() + ".mp4");
                                                    SharedConfig.saveConfig();
                                                    path2 = cacheFile62.getAbsolutePath();
                                                }
                                                final TLRPC.TL_document videoFinal2 = document3;
                                                final String parentFinal2 = parentObject5;
                                                final String finalPath22 = path2;
                                                final HashMap params32 = new HashMap<>();
                                                thumbFinal = thumb;
                                                str21 = finalPath;
                                                obj = null;
                                                thumbKeyFinal = thumbKey;
                                                if (originalPath9 != null) {
                                                }
                                                if (muted) {
                                                }
                                                thumbFinal2 = thumbFinal;
                                                thumbKeyFinal2 = thumbKeyFinal;
                                                a7 = a13;
                                                groupId5 = groupId10;
                                                photosCount2 = photosCount;
                                                enryptedLayer2 = enryptedLayer3;
                                                isEncrypted = isEncrypted2;
                                                final VideoEditedInfo videoEditedInfo32 = videoEditedInfo;
                                                a6 = a7;
                                                groupId4 = groupId5;
                                                final Bitmap thumbFinal32 = thumbFinal2;
                                                final String thumbKeyFinal32 = thumbKeyFinal2;
                                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda62
                                                    @Override // java.lang.Runnable
                                                    public final void run() {
                                                        SendMessagesHelper.lambda$prepareSendingMedia$60(thumbFinal32, thumbKeyFinal32, editingMessageObject, accountInstance, videoEditedInfo32, videoFinal2, finalPath22, params32, parentFinal2, dialog_id, reply_to_msg, info4, notify, scheduleDate);
                                                    }
                                                });
                                                photosCount = photosCount2;
                                                photosCount3 = photosCount;
                                                str19 = str4;
                                                a5 = a6;
                                                workers3 = workers8;
                                                sendAsDocumentsCaptions = arrayList5;
                                                sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                sendAsDocumentsEntities = arrayList6;
                                                sendAsDocuments3 = sendAsDocuments4;
                                                str15 = str21;
                                                groupId2 = groupId4;
                                                str20 = str3;
                                                str17 = str2;
                                            } else {
                                                isEncrypted = isEncrypted6;
                                                str21 = str;
                                                workers4 = workers8;
                                                a6 = a13;
                                                enryptedLayer2 = enryptedLayer6;
                                                groupId4 = groupId10;
                                                obj = null;
                                                info = info4;
                                            }
                                        } else {
                                            isEncrypted = isEncrypted6;
                                            str21 = str;
                                            workers4 = workers8;
                                            a6 = a13;
                                            enryptedLayer2 = enryptedLayer6;
                                            groupId4 = groupId10;
                                            obj = null;
                                            info = info4;
                                        }
                                        SendingMediaInfo info5 = info;
                                        workers8 = workers4;
                                        prepareSendingDocumentInternal(accountInstance, info5.path, info5.path, null, null, dialog_id, reply_to_msg, info5.caption, info5.entities, editingMessageObject, forceDocument, notify, scheduleDate);
                                        photosCount3 = photosCount;
                                        str19 = str4;
                                        a5 = a6;
                                        workers3 = workers8;
                                        sendAsDocumentsCaptions = arrayList5;
                                        sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                        sendAsDocumentsEntities = arrayList6;
                                        sendAsDocuments3 = sendAsDocuments4;
                                        str15 = str21;
                                        groupId2 = groupId4;
                                        str20 = str3;
                                        str17 = str2;
                                    } else {
                                        isEncrypted = isEncrypted6;
                                        String str32 = str;
                                        enryptedLayer2 = enryptedLayer6;
                                        obj = null;
                                        String originalPath11 = info4.path;
                                        String tempPath4 = info4.path;
                                        if (tempPath4 == null && info4.uri != null) {
                                            tempPath4 = AndroidUtilities.getPath(info4.uri);
                                            originalPath11 = info4.uri.toString();
                                        }
                                        if (forceDocument) {
                                            str10 = str2;
                                            str11 = str4;
                                        } else if (ImageLoader.shouldSendImageAsDocument(info4.path, info4.uri)) {
                                            str10 = str2;
                                            str11 = str4;
                                        } else {
                                            if (tempPath4 == null) {
                                                str10 = str2;
                                                str11 = str4;
                                            } else {
                                                str10 = str2;
                                                if (tempPath4.endsWith(str10)) {
                                                    str11 = str4;
                                                } else {
                                                    str11 = str4;
                                                }
                                                if (tempPath4.endsWith(str10)) {
                                                    extension = "gif";
                                                } else {
                                                    extension = "webp";
                                                }
                                                tempPath = tempPath4;
                                                isDocument = true;
                                                if (!isDocument) {
                                                    if (sendAsDocuments4 != null) {
                                                        sendAsDocuments3 = sendAsDocuments4;
                                                        sendAsDocumentsCaptions = arrayList5;
                                                        sendAsDocumentsOriginal = sendAsDocumentsOriginal6;
                                                        sendAsDocumentsEntities = arrayList6;
                                                    } else {
                                                        sendAsDocuments3 = new ArrayList<>();
                                                        sendAsDocumentsOriginal = new ArrayList<>();
                                                        sendAsDocumentsEntities = new ArrayList<>();
                                                        sendAsDocumentsCaptions = new ArrayList<>();
                                                    }
                                                    sendAsDocuments3.add(tempPath);
                                                    sendAsDocumentsOriginal.add(originalPath11);
                                                    sendAsDocumentsEntities.add(info4.caption);
                                                    sendAsDocumentsCaptions.add(info4.entities);
                                                    sendAsDocumentsOriginal3 = sendAsDocumentsOriginal;
                                                    str19 = str11;
                                                    photosCount3 = photosCount;
                                                    str20 = str3;
                                                    a5 = a13;
                                                    workers3 = workers8;
                                                    str15 = str32;
                                                    groupId2 = groupId10;
                                                    str17 = str10;
                                                } else {
                                                    if (tempPath != null) {
                                                        File temp3 = new File(tempPath);
                                                        a3 = a13;
                                                        if (!blnOriginalImg) {
                                                            str14 = str32;
                                                            tempPath2 = tempPath;
                                                            str12 = str11;
                                                            originalPath6 = originalPath11 + temp3.length() + str14 + temp3.lastModified();
                                                            str13 = str3;
                                                        } else {
                                                            tempPath2 = tempPath;
                                                            str12 = str11;
                                                            str14 = str32;
                                                            str13 = str3;
                                                            originalPath6 = originalPath11 + temp3.length() + str14 + temp3.lastModified() + str13;
                                                        }
                                                        originalPath5 = originalPath6;
                                                    } else {
                                                        tempPath2 = tempPath;
                                                        str12 = str11;
                                                        str13 = str3;
                                                        a3 = a13;
                                                        str14 = str32;
                                                        originalPath5 = null;
                                                    }
                                                    TLRPC.TL_photo photo9 = null;
                                                    String parentObject9 = null;
                                                    if (workers8 != null) {
                                                        MediaSendPrepareWorker worker2 = workers8.get(info4);
                                                        TLRPC.TL_photo photo10 = worker2.photo;
                                                        String parentObject10 = worker2.parentObject;
                                                        if (photo10 == null) {
                                                            try {
                                                                worker2.sync.await();
                                                            } catch (Exception e5) {
                                                                FileLog.e(e5);
                                                            }
                                                            photo10 = worker2.photo;
                                                            parentObject10 = worker2.parentObject;
                                                        }
                                                        parentObject3 = parentObject10;
                                                        workers3 = workers8;
                                                        str15 = str14;
                                                        str16 = str13;
                                                        str17 = str10;
                                                        str18 = "originalPath";
                                                        a4 = a3;
                                                        photo2 = photo10;
                                                    } else {
                                                        if (isEncrypted || info4.ttl != 0) {
                                                            workers3 = workers8;
                                                            str15 = str14;
                                                            str16 = str13;
                                                            str17 = str10;
                                                            str18 = "originalPath";
                                                            z = blnOriginalImg;
                                                            a4 = a3;
                                                        } else {
                                                            Object[] sentData4 = accountInstance.getMessagesStorage().getSentFile(originalPath5, !isEncrypted ? 0 : 3);
                                                            if (sentData4 != null) {
                                                                photo9 = (TLRPC.TL_photo) sentData4[0];
                                                                parentObject9 = (String) sentData4[1];
                                                            }
                                                            if (photo9 == null && info4.uri != null) {
                                                                TLRPC.TL_photo photo11 = photo9;
                                                                Object[] sentData5 = accountInstance.getMessagesStorage().getSentFile(AndroidUtilities.getPath(info4.uri), !isEncrypted ? 0 : 3);
                                                                if (sentData5 != null) {
                                                                    photo9 = (TLRPC.TL_photo) sentData5[0];
                                                                    String parentObject11 = (String) sentData5[1];
                                                                    parentObject4 = parentObject11;
                                                                } else {
                                                                    photo9 = photo11;
                                                                    parentObject4 = parentObject9;
                                                                }
                                                            } else {
                                                                photo9 = photo9;
                                                                parentObject4 = parentObject9;
                                                            }
                                                            str18 = "originalPath";
                                                            workers3 = workers8;
                                                            str15 = str14;
                                                            str16 = str13;
                                                            str17 = str10;
                                                            z = blnOriginalImg;
                                                            a4 = a3;
                                                            ensureMediaThumbExists(isEncrypted, photo9, info4.path, info4.uri, 0L);
                                                            parentObject9 = parentObject4;
                                                        }
                                                        if (photo9 != null) {
                                                            photo2 = photo9;
                                                            parentObject3 = parentObject9;
                                                        } else {
                                                            TLRPC.TL_photo photo12 = accountInstance.getSendMessagesHelper().generatePhotoSizes(info4.path, info4.uri, z);
                                                            if (isEncrypted && info4.canDeleteAfter) {
                                                                new File(info4.path).delete();
                                                            }
                                                            photo2 = photo12;
                                                            parentObject3 = parentObject9;
                                                        }
                                                    }
                                                    if (photo2 == null) {
                                                        String originalPath12 = originalPath5;
                                                        String tempPath5 = tempPath2;
                                                        groupId2 = groupId10;
                                                        a5 = a4;
                                                        String str33 = str16;
                                                        str19 = str12;
                                                        str20 = str33;
                                                        if (sendAsDocuments4 != null) {
                                                            sendAsDocuments3 = sendAsDocuments4;
                                                            sendAsDocumentsCaptions = arrayList5;
                                                            sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                            sendAsDocumentsEntities = arrayList6;
                                                        } else {
                                                            sendAsDocuments3 = new ArrayList<>();
                                                            sendAsDocumentsOriginal3 = new ArrayList<>();
                                                            sendAsDocumentsEntities = new ArrayList<>();
                                                            sendAsDocumentsCaptions = new ArrayList<>();
                                                        }
                                                        sendAsDocuments3.add(tempPath5);
                                                        sendAsDocumentsOriginal3.add(originalPath12);
                                                        sendAsDocumentsEntities.add(info4.caption);
                                                        sendAsDocumentsCaptions.add(info4.entities);
                                                        photosCount3 = photosCount;
                                                    } else {
                                                        TLRPC.TL_photo photoFinal3 = photo2;
                                                        ArrayList<String> sendAsDocuments5 = sendAsDocuments4;
                                                        final String parentFinal3 = parentObject3;
                                                        final HashMap<String, String> params4 = new HashMap<>();
                                                        final Bitmap[] bitmapFinal = new Bitmap[1];
                                                        final String[] keyFinal = new String[1];
                                                        boolean z4 = (info4.masks == null || info4.masks.isEmpty()) ? false : true;
                                                        photo2.has_stickers = z4;
                                                        if (!z4) {
                                                            sendAsDocuments2 = sendAsDocuments5;
                                                        } else {
                                                            SerializedData serializedData = new SerializedData((info4.masks.size() * 20) + 4);
                                                            serializedData.writeInt32(info4.masks.size());
                                                            int b2 = 0;
                                                            while (true) {
                                                                sendAsDocuments2 = sendAsDocuments5;
                                                                if (b2 >= info4.masks.size()) {
                                                                    break;
                                                                }
                                                                info4.masks.get(b2).serializeToStream(serializedData);
                                                                b2++;
                                                                sendAsDocuments5 = sendAsDocuments2;
                                                            }
                                                            params4.put("masks", Utilities.bytesToHex(serializedData.toByteArray()));
                                                            serializedData.cleanup();
                                                        }
                                                        if (originalPath5 != null) {
                                                            params4.put(str18, originalPath5);
                                                        }
                                                        if (groupPhotos) {
                                                            try {
                                                            } catch (Exception e6) {
                                                                e = e6;
                                                                FileLog.e(e);
                                                                if (!groupPhotos) {
                                                                }
                                                                groupId2 = groupId3;
                                                                final TLRPC.TL_photo tL_photo = photoFinal;
                                                                a5 = a4;
                                                                String str34 = str16;
                                                                str19 = str12;
                                                                str20 = str34;
                                                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda63
                                                                    @Override // java.lang.Runnable
                                                                    public final void run() {
                                                                        SendMessagesHelper.lambda$prepareSendingMedia$61(bitmapFinal, keyFinal, editingMessageObject, accountInstance, tL_photo, params4, parentFinal3, dialog_id, reply_to_msg, info4, notify, scheduleDate);
                                                                    }
                                                                });
                                                                sendAsDocuments3 = sendAsDocuments2;
                                                                photosCount3 = photosCount;
                                                                sendAsDocumentsCaptions = arrayList5;
                                                                sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                                sendAsDocumentsEntities = arrayList6;
                                                                a2 = a5 + 1;
                                                                arrayList2 = media;
                                                                count = count2;
                                                                str2 = str17;
                                                                str3 = str20;
                                                                workers = workers3;
                                                                str4 = str19;
                                                                enryptedLayer = enryptedLayer2;
                                                                isEncrypted4 = isEncrypted;
                                                                str = str15;
                                                                groupId8 = groupId2;
                                                            }
                                                            if (media.size() != 1) {
                                                                if (!groupPhotos) {
                                                                    photoFinal = photoFinal3;
                                                                    photo3 = photo2;
                                                                    groupId3 = groupId10;
                                                                } else {
                                                                    int photosCount6 = photosCount + 1;
                                                                    photoFinal = photoFinal3;
                                                                    photo3 = photo2;
                                                                    groupId3 = groupId10;
                                                                    params4.put("groupId", "" + groupId3);
                                                                    if (photosCount6 == 10 || a4 == count2 - 1) {
                                                                        params4.put("final", "1");
                                                                        lastGroupId = 0;
                                                                        photosCount = photosCount6;
                                                                    } else {
                                                                        photosCount = photosCount6;
                                                                    }
                                                                }
                                                                groupId2 = groupId3;
                                                                final TLRPC.TL_photo tL_photo2 = photoFinal;
                                                                a5 = a4;
                                                                String str342 = str16;
                                                                str19 = str12;
                                                                str20 = str342;
                                                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda63
                                                                    @Override // java.lang.Runnable
                                                                    public final void run() {
                                                                        SendMessagesHelper.lambda$prepareSendingMedia$61(bitmapFinal, keyFinal, editingMessageObject, accountInstance, tL_photo2, params4, parentFinal3, dialog_id, reply_to_msg, info4, notify, scheduleDate);
                                                                    }
                                                                });
                                                                sendAsDocuments3 = sendAsDocuments2;
                                                                photosCount3 = photosCount;
                                                                sendAsDocumentsCaptions = arrayList5;
                                                                sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                                sendAsDocumentsEntities = arrayList6;
                                                            }
                                                        }
                                                        TLRPC.PhotoSize currentPhotoObject = FileLoader.getClosestPhotoSizeWithSize(photoFinal3.sizes, AndroidUtilities.getPhotoSize());
                                                        if (currentPhotoObject != null) {
                                                            try {
                                                                keyFinal[0] = getKeyForPhotoSize(currentPhotoObject, bitmapFinal, false);
                                                            } catch (Exception e7) {
                                                                e = e7;
                                                                FileLog.e(e);
                                                                if (!groupPhotos) {
                                                                }
                                                                groupId2 = groupId3;
                                                                final TLRPC.TL_photo tL_photo22 = photoFinal;
                                                                a5 = a4;
                                                                String str3422 = str16;
                                                                str19 = str12;
                                                                str20 = str3422;
                                                                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda63
                                                                    @Override // java.lang.Runnable
                                                                    public final void run() {
                                                                        SendMessagesHelper.lambda$prepareSendingMedia$61(bitmapFinal, keyFinal, editingMessageObject, accountInstance, tL_photo22, params4, parentFinal3, dialog_id, reply_to_msg, info4, notify, scheduleDate);
                                                                    }
                                                                });
                                                                sendAsDocuments3 = sendAsDocuments2;
                                                                photosCount3 = photosCount;
                                                                sendAsDocumentsCaptions = arrayList5;
                                                                sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                                sendAsDocumentsEntities = arrayList6;
                                                                a2 = a5 + 1;
                                                                arrayList2 = media;
                                                                count = count2;
                                                                str2 = str17;
                                                                str3 = str20;
                                                                workers = workers3;
                                                                str4 = str19;
                                                                enryptedLayer = enryptedLayer2;
                                                                isEncrypted4 = isEncrypted;
                                                                str = str15;
                                                                groupId8 = groupId2;
                                                            }
                                                        }
                                                        if (!groupPhotos) {
                                                        }
                                                        groupId2 = groupId3;
                                                        final TLRPC.TL_photo tL_photo222 = photoFinal;
                                                        a5 = a4;
                                                        String str34222 = str16;
                                                        str19 = str12;
                                                        str20 = str34222;
                                                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda63
                                                            @Override // java.lang.Runnable
                                                            public final void run() {
                                                                SendMessagesHelper.lambda$prepareSendingMedia$61(bitmapFinal, keyFinal, editingMessageObject, accountInstance, tL_photo222, params4, parentFinal3, dialog_id, reply_to_msg, info4, notify, scheduleDate);
                                                            }
                                                        });
                                                        sendAsDocuments3 = sendAsDocuments2;
                                                        photosCount3 = photosCount;
                                                        sendAsDocumentsCaptions = arrayList5;
                                                        sendAsDocumentsOriginal3 = sendAsDocumentsOriginal6;
                                                        sendAsDocumentsEntities = arrayList6;
                                                    }
                                                }
                                            }
                                            if (tempPath4 == null && info4.uri != null) {
                                                if (MediaController.isGif(info4.uri)) {
                                                    originalPath11 = info4.uri.toString();
                                                    extension = "gif";
                                                    tempPath = MediaController.copyFileToCache(info4.uri, "gif");
                                                    isDocument = true;
                                                } else if (MediaController.isWebp(info4.uri)) {
                                                    originalPath11 = info4.uri.toString();
                                                    extension = "webp";
                                                    tempPath = MediaController.copyFileToCache(info4.uri, "webp");
                                                    isDocument = true;
                                                }
                                                if (!isDocument) {
                                                }
                                            }
                                            tempPath = tempPath4;
                                            isDocument = false;
                                            if (!isDocument) {
                                            }
                                        }
                                        extension = tempPath4 != null ? FileLoader.getFileExtension(new File(tempPath4)) : "";
                                        tempPath = tempPath4;
                                        isDocument = true;
                                        if (!isDocument) {
                                        }
                                    }
                                }
                                a2 = a5 + 1;
                                arrayList2 = media;
                                count = count2;
                                str2 = str17;
                                str3 = str20;
                                workers = workers3;
                                str4 = str19;
                                enryptedLayer = enryptedLayer2;
                                isEncrypted4 = isEncrypted;
                                str = str15;
                                groupId8 = groupId2;
                            }
                        }
                        photosCount = photosCount3;
                        groupId = groupId8;
                        int count22 = count;
                        ArrayList<String> sendAsDocuments42 = sendAsDocuments3;
                        if (info4.searchImage == null) {
                        }
                        a2 = a5 + 1;
                        arrayList2 = media;
                        count = count22;
                        str2 = str17;
                        str3 = str20;
                        workers = workers3;
                        str4 = str19;
                        enryptedLayer = enryptedLayer2;
                        isEncrypted4 = isEncrypted;
                        str = str15;
                        groupId8 = groupId2;
                    }
                    ArrayList<String> arrayList7 = sendAsDocumentsCaptions;
                    ArrayList<ArrayList<TLRPC.MessageEntity>> arrayList8 = sendAsDocumentsEntities;
                    ArrayList<String> sendAsDocumentsOriginal7 = sendAsDocumentsOriginal3;
                    sendAsDocuments = sendAsDocuments3;
                    if (lastGroupId != 0) {
                        final long lastGroupIdFinal = lastGroupId;
                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda64
                            @Override // java.lang.Runnable
                            public final void run() {
                                SendMessagesHelper.lambda$prepareSendingMedia$62(AccountInstance.this, lastGroupIdFinal, scheduleDate);
                            }
                        });
                    }
                    if (inputContent != null) {
                        inputContent.releasePermission();
                    }
                    if (sendAsDocuments == null && !sendAsDocuments.isEmpty()) {
                        int a14 = 0;
                        while (a14 < sendAsDocuments.size()) {
                            String str35 = sendAsDocuments.get(a14);
                            ArrayList<String> sendAsDocumentsOriginal8 = sendAsDocumentsOriginal7;
                            String str36 = sendAsDocumentsOriginal8.get(a14);
                            ArrayList<ArrayList<TLRPC.MessageEntity>> arrayList9 = arrayList8;
                            CharSequence charSequence = (CharSequence) arrayList9.get(a14);
                            ArrayList<String> arrayList10 = arrayList7;
                            ArrayList<String> sendAsDocumentsOriginal9 = (ArrayList) arrayList10.get(a14);
                            prepareSendingDocumentInternal(accountInstance, str35, str36, null, extension, dialog_id, reply_to_msg, charSequence, sendAsDocumentsOriginal9, editingMessageObject, forceDocument, notify, scheduleDate);
                            a14++;
                            sendAsDocumentsOriginal7 = sendAsDocumentsOriginal8;
                            arrayList7 = arrayList10;
                            arrayList8 = arrayList9;
                        }
                    }
                    if (BuildVars.LOGS_ENABLED) {
                        FileLog.d("total send time = " + (System.currentTimeMillis() - beginTime));
                        return;
                    }
                    return;
                }
                workers = null;
                long groupId82 = 0;
                lastGroupId = 0;
                String extension2 = null;
                ArrayList<String> sendAsDocumentsOriginal32 = null;
                a2 = 0;
                ArrayList<String> sendAsDocuments32 = null;
                int photosCount32 = 0;
                ArrayList<ArrayList<TLRPC.MessageEntity>> sendAsDocumentsEntities2 = null;
                ArrayList<String> sendAsDocumentsCaptions2 = null;
                while (a2 < count) {
                }
                ArrayList<String> arrayList72 = sendAsDocumentsCaptions2;
                ArrayList<ArrayList<TLRPC.MessageEntity>> arrayList82 = sendAsDocumentsEntities2;
                ArrayList<String> sendAsDocumentsOriginal72 = sendAsDocumentsOriginal32;
                sendAsDocuments = sendAsDocuments32;
                if (lastGroupId != 0) {
                }
                if (inputContent != null) {
                }
                if (sendAsDocuments == null) {
                }
                if (BuildVars.LOGS_ENABLED) {
                }
            }
        }
        enryptedLayer = 0;
        String str252 = "_ori";
        String str262 = ".webp";
        String str272 = ".gif";
        String str282 = "_";
        if (!isEncrypted4) {
        }
        if (!forceDocument) {
        }
        str = "_";
        str2 = ".gif";
        str3 = "_ori";
        str4 = ".webp";
        workers = null;
        long groupId822 = 0;
        lastGroupId = 0;
        String extension22 = null;
        ArrayList<String> sendAsDocumentsOriginal322 = null;
        a2 = 0;
        ArrayList<String> sendAsDocuments322 = null;
        int photosCount322 = 0;
        ArrayList<ArrayList<TLRPC.MessageEntity>> sendAsDocumentsEntities22 = null;
        ArrayList<String> sendAsDocumentsCaptions22 = null;
        while (a2 < count) {
        }
        ArrayList<String> arrayList722 = sendAsDocumentsCaptions22;
        ArrayList<ArrayList<TLRPC.MessageEntity>> arrayList822 = sendAsDocumentsEntities22;
        ArrayList<String> sendAsDocumentsOriginal722 = sendAsDocumentsOriginal322;
        sendAsDocuments = sendAsDocuments322;
        if (lastGroupId != 0) {
        }
        if (inputContent != null) {
        }
        if (sendAsDocuments == null) {
        }
        if (BuildVars.LOGS_ENABLED) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingMedia$57(MediaSendPrepareWorker worker, AccountInstance accountInstance, SendingMediaInfo info, boolean blnOriginalImg, boolean isEncrypted) {
        worker.photo = accountInstance.getSendMessagesHelper().generatePhotoSizes(info.path, info.uri, blnOriginalImg);
        if (isEncrypted && info.canDeleteAfter) {
            new File(info.path).delete();
        }
        worker.sync.countDown();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingMedia$58(MessageObject editingMessageObject, AccountInstance accountInstance, TLRPC.TL_document documentFinal, String pathFinal, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, SendingMediaInfo info, boolean notify, int scheduleDate) {
        if (editingMessageObject == null) {
            accountInstance.getSendMessagesHelper().sendMessage(documentFinal, null, pathFinal, dialog_id, reply_to_msg, info.caption, info.entities, null, params, notify, scheduleDate, 0, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, null, null, documentFinal, pathFinal, params, false, parentFinal);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingMedia$59(MessageObject editingMessageObject, AccountInstance accountInstance, TLRPC.TL_photo photoFinal, boolean needDownloadHttpFinal, SendingMediaInfo info, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, boolean notify, int scheduleDate) {
        if (editingMessageObject != null) {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, photoFinal, null, null, needDownloadHttpFinal ? info.searchImage.imageUrl : null, params, false, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().sendMessage(photoFinal, needDownloadHttpFinal ? info.searchImage.imageUrl : null, dialog_id, reply_to_msg, info.caption, info.entities, null, params, notify, scheduleDate, info.ttl, parentFinal);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingMedia$60(Bitmap thumbFinal, String thumbKeyFinal, MessageObject editingMessageObject, AccountInstance accountInstance, VideoEditedInfo videoEditedInfo, TLRPC.TL_document videoFinal, String finalPath, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, SendingMediaInfo info, boolean notify, int scheduleDate) {
        if (thumbFinal != null && thumbKeyFinal != null) {
            ImageLoader.getInstance().putImageToCache(new BitmapDrawable(thumbFinal), thumbKeyFinal);
        }
        if (editingMessageObject != null) {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, null, videoEditedInfo, videoFinal, finalPath, params, false, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().sendMessage(videoFinal, videoEditedInfo, finalPath, dialog_id, reply_to_msg, info.caption, info.entities, null, params, notify, scheduleDate, info.ttl, parentFinal);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingMedia$61(Bitmap[] bitmapFinal, String[] keyFinal, MessageObject editingMessageObject, AccountInstance accountInstance, TLRPC.TL_photo photoFinal, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, SendingMediaInfo info, boolean notify, int scheduleDate) {
        if (bitmapFinal[0] != null && keyFinal[0] != null) {
            ImageLoader.getInstance().putImageToCache(new BitmapDrawable(bitmapFinal[0]), keyFinal[0]);
        }
        if (editingMessageObject == null) {
            accountInstance.getSendMessagesHelper().sendMessage(photoFinal, null, dialog_id, reply_to_msg, info.caption, info.entities, null, params, notify, scheduleDate, info.ttl, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, photoFinal, null, null, null, params, false, parentFinal);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingMedia$62(AccountInstance accountInstance, long lastGroupIdFinal, int scheduleDate) {
        SendMessagesHelper instance = accountInstance.getSendMessagesHelper();
        ArrayList<DelayedMessage> arrayList = instance.delayedMessages.get("group_" + lastGroupIdFinal);
        if (arrayList != null && !arrayList.isEmpty()) {
            DelayedMessage message = arrayList.get(0);
            MessageObject prevMessage = message.messageObjects.get(message.messageObjects.size() - 1);
            message.finalGroupMessage = prevMessage.getId();
            prevMessage.messageOwner.params.put("final", "1");
            TLRPC.TL_messages_messages messagesRes = new TLRPC.TL_messages_messages();
            messagesRes.messages.add(prevMessage.messageOwner);
            accountInstance.getMessagesStorage().putMessages((TLRPC.messages_Messages) messagesRes, message.peer, -2, 0, false, scheduleDate != 0);
            instance.sendReadyToSendGroup(message, true, true);
        }
    }

    private static void fillVideoAttribute(String videoPath, TLRPC.TL_documentAttributeVideo attributeVideo, VideoEditedInfo videoEditedInfo) {
        boolean infoObtained = false;
        MediaMetadataRetriever mediaMetadataRetriever = null;
        try {
            try {
                try {
                    MediaMetadataRetriever mediaMetadataRetriever2 = new MediaMetadataRetriever();
                    mediaMetadataRetriever2.setDataSource(videoPath);
                    String width = mediaMetadataRetriever2.extractMetadata(18);
                    if (width != null) {
                        attributeVideo.w = Integer.parseInt(width);
                    }
                    String height = mediaMetadataRetriever2.extractMetadata(19);
                    if (height != null) {
                        attributeVideo.h = Integer.parseInt(height);
                    }
                    String duration = mediaMetadataRetriever2.extractMetadata(9);
                    if (duration != null) {
                        attributeVideo.duration = (int) Math.ceil(((float) Long.parseLong(duration)) / 1000.0f);
                    }
                    String rotation = mediaMetadataRetriever2.extractMetadata(24);
                    if (rotation != null) {
                        int val = Utilities.parseInt(rotation).intValue();
                        if (videoEditedInfo != null) {
                            videoEditedInfo.rotationValue = val;
                        } else if (val == 90 || val == 270) {
                            int temp = attributeVideo.w;
                            attributeVideo.w = attributeVideo.h;
                            attributeVideo.h = temp;
                        }
                    }
                    infoObtained = true;
                    mediaMetadataRetriever2.release();
                } catch (Throwable th) {
                    if (0 != 0) {
                        try {
                            mediaMetadataRetriever.release();
                        } catch (Exception e) {
                            FileLog.e(e);
                        }
                    }
                    throw th;
                }
            } catch (Exception e2) {
                FileLog.e(e2);
                if (0 != 0) {
                    mediaMetadataRetriever.release();
                }
            }
        } catch (Exception e3) {
            FileLog.e(e3);
        }
        if (!infoObtained) {
            try {
                MediaPlayer mp = MediaPlayer.create(ApplicationLoader.applicationContext, Uri.fromFile(new File(videoPath)));
                if (mp != null) {
                    attributeVideo.duration = (int) Math.ceil(mp.getDuration() / 1000.0f);
                    attributeVideo.w = mp.getVideoWidth();
                    attributeVideo.h = mp.getVideoHeight();
                    mp.release();
                }
            } catch (Exception e4) {
                FileLog.e(e4);
            }
        }
    }

    private static Bitmap createVideoThumbnail(String filePath, long time) {
        Bitmap bitmap = null;
        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        try {
            try {
                retriever.setDataSource(filePath);
                bitmap = retriever.getFrameAtTime(time, 1);
                retriever.release();
            } catch (Exception e) {
                retriever.release();
            } catch (Throwable th) {
                try {
                    retriever.release();
                } catch (IOException e2) {
                } catch (RuntimeException e3) {
                }
                throw th;
            }
        } catch (IOException e4) {
        } catch (RuntimeException e5) {
        }
        if (bitmap == null) {
            return null;
        }
        return bitmap;
    }

    private static VideoEditedInfo createCompressionSettings(String videoPath) {
        int compressionsCount;
        float maxSize;
        float targetBitrate;
        int[] params = new int[9];
        AnimatedFileDrawable.getVideoInfo(videoPath, params);
        if (params[0] == 0) {
            if (BuildVars.LOGS_ENABLED) {
                FileLog.d("video hasn't avc1 atom");
                return null;
            }
            return null;
        }
        int originalBitrate = params[3];
        int bitrate = params[3];
        float videoDuration = params[4];
        long videoFramesSize = params[6];
        long audioFramesSize = params[5];
        int videoFramerate = params[7];
        if (bitrate > 900000) {
            bitrate = 900000;
        }
        VideoEditedInfo videoEditedInfo = new VideoEditedInfo();
        videoEditedInfo.startTime = -1L;
        videoEditedInfo.endTime = -1L;
        videoEditedInfo.bitrate = bitrate;
        videoEditedInfo.originalPath = videoPath;
        videoEditedInfo.framerate = videoFramerate;
        videoEditedInfo.estimatedDuration = (long) Math.ceil(videoDuration);
        int i = params[1];
        videoEditedInfo.originalWidth = i;
        videoEditedInfo.resultWidth = i;
        int i2 = params[2];
        videoEditedInfo.originalHeight = i2;
        videoEditedInfo.resultHeight = i2;
        videoEditedInfo.rotationValue = params[8];
        SharedPreferences preferences = MessagesController.getGlobalMainSettings();
        int selectedCompression = preferences.getInt("compress_video2", 1);
        if (videoEditedInfo.originalWidth > 1280 || videoEditedInfo.originalHeight > 1280) {
            compressionsCount = 5;
        } else if (videoEditedInfo.originalWidth > 848 || videoEditedInfo.originalHeight > 848) {
            compressionsCount = 4;
        } else if (videoEditedInfo.originalWidth > 640 || videoEditedInfo.originalHeight > 640) {
            compressionsCount = 3;
        } else if (videoEditedInfo.originalWidth > 480 || videoEditedInfo.originalHeight > 480) {
            compressionsCount = 2;
        } else {
            compressionsCount = 1;
        }
        if (selectedCompression >= compressionsCount) {
            selectedCompression = compressionsCount - 1;
        }
        if (selectedCompression != compressionsCount - 1) {
            switch (selectedCompression) {
                case 0:
                    maxSize = 432.0f;
                    targetBitrate = 400000;
                    break;
                case 1:
                    maxSize = 640.0f;
                    targetBitrate = 900000;
                    break;
                case 2:
                    maxSize = 848.0f;
                    targetBitrate = 1100000;
                    break;
                default:
                    maxSize = 1280.0f;
                    targetBitrate = 2500000;
                    break;
            }
            float scale = maxSize / (videoEditedInfo.originalWidth > videoEditedInfo.originalHeight ? videoEditedInfo.originalWidth : videoEditedInfo.originalHeight);
            videoEditedInfo.resultWidth = Math.round((videoEditedInfo.originalWidth * scale) / 2.0f) * 2;
            videoEditedInfo.resultHeight = Math.round((videoEditedInfo.originalHeight * scale) / 2.0f) * 2;
            if (bitrate != 0) {
                bitrate = Math.min(targetBitrate, (int) (originalBitrate / scale));
                videoFramesSize = ((bitrate / 8) * videoDuration) / 1000.0f;
            }
        }
        if (selectedCompression == compressionsCount - 1) {
            videoEditedInfo.resultWidth = videoEditedInfo.originalWidth;
            videoEditedInfo.resultHeight = videoEditedInfo.originalHeight;
            videoEditedInfo.bitrate = originalBitrate;
            videoEditedInfo.estimatedSize = (int) new File(videoPath).length();
        } else {
            videoEditedInfo.bitrate = bitrate;
            videoEditedInfo.estimatedSize = (int) (audioFramesSize + videoFramesSize);
            videoEditedInfo.estimatedSize += (videoEditedInfo.estimatedSize / 32768) * 16;
        }
        if (videoEditedInfo.estimatedSize == 0) {
            videoEditedInfo.estimatedSize = 1L;
        }
        return videoEditedInfo;
    }

    public static void prepareSendingVideo(final AccountInstance accountInstance, final String videoPath, final long estimatedSize, final long duration, final int width, final int height, final VideoEditedInfo info, final long dialog_id, final MessageObject reply_to_msg, final CharSequence caption, final ArrayList<TLRPC.MessageEntity> entities, final int ttl, final MessageObject editingMessageObject, final boolean notify, final int scheduleDate) {
        if (videoPath == null || videoPath.length() == 0) {
            return;
        }
        new Thread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda9
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.lambda$prepareSendingVideo$65(VideoEditedInfo.this, videoPath, dialog_id, duration, ttl, accountInstance, height, width, estimatedSize, caption, editingMessageObject, reply_to_msg, entities, notify, scheduleDate);
            }
        }).start();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:113:0x0316  */
    /* JADX WARN: Removed duplicated region for block: B:121:0x0367  */
    /* JADX WARN: Removed duplicated region for block: B:122:0x036e  */
    /* JADX WARN: Removed duplicated region for block: B:124:0x0372  */
    /* JADX WARN: Removed duplicated region for block: B:55:0x015f  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static /* synthetic */ void lambda$prepareSendingVideo$65(VideoEditedInfo info, String videoPath, final long dialog_id, long duration, final int ttl, final AccountInstance accountInstance, int height, int width, long estimatedSize, CharSequence caption, final MessageObject editingMessageObject, final MessageObject reply_to_msg, final ArrayList entities, final boolean notify, final int scheduleDate) {
        boolean isRound;
        boolean isEncrypted;
        VideoEditedInfo videoEditedInfo;
        String path;
        boolean isRound2;
        String originalPath;
        long startTime;
        TLRPC.TL_document document;
        long startTime2;
        TLRPC.TL_document document2;
        Bitmap thumb;
        String thumbKey;
        String path2;
        TLRPC.TL_documentAttributeVideo attributeVideo;
        int i;
        int i2;
        VideoEditedInfo videoEditedInfo2 = info != null ? info : createCompressionSettings(videoPath);
        boolean isEncrypted2 = ((int) dialog_id) == 0;
        boolean isRound3 = videoEditedInfo2 != null && videoEditedInfo2.roundVideo;
        String thumbKey2 = null;
        if (videoEditedInfo2 != null || videoPath.endsWith("mp4")) {
            isRound = isRound3;
            isEncrypted = isEncrypted2;
            videoEditedInfo = videoEditedInfo2;
        } else if (!isRound3) {
            prepareSendingDocumentInternal(accountInstance, videoPath, videoPath, null, null, dialog_id, reply_to_msg, caption, entities, editingMessageObject, false, notify, scheduleDate);
            return;
        } else {
            isRound = isRound3;
            isEncrypted = isEncrypted2;
            videoEditedInfo = videoEditedInfo2;
        }
        File temp = new File(videoPath);
        String originalPath2 = videoPath + temp.length() + "_" + temp.lastModified();
        final VideoEditedInfo videoEditedInfo3 = videoEditedInfo;
        if (videoEditedInfo3 == null) {
            path = videoPath;
            isRound2 = isRound;
            originalPath = originalPath2;
            startTime = 0;
        } else {
            isRound2 = isRound;
            if (!isRound2) {
                path = videoPath;
                originalPath2 = originalPath2 + duration + "_" + videoEditedInfo3.startTime + "_" + videoEditedInfo3.endTime + (videoEditedInfo3.muted ? "_m" : "");
                if (videoEditedInfo3.resultWidth != videoEditedInfo3.originalWidth) {
                    originalPath2 = originalPath2 + "_" + videoEditedInfo3.resultWidth;
                }
            } else {
                path = videoPath;
            }
            long startTime3 = videoEditedInfo3.startTime >= 0 ? videoEditedInfo3.startTime : 0L;
            originalPath = originalPath2;
            startTime = startTime3;
        }
        String parentObject = null;
        boolean isEncrypted3 = isEncrypted;
        if (isEncrypted3 || ttl != 0) {
            document = null;
            startTime2 = startTime;
        } else {
            MessagesStorage messagesStorage = accountInstance.getMessagesStorage();
            if (!isEncrypted3) {
                document = null;
                i2 = 2;
            } else {
                document = null;
                i2 = 5;
            }
            Object[] sentData = messagesStorage.getSentFile(originalPath, i2);
            if (sentData == null) {
                startTime2 = startTime;
            } else {
                TLRPC.TL_document document3 = (TLRPC.TL_document) sentData[0];
                String parentObject2 = (String) sentData[1];
                startTime2 = startTime;
                ensureMediaThumbExists(isEncrypted3, document3, videoPath, null, startTime2);
                document2 = document3;
                parentObject = parentObject2;
                if (document2 != null) {
                    Bitmap thumb2 = createVideoThumbnail(videoPath, startTime2);
                    if (thumb2 == null) {
                        thumb2 = ThumbnailUtils.createVideoThumbnail(videoPath, 1);
                    }
                    int side = (isEncrypted3 || ttl != 0) ? 90 : 320;
                    TLRPC.PhotoSize size = ImageLoader.scaleAndSaveImage(thumb2, side, side, side > 90 ? 80 : 55, isEncrypted3);
                    if (thumb2 != null && size != null) {
                        if (!isRound2) {
                            thumb2 = null;
                        } else if (isEncrypted3) {
                            Utilities.blurBitmap(thumb2, 7, 1, thumb2.getWidth(), thumb2.getHeight(), thumb2.getRowBytes());
                            thumbKey2 = String.format(size.location.volume_id + "_" + size.location.local_id + "@%d_%d_b2", Integer.valueOf((int) (AndroidUtilities.roundMessageSize / AndroidUtilities.density)), Integer.valueOf((int) (AndroidUtilities.roundMessageSize / AndroidUtilities.density)));
                        } else {
                            Utilities.blurBitmap(thumb2, 3, 1, thumb2.getWidth(), thumb2.getHeight(), thumb2.getRowBytes());
                            thumbKey2 = String.format(size.location.volume_id + "_" + size.location.local_id + "@%d_%d_b", Integer.valueOf((int) (AndroidUtilities.roundMessageSize / AndroidUtilities.density)), Integer.valueOf((int) (AndroidUtilities.roundMessageSize / AndroidUtilities.density)));
                        }
                    }
                    TLRPC.TL_document document4 = new TLRPC.TL_document();
                    if (size != null) {
                        document4.thumbs.add(size);
                        document4.flags |= 1;
                    }
                    document4.file_reference = new byte[0];
                    document4.mime_type = MimeTypes.VIDEO_MP4;
                    accountInstance.getUserConfig().saveConfig(false);
                    if (isEncrypted3) {
                        int high_id = (int) (dialog_id >> 32);
                        TLRPC.EncryptedChat encryptedChat = accountInstance.getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
                        if (encryptedChat == null) {
                            return;
                        }
                        if (AndroidUtilities.getPeerLayerVersion(encryptedChat.layer) >= 66) {
                            attributeVideo = new TLRPC.TL_documentAttributeVideo();
                        } else {
                            attributeVideo = new TLRPC.TL_documentAttributeVideo_layer65();
                        }
                    } else {
                        attributeVideo = new TLRPC.TL_documentAttributeVideo();
                        attributeVideo.supports_streaming = true;
                    }
                    attributeVideo.round_message = isRound2;
                    document4.attributes.add(attributeVideo);
                    if (videoEditedInfo3 == null || !videoEditedInfo3.needConvert()) {
                        thumb = thumb2;
                        if (temp.exists()) {
                            document4.size = (int) temp.length();
                        }
                        fillVideoAttribute(videoPath, attributeVideo, null);
                    } else {
                        if (videoEditedInfo3.muted) {
                            document4.attributes.add(new TLRPC.TL_documentAttributeAnimated());
                            fillVideoAttribute(videoPath, attributeVideo, videoEditedInfo3);
                            videoEditedInfo3.originalWidth = attributeVideo.w;
                            videoEditedInfo3.originalHeight = attributeVideo.h;
                            thumb = thumb2;
                        } else {
                            thumb = thumb2;
                            attributeVideo.duration = (int) (duration / 1000);
                        }
                        if (videoEditedInfo3.rotationValue == 90) {
                            i = width;
                        } else if (videoEditedInfo3.rotationValue == 270) {
                            i = width;
                        } else {
                            attributeVideo.w = width;
                            attributeVideo.h = height;
                            document4.size = (int) estimatedSize;
                        }
                        attributeVideo.w = height;
                        attributeVideo.h = i;
                        document4.size = (int) estimatedSize;
                    }
                    document2 = document4;
                    thumbKey = thumbKey2;
                } else {
                    thumb = null;
                    thumbKey = null;
                }
                if (videoEditedInfo3 == null && videoEditedInfo3.needConvert()) {
                    String fileName = "-2147483648_" + SharedConfig.getLastLocalId() + ".mp4";
                    File cacheFile = new File(FileLoader.getDirectory(4), fileName);
                    SharedConfig.saveConfig();
                    path2 = cacheFile.getAbsolutePath();
                } else {
                    path2 = path;
                }
                final TLRPC.TL_document videoFinal = document2;
                final String parentFinal = parentObject;
                final String finalPath = path2;
                final HashMap<String, String> params = new HashMap<>();
                final Bitmap thumbFinal = thumb;
                final String thumbKeyFinal = thumbKey;
                final String captionFinal = caption == null ? caption.toString() : "";
                if (originalPath != null) {
                    params.put("originalPath", originalPath);
                }
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda24
                    @Override // java.lang.Runnable
                    public final void run() {
                        SendMessagesHelper.lambda$prepareSendingVideo$64(thumbFinal, thumbKeyFinal, editingMessageObject, accountInstance, videoEditedInfo3, videoFinal, finalPath, params, parentFinal, dialog_id, reply_to_msg, captionFinal, entities, notify, scheduleDate, ttl);
                    }
                });
            }
        }
        document2 = document;
        if (document2 != null) {
        }
        if (videoEditedInfo3 == null) {
        }
        path2 = path;
        final TLRPC.TL_document videoFinal2 = document2;
        final String parentFinal2 = parentObject;
        final String finalPath2 = path2;
        final HashMap params2 = new HashMap<>();
        final Bitmap thumbFinal2 = thumb;
        final String thumbKeyFinal2 = thumbKey;
        if (caption == null) {
        }
        if (originalPath != null) {
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.SendMessagesHelper$$ExternalSyntheticLambda24
            @Override // java.lang.Runnable
            public final void run() {
                SendMessagesHelper.lambda$prepareSendingVideo$64(thumbFinal2, thumbKeyFinal2, editingMessageObject, accountInstance, videoEditedInfo3, videoFinal2, finalPath2, params2, parentFinal2, dialog_id, reply_to_msg, captionFinal, entities, notify, scheduleDate, ttl);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$prepareSendingVideo$64(Bitmap thumbFinal, String thumbKeyFinal, MessageObject editingMessageObject, AccountInstance accountInstance, VideoEditedInfo videoEditedInfo, TLRPC.TL_document videoFinal, String finalPath, HashMap params, String parentFinal, long dialog_id, MessageObject reply_to_msg, String captionFinal, ArrayList entities, boolean notify, int scheduleDate, int ttl) {
        if (thumbFinal != null && thumbKeyFinal != null) {
            ImageLoader.getInstance().putImageToCache(new BitmapDrawable(thumbFinal), thumbKeyFinal);
        }
        if (editingMessageObject != null) {
            accountInstance.getSendMessagesHelper().editMessageMedia(editingMessageObject, null, videoEditedInfo, videoFinal, finalPath, params, false, parentFinal);
        } else {
            accountInstance.getSendMessagesHelper().sendMessage(videoFinal, videoEditedInfo, finalPath, dialog_id, reply_to_msg, captionFinal, entities, null, params, notify, scheduleDate, ttl, parentFinal);
        }
    }
}
