package im.uecahxyvjd.messenger;

import android.content.SharedPreferences;
import com.google.android.exoplayer2.trackselection.AdaptiveTrackSelection;
import java.io.File;
import java.io.RandomAccessFile;
import java.lang.reflect.Array;
import kotlin.UByte;
/* loaded from: classes16.dex */
public class StatsController extends BaseController {
    private static final int TYPES_COUNT = 7;
    public static final int TYPE_AUDIOS = 3;
    public static final int TYPE_CALLS = 0;
    public static final int TYPE_FILES = 5;
    public static final int TYPE_MESSAGES = 1;
    public static final int TYPE_MOBILE = 0;
    public static final int TYPE_PHOTOS = 4;
    public static final int TYPE_ROAMING = 2;
    public static final int TYPE_TOTAL = 6;
    public static final int TYPE_VIDEOS = 2;
    public static final int TYPE_WIFI = 1;
    private byte[] buffer;
    private int[] callsTotalTime;
    private long lastInternalStatsSaveTime;
    private long[][] receivedBytes;
    private int[][] receivedItems;
    private long[] resetStatsDate;
    private Runnable saveRunnable;
    private long[][] sentBytes;
    private int[][] sentItems;
    private RandomAccessFile statsFile;
    private static DispatchQueue statsSaveQueue = new DispatchQueue("statsSaveQueue");
    private static final ThreadLocal<Long> lastStatsSaveTime = new ThreadLocal<Long>() { // from class: im.uecahxyvjd.messenger.StatsController.1
        /* JADX INFO: Access modifiers changed from: protected */
        /* JADX WARN: Can't rename method to resolve collision */
        @Override // java.lang.ThreadLocal
        public Long initialValue() {
            return Long.valueOf(System.currentTimeMillis() - 1000);
        }
    };
    private static volatile StatsController[] Instance = new StatsController[3];

    /* JADX INFO: Access modifiers changed from: private */
    public byte[] intToBytes(int value) {
        byte[] bArr = this.buffer;
        bArr[0] = (byte) (value >>> 24);
        bArr[1] = (byte) (value >>> 16);
        bArr[2] = (byte) (value >>> 8);
        bArr[3] = (byte) value;
        return bArr;
    }

    private int bytesToInt(byte[] bytes) {
        return (bytes[0] << 24) | ((bytes[1] & UByte.MAX_VALUE) << 16) | ((bytes[2] & UByte.MAX_VALUE) << 8) | (bytes[3] & UByte.MAX_VALUE);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public byte[] longToBytes(long value) {
        byte[] bArr = this.buffer;
        bArr[0] = (byte) (value >>> 56);
        bArr[1] = (byte) (value >>> 48);
        bArr[2] = (byte) (value >>> 40);
        bArr[3] = (byte) (value >>> 32);
        bArr[4] = (byte) (value >>> 24);
        bArr[5] = (byte) (value >>> 16);
        bArr[6] = (byte) (value >>> 8);
        bArr[7] = (byte) value;
        return bArr;
    }

    private long bytesToLong(byte[] bytes) {
        return ((bytes[0] & 255) << 56) | ((bytes[1] & 255) << 48) | ((bytes[2] & 255) << 40) | ((bytes[3] & 255) << 32) | ((bytes[4] & 255) << 24) | ((bytes[5] & 255) << 16) | ((bytes[6] & 255) << 8) | (255 & bytes[7]);
    }

    public static StatsController getInstance(int num) {
        StatsController localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (StatsController.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    StatsController[] statsControllerArr = Instance;
                    StatsController statsController = new StatsController(num);
                    localInstance = statsController;
                    statsControllerArr[num] = statsController;
                }
            }
        }
        return localInstance;
    }

    private StatsController(int account) {
        super(account);
        this.buffer = new byte[8];
        this.sentBytes = (long[][]) Array.newInstance(Long.TYPE, 3, 7);
        this.receivedBytes = (long[][]) Array.newInstance(Long.TYPE, 3, 7);
        this.sentItems = (int[][]) Array.newInstance(Integer.TYPE, 3, 7);
        this.receivedItems = (int[][]) Array.newInstance(Integer.TYPE, 3, 7);
        this.resetStatsDate = new long[3];
        this.callsTotalTime = new int[3];
        this.saveRunnable = new Runnable() { // from class: im.uecahxyvjd.messenger.StatsController.2
            @Override // java.lang.Runnable
            public void run() {
                long newTime = System.currentTimeMillis();
                if (Math.abs(newTime - StatsController.this.lastInternalStatsSaveTime) >= AdaptiveTrackSelection.DEFAULT_MIN_TIME_BETWEEN_BUFFER_REEVALUTATION_MS) {
                    StatsController.this.lastInternalStatsSaveTime = newTime;
                    try {
                        StatsController.this.statsFile.seek(0L);
                        for (int a2 = 0; a2 < 3; a2++) {
                            for (int b = 0; b < 7; b++) {
                                RandomAccessFile randomAccessFile = StatsController.this.statsFile;
                                StatsController statsController = StatsController.this;
                                randomAccessFile.write(statsController.longToBytes(statsController.sentBytes[a2][b]), 0, 8);
                                RandomAccessFile randomAccessFile2 = StatsController.this.statsFile;
                                StatsController statsController2 = StatsController.this;
                                randomAccessFile2.write(statsController2.longToBytes(statsController2.receivedBytes[a2][b]), 0, 8);
                                RandomAccessFile randomAccessFile3 = StatsController.this.statsFile;
                                StatsController statsController3 = StatsController.this;
                                randomAccessFile3.write(statsController3.intToBytes(statsController3.sentItems[a2][b]), 0, 4);
                                RandomAccessFile randomAccessFile4 = StatsController.this.statsFile;
                                StatsController statsController4 = StatsController.this;
                                randomAccessFile4.write(statsController4.intToBytes(statsController4.receivedItems[a2][b]), 0, 4);
                            }
                            RandomAccessFile randomAccessFile5 = StatsController.this.statsFile;
                            StatsController statsController5 = StatsController.this;
                            randomAccessFile5.write(statsController5.intToBytes(statsController5.callsTotalTime[a2]), 0, 4);
                            RandomAccessFile randomAccessFile6 = StatsController.this.statsFile;
                            StatsController statsController6 = StatsController.this;
                            randomAccessFile6.write(statsController6.longToBytes(statsController6.resetStatsDate[a2]), 0, 8);
                        }
                        StatsController.this.statsFile.getFD().sync();
                    } catch (Exception e) {
                    }
                }
            }
        };
        File filesDir = ApplicationLoader.getFilesDirFixed();
        if (account != 0) {
            filesDir = new File(ApplicationLoader.getFilesDirFixed(), "account" + account + "/");
            filesDir.mkdirs();
        }
        boolean needConvert = true;
        try {
            RandomAccessFile randomAccessFile = new RandomAccessFile(new File(filesDir, "stats2.dat"), "rw");
            this.statsFile = randomAccessFile;
            if (randomAccessFile.length() > 0) {
                boolean save = false;
                for (int a2 = 0; a2 < 3; a2++) {
                    for (int b = 0; b < 7; b++) {
                        this.statsFile.readFully(this.buffer, 0, 8);
                        this.sentBytes[a2][b] = bytesToLong(this.buffer);
                        this.statsFile.readFully(this.buffer, 0, 8);
                        this.receivedBytes[a2][b] = bytesToLong(this.buffer);
                        this.statsFile.readFully(this.buffer, 0, 4);
                        this.sentItems[a2][b] = bytesToInt(this.buffer);
                        this.statsFile.readFully(this.buffer, 0, 4);
                        this.receivedItems[a2][b] = bytesToInt(this.buffer);
                    }
                    this.statsFile.readFully(this.buffer, 0, 4);
                    this.callsTotalTime[a2] = bytesToInt(this.buffer);
                    this.statsFile.readFully(this.buffer, 0, 8);
                    this.resetStatsDate[a2] = bytesToLong(this.buffer);
                    long[] jArr = this.resetStatsDate;
                    if (jArr[a2] == 0) {
                        save = true;
                        jArr[a2] = System.currentTimeMillis();
                    }
                }
                if (save) {
                    saveStats();
                }
                needConvert = false;
            }
        } catch (Exception e) {
        }
        if (needConvert) {
            SharedPreferences sharedPreferences = account == 0 ? ApplicationLoader.applicationContext.getSharedPreferences("stats", 0) : ApplicationLoader.applicationContext.getSharedPreferences("stats" + account, 0);
            boolean save2 = false;
            int a3 = 0;
            for (int i = 3; a3 < i; i = 3) {
                this.callsTotalTime[a3] = sharedPreferences.getInt("callsTotalTime" + a3, 0);
                this.resetStatsDate[a3] = sharedPreferences.getLong("resetStatsDate" + a3, 0L);
                for (int b2 = 0; b2 < 7; b2++) {
                    this.sentBytes[a3][b2] = sharedPreferences.getLong("sentBytes" + a3 + "_" + b2, 0L);
                    this.receivedBytes[a3][b2] = sharedPreferences.getLong("receivedBytes" + a3 + "_" + b2, 0L);
                    this.sentItems[a3][b2] = sharedPreferences.getInt("sentItems" + a3 + "_" + b2, 0);
                    this.receivedItems[a3][b2] = sharedPreferences.getInt("receivedItems" + a3 + "_" + b2, 0);
                }
                long[] jArr2 = this.resetStatsDate;
                if (jArr2[a3] == 0) {
                    save2 = true;
                    jArr2[a3] = System.currentTimeMillis();
                }
                a3++;
            }
            if (save2) {
                saveStats();
            }
        }
    }

    public void incrementReceivedItemsCount(int networkType, int dataType, int value) {
        int[] iArr = this.receivedItems[networkType];
        iArr[dataType] = iArr[dataType] + value;
        saveStats();
    }

    public void incrementSentItemsCount(int networkType, int dataType, int value) {
        int[] iArr = this.sentItems[networkType];
        iArr[dataType] = iArr[dataType] + value;
        saveStats();
    }

    public void incrementReceivedBytesCount(int networkType, int dataType, long value) {
        long[] jArr = this.receivedBytes[networkType];
        jArr[dataType] = jArr[dataType] + value;
        saveStats();
    }

    public void incrementSentBytesCount(int networkType, int dataType, long value) {
        long[] jArr = this.sentBytes[networkType];
        jArr[dataType] = jArr[dataType] + value;
        saveStats();
    }

    public void incrementTotalCallsTime(int networkType, int value) {
        int[] iArr = this.callsTotalTime;
        iArr[networkType] = iArr[networkType] + value;
        saveStats();
    }

    public int getRecivedItemsCount(int networkType, int dataType) {
        return this.receivedItems[networkType][dataType];
    }

    public int getSentItemsCount(int networkType, int dataType) {
        return this.sentItems[networkType][dataType];
    }

    public long getSentBytesCount(int networkType, int dataType) {
        if (dataType == 1) {
            long[] jArr = this.sentBytes[networkType];
            return (((jArr[6] - jArr[5]) - jArr[3]) - jArr[2]) - jArr[4];
        }
        return this.sentBytes[networkType][dataType];
    }

    public long getReceivedBytesCount(int networkType, int dataType) {
        if (dataType == 1) {
            long[] jArr = this.receivedBytes[networkType];
            return (((jArr[6] - jArr[5]) - jArr[3]) - jArr[2]) - jArr[4];
        }
        return this.receivedBytes[networkType][dataType];
    }

    public int getCallsTotalTime(int networkType) {
        return this.callsTotalTime[networkType];
    }

    public long getResetStatsDate(int networkType) {
        return this.resetStatsDate[networkType];
    }

    public void resetStats(int networkType) {
        this.resetStatsDate[networkType] = System.currentTimeMillis();
        for (int a2 = 0; a2 < 7; a2++) {
            this.sentBytes[networkType][a2] = 0;
            this.receivedBytes[networkType][a2] = 0;
            this.sentItems[networkType][a2] = 0;
            this.receivedItems[networkType][a2] = 0;
        }
        this.callsTotalTime[networkType] = 0;
        saveStats();
    }

    private void saveStats() {
        long newTime = System.currentTimeMillis();
        ThreadLocal<Long> threadLocal = lastStatsSaveTime;
        if (Math.abs(newTime - threadLocal.get().longValue()) >= AdaptiveTrackSelection.DEFAULT_MIN_TIME_BETWEEN_BUFFER_REEVALUTATION_MS) {
            threadLocal.set(Long.valueOf(newTime));
            statsSaveQueue.postRunnable(this.saveRunnable);
        }
    }
}
