package im.uecahxyvjd.messenger;

import android.content.SharedPreferences;
import android.os.SystemClock;
import android.util.Base64;
import im.uecahxyvjd.tgnet.SerializedData;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.utils.AppUpdater;
import java.io.File;
/* loaded from: classes16.dex */
public class UserConfig extends BaseController {
    private static volatile UserConfig[] Instance = new UserConfig[3];
    public static final int MAX_ACCOUNT_COUNT = 3;
    public static final int i_dialogsLoadOffsetAccess_1 = 5;
    public static final int i_dialogsLoadOffsetAccess_2 = 6;
    public static final int i_dialogsLoadOffsetChannelId = 4;
    public static final int i_dialogsLoadOffsetChatId = 3;
    public static final int i_dialogsLoadOffsetDate = 1;
    public static final int i_dialogsLoadOffsetId = 0;
    public static final int i_dialogsLoadOffsetUserId = 2;
    public static int selectedAccount;
    public long autoDownloadConfigLoadTime;
    public int botRatingLoadTime;
    public int clientUserId;
    private boolean configLoaded;
    public boolean contactsReimported;
    public int contactsSavedCount;
    private TLRPC.User currentUser;
    public boolean draftsLoaded;
    public boolean hasSecureData;
    public boolean hasValidDialogLoadIds;
    public int isCdnVip;
    public int lastBroadcastId;
    public int lastContactsSyncTime;
    public int lastHintsSyncTime;
    public int lastSendMessageId;
    public int loginTime;
    public long migrateOffsetAccess;
    public int migrateOffsetChannelId;
    public int migrateOffsetChatId;
    public int migrateOffsetDate;
    public int migrateOffsetId;
    public int migrateOffsetUserId;
    public boolean notificationsSettingsLoaded;
    public boolean notificationsSignUpSettingsLoaded;
    public int ratingLoadTime;
    public boolean registeredForPush;
    public volatile byte[] savedPasswordHash;
    public volatile long savedPasswordTime;
    public volatile byte[] savedSaltedPassword;
    public boolean suggestContacts;
    private final Object sync;
    public boolean syncContacts;
    public TLRPC.TL_account_tmpPassword tmpPassword;
    public TLRPC.TL_help_termsOfService unacceptedTermsOfService;
    public boolean unreadDialogsLoaded;

    public static UserConfig getInstance(int num) {
        UserConfig localInstance = Instance[num];
        if (localInstance == null) {
            synchronized (UserConfig.class) {
                localInstance = Instance[num];
                if (localInstance == null) {
                    UserConfig[] userConfigArr = Instance;
                    UserConfig userConfig = new UserConfig(num);
                    localInstance = userConfig;
                    userConfigArr[num] = userConfig;
                }
            }
        }
        return localInstance;
    }

    public UserConfig(int instance) {
        super(instance);
        this.sync = new Object();
        this.lastSendMessageId = -210000;
        this.lastBroadcastId = -1;
        this.unreadDialogsLoaded = true;
        this.migrateOffsetId = -1;
        this.migrateOffsetDate = -1;
        this.migrateOffsetUserId = -1;
        this.migrateOffsetChatId = -1;
        this.migrateOffsetChannelId = -1;
        this.migrateOffsetAccess = -1L;
        this.suggestContacts = true;
        this.syncContacts = true;
        this.isCdnVip = -1;
    }

    private SharedPreferences getPreferences() {
        if (this.currentAccount == 0) {
            return ApplicationLoader.applicationContext.getSharedPreferences("userconfing", 0);
        }
        return ApplicationLoader.applicationContext.getSharedPreferences("userconfig" + this.currentAccount, 0);
    }

    public static int getActivatedAccountsCount() {
        int count = 0;
        for (int a2 = 0; a2 < 3; a2++) {
            if (AccountInstance.getInstance(a2).getUserConfig().isClientActivated()) {
                count++;
            }
        }
        return count;
    }

    public int getNewMessageId() {
        int id;
        synchronized (this.sync) {
            id = this.lastSendMessageId;
            this.lastSendMessageId = id - 1;
        }
        return id;
    }

    public void saveConfig(boolean withFile) {
        saveConfig(withFile, null);
    }

    public void saveConfig(boolean withFile, File oldFile) {
        synchronized (this.sync) {
            try {
                SharedPreferences.Editor editor = getPreferences().edit();
                if (this.currentAccount == 0) {
                    editor.putInt("selectedAccount", selectedAccount);
                }
                editor.putBoolean("registeredForPush", this.registeredForPush);
                editor.putInt("lastSendMessageId", this.lastSendMessageId);
                editor.putInt("contactsSavedCount", this.contactsSavedCount);
                editor.putInt("lastBroadcastId", this.lastBroadcastId);
                editor.putInt("lastContactsSyncTime", this.lastContactsSyncTime);
                editor.putInt("lastHintsSyncTime", this.lastHintsSyncTime);
                editor.putBoolean("draftsLoaded", this.draftsLoaded);
                editor.putBoolean("unreadDialogsLoaded", this.unreadDialogsLoaded);
                editor.putInt("ratingLoadTime", this.ratingLoadTime);
                editor.putInt("botRatingLoadTime", this.botRatingLoadTime);
                editor.putBoolean("contactsReimported", this.contactsReimported);
                editor.putInt("loginTime", this.loginTime);
                editor.putBoolean("syncContacts", this.syncContacts);
                editor.putBoolean("suggestContacts", this.suggestContacts);
                editor.putBoolean("hasSecureData", this.hasSecureData);
                editor.putBoolean("notificationsSettingsLoaded3", this.notificationsSettingsLoaded);
                editor.putBoolean("notificationsSignUpSettingsLoaded", this.notificationsSignUpSettingsLoaded);
                editor.putLong("autoDownloadConfigLoadTime", this.autoDownloadConfigLoadTime);
                editor.putBoolean("hasValidDialogLoadIds", this.hasValidDialogLoadIds);
                editor.putInt("6migrateOffsetId", this.migrateOffsetId);
                if (this.migrateOffsetId != -1) {
                    editor.putInt("6migrateOffsetDate", this.migrateOffsetDate);
                    editor.putInt("6migrateOffsetUserId", this.migrateOffsetUserId);
                    editor.putInt("6migrateOffsetChatId", this.migrateOffsetChatId);
                    editor.putInt("6migrateOffsetChannelId", this.migrateOffsetChannelId);
                    editor.putLong("6migrateOffsetAccess", this.migrateOffsetAccess);
                }
                if (this.unacceptedTermsOfService != null) {
                    try {
                        SerializedData data = new SerializedData(this.unacceptedTermsOfService.getObjectSize());
                        this.unacceptedTermsOfService.serializeToStream(data);
                        String str = Base64.encodeToString(data.toByteArray(), 0);
                        editor.putString("terms", str);
                        data.cleanup();
                    } catch (Exception e) {
                    }
                } else {
                    editor.remove("terms");
                }
                SharedConfig.saveConfig();
                if (this.tmpPassword != null) {
                    SerializedData data2 = new SerializedData();
                    this.tmpPassword.serializeToStream(data2);
                    String string = Base64.encodeToString(data2.toByteArray(), 0);
                    editor.putString("tmpPassword", string);
                    data2.cleanup();
                } else {
                    editor.remove("tmpPassword");
                }
                if (this.currentUser != null) {
                    if (withFile) {
                        SerializedData data3 = new SerializedData();
                        this.currentUser.serializeToStream(data3);
                        String string2 = Base64.encodeToString(data3.toByteArray(), 0);
                        editor.putString("user", string2);
                        data3.cleanup();
                    }
                } else {
                    editor.remove("user");
                }
                editor.commit();
                if (oldFile != null) {
                    oldFile.delete();
                }
            } catch (Exception e2) {
                FileLog.e(e2);
            }
        }
    }

    public boolean isClientActivated() {
        boolean z;
        synchronized (this.sync) {
            z = this.currentUser != null;
        }
        return z;
    }

    public int getClientUserId() {
        int i;
        synchronized (this.sync) {
            TLRPC.User user = this.currentUser;
            i = user != null ? user.id : 0;
        }
        return i;
    }

    public String getClientPhone() {
        String str;
        synchronized (this.sync) {
            TLRPC.User user = this.currentUser;
            str = (user == null || user.phone == null) ? "" : this.currentUser.phone;
        }
        return str;
    }

    public TLRPC.User getCurrentUser() {
        TLRPC.User user;
        synchronized (this.sync) {
            user = this.currentUser;
        }
        return user;
    }

    public void setCurrentUser(TLRPC.User user) {
        synchronized (this.sync) {
            this.currentUser = user;
            this.clientUserId = user.id;
        }
    }

    /* JADX WARN: Can't wrap try/catch for region: R(21:9|(1:11)|12|(17:17|18|19|20|21|(1:25)|27|(1:29)|30|(1:34)|35|(1:39)|40|(1:42)|43|44|45)|48|18|19|20|21|(2:23|25)|27|(0)|30|(2:32|34)|35|(2:37|39)|40|(0)|43|44|45) */
    /* JADX WARN: Code restructure failed: missing block: B:26:0x0101, code lost:
        r8 = move-exception;
     */
    /* JADX WARN: Code restructure failed: missing block: B:27:0x0102, code lost:
        im.uecahxyvjd.messenger.FileLog.e(r8);
     */
    /* JADX WARN: Removed duplicated region for block: B:30:0x010f A[Catch: all -> 0x0183, TryCatch #0 {, blocks: (B:4:0x0003, B:6:0x0007, B:8:0x0009, B:10:0x0012, B:11:0x001a, B:13:0x00d1, B:18:0x00dd, B:20:0x00e0, B:22:0x00e8, B:24:0x00ee, B:28:0x0105, B:30:0x010f, B:31:0x0137, B:33:0x013f, B:35:0x0145, B:36:0x0157, B:38:0x015f, B:40:0x0165, B:41:0x0177, B:43:0x017b, B:44:0x017f, B:45:0x0181, B:27:0x0102), top: B:50:0x0003, inners: #1 }] */
    /* JADX WARN: Removed duplicated region for block: B:43:0x017b A[Catch: all -> 0x0183, TryCatch #0 {, blocks: (B:4:0x0003, B:6:0x0007, B:8:0x0009, B:10:0x0012, B:11:0x001a, B:13:0x00d1, B:18:0x00dd, B:20:0x00e0, B:22:0x00e8, B:24:0x00ee, B:28:0x0105, B:30:0x010f, B:31:0x0137, B:33:0x013f, B:35:0x0145, B:36:0x0157, B:38:0x015f, B:40:0x0165, B:41:0x0177, B:43:0x017b, B:44:0x017f, B:45:0x0181, B:27:0x0102), top: B:50:0x0003, inners: #1 }] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void loadConfig() {
        boolean z;
        int i;
        String string;
        String string2;
        TLRPC.User user;
        byte[] bytes;
        byte[] bytes2;
        String terms;
        byte[] arr;
        synchronized (this.sync) {
            if (this.configLoaded) {
                return;
            }
            SharedPreferences preferences = getPreferences();
            if (this.currentAccount == 0) {
                selectedAccount = preferences.getInt("selectedAccount", 0);
            }
            this.registeredForPush = preferences.getBoolean("registeredForPush", false);
            this.lastSendMessageId = preferences.getInt("lastSendMessageId", -210000);
            this.contactsSavedCount = preferences.getInt("contactsSavedCount", 0);
            this.lastBroadcastId = preferences.getInt("lastBroadcastId", -1);
            this.lastContactsSyncTime = preferences.getInt("lastContactsSyncTime", ((int) (System.currentTimeMillis() / 1000)) - 82800);
            this.lastHintsSyncTime = preferences.getInt("lastHintsSyncTime", ((int) (System.currentTimeMillis() / 1000)) - 90000);
            this.draftsLoaded = preferences.getBoolean("draftsLoaded", false);
            this.unreadDialogsLoaded = preferences.getBoolean("unreadDialogsLoaded", false);
            this.contactsReimported = preferences.getBoolean("contactsReimported", false);
            this.ratingLoadTime = preferences.getInt("ratingLoadTime", 0);
            this.botRatingLoadTime = preferences.getInt("botRatingLoadTime", 0);
            this.loginTime = preferences.getInt("loginTime", this.currentAccount);
            this.syncContacts = preferences.getBoolean("syncContacts", true);
            this.suggestContacts = preferences.getBoolean("suggestContacts", true);
            this.hasSecureData = preferences.getBoolean("hasSecureData", false);
            this.notificationsSettingsLoaded = preferences.getBoolean("notificationsSettingsLoaded3", false);
            this.notificationsSignUpSettingsLoaded = preferences.getBoolean("notificationsSignUpSettingsLoaded", false);
            this.autoDownloadConfigLoadTime = preferences.getLong("autoDownloadConfigLoadTime", 0L);
            if (!preferences.contains("2dialogsLoadOffsetId") && !preferences.getBoolean("hasValidDialogLoadIds", false)) {
                z = false;
                this.hasValidDialogLoadIds = z;
                terms = preferences.getString("terms", null);
                if (terms != null && (arr = Base64.decode(terms, 0)) != null) {
                    SerializedData data = new SerializedData(arr);
                    this.unacceptedTermsOfService = TLRPC.TL_help_termsOfService.TLdeserialize(data, data.readInt32(false), false);
                    data.cleanup();
                }
                i = preferences.getInt("6migrateOffsetId", 0);
                this.migrateOffsetId = i;
                if (i != -1) {
                    this.migrateOffsetDate = preferences.getInt("6migrateOffsetDate", 0);
                    this.migrateOffsetUserId = preferences.getInt("6migrateOffsetUserId", 0);
                    this.migrateOffsetChatId = preferences.getInt("6migrateOffsetChatId", 0);
                    this.migrateOffsetChannelId = preferences.getInt("6migrateOffsetChannelId", 0);
                    this.migrateOffsetAccess = preferences.getLong("6migrateOffsetAccess", 0L);
                }
                string = preferences.getString("tmpPassword", null);
                if (string != null && (bytes2 = Base64.decode(string, 0)) != null) {
                    SerializedData data2 = new SerializedData(bytes2);
                    this.tmpPassword = TLRPC.TL_account_tmpPassword.TLdeserialize(data2, data2.readInt32(false), false);
                    data2.cleanup();
                }
                string2 = preferences.getString("user", null);
                if (string2 != null && (bytes = Base64.decode(string2, 0)) != null) {
                    SerializedData data3 = new SerializedData(bytes);
                    this.currentUser = TLRPC.User.TLdeserialize(data3, data3.readInt32(false), false);
                    data3.cleanup();
                }
                user = this.currentUser;
                if (user != null) {
                    this.clientUserId = user.id;
                }
                this.configLoaded = true;
            }
            z = true;
            this.hasValidDialogLoadIds = z;
            terms = preferences.getString("terms", null);
            if (terms != null) {
                SerializedData data4 = new SerializedData(arr);
                this.unacceptedTermsOfService = TLRPC.TL_help_termsOfService.TLdeserialize(data4, data4.readInt32(false), false);
                data4.cleanup();
            }
            i = preferences.getInt("6migrateOffsetId", 0);
            this.migrateOffsetId = i;
            if (i != -1) {
            }
            string = preferences.getString("tmpPassword", null);
            if (string != null) {
                SerializedData data22 = new SerializedData(bytes2);
                this.tmpPassword = TLRPC.TL_account_tmpPassword.TLdeserialize(data22, data22.readInt32(false), false);
                data22.cleanup();
            }
            string2 = preferences.getString("user", null);
            if (string2 != null) {
                SerializedData data32 = new SerializedData(bytes);
                this.currentUser = TLRPC.User.TLdeserialize(data32, data32.readInt32(false), false);
                data32.cleanup();
            }
            user = this.currentUser;
            if (user != null) {
            }
            this.configLoaded = true;
        }
    }

    public void savePassword(byte[] hash, byte[] salted) {
        this.savedPasswordTime = SystemClock.elapsedRealtime();
        this.savedPasswordHash = hash;
        this.savedSaltedPassword = salted;
    }

    public void checkSavedPassword() {
        if ((this.savedSaltedPassword == null && this.savedPasswordHash == null) || Math.abs(SystemClock.elapsedRealtime() - this.savedPasswordTime) < 1800000) {
            return;
        }
        resetSavedPassword();
    }

    public void resetSavedPassword() {
        this.savedPasswordTime = 0L;
        if (this.savedPasswordHash != null) {
            for (int a2 = 0; a2 < this.savedPasswordHash.length; a2++) {
                this.savedPasswordHash[a2] = 0;
            }
            this.savedPasswordHash = null;
        }
        if (this.savedSaltedPassword != null) {
            for (int a3 = 0; a3 < this.savedSaltedPassword.length; a3++) {
                this.savedSaltedPassword[a3] = 0;
            }
            this.savedSaltedPassword = null;
        }
    }

    public void clearConfig() {
        getPreferences().edit().clear().commit();
        this.currentUser = null;
        this.clientUserId = 0;
        this.registeredForPush = false;
        this.contactsSavedCount = 0;
        this.lastSendMessageId = -210000;
        this.lastBroadcastId = -1;
        this.notificationsSettingsLoaded = false;
        this.notificationsSignUpSettingsLoaded = false;
        this.migrateOffsetId = -1;
        this.migrateOffsetDate = -1;
        this.migrateOffsetUserId = -1;
        this.migrateOffsetChatId = -1;
        this.migrateOffsetChannelId = -1;
        this.migrateOffsetAccess = -1L;
        this.ratingLoadTime = 0;
        this.botRatingLoadTime = 0;
        this.draftsLoaded = true;
        this.contactsReimported = true;
        this.syncContacts = true;
        this.suggestContacts = true;
        this.unreadDialogsLoaded = true;
        this.hasValidDialogLoadIds = true;
        this.unacceptedTermsOfService = null;
        this.hasSecureData = false;
        this.loginTime = (int) (System.currentTimeMillis() / 1000);
        this.lastContactsSyncTime = ((int) (System.currentTimeMillis() / 1000)) - 82800;
        this.lastHintsSyncTime = ((int) (System.currentTimeMillis() / 1000)) - 90000;
        AppUpdater.pendingAppUpdate = null;
        resetSavedPassword();
        boolean hasActivated = false;
        int a2 = 0;
        while (true) {
            if (a2 >= 3) {
                break;
            } else if (!AccountInstance.getInstance(a2).getUserConfig().isClientActivated()) {
                a2++;
            } else {
                hasActivated = true;
                break;
            }
        }
        if (!hasActivated) {
            SharedConfig.clearConfig();
        }
        saveConfig(true);
    }

    public boolean isPinnedDialogsLoaded(int folderId) {
        return getPreferences().getBoolean("2pinnedDialogsLoaded" + folderId, false);
    }

    public void setPinnedDialogsLoaded(int folderId, boolean loaded) {
        getPreferences().edit().putBoolean("2pinnedDialogsLoaded" + folderId, loaded).commit();
    }

    public int getTotalDialogsCount(int folderId) {
        return getPreferences().getInt("2totalDialogsLoadCount" + (folderId == 0 ? "" : Integer.valueOf(folderId)), 0);
    }

    public void setTotalDialogsCount(int folderId, int totalDialogsLoadCount) {
        getPreferences().edit().putInt("2totalDialogsLoadCount" + (folderId == 0 ? "" : Integer.valueOf(folderId)), totalDialogsLoadCount).commit();
    }

    public int[] getDialogLoadOffsets(int folderId) {
        SharedPreferences preferences = getPreferences();
        int dialogsLoadOffsetId = preferences.getInt("2dialogsLoadOffsetId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), this.hasValidDialogLoadIds ? 0 : -1);
        int dialogsLoadOffsetDate = preferences.getInt("2dialogsLoadOffsetDate" + (folderId == 0 ? "" : Integer.valueOf(folderId)), this.hasValidDialogLoadIds ? 0 : -1);
        int dialogsLoadOffsetUserId = preferences.getInt("2dialogsLoadOffsetUserId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), this.hasValidDialogLoadIds ? 0 : -1);
        int dialogsLoadOffsetChatId = preferences.getInt("2dialogsLoadOffsetChatId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), this.hasValidDialogLoadIds ? 0 : -1);
        int dialogsLoadOffsetChannelId = preferences.getInt("2dialogsLoadOffsetChannelId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), this.hasValidDialogLoadIds ? 0 : -1);
        long dialogsLoadOffsetAccess = preferences.getLong("2dialogsLoadOffsetAccess" + (folderId != 0 ? Integer.valueOf(folderId) : ""), this.hasValidDialogLoadIds ? 0L : -1L);
        return new int[]{dialogsLoadOffsetId, dialogsLoadOffsetDate, dialogsLoadOffsetUserId, dialogsLoadOffsetChatId, dialogsLoadOffsetChannelId, (int) dialogsLoadOffsetAccess, (int) (dialogsLoadOffsetAccess >> 32)};
    }

    public void setDialogsLoadOffset(int folderId, int dialogsLoadOffsetId, int dialogsLoadOffsetDate, int dialogsLoadOffsetUserId, int dialogsLoadOffsetChatId, int dialogsLoadOffsetChannelId, long dialogsLoadOffsetAccess) {
        SharedPreferences.Editor editor = getPreferences().edit();
        editor.putInt("2dialogsLoadOffsetId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), dialogsLoadOffsetId);
        editor.putInt("2dialogsLoadOffsetDate" + (folderId == 0 ? "" : Integer.valueOf(folderId)), dialogsLoadOffsetDate);
        editor.putInt("2dialogsLoadOffsetUserId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), dialogsLoadOffsetUserId);
        editor.putInt("2dialogsLoadOffsetChatId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), dialogsLoadOffsetChatId);
        editor.putInt("2dialogsLoadOffsetChannelId" + (folderId == 0 ? "" : Integer.valueOf(folderId)), dialogsLoadOffsetChannelId);
        editor.putLong("2dialogsLoadOffsetAccess" + (folderId != 0 ? Integer.valueOf(folderId) : ""), dialogsLoadOffsetAccess);
        editor.putBoolean("hasValidDialogLoadIds", true);
        editor.commit();
    }
}
