package im.uecahxyvjd.messenger.camera;

import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.SurfaceTexture;
import android.graphics.drawable.BitmapDrawable;
import android.hardware.Camera;
import android.media.MediaMetadataRetriever;
import android.media.MediaRecorder;
import android.media.ThumbnailUtils;
import android.os.Build;
import android.util.Base64;
import com.serenegiant.usb.UVCCamera;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ApplicationLoader;
import im.uecahxyvjd.messenger.Bitmaps;
import im.uecahxyvjd.messenger.BuildVars;
import im.uecahxyvjd.messenger.FileLoader;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.ImageLoader;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.SharedConfig;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.tgnet.SerializedData;
import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import kotlin.UByte;
/* loaded from: classes17.dex */
public class CameraController implements MediaRecorder.OnInfoListener {
    private static final int CORE_POOL_SIZE = 1;
    private static volatile CameraController Instance = null;
    private static final int KEEP_ALIVE_SECONDS = 60;
    private static final int MAX_POOL_SIZE = 1;
    protected volatile ArrayList<CameraInfo> cameraInfos;
    private boolean cameraInitied;
    private boolean loadingCameras;
    private VideoTakeCallback onVideoTakeCallback;
    private String recordedFile;
    private MediaRecorder recorder;
    protected ArrayList<String> availableFlashModes = new ArrayList<>();
    private ArrayList<Runnable> onFinishCameraInitRunnables = new ArrayList<>();
    private ThreadPoolExecutor threadPool = new ThreadPoolExecutor(1, 1, 60, TimeUnit.SECONDS, new LinkedBlockingQueue());

    /* loaded from: classes17.dex */
    public interface VideoTakeCallback {
        void onFinishVideoRecording(String str, long j);
    }

    public static CameraController getInstance() {
        CameraController localInstance = Instance;
        if (localInstance == null) {
            synchronized (CameraController.class) {
                localInstance = Instance;
                if (localInstance == null) {
                    CameraController cameraController = new CameraController();
                    localInstance = cameraController;
                    Instance = cameraController;
                }
            }
        }
        return localInstance;
    }

    public void cancelOnInitRunnable(Runnable onInitRunnable) {
        this.onFinishCameraInitRunnables.remove(onInitRunnable);
    }

    public void initCamera(Runnable onInitRunnable) {
        if (onInitRunnable != null && !this.onFinishCameraInitRunnables.contains(onInitRunnable)) {
            this.onFinishCameraInitRunnables.add(onInitRunnable);
        }
        if (this.loadingCameras || this.cameraInitied) {
            return;
        }
        this.loadingCameras = true;
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda10
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.this.m1010x3748b53d();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initCamera$3$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1010x3748b53d() {
        String cache;
        Camera.CameraInfo info;
        List<Camera.Size> list;
        String str;
        final CameraController cameraController = this;
        String str2 = "cameraCache";
        try {
            if (cameraController.cameraInfos == null) {
                SharedPreferences preferences = MessagesController.getGlobalMainSettings();
                String cache2 = preferences.getString("cameraCache", null);
                Comparator<Size> comparator = new Comparator() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda2
                    @Override // java.util.Comparator
                    public final int compare(Object obj, Object obj2) {
                        return CameraController.lambda$initCamera$0((Size) obj, (Size) obj2);
                    }
                };
                ArrayList<CameraInfo> result = new ArrayList<>();
                if (cache2 != null) {
                    SerializedData serializedData = new SerializedData(Base64.decode(cache2, 0));
                    int count = serializedData.readInt32(false);
                    for (int a2 = 0; a2 < count; a2++) {
                        CameraInfo cameraInfo = new CameraInfo(serializedData.readInt32(false), serializedData.readInt32(false));
                        int pCount = serializedData.readInt32(false);
                        for (int b = 0; b < pCount; b++) {
                            cameraInfo.previewSizes.add(new Size(serializedData.readInt32(false), serializedData.readInt32(false)));
                        }
                        int pCount2 = serializedData.readInt32(false);
                        for (int b2 = 0; b2 < pCount2; b2++) {
                            cameraInfo.pictureSizes.add(new Size(serializedData.readInt32(false), serializedData.readInt32(false)));
                        }
                        result.add(cameraInfo);
                        Collections.sort(cameraInfo.previewSizes, comparator);
                        Collections.sort(cameraInfo.pictureSizes, comparator);
                    }
                    serializedData.cleanup();
                } else {
                    int count2 = Camera.getNumberOfCameras();
                    Camera.CameraInfo info2 = new Camera.CameraInfo();
                    int bufferSize = 4;
                    int cameraId = 0;
                    while (cameraId < count2) {
                        try {
                            Camera.getCameraInfo(cameraId, info2);
                            CameraInfo cameraInfo2 = new CameraInfo(cameraId, info2.facing);
                            if (ApplicationLoader.mainInterfacePaused && ApplicationLoader.externalInterfacePaused) {
                                throw new RuntimeException("app paused");
                            }
                            Camera camera = Camera.open(cameraInfo2.getCameraId());
                            Camera.Parameters params = camera.getParameters();
                            List<Camera.Size> list2 = params.getSupportedPreviewSizes();
                            int a3 = 0;
                            while (true) {
                                cache = cache2;
                                info = info2;
                                if (a3 >= list2.size()) {
                                    break;
                                }
                                Camera.Size size = list2.get(a3);
                                List<Camera.Size> list3 = list2;
                                if (size.width == 1280 && size.height != 720) {
                                    str = str2;
                                } else if (size.height >= 2160 || size.width >= 2160) {
                                    str = str2;
                                } else {
                                    str = str2;
                                    cameraInfo2.previewSizes.add(new Size(size.width, size.height));
                                    if (BuildVars.LOGS_ENABLED) {
                                        FileLog.d("preview size = " + size.width + " " + size.height);
                                    }
                                }
                                a3++;
                                cache2 = cache;
                                info2 = info;
                                list2 = list3;
                                str2 = str;
                            }
                            String str3 = str2;
                            List<Camera.Size> list4 = params.getSupportedPictureSizes();
                            int a4 = 0;
                            while (a4 < list4.size()) {
                                Camera.Size size2 = list4.get(a4);
                                if (size2.width == 1280 && size2.height != 720) {
                                    list = list4;
                                    a4++;
                                    list4 = list;
                                }
                                if ("samsung".equals(Build.MANUFACTURER) && "jflteuc".equals(Build.PRODUCT) && size2.width >= 2048) {
                                    list = list4;
                                    a4++;
                                    list4 = list;
                                }
                                list = list4;
                                cameraInfo2.pictureSizes.add(new Size(size2.width, size2.height));
                                if (BuildVars.LOGS_ENABLED) {
                                    FileLog.d("picture size = " + size2.width + " " + size2.height);
                                }
                                a4++;
                                list4 = list;
                            }
                            camera.release();
                            result.add(cameraInfo2);
                            Collections.sort(cameraInfo2.previewSizes, comparator);
                            Collections.sort(cameraInfo2.pictureSizes, comparator);
                            bufferSize += ((cameraInfo2.previewSizes.size() + cameraInfo2.pictureSizes.size()) * 8) + 8;
                            cameraId++;
                            cache2 = cache;
                            info2 = info;
                            str2 = str3;
                        } catch (Exception e) {
                            cameraController = this;
                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda4
                                @Override // java.lang.Runnable
                                public final void run() {
                                    CameraController.this.m1009x37bf1b3c();
                                }
                            });
                            return;
                        }
                    }
                    String str4 = str2;
                    SerializedData serializedData2 = new SerializedData(bufferSize);
                    serializedData2.writeInt32(result.size());
                    for (int a5 = 0; a5 < count2; a5++) {
                        CameraInfo cameraInfo3 = result.get(a5);
                        serializedData2.writeInt32(cameraInfo3.cameraId);
                        serializedData2.writeInt32(cameraInfo3.frontCamera);
                        int pCount3 = cameraInfo3.previewSizes.size();
                        serializedData2.writeInt32(pCount3);
                        for (int b3 = 0; b3 < pCount3; b3++) {
                            Size size3 = cameraInfo3.previewSizes.get(b3);
                            serializedData2.writeInt32(size3.mWidth);
                            serializedData2.writeInt32(size3.mHeight);
                        }
                        int pCount4 = cameraInfo3.pictureSizes.size();
                        serializedData2.writeInt32(pCount4);
                        for (int b4 = 0; b4 < pCount4; b4++) {
                            Size size4 = cameraInfo3.pictureSizes.get(b4);
                            serializedData2.writeInt32(size4.mWidth);
                            serializedData2.writeInt32(size4.mHeight);
                        }
                    }
                    preferences.edit().putString(str4, Base64.encodeToString(serializedData2.toByteArray(), 0)).commit();
                    serializedData2.cleanup();
                }
                cameraController = this;
                cameraController.cameraInfos = result;
            }
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda3
                @Override // java.lang.Runnable
                public final void run() {
                    CameraController.this.m1008x3835813b();
                }
            });
        } catch (Exception e2) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$initCamera$0(Size o1, Size o2) {
        if (o1.mWidth < o2.mWidth) {
            return 1;
        }
        if (o1.mWidth > o2.mWidth) {
            return -1;
        }
        if (o1.mHeight < o2.mHeight) {
            return 1;
        }
        return o1.mHeight > o2.mHeight ? -1 : 0;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initCamera$1$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1008x3835813b() {
        this.loadingCameras = false;
        this.cameraInitied = true;
        if (!this.onFinishCameraInitRunnables.isEmpty()) {
            for (int a2 = 0; a2 < this.onFinishCameraInitRunnables.size(); a2++) {
                this.onFinishCameraInitRunnables.get(a2).run();
            }
            this.onFinishCameraInitRunnables.clear();
        }
        NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.cameraInitied, new Object[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initCamera$2$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1009x37bf1b3c() {
        this.onFinishCameraInitRunnables.clear();
        this.loadingCameras = false;
        this.cameraInitied = false;
    }

    public boolean isCameraInitied() {
        return (!this.cameraInitied || this.cameraInfos == null || this.cameraInfos.isEmpty()) ? false : true;
    }

    public void runOnThreadPool(Runnable runnable) {
        this.threadPool.execute(runnable);
    }

    public void close(final CameraSession session, final CountDownLatch countDownLatch, final Runnable beforeDestroyRunnable) {
        session.destroy();
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda6
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.lambda$close$4(beforeDestroyRunnable, session, countDownLatch);
            }
        });
        if (countDownLatch != null) {
            try {
                countDownLatch.await();
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$close$4(Runnable beforeDestroyRunnable, CameraSession session, CountDownLatch countDownLatch) {
        if (beforeDestroyRunnable != null) {
            beforeDestroyRunnable.run();
        }
        if (session.cameraInfo.camera == null) {
            return;
        }
        try {
            session.cameraInfo.camera.stopPreview();
            session.cameraInfo.camera.setPreviewCallbackWithBuffer(null);
        } catch (Exception e) {
            FileLog.e(e);
        }
        try {
            session.cameraInfo.camera.release();
        } catch (Exception e2) {
            FileLog.e(e2);
        }
        session.cameraInfo.camera = null;
        if (countDownLatch != null) {
            countDownLatch.countDown();
        }
    }

    public ArrayList<CameraInfo> getCameras() {
        return this.cameraInfos;
    }

    /* JADX WARN: Code restructure failed: missing block: B:37:0x0062, code lost:
        r1 = r3;
     */
    /* JADX WARN: Code restructure failed: missing block: B:40:0x0067, code lost:
        if (r2 <= 8) goto L68;
     */
    /* JADX WARN: Code restructure failed: missing block: B:41:0x0069, code lost:
        r3 = pack(r10, r1, 4, false);
     */
    /* JADX WARN: Code restructure failed: missing block: B:42:0x0070, code lost:
        if (r3 == 1229531648) goto L44;
     */
    /* JADX WARN: Code restructure failed: missing block: B:44:0x0075, code lost:
        if (r3 == 1296891946) goto L44;
     */
    /* JADX WARN: Code restructure failed: missing block: B:45:0x0077, code lost:
        return 0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:46:0x0078, code lost:
        if (r3 != 1229531648) goto L45;
     */
    /* JADX WARN: Code restructure failed: missing block: B:48:0x007b, code lost:
        r6 = false;
     */
    /* JADX WARN: Code restructure failed: missing block: B:49:0x007c, code lost:
        r4 = r6;
        r5 = pack(r10, r1 + 4, 4, r4) + 2;
     */
    /* JADX WARN: Code restructure failed: missing block: B:50:0x0086, code lost:
        if (r5 < 10) goto L67;
     */
    /* JADX WARN: Code restructure failed: missing block: B:51:0x0088, code lost:
        if (r5 <= r2) goto L49;
     */
    /* JADX WARN: Code restructure failed: missing block: B:53:0x008b, code lost:
        r1 = r1 + r5;
        r2 = r2 - r5;
        r5 = pack(r10, r1 - 2, 2, r4);
     */
    /* JADX WARN: Code restructure failed: missing block: B:54:0x0093, code lost:
        r6 = r5 - 1;
     */
    /* JADX WARN: Code restructure failed: missing block: B:55:0x0095, code lost:
        if (r5 <= 0) goto L66;
     */
    /* JADX WARN: Code restructure failed: missing block: B:57:0x0099, code lost:
        if (r2 < 12) goto L65;
     */
    /* JADX WARN: Code restructure failed: missing block: B:59:0x00a1, code lost:
        if (pack(r10, r1, 2, r4) != 274) goto L56;
     */
    /* JADX WARN: Code restructure failed: missing block: B:60:0x00a3, code lost:
        r5 = pack(r10, r1 + 8, 2, r4);
     */
    /* JADX WARN: Code restructure failed: missing block: B:61:0x00a9, code lost:
        switch(r5) {
            case 1: goto L64;
            case 3: goto L63;
            case 6: goto L62;
            case 8: goto L61;
            default: goto L60;
        };
     */
    /* JADX WARN: Code restructure failed: missing block: B:62:0x00ac, code lost:
        return 0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:63:0x00ad, code lost:
        return 270;
     */
    /* JADX WARN: Code restructure failed: missing block: B:65:0x00b0, code lost:
        return 90;
     */
    /* JADX WARN: Code restructure failed: missing block: B:67:0x00b3, code lost:
        return 180;
     */
    /* JADX WARN: Code restructure failed: missing block: B:69:0x00b6, code lost:
        return 0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:70:0x00b7, code lost:
        r1 = r1 + 12;
        r2 = r2 - 12;
        r5 = r6;
     */
    /* JADX WARN: Code restructure failed: missing block: B:71:0x00bd, code lost:
        return 0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:72:0x00be, code lost:
        return 0;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private static int getOrientation(byte[] jpeg) {
        if (jpeg == null) {
            return 0;
        }
        int marker = 0;
        int length = 0;
        while (true) {
            boolean z = true;
            if (marker + 3 >= jpeg.length) {
                break;
            }
            int offset = marker + 1;
            if ((jpeg[marker] & 255) != 255) {
                marker = offset;
                break;
            }
            int marker2 = jpeg[offset] & UByte.MAX_VALUE;
            if (marker2 == 255) {
                marker = offset;
            } else {
                int offset2 = offset + 1;
                if (marker2 == 216 || marker2 == 1) {
                    marker = offset2;
                } else if (marker2 != 217 && marker2 != 218) {
                    int length2 = pack(jpeg, offset2, 2, false);
                    if (length2 >= 2 && offset2 + length2 <= jpeg.length) {
                        if (marker2 == 225 && length2 >= 8 && pack(jpeg, offset2 + 2, 4, false) == 1165519206 && pack(jpeg, offset2 + 6, 2, false) == 0) {
                            length = length2 - 8;
                            marker = offset2 + 8;
                            break;
                        }
                        int offset3 = offset2 + length2;
                        length = 0;
                        marker = offset3;
                    } else {
                        break;
                    }
                } else {
                    break;
                }
            }
        }
        return 0;
    }

    private static int pack(byte[] bytes, int offset, int length, boolean littleEndian) {
        int step = 1;
        if (littleEndian) {
            offset += length - 1;
            step = -1;
        }
        int value = 0;
        while (true) {
            int length2 = length - 1;
            if (length > 0) {
                value = (value << 8) | (bytes[offset] & UByte.MAX_VALUE);
                offset += step;
                length = length2;
            } else {
                return value;
            }
        }
    }

    public boolean takePicture(final File path, CameraSession session, final Runnable callback) {
        if (session == null) {
            return false;
        }
        final CameraInfo info = session.cameraInfo;
        final boolean flipFront = session.isFlipFront();
        Camera camera = info.camera;
        try {
            camera.takePicture(null, null, new Camera.PictureCallback() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda8
                @Override // android.hardware.Camera.PictureCallback
                public final void onPictureTaken(byte[] bArr, Camera camera2) {
                    CameraController.lambda$takePicture$5(path, info, flipFront, callback, bArr, camera2);
                }
            });
            return true;
        } catch (Exception e) {
            FileLog.e(e);
            return false;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$takePicture$5(File path, CameraInfo info, boolean flipFront, Runnable callback, byte[] data, Camera camera1) {
        Bitmap bitmap = null;
        int size = (int) (AndroidUtilities.getPhotoSize() / AndroidUtilities.density);
        String key = String.format(Locale.US, "%s@%d_%d", Utilities.MD5(path.getAbsolutePath()), Integer.valueOf(size), Integer.valueOf(size));
        try {
            BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeByteArray(data, 0, data.length, options);
            float scaleFactor = Math.max(options.outWidth / AndroidUtilities.getPhotoSize(), options.outHeight / AndroidUtilities.getPhotoSize());
            if (scaleFactor < 1.0f) {
                scaleFactor = 1.0f;
            }
            options.inJustDecodeBounds = false;
            options.inSampleSize = (int) scaleFactor;
            options.inPurgeable = true;
            bitmap = BitmapFactory.decodeByteArray(data, 0, data.length, options);
        } catch (Throwable e) {
            FileLog.e(e);
        }
        try {
        } catch (Exception e2) {
            FileLog.e(e2);
        }
        if (info.frontCamera != 0 && flipFront) {
            Matrix matrix = new Matrix();
            matrix.setRotate(getOrientation(data));
            matrix.postScale(-1.0f, 1.0f);
            Bitmap scaled = Bitmaps.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
            if (scaled != bitmap) {
                bitmap.recycle();
            }
            FileOutputStream outputStream = new FileOutputStream(path);
            scaled.compress(Bitmap.CompressFormat.JPEG, 80, outputStream);
            outputStream.flush();
            outputStream.getFD().sync();
            outputStream.close();
            if (scaled != 0) {
                ImageLoader.getInstance().putImageToCache(new BitmapDrawable(scaled), key);
            }
            if (callback != null) {
                callback.run();
                return;
            }
            return;
        }
        FileOutputStream outputStream2 = new FileOutputStream(path);
        outputStream2.write(data);
        outputStream2.flush();
        outputStream2.getFD().sync();
        outputStream2.close();
        if (bitmap != null) {
            ImageLoader.getInstance().putImageToCache(new BitmapDrawable(bitmap), key);
        }
        if (callback != null) {
            callback.run();
        }
    }

    public void startPreview(final CameraSession session) {
        if (session == null) {
            return;
        }
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda0
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.lambda$startPreview$6(CameraSession.this);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$startPreview$6(CameraSession session) {
        Camera camera = session.cameraInfo.camera;
        if (camera == null) {
            try {
                CameraInfo cameraInfo = session.cameraInfo;
                Camera open = Camera.open(session.cameraInfo.cameraId);
                cameraInfo.camera = open;
                camera = open;
            } catch (Exception e) {
                session.cameraInfo.camera = null;
                if (camera != null) {
                    camera.release();
                }
                FileLog.e(e);
                return;
            }
        }
        camera.startPreview();
    }

    public void stopPreview(final CameraSession session) {
        if (session == null) {
            return;
        }
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda13
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.lambda$stopPreview$7(CameraSession.this);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$stopPreview$7(CameraSession session) {
        Camera camera = session.cameraInfo.camera;
        if (camera == null) {
            try {
                CameraInfo cameraInfo = session.cameraInfo;
                Camera open = Camera.open(session.cameraInfo.cameraId);
                cameraInfo.camera = open;
                camera = open;
            } catch (Exception e) {
                session.cameraInfo.camera = null;
                if (camera != null) {
                    camera.release();
                }
                FileLog.e(e);
                return;
            }
        }
        camera.stopPreview();
    }

    public void openRound(final CameraSession session, final SurfaceTexture texture, final Runnable callback, final Runnable configureCallback) {
        if (session == null || texture == null) {
            if (BuildVars.LOGS_ENABLED) {
                FileLog.d("failed to open round " + session + " tex = " + texture);
                return;
            }
            return;
        }
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda9
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.lambda$openRound$8(CameraSession.this, configureCallback, texture, callback);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$openRound$8(CameraSession session, Runnable configureCallback, SurfaceTexture texture, Runnable callback) {
        Camera camera = session.cameraInfo.camera;
        try {
            if (BuildVars.LOGS_ENABLED) {
                FileLog.d("start creating round camera session");
            }
            if (camera == null) {
                CameraInfo cameraInfo = session.cameraInfo;
                Camera open = Camera.open(session.cameraInfo.cameraId);
                cameraInfo.camera = open;
                camera = open;
            }
            camera.getParameters();
            session.configureRoundCamera();
            if (configureCallback != null) {
                configureCallback.run();
            }
            camera.setPreviewTexture(texture);
            camera.startPreview();
            if (callback != null) {
                AndroidUtilities.runOnUIThread(callback);
            }
            if (BuildVars.LOGS_ENABLED) {
                FileLog.d("round camera session created");
            }
        } catch (Exception e) {
            session.cameraInfo.camera = null;
            if (camera != null) {
                camera.release();
            }
            FileLog.e(e);
        }
    }

    public void open(final CameraSession session, final SurfaceTexture texture, final Runnable callback, final Runnable prestartCallback) {
        if (session == null || texture == null) {
            return;
        }
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda12
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.this.m1011lambda$open$9$imuecahxyvjdmessengercameraCameraController(session, prestartCallback, texture, callback);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$open$9$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1011lambda$open$9$imuecahxyvjdmessengercameraCameraController(CameraSession session, Runnable prestartCallback, SurfaceTexture texture, Runnable callback) {
        Camera camera = session.cameraInfo.camera;
        if (camera == null) {
            try {
                CameraInfo cameraInfo = session.cameraInfo;
                Camera open = Camera.open(session.cameraInfo.cameraId);
                cameraInfo.camera = open;
                camera = open;
            } catch (Exception e) {
                session.cameraInfo.camera = null;
                if (camera != null) {
                    camera.release();
                }
                FileLog.e(e);
                return;
            }
        }
        Camera.Parameters params = camera.getParameters();
        List<String> rawFlashModes = params.getSupportedFlashModes();
        this.availableFlashModes.clear();
        if (rawFlashModes != null) {
            for (int a2 = 0; a2 < rawFlashModes.size(); a2++) {
                String rawFlashMode = rawFlashModes.get(a2);
                if (rawFlashMode.equals("off") || rawFlashMode.equals("on") || rawFlashMode.equals("auto")) {
                    this.availableFlashModes.add(rawFlashMode);
                }
            }
            session.checkFlashMode(this.availableFlashModes.get(0));
        }
        if (prestartCallback != null) {
            prestartCallback.run();
        }
        session.configurePhotoCamera();
        camera.setPreviewTexture(texture);
        camera.startPreview();
        if (callback != null) {
            AndroidUtilities.runOnUIThread(callback);
        }
    }

    public void recordVideo(final CameraSession session, final File path, final VideoTakeCallback callback, final Runnable onVideoStartRecord) {
        if (session == null) {
            return;
        }
        final CameraInfo info = session.cameraInfo;
        final Camera camera = info.camera;
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.this.m1012x470d22b2(camera, session, path, info, callback, onVideoStartRecord);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$recordVideo$10$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1012x470d22b2(Camera camera, CameraSession session, File path, CameraInfo info, VideoTakeCallback callback, Runnable onVideoStartRecord) {
        if (camera != null) {
            try {
                try {
                    Camera.Parameters params = camera.getParameters();
                    params.setFlashMode(session.getCurrentFlashMode().equals("on") ? "torch" : "off");
                    camera.setParameters(params);
                } catch (Exception e) {
                    FileLog.e(e);
                }
                camera.unlock();
                try {
                    MediaRecorder mediaRecorder = new MediaRecorder();
                    this.recorder = mediaRecorder;
                    mediaRecorder.setCamera(camera);
                    this.recorder.setVideoSource(1);
                    this.recorder.setAudioSource(5);
                    session.configureRecorder(1, this.recorder);
                    this.recorder.setOutputFile(path.getAbsolutePath());
                    this.recorder.setMaxFileSize(1073741824L);
                    this.recorder.setVideoFrameRate(30);
                    this.recorder.setMaxDuration(0);
                    Size pictureSize = chooseOptimalSize(info.getPictureSizes(), 720, UVCCamera.DEFAULT_PREVIEW_HEIGHT, new Size(16, 9));
                    this.recorder.setVideoEncodingBitRate(1800000);
                    this.recorder.setVideoSize(pictureSize.getWidth(), pictureSize.getHeight());
                    this.recorder.setOnInfoListener(this);
                    this.recorder.prepare();
                    this.recorder.start();
                    this.onVideoTakeCallback = callback;
                    this.recordedFile = path.getAbsolutePath();
                    if (onVideoStartRecord != null) {
                        AndroidUtilities.runOnUIThread(onVideoStartRecord);
                    }
                } catch (Exception e2) {
                    this.recorder.release();
                    this.recorder = null;
                    FileLog.e(e2);
                }
            } catch (Exception e3) {
                FileLog.e(e3);
            }
        }
    }

    private void finishRecordingVideo() {
        MediaMetadataRetriever mediaMetadataRetriever = null;
        long duration = 0;
        try {
            try {
                mediaMetadataRetriever = new MediaMetadataRetriever();
                mediaMetadataRetriever.setDataSource(this.recordedFile);
                String d = mediaMetadataRetriever.extractMetadata(9);
                if (d != null) {
                    duration = (int) Math.ceil(((float) Long.parseLong(d)) / 1000.0f);
                }
            } catch (Exception e) {
                FileLog.e(e);
                if (mediaMetadataRetriever != null) {
                    try {
                        mediaMetadataRetriever.release();
                    } catch (Exception e2) {
                        e = e2;
                        FileLog.e(e);
                        final long duration2 = duration;
                        final Bitmap bitmap = ThumbnailUtils.createVideoThumbnail(this.recordedFile, 1);
                        String fileName = "-2147483648_" + SharedConfig.getLastLocalId() + ".jpg";
                        final File cacheFile = new File(FileLoader.getDirectory(4), fileName);
                        FileOutputStream stream = new FileOutputStream(cacheFile);
                        bitmap.compress(Bitmap.CompressFormat.JPEG, 80, stream);
                        SharedConfig.saveConfig();
                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda7
                            @Override // java.lang.Runnable
                            public final void run() {
                                CameraController.this.m1007x6da2f7a6(cacheFile, bitmap, duration2);
                            }
                        });
                    }
                }
            }
            try {
                mediaMetadataRetriever.release();
            } catch (Exception e3) {
                e = e3;
                FileLog.e(e);
                final long duration22 = duration;
                final Bitmap bitmap2 = ThumbnailUtils.createVideoThumbnail(this.recordedFile, 1);
                String fileName2 = "-2147483648_" + SharedConfig.getLastLocalId() + ".jpg";
                final File cacheFile2 = new File(FileLoader.getDirectory(4), fileName2);
                FileOutputStream stream2 = new FileOutputStream(cacheFile2);
                bitmap2.compress(Bitmap.CompressFormat.JPEG, 80, stream2);
                SharedConfig.saveConfig();
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda7
                    @Override // java.lang.Runnable
                    public final void run() {
                        CameraController.this.m1007x6da2f7a6(cacheFile2, bitmap2, duration22);
                    }
                });
            }
            final long duration222 = duration;
            final Bitmap bitmap22 = ThumbnailUtils.createVideoThumbnail(this.recordedFile, 1);
            String fileName22 = "-2147483648_" + SharedConfig.getLastLocalId() + ".jpg";
            final File cacheFile22 = new File(FileLoader.getDirectory(4), fileName22);
            try {
                FileOutputStream stream22 = new FileOutputStream(cacheFile22);
                bitmap22.compress(Bitmap.CompressFormat.JPEG, 80, stream22);
            } catch (Throwable e4) {
                FileLog.e(e4);
            }
            SharedConfig.saveConfig();
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda7
                @Override // java.lang.Runnable
                public final void run() {
                    CameraController.this.m1007x6da2f7a6(cacheFile22, bitmap22, duration222);
                }
            });
        } catch (Throwable th) {
            if (mediaMetadataRetriever != null) {
                try {
                    mediaMetadataRetriever.release();
                } catch (Exception e5) {
                    FileLog.e(e5);
                }
            }
            throw th;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$finishRecordingVideo$11$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1007x6da2f7a6(File cacheFile, Bitmap bitmap, long durationFinal) {
        if (this.onVideoTakeCallback != null) {
            String path = cacheFile.getAbsolutePath();
            if (bitmap != null) {
                ImageLoader.getInstance().putImageToCache(new BitmapDrawable(bitmap), Utilities.MD5(path));
            }
            this.onVideoTakeCallback.onFinishVideoRecording(path, durationFinal);
            this.onVideoTakeCallback = null;
        }
    }

    @Override // android.media.MediaRecorder.OnInfoListener
    public void onInfo(MediaRecorder mediaRecorder, int what, int extra) {
        if (what == 800 || what == 801 || what == 1) {
            MediaRecorder tempRecorder = this.recorder;
            this.recorder = null;
            if (tempRecorder != null) {
                tempRecorder.stop();
                tempRecorder.release();
            }
            if (this.onVideoTakeCallback != null) {
                finishRecordingVideo();
            }
        }
    }

    public void stopVideoRecording(final CameraSession session, final boolean abandon) {
        this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda5
            @Override // java.lang.Runnable
            public final void run() {
                CameraController.this.m1013xe8ff0cad(session, abandon);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$stopVideoRecording$13$im-uecahxyvjd-messenger-camera-CameraController  reason: not valid java name */
    public /* synthetic */ void m1013xe8ff0cad(final CameraSession session, boolean abandon) {
        MediaRecorder tempRecorder;
        try {
            CameraInfo info = session.cameraInfo;
            final Camera camera = info.camera;
            if (camera != null && (tempRecorder = this.recorder) != null) {
                this.recorder = null;
                try {
                    tempRecorder.stop();
                } catch (Exception e) {
                    FileLog.e(e);
                }
                try {
                    tempRecorder.release();
                } catch (Exception e2) {
                    FileLog.e(e2);
                }
                try {
                    camera.reconnect();
                    camera.startPreview();
                } catch (Exception e3) {
                    FileLog.e(e3);
                }
                try {
                    session.stopVideoRecording();
                } catch (Exception e4) {
                    FileLog.e(e4);
                }
            }
            try {
                Camera.Parameters params = camera.getParameters();
                params.setFlashMode("off");
                camera.setParameters(params);
            } catch (Exception e5) {
                FileLog.e(e5);
            }
            this.threadPool.execute(new Runnable() { // from class: im.uecahxyvjd.messenger.camera.CameraController$$ExternalSyntheticLambda11
                @Override // java.lang.Runnable
                public final void run() {
                    CameraController.lambda$stopVideoRecording$12(camera, session);
                }
            });
            if (!abandon && this.onVideoTakeCallback != null) {
                finishRecordingVideo();
            } else {
                this.onVideoTakeCallback = null;
            }
        } catch (Exception e6) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$stopVideoRecording$12(Camera camera, CameraSession session) {
        try {
            Camera.Parameters params = camera.getParameters();
            params.setFlashMode(session.getCurrentFlashMode());
            camera.setParameters(params);
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public static Size chooseOptimalSize(List<Size> choices, int width, int height, Size aspectRatio) {
        List<Size> bigEnough = new ArrayList<>();
        int w = aspectRatio.getWidth();
        int h = aspectRatio.getHeight();
        for (int a2 = 0; a2 < choices.size(); a2++) {
            Size option = choices.get(a2);
            if (option.getHeight() == (option.getWidth() * h) / w && option.getWidth() >= width && option.getHeight() >= height) {
                bigEnough.add(option);
            }
        }
        int a3 = bigEnough.size();
        if (a3 > 0) {
            return (Size) Collections.min(bigEnough, new CompareSizesByArea());
        }
        return (Size) Collections.max(choices, new CompareSizesByArea());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* loaded from: classes17.dex */
    public static class CompareSizesByArea implements Comparator<Size> {
        CompareSizesByArea() {
        }

        @Override // java.util.Comparator
        public int compare(Size lhs, Size rhs) {
            return Long.signum((lhs.getWidth() * lhs.getHeight()) - (rhs.getWidth() * rhs.getHeight()));
        }
    }
}
