package im.uecahxyvjd.messenger.utils.status;

import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
/* loaded from: classes17.dex */
public class StatusBarUtils {
    public static final int TYPE_FLYME = 1;
    public static final int TYPE_M = 3;
    public static final int TYPE_MIUI = 0;

    @Retention(RetentionPolicy.SOURCE)
    /* loaded from: classes17.dex */
    @interface ViewType {
    }

    public static void setStatusBarColor(Activity activity, int colorId) {
        Window window = activity.getWindow();
        window.setStatusBarColor(colorId);
    }

    public static void setTranslucentStatus(Activity activity) {
        Window window = activity.getWindow();
        View decorView = window.getDecorView();
        decorView.setSystemUiVisibility(1280);
        window.addFlags(Integer.MIN_VALUE);
        window.setStatusBarColor(0);
    }

    public static void setRootViewFitsSystemWindows(Activity activity, boolean fitSystemWindows) {
        ViewGroup rootView;
        ViewGroup winContent = (ViewGroup) activity.findViewById(16908290);
        if (winContent.getChildCount() > 0 && (rootView = (ViewGroup) winContent.getChildAt(0)) != null) {
            rootView.setFitsSystemWindows(fitSystemWindows);
        }
    }

    public static boolean setStatusBarDarkTheme(Activity activity, boolean dark) {
        if (Build.VERSION.SDK_INT >= 23) {
            setStatusBarFontIconDark(activity, 3, dark);
        } else if (OSUtils.isMiui()) {
            setStatusBarFontIconDark(activity, 0, dark);
        } else if (!OSUtils.isFlyme()) {
            return false;
        } else {
            setStatusBarFontIconDark(activity, 1, dark);
        }
        return true;
    }

    public static boolean setStatusBarFontIconDark(Activity activity, int type, boolean dark) {
        switch (type) {
            case 0:
                return setMiuiUI(activity, dark);
            case 1:
                return setFlymeUI(activity, dark);
            default:
                return setCommonUI(activity, dark);
        }
    }

    public static boolean setCommonUI(Activity activity, boolean dark) {
        View decorView;
        int vis;
        if (Build.VERSION.SDK_INT >= 23 && (decorView = activity.getWindow().getDecorView()) != null) {
            int vis2 = decorView.getSystemUiVisibility();
            if (dark) {
                vis = vis2 | 8192;
            } else {
                vis = vis2 & (-8193);
            }
            if (decorView.getSystemUiVisibility() != vis) {
                decorView.setSystemUiVisibility(vis);
                return true;
            }
            return true;
        }
        return false;
    }

    public static boolean setFlymeUI(Activity activity, boolean dark) {
        int value;
        try {
            Window window = activity.getWindow();
            WindowManager.LayoutParams lp = window.getAttributes();
            Field darkFlag = WindowManager.LayoutParams.class.getDeclaredField("MEIZU_FLAG_DARK_STATUS_BAR_ICON");
            Field meizuFlags = WindowManager.LayoutParams.class.getDeclaredField("meizuFlags");
            darkFlag.setAccessible(true);
            meizuFlags.setAccessible(true);
            int bit = darkFlag.getInt(null);
            int value2 = meizuFlags.getInt(lp);
            if (dark) {
                value = value2 | bit;
            } else {
                value = value2 & (~bit);
            }
            meizuFlags.setInt(lp, value);
            window.setAttributes(lp);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static boolean setMiuiUI(Activity activity, boolean dark) {
        try {
            Window window = activity.getWindow();
            Class<?> clazz = activity.getWindow().getClass();
            Class<?> layoutParams = Class.forName("android.view.MiuiWindowManager$LayoutParams");
            Field field = layoutParams.getField("EXTRA_FLAG_STATUS_BAR_DARK_MODE");
            int darkModeFlag = field.getInt(layoutParams);
            Method extraFlagField = clazz.getDeclaredMethod("setExtraFlags", Integer.TYPE, Integer.TYPE);
            extraFlagField.setAccessible(true);
            if (dark) {
                extraFlagField.invoke(window, Integer.valueOf(darkModeFlag), Integer.valueOf(darkModeFlag));
            } else {
                extraFlagField.invoke(window, 0, Integer.valueOf(darkModeFlag));
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static int getStatusBarHeight(Context context) {
        int resourceId = context.getResources().getIdentifier("status_bar_height", "dimen", "android");
        if (resourceId <= 0) {
            return 0;
        }
        int result = context.getResources().getDimensionPixelSize(resourceId);
        return result;
    }
}
