package im.uecahxyvjd.messenger.voip;

import android.content.SharedPreferences;
import android.media.audiofx.AcousticEchoCanceler;
import android.media.audiofx.NoiseSuppressor;
import android.os.SystemClock;
import im.uecahxyvjd.messenger.ApplicationLoader;
import im.uecahxyvjd.messenger.BuildVars;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.components.voip.VoIPHelper;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Locale;
/* loaded from: classes6.dex */
public class VoIPController {
    public static final int DATA_SAVING_ALWAYS = 2;
    public static final int DATA_SAVING_MOBILE = 1;
    public static final int DATA_SAVING_NEVER = 0;
    public static final int DATA_SAVING_ROAMING = 3;
    public static final int ERROR_AUDIO_IO = 3;
    public static final int ERROR_CONNECTION_SERVICE = -5;
    public static final int ERROR_INCOMPATIBLE = 1;
    public static final int ERROR_INSECURE_UPGRADE = -4;
    public static final int ERROR_LOCALIZED = -3;
    public static final int ERROR_PEER_OUTDATED = -1;
    public static final int ERROR_PRIVACY = -2;
    public static final int ERROR_TIMEOUT = 2;
    public static final int ERROR_UNKNOWN = 0;
    public static final int NET_TYPE_3G = 3;
    public static final int NET_TYPE_DIALUP = 10;
    public static final int NET_TYPE_EDGE = 2;
    public static final int NET_TYPE_ETHERNET = 7;
    public static final int NET_TYPE_GPRS = 1;
    public static final int NET_TYPE_HSPA = 4;
    public static final int NET_TYPE_LTE = 5;
    public static final int NET_TYPE_OTHER_HIGH_SPEED = 8;
    public static final int NET_TYPE_OTHER_LOW_SPEED = 9;
    public static final int NET_TYPE_OTHER_MOBILE = 11;
    public static final int NET_TYPE_UNKNOWN = 0;
    public static final int NET_TYPE_WIFI = 6;
    public static final int PEER_CAP_GROUP_CALLS = 1;
    public static final int STATE_ESTABLISHED = 3;
    public static final int STATE_FAILED = 4;
    public static final int STATE_RECONNECTING = 5;
    public static final int STATE_WAIT_INIT = 1;
    public static final int STATE_WAIT_INIT_ACK = 2;
    protected long callStartTime;
    protected ConnectionStateListener listener;
    protected long nativeInst;

    /* loaded from: classes6.dex */
    public interface ConnectionStateListener {
        void onCallUpgradeRequestReceived();

        void onConnectionStateChanged(int i);

        void onGroupCallKeyReceived(byte[] bArr);

        void onGroupCallKeySent();

        void onSignalBarCountChanged(int i);
    }

    public static native int getConnectionMaxLayer();

    private native void nativeConnect(long j);

    private native void nativeDebugCtl(long j, int i, int i2);

    private native String nativeGetDebugLog(long j);

    private native String nativeGetDebugString(long j);

    private native int nativeGetLastError(long j);

    private native int nativeGetPeerCapabilities(long j);

    private native long nativeGetPreferredRelayID(long j);

    private native void nativeGetStats(long j, Stats stats);

    private static native String nativeGetVersion();

    private native long nativeInit(String str);

    private static native boolean nativeNeedRate(long j);

    private native void nativeRelease(long j);

    private native void nativeRequestCallUpgrade(long j);

    private native void nativeSendGroupCallKey(long j, byte[] bArr);

    private native void nativeSetAudioOutputGainControlEnabled(long j, boolean z);

    private native void nativeSetConfig(long j, double d, double d2, int i, boolean z, boolean z2, boolean z3, String str, String str2, boolean z4);

    private native void nativeSetEchoCancellationStrength(long j, int i);

    private native void nativeSetEncryptionKey(long j, byte[] bArr, boolean z);

    private native void nativeSetMicMute(long j, boolean z);

    private static native void nativeSetNativeBufferSize(int i);

    private native void nativeSetNetworkType(long j, int i);

    private native void nativeSetProxy(long j, String str, int i, String str2, String str3);

    private native void nativeSetRemoteEndpoints(long j, TLRPC.TL_phoneConnection[] tL_phoneConnectionArr, boolean z, boolean z2, int i);

    public static native void nativeSetVideoRenderer(long j, long j2);

    public static native void nativeSetVideoSource(long j, long j2);

    private native void nativeStart(long j);

    public VoIPController() {
        this.nativeInst = 0L;
        this.nativeInst = nativeInit(new File(ApplicationLoader.applicationContext.getFilesDir(), "voip_persistent_state.json").getAbsolutePath());
    }

    public void start() {
        ensureNativeInstance();
        nativeStart(this.nativeInst);
    }

    public void connect() {
        ensureNativeInstance();
        nativeConnect(this.nativeInst);
    }

    public void setRemoteEndpoints(TLRPC.TL_phoneConnection[] endpoints, boolean allowP2p, boolean tcp, int connectionMaxLayer) {
        if (endpoints.length == 0) {
            throw new IllegalArgumentException("endpoints size is 0");
        }
        for (TLRPC.TL_phoneConnection endpoint : endpoints) {
            if (endpoint.ip == null || endpoint.ip.length() == 0) {
                throw new IllegalArgumentException("endpoint " + endpoint + " has empty/null ipv4");
            }
            if (endpoint.peer_tag != null && endpoint.peer_tag.length != 16) {
                throw new IllegalArgumentException("endpoint " + endpoint + " has peer_tag of wrong length");
            }
        }
        ensureNativeInstance();
        nativeSetRemoteEndpoints(this.nativeInst, endpoints, allowP2p, tcp, connectionMaxLayer);
    }

    public void setEncryptionKey(byte[] key, boolean isOutgoing) {
        if (key.length != 256) {
            throw new IllegalArgumentException("key length must be exactly 256 bytes but is " + key.length);
        }
        ensureNativeInstance();
        nativeSetEncryptionKey(this.nativeInst, key, isOutgoing);
    }

    public static void setNativeBufferSize(int size) {
        nativeSetNativeBufferSize(size);
    }

    public void release() {
        ensureNativeInstance();
        nativeRelease(this.nativeInst);
        this.nativeInst = 0L;
    }

    public String getDebugString() {
        ensureNativeInstance();
        return nativeGetDebugString(this.nativeInst);
    }

    protected void ensureNativeInstance() {
        if (this.nativeInst == 0) {
            throw new IllegalStateException("Native instance is not valid");
        }
    }

    public void setConnectionStateListener(ConnectionStateListener connectionStateListener) {
        this.listener = connectionStateListener;
    }

    private void handleStateChange(int state) {
        if (state == 3 && this.callStartTime == 0) {
            this.callStartTime = SystemClock.elapsedRealtime();
        }
        ConnectionStateListener connectionStateListener = this.listener;
        if (connectionStateListener != null) {
            connectionStateListener.onConnectionStateChanged(state);
        }
    }

    private void handleSignalBarsChange(int count) {
        ConnectionStateListener connectionStateListener = this.listener;
        if (connectionStateListener != null) {
            connectionStateListener.onSignalBarCountChanged(count);
        }
    }

    private void groupCallKeyReceived(byte[] key) {
        ConnectionStateListener connectionStateListener = this.listener;
        if (connectionStateListener != null) {
            connectionStateListener.onGroupCallKeyReceived(key);
        }
    }

    private void groupCallKeySent() {
        ConnectionStateListener connectionStateListener = this.listener;
        if (connectionStateListener != null) {
            connectionStateListener.onGroupCallKeySent();
        }
    }

    private void callUpgradeRequestReceived() {
        ConnectionStateListener connectionStateListener = this.listener;
        if (connectionStateListener != null) {
            connectionStateListener.onCallUpgradeRequestReceived();
        }
    }

    public void setNetworkType(int type) {
        ensureNativeInstance();
        nativeSetNetworkType(this.nativeInst, type);
    }

    public long getCallDuration() {
        return SystemClock.elapsedRealtime() - this.callStartTime;
    }

    public void setMicMute(boolean mute) {
        ensureNativeInstance();
        nativeSetMicMute(this.nativeInst, mute);
    }

    public void setConfig(double recvTimeout, double initTimeout, int dataSavingOption, long callID) {
        boolean sysAecAvailable;
        boolean sysNsAvailable;
        ensureNativeInstance();
        boolean sysAecAvailable2 = false;
        try {
            sysAecAvailable2 = AcousticEchoCanceler.isAvailable();
            boolean sysNsAvailable2 = NoiseSuppressor.isAvailable();
            sysAecAvailable = sysAecAvailable2;
            sysNsAvailable = sysNsAvailable2;
        } catch (Throwable th) {
            sysAecAvailable = sysAecAvailable2;
            sysNsAvailable = false;
        }
        SharedPreferences preferences = MessagesController.getGlobalMainSettings();
        boolean dump = preferences.getBoolean("dbg_dump_call_stats", false);
        nativeSetConfig(this.nativeInst, recvTimeout, initTimeout, dataSavingOption, (sysAecAvailable && VoIPServerConfig.getBoolean("use_system_aec", true)) ? false : true, (sysNsAvailable && VoIPServerConfig.getBoolean("use_system_ns", true)) ? false : true, true, BuildVars.DEBUG_VERSION ? getLogFilePath("voip" + callID) : getLogFilePath(callID), (BuildVars.DEBUG_VERSION && dump) ? getLogFilePath("voipStats") : null, BuildVars.DEBUG_VERSION);
    }

    public void debugCtl(int request, int param) {
        ensureNativeInstance();
        nativeDebugCtl(this.nativeInst, request, param);
    }

    public long getPreferredRelayID() {
        ensureNativeInstance();
        return nativeGetPreferredRelayID(this.nativeInst);
    }

    public int getLastError() {
        ensureNativeInstance();
        return nativeGetLastError(this.nativeInst);
    }

    public void getStats(Stats stats) {
        ensureNativeInstance();
        if (stats == null) {
            throw new NullPointerException("You're not supposed to pass null here");
        }
        nativeGetStats(this.nativeInst, stats);
    }

    public static String getVersion() {
        return nativeGetVersion();
    }

    private String getLogFilePath(String name) {
        Calendar c = Calendar.getInstance();
        return new File(ApplicationLoader.applicationContext.getExternalFilesDir(null), String.format(Locale.US, "logs/%02d_%02d_%04d_%02d_%02d_%02d_%s.txt", Integer.valueOf(c.get(5)), Integer.valueOf(c.get(2) + 1), Integer.valueOf(c.get(1)), Integer.valueOf(c.get(11)), Integer.valueOf(c.get(12)), Integer.valueOf(c.get(13)), name)).getAbsolutePath();
    }

    private String getLogFilePath(long callID) {
        File dir = VoIPHelper.getLogsDir();
        if (!BuildVars.DEBUG_VERSION) {
            File[] _logs = dir.listFiles();
            ArrayList<File> logs = new ArrayList<>();
            logs.addAll(Arrays.asList(_logs));
            while (logs.size() > 20) {
                File oldest = logs.get(0);
                Iterator<File> it = logs.iterator();
                while (it.hasNext()) {
                    File file = it.next();
                    if (file.getName().endsWith(".log") && file.lastModified() < oldest.lastModified()) {
                        oldest = file;
                    }
                }
                oldest.delete();
                logs.remove(oldest);
            }
        }
        return new File(dir, callID + ".log").getAbsolutePath();
    }

    public String getDebugLog() {
        ensureNativeInstance();
        return nativeGetDebugLog(this.nativeInst);
    }

    public void setProxy(String address, int port, String username, String password) {
        ensureNativeInstance();
        if (address == null) {
            throw new NullPointerException("address can't be null");
        }
        nativeSetProxy(this.nativeInst, address, port, username, password);
    }

    public void setAudioOutputGainControlEnabled(boolean enabled) {
        ensureNativeInstance();
        nativeSetAudioOutputGainControlEnabled(this.nativeInst, enabled);
    }

    public int getPeerCapabilities() {
        ensureNativeInstance();
        return nativeGetPeerCapabilities(this.nativeInst);
    }

    public void sendGroupCallKey(byte[] key) {
        if (key == null) {
            throw new NullPointerException("key can not be null");
        }
        if (key.length != 256) {
            throw new IllegalArgumentException("key must be 256 bytes long, got " + key.length);
        }
        ensureNativeInstance();
        nativeSendGroupCallKey(this.nativeInst, key);
    }

    public void requestCallUpgrade() {
        ensureNativeInstance();
        nativeRequestCallUpgrade(this.nativeInst);
    }

    public void setEchoCancellationStrength(int strength) {
        ensureNativeInstance();
        nativeSetEchoCancellationStrength(this.nativeInst, strength);
    }

    public boolean needRate() {
        ensureNativeInstance();
        return nativeNeedRate(this.nativeInst);
    }

    /* loaded from: classes6.dex */
    public static class Stats {
        public long bytesRecvdMobile;
        public long bytesRecvdWifi;
        public long bytesSentMobile;
        public long bytesSentWifi;

        public String toString() {
            return "Stats{bytesRecvdMobile=" + this.bytesRecvdMobile + ", bytesSentWifi=" + this.bytesSentWifi + ", bytesRecvdWifi=" + this.bytesRecvdWifi + ", bytesSentMobile=" + this.bytesSentMobile + '}';
        }
    }
}
