package im.uecahxyvjd.phoneformat;

import java.util.ArrayList;
import java.util.Iterator;
/* loaded from: classes4.dex */
public class CallingCodeInfo {
    public ArrayList<String> countries = new ArrayList<>();
    public String callingCode = "";
    public ArrayList<String> trunkPrefixes = new ArrayList<>();
    public ArrayList<String> intlPrefixes = new ArrayList<>();
    public ArrayList<RuleSet> ruleSets = new ArrayList<>();

    /* JADX INFO: Access modifiers changed from: package-private */
    public String matchingAccessCode(String str) {
        Iterator<String> it = this.intlPrefixes.iterator();
        while (it.hasNext()) {
            String code = it.next();
            if (str.startsWith(code)) {
                return code;
            }
        }
        return null;
    }

    String matchingTrunkCode(String str) {
        Iterator<String> it = this.trunkPrefixes.iterator();
        while (it.hasNext()) {
            String code = it.next();
            if (str.startsWith(code)) {
                return code;
            }
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String format(String orig) {
        String str = orig;
        String trunkPrefix = null;
        String intlPrefix = null;
        if (str.startsWith(this.callingCode)) {
            intlPrefix = this.callingCode;
            str = str.substring(intlPrefix.length());
        } else {
            String trunk = matchingTrunkCode(str);
            if (trunk != null) {
                trunkPrefix = trunk;
                str = str.substring(trunkPrefix.length());
            }
        }
        Iterator<RuleSet> it = this.ruleSets.iterator();
        while (it.hasNext()) {
            RuleSet set = it.next();
            String phone = set.format(str, intlPrefix, trunkPrefix, true);
            if (phone != null) {
                return phone;
            }
        }
        Iterator<RuleSet> it2 = this.ruleSets.iterator();
        while (it2.hasNext()) {
            RuleSet set2 = it2.next();
            String phone2 = set2.format(str, intlPrefix, trunkPrefix, false);
            if (phone2 != null) {
                return phone2;
            }
        }
        return (intlPrefix == null || str.length() == 0) ? orig : String.format("%s %s", intlPrefix, str);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String desensitization(String orig) {
        String str = orig;
        String trunkPrefix = null;
        String intlPrefix = null;
        if (str.startsWith(this.callingCode)) {
            intlPrefix = this.callingCode;
            str = str.substring(intlPrefix.length());
        } else {
            String trunk = matchingTrunkCode(str);
            if (trunk != null) {
                trunkPrefix = trunk;
                str = str.substring(trunkPrefix.length());
            }
        }
        Iterator<RuleSet> it = this.ruleSets.iterator();
        while (it.hasNext()) {
            RuleSet set = it.next();
            String phone = set.desensitization(str, intlPrefix, trunkPrefix, true);
            if (phone != null) {
                return phone;
            }
        }
        Iterator<RuleSet> it2 = this.ruleSets.iterator();
        while (it2.hasNext()) {
            RuleSet set2 = it2.next();
            String phone2 = set2.desensitization(str, intlPrefix, trunkPrefix, false);
            if (phone2 != null) {
                return phone2;
            }
        }
        return (intlPrefix == null || str.length() == 0) ? orig : String.format("%s %s", intlPrefix, str);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean isValidPhoneNumber(String orig) {
        String str = orig;
        String trunkPrefix = null;
        String intlPrefix = null;
        if (str.startsWith(this.callingCode)) {
            intlPrefix = this.callingCode;
            str = str.substring(intlPrefix.length());
        } else {
            String trunk = matchingTrunkCode(str);
            if (trunk != null) {
                trunkPrefix = trunk;
                str = str.substring(trunkPrefix.length());
            }
        }
        Iterator<RuleSet> it = this.ruleSets.iterator();
        while (it.hasNext()) {
            RuleSet set = it.next();
            boolean valid = set.isValid(str, intlPrefix, trunkPrefix, true);
            if (valid) {
                return true;
            }
        }
        Iterator<RuleSet> it2 = this.ruleSets.iterator();
        while (it2.hasNext()) {
            RuleSet set2 = it2.next();
            boolean valid2 = set2.isValid(str, intlPrefix, trunkPrefix, false);
            if (valid2) {
                return true;
            }
        }
        return false;
    }
}
