package im.uecahxyvjd.tgnet;

import com.blankj.utilcode.util.GsonUtils;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.utils.apache.StringEscapeUtils;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.List;
/* loaded from: classes7.dex */
public class TLJsonResolve {
    public static String TAG = TLJsonResolve.class.getName();

    public static String getData(TLObject response) {
        if (response == null) {
            return null;
        }
        if (response instanceof TLRPC.TL_dataJSON) {
            String jsonData = ((TLRPC.TL_dataJSON) response).data;
            return jsonData;
        }
        Field[] fields = response.getClass().getDeclaredFields();
        for (Field f : fields) {
            if (TLRPC.TL_dataJSON.class.getName().equals(f.getType().getName())) {
                f.setAccessible(true);
                try {
                    Object o = f.get(response);
                    if (o == null) {
                        return null;
                    }
                    String jsonData2 = ((TLRPC.TL_dataJSON) o).data;
                    return jsonData2;
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                    return null;
                }
            }
        }
        return null;
    }

    public static <T> TLApiModel<T> parse(String jsonData, Class<?> clz) {
        TLRPC.TL_dataJSON dataJSON = new TLRPC.TL_dataJSON();
        dataJSON.data = jsonData;
        return parse(dataJSON, clz);
    }

    public static <T> TLApiModel<T> parse(TLObject response, Class<?> clz) {
        TLApiModel model = null;
        String errorMsg = null;
        try {
            String data = getData(response);
            model = (TLApiModel) GsonUtils.fromJson(data, (Class<Object>) TLApiModel.class);
            if (model.data != null) {
                model.data = StringEscapeUtils.unescapeJava(model.data);
                if (model.data != null) {
                    try {
                        model.data = model.data.replaceAll("\"\\{", "{");
                    } catch (Exception e) {
                    }
                    try {
                        model.data = model.data.replaceAll("}\"", "}");
                    } catch (Exception e2) {
                    }
                    try {
                        model.data = model.data.replaceAll("\"\\[", "[");
                    } catch (Exception e3) {
                    }
                    try {
                        model.data = model.data.replaceAll("]\"", "]");
                    } catch (Exception e4) {
                    }
                    if (!model.data.startsWith("{")) {
                        if (model.data.startsWith("[")) {
                            model.modelList = (List) GsonUtils.fromJson(model.data, new ParameterizedTypeImpl(clz));
                        }
                    } else {
                        model.model = (T) GsonUtils.fromJson(model.data, (Class<Object>) clz);
                    }
                }
            }
        } catch (Exception e5) {
            errorMsg = e5.getMessage();
            FileLog.e(TAG + " =====> " + e5.getMessage());
        }
        if (model == null) {
            model = new TLApiModel();
            model.code = "-9999";
            if (errorMsg != null) {
                model.message = errorMsg;
            } else {
                model.message = LocaleController.getString("TLJsonResolveParseFaild", R.string.TLJsonResolveParseFaild);
            }
        }
        return model;
    }

    public static <T> TLApiModel<T> parse3(TLObject response, Class<?> clz) {
        TLApiModel model = null;
        String errorMsg = null;
        try {
            String data = getData(response);
            model = (TLApiModel) GsonUtils.fromJson(data, (Class<Object>) TLApiModel.class);
            if (model.data != null && model.data != null) {
                if (model.data.startsWith("{")) {
                    model.model = (T) GsonUtils.fromJson(model.data, (Class<Object>) clz);
                } else if (model.data.startsWith("[")) {
                    model.modelList = (List) GsonUtils.fromJson(model.data, new ParameterizedTypeImpl(clz));
                }
            }
        } catch (Exception e) {
            errorMsg = e.getMessage();
            FileLog.e(TAG + " =====> " + e.getMessage());
        }
        if (model == null) {
            model = new TLApiModel();
            model.code = "-9999";
            if (errorMsg != null) {
                model.message = errorMsg;
            } else {
                model.message = LocaleController.getString("TLJsonResolveParseFaild", R.string.TLJsonResolveParseFaild);
            }
        }
        return model;
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes7.dex */
    public static class ParameterizedTypeImpl implements ParameterizedType {
        Class clazz;

        public ParameterizedTypeImpl(Class clz) {
            this.clazz = clz;
        }

        @Override // java.lang.reflect.ParameterizedType
        public Type[] getActualTypeArguments() {
            return new Type[]{this.clazz};
        }

        @Override // java.lang.reflect.ParameterizedType
        public Type getRawType() {
            return List.class;
        }

        @Override // java.lang.reflect.ParameterizedType
        public Type getOwnerType() {
            return null;
        }
    }
}
