package im.uecahxyvjd.ui;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ApplicationLoader;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.ui.CountrySelectActivity;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.actionbar.ThemeDescription;
import im.uecahxyvjd.ui.cells.LetterSectionCell;
import im.uecahxyvjd.ui.cells.TextSettingsCell;
import im.uecahxyvjd.ui.components.EmptyTextProgressView;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.RecyclerListView;
import im.uecahxyvjd.ui.decoration.StickyDecoration;
import im.uecahxyvjd.ui.decoration.listener.GroupListener;
import im.uecahxyvjd.ui.hui.CharacterParser;
import im.uecahxyvjd.ui.hui.decoration.TopBottomDecoration;
import im.uecahxyvjd.ui.hviews.search.MrySearchView;
import im.uecahxyvjd.ui.hviews.sidebar.SideBar;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.regex.Pattern;
import org.slf4j.Marker;
/* loaded from: classes12.dex */
public class CountrySelectActivity extends BaseSearchViewFragment {
    private CountrySelectActivityDelegate delegate;
    private EmptyTextProgressView emptyView;
    private RecyclerListView listView;
    private CountryAdapter listViewAdapter;
    private boolean needChangeChar;
    private boolean needPhoneCode;
    private CountrySearchAdapter searchListViewAdapter;
    private boolean searchWas;
    private boolean searching;
    private SideBar sideBar;

    /* loaded from: classes12.dex */
    public static class Country {
        public String code;
        public String name;
        public String phoneFormat;
        public String shortname;
    }

    /* loaded from: classes12.dex */
    public interface CountrySelectActivityDelegate {
        void didSelectCountry(Country country);
    }

    public CountrySelectActivity(boolean phoneCode) {
        this.needPhoneCode = phoneCode;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onFragmentCreate() {
        return super.onFragmentCreate();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onFragmentDestroy() {
        RecyclerListView recyclerListView = this.listView;
        if (recyclerListView != null) {
            recyclerListView.setOnScrollListener(null);
        }
        super.onFragmentDestroy();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment
    protected MrySearchView getSearchView() {
        this.searchView = new MrySearchView(getParentActivity());
        int margin = AndroidUtilities.dp(10.0f);
        ((FrameLayout) this.fragmentView).addView(this.searchView, LayoutHelper.createFrame(-1, 35, margin, margin, margin, margin));
        return this.searchView;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment, im.uecahxyvjd.ui.actionbar.BaseFragment
    public View createView(Context context) {
        this.searching = false;
        this.searchWas = false;
        this.fragmentView = new FrameLayout(context);
        this.fragmentView.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundGray));
        FrameLayout frameLayout = (FrameLayout) this.fragmentView;
        initActionBar();
        super.createView(context);
        initList(frameLayout, context);
        initSideBar(frameLayout, context);
        return this.fragmentView;
    }

    private void initActionBar() {
        this.actionBar.setBackButtonImage(R.mipmap.ic_back);
        this.actionBar.setAllowOverlayTitle(true);
        this.actionBar.setCastShadows(false);
        this.actionBar.setTitle(LocaleController.getString("ChooseCountry", R.string.ChooseCountry));
        this.actionBar.setActionBarMenuOnItemClick(new ActionBar.ActionBarMenuOnItemClick() { // from class: im.uecahxyvjd.ui.CountrySelectActivity.1
            @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
            public void onItemClick(int id) {
                if (id == -1) {
                    CountrySelectActivity.this.finishFragment();
                }
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: protected */
    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment
    public void initSearchView() {
        super.initSearchView();
        this.searchView.setHintText(LocaleController.getString("Search", R.string.Search));
    }

    private void initList(FrameLayout frameLayout, Context context) {
        this.listViewAdapter = new CountryAdapter(context);
        this.searchListViewAdapter = new CountrySearchAdapter(context, this.listViewAdapter.getCountries());
        EmptyTextProgressView emptyTextProgressView = new EmptyTextProgressView(context);
        this.emptyView = emptyTextProgressView;
        emptyTextProgressView.showTextView();
        this.emptyView.setShowAtCenter(true);
        this.emptyView.setText(LocaleController.getString("NoResult", R.string.NoResult));
        frameLayout.addView(this.emptyView, LayoutHelper.createFrame(-1, -1, 0, AndroidUtilities.dp(55.0f), 0, 0));
        RecyclerListView recyclerListView = new RecyclerListView(context);
        this.listView = recyclerListView;
        recyclerListView.setOverScrollMode(2);
        this.listView.setEmptyView(this.emptyView);
        this.listView.setVerticalScrollBarEnabled(false);
        this.listView.setLayoutManager(new LinearLayoutManager(context, 1, false));
        StickyDecoration.Builder decorationBuilder = StickyDecoration.Builder.init(new GroupListener() { // from class: im.uecahxyvjd.ui.CountrySelectActivity$$ExternalSyntheticLambda1
            @Override // im.uecahxyvjd.ui.decoration.listener.GroupListener
            public final String getGroupName(int i) {
                return CountrySelectActivity.this.m1521lambda$initList$0$imuecahxyvjduiCountrySelectActivity(i);
            }
        }).setGroupBackground(Theme.getColor(Theme.key_windowBackgroundGray)).setGroupTextColor(Theme.getColor(Theme.key_windowBackgroundWhiteBlackText)).setGroupTextTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf")).setGroupHeight(AndroidUtilities.dp(24.0f)).setDivideColor(Color.parseColor("#EE96BC")).setGroupTextSize(AndroidUtilities.dp(14.0f)).setTextSideMargin(AndroidUtilities.dp(15.0f));
        StickyDecoration decoration = decorationBuilder.build();
        this.listView.addItemDecoration(decoration);
        this.listView.addItemDecoration(TopBottomDecoration.getDefaultTopBottomCornerBg(0, 10, 7.5f));
        this.listView.setAdapter(this.listViewAdapter);
        this.listView.setVerticalScrollbarPosition(LocaleController.isRTL ? 1 : 2);
        frameLayout.addView(this.listView, LayoutHelper.createFrame(-1, -1, AndroidUtilities.dp(10.0f), AndroidUtilities.dp(55.0f), AndroidUtilities.dp(10.0f), 0));
        this.listView.setOnItemClickListener(new RecyclerListView.OnItemClickListener() { // from class: im.uecahxyvjd.ui.CountrySelectActivity$$ExternalSyntheticLambda2
            @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemClickListener
            public final void onItemClick(View view, int i) {
                CountrySelectActivity.this.m1522lambda$initList$1$imuecahxyvjduiCountrySelectActivity(view, i);
            }
        });
        this.listView.setOnScrollListener(new RecyclerView.OnScrollListener() { // from class: im.uecahxyvjd.ui.CountrySelectActivity.2
            @Override // androidx.recyclerview.widget.RecyclerView.OnScrollListener
            public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                if (newState == 1 && CountrySelectActivity.this.searching && CountrySelectActivity.this.searchWas) {
                    AndroidUtilities.hideKeyboard(CountrySelectActivity.this.getParentActivity().getCurrentFocus());
                }
            }

            @Override // androidx.recyclerview.widget.RecyclerView.OnScrollListener
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                LinearLayoutManager layoutManager = (LinearLayoutManager) recyclerView.getLayoutManager();
                int firstPosition = layoutManager.findFirstVisibleItemPosition();
                String s = CountrySelectActivity.this.listViewAdapter.getLetter(firstPosition);
                if (TextUtils.isEmpty(s) && CountrySelectActivity.this.listViewAdapter.getSectionForPosition(firstPosition) == 0) {
                    s = CountrySelectActivity.this.listViewAdapter.getLetter(CountrySelectActivity.this.listViewAdapter.getPositionForSection(1));
                }
                CountrySelectActivity.this.sideBar.setChooseChar(s);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initList$0$im-uecahxyvjd-ui-CountrySelectActivity  reason: not valid java name */
    public /* synthetic */ String m1521lambda$initList$0$imuecahxyvjduiCountrySelectActivity(int position) {
        if (this.searchWas) {
            CountrySearchAdapter countrySearchAdapter = this.searchListViewAdapter;
            if (countrySearchAdapter != null && countrySearchAdapter.getItemCount() > 0) {
                String name = this.searchListViewAdapter.getItem(0).name;
                if (!TextUtils.isEmpty(name)) {
                    if (this.needChangeChar) {
                        return CharacterParser.getInstance().getSelling(name).substring(0, 1).toUpperCase();
                    }
                    return name.substring(0, 1).toUpperCase();
                }
                return null;
            }
            return null;
        } else if (this.listViewAdapter.getItemCount() > position && position > -1) {
            String letter = this.listViewAdapter.getLetter(position);
            return letter;
        } else {
            return null;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initList$1$im-uecahxyvjd-ui-CountrySelectActivity  reason: not valid java name */
    public /* synthetic */ void m1522lambda$initList$1$imuecahxyvjduiCountrySelectActivity(View view, int position) {
        Country country;
        CountrySelectActivityDelegate countrySelectActivityDelegate;
        if (this.searching && this.searchWas) {
            country = this.searchListViewAdapter.getItem(position);
        } else {
            int section = this.listViewAdapter.getSectionForPosition(position);
            int row = this.listViewAdapter.getPositionInSectionForPosition(position);
            if (row < 0 || section < 0) {
                return;
            }
            country = this.listViewAdapter.getItem(section, row);
        }
        if (position < 0) {
            return;
        }
        finishFragment();
        if (country != null && (countrySelectActivityDelegate = this.delegate) != null) {
            countrySelectActivityDelegate.didSelectCountry(country);
        }
    }

    private void initSideBar(FrameLayout frameLayout, Context context) {
        TextView textView = new TextView(context);
        textView.setTextSize(50.0f);
        textView.setGravity(17);
        textView.setTextColor(Theme.getColor(Theme.key_windowBackgroundWhiteBlackText));
        frameLayout.addView(textView, LayoutHelper.createFrame(100, 100, 17));
        SideBar sideBar = new SideBar(context);
        this.sideBar = sideBar;
        sideBar.setCharsOnly();
        this.sideBar.setTextView(textView);
        frameLayout.addView(this.sideBar, LayoutHelper.createFrame(35.0f, 420.0f, 21, 0.0f, 56.0f, 0.0f, 56.0f));
        this.sideBar.setOnTouchingLetterChangedListener(new SideBar.OnTouchingLetterChangedListener() { // from class: im.uecahxyvjd.ui.CountrySelectActivity$$ExternalSyntheticLambda0
            @Override // im.uecahxyvjd.ui.hviews.sidebar.SideBar.OnTouchingLetterChangedListener
            public final void onTouchingLetterChanged(String str) {
                CountrySelectActivity.this.m1523lambda$initSideBar$2$imuecahxyvjduiCountrySelectActivity(str);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initSideBar$2$im-uecahxyvjd-ui-CountrySelectActivity  reason: not valid java name */
    public /* synthetic */ void m1523lambda$initSideBar$2$imuecahxyvjduiCountrySelectActivity(String s) {
        int section = 0;
        if (!"#".equals(s)) {
            section = this.listViewAdapter.getSectionForChar(s.charAt(0));
        }
        int position = this.listViewAdapter.getPositionForSection(section);
        if (position != -1) {
            this.listView.getLayoutManager().scrollToPosition(position);
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onResume() {
        super.onResume();
        CountryAdapter countryAdapter = this.listViewAdapter;
        if (countryAdapter != null) {
            countryAdapter.notifyDataSetChanged();
        }
    }

    public void setCountrySelectActivityDelegate(CountrySelectActivityDelegate delegate) {
        this.delegate = delegate;
    }

    /* loaded from: classes12.dex */
    public class CountryAdapter extends RecyclerListView.SectionsAdapter {
        private Context mContext;
        private HashMap<String, ArrayList<Country>> countries = new HashMap<>();
        private ArrayList<String> sortedCountries = new ArrayList<>();

        /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
        public CountryAdapter(Context context) {
            char c;
            InputStream stream;
            BufferedReader reader;
            String n;
            char c2;
            this.mContext = context;
            String countryFileName = null;
            char c3 = 0;
            try {
                LocaleController.LocaleInfo localeInfo = LocaleController.getInstance().getCurrentLocaleInfo();
                c = 2;
                if (localeInfo != null && localeInfo.nameEnglish != null) {
                    String str = localeInfo.nameEnglish;
                    switch (str.hashCode()) {
                        case -1303729356:
                            if (str.equals("Traditional Chinese")) {
                                c2 = 2;
                                break;
                            }
                            c2 = 65535;
                            break;
                        case 60895824:
                            if (str.equals("English")) {
                                c2 = 0;
                                break;
                            }
                            c2 = 65535;
                            break;
                        case 1132825637:
                            if (str.equals("Simplified Chinese")) {
                                c2 = 1;
                                break;
                            }
                            c2 = 65535;
                            break;
                        default:
                            c2 = 65535;
                            break;
                    }
                    switch (c2) {
                        case 0:
                            countryFileName = "countries.txt";
                            break;
                        case 1:
                            countryFileName = "countries_cn.txt";
                            CountrySelectActivity.this.needChangeChar = true;
                            break;
                        case 2:
                            countryFileName = "countries_tw.txt";
                            CountrySelectActivity.this.needChangeChar = true;
                            break;
                    }
                }
                countryFileName = countryFileName == null ? "countries.txt" : countryFileName;
                ArrayList<Country> hotCountries = new ArrayList<>();
                Country c1 = new Country();
                c1.name = LocaleController.getString(R.string.China);
                c1.code = "86";
                c1.shortname = "CN";
                c1.phoneFormat = "XXX XXXX XXXX";
                hotCountries.add(c1);
                Country c22 = new Country();
                c22.name = LocaleController.getString(R.string.USA);
                c22.code = "1";
                c22.shortname = "US";
                c22.phoneFormat = "XXX XXX XXXX";
                hotCountries.add(c22);
                this.countries.put(LocaleController.getString(R.string.Hot), hotCountries);
                stream = ApplicationLoader.applicationContext.getResources().getAssets().open(countryFileName);
                reader = new BufferedReader(new InputStreamReader(stream));
            } catch (Exception e) {
                FileLog.e(e);
            }
            while (true) {
                String line = reader.readLine();
                if (line != null) {
                    String[] args = line.split(";");
                    Country c4 = new Country();
                    c4.code = args[c3];
                    c4.shortname = args[1];
                    c4.name = args[c];
                    if (args.length > 3) {
                        c4.phoneFormat = args[3];
                    }
                    if (CountrySelectActivity.this.needChangeChar) {
                        n = CharacterParser.getInstance().getSelling(c4.name.substring(0, 1)).toUpperCase();
                        if (n.length() > 1) {
                            n = n.substring(0, 1);
                        }
                    } else {
                        String n2 = c4.name;
                        n = n2.substring(0, 1).toUpperCase();
                    }
                    ArrayList<Country> arr = this.countries.get(n);
                    if (arr == null) {
                        arr = new ArrayList<>();
                        this.countries.put(n, arr);
                        this.sortedCountries.add(n);
                    }
                    arr.add(c4);
                    c3 = 0;
                    c = 2;
                } else {
                    reader.close();
                    stream.close();
                    Collections.sort(this.sortedCountries, new ChangePhoneActivity$PhoneView$$ExternalSyntheticLambda8());
                    this.sortedCountries.add(0, LocaleController.getString(R.string.Hot));
                    for (ArrayList<Country> arr2 : this.countries.values()) {
                        Collections.sort(arr2, new Comparator() { // from class: im.uecahxyvjd.ui.CountrySelectActivity$CountryAdapter$$ExternalSyntheticLambda0
                            @Override // java.util.Comparator
                            public final int compare(Object obj, Object obj2) {
                                int compareTo;
                                compareTo = ((CountrySelectActivity.Country) obj).name.compareTo(((CountrySelectActivity.Country) obj2).name);
                                return compareTo;
                            }
                        });
                    }
                    return;
                }
            }
        }

        public HashMap<String, ArrayList<Country>> getCountries() {
            return this.countries;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public Country getItem(int section, int position) {
            if (section < 0 || section >= this.sortedCountries.size()) {
                return null;
            }
            ArrayList<Country> arr = this.countries.get(this.sortedCountries.get(section));
            if (position < 0 || position >= arr.size()) {
                return null;
            }
            return arr.get(position);
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public boolean isEnabled(int section, int row) {
            ArrayList<Country> arr = this.countries.get(this.sortedCountries.get(section));
            return row < arr.size();
        }

        public int getSectionForChar(char section) {
            for (int i = 0; i < getSectionCount() - 1; i++) {
                String sortStr = this.sortedCountries.get(i);
                char firstChar = sortStr.toUpperCase().charAt(0);
                if (firstChar == section) {
                    return i;
                }
            }
            return -1;
        }

        public int getPositionForSection(int section) {
            if (section == -1) {
                return -1;
            }
            int positionStart = 0;
            int N = getSectionCount();
            for (int i = 0; i < N; i++) {
                if (i >= section) {
                    return positionStart;
                }
                int count = getCountForSection(i);
                positionStart += count;
            }
            return -1;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public int getSectionCount() {
            return this.sortedCountries.size();
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public int getCountForSection(int section) {
            int count = this.countries.get(this.sortedCountries.get(section)).size();
            if (section != this.sortedCountries.size() - 1) {
                return count + 1;
            }
            return count;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public View getSectionHeaderView(int section, View view) {
            if (view == null) {
                view = new LetterSectionCell(this.mContext);
                ((LetterSectionCell) view).setCellHeight(AndroidUtilities.dp(48.0f));
            }
            ((LetterSectionCell) view).setLetter(this.sortedCountries.get(section).toUpperCase());
            return view;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view;
            switch (viewType) {
                case 0:
                    view = new TextSettingsCell(this.mContext);
                    view.setPadding(16, 0, 16, 0);
                    break;
                default:
                    view = new View(this.mContext);
                    break;
            }
            return new RecyclerListView.Holder(view);
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public void onBindViewHolder(int section, int position, RecyclerView.ViewHolder holder) {
            if (holder.getItemViewType() == 0) {
                ArrayList<Country> arr = this.countries.get(this.sortedCountries.get(section));
                Country c = arr.get(position);
                ((TextSettingsCell) holder.itemView).setTextAndValue(c.name, CountrySelectActivity.this.needPhoneCode ? Marker.ANY_NON_NULL_MARKER + c.code : null, false);
            }
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
        public int getItemViewType(int section, int position) {
            ArrayList<Country> arr = this.countries.get(this.sortedCountries.get(section));
            return position < arr.size() ? 0 : 1;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.FastScrollAdapter
        public String getLetter(int position) {
            int section = getSectionForPosition(position);
            if (section == -1) {
                section = this.sortedCountries.size() - 1;
            }
            return this.sortedCountries.get(section);
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.FastScrollAdapter
        public int getPositionForScrollProgress(float progress) {
            return (int) (getItemCount() * progress);
        }
    }

    /* loaded from: classes12.dex */
    public class CountrySearchAdapter extends RecyclerListView.SelectionAdapter {
        private HashMap<String, ArrayList<Country>> countries;
        private Context mContext;
        private ArrayList<Country> searchResult;
        private Timer searchTimer;

        public CountrySearchAdapter(Context context, HashMap<String, ArrayList<Country>> countries) {
            this.mContext = context;
            this.countries = countries;
        }

        public void search(final String query) {
            if (query == null) {
                this.searchResult = null;
                return;
            }
            try {
                Timer timer = this.searchTimer;
                if (timer != null) {
                    timer.cancel();
                }
            } catch (Exception e) {
                FileLog.e(e);
            }
            Timer timer2 = new Timer();
            this.searchTimer = timer2;
            timer2.schedule(new TimerTask() { // from class: im.uecahxyvjd.ui.CountrySelectActivity.CountrySearchAdapter.1
                @Override // java.util.TimerTask, java.lang.Runnable
                public void run() {
                    try {
                        CountrySearchAdapter.this.searchTimer.cancel();
                        CountrySearchAdapter.this.searchTimer = null;
                    } catch (Exception e2) {
                        FileLog.e(e2);
                    }
                    CountrySearchAdapter.this.processSearch(query.toLowerCase());
                }
            }, 100L, 300L);
        }

        /* JADX INFO: Access modifiers changed from: private */
        public void processSearch(final String query) {
            Utilities.searchQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.CountrySelectActivity$CountrySearchAdapter$$ExternalSyntheticLambda1
                @Override // java.lang.Runnable
                public final void run() {
                    CountrySelectActivity.CountrySearchAdapter.this.m1524x40ae5a0(query);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$processSearch$0$im-uecahxyvjd-ui-CountrySelectActivity$CountrySearchAdapter  reason: not valid java name */
        public /* synthetic */ void m1524x40ae5a0(String query) {
            String q = query.trim().toLowerCase();
            if (q.length() == 0) {
                updateSearchResults(new ArrayList<>());
                return;
            }
            ArrayList<Country> resultArray = new ArrayList<>();
            boolean isInputEn = Pattern.compile("([a-zA-Z])").matcher(query).matches();
            String n = query.substring(0, 1).toUpperCase();
            if (!isInputEn) {
                n = CharacterParser.getInstance().getSelling(n).toUpperCase();
                if (n.length() > 1) {
                    n = n.substring(0, 1);
                }
            }
            ArrayList<Country> arr = this.countries.get(n);
            if (arr != null) {
                Iterator<Country> it = arr.iterator();
                while (it.hasNext()) {
                    Country c = it.next();
                    if (CountrySelectActivity.this.needChangeChar) {
                        if (!isInputEn) {
                            if (c.name.toLowerCase().startsWith(query)) {
                                resultArray.add(c);
                            }
                        } else if (CharacterParser.getInstance().getSelling(c.name).toLowerCase().startsWith(query)) {
                            resultArray.add(c);
                        }
                    } else if (c.name.toLowerCase().startsWith(query)) {
                        resultArray.add(c);
                    }
                }
            }
            updateSearchResults(resultArray);
        }

        private void updateSearchResults(final ArrayList<Country> arrCounties) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.CountrySelectActivity$CountrySearchAdapter$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    CountrySelectActivity.CountrySearchAdapter.this.m1525xf0d1d94f(arrCounties);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$updateSearchResults$1$im-uecahxyvjd-ui-CountrySelectActivity$CountrySearchAdapter  reason: not valid java name */
        public /* synthetic */ void m1525xf0d1d94f(ArrayList arrCounties) {
            this.searchResult = arrCounties;
            notifyDataSetChanged();
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            return true;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            ArrayList<Country> arrayList = this.searchResult;
            if (arrayList == null) {
                return 0;
            }
            return arrayList.size();
        }

        public Country getItem(int i) {
            if (i < 0 || i >= this.searchResult.size()) {
                return null;
            }
            return this.searchResult.get(i);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            return new RecyclerListView.Holder(new TextSettingsCell(this.mContext));
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            Country c = this.searchResult.get(position);
            ((TextSettingsCell) holder.itemView).setTextAndValue(c.name, CountrySelectActivity.this.needPhoneCode ? Marker.ANY_NON_NULL_MARKER + c.code : null, position != this.searchResult.size() - 1);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemViewType(int i) {
            return 0;
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public ThemeDescription[] getThemeDescriptions() {
        return new ThemeDescription[]{new ThemeDescription(this.fragmentView, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_windowBackgroundWhite), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.listView, ThemeDescription.FLAG_LISTGLOWCOLOR, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_ITEMSCOLOR, null, null, null, null, Theme.key_actionBarDefaultIcon), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_TITLECOLOR, null, null, null, null, Theme.key_actionBarDefaultTitle), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_SELECTORCOLOR, null, null, null, null, Theme.key_actionBarDefaultSelector), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_SEARCH, null, null, null, null, Theme.key_actionBarDefaultSearch), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_SEARCHPLACEHOLDER, null, null, null, null, Theme.key_actionBarDefaultSearchPlaceholder), new ThemeDescription(this.listView, ThemeDescription.FLAG_SELECTOR, null, null, null, null, Theme.key_listSelector), new ThemeDescription(this.listView, 0, new Class[]{View.class}, Theme.dividerPaint, null, null, Theme.key_divider), new ThemeDescription(this.listView, ThemeDescription.FLAG_FASTSCROLL, null, null, null, null, Theme.key_fastScrollActive), new ThemeDescription(this.listView, ThemeDescription.FLAG_FASTSCROLL, null, null, null, null, Theme.key_fastScrollInactive), new ThemeDescription(this.listView, ThemeDescription.FLAG_FASTSCROLL, null, null, null, null, Theme.key_fastScrollText), new ThemeDescription(this.emptyView, ThemeDescription.FLAG_TEXTCOLOR, null, null, null, null, Theme.key_emptyListPlaceholder), new ThemeDescription(this.listView, 0, new Class[]{TextSettingsCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, 0, new Class[]{TextSettingsCell.class}, new String[]{"valueTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteValueText), new ThemeDescription(this.listView, ThemeDescription.FLAG_SECTIONS, new Class[]{LetterSectionCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText4)};
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment, im.uecahxyvjd.ui.hviews.search.MrySearchView.ISearchViewDelegate
    public void onStart(boolean focus) {
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment, im.uecahxyvjd.ui.hviews.search.MrySearchView.ISearchViewDelegate
    public void onSearchExpand() {
        this.searching = true;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment, im.uecahxyvjd.ui.hviews.search.MrySearchView.ISearchViewDelegate
    public void onSearchCollapse() {
        CountrySearchAdapter countrySearchAdapter = this.searchListViewAdapter;
        if (countrySearchAdapter != null) {
            countrySearchAdapter.search(null);
        }
        this.searching = false;
        this.searchWas = false;
        RecyclerListView recyclerListView = this.listView;
        if (recyclerListView != null) {
            recyclerListView.setAdapter(this.listViewAdapter);
            this.listView.setFastScrollVisible(false);
        }
        EmptyTextProgressView emptyTextProgressView = this.emptyView;
        if (emptyTextProgressView != null) {
            emptyTextProgressView.setText(LocaleController.getString("ChooseCountry", R.string.ChooseCountry));
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment, im.uecahxyvjd.ui.hviews.search.MrySearchView.ISearchViewDelegate
    public void onTextChange(String text) {
        this.searchListViewAdapter.search(text);
        if (text.length() != 0) {
            this.searchWas = true;
            RecyclerListView recyclerListView = this.listView;
            if (recyclerListView != null) {
                recyclerListView.setAdapter(this.searchListViewAdapter);
                this.listView.setFastScrollVisible(false);
                return;
            }
            return;
        }
        onSearchCollapse();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseSearchViewFragment, im.uecahxyvjd.ui.hviews.search.MrySearchView.ISearchViewDelegate
    public void onActionSearch(String trim) {
    }
}
