package im.uecahxyvjd.ui;

import android.animation.Animator;
import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Configuration;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.os.Environment;
import android.os.StatFs;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.FrameLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ApplicationLoader;
import im.uecahxyvjd.messenger.BuildVars;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserObject;
import im.uecahxyvjd.ui.DocumentSelectActivity;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.ActionBarMenu;
import im.uecahxyvjd.ui.actionbar.AlertDialog;
import im.uecahxyvjd.ui.actionbar.BackDrawable;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.actionbar.ThemeDescription;
import im.uecahxyvjd.ui.cells.GraySectionCell;
import im.uecahxyvjd.ui.cells.SharedDocumentCell;
import im.uecahxyvjd.ui.components.AlertsCreator;
import im.uecahxyvjd.ui.components.EmptyTextProgressView;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.NumberTextView;
import im.uecahxyvjd.ui.components.RecyclerListView;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.StringTokenizer;
/* loaded from: classes12.dex */
public class DocumentSelectActivity extends BaseFragment {
    private static final int done = 3;
    private boolean allowMusic;
    private boolean canSelectOnlyImageFiles;
    private File currentDir;
    private DocumentSelectActivityDelegate delegate;
    private EmptyTextProgressView emptyView;
    private LinearLayoutManager layoutManager;
    private ListAdapter listAdapter;
    private RecyclerListView listView;
    private ChatActivity parentFragment;
    private boolean scrolling;
    private NumberTextView selectedMessagesCountTextView;
    private ArrayList<ListItem> items = new ArrayList<>();
    private boolean receiverRegistered = false;
    private ArrayList<HistoryEntry> history = new ArrayList<>();
    private long sizeLimit = 1610612736;
    private HashMap<String, ListItem> selectedFiles = new HashMap<>();
    private ArrayList<View> actionModeViews = new ArrayList<>();
    private ArrayList<ListItem> recentItems = new ArrayList<>();
    private int maxSelectedFiles = -1;
    private BroadcastReceiver receiver = new AnonymousClass1();

    /* loaded from: classes12.dex */
    public interface DocumentSelectActivityDelegate {
        void didSelectFiles(DocumentSelectActivity documentSelectActivity, ArrayList<String> arrayList, boolean z, int i);

        void startDocumentSelectActivity();

        void startMusicSelectActivity(BaseFragment baseFragment);

        /* renamed from: im.uecahxyvjd.ui.DocumentSelectActivity$DocumentSelectActivityDelegate$-CC */
        /* loaded from: classes12.dex */
        public final /* synthetic */ class CC {
            public static void $default$startMusicSelectActivity(DocumentSelectActivityDelegate _this, BaseFragment parentFragment) {
            }
        }
    }

    /* loaded from: classes12.dex */
    public class ListItem {
        String ext;
        File file;
        int icon;
        String subtitle;
        String thumb;
        String title;

        private ListItem() {
            DocumentSelectActivity.this = r1;
            this.subtitle = "";
            this.ext = "";
        }

        /* synthetic */ ListItem(DocumentSelectActivity x0, AnonymousClass1 x1) {
            this();
        }
    }

    /* loaded from: classes12.dex */
    public class HistoryEntry {
        File dir;
        int scrollItem;
        int scrollOffset;
        String title;

        private HistoryEntry() {
            DocumentSelectActivity.this = r1;
        }

        /* synthetic */ HistoryEntry(DocumentSelectActivity x0, AnonymousClass1 x1) {
            this();
        }
    }

    /* renamed from: im.uecahxyvjd.ui.DocumentSelectActivity$1 */
    /* loaded from: classes12.dex */
    public class AnonymousClass1 extends BroadcastReceiver {
        AnonymousClass1() {
            DocumentSelectActivity.this = this$0;
        }

        @Override // android.content.BroadcastReceiver
        public void onReceive(Context arg0, Intent intent) {
            Runnable r = new Runnable() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$1$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    DocumentSelectActivity.AnonymousClass1.this.m1561lambda$onReceive$0$imuecahxyvjduiDocumentSelectActivity$1();
                }
            };
            if ("android.intent.action.MEDIA_UNMOUNTED".equals(intent.getAction())) {
                DocumentSelectActivity.this.listView.postDelayed(r, 1000L);
            } else {
                r.run();
            }
        }

        /* renamed from: lambda$onReceive$0$im-uecahxyvjd-ui-DocumentSelectActivity$1 */
        public /* synthetic */ void m1561lambda$onReceive$0$imuecahxyvjduiDocumentSelectActivity$1() {
            try {
                if (DocumentSelectActivity.this.currentDir == null) {
                    DocumentSelectActivity.this.listRoots();
                } else {
                    DocumentSelectActivity documentSelectActivity = DocumentSelectActivity.this;
                    documentSelectActivity.listFiles(documentSelectActivity.currentDir);
                }
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
    }

    public DocumentSelectActivity(boolean music) {
        this.allowMusic = music;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onFragmentCreate() {
        loadRecentFiles();
        return super.onFragmentCreate();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onFragmentDestroy() {
        try {
            if (this.receiverRegistered) {
                ApplicationLoader.applicationContext.unregisterReceiver(this.receiver);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        super.onFragmentDestroy();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public View createView(Context context) {
        if (!this.receiverRegistered) {
            this.receiverRegistered = true;
            IntentFilter filter = new IntentFilter();
            filter.addAction("android.intent.action.MEDIA_BAD_REMOVAL");
            filter.addAction("android.intent.action.MEDIA_CHECKING");
            filter.addAction("android.intent.action.MEDIA_EJECT");
            filter.addAction("android.intent.action.MEDIA_MOUNTED");
            filter.addAction("android.intent.action.MEDIA_NOFS");
            filter.addAction("android.intent.action.MEDIA_REMOVED");
            filter.addAction("android.intent.action.MEDIA_SHARED");
            filter.addAction("android.intent.action.MEDIA_UNMOUNTABLE");
            filter.addAction("android.intent.action.MEDIA_UNMOUNTED");
            filter.addDataScheme("file");
            ApplicationLoader.applicationContext.registerReceiver(this.receiver, filter);
        }
        this.actionBar.setBackButtonDrawable(new BackDrawable(false));
        this.actionBar.setAllowOverlayTitle(true);
        this.actionBar.setTitle(LocaleController.getString("SelectFile", R.string.SelectFile));
        this.actionBar.setActionBarMenuOnItemClick(new AnonymousClass2());
        this.selectedFiles.clear();
        this.actionModeViews.clear();
        ActionBarMenu actionMode = this.actionBar.createActionMode();
        NumberTextView numberTextView = new NumberTextView(actionMode.getContext());
        this.selectedMessagesCountTextView = numberTextView;
        numberTextView.setTextSize(18);
        this.selectedMessagesCountTextView.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
        this.selectedMessagesCountTextView.setTextColor(Theme.getColor(Theme.key_actionBarActionModeDefaultIcon));
        this.selectedMessagesCountTextView.setOnTouchListener(new View.OnTouchListener() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$$ExternalSyntheticLambda0
            @Override // android.view.View.OnTouchListener
            public final boolean onTouch(View view, MotionEvent motionEvent) {
                return DocumentSelectActivity.lambda$createView$0(view, motionEvent);
            }
        });
        actionMode.addView(this.selectedMessagesCountTextView, LayoutHelper.createLinear(0, -1, 1.0f, 65, 0, 0, 0));
        this.actionModeViews.add(actionMode.addItemWithWidth(3, R.drawable.ic_ab_done, AndroidUtilities.dp(54.0f)));
        this.fragmentView = new FrameLayout(context);
        FrameLayout frameLayout = (FrameLayout) this.fragmentView;
        EmptyTextProgressView emptyTextProgressView = new EmptyTextProgressView(context);
        this.emptyView = emptyTextProgressView;
        emptyTextProgressView.showTextView();
        frameLayout.addView(this.emptyView, LayoutHelper.createFrame(-1, -1.0f));
        RecyclerListView recyclerListView = new RecyclerListView(context);
        this.listView = recyclerListView;
        recyclerListView.setVerticalScrollBarEnabled(false);
        RecyclerListView recyclerListView2 = this.listView;
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(context, 1, false);
        this.layoutManager = linearLayoutManager;
        recyclerListView2.setLayoutManager(linearLayoutManager);
        this.listView.setEmptyView(this.emptyView);
        RecyclerListView recyclerListView3 = this.listView;
        ListAdapter listAdapter = new ListAdapter(context);
        this.listAdapter = listAdapter;
        recyclerListView3.setAdapter(listAdapter);
        frameLayout.addView(this.listView, LayoutHelper.createFrame(-1, -1.0f));
        this.listView.setOnScrollListener(new RecyclerView.OnScrollListener() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity.3
            {
                DocumentSelectActivity.this = this;
            }

            @Override // androidx.recyclerview.widget.RecyclerView.OnScrollListener
            public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                DocumentSelectActivity.this.scrolling = newState != 0;
            }
        });
        this.listView.setOnItemLongClickListener(new RecyclerListView.OnItemLongClickListener() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$$ExternalSyntheticLambda1
            @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemLongClickListener
            public final boolean onItemClick(View view, int i) {
                return DocumentSelectActivity.this.m1558lambda$createView$1$imuecahxyvjduiDocumentSelectActivity(view, i);
            }
        });
        this.listView.setOnItemClickListener(new RecyclerListView.OnItemClickListener() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$$ExternalSyntheticLambda2
            @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemClickListener
            public final void onItemClick(View view, int i) {
                DocumentSelectActivity.this.m1560lambda$createView$3$imuecahxyvjduiDocumentSelectActivity(view, i);
            }
        });
        listRoots();
        return this.fragmentView;
    }

    /* renamed from: im.uecahxyvjd.ui.DocumentSelectActivity$2 */
    /* loaded from: classes12.dex */
    public class AnonymousClass2 extends ActionBar.ActionBarMenuOnItemClick {
        AnonymousClass2() {
            DocumentSelectActivity.this = this$0;
        }

        @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
        public void onItemClick(int id) {
            if (id == -1) {
                if (DocumentSelectActivity.this.actionBar.isActionModeShowed()) {
                    DocumentSelectActivity.this.selectedFiles.clear();
                    DocumentSelectActivity.this.actionBar.hideActionMode();
                    int count = DocumentSelectActivity.this.listView.getChildCount();
                    for (int a2 = 0; a2 < count; a2++) {
                        View child = DocumentSelectActivity.this.listView.getChildAt(a2);
                        if (child instanceof SharedDocumentCell) {
                            ((SharedDocumentCell) child).setChecked(false, true);
                        }
                    }
                    return;
                }
                DocumentSelectActivity.this.finishFragment();
            } else if (id == 3 && DocumentSelectActivity.this.delegate != null) {
                final ArrayList<String> files = new ArrayList<>(DocumentSelectActivity.this.selectedFiles.keySet());
                if (DocumentSelectActivity.this.parentFragment == null || !DocumentSelectActivity.this.parentFragment.isInScheduleMode()) {
                    DocumentSelectActivity.this.delegate.didSelectFiles(DocumentSelectActivity.this, files, true, 0);
                } else {
                    AlertsCreator.createScheduleDatePickerDialog(DocumentSelectActivity.this.getParentActivity(), UserObject.isUserSelf(DocumentSelectActivity.this.parentFragment.getCurrentUser()), new AlertsCreator.ScheduleDatePickerDelegate() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$2$$ExternalSyntheticLambda0
                        @Override // im.uecahxyvjd.ui.components.AlertsCreator.ScheduleDatePickerDelegate
                        public final void didSelectDate(boolean z, int i) {
                            DocumentSelectActivity.AnonymousClass2.this.m1562lambda$onItemClick$0$imuecahxyvjduiDocumentSelectActivity$2(files, z, i);
                        }
                    });
                }
            }
        }

        /* renamed from: lambda$onItemClick$0$im-uecahxyvjd-ui-DocumentSelectActivity$2 */
        public /* synthetic */ void m1562lambda$onItemClick$0$imuecahxyvjduiDocumentSelectActivity$2(ArrayList files, boolean notify, int scheduleDate) {
            DocumentSelectActivity.this.delegate.didSelectFiles(DocumentSelectActivity.this, files, notify, scheduleDate);
        }
    }

    public static /* synthetic */ boolean lambda$createView$0(View v, MotionEvent event) {
        return true;
    }

    /* renamed from: lambda$createView$1$im-uecahxyvjd-ui-DocumentSelectActivity */
    public /* synthetic */ boolean m1558lambda$createView$1$imuecahxyvjduiDocumentSelectActivity(View view, int position) {
        ListItem item;
        if (this.actionBar.isActionModeShowed() || (item = this.listAdapter.getItem(position)) == null) {
            return false;
        }
        File file = item.file;
        if (file != null && !file.isDirectory()) {
            if (!file.canRead()) {
                showErrorBox(LocaleController.getString("AccessError", R.string.AccessError));
                return false;
            } else if (this.canSelectOnlyImageFiles && item.thumb == null) {
                showErrorBox(LocaleController.formatString("PassportUploadNotImage", R.string.PassportUploadNotImage, new Object[0]));
                return false;
            } else {
                if (this.sizeLimit != 0) {
                    long length = file.length();
                    long j = this.sizeLimit;
                    if (length > j) {
                        showErrorBox(LocaleController.formatString("FileUploadLimit", R.string.FileUploadLimit, AndroidUtilities.formatFileSize(j)));
                        return false;
                    }
                }
                if (this.maxSelectedFiles >= 0) {
                    int size = this.selectedFiles.size();
                    int i = this.maxSelectedFiles;
                    if (size >= i) {
                        showErrorBox(LocaleController.formatString("PassportUploadMaxReached", R.string.PassportUploadMaxReached, LocaleController.formatPluralString("Files", i)));
                        return false;
                    }
                }
                if (file.length() == 0) {
                    return false;
                }
                this.selectedFiles.put(file.toString(), item);
                this.selectedMessagesCountTextView.setNumber(1, false);
                AnimatorSet animatorSet = new AnimatorSet();
                ArrayList<Animator> animators = new ArrayList<>();
                for (int a2 = 0; a2 < this.actionModeViews.size(); a2++) {
                    View view2 = this.actionModeViews.get(a2);
                    AndroidUtilities.clearDrawableAnimation(view2);
                    animators.add(ObjectAnimator.ofFloat(view2, "scaleY", 0.1f, 1.0f));
                }
                animatorSet.playTogether(animators);
                animatorSet.setDuration(250L);
                animatorSet.start();
                this.scrolling = false;
                if (view instanceof SharedDocumentCell) {
                    ((SharedDocumentCell) view).setChecked(true, true);
                }
                this.actionBar.showActionMode();
            }
        }
        return true;
    }

    /* renamed from: lambda$createView$3$im-uecahxyvjd-ui-DocumentSelectActivity */
    public /* synthetic */ void m1560lambda$createView$3$imuecahxyvjduiDocumentSelectActivity(View view, int position) {
        ListItem item = this.listAdapter.getItem(position);
        if (item == null) {
            return;
        }
        File file = item.file;
        if (file == null) {
            if (item.icon == R.drawable.ic_storage_gallery) {
                DocumentSelectActivityDelegate documentSelectActivityDelegate = this.delegate;
                if (documentSelectActivityDelegate != null) {
                    documentSelectActivityDelegate.startDocumentSelectActivity();
                }
                finishFragment(false);
            } else if (item.icon == R.drawable.ic_storage_music) {
                DocumentSelectActivityDelegate documentSelectActivityDelegate2 = this.delegate;
                if (documentSelectActivityDelegate2 != null) {
                    documentSelectActivityDelegate2.startMusicSelectActivity(this);
                }
            } else {
                ArrayList<HistoryEntry> arrayList = this.history;
                HistoryEntry he = arrayList.remove(arrayList.size() - 1);
                this.actionBar.setTitle(he.title);
                if (he.dir != null) {
                    listFiles(he.dir);
                } else {
                    listRoots();
                }
                this.layoutManager.scrollToPositionWithOffset(he.scrollItem, he.scrollOffset);
            }
        } else if (file.isDirectory()) {
            HistoryEntry he2 = new HistoryEntry(this, null);
            he2.scrollItem = this.layoutManager.findLastVisibleItemPosition();
            View topView = this.layoutManager.findViewByPosition(he2.scrollItem);
            if (topView != null) {
                he2.scrollOffset = topView.getTop();
            }
            he2.dir = this.currentDir;
            he2.title = this.actionBar.getTitle();
            this.history.add(he2);
            if (!listFiles(file)) {
                this.history.remove(he2);
            } else {
                this.actionBar.setTitle(item.title);
            }
        } else {
            if (!file.canRead()) {
                showErrorBox(LocaleController.getString("AccessError", R.string.AccessError));
                file = new File("/mnt/sdcard");
            }
            if (this.canSelectOnlyImageFiles && item.thumb == null) {
                showErrorBox(LocaleController.formatString("PassportUploadNotImage", R.string.PassportUploadNotImage, new Object[0]));
                return;
            }
            if (this.sizeLimit != 0) {
                long length = file.length();
                long j = this.sizeLimit;
                if (length > j) {
                    showErrorBox(LocaleController.formatString("FileUploadLimit", R.string.FileUploadLimit, AndroidUtilities.formatFileSize(j)));
                    return;
                }
            }
            if (file.length() == 0) {
                return;
            }
            if (this.actionBar.isActionModeShowed()) {
                if (this.selectedFiles.containsKey(file.toString())) {
                    this.selectedFiles.remove(file.toString());
                } else {
                    if (this.maxSelectedFiles >= 0) {
                        int size = this.selectedFiles.size();
                        int i = this.maxSelectedFiles;
                        if (size >= i) {
                            showErrorBox(LocaleController.formatString("PassportUploadMaxReached", R.string.PassportUploadMaxReached, LocaleController.formatPluralString("Files", i)));
                            return;
                        }
                    }
                    this.selectedFiles.put(file.toString(), item);
                }
                if (this.selectedFiles.isEmpty()) {
                    this.actionBar.hideActionMode();
                } else {
                    this.selectedMessagesCountTextView.setNumber(this.selectedFiles.size(), true);
                }
                this.scrolling = false;
                if (view instanceof SharedDocumentCell) {
                    ((SharedDocumentCell) view).setChecked(this.selectedFiles.containsKey(item.file.toString()), true);
                }
            } else if (this.delegate != null) {
                final ArrayList<String> files = new ArrayList<>();
                files.add(file.getAbsolutePath());
                ChatActivity chatActivity = this.parentFragment;
                if (chatActivity == null || !chatActivity.isInScheduleMode()) {
                    this.delegate.didSelectFiles(this, files, true, 0);
                } else {
                    AlertsCreator.createScheduleDatePickerDialog(getParentActivity(), UserObject.isUserSelf(this.parentFragment.getCurrentUser()), new AlertsCreator.ScheduleDatePickerDelegate() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$$ExternalSyntheticLambda3
                        @Override // im.uecahxyvjd.ui.components.AlertsCreator.ScheduleDatePickerDelegate
                        public final void didSelectDate(boolean z, int i2) {
                            DocumentSelectActivity.this.m1559lambda$createView$2$imuecahxyvjduiDocumentSelectActivity(files, z, i2);
                        }
                    });
                }
            }
        }
    }

    /* renamed from: lambda$createView$2$im-uecahxyvjd-ui-DocumentSelectActivity */
    public /* synthetic */ void m1559lambda$createView$2$imuecahxyvjduiDocumentSelectActivity(ArrayList files, boolean notify, int scheduleDate) {
        this.delegate.didSelectFiles(this, files, notify, scheduleDate);
    }

    public void setChatActivity(ChatActivity chatActivity) {
        this.parentFragment = chatActivity;
    }

    public void setMaxSelectedFiles(int value) {
        this.maxSelectedFiles = value;
    }

    public void setCanSelectOnlyImageFiles(boolean value) {
        this.canSelectOnlyImageFiles = true;
    }

    public void loadRecentFiles() {
        try {
            File[] files = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).listFiles();
            for (File file : files) {
                if (!file.isDirectory()) {
                    ListItem item = new ListItem(this, null);
                    item.title = file.getName();
                    item.file = file;
                    String fname = file.getName();
                    String[] sp = fname.split("\\.");
                    item.ext = sp.length > 1 ? sp[sp.length - 1] : "?";
                    item.subtitle = AndroidUtilities.formatFileSize(file.length());
                    String fname2 = fname.toLowerCase();
                    if (fname2.endsWith(".jpg") || fname2.endsWith(".png") || fname2.endsWith(".gif") || fname2.endsWith(".jpeg")) {
                        item.thumb = file.getAbsolutePath();
                    }
                    this.recentItems.add(item);
                }
            }
            Collections.sort(this.recentItems, new Comparator() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$$ExternalSyntheticLambda4
                @Override // java.util.Comparator
                public final int compare(Object obj, Object obj2) {
                    return DocumentSelectActivity.lambda$loadRecentFiles$4((DocumentSelectActivity.ListItem) obj, (DocumentSelectActivity.ListItem) obj2);
                }
            });
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public static /* synthetic */ int lambda$loadRecentFiles$4(ListItem o1, ListItem o2) {
        long lm = o1.file.lastModified();
        long rm = o2.file.lastModified();
        if (lm == rm) {
            return 0;
        }
        if (lm > rm) {
            return -1;
        }
        return 1;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onResume() {
        super.onResume();
        ListAdapter listAdapter = this.listAdapter;
        if (listAdapter != null) {
            listAdapter.notifyDataSetChanged();
        }
        fixLayoutInternal();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        RecyclerListView recyclerListView = this.listView;
        if (recyclerListView != null) {
            ViewTreeObserver obs = recyclerListView.getViewTreeObserver();
            obs.addOnPreDrawListener(new ViewTreeObserver.OnPreDrawListener() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity.4
                {
                    DocumentSelectActivity.this = this;
                }

                @Override // android.view.ViewTreeObserver.OnPreDrawListener
                public boolean onPreDraw() {
                    DocumentSelectActivity.this.listView.getViewTreeObserver().removeOnPreDrawListener(this);
                    DocumentSelectActivity.this.fixLayoutInternal();
                    return true;
                }
            });
        }
    }

    public void fixLayoutInternal() {
        if (this.selectedMessagesCountTextView == null) {
            return;
        }
        if (!AndroidUtilities.isTablet() && ApplicationLoader.applicationContext.getResources().getConfiguration().orientation == 2) {
            this.selectedMessagesCountTextView.setTextSize(18);
        } else {
            this.selectedMessagesCountTextView.setTextSize(20);
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onBackPressed() {
        if (this.history.size() > 0) {
            ArrayList<HistoryEntry> arrayList = this.history;
            HistoryEntry he = arrayList.remove(arrayList.size() - 1);
            this.actionBar.setTitle(he.title);
            if (he.dir != null) {
                listFiles(he.dir);
            } else {
                listRoots();
            }
            this.layoutManager.scrollToPositionWithOffset(he.scrollItem, he.scrollOffset);
            return false;
        }
        return super.onBackPressed();
    }

    public void setDelegate(DocumentSelectActivityDelegate delegate) {
        this.delegate = delegate;
    }

    public boolean listFiles(File dir) {
        if (!dir.canRead()) {
            if ((dir.getAbsolutePath().startsWith(Environment.getExternalStorageDirectory().toString()) || dir.getAbsolutePath().startsWith("/sdcard") || dir.getAbsolutePath().startsWith("/mnt/sdcard")) && !Environment.getExternalStorageState().equals("mounted") && !Environment.getExternalStorageState().equals("mounted_ro")) {
                this.currentDir = dir;
                this.items.clear();
                String state = Environment.getExternalStorageState();
                if ("shared".equals(state)) {
                    this.emptyView.setText(LocaleController.getString("UsbActive", R.string.UsbActive));
                } else {
                    this.emptyView.setText(LocaleController.getString("NotMounted", R.string.NotMounted));
                }
                AndroidUtilities.clearDrawableAnimation(this.listView);
                this.scrolling = true;
                this.listAdapter.notifyDataSetChanged();
                return true;
            }
            showErrorBox(LocaleController.getString("AccessError", R.string.AccessError));
            return false;
        }
        try {
            File[] files = dir.listFiles();
            if (files == null) {
                showErrorBox(LocaleController.getString("UnknownError", R.string.UnknownError));
                return false;
            }
            this.currentDir = dir;
            this.items.clear();
            Arrays.sort(files, new Comparator() { // from class: im.uecahxyvjd.ui.DocumentSelectActivity$$ExternalSyntheticLambda5
                @Override // java.util.Comparator
                public final int compare(Object obj, Object obj2) {
                    return DocumentSelectActivity.lambda$listFiles$5((File) obj, (File) obj2);
                }
            });
            for (File file : files) {
                if (file.getName().indexOf(46) != 0) {
                    ListItem item = new ListItem(this, null);
                    item.title = file.getName();
                    item.file = file;
                    if (file.isDirectory()) {
                        item.icon = R.drawable.ic_directory;
                        item.subtitle = LocaleController.getString("Folder", R.string.Folder);
                    } else {
                        String fname = file.getName();
                        String[] sp = fname.split("\\.");
                        item.ext = sp.length > 1 ? sp[sp.length - 1] : "?";
                        item.subtitle = AndroidUtilities.formatFileSize(file.length());
                        String fname2 = fname.toLowerCase();
                        if (fname2.endsWith(".jpg") || fname2.endsWith(".png") || fname2.endsWith(".gif") || fname2.endsWith(".jpeg")) {
                            item.thumb = file.getAbsolutePath();
                        }
                    }
                    this.items.add(item);
                }
            }
            ListItem item2 = new ListItem(this, null);
            item2.title = "..";
            if (this.history.size() > 0) {
                ArrayList<HistoryEntry> arrayList = this.history;
                HistoryEntry entry = arrayList.get(arrayList.size() - 1);
                if (entry.dir == null) {
                    item2.subtitle = LocaleController.getString("Folder", R.string.Folder);
                } else {
                    item2.subtitle = entry.dir.toString();
                }
            } else {
                item2.subtitle = LocaleController.getString("Folder", R.string.Folder);
            }
            item2.icon = R.drawable.ic_directory;
            item2.file = null;
            this.items.add(0, item2);
            AndroidUtilities.clearDrawableAnimation(this.listView);
            this.scrolling = true;
            this.listAdapter.notifyDataSetChanged();
            return true;
        } catch (Exception e) {
            showErrorBox(e.getLocalizedMessage());
            return false;
        }
    }

    public static /* synthetic */ int lambda$listFiles$5(File lhs, File rhs) {
        if (lhs.isDirectory() != rhs.isDirectory()) {
            return lhs.isDirectory() ? -1 : 1;
        }
        return lhs.getName().compareToIgnoreCase(rhs.getName());
    }

    private void showErrorBox(String error) {
        if (getParentActivity() == null) {
            return;
        }
        new AlertDialog.Builder(getParentActivity()).setTitle(LocaleController.getString("AppName", R.string.AppName)).setMessage(error).setPositiveButton(LocaleController.getString("OK", R.string.OK), null).show();
    }

    /* JADX WARN: Can't wrap try/catch for region: R(8:33|(3:(2:37|(8:39|40|41|42|(1:44)(1:54)|45|46|47))|46|47)|58|40|41|42|(0)(0)|45) */
    /* JADX WARN: Code restructure failed: missing block: B:167:0x015f, code lost:
        r0 = e;
     */
    /* JADX WARN: Removed duplicated region for block: B:160:0x0133 A[Catch: Exception -> 0x015f, all -> 0x017f, TryCatch #0 {Exception -> 0x015f, blocks: (B:158:0x0122, B:160:0x0133, B:161:0x013a), top: B:204:0x0122 }] */
    /* JADX WARN: Removed duplicated region for block: B:161:0x013a A[Catch: Exception -> 0x015f, all -> 0x017f, TRY_LEAVE, TryCatch #0 {Exception -> 0x015f, blocks: (B:158:0x0122, B:160:0x0133, B:161:0x013a), top: B:204:0x0122 }] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void listRoots() {
        String newPath;
        int index;
        this.currentDir = null;
        this.items.clear();
        HashSet<String> paths = new HashSet<>();
        String defaultPath = Environment.getExternalStorageDirectory().getPath();
        Environment.isExternalStorageRemovable();
        String defaultPathState = Environment.getExternalStorageState();
        if (defaultPathState.equals("mounted") || defaultPathState.equals("mounted_ro")) {
            ListItem ext = new ListItem(this, null);
            if (Environment.isExternalStorageRemovable()) {
                ext.title = LocaleController.getString("SdCard", R.string.SdCard);
                ext.icon = R.drawable.ic_external_storage;
            } else {
                ext.title = LocaleController.getString("InternalStorage", R.string.InternalStorage);
                ext.icon = R.drawable.ic_storage;
            }
            ext.subtitle = getRootSubtitle(defaultPath);
            ext.file = Environment.getExternalStorageDirectory();
            this.items.add(ext);
            paths.add(defaultPath);
        }
        BufferedReader bufferedReader = null;
        try {
            try {
                try {
                    bufferedReader = new BufferedReader(new FileReader("/proc/mounts"));
                    while (true) {
                        String line = bufferedReader.readLine();
                        if (line == null) {
                            break;
                        } else if (line.contains("vfat") || line.contains("/mnt")) {
                            if (BuildVars.LOGS_ENABLED) {
                                FileLog.d(line);
                            }
                            StringTokenizer tokens = new StringTokenizer(line, " ");
                            tokens.nextToken();
                            String path = tokens.nextToken();
                            if (!paths.contains(path)) {
                                if (line.contains("/dev/block/vold")) {
                                    if (!line.contains("/mnt/secure") && !line.contains("/mnt/asec") && !line.contains("/mnt/obb") && !line.contains("/dev/mapper") && !line.contains("tmpfs")) {
                                        try {
                                            if (!new File(path).isDirectory() && (index = path.lastIndexOf(47)) != -1) {
                                                newPath = "/storage/" + path.substring(index + 1);
                                                if (new File(newPath).isDirectory()) {
                                                    paths.add(newPath);
                                                    ListItem item = new ListItem(this, null);
                                                    if (!newPath.toLowerCase().contains("sd")) {
                                                        item.title = LocaleController.getString("SdCard", R.string.SdCard);
                                                    } else {
                                                        item.title = LocaleController.getString("ExternalStorage", R.string.ExternalStorage);
                                                    }
                                                    item.icon = R.drawable.ic_external_storage;
                                                    item.subtitle = getRootSubtitle(newPath);
                                                    item.file = new File(newPath);
                                                    this.items.add(item);
                                                }
                                            }
                                            item.icon = R.drawable.ic_external_storage;
                                            item.subtitle = getRootSubtitle(newPath);
                                            item.file = new File(newPath);
                                            this.items.add(item);
                                        } catch (Exception e) {
                                            e = e;
                                            FileLog.e(e);
                                        }
                                        newPath = path;
                                        paths.add(newPath);
                                        ListItem item2 = new ListItem(this, null);
                                        if (!newPath.toLowerCase().contains("sd")) {
                                        }
                                    }
                                }
                            }
                        }
                    }
                    bufferedReader.close();
                } catch (Exception e2) {
                    FileLog.e(e2);
                }
            } catch (Exception e3) {
                FileLog.e(e3);
                if (bufferedReader != null) {
                    bufferedReader.close();
                }
            }
            ListItem fs = new ListItem(this, null);
            fs.title = "/";
            fs.subtitle = LocaleController.getString("SystemRoot", R.string.SystemRoot);
            fs.icon = R.drawable.ic_directory;
            fs.file = new File("/");
            this.items.add(fs);
            try {
                File appPath = new File(Environment.getExternalStorageDirectory(), "Zaaa");
                if (appPath.exists()) {
                    ListItem fs2 = new ListItem(this, null);
                    fs2.title = LocaleController.getString(R.string.AppName);
                    fs2.subtitle = appPath.toString();
                    fs2.icon = R.drawable.ic_directory;
                    fs2.file = appPath;
                    this.items.add(fs2);
                }
            } catch (Exception e4) {
                FileLog.e(e4);
            }
            ListItem fs3 = new ListItem(this, null);
            fs3.title = LocaleController.getString("Gallery", R.string.Gallery);
            fs3.subtitle = LocaleController.getString("GalleryInfo", R.string.GalleryInfo);
            fs3.icon = R.drawable.ic_storage_gallery;
            fs3.file = null;
            this.items.add(fs3);
            if (this.allowMusic) {
                ListItem fs4 = new ListItem(this, null);
                fs4.title = LocaleController.getString("AttachMusic", R.string.AttachMusic);
                fs4.subtitle = LocaleController.getString("MusicInfo", R.string.MusicInfo);
                fs4.icon = R.drawable.ic_storage_music;
                fs4.file = null;
                this.items.add(fs4);
            }
            AndroidUtilities.clearDrawableAnimation(this.listView);
            this.scrolling = true;
            this.listAdapter.notifyDataSetChanged();
        } catch (Throwable e5) {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (Exception e6) {
                    FileLog.e(e6);
                }
            }
            throw e5;
        }
    }

    private String getRootSubtitle(String path) {
        try {
            StatFs stat = new StatFs(path);
            long total = stat.getBlockCount() * stat.getBlockSize();
            long free = stat.getAvailableBlocks() * stat.getBlockSize();
            if (total == 0) {
                return "";
            }
            return LocaleController.formatString("FreeOfTotal", R.string.FreeOfTotal, AndroidUtilities.formatFileSize(free), AndroidUtilities.formatFileSize(total));
        } catch (Exception e) {
            FileLog.e(e);
            return path;
        }
    }

    /* loaded from: classes12.dex */
    public class ListAdapter extends RecyclerListView.SelectionAdapter {
        private Context mContext;

        public ListAdapter(Context context) {
            DocumentSelectActivity.this = r1;
            this.mContext = context;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            return holder.getItemViewType() != 0;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            int count = DocumentSelectActivity.this.items.size();
            if (DocumentSelectActivity.this.history.isEmpty() && !DocumentSelectActivity.this.recentItems.isEmpty()) {
                return count + DocumentSelectActivity.this.recentItems.size() + 1;
            }
            return count;
        }

        public ListItem getItem(int position) {
            int position2;
            if (position < DocumentSelectActivity.this.items.size()) {
                return (ListItem) DocumentSelectActivity.this.items.get(position);
            }
            if (DocumentSelectActivity.this.history.isEmpty() && !DocumentSelectActivity.this.recentItems.isEmpty() && position != DocumentSelectActivity.this.items.size() && (position2 = position - (DocumentSelectActivity.this.items.size() + 1)) < DocumentSelectActivity.this.recentItems.size()) {
                return (ListItem) DocumentSelectActivity.this.recentItems.get(position2);
            }
            return null;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemViewType(int position) {
            return getItem(position) != null ? 1 : 0;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view;
            switch (viewType) {
                case 0:
                    view = new GraySectionCell(this.mContext);
                    ((GraySectionCell) view).setText(LocaleController.getString("Recent", R.string.Recent));
                    break;
                default:
                    view = new SharedDocumentCell(this.mContext);
                    break;
            }
            return new RecyclerListView.Holder(view);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            if (holder.getItemViewType() == 1) {
                ListItem item = getItem(position);
                SharedDocumentCell documentCell = (SharedDocumentCell) holder.itemView;
                if (item.icon != 0) {
                    documentCell.setTextAndValueAndTypeAndThumb(item.title, item.subtitle, null, null, item.icon);
                } else {
                    String type = item.ext.toUpperCase().substring(0, Math.min(item.ext.length(), 4));
                    documentCell.setTextAndValueAndTypeAndThumb(item.title, item.subtitle, type, item.thumb, 0);
                }
                if (item.file == null || !DocumentSelectActivity.this.actionBar.isActionModeShowed()) {
                    documentCell.setChecked(false, true ^ DocumentSelectActivity.this.scrolling);
                } else {
                    documentCell.setChecked(DocumentSelectActivity.this.selectedFiles.containsKey(item.file.toString()), true ^ DocumentSelectActivity.this.scrolling);
                }
            }
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public ThemeDescription[] getThemeDescriptions() {
        return new ThemeDescription[]{new ThemeDescription(this.fragmentView, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_windowBackgroundWhite), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.listView, ThemeDescription.FLAG_LISTGLOWCOLOR, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_ITEMSCOLOR, null, null, null, null, Theme.key_actionBarDefaultIcon), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_TITLECOLOR, null, null, null, null, Theme.key_actionBarDefaultTitle), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_SELECTORCOLOR, null, null, null, null, Theme.key_actionBarDefaultSelector), new ThemeDescription(this.listView, ThemeDescription.FLAG_SELECTOR, null, null, null, null, Theme.key_listSelector), new ThemeDescription(this.emptyView, ThemeDescription.FLAG_TEXTCOLOR, null, null, null, null, Theme.key_emptyListPlaceholder), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_AM_ITEMSCOLOR, null, null, null, null, Theme.key_actionBarActionModeDefaultIcon), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_AM_BACKGROUND, null, null, null, null, Theme.key_actionBarActionModeDefault), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_AM_TOPBACKGROUND, null, null, null, null, Theme.key_actionBarActionModeDefaultTop), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_AM_SELECTORCOLOR, null, null, null, null, Theme.key_actionBarActionModeDefaultSelector), new ThemeDescription(this.selectedMessagesCountTextView, ThemeDescription.FLAG_TEXTCOLOR, null, null, null, null, Theme.key_actionBarActionModeDefaultIcon), new ThemeDescription(this.listView, 0, new Class[]{GraySectionCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_graySectionText), new ThemeDescription(this.listView, ThemeDescription.FLAG_CELLBACKGROUNDCOLOR, new Class[]{GraySectionCell.class}, null, null, null, Theme.key_graySection), new ThemeDescription(this.listView, ThemeDescription.FLAG_TEXTCOLOR, new Class[]{SharedDocumentCell.class}, new String[]{"nameTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, ThemeDescription.FLAG_TEXTCOLOR, new Class[]{SharedDocumentCell.class}, new String[]{"dateTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText3), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKBOX, new Class[]{SharedDocumentCell.class}, new String[]{"checkBox"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_checkbox), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKBOXCHECK, new Class[]{SharedDocumentCell.class}, new String[]{"checkBox"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_checkboxCheck), new ThemeDescription(this.listView, ThemeDescription.FLAG_IMAGECOLOR, new Class[]{SharedDocumentCell.class}, new String[]{"thumbImageView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_files_folderIcon), new ThemeDescription(this.listView, ThemeDescription.FLAG_IMAGECOLOR | ThemeDescription.FLAG_BACKGROUNDFILTER, new Class[]{SharedDocumentCell.class}, new String[]{"thumbImageView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_files_folderIconBackground), new ThemeDescription(this.listView, ThemeDescription.FLAG_TEXTCOLOR, new Class[]{SharedDocumentCell.class}, new String[]{"extTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_files_iconText)};
    }
}
