package im.uecahxyvjd.ui;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.AnimatorSet;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.LongSparseArray;
import android.util.SparseArray;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.TextView;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.android.exoplayer2.upstream.cache.ContentMetadata;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ChatObject;
import im.uecahxyvjd.messenger.ContactsController;
import im.uecahxyvjd.messenger.DispatchQueue;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessagesStorage;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.DialogsActivity;
import im.uecahxyvjd.ui.NotificationsCustomSettingsActivity;
import im.uecahxyvjd.ui.NotificationsSettingsActivity;
import im.uecahxyvjd.ui.ProfileNotificationsActivity;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.ActionBarMenu;
import im.uecahxyvjd.ui.actionbar.ActionBarMenuItem;
import im.uecahxyvjd.ui.actionbar.AlertDialog;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.actionbar.ThemeDescription;
import im.uecahxyvjd.ui.adapters.SearchAdapterHelper;
import im.uecahxyvjd.ui.cells.GraySectionCell;
import im.uecahxyvjd.ui.cells.HeaderCell;
import im.uecahxyvjd.ui.cells.NotificationsCheckCell;
import im.uecahxyvjd.ui.cells.ShadowSectionCell;
import im.uecahxyvjd.ui.cells.TextCell;
import im.uecahxyvjd.ui.cells.TextCheckCell;
import im.uecahxyvjd.ui.cells.TextColorCell;
import im.uecahxyvjd.ui.cells.TextSettingsCell;
import im.uecahxyvjd.ui.cells.UserCell;
import im.uecahxyvjd.ui.components.AlertsCreator;
import im.uecahxyvjd.ui.components.EmptyTextProgressView;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.RecyclerListView;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
/* loaded from: classes12.dex */
public class NotificationsCustomSettingsActivity extends BaseFragment {
    private static final int search_button = 0;
    private ListAdapter adapter;
    private int alertRow;
    private int alertSection2Row;
    private AnimatorSet animatorSet;
    private int currentType;
    private int deleteAllRow;
    private int deleteAllSectionRow;
    private EmptyTextProgressView emptyView;
    private ArrayList<NotificationsSettingsActivity.NotificationException> exceptions;
    private int exceptionsAddRow;
    private HashMap<Long, NotificationsSettingsActivity.NotificationException> exceptionsDict;
    private int exceptionsEndRow;
    private int exceptionsSection2Row;
    private int exceptionsStartRow;
    private int groupSection2Row;
    private RecyclerListView listView;
    private int messageLedRow;
    private int messagePopupNotificationRow;
    private int messagePriorityRow;
    private int messageSectionRow;
    private int messageSoundRow;
    private int messageVibrateRow;
    private int previewRow;
    private int rowCount;
    private SearchAdapter searchAdapter;
    private boolean searchWas;
    private boolean searching;

    public NotificationsCustomSettingsActivity(int type, ArrayList<NotificationsSettingsActivity.NotificationException> notificationExceptions) {
        this(type, notificationExceptions, false);
    }

    public NotificationsCustomSettingsActivity(int type, ArrayList<NotificationsSettingsActivity.NotificationException> notificationExceptions, boolean load) {
        this.rowCount = 0;
        this.exceptionsDict = new HashMap<>();
        this.currentType = type;
        this.exceptions = notificationExceptions;
        int N = notificationExceptions.size();
        for (int a2 = 0; a2 < N; a2++) {
            NotificationsSettingsActivity.NotificationException exception = this.exceptions.get(a2);
            this.exceptionsDict.put(Long.valueOf(exception.did), exception);
        }
        if (load) {
            loadExceptions();
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onFragmentCreate() {
        updateRows();
        return super.onFragmentCreate();
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public View createView(Context context) {
        this.searching = false;
        this.searchWas = false;
        this.actionBar.setBackButtonImage(R.mipmap.ic_back);
        this.actionBar.setAllowOverlayTitle(true);
        if (this.currentType == -1) {
            this.actionBar.setTitle(LocaleController.getString("NotificationsExceptions", R.string.NotificationsExceptions));
        } else {
            this.actionBar.setTitle(LocaleController.getString("Notifications", R.string.Notifications));
        }
        this.actionBar.setActionBarMenuOnItemClick(new ActionBar.ActionBarMenuOnItemClick() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity.1
            @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
            public void onItemClick(int id) {
                if (id == -1) {
                    NotificationsCustomSettingsActivity.this.finishFragment();
                }
            }
        });
        ArrayList<NotificationsSettingsActivity.NotificationException> arrayList = this.exceptions;
        if (arrayList != null && !arrayList.isEmpty()) {
            ActionBarMenu menu = this.actionBar.createMenu();
            ActionBarMenuItem searchItem = menu.addItem(0, R.drawable.ic_ab_search).setIsSearchField(true).setActionBarMenuItemSearchListener(new ActionBarMenuItem.ActionBarMenuItemSearchListener() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity.2
                @Override // im.uecahxyvjd.ui.actionbar.ActionBarMenuItem.ActionBarMenuItemSearchListener
                public void onSearchExpand() {
                    NotificationsCustomSettingsActivity.this.searching = true;
                    NotificationsCustomSettingsActivity.this.emptyView.setShowAtCenter(true);
                }

                @Override // im.uecahxyvjd.ui.actionbar.ActionBarMenuItem.ActionBarMenuItemSearchListener
                public void onSearchCollapse() {
                    NotificationsCustomSettingsActivity.this.searchAdapter.searchDialogs(null);
                    NotificationsCustomSettingsActivity.this.searching = false;
                    NotificationsCustomSettingsActivity.this.searchWas = false;
                    NotificationsCustomSettingsActivity.this.emptyView.setText(LocaleController.getString("NoExceptions", R.string.NoExceptions));
                    NotificationsCustomSettingsActivity.this.listView.setAdapter(NotificationsCustomSettingsActivity.this.adapter);
                    NotificationsCustomSettingsActivity.this.adapter.notifyDataSetChanged();
                    NotificationsCustomSettingsActivity.this.listView.setFastScrollVisible(true);
                    NotificationsCustomSettingsActivity.this.listView.setVerticalScrollBarEnabled(false);
                    NotificationsCustomSettingsActivity.this.emptyView.setShowAtCenter(false);
                }

                @Override // im.uecahxyvjd.ui.actionbar.ActionBarMenuItem.ActionBarMenuItemSearchListener
                public void onTextChanged(EditText editText) {
                    if (NotificationsCustomSettingsActivity.this.searchAdapter == null) {
                        return;
                    }
                    String text = editText.getText().toString();
                    if (text.length() != 0) {
                        NotificationsCustomSettingsActivity.this.searchWas = true;
                        if (NotificationsCustomSettingsActivity.this.listView != null) {
                            NotificationsCustomSettingsActivity.this.emptyView.setText(LocaleController.getString("NoResult", R.string.NoResult));
                            NotificationsCustomSettingsActivity.this.emptyView.showProgress();
                            NotificationsCustomSettingsActivity.this.listView.setAdapter(NotificationsCustomSettingsActivity.this.searchAdapter);
                            NotificationsCustomSettingsActivity.this.searchAdapter.notifyDataSetChanged();
                            NotificationsCustomSettingsActivity.this.listView.setFastScrollVisible(false);
                            NotificationsCustomSettingsActivity.this.listView.setVerticalScrollBarEnabled(true);
                        }
                    }
                    NotificationsCustomSettingsActivity.this.searchAdapter.searchDialogs(text);
                }
            });
            searchItem.setSearchFieldHint(LocaleController.getString("Search", R.string.Search));
        }
        this.searchAdapter = new SearchAdapter(context);
        this.fragmentView = new FrameLayout(context);
        FrameLayout frameLayout = (FrameLayout) this.fragmentView;
        frameLayout.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundGray));
        EmptyTextProgressView emptyTextProgressView = new EmptyTextProgressView(context);
        this.emptyView = emptyTextProgressView;
        emptyTextProgressView.setTextSize(18);
        this.emptyView.setText(LocaleController.getString("NoExceptions", R.string.NoExceptions));
        this.emptyView.showTextView();
        frameLayout.addView(this.emptyView, LayoutHelper.createFrame(-1, -1.0f));
        RecyclerListView recyclerListView = new RecyclerListView(context);
        this.listView = recyclerListView;
        recyclerListView.setEmptyView(this.emptyView);
        this.listView.setLayoutManager(new LinearLayoutManager(context, 1, false));
        this.listView.setVerticalScrollBarEnabled(false);
        frameLayout.addView(this.listView, LayoutHelper.createFrame(-1, -1.0f));
        RecyclerListView recyclerListView2 = this.listView;
        ListAdapter listAdapter = new ListAdapter(context);
        this.adapter = listAdapter;
        recyclerListView2.setAdapter(listAdapter);
        this.listView.setOnItemClickListener(new RecyclerListView.OnItemClickListenerExtended() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda3
            @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemClickListenerExtended
            public final void onItemClick(View view, int i, float f, float f2) {
                NotificationsCustomSettingsActivity.this.m1825x38a9ec12(view, i, f, f2);
            }
        });
        this.listView.setOnScrollListener(new RecyclerView.OnScrollListener() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity.3
            @Override // androidx.recyclerview.widget.RecyclerView.OnScrollListener
            public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                if (newState == 1 && NotificationsCustomSettingsActivity.this.searching && NotificationsCustomSettingsActivity.this.searchWas) {
                    AndroidUtilities.hideKeyboard(NotificationsCustomSettingsActivity.this.getParentActivity().getCurrentFocus());
                }
            }

            @Override // androidx.recyclerview.widget.RecyclerView.OnScrollListener
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
            }
        });
        return this.fragmentView;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$9$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1825x38a9ec12(View view, final int position, float x, float y) {
        ArrayList<NotificationsSettingsActivity.NotificationException> arrayList;
        NotificationsSettingsActivity.NotificationException exception;
        boolean newException;
        long did;
        boolean newException2;
        NotificationsSettingsActivity.NotificationException exception2;
        NotificationsSettingsActivity.NotificationException exception3;
        boolean newException3;
        ArrayList<NotificationsSettingsActivity.NotificationException> arrayList2;
        String key;
        String path;
        boolean enabled = false;
        if (getParentActivity() != null) {
            if (this.listView.getAdapter() == this.searchAdapter || (position >= this.exceptionsStartRow && position < this.exceptionsEndRow)) {
                RecyclerView.Adapter adapter = this.listView.getAdapter();
                SearchAdapter searchAdapter = this.searchAdapter;
                if (adapter == searchAdapter) {
                    Object object = searchAdapter.getObject(position);
                    if (!(object instanceof NotificationsSettingsActivity.NotificationException)) {
                        if (object instanceof TLRPC.User) {
                            TLRPC.User user = (TLRPC.User) object;
                            did = user.id;
                        } else {
                            TLRPC.Chat chat = (TLRPC.Chat) object;
                            did = -chat.id;
                        }
                        if (this.exceptionsDict.containsKey(Long.valueOf(did))) {
                            exception2 = this.exceptionsDict.get(Long.valueOf(did));
                            newException2 = false;
                        } else {
                            NotificationsSettingsActivity.NotificationException exception4 = new NotificationsSettingsActivity.NotificationException();
                            exception4.did = did;
                            if (object instanceof TLRPC.User) {
                                TLRPC.User user2 = (TLRPC.User) object;
                                exception4.did = user2.id;
                            } else {
                                TLRPC.Chat chat2 = (TLRPC.Chat) object;
                                exception4.did = -chat2.id;
                            }
                            newException2 = true;
                            exception2 = exception4;
                        }
                        exception3 = exception2;
                        newException3 = newException2;
                        arrayList2 = this.exceptions;
                    } else {
                        arrayList2 = this.searchAdapter.searchResult;
                        exception3 = (NotificationsSettingsActivity.NotificationException) object;
                        newException3 = false;
                    }
                    arrayList = arrayList2;
                    exception = exception3;
                    newException = newException3;
                } else {
                    ArrayList<NotificationsSettingsActivity.NotificationException> arrayList3 = this.exceptions;
                    int index = position - this.exceptionsStartRow;
                    if (index < 0 || index >= arrayList3.size()) {
                        return;
                    }
                    NotificationsSettingsActivity.NotificationException exception5 = arrayList3.get(index);
                    arrayList = arrayList3;
                    exception = exception5;
                    newException = false;
                }
                if (exception != null) {
                    final boolean z = newException;
                    final ArrayList<NotificationsSettingsActivity.NotificationException> arrayList4 = arrayList;
                    final NotificationsSettingsActivity.NotificationException notificationException = exception;
                    AlertsCreator.showCustomNotificationsDialog(this, exception.did, -1, null, this.currentAccount, null, new MessagesStorage.IntCallback() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda7
                        @Override // im.uecahxyvjd.messenger.MessagesStorage.IntCallback
                        public final void run(int i) {
                            NotificationsCustomSettingsActivity.this.m1816x4db2787b(z, arrayList4, notificationException, position, i);
                        }
                    });
                    return;
                }
                return;
            }
            if (position != this.exceptionsAddRow) {
                if (position != this.deleteAllRow) {
                    if (position == this.alertRow) {
                        enabled = getNotificationsController().isGlobalNotificationsEnabled(this.currentType);
                        final NotificationsCheckCell checkCell = (NotificationsCheckCell) view;
                        final RecyclerView.ViewHolder holder = this.listView.findViewHolderForAdapterPosition(position);
                        if (!enabled) {
                            getNotificationsController().setGlobalNotificationsEnabled(this.currentType, 0);
                            checkCell.setChecked(!enabled);
                            if (holder != null) {
                                this.adapter.onBindViewHolder(holder, position);
                            }
                            checkRowsEnabled();
                        } else {
                            AlertsCreator.showCustomNotificationsDialog(this, 0L, this.currentType, this.exceptions, this.currentAccount, new MessagesStorage.IntCallback() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda10
                                @Override // im.uecahxyvjd.messenger.MessagesStorage.IntCallback
                                public final void run(int i) {
                                    NotificationsCustomSettingsActivity.this.m1820xb62072f7(checkCell, holder, position, i);
                                }
                            });
                        }
                    } else if (position != this.previewRow) {
                        if (position == this.messageSoundRow) {
                            if (!view.isEnabled()) {
                                return;
                            }
                            try {
                                SharedPreferences preferences = getNotificationsSettings();
                                Intent tmpIntent = new Intent("android.intent.action.RINGTONE_PICKER");
                                tmpIntent.putExtra("android.intent.extra.ringtone.TYPE", 2);
                                tmpIntent.putExtra("android.intent.extra.ringtone.SHOW_DEFAULT", true);
                                tmpIntent.putExtra("android.intent.extra.ringtone.SHOW_SILENT", true);
                                tmpIntent.putExtra("android.intent.extra.ringtone.DEFAULT_URI", RingtoneManager.getDefaultUri(2));
                                Uri currentSound = null;
                                String defaultPath = null;
                                Uri defaultUri = Settings.System.DEFAULT_NOTIFICATION_URI;
                                if (defaultUri != null) {
                                    defaultPath = defaultUri.getPath();
                                }
                                int i = this.currentType;
                                if (i == 1) {
                                    path = preferences.getString("GlobalSoundPath", defaultPath);
                                } else if (i == 0) {
                                    path = preferences.getString("GroupSoundPath", defaultPath);
                                } else {
                                    path = preferences.getString("ChannelSoundPath", defaultPath);
                                }
                                if (path != null && !path.equals("NoSound")) {
                                    currentSound = path.equals(defaultPath) ? defaultUri : Uri.parse(path);
                                }
                                tmpIntent.putExtra("android.intent.extra.ringtone.EXISTING_URI", currentSound);
                                startActivityForResult(tmpIntent, position);
                            } catch (Exception e) {
                                FileLog.e(e);
                            }
                        } else if (position == this.messageLedRow) {
                            if (view.isEnabled()) {
                                showDialog(AlertsCreator.createColorSelectDialog(getParentActivity(), 0L, this.currentType, new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda11
                                    @Override // java.lang.Runnable
                                    public final void run() {
                                        NotificationsCustomSettingsActivity.this.m1821xd03bf196(position);
                                    }
                                }));
                            } else {
                                return;
                            }
                        } else if (position == this.messagePopupNotificationRow) {
                            if (view.isEnabled()) {
                                showDialog(AlertsCreator.createPopupSelectDialog(getParentActivity(), this.currentType, new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda12
                                    @Override // java.lang.Runnable
                                    public final void run() {
                                        NotificationsCustomSettingsActivity.this.m1822xea577035(position);
                                    }
                                }));
                            } else {
                                return;
                            }
                        } else if (position == this.messageVibrateRow) {
                            if (!view.isEnabled()) {
                                return;
                            }
                            int i2 = this.currentType;
                            if (i2 == 1) {
                                key = "vibrate_messages";
                            } else if (i2 == 0) {
                                key = "vibrate_group";
                            } else {
                                key = "vibrate_channel";
                            }
                            showDialog(AlertsCreator.createVibrationSelectDialog(getParentActivity(), 0L, key, new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda1
                                @Override // java.lang.Runnable
                                public final void run() {
                                    NotificationsCustomSettingsActivity.this.m1823x472eed4(position);
                                }
                            }));
                        } else if (position == this.messagePriorityRow) {
                            if (view.isEnabled()) {
                                showDialog(AlertsCreator.createPrioritySelectDialog(getParentActivity(), 0L, this.currentType, new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda2
                                    @Override // java.lang.Runnable
                                    public final void run() {
                                        NotificationsCustomSettingsActivity.this.m1824x1e8e6d73(position);
                                    }
                                }));
                            } else {
                                return;
                            }
                        }
                    } else if (!view.isEnabled()) {
                        return;
                    } else {
                        SharedPreferences preferences2 = getNotificationsSettings();
                        SharedPreferences.Editor editor = preferences2.edit();
                        int i3 = this.currentType;
                        if (i3 == 1) {
                            boolean enabled2 = preferences2.getBoolean("EnablePreviewAll", true);
                            editor.putBoolean("EnablePreviewAll", !enabled2);
                            enabled = enabled2;
                        } else if (i3 == 0) {
                            boolean enabled3 = preferences2.getBoolean("EnablePreviewGroup", true);
                            editor.putBoolean("EnablePreviewGroup", !enabled3);
                            enabled = enabled3;
                        } else {
                            boolean enabled4 = preferences2.getBoolean("EnablePreviewChannel", true);
                            editor.putBoolean("EnablePreviewChannel", !enabled4);
                            enabled = enabled4;
                        }
                        editor.commit();
                        getNotificationsController().updateServerNotificationsSettings(this.currentType);
                    }
                } else {
                    AlertDialog.Builder builder = new AlertDialog.Builder(getParentActivity());
                    builder.setTitle(LocaleController.getString("NotificationsDeleteAllExceptionTitle", R.string.NotificationsDeleteAllExceptionTitle));
                    builder.setMessage(LocaleController.getString("NotificationsDeleteAllExceptionAlert", R.string.NotificationsDeleteAllExceptionAlert));
                    builder.setPositiveButton(LocaleController.getString("Delete", R.string.Delete), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda9
                        @Override // android.content.DialogInterface.OnClickListener
                        public final void onClick(DialogInterface dialogInterface, int i4) {
                            NotificationsCustomSettingsActivity.this.m1819x9c04f458(dialogInterface, i4);
                        }
                    });
                    builder.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
                    AlertDialog alertDialog = builder.create();
                    showDialog(alertDialog);
                    TextView button = (TextView) alertDialog.getButton(-1);
                    if (button != null) {
                        button.setTextColor(Theme.getColor(Theme.key_dialogTextRed2));
                    }
                }
            } else {
                Bundle args = new Bundle();
                args.putBoolean("onlySelect", true);
                args.putBoolean("checkCanWrite", false);
                int i4 = this.currentType;
                if (i4 == 0) {
                    args.putInt("dialogsType", 6);
                } else if (i4 == 2) {
                    args.putInt("dialogsType", 5);
                } else {
                    args.putInt("dialogsType", 4);
                }
                DialogsActivity activity = new DialogsActivity(args);
                activity.setDelegate(new DialogsActivity.DialogsActivityDelegate() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda8
                    @Override // im.uecahxyvjd.ui.DialogsActivity.DialogsActivityDelegate
                    public final void didSelectDialogs(DialogsActivity dialogsActivity, ArrayList arrayList5, CharSequence charSequence, boolean z2) {
                        NotificationsCustomSettingsActivity.this.m1818x81e975b9(dialogsActivity, arrayList5, charSequence, z2);
                    }
                });
                presentFragment(activity);
            }
            if (view instanceof TextCheckCell) {
                ((TextCheckCell) view).setChecked(!enabled);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$0$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1816x4db2787b(boolean newException, ArrayList arrayList, NotificationsSettingsActivity.NotificationException exception, int position, int param) {
        int time;
        int idx;
        if (param == 0) {
            if (newException) {
                return;
            }
            ArrayList<NotificationsSettingsActivity.NotificationException> arrayList2 = this.exceptions;
            if (arrayList != arrayList2 && (idx = arrayList2.indexOf(exception)) >= 0) {
                this.exceptions.remove(idx);
                this.exceptionsDict.remove(Long.valueOf(exception.did));
            }
            arrayList.remove(exception);
            if (this.exceptionsAddRow != -1 && arrayList.isEmpty() && arrayList == this.exceptions) {
                this.listView.getAdapter().notifyItemChanged(this.exceptionsAddRow);
                this.listView.getAdapter().notifyItemRemoved(this.deleteAllRow);
                this.listView.getAdapter().notifyItemRemoved(this.deleteAllSectionRow);
            }
            this.listView.getAdapter().notifyItemRemoved(position);
            updateRows();
            checkRowsEnabled();
            this.actionBar.closeSearchField();
            return;
        }
        SharedPreferences preferences = getNotificationsSettings();
        exception.hasCustom = preferences.getBoolean(ContentMetadata.KEY_CUSTOM_PREFIX + exception.did, false);
        exception.notify = preferences.getInt("notify2_" + exception.did, 0);
        if (exception.notify != 0 && (time = preferences.getInt("notifyuntil_" + exception.did, -1)) != -1) {
            exception.muteUntil = time;
        }
        if (newException) {
            this.exceptions.add(exception);
            this.exceptionsDict.put(Long.valueOf(exception.did), exception);
            updateRows();
            this.adapter.notifyDataSetChanged();
        } else {
            this.listView.getAdapter().notifyItemChanged(position);
        }
        this.actionBar.closeSearchField();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$2$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1818x81e975b9(DialogsActivity fragment, ArrayList dids, CharSequence message, boolean param) {
        Bundle args2 = new Bundle();
        args2.putLong("dialog_id", ((Long) dids.get(0)).longValue());
        args2.putBoolean("exception", true);
        ProfileNotificationsActivity profileNotificationsActivity = new ProfileNotificationsActivity(args2);
        profileNotificationsActivity.setDelegate(new ProfileNotificationsActivity.ProfileNotificationsActivityDelegate() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda0
            @Override // im.uecahxyvjd.ui.ProfileNotificationsActivity.ProfileNotificationsActivityDelegate
            public final void didCreateNewException(NotificationsSettingsActivity.NotificationException notificationException) {
                NotificationsCustomSettingsActivity.this.m1817x67cdf71a(notificationException);
            }
        });
        presentFragment(profileNotificationsActivity, true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$1$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1817x67cdf71a(NotificationsSettingsActivity.NotificationException exception) {
        this.exceptions.add(0, exception);
        updateRows();
        this.adapter.notifyDataSetChanged();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$3$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1819x9c04f458(DialogInterface dialogInterface, int i) {
        SharedPreferences preferences = getNotificationsSettings();
        SharedPreferences.Editor editor = preferences.edit();
        int N = this.exceptions.size();
        for (int a2 = 0; a2 < N; a2++) {
            NotificationsSettingsActivity.NotificationException exception = this.exceptions.get(a2);
            editor.remove("notify2_" + exception.did).remove(ContentMetadata.KEY_CUSTOM_PREFIX + exception.did);
            getMessagesStorage().setDialogFlags(exception.did, 0L);
            TLRPC.Dialog dialog = getMessagesController().dialogs_dict.get(exception.did);
            if (dialog != null) {
                dialog.notify_settings = new TLRPC.TL_peerNotifySettings();
            }
        }
        editor.commit();
        int N2 = this.exceptions.size();
        for (int a3 = 0; a3 < N2; a3++) {
            getNotificationsController().updateServerNotificationsSettings(this.exceptions.get(a3).did, false);
        }
        this.exceptions.clear();
        this.exceptionsDict.clear();
        updateRows();
        getNotificationCenter().postNotificationName(NotificationCenter.notificationsSettingsUpdated, new Object[0]);
        this.adapter.notifyDataSetChanged();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$4$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1820xb62072f7(NotificationsCheckCell checkCell, RecyclerView.ViewHolder holder, int position, int param) {
        int offUntil;
        int iconType;
        SharedPreferences preferences = getNotificationsSettings();
        int offUntil2 = this.currentType;
        if (offUntil2 == 1) {
            offUntil = preferences.getInt("EnableAll2", 0);
        } else if (offUntil2 == 0) {
            offUntil = preferences.getInt("EnableGroup2", 0);
        } else {
            offUntil = preferences.getInt("EnableChannel2", 0);
        }
        int currentTime = getConnectionsManager().getCurrentTime();
        if (offUntil < currentTime) {
            iconType = 0;
        } else if (offUntil - 31536000 >= currentTime) {
            iconType = 0;
        } else {
            iconType = 2;
        }
        checkCell.setChecked(getNotificationsController().isGlobalNotificationsEnabled(this.currentType), iconType);
        if (holder != null) {
            this.adapter.onBindViewHolder(holder, position);
        }
        checkRowsEnabled();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$5$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1821xd03bf196(int position) {
        RecyclerView.ViewHolder holder = this.listView.findViewHolderForAdapterPosition(position);
        if (holder != null) {
            this.adapter.onBindViewHolder(holder, position);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$6$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1822xea577035(int position) {
        RecyclerView.ViewHolder holder = this.listView.findViewHolderForAdapterPosition(position);
        if (holder != null) {
            this.adapter.onBindViewHolder(holder, position);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$7$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1823x472eed4(int position) {
        RecyclerView.ViewHolder holder = this.listView.findViewHolderForAdapterPosition(position);
        if (holder != null) {
            this.adapter.onBindViewHolder(holder, position);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$8$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1824x1e8e6d73(int position) {
        RecyclerView.ViewHolder holder = this.listView.findViewHolderForAdapterPosition(position);
        if (holder != null) {
            this.adapter.onBindViewHolder(holder, position);
        }
    }

    private void checkRowsEnabled() {
        if (!this.exceptions.isEmpty()) {
            return;
        }
        int count = this.listView.getChildCount();
        ArrayList<Animator> animators = new ArrayList<>();
        boolean enabled = getNotificationsController().isGlobalNotificationsEnabled(this.currentType);
        for (int a2 = 0; a2 < count; a2++) {
            View child = this.listView.getChildAt(a2);
            RecyclerListView.Holder holder = (RecyclerListView.Holder) this.listView.getChildViewHolder(child);
            switch (holder.getItemViewType()) {
                case 0:
                    HeaderCell headerCell = (HeaderCell) holder.itemView;
                    if (holder.getAdapterPosition() == this.messageSectionRow) {
                        headerCell.setEnabled(enabled, animators);
                        break;
                    } else {
                        break;
                    }
                case 1:
                    TextCheckCell textCell = (TextCheckCell) holder.itemView;
                    textCell.setEnabled(enabled, animators);
                    break;
                case 3:
                    TextColorCell textCell2 = (TextColorCell) holder.itemView;
                    textCell2.setEnabled(enabled, animators);
                    break;
                case 5:
                    TextSettingsCell textCell3 = (TextSettingsCell) holder.itemView;
                    textCell3.setEnabled(enabled, animators);
                    break;
            }
        }
        if (!animators.isEmpty()) {
            AnimatorSet animatorSet = this.animatorSet;
            if (animatorSet != null) {
                animatorSet.cancel();
            }
            AnimatorSet animatorSet2 = new AnimatorSet();
            this.animatorSet = animatorSet2;
            animatorSet2.playTogether(animators);
            this.animatorSet.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity.4
                @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
                public void onAnimationEnd(Animator animator) {
                    if (animator.equals(NotificationsCustomSettingsActivity.this.animatorSet)) {
                        NotificationsCustomSettingsActivity.this.animatorSet = null;
                    }
                }
            });
            this.animatorSet.setDuration(150L);
            this.animatorSet.start();
        }
    }

    private void loadExceptions() {
        getMessagesStorage().getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda4
            @Override // java.lang.Runnable
            public final void run() {
                NotificationsCustomSettingsActivity.this.m1828x4c648848();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:117:0x02ea  */
    /* JADX WARN: Removed duplicated region for block: B:124:0x0305 A[LOOP:3: B:123:0x0303->B:124:0x0305, LOOP_END] */
    /* JADX WARN: Removed duplicated region for block: B:127:0x031e  */
    /* JADX WARN: Removed duplicated region for block: B:98:0x029e  */
    /* renamed from: lambda$loadExceptions$11$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m1828x4c648848() {
        ArrayList<TLRPC.Chat> chats;
        ArrayList<TLRPC.User> users;
        int a2;
        int size;
        int a3;
        int size2;
        int a4;
        int size3;
        int a5;
        int size4;
        int selfId;
        Map<String, ?> values;
        SharedPreferences preferences;
        ArrayList<TLRPC.Chat> chats2;
        ArrayList<TLRPC.User> users2;
        Integer time;
        final ArrayList<NotificationsSettingsActivity.NotificationException> usersResult = new ArrayList<>();
        ArrayList<NotificationsSettingsActivity.NotificationException> chatsResult = new ArrayList<>();
        ArrayList<NotificationsSettingsActivity.NotificationException> channelsResult = new ArrayList<>();
        LongSparseArray<NotificationsSettingsActivity.NotificationException> waitingForLoadExceptions = new LongSparseArray<>();
        ArrayList<Integer> usersToLoad = new ArrayList<>();
        ArrayList<Integer> chatsToLoad = new ArrayList<>();
        ArrayList<Integer> encryptedChatsToLoad = new ArrayList<>();
        ArrayList<TLRPC.User> users3 = new ArrayList<>();
        ArrayList<TLRPC.Chat> chats3 = new ArrayList<>();
        final ArrayList<TLRPC.EncryptedChat> encryptedChats = new ArrayList<>();
        int selfId2 = getUserConfig().clientUserId;
        SharedPreferences preferences2 = getNotificationsSettings();
        Map<String, ?> values2 = preferences2.getAll();
        Iterator<Map.Entry<String, ?>> it = values2.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<String, ?> entry = it.next();
            String key = entry.getKey();
            Iterator<Map.Entry<String, ?>> it2 = it;
            if (key.startsWith("notify2_")) {
                chats2 = chats3;
                String key2 = key.replace("notify2_", "");
                users2 = users3;
                long did = Utilities.parseLong(key2).longValue();
                if (did != 0) {
                    ArrayList<NotificationsSettingsActivity.NotificationException> chatsResult2 = chatsResult;
                    ArrayList<NotificationsSettingsActivity.NotificationException> channelsResult2 = channelsResult;
                    if (did != selfId2) {
                        NotificationsSettingsActivity.NotificationException exception = new NotificationsSettingsActivity.NotificationException();
                        exception.did = did;
                        exception.hasCustom = preferences2.getBoolean(ContentMetadata.KEY_CUSTOM_PREFIX + did, false);
                        exception.notify = ((Integer) entry.getValue()).intValue();
                        if (exception.notify != 0 && (time = (Integer) values2.get("notifyuntil_" + key2)) != null) {
                            exception.muteUntil = time.intValue();
                        }
                        int lower_id = (int) did;
                        int high_id = (int) (did << 32);
                        if (lower_id == 0) {
                            channelsResult = channelsResult2;
                            chatsResult = chatsResult2;
                            if (high_id != 0) {
                                values = values2;
                                TLRPC.EncryptedChat encryptedChat = getMessagesController().getEncryptedChat(Integer.valueOf(high_id));
                                if (encryptedChat == null) {
                                    encryptedChatsToLoad.add(Integer.valueOf(high_id));
                                    waitingForLoadExceptions.put(did, exception);
                                    preferences = preferences2;
                                } else {
                                    TLRPC.User user = getMessagesController().getUser(Integer.valueOf(encryptedChat.user_id));
                                    if (user == null) {
                                        usersToLoad.add(Integer.valueOf(encryptedChat.user_id));
                                        preferences = preferences2;
                                        waitingForLoadExceptions.put(encryptedChat.user_id, exception);
                                    } else {
                                        preferences = preferences2;
                                        if (user.deleted) {
                                            it = it2;
                                            chats3 = chats2;
                                            users3 = users2;
                                            values2 = values;
                                            preferences2 = preferences;
                                        }
                                    }
                                }
                                usersResult.add(exception);
                            } else {
                                values = values2;
                                preferences = preferences2;
                            }
                        } else if (lower_id > 0) {
                            TLRPC.User user2 = getMessagesController().getUser(Integer.valueOf(lower_id));
                            if (user2 == null) {
                                usersToLoad.add(Integer.valueOf(lower_id));
                                waitingForLoadExceptions.put(did, exception);
                            } else if (user2.deleted) {
                                it = it2;
                                chats3 = chats2;
                                users3 = users2;
                                chatsResult = chatsResult2;
                                channelsResult = channelsResult2;
                            }
                            usersResult.add(exception);
                            preferences = preferences2;
                            chatsResult = chatsResult2;
                            channelsResult = channelsResult2;
                            values = values2;
                        } else {
                            TLRPC.Chat chat = getMessagesController().getChat(Integer.valueOf(-lower_id));
                            if (chat == null) {
                                chatsToLoad.add(Integer.valueOf(-lower_id));
                                waitingForLoadExceptions.put(did, exception);
                                it = it2;
                                chats3 = chats2;
                                users3 = users2;
                                chatsResult = chatsResult2;
                                channelsResult = channelsResult2;
                            } else if (chat.left || chat.kicked) {
                                channelsResult = channelsResult2;
                                chatsResult = chatsResult2;
                                it = it2;
                                chats3 = chats2;
                                users3 = users2;
                            } else if (chat.migrated_to != null) {
                                it = it2;
                                chats3 = chats2;
                                users3 = users2;
                                chatsResult = chatsResult2;
                                channelsResult = channelsResult2;
                            } else {
                                if (!ChatObject.isChannel(chat) || chat.megagroup) {
                                    channelsResult = channelsResult2;
                                    chatsResult = chatsResult2;
                                    chatsResult.add(exception);
                                } else {
                                    channelsResult = channelsResult2;
                                    channelsResult.add(exception);
                                    chatsResult = chatsResult2;
                                }
                                values = values2;
                                preferences = preferences2;
                            }
                        }
                    } else {
                        preferences = preferences2;
                        chatsResult = chatsResult2;
                        channelsResult = channelsResult2;
                        values = values2;
                    }
                } else {
                    values = values2;
                    preferences = preferences2;
                }
            } else {
                values = values2;
                preferences = preferences2;
                chats2 = chats3;
                users2 = users3;
            }
            it = it2;
            chats3 = chats2;
            users3 = users2;
            values2 = values;
            preferences2 = preferences;
        }
        ArrayList<TLRPC.Chat> chats4 = chats3;
        ArrayList<TLRPC.User> users4 = users3;
        if (waitingForLoadExceptions.size() != 0) {
            try {
                if (!encryptedChatsToLoad.isEmpty()) {
                    try {
                        getMessagesStorage().getEncryptedChatsInternal(TextUtils.join(",", encryptedChatsToLoad), encryptedChats, usersToLoad);
                    } catch (Exception e) {
                        e = e;
                        chats = chats4;
                        users = users4;
                        FileLog.e(e);
                        a2 = 0;
                        size = chats.size();
                        while (a2 < size) {
                        }
                        size2 = users.size();
                        while (a3 < size2) {
                        }
                        size3 = encryptedChats.size();
                        while (a4 < size3) {
                        }
                        size4 = waitingForLoadExceptions.size();
                        while (a5 < size4) {
                        }
                        final ArrayList<TLRPC.User> arrayList = users;
                        final ArrayList<TLRPC.Chat> arrayList2 = chats;
                        final ArrayList<NotificationsSettingsActivity.NotificationException> arrayList3 = chatsResult;
                        final ArrayList<NotificationsSettingsActivity.NotificationException> arrayList4 = channelsResult;
                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda6
                            @Override // java.lang.Runnable
                            public final void run() {
                                NotificationsCustomSettingsActivity.this.m1827x324909a9(arrayList, arrayList2, encryptedChats, usersResult, arrayList3, arrayList4);
                            }
                        });
                    }
                }
                if (usersToLoad.isEmpty()) {
                    users = users4;
                } else {
                    try {
                        users = users4;
                    } catch (Exception e2) {
                        e = e2;
                        users = users4;
                        chats = chats4;
                    }
                    try {
                        getMessagesStorage().getUsersInternal(TextUtils.join(",", usersToLoad), users);
                    } catch (Exception e3) {
                        e = e3;
                        chats = chats4;
                        FileLog.e(e);
                        a2 = 0;
                        size = chats.size();
                        while (a2 < size) {
                        }
                        size2 = users.size();
                        while (a3 < size2) {
                        }
                        size3 = encryptedChats.size();
                        while (a4 < size3) {
                        }
                        size4 = waitingForLoadExceptions.size();
                        while (a5 < size4) {
                        }
                        final ArrayList arrayList5 = users;
                        final ArrayList arrayList22 = chats;
                        final ArrayList arrayList32 = chatsResult;
                        final ArrayList arrayList42 = channelsResult;
                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda6
                            @Override // java.lang.Runnable
                            public final void run() {
                                NotificationsCustomSettingsActivity.this.m1827x324909a9(arrayList5, arrayList22, encryptedChats, usersResult, arrayList32, arrayList42);
                            }
                        });
                    }
                }
                try {
                    if (chatsToLoad.isEmpty()) {
                        chats = chats4;
                    } else {
                        chats = chats4;
                        try {
                            getMessagesStorage().getChatsInternal(TextUtils.join(",", chatsToLoad), chats);
                        } catch (Exception e4) {
                            e = e4;
                            FileLog.e(e);
                            a2 = 0;
                            size = chats.size();
                            while (a2 < size) {
                            }
                            size2 = users.size();
                            while (a3 < size2) {
                            }
                            size3 = encryptedChats.size();
                            while (a4 < size3) {
                            }
                            size4 = waitingForLoadExceptions.size();
                            while (a5 < size4) {
                            }
                            final ArrayList arrayList52 = users;
                            final ArrayList arrayList222 = chats;
                            final ArrayList arrayList322 = chatsResult;
                            final ArrayList arrayList422 = channelsResult;
                            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda6
                                @Override // java.lang.Runnable
                                public final void run() {
                                    NotificationsCustomSettingsActivity.this.m1827x324909a9(arrayList52, arrayList222, encryptedChats, usersResult, arrayList322, arrayList422);
                                }
                            });
                        }
                    }
                } catch (Exception e5) {
                    e = e5;
                    chats = chats4;
                }
            } catch (Exception e6) {
                e = e6;
                chats = chats4;
                users = users4;
            }
            a2 = 0;
            size = chats.size();
            while (a2 < size) {
                TLRPC.Chat chat2 = chats.get(a2);
                if (chat2.left || chat2.kicked) {
                    selfId = selfId2;
                } else if (chat2.migrated_to != null) {
                    selfId = selfId2;
                } else {
                    NotificationsSettingsActivity.NotificationException exception2 = waitingForLoadExceptions.get(-chat2.id);
                    selfId = selfId2;
                    waitingForLoadExceptions.remove(-chat2.id);
                    if (exception2 != null) {
                        if (!ChatObject.isChannel(chat2) || chat2.megagroup) {
                            chatsResult.add(exception2);
                        } else {
                            channelsResult.add(exception2);
                        }
                    }
                }
                a2++;
                selfId2 = selfId;
            }
            size2 = users.size();
            for (a3 = 0; a3 < size2; a3++) {
                TLRPC.User user3 = users.get(a3);
                if (!user3.deleted) {
                    waitingForLoadExceptions.remove(user3.id);
                }
            }
            size3 = encryptedChats.size();
            for (a4 = 0; a4 < size3; a4++) {
                waitingForLoadExceptions.remove(encryptedChats.get(a4).id << 32);
            }
            size4 = waitingForLoadExceptions.size();
            for (a5 = 0; a5 < size4; a5++) {
                if (((int) waitingForLoadExceptions.keyAt(a5)) < 0) {
                    chatsResult.remove(waitingForLoadExceptions.valueAt(a5));
                    channelsResult.remove(waitingForLoadExceptions.valueAt(a5));
                } else {
                    usersResult.remove(waitingForLoadExceptions.valueAt(a5));
                }
            }
        } else {
            chats = chats4;
            users = users4;
        }
        final ArrayList arrayList522 = users;
        final ArrayList arrayList2222 = chats;
        final ArrayList arrayList3222 = chatsResult;
        final ArrayList arrayList4222 = channelsResult;
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda6
            @Override // java.lang.Runnable
            public final void run() {
                NotificationsCustomSettingsActivity.this.m1827x324909a9(arrayList522, arrayList2222, encryptedChats, usersResult, arrayList3222, arrayList4222);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadExceptions$10$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1827x324909a9(ArrayList users, ArrayList chats, ArrayList encryptedChats, ArrayList usersResult, ArrayList chatsResult, ArrayList channelsResult) {
        getMessagesController().putUsers(users, true);
        getMessagesController().putChats(chats, true);
        getMessagesController().putEncryptedChats(encryptedChats, true);
        int i = this.currentType;
        if (i == 1) {
            this.exceptions = usersResult;
        } else if (i == 0) {
            this.exceptions = chatsResult;
        } else {
            this.exceptions = channelsResult;
        }
        updateRows();
        this.adapter.notifyDataSetChanged();
    }

    private void updateRows() {
        ArrayList<NotificationsSettingsActivity.NotificationException> arrayList;
        this.rowCount = 0;
        int i = this.currentType;
        if (i != -1) {
            int i2 = 0 + 1;
            this.rowCount = i2;
            this.alertRow = 0;
            int i3 = i2 + 1;
            this.rowCount = i3;
            this.alertSection2Row = i2;
            int i4 = i3 + 1;
            this.rowCount = i4;
            this.messageSectionRow = i3;
            int i5 = i4 + 1;
            this.rowCount = i5;
            this.previewRow = i4;
            int i6 = i5 + 1;
            this.rowCount = i6;
            this.messageLedRow = i5;
            int i7 = i6 + 1;
            this.rowCount = i7;
            this.messageVibrateRow = i6;
            if (i == 2) {
                this.messagePopupNotificationRow = -1;
            } else {
                this.rowCount = i7 + 1;
                this.messagePopupNotificationRow = i7;
            }
            int i8 = this.rowCount;
            this.rowCount = i8 + 1;
            this.messageSoundRow = i8;
            int i9 = this.rowCount;
            int i10 = i9 + 1;
            this.rowCount = i10;
            this.messagePriorityRow = i9;
            int i11 = i10 + 1;
            this.rowCount = i11;
            this.groupSection2Row = i10;
            this.rowCount = i11 + 1;
            this.exceptionsAddRow = i11;
        } else {
            this.alertRow = -1;
            this.alertSection2Row = -1;
            this.messageSectionRow = -1;
            this.previewRow = -1;
            this.messageLedRow = -1;
            this.messageVibrateRow = -1;
            this.messagePopupNotificationRow = -1;
            this.messageSoundRow = -1;
            this.messagePriorityRow = -1;
            this.groupSection2Row = -1;
            this.exceptionsAddRow = -1;
        }
        ArrayList<NotificationsSettingsActivity.NotificationException> arrayList2 = this.exceptions;
        if (arrayList2 != null && !arrayList2.isEmpty()) {
            int i12 = this.rowCount;
            this.exceptionsStartRow = i12;
            int size = i12 + this.exceptions.size();
            this.rowCount = size;
            this.exceptionsEndRow = size;
        } else {
            this.exceptionsStartRow = -1;
            this.exceptionsEndRow = -1;
        }
        if (this.currentType != -1 || ((arrayList = this.exceptions) != null && !arrayList.isEmpty())) {
            int i13 = this.rowCount;
            this.rowCount = i13 + 1;
            this.exceptionsSection2Row = i13;
        } else {
            this.exceptionsSection2Row = -1;
        }
        ArrayList<NotificationsSettingsActivity.NotificationException> arrayList3 = this.exceptions;
        if (arrayList3 != null && !arrayList3.isEmpty()) {
            int i14 = this.rowCount;
            int i15 = i14 + 1;
            this.rowCount = i15;
            this.deleteAllRow = i14;
            this.rowCount = i15 + 1;
            this.deleteAllSectionRow = i15;
            return;
        }
        this.deleteAllRow = -1;
        this.deleteAllSectionRow = -1;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onActivityResultFragment(int requestCode, int resultCode, Intent data) {
        Ringtone rng;
        if (resultCode == -1) {
            Uri ringtone = (Uri) data.getParcelableExtra("android.intent.extra.ringtone.PICKED_URI");
            String name = null;
            if (ringtone != null && (rng = RingtoneManager.getRingtone(getParentActivity(), ringtone)) != null) {
                if (ringtone.equals(Settings.System.DEFAULT_NOTIFICATION_URI)) {
                    name = LocaleController.getString("SoundDefault", R.string.SoundDefault);
                } else {
                    name = rng.getTitle(getParentActivity());
                }
                rng.stop();
            }
            SharedPreferences preferences = getNotificationsSettings();
            SharedPreferences.Editor editor = preferences.edit();
            int i = this.currentType;
            if (i == 1) {
                if (name != null && ringtone != null) {
                    editor.putString("GlobalSound", name);
                    editor.putString("GlobalSoundPath", ringtone.toString());
                } else {
                    editor.putString("GlobalSound", "NoSound");
                    editor.putString("GlobalSoundPath", "NoSound");
                }
            } else if (i == 0) {
                if (name != null && ringtone != null) {
                    editor.putString("GroupSound", name);
                    editor.putString("GroupSoundPath", ringtone.toString());
                } else {
                    editor.putString("GroupSound", "NoSound");
                    editor.putString("GroupSoundPath", "NoSound");
                }
            } else if (i == 2) {
                if (name != null && ringtone != null) {
                    editor.putString("ChannelSound", name);
                    editor.putString("ChannelSoundPath", ringtone.toString());
                } else {
                    editor.putString("ChannelSound", "NoSound");
                    editor.putString("ChannelSoundPath", "NoSound");
                }
            }
            editor.commit();
            getNotificationsController().updateServerNotificationsSettings(this.currentType);
            RecyclerView.ViewHolder holder = this.listView.findViewHolderForAdapterPosition(requestCode);
            if (holder != null) {
                this.adapter.onBindViewHolder(holder, requestCode);
            }
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onResume() {
        super.onResume();
        ListAdapter listAdapter = this.adapter;
        if (listAdapter != null) {
            listAdapter.notifyDataSetChanged();
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes12.dex */
    public class SearchAdapter extends RecyclerListView.SelectionAdapter {
        private Context mContext;
        private SearchAdapterHelper searchAdapterHelper;
        private ArrayList<NotificationsSettingsActivity.NotificationException> searchResult = new ArrayList<>();
        private ArrayList<CharSequence> searchResultNames = new ArrayList<>();
        private Runnable searchRunnable;

        public SearchAdapter(Context context) {
            this.mContext = context;
            SearchAdapterHelper searchAdapterHelper = new SearchAdapterHelper(true);
            this.searchAdapterHelper = searchAdapterHelper;
            searchAdapterHelper.setDelegate(new SearchAdapterHelper.SearchAdapterHelperDelegate() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$SearchAdapter$$ExternalSyntheticLambda0
                @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
                public /* synthetic */ SparseArray getExcludeUsers() {
                    return SearchAdapterHelper.SearchAdapterHelperDelegate.CC.$default$getExcludeUsers(this);
                }

                @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
                public final void onDataSetChanged() {
                    NotificationsCustomSettingsActivity.SearchAdapter.this.m1829x3dba99();
                }

                @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
                public /* synthetic */ void onSetHashtags(ArrayList arrayList, HashMap hashMap) {
                    SearchAdapterHelper.SearchAdapterHelperDelegate.CC.$default$onSetHashtags(this, arrayList, hashMap);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$new$0$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity$SearchAdapter  reason: not valid java name */
        public /* synthetic */ void m1829x3dba99() {
            if (this.searchRunnable == null && !this.searchAdapterHelper.isSearchInProgress()) {
                NotificationsCustomSettingsActivity.this.emptyView.showTextView();
            }
            notifyDataSetChanged();
        }

        public void searchDialogs(final String query) {
            if (this.searchRunnable != null) {
                Utilities.searchQueue.cancelRunnable(this.searchRunnable);
                this.searchRunnable = null;
            }
            if (query == null) {
                this.searchResult.clear();
                this.searchResultNames.clear();
                this.searchAdapterHelper.mergeResults(null);
                this.searchAdapterHelper.queryServerSearch(null, true, NotificationsCustomSettingsActivity.this.currentType != 1, true, false, 0, false, 0);
                notifyDataSetChanged();
                return;
            }
            DispatchQueue dispatchQueue = Utilities.searchQueue;
            Runnable runnable = new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$SearchAdapter$$ExternalSyntheticLambda4
                @Override // java.lang.Runnable
                public final void run() {
                    NotificationsCustomSettingsActivity.SearchAdapter.this.m1832x352040d5(query);
                }
            };
            this.searchRunnable = runnable;
            dispatchQueue.postRunnable(runnable, 300L);
        }

        /* JADX INFO: Access modifiers changed from: private */
        /* renamed from: processSearch */
        public void m1832x352040d5(final String query) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$SearchAdapter$$ExternalSyntheticLambda3
                @Override // java.lang.Runnable
                public final void run() {
                    NotificationsCustomSettingsActivity.SearchAdapter.this.m1831x50f5867f(query);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$processSearch$3$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity$SearchAdapter  reason: not valid java name */
        public /* synthetic */ void m1831x50f5867f(final String query) {
            this.searchAdapterHelper.queryServerSearch(query, true, NotificationsCustomSettingsActivity.this.currentType != 1, true, false, 0, false, 0);
            final ArrayList<NotificationsSettingsActivity.NotificationException> contactsCopy = new ArrayList<>(NotificationsCustomSettingsActivity.this.exceptions);
            Utilities.searchQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$SearchAdapter$$ExternalSyntheticLambda1
                @Override // java.lang.Runnable
                public final void run() {
                    NotificationsCustomSettingsActivity.SearchAdapter.this.m1830xb4878a20(query, contactsCopy);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* JADX WARN: Code restructure failed: missing block: B:64:0x0195, code lost:
            if (r9[0].contains(" " + r2) == false) goto L44;
         */
        /* JADX WARN: Removed duplicated region for block: B:101:0x01d2 A[SYNTHETIC] */
        /* JADX WARN: Removed duplicated region for block: B:88:0x021d A[LOOP:1: B:57:0x0163->B:88:0x021d, LOOP_END] */
        /* renamed from: lambda$processSearch$2$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity$SearchAdapter  reason: not valid java name */
        /*
            Code decompiled incorrectly, please refer to instructions dump.
        */
        public /* synthetic */ void m1830xb4878a20(String query, ArrayList contactsCopy) {
            String search1;
            char c;
            String tName;
            String search2;
            String[] search;
            int found;
            int b;
            String search12 = query.trim().toLowerCase();
            if (search12.length() == 0) {
                updateSearchResults(new ArrayList<>(), new ArrayList<>(), new ArrayList<>());
                return;
            }
            String search22 = LocaleController.getInstance().getTranslitString(search12);
            search22 = (search12.equals(search22) || search22.length() == 0) ? null : null;
            String[] search3 = new String[(search22 != null ? 1 : 0) + 1];
            search3[0] = search12;
            if (search22 != null) {
                search3[1] = search22;
            }
            ArrayList<TLObject> resultArray = new ArrayList<>();
            ArrayList<NotificationsSettingsActivity.NotificationException> exceptionsArray = new ArrayList<>();
            ArrayList<CharSequence> resultArrayNames = new ArrayList<>();
            String[] names = new String[2];
            int a2 = 0;
            while (a2 < contactsCopy.size()) {
                NotificationsSettingsActivity.NotificationException exception = (NotificationsSettingsActivity.NotificationException) contactsCopy.get(a2);
                int lower_id = (int) exception.did;
                int b2 = (int) (exception.did >> 32);
                TLObject object = null;
                if (lower_id != 0) {
                    if (lower_id > 0) {
                        TLRPC.User user = NotificationsCustomSettingsActivity.this.getMessagesController().getUser(Integer.valueOf(lower_id));
                        if (user.deleted) {
                            search1 = search12;
                            search2 = search22;
                            search = search3;
                        } else {
                            if (user == null) {
                                search1 = search12;
                            } else {
                                search1 = search12;
                                names[0] = ContactsController.formatName(user.first_name, user.last_name);
                                names[1] = user.username;
                                object = user;
                            }
                            c = 0;
                        }
                    } else {
                        search1 = search12;
                        TLRPC.Chat chat = NotificationsCustomSettingsActivity.this.getMessagesController().getChat(Integer.valueOf(-lower_id));
                        if (chat != null) {
                            if (chat.left || chat.kicked) {
                                search2 = search22;
                                search = search3;
                            } else if (chat.migrated_to != null) {
                                search2 = search22;
                                search = search3;
                            } else {
                                names[0] = chat.title;
                                names[1] = chat.username;
                                object = chat;
                            }
                        }
                        c = 0;
                    }
                    a2++;
                    search12 = search1;
                    search22 = search2;
                    search3 = search;
                } else {
                    search1 = search12;
                    TLRPC.EncryptedChat encryptedChat = NotificationsCustomSettingsActivity.this.getMessagesController().getEncryptedChat(Integer.valueOf(b2));
                    if (encryptedChat != null) {
                        TLRPC.User user2 = NotificationsCustomSettingsActivity.this.getMessagesController().getUser(Integer.valueOf(encryptedChat.user_id));
                        if (user2 != null) {
                            c = 0;
                            names[0] = ContactsController.formatName(user2.first_name, user2.last_name);
                            names[1] = user2.username;
                        } else {
                            c = 0;
                        }
                    } else {
                        c = 0;
                    }
                }
                String originalName = names[c];
                names[c] = names[c].toLowerCase();
                String tName2 = LocaleController.getInstance().getTranslitString(names[c]);
                if (names[c] != null && names[c].equals(tName2)) {
                    tName = null;
                } else {
                    tName = tName2;
                }
                int found2 = 0;
                int found3 = 0;
                while (true) {
                    search2 = search22;
                    if (found3 >= search3.length) {
                        search = search3;
                        break;
                    }
                    String q = search3[found3];
                    int high_id = b2;
                    if (names[0] != null) {
                        search = search3;
                        if (!names[0].startsWith(q)) {
                        }
                        found = 1;
                        b = found3;
                        if (found != 0) {
                            found2 = found;
                            found3 = b + 1;
                            search22 = search2;
                            search3 = search;
                            b2 = high_id;
                            originalName = originalName;
                        } else {
                            if (found == 1) {
                                resultArrayNames.add(AndroidUtilities.generateSearchName(originalName, null, q));
                            } else {
                                resultArrayNames.add(AndroidUtilities.generateSearchName("@" + names[1], null, "@" + q));
                            }
                            exceptionsArray.add(exception);
                            if (object != null) {
                                resultArray.add(object);
                            }
                        }
                    } else {
                        search = search3;
                    }
                    if (tName == null || (!tName.startsWith(q) && !tName.contains(" " + q))) {
                        if (names[1] != null) {
                            b = found3;
                            if (names[1].startsWith(q)) {
                                found = 2;
                                if (found != 0) {
                                }
                            }
                        } else {
                            b = found3;
                        }
                        found = found2;
                        if (found != 0) {
                        }
                    }
                    found = 1;
                    b = found3;
                    if (found != 0) {
                    }
                }
                a2++;
                search12 = search1;
                search22 = search2;
                search3 = search;
            }
            updateSearchResults(resultArray, exceptionsArray, resultArrayNames);
        }

        private void updateSearchResults(final ArrayList<TLObject> result, final ArrayList<NotificationsSettingsActivity.NotificationException> exceptions, final ArrayList<CharSequence> names) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$SearchAdapter$$ExternalSyntheticLambda2
                @Override // java.lang.Runnable
                public final void run() {
                    NotificationsCustomSettingsActivity.SearchAdapter.this.m1833x59f4dff0(exceptions, names, result);
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$updateSearchResults$4$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity$SearchAdapter  reason: not valid java name */
        public /* synthetic */ void m1833x59f4dff0(ArrayList exceptions, ArrayList names, ArrayList result) {
            this.searchRunnable = null;
            this.searchResult = exceptions;
            this.searchResultNames = names;
            this.searchAdapterHelper.mergeResults(result);
            if (NotificationsCustomSettingsActivity.this.searching && !this.searchAdapterHelper.isSearchInProgress()) {
                NotificationsCustomSettingsActivity.this.emptyView.showTextView();
            }
            notifyDataSetChanged();
        }

        public Object getObject(int position) {
            if (position >= 0 && position < this.searchResult.size()) {
                return this.searchResult.get(position);
            }
            int position2 = position - (this.searchResult.size() + 1);
            ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
            if (position2 >= 0 && position2 < globalSearch.size()) {
                return this.searchAdapterHelper.getGlobalSearch().get(position2);
            }
            return null;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            return true;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            int count = this.searchResult.size();
            ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
            if (!globalSearch.isEmpty()) {
                return count + globalSearch.size() + 1;
            }
            return count;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view;
            switch (viewType) {
                case 0:
                    view = new UserCell(this.mContext, 4, 0, false, true);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                default:
                    view = new GraySectionCell(this.mContext);
                    break;
            }
            return new RecyclerListView.Holder(view);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            switch (holder.getItemViewType()) {
                case 0:
                    UserCell cell = (UserCell) holder.itemView;
                    if (position < this.searchResult.size()) {
                        cell.setException(this.searchResult.get(position), this.searchResultNames.get(position), position != this.searchResult.size() - 1);
                        cell.setAddButtonVisible(false);
                        return;
                    }
                    int position2 = position - (this.searchResult.size() + 1);
                    ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
                    TLObject object = globalSearch.get(position2);
                    cell.setData(object, null, LocaleController.getString("NotificationsOn", R.string.NotificationsOn), 0, position2 != globalSearch.size() - 1);
                    cell.setAddButtonVisible(true);
                    return;
                case 1:
                    ((GraySectionCell) holder.itemView).setText(LocaleController.getString("AddToExceptions", R.string.AddToExceptions));
                    return;
                default:
                    return;
            }
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemViewType(int position) {
            if (position == this.searchResult.size()) {
                return 1;
            }
            return 0;
        }
    }

    /* loaded from: classes12.dex */
    private class ListAdapter extends RecyclerListView.SelectionAdapter {
        private Context mContext;

        public ListAdapter(Context context) {
            this.mContext = context;
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            int type = holder.getItemViewType();
            return (type == 0 || type == 4) ? false : true;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            return NotificationsCustomSettingsActivity.this.rowCount;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view;
            switch (viewType) {
                case 0:
                    view = new HeaderCell(this.mContext);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                case 1:
                    view = new TextCheckCell(this.mContext);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                case 2:
                    view = new UserCell(this.mContext, 6, 0, false);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                case 3:
                    view = new TextColorCell(this.mContext);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                case 4:
                    view = new ShadowSectionCell(this.mContext);
                    break;
                case 5:
                    view = new TextSettingsCell(this.mContext);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                case 6:
                    view = new NotificationsCheckCell(this.mContext);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
                default:
                    view = new TextCell(this.mContext);
                    view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    break;
            }
            return new RecyclerListView.Holder(view);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            boolean enabled;
            int color;
            int option;
            String value;
            int value2;
            int value3;
            String value4;
            String text;
            int offUntil;
            int iconType;
            switch (holder.getItemViewType()) {
                case 0:
                    HeaderCell headerCell = (HeaderCell) holder.itemView;
                    if (position == NotificationsCustomSettingsActivity.this.messageSectionRow) {
                        headerCell.setText(LocaleController.getString("SETTINGS", R.string.SETTINGS));
                        return;
                    }
                    return;
                case 1:
                    TextCheckCell checkCell = (TextCheckCell) holder.itemView;
                    SharedPreferences preferences = NotificationsCustomSettingsActivity.this.getNotificationsSettings();
                    if (position == NotificationsCustomSettingsActivity.this.previewRow) {
                        if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                            if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                                enabled = preferences.getBoolean("EnablePreviewGroup", true);
                            } else {
                                enabled = preferences.getBoolean("EnablePreviewChannel", true);
                            }
                        } else {
                            enabled = preferences.getBoolean("EnablePreviewAll", true);
                        }
                        checkCell.setTextAndCheck(LocaleController.getString("MessagePreview", R.string.MessagePreview), enabled, true);
                        return;
                    }
                    return;
                case 2:
                    UserCell cell = (UserCell) holder.itemView;
                    NotificationsSettingsActivity.NotificationException exception = (NotificationsSettingsActivity.NotificationException) NotificationsCustomSettingsActivity.this.exceptions.get(position - NotificationsCustomSettingsActivity.this.exceptionsStartRow);
                    cell.setException(exception, null, position != NotificationsCustomSettingsActivity.this.exceptionsEndRow - 1);
                    return;
                case 3:
                    TextColorCell textColorCell = (TextColorCell) holder.itemView;
                    SharedPreferences preferences2 = NotificationsCustomSettingsActivity.this.getNotificationsSettings();
                    if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                        if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                            color = preferences2.getInt("GroupLed", -16776961);
                        } else {
                            color = preferences2.getInt("ChannelLed", -16776961);
                        }
                    } else {
                        color = preferences2.getInt("MessagesLed", -16776961);
                    }
                    int a2 = 0;
                    while (true) {
                        if (a2 < 9) {
                            if (TextColorCell.colorsToSave[a2] != color) {
                                a2++;
                            } else {
                                color = TextColorCell.colors[a2];
                            }
                        }
                    }
                    textColorCell.setTextAndColor(LocaleController.getString("LedColor", R.string.LedColor), color, true);
                    return;
                case 4:
                    if (position == NotificationsCustomSettingsActivity.this.deleteAllSectionRow || ((position == NotificationsCustomSettingsActivity.this.groupSection2Row && NotificationsCustomSettingsActivity.this.exceptionsSection2Row == -1) || (position == NotificationsCustomSettingsActivity.this.exceptionsSection2Row && NotificationsCustomSettingsActivity.this.deleteAllRow == -1))) {
                        holder.itemView.setBackgroundDrawable(Theme.getThemedDrawable(this.mContext, (int) R.drawable.greydivider_bottom, Theme.key_windowBackgroundGrayShadow));
                        return;
                    } else {
                        holder.itemView.setBackgroundDrawable(Theme.getThemedDrawable(this.mContext, (int) R.drawable.greydivider, Theme.key_windowBackgroundGrayShadow));
                        return;
                    }
                case 5:
                    TextSettingsCell textCell = (TextSettingsCell) holder.itemView;
                    SharedPreferences preferences3 = NotificationsCustomSettingsActivity.this.getNotificationsSettings();
                    if (position == NotificationsCustomSettingsActivity.this.messageSoundRow) {
                        if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                            if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                                value4 = preferences3.getString("GroupSound", LocaleController.getString("SoundDefault", R.string.SoundDefault));
                            } else {
                                value4 = preferences3.getString("ChannelSound", LocaleController.getString("SoundDefault", R.string.SoundDefault));
                            }
                        } else {
                            value4 = preferences3.getString("GlobalSound", LocaleController.getString("SoundDefault", R.string.SoundDefault));
                        }
                        if (value4.equals("NoSound")) {
                            value4 = LocaleController.getString("NoSound", R.string.NoSound);
                        }
                        textCell.setTextAndValue(LocaleController.getString("Sound", R.string.Sound), value4, true);
                        return;
                    } else if (position == NotificationsCustomSettingsActivity.this.messageVibrateRow) {
                        if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                            if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                                value3 = preferences3.getInt("vibrate_group", 0);
                            } else {
                                value3 = preferences3.getInt("vibrate_channel", 0);
                            }
                        } else {
                            value3 = preferences3.getInt("vibrate_messages", 0);
                        }
                        if (value3 == 0) {
                            textCell.setTextAndValue(LocaleController.getString("Vibrate", R.string.Vibrate), LocaleController.getString("VibrationDefault", R.string.VibrationDefault), true);
                            return;
                        } else if (value3 == 1) {
                            textCell.setTextAndValue(LocaleController.getString("Vibrate", R.string.Vibrate), LocaleController.getString("Short", R.string.Short), true);
                            return;
                        } else if (value3 == 2) {
                            textCell.setTextAndValue(LocaleController.getString("Vibrate", R.string.Vibrate), LocaleController.getString("VibrationDisabled", R.string.VibrationDisabled), true);
                            return;
                        } else if (value3 == 3) {
                            textCell.setTextAndValue(LocaleController.getString("Vibrate", R.string.Vibrate), LocaleController.getString("Long", R.string.Long), true);
                            return;
                        } else if (value3 == 4) {
                            textCell.setTextAndValue(LocaleController.getString("Vibrate", R.string.Vibrate), LocaleController.getString("OnlyIfSilent", R.string.OnlyIfSilent), true);
                            return;
                        } else {
                            return;
                        }
                    } else if (position == NotificationsCustomSettingsActivity.this.messagePriorityRow) {
                        if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                            if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                                value2 = preferences3.getInt("priority_group", 1);
                            } else {
                                value2 = preferences3.getInt("priority_channel", 1);
                            }
                        } else {
                            value2 = preferences3.getInt("priority_messages", 1);
                        }
                        if (value2 == 0) {
                            textCell.setTextAndValue(LocaleController.getString("NotificationsImportance", R.string.NotificationsImportance), LocaleController.getString("NotificationsPriorityHigh", R.string.NotificationsPriorityHigh), true);
                            return;
                        } else if (value2 == 1 || value2 == 2) {
                            textCell.setTextAndValue(LocaleController.getString("NotificationsImportance", R.string.NotificationsImportance), LocaleController.getString("NotificationsPriorityUrgent", R.string.NotificationsPriorityUrgent), true);
                            return;
                        } else if (value2 == 4) {
                            textCell.setTextAndValue(LocaleController.getString("NotificationsImportance", R.string.NotificationsImportance), LocaleController.getString("NotificationsPriorityLow", R.string.NotificationsPriorityLow), true);
                            return;
                        } else if (value2 == 5) {
                            textCell.setTextAndValue(LocaleController.getString("NotificationsImportance", R.string.NotificationsImportance), LocaleController.getString("NotificationsPriorityMedium", R.string.NotificationsPriorityMedium), true);
                            return;
                        } else {
                            return;
                        }
                    } else if (position == NotificationsCustomSettingsActivity.this.messagePopupNotificationRow) {
                        if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                            if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                                option = preferences3.getInt("popupGroup", 0);
                            } else {
                                option = preferences3.getInt("popupChannel", 0);
                            }
                        } else {
                            option = preferences3.getInt("popupAll", 0);
                        }
                        if (option == 0) {
                            value = LocaleController.getString("NoPopup", R.string.NoPopup);
                        } else if (option == 1) {
                            value = LocaleController.getString("OnlyWhenScreenOn", R.string.OnlyWhenScreenOn);
                        } else if (option == 2) {
                            value = LocaleController.getString("OnlyWhenScreenOff", R.string.OnlyWhenScreenOff);
                        } else {
                            value = LocaleController.getString("AlwaysShowPopup", R.string.AlwaysShowPopup);
                        }
                        textCell.setTextAndValue(LocaleController.getString("PopupNotification", R.string.PopupNotification), value, true);
                        return;
                    } else {
                        return;
                    }
                case 6:
                    NotificationsCheckCell checkCell2 = (NotificationsCheckCell) holder.itemView;
                    checkCell2.setDrawLine(false);
                    StringBuilder builder = new StringBuilder();
                    SharedPreferences preferences4 = NotificationsCustomSettingsActivity.this.getNotificationsSettings();
                    if (NotificationsCustomSettingsActivity.this.currentType != 1) {
                        if (NotificationsCustomSettingsActivity.this.currentType == 0) {
                            String text2 = LocaleController.getString("NotificationsForGroups", R.string.NotificationsForGroups);
                            text = text2;
                            offUntil = preferences4.getInt("EnableGroup2", 0);
                        } else {
                            String text3 = LocaleController.getString("NotificationsForChannels", R.string.NotificationsForChannels);
                            text = text3;
                            offUntil = preferences4.getInt("EnableChannel2", 0);
                        }
                    } else {
                        String text4 = LocaleController.getString("NotificationsForPrivateChats", R.string.NotificationsForPrivateChats);
                        text = text4;
                        offUntil = preferences4.getInt("EnableAll2", 0);
                    }
                    int currentTime = NotificationsCustomSettingsActivity.this.getConnectionsManager().getCurrentTime();
                    boolean z = offUntil < currentTime;
                    boolean enabled2 = z;
                    if (z) {
                        builder.append(LocaleController.getString("NotificationsOn", R.string.NotificationsOn));
                        iconType = 0;
                    } else if (offUntil - 31536000 >= currentTime) {
                        builder.append(LocaleController.getString("NotificationsOff", R.string.NotificationsOff));
                        iconType = 0;
                    } else {
                        builder.append(LocaleController.formatString("NotificationsOffUntil", R.string.NotificationsOffUntil, LocaleController.stringForMessageListDate(offUntil)));
                        iconType = 2;
                    }
                    checkCell2.setTextAndValueAndCheck(text, builder, enabled2, iconType, false);
                    return;
                case 7:
                    TextCell textCell2 = (TextCell) holder.itemView;
                    if (position == NotificationsCustomSettingsActivity.this.exceptionsAddRow) {
                        textCell2.setTextAndIcon(LocaleController.getString("NotificationsAddAnException", R.string.NotificationsAddAnException), R.drawable.actions_addmember2, NotificationsCustomSettingsActivity.this.exceptionsStartRow != -1);
                        textCell2.setColors(Theme.key_windowBackgroundWhiteBlueIcon, Theme.key_windowBackgroundWhiteBlueButton);
                        return;
                    } else if (position == NotificationsCustomSettingsActivity.this.deleteAllRow) {
                        textCell2.setText(LocaleController.getString("NotificationsDeleteAllException", R.string.NotificationsDeleteAllException), false);
                        textCell2.setColors(null, Theme.key_windowBackgroundWhiteRedText5);
                        return;
                    } else {
                        return;
                    }
                default:
                    return;
            }
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onViewAttachedToWindow(RecyclerView.ViewHolder holder) {
            if (NotificationsCustomSettingsActivity.this.exceptions != null && NotificationsCustomSettingsActivity.this.exceptions.isEmpty()) {
                boolean enabled = NotificationsCustomSettingsActivity.this.getNotificationsController().isGlobalNotificationsEnabled(NotificationsCustomSettingsActivity.this.currentType);
                switch (holder.getItemViewType()) {
                    case 0:
                        HeaderCell headerCell = (HeaderCell) holder.itemView;
                        if (holder.getAdapterPosition() == NotificationsCustomSettingsActivity.this.messageSectionRow) {
                            headerCell.setEnabled(enabled, null);
                            return;
                        } else {
                            headerCell.setEnabled(true, null);
                            return;
                        }
                    case 1:
                        TextCheckCell textCell = (TextCheckCell) holder.itemView;
                        textCell.setEnabled(enabled, null);
                        return;
                    case 2:
                    case 4:
                    default:
                        return;
                    case 3:
                        TextColorCell textCell2 = (TextColorCell) holder.itemView;
                        textCell2.setEnabled(enabled, null);
                        return;
                    case 5:
                        TextSettingsCell textCell3 = (TextSettingsCell) holder.itemView;
                        textCell3.setEnabled(enabled, null);
                        return;
                }
            }
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemViewType(int position) {
            if (position != NotificationsCustomSettingsActivity.this.messageSectionRow) {
                if (position != NotificationsCustomSettingsActivity.this.previewRow) {
                    if (position < NotificationsCustomSettingsActivity.this.exceptionsStartRow || position >= NotificationsCustomSettingsActivity.this.exceptionsEndRow) {
                        if (position != NotificationsCustomSettingsActivity.this.messageLedRow) {
                            if (position != NotificationsCustomSettingsActivity.this.groupSection2Row && position != NotificationsCustomSettingsActivity.this.alertSection2Row && position != NotificationsCustomSettingsActivity.this.exceptionsSection2Row && position != NotificationsCustomSettingsActivity.this.deleteAllSectionRow) {
                                if (position != NotificationsCustomSettingsActivity.this.alertRow) {
                                    if (position == NotificationsCustomSettingsActivity.this.exceptionsAddRow || position == NotificationsCustomSettingsActivity.this.deleteAllRow) {
                                        return 7;
                                    }
                                    return 5;
                                }
                                return 6;
                            }
                            return 4;
                        }
                        return 3;
                    }
                    return 2;
                }
                return 1;
            }
            return 0;
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public ThemeDescription[] getThemeDescriptions() {
        ThemeDescription.ThemeDescriptionDelegate cellDelegate = new ThemeDescription.ThemeDescriptionDelegate() { // from class: im.uecahxyvjd.ui.NotificationsCustomSettingsActivity$$ExternalSyntheticLambda5
            @Override // im.uecahxyvjd.ui.actionbar.ThemeDescription.ThemeDescriptionDelegate
            public final void didSetColor() {
                NotificationsCustomSettingsActivity.this.m1826xc7644947();
            }
        };
        return new ThemeDescription[]{new ThemeDescription(this.listView, ThemeDescription.FLAG_CELLBACKGROUNDCOLOR, new Class[]{HeaderCell.class, TextCheckCell.class, TextColorCell.class, TextSettingsCell.class, UserCell.class, NotificationsCheckCell.class}, null, null, null, Theme.key_windowBackgroundWhite), new ThemeDescription(this.fragmentView, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_windowBackgroundGray), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.listView, ThemeDescription.FLAG_LISTGLOWCOLOR, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_ITEMSCOLOR, null, null, null, null, Theme.key_actionBarDefaultIcon), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_TITLECOLOR, null, null, null, null, Theme.key_actionBarDefaultTitle), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_SELECTORCOLOR, null, null, null, null, Theme.key_actionBarDefaultSelector), new ThemeDescription(this.listView, ThemeDescription.FLAG_SELECTOR, null, null, null, null, Theme.key_listSelector), new ThemeDescription(this.listView, 0, new Class[]{View.class}, Theme.dividerPaint, null, null, Theme.key_divider), new ThemeDescription(this.listView, 0, new Class[]{HeaderCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlueHeader), new ThemeDescription(this.listView, 0, new Class[]{TextCheckCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, 0, new Class[]{TextCheckCell.class}, new String[]{"valueTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText2), new ThemeDescription(this.listView, 0, new Class[]{TextCheckCell.class}, new String[]{"checkBox"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_switchTrack), new ThemeDescription(this.listView, 0, new Class[]{TextCheckCell.class}, new String[]{"checkBox"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_switchTrackChecked), new ThemeDescription(this.listView, 0, new Class[]{UserCell.class}, new String[]{"imageView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayIcon), new ThemeDescription(this.listView, 0, new Class[]{UserCell.class}, new String[]{"nameTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, 0, new Class[]{UserCell.class}, new String[]{"statusColor"}, (Paint[]) null, (Drawable[]) null, cellDelegate, Theme.key_windowBackgroundWhiteGrayText), new ThemeDescription(this.listView, 0, new Class[]{UserCell.class}, new String[]{"statusOnlineColor"}, (Paint[]) null, (Drawable[]) null, cellDelegate, Theme.key_windowBackgroundWhiteBlueText), new ThemeDescription(this.listView, 0, new Class[]{UserCell.class}, null, new Drawable[]{Theme.avatar_savedDrawable}, null, Theme.key_avatar_text), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundRed), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundOrange), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundViolet), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundGreen), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundCyan), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundBlue), new ThemeDescription(null, 0, null, null, null, cellDelegate, Theme.key_avatar_backgroundPink), new ThemeDescription(this.listView, 0, new Class[]{GraySectionCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_graySectionText), new ThemeDescription(this.listView, ThemeDescription.FLAG_CELLBACKGROUNDCOLOR, new Class[]{GraySectionCell.class}, null, null, null, Theme.key_graySection), new ThemeDescription(this.listView, 0, new Class[]{NotificationsCheckCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, 0, new Class[]{NotificationsCheckCell.class}, new String[]{"valueTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText2), new ThemeDescription(this.listView, 0, new Class[]{NotificationsCheckCell.class}, new String[]{"checkBox"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_switchTrack), new ThemeDescription(this.listView, 0, new Class[]{NotificationsCheckCell.class}, new String[]{"checkBox"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_switchTrackChecked), new ThemeDescription(this.listView, 0, new Class[]{TextColorCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, 0, new Class[]{TextSettingsCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, 0, new Class[]{TextSettingsCell.class}, new String[]{"valueTextView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteValueText), new ThemeDescription(this.listView, ThemeDescription.FLAG_BACKGROUNDFILTER, new Class[]{ShadowSectionCell.class}, null, null, null, Theme.key_windowBackgroundGrayShadow), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{TextCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlueButton), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{TextCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteRedText5), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{TextCell.class}, new String[]{"imageView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlueIcon)};
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getThemeDescriptions$12$im-uecahxyvjd-ui-NotificationsCustomSettingsActivity  reason: not valid java name */
    public /* synthetic */ void m1826xc7644947() {
        RecyclerListView recyclerListView = this.listView;
        if (recyclerListView != null) {
            int count = recyclerListView.getChildCount();
            for (int a2 = 0; a2 < count; a2++) {
                View child = this.listView.getChildAt(a2);
                if (child instanceof UserCell) {
                    ((UserCell) child).update(0);
                }
            }
        }
    }
}
