package im.uecahxyvjd.ui;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.PollCreateActivity;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.ActionBarMenu;
import im.uecahxyvjd.ui.actionbar.ActionBarMenuItem;
import im.uecahxyvjd.ui.actionbar.AlertDialog;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.SimpleTextView;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.actionbar.ThemeDescription;
import im.uecahxyvjd.ui.cells.HeaderCell;
import im.uecahxyvjd.ui.cells.PollEditTextCell;
import im.uecahxyvjd.ui.cells.ShadowSectionCell;
import im.uecahxyvjd.ui.cells.TextInfoPrivacyCell;
import im.uecahxyvjd.ui.cells.TextSettingsCell;
import im.uecahxyvjd.ui.components.AlertsCreator;
import im.uecahxyvjd.ui.components.ContextProgressView;
import im.uecahxyvjd.ui.components.EditTextBoldCursor;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.RecyclerListView;
/* loaded from: classes12.dex */
public class PollCreateActivity extends BaseFragment {
    private static final int MAX_ANSWER_LENGTH = 100;
    private static final int MAX_QUESTION_LENGTH = 255;
    private static final int done_button = 1;
    private int addAnswerRow;
    private int answerHeaderRow;
    private int answerSectionRow;
    private int answerStartRow;
    private PollCreateActivityDelegate delegate;
    private ActionBarMenuItem doneItem;
    private AnimatorSet doneItemAnimation;
    private ListAdapter listAdapter;
    private RecyclerListView listView;
    private ChatActivity parentFragment;
    private ContextProgressView progressView;
    private int questionHeaderRow;
    private int questionRow;
    private int questionSectionRow;
    private String questionString;
    private int rowCount;
    private String[] answers = new String[10];
    private int answersCount = 1;
    private int requestFieldFocusAtPosition = -1;

    /* loaded from: classes12.dex */
    public interface PollCreateActivityDelegate {
        void sendPoll(TLRPC.TL_messageMediaPoll tL_messageMediaPoll, boolean z, int i);
    }

    static /* synthetic */ int access$1410(PollCreateActivity x0) {
        int i = x0.answersCount;
        x0.answersCount = i - 1;
        return i;
    }

    /* loaded from: classes12.dex */
    public class TouchHelperCallback extends ItemTouchHelper.Callback {
        public TouchHelperCallback() {
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public boolean isLongPressDragEnabled() {
            return true;
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public int getMovementFlags(RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder) {
            if (viewHolder.getItemViewType() != 5) {
                return makeMovementFlags(0, 0);
            }
            return makeMovementFlags(3, 0);
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public boolean onMove(RecyclerView recyclerView, RecyclerView.ViewHolder source, RecyclerView.ViewHolder target) {
            if (source.getItemViewType() == target.getItemViewType()) {
                PollCreateActivity.this.listAdapter.swapElements(source.getAdapterPosition(), target.getAdapterPosition());
                return true;
            }
            return false;
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public void onChildDraw(Canvas c, RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder, float dX, float dY, int actionState, boolean isCurrentlyActive) {
            super.onChildDraw(c, recyclerView, viewHolder, dX, dY, actionState, isCurrentlyActive);
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public void onSelectedChanged(RecyclerView.ViewHolder viewHolder, int actionState) {
            if (actionState != 0) {
                PollCreateActivity.this.listView.cancelClickRunnables(false);
                viewHolder.itemView.setPressed(true);
            }
            super.onSelectedChanged(viewHolder, actionState);
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public void onSwiped(RecyclerView.ViewHolder viewHolder, int direction) {
        }

        @Override // androidx.recyclerview.widget.ItemTouchHelper.Callback
        public void clearView(RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder) {
            super.clearView(recyclerView, viewHolder);
            viewHolder.itemView.setPressed(false);
        }
    }

    public PollCreateActivity(ChatActivity chatActivity) {
        this.parentFragment = chatActivity;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onFragmentCreate() {
        super.onFragmentCreate();
        updateRows();
        return true;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public View createView(Context context) {
        this.actionBar.setBackButtonImage(R.mipmap.ic_back);
        this.actionBar.setTitle(LocaleController.getString("NewPoll", R.string.NewPoll));
        if (AndroidUtilities.isTablet()) {
            this.actionBar.setOccupyStatusBar(false);
        }
        this.actionBar.setAllowOverlayTitle(true);
        this.actionBar.setActionBarMenuOnItemClick(new AnonymousClass1());
        ActionBarMenu menu = this.actionBar.createMenu();
        this.doneItem = menu.addItemWithWidth(1, R.drawable.ic_done, AndroidUtilities.dp(56.0f), LocaleController.getString("Done", R.string.Done));
        ContextProgressView contextProgressView = new ContextProgressView(context, 1);
        this.progressView = contextProgressView;
        contextProgressView.setAlpha(0.0f);
        this.progressView.setScaleX(0.1f);
        this.progressView.setScaleY(0.1f);
        this.progressView.setVisibility(4);
        this.doneItem.addView(this.progressView, LayoutHelper.createFrame(-1, -1.0f));
        this.listAdapter = new ListAdapter(context);
        this.fragmentView = new FrameLayout(context);
        this.fragmentView.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundGray));
        FrameLayout frameLayout = (FrameLayout) this.fragmentView;
        RecyclerListView recyclerListView = new RecyclerListView(context) { // from class: im.uecahxyvjd.ui.PollCreateActivity.2
            @Override // androidx.recyclerview.widget.RecyclerView, android.view.ViewGroup, android.view.ViewParent
            public boolean requestChildRectangleOnScreen(View child, Rect rectangle, boolean immediate) {
                rectangle.bottom += AndroidUtilities.dp(60.0f);
                return super.requestChildRectangleOnScreen(child, rectangle, immediate);
            }

            /* JADX INFO: Access modifiers changed from: protected */
            @Override // im.uecahxyvjd.ui.components.RecyclerListView, androidx.recyclerview.widget.RecyclerView, android.view.View
            public void onMeasure(int widthSpec, int heightSpec) {
                super.onMeasure(widthSpec, heightSpec);
            }

            @Override // androidx.recyclerview.widget.RecyclerView, android.view.View, android.view.ViewParent
            public void requestLayout() {
                super.requestLayout();
            }
        };
        this.listView = recyclerListView;
        recyclerListView.setVerticalScrollBarEnabled(false);
        ((DefaultItemAnimator) this.listView.getItemAnimator()).setDelayAnimations(false);
        this.listView.setLayoutManager(new LinearLayoutManager(context, 1, false));
        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(new TouchHelperCallback());
        itemTouchHelper.attachToRecyclerView(this.listView);
        frameLayout.addView(this.listView, LayoutHelper.createFrame(-1, -1, 51));
        this.listView.setAdapter(this.listAdapter);
        this.listView.setOnItemClickListener(new RecyclerListView.OnItemClickListener() { // from class: im.uecahxyvjd.ui.PollCreateActivity$$ExternalSyntheticLambda1
            @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemClickListener
            public final void onItemClick(View view, int i) {
                PollCreateActivity.this.m2055lambda$createView$0$imuecahxyvjduiPollCreateActivity(view, i);
            }
        });
        checkDoneButton();
        return this.fragmentView;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: im.uecahxyvjd.ui.PollCreateActivity$1  reason: invalid class name */
    /* loaded from: classes12.dex */
    public class AnonymousClass1 extends ActionBar.ActionBarMenuOnItemClick {
        AnonymousClass1() {
        }

        @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
        public void onItemClick(int id) {
            if (id == -1) {
                if (PollCreateActivity.this.checkDiscard()) {
                    PollCreateActivity.this.finishFragment();
                }
            } else if (id == 1) {
                final TLRPC.TL_messageMediaPoll poll = new TLRPC.TL_messageMediaPoll();
                poll.poll = new TLRPC.TL_poll();
                TLRPC.TL_poll tL_poll = poll.poll;
                PollCreateActivity pollCreateActivity = PollCreateActivity.this;
                tL_poll.question = pollCreateActivity.getFixedString(pollCreateActivity.questionString);
                for (int a2 = 0; a2 < PollCreateActivity.this.answers.length; a2++) {
                    PollCreateActivity pollCreateActivity2 = PollCreateActivity.this;
                    if (!TextUtils.isEmpty(pollCreateActivity2.getFixedString(pollCreateActivity2.answers[a2]))) {
                        TLRPC.TL_pollAnswer answer = new TLRPC.TL_pollAnswer();
                        PollCreateActivity pollCreateActivity3 = PollCreateActivity.this;
                        answer.text = pollCreateActivity3.getFixedString(pollCreateActivity3.answers[a2]);
                        answer.option = new byte[1];
                        answer.option[0] = (byte) (poll.poll.answers.size() + 48);
                        poll.poll.answers.add(answer);
                    }
                }
                poll.results = new TLRPC.TL_pollResults();
                if (PollCreateActivity.this.parentFragment.isInScheduleMode()) {
                    AlertsCreator.createScheduleDatePickerDialog(PollCreateActivity.this.getParentActivity(), UserObject.isUserSelf(PollCreateActivity.this.parentFragment.getCurrentUser()), new AlertsCreator.ScheduleDatePickerDelegate() { // from class: im.uecahxyvjd.ui.PollCreateActivity$1$$ExternalSyntheticLambda0
                        @Override // im.uecahxyvjd.ui.components.AlertsCreator.ScheduleDatePickerDelegate
                        public final void didSelectDate(boolean z, int i) {
                            PollCreateActivity.AnonymousClass1.this.m2057lambda$onItemClick$0$imuecahxyvjduiPollCreateActivity$1(poll, z, i);
                        }
                    });
                    return;
                }
                PollCreateActivity.this.delegate.sendPoll(poll, true, 0);
                PollCreateActivity.this.finishFragment();
            }
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$onItemClick$0$im-uecahxyvjd-ui-PollCreateActivity$1  reason: not valid java name */
        public /* synthetic */ void m2057lambda$onItemClick$0$imuecahxyvjduiPollCreateActivity$1(TLRPC.TL_messageMediaPoll poll, boolean notify, int scheduleDate) {
            PollCreateActivity.this.delegate.sendPoll(poll, notify, scheduleDate);
            PollCreateActivity.this.finishFragment();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createView$0$im-uecahxyvjd-ui-PollCreateActivity  reason: not valid java name */
    public /* synthetic */ void m2055lambda$createView$0$imuecahxyvjduiPollCreateActivity(View view, int position) {
        if (position == this.addAnswerRow) {
            addNewField();
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onResume() {
        super.onResume();
        ListAdapter listAdapter = this.listAdapter;
        if (listAdapter != null) {
            listAdapter.notifyDataSetChanged();
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onTransitionAnimationEnd(boolean isOpen, boolean backward) {
        if (isOpen) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.PollCreateActivity$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    PollCreateActivity.this.m2056xea106925();
                }
            }, 100L);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onTransitionAnimationEnd$1$im-uecahxyvjd-ui-PollCreateActivity  reason: not valid java name */
    public /* synthetic */ void m2056xea106925() {
        RecyclerView.ViewHolder holder;
        RecyclerListView recyclerListView = this.listView;
        if (recyclerListView != null && (holder = recyclerListView.findViewHolderForAdapterPosition(this.questionRow)) != null) {
            PollEditTextCell textCell = (PollEditTextCell) holder.itemView;
            EditTextBoldCursor editText = textCell.getTextView();
            editText.requestFocus();
            AndroidUtilities.showKeyboard(editText);
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public String getFixedString(String text) {
        if (TextUtils.isEmpty(text)) {
            return text;
        }
        String text2 = AndroidUtilities.getTrimmedString(text).toString();
        while (text2.contains("\n\n\n")) {
            text2 = text2.replace("\n\n\n", "\n\n");
        }
        while (text2.startsWith("\n\n\n")) {
            text2 = text2.replace("\n\n\n", "\n\n");
        }
        return text2;
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void checkDoneButton() {
        boolean enabled = true;
        if (TextUtils.isEmpty(getFixedString(this.questionString)) || this.questionString.length() > 255) {
            enabled = false;
        } else {
            int count = 0;
            int a2 = 0;
            while (true) {
                String[] strArr = this.answers;
                if (a2 >= strArr.length) {
                    break;
                }
                if (!TextUtils.isEmpty(getFixedString(strArr[a2]))) {
                    if (this.answers[a2].length() > 100) {
                        count = 0;
                        break;
                    }
                    count++;
                }
                a2++;
            }
            if (count < 2) {
                enabled = false;
            }
        }
        this.doneItem.setEnabled(enabled);
        this.doneItem.setAlpha(enabled ? 1.0f : 0.5f);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void updateRows() {
        this.rowCount = 0;
        int i = 0 + 1;
        this.rowCount = i;
        this.questionHeaderRow = 0;
        int i2 = i + 1;
        this.rowCount = i2;
        this.questionRow = i;
        int i3 = i2 + 1;
        this.rowCount = i3;
        this.questionSectionRow = i2;
        int i4 = i3 + 1;
        this.rowCount = i4;
        this.answerHeaderRow = i3;
        int i5 = this.answersCount;
        if (i5 != 0) {
            this.answerStartRow = i4;
            this.rowCount = i4 + i5;
        } else {
            this.answerStartRow = -1;
        }
        if (i5 != this.answers.length) {
            int i6 = this.rowCount;
            this.rowCount = i6 + 1;
            this.addAnswerRow = i6;
        } else {
            this.addAnswerRow = -1;
        }
        int i7 = this.rowCount;
        this.rowCount = i7 + 1;
        this.answerSectionRow = i7;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onBackPressed() {
        return checkDiscard();
    }

    /* JADX INFO: Access modifiers changed from: private */
    public boolean checkDiscard() {
        boolean allowDiscard = TextUtils.isEmpty(getFixedString(this.questionString));
        if (allowDiscard) {
            for (int a2 = 0; a2 < this.answersCount && (allowDiscard = TextUtils.isEmpty(getFixedString(this.answers[a2]))); a2++) {
            }
        }
        if (!allowDiscard) {
            AlertDialog.Builder builder = new AlertDialog.Builder(getParentActivity());
            builder.setTitle(LocaleController.getString("CancelPollAlertTitle", R.string.CancelPollAlertTitle));
            builder.setMessage(LocaleController.getString("CancelPollAlertText", R.string.CancelPollAlertText));
            builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.PollCreateActivity$$ExternalSyntheticLambda2
                @Override // android.content.DialogInterface.OnClickListener
                public final void onClick(DialogInterface dialogInterface, int i) {
                    PollCreateActivity.this.m2054lambda$checkDiscard$2$imuecahxyvjduiPollCreateActivity(dialogInterface, i);
                }
            });
            builder.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
            showDialog(builder.create());
        }
        return allowDiscard;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$checkDiscard$2$im-uecahxyvjd-ui-PollCreateActivity  reason: not valid java name */
    public /* synthetic */ void m2054lambda$checkDiscard$2$imuecahxyvjduiPollCreateActivity(DialogInterface dialogInterface, int i) {
        finishFragment();
    }

    public void setDelegate(PollCreateActivityDelegate pollCreateActivityDelegate) {
        this.delegate = pollCreateActivityDelegate;
    }

    private void showEditDoneProgress(final boolean show) {
        AnimatorSet animatorSet = this.doneItemAnimation;
        if (animatorSet != null) {
            animatorSet.cancel();
        }
        this.doneItemAnimation = new AnimatorSet();
        if (show) {
            this.progressView.setVisibility(0);
            this.doneItem.setEnabled(false);
            this.doneItemAnimation.playTogether(ObjectAnimator.ofFloat(this.doneItem.getContentView(), View.SCALE_X, 0.1f), ObjectAnimator.ofFloat(this.doneItem.getContentView(), View.SCALE_Y, 0.1f), ObjectAnimator.ofFloat(this.doneItem.getContentView(), View.ALPHA, 0.0f), ObjectAnimator.ofFloat(this.progressView, View.SCALE_X, 1.0f), ObjectAnimator.ofFloat(this.progressView, View.SCALE_Y, 1.0f), ObjectAnimator.ofFloat(this.progressView, View.ALPHA, 1.0f));
        } else {
            this.doneItem.getContentView().setVisibility(0);
            this.doneItem.setEnabled(true);
            this.doneItemAnimation.playTogether(ObjectAnimator.ofFloat(this.progressView, View.SCALE_X, 0.1f), ObjectAnimator.ofFloat(this.progressView, View.SCALE_Y, 0.1f), ObjectAnimator.ofFloat(this.progressView, View.ALPHA, 0.0f), ObjectAnimator.ofFloat(this.doneItem.getContentView(), View.SCALE_X, 1.0f), ObjectAnimator.ofFloat(this.doneItem.getContentView(), View.SCALE_Y, 1.0f), ObjectAnimator.ofFloat(this.doneItem.getContentView(), View.ALPHA, 1.0f));
        }
        this.doneItemAnimation.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.PollCreateActivity.3
            @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
            public void onAnimationEnd(Animator animation) {
                if (PollCreateActivity.this.doneItemAnimation != null && PollCreateActivity.this.doneItemAnimation.equals(animation)) {
                    if (!show) {
                        PollCreateActivity.this.progressView.setVisibility(4);
                    } else {
                        PollCreateActivity.this.doneItem.getContentView().setVisibility(4);
                    }
                }
            }

            @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
            public void onAnimationCancel(Animator animation) {
                if (PollCreateActivity.this.doneItemAnimation != null && PollCreateActivity.this.doneItemAnimation.equals(animation)) {
                    PollCreateActivity.this.doneItemAnimation = null;
                }
            }
        });
        this.doneItemAnimation.setDuration(150L);
        this.doneItemAnimation.start();
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void setTextLeft(View cell, int index) {
        boolean z = cell instanceof HeaderCell;
        String key = Theme.key_windowBackgroundWhiteRedText5;
        if (z) {
            HeaderCell headerCell = (HeaderCell) cell;
            if (index == -1) {
                String str = this.questionString;
                int left = 255 - (str != null ? str.length() : 0);
                if (left <= 76.5f) {
                    headerCell.setText2(String.format("%d", Integer.valueOf(left)));
                    SimpleTextView textView = headerCell.getTextView2();
                    if (left >= 0) {
                        key = Theme.key_windowBackgroundWhiteGrayText3;
                    }
                    textView.setTextColor(Theme.getColor(key));
                    textView.setTag(key);
                    return;
                }
                headerCell.setText2("");
                return;
            }
            headerCell.setText2("");
        } else if ((cell instanceof PollEditTextCell) && index >= 0) {
            PollEditTextCell textCell = (PollEditTextCell) cell;
            String str2 = this.answers[index];
            int left2 = 100 - (str2 != null ? str2.length() : 0);
            if (left2 <= 30.0f) {
                textCell.setText2(String.format("%d", Integer.valueOf(left2)));
                SimpleTextView textView2 = textCell.getTextView2();
                if (left2 >= 0) {
                    key = Theme.key_windowBackgroundWhiteGrayText3;
                }
                textView2.setTextColor(Theme.getColor(key));
                textView2.setTag(key);
                return;
            }
            textCell.setText2("");
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void addNewField() {
        int i = this.answersCount + 1;
        this.answersCount = i;
        if (i == this.answers.length) {
            this.listAdapter.notifyItemRemoved(this.addAnswerRow);
        }
        this.listAdapter.notifyItemInserted(this.addAnswerRow);
        updateRows();
        this.requestFieldFocusAtPosition = (this.answerStartRow + this.answersCount) - 1;
        this.listAdapter.notifyItemChanged(this.answerSectionRow);
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes12.dex */
    public class ListAdapter extends RecyclerListView.SelectionAdapter {
        private Context mContext;

        public ListAdapter(Context context) {
            this.mContext = context;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            return PollCreateActivity.this.rowCount;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            switch (holder.getItemViewType()) {
                case 0:
                    HeaderCell cell = (HeaderCell) holder.itemView;
                    if (position != PollCreateActivity.this.questionHeaderRow) {
                        if (position == PollCreateActivity.this.answerHeaderRow) {
                            cell.setText(LocaleController.getString("PollOptions", R.string.PollOptions));
                            return;
                        }
                        return;
                    }
                    cell.setText(LocaleController.getString("Question", R.string.Question));
                    return;
                case 1:
                default:
                    return;
                case 2:
                    TextInfoPrivacyCell cell2 = (TextInfoPrivacyCell) holder.itemView;
                    cell2.setBackgroundDrawable(Theme.getThemedDrawable(this.mContext, (int) R.drawable.greydivider_bottom, Theme.key_windowBackgroundGrayShadow));
                    if (10 - PollCreateActivity.this.answersCount <= 0) {
                        cell2.setText(LocaleController.getString("AddAnOptionInfoMax", R.string.AddAnOptionInfoMax));
                        return;
                    } else {
                        cell2.setText(LocaleController.formatString("AddAnOptionInfo", R.string.AddAnOptionInfo, LocaleController.formatPluralString("Option", 10 - PollCreateActivity.this.answersCount)));
                        return;
                    }
                case 3:
                    TextSettingsCell textCell = (TextSettingsCell) holder.itemView;
                    textCell.setTextColor(Theme.getColor(Theme.key_windowBackgroundWhiteHintText));
                    textCell.setText(LocaleController.getString("AddAnOption", R.string.AddAnOption), false);
                    return;
                case 4:
                    PollEditTextCell textCell2 = (PollEditTextCell) holder.itemView;
                    textCell2.setTag(1);
                    textCell2.setTextAndHint(PollCreateActivity.this.questionString != null ? PollCreateActivity.this.questionString : "", LocaleController.getString("QuestionHint", R.string.QuestionHint), false);
                    textCell2.setTag(null);
                    return;
                case 5:
                    PollEditTextCell textCell3 = (PollEditTextCell) holder.itemView;
                    textCell3.setTag(1);
                    int index = position - PollCreateActivity.this.answerStartRow;
                    textCell3.setTextAndHint(PollCreateActivity.this.answers[index], LocaleController.getString("OptionHint", R.string.OptionHint), true);
                    textCell3.setTag(null);
                    if (PollCreateActivity.this.requestFieldFocusAtPosition == position) {
                        EditTextBoldCursor editText = textCell3.getTextView();
                        editText.requestFocus();
                        AndroidUtilities.showKeyboard(editText);
                        PollCreateActivity.this.requestFieldFocusAtPosition = -1;
                    }
                    PollCreateActivity.this.setTextLeft(holder.itemView, position - PollCreateActivity.this.answerStartRow);
                    return;
            }
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onViewAttachedToWindow(RecyclerView.ViewHolder holder) {
            int viewType = holder.getItemViewType();
            if (viewType == 0 || viewType == 5) {
                PollCreateActivity.this.setTextLeft(holder.itemView, holder.getAdapterPosition() == PollCreateActivity.this.questionHeaderRow ? -1 : 0);
            }
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            return holder.getAdapterPosition() == PollCreateActivity.this.addAnswerRow;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view;
            switch (viewType) {
                case 0:
                    View view2 = new HeaderCell(this.mContext, false, 21, 15, true);
                    view2.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    view = view2;
                    break;
                case 1:
                    view = new ShadowSectionCell(this.mContext);
                    break;
                case 2:
                    view = new TextInfoPrivacyCell(this.mContext);
                    break;
                case 3:
                    View view3 = new TextSettingsCell(this.mContext);
                    view3.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    view = view3;
                    break;
                case 4:
                    final PollEditTextCell cell = new PollEditTextCell(this.mContext, null);
                    cell.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    cell.addTextWatcher(new TextWatcher() { // from class: im.uecahxyvjd.ui.PollCreateActivity.ListAdapter.1
                        @Override // android.text.TextWatcher
                        public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                        }

                        @Override // android.text.TextWatcher
                        public void onTextChanged(CharSequence s, int start, int before, int count) {
                        }

                        @Override // android.text.TextWatcher
                        public void afterTextChanged(Editable s) {
                            if (cell.getTag() != null) {
                                return;
                            }
                            PollCreateActivity.this.questionString = s.toString();
                            RecyclerView.ViewHolder holder = PollCreateActivity.this.listView.findViewHolderForAdapterPosition(PollCreateActivity.this.questionHeaderRow);
                            if (holder != null) {
                                PollCreateActivity.this.setTextLeft(holder.itemView, -1);
                            }
                            PollCreateActivity.this.checkDoneButton();
                        }
                    });
                    view = cell;
                    break;
                default:
                    final PollEditTextCell cell2 = new PollEditTextCell(this.mContext, new View.OnClickListener() { // from class: im.uecahxyvjd.ui.PollCreateActivity$ListAdapter$$ExternalSyntheticLambda0
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view4) {
                            PollCreateActivity.ListAdapter.this.m2058xfac6fe92(view4);
                        }
                    }) { // from class: im.uecahxyvjd.ui.PollCreateActivity.ListAdapter.2
                        @Override // im.uecahxyvjd.ui.cells.PollEditTextCell
                        protected boolean drawDivider() {
                            RecyclerView.ViewHolder holder = PollCreateActivity.this.listView.findContainingViewHolder(this);
                            if (holder != null) {
                                int position = holder.getAdapterPosition();
                                if (PollCreateActivity.this.answersCount == 10 && position == (PollCreateActivity.this.answerStartRow + PollCreateActivity.this.answersCount) - 1) {
                                    return false;
                                }
                            }
                            return true;
                        }
                    };
                    cell2.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                    cell2.addTextWatcher(new TextWatcher() { // from class: im.uecahxyvjd.ui.PollCreateActivity.ListAdapter.3
                        @Override // android.text.TextWatcher
                        public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                        }

                        @Override // android.text.TextWatcher
                        public void onTextChanged(CharSequence s, int start, int before, int count) {
                        }

                        @Override // android.text.TextWatcher
                        public void afterTextChanged(Editable s) {
                            RecyclerView.ViewHolder holder = PollCreateActivity.this.listView.findContainingViewHolder(cell2);
                            if (holder != null) {
                                int position = holder.getAdapterPosition();
                                int index = position - PollCreateActivity.this.answerStartRow;
                                if (index >= 0 && index < PollCreateActivity.this.answers.length) {
                                    PollCreateActivity.this.answers[index] = s.toString();
                                    PollCreateActivity.this.setTextLeft(cell2, index);
                                    PollCreateActivity.this.checkDoneButton();
                                }
                            }
                        }
                    });
                    cell2.setShowNextButton(true);
                    EditTextBoldCursor editText = cell2.getTextView();
                    editText.setImeOptions(editText.getImeOptions() | 5);
                    editText.setOnEditorActionListener(new TextView.OnEditorActionListener() { // from class: im.uecahxyvjd.ui.PollCreateActivity$ListAdapter$$ExternalSyntheticLambda1
                        @Override // android.widget.TextView.OnEditorActionListener
                        public final boolean onEditorAction(TextView textView, int i, KeyEvent keyEvent) {
                            return PollCreateActivity.ListAdapter.this.m2059xfa509893(cell2, textView, i, keyEvent);
                        }
                    });
                    editText.setOnKeyListener(new View.OnKeyListener() { // from class: im.uecahxyvjd.ui.PollCreateActivity$ListAdapter$$ExternalSyntheticLambda2
                        @Override // android.view.View.OnKeyListener
                        public final boolean onKey(View view4, int i, KeyEvent keyEvent) {
                            return PollCreateActivity.ListAdapter.lambda$onCreateViewHolder$2(PollEditTextCell.this, view4, i, keyEvent);
                        }
                    });
                    view = cell2;
                    break;
            }
            view.setLayoutParams(new RecyclerView.LayoutParams(-1, -2));
            return new RecyclerListView.Holder(view);
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$onCreateViewHolder$0$im-uecahxyvjd-ui-PollCreateActivity$ListAdapter  reason: not valid java name */
        public /* synthetic */ void m2058xfac6fe92(View v) {
            if (v.getTag() != null) {
                return;
            }
            v.setTag(1);
            RecyclerView.ViewHolder holder = PollCreateActivity.this.listView.findContainingViewHolder((View) v.getParent());
            if (holder != null) {
                int position = holder.getAdapterPosition();
                int index = position - PollCreateActivity.this.answerStartRow;
                PollCreateActivity.this.listAdapter.notifyItemRemoved(holder.getAdapterPosition());
                System.arraycopy(PollCreateActivity.this.answers, index + 1, PollCreateActivity.this.answers, index, (PollCreateActivity.this.answers.length - 1) - index);
                PollCreateActivity.this.answers[PollCreateActivity.this.answers.length - 1] = null;
                PollCreateActivity.access$1410(PollCreateActivity.this);
                if (PollCreateActivity.this.answersCount == PollCreateActivity.this.answers.length - 1) {
                    PollCreateActivity.this.listAdapter.notifyItemInserted((PollCreateActivity.this.answerStartRow + PollCreateActivity.this.answers.length) - 1);
                }
                RecyclerView.ViewHolder holder2 = PollCreateActivity.this.listView.findViewHolderForAdapterPosition(position - 1);
                if (holder2 != null && (holder2.itemView instanceof PollEditTextCell)) {
                    PollEditTextCell editTextCell = (PollEditTextCell) holder2.itemView;
                    editTextCell.getTextView().requestFocus();
                }
                PollCreateActivity.this.checkDoneButton();
                PollCreateActivity.this.updateRows();
                PollCreateActivity.this.listAdapter.notifyItemChanged(PollCreateActivity.this.answerSectionRow);
            }
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$onCreateViewHolder$1$im-uecahxyvjd-ui-PollCreateActivity$ListAdapter  reason: not valid java name */
        public /* synthetic */ boolean m2059xfa509893(PollEditTextCell cell, TextView v, int actionId, KeyEvent event) {
            if (actionId == 5) {
                RecyclerView.ViewHolder holder = PollCreateActivity.this.listView.findContainingViewHolder(cell);
                if (holder != null) {
                    int position = holder.getAdapterPosition();
                    int index = position - PollCreateActivity.this.answerStartRow;
                    if (index == PollCreateActivity.this.answersCount - 1 && PollCreateActivity.this.answersCount < 10) {
                        PollCreateActivity.this.addNewField();
                    } else if (index != PollCreateActivity.this.answersCount - 1) {
                        RecyclerView.ViewHolder holder2 = PollCreateActivity.this.listView.findViewHolderForAdapterPosition(position + 1);
                        if (holder2 != null && (holder2.itemView instanceof PollEditTextCell)) {
                            PollEditTextCell editTextCell = (PollEditTextCell) holder2.itemView;
                            editTextCell.getTextView().requestFocus();
                        }
                    } else {
                        AndroidUtilities.hideKeyboard(cell.getTextView());
                    }
                }
                return true;
            }
            return false;
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        public static /* synthetic */ boolean lambda$onCreateViewHolder$2(PollEditTextCell cell, View v, int keyCode, KeyEvent event) {
            EditTextBoldCursor field = (EditTextBoldCursor) v;
            if (keyCode == 67 && event.getAction() == 0 && field.length() == 0) {
                cell.callOnDelete();
                return true;
            }
            return false;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemViewType(int position) {
            if (position != PollCreateActivity.this.questionHeaderRow && position != PollCreateActivity.this.answerHeaderRow) {
                if (position != PollCreateActivity.this.questionSectionRow) {
                    if (position != PollCreateActivity.this.answerSectionRow) {
                        if (position != PollCreateActivity.this.addAnswerRow) {
                            if (position == PollCreateActivity.this.questionRow) {
                                return 4;
                            }
                            return 5;
                        }
                        return 3;
                    }
                    return 2;
                }
                return 1;
            }
            return 0;
        }

        public void swapElements(int fromIndex, int toIndex) {
            int idx1 = fromIndex - PollCreateActivity.this.answerStartRow;
            int idx2 = toIndex - PollCreateActivity.this.answerStartRow;
            if (idx1 >= 0 && idx2 >= 0 && idx1 < PollCreateActivity.this.answersCount && idx2 < PollCreateActivity.this.answersCount) {
                String from = PollCreateActivity.this.answers[idx1];
                PollCreateActivity.this.answers[idx1] = PollCreateActivity.this.answers[idx2];
                PollCreateActivity.this.answers[idx2] = from;
                notifyItemMoved(fromIndex, toIndex);
            }
        }
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public ThemeDescription[] getThemeDescriptions() {
        return new ThemeDescription[]{new ThemeDescription(this.listView, ThemeDescription.FLAG_CELLBACKGROUNDCOLOR, new Class[]{HeaderCell.class, TextSettingsCell.class, PollEditTextCell.class}, null, null, null, Theme.key_windowBackgroundWhite), new ThemeDescription(this.fragmentView, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_windowBackgroundGray), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_BACKGROUND, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.listView, ThemeDescription.FLAG_LISTGLOWCOLOR, null, null, null, null, Theme.key_actionBarDefault), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_ITEMSCOLOR, null, null, null, null, Theme.key_actionBarDefaultIcon), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_TITLECOLOR, null, null, null, null, Theme.key_actionBarDefaultTitle), new ThemeDescription(this.actionBar, ThemeDescription.FLAG_AB_SELECTORCOLOR, null, null, null, null, Theme.key_actionBarDefaultSelector), new ThemeDescription(this.listView, 0, new Class[]{HeaderCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlueHeader), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{HeaderCell.class}, new String[]{"textView2"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteRedText5), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{HeaderCell.class}, new String[]{"textView2"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText3), new ThemeDescription(this.listView, ThemeDescription.FLAG_TEXTCOLOR, new Class[]{PollEditTextCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteBlackText), new ThemeDescription(this.listView, ThemeDescription.FLAG_HINTTEXTCOLOR, new Class[]{PollEditTextCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteHintText), new ThemeDescription(this.listView, ThemeDescription.FLAG_HINTTEXTCOLOR, new Class[]{PollEditTextCell.class}, new String[]{"deleteImageView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText), new ThemeDescription(this.listView, ThemeDescription.FLAG_USEBACKGROUNDDRAWABLE | ThemeDescription.FLAG_DRAWABLESELECTEDSTATE, new Class[]{PollEditTextCell.class}, new String[]{"deleteImageView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_stickers_menuSelector), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{PollEditTextCell.class}, new String[]{"textView2"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteRedText5), new ThemeDescription(this.listView, ThemeDescription.FLAG_CHECKTAG, new Class[]{PollEditTextCell.class}, new String[]{"textView2"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteGrayText3), new ThemeDescription(this.listView, ThemeDescription.FLAG_SELECTOR, null, null, null, null, Theme.key_listSelector), new ThemeDescription(this.listView, 0, new Class[]{View.class}, Theme.dividerPaint, null, null, Theme.key_divider), new ThemeDescription(this.listView, 0, new Class[]{TextSettingsCell.class}, new String[]{"textView"}, (Paint[]) null, (Drawable[]) null, (ThemeDescription.ThemeDescriptionDelegate) null, Theme.key_windowBackgroundWhiteHintText)};
    }
}
