package im.uecahxyvjd.ui;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.SurfaceTexture;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.util.SparseArray;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.TextureView;
import android.view.VelocityTracker;
import android.view.View;
import android.view.WindowInsets;
import android.view.WindowManager;
import android.view.animation.DecelerateInterpolator;
import android.widget.FrameLayout;
import androidx.recyclerview.widget.ItemTouchHelper;
import com.google.android.exoplayer2.C;
import com.google.android.exoplayer2.ui.AspectRatioFrameLayout;
import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.FileLoader;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.ImageLocation;
import im.uecahxyvjd.messenger.ImageReceiver;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.PhotoViewer;
import im.uecahxyvjd.ui.SecretMediaViewer;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.Scroller;
import im.uecahxyvjd.ui.components.VideoPlayer;
import java.io.File;
import java.util.ArrayList;
/* loaded from: classes12.dex */
public class SecretMediaViewer implements NotificationCenter.NotificationCenterDelegate, GestureDetector.OnGestureListener, GestureDetector.OnDoubleTapListener {
    private static volatile SecretMediaViewer Instance = null;
    private ActionBar actionBar;
    private float animateToClipBottom;
    private float animateToClipHorizontal;
    private float animateToClipTop;
    private float animateToScale;
    private float animateToX;
    private float animateToY;
    private long animationStartTime;
    private float animationValue;
    private AspectRatioFrameLayout aspectRatioFrameLayout;
    private float clipBottom;
    private float clipHorizontal;
    private float clipTop;
    private long closeTime;
    private boolean closeVideoAfterWatch;
    private FrameLayoutDrawer containerView;
    private int currentAccount;
    private AnimatorSet currentActionBarAnimation;
    private int currentChannelId;
    private MessageObject currentMessageObject;
    private PhotoViewer.PhotoViewerProvider currentProvider;
    private int currentRotation;
    private ImageReceiver.BitmapHolder currentThumb;
    private boolean disableShowCheck;
    private boolean discardTap;
    private boolean doubleTap;
    private float dragY;
    private boolean draggingDown;
    private GestureDetector gestureDetector;
    private AnimatorSet imageMoveAnimation;
    private boolean invalidCoords;
    private boolean isPhotoVisible;
    private boolean isPlaying;
    private boolean isVideo;
    private boolean isVisible;
    private Object lastInsets;
    private float maxX;
    private float maxY;
    private float minX;
    private float minY;
    private float moveStartX;
    private float moveStartY;
    private boolean moving;
    private long openTime;
    private Activity parentActivity;
    private Runnable photoAnimationEndRunnable;
    private int photoAnimationInProgress;
    private long photoTransitionAnimationStartTime;
    private float pinchCenterX;
    private float pinchCenterY;
    private float pinchStartDistance;
    private float pinchStartX;
    private float pinchStartY;
    private int playerRetryPlayCount;
    private Scroller scroller;
    private SecretDeleteTimer secretDeleteTimer;
    private boolean textureUploaded;
    private float translationX;
    private float translationY;
    private boolean useOvershootForScale;
    private VelocityTracker velocityTracker;
    private float videoCrossfadeAlpha;
    private long videoCrossfadeAlphaLastTime;
    private boolean videoCrossfadeStarted;
    private VideoPlayer videoPlayer;
    private TextureView videoTextureView;
    private boolean videoWatchedOneTime;
    private WindowManager.LayoutParams windowLayoutParams;
    private FrameLayout windowView;
    private boolean zoomAnimation;
    private boolean zooming;
    private ImageReceiver centerImage = new ImageReceiver();
    private int[] coords = new int[2];
    private boolean isActionBarVisible = true;
    private PhotoBackgroundDrawable photoBackgroundDrawable = new PhotoBackgroundDrawable(-16777216);
    private Paint blackPaint = new Paint();
    private float scale = 1.0f;
    private DecelerateInterpolator interpolator = new DecelerateInterpolator(1.5f);
    private float pinchStartScale = 1.0f;
    private boolean canDragDown = true;

    static /* synthetic */ int access$1210(SecretMediaViewer x0) {
        int i = x0.playerRetryPlayCount;
        x0.playerRetryPlayCount = i - 1;
        return i;
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes12.dex */
    public class FrameLayoutDrawer extends FrameLayout {
        public FrameLayoutDrawer(Context context) {
            super(context);
            setWillNotDraw(false);
        }

        @Override // android.view.View
        public boolean onTouchEvent(MotionEvent event) {
            SecretMediaViewer.this.processTouchEvent(event);
            return true;
        }

        @Override // android.view.View
        protected void onDraw(Canvas canvas) {
            SecretMediaViewer.this.onDraw(canvas);
        }

        @Override // android.view.ViewGroup
        protected boolean drawChild(Canvas canvas, View child, long drawingTime) {
            return child != SecretMediaViewer.this.aspectRatioFrameLayout && super.drawChild(canvas, child, drawingTime);
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes12.dex */
    public class SecretDeleteTimer extends FrameLayout {
        private Paint afterDeleteProgressPaint;
        private Paint circlePaint;
        private Paint deleteProgressPaint;
        private RectF deleteProgressRect;
        private long destroyTime;
        private long destroyTtl;
        private Drawable drawable;
        private ArrayList<Particle> freeParticles;
        private long lastAnimationTime;
        private Paint particlePaint;
        private ArrayList<Particle> particles;
        private boolean useVideoProgress;

        /* JADX INFO: Access modifiers changed from: private */
        /* loaded from: classes12.dex */
        public class Particle {
            float alpha;
            float currentTime;
            float lifeTime;
            float velocity;
            float vx;
            float vy;
            float x;
            float y;

            private Particle() {
            }

            /* synthetic */ Particle(SecretDeleteTimer x0, AnonymousClass1 x1) {
                this();
            }
        }

        public SecretDeleteTimer(Context context) {
            super(context);
            this.deleteProgressRect = new RectF();
            this.particles = new ArrayList<>();
            this.freeParticles = new ArrayList<>();
            setWillNotDraw(false);
            Paint paint = new Paint(1);
            this.particlePaint = paint;
            paint.setStrokeWidth(AndroidUtilities.dp(1.5f));
            this.particlePaint.setColor(-1644826);
            this.particlePaint.setStrokeCap(Paint.Cap.ROUND);
            this.particlePaint.setStyle(Paint.Style.STROKE);
            Paint paint2 = new Paint(1);
            this.deleteProgressPaint = paint2;
            paint2.setColor(-1644826);
            Paint paint3 = new Paint(1);
            this.afterDeleteProgressPaint = paint3;
            paint3.setStyle(Paint.Style.STROKE);
            this.afterDeleteProgressPaint.setStrokeCap(Paint.Cap.ROUND);
            this.afterDeleteProgressPaint.setColor(-1644826);
            this.afterDeleteProgressPaint.setStrokeWidth(AndroidUtilities.dp(2.0f));
            Paint paint4 = new Paint(1);
            this.circlePaint = paint4;
            paint4.setColor(Theme.ACTION_BAR_PHOTO_VIEWER_COLOR);
            this.drawable = context.getResources().getDrawable(R.drawable.flame_small);
            for (int a2 = 0; a2 < 40; a2++) {
                this.freeParticles.add(new Particle(this, null));
            }
        }

        /* JADX INFO: Access modifiers changed from: private */
        public void setDestroyTime(long time, long ttl, boolean videoProgress) {
            this.destroyTime = time;
            this.destroyTtl = ttl;
            this.useVideoProgress = videoProgress;
            this.lastAnimationTime = System.currentTimeMillis();
            invalidate();
        }

        private void updateParticles(long dt) {
            int count = this.particles.size();
            int a2 = 0;
            while (a2 < count) {
                Particle particle = this.particles.get(a2);
                if (particle.currentTime >= particle.lifeTime) {
                    if (this.freeParticles.size() < 40) {
                        this.freeParticles.add(particle);
                    }
                    this.particles.remove(a2);
                    a2--;
                    count--;
                } else {
                    particle.alpha = 1.0f - AndroidUtilities.decelerateInterpolator.getInterpolation(particle.currentTime / particle.lifeTime);
                    particle.x += ((particle.vx * particle.velocity) * ((float) dt)) / 500.0f;
                    particle.y += ((particle.vy * particle.velocity) * ((float) dt)) / 500.0f;
                    particle.currentTime += (float) dt;
                }
                a2++;
            }
        }

        @Override // android.widget.FrameLayout, android.view.View
        protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
            super.onMeasure(widthMeasureSpec, heightMeasureSpec);
            int y = (getMeasuredHeight() / 2) - (AndroidUtilities.dp(28.0f) / 2);
            this.deleteProgressRect.set(getMeasuredWidth() - AndroidUtilities.dp(49.0f), y, getMeasuredWidth() - AndroidUtilities.dp(21.0f), AndroidUtilities.dp(28.0f) + y);
        }

        @Override // android.view.View
        protected void onDraw(Canvas canvas) {
            float progress;
            Particle newParticle;
            float progress2;
            if (SecretMediaViewer.this.currentMessageObject != null && SecretMediaViewer.this.currentMessageObject.messageOwner.destroyTime != 0) {
                canvas.drawCircle(getMeasuredWidth() - AndroidUtilities.dp(35.0f), getMeasuredHeight() / 2, AndroidUtilities.dp(16.0f), this.circlePaint);
                if (this.useVideoProgress) {
                    if (SecretMediaViewer.this.videoPlayer != null) {
                        long duration = SecretMediaViewer.this.videoPlayer.getDuration();
                        long position = SecretMediaViewer.this.videoPlayer.getCurrentPosition();
                        if (duration != C.TIME_UNSET && position != C.TIME_UNSET) {
                            progress2 = 1.0f - (((float) position) / ((float) duration));
                        } else {
                            progress2 = 1.0f;
                        }
                        progress = progress2;
                    } else {
                        progress = 1.0f;
                    }
                } else {
                    long msTime = System.currentTimeMillis() + (ConnectionsManager.getInstance(SecretMediaViewer.this.currentAccount).getTimeDifference() * 1000);
                    progress = ((float) Math.max(0L, this.destroyTime - msTime)) / (((float) this.destroyTtl) * 1000.0f);
                }
                int x = getMeasuredWidth() - AndroidUtilities.dp(40.0f);
                int y = ((getMeasuredHeight() - AndroidUtilities.dp(14.0f)) / 2) - AndroidUtilities.dp(0.5f);
                this.drawable.setBounds(x, y, AndroidUtilities.dp(10.0f) + x, AndroidUtilities.dp(14.0f) + y);
                this.drawable.draw(canvas);
                float radProgress = progress * (-360.0f);
                canvas.drawArc(this.deleteProgressRect, -90.0f, radProgress, false, this.afterDeleteProgressPaint);
                int count = this.particles.size();
                for (int a2 = 0; a2 < count; a2++) {
                    Particle particle = this.particles.get(a2);
                    this.particlePaint.setAlpha((int) (particle.alpha * 255.0f));
                    canvas.drawPoint(particle.x, particle.y, this.particlePaint);
                }
                double vx = Math.sin((radProgress - 90.0f) * 0.017453292519943295d);
                double vy = -Math.cos((radProgress - 90.0f) * 0.017453292519943295d);
                int rad = AndroidUtilities.dp(14.0f);
                float cx = (float) (((-vy) * rad) + this.deleteProgressRect.centerX());
                float cy = (float) ((rad * vx) + this.deleteProgressRect.centerY());
                int a3 = 0;
                while (a3 < 1) {
                    if (this.freeParticles.isEmpty()) {
                        newParticle = new Particle(this, null);
                    } else {
                        newParticle = this.freeParticles.get(0);
                        this.freeParticles.remove(0);
                    }
                    newParticle.x = cx;
                    newParticle.y = cy;
                    double angle = (Utilities.random.nextInt(140) - 70) * 0.017453292519943295d;
                    if (angle < FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE) {
                        angle += 6.283185307179586d;
                    }
                    newParticle.vx = (float) ((Math.cos(angle) * vx) - (Math.sin(angle) * vy));
                    newParticle.vy = (float) ((Math.sin(angle) * vx) + (Math.cos(angle) * vy));
                    newParticle.alpha = 1.0f;
                    newParticle.currentTime = 0.0f;
                    newParticle.lifeTime = Utilities.random.nextInt(100) + 400;
                    newParticle.velocity = (Utilities.random.nextFloat() * 4.0f) + 20.0f;
                    this.particles.add(newParticle);
                    a3++;
                    count = count;
                    rad = rad;
                }
                long newTime = System.currentTimeMillis();
                long dt = newTime - this.lastAnimationTime;
                updateParticles(dt);
                this.lastAnimationTime = newTime;
                invalidate();
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes12.dex */
    public class PhotoBackgroundDrawable extends ColorDrawable {
        private Runnable drawRunnable;
        private int frame;

        public PhotoBackgroundDrawable(int color) {
            super(color);
        }

        @Override // android.graphics.drawable.ColorDrawable, android.graphics.drawable.Drawable
        public void setAlpha(int alpha) {
            if (SecretMediaViewer.this.parentActivity instanceof LaunchActivity) {
                ((LaunchActivity) SecretMediaViewer.this.parentActivity).drawerLayoutContainer.setAllowDrawContent((SecretMediaViewer.this.isPhotoVisible && alpha == 255) ? false : true);
            }
            super.setAlpha(alpha);
        }

        @Override // android.graphics.drawable.ColorDrawable, android.graphics.drawable.Drawable
        public void draw(Canvas canvas) {
            Runnable runnable;
            super.draw(canvas);
            if (getAlpha() != 0) {
                if (this.frame == 2 && (runnable = this.drawRunnable) != null) {
                    runnable.run();
                    this.drawRunnable = null;
                } else {
                    invalidateSelf();
                }
                this.frame++;
            }
        }
    }

    public static SecretMediaViewer getInstance() {
        SecretMediaViewer localInstance = Instance;
        if (localInstance == null) {
            synchronized (PhotoViewer.class) {
                localInstance = Instance;
                if (localInstance == null) {
                    SecretMediaViewer secretMediaViewer = new SecretMediaViewer();
                    localInstance = secretMediaViewer;
                    Instance = secretMediaViewer;
                }
            }
        }
        return localInstance;
    }

    public static boolean hasInstance() {
        return Instance != null;
    }

    @Override // im.uecahxyvjd.messenger.NotificationCenter.NotificationCenterDelegate
    public void didReceivedNotification(int id, int account, Object... args) {
        if (id == NotificationCenter.messagesDeleted) {
            boolean scheduled = ((Boolean) args[2]).booleanValue();
            if (scheduled || this.currentMessageObject == null || ((Integer) args[1]).intValue() != 0) {
                return;
            }
            ArrayList<Integer> markAsDeletedMessages = (ArrayList) args[0];
            if (markAsDeletedMessages.contains(Integer.valueOf(this.currentMessageObject.getId()))) {
                if (this.isVideo && !this.videoWatchedOneTime) {
                    this.closeVideoAfterWatch = true;
                } else {
                    closePhoto(true, true);
                }
            }
        } else if (id == NotificationCenter.didCreatedNewDeleteTask) {
            if (this.currentMessageObject == null || this.secretDeleteTimer == null) {
                return;
            }
            SparseArray<ArrayList<Long>> mids = (SparseArray) args[0];
            for (int i = 0; i < mids.size(); i++) {
                int key = mids.keyAt(i);
                ArrayList<Long> arr = mids.get(key);
                for (int a2 = 0; a2 < arr.size(); a2++) {
                    long mid = arr.get(a2).longValue();
                    if (a2 == 0) {
                        int channelId = (int) (mid >> 32);
                        if (channelId < 0) {
                            channelId = 0;
                        }
                        if (channelId != this.currentChannelId) {
                            return;
                        }
                    }
                    if (this.currentMessageObject.getId() == mid) {
                        this.currentMessageObject.messageOwner.destroyTime = key;
                        this.secretDeleteTimer.invalidate();
                        return;
                    }
                }
            }
        } else if (id == NotificationCenter.updateMessageMedia) {
            TLRPC.Message message = (TLRPC.Message) args[0];
            if (this.currentMessageObject.getId() == message.id) {
                if (this.isVideo && !this.videoWatchedOneTime) {
                    this.closeVideoAfterWatch = true;
                } else {
                    closePhoto(true, true);
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void preparePlayer(File file) {
        if (this.parentActivity == null) {
            return;
        }
        releasePlayer();
        if (this.videoTextureView == null) {
            AspectRatioFrameLayout aspectRatioFrameLayout = new AspectRatioFrameLayout(this.parentActivity);
            this.aspectRatioFrameLayout = aspectRatioFrameLayout;
            aspectRatioFrameLayout.setVisibility(4);
            this.containerView.addView(this.aspectRatioFrameLayout, 0, LayoutHelper.createFrame(-1, -1, 17));
            TextureView textureView = new TextureView(this.parentActivity);
            this.videoTextureView = textureView;
            textureView.setOpaque(false);
            this.aspectRatioFrameLayout.addView(this.videoTextureView, LayoutHelper.createFrame(-1, -1, 17));
        }
        this.textureUploaded = false;
        this.videoCrossfadeStarted = false;
        TextureView textureView2 = this.videoTextureView;
        this.videoCrossfadeAlpha = 0.0f;
        textureView2.setAlpha(0.0f);
        if (this.videoPlayer == null) {
            VideoPlayer videoPlayer = new VideoPlayer();
            this.videoPlayer = videoPlayer;
            videoPlayer.setTextureView(this.videoTextureView);
            this.videoPlayer.setDelegate(new AnonymousClass1(file));
        }
        this.videoPlayer.preparePlayer(Uri.fromFile(file), "other");
        this.videoPlayer.setPlayWhenReady(true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: im.uecahxyvjd.ui.SecretMediaViewer$1  reason: invalid class name */
    /* loaded from: classes12.dex */
    public class AnonymousClass1 implements VideoPlayer.VideoPlayerDelegate {
        final /* synthetic */ File val$file;

        AnonymousClass1(File file) {
            this.val$file = file;
        }

        @Override // im.uecahxyvjd.ui.components.VideoPlayer.VideoPlayerDelegate
        public void onStateChanged(boolean playWhenReady, int playbackState) {
            if (SecretMediaViewer.this.videoPlayer == null || SecretMediaViewer.this.currentMessageObject == null) {
                return;
            }
            if (playbackState == 4 || playbackState == 1) {
                try {
                    SecretMediaViewer.this.parentActivity.getWindow().clearFlags(128);
                } catch (Exception e) {
                    FileLog.e(e);
                }
            } else {
                try {
                    SecretMediaViewer.this.parentActivity.getWindow().addFlags(128);
                } catch (Exception e2) {
                    FileLog.e(e2);
                }
            }
            if (playbackState == 3 && SecretMediaViewer.this.aspectRatioFrameLayout.getVisibility() != 0) {
                SecretMediaViewer.this.aspectRatioFrameLayout.setVisibility(0);
            }
            if (!SecretMediaViewer.this.videoPlayer.isPlaying() || playbackState == 4) {
                if (SecretMediaViewer.this.isPlaying) {
                    SecretMediaViewer.this.isPlaying = false;
                    if (playbackState == 4) {
                        SecretMediaViewer.this.videoWatchedOneTime = true;
                        if (!SecretMediaViewer.this.closeVideoAfterWatch) {
                            SecretMediaViewer.this.videoPlayer.seekTo(0L);
                            SecretMediaViewer.this.videoPlayer.play();
                            return;
                        }
                        SecretMediaViewer.this.closePhoto(true, true);
                    }
                }
            } else if (!SecretMediaViewer.this.isPlaying) {
                SecretMediaViewer.this.isPlaying = true;
            }
        }

        @Override // im.uecahxyvjd.ui.components.VideoPlayer.VideoPlayerDelegate
        public void onError(Exception e) {
            if (SecretMediaViewer.this.playerRetryPlayCount > 0) {
                SecretMediaViewer.access$1210(SecretMediaViewer.this);
                final File file = this.val$file;
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$1$$ExternalSyntheticLambda0
                    @Override // java.lang.Runnable
                    public final void run() {
                        SecretMediaViewer.AnonymousClass1.this.m2159lambda$onError$0$imuecahxyvjduiSecretMediaViewer$1(file);
                    }
                }, 100L);
                return;
            }
            FileLog.e(e);
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$onError$0$im-uecahxyvjd-ui-SecretMediaViewer$1  reason: not valid java name */
        public /* synthetic */ void m2159lambda$onError$0$imuecahxyvjduiSecretMediaViewer$1(File file) {
            SecretMediaViewer.this.preparePlayer(file);
        }

        @Override // im.uecahxyvjd.ui.components.VideoPlayer.VideoPlayerDelegate
        public void onVideoSizeChanged(int width, int height, int unappliedRotationDegrees, float pixelWidthHeightRatio) {
            if (SecretMediaViewer.this.aspectRatioFrameLayout != null) {
                if (unappliedRotationDegrees == 90 || unappliedRotationDegrees == 270) {
                    width = height;
                    height = width;
                }
                SecretMediaViewer.this.aspectRatioFrameLayout.setAspectRatio(height == 0 ? 1.0f : (width * pixelWidthHeightRatio) / height, unappliedRotationDegrees);
            }
        }

        @Override // im.uecahxyvjd.ui.components.VideoPlayer.VideoPlayerDelegate
        public void onRenderedFirstFrame() {
            if (!SecretMediaViewer.this.textureUploaded) {
                SecretMediaViewer.this.textureUploaded = true;
                SecretMediaViewer.this.containerView.invalidate();
            }
        }

        @Override // im.uecahxyvjd.ui.components.VideoPlayer.VideoPlayerDelegate
        public boolean onSurfaceDestroyed(SurfaceTexture surfaceTexture) {
            return false;
        }

        @Override // im.uecahxyvjd.ui.components.VideoPlayer.VideoPlayerDelegate
        public void onSurfaceTextureUpdated(SurfaceTexture surfaceTexture) {
        }
    }

    private void releasePlayer() {
        VideoPlayer videoPlayer = this.videoPlayer;
        if (videoPlayer != null) {
            this.playerRetryPlayCount = 0;
            videoPlayer.releasePlayer(true);
            this.videoPlayer = null;
        }
        try {
            Activity activity = this.parentActivity;
            if (activity != null) {
                activity.getWindow().clearFlags(128);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        AspectRatioFrameLayout aspectRatioFrameLayout = this.aspectRatioFrameLayout;
        if (aspectRatioFrameLayout != null) {
            this.containerView.removeView(aspectRatioFrameLayout);
            this.aspectRatioFrameLayout = null;
        }
        if (this.videoTextureView != null) {
            this.videoTextureView = null;
        }
        this.isPlaying = false;
    }

    public void setParentActivity(Activity activity) {
        int i = UserConfig.selectedAccount;
        this.currentAccount = i;
        this.centerImage.setCurrentAccount(i);
        if (this.parentActivity == activity) {
            return;
        }
        this.parentActivity = activity;
        this.scroller = new Scroller(activity);
        FrameLayout frameLayout = new FrameLayout(activity) { // from class: im.uecahxyvjd.ui.SecretMediaViewer.2
            @Override // android.widget.FrameLayout, android.view.View
            protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
                int widthSize = View.MeasureSpec.getSize(widthMeasureSpec);
                int heightSize = View.MeasureSpec.getSize(heightMeasureSpec);
                if (SecretMediaViewer.this.lastInsets != null) {
                    WindowInsets insets = (WindowInsets) SecretMediaViewer.this.lastInsets;
                    if (AndroidUtilities.incorrectDisplaySizeFix) {
                        if (heightSize > AndroidUtilities.displaySize.y) {
                            heightSize = AndroidUtilities.displaySize.y;
                        }
                        heightSize += AndroidUtilities.statusBarHeight;
                    }
                    heightSize -= insets.getSystemWindowInsetBottom();
                    widthSize -= insets.getSystemWindowInsetRight();
                } else if (heightSize > AndroidUtilities.displaySize.y) {
                    heightSize = AndroidUtilities.displaySize.y;
                }
                setMeasuredDimension(widthSize, heightSize);
                if (SecretMediaViewer.this.lastInsets != null) {
                    widthSize -= ((WindowInsets) SecretMediaViewer.this.lastInsets).getSystemWindowInsetLeft();
                }
                SecretMediaViewer.this.containerView.measure(View.MeasureSpec.makeMeasureSpec(widthSize, 1073741824), View.MeasureSpec.makeMeasureSpec(heightSize, 1073741824));
            }

            @Override // android.widget.FrameLayout, android.view.ViewGroup, android.view.View
            protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
                int x = SecretMediaViewer.this.lastInsets != null ? 0 + ((WindowInsets) SecretMediaViewer.this.lastInsets).getSystemWindowInsetLeft() : 0;
                SecretMediaViewer.this.containerView.layout(x, 0, SecretMediaViewer.this.containerView.getMeasuredWidth() + x, SecretMediaViewer.this.containerView.getMeasuredHeight());
                if (changed) {
                    if (SecretMediaViewer.this.imageMoveAnimation == null) {
                        SecretMediaViewer.this.scale = 1.0f;
                        SecretMediaViewer.this.translationX = 0.0f;
                        SecretMediaViewer.this.translationY = 0.0f;
                    }
                    SecretMediaViewer secretMediaViewer = SecretMediaViewer.this;
                    secretMediaViewer.updateMinMax(secretMediaViewer.scale);
                }
            }
        };
        this.windowView = frameLayout;
        frameLayout.setBackgroundDrawable(this.photoBackgroundDrawable);
        this.windowView.setFocusable(true);
        this.windowView.setFocusableInTouchMode(true);
        FrameLayoutDrawer frameLayoutDrawer = new FrameLayoutDrawer(activity) { // from class: im.uecahxyvjd.ui.SecretMediaViewer.3
            @Override // android.widget.FrameLayout, android.view.ViewGroup, android.view.View
            protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
                super.onLayout(changed, left, top, right, bottom);
                if (SecretMediaViewer.this.secretDeleteTimer != null) {
                    int y = ((ActionBar.getCurrentActionBarHeight() - SecretMediaViewer.this.secretDeleteTimer.getMeasuredHeight()) / 2) + AndroidUtilities.statusBarHeight;
                    SecretMediaViewer.this.secretDeleteTimer.layout(SecretMediaViewer.this.secretDeleteTimer.getLeft(), y, SecretMediaViewer.this.secretDeleteTimer.getRight(), SecretMediaViewer.this.secretDeleteTimer.getMeasuredHeight() + y);
                }
            }
        };
        this.containerView = frameLayoutDrawer;
        frameLayoutDrawer.setFocusable(false);
        this.windowView.addView(this.containerView);
        FrameLayout.LayoutParams layoutParams = (FrameLayout.LayoutParams) this.containerView.getLayoutParams();
        layoutParams.width = -1;
        layoutParams.height = -1;
        layoutParams.gravity = 51;
        this.containerView.setLayoutParams(layoutParams);
        this.containerView.setFitsSystemWindows(true);
        this.containerView.setOnApplyWindowInsetsListener(new View.OnApplyWindowInsetsListener() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$$ExternalSyntheticLambda0
            @Override // android.view.View.OnApplyWindowInsetsListener
            public final WindowInsets onApplyWindowInsets(View view, WindowInsets windowInsets) {
                return SecretMediaViewer.this.m2158lambda$setParentActivity$0$imuecahxyvjduiSecretMediaViewer(view, windowInsets);
            }
        });
        this.containerView.setSystemUiVisibility(1280);
        GestureDetector gestureDetector = new GestureDetector(this.containerView.getContext(), this);
        this.gestureDetector = gestureDetector;
        gestureDetector.setOnDoubleTapListener(this);
        ActionBar actionBar = new ActionBar(activity);
        this.actionBar = actionBar;
        actionBar.setTitleColor(-1);
        this.actionBar.setSubtitleColor(-1);
        this.actionBar.setBackgroundColor(Theme.ACTION_BAR_PHOTO_VIEWER_COLOR);
        this.actionBar.setOccupyStatusBar(true);
        this.actionBar.setItemsBackgroundColor(Theme.ACTION_BAR_WHITE_SELECTOR_COLOR, false);
        this.actionBar.setBackButtonImage(R.mipmap.ic_back);
        this.actionBar.setTitleRightMargin(AndroidUtilities.dp(70.0f));
        this.containerView.addView(this.actionBar, LayoutHelper.createFrame(-1, -2.0f));
        this.actionBar.setActionBarMenuOnItemClick(new ActionBar.ActionBarMenuOnItemClick() { // from class: im.uecahxyvjd.ui.SecretMediaViewer.4
            @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
            public void onItemClick(int id) {
                if (id == -1) {
                    SecretMediaViewer.this.closePhoto(true, false);
                }
            }
        });
        SecretDeleteTimer secretDeleteTimer = new SecretDeleteTimer(activity);
        this.secretDeleteTimer = secretDeleteTimer;
        this.containerView.addView(secretDeleteTimer, LayoutHelper.createFrame(119.0f, 48.0f, 53, 0.0f, 0.0f, 0.0f, 0.0f));
        WindowManager.LayoutParams layoutParams2 = new WindowManager.LayoutParams();
        this.windowLayoutParams = layoutParams2;
        layoutParams2.height = -1;
        this.windowLayoutParams.format = -3;
        this.windowLayoutParams.width = -1;
        this.windowLayoutParams.gravity = 48;
        this.windowLayoutParams.type = 99;
        this.windowLayoutParams.flags = -2147417848;
        this.windowLayoutParams.flags |= 8192;
        this.centerImage.setParentView(this.containerView);
        this.centerImage.setForceCrossfade(true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$setParentActivity$0$im-uecahxyvjd-ui-SecretMediaViewer  reason: not valid java name */
    public /* synthetic */ WindowInsets m2158lambda$setParentActivity$0$imuecahxyvjduiSecretMediaViewer(View v, WindowInsets insets) {
        WindowInsets oldInsets = (WindowInsets) this.lastInsets;
        this.lastInsets = insets;
        if (oldInsets == null || !oldInsets.toString().equals(insets.toString())) {
            this.windowView.requestLayout();
        }
        return insets.consumeSystemWindowInsets();
    }

    public void openMedia(MessageObject messageObject, PhotoViewer.PhotoViewerProvider provider) {
        PhotoViewer.PlaceProviderObject object;
        final PhotoViewer.PlaceProviderObject object2;
        char c;
        if (this.parentActivity == null || messageObject == null || !messageObject.needDrawBluredPreview() || provider == null || (object = provider.getPlaceForPhoto(messageObject, null, 0, true)) == null) {
            return;
        }
        this.currentProvider = provider;
        this.openTime = System.currentTimeMillis();
        this.closeTime = 0L;
        this.isActionBarVisible = true;
        this.isPhotoVisible = true;
        this.draggingDown = false;
        AspectRatioFrameLayout aspectRatioFrameLayout = this.aspectRatioFrameLayout;
        if (aspectRatioFrameLayout != null) {
            aspectRatioFrameLayout.setVisibility(4);
        }
        releasePlayer();
        this.pinchStartDistance = 0.0f;
        this.pinchStartScale = 1.0f;
        this.pinchCenterX = 0.0f;
        this.pinchCenterY = 0.0f;
        this.pinchStartX = 0.0f;
        this.pinchStartY = 0.0f;
        this.moveStartX = 0.0f;
        this.moveStartY = 0.0f;
        this.zooming = false;
        this.moving = false;
        this.doubleTap = false;
        this.invalidCoords = false;
        this.canDragDown = true;
        updateMinMax(this.scale);
        this.photoBackgroundDrawable.setAlpha(0);
        this.containerView.setAlpha(1.0f);
        this.containerView.setVisibility(0);
        this.secretDeleteTimer.setAlpha(1.0f);
        this.isVideo = false;
        this.videoWatchedOneTime = false;
        this.closeVideoAfterWatch = false;
        this.disableShowCheck = true;
        this.centerImage.setManualAlphaAnimator(false);
        RectF drawRegion = object.imageReceiver.getDrawRegion();
        float width = drawRegion.width();
        float height = drawRegion.height();
        int viewWidth = AndroidUtilities.displaySize.x;
        int viewHeight = AndroidUtilities.statusBarHeight + AndroidUtilities.displaySize.y;
        this.scale = Math.max(width / viewWidth, height / viewHeight);
        this.translationX = ((object.viewX + drawRegion.left) + (width / 2.0f)) - (viewWidth / 2);
        this.translationY = ((object.viewY + drawRegion.top) + (height / 2.0f)) - (viewHeight / 2);
        this.clipHorizontal = Math.abs(drawRegion.left - object.imageReceiver.getImageX());
        int clipVertical = (int) Math.abs(drawRegion.top - object.imageReceiver.getImageY());
        int[] coords2 = new int[2];
        object.parentView.getLocationInWindow(coords2);
        float f = ((coords2[1] - 0) - (object.viewY + drawRegion.top)) + object.clipTopAddition;
        this.clipTop = f;
        if (f < 0.0f) {
            this.clipTop = 0.0f;
        }
        float height2 = (((object.viewY + drawRegion.top) + ((int) height)) - ((coords2[1] + object.parentView.getHeight()) - 0)) + object.clipBottomAddition;
        this.clipBottom = height2;
        if (height2 < 0.0f) {
            this.clipBottom = 0.0f;
        }
        this.clipTop = Math.max(this.clipTop, clipVertical);
        this.clipBottom = Math.max(this.clipBottom, clipVertical);
        this.animationStartTime = System.currentTimeMillis();
        this.animateToX = 0.0f;
        this.animateToY = 0.0f;
        this.animateToClipBottom = 0.0f;
        this.animateToClipHorizontal = 0.0f;
        this.animateToClipTop = 0.0f;
        this.animateToScale = 1.0f;
        this.zoomAnimation = true;
        NotificationCenter.getInstance(this.currentAccount).addObserver(this, NotificationCenter.messagesDeleted);
        NotificationCenter.getInstance(this.currentAccount).addObserver(this, NotificationCenter.updateMessageMedia);
        NotificationCenter.getInstance(this.currentAccount).addObserver(this, NotificationCenter.didCreatedNewDeleteTask);
        this.currentChannelId = messageObject.messageOwner.to_id != null ? messageObject.messageOwner.to_id.channel_id : 0;
        toggleActionBar(true, false);
        this.currentMessageObject = messageObject;
        TLRPC.Document document = messageObject.getDocument();
        ImageReceiver.BitmapHolder bitmapHolder = this.currentThumb;
        if (bitmapHolder != null) {
            bitmapHolder.release();
            this.currentThumb = null;
        }
        this.currentThumb = object.imageReceiver.getThumbBitmapSafe();
        if (document == null) {
            object2 = object;
            c = 4;
            this.actionBar.setTitle(LocaleController.getString("DisappearingPhoto", R.string.DisappearingPhoto));
            TLRPC.PhotoSize sizeFull = FileLoader.getClosestPhotoSizeWithSize(messageObject.photoThumbs, AndroidUtilities.getPhotoSize());
            this.centerImage.setImage(ImageLocation.getForObject(sizeFull, messageObject.photoThumbsObject), (String) null, this.currentThumb != null ? new BitmapDrawable(this.currentThumb.bitmap) : null, -1, (String) null, messageObject, 2);
            this.secretDeleteTimer.setDestroyTime(messageObject.messageOwner.destroyTime * 1000, messageObject.messageOwner.ttl, false);
        } else if (!MessageObject.isGifDocument(document)) {
            object2 = object;
            c = 4;
            this.playerRetryPlayCount = 1;
            this.actionBar.setTitle(LocaleController.getString("DisappearingVideo", R.string.DisappearingVideo));
            File f2 = new File(messageObject.messageOwner.attachPath);
            if (f2.exists()) {
                preparePlayer(f2);
            } else {
                File file = FileLoader.getPathToMessage(messageObject.messageOwner);
                File encryptedFile = new File(file.getAbsolutePath() + ".enc");
                if (encryptedFile.exists()) {
                    file = encryptedFile;
                }
                preparePlayer(file);
            }
            this.isVideo = true;
            this.centerImage.setImage((ImageLocation) null, (String) null, this.currentThumb != null ? new BitmapDrawable(this.currentThumb.bitmap) : null, -1, (String) null, messageObject, 2);
            long destroyTime = messageObject.messageOwner.destroyTime * 1000;
            long currentTime = System.currentTimeMillis() + (ConnectionsManager.getInstance(this.currentAccount).getTimeDifference() * 1000);
            long timeToDestroy = destroyTime - currentTime;
            long duration = messageObject.getDuration() * 1000;
            if (duration > timeToDestroy) {
                this.secretDeleteTimer.setDestroyTime(-1L, -1L, true);
            } else {
                this.secretDeleteTimer.setDestroyTime(messageObject.messageOwner.destroyTime * 1000, messageObject.messageOwner.ttl, false);
            }
        } else {
            this.actionBar.setTitle(LocaleController.getString("DisappearingGif", R.string.DisappearingGif));
            c = 4;
            object2 = object;
            this.centerImage.setImage(ImageLocation.getForDocument(document), (String) null, this.currentThumb != null ? new BitmapDrawable(this.currentThumb.bitmap) : null, -1, (String) null, messageObject, 1);
            this.secretDeleteTimer.setDestroyTime(messageObject.messageOwner.destroyTime * 1000, messageObject.messageOwner.ttl, false);
        }
        try {
            if (this.windowView.getParent() != null) {
                WindowManager wm = (WindowManager) this.parentActivity.getSystemService("window");
                wm.removeView(this.windowView);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        WindowManager wm2 = (WindowManager) this.parentActivity.getSystemService("window");
        wm2.addView(this.windowView, this.windowLayoutParams);
        this.secretDeleteTimer.invalidate();
        this.isVisible = true;
        AnimatorSet animatorSet = new AnimatorSet();
        this.imageMoveAnimation = animatorSet;
        Animator[] animatorArr = new Animator[5];
        animatorArr[0] = ObjectAnimator.ofFloat(this.actionBar, "alpha", 0.0f, 1.0f);
        animatorArr[1] = ObjectAnimator.ofFloat(this.secretDeleteTimer, "alpha", 0.0f, 1.0f);
        animatorArr[2] = ObjectAnimator.ofInt(this.photoBackgroundDrawable, "alpha", 0, 255);
        animatorArr[3] = ObjectAnimator.ofFloat(this.secretDeleteTimer, "alpha", 0.0f, 1.0f);
        animatorArr[c] = ObjectAnimator.ofFloat(this, "animationValue", 0.0f, 1.0f);
        animatorSet.playTogether(animatorArr);
        this.photoAnimationInProgress = 3;
        this.photoAnimationEndRunnable = new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$$ExternalSyntheticLambda2
            @Override // java.lang.Runnable
            public final void run() {
                SecretMediaViewer.this.m2156lambda$openMedia$1$imuecahxyvjduiSecretMediaViewer();
            }
        };
        this.imageMoveAnimation.setDuration(250L);
        this.imageMoveAnimation.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.SecretMediaViewer.5
            @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
            public void onAnimationEnd(Animator animation) {
                if (SecretMediaViewer.this.photoAnimationEndRunnable != null) {
                    SecretMediaViewer.this.photoAnimationEndRunnable.run();
                    SecretMediaViewer.this.photoAnimationEndRunnable = null;
                }
            }
        });
        this.photoTransitionAnimationStartTime = System.currentTimeMillis();
        this.containerView.setLayerType(2, null);
        this.imageMoveAnimation.setInterpolator(new DecelerateInterpolator());
        this.photoBackgroundDrawable.frame = 0;
        this.photoBackgroundDrawable.drawRunnable = new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                SecretMediaViewer.this.m2157lambda$openMedia$2$imuecahxyvjduiSecretMediaViewer(object2);
            }
        };
        this.imageMoveAnimation.start();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$openMedia$1$im-uecahxyvjd-ui-SecretMediaViewer  reason: not valid java name */
    public /* synthetic */ void m2156lambda$openMedia$1$imuecahxyvjduiSecretMediaViewer() {
        this.photoAnimationInProgress = 0;
        this.imageMoveAnimation = null;
        if (this.containerView != null) {
            this.containerView.setLayerType(0, null);
            this.containerView.invalidate();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$openMedia$2$im-uecahxyvjd-ui-SecretMediaViewer  reason: not valid java name */
    public /* synthetic */ void m2157lambda$openMedia$2$imuecahxyvjduiSecretMediaViewer(PhotoViewer.PlaceProviderObject object) {
        this.disableShowCheck = false;
        object.imageReceiver.setVisible(false, true);
    }

    public boolean isShowingImage(MessageObject object) {
        MessageObject messageObject;
        return (!this.isVisible || this.disableShowCheck || object == null || (messageObject = this.currentMessageObject) == null || messageObject.getId() != object.getId()) ? false : true;
    }

    private void toggleActionBar(boolean show, boolean animated) {
        if (show) {
            this.actionBar.setVisibility(0);
        }
        this.actionBar.setEnabled(show);
        this.isActionBarVisible = show;
        if (animated) {
            ArrayList<Animator> arrayList = new ArrayList<>();
            ActionBar actionBar = this.actionBar;
            float[] fArr = new float[1];
            fArr[0] = show ? 1.0f : 0.0f;
            arrayList.add(ObjectAnimator.ofFloat(actionBar, "alpha", fArr));
            AnimatorSet animatorSet = new AnimatorSet();
            this.currentActionBarAnimation = animatorSet;
            animatorSet.playTogether(arrayList);
            if (!show) {
                this.currentActionBarAnimation.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.SecretMediaViewer.6
                    @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
                    public void onAnimationEnd(Animator animation) {
                        if (SecretMediaViewer.this.currentActionBarAnimation != null && SecretMediaViewer.this.currentActionBarAnimation.equals(animation)) {
                            SecretMediaViewer.this.actionBar.setVisibility(8);
                            SecretMediaViewer.this.currentActionBarAnimation = null;
                        }
                    }
                });
            }
            this.currentActionBarAnimation.setDuration(200L);
            this.currentActionBarAnimation.start();
            return;
        }
        this.actionBar.setAlpha(show ? 1.0f : 0.0f);
        if (!show) {
            this.actionBar.setVisibility(8);
        }
    }

    public boolean isVisible() {
        return this.isVisible;
    }

    public void destroyPhotoViewer() {
        FrameLayout frameLayout;
        NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.messagesDeleted);
        NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.updateMessageMedia);
        NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.didCreatedNewDeleteTask);
        this.isVisible = false;
        this.currentProvider = null;
        ImageReceiver.BitmapHolder bitmapHolder = this.currentThumb;
        if (bitmapHolder != null) {
            bitmapHolder.release();
            this.currentThumb = null;
        }
        releasePlayer();
        if (this.parentActivity != null && (frameLayout = this.windowView) != null) {
            try {
                if (frameLayout.getParent() != null) {
                    WindowManager wm = (WindowManager) this.parentActivity.getSystemService("window");
                    wm.removeViewImmediate(this.windowView);
                }
                this.windowView = null;
            } catch (Exception e) {
                FileLog.e(e);
            }
        }
        Instance = null;
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* JADX WARN: Removed duplicated region for block: B:102:0x031f  */
    /* JADX WARN: Removed duplicated region for block: B:89:0x02bd  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void onDraw(Canvas canvas) {
        float currentScale;
        float currentTranslationY;
        float currentTranslationX;
        float currentClipTop;
        float currentClipBottom;
        float currentClipHorizontal;
        int bitmapWidth;
        int bitmapWidth2;
        int width;
        int height;
        float av;
        if (!this.isPhotoVisible) {
            return;
        }
        float aty = -1.0f;
        if (this.imageMoveAnimation != null) {
            if (!this.scroller.isFinished()) {
                this.scroller.abortAnimation();
            }
            if (this.useOvershootForScale) {
                float f = this.animationValue;
                if (f < 0.9f) {
                    av = f / 0.9f;
                    float f2 = this.scale;
                    currentScale = f2 + (((this.animateToScale * 1.02f) - f2) * av);
                } else {
                    float f3 = this.animateToScale;
                    currentScale = f3 + (0.01999998f * f3 * (1.0f - ((f - 0.9f) / 0.100000024f)));
                    av = 1.0f;
                }
                float av2 = this.translationY;
                currentTranslationY = av2 + ((this.animateToY - av2) * av);
                float f4 = this.translationX;
                currentTranslationX = f4 + ((this.animateToX - f4) * av);
                float f5 = this.clipTop;
                currentClipTop = f5 + ((this.animateToClipTop - f5) * av);
                float f6 = this.clipBottom;
                currentClipBottom = f6 + ((this.animateToClipBottom - f6) * av);
                float f7 = this.clipHorizontal;
                currentClipHorizontal = f7 + ((this.animateToClipHorizontal - f7) * av);
            } else {
                float f8 = this.scale;
                float f9 = this.animationValue;
                currentScale = f8 + ((this.animateToScale - f8) * f9);
                float f10 = this.translationY;
                currentTranslationY = f10 + ((this.animateToY - f10) * f9);
                float f11 = this.translationX;
                currentTranslationX = f11 + ((this.animateToX - f11) * f9);
                float f12 = this.clipTop;
                currentClipTop = f12 + ((this.animateToClipTop - f12) * f9);
                float f13 = this.clipBottom;
                currentClipBottom = f13 + ((this.animateToClipBottom - f13) * f9);
                float f14 = this.clipHorizontal;
                currentClipHorizontal = f14 + ((this.animateToClipHorizontal - f14) * f9);
            }
            if (this.animateToScale == 1.0f && this.scale == 1.0f && this.translationX == 0.0f) {
                aty = currentTranslationY;
            }
            this.containerView.invalidate();
        } else {
            if (this.animationStartTime != 0) {
                this.translationX = this.animateToX;
                this.translationY = this.animateToY;
                this.clipBottom = this.animateToClipBottom;
                this.clipTop = this.animateToClipTop;
                this.clipHorizontal = this.animateToClipHorizontal;
                float f15 = this.animateToScale;
                this.scale = f15;
                this.animationStartTime = 0L;
                updateMinMax(f15);
                this.zoomAnimation = false;
                this.useOvershootForScale = false;
            }
            if (!this.scroller.isFinished() && this.scroller.computeScrollOffset()) {
                if (this.scroller.getStartX() < this.maxX && this.scroller.getStartX() > this.minX) {
                    this.translationX = this.scroller.getCurrX();
                }
                if (this.scroller.getStartY() < this.maxY && this.scroller.getStartY() > this.minY) {
                    this.translationY = this.scroller.getCurrY();
                }
                this.containerView.invalidate();
            }
            currentScale = this.scale;
            currentTranslationY = this.translationY;
            currentTranslationX = this.translationX;
            currentClipTop = this.clipTop;
            currentClipBottom = this.clipBottom;
            currentClipHorizontal = this.clipHorizontal;
            if (!this.moving) {
                aty = this.translationY;
            }
        }
        float translateX = currentTranslationX;
        float scaleDiff = 0.0f;
        float alpha = 1.0f;
        if (this.photoAnimationInProgress != 3) {
            if (this.scale == 1.0f && aty != -1.0f && !this.zoomAnimation) {
                float maxValue = getContainerViewHeight() / 4.0f;
                this.photoBackgroundDrawable.setAlpha((int) Math.max(127.0f, (1.0f - (Math.min(Math.abs(aty), maxValue) / maxValue)) * 255.0f));
            } else {
                this.photoBackgroundDrawable.setAlpha(255);
            }
            if (!this.zoomAnimation) {
                float f16 = this.maxX;
                if (translateX > f16) {
                    float alpha2 = Math.min(1.0f, (translateX - f16) / canvas.getWidth());
                    scaleDiff = alpha2 * 0.3f;
                    alpha = 1.0f - alpha2;
                    translateX = this.maxX;
                }
            }
        }
        AspectRatioFrameLayout aspectRatioFrameLayout = this.aspectRatioFrameLayout;
        boolean drawTextureView = aspectRatioFrameLayout != null && aspectRatioFrameLayout.getVisibility() == 0;
        canvas.save();
        float sc = currentScale - scaleDiff;
        canvas.translate((getContainerViewWidth() / 2) + translateX, (getContainerViewHeight() / 2) + currentTranslationY);
        canvas.scale(sc, sc);
        int bitmapWidth3 = this.centerImage.getBitmapWidth();
        int bitmapHeight = this.centerImage.getBitmapHeight();
        if (!drawTextureView) {
            bitmapWidth = bitmapWidth3;
        } else if (this.textureUploaded) {
            float scale1 = bitmapWidth3 / bitmapHeight;
            bitmapWidth = bitmapWidth3;
            float scale2 = this.videoTextureView.getMeasuredWidth() / this.videoTextureView.getMeasuredHeight();
            if (Math.abs(scale1 - scale2) > 0.01f) {
                bitmapWidth2 = this.videoTextureView.getMeasuredWidth();
                bitmapHeight = this.videoTextureView.getMeasuredHeight();
                float scale = Math.min(getContainerViewHeight() / bitmapHeight, getContainerViewWidth() / bitmapWidth2);
                width = (int) (bitmapWidth2 * scale);
                height = (int) (bitmapHeight * scale);
                canvas.clipRect(((-width) / 2) + (currentClipHorizontal / sc), ((-height) / 2) + (currentClipTop / sc), (width / 2) - (currentClipHorizontal / sc), (height / 2) - (currentClipBottom / sc));
                if (drawTextureView || !this.textureUploaded || !this.videoCrossfadeStarted || this.videoCrossfadeAlpha != 1.0f) {
                    this.centerImage.setAlpha(alpha);
                    this.centerImage.setImageCoords((-width) / 2, (-height) / 2, width, height);
                    this.centerImage.draw(canvas);
                }
                if (drawTextureView) {
                    if (!this.videoCrossfadeStarted && this.textureUploaded) {
                        this.videoCrossfadeStarted = true;
                        this.videoCrossfadeAlpha = 0.0f;
                        this.videoCrossfadeAlphaLastTime = System.currentTimeMillis();
                    }
                    canvas.translate((-width) / 2, (-height) / 2);
                    this.videoTextureView.setAlpha(this.videoCrossfadeAlpha * alpha);
                    this.aspectRatioFrameLayout.draw(canvas);
                    if (this.videoCrossfadeStarted && this.videoCrossfadeAlpha < 1.0f) {
                        long newUpdateTime = System.currentTimeMillis();
                        long dt = newUpdateTime - this.videoCrossfadeAlphaLastTime;
                        this.videoCrossfadeAlphaLastTime = newUpdateTime;
                        this.videoCrossfadeAlpha += ((float) dt) / 200.0f;
                        this.containerView.invalidate();
                        if (this.videoCrossfadeAlpha > 1.0f) {
                            this.videoCrossfadeAlpha = 1.0f;
                        }
                    }
                }
                canvas.restore();
            }
        } else {
            bitmapWidth = bitmapWidth3;
        }
        bitmapWidth2 = bitmapWidth;
        float scale3 = Math.min(getContainerViewHeight() / bitmapHeight, getContainerViewWidth() / bitmapWidth2);
        width = (int) (bitmapWidth2 * scale3);
        height = (int) (bitmapHeight * scale3);
        canvas.clipRect(((-width) / 2) + (currentClipHorizontal / sc), ((-height) / 2) + (currentClipTop / sc), (width / 2) - (currentClipHorizontal / sc), (height / 2) - (currentClipBottom / sc));
        if (drawTextureView) {
        }
        this.centerImage.setAlpha(alpha);
        this.centerImage.setImageCoords((-width) / 2, (-height) / 2, width, height);
        this.centerImage.draw(canvas);
        if (drawTextureView) {
        }
        canvas.restore();
    }

    public float getVideoCrossfadeAlpha() {
        return this.videoCrossfadeAlpha;
    }

    public void setVideoCrossfadeAlpha(float value) {
        this.videoCrossfadeAlpha = value;
        this.containerView.invalidate();
    }

    private boolean checkPhotoAnimation() {
        if (this.photoAnimationInProgress != 0 && Math.abs(this.photoTransitionAnimationStartTime - System.currentTimeMillis()) >= 500) {
            Runnable runnable = this.photoAnimationEndRunnable;
            if (runnable != null) {
                runnable.run();
                this.photoAnimationEndRunnable = null;
            }
            this.photoAnimationInProgress = 0;
        }
        return this.photoAnimationInProgress != 0;
    }

    public long getOpenTime() {
        return this.openTime;
    }

    public long getCloseTime() {
        return this.closeTime;
    }

    public MessageObject getCurrentMessageObject() {
        return this.currentMessageObject;
    }

    public void closePhoto(boolean animated, boolean byDelete) {
        PhotoViewer.PlaceProviderObject object;
        final PhotoViewer.PlaceProviderObject object2;
        if (this.parentActivity != null && this.isPhotoVisible && !checkPhotoAnimation()) {
            NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.messagesDeleted);
            NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.updateMessageMedia);
            NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.didCreatedNewDeleteTask);
            this.isActionBarVisible = false;
            VelocityTracker velocityTracker = this.velocityTracker;
            if (velocityTracker != null) {
                velocityTracker.recycle();
                this.velocityTracker = null;
            }
            this.closeTime = System.currentTimeMillis();
            if (this.currentProvider != null && !(this.currentMessageObject.messageOwner.media.photo instanceof TLRPC.TL_photoEmpty) && !(this.currentMessageObject.messageOwner.media.document instanceof TLRPC.TL_documentEmpty)) {
                object = this.currentProvider.getPlaceForPhoto(this.currentMessageObject, null, 0, true);
            } else {
                object = null;
            }
            VideoPlayer videoPlayer = this.videoPlayer;
            if (videoPlayer != null) {
                videoPlayer.pause();
            }
            if (animated) {
                this.photoAnimationInProgress = 3;
                this.containerView.invalidate();
                this.imageMoveAnimation = new AnimatorSet();
                if (object != null && object.imageReceiver.getThumbBitmap() != null && !byDelete) {
                    object.imageReceiver.setVisible(false, true);
                    RectF drawRegion = object.imageReceiver.getDrawRegion();
                    float width = drawRegion.right - drawRegion.left;
                    float height = drawRegion.bottom - drawRegion.top;
                    int viewWidth = AndroidUtilities.displaySize.x;
                    int viewHeight = AndroidUtilities.displaySize.y + AndroidUtilities.statusBarHeight;
                    this.animateToScale = Math.max(width / viewWidth, height / viewHeight);
                    this.animateToX = ((object.viewX + drawRegion.left) + (width / 2.0f)) - (viewWidth / 2);
                    this.animateToY = ((object.viewY + drawRegion.top) + (height / 2.0f)) - (viewHeight / 2);
                    this.animateToClipHorizontal = Math.abs(drawRegion.left - object.imageReceiver.getImageX());
                    int clipVertical = (int) Math.abs(drawRegion.top - object.imageReceiver.getImageY());
                    int[] coords2 = new int[2];
                    object.parentView.getLocationInWindow(coords2);
                    float f = ((coords2[1] - 0) - (object.viewY + drawRegion.top)) + object.clipTopAddition;
                    this.animateToClipTop = f;
                    if (f < 0.0f) {
                        this.animateToClipTop = 0.0f;
                    }
                    float height2 = (((object.viewY + drawRegion.top) + ((int) height)) - ((coords2[1] + object.parentView.getHeight()) - 0)) + object.clipBottomAddition;
                    this.animateToClipBottom = height2;
                    if (height2 < 0.0f) {
                        this.animateToClipBottom = 0.0f;
                    }
                    object2 = object;
                    this.animationStartTime = System.currentTimeMillis();
                    this.animateToClipBottom = Math.max(this.animateToClipBottom, clipVertical);
                    this.animateToClipTop = Math.max(this.animateToClipTop, clipVertical);
                    this.zoomAnimation = true;
                } else {
                    object2 = object;
                    int h = AndroidUtilities.displaySize.y + AndroidUtilities.statusBarHeight;
                    this.animateToY = this.translationY >= 0.0f ? h : -h;
                }
                if (this.isVideo) {
                    this.videoCrossfadeStarted = false;
                    this.textureUploaded = false;
                    this.imageMoveAnimation.playTogether(ObjectAnimator.ofInt(this.photoBackgroundDrawable, "alpha", 0), ObjectAnimator.ofFloat(this, "animationValue", 0.0f, 1.0f), ObjectAnimator.ofFloat(this.actionBar, "alpha", 0.0f), ObjectAnimator.ofFloat(this.secretDeleteTimer, "alpha", 0.0f), ObjectAnimator.ofFloat(this, "videoCrossfadeAlpha", 0.0f));
                } else {
                    this.centerImage.setManualAlphaAnimator(true);
                    this.imageMoveAnimation.playTogether(ObjectAnimator.ofInt(this.photoBackgroundDrawable, "alpha", 0), ObjectAnimator.ofFloat(this, "animationValue", 0.0f, 1.0f), ObjectAnimator.ofFloat(this.actionBar, "alpha", 0.0f), ObjectAnimator.ofFloat(this.secretDeleteTimer, "alpha", 0.0f), ObjectAnimator.ofFloat(this.centerImage, "currentAlpha", 0.0f));
                }
                this.photoAnimationEndRunnable = new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$$ExternalSyntheticLambda4
                    @Override // java.lang.Runnable
                    public final void run() {
                        SecretMediaViewer.this.m2153lambda$closePhoto$3$imuecahxyvjduiSecretMediaViewer(object2);
                    }
                };
                this.imageMoveAnimation.setInterpolator(new DecelerateInterpolator());
                this.imageMoveAnimation.setDuration(250L);
                this.imageMoveAnimation.addListener(new AnonymousClass7(object2));
                this.photoTransitionAnimationStartTime = System.currentTimeMillis();
                this.containerView.setLayerType(2, null);
                this.imageMoveAnimation.start();
                return;
            }
            final PhotoViewer.PlaceProviderObject object3 = object;
            AnimatorSet animatorSet = new AnimatorSet();
            animatorSet.playTogether(ObjectAnimator.ofFloat(this.containerView, "scaleX", 0.9f), ObjectAnimator.ofFloat(this.containerView, "scaleY", 0.9f), ObjectAnimator.ofInt(this.photoBackgroundDrawable, "alpha", 0), ObjectAnimator.ofFloat(this.actionBar, "alpha", 0.0f));
            this.photoAnimationInProgress = 2;
            this.photoAnimationEndRunnable = new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$$ExternalSyntheticLambda5
                @Override // java.lang.Runnable
                public final void run() {
                    SecretMediaViewer.this.m2154lambda$closePhoto$4$imuecahxyvjduiSecretMediaViewer(object3);
                }
            };
            animatorSet.setDuration(200L);
            animatorSet.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.SecretMediaViewer.8
                @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
                public void onAnimationEnd(Animator animation) {
                    if (SecretMediaViewer.this.photoAnimationEndRunnable != null) {
                        SecretMediaViewer.this.photoAnimationEndRunnable.run();
                        SecretMediaViewer.this.photoAnimationEndRunnable = null;
                    }
                }
            });
            this.photoTransitionAnimationStartTime = System.currentTimeMillis();
            this.containerView.setLayerType(2, null);
            animatorSet.start();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$closePhoto$3$im-uecahxyvjd-ui-SecretMediaViewer  reason: not valid java name */
    public /* synthetic */ void m2153lambda$closePhoto$3$imuecahxyvjduiSecretMediaViewer(PhotoViewer.PlaceProviderObject object) {
        this.imageMoveAnimation = null;
        this.photoAnimationInProgress = 0;
        this.containerView.setLayerType(0, null);
        this.containerView.setVisibility(4);
        onPhotoClosed(object);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: im.uecahxyvjd.ui.SecretMediaViewer$7  reason: invalid class name */
    /* loaded from: classes12.dex */
    public class AnonymousClass7 extends AnimatorListenerAdapter {
        final /* synthetic */ PhotoViewer.PlaceProviderObject val$object;

        AnonymousClass7(PhotoViewer.PlaceProviderObject placeProviderObject) {
            this.val$object = placeProviderObject;
        }

        @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
        public void onAnimationEnd(Animator animation) {
            PhotoViewer.PlaceProviderObject placeProviderObject = this.val$object;
            if (placeProviderObject != null) {
                placeProviderObject.imageReceiver.setVisible(true, true);
            }
            SecretMediaViewer.this.isVisible = false;
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$7$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    SecretMediaViewer.AnonymousClass7.this.m2160lambda$onAnimationEnd$0$imuecahxyvjduiSecretMediaViewer$7();
                }
            });
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$onAnimationEnd$0$im-uecahxyvjd-ui-SecretMediaViewer$7  reason: not valid java name */
        public /* synthetic */ void m2160lambda$onAnimationEnd$0$imuecahxyvjduiSecretMediaViewer$7() {
            if (SecretMediaViewer.this.photoAnimationEndRunnable != null) {
                SecretMediaViewer.this.photoAnimationEndRunnable.run();
                SecretMediaViewer.this.photoAnimationEndRunnable = null;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$closePhoto$4$im-uecahxyvjd-ui-SecretMediaViewer  reason: not valid java name */
    public /* synthetic */ void m2154lambda$closePhoto$4$imuecahxyvjduiSecretMediaViewer(PhotoViewer.PlaceProviderObject object) {
        if (this.containerView == null) {
            return;
        }
        this.containerView.setLayerType(0, null);
        this.containerView.setVisibility(4);
        this.photoAnimationInProgress = 0;
        onPhotoClosed(object);
        this.containerView.setScaleX(1.0f);
        this.containerView.setScaleY(1.0f);
    }

    private void onPhotoClosed(PhotoViewer.PlaceProviderObject object) {
        this.isVisible = false;
        this.currentProvider = null;
        this.disableShowCheck = false;
        releasePlayer();
        new ArrayList();
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.SecretMediaViewer$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                SecretMediaViewer.this.m2155lambda$onPhotoClosed$5$imuecahxyvjduiSecretMediaViewer();
            }
        }, 50L);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onPhotoClosed$5$im-uecahxyvjd-ui-SecretMediaViewer  reason: not valid java name */
    public /* synthetic */ void m2155lambda$onPhotoClosed$5$imuecahxyvjduiSecretMediaViewer() {
        ImageReceiver.BitmapHolder bitmapHolder = this.currentThumb;
        if (bitmapHolder != null) {
            bitmapHolder.release();
            this.currentThumb = null;
        }
        this.centerImage.setImageBitmap((Bitmap) null);
        try {
            if (this.windowView.getParent() != null) {
                WindowManager wm = (WindowManager) this.parentActivity.getSystemService("window");
                wm.removeView(this.windowView);
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        this.isPhotoVisible = false;
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void updateMinMax(float scale) {
        int maxW = ((int) ((this.centerImage.getImageWidth() * scale) - getContainerViewWidth())) / 2;
        int maxH = ((int) ((this.centerImage.getImageHeight() * scale) - getContainerViewHeight())) / 2;
        if (maxW > 0) {
            this.minX = -maxW;
            this.maxX = maxW;
        } else {
            this.maxX = 0.0f;
            this.minX = 0.0f;
        }
        if (maxH > 0) {
            this.minY = -maxH;
            this.maxY = maxH;
            return;
        }
        this.maxY = 0.0f;
        this.minY = 0.0f;
    }

    private int getContainerViewWidth() {
        return this.containerView.getWidth();
    }

    private int getContainerViewHeight() {
        return this.containerView.getHeight();
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* JADX WARN: Removed duplicated region for block: B:93:0x01dd  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean processTouchEvent(MotionEvent ev) {
        if (this.photoAnimationInProgress == 0 && this.animationStartTime == 0) {
            if (ev.getPointerCount() == 1 && this.gestureDetector.onTouchEvent(ev) && this.doubleTap) {
                this.doubleTap = false;
                this.moving = false;
                this.zooming = false;
                checkMinMax(false);
                return true;
            }
            if (ev.getActionMasked() == 0 || ev.getActionMasked() == 5) {
                this.discardTap = false;
                if (!this.scroller.isFinished()) {
                    this.scroller.abortAnimation();
                }
                if (!this.draggingDown) {
                    if (ev.getPointerCount() == 2) {
                        this.pinchStartDistance = (float) Math.hypot(ev.getX(1) - ev.getX(0), ev.getY(1) - ev.getY(0));
                        this.pinchStartScale = this.scale;
                        this.pinchCenterX = (ev.getX(0) + ev.getX(1)) / 2.0f;
                        this.pinchCenterY = (ev.getY(0) + ev.getY(1)) / 2.0f;
                        this.pinchStartX = this.translationX;
                        this.pinchStartY = this.translationY;
                        this.zooming = true;
                        this.moving = false;
                        VelocityTracker velocityTracker = this.velocityTracker;
                        if (velocityTracker != null) {
                            velocityTracker.clear();
                        }
                    } else if (ev.getPointerCount() == 1) {
                        this.moveStartX = ev.getX();
                        float y = ev.getY();
                        this.moveStartY = y;
                        this.dragY = y;
                        this.draggingDown = false;
                        this.canDragDown = true;
                        VelocityTracker velocityTracker2 = this.velocityTracker;
                        if (velocityTracker2 != null) {
                            velocityTracker2.clear();
                        }
                    }
                }
            } else if (ev.getActionMasked() == 2) {
                if (ev.getPointerCount() == 2 && !this.draggingDown && this.zooming) {
                    this.discardTap = true;
                    this.scale = (((float) Math.hypot(ev.getX(1) - ev.getX(0), ev.getY(1) - ev.getY(0))) / this.pinchStartDistance) * this.pinchStartScale;
                    this.translationX = (this.pinchCenterX - (getContainerViewWidth() / 2)) - (((this.pinchCenterX - (getContainerViewWidth() / 2)) - this.pinchStartX) * (this.scale / this.pinchStartScale));
                    float containerViewHeight = this.pinchCenterY - (getContainerViewHeight() / 2);
                    float containerViewHeight2 = (this.pinchCenterY - (getContainerViewHeight() / 2)) - this.pinchStartY;
                    float f = this.scale;
                    this.translationY = containerViewHeight - (containerViewHeight2 * (f / this.pinchStartScale));
                    updateMinMax(f);
                    this.containerView.invalidate();
                } else if (ev.getPointerCount() == 1) {
                    VelocityTracker velocityTracker3 = this.velocityTracker;
                    if (velocityTracker3 != null) {
                        velocityTracker3.addMovement(ev);
                    }
                    float dx = Math.abs(ev.getX() - this.moveStartX);
                    float dy = Math.abs(ev.getY() - this.dragY);
                    if (dx > AndroidUtilities.dp(3.0f) || dy > AndroidUtilities.dp(3.0f)) {
                        this.discardTap = true;
                    }
                    if (this.canDragDown && !this.draggingDown && this.scale == 1.0f && dy >= AndroidUtilities.dp(30.0f) && dy / 2.0f > dx) {
                        this.draggingDown = true;
                        this.moving = false;
                        this.dragY = ev.getY();
                        if (this.isActionBarVisible) {
                            toggleActionBar(false, true);
                        }
                        return true;
                    } else if (this.draggingDown) {
                        this.translationY = ev.getY() - this.dragY;
                        this.containerView.invalidate();
                    } else if (!this.invalidCoords && this.animationStartTime == 0) {
                        float moveDx = this.moveStartX - ev.getX();
                        float moveDy = this.moveStartY - ev.getY();
                        if (this.moving || ((this.scale == 1.0f && Math.abs(moveDy) + AndroidUtilities.dp(12.0f) < Math.abs(moveDx)) || this.scale != 1.0f)) {
                            if (!this.moving) {
                                moveDx = 0.0f;
                                moveDy = 0.0f;
                                this.moving = true;
                                this.canDragDown = false;
                            }
                            this.moveStartX = ev.getX();
                            this.moveStartY = ev.getY();
                            updateMinMax(this.scale);
                            float f2 = this.translationX;
                            if (f2 < this.minX || f2 > this.maxX) {
                                moveDx /= 3.0f;
                            }
                            float f3 = this.maxY;
                            if (f3 == 0.0f) {
                                float f4 = this.minY;
                                if (f4 == 0.0f) {
                                    float f5 = this.translationY;
                                    if (f5 - moveDy < f4) {
                                        this.translationY = f4;
                                        moveDy = 0.0f;
                                    } else if (f5 - moveDy > f3) {
                                        this.translationY = f3;
                                        moveDy = 0.0f;
                                    }
                                    this.translationX = f2 - moveDx;
                                    if (this.scale != 1.0f) {
                                        this.translationY -= moveDy;
                                    }
                                    this.containerView.invalidate();
                                }
                            }
                            float f6 = this.translationY;
                            if (f6 < this.minY || f6 > f3) {
                                moveDy /= 3.0f;
                            }
                            this.translationX = f2 - moveDx;
                            if (this.scale != 1.0f) {
                            }
                            this.containerView.invalidate();
                        }
                    } else {
                        this.invalidCoords = false;
                        this.moveStartX = ev.getX();
                        this.moveStartY = ev.getY();
                    }
                }
            } else if (ev.getActionMasked() == 3 || ev.getActionMasked() == 1 || ev.getActionMasked() == 6) {
                if (this.zooming) {
                    this.invalidCoords = true;
                    float f7 = this.scale;
                    if (f7 < 1.0f) {
                        updateMinMax(1.0f);
                        animateTo(1.0f, 0.0f, 0.0f, true);
                    } else if (f7 > 3.0f) {
                        float atx = (this.pinchCenterX - (getContainerViewWidth() / 2)) - (((this.pinchCenterX - (getContainerViewWidth() / 2)) - this.pinchStartX) * (3.0f / this.pinchStartScale));
                        float aty = (this.pinchCenterY - (getContainerViewHeight() / 2)) - (((this.pinchCenterY - (getContainerViewHeight() / 2)) - this.pinchStartY) * (3.0f / this.pinchStartScale));
                        updateMinMax(3.0f);
                        if (atx < this.minX) {
                            atx = this.minX;
                        } else if (atx > this.maxX) {
                            atx = this.maxX;
                        }
                        if (aty < this.minY) {
                            aty = this.minY;
                        } else if (aty > this.maxY) {
                            aty = this.maxY;
                        }
                        animateTo(3.0f, atx, aty, true);
                    } else {
                        checkMinMax(true);
                    }
                    this.zooming = false;
                } else if (this.draggingDown) {
                    if (Math.abs(this.dragY - ev.getY()) > getContainerViewHeight() / 6.0f) {
                        closePhoto(true, false);
                    } else {
                        animateTo(1.0f, 0.0f, 0.0f, false);
                    }
                    this.draggingDown = false;
                } else if (this.moving) {
                    float moveToX = this.translationX;
                    float moveToY = this.translationY;
                    updateMinMax(this.scale);
                    this.moving = false;
                    this.canDragDown = true;
                    VelocityTracker velocityTracker4 = this.velocityTracker;
                    if (velocityTracker4 != null && this.scale == 1.0f) {
                        velocityTracker4.computeCurrentVelocity(1000);
                    }
                    float f8 = this.translationX;
                    if (f8 < this.minX) {
                        moveToX = this.minX;
                    } else if (f8 > this.maxX) {
                        moveToX = this.maxX;
                    }
                    float f9 = this.translationY;
                    if (f9 < this.minY) {
                        moveToY = this.minY;
                    } else if (f9 > this.maxY) {
                        moveToY = this.maxY;
                    }
                    animateTo(this.scale, moveToX, moveToY, false);
                }
            }
            return false;
        }
        return false;
    }

    private void checkMinMax(boolean zoom) {
        float moveToX = this.translationX;
        float moveToY = this.translationY;
        updateMinMax(this.scale);
        float f = this.translationX;
        if (f < this.minX) {
            moveToX = this.minX;
        } else if (f > this.maxX) {
            moveToX = this.maxX;
        }
        float f2 = this.translationY;
        if (f2 < this.minY) {
            moveToY = this.minY;
        } else if (f2 > this.maxY) {
            moveToY = this.maxY;
        }
        animateTo(this.scale, moveToX, moveToY, zoom);
    }

    private void animateTo(float newScale, float newTx, float newTy, boolean isZoom) {
        animateTo(newScale, newTx, newTy, isZoom, ItemTouchHelper.Callback.DEFAULT_SWIPE_ANIMATION_DURATION);
    }

    private void animateTo(float newScale, float newTx, float newTy, boolean isZoom, int duration) {
        if (this.scale == newScale && this.translationX == newTx && this.translationY == newTy) {
            return;
        }
        this.zoomAnimation = isZoom;
        this.animateToScale = newScale;
        this.animateToX = newTx;
        this.animateToY = newTy;
        this.animationStartTime = System.currentTimeMillis();
        AnimatorSet animatorSet = new AnimatorSet();
        this.imageMoveAnimation = animatorSet;
        animatorSet.playTogether(ObjectAnimator.ofFloat(this, "animationValue", 0.0f, 1.0f));
        this.imageMoveAnimation.setInterpolator(this.interpolator);
        this.imageMoveAnimation.setDuration(duration);
        this.imageMoveAnimation.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.SecretMediaViewer.9
            @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
            public void onAnimationEnd(Animator animation) {
                SecretMediaViewer.this.imageMoveAnimation = null;
                SecretMediaViewer.this.containerView.invalidate();
            }
        });
        this.imageMoveAnimation.start();
    }

    public void setAnimationValue(float value) {
        this.animationValue = value;
        this.containerView.invalidate();
    }

    public float getAnimationValue() {
        return this.animationValue;
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onDown(MotionEvent e) {
        return false;
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public void onShowPress(MotionEvent e) {
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onSingleTapUp(MotionEvent e) {
        return false;
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
        return false;
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public void onLongPress(MotionEvent e) {
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
        if (this.scale != 1.0f) {
            this.scroller.abortAnimation();
            this.scroller.fling(Math.round(this.translationX), Math.round(this.translationY), Math.round(velocityX), Math.round(velocityY), (int) this.minX, (int) this.maxX, (int) this.minY, (int) this.maxY);
            this.containerView.postInvalidate();
            return false;
        }
        return false;
    }

    @Override // android.view.GestureDetector.OnDoubleTapListener
    public boolean onSingleTapConfirmed(MotionEvent e) {
        if (this.discardTap) {
            return false;
        }
        toggleActionBar(!this.isActionBarVisible, true);
        return true;
    }

    @Override // android.view.GestureDetector.OnDoubleTapListener
    public boolean onDoubleTap(MotionEvent e) {
        float f = this.scale;
        if ((f != 1.0f || (this.translationY == 0.0f && this.translationX == 0.0f)) && this.animationStartTime == 0 && this.photoAnimationInProgress == 0) {
            if (f == 1.0f) {
                float atx = (e.getX() - (getContainerViewWidth() / 2)) - (((e.getX() - (getContainerViewWidth() / 2)) - this.translationX) * (3.0f / this.scale));
                float aty = (e.getY() - (getContainerViewHeight() / 2)) - (((e.getY() - (getContainerViewHeight() / 2)) - this.translationY) * (3.0f / this.scale));
                updateMinMax(3.0f);
                if (atx < this.minX) {
                    atx = this.minX;
                } else if (atx > this.maxX) {
                    atx = this.maxX;
                }
                if (aty < this.minY) {
                    aty = this.minY;
                } else if (aty > this.maxY) {
                    aty = this.maxY;
                }
                animateTo(3.0f, atx, aty, true);
            } else {
                animateTo(1.0f, 0.0f, 0.0f, true);
            }
            this.doubleTap = true;
            return true;
        }
        return false;
    }

    @Override // android.view.GestureDetector.OnDoubleTapListener
    public boolean onDoubleTapEvent(MotionEvent e) {
        return false;
    }

    private boolean scaleToFill() {
        return false;
    }
}
