package im.uecahxyvjd.ui.adapters;

import android.content.Context;
import android.text.SpannableStringBuilder;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.util.LongSparseArray;
import android.util.SparseArray;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ChatObject;
import im.uecahxyvjd.messenger.ContactsController;
import im.uecahxyvjd.messenger.DispatchQueue;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MediaDataController;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.messenger.MessagesStorage;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.messenger.UserObject;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.phoneformat.PhoneFormat;
import im.uecahxyvjd.sqlite.SQLiteCursor;
import im.uecahxyvjd.sqlite.SQLitePreparedStatement;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.NativeByteBuffer;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.adapters.DialogsSearchAdapter;
import im.uecahxyvjd.ui.adapters.SearchAdapterHelper;
import im.uecahxyvjd.ui.cells.DialogCell;
import im.uecahxyvjd.ui.cells.GraySectionCell;
import im.uecahxyvjd.ui.cells.HashtagSearchCell;
import im.uecahxyvjd.ui.cells.HintDialogCell;
import im.uecahxyvjd.ui.cells.LoadingCell;
import im.uecahxyvjd.ui.cells.ProfileSearchCell;
import im.uecahxyvjd.ui.cells.TextCell;
import im.uecahxyvjd.ui.components.RecyclerListView;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Locale;
import org.slf4j.Marker;
/* loaded from: classes18.dex */
public class DialogsSearchAdapter extends RecyclerListView.SelectionAdapter {
    private int currentAccount;
    private DialogsSearchAdapterDelegate delegate;
    private int dialogsType;
    private RecyclerListView innerListView;
    private String lastMessagesSearchString;
    private int lastReqId;
    private int lastSearchId;
    private String lastSearchText;
    private Context mContext;
    private int mProfileSearchCellMarginRight;
    private boolean messagesSearchEndReached;
    private int needMessagesSearch;
    private int nextSearchRate;
    private ArrayList<RecentSearchObject> recentSearchObjects;
    private LongSparseArray<RecentSearchObject> recentSearchObjectsById;
    private int reqId;
    private SearchAdapterHelper searchAdapterHelper;
    private ArrayList<TLObject> searchResult;
    private ArrayList<String> searchResultHashtags;
    private ArrayList<MessageObject> searchResultMessages;
    private ArrayList<CharSequence> searchResultNames;
    private Runnable searchRunnable;
    private Runnable searchRunnable2;
    private boolean searchWas;
    private int selfUserId;

    /* loaded from: classes18.dex */
    public interface DialogsSearchAdapterDelegate {
        void didPressedOnSubDialog(long j);

        void needClearList();

        void needRemoveHint(int i);

        void searchStateChanged(boolean z);
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes18.dex */
    public class DialogSearchResult {
        public int date;
        public CharSequence name;
        public TLObject object;

        private DialogSearchResult() {
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* loaded from: classes18.dex */
    public static class RecentSearchObject {
        int date;
        long did;
        TLObject object;

        protected RecentSearchObject() {
        }
    }

    /* loaded from: classes18.dex */
    private class CategoryAdapterRecycler extends RecyclerListView.SelectionAdapter {
        private CategoryAdapterRecycler() {
        }

        public void setIndex(int value) {
            notifyDataSetChanged();
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = new HintDialogCell(DialogsSearchAdapter.this.mContext);
            view.setLayoutParams(new RecyclerView.LayoutParams(AndroidUtilities.dp(80.0f), AndroidUtilities.dp(86.0f)));
            return new RecyclerListView.Holder(view);
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            return true;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            HintDialogCell cell = (HintDialogCell) holder.itemView;
            TLRPC.TL_topPeer peer = MediaDataController.getInstance(DialogsSearchAdapter.this.currentAccount).hints.get(position);
            new TLRPC.TL_dialog();
            TLRPC.Chat chat = null;
            TLRPC.User user = null;
            int did = 0;
            if (peer.peer.user_id != 0) {
                did = peer.peer.user_id;
                user = MessagesController.getInstance(DialogsSearchAdapter.this.currentAccount).getUser(Integer.valueOf(peer.peer.user_id));
            } else if (peer.peer.channel_id != 0) {
                did = -peer.peer.channel_id;
                chat = MessagesController.getInstance(DialogsSearchAdapter.this.currentAccount).getChat(Integer.valueOf(peer.peer.channel_id));
            } else if (peer.peer.chat_id != 0) {
                did = -peer.peer.chat_id;
                chat = MessagesController.getInstance(DialogsSearchAdapter.this.currentAccount).getChat(Integer.valueOf(peer.peer.chat_id));
            }
            cell.setTag(Integer.valueOf(did));
            String name = "";
            if (user != null) {
                name = UserObject.getFirstName(user);
            } else if (chat != null) {
                name = chat.title;
            }
            cell.setDialog(did, true, name);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            return MediaDataController.getInstance(DialogsSearchAdapter.this.currentAccount).hints.size();
        }
    }

    public DialogsSearchAdapter(Context context, int messagesSearch, int type) {
        this(context, messagesSearch, type, 0);
    }

    public DialogsSearchAdapter(Context context, int messagesSearch, int type, int profileSearchCellMarginRight) {
        this.searchResult = new ArrayList<>();
        this.searchResultNames = new ArrayList<>();
        this.searchResultMessages = new ArrayList<>();
        this.searchResultHashtags = new ArrayList<>();
        this.reqId = 0;
        this.lastSearchId = 0;
        this.currentAccount = UserConfig.selectedAccount;
        this.recentSearchObjects = new ArrayList<>();
        this.recentSearchObjectsById = new LongSparseArray<>();
        SearchAdapterHelper searchAdapterHelper = new SearchAdapterHelper(false);
        this.searchAdapterHelper = searchAdapterHelper;
        searchAdapterHelper.setDelegate(new SearchAdapterHelper.SearchAdapterHelperDelegate() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter.1
            @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
            public /* synthetic */ SparseArray getExcludeUsers() {
                return SearchAdapterHelper.SearchAdapterHelperDelegate.CC.$default$getExcludeUsers(this);
            }

            @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
            public void onDataSetChanged() {
                DialogsSearchAdapter.this.searchWas = true;
                if (!DialogsSearchAdapter.this.searchAdapterHelper.isSearchInProgress() && DialogsSearchAdapter.this.delegate != null) {
                    DialogsSearchAdapter.this.delegate.searchStateChanged(false);
                }
                DialogsSearchAdapter.this.notifyDataSetChanged();
            }

            @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
            public void onSetHashtags(ArrayList<SearchAdapterHelper.HashtagObject> arrayList, HashMap<String, SearchAdapterHelper.HashtagObject> hashMap) {
                for (int a2 = 0; a2 < arrayList.size(); a2++) {
                    DialogsSearchAdapter.this.searchResultHashtags.add(arrayList.get(a2).hashtag);
                }
                if (DialogsSearchAdapter.this.delegate != null) {
                    DialogsSearchAdapter.this.delegate.searchStateChanged(false);
                }
                DialogsSearchAdapter.this.notifyDataSetChanged();
            }
        });
        this.mContext = context;
        this.needMessagesSearch = messagesSearch;
        this.dialogsType = type;
        this.selfUserId = UserConfig.getInstance(this.currentAccount).getClientUserId();
        loadRecentSearch();
        MediaDataController.getInstance(this.currentAccount).loadHints(true);
        this.mProfileSearchCellMarginRight = profileSearchCellMarginRight;
    }

    public RecyclerListView getInnerListView() {
        return this.innerListView;
    }

    public void setDelegate(DialogsSearchAdapterDelegate delegate) {
        this.delegate = delegate;
    }

    public boolean isMessagesSearchEndReached() {
        return this.messagesSearchEndReached;
    }

    public void loadMoreSearchMessages() {
        searchMessagesInternal(this.lastMessagesSearchString);
    }

    public String getLastSearchString() {
        return this.lastMessagesSearchString;
    }

    private void searchMessagesInternal(String query) {
        if (this.needMessagesSearch != 0) {
            if (TextUtils.isEmpty(this.lastMessagesSearchString) && TextUtils.isEmpty(query)) {
                return;
            }
            if (this.reqId != 0) {
                ConnectionsManager.getInstance(this.currentAccount).cancelRequest(this.reqId, true);
                this.reqId = 0;
            }
            if (TextUtils.isEmpty(query)) {
                this.searchResultMessages.clear();
                this.lastReqId = 0;
                this.lastMessagesSearchString = null;
                this.searchWas = false;
                notifyDataSetChanged();
                DialogsSearchAdapterDelegate dialogsSearchAdapterDelegate = this.delegate;
                if (dialogsSearchAdapterDelegate != null) {
                    dialogsSearchAdapterDelegate.searchStateChanged(false);
                }
            }
        }
    }

    public boolean hasRecentRearch() {
        int i = this.dialogsType;
        return (i == 4 || i == 5 || i == 6 || (this.recentSearchObjects.isEmpty() && MediaDataController.getInstance(this.currentAccount).hints.isEmpty())) ? false : true;
    }

    public boolean isRecentSearchDisplayed() {
        int i;
        return (this.needMessagesSearch == 2 || this.searchWas || (this.recentSearchObjects.isEmpty() && MediaDataController.getInstance(this.currentAccount).hints.isEmpty()) || (i = this.dialogsType) == 4 || i == 5 || i == 6) ? false : true;
    }

    public void loadRecentSearch() {
        MessagesStorage.getInstance(this.currentAccount).getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda3
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2510x7bce396b();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadRecentSearch$2$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2510x7bce396b() {
        ArrayList<Integer> chatsToLoad;
        try {
            int i = 0;
            SQLiteCursor cursor = MessagesStorage.getInstance(this.currentAccount).getDatabase().queryFinalized("SELECT did, date FROM search_recent WHERE 1", new Object[0]);
            ArrayList<Integer> usersToLoad = new ArrayList<>();
            ArrayList<Integer> chatsToLoad2 = new ArrayList<>();
            ArrayList<Integer> encryptedToLoad = new ArrayList<>();
            new ArrayList();
            final ArrayList<RecentSearchObject> arrayList = new ArrayList<>();
            final LongSparseArray<RecentSearchObject> hashMap = new LongSparseArray<>();
            while (cursor.next()) {
                long did = cursor.longValue(i);
                boolean add = false;
                int lower_id = (int) did;
                int high_id = (int) (did >> 32);
                if (lower_id != 0) {
                    if (lower_id > 0) {
                        if (this.dialogsType != 2 && !usersToLoad.contains(Integer.valueOf(lower_id))) {
                            usersToLoad.add(Integer.valueOf(lower_id));
                            add = true;
                        }
                    } else if (!chatsToLoad2.contains(Integer.valueOf(-lower_id))) {
                        chatsToLoad2.add(Integer.valueOf(-lower_id));
                        add = true;
                    }
                } else {
                    int i2 = this.dialogsType;
                    if ((i2 == 0 || i2 == 3) && !encryptedToLoad.contains(Integer.valueOf(high_id))) {
                        encryptedToLoad.add(Integer.valueOf(high_id));
                        add = true;
                    }
                }
                if (!add) {
                    chatsToLoad = chatsToLoad2;
                } else {
                    RecentSearchObject recentSearchObject = new RecentSearchObject();
                    recentSearchObject.did = did;
                    recentSearchObject.date = cursor.intValue(1);
                    arrayList.add(recentSearchObject);
                    chatsToLoad = chatsToLoad2;
                    hashMap.put(recentSearchObject.did, recentSearchObject);
                }
                chatsToLoad2 = chatsToLoad;
                i = 0;
            }
            ArrayList<Integer> chatsToLoad3 = chatsToLoad2;
            cursor.dispose();
            ArrayList<TLRPC.User> users = new ArrayList<>();
            if (!encryptedToLoad.isEmpty()) {
                ArrayList<TLRPC.EncryptedChat> encryptedChats = new ArrayList<>();
                MessagesStorage.getInstance(this.currentAccount).getEncryptedChatsInternal(TextUtils.join(",", encryptedToLoad), encryptedChats, usersToLoad);
                for (int a2 = 0; a2 < encryptedChats.size(); a2++) {
                    hashMap.get(encryptedChats.get(a2).id << 32).object = encryptedChats.get(a2);
                }
            }
            if (!chatsToLoad3.isEmpty()) {
                ArrayList<TLRPC.Chat> chats = new ArrayList<>();
                MessagesStorage.getInstance(this.currentAccount).getChatsInternal(TextUtils.join(",", chatsToLoad3), chats);
                for (int a3 = 0; a3 < chats.size(); a3++) {
                    TLRPC.Chat chat = chats.get(a3);
                    long did2 = -chat.id;
                    if (chat.migrated_to != null) {
                        RecentSearchObject recentSearchObject2 = hashMap.get(did2);
                        hashMap.remove(did2);
                        if (recentSearchObject2 != null) {
                            arrayList.remove(recentSearchObject2);
                        }
                    } else {
                        hashMap.get(did2).object = chat;
                    }
                }
            }
            if (!usersToLoad.isEmpty()) {
                MessagesStorage.getInstance(this.currentAccount).getUsersInternal(TextUtils.join(",", usersToLoad), users);
                for (int a4 = 0; a4 < users.size(); a4++) {
                    TLRPC.User user = users.get(a4);
                    RecentSearchObject recentSearchObject3 = hashMap.get(user.id);
                    if (recentSearchObject3 != null) {
                        recentSearchObject3.object = user;
                    }
                }
            }
            Collections.sort(arrayList, new Comparator() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda1
                @Override // java.util.Comparator
                public final int compare(Object obj, Object obj2) {
                    return DialogsSearchAdapter.lambda$loadRecentSearch$0((DialogsSearchAdapter.RecentSearchObject) obj, (DialogsSearchAdapter.RecentSearchObject) obj2);
                }
            });
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda2
                @Override // java.lang.Runnable
                public final void run() {
                    DialogsSearchAdapter.this.m2509x4203978c(arrayList, hashMap);
                }
            });
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$loadRecentSearch$0(RecentSearchObject lhs, RecentSearchObject rhs) {
        if (lhs.date < rhs.date) {
            return 1;
        }
        if (lhs.date > rhs.date) {
            return -1;
        }
        return 0;
    }

    public void putRecentSearch(final long did, TLObject object) {
        RecentSearchObject recentSearchObject = this.recentSearchObjectsById.get(did);
        if (recentSearchObject == null) {
            recentSearchObject = new RecentSearchObject();
            this.recentSearchObjectsById.put(did, recentSearchObject);
        } else {
            this.recentSearchObjects.remove(recentSearchObject);
        }
        this.recentSearchObjects.add(0, recentSearchObject);
        recentSearchObject.did = did;
        recentSearchObject.object = object;
        recentSearchObject.date = (int) (System.currentTimeMillis() / 1000);
        notifyDataSetChanged();
        MessagesStorage.getInstance(this.currentAccount).getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda13
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2515xc852609b(did);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$putRecentSearch$3$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2515xc852609b(long did) {
        try {
            SQLitePreparedStatement state = MessagesStorage.getInstance(this.currentAccount).getDatabase().executeFast("REPLACE INTO search_recent VALUES(?, ?)");
            state.requery();
            state.bindLong(1, did);
            state.bindInteger(2, (int) (System.currentTimeMillis() / 1000));
            state.step();
            state.dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public void clearRecentSearch() {
        this.recentSearchObjectsById = new LongSparseArray<>();
        this.recentSearchObjects = new ArrayList<>();
        notifyDataSetChanged();
        MessagesStorage.getInstance(this.currentAccount).getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda0
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2508x812c8cbc();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$clearRecentSearch$4$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2508x812c8cbc() {
        try {
            MessagesStorage.getInstance(this.currentAccount).getDatabase().executeFast("DELETE FROM search_recent WHERE 1").stepThis().dispose();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public void addHashtagsFromMessage(CharSequence message) {
        this.searchAdapterHelper.addHashtagsFromMessage(message);
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* renamed from: setRecentSearch */
    public void m2509x4203978c(ArrayList<RecentSearchObject> arrayList, LongSparseArray<RecentSearchObject> hashMap) {
        this.recentSearchObjects = arrayList;
        this.recentSearchObjectsById = hashMap;
        for (int a2 = 0; a2 < this.recentSearchObjects.size(); a2++) {
            RecentSearchObject recentSearchObject = this.recentSearchObjects.get(a2);
            if (recentSearchObject.object instanceof TLRPC.User) {
                MessagesController.getInstance(this.currentAccount).putUser((TLRPC.User) recentSearchObject.object, true);
            } else if (recentSearchObject.object instanceof TLRPC.Chat) {
                MessagesController.getInstance(this.currentAccount).putChat((TLRPC.Chat) recentSearchObject.object, true);
            } else if (recentSearchObject.object instanceof TLRPC.EncryptedChat) {
                MessagesController.getInstance(this.currentAccount).putEncryptedChat((TLRPC.EncryptedChat) recentSearchObject.object, true);
            }
        }
        notifyDataSetChanged();
    }

    private void searchDialogsInternal(final String query, final int searchId) {
        if (this.needMessagesSearch == 2) {
            return;
        }
        MessagesStorage.getInstance(this.currentAccount).getStorageQueue().postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda7
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2518x526ad58a(query, searchId);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:194:0x0569 A[Catch: Exception -> 0x0788, LOOP:10: B:149:0x03d6->B:194:0x0569, LOOP_END, TryCatch #1 {Exception -> 0x0788, blocks: (B:136:0x0385, B:137:0x03a8, B:139:0x03ae, B:142:0x03c3, B:150:0x03d8, B:152:0x03db, B:165:0x0400, B:173:0x0426, B:182:0x0448, B:184:0x04d5, B:185:0x04d7, B:187:0x04e3, B:190:0x04ef, B:192:0x0551, B:191:0x0520, B:194:0x0569, B:197:0x05a3, B:199:0x05bb, B:200:0x05c1, B:202:0x05c7, B:204:0x05d1, B:206:0x05d5, B:207:0x05d8, B:208:0x05db, B:209:0x05ee, B:211:0x05f4), top: B:282:0x0385 }] */
    /* JADX WARN: Removed duplicated region for block: B:261:0x074a A[Catch: Exception -> 0x0786, LOOP:8: B:230:0x066a->B:261:0x074a, LOOP_END, TryCatch #2 {Exception -> 0x0786, blocks: (B:214:0x060a, B:216:0x060f, B:217:0x0623, B:219:0x0629, B:222:0x0636, B:225:0x064a, B:227:0x0657, B:229:0x0662, B:231:0x066c, B:233:0x067a, B:236:0x0699, B:238:0x069f, B:242:0x06b9, B:249:0x06cd, B:251:0x06d8, B:253:0x06ec, B:257:0x0701, B:259:0x073a, B:258:0x070e, B:261:0x074a, B:264:0x0772), top: B:284:0x060a }] */
    /* JADX WARN: Removed duplicated region for block: B:306:0x01fd A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:328:0x06cd A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:333:0x03f9 A[SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:92:0x0282 A[Catch: Exception -> 0x078c, LOOP:2: B:66:0x01c8->B:92:0x0282, LOOP_END, TryCatch #0 {Exception -> 0x078c, blocks: (B:3:0x0004, B:5:0x0020, B:7:0x0037, B:9:0x0045, B:16:0x0053, B:18:0x005a, B:19:0x005c, B:20:0x0088, B:22:0x008f, B:25:0x00b6, B:27:0x00bd, B:32:0x00ce, B:34:0x00d8, B:35:0x00e0, B:38:0x00f1, B:40:0x00fc, B:41:0x0105, B:45:0x010e, B:47:0x0118, B:49:0x012a, B:51:0x013b, B:53:0x015e, B:56:0x016a, B:57:0x0193, B:59:0x0199, B:62:0x01ae, B:64:0x01b6, B:65:0x01bd, B:67:0x01ca, B:70:0x01da, B:74:0x01e3, B:80:0x01fd, B:82:0x0208, B:84:0x022a, B:88:0x023b, B:90:0x0271, B:89:0x0246, B:92:0x0282, B:95:0x02a8, B:97:0x02c2, B:99:0x02c8, B:100:0x02ed, B:102:0x02f3, B:105:0x0308, B:107:0x030c, B:110:0x0316, B:113:0x031d, B:114:0x0320, B:116:0x0327, B:118:0x0335, B:120:0x0339, B:122:0x033f, B:126:0x034b, B:132:0x0379, B:134:0x037f, B:146:0x03cf, B:155:0x03e5, B:159:0x03ee, B:169:0x040f, B:177:0x0430), top: B:281:0x0004 }] */
    /* renamed from: lambda$searchDialogsInternal$6$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m2518x526ad58a(String query, int searchId) {
        int resultCount;
        String str;
        String str2;
        String[] search;
        String str3;
        String str4;
        ArrayList<TLRPC.User> encUsers;
        String str5;
        String str6;
        ArrayList<Integer> chatsToLoad;
        ArrayList<Integer> encryptedToLoad;
        ArrayList<TLRPC.User> encUsers2;
        TLRPC.EncryptedChat chat;
        TLRPC.User user;
        String username;
        String[] search2;
        int usernamePos;
        String str7;
        int found;
        int i;
        String str8;
        String str9;
        String name;
        String name2;
        String search22;
        String str10 = " ";
        try {
            String savedMessages = LocaleController.getString("SavedMessages", R.string.SavedMessages).toLowerCase();
            String search1 = query.trim().toLowerCase();
            if (search1.length() == 0) {
                this.lastSearchId = -1;
                updateSearchResults(new ArrayList<>(), new ArrayList<>(), new ArrayList<>(), this.lastSearchId);
                return;
            }
            String search23 = LocaleController.getInstance().getTranslitString(search1);
            search23 = (search1.equals(search23) || search23.length() == 0) ? null : null;
            int i2 = 0;
            String[] search3 = new String[(search23 != null ? 1 : 0) + 1];
            search3[0] = search1;
            if (search23 != null) {
                search3[1] = search23;
            }
            ArrayList<Integer> usersToLoad = new ArrayList<>();
            ArrayList<Integer> chatsToLoad2 = new ArrayList<>();
            ArrayList<Integer> encryptedToLoad2 = new ArrayList<>();
            ArrayList<TLRPC.User> encUsers3 = new ArrayList<>();
            int high_id = 0;
            LongSparseArray<DialogSearchResult> dialogsResult = new LongSparseArray<>();
            SQLiteCursor cursor = MessagesStorage.getInstance(this.currentAccount).getDatabase().queryFinalized("SELECT did, date FROM dialogs ORDER BY date DESC LIMIT 600", new Object[0]);
            while (cursor.next()) {
                long id = cursor.longValue(i2);
                DialogSearchResult dialogSearchResult = new DialogSearchResult();
                dialogSearchResult.date = cursor.intValue(1);
                String[] search4 = search3;
                dialogsResult.put(id, dialogSearchResult);
                int lower_id = (int) id;
                ArrayList<TLRPC.User> encUsers4 = encUsers3;
                int resultCount2 = high_id;
                int high_id2 = (int) (id >> 32);
                if (lower_id != 0) {
                    if (lower_id > 0) {
                        int i3 = this.dialogsType;
                        search22 = search23;
                        if (i3 == 4 && lower_id == this.selfUserId) {
                            search3 = search4;
                            encUsers3 = encUsers4;
                            high_id = resultCount2;
                            search23 = search22;
                            i2 = 0;
                        } else if (i3 != 2 && !usersToLoad.contains(Integer.valueOf(lower_id))) {
                            usersToLoad.add(Integer.valueOf(lower_id));
                        }
                    } else {
                        search22 = search23;
                        if (this.dialogsType == 4) {
                            search3 = search4;
                            encUsers3 = encUsers4;
                            high_id = resultCount2;
                            search23 = search22;
                            i2 = 0;
                        } else if (!chatsToLoad2.contains(Integer.valueOf(-lower_id))) {
                            chatsToLoad2.add(Integer.valueOf(-lower_id));
                        }
                    }
                } else {
                    search22 = search23;
                    int i4 = this.dialogsType;
                    if ((i4 == 0 || i4 == 3) && !encryptedToLoad2.contains(Integer.valueOf(high_id2))) {
                        encryptedToLoad2.add(Integer.valueOf(high_id2));
                    }
                }
                search3 = search4;
                encUsers3 = encUsers4;
                high_id = resultCount2;
                search23 = search22;
                i2 = 0;
            }
            String[] search5 = search3;
            ArrayList<TLRPC.User> encUsers5 = encUsers3;
            int resultCount3 = high_id;
            cursor.dispose();
            if (!savedMessages.contains(search1)) {
                resultCount = resultCount3;
            } else {
                TLRPC.User user2 = UserConfig.getInstance(this.currentAccount).getCurrentUser();
                DialogSearchResult dialogSearchResult2 = new DialogSearchResult();
                dialogSearchResult2.date = Integer.MAX_VALUE;
                dialogSearchResult2.name = savedMessages;
                dialogSearchResult2.object = user2;
                dialogsResult.put(user2.id, dialogSearchResult2);
                resultCount = resultCount3 + 1;
            }
            String str11 = ";;;";
            String str12 = ",";
            if (usersToLoad.isEmpty()) {
                str = ";;;";
                str2 = ",";
                search = search5;
            } else {
                SQLiteCursor cursor2 = MessagesStorage.getInstance(this.currentAccount).getDatabase().queryFinalized(String.format(Locale.US, "SELECT data, status, name FROM users WHERE uid IN(%s)", TextUtils.join(",", usersToLoad)), new Object[0]);
                while (cursor2.next()) {
                    String name3 = cursor2.stringValue(2);
                    String tName = LocaleController.getInstance().getTranslitString(name3);
                    if (name3.equals(tName)) {
                        tName = null;
                    }
                    String username2 = null;
                    int usernamePos2 = name3.lastIndexOf(str11);
                    if (usernamePos2 != -1) {
                        username2 = name3.substring(usernamePos2 + 3);
                    }
                    ArrayList<Integer> usersToLoad2 = usersToLoad;
                    String[] search6 = search5;
                    int usernamePos3 = search6.length;
                    String found2 = null;
                    int found3 = 0;
                    while (true) {
                        if (found3 >= usernamePos3) {
                            str8 = str11;
                            str9 = str12;
                            break;
                        }
                        String q = search6[found3];
                        int i5 = usernamePos3;
                        if (!name3.contains(q) && (tName == null || !tName.contains(q))) {
                            if (username2 != null && username2.startsWith(q)) {
                                name = name3;
                                name2 = 2;
                            } else {
                                name = name3;
                                name2 = found2;
                            }
                            if (name2 != null) {
                                found3++;
                                found2 = name2;
                                usernamePos3 = i5;
                                name3 = name;
                            } else {
                                NativeByteBuffer data = cursor2.byteBufferValue(0);
                                if (data != null) {
                                    TLRPC.User user3 = TLRPC.User.TLdeserialize(data, data.readInt32(false), false);
                                    data.reuse();
                                    str8 = str11;
                                    str9 = str12;
                                    DialogSearchResult dialogSearchResult3 = dialogsResult.get(user3.id);
                                    if (user3.status != null) {
                                        user3.status.expires = cursor2.intValue(1);
                                    }
                                    if (name2 == 1) {
                                        dialogSearchResult3.name = AndroidUtilities.generateSearchName(user3.first_name, user3.last_name, q);
                                    } else {
                                        dialogSearchResult3.name = AndroidUtilities.generateSearchName("@" + user3.username, null, "@" + q);
                                    }
                                    dialogSearchResult3.object = user3;
                                    resultCount++;
                                } else {
                                    str8 = str11;
                                    str9 = str12;
                                }
                            }
                        }
                        name = name3;
                        name2 = 1;
                        if (name2 != null) {
                        }
                    }
                    search5 = search6;
                    usersToLoad = usersToLoad2;
                    str11 = str8;
                    str12 = str9;
                }
                str = str11;
                str2 = str12;
                search = search5;
                cursor2.dispose();
            }
            if (chatsToLoad2.isEmpty()) {
                str3 = str2;
            } else {
                str3 = str2;
                SQLiteCursor cursor3 = MessagesStorage.getInstance(this.currentAccount).getDatabase().queryFinalized(String.format(Locale.US, "SELECT data, name FROM chats WHERE uid IN(%s)", TextUtils.join(str3, chatsToLoad2)), new Object[0]);
                while (cursor3.next()) {
                    String name4 = cursor3.stringValue(1);
                    String tName2 = LocaleController.getInstance().getTranslitString(name4);
                    if (name4.equals(tName2)) {
                        tName2 = null;
                    }
                    int length = search.length;
                    while (i < length) {
                        String q2 = search[i];
                        i = (!name4.contains(q2) && (tName2 == null || !tName2.contains(q2))) ? i + 1 : 0;
                        NativeByteBuffer data2 = cursor3.byteBufferValue(0);
                        if (data2 != null) {
                            TLRPC.Chat chat2 = TLRPC.Chat.TLdeserialize(data2, data2.readInt32(false), false);
                            data2.reuse();
                            if (chat2 != null && !chat2.deactivated) {
                                if (ChatObject.isChannel(chat2) && ChatObject.isNotInChat(chat2)) {
                                }
                                long dialog_id = -chat2.id;
                                DialogSearchResult dialogSearchResult4 = dialogsResult.get(dialog_id);
                                dialogSearchResult4.name = AndroidUtilities.generateSearchName(chat2.title, null, q2);
                                dialogSearchResult4.object = chat2;
                                resultCount++;
                            }
                        }
                    }
                }
                cursor3.dispose();
            }
            if (encryptedToLoad2.isEmpty()) {
                str4 = " ";
                encUsers = encUsers5;
                str5 = str;
            } else {
                try {
                    SQLiteCursor cursor4 = MessagesStorage.getInstance(this.currentAccount).getDatabase().queryFinalized(String.format(Locale.US, "SELECT q.data, u.name, q.user, q.g, q.authkey, q.ttl, u.data, u.status, q.layer, q.seq_in, q.seq_out, q.use_count, q.exchange_id, q.key_date, q.fprint, q.fauthkey, q.khash, q.in_seq_no, q.admin_id, q.mtproto_seq FROM enc_chats as q INNER JOIN users as u ON q.user = u.uid WHERE q.uid IN(%s)", TextUtils.join(str3, encryptedToLoad2)), new Object[0]);
                    while (cursor4.next()) {
                        String name5 = cursor4.stringValue(1);
                        String tName3 = LocaleController.getInstance().getTranslitString(name5);
                        if (name5.equals(tName3)) {
                            tName3 = null;
                        }
                        String username3 = null;
                        String str13 = str;
                        int usernamePos4 = name5.lastIndexOf(str13);
                        if (usernamePos4 != -1) {
                            username3 = name5.substring(usernamePos4 + 2);
                        }
                        char c = 0;
                        int a2 = 0;
                        while (true) {
                            int usernamePos5 = usernamePos4;
                            if (a2 >= search.length) {
                                str6 = str10;
                                chatsToLoad = chatsToLoad2;
                                encryptedToLoad = encryptedToLoad2;
                                encUsers2 = encUsers5;
                                break;
                            }
                            String q3 = search[a2];
                            if (!name5.contains(q3) && (tName3 == null || !tName3.contains(q3))) {
                                if (username3 != null && username3.startsWith(q3)) {
                                    c = 2;
                                }
                                if (c != 0) {
                                    a2++;
                                    usernamePos4 = usernamePos5;
                                    str10 = str10;
                                } else {
                                    NativeByteBuffer data3 = cursor4.byteBufferValue(0);
                                    if (data3 == null) {
                                        chat = null;
                                    } else {
                                        TLRPC.EncryptedChat chat3 = TLRPC.EncryptedChat.TLdeserialize(data3, data3.readInt32(false), false);
                                        data3.reuse();
                                        chat = chat3;
                                    }
                                    NativeByteBuffer data4 = cursor4.byteBufferValue(6);
                                    if (data4 == null) {
                                        chatsToLoad = chatsToLoad2;
                                        user = null;
                                    } else {
                                        chatsToLoad = chatsToLoad2;
                                        TLRPC.User user4 = TLRPC.User.TLdeserialize(data4, data4.readInt32(false), false);
                                        data4.reuse();
                                        user = user4;
                                    }
                                    if (chat == null || user == null) {
                                        str6 = str10;
                                        encryptedToLoad = encryptedToLoad2;
                                        encUsers2 = encUsers5;
                                    } else {
                                        encryptedToLoad = encryptedToLoad2;
                                        DialogSearchResult dialogSearchResult5 = dialogsResult.get(chat.id << 32);
                                        chat.user_id = cursor4.intValue(2);
                                        chat.a_or_b = cursor4.byteArrayValue(3);
                                        chat.auth_key = cursor4.byteArrayValue(4);
                                        chat.ttl = cursor4.intValue(5);
                                        chat.layer = cursor4.intValue(8);
                                        chat.seq_in = cursor4.intValue(9);
                                        chat.seq_out = cursor4.intValue(10);
                                        int use_count = cursor4.intValue(11);
                                        chat.key_use_count_in = (short) (use_count >> 16);
                                        chat.key_use_count_out = (short) use_count;
                                        str6 = str10;
                                        chat.exchange_id = cursor4.longValue(12);
                                        chat.key_create_date = cursor4.intValue(13);
                                        chat.future_key_fingerprint = cursor4.longValue(14);
                                        chat.future_auth_key = cursor4.byteArrayValue(15);
                                        chat.key_hash = cursor4.byteArrayValue(16);
                                        chat.in_seq_no = cursor4.intValue(17);
                                        int admin_id = cursor4.intValue(18);
                                        if (admin_id != 0) {
                                            chat.admin_id = admin_id;
                                        }
                                        chat.mtproto_seq = cursor4.intValue(19);
                                        if (user.status != null) {
                                            user.status.expires = cursor4.intValue(7);
                                        }
                                        if (c == 1) {
                                            dialogSearchResult5.name = new SpannableStringBuilder(ContactsController.formatName(user.first_name, user.last_name));
                                            ((SpannableStringBuilder) dialogSearchResult5.name).setSpan(new ForegroundColorSpan(Theme.getColor(Theme.key_chats_secretName)), 0, dialogSearchResult5.name.length(), 33);
                                        } else {
                                            dialogSearchResult5.name = AndroidUtilities.generateSearchName("@" + user.username, null, "@" + q3);
                                        }
                                        dialogSearchResult5.object = chat;
                                        encUsers2 = encUsers5;
                                        encUsers2.add(user);
                                        resultCount++;
                                    }
                                }
                            }
                            c = 1;
                            if (c != 0) {
                            }
                        }
                        encUsers5 = encUsers2;
                        str = str13;
                        chatsToLoad2 = chatsToLoad;
                        encryptedToLoad2 = encryptedToLoad;
                        str10 = str6;
                    }
                    str4 = str10;
                    encUsers = encUsers5;
                    str5 = str;
                    cursor4.dispose();
                } catch (Exception e) {
                    e = e;
                    FileLog.e(e);
                }
            }
            ArrayList<DialogSearchResult> searchResults = new ArrayList<>(resultCount);
            for (int a3 = 0; a3 < dialogsResult.size(); a3++) {
                DialogSearchResult dialogSearchResult6 = dialogsResult.valueAt(a3);
                if (dialogSearchResult6.object != null && dialogSearchResult6.name != null) {
                    searchResults.add(dialogSearchResult6);
                }
            }
            Collections.sort(searchResults, new Comparator() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda8
                @Override // java.util.Comparator
                public final int compare(Object obj, Object obj2) {
                    return DialogsSearchAdapter.lambda$searchDialogsInternal$5((DialogsSearchAdapter.DialogSearchResult) obj, (DialogsSearchAdapter.DialogSearchResult) obj2);
                }
            });
            ArrayList<TLObject> resultArray = new ArrayList<>();
            ArrayList<CharSequence> resultArrayNames = new ArrayList<>();
            for (int a4 = 0; a4 < searchResults.size(); a4++) {
                DialogSearchResult dialogSearchResult7 = searchResults.get(a4);
                resultArray.add(dialogSearchResult7.object);
                resultArrayNames.add(dialogSearchResult7.name);
            }
            try {
                if (this.dialogsType != 2) {
                    SQLiteCursor cursor5 = MessagesStorage.getInstance(this.currentAccount).getDatabase().queryFinalized("SELECT u.data, u.status, u.name, u.uid FROM users as u INNER JOIN contacts as c ON u.uid = c.uid", new Object[0]);
                    while (cursor5.next()) {
                        int uid = cursor5.intValue(3);
                        if (dialogsResult.indexOfKey(uid) < 0) {
                            String name6 = cursor5.stringValue(2);
                            String tName4 = LocaleController.getInstance().getTranslitString(name6);
                            if (name6.equals(tName4)) {
                                tName4 = null;
                            }
                            int found4 = name6.lastIndexOf(str5);
                            if (found4 == -1) {
                                username = null;
                            } else {
                                username = name6.substring(found4 + 3);
                            }
                            int found5 = 0;
                            ArrayList<DialogSearchResult> searchResults2 = searchResults;
                            int length2 = search.length;
                            String str14 = str5;
                            int i6 = 0;
                            while (true) {
                                if (i6 >= length2) {
                                    search2 = search;
                                    break;
                                }
                                String q4 = search[i6];
                                int i7 = length2;
                                if (name6.startsWith(q4)) {
                                    usernamePos = found4;
                                    search2 = search;
                                    str7 = str4;
                                } else {
                                    usernamePos = found4;
                                    search2 = search;
                                    str7 = str4;
                                    if (!name6.contains(str7 + q4) && (tName4 == null || (!tName4.startsWith(q4) && !tName4.contains(str7 + q4)))) {
                                        if (username != null && username.startsWith(q4)) {
                                            found = 2;
                                        } else {
                                            found = found5;
                                        }
                                        if (found != 0) {
                                            str4 = str7;
                                            i6++;
                                            found5 = found;
                                            uid = uid;
                                            length2 = i7;
                                            found4 = usernamePos;
                                            search = search2;
                                        } else {
                                            NativeByteBuffer data5 = cursor5.byteBufferValue(0);
                                            if (data5 != null) {
                                                str4 = str7;
                                                TLRPC.User user5 = TLRPC.User.TLdeserialize(data5, data5.readInt32(false), false);
                                                data5.reuse();
                                                if (user5.status != null) {
                                                    user5.status.expires = cursor5.intValue(1);
                                                }
                                                if (found == 1) {
                                                    resultArrayNames.add(AndroidUtilities.generateSearchName(user5.first_name, user5.last_name, q4));
                                                } else {
                                                    resultArrayNames.add(AndroidUtilities.generateSearchName("@" + user5.username, null, "@" + q4));
                                                }
                                                resultArray.add(user5);
                                            } else {
                                                str4 = str7;
                                            }
                                        }
                                    }
                                }
                                found = 1;
                                if (found != 0) {
                                }
                            }
                            searchResults = searchResults2;
                            search = search2;
                            str5 = str14;
                        }
                    }
                    cursor5.dispose();
                }
            } catch (Exception e2) {
                e = e2;
                FileLog.e(e);
            }
            try {
                updateSearchResults(resultArray, resultArrayNames, encUsers, searchId);
            } catch (Exception e3) {
                e = e3;
                FileLog.e(e);
            }
        } catch (Exception e4) {
            e = e4;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$searchDialogsInternal$5(DialogSearchResult lhs, DialogSearchResult rhs) {
        if (lhs.date < rhs.date) {
            return 1;
        }
        if (lhs.date > rhs.date) {
            return -1;
        }
        return 0;
    }

    private void updateSearchResults(final ArrayList<TLObject> result, final ArrayList<CharSequence> names, final ArrayList<TLRPC.User> encUsers, final int searchId) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda4
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2519x49fb27a4(searchId, result, encUsers, names);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$updateSearchResults$7$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2519x49fb27a4(int searchId, ArrayList result, ArrayList encUsers, ArrayList names) {
        if (searchId != this.lastSearchId) {
            return;
        }
        this.searchWas = true;
        for (int a2 = 0; a2 < result.size(); a2++) {
            TLObject obj = (TLObject) result.get(a2);
            if (obj instanceof TLRPC.User) {
                TLRPC.User user = (TLRPC.User) obj;
                MessagesController.getInstance(this.currentAccount).putUser(user, true);
            } else if (obj instanceof TLRPC.Chat) {
                TLRPC.Chat chat = (TLRPC.Chat) obj;
                MessagesController.getInstance(this.currentAccount).putChat(chat, true);
            } else if (obj instanceof TLRPC.EncryptedChat) {
                TLRPC.EncryptedChat chat2 = (TLRPC.EncryptedChat) obj;
                MessagesController.getInstance(this.currentAccount).putEncryptedChat(chat2, true);
            }
        }
        int a3 = this.currentAccount;
        MessagesController.getInstance(a3).putUsers(encUsers, true);
        this.searchResult = result;
        this.searchResultNames = names;
        this.searchAdapterHelper.mergeResults(result);
        notifyDataSetChanged();
        if (this.delegate != null) {
            if (getItemCount() == 0 && (this.searchRunnable2 != null || this.searchAdapterHelper.isSearchInProgress())) {
                this.delegate.searchStateChanged(true);
            } else {
                this.delegate.searchStateChanged(false);
            }
        }
    }

    public boolean isHashtagSearch() {
        return !this.searchResultHashtags.isEmpty();
    }

    public void clearRecentHashtags() {
        this.searchAdapterHelper.clearRecentHashtags();
        this.searchResultHashtags.clear();
        notifyDataSetChanged();
    }

    public void searchDialogs(final String text) {
        final String query;
        if (text != null && text.equals(this.lastSearchText)) {
            return;
        }
        this.lastSearchText = text;
        if (this.searchRunnable != null) {
            Utilities.searchQueue.cancelRunnable(this.searchRunnable);
            this.searchRunnable = null;
        }
        Runnable runnable = this.searchRunnable2;
        if (runnable != null) {
            AndroidUtilities.cancelRunOnUIThread(runnable);
            this.searchRunnable2 = null;
        }
        if (text != null) {
            query = text.trim();
        } else {
            query = null;
        }
        if (TextUtils.isEmpty(query)) {
            this.searchAdapterHelper.unloadRecentHashtags();
            this.searchResult.clear();
            this.searchResultNames.clear();
            this.searchResultHashtags.clear();
            this.searchAdapterHelper.mergeResults(null);
            if (this.needMessagesSearch != 2) {
                this.searchAdapterHelper.queryServerSearch(null, true, true, true, true, 0, this.dialogsType == 0, 0);
            }
            this.searchWas = false;
            this.lastSearchId = -1;
            searchMessagesInternal(null);
            notifyDataSetChanged();
            return;
        }
        if (this.needMessagesSearch != 2 && query.startsWith("#") && query.length() == 1) {
            this.messagesSearchEndReached = true;
            if (this.searchAdapterHelper.loadRecentHashtags()) {
                this.searchResultMessages.clear();
                this.searchResultHashtags.clear();
                ArrayList<SearchAdapterHelper.HashtagObject> hashtags = this.searchAdapterHelper.getHashtags();
                for (int a2 = 0; a2 < hashtags.size(); a2++) {
                    this.searchResultHashtags.add(hashtags.get(a2).hashtag);
                }
                DialogsSearchAdapterDelegate dialogsSearchAdapterDelegate = this.delegate;
                if (dialogsSearchAdapterDelegate != null) {
                    dialogsSearchAdapterDelegate.searchStateChanged(false);
                }
            }
            notifyDataSetChanged();
        } else {
            this.searchResultHashtags.clear();
            notifyDataSetChanged();
        }
        final int searchId = this.lastSearchId + 1;
        this.lastSearchId = searchId;
        DispatchQueue dispatchQueue = Utilities.searchQueue;
        Runnable runnable2 = new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda12
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2517x5a775664(query, searchId, text);
            }
        };
        this.searchRunnable = runnable2;
        dispatchQueue.postRunnable(runnable2, 300L);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$searchDialogs$9$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2517x5a775664(final String query, final int searchId, final String text) {
        this.searchRunnable = null;
        searchDialogsInternal(query, searchId);
        Runnable runnable = new Runnable() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda9
            @Override // java.lang.Runnable
            public final void run() {
                DialogsSearchAdapter.this.m2516x20acb485(searchId, query, text);
            }
        };
        this.searchRunnable2 = runnable;
        AndroidUtilities.runOnUIThread(runnable);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$searchDialogs$8$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2516x20acb485(int searchId, String query, String text) {
        this.searchRunnable2 = null;
        if (searchId != this.lastSearchId) {
            return;
        }
        if (this.needMessagesSearch != 2) {
            SearchAdapterHelper searchAdapterHelper = this.searchAdapterHelper;
            int i = this.dialogsType;
            searchAdapterHelper.queryServerSearch(query, true, i != 4, true, i != 4, 0, i == 0, 0);
        }
        searchMessagesInternal(text);
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemCount() {
        if (isRecentSearchDisplayed()) {
            return (!this.recentSearchObjects.isEmpty() ? this.recentSearchObjects.size() + 1 : 0) + (MediaDataController.getInstance(this.currentAccount).hints.isEmpty() ? 0 : 2);
        } else if (!this.searchResultHashtags.isEmpty()) {
            return this.searchResultHashtags.size() + 1;
        } else {
            int count = this.searchResult.size();
            int localServerCount = this.searchAdapterHelper.getLocalServerSearch().size();
            int globalCount = this.searchAdapterHelper.getGlobalSearch().size();
            int phoneCount = this.searchAdapterHelper.getPhoneSearch().size();
            int messagesCount = this.searchResultMessages.size();
            int count2 = count + localServerCount;
            if (globalCount != 0) {
                count2 += globalCount + 1;
            }
            if (phoneCount != 0) {
                count2 += phoneCount;
            }
            if (messagesCount != 0) {
                return count2 + messagesCount + 1 + (!this.messagesSearchEndReached ? 1 : 0);
            }
            return count2;
        }
    }

    public Object getItem(int i) {
        int messagesCount;
        TLRPC.Chat chat;
        if (isRecentSearchDisplayed()) {
            messagesCount = MediaDataController.getInstance(this.currentAccount).hints.isEmpty() ? 0 : 2;
            int offset = messagesCount;
            if (i <= offset || (i - 1) - offset >= this.recentSearchObjects.size()) {
                return null;
            }
            TLObject object = this.recentSearchObjects.get((i - 1) - offset).object;
            if (object instanceof TLRPC.User) {
                TLRPC.User user = MessagesController.getInstance(this.currentAccount).getUser(Integer.valueOf(((TLRPC.User) object).id));
                if (user != null) {
                    return user;
                }
                return object;
            } else if ((object instanceof TLRPC.Chat) && (chat = MessagesController.getInstance(this.currentAccount).getChat(Integer.valueOf(((TLRPC.Chat) object).id))) != null) {
                return chat;
            } else {
                return object;
            }
        } else if (!this.searchResultHashtags.isEmpty()) {
            if (i > 0) {
                return this.searchResultHashtags.get(i - 1);
            }
            return null;
        } else {
            ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
            ArrayList<TLObject> localServerSearch = this.searchAdapterHelper.getLocalServerSearch();
            ArrayList<Object> phoneSearch = this.searchAdapterHelper.getPhoneSearch();
            int localCount = this.searchResult.size();
            int localServerCount = localServerSearch.size();
            int phoneCount = phoneSearch.size();
            int globalCount = globalSearch.isEmpty() ? 0 : globalSearch.size() + 1;
            messagesCount = this.searchResultMessages.isEmpty() ? 0 : this.searchResultMessages.size() + 1;
            if (i >= 0 && i < localCount) {
                return this.searchResult.get(i);
            }
            int i2 = i - localCount;
            if (i2 >= 0 && i2 < localServerCount) {
                return localServerSearch.get(i2);
            }
            int i3 = i2 - localServerCount;
            if (i3 >= 0 && i3 < phoneCount) {
                return phoneSearch.get(i3);
            }
            int i4 = i3 - phoneCount;
            if (i4 > 0 && i4 < globalCount) {
                return globalSearch.get(i4 - 1);
            }
            int i5 = i4 - globalCount;
            if (i5 <= 0 || i5 >= messagesCount) {
                return null;
            }
            return this.searchResultMessages.get(i5 - 1);
        }
    }

    public boolean isGlobalSearch(int i) {
        if (!isRecentSearchDisplayed() && this.searchResultHashtags.isEmpty()) {
            ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
            ArrayList<TLObject> localServerSearch = this.searchAdapterHelper.getLocalServerSearch();
            int localCount = this.searchResult.size();
            int localServerCount = localServerSearch.size();
            int phoneCount = this.searchAdapterHelper.getPhoneSearch().size();
            int globalCount = globalSearch.isEmpty() ? 0 : globalSearch.size() + 1;
            if (!this.searchResultMessages.isEmpty()) {
                int size = this.searchResultMessages.size() + 1;
            }
            if (i < 0 || i >= localCount) {
                int i2 = i - localCount;
                if (i2 < 0 || i2 >= localServerCount) {
                    int i3 = i2 - localServerCount;
                    if (i3 <= 0 || i3 >= phoneCount) {
                        int i4 = i3 - phoneCount;
                        if (i4 <= 0 || i4 >= globalCount) {
                            int i5 = i4 - globalCount;
                            return false;
                        }
                        return true;
                    }
                    return false;
                }
                return false;
            }
            return false;
        }
        return false;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public long getItemId(int i) {
        return i;
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
    public boolean isEnabled(RecyclerView.ViewHolder holder) {
        int type = holder.getItemViewType();
        return (type == 1 || type == 3) ? false : true;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        int i;
        View view = null;
        switch (viewType) {
            case 0:
                view = new ProfileSearchCell(this.mContext);
                view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                break;
            case 1:
                view = new GraySectionCell(this.mContext);
                break;
            case 2:
                view = new DialogCell(this.mContext, false, true);
                break;
            case 3:
                view = new LoadingCell(this.mContext);
                break;
            case 4:
                view = new HashtagSearchCell(this.mContext);
                break;
            case 5:
                RecyclerListView horizontalListView = new RecyclerListView(this.mContext) { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter.2
                    @Override // im.uecahxyvjd.ui.components.RecyclerListView, androidx.recyclerview.widget.RecyclerView, android.view.ViewGroup
                    public boolean onInterceptTouchEvent(MotionEvent e) {
                        if (getParent() != null && getParent().getParent() != null) {
                            getParent().getParent().requestDisallowInterceptTouchEvent(true);
                        }
                        return super.onInterceptTouchEvent(e);
                    }
                };
                horizontalListView.setTag(9);
                horizontalListView.setItemAnimator(null);
                horizontalListView.setLayoutAnimation(null);
                LinearLayoutManager layoutManager = new LinearLayoutManager(this.mContext) { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter.3
                    @Override // androidx.recyclerview.widget.LinearLayoutManager, androidx.recyclerview.widget.RecyclerView.LayoutManager
                    public boolean supportsPredictiveItemAnimations() {
                        return false;
                    }
                };
                layoutManager.setOrientation(0);
                horizontalListView.setLayoutManager(layoutManager);
                horizontalListView.setAdapter(new CategoryAdapterRecycler());
                horizontalListView.setOnItemClickListener(new RecyclerListView.OnItemClickListener() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda10
                    @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemClickListener
                    public final void onItemClick(View view2, int i2) {
                        DialogsSearchAdapter.this.m2513x402e1505(view2, i2);
                    }
                });
                horizontalListView.setOnItemLongClickListener(new RecyclerListView.OnItemLongClickListener() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda11
                    @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemLongClickListener
                    public final boolean onItemClick(View view2, int i2) {
                        return DialogsSearchAdapter.this.m2514x79f8b6e4(view2, i2);
                    }
                });
                view = horizontalListView;
                this.innerListView = horizontalListView;
                break;
            case 6:
                view = new TextCell(this.mContext, 16);
                break;
        }
        if (viewType == 5) {
            view.setLayoutParams(new RecyclerView.LayoutParams(-1, AndroidUtilities.dp(86.0f)));
        } else {
            RecyclerView.LayoutParams lp = new RecyclerView.LayoutParams(-1, -2);
            if (viewType == 0 && (i = this.mProfileSearchCellMarginRight) != 0) {
                lp.rightMargin = i;
            }
            view.setLayoutParams(lp);
        }
        return new RecyclerListView.Holder(view);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onCreateViewHolder$10$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2513x402e1505(View view1, int position) {
        DialogsSearchAdapterDelegate dialogsSearchAdapterDelegate = this.delegate;
        if (dialogsSearchAdapterDelegate != null) {
            dialogsSearchAdapterDelegate.didPressedOnSubDialog(((Integer) view1.getTag()).intValue());
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onCreateViewHolder$11$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ boolean m2514x79f8b6e4(View view12, int position) {
        DialogsSearchAdapterDelegate dialogsSearchAdapterDelegate = this.delegate;
        if (dialogsSearchAdapterDelegate != null) {
            dialogsSearchAdapterDelegate.needRemoveHint(((Integer) view12.getTag()).intValue());
            return true;
        }
        return true;
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Removed duplicated region for block: B:162:0x040e  */
    /* JADX WARN: Removed duplicated region for block: B:163:0x0410  */
    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        TLRPC.User user;
        TLRPC.Chat chat;
        TLRPC.EncryptedChat encryptedChat;
        TLRPC.Chat chat2;
        int phoneCount2;
        String nameSearch;
        String foundUserName;
        boolean isRecent;
        int index;
        CharSequence name;
        CharSequence username;
        CharSequence membersString;
        switch (holder.getItemViewType()) {
            case 0:
                ProfileSearchCell cell = (ProfileSearchCell) holder.itemView;
                CharSequence username2 = null;
                CharSequence name2 = null;
                String un = null;
                Object obj = getItem(position);
                if (obj instanceof TLRPC.User) {
                    TLRPC.User user2 = (TLRPC.User) obj;
                    un = user2.username;
                    user = user2;
                    chat = null;
                    encryptedChat = null;
                } else if (obj instanceof TLRPC.Chat) {
                    TLRPC.Chat chat3 = MessagesController.getInstance(this.currentAccount).getChat(Integer.valueOf(((TLRPC.Chat) obj).id));
                    if (chat3 != null) {
                        chat2 = chat3;
                    } else {
                        TLRPC.Chat chat4 = (TLRPC.Chat) obj;
                        chat2 = chat4;
                    }
                    un = chat2.username;
                    user = null;
                    chat = chat2;
                    encryptedChat = null;
                } else if (!(obj instanceof TLRPC.EncryptedChat)) {
                    user = null;
                    chat = null;
                    encryptedChat = null;
                } else {
                    TLRPC.EncryptedChat encryptedChat2 = MessagesController.getInstance(this.currentAccount).getEncryptedChat(Integer.valueOf(((TLRPC.EncryptedChat) obj).id));
                    TLRPC.User user3 = MessagesController.getInstance(this.currentAccount).getUser(Integer.valueOf(encryptedChat2.user_id));
                    user = user3;
                    chat = null;
                    encryptedChat = encryptedChat2;
                }
                if (isRecentSearchDisplayed()) {
                    cell.useSeparator = position != getItemCount() - 1;
                    isRecent = true;
                } else {
                    ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
                    ArrayList<Object> phoneSearch = this.searchAdapterHelper.getPhoneSearch();
                    int localCount = this.searchResult.size();
                    int localServerCount = this.searchAdapterHelper.getLocalServerSearch().size();
                    int phoneCount = phoneSearch.size();
                    if (phoneCount > 0 && (phoneSearch.get(phoneCount - 1) instanceof String)) {
                        int phoneCount22 = phoneCount - 2;
                        phoneCount2 = phoneCount22;
                    } else {
                        phoneCount2 = phoneCount;
                    }
                    int globalCount = globalSearch.isEmpty() ? 0 : globalSearch.size() + 1;
                    cell.useSeparator = (position == getItemCount() - 1 || position == ((localCount + phoneCount2) + localServerCount) - 1 || position == (((localCount + globalCount) + phoneCount) + localServerCount) - 1) ? false : true;
                    if (getItemCount() == 1) {
                        cell.setBackground(Theme.createRoundRectDrawable(AndroidUtilities.dp(5.0f), Theme.getColor(Theme.key_windowBackgroundWhite)));
                    } else if (position == 0) {
                        cell.setBackground(Theme.createRoundRectDrawable(AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), 0.0f, 0.0f, Theme.getColor(Theme.key_windowBackgroundWhite)));
                    } else if (position == getItemCount() - 1) {
                        cell.setBackground(Theme.createRoundRectDrawable(0.0f, 0.0f, AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), Theme.getColor(Theme.key_windowBackgroundWhite)));
                    }
                    if (position < this.searchResult.size()) {
                        name2 = this.searchResultNames.get(position);
                        if (name2 != null && user != null && user.username != null && user.username.length() > 0 && name2.toString().startsWith("@" + user.username)) {
                            username2 = name2;
                            name2 = null;
                            isRecent = false;
                        }
                    } else {
                        String foundUserName2 = this.searchAdapterHelper.getLastFoundUsername();
                        if (!TextUtils.isEmpty(foundUserName2)) {
                            if (user != null) {
                                String str = user.first_name;
                                String nameSearch2 = user.last_name;
                                nameSearch = ContactsController.formatName(str, nameSearch2);
                            } else if (chat == null) {
                                nameSearch = null;
                            } else {
                                nameSearch = chat.title;
                            }
                            if (nameSearch != null && (index = AndroidUtilities.indexOfIgnoreCase(nameSearch, foundUserName2)) != -1) {
                                SpannableStringBuilder spannableStringBuilder = new SpannableStringBuilder(nameSearch);
                                spannableStringBuilder.setSpan(new ForegroundColorSpan(Theme.getColor(Theme.key_windowBackgroundWhiteBlueText4)), index, foundUserName2.length() + index, 33);
                                name2 = spannableStringBuilder;
                                isRecent = false;
                            }
                            if (un != null) {
                                if (!foundUserName2.startsWith("@")) {
                                    foundUserName = foundUserName2;
                                } else {
                                    foundUserName = foundUserName2.substring(1);
                                }
                                try {
                                    SpannableStringBuilder spannableStringBuilder2 = new SpannableStringBuilder();
                                    spannableStringBuilder2.append((CharSequence) "@");
                                    spannableStringBuilder2.append((CharSequence) un);
                                    int indexOfIgnoreCase = AndroidUtilities.indexOfIgnoreCase(un, foundUserName);
                                    int index2 = indexOfIgnoreCase;
                                    if (indexOfIgnoreCase != -1) {
                                        int len = foundUserName.length();
                                        if (index2 == 0) {
                                            len++;
                                        } else {
                                            index2++;
                                        }
                                        spannableStringBuilder2.setSpan(new ForegroundColorSpan(Theme.getColor(Theme.key_windowBackgroundWhiteBlueText4)), index2, index2 + len, 33);
                                    }
                                    username2 = spannableStringBuilder2;
                                    isRecent = false;
                                } catch (Exception e) {
                                    username2 = un;
                                    FileLog.e(e);
                                    isRecent = false;
                                }
                            }
                        }
                    }
                    isRecent = false;
                }
                boolean savedMessages = false;
                if (user != null && user.id == this.selfUserId) {
                    username2 = null;
                    savedMessages = true;
                    name = LocaleController.getString("SavedMessages", R.string.SavedMessages);
                } else {
                    name = name2;
                }
                if (chat != null && chat.participants_count != 0) {
                    if (ChatObject.isChannel(chat) && !chat.megagroup) {
                        membersString = LocaleController.formatPluralString("Subscribers", chat.participants_count);
                    } else {
                        membersString = LocaleController.formatPluralString("Members", chat.participants_count);
                    }
                    if (username2 instanceof SpannableStringBuilder) {
                        ((SpannableStringBuilder) username2).append((CharSequence) ", ").append(membersString);
                    } else {
                        username = !TextUtils.isEmpty(username2) ? TextUtils.concat(username2, ", ", membersString) : membersString;
                        cell.setData(user == null ? user : chat, encryptedChat, name, username, isRecent, savedMessages);
                        break;
                    }
                }
                username = username2;
                cell.setData(user == null ? user : chat, encryptedChat, name, username, isRecent, savedMessages);
                break;
            case 1:
                GraySectionCell cell2 = (GraySectionCell) holder.itemView;
                if (!isRecentSearchDisplayed()) {
                    if (!this.searchResultHashtags.isEmpty()) {
                        cell2.setText(LocaleController.getString("Hashtags", R.string.Hashtags), LocaleController.getString("ClearButton", R.string.ClearButton), new View.OnClickListener() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda6
                            @Override // android.view.View.OnClickListener
                            public final void onClick(View view) {
                                DialogsSearchAdapter.this.m2512x7244cd63(view);
                            }
                        });
                        break;
                    } else {
                        ArrayList<TLObject> globalSearch2 = this.searchAdapterHelper.getGlobalSearch();
                        int localCount2 = this.searchResult.size();
                        int localServerCount2 = this.searchAdapterHelper.getLocalServerSearch().size();
                        int phoneCount3 = this.searchAdapterHelper.getPhoneSearch().size();
                        int globalCount2 = globalSearch2.isEmpty() ? 0 : globalSearch2.size() + 1;
                        if (!this.searchResultMessages.isEmpty()) {
                            int size = this.searchResultMessages.size() + 1;
                        }
                        int position2 = position - (localCount2 + localServerCount2);
                        if (position2 >= 0 && position2 < phoneCount3) {
                            cell2.setText(LocaleController.getString("PhoneNumberSearch", R.string.UserNameOrPhoneNumberSearch));
                            return;
                        }
                        int position3 = position2 - phoneCount3;
                        if (position3 >= 0 && position3 < globalCount2) {
                            cell2.setText(LocaleController.getString("GlobalSearch", R.string.GlobalSearch));
                            return;
                        } else {
                            cell2.setText(LocaleController.getString("SearchMessages", R.string.SearchMessages));
                            return;
                        }
                    }
                } else {
                    int offset = !MediaDataController.getInstance(this.currentAccount).hints.isEmpty() ? 2 : 0;
                    if (position >= offset) {
                        cell2.setText(LocaleController.getString("Recent", R.string.Recent), LocaleController.getString("ClearButton", R.string.ClearButton), new View.OnClickListener() { // from class: im.uecahxyvjd.ui.adapters.DialogsSearchAdapter$$ExternalSyntheticLambda5
                            @Override // android.view.View.OnClickListener
                            public final void onClick(View view) {
                                DialogsSearchAdapter.this.m2511x387a2b84(view);
                            }
                        });
                        break;
                    } else {
                        cell2.setText(LocaleController.getString("ChatHints", R.string.ChatHints));
                        break;
                    }
                }
            case 2:
                DialogCell cell3 = (DialogCell) holder.itemView;
                cell3.useSeparator = position != getItemCount() - 1;
                MessageObject messageObject = (MessageObject) getItem(position);
                cell3.setDialog(messageObject.getDialogId(), messageObject, messageObject.messageOwner.date);
                break;
            case 4:
                HashtagSearchCell cell4 = (HashtagSearchCell) holder.itemView;
                cell4.setText(this.searchResultHashtags.get(position - 1));
                cell4.setNeedDivider(position != this.searchResultHashtags.size());
                break;
            case 5:
                RecyclerListView recyclerListView = (RecyclerListView) holder.itemView;
                ((CategoryAdapterRecycler) recyclerListView.getAdapter()).setIndex(position / 2);
                break;
            case 6:
                String str2 = (String) getItem(position);
                TextCell cell5 = (TextCell) holder.itemView;
                cell5.setColors(null, Theme.key_windowBackgroundWhiteBlueText2);
                cell5.setText(LocaleController.formatString("AddContactByPhone", R.string.AddContactByPhone, PhoneFormat.getInstance().format(Marker.ANY_NON_NULL_MARKER + str2)), false);
                break;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$12$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2511x387a2b84(View v) {
        DialogsSearchAdapterDelegate dialogsSearchAdapterDelegate = this.delegate;
        if (dialogsSearchAdapterDelegate != null) {
            dialogsSearchAdapterDelegate.needClearList();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$13$im-uecahxyvjd-ui-adapters-DialogsSearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2512x7244cd63(View v) {
        DialogsSearchAdapterDelegate dialogsSearchAdapterDelegate = this.delegate;
        if (dialogsSearchAdapterDelegate != null) {
            dialogsSearchAdapterDelegate.needClearList();
        }
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemViewType(int i) {
        if (isRecentSearchDisplayed()) {
            int offset = MediaDataController.getInstance(this.currentAccount).hints.isEmpty() ? 0 : 2;
            if (i <= offset) {
                return (i == offset || i % 2 == 0) ? 1 : 5;
            }
            return 0;
        } else if (!this.searchResultHashtags.isEmpty()) {
            return i == 0 ? 1 : 4;
        } else {
            ArrayList<TLObject> globalSearch = this.searchAdapterHelper.getGlobalSearch();
            int localCount = this.searchResult.size();
            int localServerCount = this.searchAdapterHelper.getLocalServerSearch().size();
            int phoneCount = this.searchAdapterHelper.getPhoneSearch().size();
            int globalCount = globalSearch.isEmpty() ? 0 : globalSearch.size() + 1;
            int messagesCount = this.searchResultMessages.isEmpty() ? 0 : this.searchResultMessages.size() + 1;
            if (i < 0 || i >= localCount) {
                int i2 = i - localCount;
                if (i2 < 0 || i2 >= localServerCount) {
                    int i3 = i2 - localServerCount;
                    if (i3 >= 0 && i3 < phoneCount) {
                        Object object = getItem(i3);
                        if (object instanceof String) {
                            String str = (String) object;
                            return "section".equals(str) ? 1 : 6;
                        }
                        return 0;
                    }
                    int i4 = i3 - phoneCount;
                    if (i4 >= 0 && i4 < globalCount) {
                        return i4 == 0 ? 1 : 0;
                    }
                    int i5 = i4 - globalCount;
                    if (i5 < 0 || i5 >= messagesCount) {
                        return 3;
                    }
                    return i5 == 0 ? 1 : 2;
                }
                return 0;
            }
            return 0;
        }
    }
}
