package im.uecahxyvjd.ui.adapters;

import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import androidx.recyclerview.widget.RecyclerView;
import com.baidu.location.BDLocation;
import com.baidu.mapapi.model.LatLng;
import com.baidu.mapapi.utils.DistanceUtil;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.LocationController;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.LocationActivity;
import im.uecahxyvjd.ui.cells.EmptyCell;
import im.uecahxyvjd.ui.cells.GraySectionCell;
import im.uecahxyvjd.ui.cells.LocationCell;
import im.uecahxyvjd.ui.cells.LocationLoadingCell;
import im.uecahxyvjd.ui.cells.LocationPoweredCell;
import im.uecahxyvjd.ui.cells.SendLocationCell;
import im.uecahxyvjd.ui.cells.SharingLiveLocationCell;
import im.uecahxyvjd.ui.components.RecyclerListView;
import java.util.ArrayList;
import java.util.Locale;
@Deprecated
/* loaded from: classes18.dex */
public class LocationActivityAdapter extends BaseLocationAdapter implements LocationController.LocationFetchCallback {
    private String addressName;
    private TLRPC.TL_channelLocation chatLocation;
    private MessageObject currentMessageObject;
    private BDLocation customLocation;
    private long dialogId;
    private boolean fetchingLocation;
    private BDLocation gpsLocation;
    private int locationType;
    private Context mContext;
    private int overScrollHeight;
    private BDLocation previousFetchedLocation;
    private boolean pulledUp;
    private SendLocationCell sendLocationCell;
    private int currentAccount = UserConfig.selectedAccount;
    private int shareLiveLocationPotistion = -1;
    private ArrayList<LocationActivity.LiveLocation> currentLiveLocations = new ArrayList<>();

    public LocationActivityAdapter(Context context, int type, long did) {
        this.mContext = context;
        this.locationType = type;
        this.dialogId = did;
    }

    public void setOverScrollHeight(int value) {
        this.overScrollHeight = value;
    }

    public void setGpsLocation(BDLocation location) {
        int i;
        boolean notSet = this.gpsLocation == null;
        this.gpsLocation = location;
        if (this.customLocation == null) {
            fetchLocationAddress();
        }
        if (notSet && (i = this.shareLiveLocationPotistion) > 0) {
            notifyItemChanged(i);
        }
        if (this.currentMessageObject != null) {
            notifyItemChanged(1);
            updateLiveLocations();
        } else if (this.locationType != 2) {
            updateCell();
        } else {
            updateLiveLocations();
        }
    }

    public void updateLiveLocations() {
        if (!this.currentLiveLocations.isEmpty()) {
            notifyItemRangeChanged(2, this.currentLiveLocations.size());
        }
    }

    public void setCustomLocation(BDLocation location) {
        this.customLocation = location;
        fetchLocationAddress();
        updateCell();
    }

    public void setLiveLocations(ArrayList<LocationActivity.LiveLocation> liveLocations) {
        this.currentLiveLocations = new ArrayList<>(liveLocations);
        int uid = UserConfig.getInstance(this.currentAccount).getClientUserId();
        int a2 = 0;
        while (true) {
            if (a2 >= this.currentLiveLocations.size()) {
                break;
            } else if (this.currentLiveLocations.get(a2).id != uid) {
                a2++;
            } else {
                this.currentLiveLocations.remove(a2);
                break;
            }
        }
        notifyDataSetChanged();
    }

    public void setMessageObject(MessageObject messageObject) {
        this.currentMessageObject = messageObject;
        notifyDataSetChanged();
    }

    public void setChatLocation(TLRPC.TL_channelLocation location) {
        this.chatLocation = location;
    }

    private void updateCell() {
        String address;
        SendLocationCell sendLocationCell = this.sendLocationCell;
        if (sendLocationCell != null) {
            if (this.locationType != 4) {
                if (this.customLocation != null) {
                    sendLocationCell.setText(LocaleController.getString("SendSelectedLocation", R.string.SendSelectedLocation), String.format(Locale.US, "(%f,%f)", Double.valueOf(this.customLocation.getLatitude()), Double.valueOf(this.customLocation.getLongitude())));
                    return;
                } else if (this.gpsLocation != null) {
                    sendLocationCell.setText(LocaleController.getString("SendLocation", R.string.SendLocation), LocaleController.formatString("AccurateTo", R.string.AccurateTo, LocaleController.formatPluralString("Meters", this.gpsLocation.getGpsAccuracyStatus())));
                    return;
                } else {
                    sendLocationCell.setText(LocaleController.getString("SendLocation", R.string.SendLocation), LocaleController.getString("Loading", R.string.Loading));
                    return;
                }
            }
            if (this.addressName != null) {
                address = this.addressName;
            } else {
                BDLocation bDLocation = this.customLocation;
                if ((bDLocation == null && this.gpsLocation == null) || this.fetchingLocation) {
                    address = LocaleController.getString("Loading", R.string.Loading);
                } else if (bDLocation != null) {
                    address = String.format(Locale.US, "(%f,%f)", Double.valueOf(this.customLocation.getLatitude()), Double.valueOf(this.customLocation.getLongitude()));
                } else if (this.gpsLocation != null) {
                    address = String.format(Locale.US, "(%f,%f)", Double.valueOf(this.gpsLocation.getLatitude()), Double.valueOf(this.gpsLocation.getLongitude()));
                } else {
                    address = LocaleController.getString("Loading", R.string.Loading);
                }
            }
            this.sendLocationCell.setText(LocaleController.getString("ChatSetThisLocation", R.string.ChatSetThisLocation), address);
        }
    }

    private String getAddressName() {
        return this.addressName;
    }

    @Override // im.uecahxyvjd.messenger.LocationController.LocationFetchCallback
    public void onLocationAddressAvailable(String address, String displayAddress, BDLocation location) {
    }

    public void fetchLocationAddress() {
        BDLocation location;
        if (this.locationType != 4) {
            return;
        }
        if (this.customLocation != null) {
            location = this.customLocation;
        } else {
            BDLocation location2 = this.gpsLocation;
            if (location2 != null) {
                location = this.gpsLocation;
            } else {
                return;
            }
        }
        LatLng newLatLng = new LatLng(location.getLatitude(), location.getLongitude());
        LatLng preLatLng = new LatLng(this.previousFetchedLocation.getLatitude(), this.previousFetchedLocation.getLongitude());
        if (this.previousFetchedLocation == null || DistanceUtil.getDistance(newLatLng, preLatLng) > 100.0d) {
            this.addressName = null;
        }
        updateCell();
        this.fetchingLocation = true;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemCount() {
        int i = this.locationType;
        if (i == 5 || i == 4) {
            return 2;
        }
        if (this.currentMessageObject != null) {
            return (this.currentLiveLocations.isEmpty() ? 0 : this.currentLiveLocations.size() + 2) + 2;
        } else if (i == 2) {
            return this.currentLiveLocations.size() + 2;
        } else {
            if (this.searching || (!this.searching && this.places.isEmpty())) {
                return this.locationType != 0 ? 5 : 4;
            } else if (this.locationType == 1) {
                return this.places.size() + 4 + (1 ^ (this.places.isEmpty() ? 1 : 0));
            } else {
                return this.places.size() + 3 + (1 ^ (this.places.isEmpty() ? 1 : 0));
            }
        }
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view;
        switch (viewType) {
            case 0:
                view = new EmptyCell(this.mContext);
                break;
            case 1:
                view = new SendLocationCell(this.mContext, false);
                break;
            case 2:
                view = new GraySectionCell(this.mContext);
                break;
            case 3:
                view = new LocationCell(this.mContext);
                break;
            case 4:
                view = new LocationLoadingCell(this.mContext);
                break;
            case 5:
                view = new LocationPoweredCell(this.mContext);
                break;
            case 6:
                SendLocationCell cell = new SendLocationCell(this.mContext, true);
                cell.setDialogId(this.dialogId);
                view = cell;
                break;
            default:
                Context context = this.mContext;
                int i = this.locationType;
                view = new SharingLiveLocationCell(context, true, (i == 4 || i == 5) ? 16 : 54);
                break;
        }
        return new RecyclerListView.Holder(view);
    }

    public void setPulledUp() {
        if (this.pulledUp) {
            return;
        }
        this.pulledUp = true;
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.LocationActivityAdapter$$ExternalSyntheticLambda0
            @Override // java.lang.Runnable
            public final void run() {
                LocationActivityAdapter.this.m2521x5d07f9f8();
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$setPulledUp$0$im-uecahxyvjd-ui-adapters-LocationActivityAdapter  reason: not valid java name */
    public /* synthetic */ void m2521x5d07f9f8() {
        notifyItemChanged(this.locationType == 0 ? 2 : 3);
    }

    public boolean isPulledUp() {
        return this.pulledUp;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        switch (holder.getItemViewType()) {
            case 0:
                ((EmptyCell) holder.itemView).setHeight(this.overScrollHeight);
                return;
            case 1:
                this.sendLocationCell = (SendLocationCell) holder.itemView;
                updateCell();
                return;
            case 2:
                if (this.currentMessageObject != null) {
                    ((GraySectionCell) holder.itemView).setText(LocaleController.getString("LiveLocations", R.string.LiveLocations));
                    return;
                } else if (this.pulledUp) {
                    ((GraySectionCell) holder.itemView).setText(LocaleController.getString("NearbyPlaces", R.string.NearbyPlaces));
                    return;
                } else {
                    ((GraySectionCell) holder.itemView).setText(LocaleController.getString("ShowNearbyPlaces", R.string.ShowNearbyPlaces));
                    return;
                }
            case 3:
                if (this.locationType == 0) {
                    ((LocationCell) holder.itemView).setLocation(this.places.get(position - 3), this.iconUrls.get(position - 3), true);
                    return;
                } else {
                    ((LocationCell) holder.itemView).setLocation(this.places.get(position - 4), this.iconUrls.get(position - 4), true);
                    return;
                }
            case 4:
                ((LocationLoadingCell) holder.itemView).setLoading(this.searching);
                return;
            case 5:
            default:
                return;
            case 6:
                ((SendLocationCell) holder.itemView).setHasLocation(this.gpsLocation != null);
                return;
            case 7:
                SharingLiveLocationCell locationCell = (SharingLiveLocationCell) holder.itemView;
                TLRPC.TL_channelLocation tL_channelLocation = this.chatLocation;
                if (tL_channelLocation != null) {
                    locationCell.setDialog(this.dialogId, tL_channelLocation);
                    return;
                }
                return;
        }
    }

    public Object getItem(int i) {
        int i2 = this.locationType;
        if (i2 == 4) {
            if (this.addressName == null) {
                return null;
            }
            TLRPC.TL_messageMediaVenue venue = new TLRPC.TL_messageMediaVenue();
            venue.address = this.addressName;
            venue.geo = new TLRPC.TL_geoPoint();
            if (this.customLocation != null) {
                venue.geo.lat = this.customLocation.getLatitude();
                venue.geo._long = this.customLocation.getLongitude();
            } else if (this.gpsLocation != null) {
                venue.geo.lat = this.gpsLocation.getLatitude();
                venue.geo._long = this.gpsLocation.getLongitude();
            }
            return venue;
        }
        MessageObject messageObject = this.currentMessageObject;
        if (messageObject != null) {
            if (i == 1) {
                return messageObject;
            }
            if (i > 3 && i < this.places.size() + 3) {
                return this.currentLiveLocations.get(i - 4);
            }
        } else if (i2 == 2) {
            if (i >= 2) {
                return this.currentLiveLocations.get(i - 2);
            }
            return null;
        } else if (i2 == 1) {
            if (i > 3 && i < this.places.size() + 4) {
                return this.places.get(i - 4);
            }
        } else if (i > 2 && i < this.places.size() + 3) {
            return this.places.get(i - 3);
        }
        return null;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemViewType(int position) {
        if (position == 0) {
            return 0;
        }
        int i = this.locationType;
        if (i == 5) {
            return 7;
        }
        if (i == 4) {
            return 1;
        }
        if (this.currentMessageObject != null) {
            if (position == 2) {
                return 2;
            }
            if (position == 3) {
                this.shareLiveLocationPotistion = position;
                return 6;
            }
            return 7;
        } else if (i == 2) {
            if (position == 1) {
                this.shareLiveLocationPotistion = position;
                return 6;
            }
            return 7;
        } else {
            if (i == 1) {
                if (position == 1) {
                    return 1;
                }
                if (position == 2) {
                    this.shareLiveLocationPotistion = position;
                    return 6;
                } else if (position == 3) {
                    return 2;
                } else {
                    if (this.searching || (!this.searching && this.places.isEmpty())) {
                        return 4;
                    }
                    if (position == this.places.size() + 4) {
                        return 5;
                    }
                }
            } else if (position == 1) {
                return 1;
            } else {
                if (position == 2) {
                    return 2;
                }
                if (this.searching || (!this.searching && this.places.isEmpty())) {
                    return 4;
                }
                if (position == this.places.size() + 3) {
                    return 5;
                }
            }
            return 3;
        }
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
    public boolean isEnabled(RecyclerView.ViewHolder holder) {
        int viewType = holder.getItemViewType();
        return viewType == 6 ? (LocationController.getInstance(this.currentAccount).getSharingLocationInfo(this.dialogId) == null && this.gpsLocation == null) ? false : true : viewType == 1 || viewType == 3 || viewType == 7;
    }
}
