package im.uecahxyvjd.ui.adapters;

import android.content.Context;
import android.text.SpannableStringBuilder;
import android.text.style.ForegroundColorSpan;
import android.util.SparseArray;
import android.view.View;
import android.view.ViewGroup;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ContactsController;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.phoneformat.PhoneFormat;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.adapters.SearchAdapterHelper;
import im.uecahxyvjd.ui.cells.GraySectionCell;
import im.uecahxyvjd.ui.cells.ProfileSearchCell;
import im.uecahxyvjd.ui.cells.TextCell;
import im.uecahxyvjd.ui.cells.UserCell;
import im.uecahxyvjd.ui.components.RecyclerListView;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;
import org.slf4j.Marker;
/* loaded from: classes18.dex */
public class SearchAdapter extends RecyclerListView.SelectionAdapter {
    private boolean allowBots;
    private boolean allowChats;
    private boolean allowPhoneNumbers;
    private boolean allowUsernameSearch;
    private int channelId;
    private SparseArray<?> checkedMap;
    private SparseArray<TLRPC.User> ignoreUsers;
    private Context mContext;
    private boolean onlyMutual;
    private SearchAdapterHelper searchAdapterHelper;
    private Timer searchTimer;
    private boolean useUserCell;
    private ArrayList<TLObject> searchResult = new ArrayList<>();
    private ArrayList<CharSequence> searchResultNames = new ArrayList<>();
    private int miViewType = 0;

    public SearchAdapter(Context context, SparseArray<TLRPC.User> arg1, boolean usernameSearch, boolean mutual, boolean chats, boolean bots, boolean phones, int searchChannelId) {
        this.mContext = context;
        this.ignoreUsers = arg1;
        this.onlyMutual = mutual;
        this.allowUsernameSearch = usernameSearch;
        this.allowChats = chats;
        this.allowBots = bots;
        this.channelId = searchChannelId;
        this.allowPhoneNumbers = phones;
        SearchAdapterHelper searchAdapterHelper = new SearchAdapterHelper(true);
        this.searchAdapterHelper = searchAdapterHelper;
        searchAdapterHelper.setDelegate(new SearchAdapterHelper.SearchAdapterHelperDelegate() { // from class: im.uecahxyvjd.ui.adapters.SearchAdapter.1
            @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
            public void onDataSetChanged() {
                SearchAdapter.this.notifyDataSetChanged();
            }

            @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
            public void onSetHashtags(ArrayList<SearchAdapterHelper.HashtagObject> arrayList, HashMap<String, SearchAdapterHelper.HashtagObject> hashMap) {
            }

            @Override // im.uecahxyvjd.ui.adapters.SearchAdapterHelper.SearchAdapterHelperDelegate
            public SparseArray<TLRPC.User> getExcludeUsers() {
                return SearchAdapter.this.ignoreUsers;
            }
        });
    }

    public void setCheckedMap(SparseArray<?> map) {
        this.checkedMap = map;
    }

    public void setUseUserCell(boolean value) {
        this.useUserCell = value;
    }

    public void searchDialogs(final String query) {
        try {
            Timer timer = this.searchTimer;
            if (timer != null) {
                timer.cancel();
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        if (query == null) {
            this.searchResult.clear();
            this.searchResultNames.clear();
            if (this.allowUsernameSearch) {
                this.searchAdapterHelper.queryServerSearch(null, true, this.allowChats, this.allowBots, true, this.channelId, this.allowPhoneNumbers, 0);
            }
            notifyDataSetChanged();
            return;
        }
        Timer timer2 = new Timer();
        this.searchTimer = timer2;
        timer2.schedule(new TimerTask() { // from class: im.uecahxyvjd.ui.adapters.SearchAdapter.2
            @Override // java.util.TimerTask, java.lang.Runnable
            public void run() {
                try {
                    SearchAdapter.this.searchTimer.cancel();
                    SearchAdapter.this.searchTimer = null;
                } catch (Exception e2) {
                    FileLog.e(e2);
                }
                SearchAdapter.this.processSearch(query);
            }
        }, 200L, 300L);
    }

    public void setMiViewType(int miViewType) {
        this.miViewType = miViewType;
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void processSearch(final String query) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.SearchAdapter$$ExternalSyntheticLambda0
            @Override // java.lang.Runnable
            public final void run() {
                SearchAdapter.this.m2538lambda$processSearch$1$imuecahxyvjduiadaptersSearchAdapter(query);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$processSearch$1$im-uecahxyvjd-ui-adapters-SearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2538lambda$processSearch$1$imuecahxyvjduiadaptersSearchAdapter(final String query) {
        if (this.allowUsernameSearch) {
            this.searchAdapterHelper.queryServerSearch(query, true, this.allowChats, this.allowBots, true, this.channelId, this.allowPhoneNumbers, -1);
        }
        final int currentAccount = UserConfig.selectedAccount;
        final ArrayList<TLRPC.Contact> contactsCopy = new ArrayList<>(ContactsController.getInstance(currentAccount).contacts);
        Utilities.searchQueue.postRunnable(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.SearchAdapter$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                SearchAdapter.this.m2537lambda$processSearch$0$imuecahxyvjduiadaptersSearchAdapter(query, contactsCopy, currentAccount);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Removed duplicated region for block: B:58:0x011d A[LOOP:1: B:38:0x00b0->B:58:0x011d, LOOP_END] */
    /* JADX WARN: Removed duplicated region for block: B:71:0x00dc A[SYNTHETIC] */
    /* renamed from: lambda$processSearch$0$im-uecahxyvjd-ui-adapters-SearchAdapter  reason: not valid java name */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public /* synthetic */ void m2537lambda$processSearch$0$imuecahxyvjduiadaptersSearchAdapter(String query, ArrayList contactsCopy, int currentAccount) {
        String search1;
        String search12 = query.trim().toLowerCase();
        if (search12.length() == 0) {
            updateSearchResults(new ArrayList<>(), new ArrayList<>());
            return;
        }
        String search2 = LocaleController.getInstance().getTranslitString(search12);
        search2 = (search12.equals(search2) || search2.length() == 0) ? null : null;
        int i = 0;
        String[] search = new String[(search2 != null ? 1 : 0) + 1];
        search[0] = search12;
        if (search2 != null) {
            search[1] = search2;
        }
        ArrayList<TLObject> resultArray = new ArrayList<>();
        ArrayList<CharSequence> resultArrayNames = new ArrayList<>();
        int a2 = 0;
        while (a2 < contactsCopy.size()) {
            TLRPC.Contact contact = (TLRPC.Contact) contactsCopy.get(a2);
            TLRPC.User user = MessagesController.getInstance(currentAccount).getUser(Integer.valueOf(contact.user_id));
            if (user.id != UserConfig.getInstance(currentAccount).getClientUserId()) {
                if (!this.onlyMutual || user.mutual_contact) {
                    SparseArray<TLRPC.User> sparseArray = this.ignoreUsers;
                    if (sparseArray != null && sparseArray.indexOfKey(contact.user_id) >= 0) {
                        search1 = search12;
                    } else {
                        String name = ContactsController.formatName(user.first_name, user.last_name).toLowerCase();
                        String tName = LocaleController.getInstance().getTranslitString(name);
                        if (name.equals(tName)) {
                            tName = null;
                        }
                        int found = 0;
                        int length = search.length;
                        while (true) {
                            if (i >= length) {
                                search1 = search12;
                                break;
                            }
                            String q = search[i];
                            if (name.contains(q)) {
                                search1 = search12;
                            } else if (tName == null || !tName.contains(q)) {
                                search1 = search12;
                                if (user.username != null && user.username.contains(q)) {
                                    found = 2;
                                }
                                if (found != 0) {
                                    i++;
                                    search12 = search1;
                                } else {
                                    if (found == 1) {
                                        resultArrayNames.add(AndroidUtilities.generateSearchName(user.first_name, user.last_name, q));
                                    } else {
                                        resultArrayNames.add(AndroidUtilities.generateSearchName("@" + user.username, null, "@" + q));
                                    }
                                    resultArray.add(user);
                                }
                            } else {
                                search1 = search12;
                            }
                            found = 1;
                            if (found != 0) {
                            }
                        }
                    }
                } else {
                    search1 = search12;
                }
            } else {
                search1 = search12;
            }
            a2++;
            search12 = search1;
            i = 0;
        }
        updateSearchResults(resultArray, resultArrayNames);
    }

    private void updateSearchResults(final ArrayList<TLObject> users, final ArrayList<CharSequence> names) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.adapters.SearchAdapter$$ExternalSyntheticLambda2
            @Override // java.lang.Runnable
            public final void run() {
                SearchAdapter.this.m2539xcd39dcfa(users, names);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$updateSearchResults$2$im-uecahxyvjd-ui-adapters-SearchAdapter  reason: not valid java name */
    public /* synthetic */ void m2539xcd39dcfa(ArrayList users, ArrayList names) {
        this.searchResult = users;
        this.searchResultNames = names;
        this.searchAdapterHelper.mergeResults(users);
        notifyDataSetChanged();
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
    public boolean isEnabled(RecyclerView.ViewHolder holder) {
        int type = holder.getItemViewType();
        return type == 0 || type == 2;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemCount() {
        int count = this.searchResult.size();
        int globalCount = this.searchAdapterHelper.getGlobalSearch().size();
        if (globalCount != 0) {
            count += globalCount + 1;
        }
        int phoneCount = this.searchAdapterHelper.getPhoneSearch().size();
        if (phoneCount != 0) {
            return count + phoneCount;
        }
        return count;
    }

    public boolean isGlobalSearch(int i) {
        int localCount = this.searchResult.size();
        int globalCount = this.searchAdapterHelper.getGlobalSearch().size();
        int phoneCount = this.searchAdapterHelper.getPhoneSearch().size();
        if (i >= 0 && i < localCount) {
            return false;
        }
        if ((i > localCount && i < localCount + phoneCount) || i <= localCount + phoneCount || i > globalCount + phoneCount + localCount) {
            return false;
        }
        return true;
    }

    public Object getItem(int i) {
        int localCount = this.searchResult.size();
        int globalCount = this.searchAdapterHelper.getGlobalSearch().size();
        int phoneCount = this.searchAdapterHelper.getPhoneSearch().size();
        if (i >= 0 && i < localCount) {
            return this.searchResult.get(i);
        }
        int i2 = i - localCount;
        if (i2 >= 0 && i2 < phoneCount) {
            return this.searchAdapterHelper.getPhoneSearch().get(i2);
        }
        int i3 = i2 - phoneCount;
        if (i3 > 0 && i3 <= globalCount) {
            return this.searchAdapterHelper.getGlobalSearch().get(i3 - 1);
        }
        return null;
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view;
        switch (viewType) {
            case 0:
                if (this.useUserCell) {
                    view = new UserCell(this.mContext, 1, 1, false);
                    if (this.checkedMap != null) {
                        ((UserCell) view).setChecked(false, false);
                    }
                } else {
                    view = new ProfileSearchCell(this.mContext);
                }
                view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                break;
            case 1:
                view = new GraySectionCell(this.mContext);
                break;
            default:
                view = new TextCell(this.mContext, 16);
                break;
        }
        return new RecyclerListView.Holder(view);
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Removed duplicated region for block: B:49:0x0140  */
    /* JADX WARN: Removed duplicated region for block: B:61:0x0167  */
    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        int id;
        String un;
        String foundUserName;
        CharSequence username;
        CharSequence name;
        boolean z = false;
        switch (holder.getItemViewType()) {
            case 0:
                TLObject object = (TLObject) getItem(position);
                if (object != null) {
                    if (object instanceof TLRPC.User) {
                        String un2 = ((TLRPC.User) object).username;
                        int id2 = ((TLRPC.User) object).id;
                        id = id2;
                        un = un2;
                    } else if (!(object instanceof TLRPC.Chat)) {
                        id = 0;
                        un = null;
                    } else {
                        String un3 = ((TLRPC.Chat) object).username;
                        int id3 = ((TLRPC.Chat) object).id;
                        id = id3;
                        un = un3;
                    }
                    CharSequence name2 = null;
                    if (position < this.searchResult.size()) {
                        name2 = this.searchResultNames.get(position);
                        if (name2 != null && un != null && un.length() > 0 && name2.toString().startsWith("@" + un)) {
                            username = name2;
                            name = null;
                            if (this.useUserCell) {
                                UserCell userCell = (UserCell) holder.itemView;
                                userCell.setMiViewType(this.miViewType);
                                if (this.miViewType == 0) {
                                    userCell.setData(object, name, username, 0);
                                } else {
                                    userCell.setData(object, name, null, 0);
                                }
                                SparseArray<?> sparseArray = this.checkedMap;
                                if (sparseArray != null) {
                                    userCell.setChecked(sparseArray.indexOfKey(id) >= 0, false);
                                    return;
                                }
                                return;
                            }
                            ProfileSearchCell profileSearchCell = (ProfileSearchCell) holder.itemView;
                            profileSearchCell.setMiViewType(this.miViewType);
                            if (this.miViewType == 0) {
                                profileSearchCell.setData(object, null, name, username, false, false);
                            } else {
                                profileSearchCell.setData(object, null, name, null, false, false);
                            }
                            if (position != getItemCount() - 1 && position != this.searchResult.size() - 1) {
                                z = true;
                            }
                            profileSearchCell.useSeparator = z;
                            if (getItemCount() == 1) {
                                profileSearchCell.setBackground(Theme.createRoundRectDrawable(AndroidUtilities.dp(5.0f), Theme.getColor(Theme.key_windowBackgroundWhite)));
                                return;
                            } else if (position == 0) {
                                profileSearchCell.setBackground(Theme.createRoundRectDrawable(AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), 0.0f, 0.0f, Theme.getColor(Theme.key_windowBackgroundWhite)));
                                return;
                            } else if (position == getItemCount() - 1) {
                                profileSearchCell.setBackground(Theme.createRoundRectDrawable(0.0f, 0.0f, AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), Theme.getColor(Theme.key_windowBackgroundWhite)));
                                return;
                            } else {
                                return;
                            }
                        }
                    } else if (position > this.searchResult.size() && un != null) {
                        String foundUserName2 = this.searchAdapterHelper.getLastFoundUsername();
                        if (!foundUserName2.startsWith("@")) {
                            foundUserName = foundUserName2;
                        } else {
                            foundUserName = foundUserName2.substring(1);
                        }
                        try {
                            SpannableStringBuilder spannableStringBuilder = new SpannableStringBuilder();
                            spannableStringBuilder.append((CharSequence) "@");
                            spannableStringBuilder.append((CharSequence) un);
                            int indexOfIgnoreCase = AndroidUtilities.indexOfIgnoreCase(un, foundUserName);
                            int index = indexOfIgnoreCase;
                            if (indexOfIgnoreCase != -1) {
                                int len = foundUserName.length();
                                if (index == 0) {
                                    len++;
                                } else {
                                    index++;
                                }
                                spannableStringBuilder.setSpan(new ForegroundColorSpan(Theme.getColor(Theme.key_windowBackgroundWhiteBlueText4)), index, index + len, 33);
                            }
                            name = null;
                            username = spannableStringBuilder;
                        } catch (Exception e) {
                            FileLog.e(e);
                            username = un;
                            name = null;
                        }
                        if (this.useUserCell) {
                        }
                    }
                    username = null;
                    name = name2;
                    if (this.useUserCell) {
                    }
                } else {
                    return;
                }
                break;
            case 1:
                GraySectionCell cell = (GraySectionCell) holder.itemView;
                if (getItem(position) == null) {
                    cell.setText(LocaleController.getString("GlobalSearch", R.string.GlobalSearch));
                    return;
                } else {
                    cell.setText(LocaleController.getString("PhoneNumberSearch", R.string.UserNameOrPhoneNumberSearch));
                    return;
                }
            case 2:
                String str = (String) getItem(position);
                TextCell cell2 = (TextCell) holder.itemView;
                cell2.setColors(null, Theme.key_windowBackgroundWhiteBlueText2);
                cell2.setText(LocaleController.formatString("AddContactByPhone", R.string.AddContactByPhone, PhoneFormat.getInstance().format(Marker.ANY_NON_NULL_MARKER + str)), false);
                return;
            default:
                return;
        }
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemViewType(int i) {
        Object item = getItem(i);
        if (item == null) {
            return 1;
        }
        if (item instanceof String) {
            String str = (String) item;
            if ("section".equals(str)) {
                return 1;
            }
            return 2;
        }
        return 0;
    }
}
