package im.uecahxyvjd.ui.cells;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.RectF;
import android.text.Layout;
import android.text.Spannable;
import android.text.StaticLayout;
import android.text.TextUtils;
import android.text.style.ClickableSpan;
import android.text.style.URLSpan;
import android.view.MotionEvent;
import android.view.View;
import android.view.accessibility.AccessibilityNodeInfo;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.FileLoader;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.ImageLocation;
import im.uecahxyvjd.messenger.ImageReceiver;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.messenger.browser.Browser;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPCRedpacket;
import im.uecahxyvjd.ui.PhotoViewer;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.components.AvatarDrawable;
/* loaded from: classes9.dex */
public class ChatActionCell extends BaseCell {
    private AvatarDrawable avatarDrawable;
    private int currentAccount;
    private MessageObject currentMessageObject;
    private int customDate;
    private CharSequence customText;
    private ChatActionCellDelegate delegate;
    private boolean hasReplyMessage;
    private boolean imagePressed;
    private ImageReceiver imageReceiver;
    private float lastTouchX;
    private float lastTouchY;
    private URLSpan pressedLink;
    private ClickableSpan pressedRedLink;
    private int previousWidth;
    private int textHeight;
    private StaticLayout textLayout;
    private int textWidth;
    private int textX;
    private int textXLeft;
    private int textY;
    private boolean wasLayout;

    /* loaded from: classes9.dex */
    public interface ChatActionCellDelegate {
        void didClickImage(ChatActionCell chatActionCell);

        void didLongPress(ChatActionCell chatActionCell, float f, float f2);

        void didPressBotButton(MessageObject messageObject, TLRPC.KeyboardButton keyboardButton);

        void didPressReplyMessage(ChatActionCell chatActionCell, int i);

        void didRedUrl(MessageObject messageObject);

        void needOpenUserProfile(int i);

        /* renamed from: im.uecahxyvjd.ui.cells.ChatActionCell$ChatActionCellDelegate$-CC */
        /* loaded from: classes9.dex */
        public final /* synthetic */ class CC {
            public static void $default$didClickImage(ChatActionCellDelegate _this, ChatActionCell cell) {
            }

            public static void $default$didLongPress(ChatActionCellDelegate _this, ChatActionCell cell, float x, float y) {
            }

            public static void $default$needOpenUserProfile(ChatActionCellDelegate _this, int uid) {
            }

            public static void $default$didPressBotButton(ChatActionCellDelegate _this, MessageObject messageObject, TLRPC.KeyboardButton button) {
            }

            public static void $default$didPressReplyMessage(ChatActionCellDelegate _this, ChatActionCell cell, int id) {
            }

            public static void $default$didRedUrl(ChatActionCellDelegate _this, MessageObject messageObject) {
            }
        }
    }

    public ChatActionCell(Context context) {
        super(context);
        this.currentAccount = UserConfig.selectedAccount;
        ImageReceiver imageReceiver = new ImageReceiver(this);
        this.imageReceiver = imageReceiver;
        imageReceiver.setRoundRadius(AndroidUtilities.dp(32.0f));
        this.avatarDrawable = new AvatarDrawable();
    }

    public void setDelegate(ChatActionCellDelegate delegate) {
        this.delegate = delegate;
    }

    public void setCustomDate(int date, boolean scheduled) {
        if (this.customDate == date) {
            return;
        }
        CharSequence newText = scheduled ? LocaleController.formatString("MessageScheduledOn", R.string.MessageScheduledOn, LocaleController.formatDateChat(date)) : LocaleController.formatDateChat(date);
        CharSequence charSequence = this.customText;
        if (charSequence != null && TextUtils.equals(newText, charSequence)) {
            return;
        }
        this.customDate = date;
        this.customText = newText;
        if (getMeasuredWidth() != 0) {
            createLayout(this.customText, getMeasuredWidth());
            invalidate();
        }
        if (!this.wasLayout) {
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.cells.ChatActionCell$$ExternalSyntheticLambda0
                @Override // java.lang.Runnable
                public final void run() {
                    ChatActionCell.this.requestLayout();
                }
            });
        } else {
            buildLayout();
        }
    }

    public void setMessageObject(MessageObject messageObject) {
        if (this.currentMessageObject == messageObject && (this.hasReplyMessage || messageObject.replyMessageObject == null)) {
            return;
        }
        this.currentMessageObject = messageObject;
        messageObject.setDelegate(new MessageObject.Delegate() { // from class: im.uecahxyvjd.ui.cells.ChatActionCell.1
            {
                ChatActionCell.this = this;
            }

            @Override // im.uecahxyvjd.messenger.MessageObject.Delegate
            public void onClickRed() {
                if (ChatActionCell.this.delegate != null) {
                    ChatActionCell.this.delegate.didRedUrl(ChatActionCell.this.currentMessageObject);
                }
            }
        });
        this.hasReplyMessage = messageObject.replyMessageObject != null;
        this.previousWidth = 0;
        if (this.currentMessageObject.type == 11) {
            int id = 0;
            if (messageObject.messageOwner.to_id != null) {
                if (messageObject.messageOwner.to_id.chat_id != 0) {
                    id = messageObject.messageOwner.to_id.chat_id;
                } else if (messageObject.messageOwner.to_id.channel_id != 0) {
                    id = messageObject.messageOwner.to_id.channel_id;
                } else {
                    id = messageObject.messageOwner.to_id.user_id;
                    if (id == UserConfig.getInstance(this.currentAccount).getClientUserId()) {
                        id = messageObject.messageOwner.from_id;
                    }
                }
            }
            this.avatarDrawable.setInfo(id, null, null);
            if (this.currentMessageObject.messageOwner.action instanceof TLRPC.TL_messageActionUserUpdatedPhoto) {
                this.imageReceiver.setImage(null, null, this.avatarDrawable, null, this.currentMessageObject, 0);
            } else {
                TLRPC.PhotoSize photo = FileLoader.getClosestPhotoSizeWithSize(this.currentMessageObject.photoThumbs, AndroidUtilities.dp(64.0f));
                if (photo != null) {
                    this.imageReceiver.setImage(ImageLocation.getForObject(photo, this.currentMessageObject.photoThumbsObject), "50_50", this.avatarDrawable, null, this.currentMessageObject, 0);
                } else {
                    this.imageReceiver.setImageBitmap(this.avatarDrawable);
                }
            }
            this.imageReceiver.setVisible(true ^ PhotoViewer.isShowingImage(this.currentMessageObject), false);
        } else {
            this.imageReceiver.setImageBitmap((Bitmap) null);
        }
        requestLayout();
    }

    public MessageObject getMessageObject() {
        return this.currentMessageObject;
    }

    public ImageReceiver getPhotoImage() {
        return this.imageReceiver;
    }

    @Override // im.uecahxyvjd.ui.cells.BaseCell
    protected void onLongPress() {
        ChatActionCellDelegate chatActionCellDelegate = this.delegate;
        if (chatActionCellDelegate != null) {
            chatActionCellDelegate.didLongPress(this, this.lastTouchX, this.lastTouchY);
        }
    }

    @Override // android.view.ViewGroup, android.view.View
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
    }

    @Override // android.view.ViewGroup, android.view.View
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        this.wasLayout = false;
    }

    @Override // android.view.View
    public boolean onTouchEvent(MotionEvent event) {
        if (this.currentMessageObject == null) {
            return super.onTouchEvent(event);
        }
        float x = event.getX();
        this.lastTouchX = x;
        float y = event.getY();
        this.lastTouchY = y;
        boolean result = false;
        if (event.getAction() == 0) {
            if (this.delegate != null) {
                if (this.currentMessageObject.type == 11 && this.imageReceiver.isInsideImage(x, y)) {
                    this.imagePressed = true;
                    result = true;
                }
                if (result) {
                    startCheckLongPress();
                }
            }
        } else {
            if (event.getAction() != 2) {
                cancelCheckLongPress();
            }
            if (this.imagePressed) {
                if (event.getAction() == 1) {
                    this.imagePressed = false;
                    ChatActionCellDelegate chatActionCellDelegate = this.delegate;
                    if (chatActionCellDelegate != null) {
                        chatActionCellDelegate.didClickImage(this);
                        playSoundEffect(0);
                    }
                } else if (event.getAction() == 3) {
                    this.imagePressed = false;
                } else if (event.getAction() == 2 && !this.imageReceiver.isInsideImage(x, y)) {
                    this.imagePressed = false;
                }
            }
        }
        if (!result && (event.getAction() == 0 || ((this.pressedLink != null || this.pressedRedLink != null) && event.getAction() == 1))) {
            int i = this.textX;
            if (x >= i) {
                int i2 = this.textY;
                if (y >= i2 && x <= i + this.textWidth && y <= this.textHeight + i2) {
                    float x2 = x - this.textXLeft;
                    int line = this.textLayout.getLineForVertical((int) (y - i2));
                    int off = this.textLayout.getOffsetForHorizontal(line, x2);
                    float left = this.textLayout.getLineLeft(line);
                    if (left <= x2 && this.textLayout.getLineWidth(line) + left >= x2 && (this.currentMessageObject.messageText instanceof Spannable) && !(this.currentMessageObject.messageOwner.action instanceof TLRPCRedpacket.CL_messagesActionReceivedRpkTransfer)) {
                        Spannable buffer = (Spannable) this.currentMessageObject.messageText;
                        URLSpan[] link = (URLSpan[]) buffer.getSpans(off, off, URLSpan.class);
                        if (link.length != 0) {
                            if (event.getAction() == 0) {
                                this.pressedLink = link[0];
                                result = true;
                            } else if (link[0] == this.pressedLink) {
                                if (this.delegate != null) {
                                    String url = link[0].getURL();
                                    if (url.startsWith("game")) {
                                        this.delegate.didPressReplyMessage(this, this.currentMessageObject.messageOwner.reply_to_msg_id);
                                    } else if (url.startsWith("http")) {
                                        Browser.openUrl(getContext(), url);
                                    } else {
                                        this.delegate.needOpenUserProfile(Integer.parseInt(url));
                                    }
                                }
                                result = true;
                            }
                        } else {
                            this.pressedLink = null;
                        }
                    } else if (this.currentMessageObject.messageOwner.action instanceof TLRPCRedpacket.CL_messagesActionReceivedRpkTransfer) {
                        Spannable buffer2 = (Spannable) this.currentMessageObject.messageText;
                        ClickableSpan[] link2 = (ClickableSpan[]) buffer2.getSpans(off, off, ClickableSpan.class);
                        if (link2.length != 0) {
                            if (event.getAction() == 0) {
                                this.pressedRedLink = link2[0];
                                result = true;
                            } else if (link2[0] == this.pressedRedLink) {
                                ChatActionCellDelegate chatActionCellDelegate2 = this.delegate;
                                if (chatActionCellDelegate2 != null) {
                                    chatActionCellDelegate2.didRedUrl(this.currentMessageObject);
                                }
                                result = true;
                            }
                        } else {
                            this.pressedRedLink = null;
                        }
                    } else {
                        this.pressedRedLink = null;
                    }
                }
            }
            this.pressedLink = null;
            this.pressedRedLink = null;
        }
        if (!result) {
            return super.onTouchEvent(event);
        }
        return result;
    }

    private void createLayout(CharSequence text, int width) {
        int maxWidth = width - AndroidUtilities.dp(30.0f);
        MessageObject messageObject = this.currentMessageObject;
        if (messageObject != null && messageObject.messageOwner != null && this.currentMessageObject.messageOwner.action != null && (this.currentMessageObject.messageOwner.action instanceof TLRPCRedpacket.CL_messagesActionReceivedRpkTransfer)) {
            Theme.chat_actionTextPaint.setColor(Theme.getColor(Theme.key_chat_redpacketServiceText));
            Theme.chat_actionTextPaint.linkColor = Theme.getColor(Theme.key_chat_redpacketLinkServiceText);
        } else {
            Theme.chat_actionTextPaint.setColor(Theme.getColor(Theme.key_chat_serviceText));
            Theme.chat_actionTextPaint.linkColor = Theme.getColor(Theme.key_chat_serviceLink);
        }
        StaticLayout staticLayout = new StaticLayout(text, Theme.chat_actionTextPaint, maxWidth, Layout.Alignment.ALIGN_CENTER, 1.0f, 0.0f, false);
        this.textLayout = staticLayout;
        this.textHeight = 0;
        this.textWidth = 0;
        try {
            int linesCount = staticLayout.getLineCount();
            for (int a2 = 0; a2 < linesCount; a2++) {
                try {
                    float lineWidth = this.textLayout.getLineWidth(a2);
                    if (lineWidth > maxWidth) {
                        lineWidth = maxWidth;
                    }
                    this.textHeight = (int) Math.max(this.textHeight, Math.ceil(this.textLayout.getLineBottom(a2)));
                    this.textWidth = (int) Math.max(this.textWidth, Math.ceil(lineWidth));
                } catch (Exception e) {
                    FileLog.e(e);
                    return;
                }
            }
        } catch (Exception e2) {
            FileLog.e(e2);
        }
        this.textX = (width - this.textWidth) / 2;
        this.textY = AndroidUtilities.dp(11.0f);
        this.textXLeft = (width - this.textLayout.getWidth()) / 2;
    }

    @Override // android.view.View
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        if (this.currentMessageObject == null && this.customText == null) {
            setMeasuredDimension(View.MeasureSpec.getSize(widthMeasureSpec), this.textHeight + AndroidUtilities.dp(20.0f));
            return;
        }
        int width = Math.max(AndroidUtilities.dp(30.0f), View.MeasureSpec.getSize(widthMeasureSpec));
        if (this.previousWidth != width) {
            this.wasLayout = true;
            this.previousWidth = width;
            buildLayout();
        }
        MessageObject messageObject = this.currentMessageObject;
        int i = 0;
        if (messageObject != null && (messageObject.messageOwner.media instanceof TLRPCRedpacket.CL_messagesPayBillOverMedia) && TextUtils.isEmpty(this.currentMessageObject.messageText)) {
            setMeasuredDimension(width, 0);
            return;
        }
        int i2 = this.textHeight;
        MessageObject messageObject2 = this.currentMessageObject;
        if (messageObject2 != null && messageObject2.type == 11) {
            i = 70;
        }
        setMeasuredDimension(width, i2 + AndroidUtilities.dp(20 + i));
    }

    private void buildLayout() {
        CharSequence text;
        MessageObject messageObject = this.currentMessageObject;
        if (messageObject != null) {
            if (messageObject.messageOwner != null && this.currentMessageObject.messageOwner.media != null && this.currentMessageObject.messageOwner.media.ttl_seconds != 0) {
                if (this.currentMessageObject.messageOwner.media.photo instanceof TLRPC.TL_photoEmpty) {
                    text = LocaleController.getString("AttachPhotoExpired", R.string.AttachPhotoExpired);
                } else if (this.currentMessageObject.messageOwner.media.document instanceof TLRPC.TL_documentEmpty) {
                    text = LocaleController.getString("AttachVideoExpired", R.string.AttachVideoExpired);
                } else {
                    text = this.currentMessageObject.messageText;
                }
            } else {
                text = this.currentMessageObject.messageText;
            }
        } else {
            text = this.customText;
        }
        createLayout(text, this.previousWidth);
        MessageObject messageObject2 = this.currentMessageObject;
        if (messageObject2 != null && messageObject2.type == 11) {
            this.imageReceiver.setImageCoords((this.previousWidth - AndroidUtilities.dp(64.0f)) / 2, this.textHeight + AndroidUtilities.dp(15.0f), AndroidUtilities.dp(64.0f), AndroidUtilities.dp(64.0f));
        }
    }

    public int getCustomDate() {
        return this.customDate;
    }

    private int findMaxWidthAroundLine(int line) {
        int width = (int) Math.ceil(this.textLayout.getLineWidth(line));
        int count = this.textLayout.getLineCount();
        for (int a2 = line + 1; a2 < count; a2++) {
            int w = (int) Math.ceil(this.textLayout.getLineWidth(a2));
            if (Math.abs(w - width) >= AndroidUtilities.dp(10.0f)) {
                break;
            }
            width = Math.max(w, width);
        }
        for (int a3 = line - 1; a3 >= 0; a3--) {
            int w2 = (int) Math.ceil(this.textLayout.getLineWidth(a3));
            if (Math.abs(w2 - width) >= AndroidUtilities.dp(10.0f)) {
                break;
            }
            width = Math.max(w2, width);
        }
        return width;
    }

    @Override // android.view.View
    protected void onDraw(Canvas canvas) {
        MessageObject messageObject = this.currentMessageObject;
        if (messageObject != null && messageObject.type == 11) {
            this.imageReceiver.draw(canvas);
        }
        StaticLayout staticLayout = this.textLayout;
        if (staticLayout != null) {
            int count = staticLayout.getLineCount();
            int previousLineBottom = 0;
            int finalWidth = AndroidUtilities.dp(50.0f);
            int finalHeight = AndroidUtilities.dp(6.0f);
            int finalY = AndroidUtilities.dp(8.0f);
            for (int a2 = 0; a2 < count; a2++) {
                int width = findMaxWidthAroundLine(a2);
                if (width > finalWidth) {
                    finalWidth = width;
                }
                int lineBottom = this.textLayout.getLineBottom(a2);
                int height = lineBottom - previousLineBottom;
                finalHeight += height;
                previousLineBottom = lineBottom;
            }
            int finalWidth2 = finalWidth + AndroidUtilities.dp(11.0f);
            int finalX = (getMeasuredWidth() - finalWidth2) / 2;
            RectF rectF = new RectF(finalX, finalY, finalX + finalWidth2, finalHeight + finalY);
            canvas.drawRoundRect(rectF, AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), Theme.chat_actionBackgroundPaint2);
            canvas.save();
            canvas.translate(this.textXLeft, this.textY);
            MessageObject messageObject2 = this.currentMessageObject;
            if (messageObject2 != null && messageObject2.messageOwner != null && this.currentMessageObject.messageOwner.action != null && (this.currentMessageObject.messageOwner.action instanceof TLRPCRedpacket.CL_messagesActionReceivedRpkTransfer)) {
                Theme.chat_actionTextPaint.setColor(Theme.getColor(Theme.key_chat_redpacketServiceText));
                Theme.chat_actionTextPaint.linkColor = Theme.getColor(Theme.key_chat_redpacketServiceText);
            } else {
                Theme.chat_actionTextPaint.setColor(Theme.getColor(Theme.key_chat_serviceText));
                Theme.chat_actionTextPaint.linkColor = Theme.getColor(Theme.key_chat_serviceLink);
            }
            this.textLayout.draw(canvas);
            canvas.restore();
        }
    }

    @Override // android.view.View
    public void onInitializeAccessibilityNodeInfo(AccessibilityNodeInfo info) {
        super.onInitializeAccessibilityNodeInfo(info);
        if (TextUtils.isEmpty(this.customText) && this.currentMessageObject == null) {
            return;
        }
        info.setText(!TextUtils.isEmpty(this.customText) ? this.customText : this.currentMessageObject.messageText);
        info.setEnabled(true);
    }
}
