package im.uecahxyvjd.ui.cells;

import android.content.Context;
import android.graphics.Canvas;
import android.net.Uri;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.text.TextUtils;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.accessibility.AccessibilityNodeInfo;
import android.widget.FrameLayout;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.FileLoader;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.ImageLocation;
import im.uecahxyvjd.messenger.ImageReceiver;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.messenger.browser.Browser;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.components.CheckBox2;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.LetterDrawable;
import im.uecahxyvjd.ui.components.LinkPath;
import java.util.ArrayList;
import java.util.Locale;
/* loaded from: classes9.dex */
public class SharedLinkCell extends FrameLayout {
    private CheckBox2 checkBox;
    private boolean checkingForLongPress;
    private SharedLinkCellDelegate delegate;
    private int description2Y;
    private StaticLayout descriptionLayout;
    private StaticLayout descriptionLayout2;
    private TextPaint descriptionTextPaint;
    private int descriptionY;
    private boolean drawLinkImageView;
    private LetterDrawable letterDrawable;
    private ImageReceiver linkImageView;
    private ArrayList<StaticLayout> linkLayout;
    private boolean linkPreviewPressed;
    private int linkY;
    ArrayList<String> links;
    private MessageObject message;
    private boolean needDivider;
    private CheckForLongPress pendingCheckForLongPress;
    private CheckForTap pendingCheckForTap;
    private int pressCount;
    private int pressedLink;
    private StaticLayout titleLayout;
    private TextPaint titleTextPaint;
    private int titleY;
    private LinkPath urlPath;

    /* loaded from: classes9.dex */
    public interface SharedLinkCellDelegate {
        boolean canPerformActions();

        void needOpenWebView(TLRPC.WebPage webPage);

        void onLinkLongPress(String str);
    }

    static /* synthetic */ int access$104(SharedLinkCell x0) {
        int i = x0.pressCount + 1;
        x0.pressCount = i;
        return i;
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes9.dex */
    public final class CheckForTap implements Runnable {
        private CheckForTap() {
        }

        @Override // java.lang.Runnable
        public void run() {
            if (SharedLinkCell.this.pendingCheckForLongPress == null) {
                SharedLinkCell.this.pendingCheckForLongPress = new CheckForLongPress();
            }
            SharedLinkCell.this.pendingCheckForLongPress.currentPressCount = SharedLinkCell.access$104(SharedLinkCell.this);
            SharedLinkCell sharedLinkCell = SharedLinkCell.this;
            sharedLinkCell.postDelayed(sharedLinkCell.pendingCheckForLongPress, ViewConfiguration.getLongPressTimeout() - ViewConfiguration.getTapTimeout());
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* loaded from: classes9.dex */
    public class CheckForLongPress implements Runnable {
        public int currentPressCount;

        CheckForLongPress() {
        }

        @Override // java.lang.Runnable
        public void run() {
            if (SharedLinkCell.this.checkingForLongPress && SharedLinkCell.this.getParent() != null && this.currentPressCount == SharedLinkCell.this.pressCount) {
                SharedLinkCell.this.checkingForLongPress = false;
                SharedLinkCell.this.performHapticFeedback(0);
                if (SharedLinkCell.this.pressedLink >= 0) {
                    SharedLinkCell.this.delegate.onLinkLongPress(SharedLinkCell.this.links.get(SharedLinkCell.this.pressedLink));
                }
                MotionEvent event = MotionEvent.obtain(0L, 0L, 3, 0.0f, 0.0f, 0);
                SharedLinkCell.this.onTouchEvent(event);
                event.recycle();
            }
        }
    }

    protected void startCheckLongPress() {
        if (this.checkingForLongPress) {
            return;
        }
        this.checkingForLongPress = true;
        if (this.pendingCheckForTap == null) {
            this.pendingCheckForTap = new CheckForTap();
        }
        postDelayed(this.pendingCheckForTap, ViewConfiguration.getTapTimeout());
    }

    protected void cancelCheckLongPress() {
        this.checkingForLongPress = false;
        CheckForLongPress checkForLongPress = this.pendingCheckForLongPress;
        if (checkForLongPress != null) {
            removeCallbacks(checkForLongPress);
        }
        CheckForTap checkForTap = this.pendingCheckForTap;
        if (checkForTap != null) {
            removeCallbacks(checkForTap);
        }
    }

    public SharedLinkCell(Context context) {
        super(context);
        this.checkingForLongPress = false;
        this.pendingCheckForLongPress = null;
        this.pressCount = 0;
        this.pendingCheckForTap = null;
        this.links = new ArrayList<>();
        this.linkLayout = new ArrayList<>();
        this.titleY = AndroidUtilities.dp(10.0f);
        this.descriptionY = AndroidUtilities.dp(30.0f);
        this.description2Y = AndroidUtilities.dp(30.0f);
        setFocusable(true);
        LinkPath linkPath = new LinkPath();
        this.urlPath = linkPath;
        linkPath.setUseRoundRect(true);
        TextPaint textPaint = new TextPaint(1);
        this.titleTextPaint = textPaint;
        textPaint.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
        this.titleTextPaint.setColor(Theme.getColor(Theme.key_windowBackgroundWhiteBlackText));
        this.descriptionTextPaint = new TextPaint(1);
        this.titleTextPaint.setTextSize(AndroidUtilities.dp(14.0f));
        this.descriptionTextPaint.setTextSize(AndroidUtilities.dp(14.0f));
        setWillNotDraw(false);
        ImageReceiver imageReceiver = new ImageReceiver(this);
        this.linkImageView = imageReceiver;
        imageReceiver.setRoundRadius(AndroidUtilities.dp(4.0f));
        this.letterDrawable = new LetterDrawable();
        CheckBox2 checkBox2 = new CheckBox2(context, 21);
        this.checkBox = checkBox2;
        checkBox2.setVisibility(4);
        this.checkBox.setColor(null, Theme.key_windowBackgroundWhite, Theme.key_checkboxCheck);
        this.checkBox.setDrawUnchecked(false);
        this.checkBox.setDrawBackgroundAsArc(2);
        addView(this.checkBox, LayoutHelper.createFrame(24.0f, 24.0f, (LocaleController.isRTL ? 5 : 3) | 48, LocaleController.isRTL ? 0.0f : 44.0f, 44.0f, LocaleController.isRTL ? 44.0f : 0.0f, 0.0f));
    }

    /* JADX WARN: Removed duplicated region for block: B:69:0x0180 A[Catch: Exception -> 0x023c, TryCatch #2 {Exception -> 0x023c, blocks: (B:52:0x0115, B:54:0x0119, B:57:0x011e, B:60:0x0124, B:62:0x012a, B:64:0x0165, B:66:0x0173, B:91:0x0204, B:93:0x0210, B:95:0x021c, B:96:0x0235, B:67:0x017c, B:69:0x0180, B:72:0x0198, B:74:0x019e, B:78:0x01ad, B:80:0x01b6, B:82:0x01c2, B:83:0x01c9, B:84:0x01e8, B:86:0x01ec, B:88:0x01fa, B:70:0x0193), top: B:201:0x0115 }] */
    /* JADX WARN: Removed duplicated region for block: B:70:0x0193 A[Catch: Exception -> 0x023c, TryCatch #2 {Exception -> 0x023c, blocks: (B:52:0x0115, B:54:0x0119, B:57:0x011e, B:60:0x0124, B:62:0x012a, B:64:0x0165, B:66:0x0173, B:91:0x0204, B:93:0x0210, B:95:0x021c, B:96:0x0235, B:67:0x017c, B:69:0x0180, B:72:0x0198, B:74:0x019e, B:78:0x01ad, B:80:0x01b6, B:82:0x01c2, B:83:0x01c9, B:84:0x01e8, B:86:0x01ec, B:88:0x01fa, B:70:0x0193), top: B:201:0x0115 }] */
    /* JADX WARN: Removed duplicated region for block: B:76:0x01aa  */
    /* JADX WARN: Removed duplicated region for block: B:82:0x01c2 A[Catch: Exception -> 0x023c, TryCatch #2 {Exception -> 0x023c, blocks: (B:52:0x0115, B:54:0x0119, B:57:0x011e, B:60:0x0124, B:62:0x012a, B:64:0x0165, B:66:0x0173, B:91:0x0204, B:93:0x0210, B:95:0x021c, B:96:0x0235, B:67:0x017c, B:69:0x0180, B:72:0x0198, B:74:0x019e, B:78:0x01ad, B:80:0x01b6, B:82:0x01c2, B:83:0x01c9, B:84:0x01e8, B:86:0x01ec, B:88:0x01fa, B:70:0x0193), top: B:201:0x0115 }] */
    /* JADX WARN: Removed duplicated region for block: B:91:0x0204 A[Catch: Exception -> 0x023c, TryCatch #2 {Exception -> 0x023c, blocks: (B:52:0x0115, B:54:0x0119, B:57:0x011e, B:60:0x0124, B:62:0x012a, B:64:0x0165, B:66:0x0173, B:91:0x0204, B:93:0x0210, B:95:0x021c, B:96:0x0235, B:67:0x017c, B:69:0x0180, B:72:0x0198, B:74:0x019e, B:78:0x01ad, B:80:0x01b6, B:82:0x01c2, B:83:0x01c9, B:84:0x01e8, B:86:0x01ec, B:88:0x01fa, B:70:0x0193), top: B:201:0x0115 }] */
    @Override // android.widget.FrameLayout, android.view.View
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        String webPageLink;
        boolean hasPhoto;
        String title;
        String description;
        String description2;
        int a2;
        CharSequence linkFinal;
        String description22;
        String link;
        int index;
        int index2;
        this.drawLinkImageView = false;
        this.descriptionLayout = null;
        this.titleLayout = null;
        this.descriptionLayout2 = null;
        this.linkLayout.clear();
        this.links.clear();
        int maxWidth = (View.MeasureSpec.getSize(widthMeasureSpec) - AndroidUtilities.dp(AndroidUtilities.leftBaseline)) - AndroidUtilities.dp(8.0f);
        String title2 = null;
        String description3 = null;
        int i = 1;
        if ((this.message.messageOwner.media instanceof TLRPC.TL_messageMediaWebPage) && (this.message.messageOwner.media.webpage instanceof TLRPC.TL_webPage)) {
            TLRPC.WebPage webPage = this.message.messageOwner.media.webpage;
            if (this.message.photoThumbs == null && webPage.photo != null) {
                this.message.generateThumbs(true);
            }
            boolean hasPhoto2 = (webPage.photo == null || this.message.photoThumbs == null) ? false : true;
            title2 = webPage.title;
            if (title2 == null) {
                title2 = webPage.site_name;
            }
            description3 = webPage.description;
            String webPageLink2 = webPage.url;
            webPageLink = webPageLink2;
            hasPhoto = hasPhoto2;
        } else {
            webPageLink = null;
            hasPhoto = false;
        }
        MessageObject messageObject = this.message;
        if (messageObject != null && !messageObject.messageOwner.entities.isEmpty()) {
            int a3 = 0;
            String str = description3;
            String title3 = title2;
            String link2 = null;
            String description4 = str;
            while (a3 < this.message.messageOwner.entities.size()) {
                TLRPC.MessageEntity entity = this.message.messageOwner.entities.get(a3);
                if (entity.length > 0 && entity.offset >= 0 && entity.offset < this.message.messageOwner.message.length()) {
                    if (entity.offset + entity.length > this.message.messageOwner.message.length()) {
                        entity.length = this.message.messageOwner.message.length() - entity.offset;
                    }
                    try {
                        if (a3 == 0 && webPageLink != null && (entity.offset != 0 || entity.length != this.message.messageOwner.message.length())) {
                            if (this.message.messageOwner.entities.size() == i) {
                                if (description4 == null) {
                                    String description23 = this.message.messageOwner.message;
                                    description22 = description23;
                                }
                            } else {
                                String description24 = this.message.messageOwner.message;
                                description22 = description24;
                            }
                            link = null;
                            if (!(entity instanceof TLRPC.TL_messageEntityTextUrl) && !(entity instanceof TLRPC.TL_messageEntityUrl)) {
                                if ((entity instanceof TLRPC.TL_messageEntityEmail) && (title3 == null || title3.length() == 0)) {
                                    link = "mailto:" + this.message.messageOwner.message.substring(entity.offset, entity.offset + entity.length);
                                    title3 = this.message.messageOwner.message.substring(entity.offset, entity.offset + entity.length);
                                    if (entity.offset == 0 || entity.length != this.message.messageOwner.message.length()) {
                                        description4 = this.message.messageOwner.message;
                                    }
                                }
                                if (link != null) {
                                    if (link.toLowerCase().indexOf("http") != 0 && link.toLowerCase().indexOf("mailto") != 0) {
                                        this.links.add("http://" + link);
                                    } else {
                                        this.links.add(link);
                                    }
                                }
                                link2 = description22;
                            }
                            if (!(entity instanceof TLRPC.TL_messageEntityUrl)) {
                                link = this.message.messageOwner.message.substring(entity.offset, entity.offset + entity.length);
                            } else {
                                link = entity.url;
                            }
                            if (title3 != null || title3.length() == 0) {
                                String title4 = link;
                                Uri uri = Uri.parse(title4);
                                title3 = uri.getHost();
                                if (title3 == null) {
                                    title3 = link;
                                }
                                if (title3 != null && (index = title3.lastIndexOf(46)) >= 0) {
                                    String title5 = title3.substring(0, index);
                                    index2 = title5.lastIndexOf(46);
                                    if (index2 >= 0) {
                                        title5 = title5.substring(index2 + 1);
                                    }
                                    title3 = title5.substring(0, 1).toUpperCase() + title5.substring(1);
                                }
                                if (entity.offset == 0 || entity.length != this.message.messageOwner.message.length()) {
                                    description4 = this.message.messageOwner.message;
                                }
                            }
                            if (link != null) {
                            }
                            link2 = description22;
                        }
                        if (!(entity instanceof TLRPC.TL_messageEntityTextUrl)) {
                            if (entity instanceof TLRPC.TL_messageEntityEmail) {
                                link = "mailto:" + this.message.messageOwner.message.substring(entity.offset, entity.offset + entity.length);
                                title3 = this.message.messageOwner.message.substring(entity.offset, entity.offset + entity.length);
                                if (entity.offset == 0) {
                                }
                                description4 = this.message.messageOwner.message;
                            }
                            if (link != null) {
                            }
                            link2 = description22;
                        }
                        if (!(entity instanceof TLRPC.TL_messageEntityUrl)) {
                        }
                        if (title3 != null) {
                        }
                        String title42 = link;
                        Uri uri2 = Uri.parse(title42);
                        title3 = uri2.getHost();
                        if (title3 == null) {
                        }
                        if (title3 != null) {
                            String title52 = title3.substring(0, index);
                            index2 = title52.lastIndexOf(46);
                            if (index2 >= 0) {
                            }
                            title3 = title52.substring(0, 1).toUpperCase() + title52.substring(1);
                        }
                        if (entity.offset == 0) {
                        }
                        description4 = this.message.messageOwner.message;
                        if (link != null) {
                        }
                        link2 = description22;
                    } catch (Exception e) {
                        FileLog.e(e);
                        link2 = description22;
                    }
                    description22 = link2;
                    link = null;
                }
                a3++;
                i = 1;
            }
            description2 = link2;
            title = title3;
            description = description4;
        } else {
            title = title2;
            description = description3;
            description2 = null;
        }
        if (webPageLink != null && this.links.isEmpty()) {
            this.links.add(webPageLink);
        }
        if (title != null) {
            try {
                StaticLayout generateStaticLayout = ChatMessageCell.generateStaticLayout(title, this.titleTextPaint, maxWidth, maxWidth, 0, 3);
                this.titleLayout = generateStaticLayout;
                if (generateStaticLayout.getLineCount() > 0) {
                    int i2 = this.titleY;
                    StaticLayout staticLayout = this.titleLayout;
                    this.descriptionY = i2 + staticLayout.getLineBottom(staticLayout.getLineCount() - 1) + AndroidUtilities.dp(4.0f);
                }
            } catch (Exception e2) {
                FileLog.e(e2);
            }
            this.letterDrawable.setTitle(title);
        }
        this.description2Y = this.descriptionY;
        StaticLayout staticLayout2 = this.titleLayout;
        int desctiptionLines = Math.max(1, 4 - (staticLayout2 != null ? staticLayout2.getLineCount() : 0));
        if (description != null) {
            try {
                StaticLayout generateStaticLayout2 = ChatMessageCell.generateStaticLayout(description, this.descriptionTextPaint, maxWidth, maxWidth, 0, desctiptionLines);
                this.descriptionLayout = generateStaticLayout2;
                if (generateStaticLayout2.getLineCount() > 0) {
                    int i3 = this.descriptionY;
                    StaticLayout staticLayout3 = this.descriptionLayout;
                    this.description2Y = i3 + staticLayout3.getLineBottom(staticLayout3.getLineCount() - 1) + AndroidUtilities.dp(5.0f);
                }
            } catch (Exception e3) {
                FileLog.e(e3);
            }
        }
        if (description2 != null) {
            try {
                this.descriptionLayout2 = ChatMessageCell.generateStaticLayout(description2, this.descriptionTextPaint, maxWidth, maxWidth, 0, desctiptionLines);
                if (this.descriptionLayout != null) {
                    this.description2Y += AndroidUtilities.dp(10.0f);
                }
            } catch (Exception e4) {
                FileLog.e(e4);
            }
        }
        if (!this.links.isEmpty()) {
            int a4 = 0;
            while (a4 < this.links.size()) {
                try {
                    String link3 = this.links.get(a4);
                    int width = (int) Math.ceil(this.descriptionTextPaint.measureText(link3));
                    linkFinal = TextUtils.ellipsize(link3.replace('\n', ' '), this.descriptionTextPaint, Math.min(width, maxWidth), TextUtils.TruncateAt.MIDDLE);
                    a2 = a4;
                } catch (Exception e5) {
                    e = e5;
                    a2 = a4;
                }
                try {
                    StaticLayout layout = new StaticLayout(linkFinal, this.descriptionTextPaint, maxWidth, Layout.Alignment.ALIGN_NORMAL, 1.0f, 0.0f, false);
                    this.linkY = this.description2Y;
                    StaticLayout staticLayout4 = this.descriptionLayout2;
                    if (staticLayout4 != null && staticLayout4.getLineCount() != 0) {
                        int i4 = this.linkY;
                        StaticLayout staticLayout5 = this.descriptionLayout2;
                        this.linkY = i4 + staticLayout5.getLineBottom(staticLayout5.getLineCount() - 1) + AndroidUtilities.dp(5.0f);
                    }
                    this.linkLayout.add(layout);
                } catch (Exception e6) {
                    e = e6;
                    FileLog.e(e);
                    a4 = a2 + 1;
                }
                a4 = a2 + 1;
            }
        }
        int maxPhotoWidth = AndroidUtilities.dp(52.0f);
        int x = LocaleController.isRTL ? (View.MeasureSpec.getSize(widthMeasureSpec) - AndroidUtilities.dp(10.0f)) - maxPhotoWidth : AndroidUtilities.dp(10.0f);
        this.letterDrawable.setBounds(x, AndroidUtilities.dp(11.0f), x + maxPhotoWidth, AndroidUtilities.dp(63.0f));
        if (hasPhoto) {
            TLRPC.PhotoSize currentPhotoObject = FileLoader.getClosestPhotoSizeWithSize(this.message.photoThumbs, maxPhotoWidth, true);
            TLRPC.PhotoSize currentPhotoObjectThumb = FileLoader.getClosestPhotoSizeWithSize(this.message.photoThumbs, 80);
            if (currentPhotoObjectThumb == currentPhotoObject) {
                currentPhotoObjectThumb = null;
            }
            currentPhotoObject.size = -1;
            if (currentPhotoObjectThumb != null) {
                currentPhotoObjectThumb.size = -1;
            }
            this.linkImageView.setImageCoords(x, AndroidUtilities.dp(11.0f), maxPhotoWidth, maxPhotoWidth);
            FileLoader.getAttachFileName(currentPhotoObject);
            String filter = String.format(Locale.US, "%d_%d", Integer.valueOf(maxPhotoWidth), Integer.valueOf(maxPhotoWidth));
            String thumbFilter = String.format(Locale.US, "%d_%d_b", Integer.valueOf(maxPhotoWidth), Integer.valueOf(maxPhotoWidth));
            this.linkImageView.setImage(ImageLocation.getForObject(currentPhotoObject, this.message.photoThumbsObject), filter, ImageLocation.getForObject(currentPhotoObjectThumb, this.message.photoThumbsObject), thumbFilter, 0, null, this.message, 0);
            this.drawLinkImageView = true;
        }
        int height = 0;
        StaticLayout staticLayout6 = this.titleLayout;
        if (staticLayout6 != null && staticLayout6.getLineCount() != 0) {
            StaticLayout staticLayout7 = this.titleLayout;
            height = 0 + staticLayout7.getLineBottom(staticLayout7.getLineCount() - 1) + AndroidUtilities.dp(4.0f);
        }
        StaticLayout staticLayout8 = this.descriptionLayout;
        if (staticLayout8 != null && staticLayout8.getLineCount() != 0) {
            StaticLayout staticLayout9 = this.descriptionLayout;
            height += staticLayout9.getLineBottom(staticLayout9.getLineCount() - 1) + AndroidUtilities.dp(5.0f);
        }
        StaticLayout staticLayout10 = this.descriptionLayout2;
        if (staticLayout10 != null && staticLayout10.getLineCount() != 0) {
            StaticLayout staticLayout11 = this.descriptionLayout2;
            height += staticLayout11.getLineBottom(staticLayout11.getLineCount() - 1) + AndroidUtilities.dp(5.0f);
            if (this.descriptionLayout != null) {
                height += AndroidUtilities.dp(10.0f);
            }
        }
        for (int a5 = 0; a5 < this.linkLayout.size(); a5++) {
            StaticLayout layout2 = this.linkLayout.get(a5);
            if (layout2.getLineCount() > 0) {
                height += layout2.getLineBottom(layout2.getLineCount() - 1);
            }
        }
        this.checkBox.measure(View.MeasureSpec.makeMeasureSpec(AndroidUtilities.dp(24.0f), 1073741824), View.MeasureSpec.makeMeasureSpec(AndroidUtilities.dp(24.0f), 1073741824));
        setMeasuredDimension(View.MeasureSpec.getSize(widthMeasureSpec), Math.max(AndroidUtilities.dp(76.0f), AndroidUtilities.dp(17.0f) + height) + (this.needDivider ? 1 : 0));
    }

    public void setLink(MessageObject messageObject, boolean divider) {
        this.needDivider = divider;
        resetPressedLink();
        this.message = messageObject;
        requestLayout();
    }

    public void setDelegate(SharedLinkCellDelegate sharedLinkCellDelegate) {
        this.delegate = sharedLinkCellDelegate;
    }

    public MessageObject getMessage() {
        return this.message;
    }

    @Override // android.view.ViewGroup, android.view.View
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        if (this.drawLinkImageView) {
            this.linkImageView.onDetachedFromWindow();
        }
    }

    @Override // android.view.ViewGroup, android.view.View
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        if (this.drawLinkImageView) {
            this.linkImageView.onAttachedToWindow();
        }
    }

    @Override // android.view.View
    public boolean onTouchEvent(MotionEvent event) {
        SharedLinkCellDelegate sharedLinkCellDelegate;
        boolean result = false;
        if (this.message != null && !this.linkLayout.isEmpty() && (sharedLinkCellDelegate = this.delegate) != null && sharedLinkCellDelegate.canPerformActions()) {
            if (event.getAction() == 0 || (this.linkPreviewPressed && event.getAction() == 1)) {
                int x = (int) event.getX();
                int y = (int) event.getY();
                boolean ok = false;
                int a2 = 0;
                int offset = 0;
                while (true) {
                    if (a2 >= this.linkLayout.size()) {
                        break;
                    }
                    StaticLayout layout = this.linkLayout.get(a2);
                    if (layout.getLineCount() > 0) {
                        int height = layout.getLineBottom(layout.getLineCount() - 1);
                        int linkPosX = AndroidUtilities.dp(LocaleController.isRTL ? 8.0f : AndroidUtilities.leftBaseline);
                        if (x >= linkPosX + layout.getLineLeft(0) && x <= linkPosX + layout.getLineWidth(0)) {
                            int i = this.linkY;
                            if (y >= i + offset && y <= i + offset + height) {
                                ok = true;
                                if (event.getAction() == 0) {
                                    resetPressedLink();
                                    this.pressedLink = a2;
                                    this.linkPreviewPressed = true;
                                    startCheckLongPress();
                                    try {
                                        this.urlPath.setCurrentLayout(layout, 0, 0.0f);
                                        layout.getSelectionPath(0, layout.getText().length(), this.urlPath);
                                    } catch (Exception e) {
                                        FileLog.e(e);
                                    }
                                    result = true;
                                } else if (this.linkPreviewPressed) {
                                    try {
                                        TLRPC.WebPage webPage = (this.pressedLink != 0 || this.message.messageOwner.media == null) ? null : this.message.messageOwner.media.webpage;
                                        if (webPage == null || webPage.embed_url == null || webPage.embed_url.length() == 0) {
                                            Browser.openUrl(getContext(), this.links.get(this.pressedLink));
                                        } else {
                                            this.delegate.needOpenWebView(webPage);
                                        }
                                    } catch (Exception e2) {
                                        FileLog.e(e2);
                                    }
                                    resetPressedLink();
                                    result = true;
                                }
                            }
                        }
                        offset += height;
                    }
                    a2++;
                }
                if (!ok) {
                    resetPressedLink();
                }
            } else if (event.getAction() == 3) {
                resetPressedLink();
            }
        } else {
            resetPressedLink();
        }
        return result || super.onTouchEvent(event);
    }

    public String getLink(int num) {
        if (num < 0 || num >= this.links.size()) {
            return null;
        }
        return this.links.get(num);
    }

    protected void resetPressedLink() {
        this.pressedLink = -1;
        this.linkPreviewPressed = false;
        cancelCheckLongPress();
        invalidate();
    }

    public void setChecked(boolean checked, boolean animated) {
        if (this.checkBox.getVisibility() != 0) {
            this.checkBox.setVisibility(0);
        }
        this.checkBox.setChecked(checked, animated);
    }

    @Override // android.view.View
    protected void onDraw(Canvas canvas) {
        if (this.titleLayout != null) {
            canvas.save();
            canvas.translate(AndroidUtilities.dp(LocaleController.isRTL ? 8.0f : AndroidUtilities.leftBaseline), this.titleY);
            this.titleLayout.draw(canvas);
            canvas.restore();
        }
        if (this.descriptionLayout != null) {
            this.descriptionTextPaint.setColor(Theme.getColor(Theme.key_windowBackgroundWhiteBlackText));
            canvas.save();
            canvas.translate(AndroidUtilities.dp(LocaleController.isRTL ? 8.0f : AndroidUtilities.leftBaseline), this.descriptionY);
            this.descriptionLayout.draw(canvas);
            canvas.restore();
        }
        if (this.descriptionLayout2 != null) {
            this.descriptionTextPaint.setColor(Theme.getColor(Theme.key_windowBackgroundWhiteBlackText));
            canvas.save();
            canvas.translate(AndroidUtilities.dp(LocaleController.isRTL ? 8.0f : AndroidUtilities.leftBaseline), this.description2Y);
            this.descriptionLayout2.draw(canvas);
            canvas.restore();
        }
        if (!this.linkLayout.isEmpty()) {
            this.descriptionTextPaint.setColor(Theme.getColor(Theme.key_windowBackgroundWhiteLinkText));
            int offset = 0;
            for (int a2 = 0; a2 < this.linkLayout.size(); a2++) {
                StaticLayout layout = this.linkLayout.get(a2);
                if (layout.getLineCount() > 0) {
                    canvas.save();
                    canvas.translate(AndroidUtilities.dp(LocaleController.isRTL ? 8.0f : AndroidUtilities.leftBaseline), this.linkY + offset);
                    if (this.pressedLink == a2) {
                        canvas.drawPath(this.urlPath, Theme.linkSelectionPaint);
                    }
                    layout.draw(canvas);
                    canvas.restore();
                    offset += layout.getLineBottom(layout.getLineCount() - 1);
                }
            }
        }
        this.letterDrawable.draw(canvas);
        if (this.drawLinkImageView) {
            this.linkImageView.draw(canvas);
        }
        if (this.needDivider) {
            if (LocaleController.isRTL) {
                canvas.drawLine(0.0f, getMeasuredHeight() - 1, getMeasuredWidth() - AndroidUtilities.dp(AndroidUtilities.leftBaseline), getMeasuredHeight() - 1, Theme.dividerPaint);
            } else {
                canvas.drawLine(AndroidUtilities.dp(AndroidUtilities.leftBaseline), getMeasuredHeight() - 1, getMeasuredWidth(), getMeasuredHeight() - 1, Theme.dividerPaint);
            }
        }
    }

    @Override // android.view.View
    public void onInitializeAccessibilityNodeInfo(AccessibilityNodeInfo info) {
        super.onInitializeAccessibilityNodeInfo(info);
        StringBuilder sb = new StringBuilder();
        StaticLayout staticLayout = this.titleLayout;
        if (staticLayout != null) {
            sb.append(staticLayout.getText());
        }
        if (this.descriptionLayout != null) {
            sb.append(", ");
            sb.append(this.descriptionLayout.getText());
        }
        if (this.descriptionLayout2 != null) {
            sb.append(", ");
            sb.append(this.descriptionLayout2.getText());
        }
        if (this.checkBox.isChecked()) {
            info.setChecked(true);
            info.setCheckable(true);
        }
    }
}
