package im.uecahxyvjd.ui.components;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.text.Html;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.SpannableStringBuilder;
import android.text.TextUtils;
import android.text.style.URLSpan;
import android.util.Base64;
import android.util.SparseArray;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import androidx.fragment.app.FragmentActivity;
import im.uecahxyvjd.messenger.AccountInstance;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.BuildVars;
import im.uecahxyvjd.messenger.ChatObject;
import im.uecahxyvjd.messenger.ContactsController;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.ImageLocation;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.messenger.MessagesStorage;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.NotificationsController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.SecretChatHelper;
import im.uecahxyvjd.messenger.SharedConfig;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.messenger.UserObject;
import im.uecahxyvjd.messenger.Utilities;
import im.uecahxyvjd.messenger.browser.Browser;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.SerializedData;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPCChats;
import im.uecahxyvjd.ui.CacheControlActivity;
import im.uecahxyvjd.ui.ChatActivity;
import im.uecahxyvjd.ui.LanguageSelectActivity;
import im.uecahxyvjd.ui.LaunchActivity;
import im.uecahxyvjd.ui.NotificationsCustomSettingsActivity;
import im.uecahxyvjd.ui.NotificationsSettingsActivity;
import im.uecahxyvjd.ui.ProfileNotificationsActivity;
import im.uecahxyvjd.ui.ReportOtherActivity;
import im.uecahxyvjd.ui.actionbar.AlertDialog;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.BottomSheet;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.cells.AccountSelectCell;
import im.uecahxyvjd.ui.cells.CheckBoxCell;
import im.uecahxyvjd.ui.cells.RadioColorCell;
import im.uecahxyvjd.ui.cells.TextColorCell;
import im.uecahxyvjd.ui.components.NumberPicker;
import im.uecahxyvjd.ui.components.toast.ToastUtils;
import im.uecahxyvjd.ui.hviews.dialogs.XDialog;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Locale;
import java.util.concurrent.CountDownLatch;
/* loaded from: classes10.dex */
public class AlertsCreator {

    /* loaded from: classes10.dex */
    public interface AccountSelectDelegate {
        void didSelectAccount(int i);
    }

    /* loaded from: classes10.dex */
    public interface DatePickerDelegate {
        void didSelectDate(int i, int i2, int i3);
    }

    /* loaded from: classes10.dex */
    public interface PaymentAlertDelegate {
        void didPressedNewCard();
    }

    /* loaded from: classes10.dex */
    public interface ScheduleDatePickerDelegate {
        void didSelectDate(boolean z, int i);
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    public static Dialog processError(int currentAccount, TLRPC.TL_error error, BaseFragment fragment, TLObject request, Object... args) {
        char c;
        char c2;
        char c3;
        char c4;
        if (error.code == 406 || error.text == null) {
            return null;
        }
        if ((request instanceof TLRPC.TL_account_saveSecureValue) || (request instanceof TLRPC.TL_account_getAuthorizationForm)) {
            if (error.text.contains("PHONE_NUMBER_INVALID")) {
                showSimpleAlert(fragment, LocaleController.getString("InvalidPhoneNumber", R.string.InvalidPhoneNumber));
                return null;
            } else if (error.text.startsWith("FLOOD_WAIT")) {
                showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
                return null;
            } else if (!"APP_VERSION_OUTDATED".equals(error.text)) {
                showSimpleAlert(fragment, LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred) + "\n" + error.text);
                return null;
            } else {
                showUpdateAppAlert(fragment.getParentActivity(), LocaleController.getString("UpdateAppAlert", R.string.UpdateAppAlert), true);
                return null;
            }
        } else if ((request instanceof TLRPC.TL_channels_joinChannel) || (request instanceof TLRPC.TL_channels_editAdmin) || (request instanceof TLRPC.TL_channels_inviteToChannel) || (request instanceof TLRPC.TL_messages_addChatUser) || (request instanceof TLRPC.TL_messages_startBot) || (request instanceof TLRPC.TL_channels_editBanned) || (request instanceof TLRPC.TL_messages_editChatDefaultBannedRights) || (request instanceof TLRPC.TL_messages_editChatAdmin) || (request instanceof TLRPC.TL_messages_migrateChat)) {
            if (fragment != null) {
                showAddUserAlert(error.text, fragment, (args == null || args.length <= 0) ? false : ((Boolean) args[0]).booleanValue(), request);
                return null;
            } else if (error.text.equals("PEER_FLOOD")) {
                NotificationCenter.getInstance(currentAccount).postNotificationName(NotificationCenter.needShowAlert, 1);
                return null;
            } else {
                return null;
            }
        } else if (request instanceof TLRPC.TL_messages_createChat) {
            if (error.text.startsWith("FLOOD_WAIT")) {
                showFloodWaitAlert(error.text, fragment);
                return null;
            }
            showAddUserAlert(error.text, fragment, false, request);
            return null;
        } else if (request instanceof TLRPCChats.TL_channels_createChannel_v1) {
            if (error.text.startsWith("FLOOD_WAIT")) {
                showFloodWaitAlert(error.text, fragment);
                return null;
            } else if (error.text.startsWith("GROUP_REPEAT_CREATE")) {
                TLRPCChats.TL_channels_createChannel_v1 reqCreateChannel = (TLRPCChats.TL_channels_createChannel_v1) request;
                if (reqCreateChannel.megagroup) {
                    showSimpleAlert(fragment, LocaleController.getString(R.string.ThisGroupExist));
                    return null;
                } else if (reqCreateChannel.broadcast) {
                    showSimpleAlert(fragment, LocaleController.getString(R.string.ThisChannelExist));
                    return null;
                } else {
                    return null;
                }
            } else {
                showAddUserAlert(error.text, fragment, false, request);
                return null;
            }
        } else if (request instanceof TLRPC.TL_channels_createChannel) {
            if (error.text.startsWith("FLOOD_WAIT")) {
                showFloodWaitAlert(error.text, fragment);
                return null;
            } else if (error.text.startsWith("GROUP_REPEAT_CREATE")) {
                TLRPC.TL_channels_createChannel reqCreateChannel2 = (TLRPC.TL_channels_createChannel) request;
                if (reqCreateChannel2.megagroup) {
                    showSimpleAlert(fragment, LocaleController.getString(R.string.ThisGroupExist));
                    return null;
                } else if (reqCreateChannel2.broadcast) {
                    showSimpleAlert(fragment, LocaleController.getString(R.string.ThisChannelExist));
                    return null;
                } else {
                    return null;
                }
            } else {
                showAddUserAlert(error.text, fragment, false, request);
                return null;
            }
        } else if (request instanceof TLRPC.TL_messages_editMessage) {
            if (!error.text.equals("MESSAGE_NOT_MODIFIED")) {
                if (fragment != null) {
                    showSimpleAlert(fragment, LocaleController.getString("EditMessageError", R.string.EditMessageError));
                    return null;
                }
                showSimpleToast(fragment, LocaleController.getString("EditMessageError", R.string.EditMessageError));
                return null;
            }
            return null;
        } else if ((request instanceof TLRPC.TL_messages_sendMessage) || (request instanceof TLRPC.TL_messages_sendMedia) || (request instanceof TLRPC.TL_messages_sendInlineBotResult) || (request instanceof TLRPC.TL_messages_forwardMessages) || (request instanceof TLRPC.TL_messages_sendMultiMedia) || (request instanceof TLRPC.TL_messages_sendScheduledMessages)) {
            String str = error.text;
            switch (str.hashCode()) {
                case -1809401834:
                    if (str.equals("USER_BANNED_IN_CHANNEL")) {
                        c = 1;
                        break;
                    }
                    c = 65535;
                    break;
                case -454039871:
                    if (str.equals("PEER_FLOOD")) {
                        c = 0;
                        break;
                    }
                    c = 65535;
                    break;
                case -33935358:
                    if (str.equals("MUTUALCONTACTNEED")) {
                        c = 3;
                        break;
                    }
                    c = 65535;
                    break;
                case 1169786080:
                    if (str.equals("SCHEDULE_TOO_MUCH")) {
                        c = 2;
                        break;
                    }
                    c = 65535;
                    break;
                default:
                    c = 65535;
                    break;
            }
            switch (c) {
                case 0:
                    NotificationCenter.getInstance(currentAccount).postNotificationName(NotificationCenter.needShowAlert, 0);
                    return null;
                case 1:
                    NotificationCenter.getInstance(currentAccount).postNotificationName(NotificationCenter.needShowAlert, 5);
                    return null;
                case 2:
                    showSimpleToast(fragment, LocaleController.getString("MessageScheduledLimitReached", R.string.MessageScheduledLimitReached));
                    return null;
                case 3:
                    NotificationCenter.getInstance(currentAccount).postNotificationName(NotificationCenter.contactRelationShip, 101);
                    return null;
                default:
                    return null;
            }
        } else if (request instanceof TLRPC.TL_messages_importChatInvite) {
            if (error.text.startsWith("FLOOD_WAIT")) {
                showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
                return null;
            } else if (error.text.equals("USERS_TOO_MUCH")) {
                showSimpleAlert(fragment, LocaleController.getString("JoinToGroupErrorFull", R.string.JoinToGroupErrorFull));
                return null;
            } else {
                showSimpleAlert(fragment, LocaleController.getString("JoinToGroupErrorNotExist", R.string.JoinToGroupErrorNotExist));
                return null;
            }
        } else if (request instanceof TLRPC.TL_messages_getAttachedStickers) {
            if (fragment != null && fragment.getParentActivity() != null) {
                ToastUtils.show((int) R.string.ErrorOccurred);
                return null;
            }
            return null;
        } else if ((request instanceof TLRPC.TL_account_confirmPhone) || (request instanceof TLRPC.TL_account_verifyPhone) || (request instanceof TLRPC.TL_account_verifyEmail)) {
            if (error.text.contains("PHONE_CODE_EMPTY") || error.text.contains("PHONE_CODE_INVALID") || error.text.contains("CODE_INVALID") || error.text.contains("CODE_EMPTY")) {
                return showSimpleAlert(fragment, LocaleController.getString("InvalidCode", R.string.InvalidCode));
            }
            if (error.text.contains("PHONE_CODE_EXPIRED") || error.text.contains("EMAIL_VERIFY_EXPIRED")) {
                return showSimpleAlert(fragment, LocaleController.getString("CodeExpired", R.string.CodeExpired));
            }
            if (error.text.startsWith("FLOOD_WAIT")) {
                return showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
            }
            return showSimpleAlert(fragment, error.text);
        } else if (request instanceof TLRPC.TL_auth_resendCode) {
            if (error.text.contains("PHONE_NUMBER_INVALID")) {
                return showSimpleAlert(fragment, LocaleController.getString("InvalidPhoneNumber", R.string.InvalidPhoneNumber));
            }
            if (error.text.contains("PHONE_CODE_EMPTY") || error.text.contains("PHONE_CODE_INVALID")) {
                return showSimpleAlert(fragment, LocaleController.getString("InvalidCode", R.string.InvalidCode));
            }
            if (error.text.contains("PHONE_CODE_EXPIRED")) {
                return showSimpleAlert(fragment, LocaleController.getString("CodeExpired", R.string.CodeExpired));
            }
            if (error.text.startsWith("FLOOD_WAIT")) {
                return showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
            }
            if (error.code != -1000) {
                return showSimpleAlert(fragment, LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred) + "\n" + error.text);
            }
            return null;
        } else if (request instanceof TLRPC.TL_account_sendConfirmPhoneCode) {
            if (error.code == 400) {
                return showSimpleAlert(fragment, LocaleController.getString("CancelLinkExpired", R.string.CancelLinkExpired));
            }
            if (error.text != null) {
                if (error.text.startsWith("FLOOD_WAIT")) {
                    return showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
                }
                return showSimpleAlert(fragment, LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred));
            }
            return null;
        } else if (request instanceof TLRPC.TL_account_changePhone) {
            if (error.text.contains("PHONE_NUMBER_INVALID")) {
                showSimpleAlert(fragment, LocaleController.getString("InvalidPhoneNumber", R.string.InvalidPhoneNumber));
                return null;
            } else if (error.text.contains("PHONE_CODE_EMPTY") || error.text.contains("PHONE_CODE_INVALID")) {
                showSimpleAlert(fragment, LocaleController.getString("InvalidCode", R.string.InvalidCode));
                return null;
            } else if (error.text.contains("PHONE_CODE_EXPIRED")) {
                showSimpleAlert(fragment, LocaleController.getString("CodeExpired", R.string.CodeExpired));
                return null;
            } else if (error.text.startsWith("FLOOD_WAIT")) {
                showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
                return null;
            } else {
                showSimpleAlert(fragment, error.text);
                return null;
            }
        } else if (request instanceof TLRPC.TL_account_sendChangePhoneCode) {
            if (error.text.contains("PHONE_NUMBER_INVALID")) {
                showSimpleAlert(fragment, LocaleController.getString("InvalidPhoneNumber", R.string.InvalidPhoneNumber));
                return null;
            } else if (error.text.contains("PHONE_CODE_EMPTY") || error.text.contains("PHONE_CODE_INVALID")) {
                showSimpleAlert(fragment, LocaleController.getString("InvalidCode", R.string.InvalidCode));
                return null;
            } else if (error.text.contains("PHONE_CODE_EXPIRED")) {
                showSimpleAlert(fragment, LocaleController.getString("CodeExpired", R.string.CodeExpired));
                return null;
            } else if (error.text.startsWith("FLOOD_WAIT")) {
                showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
                return null;
            } else if (!error.text.startsWith("PHONE_NUMBER_OCCUPIED")) {
                if (error.text.contains("IPORDE_LIMIT")) {
                    showSimpleAlert(fragment, LocaleController.getString("IpOrDeLimit", R.string.IpOrDeLimit));
                    return null;
                } else if (error.text.equals("INTERNAL")) {
                    showSimpleAlert(fragment, LocaleController.getString("InternalError", R.string.InternalError));
                    return null;
                } else {
                    showSimpleAlert(fragment, LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred));
                    return null;
                }
            } else {
                showSimpleAlert(fragment, LocaleController.formatString("ChangePhoneNumberOccupied", R.string.ChangePhoneNumberOccupied, (String) args[0]));
                return null;
            }
        } else if (request instanceof TLRPC.TL_updateUserName) {
            String str2 = error.text;
            switch (str2.hashCode()) {
                case 288843630:
                    if (str2.equals("USERNAME_INVALID")) {
                        c4 = 0;
                        break;
                    }
                    c4 = 65535;
                    break;
                case 533175271:
                    if (str2.equals("USERNAME_OCCUPIED")) {
                        c4 = 1;
                        break;
                    }
                    c4 = 65535;
                    break;
                default:
                    c4 = 65535;
                    break;
            }
            switch (c4) {
                case 0:
                    showSimpleAlert(fragment, LocaleController.getString("UsernameInvalid", R.string.UsernameInvalid));
                    return null;
                case 1:
                    showSimpleAlert(fragment, LocaleController.getString("UsernameInUse", R.string.UsernameInUse));
                    return null;
                default:
                    showSimpleAlert(fragment, LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred));
                    return null;
            }
        } else if (request instanceof TLRPC.TL_contacts_importContacts) {
            if (error != null && !error.text.startsWith("FLOOD_WAIT")) {
                showSimpleAlert(fragment, LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred) + "\n" + error.text);
                return null;
            }
            showSimpleAlert(fragment, LocaleController.getString("FloodWait", R.string.FloodWait));
            return null;
        } else if ((request instanceof TLRPC.TL_account_getPassword) || (request instanceof TLRPC.TL_account_getTmpPassword)) {
            if (error.text.startsWith("FLOOD_WAIT")) {
                showSimpleToast(fragment, getFloodWaitString(error.text));
                return null;
            }
            showSimpleToast(fragment, error.text);
            return null;
        } else if (request instanceof TLRPC.TL_payments_sendPaymentForm) {
            String str3 = error.text;
            switch (str3.hashCode()) {
                case -1144062453:
                    if (str3.equals("BOT_PRECHECKOUT_FAILED")) {
                        c3 = 0;
                        break;
                    }
                    c3 = 65535;
                    break;
                case -784238410:
                    if (str3.equals("PAYMENT_FAILED")) {
                        c3 = 1;
                        break;
                    }
                    c3 = 65535;
                    break;
                default:
                    c3 = 65535;
                    break;
            }
            switch (c3) {
                case 0:
                    showSimpleToast(fragment, LocaleController.getString("PaymentPrecheckoutFailed", R.string.PaymentPrecheckoutFailed));
                    return null;
                case 1:
                    showSimpleToast(fragment, LocaleController.getString("PaymentFailed", R.string.PaymentFailed));
                    return null;
                default:
                    showSimpleToast(fragment, error.text);
                    return null;
            }
        } else if (request instanceof TLRPC.TL_payments_validateRequestedInfo) {
            String str4 = error.text;
            switch (str4.hashCode()) {
                case 1758025548:
                    if (str4.equals("SHIPPING_NOT_AVAILABLE")) {
                        c2 = 0;
                        break;
                    }
                default:
                    c2 = 65535;
                    break;
            }
            switch (c2) {
                case 0:
                    showSimpleToast(fragment, LocaleController.getString("PaymentNoShippingMethod", R.string.PaymentNoShippingMethod));
                    return null;
                default:
                    showSimpleToast(fragment, error.text);
                    return null;
            }
        } else if (request instanceof TLRPC.TL_account_updateUsername) {
            if ("ALREDY_CHANGE".equals(error.text)) {
                showSimpleAlert(fragment, LocaleController.getString("AlreadyChangeAppNameCodeTips", R.string.AlreadyChangeAppNameCodeTips));
                return null;
            }
            showSimpleAlert(fragment, LocaleController.getString(R.string.OperationFailedPleaseTryAgain));
            return null;
        } else {
            return null;
        }
    }

    public static Toast showSimpleToast(BaseFragment baseFragment, String text) {
        ToastUtils.show((CharSequence) text);
        return ToastUtils.getToast();
    }

    public static AlertDialog showUpdateAppAlert(final Context context, String text, boolean updateApp) {
        if (context == null || text == null) {
            return null;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(LocaleController.getString("AppName", R.string.AppName));
        builder.setMessage(text);
        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
        if (updateApp) {
            builder.setNegativeButton(LocaleController.getString("UpdateApp", R.string.UpdateApp), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda47
                @Override // android.content.DialogInterface.OnClickListener
                public final void onClick(DialogInterface dialogInterface, int i) {
                    Browser.openUrl(context, BuildVars.PLAYSTORE_APP_URL);
                }
            });
        }
        return builder.show();
    }

    public static AlertDialog.Builder createLanguageAlert(final LaunchActivity activity, final TLRPC.TL_langPackLanguage language) {
        String str;
        int end;
        if (language == null) {
            return null;
        }
        language.lang_code = language.lang_code.replace('-', '_').toLowerCase();
        language.plural_code = language.plural_code.replace('-', '_').toLowerCase();
        if (language.base_lang_code != null) {
            language.base_lang_code = language.base_lang_code.replace('-', '_').toLowerCase();
        }
        final AlertDialog.Builder builder = new AlertDialog.Builder(activity);
        LocaleController.LocaleInfo currentInfo = LocaleController.getInstance().getCurrentLocaleInfo();
        if (currentInfo.shortName.equals(language.lang_code)) {
            builder.setTitle(LocaleController.getString("Language", R.string.Language));
            str = LocaleController.formatString("LanguageSame", R.string.LanguageSame, language.name);
            builder.setNegativeButton(LocaleController.getString("OK", R.string.OK), null);
            builder.setNeutralButton(LocaleController.getString("SETTINGS", R.string.SETTINGS), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda18
                @Override // android.content.DialogInterface.OnClickListener
                public final void onClick(DialogInterface dialogInterface, int i) {
                    LaunchActivity.this.m1665lambda$runLinkRequest$28$imuecahxyvjduiLaunchActivity(new LanguageSelectActivity());
                }
            });
        } else if (language.strings_count == 0) {
            builder.setTitle(LocaleController.getString("LanguageUnknownTitle", R.string.LanguageUnknownTitle));
            str = LocaleController.formatString("LanguageUnknownCustomAlert", R.string.LanguageUnknownCustomAlert, language.name);
            builder.setNegativeButton(LocaleController.getString("OK", R.string.OK), null);
        } else {
            builder.setTitle(LocaleController.getString("LanguageTitle", R.string.LanguageTitle));
            str = language.official ? LocaleController.formatString("LanguageAlert", R.string.LanguageAlert, language.name, Integer.valueOf((int) Math.ceil((language.translated_count / language.strings_count) * 100.0f))) : LocaleController.formatString("LanguageCustomAlert", R.string.LanguageCustomAlert, language.name, Integer.valueOf((int) Math.ceil((language.translated_count / language.strings_count) * 100.0f)));
            builder.setPositiveButton(LocaleController.getString("Change", R.string.Change), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda19
                @Override // android.content.DialogInterface.OnClickListener
                public final void onClick(DialogInterface dialogInterface, int i) {
                    AlertsCreator.lambda$createLanguageAlert$2(TLRPC.TL_langPackLanguage.this, activity, dialogInterface, i);
                }
            });
            builder.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        }
        SpannableStringBuilder spanned = new SpannableStringBuilder(AndroidUtilities.replaceTags(str));
        int start = TextUtils.indexOf((CharSequence) spanned, '[');
        if (start != -1) {
            end = TextUtils.indexOf((CharSequence) spanned, ']', start + 1);
            if (start != -1 && end != -1) {
                spanned.delete(end, end + 1);
                spanned.delete(start, start + 1);
            }
        } else {
            end = -1;
        }
        if (start != -1 && end != -1) {
            spanned.setSpan(new URLSpanNoUnderline(language.translations_url) { // from class: im.uecahxyvjd.ui.components.AlertsCreator.1
                @Override // im.uecahxyvjd.ui.components.URLSpanNoUnderline, android.text.style.URLSpan, android.text.style.ClickableSpan
                public void onClick(View widget) {
                    builder.getDismissRunnable().run();
                    super.onClick(widget);
                }
            }, start, end - 1, 33);
        }
        TextView message = new TextView(activity);
        message.setText(spanned);
        message.setTextSize(1, 16.0f);
        message.setLinkTextColor(Theme.getColor(Theme.key_dialogTextLink));
        message.setHighlightColor(Theme.getColor(Theme.key_dialogLinkSelection));
        message.setPadding(AndroidUtilities.dp(23.0f), 0, AndroidUtilities.dp(23.0f), 0);
        message.setMovementMethod(new AndroidUtilities.LinkMovementMethodMy());
        message.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
        builder.setView(message);
        return builder;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createLanguageAlert$2(TLRPC.TL_langPackLanguage language, LaunchActivity activity, DialogInterface dialogInterface, int i) {
        String key;
        if (language.official) {
            key = "remote_" + language.lang_code;
        } else {
            key = "unofficial_" + language.lang_code;
        }
        LocaleController.LocaleInfo localeInfo = LocaleController.getInstance().getLanguageFromDict(key);
        if (localeInfo == null) {
            localeInfo = new LocaleController.LocaleInfo();
            localeInfo.name = language.native_name;
            localeInfo.nameEnglish = language.name;
            localeInfo.shortName = language.lang_code;
            localeInfo.baseLangCode = language.base_lang_code;
            localeInfo.pluralLangCode = language.plural_code;
            localeInfo.isRtl = language.rtl;
            if (language.official) {
                localeInfo.pathToFile = "remote";
            } else {
                localeInfo.pathToFile = "unofficial";
            }
        }
        LocaleController.getInstance().applyLanguage(localeInfo, true, false, false, true, UserConfig.selectedAccount);
        activity.rebuildAllFragments(true);
    }

    public static boolean checkSlowMode(Context context, int currentAccount, long did, boolean few) {
        TLRPC.Chat chat;
        int lowerId = (int) did;
        if (lowerId < 0 && (chat = MessagesController.getInstance(currentAccount).getChat(Integer.valueOf(-lowerId))) != null && chat.slowmode_enabled && !ChatObject.hasAdminRights(chat)) {
            if (!few) {
                TLRPC.ChatFull chatFull = MessagesController.getInstance(currentAccount).getChatFull(chat.id);
                if (chatFull == null) {
                    chatFull = MessagesStorage.getInstance(currentAccount).loadChatInfo(chat.id, new CountDownLatch(1), false, false);
                }
                if (chatFull != null && chatFull.slowmode_next_send_date >= ConnectionsManager.getInstance(currentAccount).getCurrentTime()) {
                    few = true;
                }
            }
            if (few) {
                createSimpleAlert(context, chat.title, LocaleController.getString("SlowmodeSendError", R.string.SlowmodeSendError)).show();
                return true;
            }
        }
        return false;
    }

    public static AlertDialog.Builder createSimpleAlert(Context context, String text) {
        return createSimpleAlert(context, null, text);
    }

    public static AlertDialog.Builder createSimpleAlert(Context context, String title, String text) {
        if (text == null) {
            return null;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title == null ? LocaleController.getString("AppName", R.string.AppName) : title);
        builder.setMessage(text);
        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
        return builder;
    }

    public static Dialog showSimpleAlert(BaseFragment baseFragment, String text) {
        return showSimpleAlert(baseFragment, null, text);
    }

    public static Dialog showSimpleAlert(BaseFragment baseFragment, String title, String text) {
        if (text == null || baseFragment == null || baseFragment.getParentActivity() == null) {
            return null;
        }
        AlertDialog.Builder builder = createSimpleAlert(baseFragment.getParentActivity(), title, text);
        Dialog dialog = builder.create();
        baseFragment.showDialog(dialog);
        return dialog;
    }

    public static void showBlockReportSpamAlert(BaseFragment fragment, final long dialog_id, final TLRPC.User currentUser, final TLRPC.Chat currentChat, final TLRPC.EncryptedChat encryptedChat, final boolean isLocation, TLRPC.ChatFull chatInfo, final MessagesStorage.IntCallback callback) {
        CharSequence reportText;
        CheckBoxCell[] cells;
        CharSequence reportText2;
        if (fragment == null || fragment.getParentActivity() == null) {
            return;
        }
        final AccountInstance accountInstance = fragment.getAccountInstance();
        AlertDialog.Builder builder = new AlertDialog.Builder(fragment.getParentActivity());
        SharedPreferences preferences = MessagesController.getNotificationsSettings(fragment.getCurrentAccount());
        boolean showReport = preferences.getBoolean("dialog_bar_report" + dialog_id, false);
        int i = 1;
        if (currentUser != null) {
            builder.setTitle(LocaleController.formatString("BlockUserTitle", R.string.BlockUserTitle, UserObject.getFirstName(currentUser)));
            builder.setMessage(AndroidUtilities.replaceTags(LocaleController.formatString("BlockUserAlert", R.string.BlockUserAlert, UserObject.getFirstName(currentUser))));
            CharSequence reportText3 = LocaleController.getString("BlockContact", R.string.BlockContact);
            final CheckBoxCell[] cells2 = new CheckBoxCell[2];
            LinearLayout linearLayout = new LinearLayout(fragment.getParentActivity());
            linearLayout.setOrientation(1);
            int a2 = 0;
            for (int i2 = 2; a2 < i2; i2 = 2) {
                if (a2 == 0 && !showReport) {
                    reportText2 = reportText3;
                } else {
                    cells2[a2] = new CheckBoxCell(fragment.getParentActivity(), i);
                    cells2[a2].setBackgroundDrawable(Theme.getSelectorDrawable(false));
                    cells2[a2].setTag(Integer.valueOf(a2));
                    if (a2 == 0) {
                        reportText2 = reportText3;
                        cells2[a2].setText(LocaleController.getString("DeleteReportSpam", R.string.DeleteReportSpam), "", true, false);
                    } else {
                        reportText2 = reportText3;
                        if (a2 == 1) {
                            cells2[a2].setText(LocaleController.formatString("DeleteThisChat", R.string.DeleteThisChat, new Object[0]), "", true, false);
                        }
                    }
                    cells2[a2].setPadding(LocaleController.isRTL ? AndroidUtilities.dp(16.0f) : AndroidUtilities.dp(8.0f), 0, LocaleController.isRTL ? AndroidUtilities.dp(8.0f) : AndroidUtilities.dp(16.0f), 0);
                    linearLayout.addView(cells2[a2], LayoutHelper.createLinear(-1, -2));
                    cells2[a2].setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda7
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            AlertsCreator.lambda$showBlockReportSpamAlert$3(cells2, view);
                        }
                    });
                }
                a2++;
                reportText3 = reportText2;
                i = 1;
            }
            builder.setCustomViewOffset(12);
            builder.setView(linearLayout);
            cells = cells2;
            reportText = reportText3;
        } else {
            if (currentChat != null && isLocation) {
                builder.setTitle(LocaleController.getString("ReportUnrelatedGroup", R.string.ReportUnrelatedGroup));
                if (chatInfo != null && (chatInfo.location instanceof TLRPC.TL_channelLocation)) {
                    TLRPC.TL_channelLocation location = (TLRPC.TL_channelLocation) chatInfo.location;
                    builder.setMessage(AndroidUtilities.replaceTags(LocaleController.formatString("ReportUnrelatedGroupText", R.string.ReportUnrelatedGroupText, location.address)));
                } else {
                    builder.setMessage(LocaleController.getString("ReportUnrelatedGroupTextNoAddress", R.string.ReportUnrelatedGroupTextNoAddress));
                }
            } else {
                builder.setTitle(LocaleController.getString("ReportSpamTitle", R.string.ReportSpamTitle));
                if (ChatObject.isChannel(currentChat) && !currentChat.megagroup) {
                    builder.setMessage(LocaleController.getString("ReportSpamAlertChannel", R.string.ReportSpamAlertChannel));
                } else {
                    builder.setMessage(LocaleController.getString("ReportSpamAlertGroup", R.string.ReportSpamAlertGroup));
                }
            }
            reportText = LocaleController.getString("ReportChat", R.string.ReportChat);
            cells = null;
        }
        final CheckBoxCell[] checkBoxCellArr = cells;
        builder.setPositiveButton(reportText, new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda8
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i3) {
                AlertsCreator.lambda$showBlockReportSpamAlert$4(TLRPC.User.this, accountInstance, checkBoxCellArr, dialog_id, currentChat, encryptedChat, isLocation, callback, dialogInterface, i3);
            }
        });
        builder.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        AlertDialog dialog = builder.create();
        fragment.showDialog(dialog);
        TextView button = (TextView) dialog.getButton(-1);
        if (button != null) {
            button.setTextColor(Theme.getColor(Theme.key_dialogTextRed2));
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$showBlockReportSpamAlert$3(CheckBoxCell[] cells, View v) {
        Integer num = (Integer) v.getTag();
        cells[num.intValue()].setChecked(!cells[num.intValue()].isChecked(), true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$showBlockReportSpamAlert$4(TLRPC.User currentUser, AccountInstance accountInstance, CheckBoxCell[] cells, long dialog_id, TLRPC.Chat currentChat, TLRPC.EncryptedChat encryptedChat, boolean isLocation, MessagesStorage.IntCallback callback, DialogInterface dialogInterface, int i) {
        if (currentUser != null) {
            accountInstance.getMessagesController().blockUser(currentUser.id);
        }
        if (cells == null || (cells[0] != null && cells[0].isChecked())) {
            accountInstance.getMessagesController().reportSpam(dialog_id, currentUser, currentChat, encryptedChat, currentChat != null && isLocation);
        }
        if (cells == null || cells[1].isChecked()) {
            if (currentChat != null) {
                if (ChatObject.isNotInChat(currentChat)) {
                    accountInstance.getMessagesController().deleteDialog(dialog_id, 0);
                } else {
                    accountInstance.getMessagesController().deleteUserFromChat((int) (-dialog_id), accountInstance.getMessagesController().getUser(Integer.valueOf(accountInstance.getUserConfig().getClientUserId())), null);
                }
            } else {
                accountInstance.getMessagesController().deleteDialog(dialog_id, 0);
            }
            callback.run(1);
            return;
        }
        callback.run(0);
    }

    public static void showCustomNotificationsDialog(BaseFragment parentFragment, long did, int globalType, ArrayList<NotificationsSettingsActivity.NotificationException> exceptions, int currentAccount, MessagesStorage.IntCallback callback) {
        showCustomNotificationsDialog(parentFragment, did, globalType, exceptions, currentAccount, callback, null);
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r10v0 */
    /* JADX WARN: Type inference failed for: r10v1, types: [boolean, int] */
    /* JADX WARN: Type inference failed for: r10v3 */
    /* JADX WARN: Type inference failed for: r15v0 */
    /* JADX WARN: Type inference failed for: r15v1, types: [boolean, int] */
    /* JADX WARN: Type inference failed for: r15v3 */
    public static void showCustomNotificationsDialog(final BaseFragment parentFragment, final long did, final int globalType, final ArrayList<NotificationsSettingsActivity.NotificationException> exceptions, final int currentAccount, final MessagesStorage.IntCallback callback, final MessagesStorage.IntCallback resultCallback) {
        int a2;
        final AlertDialog.Builder builder;
        boolean defaultEnabled;
        LinearLayout linearLayout;
        String[] descriptions;
        Drawable drawable;
        if (parentFragment == null || parentFragment.getParentActivity() == null) {
            return;
        }
        boolean defaultEnabled2 = NotificationsController.getInstance(currentAccount).isGlobalNotificationsEnabled(did);
        String[] strArr = new String[5];
        ?? r15 = 0;
        strArr[0] = LocaleController.getString("NotificationsTurnOn", R.string.NotificationsTurnOn);
        ?? r10 = 1;
        strArr[1] = LocaleController.formatString("MuteFor", R.string.MuteFor, LocaleController.formatPluralString("Hours", 1));
        strArr[2] = LocaleController.formatString("MuteFor", R.string.MuteFor, LocaleController.formatPluralString("Days", 2));
        Drawable drawable2 = null;
        strArr[3] = (did == 0 && (parentFragment instanceof NotificationsCustomSettingsActivity)) ? null : LocaleController.getString("NotificationsCustomize", R.string.NotificationsCustomize);
        strArr[4] = LocaleController.getString("NotificationsTurnOff", R.string.NotificationsTurnOff);
        String[] descriptions2 = strArr;
        int[] icons = {R.drawable.notifications_on, R.drawable.notifications_mute1h, R.drawable.notifications_mute2d, R.drawable.notifications_settings, R.drawable.notifications_off};
        LinearLayout linearLayout2 = new LinearLayout(parentFragment.getParentActivity());
        linearLayout2.setOrientation(1);
        AlertDialog.Builder builder2 = new AlertDialog.Builder(parentFragment.getParentActivity());
        int a3 = 0;
        while (a3 < descriptions2.length) {
            if (descriptions2[a3] == null) {
                a2 = a3;
                builder = builder2;
                descriptions = descriptions2;
                drawable = drawable2;
                defaultEnabled = defaultEnabled2;
                linearLayout = linearLayout2;
            } else {
                TextView textView = new TextView(parentFragment.getParentActivity());
                Drawable drawable3 = parentFragment.getParentActivity().getResources().getDrawable(icons[a3]);
                if (a3 == descriptions2.length - r10) {
                    textView.setTextColor(Theme.getColor(Theme.key_dialogTextRed));
                    drawable3.setColorFilter(new PorterDuffColorFilter(Theme.getColor(Theme.key_dialogRedIcon), PorterDuff.Mode.MULTIPLY));
                } else {
                    textView.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
                    drawable3.setColorFilter(new PorterDuffColorFilter(Theme.getColor(Theme.key_dialogIcon), PorterDuff.Mode.MULTIPLY));
                }
                textView.setTextSize(r10, 16.0f);
                textView.setLines(r10);
                textView.setMaxLines(r10);
                textView.setCompoundDrawablesWithIntrinsicBounds(drawable3, drawable2, drawable2, drawable2);
                textView.setTag(Integer.valueOf(a3));
                textView.setBackgroundDrawable(Theme.getSelectorDrawable(r15));
                textView.setPadding(AndroidUtilities.dp(24.0f), r15, AndroidUtilities.dp(24.0f), r15);
                textView.setSingleLine(r10);
                textView.setGravity(19);
                textView.setCompoundDrawablePadding(AndroidUtilities.dp(26.0f));
                textView.setText(descriptions2[a3]);
                linearLayout2.addView(textView, LayoutHelper.createLinear(-1, 48, 51));
                final boolean z = defaultEnabled2;
                a2 = a3;
                builder = builder2;
                defaultEnabled = defaultEnabled2;
                linearLayout = linearLayout2;
                descriptions = descriptions2;
                drawable = drawable2;
                textView.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda52
                    @Override // android.view.View.OnClickListener
                    public final void onClick(View view) {
                        AlertsCreator.lambda$showCustomNotificationsDialog$5(did, currentAccount, z, resultCallback, globalType, parentFragment, exceptions, callback, builder, view);
                    }
                });
            }
            a3 = a2 + 1;
            linearLayout2 = linearLayout;
            builder2 = builder;
            defaultEnabled2 = defaultEnabled;
            descriptions2 = descriptions;
            drawable2 = drawable;
            r10 = 1;
            r15 = 0;
        }
        AlertDialog.Builder builder3 = builder2;
        builder3.setTitle(LocaleController.getString("Notifications", R.string.Notifications));
        builder3.setView(linearLayout2);
        parentFragment.showDialog(builder3.create());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$showCustomNotificationsDialog$5(long did, int currentAccount, boolean defaultEnabled, MessagesStorage.IntCallback resultCallback, int globalType, BaseFragment parentFragment, ArrayList exceptions, MessagesStorage.IntCallback callback, AlertDialog.Builder builder, View v) {
        long flags;
        int i = ((Integer) v.getTag()).intValue();
        if (i == 0) {
            if (did != 0) {
                SharedPreferences preferences = MessagesController.getNotificationsSettings(currentAccount);
                SharedPreferences.Editor editor = preferences.edit();
                if (defaultEnabled) {
                    editor.remove("notify2_" + did);
                } else {
                    editor.putInt("notify2_" + did, 0);
                }
                MessagesStorage.getInstance(currentAccount).setDialogFlags(did, 0L);
                editor.commit();
                TLRPC.Dialog dialog = MessagesController.getInstance(currentAccount).dialogs_dict.get(did);
                if (dialog != null) {
                    dialog.notify_settings = new TLRPC.TL_peerNotifySettings();
                }
                NotificationsController.getInstance(currentAccount).updateServerNotificationsSettings(did);
                if (resultCallback != null) {
                    if (defaultEnabled) {
                        resultCallback.run(0);
                    } else {
                        resultCallback.run(1);
                    }
                }
            } else {
                NotificationsController.getInstance(currentAccount).setGlobalNotificationsEnabled(globalType, 0);
            }
        } else if (i != 3) {
            int untilTime = ConnectionsManager.getInstance(currentAccount).getCurrentTime();
            if (i == 1) {
                untilTime += 3600;
            } else if (i == 2) {
                untilTime += 172800;
            } else if (i == 4) {
                untilTime = Integer.MAX_VALUE;
            }
            if (did != 0) {
                SharedPreferences preferences2 = MessagesController.getNotificationsSettings(currentAccount);
                SharedPreferences.Editor editor2 = preferences2.edit();
                if (i != 4) {
                    editor2.putInt("notify2_" + did, 3);
                    editor2.putInt("notifyuntil_" + did, untilTime);
                    flags = (untilTime << 32) | 1;
                } else if (!defaultEnabled) {
                    editor2.remove("notify2_" + did);
                    flags = 0;
                } else {
                    editor2.putInt("notify2_" + did, 2);
                    flags = 1;
                }
                NotificationsController.getInstance(currentAccount).removeNotificationsForDialog(did);
                MessagesStorage.getInstance(currentAccount).setDialogFlags(did, flags);
                editor2.commit();
                TLRPC.Dialog dialog2 = MessagesController.getInstance(currentAccount).dialogs_dict.get(did);
                if (dialog2 != null) {
                    dialog2.notify_settings = new TLRPC.TL_peerNotifySettings();
                    if (i != 4 || defaultEnabled) {
                        dialog2.notify_settings.mute_until = untilTime;
                    }
                }
                NotificationsController.getInstance(currentAccount).updateServerNotificationsSettings(did);
                if (resultCallback != null) {
                    if (i == 4 && !defaultEnabled) {
                        resultCallback.run(0);
                    } else {
                        resultCallback.run(1);
                    }
                }
            } else if (i == 4) {
                NotificationsController.getInstance(currentAccount).setGlobalNotificationsEnabled(globalType, Integer.MAX_VALUE);
            } else {
                NotificationsController.getInstance(currentAccount).setGlobalNotificationsEnabled(globalType, untilTime);
            }
        } else if (did != 0) {
            Bundle args = new Bundle();
            args.putLong("dialog_id", did);
            parentFragment.presentFragment(new ProfileNotificationsActivity(args));
        } else {
            parentFragment.presentFragment(new NotificationsCustomSettingsActivity(globalType, exceptions));
        }
        if (callback != null) {
            callback.run(i);
        }
        builder.getDismissRunnable().run();
    }

    public static AlertDialog showSecretLocationAlert(Context context, int currentAccount, final Runnable onSelectRunnable, boolean inChat) {
        ArrayList<String> arrayList = new ArrayList<>();
        int providers = MessagesController.getInstance(currentAccount).availableMapProviders;
        if ((providers & 1) != 0) {
            arrayList.add(LocaleController.getString("MapPreviewProviderApp", R.string.MapPreviewProviderApp));
        }
        if ((providers & 2) != 0) {
            arrayList.add(LocaleController.getString("MapPreviewProviderGoogle", R.string.MapPreviewProviderGoogle));
        }
        if ((providers & 4) != 0) {
            arrayList.add(LocaleController.getString("MapPreviewProviderYandex", R.string.MapPreviewProviderYandex));
        }
        arrayList.add(LocaleController.getString("MapPreviewProviderNobody", R.string.MapPreviewProviderNobody));
        AlertDialog.Builder builder = new AlertDialog.Builder(context).setTitle(LocaleController.getString("ChooseMapPreviewProvider", R.string.ChooseMapPreviewProvider)).setItems((CharSequence[]) arrayList.toArray(new String[0]), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda53
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$showSecretLocationAlert$6(onSelectRunnable, dialogInterface, i);
            }
        });
        if (!inChat) {
            builder.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        }
        AlertDialog dialog = builder.show();
        if (inChat) {
            dialog.setCanceledOnTouchOutside(false);
        }
        return dialog;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$showSecretLocationAlert$6(Runnable onSelectRunnable, DialogInterface dialog, int which) {
        SharedConfig.setSecretMapPreviewType(which);
        if (onSelectRunnable != null) {
            onSelectRunnable.run();
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public static void updateDayPicker(NumberPicker dayPicker, NumberPicker monthPicker, NumberPicker yearPicker) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(2, monthPicker.getValue());
        calendar.set(1, yearPicker.getValue());
        dayPicker.setMinValue(1);
        dayPicker.setMaxValue(calendar.getActualMaximum(5));
    }

    private static void checkPickerDate(NumberPicker dayPicker, NumberPicker monthPicker, NumberPicker yearPicker) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(System.currentTimeMillis());
        int currentYear = calendar.get(1);
        int currentMonth = calendar.get(2);
        int currentDay = calendar.get(5);
        if (currentYear > yearPicker.getValue()) {
            yearPicker.setValue(currentYear);
        }
        if (yearPicker.getValue() == currentYear) {
            if (currentMonth > monthPicker.getValue()) {
                monthPicker.setValue(currentMonth);
            }
            if (currentMonth == monthPicker.getValue() && currentDay > dayPicker.getValue()) {
                dayPicker.setValue(currentDay);
            }
        }
    }

    public static AlertDialog createSupportAlert(final BaseFragment fragment) {
        if (fragment == null || fragment.getParentActivity() == null) {
            return null;
        }
        TextView message = new TextView(fragment.getParentActivity());
        Spannable spanned = new SpannableString(Html.fromHtml(LocaleController.getString("AskAQuestionInfo", R.string.AskAQuestionInfo).replace("\n", "<br>")));
        URLSpan[] spans = (URLSpan[]) spanned.getSpans(0, spanned.length(), URLSpan.class);
        for (URLSpan span : spans) {
            int start = spanned.getSpanStart(span);
            int end = spanned.getSpanEnd(span);
            spanned.removeSpan(span);
            spanned.setSpan(new URLSpanNoUnderline(span.getURL()) { // from class: im.uecahxyvjd.ui.components.AlertsCreator.2
                @Override // im.uecahxyvjd.ui.components.URLSpanNoUnderline, android.text.style.URLSpan, android.text.style.ClickableSpan
                public void onClick(View widget) {
                    fragment.dismissCurrentDialog();
                    super.onClick(widget);
                }
            }, start, end, 0);
        }
        message.setText(spanned);
        message.setTextSize(1, 16.0f);
        message.setLinkTextColor(Theme.getColor(Theme.key_dialogTextLink));
        message.setHighlightColor(Theme.getColor(Theme.key_dialogLinkSelection));
        message.setPadding(AndroidUtilities.dp(23.0f), 0, AndroidUtilities.dp(23.0f), 0);
        message.setMovementMethod(new AndroidUtilities.LinkMovementMethodMy());
        message.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
        AlertDialog.Builder builder1 = new AlertDialog.Builder(fragment.getParentActivity());
        builder1.setView(message);
        builder1.setTitle(LocaleController.getString("AskAQuestion", R.string.AskAQuestion));
        builder1.setPositiveButton(LocaleController.getString("AskButton", R.string.AskButton), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda36
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.performAskAQuestion(BaseFragment.this);
            }
        });
        builder1.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder1.create();
    }

    /* JADX INFO: Access modifiers changed from: private */
    public static void performAskAQuestion(final BaseFragment fragment) {
        String userString;
        final int currentAccount = fragment.getCurrentAccount();
        final SharedPreferences preferences = MessagesController.getMainSettings(currentAccount);
        int uid = preferences.getInt("support_id", 0);
        TLRPC.User supportUser = null;
        if (uid != 0 && (supportUser = MessagesController.getInstance(currentAccount).getUser(Integer.valueOf(uid))) == null && (userString = preferences.getString("support_user", null)) != null) {
            try {
                byte[] datacentersBytes = Base64.decode(userString, 0);
                if (datacentersBytes != null) {
                    SerializedData data = new SerializedData(datacentersBytes);
                    supportUser = TLRPC.User.TLdeserialize(data, data.readInt32(false), false);
                    if (supportUser != null && supportUser.id == 333000) {
                        supportUser = null;
                    }
                    data.cleanup();
                }
            } catch (Exception e) {
                FileLog.e(e);
                supportUser = null;
            }
        }
        if (supportUser == null) {
            final AlertDialog progressDialog = new AlertDialog(fragment.getParentActivity(), 3);
            progressDialog.setCanCancel(false);
            progressDialog.show();
            TLRPC.TL_help_getSupport req = new TLRPC.TL_help_getSupport();
            ConnectionsManager.getInstance(currentAccount).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda23
                @Override // im.uecahxyvjd.tgnet.RequestDelegate
                public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                    AlertsCreator.lambda$performAskAQuestion$10(preferences, progressDialog, currentAccount, fragment, tLObject, tL_error);
                }
            });
            return;
        }
        MessagesController.getInstance(currentAccount).putUser(supportUser, true);
        Bundle args = new Bundle();
        args.putInt("user_id", supportUser.id);
        fragment.presentFragment(new ChatActivity(args));
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$performAskAQuestion$10(final SharedPreferences preferences, final AlertDialog progressDialog, final int currentAccount, final BaseFragment fragment, TLObject response, TLRPC.TL_error error) {
        if (error == null) {
            final TLRPC.TL_help_support res = (TLRPC.TL_help_support) response;
            AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda5
                @Override // java.lang.Runnable
                public final void run() {
                    AlertsCreator.lambda$performAskAQuestion$8(preferences, res, progressDialog, currentAccount, fragment);
                }
            });
            return;
        }
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda6
            @Override // java.lang.Runnable
            public final void run() {
                AlertsCreator.lambda$performAskAQuestion$9(AlertDialog.this);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$performAskAQuestion$8(SharedPreferences preferences, TLRPC.TL_help_support res, AlertDialog progressDialog, int currentAccount, BaseFragment fragment) {
        SharedPreferences.Editor editor = preferences.edit();
        editor.putInt("support_id", res.user.id);
        SerializedData data = new SerializedData();
        res.user.serializeToStream(data);
        editor.putString("support_user", Base64.encodeToString(data.toByteArray(), 0));
        editor.commit();
        data.cleanup();
        try {
            progressDialog.dismiss();
        } catch (Exception e) {
            FileLog.e(e);
        }
        ArrayList<TLRPC.User> users = new ArrayList<>();
        users.add(res.user);
        MessagesStorage.getInstance(currentAccount).putUsersAndChats(users, null, true, true);
        MessagesController.getInstance(currentAccount).putUser(res.user, false);
        Bundle args = new Bundle();
        args.putInt("user_id", res.user.id);
        fragment.presentFragment(new ChatActivity(args));
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$performAskAQuestion$9(AlertDialog progressDialog) {
        try {
            progressDialog.dismiss();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    public static void createClearOrDeleteDialogAlert(BaseFragment fragment, boolean clear, TLRPC.Chat chat, TLRPC.User user, boolean secret, MessagesStorage.BooleanCallback onProcessRunnable) {
        createClearOrDeleteDialogAlert(fragment, clear, false, false, chat, user, secret, onProcessRunnable);
    }

    /* JADX WARN: Removed duplicated region for block: B:102:0x028f  */
    /* JADX WARN: Removed duplicated region for block: B:106:0x029e  */
    /* JADX WARN: Removed duplicated region for block: B:107:0x02b0  */
    /* JADX WARN: Removed duplicated region for block: B:151:0x042f  */
    /* JADX WARN: Removed duplicated region for block: B:152:0x043b  */
    /* JADX WARN: Removed duplicated region for block: B:173:0x04f7  */
    /* JADX WARN: Removed duplicated region for block: B:177:? A[RETURN, SYNTHETIC] */
    /* JADX WARN: Removed duplicated region for block: B:98:0x0277  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static void createClearOrDeleteDialogAlert(final BaseFragment fragment, final boolean clear, final boolean admin, final boolean second, final TLRPC.Chat chat, final TLRPC.User user, final boolean secret, final MessagesStorage.BooleanCallback onProcessRunnable) {
        XDialog.Builder builder;
        int revokeTimeLimit;
        boolean z;
        String str;
        String str2;
        int revokeTimeLimit2;
        String actionText;
        TextView button;
        char c;
        if (fragment == null || fragment.getParentActivity() == null) {
            return;
        }
        if (chat == null && user == null) {
            return;
        }
        int account = fragment.getCurrentAccount();
        Context context = fragment.getParentActivity();
        XDialog.Builder builder2 = new XDialog.Builder(context);
        int selfUserId = UserConfig.getInstance(account).getClientUserId();
        final CheckBoxCell[] cell = new CheckBoxCell[1];
        TextView messageTextView = new TextView(context);
        messageTextView.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
        messageTextView.setTextSize(1, 16.0f);
        messageTextView.setGravity((LocaleController.isRTL ? 5 : 3) | 48);
        final boolean clearingCache = ChatObject.isChannel(chat) && !TextUtils.isEmpty(chat.username);
        FrameLayout frameLayout = new FrameLayout(context) { // from class: im.uecahxyvjd.ui.components.AlertsCreator.3
            @Override // android.widget.FrameLayout, android.view.View
            protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
                super.onMeasure(widthMeasureSpec, heightMeasureSpec);
                if (cell[0] != null) {
                    setMeasuredDimension(getMeasuredWidth(), getMeasuredHeight() + cell[0].getMeasuredHeight() + AndroidUtilities.dp(7.0f));
                }
            }
        };
        builder2.setView(frameLayout);
        AvatarDrawable avatarDrawable = new AvatarDrawable();
        avatarDrawable.setTextSize(AndroidUtilities.dp(12.0f));
        BackupImageView imageView = new BackupImageView(context);
        imageView.setRoundRadius(AndroidUtilities.dp(20.0f));
        frameLayout.addView(imageView, LayoutHelper.createFrame(40.0f, 40.0f, (LocaleController.isRTL ? 5 : 3) | 48, 22.0f, 15.0f, 22.0f, 0.0f));
        TextView textView = new TextView(context);
        textView.setTextColor(Theme.getColor(Theme.key_actionBarDefaultSubmenuItem));
        textView.setTextSize(1, 20.0f);
        textView.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
        textView.setLines(1);
        textView.setMaxLines(1);
        textView.setSingleLine(true);
        textView.setGravity((LocaleController.isRTL ? 5 : 3) | 16);
        textView.setEllipsize(TextUtils.TruncateAt.END);
        if (clear) {
            if (clearingCache) {
                builder = builder2;
                textView.setText(LocaleController.getString("ClearHistoryCache", R.string.ClearHistoryCache));
            } else {
                builder = builder2;
                textView.setText(LocaleController.getString("ClearHistory", R.string.ClearHistory));
            }
        } else {
            builder = builder2;
            if (admin) {
                if (ChatObject.isChannel(chat)) {
                    if (chat.megagroup) {
                        textView.setText(LocaleController.getString("DeleteMegaMenu", R.string.DeleteMegaMenu));
                    } else {
                        textView.setText(LocaleController.getString("ChannelDeleteMenu", R.string.ChannelDeleteMenu));
                    }
                } else {
                    textView.setText(LocaleController.getString("DeleteMegaMenu", R.string.DeleteMegaMenu));
                }
            } else {
                textView.setText(LocaleController.getString("DeleteChatUser", R.string.DeleteChatUser));
            }
        }
        frameLayout.addView(textView, LayoutHelper.createFrame(-1.0f, -2.0f, (LocaleController.isRTL ? 5 : 3) | 48, LocaleController.isRTL ? 21 : 76, 21.0f, LocaleController.isRTL ? 76 : 21, 0.0f));
        frameLayout.addView(messageTextView, LayoutHelper.createFrame(-2.0f, -2.0f, (LocaleController.isRTL ? 5 : 3) | 48, 24.0f, 67.0f, 24.0f, 9.0f));
        if (user == null || user.bot || user.id == selfUserId || !MessagesController.getInstance(account).canRevokePmInbox) {
        }
        if (user != null) {
            revokeTimeLimit = MessagesController.getInstance(account).revokeTimePmLimit;
        } else {
            revokeTimeLimit = MessagesController.getInstance(account).revokeTimeLimit;
        }
        if (!secret && user != null && 0 != 0) {
            if (revokeTimeLimit == Integer.MAX_VALUE) {
                z = true;
                boolean canDeleteInbox = z;
                final boolean[] deleteForAll = new boolean[1];
                if (!second || !canDeleteInbox) {
                    str = "ClearHistory";
                    str2 = "DeleteChatUser";
                    revokeTimeLimit2 = revokeTimeLimit;
                } else {
                    revokeTimeLimit2 = revokeTimeLimit;
                    cell[0] = new CheckBoxCell(context, 1);
                    cell[0].setBackgroundDrawable(Theme.getSelectorDrawable(false));
                    if (clear) {
                        str2 = "DeleteChatUser";
                        c = 0;
                        cell[0].setText(LocaleController.formatString("ClearHistoryOptionAlso", R.string.ClearHistoryOptionAlso, UserObject.getFirstName(user)), "", false, false);
                        str = "ClearHistory";
                    } else {
                        str2 = "DeleteChatUser";
                        c = 0;
                        str = "ClearHistory";
                        cell[0].setText(LocaleController.formatString("DeleteMessagesOptionAlso", R.string.DeleteMessagesOptionAlso, UserObject.getFirstName(user)), "", false, false);
                    }
                    cell[c].setPadding(LocaleController.isRTL ? AndroidUtilities.dp(16.0f) : AndroidUtilities.dp(8.0f), 0, LocaleController.isRTL ? AndroidUtilities.dp(8.0f) : AndroidUtilities.dp(16.0f), 0);
                    frameLayout.addView(cell[0], LayoutHelper.createFrame(-1.0f, 48.0f, 83, 0.0f, 0.0f, 0.0f, 0.0f));
                    cell[0].setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda11
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            AlertsCreator.lambda$createClearOrDeleteDialogAlert$11(deleteForAll, view);
                        }
                    });
                }
                if (user == null) {
                    if (user.id == selfUserId) {
                        avatarDrawable.setAvatarType(2);
                        imageView.setImage((ImageLocation) null, (String) null, avatarDrawable, user);
                    } else {
                        avatarDrawable.setInfo(user);
                        imageView.setImage(ImageLocation.getForUser(user, false), "50_50", avatarDrawable, user);
                    }
                } else if (chat != null) {
                    avatarDrawable.setInfo(chat);
                    imageView.setImage(ImageLocation.getForChat(chat, false), "50_50", avatarDrawable, chat);
                }
                if (!second) {
                    messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.getString("DeleteAllMessagesAlert", R.string.DeleteAllMessagesAlert)));
                } else if (clear) {
                    if (user != null) {
                        if (secret) {
                            messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("AreYouSureClearHistoryWithSecretUser", R.string.AreYouSureClearHistoryWithSecretUser, UserObject.getUserName(user))));
                        } else if (user.id == selfUserId) {
                            messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.getString("AreYouSureClearHistorySavedMessages", R.string.AreYouSureClearHistorySavedMessages)));
                        } else {
                            messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("AreYouSureClearHistoryWithUser", R.string.AreYouSureClearHistoryWithUser, UserObject.getUserName(user))));
                        }
                    } else if (chat != null) {
                        if (!ChatObject.isChannel(chat) || (chat.megagroup && TextUtils.isEmpty(chat.username))) {
                            messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("AreYouSureClearHistoryWithChat", R.string.AreYouSureClearHistoryWithChat, chat.title)));
                        } else if (chat.megagroup) {
                            messageTextView.setText(LocaleController.getString("AreYouSureClearHistoryGroup", R.string.AreYouSureClearHistoryGroup));
                        } else {
                            messageTextView.setText(LocaleController.getString("AreYouSureClearHistoryChannel", R.string.AreYouSureClearHistoryChannel));
                        }
                    }
                } else if (admin) {
                    if (ChatObject.isChannel(chat)) {
                        if (chat.megagroup) {
                            messageTextView.setText(LocaleController.getString("MegaDeleteAlert", R.string.MegaDeleteAlert));
                        } else {
                            messageTextView.setText(LocaleController.getString("ChannelDeleteAlert", R.string.ChannelDeleteAlert));
                        }
                    } else {
                        messageTextView.setText(LocaleController.getString("AreYouSureDeleteAndExit", R.string.AreYouSureDeleteAndExit));
                    }
                } else if (user != null) {
                    if (secret) {
                        messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("AreYouSureDeleteThisChatWithSecretUser", R.string.AreYouSureDeleteThisChatWithSecretUser, UserObject.getUserName(user))));
                    } else if (user.id == selfUserId) {
                        messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.getString("AreYouSureDeleteThisChatSavedMessages", R.string.AreYouSureDeleteThisChatSavedMessages)));
                    } else {
                        messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("AreYouSureDeleteThisChatWithUser", R.string.AreYouSureDeleteThisChatWithUser, UserObject.getUserName(user))));
                    }
                } else if (!ChatObject.isChannel(chat)) {
                    messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("AreYouSureDeleteAndExitName", R.string.AreYouSureDeleteAndExitName, chat.title)));
                } else if (chat.megagroup) {
                    messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("MegaLeaveAlertWithName", R.string.MegaLeaveAlertWithName, chat.title)));
                } else {
                    messageTextView.setText(AndroidUtilities.replaceTags(LocaleController.formatString("ChannelLeaveAlertWithName", R.string.ChannelLeaveAlertWithName, chat.title)));
                }
                if (!second) {
                    actionText = LocaleController.getString("DeleteAll", R.string.DeleteAll);
                } else if (clear) {
                    if (clearingCache) {
                        actionText = LocaleController.getString("ClearHistoryCache", R.string.ClearHistoryCache);
                    } else {
                        actionText = LocaleController.getString(str, R.string.ClearHistory);
                    }
                } else if (admin) {
                    if (ChatObject.isChannel(chat)) {
                        if (chat.megagroup) {
                            actionText = LocaleController.getString("DeleteMega", R.string.DeleteMega);
                        } else {
                            actionText = LocaleController.getString("ChannelDelete", R.string.ChannelDelete);
                        }
                    } else {
                        actionText = LocaleController.getString("DeleteMega", R.string.DeleteMega);
                    }
                } else if (ChatObject.isChannel(chat)) {
                    if (chat.megagroup) {
                        actionText = LocaleController.getString("LeaveMegaMenu", R.string.LeaveMegaMenu);
                    } else {
                        actionText = LocaleController.getString("LeaveChannelMenu", R.string.LeaveChannelMenu);
                    }
                } else {
                    actionText = LocaleController.getString(str2, R.string.DeleteChatUser);
                }
                XDialog.Builder builder3 = builder;
                builder3.setPositiveButton(actionText, new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda22
                    @Override // android.content.DialogInterface.OnClickListener
                    public final void onClick(DialogInterface dialogInterface, int i) {
                        AlertsCreator.lambda$createClearOrDeleteDialogAlert$13(TLRPC.User.this, clearingCache, second, deleteForAll, fragment, clear, admin, chat, secret, onProcessRunnable, dialogInterface, i);
                    }
                });
                builder3.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
                XDialog alertDialog = builder3.create();
                fragment.showDialog(alertDialog);
                button = (TextView) alertDialog.getButton(-1);
                if (button == null) {
                    button.setTextColor(Theme.getColor(Theme.key_dialogTextRed2));
                    return;
                }
                return;
            }
        }
        z = false;
        boolean canDeleteInbox2 = z;
        final boolean[] deleteForAll2 = new boolean[1];
        if (!second) {
        }
        str = "ClearHistory";
        str2 = "DeleteChatUser";
        revokeTimeLimit2 = revokeTimeLimit;
        if (user == null) {
        }
        if (!second) {
        }
        if (!second) {
        }
        XDialog.Builder builder32 = builder;
        builder32.setPositiveButton(actionText, new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda22
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createClearOrDeleteDialogAlert$13(TLRPC.User.this, clearingCache, second, deleteForAll2, fragment, clear, admin, chat, secret, onProcessRunnable, dialogInterface, i);
            }
        });
        builder32.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        XDialog alertDialog2 = builder32.create();
        fragment.showDialog(alertDialog2);
        button = (TextView) alertDialog2.getButton(-1);
        if (button == null) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createClearOrDeleteDialogAlert$11(boolean[] deleteForAll, View v) {
        CheckBoxCell cell1 = (CheckBoxCell) v;
        deleteForAll[0] = !deleteForAll[0];
        cell1.setChecked(deleteForAll[0], true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createClearOrDeleteDialogAlert$13(final TLRPC.User user, boolean clearingCache, boolean second, final boolean[] deleteForAll, final BaseFragment fragment, final boolean clear, final boolean admin, final TLRPC.Chat chat, final boolean secret, final MessagesStorage.BooleanCallback onProcessRunnable, DialogInterface dialogInterface, int i) {
        boolean z = false;
        if (user != null && !clearingCache && !second && deleteForAll[0]) {
            MessagesStorage.getInstance(fragment.getCurrentAccount()).getMessagesCount(user.id, new MessagesStorage.IntCallback() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda49
                @Override // im.uecahxyvjd.messenger.MessagesStorage.IntCallback
                public final void run(int i2) {
                    AlertsCreator.lambda$createClearOrDeleteDialogAlert$12(BaseFragment.this, clear, admin, chat, user, secret, onProcessRunnable, deleteForAll, i2);
                }
            });
        } else if (onProcessRunnable != null) {
            onProcessRunnable.run((second || deleteForAll[0]) ? true : true);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createClearOrDeleteDialogAlert$12(BaseFragment fragment, boolean clear, boolean admin, TLRPC.Chat chat, TLRPC.User user, boolean secret, MessagesStorage.BooleanCallback onProcessRunnable, boolean[] deleteForAll, int count) {
        if (count >= 50) {
            createClearOrDeleteDialogAlert(fragment, clear, admin, true, chat, user, secret, onProcessRunnable);
        } else if (onProcessRunnable != null) {
            onProcessRunnable.run(deleteForAll[0]);
        }
    }

    public static AlertDialog.Builder createDatePickerDialog(Context context, int minYear, int maxYear, int currentYearDiff, int selectedDay, int selectedMonth, int selectedYear, String title, final boolean checkMinDate, final DatePickerDelegate datePickerDelegate) {
        if (context == null) {
            return null;
        }
        LinearLayout linearLayout = new LinearLayout(context);
        linearLayout.setOrientation(0);
        linearLayout.setWeightSum(1.0f);
        final NumberPicker monthPicker = new NumberPicker(context);
        final NumberPicker dayPicker = new NumberPicker(context);
        final NumberPicker yearPicker = new NumberPicker(context);
        linearLayout.addView(dayPicker, LayoutHelper.createLinear(0, -2, 0.3f));
        dayPicker.setOnScrollListener(new NumberPicker.OnScrollListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda56
            @Override // im.uecahxyvjd.ui.components.NumberPicker.OnScrollListener
            public final void onScrollStateChange(NumberPicker numberPicker, int i) {
                AlertsCreator.lambda$createDatePickerDialog$14(checkMinDate, dayPicker, monthPicker, yearPicker, numberPicker, i);
            }
        });
        monthPicker.setMinValue(0);
        monthPicker.setMaxValue(11);
        linearLayout.addView(monthPicker, LayoutHelper.createLinear(0, -2, 0.3f));
        monthPicker.setFormatter(new NumberPicker.Formatter() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda57
            @Override // im.uecahxyvjd.ui.components.NumberPicker.Formatter
            public final String format(int i) {
                return AlertsCreator.lambda$createDatePickerDialog$15(i);
            }
        });
        monthPicker.setOnValueChangedListener(new NumberPicker.OnValueChangeListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda58
            @Override // im.uecahxyvjd.ui.components.NumberPicker.OnValueChangeListener
            public final void onValueChange(NumberPicker numberPicker, int i, int i2) {
                AlertsCreator.updateDayPicker(NumberPicker.this, monthPicker, yearPicker);
            }
        });
        monthPicker.setOnScrollListener(new NumberPicker.OnScrollListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda1
            @Override // im.uecahxyvjd.ui.components.NumberPicker.OnScrollListener
            public final void onScrollStateChange(NumberPicker numberPicker, int i) {
                AlertsCreator.lambda$createDatePickerDialog$17(checkMinDate, dayPicker, monthPicker, yearPicker, numberPicker, i);
            }
        });
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(System.currentTimeMillis());
        int currentYear = calendar.get(1);
        yearPicker.setMinValue(currentYear + minYear);
        yearPicker.setMaxValue(currentYear + maxYear);
        yearPicker.setValue(currentYear + currentYearDiff);
        linearLayout.addView(yearPicker, LayoutHelper.createLinear(0, -2, 0.4f));
        yearPicker.setOnValueChangedListener(new NumberPicker.OnValueChangeListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda2
            @Override // im.uecahxyvjd.ui.components.NumberPicker.OnValueChangeListener
            public final void onValueChange(NumberPicker numberPicker, int i, int i2) {
                AlertsCreator.updateDayPicker(NumberPicker.this, monthPicker, yearPicker);
            }
        });
        yearPicker.setOnScrollListener(new NumberPicker.OnScrollListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda3
            @Override // im.uecahxyvjd.ui.components.NumberPicker.OnScrollListener
            public final void onScrollStateChange(NumberPicker numberPicker, int i) {
                AlertsCreator.lambda$createDatePickerDialog$19(checkMinDate, dayPicker, monthPicker, yearPicker, numberPicker, i);
            }
        });
        updateDayPicker(dayPicker, monthPicker, yearPicker);
        if (checkMinDate) {
            checkPickerDate(dayPicker, monthPicker, yearPicker);
        }
        if (selectedDay != -1) {
            dayPicker.setValue(selectedDay);
            monthPicker.setValue(selectedMonth);
            yearPicker.setValue(selectedYear);
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title);
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("Set", R.string.Set), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda4
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createDatePickerDialog$20(checkMinDate, dayPicker, monthPicker, yearPicker, datePickerDelegate, dialogInterface, i);
            }
        });
        builder.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDatePickerDialog$14(boolean checkMinDate, NumberPicker dayPicker, NumberPicker monthPicker, NumberPicker yearPicker, NumberPicker view, int scrollState) {
        if (checkMinDate && scrollState == 0) {
            checkPickerDate(dayPicker, monthPicker, yearPicker);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ String lambda$createDatePickerDialog$15(int value) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(5, 1);
        calendar.set(2, value);
        return calendar.getDisplayName(2, 1, Locale.getDefault());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDatePickerDialog$17(boolean checkMinDate, NumberPicker dayPicker, NumberPicker monthPicker, NumberPicker yearPicker, NumberPicker view, int scrollState) {
        if (checkMinDate && scrollState == 0) {
            checkPickerDate(dayPicker, monthPicker, yearPicker);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDatePickerDialog$19(boolean checkMinDate, NumberPicker dayPicker, NumberPicker monthPicker, NumberPicker yearPicker, NumberPicker view, int scrollState) {
        if (checkMinDate && scrollState == 0) {
            checkPickerDate(dayPicker, monthPicker, yearPicker);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDatePickerDialog$20(boolean checkMinDate, NumberPicker dayPicker, NumberPicker monthPicker, NumberPicker yearPicker, DatePickerDelegate datePickerDelegate, DialogInterface dialog, int which) {
        if (checkMinDate) {
            checkPickerDate(dayPicker, monthPicker, yearPicker);
        }
        datePickerDelegate.didSelectDate(yearPicker.getValue(), monthPicker.getValue(), dayPicker.getValue());
    }

    /* JADX INFO: Access modifiers changed from: private */
    public static boolean checkScheduleDate(TextView button, boolean reminder, NumberPicker dayPicker, NumberPicker hourPicker, NumberPicker minutePicker) {
        long currentTime;
        int num;
        int day = dayPicker.getValue();
        int hour = hourPicker.getValue();
        int minute = minutePicker.getValue();
        Calendar calendar = Calendar.getInstance();
        long systemTime = System.currentTimeMillis();
        calendar.setTimeInMillis(systemTime);
        int currentYear = calendar.get(1);
        int currentDay = calendar.get(6);
        calendar.setTimeInMillis(System.currentTimeMillis() + (day * 24 * 3600 * 1000));
        calendar.set(11, hour);
        calendar.set(12, minute);
        long currentTime2 = calendar.getTimeInMillis();
        if (currentTime2 <= systemTime + 60000) {
            currentTime = currentTime2;
            calendar.setTimeInMillis(systemTime + 60000);
            if (currentDay != calendar.get(6)) {
                day = 1;
                dayPicker.setValue(1);
            }
            int i = calendar.get(11);
            hour = i;
            hourPicker.setValue(i);
            int i2 = calendar.get(12);
            minute = i2;
            minutePicker.setValue(i2);
        } else {
            currentTime = currentTime2;
        }
        int selectedYear = calendar.get(1);
        calendar.setTimeInMillis(System.currentTimeMillis() + (day * 24 * 3600 * 1000));
        calendar.set(11, hour);
        calendar.set(12, minute);
        if (button != null) {
            long time = calendar.getTimeInMillis();
            if (day == 0) {
                num = 0;
            } else if (currentYear == selectedYear) {
                num = 1;
            } else {
                num = 2;
            }
            if (reminder) {
                num += 3;
            }
            button.setText(LocaleController.getInstance().formatterScheduleSend[num].format(time));
        }
        return currentTime - systemTime > 60000;
    }

    public static BottomSheet.Builder createScheduleDatePickerDialog(Context context, boolean reminder, ScheduleDatePickerDelegate datePickerDelegate) {
        return createScheduleDatePickerDialog(context, reminder, -1L, datePickerDelegate, null);
    }

    public static BottomSheet.Builder createScheduleDatePickerDialog(Context context, boolean reminder, ScheduleDatePickerDelegate datePickerDelegate, Runnable cancelRunnable) {
        return createScheduleDatePickerDialog(context, reminder, -1L, datePickerDelegate, cancelRunnable);
    }

    public static BottomSheet.Builder createScheduleDatePickerDialog(Context context, final boolean reminder, long currentDate, final ScheduleDatePickerDelegate datePickerDelegate, final Runnable cancelRunnable) {
        String str;
        int i;
        NumberPicker.OnValueChangeListener onValueChangeListener;
        Calendar calendar;
        if (context != null) {
            final BottomSheet.Builder builder = new BottomSheet.Builder(context, false, 1);
            builder.setApplyBottomPadding(false);
            final NumberPicker dayPicker = new NumberPicker(context);
            dayPicker.setTextOffset(AndroidUtilities.dp(10.0f));
            dayPicker.setItemCount(5);
            final NumberPicker hourPicker = new NumberPicker(context);
            hourPicker.setItemCount(5);
            hourPicker.setTextOffset(-AndroidUtilities.dp(10.0f));
            final NumberPicker minutePicker = new NumberPicker(context);
            minutePicker.setItemCount(5);
            minutePicker.setTextOffset(-AndroidUtilities.dp(34.0f));
            LinearLayout container = new LinearLayout(context) { // from class: im.uecahxyvjd.ui.components.AlertsCreator.4
                boolean ignoreLayout = false;

                @Override // android.widget.LinearLayout, android.view.View
                protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
                    int count;
                    this.ignoreLayout = true;
                    if (AndroidUtilities.displaySize.x > AndroidUtilities.displaySize.y) {
                        count = 3;
                    } else {
                        count = 5;
                    }
                    dayPicker.setItemCount(count);
                    hourPicker.setItemCount(count);
                    minutePicker.setItemCount(count);
                    dayPicker.getLayoutParams().height = AndroidUtilities.dp(54.0f) * count;
                    hourPicker.getLayoutParams().height = AndroidUtilities.dp(54.0f) * count;
                    minutePicker.getLayoutParams().height = AndroidUtilities.dp(54.0f) * count;
                    this.ignoreLayout = false;
                    super.onMeasure(widthMeasureSpec, heightMeasureSpec);
                }

                @Override // android.view.View, android.view.ViewParent
                public void requestLayout() {
                    if (this.ignoreLayout) {
                        return;
                    }
                    super.requestLayout();
                }
            };
            container.setOrientation(1);
            TextView titleView = new TextView(context);
            if (reminder) {
                str = "SetReminder";
                i = R.string.SetReminder;
            } else {
                str = "ScheduleMessage";
                i = R.string.ScheduleMessage;
            }
            titleView.setText(LocaleController.getString(str, i));
            titleView.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
            titleView.setTextSize(1, 20.0f);
            titleView.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
            container.addView(titleView, LayoutHelper.createLinear(-1, -2, 51, 22, 12, 22, 4));
            titleView.setOnTouchListener(new View.OnTouchListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda28
                @Override // android.view.View.OnTouchListener
                public final boolean onTouch(View view, MotionEvent motionEvent) {
                    return AlertsCreator.lambda$createScheduleDatePickerDialog$21(view, motionEvent);
                }
            });
            LinearLayout linearLayout = new LinearLayout(context);
            linearLayout.setOrientation(0);
            linearLayout.setWeightSum(1.0f);
            container.addView(linearLayout, LayoutHelper.createLinear(-1, -2));
            final long currentTime = System.currentTimeMillis();
            final Calendar calendar2 = Calendar.getInstance();
            calendar2.setTimeInMillis(currentTime);
            final int currentYear = calendar2.get(1);
            final TextView buttonTextView = new TextView(context);
            linearLayout.addView(dayPicker, LayoutHelper.createLinear(0, 270, 0.5f));
            dayPicker.setMinValue(0);
            dayPicker.setMaxValue(365);
            dayPicker.setWrapSelectorWheel(false);
            dayPicker.setFormatter(new NumberPicker.Formatter() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda29
                @Override // im.uecahxyvjd.ui.components.NumberPicker.Formatter
                public final String format(int i2) {
                    return AlertsCreator.lambda$createScheduleDatePickerDialog$22(currentTime, calendar2, currentYear, i2);
                }
            });
            NumberPicker.OnValueChangeListener onValueChangeListener2 = new NumberPicker.OnValueChangeListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda30
                @Override // im.uecahxyvjd.ui.components.NumberPicker.OnValueChangeListener
                public final void onValueChange(NumberPicker numberPicker, int i2, int i3) {
                    AlertsCreator.checkScheduleDate(buttonTextView, reminder, dayPicker, hourPicker, minutePicker);
                }
            };
            dayPicker.setOnValueChangedListener(onValueChangeListener2);
            hourPicker.setMinValue(0);
            hourPicker.setMaxValue(23);
            linearLayout.addView(hourPicker, LayoutHelper.createLinear(0, 270, 0.2f));
            hourPicker.setFormatter(new NumberPicker.Formatter() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda31
                @Override // im.uecahxyvjd.ui.components.NumberPicker.Formatter
                public final String format(int i2) {
                    String format;
                    format = String.format("%02d", Integer.valueOf(i2));
                    return format;
                }
            });
            hourPicker.setOnValueChangedListener(onValueChangeListener2);
            minutePicker.setMinValue(0);
            minutePicker.setMaxValue(59);
            minutePicker.setValue(0);
            minutePicker.setFormatter(new NumberPicker.Formatter() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda32
                @Override // im.uecahxyvjd.ui.components.NumberPicker.Formatter
                public final String format(int i2) {
                    String format;
                    format = String.format("%02d", Integer.valueOf(i2));
                    return format;
                }
            });
            linearLayout.addView(minutePicker, LayoutHelper.createLinear(0, 270, 0.3f));
            minutePicker.setOnValueChangedListener(onValueChangeListener2);
            if (currentDate <= 0) {
                onValueChangeListener = onValueChangeListener2;
                calendar = calendar2;
            } else {
                long currentDate2 = 1000 * currentDate;
                calendar = calendar2;
                calendar.setTimeInMillis(System.currentTimeMillis());
                calendar.set(12, 0);
                calendar.set(11, 0);
                onValueChangeListener = onValueChangeListener2;
                int days = (int) ((currentDate2 - calendar.getTimeInMillis()) / 86400000);
                calendar.setTimeInMillis(currentDate2);
                if (days >= 0) {
                    minutePicker.setValue(calendar.get(12));
                    hourPicker.setValue(calendar.get(11));
                    dayPicker.setValue(days);
                }
            }
            final boolean[] canceled = {true};
            checkScheduleDate(buttonTextView, reminder, dayPicker, hourPicker, minutePicker);
            buttonTextView.setPadding(AndroidUtilities.dp(34.0f), 0, AndroidUtilities.dp(34.0f), 0);
            buttonTextView.setGravity(17);
            buttonTextView.setTextColor(Theme.getColor(Theme.key_featuredStickers_buttonText));
            buttonTextView.setTextSize(1, 14.0f);
            buttonTextView.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
            buttonTextView.setBackgroundDrawable(Theme.createSimpleSelectorRoundRectDrawable(AndroidUtilities.dp(4.0f), Theme.getColor(Theme.key_featuredStickers_addButton), Theme.getColor(Theme.key_featuredStickers_addButtonPressed)));
            container.addView(buttonTextView, LayoutHelper.createLinear(-1, 48, 83, 16, 15, 16, 16));
            final Calendar calendar3 = calendar;
            buttonTextView.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda34
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createScheduleDatePickerDialog$26(canceled, reminder, dayPicker, hourPicker, minutePicker, calendar3, datePickerDelegate, builder, view);
                }
            });
            builder.setCustomView(container);
            builder.show().setOnDismissListener(new DialogInterface.OnDismissListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda35
                @Override // android.content.DialogInterface.OnDismissListener
                public final void onDismiss(DialogInterface dialogInterface) {
                    AlertsCreator.lambda$createScheduleDatePickerDialog$27(cancelRunnable, canceled, dialogInterface);
                }
            });
            return builder;
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ boolean lambda$createScheduleDatePickerDialog$21(View v, MotionEvent event) {
        return true;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ String lambda$createScheduleDatePickerDialog$22(long currentTime, Calendar calendar, int currentYear, int value) {
        if (value == 0) {
            return LocaleController.getString("MessageScheduleToday", R.string.MessageScheduleToday);
        }
        long date = (value * 86400000) + currentTime;
        calendar.setTimeInMillis(date);
        int year = calendar.get(1);
        if (year == currentYear) {
            return LocaleController.getInstance().formatterScheduleDay.format(date);
        }
        return LocaleController.getInstance().formatterScheduleYear.format(date);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createScheduleDatePickerDialog$26(boolean[] canceled, boolean reminder, NumberPicker dayPicker, NumberPicker hourPicker, NumberPicker minutePicker, Calendar calendar, ScheduleDatePickerDelegate datePickerDelegate, BottomSheet.Builder builder, View v) {
        canceled[0] = false;
        boolean setSeconds = checkScheduleDate(null, reminder, dayPicker, hourPicker, minutePicker);
        calendar.setTimeInMillis(System.currentTimeMillis() + (dayPicker.getValue() * 24 * 3600 * 1000));
        calendar.set(11, hourPicker.getValue());
        calendar.set(12, minutePicker.getValue());
        if (setSeconds) {
            calendar.set(13, 0);
        }
        datePickerDelegate.didSelectDate(true, (int) (calendar.getTimeInMillis() / 1000));
        builder.getDismissRunnable().run();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createScheduleDatePickerDialog$27(Runnable cancelRunnable, boolean[] canceled, DialogInterface dialog) {
        if (cancelRunnable != null && canceled[0]) {
            cancelRunnable.run();
        }
    }

    public static Dialog createMuteAlert(Context context, final long dialog_id) {
        if (context == null) {
            return null;
        }
        BottomSheet.Builder builder = new BottomSheet.Builder(context);
        builder.setTitle(LocaleController.getString("Notifications", R.string.Notifications));
        CharSequence[] items = {LocaleController.formatString("MuteFor", R.string.MuteFor, LocaleController.formatPluralString("Hours", 1)), LocaleController.formatString("MuteFor", R.string.MuteFor, LocaleController.formatPluralString("Hours", 8)), LocaleController.formatString("MuteFor", R.string.MuteFor, LocaleController.formatPluralString("Days", 2)), LocaleController.getString("MuteDisable", R.string.MuteDisable)};
        builder.setItems(items, new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda38
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createMuteAlert$28(dialog_id, dialogInterface, i);
            }
        });
        return builder.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createMuteAlert$28(long dialog_id, DialogInterface dialogInterface, int i) {
        int setting;
        if (i == 0) {
            setting = 0;
        } else if (i == 1) {
            setting = 1;
        } else if (i == 2) {
            setting = 2;
        } else {
            setting = 3;
        }
        NotificationsController.getInstance(UserConfig.selectedAccount).setDialogNotificationsSettings(dialog_id, setting);
    }

    public static void createReportAlert(Context context, final long dialog_id, final int messageId, final BaseFragment parentFragment) {
        if (context == null || parentFragment == null) {
            return;
        }
        BottomSheet.Builder builder = new BottomSheet.Builder(context);
        builder.setTitle(LocaleController.getString("ReportChat", R.string.ReportChat));
        CharSequence[] items = {LocaleController.getString("ReportChatSpam", R.string.ReportChatSpam), LocaleController.getString("ReportChatViolence", R.string.ReportChatViolence), LocaleController.getString("ReportChatChild", R.string.ReportChatChild), LocaleController.getString("ReportChatPornography", R.string.ReportChatPornography), LocaleController.getString("ReportChatOther", R.string.ReportChatOther)};
        builder.setItems(items, new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda14
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createReportAlert$30(dialog_id, messageId, parentFragment, dialogInterface, i);
            }
        });
        BottomSheet sheet = builder.create();
        parentFragment.showDialog(sheet);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    public static /* synthetic */ void lambda$createReportAlert$30(long dialog_id, int messageId, BaseFragment parentFragment, DialogInterface dialogInterface, int i) {
        TLObject req;
        if (i == 4) {
            Bundle args = new Bundle();
            args.putLong("dialog_id", dialog_id);
            args.putLong("message_id", messageId);
            parentFragment.presentFragment(new ReportOtherActivity(args));
            return;
        }
        TLRPC.InputPeer peer = MessagesController.getInstance(UserConfig.selectedAccount).getInputPeer((int) dialog_id);
        if (messageId != 0) {
            TLRPC.TL_messages_report request = new TLRPC.TL_messages_report();
            request.peer = peer;
            request.id.add(Integer.valueOf(messageId));
            if (i == 0) {
                request.reason = new TLRPC.TL_inputReportReasonSpam();
            } else if (i == 1) {
                request.reason = new TLRPC.TL_inputReportReasonViolence();
            } else if (i == 2) {
                request.reason = new TLRPC.TL_inputReportReasonChildAbuse();
            } else if (i == 3) {
                request.reason = new TLRPC.TL_inputReportReasonPornography();
            }
            req = request;
        } else {
            TLRPC.TL_account_reportPeer request2 = new TLRPC.TL_account_reportPeer();
            request2.peer = peer;
            if (i == 0) {
                request2.reason = new TLRPC.TL_inputReportReasonSpam();
            } else if (i == 1) {
                request2.reason = new TLRPC.TL_inputReportReasonViolence();
            } else if (i == 2) {
                request2.reason = new TLRPC.TL_inputReportReasonChildAbuse();
            } else if (i == 3) {
                request2.reason = new TLRPC.TL_inputReportReasonPornography();
            }
            req = request2;
        }
        ConnectionsManager.getInstance(UserConfig.selectedAccount).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda0
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                AlertsCreator.lambda$createReportAlert$29(tLObject, tL_error);
            }
        });
        ToastUtils.show((int) R.string.ReportChatSent);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createReportAlert$29(TLObject response, TLRPC.TL_error error) {
    }

    private static String getFloodWaitString(String error) {
        String timeString;
        int time = Utilities.parseInt(error).intValue();
        if (time < 60) {
            timeString = LocaleController.formatPluralString("Seconds", time);
        } else {
            timeString = LocaleController.formatPluralString("Minutes", time / 60);
        }
        return LocaleController.formatString("FloodWaitTime", R.string.FloodWaitTime, timeString);
    }

    public static void showFloodWaitAlert(String error, BaseFragment fragment) {
        String timeString;
        if (error == null || !error.startsWith("FLOOD_WAIT") || fragment == null || fragment.getParentActivity() == null) {
            return;
        }
        int time = Utilities.parseInt(error).intValue();
        if (time < 60) {
            timeString = LocaleController.formatPluralString("Seconds", time);
        } else {
            timeString = LocaleController.formatPluralString("Minutes", time / 60);
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(fragment.getParentActivity());
        builder.setTitle(LocaleController.getString("AppName", R.string.AppName));
        builder.setMessage(LocaleController.formatString("FloodWaitTime", R.string.FloodWaitTime, timeString));
        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
        fragment.showDialog(builder.create(), true, null);
    }

    public static void showSendMediaAlert(int result, BaseFragment fragment) {
        if (result == 0) {
            return;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(fragment.getParentActivity());
        builder.setTitle(LocaleController.getString("AppName", R.string.AppName));
        if (result == 1) {
            builder.setMessage(LocaleController.getString("ErrorSendRestrictedStickers", R.string.ErrorSendRestrictedStickers));
        } else if (result == 2) {
            builder.setMessage(LocaleController.getString("ErrorSendRestrictedMedia", R.string.ErrorSendRestrictedMedia));
        } else if (result == 3) {
            builder.setMessage(LocaleController.getString("ErrorSendRestrictedPolls", R.string.ErrorSendRestrictedPolls));
        } else if (result == 4) {
            builder.setMessage(LocaleController.getString("ErrorSendRestrictedStickersAll", R.string.ErrorSendRestrictedStickersAll));
        } else if (result == 5) {
            builder.setMessage(LocaleController.getString("ErrorSendRestrictedMediaAll", R.string.ErrorSendRestrictedMediaAll));
        } else if (result == 6) {
            builder.setMessage(LocaleController.getString("ErrorSendRestrictedPollsAll", R.string.ErrorSendRestrictedPollsAll));
        }
        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
        fragment.showDialog(builder.create(), true, null);
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    public static void showAddUserAlert(String error, final BaseFragment fragment, boolean isChannel, TLObject request) {
        char c;
        if (error == null || fragment == null || fragment.getParentActivity() == null) {
            return;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(fragment.getParentActivity());
        builder.setTitle(LocaleController.getString("AppName", R.string.AppName));
        switch (error.hashCode()) {
            case -2120721660:
                if (error.equals("CHANNELS_ADMIN_LOCATED_TOO_MUCH")) {
                    c = 17;
                    break;
                }
                c = 65535;
                break;
            case -2012133105:
                if (error.equals("CHANNELS_ADMIN_PUBLIC_TOO_MUCH")) {
                    c = 16;
                    break;
                }
                c = 65535;
                break;
            case -1763467626:
                if (error.equals("USERS_TOO_FEW")) {
                    c = '\t';
                    break;
                }
                c = 65535;
                break;
            case -538116776:
                if (error.equals("USER_BLOCKED")) {
                    c = 1;
                    break;
                }
                c = 65535;
                break;
            case -512775857:
                if (error.equals("USER_RESTRICTED")) {
                    c = '\n';
                    break;
                }
                c = 65535;
                break;
            case -454039871:
                if (error.equals("PEER_FLOOD")) {
                    c = 0;
                    break;
                }
                c = 65535;
                break;
            case -420079733:
                if (error.equals("BOTS_TOO_MUCH")) {
                    c = 7;
                    break;
                }
                c = 65535;
                break;
            case 98635865:
                if (error.equals("USER_KICKED")) {
                    c = '\r';
                    break;
                }
                c = 65535;
                break;
            case 517420851:
                if (error.equals("USER_BOT")) {
                    c = 2;
                    break;
                }
                c = 65535;
                break;
            case 845559454:
                if (error.equals("YOU_BLOCKED_USER")) {
                    c = 11;
                    break;
                }
                c = 65535;
                break;
            case 916342611:
                if (error.equals("USER_ADMIN_INVALID")) {
                    c = 15;
                    break;
                }
                c = 65535;
                break;
            case 1047173446:
                if (error.equals("CHAT_ADMIN_BAN_REQUIRED")) {
                    c = '\f';
                    break;
                }
                c = 65535;
                break;
            case 1167301807:
                if (error.equals("USERS_TOO_MUCH")) {
                    c = 4;
                    break;
                }
                c = 65535;
                break;
            case 1227003815:
                if (error.equals("USER_ID_INVALID")) {
                    c = 3;
                    break;
                }
                c = 65535;
                break;
            case 1253103379:
                if (error.equals("ADMINS_TOO_MUCH")) {
                    c = 6;
                    break;
                }
                c = 65535;
                break;
            case 1355367367:
                if (error.equals("CHANNELS_TOO_MUCH")) {
                    c = 18;
                    break;
                }
                c = 65535;
                break;
            case 1623167701:
                if (error.equals("USER_NOT_MUTUAL_CONTACT")) {
                    c = 5;
                    break;
                }
                c = 65535;
                break;
            case 1754587486:
                if (error.equals("CHAT_ADMIN_INVITE_REQUIRED")) {
                    c = 14;
                    break;
                }
                c = 65535;
                break;
            case 1916725894:
                if (error.equals("USER_PRIVACY_RESTRICTED")) {
                    c = '\b';
                    break;
                }
                c = 65535;
                break;
            default:
                c = 65535;
                break;
        }
        switch (c) {
            case 0:
                builder.setMessage(LocaleController.getString("NobodyLikesSpam2", R.string.NobodyLikesSpam2));
                builder.setNegativeButton(LocaleController.getString("MoreInfo", R.string.MoreInfo), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda39
                    @Override // android.content.DialogInterface.OnClickListener
                    public final void onClick(DialogInterface dialogInterface, int i) {
                        MessagesController.getInstance(r0.getCurrentAccount()).openByUserName("spambot", BaseFragment.this, 1);
                    }
                });
                break;
            case 1:
            case 2:
            case 3:
                if (isChannel) {
                    builder.setMessage(LocaleController.getString("ChannelUserCantAdd", R.string.ChannelUserCantAdd));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("GroupUserCantAdd", R.string.GroupUserCantAdd));
                    break;
                }
            case 4:
                if (isChannel) {
                    builder.setMessage(LocaleController.getString("ChannelUserAddLimit", R.string.ChannelUserAddLimit));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("GroupUserAddLimit", R.string.GroupUserAddLimit));
                    break;
                }
            case 5:
                if (isChannel) {
                    builder.setMessage(LocaleController.getString("ChannelUserLeftError", R.string.ChannelUserLeftError));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("GroupUserLeftError", R.string.GroupUserLeftError));
                    break;
                }
            case 6:
                if (isChannel) {
                    builder.setMessage(LocaleController.getString("ChannelUserCantAdmin", R.string.ChannelUserCantAdmin));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("GroupUserCantAdmin", R.string.GroupUserCantAdmin));
                    break;
                }
            case 7:
                if (isChannel) {
                    builder.setMessage(LocaleController.getString("ChannelUserCantBot", R.string.ChannelUserCantBot));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("GroupUserCantBot", R.string.GroupUserCantBot));
                    break;
                }
            case '\b':
                if (isChannel) {
                    builder.setMessage(LocaleController.getString("InviteToChannelError", R.string.InviteToChannelError));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("InviteToGroupError", R.string.InviteToGroupError));
                    break;
                }
            case '\t':
                builder.setMessage(LocaleController.getString("CreateGroupError", R.string.CreateGroupError));
                break;
            case '\n':
                builder.setMessage(LocaleController.getString("UserRestricted", R.string.UserRestricted));
                break;
            case 11:
                builder.setMessage(LocaleController.getString("YouBlockedUser", R.string.YouBlockedUser));
                break;
            case '\f':
            case '\r':
                builder.setMessage(LocaleController.getString("AddAdminErrorBlacklisted", R.string.AddAdminErrorBlacklisted));
                break;
            case 14:
                builder.setMessage(LocaleController.getString("AddAdminErrorNotAMember", R.string.AddAdminErrorNotAMember));
                break;
            case 15:
                builder.setMessage(LocaleController.getString("AddBannedErrorAdmin", R.string.AddBannedErrorAdmin));
                break;
            case 16:
                builder.setMessage(LocaleController.getString("PublicChannelsTooMuch", R.string.PublicChannelsTooMuch));
                break;
            case 17:
                builder.setMessage(LocaleController.getString("LocatedChannelsTooMuch", R.string.LocatedChannelsTooMuch));
                break;
            case 18:
                if (request instanceof TLRPC.TL_channels_createChannel) {
                    builder.setMessage(LocaleController.getString("ChannelTooMuch", R.string.ChannelTooMuch));
                    break;
                } else {
                    builder.setMessage(LocaleController.getString("ChannelTooMuchJoin", R.string.ChannelTooMuchJoin));
                    break;
                }
            default:
                builder.setMessage(LocaleController.getString("ErrorOccurred", R.string.ErrorOccurred) + "\n" + error);
                break;
        }
        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), null);
        fragment.showDialog(builder.create(), true, null);
    }

    public static Dialog createColorSelectDialog(Activity parentActivity, final long dialog_id, final int globalType, final Runnable onSelect) {
        int currentColor;
        SharedPreferences preferences = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        if (dialog_id != 0) {
            if (preferences.contains("color_" + dialog_id)) {
                currentColor = preferences.getInt("color_" + dialog_id, -16776961);
            } else if (((int) dialog_id) < 0) {
                currentColor = preferences.getInt("GroupLed", -16776961);
            } else {
                int currentColor2 = preferences.getInt("MessagesLed", -16776961);
                currentColor = currentColor2;
            }
        } else if (globalType == 1) {
            currentColor = preferences.getInt("MessagesLed", -16776961);
        } else if (globalType == 0) {
            currentColor = preferences.getInt("GroupLed", -16776961);
        } else {
            currentColor = preferences.getInt("ChannelLed", -16776961);
        }
        final LinearLayout linearLayout = new LinearLayout(parentActivity);
        linearLayout.setOrientation(1);
        String[] descriptions = {LocaleController.getString("ColorRed", R.string.ColorRed), LocaleController.getString("ColorOrange", R.string.ColorOrange), LocaleController.getString("ColorYellow", R.string.ColorYellow), LocaleController.getString("ColorGreen", R.string.ColorGreen), LocaleController.getString("ColorCyan", R.string.ColorCyan), LocaleController.getString("ColorBlue", R.string.ColorBlue), LocaleController.getString("ColorViolet", R.string.ColorViolet), LocaleController.getString("ColorPink", R.string.ColorPink), LocaleController.getString("ColorWhite", R.string.ColorWhite)};
        final int[] selectedColor = {currentColor};
        for (int a2 = 0; a2 < 9; a2++) {
            RadioColorCell cell = new RadioColorCell(parentActivity);
            boolean z = false;
            cell.setPadding(AndroidUtilities.dp(4.0f), 0, AndroidUtilities.dp(4.0f), 0);
            cell.setTag(Integer.valueOf(a2));
            cell.setCheckColor(TextColorCell.colors[a2], TextColorCell.colors[a2]);
            String str = descriptions[a2];
            if (currentColor == TextColorCell.colorsToSave[a2]) {
                z = true;
            }
            cell.setTextAndValue(str, z);
            linearLayout.addView(cell);
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda33
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createColorSelectDialog$32(linearLayout, selectedColor, view);
                }
            });
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        builder.setTitle(LocaleController.getString("LedColor", R.string.LedColor));
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("Set", R.string.Set), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda44
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createColorSelectDialog$33(dialog_id, selectedColor, globalType, onSelect, dialogInterface, i);
            }
        });
        builder.setNeutralButton(LocaleController.getString("LedDisabled", R.string.LedDisabled), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda54
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createColorSelectDialog$34(dialog_id, globalType, onSelect, dialogInterface, i);
            }
        });
        if (dialog_id != 0) {
            builder.setNegativeButton(LocaleController.getString("Default", R.string.Default), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda55
                @Override // android.content.DialogInterface.OnClickListener
                public final void onClick(DialogInterface dialogInterface, int i) {
                    AlertsCreator.lambda$createColorSelectDialog$35(dialog_id, onSelect, dialogInterface, i);
                }
            });
        }
        return builder.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createColorSelectDialog$32(LinearLayout linearLayout, int[] selectedColor, View v) {
        int count = linearLayout.getChildCount();
        int a1 = 0;
        while (true) {
            boolean z = false;
            if (a1 < count) {
                RadioColorCell cell1 = (RadioColorCell) linearLayout.getChildAt(a1);
                if (cell1 == v) {
                    z = true;
                }
                cell1.setChecked(z, true);
                a1++;
            } else {
                selectedColor[0] = TextColorCell.colorsToSave[((Integer) v.getTag()).intValue()];
                return;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createColorSelectDialog$33(long dialog_id, int[] selectedColor, int globalType, Runnable onSelect, DialogInterface dialogInterface, int which) {
        SharedPreferences preferences1 = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        SharedPreferences.Editor editor = preferences1.edit();
        if (dialog_id != 0) {
            editor.putInt("color_" + dialog_id, selectedColor[0]);
        } else if (globalType == 1) {
            editor.putInt("MessagesLed", selectedColor[0]);
        } else if (globalType == 0) {
            editor.putInt("GroupLed", selectedColor[0]);
        } else {
            editor.putInt("ChannelLed", selectedColor[0]);
        }
        editor.commit();
        if (onSelect != null) {
            onSelect.run();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createColorSelectDialog$34(long dialog_id, int globalType, Runnable onSelect, DialogInterface dialog, int which) {
        SharedPreferences preferences12 = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        SharedPreferences.Editor editor = preferences12.edit();
        if (dialog_id != 0) {
            editor.putInt("color_" + dialog_id, 0);
        } else if (globalType == 1) {
            editor.putInt("MessagesLed", 0);
        } else if (globalType == 0) {
            editor.putInt("GroupLed", 0);
        } else {
            editor.putInt("ChannelLed", 0);
        }
        editor.commit();
        if (onSelect != null) {
            onSelect.run();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createColorSelectDialog$35(long dialog_id, Runnable onSelect, DialogInterface dialog, int which) {
        SharedPreferences preferences13 = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        SharedPreferences.Editor editor = preferences13.edit();
        editor.remove("color_" + dialog_id);
        editor.commit();
        if (onSelect != null) {
            onSelect.run();
        }
    }

    public static Dialog createVibrationSelectDialog(Activity parentActivity, long dialog_id, boolean globalGroup, boolean globalAll, Runnable onSelect) {
        String prefix;
        if (dialog_id != 0) {
            prefix = "vibrate_";
        } else {
            prefix = globalGroup ? "vibrate_group" : "vibrate_messages";
        }
        return createVibrationSelectDialog(parentActivity, dialog_id, prefix, onSelect);
    }

    public static Dialog createVibrationSelectDialog(Activity parentActivity, final long dialog_id, final String prefKeyPrefix, final Runnable onSelect) {
        String[] descriptions;
        Activity activity = parentActivity;
        SharedPreferences preferences = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        final int[] selected = new int[1];
        int i = 0;
        if (dialog_id != 0) {
            selected[0] = preferences.getInt(prefKeyPrefix + dialog_id, 0);
            if (selected[0] == 3) {
                selected[0] = 2;
            } else if (selected[0] == 2) {
                selected[0] = 3;
            }
            descriptions = new String[]{LocaleController.getString("VibrationDefault", R.string.VibrationDefault), LocaleController.getString("Short", R.string.Short), LocaleController.getString("Long", R.string.Long), LocaleController.getString("VibrationDisabled", R.string.VibrationDisabled)};
        } else {
            selected[0] = preferences.getInt(prefKeyPrefix, 0);
            if (selected[0] == 0) {
                selected[0] = 1;
            } else if (selected[0] == 1) {
                selected[0] = 2;
            } else if (selected[0] == 2) {
                selected[0] = 0;
            }
            descriptions = new String[]{LocaleController.getString("VibrationDisabled", R.string.VibrationDisabled), LocaleController.getString("VibrationDefault", R.string.VibrationDefault), LocaleController.getString("Short", R.string.Short), LocaleController.getString("Long", R.string.Long), LocaleController.getString("OnlyIfSilent", R.string.OnlyIfSilent)};
        }
        LinearLayout linearLayout = new LinearLayout(activity);
        linearLayout.setOrientation(1);
        AlertDialog.Builder builder = new AlertDialog.Builder(activity);
        int a2 = 0;
        while (a2 < descriptions.length) {
            RadioColorCell cell = new RadioColorCell(activity);
            cell.setPadding(AndroidUtilities.dp(4.0f), i, AndroidUtilities.dp(4.0f), i);
            cell.setTag(Integer.valueOf(a2));
            cell.setCheckColor(Theme.getColor(Theme.key_radioBackground), Theme.getColor(Theme.key_dialogRadioBackgroundChecked));
            cell.setTextAndValue(descriptions[a2], selected[i] == a2);
            linearLayout.addView(cell);
            int a3 = a2;
            final AlertDialog.Builder builder2 = builder;
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda26
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createVibrationSelectDialog$36(selected, dialog_id, prefKeyPrefix, builder2, onSelect, view);
                }
            });
            a2 = a3 + 1;
            activity = parentActivity;
            builder = builder;
            i = 0;
        }
        AlertDialog.Builder builder3 = builder;
        builder3.setTitle(LocaleController.getString("Vibrate", R.string.Vibrate));
        builder3.setView(linearLayout);
        builder3.setPositiveButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder3.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createVibrationSelectDialog$36(int[] selected, long dialog_id, String prefKeyPrefix, AlertDialog.Builder builder, Runnable onSelect, View v) {
        selected[0] = ((Integer) v.getTag()).intValue();
        SharedPreferences preferences1 = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        SharedPreferences.Editor editor = preferences1.edit();
        if (dialog_id != 0) {
            if (selected[0] == 0) {
                editor.putInt(prefKeyPrefix + dialog_id, 0);
            } else if (selected[0] == 1) {
                editor.putInt(prefKeyPrefix + dialog_id, 1);
            } else if (selected[0] == 2) {
                editor.putInt(prefKeyPrefix + dialog_id, 3);
            } else if (selected[0] == 3) {
                editor.putInt(prefKeyPrefix + dialog_id, 2);
            }
        } else if (selected[0] == 0) {
            editor.putInt(prefKeyPrefix, 2);
        } else if (selected[0] == 1) {
            editor.putInt(prefKeyPrefix, 0);
        } else if (selected[0] == 2) {
            editor.putInt(prefKeyPrefix, 1);
        } else if (selected[0] == 3) {
            editor.putInt(prefKeyPrefix, 3);
        } else if (selected[0] == 4) {
            editor.putInt(prefKeyPrefix, 4);
        }
        editor.commit();
        builder.getDismissRunnable().run();
        if (onSelect != null) {
            onSelect.run();
        }
    }

    public static Dialog createLocationUpdateDialog(Activity parentActivity, TLRPC.User user, final MessagesStorage.IntCallback callback) {
        final int[] selected = new int[1];
        String[] descriptions = {LocaleController.getString("SendLiveLocationFor15m", R.string.SendLiveLocationFor15m), LocaleController.getString("SendLiveLocationFor1h", R.string.SendLiveLocationFor1h), LocaleController.getString("SendLiveLocationFor8h", R.string.SendLiveLocationFor8h)};
        final LinearLayout linearLayout = new LinearLayout(parentActivity);
        linearLayout.setOrientation(1);
        TextView titleTextView = new TextView(parentActivity);
        if (user != null) {
            titleTextView.setText(LocaleController.formatString("LiveLocationAlertPrivate", R.string.LiveLocationAlertPrivate, UserObject.getFirstName(user)));
        } else {
            titleTextView.setText(LocaleController.getString("LiveLocationAlertGroup", R.string.LiveLocationAlertGroup));
        }
        titleTextView.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
        titleTextView.setTextSize(1, 16.0f);
        titleTextView.setGravity((LocaleController.isRTL ? 5 : 3) | 48);
        linearLayout.addView(titleTextView, LayoutHelper.createLinear(-2, -2, (LocaleController.isRTL ? 5 : 3) | 48, 24, 0, 24, 8));
        int a2 = 0;
        while (a2 < descriptions.length) {
            RadioColorCell cell = new RadioColorCell(parentActivity);
            cell.setPadding(AndroidUtilities.dp(4.0f), 0, AndroidUtilities.dp(4.0f), 0);
            cell.setTag(Integer.valueOf(a2));
            cell.setCheckColor(Theme.getColor(Theme.key_radioBackground), Theme.getColor(Theme.key_dialogRadioBackgroundChecked));
            cell.setTextAndValue(descriptions[a2], selected[0] == a2);
            linearLayout.addView(cell);
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda20
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createLocationUpdateDialog$37(selected, linearLayout, view);
                }
            });
            a2++;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        builder.setTopImage(new ShareLocationDrawable(parentActivity, 0), Theme.getColor(Theme.key_dialogTopBackground));
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("ShareFile", R.string.ShareFile), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda21
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createLocationUpdateDialog$38(selected, callback, dialogInterface, i);
            }
        });
        builder.setNeutralButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createLocationUpdateDialog$37(int[] selected, LinearLayout linearLayout, View v) {
        int num = ((Integer) v.getTag()).intValue();
        selected[0] = num;
        int count = linearLayout.getChildCount();
        for (int a1 = 0; a1 < count; a1++) {
            View child = linearLayout.getChildAt(a1);
            if (child instanceof RadioColorCell) {
                ((RadioColorCell) child).setChecked(child == v, true);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createLocationUpdateDialog$38(int[] selected, MessagesStorage.IntCallback callback, DialogInterface dialog, int which) {
        int time;
        if (selected[0] == 0) {
            time = 900;
        } else {
            int time2 = selected[0];
            if (time2 == 1) {
                time = 3600;
            } else {
                time = 28800;
            }
        }
        callback.run(time);
    }

    public static AlertDialog.Builder createContactsPermissionDialog(Activity parentActivity, final MessagesStorage.IntCallback callback) {
        AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        builder.setTopImage(R.drawable.permissions_contacts, Theme.getColor(Theme.key_dialogTopBackground));
        builder.setMessage(AndroidUtilities.replaceTags(LocaleController.getString("ContactsPermissionAlert", R.string.ContactsPermissionAlert)));
        builder.setPositiveButton(LocaleController.getString("ContactsPermissionAlertContinue", R.string.ContactsPermissionAlertContinue), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda50
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                MessagesStorage.IntCallback.this.run(1);
            }
        });
        builder.setNegativeButton(LocaleController.getString("ContactsPermissionAlertNotNow", R.string.ContactsPermissionAlertNotNow), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda51
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                MessagesStorage.IntCallback.this.run(0);
            }
        });
        return builder;
    }

    public static Dialog createFreeSpaceDialog(final LaunchActivity parentActivity) {
        final int[] selected = new int[1];
        if (SharedConfig.keepMedia == 2) {
            selected[0] = 3;
        } else if (SharedConfig.keepMedia == 0) {
            selected[0] = 1;
        } else if (SharedConfig.keepMedia == 1) {
            selected[0] = 2;
        } else if (SharedConfig.keepMedia == 3) {
            selected[0] = 0;
        }
        String[] descriptions = {LocaleController.formatPluralString("Days", 3), LocaleController.formatPluralString("Weeks", 1), LocaleController.formatPluralString("Months", 1), LocaleController.getString("LowDiskSpaceNeverRemove", R.string.LowDiskSpaceNeverRemove)};
        final LinearLayout linearLayout = new LinearLayout(parentActivity);
        linearLayout.setOrientation(1);
        TextView titleTextView = new TextView(parentActivity);
        titleTextView.setText(LocaleController.getString("LowDiskSpaceTitle2", R.string.LowDiskSpaceTitle2));
        titleTextView.setTextColor(Theme.getColor(Theme.key_dialogTextBlack));
        titleTextView.setTextSize(1, 16.0f);
        titleTextView.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
        titleTextView.setGravity((LocaleController.isRTL ? 5 : 3) | 48);
        linearLayout.addView(titleTextView, LayoutHelper.createLinear(-2, -2, (LocaleController.isRTL ? 5 : 3) | 48, 24, 0, 24, 8));
        int a2 = 0;
        while (a2 < descriptions.length) {
            RadioColorCell cell = new RadioColorCell(parentActivity);
            cell.setPadding(AndroidUtilities.dp(4.0f), 0, AndroidUtilities.dp(4.0f), 0);
            cell.setTag(Integer.valueOf(a2));
            cell.setCheckColor(Theme.getColor(Theme.key_radioBackground), Theme.getColor(Theme.key_dialogRadioBackgroundChecked));
            cell.setTextAndValue(descriptions[a2], selected[0] == a2);
            linearLayout.addView(cell);
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda15
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createFreeSpaceDialog$41(selected, linearLayout, view);
                }
            });
            a2++;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        builder.setTitle(LocaleController.getString("LowDiskSpaceTitle", R.string.LowDiskSpaceTitle));
        builder.setMessage(LocaleController.getString("LowDiskSpaceMessage", R.string.LowDiskSpaceMessage));
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("OK", R.string.OK), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda16
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                SharedConfig.setKeepMedia(selected[0]);
            }
        });
        builder.setNeutralButton(LocaleController.getString("ClearMediaCache", R.string.ClearMediaCache), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda17
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                LaunchActivity.this.m1665lambda$runLinkRequest$28$imuecahxyvjduiLaunchActivity(new CacheControlActivity());
            }
        });
        return builder.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createFreeSpaceDialog$41(int[] selected, LinearLayout linearLayout, View v) {
        int num = ((Integer) v.getTag()).intValue();
        if (num == 0) {
            selected[0] = 3;
        } else if (num == 1) {
            selected[0] = 0;
        } else if (num == 2) {
            selected[0] = 1;
        } else if (num == 3) {
            selected[0] = 2;
        }
        int count = linearLayout.getChildCount();
        for (int a1 = 0; a1 < count; a1++) {
            View child = linearLayout.getChildAt(a1);
            if (child instanceof RadioColorCell) {
                ((RadioColorCell) child).setChecked(child == v, true);
            }
        }
    }

    public static Dialog createPrioritySelectDialog(Activity parentActivity, final long dialog_id, final int globalType, final Runnable onSelect) {
        String[] descriptions;
        Activity activity = parentActivity;
        final SharedPreferences preferences = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        final int[] selected = new int[1];
        int i = 0;
        if (dialog_id != 0) {
            selected[0] = preferences.getInt("priority_" + dialog_id, 3);
            if (selected[0] == 3) {
                selected[0] = 0;
            } else if (selected[0] == 4) {
                selected[0] = 1;
            } else if (selected[0] == 5) {
                selected[0] = 2;
            } else if (selected[0] == 0) {
                selected[0] = 3;
            } else {
                selected[0] = 4;
            }
            descriptions = new String[]{LocaleController.getString("NotificationsPrioritySettings", R.string.NotificationsPrioritySettings), LocaleController.getString("NotificationsPriorityLow", R.string.NotificationsPriorityLow), LocaleController.getString("NotificationsPriorityMedium", R.string.NotificationsPriorityMedium), LocaleController.getString("NotificationsPriorityHigh", R.string.NotificationsPriorityHigh), LocaleController.getString("NotificationsPriorityUrgent", R.string.NotificationsPriorityUrgent)};
        } else {
            if (dialog_id == 0) {
                if (globalType == 1) {
                    selected[0] = preferences.getInt("priority_messages", 1);
                } else if (globalType == 0) {
                    selected[0] = preferences.getInt("priority_group", 1);
                } else if (globalType == 2) {
                    selected[0] = preferences.getInt("priority_channel", 1);
                }
            }
            if (selected[0] == 4) {
                selected[0] = 0;
            } else if (selected[0] == 5) {
                selected[0] = 1;
            } else if (selected[0] == 0) {
                selected[0] = 2;
            } else {
                selected[0] = 3;
            }
            descriptions = new String[]{LocaleController.getString("NotificationsPriorityLow", R.string.NotificationsPriorityLow), LocaleController.getString("NotificationsPriorityMedium", R.string.NotificationsPriorityMedium), LocaleController.getString("NotificationsPriorityHigh", R.string.NotificationsPriorityHigh), LocaleController.getString("NotificationsPriorityUrgent", R.string.NotificationsPriorityUrgent)};
        }
        LinearLayout linearLayout = new LinearLayout(activity);
        linearLayout.setOrientation(1);
        AlertDialog.Builder builder = new AlertDialog.Builder(activity);
        int a2 = 0;
        while (a2 < descriptions.length) {
            RadioColorCell cell = new RadioColorCell(activity);
            cell.setPadding(AndroidUtilities.dp(4.0f), i, AndroidUtilities.dp(4.0f), i);
            cell.setTag(Integer.valueOf(a2));
            cell.setCheckColor(Theme.getColor(Theme.key_radioBackground), Theme.getColor(Theme.key_dialogRadioBackgroundChecked));
            cell.setTextAndValue(descriptions[a2], selected[i] == a2);
            linearLayout.addView(cell);
            final AlertDialog.Builder builder2 = builder;
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda24
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createPrioritySelectDialog$44(selected, dialog_id, globalType, preferences, builder2, onSelect, view);
                }
            });
            a2++;
            activity = parentActivity;
            linearLayout = linearLayout;
            builder = builder2;
            i = 0;
        }
        AlertDialog.Builder builder3 = builder;
        builder3.setTitle(LocaleController.getString("NotificationsImportance", R.string.NotificationsImportance));
        builder3.setView(linearLayout);
        builder3.setPositiveButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder3.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createPrioritySelectDialog$44(int[] selected, long dialog_id, int globalType, SharedPreferences preferences, AlertDialog.Builder builder, Runnable onSelect, View v) {
        int option;
        int option2;
        selected[0] = ((Integer) v.getTag()).intValue();
        SharedPreferences preferences1 = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        SharedPreferences.Editor editor = preferences1.edit();
        if (dialog_id != 0) {
            if (selected[0] == 0) {
                option2 = 3;
            } else if (selected[0] == 1) {
                option2 = 4;
            } else if (selected[0] == 2) {
                option2 = 5;
            } else {
                int option3 = selected[0];
                if (option3 == 3) {
                    option2 = 0;
                } else {
                    option2 = 1;
                }
            }
            editor.putInt("priority_" + dialog_id, option2);
        } else {
            if (selected[0] == 0) {
                option = 4;
            } else {
                int option4 = selected[0];
                if (option4 == 1) {
                    option = 5;
                } else {
                    int option5 = selected[0];
                    if (option5 == 2) {
                        option = 0;
                    } else {
                        option = 1;
                    }
                }
            }
            if (globalType == 1) {
                editor.putInt("priority_messages", option);
                selected[0] = preferences.getInt("priority_messages", 1);
            } else if (globalType == 0) {
                editor.putInt("priority_group", option);
                selected[0] = preferences.getInt("priority_group", 1);
            } else if (globalType == 2) {
                editor.putInt("priority_channel", option);
                selected[0] = preferences.getInt("priority_channel", 1);
            }
        }
        editor.commit();
        builder.getDismissRunnable().run();
        if (onSelect != null) {
            onSelect.run();
        }
    }

    public static Dialog createPopupSelectDialog(Activity parentActivity, final int globalType, final Runnable onSelect) {
        SharedPreferences preferences = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        final int[] selected = new int[1];
        if (globalType == 1) {
            selected[0] = preferences.getInt("popupAll", 0);
        } else if (globalType == 0) {
            selected[0] = preferences.getInt("popupGroup", 0);
        } else {
            selected[0] = preferences.getInt("popupChannel", 0);
        }
        String[] descriptions = {LocaleController.getString("NoPopup", R.string.NoPopup), LocaleController.getString("OnlyWhenScreenOn", R.string.OnlyWhenScreenOn), LocaleController.getString("OnlyWhenScreenOff", R.string.OnlyWhenScreenOff), LocaleController.getString("AlwaysShowPopup", R.string.AlwaysShowPopup)};
        LinearLayout linearLayout = new LinearLayout(parentActivity);
        linearLayout.setOrientation(1);
        final AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        int a2 = 0;
        while (a2 < descriptions.length) {
            RadioColorCell cell = new RadioColorCell(parentActivity);
            cell.setTag(Integer.valueOf(a2));
            cell.setPadding(AndroidUtilities.dp(4.0f), 0, AndroidUtilities.dp(4.0f), 0);
            cell.setCheckColor(Theme.getColor(Theme.key_radioBackground), Theme.getColor(Theme.key_dialogRadioBackgroundChecked));
            cell.setTextAndValue(descriptions[a2], selected[0] == a2);
            linearLayout.addView(cell);
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda27
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createPopupSelectDialog$45(selected, globalType, builder, onSelect, view);
                }
            });
            a2++;
        }
        builder.setTitle(LocaleController.getString("PopupNotification", R.string.PopupNotification));
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createPopupSelectDialog$45(int[] selected, int globalType, AlertDialog.Builder builder, Runnable onSelect, View v) {
        selected[0] = ((Integer) v.getTag()).intValue();
        SharedPreferences preferences1 = MessagesController.getNotificationsSettings(UserConfig.selectedAccount);
        SharedPreferences.Editor editor = preferences1.edit();
        if (globalType == 1) {
            editor.putInt("popupAll", selected[0]);
        } else if (globalType == 0) {
            editor.putInt("popupGroup", selected[0]);
        } else {
            editor.putInt("popupChannel", selected[0]);
        }
        editor.commit();
        builder.getDismissRunnable().run();
        if (onSelect != null) {
            onSelect.run();
        }
    }

    public static Dialog createSingleChoiceDialog(Activity parentActivity, String[] options, String title, int selected, final DialogInterface.OnClickListener listener) {
        LinearLayout linearLayout = new LinearLayout(parentActivity);
        linearLayout.setOrientation(1);
        final AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        for (int a2 = 0; a2 < options.length; a2++) {
            RadioColorCell cell = new RadioColorCell(parentActivity);
            boolean z = false;
            cell.setPadding(AndroidUtilities.dp(4.0f), 0, AndroidUtilities.dp(4.0f), 0);
            cell.setTag(Integer.valueOf(a2));
            cell.setCheckColor(Theme.getColor(Theme.key_radioBackground), Theme.getColor(Theme.key_dialogRadioBackgroundChecked));
            String str = options[a2];
            if (selected == a2) {
                z = true;
            }
            cell.setTextAndValue(str, z);
            linearLayout.addView(cell);
            cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda25
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    AlertsCreator.lambda$createSingleChoiceDialog$46(AlertDialog.Builder.this, listener, view);
                }
            });
        }
        builder.setTitle(title);
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        return builder.create();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createSingleChoiceDialog$46(AlertDialog.Builder builder, DialogInterface.OnClickListener listener, View v) {
        int sel = ((Integer) v.getTag()).intValue();
        builder.getDismissRunnable().run();
        listener.onClick(null, sel);
    }

    public static AlertDialog.Builder createTTLAlert(Context context, final TLRPC.EncryptedChat encryptedChat) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(LocaleController.getString("MessageLifetime", R.string.MessageLifetime));
        final NumberPicker numberPicker = new NumberPicker(context);
        numberPicker.setMinValue(0);
        numberPicker.setMaxValue(20);
        if (encryptedChat.ttl > 0 && encryptedChat.ttl < 16) {
            numberPicker.setValue(encryptedChat.ttl);
        } else if (encryptedChat.ttl == 30) {
            numberPicker.setValue(16);
        } else if (encryptedChat.ttl == 60) {
            numberPicker.setValue(17);
        } else if (encryptedChat.ttl == 3600) {
            numberPicker.setValue(18);
        } else if (encryptedChat.ttl == 86400) {
            numberPicker.setValue(19);
        } else if (encryptedChat.ttl == 604800) {
            numberPicker.setValue(20);
        } else if (encryptedChat.ttl == 0) {
            numberPicker.setValue(0);
        }
        numberPicker.setFormatter(new NumberPicker.Formatter() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda12
            @Override // im.uecahxyvjd.ui.components.NumberPicker.Formatter
            public final String format(int i) {
                return AlertsCreator.lambda$createTTLAlert$47(i);
            }
        });
        builder.setView(numberPicker);
        builder.setNegativeButton(LocaleController.getString("Done", R.string.Done), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda13
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i) {
                AlertsCreator.lambda$createTTLAlert$48(TLRPC.EncryptedChat.this, numberPicker, dialogInterface, i);
            }
        });
        return builder;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ String lambda$createTTLAlert$47(int value) {
        if (value == 0) {
            return LocaleController.getString("ShortMessageLifetimeForever", R.string.ShortMessageLifetimeForever);
        }
        if (value >= 1 && value < 16) {
            return LocaleController.formatTTLString(value);
        }
        if (value == 16) {
            return LocaleController.formatTTLString(30);
        }
        if (value == 17) {
            return LocaleController.formatTTLString(60);
        }
        if (value == 18) {
            return LocaleController.formatTTLString(3600);
        }
        if (value == 19) {
            return LocaleController.formatTTLString(86400);
        }
        if (value == 20) {
            return LocaleController.formatTTLString(604800);
        }
        return "";
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createTTLAlert$48(TLRPC.EncryptedChat encryptedChat, NumberPicker numberPicker, DialogInterface dialog, int which) {
        int oldValue = encryptedChat.ttl;
        int which2 = numberPicker.getValue();
        if (which2 >= 0 && which2 < 16) {
            encryptedChat.ttl = which2;
        } else if (which2 == 16) {
            encryptedChat.ttl = 30;
        } else if (which2 == 17) {
            encryptedChat.ttl = 60;
        } else if (which2 == 18) {
            encryptedChat.ttl = 3600;
        } else if (which2 == 19) {
            encryptedChat.ttl = 86400;
        } else if (which2 == 20) {
            encryptedChat.ttl = 604800;
        }
        if (oldValue != encryptedChat.ttl) {
            SecretChatHelper.getInstance(UserConfig.selectedAccount).sendTTLMessage(encryptedChat, null);
            MessagesStorage.getInstance(UserConfig.selectedAccount).updateEncryptedChatTTL(encryptedChat);
        }
    }

    public static AlertDialog createAccountSelectDialog(Activity parentActivity, final AccountSelectDelegate delegate) {
        if (UserConfig.getActivatedAccountsCount() < 2) {
            return null;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(parentActivity);
        final Runnable dismissRunnable = builder.getDismissRunnable();
        final AlertDialog[] alertDialog = new AlertDialog[1];
        LinearLayout linearLayout = new LinearLayout(parentActivity);
        linearLayout.setOrientation(1);
        for (int a2 = 0; a2 < 3; a2++) {
            TLRPC.User u = UserConfig.getInstance(a2).getCurrentUser();
            if (u != null) {
                AccountSelectCell cell = new AccountSelectCell(parentActivity);
                cell.setAccount(a2, false);
                cell.setPadding(AndroidUtilities.dp(14.0f), 0, AndroidUtilities.dp(14.0f), 0);
                cell.setBackgroundDrawable(Theme.getSelectorDrawable(false));
                linearLayout.addView(cell, LayoutHelper.createLinear(-1, 50));
                cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda37
                    @Override // android.view.View.OnClickListener
                    public final void onClick(View view) {
                        AlertsCreator.lambda$createAccountSelectDialog$49(alertDialog, dismissRunnable, delegate, view);
                    }
                });
            }
        }
        builder.setTitle(LocaleController.getString("SelectAccount", R.string.SelectAccount));
        builder.setView(linearLayout);
        builder.setPositiveButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        AlertDialog create = builder.create();
        alertDialog[0] = create;
        return create;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createAccountSelectDialog$49(AlertDialog[] alertDialog, Runnable dismissRunnable, AccountSelectDelegate delegate, View v) {
        if (alertDialog[0] != null) {
            alertDialog[0].setOnDismissListener(null);
        }
        dismissRunnable.run();
        AccountSelectCell cell1 = (AccountSelectCell) v;
        delegate.didSelectAccount(cell1.getAccountNumber());
    }

    /* JADX WARN: Removed duplicated region for block: B:209:0x044f  */
    /* JADX WARN: Removed duplicated region for block: B:268:0x05bd  */
    /* JADX WARN: Removed duplicated region for block: B:269:0x05ca  */
    /* JADX WARN: Removed duplicated region for block: B:295:0x0685  */
    /* JADX WARN: Removed duplicated region for block: B:296:0x068d  */
    /* JADX WARN: Removed duplicated region for block: B:299:0x06b4  */
    /* JADX WARN: Removed duplicated region for block: B:327:? A[RETURN, SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static void createDeleteMessagesAlert(final BaseFragment fragment, final TLRPC.User user, final TLRPC.Chat chat, final TLRPC.EncryptedChat encryptedChat, final TLRPC.ChatFull chatInfo, final long mergeDialogId, final MessageObject selectedMessage, final SparseArray<MessageObject>[] selectedMessages, final MessageObject.GroupedMessages selectedGroup, final boolean scheduled, int loadParticipant, final Runnable onDelete) {
        Activity parentActivity;
        int count;
        long dialogId;
        int revokeTimeLimit;
        int revokeTimeLimit2;
        boolean[] checks;
        int count2;
        Activity activity;
        int currentAccount;
        BaseFragment baseFragment;
        TLRPC.Chat chat2;
        boolean[] deleteForAll;
        final boolean[] deleteForAll2;
        int count3;
        TLRPC.User actionUser;
        boolean hasDeleteForAllCheck;
        boolean hasNotOut;
        int myMessagesCount;
        boolean z;
        boolean hasOutgoing;
        TextView button;
        int count4;
        AlertDialog.Builder builder;
        TLRPC.User actionUser2;
        int revokeTimeLimit3;
        FragmentActivity fragmentActivity;
        int count5;
        final boolean[] deleteForAll3;
        TLRPC.User actionUser3;
        final boolean[] checks2;
        TLRPC.User actionUser4;
        if (fragment == null) {
            return;
        }
        if ((user == null && chat == null && encryptedChat == null) || (parentActivity = fragment.getParentActivity()) == null) {
            return;
        }
        final int currentAccount2 = fragment.getCurrentAccount();
        AlertDialog.Builder builder2 = new AlertDialog.Builder(parentActivity);
        if (selectedGroup != null) {
            count = selectedGroup.messages.size();
        } else if (selectedMessage == null) {
            count = selectedMessages[0].size() + selectedMessages[1].size();
        } else {
            count = 1;
        }
        if (encryptedChat != null) {
            dialogId = encryptedChat.id << 32;
        } else if (user != null) {
            dialogId = user.id;
        } else {
            dialogId = -chat.id;
        }
        boolean[] checks3 = new boolean[3];
        boolean[] deleteForAll4 = new boolean[1];
        TLRPC.User actionUser5 = null;
        boolean canRevokeInbox = user != null && MessagesController.getInstance(currentAccount2).canRevokePmInbox;
        if (user != null) {
            revokeTimeLimit = MessagesController.getInstance(currentAccount2).revokeTimePmLimit;
        } else {
            revokeTimeLimit = MessagesController.getInstance(currentAccount2).revokeTimeLimit;
        }
        boolean hasDeleteForAllCheck2 = false;
        boolean hasNotOut2 = false;
        int myMessagesCount2 = 0;
        boolean canDeleteInbox = encryptedChat == null && user != null && canRevokeInbox && revokeTimeLimit == Integer.MAX_VALUE;
        if (chat == null) {
            revokeTimeLimit2 = revokeTimeLimit;
            checks = checks3;
            count2 = count;
            activity = parentActivity;
            currentAccount = currentAccount2;
            baseFragment = fragment;
            chat2 = chat;
            deleteForAll = deleteForAll4;
        } else if (!chat.megagroup || scheduled) {
            revokeTimeLimit2 = revokeTimeLimit;
            count2 = count;
            activity = parentActivity;
            currentAccount = currentAccount2;
            baseFragment = fragment;
            checks = checks3;
            chat2 = chat;
            deleteForAll = deleteForAll4;
        } else {
            boolean canBan = ChatObject.canBlockUsers(chat);
            int currentDate = ConnectionsManager.getInstance(currentAccount2).getCurrentTime();
            if (selectedMessage != null) {
                if (selectedMessage.messageOwner.action == null || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionEmpty) || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionChatDeleteUser) || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionChatJoinedByLink) || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionChatAddUser)) {
                    count4 = count;
                    actionUser5 = MessagesController.getInstance(currentAccount2).getUser(Integer.valueOf(selectedMessage.messageOwner.from_id));
                } else {
                    count4 = count;
                }
                boolean hasOutgoing2 = !selectedMessage.isSendError() && selectedMessage.getDialogId() == mergeDialogId && (selectedMessage.messageOwner.action == null || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionEmpty)) && selectedMessage.isOut() && currentDate - selectedMessage.messageOwner.date <= revokeTimeLimit;
                if (hasOutgoing2) {
                    myMessagesCount2 = 0 + 1;
                }
                builder = builder2;
                actionUser2 = actionUser5;
            } else {
                count4 = count;
                int from_id = -1;
                int a2 = 1;
                while (true) {
                    if (a2 < 0) {
                        builder = builder2;
                        break;
                    }
                    builder = builder2;
                    for (int b = 0; b < selectedMessages[a2].size(); b++) {
                        MessageObject msg = selectedMessages[a2].valueAt(b);
                        if (from_id == -1) {
                            from_id = msg.messageOwner.from_id;
                        }
                        if (from_id < 0 || from_id != msg.messageOwner.from_id) {
                            from_id = -2;
                            break;
                        }
                    }
                    if (from_id == -2) {
                        break;
                    }
                    a2--;
                    builder2 = builder;
                }
                for (int a3 = 1; a3 >= 0; a3--) {
                    for (int b2 = 0; b2 < selectedMessages[a3].size(); b2++) {
                        MessageObject msg2 = selectedMessages[a3].valueAt(b2);
                        if (a3 == 1 && msg2.isOut() && msg2.messageOwner.action == null && currentDate - msg2.messageOwner.date <= revokeTimeLimit) {
                            myMessagesCount2++;
                        }
                    }
                }
                if (from_id == -1) {
                    actionUser2 = null;
                } else {
                    TLRPC.User actionUser6 = MessagesController.getInstance(currentAccount2).getUser(Integer.valueOf(from_id));
                    actionUser2 = actionUser6;
                }
            }
            if (actionUser2 == null || actionUser2.id == UserConfig.getInstance(currentAccount2).getClientUserId()) {
                revokeTimeLimit3 = revokeTimeLimit;
                fragmentActivity = parentActivity;
                currentAccount = currentAccount2;
                baseFragment = fragment;
                checks = checks3;
                count5 = count4;
                builder2 = builder;
                if (0 != 0 || myMessagesCount2 <= 0) {
                    chat2 = chat;
                    deleteForAll3 = deleteForAll4;
                    actionUser3 = null;
                } else {
                    FrameLayout frameLayout = new FrameLayout(fragmentActivity);
                    CheckBoxCell cell = new CheckBoxCell(fragmentActivity, 1);
                    cell.setBackgroundDrawable(Theme.getSelectorDrawable(false));
                    chat2 = chat;
                    if (chat2 == null || 0 == 0) {
                        cell.setText(LocaleController.getString("DeleteMessagesOption", R.string.DeleteMessagesOption), "", false, false);
                    } else {
                        cell.setText(LocaleController.getString("DeleteForAll", R.string.DeleteForAll), "", false, false);
                    }
                    cell.setPadding(LocaleController.isRTL ? AndroidUtilities.dp(16.0f) : AndroidUtilities.dp(8.0f), 0, LocaleController.isRTL ? AndroidUtilities.dp(8.0f) : AndroidUtilities.dp(16.0f), 0);
                    frameLayout.addView(cell, LayoutHelper.createFrame(-1.0f, 48.0f, 51, 0.0f, 0.0f, 0.0f, 0.0f));
                    deleteForAll3 = deleteForAll4;
                    cell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda43
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            AlertsCreator.lambda$createDeleteMessagesAlert$55(deleteForAll3, view);
                        }
                    });
                    builder2.setView(frameLayout);
                    builder2.setCustomViewOffset(9);
                    hasDeleteForAllCheck2 = true;
                    actionUser3 = actionUser2;
                }
            } else if (loadParticipant != 1 || chat.creator) {
                revokeTimeLimit3 = revokeTimeLimit;
                currentAccount = currentAccount2;
                TLRPC.User actionUser7 = actionUser2;
                baseFragment = fragment;
                boolean[] checks4 = checks3;
                count5 = count4;
                AlertDialog.Builder builder3 = builder;
                fragmentActivity = parentActivity;
                FrameLayout frameLayout2 = new FrameLayout(fragmentActivity);
                int num = 0;
                int a4 = 0;
                while (a4 < 3) {
                    if ((loadParticipant == 2 || !canBan) && a4 == 0) {
                        checks2 = checks4;
                        actionUser4 = actionUser7;
                    } else {
                        CheckBoxCell cell2 = new CheckBoxCell(fragmentActivity, 1);
                        cell2.setBackgroundDrawable(Theme.getSelectorDrawable(false));
                        cell2.setTag(Integer.valueOf(a4));
                        if (a4 == 0) {
                            cell2.setText(LocaleController.getString("DeleteBanUser", R.string.DeleteBanUser), "", false, false);
                            actionUser4 = actionUser7;
                        } else if (a4 == 1) {
                            cell2.setText(LocaleController.getString("DeleteReportSpam", R.string.DeleteReportSpam), "", false, false);
                            actionUser4 = actionUser7;
                        } else if (a4 == 2) {
                            actionUser4 = actionUser7;
                            cell2.setText(LocaleController.formatString("DeleteAllFrom", R.string.DeleteAllFrom, ContactsController.formatName(actionUser4.first_name, actionUser4.last_name)), "", false, false);
                        } else {
                            actionUser4 = actionUser7;
                        }
                        cell2.setPadding(LocaleController.isRTL ? AndroidUtilities.dp(16.0f) : AndroidUtilities.dp(8.0f), 0, LocaleController.isRTL ? AndroidUtilities.dp(8.0f) : AndroidUtilities.dp(16.0f), 0);
                        frameLayout2.addView(cell2, LayoutHelper.createFrame(-1.0f, 48.0f, 51, 0.0f, num * 48, 0.0f, 0.0f));
                        checks2 = checks4;
                        cell2.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda42
                            @Override // android.view.View.OnClickListener
                            public final void onClick(View view) {
                                AlertsCreator.lambda$createDeleteMessagesAlert$54(checks2, view);
                            }
                        });
                        num++;
                    }
                    a4++;
                    actionUser7 = actionUser4;
                    checks4 = checks2;
                }
                checks = checks4;
                builder2 = builder3;
                builder2.setView(frameLayout2);
                chat2 = chat;
                actionUser3 = actionUser7;
                deleteForAll3 = deleteForAll4;
            } else {
                final AlertDialog[] progressDialog = {new AlertDialog(parentActivity, 3)};
                TLRPC.TL_channels_getParticipant req = new TLRPC.TL_channels_getParticipant();
                req.channel = MessagesController.getInputChannel(chat);
                req.user_id = MessagesController.getInstance(currentAccount2).getInputUser(actionUser2);
                final int requestId = ConnectionsManager.getInstance(currentAccount2).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda40
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda9
                            @Override // java.lang.Runnable
                            public final void run() {
                                AlertsCreator.lambda$createDeleteMessagesAlert$50(r1, tLObject, r3, r4, r5, r6, r7, r8, r10, r11, r12, r13, r14);
                            }
                        });
                    }
                });
                AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda41
                    @Override // java.lang.Runnable
                    public final void run() {
                        AlertsCreator.lambda$createDeleteMessagesAlert$53(progressDialog, currentAccount2, requestId, fragment);
                    }
                }, 1000L);
                return;
            }
            deleteForAll2 = deleteForAll3;
            actionUser = actionUser3;
            hasDeleteForAllCheck = hasDeleteForAllCheck2;
            hasNotOut = false;
            myMessagesCount = myMessagesCount2;
            count3 = count5;
            final TLRPC.User userFinal = actionUser;
            final int i = currentAccount;
            final long j = dialogId;
            final boolean[] zArr = deleteForAll2;
            final boolean[] zArr2 = checks;
            builder2.setPositiveButton(LocaleController.getString("Delete", R.string.Delete), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda46
                @Override // android.content.DialogInterface.OnClickListener
                public final void onClick(DialogInterface dialogInterface, int i2) {
                    AlertsCreator.lambda$createDeleteMessagesAlert$58(MessageObject.this, selectedGroup, encryptedChat, i, j, zArr, scheduled, selectedMessages, userFinal, zArr2, chat, chatInfo, onDelete, dialogInterface, i2);
                }
            });
            if (count3 != 1) {
                builder2.setTitle(LocaleController.getString("DeleteSingleMessagesTitle", R.string.DeleteSingleMessagesTitle));
            } else {
                builder2.setTitle(LocaleController.formatString("DeleteMessagesTitle", R.string.DeleteMessagesTitle, LocaleController.formatPluralString("messages", count3)));
            }
            if (chat2 == null && hasNotOut) {
                if (hasDeleteForAllCheck && myMessagesCount != count3) {
                    builder2.setMessage(LocaleController.formatString("DeleteMessagesTextGroupPart", R.string.DeleteMessagesTextGroupPart, LocaleController.formatPluralString("messages", myMessagesCount)));
                } else if (count3 == 1) {
                    builder2.setMessage(LocaleController.getString("AreYouSureDeleteSingleMessage", R.string.AreYouSureDeleteSingleMessage));
                } else {
                    builder2.setMessage(LocaleController.getString("AreYouSureDeleteFewMessages", R.string.AreYouSureDeleteFewMessages));
                }
            } else if (!hasDeleteForAllCheck && !canDeleteInbox && myMessagesCount != count3) {
                if (chat2 != null) {
                    builder2.setMessage(LocaleController.formatString("DeleteMessagesTextGroup", R.string.DeleteMessagesTextGroup, LocaleController.formatPluralString("messages", myMessagesCount)));
                } else {
                    builder2.setMessage(AndroidUtilities.replaceTags(LocaleController.formatString("DeleteMessagesText", R.string.DeleteMessagesText, LocaleController.formatPluralString("messages", myMessagesCount), UserObject.getFirstName(user))));
                }
            } else if (chat2 == null && chat2.megagroup) {
                if (count3 == 1) {
                    builder2.setMessage(LocaleController.getString("AreYouSureDeleteSingleMessageMega", R.string.AreYouSureDeleteSingleMessageMega));
                } else {
                    builder2.setMessage(LocaleController.getString("AreYouSureDeleteFewMessagesMega", R.string.AreYouSureDeleteFewMessagesMega));
                }
            } else if (count3 != 1) {
                builder2.setMessage(LocaleController.getString("AreYouSureDeleteSingleMessage", R.string.AreYouSureDeleteSingleMessage));
            } else {
                builder2.setMessage(LocaleController.getString("AreYouSureDeleteFewMessages", R.string.AreYouSureDeleteFewMessages));
            }
            builder2.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
            AlertDialog dialog = builder2.create();
            baseFragment.showDialog(dialog);
            button = (TextView) dialog.getButton(-1);
            if (button == null) {
                button.setTextColor(Theme.getColor(Theme.key_dialogTextRed2));
                return;
            }
            return;
        }
        if (!scheduled && !ChatObject.isChannel(chat) && encryptedChat == null) {
            int currentDate2 = ConnectionsManager.getInstance(currentAccount).getCurrentTime();
            deleteForAll2 = deleteForAll;
            if ((user == null || user.id == UserConfig.getInstance(currentAccount).getClientUserId() || user.bot) && chat2 == null) {
                myMessagesCount = 0;
            } else if (selectedMessage != null) {
                if (!selectedMessage.isSendError()) {
                    if ((selectedMessage.messageOwner.action == null || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionEmpty) || (selectedMessage.messageOwner.action instanceof TLRPC.TL_messageActionPhoneCall)) && (selectedMessage.isOut() || canRevokeInbox || ChatObject.hasAdminRights(chat))) {
                        if (currentDate2 - selectedMessage.messageOwner.date <= revokeTimeLimit2) {
                            hasOutgoing = true;
                            if (hasOutgoing) {
                                myMessagesCount2 = 0 + 1;
                            }
                            hasNotOut2 = !selectedMessage.isOut();
                            myMessagesCount = myMessagesCount2;
                        }
                    }
                }
                hasOutgoing = false;
                if (hasOutgoing) {
                }
                hasNotOut2 = !selectedMessage.isOut();
                myMessagesCount = myMessagesCount2;
            } else {
                int revokeTimeLimit4 = revokeTimeLimit2;
                for (int a5 = 1; a5 >= 0; a5--) {
                    for (int b3 = 0; b3 < selectedMessages[a5].size(); b3++) {
                        MessageObject msg3 = selectedMessages[a5].valueAt(b3);
                        if ((msg3.messageOwner.action == null || (msg3.messageOwner.action instanceof TLRPC.TL_messageActionEmpty) || (msg3.messageOwner.action instanceof TLRPC.TL_messageActionPhoneCall)) && ((msg3.isOut() || canRevokeInbox || (chat2 != null && ChatObject.canBlockUsers(chat))) && currentDate2 - msg3.messageOwner.date <= revokeTimeLimit4)) {
                            myMessagesCount2++;
                            if (!hasNotOut2 && !msg3.isOut()) {
                                hasNotOut2 = true;
                            }
                        }
                    }
                }
                myMessagesCount = myMessagesCount2;
            }
            if (myMessagesCount <= 0) {
                count3 = count2;
                actionUser = null;
                hasDeleteForAllCheck = false;
                hasNotOut = hasNotOut2;
            } else {
                FrameLayout frameLayout3 = new FrameLayout(activity);
                CheckBoxCell cell3 = new CheckBoxCell(activity, 1);
                cell3.setBackgroundDrawable(Theme.getSelectorDrawable(false));
                if (canDeleteInbox) {
                    cell3.setText(LocaleController.formatString("DeleteMessagesOptionAlso", R.string.DeleteMessagesOptionAlso, UserObject.getFirstName(user)), "", false, false);
                    count3 = count2;
                } else {
                    if (chat2 != null) {
                        if (hasNotOut2) {
                            count3 = count2;
                        } else {
                            count3 = count2;
                            z = myMessagesCount != count3 ? false : false;
                        }
                        cell3.setText(LocaleController.getString("DeleteForAll", R.string.DeleteForAll), "", false, false);
                    } else {
                        count3 = count2;
                    }
                    cell3.setText(LocaleController.getString("DeleteMessagesOption", R.string.DeleteMessagesOption), "", z, z);
                }
                cell3.setPadding(LocaleController.isRTL ? AndroidUtilities.dp(16.0f) : AndroidUtilities.dp(8.0f), 0, LocaleController.isRTL ? AndroidUtilities.dp(8.0f) : AndroidUtilities.dp(16.0f), 0);
                frameLayout3.addView(cell3, LayoutHelper.createFrame(-1.0f, 48.0f, 51, 0.0f, 0.0f, 0.0f, 0.0f));
                cell3.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda45
                    @Override // android.view.View.OnClickListener
                    public final void onClick(View view) {
                        AlertsCreator.lambda$createDeleteMessagesAlert$56(deleteForAll2, view);
                    }
                });
                builder2.setView(frameLayout3);
                builder2.setCustomViewOffset(9);
                actionUser = null;
                hasDeleteForAllCheck = true;
                hasNotOut = hasNotOut2;
            }
        } else {
            deleteForAll2 = deleteForAll;
            count3 = count2;
            actionUser = null;
            hasDeleteForAllCheck = false;
            hasNotOut = false;
            myMessagesCount = 0;
        }
        final TLRPC.User userFinal2 = actionUser;
        final int i2 = currentAccount;
        final long j2 = dialogId;
        final boolean[] zArr3 = deleteForAll2;
        final boolean[] zArr22 = checks;
        builder2.setPositiveButton(LocaleController.getString("Delete", R.string.Delete), new DialogInterface.OnClickListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda46
            @Override // android.content.DialogInterface.OnClickListener
            public final void onClick(DialogInterface dialogInterface, int i22) {
                AlertsCreator.lambda$createDeleteMessagesAlert$58(MessageObject.this, selectedGroup, encryptedChat, i2, j2, zArr3, scheduled, selectedMessages, userFinal2, zArr22, chat, chatInfo, onDelete, dialogInterface, i22);
            }
        });
        if (count3 != 1) {
        }
        if (chat2 == null) {
        }
        if (!hasDeleteForAllCheck) {
        }
        if (chat2 == null) {
        }
        if (count3 != 1) {
        }
        builder2.setNegativeButton(LocaleController.getString("Cancel", R.string.Cancel), null);
        AlertDialog dialog2 = builder2.create();
        baseFragment.showDialog(dialog2);
        button = (TextView) dialog2.getButton(-1);
        if (button == null) {
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$50(AlertDialog[] progressDialog, TLObject response, BaseFragment fragment, TLRPC.User user, TLRPC.Chat chat, TLRPC.EncryptedChat encryptedChat, TLRPC.ChatFull chatInfo, long mergeDialogId, MessageObject selectedMessage, SparseArray[] selectedMessages, MessageObject.GroupedMessages selectedGroup, boolean scheduled, Runnable onDelete) {
        try {
            progressDialog[0].dismiss();
        } catch (Throwable th) {
        }
        progressDialog[0] = null;
        int loadType = 2;
        if (response != null) {
            TLRPC.TL_channels_channelParticipant participant = (TLRPC.TL_channels_channelParticipant) response;
            if (!(participant.participant instanceof TLRPC.TL_channelParticipantAdmin) && !(participant.participant instanceof TLRPC.TL_channelParticipantCreator)) {
                loadType = 0;
            }
        }
        createDeleteMessagesAlert(fragment, user, chat, encryptedChat, chatInfo, mergeDialogId, selectedMessage, selectedMessages, selectedGroup, scheduled, loadType, onDelete);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$53(AlertDialog[] progressDialog, final int currentAccount, final int requestId, BaseFragment fragment) {
        if (progressDialog[0] == null) {
            return;
        }
        progressDialog[0].setOnCancelListener(new DialogInterface.OnCancelListener() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda10
            @Override // android.content.DialogInterface.OnCancelListener
            public final void onCancel(DialogInterface dialogInterface) {
                ConnectionsManager.getInstance(currentAccount).cancelRequest(requestId, true);
            }
        });
        fragment.showDialog(progressDialog[0]);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$54(boolean[] checks, View v) {
        if (!v.isEnabled()) {
            return;
        }
        CheckBoxCell cell13 = (CheckBoxCell) v;
        Integer num1 = (Integer) cell13.getTag();
        checks[num1.intValue()] = !checks[num1.intValue()];
        cell13.setChecked(checks[num1.intValue()], true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$55(boolean[] deleteForAll, View v) {
        CheckBoxCell cell12 = (CheckBoxCell) v;
        deleteForAll[0] = !deleteForAll[0];
        cell12.setChecked(deleteForAll[0], true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$56(boolean[] deleteForAll, View v) {
        CheckBoxCell cell1 = (CheckBoxCell) v;
        deleteForAll[0] = !deleteForAll[0];
        cell1.setChecked(deleteForAll[0], true);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Removed duplicated region for block: B:44:0x00fb  */
    /* JADX WARN: Removed duplicated region for block: B:79:0x0131 A[SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$58(MessageObject selectedMessage, MessageObject.GroupedMessages selectedGroup, TLRPC.EncryptedChat encryptedChat, int currentAccount, long dialogId, boolean[] deleteForAll, boolean scheduled, SparseArray[] selectedMessages, TLRPC.User userFinal, boolean[] checks, TLRPC.Chat chat, TLRPC.ChatFull chatInfo, Runnable onDelete, DialogInterface dialogInterface, int i) {
        int i2;
        ArrayList<Integer> ids;
        int channelId;
        ArrayList<Long> random_ids;
        ArrayList<Integer> ids2 = null;
        int i3 = 10;
        int i4 = 0;
        if (selectedMessage != null) {
            ArrayList<Integer> ids3 = new ArrayList<>();
            ArrayList<Long> random_ids2 = null;
            if (selectedGroup != null) {
                for (int a2 = 0; a2 < selectedGroup.messages.size(); a2++) {
                    MessageObject messageObject = selectedGroup.messages.get(a2);
                    ids3.add(Integer.valueOf(messageObject.getId()));
                    if (encryptedChat != null && messageObject.messageOwner.random_id != 0 && messageObject.type != 10) {
                        if (random_ids2 == null) {
                            random_ids2 = new ArrayList<>();
                        }
                        random_ids2.add(Long.valueOf(messageObject.messageOwner.random_id));
                    }
                }
                random_ids = random_ids2;
            } else {
                ids3.add(Integer.valueOf(selectedMessage.getId()));
                if (encryptedChat != null && selectedMessage.messageOwner.random_id != 0 && selectedMessage.type != 10) {
                    ArrayList<Long> random_ids3 = new ArrayList<>();
                    random_ids3.add(Long.valueOf(selectedMessage.messageOwner.random_id));
                    random_ids = random_ids3;
                } else {
                    random_ids = null;
                }
            }
            MessagesController.getInstance(currentAccount).deleteMessages(ids3, random_ids, encryptedChat, dialogId, selectedMessage.messageOwner.to_id.channel_id, deleteForAll[0], scheduled);
            ids = ids3;
            i2 = 0;
        } else {
            int a3 = 1;
            while (a3 >= 0) {
                ArrayList<Integer> ids4 = new ArrayList<>();
                for (int b = 0; b < selectedMessages[a3].size(); b++) {
                    ids4.add(Integer.valueOf(selectedMessages[a3].keyAt(b)));
                }
                ArrayList<Long> random_ids4 = null;
                if (!ids4.isEmpty()) {
                    MessageObject msg = (MessageObject) selectedMessages[a3].get(ids4.get(i4).intValue());
                    if (0 == 0 && msg.messageOwner.to_id.channel_id != 0) {
                        int channelId2 = msg.messageOwner.to_id.channel_id;
                        channelId = channelId2;
                        if (encryptedChat == null) {
                            random_ids4 = new ArrayList<>();
                            for (int b2 = 0; b2 < selectedMessages[a3].size(); b2++) {
                                MessageObject msg2 = (MessageObject) selectedMessages[a3].valueAt(b2);
                                if (msg2.messageOwner.random_id != 0 && msg2.type != i3) {
                                    random_ids4.add(Long.valueOf(msg2.messageOwner.random_id));
                                }
                            }
                        }
                        MessagesController.getInstance(currentAccount).deleteMessages(ids4, random_ids4, encryptedChat, dialogId, channelId, deleteForAll[i4], scheduled);
                        selectedMessages[a3].clear();
                        a3--;
                        ids2 = ids4;
                        i4 = 0;
                        i3 = 10;
                    }
                }
                channelId = 0;
                if (encryptedChat == null) {
                }
                MessagesController.getInstance(currentAccount).deleteMessages(ids4, random_ids4, encryptedChat, dialogId, channelId, deleteForAll[i4], scheduled);
                selectedMessages[a3].clear();
                a3--;
                ids2 = ids4;
                i4 = 0;
                i3 = 10;
            }
            i2 = 0;
            ids = ids2;
        }
        if (userFinal != null) {
            if (checks[i2]) {
                MessagesController.getInstance(currentAccount).deleteUserFromChat(chat.id, userFinal, chatInfo);
            }
            if (checks[1]) {
                TLRPC.TL_channels_reportSpam req = new TLRPC.TL_channels_reportSpam();
                req.channel = MessagesController.getInputChannel(chat);
                req.user_id = MessagesController.getInstance(currentAccount).getInputUser(userFinal);
                req.id = ids;
                ConnectionsManager.getInstance(currentAccount).sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.components.AlertsCreator$$ExternalSyntheticLambda48
                    @Override // im.uecahxyvjd.tgnet.RequestDelegate
                    public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                        AlertsCreator.lambda$createDeleteMessagesAlert$57(tLObject, tL_error);
                    }
                });
            }
            if (checks[2]) {
                MessagesController.getInstance(currentAccount).deleteUserChannelHistory(chat, userFinal, i2);
            }
        }
        if (onDelete != null) {
            onDelete.run();
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ void lambda$createDeleteMessagesAlert$57(TLObject response, TLRPC.TL_error error) {
    }
}
