package im.uecahxyvjd.ui.components;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.Shader;
import android.view.View;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.ImageReceiver;
/* loaded from: classes10.dex */
public class ClippingImageView extends View {
    private float animationProgress;
    private float[][] animationValues;
    private RectF bitmapRect;
    private BitmapShader bitmapShader;
    private ImageReceiver.BitmapHolder bmp;
    private int clipBottom;
    private int clipLeft;
    private int clipRight;
    private int clipTop;
    private RectF drawRect;
    private int imageX;
    private int imageY;
    private Matrix matrix;
    private boolean needRadius;
    private int orientation;
    private Paint paint;
    private int radius;
    private Paint roundPaint;
    private RectF roundRect;
    private Matrix shaderMatrix;

    public ClippingImageView(Context context) {
        super(context);
        Paint paint = new Paint(2);
        this.paint = paint;
        paint.setFilterBitmap(true);
        this.matrix = new Matrix();
        this.drawRect = new RectF();
        this.bitmapRect = new RectF();
        this.roundPaint = new Paint(3);
        this.roundRect = new RectF();
        this.shaderMatrix = new Matrix();
    }

    public void setAnimationValues(float[][] values) {
        this.animationValues = values;
    }

    public float getAnimationProgress() {
        return this.animationProgress;
    }

    public void setAnimationProgress(float progress) {
        this.animationProgress = progress;
        try {
            float[][] fArr = this.animationValues;
            float f = fArr[0][0];
            setScaleX(f + ((fArr[1][0] - f) * progress));
            float[][] fArr2 = this.animationValues;
            float f2 = fArr2[0][1];
            setScaleY(f2 + ((fArr2[1][1] - f2) * this.animationProgress));
            float[][] fArr3 = this.animationValues;
            float f3 = fArr3[0][2];
            setTranslationX(f3 + ((fArr3[1][2] - f3) * this.animationProgress));
            float[][] fArr4 = this.animationValues;
            float f4 = fArr4[0][3];
            setTranslationY(f4 + ((fArr4[1][3] - f4) * this.animationProgress));
            float[][] fArr5 = this.animationValues;
            float f5 = fArr5[0][4];
            setClipHorizontal((int) (f5 + ((fArr5[1][4] - f5) * this.animationProgress)));
            float[][] fArr6 = this.animationValues;
            float f6 = fArr6[0][5];
            setClipTop((int) (f6 + ((fArr6[1][5] - f6) * this.animationProgress)));
            float[][] fArr7 = this.animationValues;
            float f7 = fArr7[0][6];
            setClipBottom((int) (f7 + ((fArr7[1][6] - f7) * this.animationProgress)));
            float[][] fArr8 = this.animationValues;
            float f8 = fArr8[0][7];
            setRadius((int) (f8 + ((fArr8[1][7] - f8) * this.animationProgress)));
            float[][] fArr9 = this.animationValues;
            float[] fArr10 = fArr9[0];
            if (fArr10.length > 8) {
                float f9 = fArr10[8];
                setImageY((int) (f9 + ((fArr9[1][8] - f9) * this.animationProgress)));
                float[][] fArr11 = this.animationValues;
                float f10 = fArr11[0][9];
                setImageX((int) (f10 + ((fArr11[1][9] - f10) * this.animationProgress)));
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        invalidate();
    }

    public void getClippedVisibleRect(RectF rect) {
        rect.left = getTranslationX();
        rect.top = getTranslationY();
        rect.right = rect.left + (getMeasuredWidth() * getScaleX());
        rect.bottom = rect.top + (getMeasuredHeight() * getScaleY());
        rect.left += this.clipLeft;
        rect.top += this.clipTop;
        rect.right -= this.clipRight;
        rect.bottom -= this.clipBottom;
    }

    public int getClipBottom() {
        return this.clipBottom;
    }

    public int getClipHorizontal() {
        return this.clipRight;
    }

    public int getClipLeft() {
        return this.clipLeft;
    }

    public int getClipRight() {
        return this.clipRight;
    }

    public int getClipTop() {
        return this.clipTop;
    }

    public int getRadius() {
        return this.radius;
    }

    @Override // android.view.View
    public void onDraw(Canvas canvas) {
        ImageReceiver.BitmapHolder bitmapHolder;
        if (getVisibility() == 0 && (bitmapHolder = this.bmp) != null && !bitmapHolder.isRecycled()) {
            float scaleY = getScaleY();
            canvas.save();
            if (this.needRadius) {
                this.shaderMatrix.reset();
                this.roundRect.set(this.imageX / scaleY, this.imageY / scaleY, getWidth() - (this.imageX / scaleY), getHeight() - (this.imageY / scaleY));
                this.bitmapRect.set(0.0f, 0.0f, this.bmp.getWidth(), this.bmp.getHeight());
                AndroidUtilities.setRectToRect(this.shaderMatrix, this.bitmapRect, this.roundRect, this.orientation, false);
                this.bitmapShader.setLocalMatrix(this.shaderMatrix);
                canvas.clipRect(this.clipLeft / scaleY, this.clipTop / scaleY, getWidth() - (this.clipRight / scaleY), getHeight() - (this.clipBottom / scaleY));
                RectF rectF = this.roundRect;
                int i = this.radius;
                canvas.drawRoundRect(rectF, i, i, this.roundPaint);
            } else {
                int i2 = this.orientation;
                if (i2 == 90 || i2 == 270) {
                    this.drawRect.set((-getHeight()) / 2, (-getWidth()) / 2, getHeight() / 2, getWidth() / 2);
                    this.matrix.setRectToRect(this.bitmapRect, this.drawRect, Matrix.ScaleToFit.FILL);
                    this.matrix.postRotate(this.orientation, 0.0f, 0.0f);
                    this.matrix.postTranslate(getWidth() / 2, getHeight() / 2);
                } else if (i2 == 180) {
                    this.drawRect.set((-getWidth()) / 2, (-getHeight()) / 2, getWidth() / 2, getHeight() / 2);
                    this.matrix.setRectToRect(this.bitmapRect, this.drawRect, Matrix.ScaleToFit.FILL);
                    this.matrix.postRotate(this.orientation, 0.0f, 0.0f);
                    this.matrix.postTranslate(getWidth() / 2, getHeight() / 2);
                } else {
                    this.drawRect.set(0.0f, 0.0f, getWidth(), getHeight());
                    this.matrix.setRectToRect(this.bitmapRect, this.drawRect, Matrix.ScaleToFit.FILL);
                }
                canvas.clipRect(this.clipLeft / scaleY, this.clipTop / scaleY, getWidth() - (this.clipRight / scaleY), getHeight() - (this.clipBottom / scaleY));
                try {
                    canvas.drawBitmap(this.bmp.bitmap, this.matrix, this.paint);
                } catch (Exception e) {
                    FileLog.e(e);
                }
            }
            canvas.restore();
        }
    }

    public void setClipBottom(int value) {
        this.clipBottom = value;
        invalidate();
    }

    public void setClipHorizontal(int value) {
        this.clipRight = value;
        this.clipLeft = value;
        invalidate();
    }

    public void setClipLeft(int value) {
        this.clipLeft = value;
        invalidate();
    }

    public void setClipRight(int value) {
        this.clipRight = value;
        invalidate();
    }

    public void setClipTop(int value) {
        this.clipTop = value;
        invalidate();
    }

    public void setClipVertical(int value) {
        this.clipBottom = value;
        this.clipTop = value;
        invalidate();
    }

    public void setImageY(int value) {
        this.imageY = value;
    }

    public void setImageX(int value) {
        this.imageX = value;
    }

    public void setOrientation(int angle) {
        this.orientation = angle;
    }

    public void setImageBitmap(ImageReceiver.BitmapHolder bitmap) {
        ImageReceiver.BitmapHolder bitmapHolder = this.bmp;
        if (bitmapHolder != null) {
            bitmapHolder.release();
            this.bitmapShader = null;
        }
        this.bmp = bitmap;
        if (bitmap != null && bitmap.bitmap != null) {
            this.bitmapRect.set(0.0f, 0.0f, bitmap.getWidth(), bitmap.getHeight());
            if (this.needRadius) {
                BitmapShader bitmapShader = new BitmapShader(this.bmp.bitmap, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP);
                this.bitmapShader = bitmapShader;
                this.roundPaint.setShader(bitmapShader);
            }
        }
        invalidate();
    }

    public Bitmap getBitmap() {
        ImageReceiver.BitmapHolder bitmapHolder = this.bmp;
        if (bitmapHolder != null) {
            return bitmapHolder.bitmap;
        }
        return null;
    }

    public int getOrientation() {
        return this.orientation;
    }

    public void setNeedRadius(boolean value) {
        this.needRadius = value;
    }

    public void setRadius(int value) {
        this.radius = value;
    }
}
