package im.uecahxyvjd.ui.components;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.FileLoader;
import im.uecahxyvjd.messenger.ImageLocation;
import im.uecahxyvjd.messenger.ImageReceiver;
import im.uecahxyvjd.messenger.MessageObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.Theme;
import java.util.ArrayList;
/* loaded from: classes10.dex */
public class GroupedPhotosListView extends View implements GestureDetector.OnGestureListener {
    private boolean animateAllLine;
    private int animateToDX;
    private int animateToDXStart;
    private int animateToItem;
    private Paint backgroundPaint;
    private long currentGroupId;
    private int currentImage;
    private float currentItemProgress;
    private ArrayList<Object> currentObjects;
    public ArrayList<ImageLocation> currentPhotos;
    private GroupedPhotosListViewDelegate delegate;
    private int drawDx;
    private GestureDetector gestureDetector;
    private boolean ignoreChanges;
    private ArrayList<ImageReceiver> imagesToDraw;
    private int itemHeight;
    private int itemSpacing;
    private int itemWidth;
    private int itemY;
    private long lastUpdateTime;
    private float moveLineProgress;
    private boolean moving;
    private int nextImage;
    private float nextItemProgress;
    private int nextPhotoScrolling;
    private android.widget.Scroller scroll;
    private boolean scrolling;
    private boolean stopedScrolling;
    private ArrayList<ImageReceiver> unusedReceivers;

    /* loaded from: classes10.dex */
    public interface GroupedPhotosListViewDelegate {
        int getAvatarsDialogId();

        int getCurrentAccount();

        int getCurrentIndex();

        ArrayList<MessageObject> getImagesArr();

        ArrayList<ImageLocation> getImagesArrLocations();

        ArrayList<TLRPC.PageBlock> getPageBlockArr();

        Object getParentObject();

        int getSlideshowMessageId();

        void setCurrentIndex(int i);
    }

    public GroupedPhotosListView(Context context) {
        super(context);
        this.backgroundPaint = new Paint();
        this.unusedReceivers = new ArrayList<>();
        this.imagesToDraw = new ArrayList<>();
        this.currentPhotos = new ArrayList<>();
        this.currentObjects = new ArrayList<>();
        this.currentItemProgress = 1.0f;
        this.nextItemProgress = 0.0f;
        this.animateToItem = -1;
        this.nextPhotoScrolling = -1;
        this.gestureDetector = new GestureDetector(context, this);
        this.scroll = new android.widget.Scroller(context);
        this.itemWidth = AndroidUtilities.dp(42.0f);
        this.itemHeight = AndroidUtilities.dp(56.0f);
        this.itemSpacing = AndroidUtilities.dp(1.0f);
        this.itemY = AndroidUtilities.dp(3.0f);
        this.backgroundPaint.setColor(Theme.ACTION_BAR_PHOTO_VIEWER_COLOR);
    }

    public void clear() {
        this.currentPhotos.clear();
        this.currentObjects.clear();
        this.imagesToDraw.clear();
    }

    /* JADX WARN: Multi-variable type inference failed */
    public void fillList() {
        ArrayList<ImageLocation> imagesArrLocations;
        boolean changed;
        int currentAccount;
        Object currentObject;
        int i;
        int min;
        ArrayList<MessageObject> imagesArr;
        int i2;
        if (this.ignoreChanges) {
            this.ignoreChanges = false;
            return;
        }
        int currentIndex = this.delegate.getCurrentIndex();
        ArrayList<ImageLocation> imagesArrLocations2 = this.delegate.getImagesArrLocations();
        ArrayList<MessageObject> imagesArr2 = this.delegate.getImagesArr();
        ArrayList<TLRPC.PageBlock> pageBlockArr = this.delegate.getPageBlockArr();
        int slideshowMessageId = this.delegate.getSlideshowMessageId();
        int currentAccount2 = this.delegate.getCurrentAccount();
        boolean changed2 = false;
        int newCount = 0;
        Object currentObject2 = null;
        if (imagesArrLocations2 != null && !imagesArrLocations2.isEmpty()) {
            Object location = (ImageLocation) imagesArrLocations2.get(currentIndex);
            newCount = imagesArrLocations2.size();
            currentObject = location;
            imagesArrLocations = imagesArrLocations2;
        } else if (imagesArr2 == null || imagesArr2.isEmpty()) {
            imagesArrLocations = imagesArrLocations2;
            if (pageBlockArr != null && !pageBlockArr.isEmpty()) {
                TLRPC.PageBlock pageBlock = pageBlockArr.get(currentIndex);
                currentObject2 = pageBlock;
                if (pageBlock.groupId != this.currentGroupId) {
                    changed2 = true;
                    this.currentGroupId = pageBlock.groupId;
                    currentObject = currentObject2;
                } else {
                    int size = pageBlockArr.size();
                    for (int a2 = currentIndex; a2 < size; a2++) {
                        TLRPC.PageBlock object = pageBlockArr.get(a2);
                        if (object.groupId != this.currentGroupId) {
                            break;
                        }
                        newCount++;
                    }
                    for (int a3 = currentIndex - 1; a3 >= 0; a3--) {
                        TLRPC.PageBlock object2 = pageBlockArr.get(a3);
                        if (object2.groupId != this.currentGroupId) {
                            break;
                        }
                        newCount++;
                    }
                }
            }
            changed2 = false;
            currentObject = currentObject2;
        } else {
            MessageObject messageObject = imagesArr2.get(currentIndex);
            Object currentObject3 = messageObject;
            if (messageObject.getGroupIdForUse() != this.currentGroupId) {
                changed2 = true;
                this.currentGroupId = messageObject.getGroupIdForUse();
                imagesArrLocations = imagesArrLocations2;
                currentObject = currentObject3;
            } else {
                int max = Math.min(currentIndex + 10, imagesArr2.size());
                int a4 = currentIndex;
                while (true) {
                    if (a4 >= max) {
                        imagesArrLocations = imagesArrLocations2;
                        break;
                    }
                    MessageObject object3 = imagesArr2.get(a4);
                    if (slideshowMessageId == 0) {
                        imagesArrLocations = imagesArrLocations2;
                        if (object3.getGroupIdForUse() != this.currentGroupId) {
                            break;
                        }
                    } else {
                        imagesArrLocations = imagesArrLocations2;
                    }
                    newCount++;
                    a4++;
                    imagesArrLocations2 = imagesArrLocations;
                }
                int min2 = Math.max(currentIndex - 10, 0);
                int a5 = currentIndex - 1;
                while (true) {
                    if (a5 < min2) {
                        changed = changed2;
                        break;
                    }
                    MessageObject object4 = imagesArr2.get(a5);
                    if (slideshowMessageId == 0) {
                        currentAccount = currentAccount2;
                        changed = changed2;
                        if (object4.getGroupIdForUse() != this.currentGroupId) {
                            break;
                        }
                    } else {
                        currentAccount = currentAccount2;
                        changed = changed2;
                    }
                    newCount++;
                    a5--;
                    currentAccount2 = currentAccount;
                    changed2 = changed;
                }
                changed2 = changed;
                currentObject = currentObject3;
            }
        }
        if (currentObject == null) {
            return;
        }
        if (!changed2) {
            if (newCount != this.currentPhotos.size() || this.currentObjects.indexOf(currentObject) == -1) {
                changed2 = true;
            } else {
                int newImageIndex = this.currentObjects.indexOf(currentObject);
                int i3 = this.currentImage;
                if (i3 != newImageIndex && newImageIndex != -1) {
                    if (!this.animateAllLine) {
                        fillImages(true, (i3 - newImageIndex) * (this.itemWidth + this.itemSpacing));
                        this.currentImage = newImageIndex;
                        i2 = 0;
                        this.moving = false;
                    } else {
                        this.animateToItem = newImageIndex;
                        this.nextImage = newImageIndex;
                        this.animateToDX = (i3 - newImageIndex) * (this.itemWidth + this.itemSpacing);
                        this.moving = true;
                        this.animateAllLine = false;
                        this.lastUpdateTime = System.currentTimeMillis();
                        invalidate();
                        i2 = 0;
                    }
                    this.drawDx = i2;
                }
            }
        }
        if (changed2) {
            this.animateAllLine = false;
            this.currentPhotos.clear();
            this.currentObjects.clear();
            if (imagesArrLocations != null && !imagesArrLocations.isEmpty()) {
                ArrayList<ImageLocation> imagesArrLocations3 = imagesArrLocations;
                this.currentObjects.addAll(imagesArrLocations3);
                this.currentPhotos.addAll(imagesArrLocations3);
                this.currentImage = currentIndex;
                this.animateToItem = -1;
            } else if (imagesArr2 != null && !imagesArr2.isEmpty()) {
                if (this.currentGroupId != 0 || slideshowMessageId != 0) {
                    int max2 = Math.min(currentIndex + 10, imagesArr2.size());
                    int a6 = currentIndex;
                    while (true) {
                        i = 56;
                        if (a6 >= max2) {
                            break;
                        }
                        MessageObject object5 = imagesArr2.get(a6);
                        if (slideshowMessageId == 0 && object5.getGroupIdForUse() != this.currentGroupId) {
                            break;
                        }
                        this.currentObjects.add(object5);
                        this.currentPhotos.add(ImageLocation.getForObject(FileLoader.getClosestPhotoSizeWithSize(object5.photoThumbs, 56, true), object5.photoThumbsObject));
                        a6++;
                    }
                    this.currentImage = 0;
                    this.animateToItem = -1;
                    int min3 = Math.max(currentIndex - 10, 0);
                    int a7 = currentIndex - 1;
                    while (a7 >= min3) {
                        MessageObject object6 = imagesArr2.get(a7);
                        if (slideshowMessageId == 0) {
                            min = min3;
                            imagesArr = imagesArr2;
                            if (object6.getGroupIdForUse() != this.currentGroupId) {
                                break;
                            }
                        } else {
                            min = min3;
                            imagesArr = imagesArr2;
                        }
                        this.currentObjects.add(0, object6);
                        this.currentPhotos.add(0, ImageLocation.getForObject(FileLoader.getClosestPhotoSizeWithSize(object6.photoThumbs, i, true), object6.photoThumbsObject));
                        this.currentImage++;
                        a7--;
                        imagesArr2 = imagesArr;
                        min3 = min;
                        i = 56;
                    }
                }
            } else if (pageBlockArr != null && !pageBlockArr.isEmpty()) {
                if (this.currentGroupId != 0) {
                    int a8 = currentIndex;
                    int size2 = pageBlockArr.size();
                    while (a8 < size2) {
                        TLRPC.PageBlock object7 = pageBlockArr.get(a8);
                        int slideshowMessageId2 = slideshowMessageId;
                        if (object7.groupId != this.currentGroupId) {
                            break;
                        }
                        this.currentObjects.add(object7);
                        this.currentPhotos.add(ImageLocation.getForObject(object7.thumb, object7.thumbObject));
                        a8++;
                        slideshowMessageId = slideshowMessageId2;
                    }
                    this.currentImage = 0;
                    this.animateToItem = -1;
                    for (int a9 = currentIndex - 1; a9 >= 0; a9--) {
                        TLRPC.PageBlock object8 = pageBlockArr.get(a9);
                        if (object8.groupId != this.currentGroupId) {
                            break;
                        }
                        this.currentObjects.add(0, object8);
                        this.currentPhotos.add(0, ImageLocation.getForObject(object8.thumb, object8.thumbObject));
                        this.currentImage++;
                    }
                }
            }
            if (this.currentPhotos.size() == 1) {
                this.currentPhotos.clear();
                this.currentObjects.clear();
            }
            fillImages(false, 0);
        }
    }

    public void setMoveProgress(float progress) {
        if (this.scrolling || this.animateToItem >= 0) {
            return;
        }
        if (progress > 0.0f) {
            this.nextImage = this.currentImage - 1;
        } else {
            this.nextImage = this.currentImage + 1;
        }
        int i = this.nextImage;
        if (i >= 0 && i < this.currentPhotos.size()) {
            this.currentItemProgress = 1.0f - Math.abs(progress);
        } else {
            this.currentItemProgress = 1.0f;
        }
        this.nextItemProgress = 1.0f - this.currentItemProgress;
        this.moving = progress != 0.0f;
        invalidate();
        if (!this.currentPhotos.isEmpty()) {
            if (progress >= 0.0f || this.currentImage != this.currentPhotos.size() - 1) {
                if (progress > 0.0f && this.currentImage == 0) {
                    return;
                }
                int i2 = (int) ((this.itemWidth + this.itemSpacing) * progress);
                this.drawDx = i2;
                fillImages(true, i2);
            }
        }
    }

    private ImageReceiver getFreeReceiver() {
        ImageReceiver receiver;
        if (this.unusedReceivers.isEmpty()) {
            receiver = new ImageReceiver(this);
        } else {
            receiver = this.unusedReceivers.get(0);
            this.unusedReceivers.remove(0);
        }
        this.imagesToDraw.add(receiver);
        receiver.setCurrentAccount(this.delegate.getCurrentAccount());
        return receiver;
    }

    private void fillImages(boolean move, int dx) {
        int addRightIndex;
        int addLeftIndex;
        Object obj;
        Object obj2;
        int i = 0;
        if (!move && !this.imagesToDraw.isEmpty()) {
            this.unusedReceivers.addAll(this.imagesToDraw);
            this.imagesToDraw.clear();
            this.moving = false;
            this.moveLineProgress = 1.0f;
            this.currentItemProgress = 1.0f;
            this.nextItemProgress = 0.0f;
        }
        invalidate();
        if (getMeasuredWidth() == 0 || this.currentPhotos.isEmpty()) {
            return;
        }
        int width = getMeasuredWidth();
        int startX = (getMeasuredWidth() / 2) - (this.itemWidth / 2);
        if (move) {
            addRightIndex = Integer.MIN_VALUE;
            addLeftIndex = Integer.MAX_VALUE;
            int count = this.imagesToDraw.size();
            int a2 = 0;
            while (a2 < count) {
                ImageReceiver receiver = this.imagesToDraw.get(a2);
                int num = receiver.getParam();
                int i2 = this.itemWidth;
                int x = ((num - this.currentImage) * (this.itemSpacing + i2)) + startX + dx;
                if (x > width || i2 + x < 0) {
                    this.unusedReceivers.add(receiver);
                    this.imagesToDraw.remove(a2);
                    count--;
                    a2--;
                }
                addLeftIndex = Math.min(addLeftIndex, num - 1);
                addRightIndex = Math.max(addRightIndex, num + 1);
                a2++;
            }
        } else {
            addRightIndex = this.currentImage;
            addLeftIndex = this.currentImage - 1;
        }
        if (addRightIndex != Integer.MIN_VALUE) {
            int count2 = this.currentPhotos.size();
            int a3 = addRightIndex;
            while (a3 < count2) {
                int x2 = ((a3 - this.currentImage) * (this.itemWidth + this.itemSpacing)) + startX + dx;
                if (x2 >= width) {
                    break;
                }
                ImageLocation location = this.currentPhotos.get(a3);
                ImageReceiver receiver2 = getFreeReceiver();
                receiver2.setImageCoords(x2, this.itemY, this.itemWidth, this.itemHeight);
                if (this.currentObjects.get(i) instanceof MessageObject) {
                    obj2 = this.currentObjects.get(a3);
                } else if (!(this.currentObjects.get(i) instanceof TLRPC.PageBlock)) {
                    obj2 = "avatar_" + this.delegate.getAvatarsDialogId();
                } else {
                    obj2 = this.delegate.getParentObject();
                }
                Object parent = obj2;
                receiver2.setImage(null, null, location, "80_80", 0, null, parent, 1);
                receiver2.setParam(a3);
                a3++;
                i = 0;
            }
        }
        if (addLeftIndex != Integer.MAX_VALUE) {
            for (int a4 = addLeftIndex; a4 >= 0; a4--) {
                int i3 = this.itemWidth;
                int x3 = ((a4 - this.currentImage) * (this.itemSpacing + i3)) + startX + dx + i3;
                if (x3 > 0) {
                    ImageLocation location2 = this.currentPhotos.get(a4);
                    ImageReceiver receiver3 = getFreeReceiver();
                    receiver3.setImageCoords(x3, this.itemY, this.itemWidth, this.itemHeight);
                    if (this.currentObjects.get(0) instanceof MessageObject) {
                        obj = this.currentObjects.get(a4);
                    } else if (!(this.currentObjects.get(0) instanceof TLRPC.PageBlock)) {
                        obj = "avatar_" + this.delegate.getAvatarsDialogId();
                    } else {
                        obj = this.delegate.getParentObject();
                    }
                    Object parent2 = obj;
                    receiver3.setImage(null, null, location2, "80_80", 0, null, parent2, 1);
                    receiver3.setParam(a4);
                } else {
                    return;
                }
            }
        }
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onDown(MotionEvent e) {
        if (!this.scroll.isFinished()) {
            this.scroll.abortAnimation();
        }
        this.animateToItem = -1;
        return true;
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public void onShowPress(MotionEvent e) {
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onSingleTapUp(MotionEvent e) {
        int currentIndex = this.delegate.getCurrentIndex();
        ArrayList<ImageLocation> imagesArrLocations = this.delegate.getImagesArrLocations();
        ArrayList<MessageObject> imagesArr = this.delegate.getImagesArr();
        ArrayList<TLRPC.PageBlock> pageBlockArr = this.delegate.getPageBlockArr();
        stopScrolling();
        int count = this.imagesToDraw.size();
        for (int a2 = 0; a2 < count; a2++) {
            ImageReceiver receiver = this.imagesToDraw.get(a2);
            if (receiver.isInsideImage(e.getX(), e.getY())) {
                int num = receiver.getParam();
                if (num < 0 || num >= this.currentObjects.size()) {
                    return true;
                }
                if (imagesArr != null && !imagesArr.isEmpty()) {
                    MessageObject messageObject = (MessageObject) this.currentObjects.get(num);
                    int idx = imagesArr.indexOf(messageObject);
                    if (currentIndex == idx) {
                        return true;
                    }
                    this.moveLineProgress = 1.0f;
                    this.animateAllLine = true;
                    this.delegate.setCurrentIndex(idx);
                    return false;
                } else if (pageBlockArr != null && !pageBlockArr.isEmpty()) {
                    TLRPC.PageBlock pageBlock = (TLRPC.PageBlock) this.currentObjects.get(num);
                    int idx2 = pageBlockArr.indexOf(pageBlock);
                    if (currentIndex == idx2) {
                        return true;
                    }
                    this.moveLineProgress = 1.0f;
                    this.animateAllLine = true;
                    this.delegate.setCurrentIndex(idx2);
                    return false;
                } else if (imagesArrLocations != null && !imagesArrLocations.isEmpty()) {
                    ImageLocation location = (ImageLocation) this.currentObjects.get(num);
                    int idx3 = imagesArrLocations.indexOf(location);
                    if (currentIndex == idx3) {
                        return true;
                    }
                    this.moveLineProgress = 1.0f;
                    this.animateAllLine = true;
                    this.delegate.setCurrentIndex(idx3);
                    return false;
                } else {
                    return false;
                }
            }
        }
        return false;
    }

    private void updateAfterScroll() {
        int dx;
        int indexChange;
        int indexChange2 = 0;
        int dx2 = this.drawDx;
        int abs = Math.abs(dx2);
        int i = this.itemWidth;
        int i2 = this.itemSpacing;
        if (abs > (i / 2) + i2) {
            if (dx2 > 0) {
                dx = dx2 - ((i / 2) + i2);
                indexChange = 0 + 1;
            } else {
                dx = dx2 + (i / 2) + i2;
                indexChange = 0 - 1;
            }
            indexChange2 = indexChange + (dx / (i + (i2 * 2)));
        }
        this.nextPhotoScrolling = this.currentImage - indexChange2;
        int currentIndex = this.delegate.getCurrentIndex();
        ArrayList<ImageLocation> imagesArrLocations = this.delegate.getImagesArrLocations();
        ArrayList<MessageObject> imagesArr = this.delegate.getImagesArr();
        ArrayList<TLRPC.PageBlock> pageBlockArr = this.delegate.getPageBlockArr();
        int i3 = this.nextPhotoScrolling;
        if (currentIndex != i3 && i3 >= 0 && i3 < this.currentPhotos.size()) {
            Object photo = this.currentObjects.get(this.nextPhotoScrolling);
            int nextPhoto = -1;
            if (imagesArr != null && !imagesArr.isEmpty()) {
                MessageObject messageObject = (MessageObject) photo;
                nextPhoto = imagesArr.indexOf(messageObject);
            } else if (pageBlockArr != null && !pageBlockArr.isEmpty()) {
                TLRPC.PageBlock pageBlock = (TLRPC.PageBlock) photo;
                nextPhoto = pageBlockArr.indexOf(pageBlock);
            } else if (imagesArrLocations != null && !imagesArrLocations.isEmpty()) {
                ImageLocation location = (ImageLocation) photo;
                nextPhoto = imagesArrLocations.indexOf(location);
            }
            if (nextPhoto >= 0) {
                this.ignoreChanges = true;
                this.delegate.setCurrentIndex(nextPhoto);
            }
        }
        if (!this.scrolling) {
            this.scrolling = true;
            this.stopedScrolling = false;
        }
        fillImages(true, this.drawDx);
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
        this.drawDx = (int) (this.drawDx - distanceX);
        int min = getMinScrollX();
        int max = getMaxScrollX();
        int i = this.drawDx;
        if (i < min) {
            this.drawDx = min;
        } else if (i > max) {
            this.drawDx = max;
        }
        updateAfterScroll();
        return false;
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public void onLongPress(MotionEvent e) {
    }

    @Override // android.view.GestureDetector.OnGestureListener
    public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
        this.scroll.abortAnimation();
        if (this.currentPhotos.size() >= 10) {
            this.scroll.fling(this.drawDx, 0, Math.round(velocityX), 0, getMinScrollX(), getMaxScrollX(), 0, 0);
            return false;
        }
        return false;
    }

    private void stopScrolling() {
        this.scrolling = false;
        if (!this.scroll.isFinished()) {
            this.scroll.abortAnimation();
        }
        int i = this.nextPhotoScrolling;
        if (i >= 0 && i < this.currentObjects.size()) {
            this.stopedScrolling = true;
            int i2 = this.nextPhotoScrolling;
            this.animateToItem = i2;
            this.nextImage = i2;
            this.animateToDX = (this.currentImage - i2) * (this.itemWidth + this.itemSpacing);
            this.animateToDXStart = this.drawDx;
            this.moveLineProgress = 1.0f;
            this.nextPhotoScrolling = -1;
        }
        invalidate();
    }

    @Override // android.view.View
    public boolean onTouchEvent(MotionEvent event) {
        boolean z = false;
        if (this.currentPhotos.isEmpty() || getAlpha() != 1.0f) {
            return false;
        }
        boolean result = (this.gestureDetector.onTouchEvent(event) || super.onTouchEvent(event)) ? true : true;
        if (this.scrolling && event.getAction() == 1 && this.scroll.isFinished()) {
            stopScrolling();
        }
        return result;
    }

    private int getMinScrollX() {
        return (-((this.currentPhotos.size() - this.currentImage) - 1)) * (this.itemWidth + (this.itemSpacing * 2));
    }

    private int getMaxScrollX() {
        return this.currentImage * (this.itemWidth + (this.itemSpacing * 2));
    }

    @Override // android.view.View
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        fillImages(false, 0);
    }

    @Override // android.view.View
    protected void onDraw(Canvas canvas) {
        int trueWidth;
        int i;
        int nextTrueWidth;
        int i2;
        float interpolation;
        int i3;
        int i4;
        int count;
        int maxItemWidth;
        if (this.imagesToDraw.isEmpty()) {
            return;
        }
        canvas.drawRect(0.0f, 0.0f, getMeasuredWidth(), getMeasuredHeight(), this.backgroundPaint);
        int count2 = this.imagesToDraw.size();
        int moveX = this.drawDx;
        int maxItemWidth2 = (int) (this.itemWidth * 2.0f);
        int padding = AndroidUtilities.dp(8.0f);
        ImageLocation object = this.currentPhotos.get(this.currentImage);
        if (object != null && object.photoSize != null) {
            trueWidth = Math.max(this.itemWidth, (int) (object.photoSize.w * (this.itemHeight / object.photoSize.h)));
        } else {
            trueWidth = this.itemHeight;
        }
        int trueWidth2 = Math.min(maxItemWidth2, trueWidth);
        float f = this.currentItemProgress;
        int currentPaddings = (int) (padding * 2 * f);
        int trueWidth3 = this.itemWidth + ((int) ((trueWidth2 - i) * f)) + currentPaddings;
        int trueWidth4 = this.nextImage;
        if (trueWidth4 >= 0 && trueWidth4 < this.currentPhotos.size()) {
            ImageLocation object2 = this.currentPhotos.get(this.nextImage);
            if (object2 != null && object2.photoSize != null) {
                nextTrueWidth = Math.max(this.itemWidth, (int) (object2.photoSize.w * (this.itemHeight / object2.photoSize.h)));
            } else {
                nextTrueWidth = this.itemHeight;
            }
        } else {
            nextTrueWidth = this.itemWidth;
        }
        int nextTrueWidth2 = Math.min(maxItemWidth2, nextTrueWidth);
        float f2 = this.nextItemProgress;
        int nextPaddings = (int) (padding * 2 * f2);
        float f3 = moveX;
        int moveX2 = (int) (f3 + ((((nextTrueWidth2 + nextPaddings) - i2) / 2) * f2 * (this.nextImage > this.currentImage ? -1 : 1)));
        int nextTrueWidth3 = this.itemWidth + ((int) ((nextTrueWidth2 - i2) * f2)) + nextPaddings;
        int nextTrueWidth4 = getMeasuredWidth();
        int startX = (nextTrueWidth4 - trueWidth3) / 2;
        int a2 = 0;
        while (a2 < count2) {
            ImageReceiver receiver = this.imagesToDraw.get(a2);
            int num = receiver.getParam();
            int i5 = this.currentImage;
            if (num == i5) {
                receiver.setImageX(startX + moveX2 + (currentPaddings / 2));
                receiver.setImageWidth(trueWidth3 - currentPaddings);
                count = count2;
                maxItemWidth = maxItemWidth2;
            } else {
                int i6 = this.nextImage;
                if (i6 < i5) {
                    if (num < i5) {
                        if (num <= i6) {
                            int i7 = this.itemWidth;
                            count = count2;
                            int count3 = this.itemSpacing;
                            receiver.setImageX((((((receiver.getParam() - this.currentImage) + 1) * (i7 + count3)) + startX) - (count3 + nextTrueWidth3)) + moveX2);
                            maxItemWidth = maxItemWidth2;
                        } else {
                            count = count2;
                            int count4 = receiver.getParam();
                            receiver.setImageX(((count4 - this.currentImage) * (this.itemWidth + this.itemSpacing)) + startX + moveX2);
                            maxItemWidth = maxItemWidth2;
                        }
                    } else {
                        count = count2;
                        int count5 = startX + trueWidth3;
                        maxItemWidth = maxItemWidth2;
                        receiver.setImageX(count5 + this.itemSpacing + (((receiver.getParam() - this.currentImage) - 1) * (this.itemWidth + this.itemSpacing)) + moveX2);
                    }
                } else {
                    count = count2;
                    maxItemWidth = maxItemWidth2;
                    if (num < i5) {
                        receiver.setImageX(((receiver.getParam() - this.currentImage) * (this.itemWidth + this.itemSpacing)) + startX + moveX2);
                    } else if (num <= i6) {
                        receiver.setImageX(startX + trueWidth3 + this.itemSpacing + (((receiver.getParam() - this.currentImage) - 1) * (this.itemWidth + this.itemSpacing)) + moveX2);
                    } else {
                        int i8 = this.itemWidth;
                        int i9 = this.itemSpacing;
                        receiver.setImageX(startX + trueWidth3 + this.itemSpacing + (((receiver.getParam() - this.currentImage) - 2) * (i8 + i9)) + i9 + nextTrueWidth3 + moveX2);
                    }
                }
                if (num == this.nextImage) {
                    receiver.setImageWidth(nextTrueWidth3 - nextPaddings);
                    receiver.setImageX(receiver.getImageX() + (nextPaddings / 2));
                } else {
                    receiver.setImageWidth(this.itemWidth);
                }
            }
            receiver.draw(canvas);
            a2++;
            count2 = count;
            maxItemWidth2 = maxItemWidth;
        }
        long newTime = System.currentTimeMillis();
        long dt = newTime - this.lastUpdateTime;
        if (dt > 17) {
            dt = 17;
        }
        this.lastUpdateTime = newTime;
        int i10 = this.animateToItem;
        if (i10 >= 0) {
            float f4 = this.moveLineProgress;
            if (f4 > 0.0f) {
                this.moveLineProgress = f4 - (((float) dt) / 200.0f);
                if (i10 != this.currentImage) {
                    this.nextItemProgress = CubicBezierInterpolator.EASE_OUT.getInterpolation(1.0f - this.moveLineProgress);
                    if (!this.stopedScrolling) {
                        this.currentItemProgress = CubicBezierInterpolator.EASE_OUT.getInterpolation(this.moveLineProgress);
                        this.drawDx = (int) Math.ceil(this.nextItemProgress * this.animateToDX);
                    } else {
                        float f5 = this.currentItemProgress;
                        if (f5 > 0.0f) {
                            float f6 = f5 - (((float) dt) / 200.0f);
                            this.currentItemProgress = f6;
                            if (f6 < 0.0f) {
                                this.currentItemProgress = 0.0f;
                            }
                        }
                        this.drawDx = this.animateToDXStart + ((int) Math.ceil(interpolation * (this.animateToDX - i3)));
                    }
                } else {
                    float f7 = this.currentItemProgress;
                    if (f7 < 1.0f) {
                        float f8 = f7 + (((float) dt) / 200.0f);
                        this.currentItemProgress = f8;
                        if (f8 > 1.0f) {
                            this.currentItemProgress = 1.0f;
                        }
                    }
                    this.drawDx = this.animateToDXStart + ((int) Math.ceil(this.currentItemProgress * (this.animateToDX - i4)));
                }
                if (this.moveLineProgress <= 0.0f) {
                    this.currentImage = this.animateToItem;
                    this.moveLineProgress = 1.0f;
                    this.currentItemProgress = 1.0f;
                    this.nextItemProgress = 0.0f;
                    this.moving = false;
                    this.stopedScrolling = false;
                    this.drawDx = 0;
                    this.animateToItem = -1;
                }
            }
            fillImages(true, this.drawDx);
            invalidate();
        }
        if (this.scrolling) {
            float f9 = this.currentItemProgress;
            if (f9 > 0.0f) {
                float f10 = f9 - (((float) dt) / 200.0f);
                this.currentItemProgress = f10;
                if (f10 < 0.0f) {
                    this.currentItemProgress = 0.0f;
                }
                invalidate();
            }
        }
        if (!this.scroll.isFinished()) {
            if (this.scroll.computeScrollOffset()) {
                this.drawDx = this.scroll.getCurrX();
                updateAfterScroll();
                invalidate();
            }
            if (this.scroll.isFinished()) {
                stopScrolling();
            }
        }
    }

    public void setDelegate(GroupedPhotosListViewDelegate groupedPhotosListViewDelegate) {
        this.delegate = groupedPhotosListViewDelegate;
    }
}
