package im.uecahxyvjd.ui.components;

import android.graphics.Canvas;
import android.graphics.ColorFilter;
import android.graphics.CornerPathEffect;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;
import android.text.TextPaint;
import android.view.animation.DecelerateInterpolator;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.ui.actionbar.Theme;
/* loaded from: classes10.dex */
public class MediaActionDrawable extends Drawable {
    private static final float CANCEL_TO_CHECK_STAGE1 = 0.5f;
    private static final float CANCEL_TO_CHECK_STAGE2 = 0.5f;
    private static final float DOWNLOAD_TO_CANCEL_STAGE1 = 0.5f;
    private static final float DOWNLOAD_TO_CANCEL_STAGE2 = 0.2f;
    private static final float DOWNLOAD_TO_CANCEL_STAGE3 = 0.3f;
    private static final float EPS = 0.001f;
    public static final int ICON_CANCEL = 3;
    public static final int ICON_CANCEL_FILL = 14;
    public static final int ICON_CANCEL_NOPROFRESS = 12;
    public static final int ICON_CANCEL_PERCENT = 13;
    public static final int ICON_CHECK = 6;
    public static final int ICON_DOWNLOAD = 2;
    public static final int ICON_EMPTY = 10;
    public static final int ICON_EMPTY_NOPROGRESS = 11;
    public static final int ICON_FILE = 5;
    public static final int ICON_FILE_APK = 20;
    public static final int ICON_FILE_COMPRESS = 15;
    public static final int ICON_FILE_DOC = 16;
    public static final int ICON_FILE_IPA = 21;
    public static final int ICON_FILE_PDF = 19;
    public static final int ICON_FILE_TXT = 18;
    public static final int ICON_FILE_XLS = 17;
    public static final int ICON_FIRE = 7;
    public static final int ICON_GIF = 8;
    public static final int ICON_NONE = 4;
    public static final int ICON_PAUSE = 1;
    public static final int ICON_PLAY = 0;
    public static final int ICON_SECRETCHECK = 9;
    private static final int pauseRotation = 90;
    private static final int playRotation = 0;
    private float animatedDownloadProgress;
    private boolean animatingTransition;
    private ColorFilter colorFilter;
    private int currentIcon;
    private MediaActionDrawableDelegate delegate;
    private float downloadProgress;
    private float downloadProgressAnimationStart;
    private float downloadProgressTime;
    private float downloadRadOffset;
    private boolean isMini;
    private long lastAnimationTime;
    private int nextIcon;
    private String percentString;
    private int percentStringWidth;
    private float savedTransitionProgress;
    private static final float[] playPath1 = {18.0f, 15.0f, 34.0f, 24.0f, 34.0f, 24.0f, 18.0f, 24.0f, 18.0f, 24.0f};
    private static final float[] playPath2 = {18.0f, 33.0f, 34.0f, 24.0f, 34.0f, 24.0f, 18.0f, 24.0f, 18.0f, 24.0f};
    private static final float[] playFinalPath = {18.0f, 15.0f, 34.0f, 24.0f, 18.0f, 33.0f};
    private static final float[] pausePath1 = {16.0f, 17.0f, 32.0f, 17.0f, 32.0f, 22.0f, 16.0f, 22.0f, 16.0f, 19.5f};
    private static final float[] pausePath2 = {16.0f, 31.0f, 32.0f, 31.0f, 32.0f, 26.0f, 16.0f, 26.0f, 16.0f, 28.5f};
    private TextPaint textPaint = new TextPaint(1);
    private Paint paint = new Paint(1);
    private Paint paint2 = new Paint(1);
    private Paint paint3 = new Paint(1);
    private Path path1 = new Path();
    private Path path2 = new Path();
    private RectF rect = new RectF();
    private float scale = 1.0f;
    private DecelerateInterpolator interpolator = new DecelerateInterpolator();
    private float transitionAnimationTime = 400.0f;
    private int lastPercent = -1;
    private float overrideAlpha = 1.0f;
    private float transitionProgress = 1.0f;

    /* loaded from: classes10.dex */
    public interface MediaActionDrawableDelegate {
        void invalidate();
    }

    public MediaActionDrawable() {
        this.paint.setColor(-1);
        this.paint.setStrokeCap(Paint.Cap.ROUND);
        this.paint.setStrokeWidth(AndroidUtilities.dp(3.0f));
        this.paint.setStyle(Paint.Style.STROKE);
        this.paint3.setColor(-1);
        this.textPaint.setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
        this.textPaint.setTextSize(AndroidUtilities.dp(13.0f));
        this.textPaint.setColor(-1);
        this.paint2.setColor(-1);
        this.paint2.setPathEffect(new CornerPathEffect(AndroidUtilities.dp(2.0f)));
    }

    @Override // android.graphics.drawable.Drawable
    public void setAlpha(int alpha) {
    }

    public void setOverrideAlpha(float alpha) {
        this.overrideAlpha = alpha;
    }

    @Override // android.graphics.drawable.Drawable
    public void setColorFilter(ColorFilter colorFilter) {
        this.paint.setColorFilter(colorFilter);
        this.paint2.setColorFilter(colorFilter);
        this.paint3.setColorFilter(colorFilter);
        this.textPaint.setColorFilter(colorFilter);
    }

    public void setColor(int value) {
        this.paint.setColor(value | (-16777216));
        this.paint2.setColor(value | (-16777216));
        this.paint3.setColor(value | (-16777216));
        this.textPaint.setColor((-16777216) | value);
        this.colorFilter = new PorterDuffColorFilter(value, PorterDuff.Mode.MULTIPLY);
    }

    public int getColor() {
        return this.paint.getColor();
    }

    public void setMini(boolean value) {
        this.isMini = value;
        this.paint.setStrokeWidth(AndroidUtilities.dp(value ? 2.0f : 3.0f));
    }

    @Override // android.graphics.drawable.Drawable
    public int getOpacity() {
        return -2;
    }

    public void setDelegate(MediaActionDrawableDelegate mediaActionDrawableDelegate) {
        this.delegate = mediaActionDrawableDelegate;
    }

    public boolean setIcon(int icon, boolean animated) {
        int i;
        int i2;
        if (this.currentIcon == icon && (i2 = this.nextIcon) != icon) {
            this.currentIcon = i2;
            this.transitionProgress = 1.0f;
        }
        if (animated) {
            int i3 = this.currentIcon;
            if (i3 == icon || (i = this.nextIcon) == icon) {
                return false;
            }
            if (i3 == 2 && (icon == 3 || icon == 14)) {
                this.transitionAnimationTime = 400.0f;
            } else if (i3 != 4 && icon == 6) {
                this.transitionAnimationTime = 360.0f;
            } else if ((i3 == 4 && icon == 14) || (i3 == 14 && icon == 4)) {
                this.transitionAnimationTime = 160.0f;
            } else {
                this.transitionAnimationTime = 220.0f;
            }
            if (this.animatingTransition) {
                this.currentIcon = i;
            }
            this.animatingTransition = true;
            this.nextIcon = icon;
            this.savedTransitionProgress = this.transitionProgress;
            this.transitionProgress = 0.0f;
        } else if (this.currentIcon == icon) {
            return false;
        } else {
            this.animatingTransition = false;
            this.nextIcon = icon;
            this.currentIcon = icon;
            this.savedTransitionProgress = this.transitionProgress;
            this.transitionProgress = 1.0f;
        }
        if (icon == 3 || icon == 14) {
            this.downloadRadOffset = 112.0f;
            this.animatedDownloadProgress = 0.0f;
            this.downloadProgressAnimationStart = 0.0f;
            this.downloadProgressTime = 0.0f;
        }
        invalidateSelf();
        return true;
    }

    public int getCurrentIcon() {
        return this.nextIcon;
    }

    public int getPreviousIcon() {
        return this.currentIcon;
    }

    public void setProgress(float value, boolean animated) {
        if (!animated) {
            this.animatedDownloadProgress = value;
            this.downloadProgressAnimationStart = value;
        } else {
            if (this.animatedDownloadProgress > value) {
                this.animatedDownloadProgress = value;
            }
            this.downloadProgressAnimationStart = this.animatedDownloadProgress;
        }
        this.downloadProgress = value;
        this.downloadProgressTime = 0.0f;
        invalidateSelf();
    }

    private float getCircleValue(float value) {
        while (value > 360.0f) {
            value -= 360.0f;
        }
        return value;
    }

    public float getProgressAlpha() {
        return 1.0f - this.transitionProgress;
    }

    public float getTransitionProgress() {
        if (this.animatingTransition) {
            return this.transitionProgress;
        }
        return 1.0f;
    }

    @Override // android.graphics.drawable.Drawable
    public void setBounds(int left, int top, int right, int bottom) {
        super.setBounds(left, top, right, bottom);
        float intrinsicWidth = (right - left) / getIntrinsicWidth();
        this.scale = intrinsicWidth;
        if (intrinsicWidth < 0.7f) {
            this.paint.setStrokeWidth(AndroidUtilities.dp(2.0f));
        }
    }

    @Override // android.graphics.drawable.Drawable
    public void invalidateSelf() {
        super.invalidateSelf();
        MediaActionDrawableDelegate mediaActionDrawableDelegate = this.delegate;
        if (mediaActionDrawableDelegate != null) {
            mediaActionDrawableDelegate.invalidate();
        }
    }

    /* JADX WARN: Code restructure failed: missing block: B:356:0x08aa, code lost:
        if (r40.nextIcon != 1) goto L278;
     */
    /* JADX WARN: Removed duplicated region for block: B:220:0x0626  */
    /* JADX WARN: Removed duplicated region for block: B:221:0x062d  */
    /* JADX WARN: Removed duplicated region for block: B:227:0x0655  */
    /* JADX WARN: Removed duplicated region for block: B:228:0x0658  */
    /* JADX WARN: Removed duplicated region for block: B:233:0x0663  */
    /* JADX WARN: Removed duplicated region for block: B:234:0x0666  */
    /* JADX WARN: Removed duplicated region for block: B:239:0x0673  */
    /* JADX WARN: Removed duplicated region for block: B:240:0x0676  */
    /* JADX WARN: Removed duplicated region for block: B:245:0x0684  */
    /* JADX WARN: Removed duplicated region for block: B:246:0x0687  */
    /* JADX WARN: Removed duplicated region for block: B:251:0x0693  */
    /* JADX WARN: Removed duplicated region for block: B:252:0x0696  */
    /* JADX WARN: Removed duplicated region for block: B:257:0x06a4  */
    /* JADX WARN: Removed duplicated region for block: B:258:0x06a7  */
    /* JADX WARN: Removed duplicated region for block: B:263:0x06b5  */
    /* JADX WARN: Removed duplicated region for block: B:264:0x06b8  */
    /* JADX WARN: Removed duplicated region for block: B:269:0x06c6  */
    /* JADX WARN: Removed duplicated region for block: B:270:0x06c9  */
    /* JADX WARN: Removed duplicated region for block: B:275:0x06d7  */
    /* JADX WARN: Removed duplicated region for block: B:276:0x06da  */
    /* JADX WARN: Removed duplicated region for block: B:281:0x06e8  */
    /* JADX WARN: Removed duplicated region for block: B:282:0x06ed  */
    /* JADX WARN: Removed duplicated region for block: B:294:0x0715  */
    /* JADX WARN: Removed duplicated region for block: B:295:0x0718  */
    /* JADX WARN: Removed duplicated region for block: B:298:0x0734  */
    /* JADX WARN: Removed duplicated region for block: B:301:0x077f  */
    /* JADX WARN: Removed duplicated region for block: B:308:0x0792  */
    /* JADX WARN: Removed duplicated region for block: B:309:0x0796  */
    /* JADX WARN: Removed duplicated region for block: B:315:0x07a9  */
    /* JADX WARN: Removed duplicated region for block: B:316:0x07ac  */
    /* JADX WARN: Removed duplicated region for block: B:319:0x07c6  */
    /* JADX WARN: Removed duplicated region for block: B:322:0x0803  */
    /* JADX WARN: Removed duplicated region for block: B:329:0x0814  */
    /* JADX WARN: Removed duplicated region for block: B:330:0x0817  */
    /* JADX WARN: Removed duplicated region for block: B:336:0x083f  */
    /* JADX WARN: Removed duplicated region for block: B:344:0x0887  */
    /* JADX WARN: Removed duplicated region for block: B:355:0x08a7  */
    /* JADX WARN: Removed duplicated region for block: B:358:0x08ad  */
    /* JADX WARN: Removed duplicated region for block: B:370:0x08d5  */
    /* JADX WARN: Removed duplicated region for block: B:371:0x08d9  */
    /* JADX WARN: Removed duplicated region for block: B:372:0x08e0  */
    /* JADX WARN: Removed duplicated region for block: B:375:0x08ed  */
    /* JADX WARN: Removed duplicated region for block: B:376:0x08f1  */
    /* JADX WARN: Removed duplicated region for block: B:377:0x08f8  */
    /* JADX WARN: Removed duplicated region for block: B:379:0x0900  */
    /* JADX WARN: Removed duplicated region for block: B:382:0x090a A[ADDED_TO_REGION] */
    /* JADX WARN: Removed duplicated region for block: B:393:0x09cb  */
    /* JADX WARN: Removed duplicated region for block: B:408:0x0a86  */
    /* JADX WARN: Removed duplicated region for block: B:418:0x0bc6  */
    /* JADX WARN: Removed duplicated region for block: B:430:0x0bfa  */
    /* JADX WARN: Removed duplicated region for block: B:438:0x0c22  */
    /* JADX WARN: Removed duplicated region for block: B:441:0x0c40  */
    /* JADX WARN: Removed duplicated region for block: B:444:0x0c6e  */
    /* JADX WARN: Removed duplicated region for block: B:446:0x0c8c  */
    /* JADX WARN: Removed duplicated region for block: B:462:0x0ce0  */
    /* JADX WARN: Removed duplicated region for block: B:464:0x0ce6  */
    /* JADX WARN: Removed duplicated region for block: B:480:0x0d3f  */
    /* JADX WARN: Removed duplicated region for block: B:495:0x0d74  */
    /* JADX WARN: Removed duplicated region for block: B:504:0x0da8  */
    @Override // android.graphics.drawable.Drawable
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public void draw(Canvas canvas) {
        float transition;
        float yStart2;
        float y1;
        float y2;
        float x1;
        float x2;
        float y3;
        float currentProgress;
        float currentBackProgress;
        float currentProgress2;
        float currentProgress22;
        float rotation;
        int alpha;
        float rotation2;
        float transition2;
        int i;
        float previowsDrawableScale;
        float drawableScale;
        Drawable nextDrawable;
        Drawable previousDrawable;
        int i2;
        float previowsDrawableScale2;
        int i3;
        int i4;
        float transition3;
        int i5;
        int i6;
        float transition4;
        int newPercent;
        String format;
        int i7;
        int i8;
        float p;
        float[] p1;
        float[] p2;
        int rotation1;
        float[] p3;
        float[] p4;
        int rotation22;
        Drawable nextDrawable2;
        int cx;
        int cy;
        float drawableScale2;
        Drawable previousDrawable2;
        int cy2;
        int i9;
        float drawableScale3;
        int cx2;
        int cy3;
        float drawableScale4;
        int i10;
        float progress2;
        float progress1;
        Drawable nextDrawable3;
        long dt;
        int i11;
        int i12;
        int i13;
        int i14;
        float progress;
        float backProgress;
        float f;
        int alpha2;
        float iconScale;
        float iconScaleX;
        float d;
        float rotation3;
        float iconScaleY;
        float iconScale2;
        float rotation4;
        float iconScaleX2;
        float iconScaleY2;
        float rotation5;
        float iconScale3;
        float rotation6;
        float iconScaleX3;
        int i15;
        int i16;
        float d2;
        int alpha3;
        int alpha4;
        int i17;
        android.graphics.Rect bounds = getBounds();
        int cx3 = bounds.centerX();
        int cy4 = bounds.centerY();
        int i18 = this.nextIcon;
        if (i18 != 4) {
            if ((i18 == 6 || i18 == 10) && this.currentIcon == 4) {
                canvas.save();
                float f2 = this.transitionProgress;
                canvas.scale(f2, f2, cx3, cy4);
            }
        } else {
            float progress3 = 1.0f - this.transitionProgress;
            canvas.save();
            canvas.scale(progress3, progress3, cx3, cy4);
        }
        AndroidUtilities.dp(3.0f);
        if (this.currentIcon == 2 || this.nextIcon == 2) {
            float yStart = cy4 - (AndroidUtilities.dp(9.0f) * this.scale);
            float yEnd = cy4 + (AndroidUtilities.dp(9.0f) * this.scale);
            float yEnd2 = cy4 + (AndroidUtilities.dp(12.0f) * this.scale);
            int i19 = this.currentIcon;
            if ((i19 == 3 || i19 == 14) && this.nextIcon == 2) {
                this.paint.setAlpha((int) (Math.min(1.0f, this.transitionProgress / 0.5f) * 255.0f));
                float transition5 = this.transitionProgress;
                transition = transition5;
                yStart2 = cy4 + (AndroidUtilities.dp(12.0f) * this.scale);
            } else {
                int i20 = this.nextIcon;
                if (i20 != 3 && i20 != 14 && i20 != 2) {
                    this.paint.setAlpha((int) (Math.min(1.0f, this.savedTransitionProgress / 0.5f) * 255.0f * (1.0f - this.transitionProgress)));
                    transition2 = this.savedTransitionProgress;
                } else {
                    this.paint.setAlpha(255);
                    transition2 = this.transitionProgress;
                }
                transition = transition2;
                yStart2 = cy4 + (AndroidUtilities.dp(1.0f) * this.scale);
            }
            if (this.animatingTransition) {
                float progress4 = transition;
                int i21 = this.nextIcon;
                if (i21 != 2 && progress4 > 0.5f) {
                    float d3 = AndroidUtilities.dp(13.0f) * this.scale;
                    float progress5 = progress4 - 0.5f;
                    float currentProgress3 = progress5 / 0.5f;
                    if (progress5 > DOWNLOAD_TO_CANCEL_STAGE2) {
                        currentProgress2 = 1.0f;
                        currentProgress22 = (progress5 - DOWNLOAD_TO_CANCEL_STAGE2) / DOWNLOAD_TO_CANCEL_STAGE3;
                    } else {
                        currentProgress2 = progress5 / DOWNLOAD_TO_CANCEL_STAGE2;
                        currentProgress22 = 0.0f;
                    }
                    this.rect.set(cx3 - d3, yEnd2 - (d3 / 2.0f), cx3, yEnd2 + (d3 / 2.0f));
                    float start = currentProgress22 * 100.0f;
                    canvas.drawArc(this.rect, start, (104.0f * currentProgress3) - start, false, this.paint);
                    y1 = yStart2 + ((yEnd2 - yStart2) * currentProgress2);
                    y3 = yEnd2;
                    y2 = yEnd2;
                    float f3 = cx3;
                    x2 = f3;
                    x1 = f3;
                    if (currentProgress22 > 0.0f) {
                        if (this.nextIcon == 14) {
                            rotation = 0.0f;
                        } else {
                            rotation = (-45.0f) * (1.0f - currentProgress22);
                        }
                        float d4 = AndroidUtilities.dp(7.0f) * currentProgress22 * this.scale;
                        int alpha5 = (int) (currentProgress22 * 255.0f);
                        int i22 = this.nextIcon;
                        if (i22 != 3 && i22 != 14 && i22 != 2) {
                            float backProgress2 = 1.0f - Math.min(1.0f, this.transitionProgress / 0.5f);
                            alpha = (int) (alpha5 * backProgress2);
                        } else {
                            alpha = alpha5;
                        }
                        if (rotation != 0.0f) {
                            canvas.save();
                            canvas.rotate(rotation, cx3, cy4);
                        }
                        if (alpha != 0) {
                            this.paint.setAlpha(alpha);
                            if (this.nextIcon != 14) {
                                rotation2 = rotation;
                                canvas.drawLine(cx3 - d4, cy4 - d4, cx3 + d4, cy4 + d4, this.paint);
                                canvas.drawLine(cx3 + d4, cy4 - d4, cx3 - d4, cy4 + d4, this.paint);
                            } else {
                                this.paint3.setAlpha(alpha);
                                this.rect.set(cx3 - AndroidUtilities.dp(3.5f), cy4 - AndroidUtilities.dp(3.5f), cx3 + AndroidUtilities.dp(3.5f), cy4 + AndroidUtilities.dp(3.5f));
                                canvas.drawRoundRect(this.rect, AndroidUtilities.dp(2.0f), AndroidUtilities.dp(2.0f), this.paint3);
                                this.paint.setAlpha((int) (alpha * 0.15f));
                                int diff = AndroidUtilities.dp(this.isMini ? 2.0f : 4.0f);
                                this.rect.set(bounds.left + diff, bounds.top + diff, bounds.right - diff, bounds.bottom - diff);
                                rotation2 = rotation;
                                canvas.drawArc(this.rect, 0.0f, 360.0f, false, this.paint);
                                this.paint.setAlpha(alpha);
                            }
                        } else {
                            rotation2 = rotation;
                        }
                        if (rotation2 != 0.0f) {
                            canvas.restore();
                        }
                    }
                } else {
                    if (i21 == 2) {
                        currentBackProgress = transition;
                        currentProgress = 1.0f - currentBackProgress;
                    } else {
                        currentProgress = transition / 0.5f;
                        currentBackProgress = 1.0f - currentProgress;
                    }
                    y1 = yStart + ((yStart2 - yStart) * currentProgress);
                    y2 = yEnd + ((yEnd2 - yEnd) * currentProgress);
                    x1 = cx3 - ((AndroidUtilities.dp(8.0f) * currentBackProgress) * this.scale);
                    x2 = cx3 + (AndroidUtilities.dp(8.0f) * currentBackProgress * this.scale);
                    y3 = y2 - ((AndroidUtilities.dp(8.0f) * currentBackProgress) * this.scale);
                }
            } else {
                y1 = yStart;
                y2 = yEnd;
                x1 = cx3 - (AndroidUtilities.dp(8.0f) * this.scale);
                x2 = cx3 + (AndroidUtilities.dp(8.0f) * this.scale);
                y3 = y2 - (AndroidUtilities.dp(8.0f) * this.scale);
            }
            if (y1 != y2) {
                canvas.drawLine(cx3, y1, cx3, y2, this.paint);
            }
            if (x1 != cx3) {
                float f4 = y3;
                float f5 = y2;
                canvas.drawLine(x1, f4, cx3, f5, this.paint);
                canvas.drawLine(x2, f4, cx3, f5, this.paint);
            }
        }
        int i23 = this.currentIcon;
        if (i23 != 3 && i23 != 14) {
            if (i23 != 4 || ((i17 = this.nextIcon) != 14 && i17 != 3)) {
                if (i23 == 10 || this.nextIcon == 10 || i23 == 13) {
                    int i24 = this.nextIcon;
                    if (i24 == 4 || i24 == 6) {
                        float backProgress3 = 1.0f - this.transitionProgress;
                        alpha4 = (int) (backProgress3 * 255.0f);
                    } else {
                        alpha4 = 255;
                    }
                    if (alpha4 != 0) {
                        this.paint.setAlpha((int) (alpha4 * this.overrideAlpha));
                        float rad = Math.max(4.0f, this.animatedDownloadProgress * 360.0f);
                        int diff2 = AndroidUtilities.dp(this.isMini ? 2.0f : 4.0f);
                        this.rect.set(bounds.left + diff2, bounds.top + diff2, bounds.right - diff2, bounds.bottom - diff2);
                        canvas.drawArc(this.rect, this.downloadRadOffset, rad, false, this.paint);
                    }
                }
                Drawable nextDrawable4 = null;
                Drawable previousDrawable3 = null;
                i = this.currentIcon;
                if (i != this.nextIcon) {
                    previowsDrawableScale = 1.0f;
                    drawableScale = 1.0f;
                } else if (i == 4) {
                    float drawableScale5 = this.transitionProgress;
                    previowsDrawableScale = 1.0f - this.transitionProgress;
                    drawableScale = drawableScale5;
                } else {
                    float previowsDrawableScale3 = this.transitionProgress;
                    float drawableScale6 = Math.min(1.0f, previowsDrawableScale3 / 0.5f);
                    previowsDrawableScale = Math.max(0.0f, 1.0f - (this.transitionProgress / 0.5f));
                    drawableScale = drawableScale6;
                }
                if (this.nextIcon != 5) {
                    nextDrawable4 = Theme.chat_fileNoneIcon;
                } else if (this.currentIcon == 5) {
                    previousDrawable3 = Theme.chat_fileNoneIcon;
                }
                if (this.nextIcon != 7) {
                    nextDrawable4 = Theme.chat_flameIcon;
                } else if (this.currentIcon == 7) {
                    previousDrawable3 = Theme.chat_flameIcon;
                }
                if (this.nextIcon != 8) {
                    nextDrawable4 = Theme.chat_gifIcon;
                } else if (this.currentIcon == 8) {
                    previousDrawable3 = Theme.chat_gifIcon;
                }
                if (this.nextIcon != 15) {
                    nextDrawable4 = Theme.chat_fileCompressIcon;
                } else if (this.currentIcon == 15) {
                    previousDrawable3 = Theme.chat_fileCompressIcon;
                }
                if (this.nextIcon != 16) {
                    nextDrawable4 = Theme.chat_fileDocIcon;
                } else if (this.currentIcon == 16) {
                    previousDrawable3 = Theme.chat_fileDocIcon;
                }
                if (this.nextIcon != 17) {
                    nextDrawable4 = Theme.chat_fileXlsIcon;
                } else if (this.currentIcon == 17) {
                    previousDrawable3 = Theme.chat_fileXlsIcon;
                }
                if (this.nextIcon != 19) {
                    nextDrawable4 = Theme.chat_filePdfIcon;
                } else if (this.currentIcon == 19) {
                    previousDrawable3 = Theme.chat_filePdfIcon;
                }
                if (this.nextIcon != 18) {
                    nextDrawable4 = Theme.chat_fileTxtIcon;
                } else if (this.currentIcon == 18) {
                    previousDrawable3 = Theme.chat_fileTxtIcon;
                }
                if (this.nextIcon != 20) {
                    nextDrawable4 = Theme.chat_fileApkIcon;
                } else if (this.currentIcon == 20) {
                    previousDrawable3 = Theme.chat_fileApkIcon;
                }
                if (this.nextIcon != 21) {
                    nextDrawable = Theme.chat_fileIpaIcon;
                    previousDrawable = previousDrawable3;
                } else if (this.currentIcon != 21) {
                    nextDrawable = nextDrawable4;
                    previousDrawable = previousDrawable3;
                } else {
                    nextDrawable = nextDrawable4;
                    previousDrawable = Theme.chat_fileIpaIcon;
                }
                i2 = this.currentIcon;
                if (i2 != 9 || this.nextIcon == 9) {
                    this.paint.setAlpha(i2 != this.nextIcon ? 255 : (int) (this.transitionProgress * 255.0f));
                    int y = cy4 + AndroidUtilities.dp(7.0f);
                    int x = cx3 - AndroidUtilities.dp(3.0f);
                    if (this.currentIcon != this.nextIcon) {
                        canvas.save();
                        float f6 = this.transitionProgress;
                        canvas.scale(f6, f6, cx3, cy4);
                    }
                    previowsDrawableScale2 = previowsDrawableScale;
                    canvas.drawLine(x - AndroidUtilities.dp(6.0f), y - AndroidUtilities.dp(6.0f), x, y, this.paint);
                    canvas.drawLine(x, y, x + AndroidUtilities.dp(12.0f), y - AndroidUtilities.dp(12.0f), this.paint);
                    if (this.currentIcon != this.nextIcon) {
                        canvas.restore();
                    }
                } else {
                    previowsDrawableScale2 = previowsDrawableScale;
                }
                i3 = this.currentIcon;
                if (i3 != 12 || this.nextIcon == 12) {
                    i4 = this.nextIcon;
                    if (i3 != i4) {
                        transition3 = 1.0f;
                    } else if (i4 == 13) {
                        transition3 = this.transitionProgress;
                    } else {
                        float transition6 = this.transitionProgress;
                        transition3 = 1.0f - transition6;
                    }
                    this.paint.setAlpha(i3 != i4 ? 255 : (int) (transition3 * 255.0f));
                    int dp = cy4 + AndroidUtilities.dp(7.0f);
                    int dp2 = cx3 - AndroidUtilities.dp(3.0f);
                    if (this.currentIcon != this.nextIcon) {
                        canvas.save();
                        canvas.scale(transition3, transition3, cx3, cy4);
                    }
                    float d5 = AndroidUtilities.dp(7.0f) * this.scale;
                    canvas.drawLine(cx3 - d5, cy4 - d5, cx3 + d5, cy4 + d5, this.paint);
                    canvas.drawLine(cx3 + d5, cy4 - d5, cx3 - d5, cy4 + d5, this.paint);
                    if (this.currentIcon != this.nextIcon) {
                        canvas.restore();
                    }
                }
                i5 = this.currentIcon;
                if (i5 != 13 || this.nextIcon == 13) {
                    i6 = this.nextIcon;
                    if (i5 != i6) {
                        transition4 = 1.0f;
                    } else if (i6 == 13) {
                        transition4 = this.transitionProgress;
                    } else {
                        float transition7 = this.transitionProgress;
                        transition4 = 1.0f - transition7;
                    }
                    this.textPaint.setAlpha((int) (transition4 * 255.0f));
                    int y4 = AndroidUtilities.dp(5.0f) + cy4;
                    int x3 = cx3 - (this.percentStringWidth / 2);
                    if (this.currentIcon != this.nextIcon) {
                        canvas.save();
                        canvas.scale(transition4, transition4, cx3, cy4);
                    }
                    newPercent = (int) (this.animatedDownloadProgress * 100.0f);
                    if (this.percentString != null || newPercent != this.lastPercent) {
                        this.lastPercent = newPercent;
                        this.percentString = String.format("%d%%", Integer.valueOf(newPercent));
                        this.percentStringWidth = (int) Math.ceil(this.textPaint.measureText(format));
                    }
                    canvas.drawText(this.percentString, x3, y4, this.textPaint);
                    if (this.currentIcon != this.nextIcon) {
                        canvas.restore();
                    }
                }
                i7 = this.currentIcon;
                if (i7 != 0 || i7 == 1 || (i14 = this.nextIcon) == 0 || i14 == 1) {
                    i8 = i7 != 0 ? 1 : 1;
                    if (i7 == i8 || this.nextIcon != 0) {
                        p = 0.0f;
                        this.path1.reset();
                        this.path2.reset();
                        float[] finalPath = null;
                        switch (this.currentIcon) {
                            case 0:
                                p1 = playPath1;
                                p2 = playPath2;
                                finalPath = playFinalPath;
                                rotation1 = 0;
                                break;
                            case 1:
                                p1 = pausePath1;
                                p2 = pausePath2;
                                rotation1 = 90;
                                break;
                            default:
                                rotation1 = 0;
                                p1 = null;
                                p2 = null;
                                break;
                        }
                        switch (this.nextIcon) {
                            case 0:
                                p3 = playPath1;
                                p4 = playPath2;
                                rotation22 = 0;
                                break;
                            case 1:
                                p3 = pausePath1;
                                p4 = pausePath2;
                                rotation22 = 90;
                                break;
                            default:
                                rotation22 = 0;
                                p3 = null;
                                p4 = null;
                                break;
                        }
                        if (p1 == null) {
                            p1 = p3;
                            p2 = p4;
                            p3 = null;
                            p4 = null;
                        }
                        nextDrawable2 = nextDrawable;
                        if (!this.animatingTransition || finalPath == null) {
                            cx = cx3;
                            cy = cy4;
                            drawableScale2 = drawableScale;
                            previousDrawable2 = previousDrawable;
                            if (p3 == null) {
                                for (int a2 = 0; a2 < 5; a2++) {
                                    if (a2 == 0) {
                                        this.path1.moveTo(AndroidUtilities.dp(p1[a2 * 2]) * this.scale, AndroidUtilities.dp(p1[(a2 * 2) + 1]) * this.scale);
                                        this.path2.moveTo(AndroidUtilities.dp(p2[a2 * 2]) * this.scale, AndroidUtilities.dp(p2[(a2 * 2) + 1]) * this.scale);
                                    } else {
                                        this.path1.lineTo(AndroidUtilities.dp(p1[a2 * 2]) * this.scale, AndroidUtilities.dp(p1[(a2 * 2) + 1]) * this.scale);
                                        this.path2.lineTo(AndroidUtilities.dp(p2[a2 * 2]) * this.scale, AndroidUtilities.dp(p2[(a2 * 2) + 1]) * this.scale);
                                    }
                                }
                                int a3 = this.nextIcon;
                                if (a3 == 4) {
                                    this.paint2.setAlpha((int) ((1.0f - this.transitionProgress) * 255.0f));
                                    cy2 = 255;
                                } else {
                                    this.paint2.setAlpha(this.currentIcon == a3 ? 255 : (int) (this.transitionProgress * 255.0f));
                                    cy2 = 255;
                                }
                            } else {
                                for (int a4 = 0; a4 < 5; a4++) {
                                    if (a4 == 0) {
                                        this.path1.moveTo(AndroidUtilities.dp(p1[a4 * 2] + ((p3[a4 * 2] - p1[a4 * 2]) * p)) * this.scale, AndroidUtilities.dp(p1[(a4 * 2) + 1] + ((p3[(a4 * 2) + 1] - p1[(a4 * 2) + 1]) * p)) * this.scale);
                                        this.path2.moveTo(AndroidUtilities.dp(p2[a4 * 2] + ((p4[a4 * 2] - p2[a4 * 2]) * p)) * this.scale, AndroidUtilities.dp(p2[(a4 * 2) + 1] + ((p4[(a4 * 2) + 1] - p2[(a4 * 2) + 1]) * p)) * this.scale);
                                    } else {
                                        this.path1.lineTo(AndroidUtilities.dp(p1[a4 * 2] + ((p3[a4 * 2] - p1[a4 * 2]) * p)) * this.scale, AndroidUtilities.dp(p1[(a4 * 2) + 1] + ((p3[(a4 * 2) + 1] - p1[(a4 * 2) + 1]) * p)) * this.scale);
                                        this.path2.lineTo(AndroidUtilities.dp(p2[a4 * 2] + ((p4[a4 * 2] - p2[a4 * 2]) * p)) * this.scale, AndroidUtilities.dp(p2[(a4 * 2) + 1] + ((p4[(a4 * 2) + 1] - p2[(a4 * 2) + 1]) * p)) * this.scale);
                                    }
                                }
                                cy2 = 255;
                                this.paint2.setAlpha(255);
                            }
                        } else {
                            int a5 = 0;
                            while (true) {
                                previousDrawable2 = previousDrawable;
                                if (a5 < finalPath.length / 2) {
                                    if (a5 == 0) {
                                        drawableScale4 = drawableScale;
                                        cy3 = cy4;
                                        cx2 = cx3;
                                        this.path1.moveTo(AndroidUtilities.dp(finalPath[a5 * 2]) * this.scale, AndroidUtilities.dp(finalPath[(a5 * 2) + 1]) * this.scale);
                                        this.path2.moveTo(AndroidUtilities.dp(finalPath[a5 * 2]) * this.scale, AndroidUtilities.dp(finalPath[(a5 * 2) + 1]) * this.scale);
                                    } else {
                                        cx2 = cx3;
                                        cy3 = cy4;
                                        drawableScale4 = drawableScale;
                                        this.path1.lineTo(AndroidUtilities.dp(finalPath[a5 * 2]) * this.scale, AndroidUtilities.dp(finalPath[(a5 * 2) + 1]) * this.scale);
                                        this.path2.lineTo(AndroidUtilities.dp(finalPath[a5 * 2]) * this.scale, AndroidUtilities.dp(finalPath[(a5 * 2) + 1]) * this.scale);
                                    }
                                    a5++;
                                    previousDrawable = previousDrawable2;
                                    drawableScale = drawableScale4;
                                    cy4 = cy3;
                                    cx3 = cx2;
                                } else {
                                    cx = cx3;
                                    cy = cy4;
                                    drawableScale2 = drawableScale;
                                    cy2 = 255;
                                }
                            }
                        }
                        this.path1.close();
                        this.path2.close();
                        canvas.save();
                        canvas.translate(bounds.left, bounds.top);
                        canvas.rotate(rotation1 + ((rotation22 - rotation1) * p), cx - bounds.left, cy - bounds.top);
                        i9 = this.currentIcon;
                        if ((i9 != 0 || i9 == 1) && i9 != 4) {
                            drawableScale3 = drawableScale2;
                        } else {
                            drawableScale3 = drawableScale2;
                            canvas.scale(drawableScale3, drawableScale3, cx - bounds.left, cy - bounds.top);
                        }
                        canvas.drawPath(this.path1, this.paint2);
                        canvas.drawPath(this.path2, this.paint2);
                        canvas.restore();
                    }
                    p = this.animatingTransition ? this.interpolator.getInterpolation(this.transitionProgress) : 0.0f;
                    this.path1.reset();
                    this.path2.reset();
                    float[] finalPath2 = null;
                    switch (this.currentIcon) {
                    }
                    switch (this.nextIcon) {
                    }
                    if (p1 == null) {
                    }
                    nextDrawable2 = nextDrawable;
                    if (!this.animatingTransition) {
                    }
                    cx = cx3;
                    cy = cy4;
                    drawableScale2 = drawableScale;
                    previousDrawable2 = previousDrawable;
                    if (p3 == null) {
                    }
                    this.path1.close();
                    this.path2.close();
                    canvas.save();
                    canvas.translate(bounds.left, bounds.top);
                    canvas.rotate(rotation1 + ((rotation22 - rotation1) * p), cx - bounds.left, cy - bounds.top);
                    i9 = this.currentIcon;
                    if (i9 != 0) {
                    }
                    drawableScale3 = drawableScale2;
                    canvas.drawPath(this.path1, this.paint2);
                    canvas.drawPath(this.path2, this.paint2);
                    canvas.restore();
                } else {
                    cx = cx3;
                    cy = cy4;
                    nextDrawable2 = nextDrawable;
                    previousDrawable2 = previousDrawable;
                    cy2 = 255;
                    drawableScale3 = drawableScale;
                }
                i10 = this.currentIcon;
                if (i10 != 6 || this.nextIcon == 6) {
                    if (i10 == 6) {
                        float f7 = this.transitionProgress;
                        if (f7 > 0.5f) {
                            float progress6 = (f7 - 0.5f) / 0.5f;
                            float progress12 = 1.0f - Math.min(1.0f, progress6 / 0.5f);
                            progress2 = progress6 > 0.5f ? (progress6 - 0.5f) / 0.5f : 0.0f;
                            progress1 = progress12;
                        } else {
                            progress2 = 0.0f;
                            progress1 = 1.0f;
                        }
                    } else {
                        progress2 = 1.0f;
                        progress1 = 0.0f;
                    }
                    int y5 = cy + AndroidUtilities.dp(7.0f);
                    int x4 = cx - AndroidUtilities.dp(3.0f);
                    this.paint.setAlpha(cy2);
                    if (progress1 < 1.0f) {
                        canvas.drawLine(x4 - AndroidUtilities.dp(6.0f), y5 - AndroidUtilities.dp(6.0f), x4 - (AndroidUtilities.dp(6.0f) * progress1), y5 - (AndroidUtilities.dp(6.0f) * progress1), this.paint);
                    }
                    if (progress2 > 0.0f) {
                        canvas.drawLine(x4, y5, (AndroidUtilities.dp(12.0f) * progress2) + x4, y5 - (AndroidUtilities.dp(12.0f) * progress2), this.paint);
                    }
                }
                if (previousDrawable2 == null) {
                    nextDrawable3 = nextDrawable2;
                    Drawable previousDrawable4 = previousDrawable2;
                    if (previousDrawable4 != nextDrawable3) {
                        int w = (int) (previousDrawable4.getIntrinsicWidth() * previowsDrawableScale2);
                        int h = (int) (previousDrawable4.getIntrinsicHeight() * previowsDrawableScale2);
                        int i25 = this.currentIcon;
                        if (i25 == 15 || i25 == 16 || i25 == 17 || i25 != 19) {
                        }
                        previousDrawable4.setAlpha(i25 == this.nextIcon ? 255 : (int) ((1.0f - this.transitionProgress) * 255.0f));
                        previousDrawable4.setBounds(cx - (w / 2), cy - (h / 2), cx + (w / 2), cy + (h / 2));
                        previousDrawable4.draw(canvas);
                    }
                } else {
                    nextDrawable3 = nextDrawable2;
                }
                if (nextDrawable3 != null) {
                    int w2 = (int) (nextDrawable3.getIntrinsicWidth() * drawableScale3);
                    int h2 = (int) (nextDrawable3.getIntrinsicHeight() * drawableScale3);
                    int i26 = this.nextIcon;
                    if (i26 == 15 || (i13 = this.currentIcon) == 16 || i13 == 17 || i13 != 19) {
                    }
                    nextDrawable3.setAlpha(this.currentIcon == i26 ? 255 : (int) (this.transitionProgress * 255.0f));
                    nextDrawable3.setBounds(cx - (w2 / 2), cy - (h2 / 2), cx + (w2 / 2), cy + (h2 / 2));
                    nextDrawable3.draw(canvas);
                }
                long newTime = System.currentTimeMillis();
                dt = newTime - this.lastAnimationTime;
                if (dt > 17) {
                    dt = 17;
                }
                this.lastAnimationTime = newTime;
                i11 = this.currentIcon;
                if (i11 != 3 || i11 == 14 || ((i11 == 4 && this.nextIcon == 14) || i11 == 10 || i11 == 13)) {
                    float f8 = this.downloadRadOffset + (((float) (360 * dt)) / 2500.0f);
                    this.downloadRadOffset = f8;
                    this.downloadRadOffset = getCircleValue(f8);
                    if (this.nextIcon != 2) {
                        float f9 = this.downloadProgress;
                        float f10 = this.downloadProgressAnimationStart;
                        float progressDiff = f9 - f10;
                        if (progressDiff > 0.0f) {
                            float f11 = this.downloadProgressTime + ((float) dt);
                            this.downloadProgressTime = f11;
                            if (f11 < 200.0f) {
                                this.animatedDownloadProgress = f10 + (this.interpolator.getInterpolation(f11 / 200.0f) * progressDiff);
                            } else {
                                this.animatedDownloadProgress = f9;
                                this.downloadProgressAnimationStart = f9;
                                this.downloadProgressTime = 0.0f;
                            }
                        }
                    }
                    invalidateSelf();
                }
                if (this.animatingTransition) {
                    float f12 = this.transitionProgress;
                    if (f12 < 1.0f) {
                        float f13 = f12 + (((float) dt) / this.transitionAnimationTime);
                        this.transitionProgress = f13;
                        if (f13 >= 1.0f) {
                            this.currentIcon = this.nextIcon;
                            this.transitionProgress = 1.0f;
                            this.animatingTransition = false;
                        }
                        invalidateSelf();
                    }
                }
                i12 = this.nextIcon;
                if (i12 != 4 || ((i12 == 6 || i12 == 10) && this.currentIcon == 4)) {
                    canvas.restore();
                }
                return;
            }
        }
        int i27 = this.nextIcon;
        if (i27 == 2) {
            float f14 = this.transitionProgress;
            if (f14 <= 0.5f) {
                float backProgress4 = 1.0f - (f14 / 0.5f);
                d2 = AndroidUtilities.dp(7.0f) * backProgress4 * this.scale;
                alpha3 = (int) (backProgress4 * 255.0f);
            } else {
                d2 = 0.0f;
                alpha3 = 0;
            }
            alpha2 = alpha3;
            iconScale = 1.0f;
            iconScaleX = 0.0f;
            d = d2;
            f = 1.0f;
            rotation3 = 0.0f;
            iconScaleY = 0.0f;
        } else if (i27 == 0 || i27 == 1 || i27 == 5 || i27 == 8 || i27 == 9 || i27 == 7 || i27 == 6 || i27 == 15 || i27 == 16 || i27 == 17 || i27 == 19 || i27 == 18 || i27 == 20 || i27 == 21) {
            if (i27 != 6) {
                progress = this.transitionProgress;
                backProgress = 1.0f - progress;
            } else {
                progress = Math.min(1.0f, this.transitionProgress / 0.5f);
                backProgress = 1.0f - progress;
            }
            float rotation7 = progress * 45.0f;
            float d6 = AndroidUtilities.dp(7.0f) * backProgress * this.scale;
            f = 1.0f;
            alpha2 = (int) (Math.min(1.0f, backProgress * 2.0f) * 255.0f);
            iconScale = 1.0f;
            iconScaleX = 0.0f;
            d = d6;
            rotation3 = 0.0f;
            iconScaleY = rotation7;
        } else if (i27 == 4) {
            float progress7 = this.transitionProgress;
            float backProgress5 = 1.0f - progress7;
            float d7 = AndroidUtilities.dp(7.0f) * this.scale;
            int alpha6 = (int) (backProgress5 * 255.0f);
            if (this.currentIcon == 14) {
                rotation5 = 0.0f;
                iconScale3 = backProgress5;
            } else {
                rotation5 = 45.0f * progress7;
                iconScale3 = 1.0f;
            }
            iconScale = iconScale3;
            iconScaleX = 0.0f;
            d = d7;
            f = 1.0f;
            rotation3 = 0.0f;
            iconScaleY = rotation5;
            alpha2 = alpha6;
        } else if (i27 == 14 || i27 == 3) {
            float d8 = this.transitionProgress;
            float backProgress6 = 1.0f - d8;
            if (i23 == 4) {
                iconScale2 = d8;
                rotation4 = 0.0f;
            } else {
                float rotation8 = 45.0f * backProgress6;
                iconScale2 = 1.0f;
                rotation4 = rotation8;
            }
            float d9 = AndroidUtilities.dp(7.0f) * this.scale;
            alpha2 = (int) (d8 * 255.0f);
            if (this.nextIcon == 14) {
                iconScaleX2 = bounds.left;
                iconScaleY2 = bounds.top;
            } else {
                iconScaleX2 = bounds.centerX();
                iconScaleY2 = bounds.centerY();
            }
            d = d9;
            iconScale = iconScale2;
            iconScaleX = iconScaleX2;
            f = 1.0f;
            float f15 = rotation4;
            rotation3 = iconScaleY2;
            iconScaleY = f15;
        } else {
            alpha2 = 255;
            iconScale = 1.0f;
            iconScaleX = 0.0f;
            d = this.scale * AndroidUtilities.dp(7.0f);
            f = 1.0f;
            rotation3 = 0.0f;
            iconScaleY = 0.0f;
        }
        if (iconScale != f) {
            canvas.save();
            canvas.scale(iconScale, iconScale, iconScaleX, rotation3);
        }
        if (iconScaleY != 0.0f) {
            canvas.save();
            canvas.rotate(iconScaleY, cx3, cy4);
        }
        if (alpha2 != 0) {
            this.paint.setAlpha((int) (alpha2 * this.overrideAlpha));
            if (this.currentIcon != 14) {
                if (this.nextIcon == 14) {
                    rotation6 = iconScaleY;
                    iconScaleX3 = 4.0f;
                } else {
                    float iconScaleY3 = cy4 + d;
                    iconScaleX3 = 4.0f;
                    rotation6 = iconScaleY;
                    canvas.drawLine(cx3 - d, cy4 - d, cx3 + d, iconScaleY3, this.paint);
                    canvas.drawLine(cx3 + d, cy4 - d, cx3 - d, cy4 + d, this.paint);
                }
            } else {
                rotation6 = iconScaleY;
                iconScaleX3 = 4.0f;
            }
            this.paint3.setAlpha((int) (alpha2 * this.overrideAlpha));
            this.rect.set(cx3 - AndroidUtilities.dp(3.5f), cy4 - AndroidUtilities.dp(3.5f), AndroidUtilities.dp(3.5f) + cx3, AndroidUtilities.dp(3.5f) + cy4);
            canvas.drawRoundRect(this.rect, AndroidUtilities.dp(2.0f), AndroidUtilities.dp(2.0f), this.paint3);
        } else {
            rotation6 = iconScaleY;
            iconScaleX3 = 4.0f;
        }
        if (rotation6 != 0.0f) {
            canvas.restore();
        }
        int i28 = this.currentIcon;
        if ((i28 == 3 || i28 == 14 || (i28 == 4 && ((i16 = this.nextIcon) == 14 || i16 == 3))) && alpha2 != 0) {
            float rad2 = Math.max(iconScaleX3, this.animatedDownloadProgress * 360.0f);
            int diff3 = AndroidUtilities.dp(this.isMini ? 2.0f : 4.0f);
            this.rect.set(bounds.left + diff3, bounds.top + diff3, bounds.right - diff3, bounds.bottom - diff3);
            int i29 = this.currentIcon;
            if (i29 == 14 || (i29 == 4 && ((i15 = this.nextIcon) == 14 || i15 == 3))) {
                this.paint.setAlpha((int) (alpha2 * 0.15f * this.overrideAlpha));
                canvas.drawArc(this.rect, 0.0f, 360.0f, false, this.paint);
                this.paint.setAlpha(alpha2);
            }
            canvas.drawArc(this.rect, this.downloadRadOffset, rad2, false, this.paint);
        }
        if (iconScale != 1.0f) {
            canvas.restore();
        }
        Drawable nextDrawable42 = null;
        Drawable previousDrawable32 = null;
        i = this.currentIcon;
        if (i != this.nextIcon) {
        }
        if (this.nextIcon != 5) {
        }
        if (this.nextIcon != 7) {
        }
        if (this.nextIcon != 8) {
        }
        if (this.nextIcon != 15) {
        }
        if (this.nextIcon != 16) {
        }
        if (this.nextIcon != 17) {
        }
        if (this.nextIcon != 19) {
        }
        if (this.nextIcon != 18) {
        }
        if (this.nextIcon != 20) {
        }
        if (this.nextIcon != 21) {
        }
        i2 = this.currentIcon;
        if (i2 != 9) {
        }
        this.paint.setAlpha(i2 != this.nextIcon ? 255 : (int) (this.transitionProgress * 255.0f));
        int y6 = cy4 + AndroidUtilities.dp(7.0f);
        int x5 = cx3 - AndroidUtilities.dp(3.0f);
        if (this.currentIcon != this.nextIcon) {
        }
        previowsDrawableScale2 = previowsDrawableScale;
        canvas.drawLine(x5 - AndroidUtilities.dp(6.0f), y6 - AndroidUtilities.dp(6.0f), x5, y6, this.paint);
        canvas.drawLine(x5, y6, x5 + AndroidUtilities.dp(12.0f), y6 - AndroidUtilities.dp(12.0f), this.paint);
        if (this.currentIcon != this.nextIcon) {
        }
        i3 = this.currentIcon;
        if (i3 != 12) {
        }
        i4 = this.nextIcon;
        if (i3 != i4) {
        }
        this.paint.setAlpha(i3 != i4 ? 255 : (int) (transition3 * 255.0f));
        int dp3 = cy4 + AndroidUtilities.dp(7.0f);
        int dp22 = cx3 - AndroidUtilities.dp(3.0f);
        if (this.currentIcon != this.nextIcon) {
        }
        float d52 = AndroidUtilities.dp(7.0f) * this.scale;
        canvas.drawLine(cx3 - d52, cy4 - d52, cx3 + d52, cy4 + d52, this.paint);
        canvas.drawLine(cx3 + d52, cy4 - d52, cx3 - d52, cy4 + d52, this.paint);
        if (this.currentIcon != this.nextIcon) {
        }
        i5 = this.currentIcon;
        if (i5 != 13) {
        }
        i6 = this.nextIcon;
        if (i5 != i6) {
        }
        this.textPaint.setAlpha((int) (transition4 * 255.0f));
        int y42 = AndroidUtilities.dp(5.0f) + cy4;
        int x32 = cx3 - (this.percentStringWidth / 2);
        if (this.currentIcon != this.nextIcon) {
        }
        newPercent = (int) (this.animatedDownloadProgress * 100.0f);
        if (this.percentString != null) {
        }
        this.lastPercent = newPercent;
        this.percentString = String.format("%d%%", Integer.valueOf(newPercent));
        this.percentStringWidth = (int) Math.ceil(this.textPaint.measureText(format));
        canvas.drawText(this.percentString, x32, y42, this.textPaint);
        if (this.currentIcon != this.nextIcon) {
        }
        i7 = this.currentIcon;
        if (i7 != 0) {
        }
        if (i7 != 0) {
        }
        if (i7 == i8) {
        }
        p = 0.0f;
        this.path1.reset();
        this.path2.reset();
        float[] finalPath22 = null;
        switch (this.currentIcon) {
        }
        switch (this.nextIcon) {
        }
        if (p1 == null) {
        }
        nextDrawable2 = nextDrawable;
        if (!this.animatingTransition) {
        }
        cx = cx3;
        cy = cy4;
        drawableScale2 = drawableScale;
        previousDrawable2 = previousDrawable;
        if (p3 == null) {
        }
        this.path1.close();
        this.path2.close();
        canvas.save();
        canvas.translate(bounds.left, bounds.top);
        canvas.rotate(rotation1 + ((rotation22 - rotation1) * p), cx - bounds.left, cy - bounds.top);
        i9 = this.currentIcon;
        if (i9 != 0) {
        }
        drawableScale3 = drawableScale2;
        canvas.drawPath(this.path1, this.paint2);
        canvas.drawPath(this.path2, this.paint2);
        canvas.restore();
        i10 = this.currentIcon;
        if (i10 != 6) {
        }
        if (i10 == 6) {
        }
        int y52 = cy + AndroidUtilities.dp(7.0f);
        int x42 = cx - AndroidUtilities.dp(3.0f);
        this.paint.setAlpha(cy2);
        if (progress1 < 1.0f) {
        }
        if (progress2 > 0.0f) {
        }
        if (previousDrawable2 == null) {
        }
        if (nextDrawable3 != null) {
        }
        long newTime2 = System.currentTimeMillis();
        dt = newTime2 - this.lastAnimationTime;
        if (dt > 17) {
        }
        this.lastAnimationTime = newTime2;
        i11 = this.currentIcon;
        if (i11 != 3) {
        }
        float f82 = this.downloadRadOffset + (((float) (360 * dt)) / 2500.0f);
        this.downloadRadOffset = f82;
        this.downloadRadOffset = getCircleValue(f82);
        if (this.nextIcon != 2) {
        }
        invalidateSelf();
        if (this.animatingTransition) {
        }
        i12 = this.nextIcon;
        if (i12 != 4) {
        }
        canvas.restore();
    }

    @Override // android.graphics.drawable.Drawable
    public int getIntrinsicWidth() {
        return AndroidUtilities.dp(48.0f);
    }

    @Override // android.graphics.drawable.Drawable
    public int getIntrinsicHeight() {
        return AndroidUtilities.dp(48.0f);
    }

    @Override // android.graphics.drawable.Drawable
    public int getMinimumWidth() {
        return AndroidUtilities.dp(48.0f);
    }

    @Override // android.graphics.drawable.Drawable
    public int getMinimumHeight() {
        return AndroidUtilities.dp(48.0f);
    }
}
