package im.uecahxyvjd.ui.components;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.ObjectAnimator;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Typeface;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.view.View;
import androidx.core.app.NotificationCompat;
import im.uecahxyvjd.messenger.AndroidUtilities;
import java.util.ArrayList;
import java.util.Locale;
/* loaded from: classes10.dex */
public class NumberTextView extends View {
    private ObjectAnimator animator;
    private int currentNumber;
    private ArrayList<StaticLayout> letters;
    private ArrayList<StaticLayout> oldLetters;
    private float progress;
    private TextPaint textPaint;

    public NumberTextView(Context context) {
        super(context);
        this.letters = new ArrayList<>();
        this.oldLetters = new ArrayList<>();
        this.textPaint = new TextPaint(1);
        this.progress = 0.0f;
        this.currentNumber = 1;
    }

    public void setProgress(float value) {
        if (this.progress == value) {
            return;
        }
        this.progress = value;
        invalidate();
    }

    public float getProgress() {
        return this.progress;
    }

    public void setNumber(int number, boolean animated) {
        if (this.currentNumber == number && animated) {
            return;
        }
        ObjectAnimator objectAnimator = this.animator;
        if (objectAnimator != null) {
            objectAnimator.cancel();
            this.animator = null;
        }
        this.oldLetters.clear();
        this.oldLetters.addAll(this.letters);
        this.letters.clear();
        String oldText = String.format(Locale.US, "%d", Integer.valueOf(this.currentNumber));
        String text = String.format(Locale.US, "%d", Integer.valueOf(number));
        boolean forwardAnimation = number > this.currentNumber;
        this.currentNumber = number;
        this.progress = 0.0f;
        int a2 = 0;
        while (a2 < text.length()) {
            String ch = text.substring(a2, a2 + 1);
            String oldCh = (this.oldLetters.isEmpty() || a2 >= oldText.length()) ? null : oldText.substring(a2, a2 + 1);
            if (oldCh == null || !oldCh.equals(ch)) {
                TextPaint textPaint = this.textPaint;
                StaticLayout layout = new StaticLayout(ch, textPaint, (int) Math.ceil(textPaint.measureText(ch)), Layout.Alignment.ALIGN_NORMAL, 1.0f, 0.0f, false);
                this.letters.add(layout);
            } else {
                this.letters.add(this.oldLetters.get(a2));
                this.oldLetters.set(a2, null);
            }
            a2++;
        }
        if (animated && !this.oldLetters.isEmpty()) {
            float[] fArr = new float[2];
            fArr[0] = forwardAnimation ? -1.0f : 1.0f;
            fArr[1] = 0.0f;
            ObjectAnimator ofFloat = ObjectAnimator.ofFloat(this, NotificationCompat.CATEGORY_PROGRESS, fArr);
            this.animator = ofFloat;
            ofFloat.setDuration(150L);
            this.animator.addListener(new AnimatorListenerAdapter() { // from class: im.uecahxyvjd.ui.components.NumberTextView.1
                @Override // android.animation.AnimatorListenerAdapter, android.animation.Animator.AnimatorListener
                public void onAnimationEnd(Animator animation) {
                    NumberTextView.this.animator = null;
                    NumberTextView.this.oldLetters.clear();
                }
            });
            this.animator.start();
        }
        invalidate();
    }

    public void setTextSize(int size) {
        this.textPaint.setTextSize(AndroidUtilities.dp(size));
        this.oldLetters.clear();
        this.letters.clear();
        setNumber(this.currentNumber, false);
    }

    public void setTextColor(int value) {
        this.textPaint.setColor(value);
        invalidate();
    }

    public void setTypeface(Typeface typeface) {
        this.textPaint.setTypeface(typeface);
        this.oldLetters.clear();
        this.letters.clear();
        setNumber(this.currentNumber, false);
    }

    @Override // android.view.View
    protected void onDraw(Canvas canvas) {
        if (this.letters.isEmpty()) {
            return;
        }
        float height = this.letters.get(0).getHeight();
        canvas.save();
        canvas.translate(getPaddingLeft(), (getMeasuredHeight() - height) / 2.0f);
        int count = Math.max(this.letters.size(), this.oldLetters.size());
        int a2 = 0;
        while (a2 < count) {
            canvas.save();
            StaticLayout old = a2 < this.oldLetters.size() ? this.oldLetters.get(a2) : null;
            StaticLayout layout = a2 < this.letters.size() ? this.letters.get(a2) : null;
            float f = this.progress;
            if (f > 0.0f) {
                if (old != null) {
                    this.textPaint.setAlpha((int) (f * 255.0f));
                    canvas.save();
                    canvas.translate(0.0f, (this.progress - 1.0f) * height);
                    old.draw(canvas);
                    canvas.restore();
                    if (layout != null) {
                        this.textPaint.setAlpha((int) ((1.0f - this.progress) * 255.0f));
                        canvas.translate(0.0f, this.progress * height);
                    }
                } else {
                    this.textPaint.setAlpha(255);
                }
            } else if (f < 0.0f) {
                if (old != null) {
                    this.textPaint.setAlpha((int) ((-f) * 255.0f));
                    canvas.save();
                    canvas.translate(0.0f, (this.progress + 1.0f) * height);
                    old.draw(canvas);
                    canvas.restore();
                }
                if (layout != null) {
                    if (a2 == count - 1 || old != null) {
                        this.textPaint.setAlpha((int) ((this.progress + 1.0f) * 255.0f));
                        canvas.translate(0.0f, this.progress * height);
                    } else {
                        this.textPaint.setAlpha(255);
                    }
                }
            } else if (layout != null) {
                this.textPaint.setAlpha(255);
            }
            if (layout != null) {
                layout.draw(canvas);
            }
            canvas.restore();
            canvas.translate(layout != null ? layout.getLineWidth(0) : old.getLineWidth(0) + AndroidUtilities.dp(1.0f), 0.0f);
            a2++;
        }
        canvas.restore();
    }
}
