package im.uecahxyvjd.ui.components.banner.adapter;

import android.view.View;
import android.view.ViewGroup;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.RecyclerView.ViewHolder;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.ui.components.banner.holder.IViewHolder;
import im.uecahxyvjd.ui.components.banner.listener.OnBannerListener;
import im.uecahxyvjd.ui.components.banner.util.BannerUtils;
import java.util.ArrayList;
import java.util.List;
/* loaded from: classes13.dex */
public abstract class BannerAdapter<T, VH extends RecyclerView.ViewHolder> extends RecyclerView.Adapter<VH> implements IViewHolder<T, VH> {
    protected List<T> mDatas = new ArrayList();
    private int mIncreaseCount = 2;
    private OnBannerListener<T> mOnBannerListener;
    private VH mViewHolder;

    public BannerAdapter(List<T> datas) {
        setDatas(datas);
    }

    public void setDatas(List<T> datas) {
        if (datas == null) {
            datas = new ArrayList();
        }
        this.mDatas = datas;
        notifyDataSetChanged();
    }

    public T getData(int position) {
        return this.mDatas.get(position);
    }

    public T getRealData(int position) {
        return this.mDatas.get(getRealPosition(position));
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public final void onBindViewHolder(VH holder, int position) {
        this.mViewHolder = holder;
        final int real = getRealPosition(position);
        final T data = this.mDatas.get(real);
        holder.itemView.setTag(R.id.banner_data_key, data);
        holder.itemView.setTag(R.id.banner_pos_key, Integer.valueOf(real));
        onBindView(holder, this.mDatas.get(real), real, getRealCount());
        if (this.mOnBannerListener != null) {
            holder.itemView.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.banner.adapter.BannerAdapter$$ExternalSyntheticLambda1
                @Override // android.view.View.OnClickListener
                public final void onClick(View view) {
                    BannerAdapter.this.m2874xe557daff(data, real, view);
                }
            });
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$0$im-uecahxyvjd-ui-components-banner-adapter-BannerAdapter  reason: not valid java name */
    public /* synthetic */ void m2874xe557daff(Object data, int real, View view) {
        this.mOnBannerListener.OnBannerClick(data, real);
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public VH onCreateViewHolder(ViewGroup parent, int viewType) {
        final VH vh = (VH) onCreateHolder(parent, viewType);
        vh.itemView.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.components.banner.adapter.BannerAdapter$$ExternalSyntheticLambda0
            @Override // android.view.View.OnClickListener
            public final void onClick(View view) {
                BannerAdapter.this.m2875xa0113bbf(vh, view);
            }
        });
        return vh;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Multi-variable type inference failed */
    /* renamed from: lambda$onCreateViewHolder$1$im-uecahxyvjd-ui-components-banner-adapter-BannerAdapter  reason: not valid java name */
    public /* synthetic */ void m2875xa0113bbf(RecyclerView.ViewHolder vh, View v) {
        if (this.mOnBannerListener != null) {
            Object tag = vh.itemView.getTag(R.id.banner_data_key);
            int real = ((Integer) vh.itemView.getTag(R.id.banner_pos_key)).intValue();
            this.mOnBannerListener.OnBannerClick(tag, real);
        }
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemCount() {
        return getRealCount() > 1 ? getRealCount() + this.mIncreaseCount : getRealCount();
    }

    public int getRealCount() {
        List<T> list = this.mDatas;
        if (list == null) {
            return 0;
        }
        return list.size();
    }

    public int getRealPosition(int position) {
        return BannerUtils.getRealPosition(this.mIncreaseCount == 2, position, getRealCount());
    }

    public void setOnBannerListener(OnBannerListener<T> listener) {
        this.mOnBannerListener = listener;
    }

    public VH getViewHolder() {
        return this.mViewHolder;
    }

    public void setIncreaseCount(int increaseCount) {
        this.mIncreaseCount = increaseCount;
    }
}
