package im.uecahxyvjd.ui.components.paint;

import android.graphics.Bitmap;
import android.graphics.PointF;
import com.google.android.gms.vision.face.Face;
import com.google.android.gms.vision.face.Landmark;
import im.uecahxyvjd.ui.components.Size;
import java.util.List;
/* loaded from: classes4.dex */
public class PhotoFace {
    private float angle;
    private im.uecahxyvjd.ui.components.Point chinPoint;
    private im.uecahxyvjd.ui.components.Point eyesCenterPoint;
    private float eyesDistance;
    private im.uecahxyvjd.ui.components.Point foreheadPoint;
    private im.uecahxyvjd.ui.components.Point mouthPoint;
    private float width;

    public PhotoFace(Face face, Bitmap sourceBitmap, Size targetSize, boolean sideward) {
        List<Landmark> landmarks = face.getLandmarks();
        im.uecahxyvjd.ui.components.Point leftEyePoint = null;
        im.uecahxyvjd.ui.components.Point rightEyePoint = null;
        im.uecahxyvjd.ui.components.Point leftMouthPoint = null;
        im.uecahxyvjd.ui.components.Point rightMouthPoint = null;
        for (Landmark landmark : landmarks) {
            PointF point = landmark.getPosition();
            switch (landmark.getType()) {
                case 4:
                    leftEyePoint = transposePoint(point, sourceBitmap, targetSize, sideward);
                    break;
                case 5:
                    leftMouthPoint = transposePoint(point, sourceBitmap, targetSize, sideward);
                    break;
                case 10:
                    rightEyePoint = transposePoint(point, sourceBitmap, targetSize, sideward);
                    break;
                case 11:
                    rightMouthPoint = transposePoint(point, sourceBitmap, targetSize, sideward);
                    break;
            }
        }
        if (leftEyePoint != null && rightEyePoint != null) {
            if (leftEyePoint.x < rightEyePoint.x) {
                im.uecahxyvjd.ui.components.Point temp = leftEyePoint;
                leftEyePoint = rightEyePoint;
                rightEyePoint = temp;
            }
            this.eyesCenterPoint = new im.uecahxyvjd.ui.components.Point((leftEyePoint.x * 0.5f) + (rightEyePoint.x * 0.5f), (leftEyePoint.y * 0.5f) + (rightEyePoint.y * 0.5f));
            this.eyesDistance = (float) Math.hypot(rightEyePoint.x - leftEyePoint.x, rightEyePoint.y - leftEyePoint.y);
            float degrees = (float) Math.toDegrees(Math.atan2(rightEyePoint.y - leftEyePoint.y, rightEyePoint.x - leftEyePoint.x) + 3.141592653589793d);
            this.angle = degrees;
            float f = this.eyesDistance;
            this.width = 2.35f * f;
            float foreheadHeight = f * 0.8f;
            float upAngle = (float) Math.toRadians(degrees - 90.0f);
            this.foreheadPoint = new im.uecahxyvjd.ui.components.Point(this.eyesCenterPoint.x + (((float) Math.cos(upAngle)) * foreheadHeight), this.eyesCenterPoint.y + (((float) Math.sin(upAngle)) * foreheadHeight));
        }
        if (leftMouthPoint != null && rightMouthPoint != null) {
            if (leftMouthPoint.x < rightMouthPoint.x) {
                im.uecahxyvjd.ui.components.Point leftMouthPoint2 = rightMouthPoint;
                rightMouthPoint = leftMouthPoint;
                leftMouthPoint = leftMouthPoint2;
            }
            this.mouthPoint = new im.uecahxyvjd.ui.components.Point((leftMouthPoint.x * 0.5f) + (rightMouthPoint.x * 0.5f), (leftMouthPoint.y * 0.5f) + (rightMouthPoint.y * 0.5f));
            float chinDepth = this.eyesDistance * 0.7f;
            float downAngle = (float) Math.toRadians(this.angle + 90.0f);
            this.chinPoint = new im.uecahxyvjd.ui.components.Point(this.mouthPoint.x + (((float) Math.cos(downAngle)) * chinDepth), this.mouthPoint.y + (((float) Math.sin(downAngle)) * chinDepth));
        }
    }

    public boolean isSufficient() {
        return this.eyesCenterPoint != null;
    }

    private im.uecahxyvjd.ui.components.Point transposePoint(PointF point, Bitmap sourceBitmap, Size targetSize, boolean sideward) {
        float bitmapW = sideward ? sourceBitmap.getHeight() : sourceBitmap.getWidth();
        float bitmapH = sideward ? sourceBitmap.getWidth() : sourceBitmap.getHeight();
        return new im.uecahxyvjd.ui.components.Point((targetSize.width * point.x) / bitmapW, (targetSize.height * point.y) / bitmapH);
    }

    public im.uecahxyvjd.ui.components.Point getPointForAnchor(int anchor) {
        switch (anchor) {
            case 0:
                return this.foreheadPoint;
            case 1:
                return this.eyesCenterPoint;
            case 2:
                return this.mouthPoint;
            case 3:
                return this.chinPoint;
            default:
                return null;
        }
    }

    public float getWidthForAnchor(int anchor) {
        if (anchor == 1) {
            return this.eyesDistance;
        }
        return this.width;
    }

    public float getAngle() {
        return this.angle;
    }
}
