package im.uecahxyvjd.ui.components.paint.views;

import android.content.Context;
import android.graphics.Paint;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.ui.components.Point;
import im.uecahxyvjd.ui.components.Rect;
import java.util.UUID;
/* loaded from: classes16.dex */
public class EntityView extends FrameLayout {
    private boolean announcedSelection;
    private EntityViewDelegate delegate;
    private GestureDetector gestureDetector;
    private boolean hasPanned;
    private boolean hasReleased;
    private boolean hasTransformed;
    private int offsetX;
    private int offsetY;
    protected Point position;
    private float previousLocationX;
    private float previousLocationY;
    private boolean recognizedLongPress;
    protected SelectionView selectionView;
    private UUID uuid;

    /* loaded from: classes16.dex */
    public interface EntityViewDelegate {
        boolean allowInteraction(EntityView entityView);

        boolean onEntityLongClicked(EntityView entityView);

        boolean onEntitySelected(EntityView entityView);
    }

    public EntityView(Context context, Point pos) {
        super(context);
        this.hasPanned = false;
        this.hasReleased = false;
        this.hasTransformed = false;
        this.announcedSelection = false;
        this.recognizedLongPress = false;
        this.position = new Point();
        this.uuid = UUID.randomUUID();
        this.position = pos;
        this.gestureDetector = new GestureDetector(context, new GestureDetector.SimpleOnGestureListener() { // from class: im.uecahxyvjd.ui.components.paint.views.EntityView.1
            @Override // android.view.GestureDetector.SimpleOnGestureListener, android.view.GestureDetector.OnGestureListener
            public void onLongPress(MotionEvent e) {
                if (!EntityView.this.hasPanned && !EntityView.this.hasTransformed && !EntityView.this.hasReleased) {
                    EntityView.this.recognizedLongPress = true;
                    if (EntityView.this.delegate != null) {
                        EntityView.this.performHapticFeedback(0);
                        EntityView.this.delegate.onEntityLongClicked(EntityView.this);
                    }
                }
            }
        });
    }

    public UUID getUUID() {
        return this.uuid;
    }

    public Point getPosition() {
        return this.position;
    }

    public void setPosition(Point value) {
        this.position = value;
        updatePosition();
    }

    public float getScale() {
        return getScaleX();
    }

    public void setScale(float scale) {
        setScaleX(scale);
        setScaleY(scale);
    }

    public void setDelegate(EntityViewDelegate entityViewDelegate) {
        this.delegate = entityViewDelegate;
    }

    public void setOffset(int x, int y) {
        this.offsetX = x;
        this.offsetY = y;
    }

    @Override // android.view.ViewGroup
    public boolean onInterceptTouchEvent(MotionEvent ev) {
        return this.delegate.allowInteraction(this);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public boolean onTouchMove(float x, float y) {
        float scale = ((View) getParent()).getScaleX();
        Point translation = new Point((x - this.previousLocationX) / scale, (y - this.previousLocationY) / scale);
        float distance = (float) Math.hypot(translation.x, translation.y);
        float minDistance = this.hasPanned ? 6.0f : 16.0f;
        if (distance > minDistance) {
            pan(translation);
            this.previousLocationX = x;
            this.previousLocationY = y;
            this.hasPanned = true;
            return true;
        }
        return false;
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void onTouchUp() {
        EntityViewDelegate entityViewDelegate;
        if (!this.recognizedLongPress && !this.hasPanned && !this.hasTransformed && !this.announcedSelection && (entityViewDelegate = this.delegate) != null) {
            entityViewDelegate.onEntitySelected(this);
        }
        this.recognizedLongPress = false;
        this.hasPanned = false;
        this.hasTransformed = false;
        this.hasReleased = true;
        this.announcedSelection = false;
    }

    @Override // android.view.View
    public boolean onTouchEvent(MotionEvent event) {
        EntityViewDelegate entityViewDelegate;
        if (event.getPointerCount() > 1 || !this.delegate.allowInteraction(this)) {
            return false;
        }
        float x = event.getRawX();
        float y = event.getRawY();
        int action = event.getActionMasked();
        boolean handled = false;
        switch (action) {
            case 0:
            case 5:
                if (!isSelected() && (entityViewDelegate = this.delegate) != null) {
                    entityViewDelegate.onEntitySelected(this);
                    this.announcedSelection = true;
                }
                this.previousLocationX = x;
                this.previousLocationY = y;
                handled = true;
                this.hasReleased = false;
                break;
            case 1:
            case 3:
            case 6:
                onTouchUp();
                handled = true;
                break;
            case 2:
                handled = onTouchMove(x, y);
                break;
        }
        this.gestureDetector.onTouchEvent(event);
        return handled;
    }

    public void pan(Point translation) {
        this.position.x += translation.x;
        this.position.y += translation.y;
        updatePosition();
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void updatePosition() {
        float halfWidth = getWidth() / 2.0f;
        float halfHeight = getHeight() / 2.0f;
        setX(this.position.x - halfWidth);
        setY(this.position.y - halfHeight);
        updateSelectionView();
    }

    public void scale(float scale) {
        float newScale = Math.max(getScale() * scale, 0.1f);
        setScale(newScale);
        updateSelectionView();
    }

    public void rotate(float angle) {
        setRotation(angle);
        updateSelectionView();
    }

    protected Rect getSelectionBounds() {
        return new Rect(0.0f, 0.0f, 0.0f, 0.0f);
    }

    @Override // android.view.View
    public boolean isSelected() {
        return this.selectionView != null;
    }

    protected SelectionView createSelectionView() {
        return null;
    }

    public void updateSelectionView() {
        SelectionView selectionView = this.selectionView;
        if (selectionView != null) {
            selectionView.updatePosition();
        }
    }

    public void select(ViewGroup selectionContainer) {
        SelectionView selectionView = createSelectionView();
        this.selectionView = selectionView;
        selectionContainer.addView(selectionView);
        selectionView.updatePosition();
    }

    public void deselect() {
        SelectionView selectionView = this.selectionView;
        if (selectionView == null) {
            return;
        }
        if (selectionView.getParent() != null) {
            ((ViewGroup) this.selectionView.getParent()).removeView(this.selectionView);
        }
        this.selectionView = null;
    }

    public void setSelectionVisibility(boolean visible) {
        SelectionView selectionView = this.selectionView;
        if (selectionView == null) {
            return;
        }
        selectionView.setVisibility(visible ? 0 : 8);
    }

    /* loaded from: classes16.dex */
    public class SelectionView extends FrameLayout {
        public static final int SELECTION_LEFT_HANDLE = 1;
        public static final int SELECTION_RIGHT_HANDLE = 2;
        public static final int SELECTION_WHOLE_HANDLE = 3;
        private int currentHandle;
        protected Paint dotPaint;
        protected Paint dotStrokePaint;
        protected Paint paint;

        public SelectionView(Context context) {
            super(context);
            this.paint = new Paint(1);
            this.dotPaint = new Paint(1);
            this.dotStrokePaint = new Paint(1);
            setWillNotDraw(false);
            this.paint.setColor(-1);
            this.dotPaint.setColor(-12793105);
            this.dotStrokePaint.setColor(-1);
            this.dotStrokePaint.setStyle(Paint.Style.STROKE);
            this.dotStrokePaint.setStrokeWidth(AndroidUtilities.dp(1.0f));
        }

        protected void updatePosition() {
            Rect bounds = EntityView.this.getSelectionBounds();
            FrameLayout.LayoutParams layoutParams = (FrameLayout.LayoutParams) getLayoutParams();
            layoutParams.leftMargin = ((int) bounds.x) + EntityView.this.offsetX;
            layoutParams.topMargin = ((int) bounds.y) + EntityView.this.offsetY;
            layoutParams.width = (int) bounds.width;
            layoutParams.height = (int) bounds.height;
            setLayoutParams(layoutParams);
            setRotation(EntityView.this.getRotation());
        }

        protected int pointInsideHandle(float x, float y) {
            return 0;
        }

        @Override // android.view.View
        public boolean onTouchEvent(MotionEvent event) {
            boolean handled;
            boolean handled2;
            int action = event.getActionMasked();
            switch (action) {
                case 0:
                case 5:
                    int handle = pointInsideHandle(event.getX(), event.getY());
                    if (handle == 0) {
                        handled = false;
                        break;
                    } else {
                        this.currentHandle = handle;
                        EntityView.this.previousLocationX = event.getRawX();
                        EntityView.this.previousLocationY = event.getRawY();
                        EntityView.this.hasReleased = false;
                        handled = true;
                        break;
                    }
                case 1:
                case 3:
                case 6:
                    EntityView.this.onTouchUp();
                    this.currentHandle = 0;
                    handled = true;
                    break;
                case 2:
                    int i = this.currentHandle;
                    if (i == 3) {
                        float x = event.getRawX();
                        float y = event.getRawY();
                        handled = EntityView.this.onTouchMove(x, y);
                        break;
                    } else if (i != 0) {
                        EntityView.this.hasTransformed = true;
                        Point translation = new Point(event.getRawX() - EntityView.this.previousLocationX, event.getRawY() - EntityView.this.previousLocationY);
                        float radAngle = (float) Math.toRadians(getRotation());
                        float delta = (float) ((translation.x * Math.cos(radAngle)) + (translation.y * Math.sin(radAngle)));
                        if (this.currentHandle == 1) {
                            delta *= -1.0f;
                        }
                        float scaleDelta = ((2.0f * delta) / getWidth()) + 1.0f;
                        EntityView.this.scale(scaleDelta);
                        float centerX = getLeft() + (getWidth() / 2);
                        float centerY = getTop() + (getHeight() / 2);
                        float parentX = event.getRawX() - ((View) getParent()).getLeft();
                        float parentY = (event.getRawY() - ((View) getParent()).getTop()) - AndroidUtilities.statusBarHeight;
                        float angle = 0.0f;
                        int i2 = this.currentHandle;
                        if (i2 == 1) {
                            angle = (float) Math.atan2(centerY - parentY, centerX - parentX);
                        } else if (i2 == 2) {
                            angle = (float) Math.atan2(parentY - centerY, parentX - centerX);
                        }
                        EntityView.this.rotate((float) Math.toDegrees(angle));
                        EntityView.this.previousLocationX = event.getRawX();
                        EntityView.this.previousLocationY = event.getRawY();
                        handled = true;
                        break;
                    } else {
                        handled2 = false;
                        handled = handled2;
                        break;
                    }
                case 4:
                default:
                    handled2 = false;
                    handled = handled2;
                    break;
            }
            if (this.currentHandle == 3) {
                EntityView.this.gestureDetector.onTouchEvent(event);
            }
            return handled;
        }
    }
}
