package im.uecahxyvjd.ui.components.toast;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.widget.Toast;
import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;
/* loaded from: classes4.dex */
public class ToastStrategy extends Handler {
    private static final int DELAY_TIMEOUT = 300;
    public static final int LONG_DURATION_TIMEOUT = 3500;
    private static final int MAX_TOAST_CAPACITY = 3;
    public static final int SHORT_DURATION_TIMEOUT = 2000;
    private static final int TYPE_CANCEL = 3;
    private static final int TYPE_CONTINUE = 2;
    private static final int TYPE_SHOW = 1;
    private volatile boolean isShow;
    private volatile Queue<CharSequence> mQueue;
    private Toast mToast;

    public ToastStrategy() {
        super(Looper.getMainLooper());
        this.mQueue = getToastQueue();
    }

    public void bind(Toast toast) {
        this.mToast = toast;
    }

    public void show(CharSequence text) {
        if ((this.mQueue.isEmpty() || !this.mQueue.contains(text)) && !this.mQueue.offer(text)) {
            this.mQueue.poll();
            this.mQueue.offer(text);
        }
        if (!this.isShow) {
            this.isShow = true;
            sendEmptyMessageDelayed(1, 300L);
        }
    }

    public void cancel() {
        if (this.isShow) {
            this.isShow = false;
            sendEmptyMessage(3);
        }
    }

    @Override // android.os.Handler
    public void handleMessage(Message msg) {
        switch (msg.what) {
            case 1:
                CharSequence text = this.mQueue.peek();
                if (text != null) {
                    this.mToast.setText(text);
                    this.mToast.show();
                    sendEmptyMessageDelayed(2, getToastDuration(text) + 300);
                    return;
                }
                this.isShow = false;
                return;
            case 2:
                this.mQueue.poll();
                if (!this.mQueue.isEmpty()) {
                    sendEmptyMessage(1);
                    return;
                } else {
                    this.isShow = false;
                    return;
                }
            case 3:
                this.isShow = false;
                this.mQueue.clear();
                this.mToast.cancel();
                return;
            default:
                return;
        }
    }

    public Queue<CharSequence> getToastQueue() {
        return new ArrayBlockingQueue(3);
    }

    public int getToastDuration(CharSequence text) {
        if (text.length() > 20) {
            return LONG_DURATION_TIMEOUT;
        }
        return 2000;
    }
}
