package im.uecahxyvjd.ui.decoration;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.view.View;
import android.view.ViewGroup;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.ui.decoration.ClickInfo;
import im.uecahxyvjd.ui.decoration.cache.CacheUtil;
import im.uecahxyvjd.ui.decoration.listener.GroupXListener;
import im.uecahxyvjd.ui.decoration.listener.OnGroupClickListener;
import im.uecahxyvjd.ui.decoration.util.ViewUtil;
import java.util.ArrayList;
import java.util.List;
/* loaded from: classes6.dex */
public class StickyXDecoration extends BaseDecoration {
    private CacheUtil<Bitmap> mBitmapCache;
    private GroupXListener mGroupListener;
    private Paint mGroutPaint;
    private CacheUtil<View> mHeadViewCache;

    private StickyXDecoration(GroupXListener groupListener) {
        this.mBitmapCache = new CacheUtil<>();
        this.mHeadViewCache = new CacheUtil<>();
        this.mGroupListener = groupListener;
        this.mGroutPaint = new Paint();
    }

    @Override // im.uecahxyvjd.ui.decoration.BaseDecoration, androidx.recyclerview.widget.RecyclerView.ItemDecoration
    public void onDrawOver(Canvas c, RecyclerView parent, RecyclerView.State state) {
        int childCount;
        int childCount2;
        int bottom;
        super.onDrawOver(c, parent, state);
        int itemCount = state.getItemCount();
        int childCount3 = parent.getChildCount();
        int left = parent.getPaddingLeft();
        int right = parent.getWidth() - parent.getPaddingRight();
        int i = 0;
        while (i < childCount3) {
            View childView = parent.getChildAt(i);
            int position = parent.getChildAdapterPosition(childView);
            int realPosition = getRealPosition(position);
            if (!isFirstInGroup(realPosition)) {
                if (isFirstInRecyclerView(realPosition, i)) {
                    childCount = childCount3;
                    childCount2 = realPosition;
                } else {
                    childCount = childCount3;
                    drawDivide(c, parent, childView, realPosition, left, right);
                    i++;
                    childCount3 = childCount;
                }
            } else {
                childCount = childCount3;
                childCount2 = realPosition;
            }
            int viewBottom = childView.getBottom();
            int bottom2 = Math.max(this.mGroupHeight, childView.getTop() + parent.getPaddingTop());
            if (position + 1 < itemCount && isLastLineInGroup(parent, childCount2) && viewBottom < bottom2) {
                bottom = viewBottom;
            } else {
                bottom = bottom2;
            }
            drawDecoration(c, childCount2, left, right, bottom);
            i++;
            childCount3 = childCount;
        }
    }

    private void drawDecoration(Canvas c, int realPosition, int left, int right, int bottom) {
        View groupView;
        Bitmap bitmap;
        c.drawRect(left, bottom - this.mGroupHeight, right, bottom, this.mGroutPaint);
        int firstPositionInGroup = getFirstInGroupWithCash(realPosition);
        if (this.mHeadViewCache.get(firstPositionInGroup) == null) {
            groupView = getGroupView(firstPositionInGroup);
            if (groupView == null) {
                return;
            }
            measureAndLayoutView(groupView, left, right);
            this.mHeadViewCache.put(firstPositionInGroup, groupView);
        } else {
            groupView = this.mHeadViewCache.get(firstPositionInGroup);
        }
        if (this.mBitmapCache.get(firstPositionInGroup) != null) {
            bitmap = this.mBitmapCache.get(firstPositionInGroup);
        } else {
            bitmap = Bitmap.createBitmap(groupView.getDrawingCache());
            this.mBitmapCache.put(firstPositionInGroup, bitmap);
        }
        c.drawBitmap(bitmap, left, bottom - this.mGroupHeight, (Paint) null);
        if (this.mOnGroupClickListener != null) {
            setClickInfo(groupView, left, bottom, realPosition);
        }
    }

    private void measureAndLayoutView(View groupView, int left, int right) {
        groupView.setDrawingCacheEnabled(true);
        ViewGroup.LayoutParams layoutParams = new ViewGroup.LayoutParams(right, this.mGroupHeight);
        groupView.setLayoutParams(layoutParams);
        groupView.measure(View.MeasureSpec.makeMeasureSpec(right, 1073741824), View.MeasureSpec.makeMeasureSpec(this.mGroupHeight, 1073741824));
        groupView.layout(left, 0 - this.mGroupHeight, right, 0);
    }

    private void setClickInfo(View groupView, int parentLeft, int parentBottom, int realPosition) {
        int parentTop = parentBottom - this.mGroupHeight;
        List<ClickInfo.DetailInfo> list = new ArrayList<>();
        List<View> viewList = ViewUtil.getChildViewWithId(groupView);
        for (View view : viewList) {
            int top = view.getTop() + parentTop;
            int bottom = view.getBottom() + parentTop;
            int left = view.getLeft() + parentLeft;
            int right = view.getRight() + parentLeft;
            list.add(new ClickInfo.DetailInfo(view.getId(), left, right, top, bottom));
            parentTop = parentTop;
        }
        ClickInfo clickInfo = new ClickInfo(parentBottom, list);
        clickInfo.mGroupId = groupView.getId();
        this.stickyHeaderPosArray.put(Integer.valueOf(realPosition), clickInfo);
    }

    @Override // im.uecahxyvjd.ui.decoration.BaseDecoration
    String getGroupName(int realPosition) {
        GroupXListener groupXListener = this.mGroupListener;
        if (groupXListener != null) {
            return groupXListener.getGroupName(realPosition);
        }
        return null;
    }

    private View getGroupView(int realPosition) {
        GroupXListener groupXListener = this.mGroupListener;
        if (groupXListener != null) {
            return groupXListener.getGroupView(realPosition);
        }
        return null;
    }

    public void setCacheEnable(boolean b) {
        this.mHeadViewCache.isCacheable(b);
    }

    public void clearCache() {
        this.mHeadViewCache.clean();
        this.mBitmapCache.clean();
    }

    public void notifyRedraw(RecyclerView recyclerView, View viewGroup, int realPosition) {
        viewGroup.setDrawingCacheEnabled(false);
        int firstPositionInGroup = getFirstInGroupWithCash(realPosition);
        this.mBitmapCache.remove(firstPositionInGroup);
        this.mHeadViewCache.remove(firstPositionInGroup);
        int left = recyclerView.getPaddingLeft();
        int right = recyclerView.getWidth() - recyclerView.getPaddingRight();
        measureAndLayoutView(viewGroup, left, right);
        this.mHeadViewCache.put(firstPositionInGroup, viewGroup);
        recyclerView.invalidate();
    }

    /* loaded from: classes6.dex */
    public static class Builder {
        StickyXDecoration mDecoration;

        private Builder(GroupXListener listener) {
            this.mDecoration = new StickyXDecoration(listener);
        }

        public static Builder init(GroupXListener listener) {
            return new Builder(listener);
        }

        public Builder setGroupHeight(int groutHeight) {
            this.mDecoration.mGroupHeight = groutHeight;
            return this;
        }

        public Builder setGroupBackground(int background) {
            this.mDecoration.mGroupBackground = background;
            this.mDecoration.mGroutPaint.setColor(this.mDecoration.mGroupBackground);
            return this;
        }

        public Builder setDivideHeight(int height) {
            this.mDecoration.mDivideHeight = height;
            return this;
        }

        public Builder setDivideColor(int color) {
            this.mDecoration.mDivideColor = color;
            this.mDecoration.mDividePaint.setColor(this.mDecoration.mDivideColor);
            return this;
        }

        public Builder setOnClickListener(OnGroupClickListener listener) {
            this.mDecoration.setOnGroupClickListener(listener);
            return this;
        }

        public Builder resetSpan(RecyclerView recyclerView, GridLayoutManager gridLayoutManager) {
            this.mDecoration.resetSpan(recyclerView, gridLayoutManager);
            return this;
        }

        public Builder setCacheEnable(boolean b) {
            this.mDecoration.setCacheEnable(b);
            return this;
        }

        public Builder setHeaderCount(int headerCount) {
            if (headerCount >= 0) {
                this.mDecoration.mHeaderCount = headerCount;
            }
            return this;
        }

        public StickyXDecoration build() {
            return this.mDecoration;
        }
    }
}
