package im.uecahxyvjd.ui.expand;

import android.view.ViewGroup;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.ui.expand.models.ExpandableGroup;
import im.uecahxyvjd.ui.expand.models.ExpandableListPosition;
import im.uecahxyvjd.ui.expand.viewholders.ChildViewHolder;
import im.uecahxyvjd.ui.expand.viewholders.GroupViewHolder;
import java.util.List;
/* loaded from: classes10.dex */
public abstract class MultiTypeExpandableRecyclerViewAdapter<GVH extends GroupViewHolder, CVH extends ChildViewHolder> extends ExpandableRecyclerViewAdapter<GVH, CVH> {
    public MultiTypeExpandableRecyclerViewAdapter(List<? extends ExpandableGroup> groups) {
        super(groups);
    }

    @Override // im.uecahxyvjd.ui.expand.ExpandableRecyclerViewAdapter, androidx.recyclerview.widget.RecyclerView.Adapter
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        if (isGroup(viewType)) {
            GVH gvh = onCreateGroupViewHolder(parent, viewType);
            gvh.setOnGroupClickListener(this);
            return gvh;
        } else if (isChild(viewType)) {
            CVH cvh = onCreateChildViewHolder(parent, viewType);
            return cvh;
        } else {
            throw new IllegalArgumentException("viewType is not valid");
        }
    }

    /* JADX WARN: Multi-variable type inference failed */
    @Override // im.uecahxyvjd.ui.expand.ExpandableRecyclerViewAdapter, androidx.recyclerview.widget.RecyclerView.Adapter
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        ExpandableListPosition listPos = this.expandableList.getUnflattenedPosition(position);
        ExpandableGroup group = this.expandableList.getExpandableGroup(listPos);
        if (isGroup(getItemViewType(position))) {
            onBindGroupViewHolder((GroupViewHolder) holder, position, group);
            if (isGroupExpanded(group)) {
                ((GroupViewHolder) holder).expand();
            } else {
                ((GroupViewHolder) holder).collapse();
            }
        } else if (isChild(getItemViewType(position))) {
            onBindChildViewHolder((ChildViewHolder) holder, position, group, listPos.childPos);
        }
    }

    @Override // im.uecahxyvjd.ui.expand.ExpandableRecyclerViewAdapter, androidx.recyclerview.widget.RecyclerView.Adapter
    public int getItemViewType(int position) {
        ExpandableListPosition listPosition = this.expandableList.getUnflattenedPosition(position);
        ExpandableGroup group = this.expandableList.getExpandableGroup(listPosition);
        int viewType = listPosition.type;
        switch (viewType) {
            case 1:
                return getChildViewType(position, group, listPosition.childPos);
            case 2:
                return getGroupViewType(position, group);
            default:
                return viewType;
        }
    }

    public int getChildViewType(int position, ExpandableGroup group, int childIndex) {
        return super.getItemViewType(position);
    }

    public int getGroupViewType(int position, ExpandableGroup group) {
        return super.getItemViewType(position);
    }

    public boolean isGroup(int viewType) {
        return viewType == 2;
    }

    public boolean isChild(int viewType) {
        return viewType == 1;
    }
}
