package im.uecahxyvjd.ui.fragments;

import android.app.Activity;
import android.content.Context;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.bjz.comm.net.utils.RxHelper;
import com.blankj.utilcode.util.ScreenUtils;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ApplicationLoader;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPC2;
import im.uecahxyvjd.ui.WebviewActivity;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.cells.ShadowSectionCell;
import im.uecahxyvjd.ui.components.LayoutHelper;
import im.uecahxyvjd.ui.components.RecyclerListView;
import im.uecahxyvjd.ui.components.banner.Banner;
import im.uecahxyvjd.ui.components.banner.adapter.BannerAdapter;
import im.uecahxyvjd.ui.components.banner.indicator.RectangleIndicator;
import im.uecahxyvjd.ui.components.banner.listener.OnBannerListener;
import im.uecahxyvjd.ui.components.toast.ToastUtils;
import im.uecahxyvjd.ui.constants.Constants;
import im.uecahxyvjd.ui.fragments.DiscoveryFragment;
import im.uecahxyvjd.ui.hcells.IndexTextCell;
import im.uecahxyvjd.ui.hui.adapter.pageAdapter.PageHolder;
import im.uecahxyvjd.ui.hui.discovery.ActionIntroActivity;
import im.uecahxyvjd.ui.hui.discovery.NearPersonAndGroupActivity;
import im.uecahxyvjd.ui.hui.discovery.QrScanActivity;
import im.uecahxyvjd.ui.hui.discoveryweb.DiscoveryJumpToPage;
import im.uecahxyvjd.ui.hui.friendscircle.glide.GlideUtils;
import im.uecahxyvjd.ui.hui.friendscircle_v1.FriendsCircleActivity;
import im.uecahxyvjd.ui.hui.friendscircle_v1.ui.FcAlbumActivity;
import im.uecahxyvjd.ui.hui.hotGroup.HotGroupRecommendActivity;
import im.uecahxyvjd.ui.hui.visualcall.PermissionUtils;
import im.uecahxyvjd.ui.hviews.sliding.SlidingLayout;
/* loaded from: classes10.dex */
public class DiscoveryFragment extends BaseFmts implements Constants {
    private static final String TAG = "DiscoveryFragment";
    private Delegate delegate;
    private int extraDataReqToken;
    private boolean hasGps;
    private ListAdapter listAdapter;
    private RecyclerListView listView;
    private int rowCount;
    private int startSectionRow = -1;
    private int bannerStartRow = -1;
    private int bannerRow = -1;
    private int bannerEndRow = -1;
    private int friendsHubRow = -1;
    private int friendsHubEmptyRow = -1;
    private int scanRow = -1;
    private int nearbyRow = -1;
    private int nearbyEmptyRow = -1;
    private int gameCenterRow = -1;
    private int gameCenterEmptyRow = -1;
    private int miniProgramRow = -1;
    private int lastSectionRow = -1;
    private int album = -1;
    private int albumEmptyRow = -1;
    private int recommendChannel = -1;
    private int recommendChannelEmptyRow = -1;
    private int extraDataStartRow = -1;
    private int extraDataEndRow = -1;

    /* loaded from: classes10.dex */
    public interface Delegate {
        TLRPC2.TL_DiscoveryPageSetting getDiscoveryPageData();
    }

    @Override // im.uecahxyvjd.ui.fragments.BaseFmts, androidx.fragment.app.Fragment
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        try {
            this.hasGps = ApplicationLoader.applicationContext.getPackageManager().hasSystemFeature("android.hardware.location.gps");
        } catch (Throwable th) {
            this.hasGps = false;
        }
        updateRow();
    }

    @Override // im.uecahxyvjd.ui.fragments.BaseFmts, androidx.fragment.app.Fragment
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        FrameLayout frameLayout = new FrameLayout(this.context);
        this.fragmentView = frameLayout;
        this.actionBar = createActionBar();
        this.actionBar.setAllowOverlayTitle(true);
        this.actionBar.setTitle(LocaleController.getString("Discovery", R.string.Discovery));
        frameLayout.addView(this.actionBar, LayoutHelper.createFrame(-1, -2.0f));
        SlidingLayout root = new SlidingLayout(this.context);
        root.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundGray));
        frameLayout.addView(root, LayoutHelper.createFrameWithActionBar(-1, -1));
        RecyclerListView recyclerListView = new RecyclerListView(this.context);
        this.listView = recyclerListView;
        recyclerListView.setClipToPadding(false);
        this.listView.setClipChildren(false);
        this.listView.setVerticalScrollBarEnabled(false);
        this.listView.setLayoutManager(new LinearLayoutManager(this.context, 1, false));
        root.addView(this.listView, LayoutHelper.createFrame(-1, -1.0f));
        View headerShadow = new View(this.context);
        headerShadow.setBackground(getResources().getDrawable(R.drawable.header_shadow).mutate());
        frameLayout.addView(headerShadow, LayoutHelper.createFrameWithActionBar(-1, 1));
        ListAdapter listAdapter = new ListAdapter(this.context);
        this.listAdapter = listAdapter;
        this.listView.setAdapter(listAdapter);
        this.listView.setOnItemClickListener(new RecyclerListView.OnItemClickListener() { // from class: im.uecahxyvjd.ui.fragments.DiscoveryFragment$$ExternalSyntheticLambda2
            @Override // im.uecahxyvjd.ui.components.RecyclerListView.OnItemClickListener
            public final void onItemClick(View view, int i) {
                DiscoveryFragment.this.m2949x9b519aa7(view, i);
            }
        });
        return this.fragmentView;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onCreateView$0$im-uecahxyvjd-ui-fragments-DiscoveryFragment  reason: not valid java name */
    public /* synthetic */ void m2949x9b519aa7(View view, int position) {
        TLRPC2.TL_DiscoveryPageSetting_SM s;
        Activity activity;
        if (position == this.nearbyRow && this.hasGps) {
            if (Build.VERSION.SDK_INT >= 23 && (activity = getParentActivity()) != null && activity.checkSelfPermission(PermissionUtils.PERMISSION_ACCESS_COARSE_LOCATION) != 0) {
                presentFragment(new ActionIntroActivity(1));
                return;
            }
            boolean enabled = true;
            if (Build.VERSION.SDK_INT >= 28) {
                LocationManager lm = (LocationManager) ApplicationLoader.applicationContext.getSystemService("location");
                enabled = lm.isLocationEnabled();
            } else {
                try {
                    int mode = Settings.Secure.getInt(ApplicationLoader.applicationContext.getContentResolver(), "location_mode", 0);
                    enabled = mode != 0;
                } catch (Throwable e) {
                    FileLog.e(e);
                }
                presentFragment(new NearPersonAndGroupActivity());
            }
            if (!enabled) {
                presentFragment(new ActionIntroActivity(4));
            } else {
                presentFragment(new NearPersonAndGroupActivity());
            }
        } else if (position != this.miniProgramRow) {
            if (position == this.friendsHubRow) {
                presentFragment(new FriendsCircleActivity());
                NotificationCenter.getGlobalInstance().postNotificationName(NotificationCenter.userFriendsCircleUpdate, new Object[0]);
            } else if (position != this.gameCenterRow) {
                if (position == this.scanRow) {
                    presentFragment(new QrScanActivity());
                } else if (position == this.album) {
                    presentFragment(new FcAlbumActivity());
                } else if (position == this.recommendChannel) {
                    if (getUserConfig().isClientActivated()) {
                        presentFragment(new HotGroupRecommendActivity());
                    }
                } else {
                    int i = this.extraDataStartRow;
                    if (i != -1 && position > i && position < this.extraDataEndRow) {
                        int cpo = (position - i) - 1;
                        Delegate delegate = this.delegate;
                        if (delegate == null || delegate.getDiscoveryPageData() == null || cpo < 0 || cpo >= this.delegate.getDiscoveryPageData().getS().size() || (s = this.delegate.getDiscoveryPageData().getS().get(cpo)) == null) {
                            return;
                        }
                        if (TextUtils.isEmpty(s.getUrl())) {
                            getExtraDataLoginUrl(s);
                        } else if (!TextUtils.isEmpty(s.getUrl()) && (s.getUrl().startsWith("http:") || s.getUrl().startsWith("https:"))) {
                            presentFragment(DiscoveryJumpToPage.toPage(s.getTitle(), s.getUrl()));
                        } else {
                            ToastUtils.show((int) R.string.CancelLinkExpired);
                        }
                    } else if (!(view instanceof ShadowSectionCell)) {
                        ToastUtils.show((int) R.string.NotSupport);
                    }
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    @Override // im.uecahxyvjd.ui.fragments.BaseFmts
    public void lazyLoadData() {
        super.lazyLoadData();
    }

    private void updateRow() {
        this.rowCount = 0;
        this.rowCount = 0 + 1;
        this.bannerStartRow = 0;
        Delegate delegate = this.delegate;
        if (delegate != null && delegate.getDiscoveryPageData() != null && this.delegate.getDiscoveryPageData().getG().size() > 0) {
            int i = this.rowCount;
            int i2 = i + 1;
            this.rowCount = i2;
            this.bannerRow = i;
            this.rowCount = i2 + 1;
            this.bannerEndRow = i2;
        }
        int i3 = this.rowCount;
        this.rowCount = i3 + 1;
        this.scanRow = i3;
        Delegate delegate2 = this.delegate;
        if (delegate2 != null && delegate2.getDiscoveryPageData() != null && this.delegate.getDiscoveryPageData().getS().size() > 0) {
            int i4 = this.rowCount;
            int i5 = i4 + 1;
            this.rowCount = i5;
            this.extraDataStartRow = i4;
            int size = i5 + this.delegate.getDiscoveryPageData().getS().size();
            this.rowCount = size;
            this.rowCount = size + 1;
            this.extraDataEndRow = size;
        }
        int i6 = this.rowCount;
        this.rowCount = i6 + 1;
        this.lastSectionRow = i6;
        ListAdapter listAdapter = this.listAdapter;
        if (listAdapter != null) {
            listAdapter.notifyDataSetChanged();
        }
    }

    private void getExtraDataLoginUrl(final TLRPC2.TL_DiscoveryPageSetting_SM s) {
        if (s == null || TextUtils.isEmpty(s.getTitle()) || this.extraDataReqToken != 0) {
            return;
        }
        TLRPC2.TL_GetLoginUrl req = new TLRPC2.TL_GetLoginUrl();
        req.app_code = s.getTitle();
        this.extraDataReqToken = getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.fragments.DiscoveryFragment$$ExternalSyntheticLambda0
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                DiscoveryFragment.this.m2948x4d6bbc1b(s, tLObject, tL_error);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getExtraDataLoginUrl$2$im-uecahxyvjd-ui-fragments-DiscoveryFragment  reason: not valid java name */
    public /* synthetic */ void m2948x4d6bbc1b(final TLRPC2.TL_DiscoveryPageSetting_SM s, final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.fragments.DiscoveryFragment$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                DiscoveryFragment.this.m2947x93f42e7c(error, response, s);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$getExtraDataLoginUrl$1$im-uecahxyvjd-ui-fragments-DiscoveryFragment  reason: not valid java name */
    public /* synthetic */ void m2947x93f42e7c(TLRPC.TL_error error, TLObject response, TLRPC2.TL_DiscoveryPageSetting_SM s) {
        if (error == null && (response instanceof TLRPC2.TL_LoginUrlInfo)) {
            TLRPC2.TL_LoginUrlInfo res = (TLRPC2.TL_LoginUrlInfo) response;
            String url = res.url;
            if (!TextUtils.isEmpty(url) && (url.startsWith("http:") || url.startsWith("https:"))) {
                presentFragment(DiscoveryJumpToPage.toPage(s.getTitle(), url));
            } else {
                ToastUtils.show((int) R.string.CancelLinkExpired);
            }
        } else {
            ToastUtils.show((int) R.string.FailedToGetLink);
            FileLog.e("DiscoveryFragment getExtraDataLoginUrl error: " + (error != null ? "errCode:" + error.code + ", errText:" + error.text : "error is null"));
        }
        this.extraDataReqToken = 0;
    }

    public void setDelegate(Delegate delegate) {
        this.delegate = delegate;
    }

    @Override // androidx.fragment.app.Fragment
    public void onResume() {
        super.onResume();
    }

    @Override // im.uecahxyvjd.ui.fragments.BaseFmts, androidx.fragment.app.Fragment
    public void onDestroy() {
        RxHelper.getInstance().m20lambda$sendSimpleRequest$0$combjzcommnetutilsRxHelper(getClass().getSimpleName());
        if (this.extraDataReqToken != 0) {
            getConnectionsManager().cancelRequest(this.extraDataReqToken, false);
            this.extraDataReqToken = 0;
        }
        super.onDestroy();
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: classes10.dex */
    public class ListAdapter extends RecyclerListView.SelectionAdapter {
        private Context mContext;

        public ListAdapter(Context context) {
            this.mContext = context;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemCount() {
            return DiscoveryFragment.this.rowCount;
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            switch (holder.getItemViewType()) {
                case 0:
                    if (position != DiscoveryFragment.this.friendsHubRow) {
                        if (position != DiscoveryFragment.this.scanRow) {
                            if (position != DiscoveryFragment.this.nearbyRow) {
                                if (position != DiscoveryFragment.this.miniProgramRow) {
                                    if (position != DiscoveryFragment.this.gameCenterRow) {
                                        if (position != DiscoveryFragment.this.album) {
                                            if (position != DiscoveryFragment.this.recommendChannel) {
                                                if (position > DiscoveryFragment.this.extraDataStartRow && position < DiscoveryFragment.this.extraDataEndRow) {
                                                    int cpo = (position - DiscoveryFragment.this.extraDataStartRow) - 1;
                                                    if (DiscoveryFragment.this.delegate != null && DiscoveryFragment.this.delegate.getDiscoveryPageData() != null && cpo >= 0 && cpo < DiscoveryFragment.this.delegate.getDiscoveryPageData().getS().size()) {
                                                        try {
                                                            TLRPC2.TL_DiscoveryPageSetting_SM item = DiscoveryFragment.this.delegate.getDiscoveryPageData().getS().get(cpo);
                                                            ((IndexTextCell) holder.itemView).setTextAndIcon(item.getTitle(), item.getLogo(), AndroidUtilities.dp(7.0f), 0, cpo == DiscoveryFragment.this.extraDataEndRow - 1);
                                                            return;
                                                        } catch (Exception e) {
                                                            return;
                                                        }
                                                    }
                                                    return;
                                                }
                                                return;
                                            }
                                            ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("HotChannelRecommend", R.string.HotChannelRecommend), R.mipmap.ic_fire, R.mipmap.icon_arrow_right, false);
                                            return;
                                        }
                                        ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("MyAlbum", R.string.MyAlbum), R.drawable.fmt_discoveryv2_album, R.mipmap.icon_arrow_right, false);
                                        return;
                                    }
                                    ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("GameCenter", R.string.GameCenter), R.mipmap.fmt_discovery_games, R.mipmap.icon_arrow_right, false);
                                    return;
                                }
                                ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("MiniProgram", R.string.MiniProgram), R.mipmap.fmt_discovery_mini_program, R.mipmap.icon_arrow_right, false);
                                return;
                            }
                            ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("PeopleNearby", R.string.PeopleNearby), R.drawable.fmt_discoveryv2_nearby, R.mipmap.icon_arrow_right, false);
                            return;
                        }
                        ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("Scan", R.string.Scan), R.drawable.fmt_discoveryv2_scan, R.mipmap.icon_arrow_right, false);
                        return;
                    }
                    ((IndexTextCell) holder.itemView).setTextAndIcon(LocaleController.getString("FriendHub", R.string.FriendHub), R.drawable.fmt_discoveryv2_friends_hub, R.mipmap.icon_arrow_right, false);
                    return;
                case 1:
                    View view = new View(this.mContext);
                    RecyclerView.LayoutParams layoutParams = new RecyclerView.LayoutParams(-1, AndroidUtilities.dp(10.0f));
                    view.setLayoutParams(layoutParams);
                    return;
                case 2:
                    View view2 = holder.itemView;
                    Banner<TLRPC2.TL_DiscoveryPageSetting_GM, BannerAdapter<TLRPC2.TL_DiscoveryPageSetting_GM, PageHolder>> banner = (Banner) view2;
                    BannerAdapter<TLRPC2.TL_DiscoveryPageSetting_GM, PageHolder> adapter = banner.getAdapter();
                    if (banner.getAdapter() == null) {
                        adapter = new BannerAdapter<TLRPC2.TL_DiscoveryPageSetting_GM, PageHolder>(null) { // from class: im.uecahxyvjd.ui.fragments.DiscoveryFragment.ListAdapter.1
                            @Override // im.uecahxyvjd.ui.components.banner.holder.IViewHolder
                            public PageHolder onCreateHolder(ViewGroup parent, int viewType) {
                                ImageView iv = new ImageView(DiscoveryFragment.this.getParentActivity());
                                iv.setScaleType(ImageView.ScaleType.FIT_XY);
                                RecyclerView.LayoutParams lp = new RecyclerView.LayoutParams(-1, -1);
                                iv.setLayoutParams(lp);
                                return new PageHolder(iv);
                            }

                            @Override // im.uecahxyvjd.ui.components.banner.holder.IViewHolder
                            public void onBindView(PageHolder holder2, TLRPC2.TL_DiscoveryPageSetting_GM data, int position2, int size) {
                                ImageView iv = (ImageView) holder2.itemView;
                                GlideUtils.getInstance().load(data.getPic(), iv.getContext(), iv, R.mipmap.banner_discovery1);
                            }
                        };
                        banner.setAdapter(adapter);
                    }
                    banner.setOnBannerListener(new OnBannerListener() { // from class: im.uecahxyvjd.ui.fragments.DiscoveryFragment$ListAdapter$$ExternalSyntheticLambda0
                        @Override // im.uecahxyvjd.ui.components.banner.listener.OnBannerListener
                        public final void OnBannerClick(Object obj, int i) {
                            DiscoveryFragment.ListAdapter.this.m2950xcf494507((TLRPC2.TL_DiscoveryPageSetting_GM) obj, i);
                        }
                    });
                    if (DiscoveryFragment.this.delegate != null && DiscoveryFragment.this.delegate.getDiscoveryPageData() != null) {
                        adapter.setDatas(DiscoveryFragment.this.delegate.getDiscoveryPageData().getG());
                    }
                    adapter.notifyDataSetChanged();
                    return;
                case 3:
                    View view3 = new View(this.mContext);
                    RecyclerView.LayoutParams layoutParams2 = new RecyclerView.LayoutParams(-1, AndroidUtilities.dp(15.0f));
                    view3.setLayoutParams(layoutParams2);
                    return;
                default:
                    return;
            }
        }

        /* JADX INFO: Access modifiers changed from: package-private */
        /* renamed from: lambda$onBindViewHolder$0$im-uecahxyvjd-ui-fragments-DiscoveryFragment$ListAdapter  reason: not valid java name */
        public /* synthetic */ void m2950xcf494507(TLRPC2.TL_DiscoveryPageSetting_GM data, int position1) {
            DiscoveryFragment.this.presentFragment(new WebviewActivity(data.getUrl(), (String) null));
        }

        @Override // im.uecahxyvjd.ui.components.RecyclerListView.SelectionAdapter
        public boolean isEnabled(RecyclerView.ViewHolder holder) {
            int position = holder.getAdapterPosition();
            return position == DiscoveryFragment.this.friendsHubRow || position == DiscoveryFragment.this.scanRow || position == DiscoveryFragment.this.nearbyRow || position == DiscoveryFragment.this.gameCenterRow || position == DiscoveryFragment.this.miniProgramRow || position == DiscoveryFragment.this.album || position == DiscoveryFragment.this.recommendChannel || (DiscoveryFragment.this.extraDataStartRow > 0 && position > DiscoveryFragment.this.extraDataStartRow && position < DiscoveryFragment.this.extraDataEndRow);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view;
            if (viewType == 1 || viewType == 3) {
                View view2 = viewType == 1 ? new ShadowSectionCell(this.mContext) : new ShadowSectionCell(this.mContext, 15);
                view2.setLayoutParams(new RecyclerView.LayoutParams(-1, -2));
                view = view2;
            } else if (viewType == 2) {
                Banner banner = new Banner(DiscoveryFragment.this.getContext());
                banner.setBannerRound(AndroidUtilities.dp(10.0f));
                banner.setLoopTime(6000L);
                int padding = AndroidUtilities.dp(2.0f);
                RectangleIndicator indicator = new RectangleIndicator(DiscoveryFragment.this.getContext());
                indicator.setPadding(padding, padding, padding, padding);
                indicator.setBackground(Theme.createRoundRectDrawable(AndroidUtilities.dp(5.0f), AndroidUtilities.alphaColor(0.5f, -1)));
                banner.setIndicator(indicator).setIndicatorNormalWidth(AndroidUtilities.dp(6.0f)).setIndicatorHeight(AndroidUtilities.dp(6.0f)).setIndicatorSpace(AndroidUtilities.dp(5.0f)).setIndicatorSelectedWidth(AndroidUtilities.dp(15.0f)).setIndicatorRadius(AndroidUtilities.dp(6.0f)).setIndicatorSelectedColor(Theme.getColor(Theme.key_windowBackgroundWhiteBlueButton)).setIndicatorNormalColor(-1);
                int h = (int) ((ScreenUtils.getScreenWidth() - AndroidUtilities.dp(30.0f)) / 3.45f);
                RecyclerView.LayoutParams lp = new RecyclerView.LayoutParams(-1, h);
                lp.leftMargin = AndroidUtilities.dp(15.0f);
                lp.rightMargin = AndroidUtilities.dp(15.0f);
                banner.setLayoutParams(lp);
                banner.setClipToPadding(false);
                banner.setClipChildren(false);
                view = banner;
            } else {
                View view3 = new IndexTextCell(this.mContext);
                view3.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                view3.setLayoutParams(new RecyclerView.LayoutParams(-1, -2));
                view = view3;
            }
            return new RecyclerListView.Holder(view);
        }

        @Override // androidx.recyclerview.widget.RecyclerView.Adapter
        public int getItemViewType(int position) {
            if (position != DiscoveryFragment.this.friendsHubEmptyRow && position != DiscoveryFragment.this.nearbyEmptyRow && position != DiscoveryFragment.this.gameCenterEmptyRow && position != DiscoveryFragment.this.lastSectionRow && position != DiscoveryFragment.this.startSectionRow && position != DiscoveryFragment.this.albumEmptyRow && position != DiscoveryFragment.this.recommendChannelEmptyRow && position != DiscoveryFragment.this.extraDataStartRow && position != DiscoveryFragment.this.extraDataEndRow) {
                if (position != DiscoveryFragment.this.bannerRow) {
                    if (position == DiscoveryFragment.this.bannerStartRow || position == DiscoveryFragment.this.bannerEndRow) {
                        return 3;
                    }
                    return 0;
                }
                return 2;
            }
            return 1;
        }
    }
}
