package im.uecahxyvjd.ui.fragments.adapter;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import androidx.recyclerview.widget.RecyclerView;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ContactsController;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.cells.EmptyCell;
import im.uecahxyvjd.ui.cells.LetterSectionCell;
import im.uecahxyvjd.ui.components.RecyclerListView;
import im.uecahxyvjd.ui.hcells.ContactUserCell;
import im.uecahxyvjd.ui.hviews.MryTextView;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
/* loaded from: classes13.dex */
public class FmtContactsAdapter extends RecyclerListView.SectionsAdapter {
    private SparseArray<?> checkedMap;
    private int classGuid;
    private FmtContactsAdapterDelegate delegate;
    private boolean disableSections;
    private boolean hasGps;
    private OnContactHeaderItemClickListener headerListener;
    private SparseArray<TLRPC.User> ignoreUsers;
    private boolean isAdmin;
    private boolean isChannel;
    private Context mContext;
    private boolean needPhonebook;
    private ArrayList<TLRPC.Contact> onlineContacts;
    private int onlyUsers;
    private boolean scrolling;
    private int sortType;
    private int currentAccount = UserConfig.selectedAccount;
    private HashMap<Integer, Integer> userPositionMap = new HashMap<>();

    /* loaded from: classes13.dex */
    public interface FmtContactsAdapterDelegate {
        void onDeleteItem(int i);
    }

    /* loaded from: classes13.dex */
    public interface OnContactHeaderItemClickListener {
        void onItemClick(View view);
    }

    public FmtContactsAdapter(Context mContext, int onlyUsersType, boolean needPhonebook, SparseArray<TLRPC.User> ignores, int flags, boolean hasGps) {
        this.mContext = mContext;
        this.onlyUsers = onlyUsersType;
        this.needPhonebook = needPhonebook;
        this.ignoreUsers = ignores;
        this.isAdmin = flags != 0;
        this.isChannel = flags == 2;
        this.hasGps = hasGps;
    }

    public void setDisableSections(boolean value) {
        this.disableSections = value;
    }

    public void setSortType(int value) {
        this.sortType = value;
        if (value == 2) {
            if (this.onlineContacts == null) {
                this.onlineContacts = new ArrayList<>(ContactsController.getInstance(this.currentAccount).contacts);
                int selfId = UserConfig.getInstance(this.currentAccount).clientUserId;
                int i = 0;
                int len = this.onlineContacts.size();
                while (true) {
                    if (i >= len) {
                        break;
                    } else if (this.onlineContacts.get(i).user_id != selfId) {
                        i++;
                    } else {
                        this.onlineContacts.remove(i);
                        break;
                    }
                }
            }
            sortOnlineContacts();
            return;
        }
        notifyDataSetChanged();
    }

    public int getSectionForChar(char section) {
        ArrayList<String> sortedUsersSectionsArray = ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        for (int i = 0; i < getSectionCount() - 1; i++) {
            String sortStr = sortedUsersSectionsArray.get(i);
            char firstChar = sortStr.toUpperCase().charAt(0);
            if (firstChar == section) {
                return i + 1;
            }
        }
        return -1;
    }

    public int getPositionForSection(int section) {
        if (section == -1) {
            return -1;
        }
        int positionStart = 0;
        int N = getSectionCount();
        for (int i = 0; i < N; i++) {
            if (i >= section) {
                return positionStart;
            }
            int count = getCountForSection(i);
            positionStart += count;
        }
        return -1;
    }

    public void sortOnlineContacts() {
        if (this.onlineContacts == null) {
            return;
        }
        try {
            final int currentTime = ConnectionsManager.getInstance(this.currentAccount).getCurrentTime();
            final MessagesController messagesController = MessagesController.getInstance(this.currentAccount);
            Collections.sort(this.onlineContacts, new Comparator() { // from class: im.uecahxyvjd.ui.fragments.adapter.FmtContactsAdapter$$ExternalSyntheticLambda4
                @Override // java.util.Comparator
                public final int compare(Object obj, Object obj2) {
                    return FmtContactsAdapter.lambda$sortOnlineContacts$0(MessagesController.this, currentTime, (TLRPC.Contact) obj, (TLRPC.Contact) obj2);
                }
            });
            notifyDataSetChanged();
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static /* synthetic */ int lambda$sortOnlineContacts$0(MessagesController messagesController, int currentTime, TLRPC.Contact objPre, TLRPC.Contact objNext) {
        TLRPC.User userPre = messagesController.getUser(Integer.valueOf(objPre.user_id));
        TLRPC.User userNext = messagesController.getUser(Integer.valueOf(objNext.user_id));
        int status1 = 0;
        int status2 = 0;
        if (userPre != null) {
            if (userPre.self) {
                status1 = currentTime + 50000;
            } else if (userPre.status != null) {
                status1 = userPre.status.expires;
            }
        }
        if (userNext != null) {
            if (userNext.self) {
                status2 = currentTime + 50000;
            } else if (userNext.status != null) {
                status2 = userNext.status.expires;
            }
        }
        if (status1 > 0 && status2 > 0) {
            if (status1 > status2) {
                return 1;
            }
            return status1 < status2 ? -1 : 0;
        } else if (status1 < 0 && status2 < 0) {
            if (status1 > status2) {
                return 1;
            }
            return status1 < status2 ? -1 : 0;
        } else if ((status1 >= 0 || status2 <= 0) && (status1 != 0 || status2 == 0)) {
            return ((status2 >= 0 || status1 <= 0) && (status2 != 0 || status1 == 0)) ? 0 : 1;
        } else {
            return -1;
        }
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public Object getItem(int section, int position) {
        HashMap<String, ArrayList<TLRPC.Contact>> usersSectionsDict = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict : ContactsController.getInstance(this.currentAccount).usersSectionsDict;
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        if (this.onlyUsers != 0 && !this.isAdmin) {
            if (section < sortedUsersSectionsArray.size()) {
                ArrayList<TLRPC.Contact> arr = usersSectionsDict.get(sortedUsersSectionsArray.get(section));
                if (position < arr.size()) {
                    return MessagesController.getInstance(this.currentAccount).getUser(Integer.valueOf(arr.get(position).user_id));
                }
            }
            return null;
        } else if (section == 0) {
            return null;
        } else {
            if (this.sortType == 2) {
                if (section == 1) {
                    if (position < this.onlineContacts.size()) {
                        return MessagesController.getInstance(this.currentAccount).getUser(Integer.valueOf(this.onlineContacts.get(position).user_id));
                    }
                    return null;
                }
            } else if (section - 1 < sortedUsersSectionsArray.size()) {
                ArrayList<TLRPC.Contact> arr2 = usersSectionsDict.get(sortedUsersSectionsArray.get(section - 1));
                if (position < arr2.size()) {
                    return MessagesController.getInstance(this.currentAccount).getUser(Integer.valueOf(arr2.get(position).user_id));
                }
                return null;
            }
            if (this.needPhonebook) {
                return ContactsController.getInstance(this.currentAccount).phoneBookContacts.get(position);
            }
            return null;
        }
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.FastScrollAdapter
    public String getLetter(int position) {
        if (this.sortType == 2) {
            return null;
        }
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        int section = getSectionForPosition(position);
        if (section == -1) {
            section = sortedUsersSectionsArray.size() - 1;
        }
        if (this.onlyUsers != 0 && !this.isAdmin) {
            if (section >= 0 && section < sortedUsersSectionsArray.size()) {
                return sortedUsersSectionsArray.get(section);
            }
        } else if (section > 0 && section <= sortedUsersSectionsArray.size()) {
            return sortedUsersSectionsArray.get(section - 1);
        }
        return null;
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.FastScrollAdapter
    public int getPositionForScrollProgress(float progress) {
        return (int) (getItemCount() * progress);
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public int getSectionCount() {
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        int count = sortedUsersSectionsArray.size();
        if (this.onlyUsers == 0) {
            count++;
        }
        if (this.isAdmin) {
            return count + 1;
        }
        return count;
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public int getCountForSection(int section) {
        ArrayList<TLRPC.Contact> arr;
        HashMap<String, ArrayList<TLRPC.Contact>> usersSectionsDict = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict : ContactsController.getInstance(this.currentAccount).usersSectionsDict;
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        if (section == 0) {
            return 2;
        }
        if (section - 1 < sortedUsersSectionsArray.size() && (arr = usersSectionsDict.get(sortedUsersSectionsArray.get(section - 1))) != null) {
            int count = arr.size();
            if (section - 1 == sortedUsersSectionsArray.size() - 1) {
                return count + 1;
            }
            return count;
        } else if (this.needPhonebook) {
            return ContactsController.getInstance(this.currentAccount).phoneBookContacts.size();
        } else {
            return 0;
        }
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public boolean isEnabled(int section, int row) {
        HashMap<String, ArrayList<TLRPC.Contact>> usersSectionsDict = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict : ContactsController.getInstance(this.currentAccount).usersSectionsDict;
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        if (this.onlyUsers != 0 && !this.isAdmin) {
            ArrayList<TLRPC.Contact> arr = usersSectionsDict.get(sortedUsersSectionsArray.get(section));
            return row < arr.size();
        } else if (section == 0) {
            return false;
        } else {
            if (this.sortType == 2) {
                return section != 1 || row < this.onlineContacts.size();
            } else if (section - 1 < sortedUsersSectionsArray.size()) {
                ArrayList<TLRPC.Contact> arr2 = usersSectionsDict.get(sortedUsersSectionsArray.get(section - 1));
                return row < arr2.size();
            }
            return true;
        }
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public int getItemViewType(int section, int position) {
        HashMap<String, ArrayList<TLRPC.Contact>> usersSectionsDict = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict : ContactsController.getInstance(this.currentAccount).usersSectionsDict;
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        if (section == 0) {
            return position == 0 ? 1 : 2;
        } else if (section - 1 >= sortedUsersSectionsArray.size()) {
            return 1;
        } else {
            ArrayList<TLRPC.Contact> arr = usersSectionsDict.get(sortedUsersSectionsArray.get(section - 1));
            return section + (-1) == sortedUsersSectionsArray.size() - 1 ? position < arr.size() ? 0 : 5 : position < arr.size() ? 0 : 4;
        }
    }

    @Override // androidx.recyclerview.widget.RecyclerView.Adapter
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view;
        switch (viewType) {
            case 0:
                View view2 = LayoutInflater.from(this.mContext).inflate(R.layout.item_contacts_layout, (ViewGroup) null);
                view2.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                view = view2;
                break;
            case 1:
                View view3 = LayoutInflater.from(this.mContext).inflate(R.layout.item_contacts_header, parent, false);
                view3.setMinimumHeight(AndroidUtilities.dp(105.0f));
                view3.setBackground(Theme.createRoundRectDrawable(AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), 0.0f, 0.0f, Theme.getColor(Theme.key_windowBackgroundWhite)));
                view = view3;
                break;
            case 2:
                view = new EmptyCell(this.mContext, AndroidUtilities.dp(10.0f));
                view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundGray));
                break;
            case 3:
                view = new EmptyCell(this.mContext, AndroidUtilities.dp(46.0f));
                view.setBackgroundColor(16776960);
                break;
            case 4:
            default:
                view = new View(this.mContext);
                view.setBackgroundColor(16760097);
                view.setMinimumHeight(AndroidUtilities.dp(48.0f));
                view.setPadding(AndroidUtilities.dp(LocaleController.isRTL ? 28.0f : 72.0f), AndroidUtilities.dp(8.0f), AndroidUtilities.dp(LocaleController.isRTL ? 72.0f : 28.0f), AndroidUtilities.dp(8.0f));
                break;
            case 5:
                view = new MryTextView(this.mContext);
                view.setBackgroundColor(0);
                view.setMinimumHeight(AndroidUtilities.dp(70.0f));
                ((MryTextView) view).setGravity(17);
                ((MryTextView) view).setTextSize(13.0f);
                break;
        }
        view.setLayoutParams(new RecyclerView.LayoutParams(-1, -2));
        return new RecyclerListView.Holder(view);
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public void onBindViewHolder(int section, int position, RecyclerView.ViewHolder holder) {
        ArrayList<TLRPC.Contact> arr;
        switch (holder.getItemViewType()) {
            case 0:
                ContactUserCell userCell = (ContactUserCell) holder.itemView.findViewById(R.id.contactUserCell);
                userCell.setAvatarPadding((this.sortType == 2 || this.disableSections) ? 6 : 58);
                if (this.sortType == 2) {
                    arr = this.onlineContacts;
                } else {
                    HashMap<String, ArrayList<TLRPC.Contact>> usersSectionsDict = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict : ContactsController.getInstance(this.currentAccount).usersSectionsDict;
                    ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
                    arr = usersSectionsDict.get(sortedUsersSectionsArray.get(section - ((this.onlyUsers == 0 || this.isAdmin) ? 1 : 0)));
                }
                TLRPC.User user = MessagesController.getInstance(this.currentAccount).getUser(Integer.valueOf(arr.get(position).user_id));
                if (MessagesController.getInstance(this.currentAccount).getUserFull(user.id) == null) {
                    MessagesController.getInstance(this.currentAccount).loadUserInfo(user, true, this.classGuid);
                } else {
                    userCell.setUserFull(MessagesController.getInstance(this.currentAccount).getUserFull(user.id));
                }
                if (section == getSectionCount() - 1 && position == arr.size() - 1) {
                    userCell.setData(user, null, null, 0);
                    holder.itemView.setBackground(Theme.createRoundRectDrawable(0.0f, 0.0f, AndroidUtilities.dp(5.0f), AndroidUtilities.dp(5.0f), Theme.getColor(Theme.key_windowBackgroundWhite)));
                } else {
                    userCell.setData(user, null, null, 0, true);
                }
                SparseArray<?> sparseArray = this.checkedMap;
                if (sparseArray != null) {
                    userCell.setChecked(sparseArray.indexOfKey(user.id) >= 0, !this.scrolling);
                }
                SparseArray<TLRPC.User> sparseArray2 = this.ignoreUsers;
                if (sparseArray2 != null) {
                    if (sparseArray2.indexOfKey(user.id) >= 0) {
                        userCell.setAlpha(0.5f);
                    } else {
                        userCell.setAlpha(1.0f);
                    }
                }
                this.userPositionMap.put(Integer.valueOf(user.id), Integer.valueOf(getPositionForSection(section) + position));
                return;
            case 1:
                if (section == 0 && this.needPhonebook) {
                    LinearLayout llNewFriend = (LinearLayout) holder.itemView.findViewById(R.id.ll_new_friend);
                    LinearLayout llMyGrouping = (LinearLayout) holder.itemView.findViewById(R.id.ll_my_grouping);
                    LinearLayout llMyGroup = (LinearLayout) holder.itemView.findViewById(R.id.ll_my_group);
                    LinearLayout llMyChannel = (LinearLayout) holder.itemView.findViewById(R.id.ll_my_channel);
                    ImageView ivUnread = (ImageView) holder.itemView.findViewById(R.id.iv_unread);
                    SharedPreferences sharedPreferences = MessagesController.getMainSettings(this.currentAccount);
                    int value = sharedPreferences.getInt("contacts_apply_count", 0);
                    ivUnread.setVisibility(value <= 0 ? 8 : 0);
                    llNewFriend.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.fragments.adapter.FmtContactsAdapter$$ExternalSyntheticLambda0
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            FmtContactsAdapter.this.m2972x993fdb14(view);
                        }
                    });
                    llMyGrouping.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.fragments.adapter.FmtContactsAdapter$$ExternalSyntheticLambda1
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            FmtContactsAdapter.this.m2973xc2943055(view);
                        }
                    });
                    llMyGroup.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.fragments.adapter.FmtContactsAdapter$$ExternalSyntheticLambda2
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            FmtContactsAdapter.this.m2974xebe88596(view);
                        }
                    });
                    llMyChannel.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.fragments.adapter.FmtContactsAdapter$$ExternalSyntheticLambda3
                        @Override // android.view.View.OnClickListener
                        public final void onClick(View view) {
                            FmtContactsAdapter.this.m2975x153cdad7(view);
                        }
                    });
                    return;
                }
                return;
            case 5:
                MryTextView tvAllContractCounts = (MryTextView) holder.itemView;
                tvAllContractCounts.setTextColor(Theme.key_windowBackgroundWhiteGrayText8);
                HashMap<String, ArrayList<TLRPC.Contact>> usersSectionsDict2 = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict : ContactsController.getInstance(this.currentAccount).usersSectionsDict;
                int allCounts = 0;
                if (usersSectionsDict2 != null) {
                    for (Map.Entry<String, ArrayList<TLRPC.Contact>> e : usersSectionsDict2.entrySet()) {
                        allCounts += e.getValue().size();
                    }
                }
                tvAllContractCounts.setText(allCounts + LocaleController.getString(R.string.CountOfContractsPeople));
                return;
            default:
                return;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$1$im-uecahxyvjd-ui-fragments-adapter-FmtContactsAdapter  reason: not valid java name */
    public /* synthetic */ void m2972x993fdb14(View v) {
        OnContactHeaderItemClickListener onContactHeaderItemClickListener = this.headerListener;
        if (onContactHeaderItemClickListener != null) {
            onContactHeaderItemClickListener.onItemClick(v);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$2$im-uecahxyvjd-ui-fragments-adapter-FmtContactsAdapter  reason: not valid java name */
    public /* synthetic */ void m2973xc2943055(View v) {
        OnContactHeaderItemClickListener onContactHeaderItemClickListener = this.headerListener;
        if (onContactHeaderItemClickListener != null) {
            onContactHeaderItemClickListener.onItemClick(v);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$3$im-uecahxyvjd-ui-fragments-adapter-FmtContactsAdapter  reason: not valid java name */
    public /* synthetic */ void m2974xebe88596(View v) {
        OnContactHeaderItemClickListener onContactHeaderItemClickListener = this.headerListener;
        if (onContactHeaderItemClickListener != null) {
            onContactHeaderItemClickListener.onItemClick(v);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$onBindViewHolder$4$im-uecahxyvjd-ui-fragments-adapter-FmtContactsAdapter  reason: not valid java name */
    public /* synthetic */ void m2975x153cdad7(View v) {
        OnContactHeaderItemClickListener onContactHeaderItemClickListener = this.headerListener;
        if (onContactHeaderItemClickListener != null) {
            onContactHeaderItemClickListener.onItemClick(v);
        }
    }

    @Override // im.uecahxyvjd.ui.components.RecyclerListView.SectionsAdapter
    public View getSectionHeaderView(int section, View view) {
        if (this.onlyUsers == 2) {
            HashMap<String, ArrayList<TLRPC.Contact>> hashMap = ContactsController.getInstance(this.currentAccount).usersMutualSectionsDict;
        } else {
            HashMap<String, ArrayList<TLRPC.Contact>> hashMap2 = ContactsController.getInstance(this.currentAccount).usersSectionsDict;
        }
        ArrayList<String> sortedUsersSectionsArray = this.onlyUsers == 2 ? ContactsController.getInstance(this.currentAccount).sortedUsersMutualSectionsArray : ContactsController.getInstance(this.currentAccount).sortedUsersSectionsArray;
        if (view == null) {
            view = new LetterSectionCell(this.mContext);
        }
        LetterSectionCell cell = (LetterSectionCell) view;
        if (this.sortType == 2 || this.disableSections) {
            cell.setLetter("");
        } else if (this.onlyUsers != 0 && !this.isAdmin) {
            if (section < sortedUsersSectionsArray.size()) {
                cell.setLetter(sortedUsersSectionsArray.get(section));
            } else {
                cell.setLetter("");
            }
        } else if (section == 0) {
            cell.setLetter("");
        } else if (section - 1 < sortedUsersSectionsArray.size()) {
            cell.setLetter(sortedUsersSectionsArray.get(section - 1));
        } else {
            cell.setLetter("");
        }
        return view;
    }

    public void setDelegate(FmtContactsAdapterDelegate delegate) {
        this.delegate = delegate;
    }

    public void setOnContactHeaderItemClickListener(OnContactHeaderItemClickListener listener) {
        this.headerListener = listener;
    }

    public void setClassGuid(int classGuid) {
        this.classGuid = classGuid;
    }

    public int getItemPosition(int userId) {
        if (this.userPositionMap.get(Integer.valueOf(userId)) != null) {
            return this.userPositionMap.get(Integer.valueOf(userId)).intValue();
        }
        return -1;
    }
}
