package im.uecahxyvjd.ui.hui.friendscircle_v1.base;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;
import androidx.core.content.FileProvider;
import androidx.exifinterface.media.ExifInterface;
import com.bjz.comm.net.base.DataListener;
import com.bjz.comm.net.bean.BResponse;
import com.bjz.comm.net.bean.FcMediaResponseBean;
import com.bjz.comm.net.bean.FcReplyBean;
import com.bjz.comm.net.mvp.contract.BaseFcContract;
import com.bjz.comm.net.mvp.presenter.FcCommonPresenter;
import com.bjz.comm.net.utils.HttpUtils;
import com.socks.library.KLog;
import im.uecahxyvjd.messenger.AccountInstance;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.ApplicationLoader;
import im.uecahxyvjd.messenger.ContactsController;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.LocationController;
import im.uecahxyvjd.messenger.MediaController;
import im.uecahxyvjd.messenger.MediaDataController;
import im.uecahxyvjd.messenger.MessagesController;
import im.uecahxyvjd.messenger.MessagesStorage;
import im.uecahxyvjd.messenger.NotificationsController;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.messenger.SendMessagesHelper;
import im.uecahxyvjd.messenger.UserConfig;
import im.uecahxyvjd.messenger.VideoEditedInfo;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.ui.LaunchActivity;
import im.uecahxyvjd.ui.PhotoAlbumPickerActivity;
import im.uecahxyvjd.ui.PhotoViewer;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.ActionBarLayout;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.hui.discovery.ActionIntroActivity;
import im.uecahxyvjd.ui.hui.discovery.NearPersonAndGroupActivity;
import im.uecahxyvjd.ui.hui.friendscircle_v1.helper.FcDBHelper;
import im.uecahxyvjd.ui.hui.friendscircle_v1.utils.TimeUtils;
import im.uecahxyvjd.ui.hui.friendscircle_v1.view.FcCommMenuDialog;
import im.uecahxyvjd.ui.hui.friendscircle_v1.view.toast.FcToastUtils;
import im.uecahxyvjd.ui.hui.visualcall.PermissionUtils;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
/* loaded from: classes10.dex */
public abstract class BaseFcFragment extends LazyLoadFragment implements BaseFcContract.IFcCommView {
    protected ActionBar actionBar;
    protected Bundle arguments;
    protected int classGuid;
    protected int currentAccount;
    public String currentPicturePath;
    protected Drawable defaultActionBarBackgroundDrawable;
    private FcCommMenuDialog dialogCommonList;
    protected View fragmentView;
    protected boolean inPreviewMode;
    private FcCommonPresenter mCommPresenter;
    protected Context mContext;
    protected ActionBarLayout parentLayout;
    protected Dialog visibleDialog;

    protected abstract int getLayoutRes();

    protected abstract void initData();

    protected abstract void initView();

    public BaseFcFragment() {
        this(null);
    }

    public BaseFcFragment(Bundle args) {
        this.currentAccount = UserConfig.selectedAccount;
        this.classGuid = ConnectionsManager.generateClassGuid();
        this.arguments = args;
        this.defaultActionBarBackgroundDrawable = new ColorDrawable(Theme.getColor(Theme.key_actionBarDefault));
    }

    @Override // androidx.fragment.app.Fragment
    public void onAttach(Context context) {
        super.onAttach(context);
        this.mContext = context;
    }

    @Override // androidx.fragment.app.Fragment
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        this.fragmentView = LayoutInflater.from(this.mContext).inflate(getLayoutRes(), (ViewGroup) null);
        initView();
        return this.fragmentView;
    }

    @Override // im.uecahxyvjd.ui.hui.friendscircle_v1.base.LazyLoadFragment, androidx.fragment.app.Fragment
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        this.parentLayout = getParentLayout();
        this.mCommPresenter = new FcCommonPresenter(this);
    }

    @Override // im.uecahxyvjd.ui.hui.friendscircle_v1.base.LazyLoadFragment
    protected void loadData() {
        initData();
    }

    @Override // im.uecahxyvjd.ui.hui.friendscircle_v1.base.LazyLoadFragment, androidx.fragment.app.Fragment
    public void onDestroy() {
        getConnectionsManager().cancelRequestsForGuid(this.classGuid);
        super.onDestroy();
        FcCommonPresenter fcCommonPresenter = this.mCommPresenter;
        if (fcCommonPresenter != null) {
            fcCommonPresenter.unSubscribeTask();
        }
    }

    protected ActionBar createActionBar() {
        ActionBar actionBar = new ActionBar(this.mContext);
        this.actionBar = actionBar;
        actionBar.setBackground(this.defaultActionBarBackgroundDrawable);
        this.actionBar.setItemsBackgroundColor(Theme.getColor(Theme.key_actionBarDefaultSelector), false);
        this.actionBar.setItemsBackgroundColor(Theme.getColor(Theme.key_actionBarActionModeDefaultSelector), true);
        this.actionBar.setItemsColor(Theme.getColor(Theme.key_actionBarDefaultIcon), false);
        this.actionBar.setItemsColor(Theme.getColor(Theme.key_actionBarActionModeDefaultIcon), true);
        this.actionBar.setCastShadows(false);
        if (this.inPreviewMode) {
            this.actionBar.setOccupyStatusBar(false);
        }
        ActionBarLayout actionBarLayout = this.parentLayout;
        if (actionBarLayout != null) {
            actionBarLayout.setBackgroundColor(Theme.getColor(Theme.key_actionBarDefault));
        }
        return this.actionBar;
    }

    private void convertFontToBold(ViewGroup viewGroup) {
        if (viewGroup != null) {
            int count = viewGroup.getChildCount();
            for (int i = 0; i < count; i++) {
                View view = viewGroup.getChildAt(i);
                if (view instanceof ViewGroup) {
                    convertFontToBold((ViewGroup) view);
                } else if (this.mContext.getResources().getString(R.string.Typeface_Bold).equals(view.getTag())) {
                    if (view instanceof TextView) {
                        ((TextView) view).setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
                    } else if (view instanceof EditText) {
                        ((EditText) view).setTypeface(AndroidUtilities.getTypeface("fonts/rmedium.ttf"));
                    }
                }
            }
        }
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    private void setBackground(ViewGroup viewGroup) {
        char c;
        int count = viewGroup.getChildCount();
        for (int i = 0; i < count; i++) {
            View view = viewGroup.getChildAt(i);
            Object tag = view.getTag();
            if (tag != null) {
                String key = (String) tag;
                switch (key.hashCode()) {
                    case -1397026623:
                        if (key.equals(Theme.key_windowBackgroundGray)) {
                            c = 1;
                            break;
                        }
                        c = 65535;
                        break;
                    case -343666293:
                        if (key.equals(Theme.key_windowBackgroundWhite)) {
                            c = 0;
                            break;
                        }
                        c = 65535;
                        break;
                    case 1658529622:
                        if (key.equals(Theme.key_walletDefaultBackground)) {
                            c = 2;
                            break;
                        }
                        c = 65535;
                        break;
                    default:
                        c = 65535;
                        break;
                }
                switch (c) {
                    case 0:
                        view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundWhite));
                        break;
                    case 1:
                        view.setBackgroundColor(Theme.getColor(Theme.key_windowBackgroundGray));
                        break;
                    case 2:
                        view.setBackgroundColor(Theme.getColor(Theme.key_walletDefaultBackground));
                        break;
                }
            }
            if (view instanceof ViewGroup) {
                setBackground((ViewGroup) view);
            }
        }
    }

    @Override // im.uecahxyvjd.ui.hui.friendscircle_v1.base.LazyLoadFragment, androidx.fragment.app.Fragment
    public void onPause() {
        super.onPause();
        AndroidUtilities.hideKeyboard(this.fragmentView.findFocus());
    }

    public Activity getParentActivity() {
        return getActivity();
    }

    public boolean presentFragment(BaseFragment fragment) {
        ActionBarLayout parentLayout = getParentLayout();
        this.parentLayout = parentLayout;
        return parentLayout != null && parentLayout.presentFragment(fragment);
    }

    public boolean presentFragment(BaseFragment fragment, boolean removeLast) {
        ActionBarLayout actionBarLayout = this.parentLayout;
        return actionBarLayout != null && actionBarLayout.presentFragment(fragment, removeLast);
    }

    public BaseFragment getCurrentFragment() {
        return getParentLayout().getCurrentFragment();
    }

    private ActionBarLayout getParentLayout() {
        return ((LaunchActivity) Objects.requireNonNull(getActivity())).getActionBarLayout();
    }

    public Dialog showDialog(Dialog dialog) {
        return showDialog(dialog, false, null);
    }

    public Dialog showDialog(Dialog dialog, DialogInterface.OnDismissListener onDismissListener) {
        return showDialog(dialog, false, onDismissListener);
    }

    public Dialog showDialog(Dialog dialog, boolean allowInTransition, final DialogInterface.OnDismissListener onDismissListener) {
        ActionBarLayout actionBarLayout;
        if (dialog == null || (actionBarLayout = this.parentLayout) == null || (!allowInTransition && actionBarLayout.checkTransitionAnimation())) {
            return null;
        }
        try {
            Dialog dialog2 = this.visibleDialog;
            if (dialog2 != null) {
                dialog2.dismiss();
                this.visibleDialog = null;
            }
        } catch (Exception e) {
            FileLog.e(e);
        }
        try {
            this.visibleDialog = dialog;
            dialog.setCanceledOnTouchOutside(true);
            this.visibleDialog.setOnDismissListener(new DialogInterface.OnDismissListener() { // from class: im.uecahxyvjd.ui.hui.friendscircle_v1.base.BaseFcFragment$$ExternalSyntheticLambda0
                @Override // android.content.DialogInterface.OnDismissListener
                public final void onDismiss(DialogInterface dialogInterface) {
                    BaseFcFragment.this.m3246xa801ba67(onDismissListener, dialogInterface);
                }
            });
            this.visibleDialog.show();
            return this.visibleDialog;
        } catch (Exception e2) {
            FileLog.e(e2);
            return null;
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$showDialog$0$im-uecahxyvjd-ui-hui-friendscircle_v1-base-BaseFcFragment  reason: not valid java name */
    public /* synthetic */ void m3246xa801ba67(DialogInterface.OnDismissListener onDismissListener, DialogInterface dialog1) {
        if (onDismissListener != null) {
            onDismissListener.onDismiss(dialog1);
        }
        onDialogDismiss(this.visibleDialog);
        this.visibleDialog = null;
    }

    public void dismissCurrentDialog() {
        Dialog dialog = this.visibleDialog;
        if (dialog == null) {
            return;
        }
        try {
            dialog.dismiss();
            this.visibleDialog = null;
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    protected void onDialogDismiss(Dialog dialog) {
    }

    public int getClassGuid() {
        return this.classGuid;
    }

    public Dialog getVisibleDialog() {
        return this.visibleDialog;
    }

    public AccountInstance getAccountInstance() {
        return AccountInstance.getInstance(this.currentAccount);
    }

    public UserConfig getUserConfig() {
        return getAccountInstance().getUserConfig();
    }

    protected MessagesController getMessagesController() {
        return getAccountInstance().getMessagesController();
    }

    protected ContactsController getContactsController() {
        return getAccountInstance().getContactsController();
    }

    protected MediaDataController getMediaDataController() {
        return getAccountInstance().getMediaDataController();
    }

    protected ConnectionsManager getConnectionsManager() {
        return getAccountInstance().getConnectionsManager();
    }

    protected LocationController getLocationController() {
        return getAccountInstance().getLocationController();
    }

    protected NotificationsController getNotificationsController() {
        return getAccountInstance().getNotificationsController();
    }

    protected MessagesStorage getMessagesStorage() {
        return getAccountInstance().getMessagesStorage();
    }

    protected SendMessagesHelper getSendMessagesHelper() {
        return getAccountInstance().getSendMessagesHelper();
    }

    protected void showBottomSheet() {
        if (getParentActivity() != null) {
            if (this.dialogCommonList == null) {
                List<String> list = new ArrayList<>();
                list.add("从手机相册选择");
                list.add("拍一张");
                FcCommMenuDialog fcCommMenuDialog = new FcCommMenuDialog(getParentActivity(), list, (List<Integer>) null, Theme.getColor(Theme.key_dialogTextBlack), new FcCommMenuDialog.RecyclerviewItemClickCallBack() { // from class: im.uecahxyvjd.ui.hui.friendscircle_v1.base.BaseFcFragment.1
                    @Override // im.uecahxyvjd.ui.hui.friendscircle_v1.view.FcCommMenuDialog.RecyclerviewItemClickCallBack
                    public void onRecyclerviewItemClick(int position) {
                        if (position == 0) {
                            BaseFcFragment.this.openGallery();
                        } else if (position == 1) {
                            BaseFcFragment.this.openCamera();
                        }
                    }
                }, 1);
                this.dialogCommonList = fcCommMenuDialog;
                fcCommMenuDialog.setTitle(LocaleController.getString("fc_change_bg", R.string.fc_change_bg), -7631463, 15);
            }
            if (this.dialogCommonList.isShowing()) {
                this.dialogCommonList.dismiss();
            } else {
                this.dialogCommonList.show();
            }
        }
    }

    public void openGallery() {
        if (Build.VERSION.SDK_INT >= 23 && getParentActivity() != null && getParentActivity().checkSelfPermission(PermissionUtils.PERMISSION_READ_EXTERNAL_STORAGE) != 0) {
            getParentActivity().requestPermissions(new String[]{PermissionUtils.PERMISSION_READ_EXTERNAL_STORAGE}, 4);
            return;
        }
        PhotoAlbumPickerActivity fragment = new PhotoAlbumPickerActivity(1, false, false, null, true);
        fragment.setDelegate(new PhotoAlbumPickerActivity.PhotoAlbumPickerActivityDelegate() { // from class: im.uecahxyvjd.ui.hui.friendscircle_v1.base.BaseFcFragment.2
            @Override // im.uecahxyvjd.ui.PhotoAlbumPickerActivity.PhotoAlbumPickerActivityDelegate
            public void didSelectPhotos(ArrayList<SendMessagesHelper.SendingMediaInfo> photos, boolean notify, int scheduleDate, boolean blnOriginalImg) {
                if (photos.get(0) != null) {
                    KLog.d("-------相册的背景图-" + photos.get(0).path);
                    BaseFcFragment.this.didSelectOnePhoto(photos.get(0).path);
                }
            }

            @Override // im.uecahxyvjd.ui.PhotoAlbumPickerActivity.PhotoAlbumPickerActivityDelegate
            public void startPhotoSelectActivity() {
            }
        });
        presentFragment(fragment);
    }

    public void openCamera() {
        if (getParentActivity() == null) {
            return;
        }
        try {
            if (Build.VERSION.SDK_INT >= 23 && getParentActivity().checkSelfPermission("android.permission.CAMERA") != 0) {
                getParentActivity().requestPermissions(new String[]{"android.permission.CAMERA"}, 19);
                return;
            }
            Intent takePictureIntent = new Intent("android.media.action.IMAGE_CAPTURE");
            File image = AndroidUtilities.generatePicturePath();
            if (image != null) {
                if (Build.VERSION.SDK_INT >= 24) {
                    takePictureIntent.putExtra("output", FileProvider.getUriForFile(getParentActivity(), "im.uecahxyvjd.messenger.provider", image));
                    takePictureIntent.addFlags(2);
                    takePictureIntent.addFlags(1);
                } else {
                    takePictureIntent.putExtra("output", Uri.fromFile(image));
                }
                this.currentPicturePath = image.getAbsolutePath();
            }
            startActivityForResult(takePictureIntent, 133);
        } catch (Exception e) {
            FileLog.e(e);
        }
    }

    @Override // androidx.fragment.app.Fragment
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == -1 && requestCode == 133) {
            PhotoViewer.getInstance().setParentActivity(getParentActivity());
            int orientation = 0;
            try {
                ExifInterface ei = new ExifInterface(this.currentPicturePath);
                int exif = ei.getAttributeInt(ExifInterface.TAG_ORIENTATION, 1);
                switch (exif) {
                    case 3:
                        orientation = 180;
                        break;
                    case 6:
                        orientation = 90;
                        break;
                    case 8:
                        orientation = 270;
                        break;
                }
            } catch (Exception e) {
                FileLog.e(e);
            }
            final ArrayList<Object> arrayList = new ArrayList<>();
            arrayList.add(new MediaController.PhotoEntry(0, 0, 0L, this.currentPicturePath, orientation, false));
            PhotoViewer.getInstance().setIsFcCrop(true);
            PhotoViewer.getInstance().openPhotoForSelect(arrayList, 0, 1, new PhotoViewer.EmptyPhotoViewerProvider() { // from class: im.uecahxyvjd.ui.hui.friendscircle_v1.base.BaseFcFragment.3
                @Override // im.uecahxyvjd.ui.PhotoViewer.EmptyPhotoViewerProvider, im.uecahxyvjd.ui.PhotoViewer.PhotoViewerProvider
                public void sendButtonPressed(int index, VideoEditedInfo videoEditedInfo, boolean notify, int scheduleDate) {
                    String path = null;
                    MediaController.PhotoEntry photoEntry = (MediaController.PhotoEntry) arrayList.get(0);
                    if (photoEntry.imagePath != null) {
                        path = photoEntry.imagePath;
                    } else if (photoEntry.path != null) {
                        path = photoEntry.path;
                    }
                    KLog.d("-------拍照的背景图-" + path);
                    BaseFcFragment.this.didSelectOnePhoto(path);
                }

                @Override // im.uecahxyvjd.ui.PhotoViewer.EmptyPhotoViewerProvider, im.uecahxyvjd.ui.PhotoViewer.PhotoViewerProvider
                public boolean allowCaption() {
                    return false;
                }

                @Override // im.uecahxyvjd.ui.PhotoViewer.EmptyPhotoViewerProvider, im.uecahxyvjd.ui.PhotoViewer.PhotoViewerProvider
                public boolean canScrollAway() {
                    return false;
                }
            }, null);
            AndroidUtilities.addMediaToGallery(this.currentPicturePath);
            this.currentPicturePath = null;
        }
    }

    protected void didSelectOnePhoto(String photosPath) {
    }

    @Override // com.bjz.comm.net.mvp.contract.BaseFcContract.IFcCommView
    public void getUploadUrlFailed(String msg) {
        FcToastUtils.show((CharSequence) (msg == null ? LocaleController.getString("friendscircle_fail_to_get_ip_location", R.string.friendscircle_fail_to_get_ip_location) : msg));
    }

    protected void uploadFile(String path, DataListener<BResponse<FcMediaResponseBean>> listener) {
        if (!TextUtils.isEmpty(path)) {
            File file = new File(path);
            if (file.exists()) {
                this.mCommPresenter.uploadFile(file, listener);
            }
        }
    }

    @Override // com.bjz.comm.net.mvp.contract.BaseFcContract.IFcCommView
    public void onUploadFileSucc(FcMediaResponseBean data, String msg) {
    }

    @Override // com.bjz.comm.net.mvp.contract.BaseFcContract.IFcCommView
    public void onUploadFileError(String msg) {
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void downloadFileToLocal(final String path) {
        String tips = LocaleController.getString(R.string.save_pic);
        String suffix = ".jpg";
        if (!TextUtils.isEmpty(path)) {
            if (path.toLowerCase().endsWith(".mp4")) {
                tips = LocaleController.getString(R.string.save_video);
                suffix = ".mp4";
            } else if (path.toLowerCase().endsWith(".gif")) {
                tips = LocaleController.getString(R.string.save_pic);
                suffix = ".gif";
            } else {
                tips = LocaleController.getString(R.string.save_pic);
                suffix = ".jpg";
            }
        }
        List<String> list = new ArrayList<>();
        list.add(tips);
        final String finalSuffix = suffix;
        FcCommMenuDialog dialogCommonList = new FcCommMenuDialog(getParentActivity(), list, (List<Integer>) null, Theme.getColor(Theme.key_windowBackgroundWhiteBlackText), new FcCommMenuDialog.RecyclerviewItemClickCallBack() { // from class: im.uecahxyvjd.ui.hui.friendscircle_v1.base.BaseFcFragment.4
            @Override // im.uecahxyvjd.ui.hui.friendscircle_v1.view.FcCommMenuDialog.RecyclerviewItemClickCallBack
            public void onRecyclerviewItemClick(int position) {
                if (position == 0) {
                    BaseFcFragment.this.mCommPresenter.downloadFile(HttpUtils.getInstance().getDownloadFileUrl() + path, AndroidUtilities.getAlbumDir(false).getAbsolutePath(), TimeUtils.getCurrentTime() + finalSuffix);
                }
            }
        }, 1);
        dialogCommonList.show();
    }

    @Override // com.bjz.comm.net.mvp.contract.BaseFcContract.IFcCommView
    public void onDownloadFileSucc(File file) {
        AndroidUtilities.addMediaToGallery(Uri.fromFile(file));
        FcToastUtils.show((CharSequence) LocaleController.getString(R.string.save_album_success));
    }

    @Override // com.bjz.comm.net.mvp.contract.BaseFcContract.IFcCommView
    public void onDownloadFileError(String msg) {
        FcToastUtils.show((CharSequence) LocaleController.getString(R.string.save_album_error));
    }

    @Override // com.bjz.comm.net.mvp.contract.BaseFcContract.IFcCommView
    public void onError(String msg) {
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void saveNewFcToLocal(T t) {
        FcDBHelper.getInstance().insert(t);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void updateLocalFollowStatus(Class<T> cla, long createBy, boolean isFollow) {
        FcDBHelper.getInstance().updateItemFollowStatus(cla, createBy, isFollow);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void updateLocalItemPermissionStatus(Class<T> cla, long forumID, int permission) {
        FcDBHelper.getInstance().updateItemPermissionStatus(cla, forumID, permission);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void deleteLocalItemById(Class<T> cla, long forumId) {
        FcDBHelper.getInstance().deleteItemById(cla, forumId);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void deleteLocalItemByUserId(Class<T> cla, long userId) {
        FcDBHelper.getInstance().deleteItemByUserId(cla, userId);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void updateLocalItemLikeStatus(Class<T> cla, long forumId, boolean isLike, int ThumbUp) {
        FcDBHelper.getInstance().updateItemLikeStatus(cla, forumId, isLike, ThumbUp);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void updateLocalReplyStatus(Class<T> cla, long forumID, FcReplyBean mFcReplyBean) {
        KLog.d("------------------添加" + forumID);
        FcDBHelper.getInstance().updateItemReply(cla, forumID, mFcReplyBean);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public <T> void deleteLocalReply(Class<T> cla, long forumID, long commentID, int commentCount) {
        FcDBHelper.getInstance().deleteReply(cla, forumID, commentID, commentCount);
    }

    public void gotoNearBy() {
        Activity activity;
        if (Build.VERSION.SDK_INT >= 23 && (activity = getParentActivity()) != null && activity.checkSelfPermission(PermissionUtils.PERMISSION_ACCESS_COARSE_LOCATION) != 0) {
            presentFragment(new ActionIntroActivity(1));
            return;
        }
        boolean enabled = true;
        if (Build.VERSION.SDK_INT >= 28) {
            LocationManager lm = (LocationManager) ApplicationLoader.applicationContext.getSystemService("location");
            enabled = lm.isLocationEnabled();
        } else {
            try {
                int mode = Settings.Secure.getInt(ApplicationLoader.applicationContext.getContentResolver(), "location_mode", 0);
                enabled = mode != 0;
            } catch (Throwable e) {
                FileLog.e(e);
            }
        }
        if (!enabled) {
            presentFragment(new ActionIntroActivity(4));
        } else {
            presentFragment(new NearPersonAndGroupActivity());
        }
    }
}
