package im.uecahxyvjd.ui.hui.friendscircle_v1.view.toast;

import android.app.AppOpsManager;
import android.app.Application;
import android.app.NotificationManager;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.res.Resources;
import android.graphics.drawable.GradientDrawable;
import android.os.Build;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.ui.components.toast.BaseToast;
import im.uecahxyvjd.ui.components.toast.SafeToast;
import im.uecahxyvjd.ui.components.toast.SupportToast;
import im.uecahxyvjd.ui.components.toast.ToastStrategy;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
/* loaded from: classes8.dex */
public final class FcToastUtils {
    private static boolean changed = false;
    private static ToastStrategy sStrategy;
    private static ToastBlackStyle sStyle;
    private static Toast sToast;

    private FcToastUtils() {
    }

    public static void init(Context context) {
        checkNullPointer(context);
        if (sStrategy == null) {
            setToastHandler(new ToastStrategy());
        }
        if (sStyle == null) {
            initStyle(new ToastBlackStyle());
        }
        if (isNotificationEnabled(context)) {
            if (Build.VERSION.SDK_INT == 25) {
                setToast(new SafeToast(context));
            } else {
                setToast(new BaseToast(context));
            }
        } else {
            setToast(new SupportToast(context));
        }
        setView(createTextView(context.getApplicationContext()));
        setGravity(sStyle.getGravity(), sStyle.getXOffset(), sStyle.getYOffset());
    }

    public static void show(Object object) {
        if (object == null || (object + "").length() == 0 || "null".equals(object + "")) {
            return;
        }
        show((CharSequence) object.toString());
    }

    public static void show(int id) {
        checkToastState();
        try {
            show((CharSequence) LocaleController.getString(id + "", id));
        } catch (Resources.NotFoundException e) {
            show((CharSequence) String.valueOf(id));
        }
    }

    public static void showFormat(int resId, Object... args) {
        checkToastState();
        try {
            show((CharSequence) LocaleController.formatString(resId + "", resId, args));
        } catch (Resources.NotFoundException e) {
            FileLog.e("id not exits");
        }
    }

    public static void setChanged(boolean changed2) {
        changed = changed2;
    }

    public static synchronized void show(CharSequence text) {
        synchronized (FcToastUtils.class) {
            if (text == null) {
                return;
            }
            checkToastState();
            if (changed) {
                reset();
            }
            sStrategy.show(text);
        }
    }

    public static synchronized void cancel() {
        synchronized (FcToastUtils.class) {
            checkToastState();
            sStrategy.cancel();
        }
    }

    public static void setGravity(int gravity, int xOffset, int yOffset) {
        checkToastState();
        sToast.setGravity(Gravity.getAbsoluteGravity(gravity, sToast.getView().getResources().getConfiguration().getLayoutDirection()), xOffset, yOffset);
    }

    public static void setView(int layoutId) {
        checkToastState();
        setView(View.inflate(sToast.getView().getContext().getApplicationContext(), layoutId, null));
    }

    public static void setView(View view) {
        checkToastState();
        checkNullPointer(view);
        Context context = view.getContext();
        if (!(context instanceof Application)) {
            throw new IllegalArgumentException("The view must be initialized using the context of the application");
        }
        Toast toast = sToast;
        if (toast != null) {
            toast.cancel();
            sToast.setView(view);
        }
    }

    public static <V extends View> V getView() {
        checkToastState();
        return (V) sToast.getView();
    }

    public static void initStyle(ToastBlackStyle style) {
        checkNullPointer(style);
        sStyle = style;
        Toast toast = sToast;
        if (toast != null) {
            toast.cancel();
            Toast toast2 = sToast;
            toast2.setView(createTextView(toast2.getView().getContext().getApplicationContext()));
            sToast.setGravity(sStyle.getGravity(), sStyle.getXOffset(), sStyle.getYOffset());
        }
    }

    public static void setToast(Toast toast) {
        checkNullPointer(toast);
        sToast = toast;
        ToastStrategy toastStrategy = sStrategy;
        if (toastStrategy != null) {
            toastStrategy.bind(toast);
        }
    }

    public static void setToastHandler(ToastStrategy handler) {
        checkNullPointer(handler);
        sStrategy = handler;
        Toast toast = sToast;
        if (toast != null) {
            handler.bind(toast);
        }
    }

    public static Toast getToast() {
        return sToast;
    }

    private static void checkToastState() {
        if (sToast == null) {
            throw new IllegalStateException("ToastUtils has not been initialized");
        }
    }

    private static void checkNullPointer(Object object) {
        if (object == null) {
            throw new NullPointerException("are you ok?");
        }
    }

    private static void reset() {
        View view = getView();
        if (view instanceof TextView) {
            TextView textView = (TextView) view;
            textView.setPaddingRelative(sStyle.getPaddingStart(), sStyle.getPaddingTop(), sStyle.getPaddingEnd(), sStyle.getPaddingBottom());
            textView.setTextSize(sStyle.getTextSize());
            textView.setCompoundDrawables(null, null, null, null);
            GradientDrawable drawable = new GradientDrawable();
            drawable.setColor(sStyle.getBackgroundColor());
            drawable.setCornerRadius(sStyle.getCornerRadius());
            textView.setBackground(drawable);
            changed = false;
        }
    }

    private static TextView createTextView(Context context) {
        GradientDrawable drawable = new GradientDrawable();
        drawable.setColor(sStyle.getBackgroundColor());
        drawable.setCornerRadius(sStyle.getCornerRadius());
        TextView textView = new TextView(context);
        textView.setId(16908299);
        textView.setTextColor(sStyle.getTextColor());
        textView.setTextSize(sStyle.getTextSize());
        textView.setPaddingRelative(sStyle.getPaddingStart(), sStyle.getPaddingTop(), sStyle.getPaddingEnd(), sStyle.getPaddingBottom());
        textView.setLayoutParams(new ViewGroup.LayoutParams(-1, -2));
        textView.setBackground(drawable);
        if (sStyle.getMaxLines() > 0) {
            textView.setMaxLines(sStyle.getMaxLines());
        }
        return textView;
    }

    private static boolean isNotificationEnabled(Context context) {
        if (Build.VERSION.SDK_INT >= 24) {
            return ((NotificationManager) context.getSystemService("notification")).areNotificationsEnabled();
        }
        AppOpsManager appOps = (AppOpsManager) context.getSystemService("appops");
        ApplicationInfo appInfo = context.getApplicationInfo();
        String pkg = context.getApplicationContext().getPackageName();
        int uid = appInfo.uid;
        try {
            Class<?> appOpsClass = Class.forName(AppOpsManager.class.getName());
            Method checkOpNoThrowMethod = appOpsClass.getMethod("checkOpNoThrow", Integer.TYPE, Integer.TYPE, String.class);
            Field opPostNotificationValue = appOpsClass.getDeclaredField("OP_POST_NOTIFICATION");
            int value = ((Integer) opPostNotificationValue.get(Integer.class)).intValue();
            return ((Integer) checkOpNoThrowMethod.invoke(appOps, Integer.valueOf(value), Integer.valueOf(uid), pkg)).intValue() == 0;
        } catch (ClassNotFoundException | IllegalAccessException | NoSuchFieldException | NoSuchMethodException | RuntimeException | InvocationTargetException e) {
            return true;
        }
    }
}
