package im.uecahxyvjd.ui.hviews.sidebar;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;
import androidx.exifinterface.media.ExifInterface;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.ui.actionbar.Theme;
/* loaded from: classes8.dex */
public class SideBar extends View {

    /* renamed from: a  reason: collision with root package name */
    public static String[] f615a = {ExifInterface.GPS_MEASUREMENT_IN_PROGRESS, "B", "C", "D", ExifInterface.LONGITUDE_EAST, "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", ExifInterface.LATITUDE_SOUTH, ExifInterface.GPS_DIRECTION_TRUE, "U", ExifInterface.GPS_MEASUREMENT_INTERRUPTED, ExifInterface.LONGITUDE_WEST, "X", "Y", "Z", "#"};
    public static String[] b = {"↑", "☆", ExifInterface.GPS_MEASUREMENT_IN_PROGRESS, "B", "C", "D", ExifInterface.LONGITUDE_EAST, "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", ExifInterface.LATITUDE_SOUTH, ExifInterface.GPS_DIRECTION_TRUE, "U", ExifInterface.GPS_MEASUREMENT_INTERRUPTED, ExifInterface.LONGITUDE_WEST, "X", "Y", "Z", "#"};
    private Paint bitmapPaint;
    public String[] chars;
    private int choose;
    private int height;
    private Context mContext;
    private TextView mTextDialog;
    private OnTouchingLetterChangedListener onTouchingLetterChangedListener;
    private boolean onlyChar;
    private Paint textPaint;

    /* loaded from: classes8.dex */
    public interface OnTouchingLetterChangedListener {
        void onTouchingLetterChanged(String str);
    }

    public void setTextView(TextView mTextDialog) {
        this.mTextDialog = mTextDialog;
    }

    public SideBar(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        this.choose = -1;
        this.textPaint = new Paint();
        this.bitmapPaint = new Paint();
        this.onlyChar = true;
        this.mContext = context;
        this.chars = f615a;
    }

    public SideBar(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public SideBar(Context context) {
        this(context, null);
    }

    public void setOnlyChar(boolean flag) {
        this.onlyChar = flag;
        if (flag) {
            this.chars = f615a;
        } else {
            this.chars = b;
        }
        requestLayout();
        invalidate();
    }

    public void setCharsOnly() {
        this.chars = f615a;
    }

    public void setTextColor(int color) {
        this.textPaint.setColor(color);
    }

    @Override // android.view.View
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        int defaultCharHeight = AndroidUtilities.dp(15.0f);
        int width = AndroidUtilities.dp(35.0f);
        int height = Math.min(this.chars.length * defaultCharHeight, AndroidUtilities.dp(500.0f));
        setMeasuredDimension(width, height);
    }

    @Override // android.view.View
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        int width = getWidth();
        int height = getHeight();
        this.height = height;
        int singleHeight = height / this.chars.length;
        Bitmap bitmap = BitmapFactory.decodeResource(getResources(), R.mipmap.ic_slide_bar_char_bg);
        for (int i = 0; i < this.chars.length; i++) {
            this.textPaint.setColor(Theme.getColor(Theme.key_sidebar_textDefaultColor));
            this.textPaint.setTypeface(Typeface.DEFAULT);
            this.textPaint.setAntiAlias(true);
            this.textPaint.setTextSize(dip2px(this.mContext, 9.5f));
            this.textPaint.setTextAlign(Paint.Align.CENTER);
            if (i == this.choose) {
                this.textPaint.setColor(-1);
                this.textPaint.setFakeBoldText(true);
                this.bitmapPaint.setAlpha(225);
            } else {
                this.bitmapPaint.setAlpha(0);
            }
            canvas.drawBitmap(bitmap, ((width / 2) - (bitmap.getWidth() / 2)) - AndroidUtilities.dp(1.0f), ((singleHeight - bitmap.getHeight()) / 2) + (singleHeight * i), this.bitmapPaint);
            Paint.FontMetrics fontMetrics = this.textPaint.getFontMetrics();
            float distance = ((fontMetrics.bottom - fontMetrics.top) / 2.0f) - fontMetrics.bottom;
            float baseline = (singleHeight / 2) + distance + (singleHeight * i);
            canvas.drawText(this.chars[i], width / 2, baseline, this.textPaint);
            this.textPaint.reset();
        }
    }

    @Override // android.view.View
    public boolean dispatchTouchEvent(MotionEvent event) {
        int action = event.getAction();
        float y = event.getY();
        int oldChoose = this.choose;
        OnTouchingLetterChangedListener listener = this.onTouchingLetterChangedListener;
        String[] strArr = this.chars;
        int c = (int) ((y / this.height) * strArr.length);
        switch (action) {
            case 1:
                if ((c < 0 && !this.onlyChar) || (c > 0 && c < strArr.length && strArr[c].equals("↑"))) {
                    this.choose = -1;
                }
                invalidate();
                TextView textView = this.mTextDialog;
                if (textView != null) {
                    textView.setVisibility(4);
                    return true;
                }
                return true;
            default:
                if (oldChoose != c && c >= 0 && c < strArr.length) {
                    if (listener != null) {
                        listener.onTouchingLetterChanged(strArr[c]);
                    }
                    TextView textView2 = this.mTextDialog;
                    if (textView2 != null) {
                        textView2.setText(this.chars[c]);
                        this.mTextDialog.setVisibility(0);
                    }
                    this.choose = c;
                    invalidate();
                    return true;
                }
                return true;
        }
    }

    public void setOnTouchingLetterChangedListener(OnTouchingLetterChangedListener onTouchingLetterChangedListener) {
        this.onTouchingLetterChangedListener = onTouchingLetterChangedListener;
    }

    public static int dip2px(Context context, float dpValue) {
        float scale = context.getResources().getDisplayMetrics().density;
        return (int) ((dpValue * scale) + 0.5f);
    }

    public void setChars(String[] chars) {
        if (chars.length != 0) {
            if (this.onlyChar) {
                this.chars = chars;
                this.choose = 0;
            } else {
                String[] newChars = new String[chars.length + 2];
                newChars[0] = "↑";
                newChars[1] = "☆";
                for (int i = 0; i < chars.length; i++) {
                    newChars[i + 2] = chars[i];
                }
                this.chars = newChars;
                this.choose = 2;
            }
            requestLayout();
            invalidate();
        }
    }

    public void setChooseChar(String c) {
        int i = 0;
        while (true) {
            String[] strArr = this.chars;
            if (i < strArr.length) {
                if (strArr[i].equals(c)) {
                    this.choose = i;
                    invalidate();
                }
                i++;
            } else {
                return;
            }
        }
    }
}
