package im.uecahxyvjd.ui.hviews.sliding;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewConfiguration;
import android.widget.FrameLayout;
import androidx.core.view.MotionEventCompat;
import androidx.core.view.ViewCompat;
import im.uecahxyvjd.messenger.R;
/* loaded from: classes9.dex */
public class SlidingLayout extends FrameLayout {
    private static final int INVALID_POINTER = -1;
    private static final int RESET_DURATION = 200;
    public static final int SLIDING_DISTANCE_UNDEFINED = -1;
    public static final int SLIDING_MODE_BOTH = 0;
    public static final int SLIDING_MODE_BOTTOM = 2;
    public static final int SLIDING_MODE_TOP = 1;
    public static final int SLIDING_POINTER_MODE_MORE = 1;
    public static final int SLIDING_POINTER_MODE_ONE = 0;
    private static final int SMOOTH_DURATION = 1000;
    public static final int STATE_IDLE = 1;
    public static final int STATE_SLIDING = 2;
    private int mActivePointerId;
    private View mBackgroundView;
    private int mBackgroundViewLayoutId;
    private View.OnTouchListener mDelegateTouchListener;
    private View mFollowView;
    private float mInitialDownY;
    private float mInitialMotionY;
    private boolean mIsBeingDragged;
    private float mLastMotionY;
    private SlidingListener mSlidingListener;
    private int mSlidingMode;
    private float mSlidingOffset;
    private int mSlidingPointerMode;
    private int mSlidingTopMaxDistance;
    private View mTargetView;
    private int mTouchSlop;

    /* loaded from: classes9.dex */
    public interface SlidingListener {
        void onSlidingChangePointer(View view, int i);

        void onSlidingOffset(View view, float f);

        void onSlidingStateChange(View view, int i);
    }

    public SlidingLayout(Context context) {
        this(context, null);
    }

    public SlidingLayout(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public SlidingLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.mBackgroundViewLayoutId = 0;
        this.mActivePointerId = -1;
        this.mSlidingOffset = 0.5f;
        this.mSlidingMode = 0;
        this.mSlidingPointerMode = 1;
        this.mSlidingTopMaxDistance = -1;
        init(context, attrs);
    }

    private void init(Context context, AttributeSet attrs) {
        TypedArray a2 = context.obtainStyledAttributes(attrs, R.styleable.SlidingLayout);
        this.mBackgroundViewLayoutId = a2.getResourceId(0, this.mBackgroundViewLayoutId);
        this.mSlidingMode = a2.getInteger(1, 0);
        this.mSlidingPointerMode = a2.getInteger(2, 1);
        this.mSlidingTopMaxDistance = a2.getDimensionPixelSize(3, -1);
        a2.recycle();
        if (this.mBackgroundViewLayoutId != 0) {
            View view = View.inflate(getContext(), this.mBackgroundViewLayoutId, null);
            setBackgroundView(view);
        }
        this.mTouchSlop = ViewConfiguration.get(getContext()).getScaledTouchSlop();
    }

    public void setBackgroundView(View view) {
        View view2 = this.mBackgroundView;
        if (view2 != null) {
            removeView(view2);
        }
        this.mBackgroundView = view;
        addView(view, 0);
    }

    public View getBackgroundView() {
        return this.mBackgroundView;
    }

    public void setSlidingDistance(int distance) {
        this.mSlidingTopMaxDistance = distance;
    }

    public int setSlidingDistance() {
        return this.mSlidingTopMaxDistance;
    }

    public float getSlidingOffset() {
        return this.mSlidingOffset;
    }

    public void setSlidingOffset(float slidingOffset) {
        this.mSlidingOffset = slidingOffset;
    }

    public void setSlidingListener(SlidingListener slidingListener) {
        this.mSlidingListener = slidingListener;
    }

    @Override // android.widget.FrameLayout, android.view.ViewGroup, android.view.View
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        if (getChildCount() != 0 && this.mTargetView == null) {
            ensureTarget();
        }
    }

    private void ensureTarget() {
        if (this.mTargetView == null) {
            this.mTargetView = getChildAt(getChildCount() - 1);
        }
    }

    public void setTargetView(View view) {
        View view2 = this.mTargetView;
        if (view2 != null) {
            removeView(view2);
        }
        this.mTargetView = view;
        addView(view);
    }

    public void setFollowView(View view) {
        this.mFollowView = view;
    }

    @Override // android.view.View
    public void setOnTouchListener(View.OnTouchListener l) {
        this.mDelegateTouchListener = l;
    }

    public View getTargetView() {
        return this.mTargetView;
    }

    public float getSlidingDistance() {
        return getInstrument().getTranslationY(getTargetView());
    }

    public Instrument getInstrument() {
        return Instrument.getInstance();
    }

    @Override // android.view.ViewGroup
    public boolean onInterceptTouchEvent(MotionEvent ev) {
        ensureTarget();
        int action = MotionEventCompat.getActionMasked(ev);
        switch (action) {
            case 0:
                int pointerId = MotionEventCompat.getPointerId(ev, 0);
                this.mActivePointerId = pointerId;
                this.mIsBeingDragged = false;
                float initialDownY = getMotionEventY(ev, pointerId);
                if (initialDownY != -1.0f) {
                    this.mInitialDownY = initialDownY;
                    break;
                } else {
                    return false;
                }
            case 1:
            case 3:
                this.mIsBeingDragged = false;
                this.mActivePointerId = -1;
                break;
            case 2:
                int i = this.mActivePointerId;
                if (i != -1) {
                    float y = getMotionEventY(ev, i);
                    if (y != -1.0f) {
                        float yDiff = this.mInitialDownY;
                        if (y > yDiff) {
                            if (y - yDiff > this.mTouchSlop && !this.mIsBeingDragged && !canChildScrollUp()) {
                                float f = this.mInitialDownY + this.mTouchSlop;
                                this.mInitialMotionY = f;
                                this.mLastMotionY = f;
                                this.mIsBeingDragged = true;
                                break;
                            }
                        } else if (y < yDiff && yDiff - y > this.mTouchSlop && !this.mIsBeingDragged && !canChildScrollDown()) {
                            float f2 = this.mInitialDownY + this.mTouchSlop;
                            this.mInitialMotionY = f2;
                            this.mLastMotionY = f2;
                            this.mIsBeingDragged = true;
                            break;
                        }
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
                break;
        }
        return this.mIsBeingDragged;
    }

    private float getMotionEventY(MotionEvent ev, int activePointerId) {
        int index = MotionEventCompat.findPointerIndex(ev, activePointerId);
        if (index < 0) {
            return -1.0f;
        }
        return MotionEventCompat.getY(ev, index);
    }

    public boolean canChildScrollUp() {
        return ViewCompat.canScrollVertically(this.mTargetView, -1);
    }

    public boolean canChildScrollDown() {
        return ViewCompat.canScrollVertically(this.mTargetView, 1);
    }

    @Override // android.view.ViewGroup, android.view.View
    public boolean dispatchTouchEvent(MotionEvent event) {
        return super.dispatchTouchEvent(event);
    }

    @Override // android.view.View
    public boolean onTouchEvent(MotionEvent event) {
        float delta;
        float movemment;
        View.OnTouchListener onTouchListener = this.mDelegateTouchListener;
        if (onTouchListener == null || !onTouchListener.onTouch(this, event)) {
            switch (event.getAction()) {
                case 1:
                case 3:
                    SlidingListener slidingListener = this.mSlidingListener;
                    if (slidingListener != null) {
                        slidingListener.onSlidingStateChange(this, 1);
                    }
                    getInstrument().reset(this.mTargetView, this.mFollowView, 200L);
                    break;
                case 2:
                    if (this.mSlidingPointerMode == 1) {
                        int activePointerId = MotionEventCompat.getPointerId(event, event.getPointerCount() - 1);
                        if (this.mActivePointerId != activePointerId) {
                            this.mActivePointerId = activePointerId;
                            float motionEventY = getMotionEventY(event, activePointerId);
                            this.mInitialDownY = motionEventY;
                            float f = motionEventY + this.mTouchSlop;
                            this.mInitialMotionY = f;
                            this.mLastMotionY = f;
                            SlidingListener slidingListener2 = this.mSlidingListener;
                            if (slidingListener2 != null) {
                                slidingListener2.onSlidingChangePointer(this.mTargetView, activePointerId);
                            }
                        }
                        float delta2 = getMotionEventY(event, this.mActivePointerId) - this.mLastMotionY;
                        float tempOffset = 1.0f - (Math.abs(getInstrument().getTranslationY(this.mTargetView) + delta2) / this.mTargetView.getMeasuredHeight());
                        delta = getInstrument().getTranslationY(this.mTargetView) + (this.mSlidingOffset * delta2 * tempOffset);
                        this.mLastMotionY = getMotionEventY(event, this.mActivePointerId);
                        movemment = getMotionEventY(event, this.mActivePointerId) - this.mInitialMotionY;
                    } else {
                        float tempOffset2 = 1.0f - Math.abs(getInstrument().getTranslationY(this.mTargetView) / this.mTargetView.getMeasuredHeight());
                        delta = (event.getY() - this.mInitialMotionY) * this.mSlidingOffset * tempOffset2;
                        movemment = event.getY() - this.mInitialMotionY;
                    }
                    float distance = getSlidingDistance();
                    switch (this.mSlidingMode) {
                        case 0:
                            getInstrument().slidingByDelta(this.mTargetView, this.mFollowView, delta);
                            break;
                        case 1:
                            if (movemment >= 0.0f || distance > 0.0f) {
                                if (delta < 0.0f) {
                                    delta = 0.0f;
                                }
                                int i = this.mSlidingTopMaxDistance;
                                if (i != -1 && delta >= i) {
                                    delta = i;
                                }
                                getInstrument().slidingByDelta(this.mTargetView, this.mFollowView, delta);
                                break;
                            }
                            break;
                        case 2:
                            if (movemment <= 0.0f || distance < 0.0f) {
                                if (delta > 0.0f) {
                                    delta = 0.0f;
                                }
                                getInstrument().slidingByDelta(this.mTargetView, this.mFollowView, delta);
                                break;
                            }
                            break;
                    }
                    SlidingListener slidingListener3 = this.mSlidingListener;
                    if (slidingListener3 != null) {
                        slidingListener3.onSlidingStateChange(this, 2);
                        this.mSlidingListener.onSlidingOffset(this, delta);
                        break;
                    }
                    break;
            }
            return true;
        }
        return true;
    }

    public void setSlidingMode(int mode) {
        this.mSlidingMode = mode;
    }

    public int getSlidingMode() {
        return this.mSlidingMode;
    }

    public void smoothScrollTo(float y) {
        getInstrument().smoothTo(this.mTargetView, y, 1000L);
    }

    @Override // android.view.ViewGroup, android.view.View
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
    }

    @Override // android.view.ViewGroup, android.view.View
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        View view = this.mTargetView;
        if (view != null) {
            view.clearAnimation();
        }
        this.mSlidingMode = 0;
        this.mTargetView = null;
        this.mBackgroundView = null;
        this.mSlidingListener = null;
    }
}
