package im.uecahxyvjd.ui.utils;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.text.TextUtils;
import com.fm.openinstall.OpenInstall;
import com.fm.openinstall.listener.AppInstallAdapter;
import com.fm.openinstall.listener.AppWakeUpAdapter;
import com.fm.openinstall.model.AppData;
import im.uecahxyvjd.messenger.FileLog;
import im.uecahxyvjd.messenger.MessagesController;
/* loaded from: classes9.dex */
public class OpenInstallUitl {
    private static volatile OpenInstallUitl Instance = null;
    public static final String TAG = "OpenInstallUitl";
    private AppInstallAdapter installCallback;
    private AppWakeUpAdapter wakeUpcallback;

    public static OpenInstallUitl getInstance() {
        if (Instance == null) {
            synchronized (OpenInstallUitl.class) {
                if (Instance == null) {
                    Instance = new OpenInstallUitl();
                }
            }
        }
        return Instance;
    }

    public static void init(Context context) {
        OpenInstall.init(context);
    }

    public static void reportRegister() {
        OpenInstall.reportRegister();
    }

    public static void reportEffectPoint(String pointId, long pointValue) {
        OpenInstall.reportEffectPoint(pointId, pointValue);
    }

    private OpenInstallUitl() {
    }

    public void getInstallOrWakeUp(Intent intent) {
        final SharedPreferences sp = MessagesController.getGlobalMainSettings();
        if (sp != null) {
            if (this.installCallback == null) {
                this.installCallback = new AppInstallAdapter() { // from class: im.uecahxyvjd.ui.utils.OpenInstallUitl.1
                    @Override // com.fm.openinstall.listener.AppInstallAdapter
                    public void onInstall(AppData appData) {
                        SharedPreferences.Editor editor;
                        FileLog.d(OpenInstallUitl.TAG, "onInstall ---> , appData=" + (appData != null ? appData.toString() : "null"));
                        if (appData != null && (editor = sp.edit()) != null) {
                            boolean needC = false;
                            if (!TextUtils.isEmpty(appData.data)) {
                                editor.putString("Op_data", appData.data);
                                needC = true;
                            }
                            if (!TextUtils.isEmpty(appData.channel)) {
                                editor.putString("Op_channel", appData.channel);
                                needC = true;
                            }
                            if (needC) {
                                editor.commit();
                            }
                        }
                    }
                };
            }
            OpenInstall.getInstall(this.installCallback);
        }
        if (this.wakeUpcallback == null) {
            this.wakeUpcallback = new AppWakeUpAdapter() { // from class: im.uecahxyvjd.ui.utils.OpenInstallUitl.2
                @Override // com.fm.openinstall.listener.AppWakeUpAdapter
                public void onWakeUp(AppData appData) {
                    FileLog.d(OpenInstallUitl.TAG, "onWakeUp ---> , appData=" + (appData != null ? appData.toString() : "null"));
                }
            };
        }
        OpenInstall.getWakeUpAlwaysCallback(intent, this.wakeUpcallback);
    }

    public void onDestroy() {
        this.wakeUpcallback = null;
        this.installCallback = null;
    }
}
