package im.uecahxyvjd.ui.wallet;

import android.content.Context;
import android.graphics.Typeface;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import com.blankj.utilcode.util.ColorUtils;
import com.blankj.utilcode.util.SpanUtils;
import com.litesuits.orm.db.assit.SQLBuilder;
import com.scwang.smartrefresh.layout.SmartRefreshLayout;
import com.scwang.smartrefresh.layout.api.RefreshLayout;
import com.scwang.smartrefresh.layout.listener.OnRefreshListener;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.tgnet.ConnectionsManager;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLApiModel;
import im.uecahxyvjd.tgnet.TLJsonResolve;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPCWallet;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.ActionBarMenu;
import im.uecahxyvjd.ui.actionbar.ActionBarMenuItem;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.components.AppTextView;
import im.uecahxyvjd.ui.components.TextCell;
import im.uecahxyvjd.ui.components.toast.ToastUtils;
import im.uecahxyvjd.ui.hui.wallet_public.bean.WalletAccountInfo;
import im.uecahxyvjd.ui.hui.wallet_public.bean.WalletConfigBean;
import im.uecahxyvjd.ui.hui.wallet_public.utils.WalletDialogUtil;
import im.uecahxyvjd.ui.hui.wallet_public.utils.WalletErrorUtil;
import im.uecahxyvjd.ui.load.SpinKitView;
import im.uecahxyvjd.ui.utils.number.MoneyUtil;
import im.uecahxyvjd.ui.wallet.utils.AnimationUtils;
import im.uecahxyvjd.ui.wallet.utils.ExceptionUtils;
/* loaded from: classes18.dex */
public class WalletActivity extends BaseFragment implements NotificationCenter.NotificationCenterDelegate {
    private TextCell bankCell;
    private AppTextView btnSet;
    private boolean hide = true;
    private ImageView ivHide;
    private ImageView ivTip;
    private TextView ivUnit;
    private SpinKitView loadView;
    private ActionBarMenuItem menuItem;
    private TextCell recordCell;
    private SmartRefreshLayout refreshLayout;
    private LinearLayout tipLayout;
    private AppTextView tvCharge;
    private AppTextView tvDesc;
    private AppTextView tvTips;
    private TextView tvTotal;
    private AppTextView tvTotalTip;
    private AppTextView tvWithdraw;
    private LinearLayout walletLayout;

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onFragmentCreate() {
        super.onFragmentCreate();
        NotificationCenter.getInstance(this.currentAccount).addObserver(this, NotificationCenter.walletInfoNeedReload);
        NotificationCenter.getInstance(this.currentAccount).addObserver(this, NotificationCenter.paymentPasswordDidSet);
        return true;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onFragmentDestroy() {
        super.onFragmentDestroy();
        NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.walletInfoNeedReload);
        NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.paymentPasswordDidSet);
        ConnectionsManager.getInstance(this.currentAccount).cancelRequestsForGuid(this.classGuid);
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public View createView(Context context) {
        this.fragmentView = LayoutInflater.from(context).inflate(R.layout.activity_wallet_layout, (ViewGroup) null);
        initActionBar();
        initViews();
        return this.fragmentView;
    }

    private void initActionBar() {
        this.actionBar.setTitle(LocaleController.getString(R.string.MyWallet));
        this.actionBar.setBackButtonImage(R.mipmap.ic_back);
        this.actionBar.setCastShadows(false);
        ActionBarMenu menu = this.actionBar.createMenu();
        this.menuItem = menu.addItem(1, R.mipmap.ic_more);
        this.actionBar.setActionBarMenuOnItemClick(new ActionBar.ActionBarMenuOnItemClick() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity.1
            @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
            public void onItemClick(int id) {
                if (id == -1) {
                    WalletActivity.this.finishFragment();
                } else if (id == 1) {
                    WalletManagementActivity fragment = new WalletManagementActivity();
                    WalletActivity.this.presentFragment(fragment);
                }
            }
        });
        this.menuItem.setVisibility(8);
        this.fragmentView.postDelayed(new Runnable() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity.2
            @Override // java.lang.Runnable
            public void run() {
                WalletActivity.this.loadWalletInfo();
            }
        }, 1000L);
    }

    private void initViews() {
        this.refreshLayout = (SmartRefreshLayout) this.fragmentView.findViewById(R.id.refreshLayout);
        this.loadView = (SpinKitView) this.fragmentView.findViewById(R.id.loadView);
        this.tipLayout = (LinearLayout) this.fragmentView.findViewById(R.id.tipLayout);
        this.tvTotalTip = (AppTextView) this.fragmentView.findViewById(R.id.tvTotalTip);
        this.ivTip = (ImageView) this.fragmentView.findViewById(R.id.ivTip);
        this.tvTips = (AppTextView) this.fragmentView.findViewById(R.id.tvTips);
        this.tvDesc = (AppTextView) this.fragmentView.findViewById(R.id.tvDesc);
        this.btnSet = (AppTextView) this.fragmentView.findViewById(R.id.btnSet);
        this.walletLayout = (LinearLayout) this.fragmentView.findViewById(R.id.walletLayout);
        this.ivHide = (ImageView) this.fragmentView.findViewById(R.id.ivHide);
        this.tvTotal = (TextView) this.fragmentView.findViewById(R.id.tvTotal);
        this.ivUnit = (TextView) this.fragmentView.findViewById(R.id.ivUnit);
        this.tvWithdraw = (AppTextView) this.fragmentView.findViewById(R.id.tvWithdraw);
        this.tvCharge = (AppTextView) this.fragmentView.findViewById(R.id.tvCharge);
        this.bankCell = (TextCell) this.fragmentView.findViewById(R.id.bankCell);
        this.recordCell = (TextCell) this.fragmentView.findViewById(R.id.recordCell);
        this.tipLayout.setVisibility(8);
        this.walletLayout.setVisibility(8);
        this.refreshLayout.setEnableLoadMore(false);
        this.refreshLayout.setOnRefreshListener(new OnRefreshListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity.3
            @Override // com.scwang.smartrefresh.layout.listener.OnRefreshListener
            public void onRefresh(RefreshLayout refreshLayout) {
                WalletActivity.this.loadWalletInfo(true);
            }
        });
        SpanUtils.with(this.tvTotalTip).append(LocaleController.getString(R.string.TotalAssets)).append(SQLBuilder.PARENTHESES_LEFT).append(LocaleController.getString(R.string.UnitCNY)).append(SQLBuilder.PARENTHESES_RIGHT).create();
        this.bankCell.setData(R.mipmap.ic_bank_card, LocaleController.getString(R.string.BankCard), R.mipmap.icon_arrow_right, true);
        this.recordCell.setData(R.mipmap.ic_balance_change, LocaleController.getString(R.string.TransactionDetails2), R.mipmap.icon_arrow_right, true);
        this.bankCell.clearColorFilter();
        this.recordCell.clearColorFilter();
        this.bankCell.setTitleSize(16);
        this.recordCell.setTitleSize(16);
        this.bankCell.setTypeface(Typeface.DEFAULT_BOLD);
        this.recordCell.setTypeface(Typeface.DEFAULT_BOLD);
        this.bankCell.setBackground(Theme.getSelectorDrawable(false));
        this.recordCell.setBackground(Theme.getSelectorDrawable(false));
        this.tvWithdraw.setBackground(Theme.getSelectorDrawable(false));
        this.tvCharge.setBackground(Theme.getSelectorDrawable(false));
        showLoading();
        hideCash(true);
        this.tvWithdraw.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda0
            @Override // android.view.View.OnClickListener
            public final void onClick(View view) {
                WalletActivity.this.m3902lambda$initViews$0$imuecahxyvjduiwalletWalletActivity(view);
            }
        });
        this.tvCharge.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda1
            @Override // android.view.View.OnClickListener
            public final void onClick(View view) {
                WalletActivity.this.m3903lambda$initViews$1$imuecahxyvjduiwalletWalletActivity(view);
            }
        });
        this.bankCell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda2
            @Override // android.view.View.OnClickListener
            public final void onClick(View view) {
                WalletActivity.this.m3904lambda$initViews$2$imuecahxyvjduiwalletWalletActivity(view);
            }
        });
        this.recordCell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda3
            @Override // android.view.View.OnClickListener
            public final void onClick(View view) {
                WalletActivity.this.m3905lambda$initViews$3$imuecahxyvjduiwalletWalletActivity(view);
            }
        });
        this.ivHide.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda4
            @Override // android.view.View.OnClickListener
            public final void onClick(View view) {
                WalletActivity.this.m3906lambda$initViews$4$imuecahxyvjduiwalletWalletActivity(view);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initViews$0$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3902lambda$initViews$0$imuecahxyvjduiwalletWalletActivity(View view) {
        WalletWithdrawActivity fragment = new WalletWithdrawActivity();
        presentFragment(fragment);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initViews$1$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3903lambda$initViews$1$imuecahxyvjduiwalletWalletActivity(View view) {
        presentFragment(new WalletTopUpActivity());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initViews$2$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3904lambda$initViews$2$imuecahxyvjduiwalletWalletActivity(View view) {
        presentFragment(new WalletBankCardsActivity());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initViews$3$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3905lambda$initViews$3$imuecahxyvjduiwalletWalletActivity(View view) {
        presentFragment(new WalletRecordsActivity());
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$initViews$4$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3906lambda$initViews$4$imuecahxyvjduiwalletWalletActivity(View view) {
        boolean z = !this.hide;
        this.hide = z;
        hideCash(z);
    }

    private void hideCash(boolean hide) {
        if (hide) {
            this.tvTotal.setText("******");
            this.ivUnit.setVisibility(8);
            this.ivHide.setImageResource(R.mipmap.ic_wallet_total_no_view);
            return;
        }
        this.tvTotal.setText(MoneyUtil.formatToString(getWalletController().getAccountInfo().getCashAmount() / 100.0d, 2));
        this.ivUnit.setVisibility(0);
        this.ivHide.setImageResource(R.mipmap.ic_wallet_total_view);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void showLoading() {
        this.tipLayout.setVisibility(0);
        this.walletLayout.setVisibility(8);
        this.ivTip.setVisibility(8);
        this.loadView.setVisibility(0);
        this.tvTips.setTextColor(ColorUtils.getColor(R.color.text_descriptive_color));
        this.tvTips.setText(LocaleController.getString(R.string.NowLoading));
        this.tvDesc.setVisibility(8);
        this.btnSet.setVisibility(8);
    }

    private void showCreateTip() {
        this.menuItem.setVisibility(8);
        this.walletLayout.setVisibility(8);
        this.tipLayout.setVisibility(0);
        this.loadView.setVisibility(8);
        AnimationUtils.executeAlphaScaleDisplayAnimation(this.tipLayout);
        this.ivTip.setVisibility(0);
        this.tvDesc.setVisibility(0);
        this.btnSet.setVisibility(0);
        this.tvTips.setTextColor(ColorUtils.getColor(R.color.text_primary_color));
        this.tvTips.setText("钱包未创建");
        this.tvDesc.setText("点击下方按钮创建钱包");
        this.btnSet.setText("创建钱包");
        this.btnSet.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity.4
            @Override // android.view.View.OnClickListener
            public void onClick(View view) {
                WalletActivity.this.createWallet();
            }
        });
    }

    private void showLockTip() {
        this.menuItem.setVisibility(8);
        this.walletLayout.setVisibility(8);
        this.tipLayout.setVisibility(0);
        this.loadView.setVisibility(8);
        AnimationUtils.executeAlphaScaleDisplayAnimation(this.tipLayout);
        this.ivTip.setVisibility(0);
        this.tvDesc.setVisibility(0);
        this.btnSet.setVisibility(8);
        this.tvTips.setTextColor(ColorUtils.getColor(R.color.text_primary_color));
        this.tvTips.setText(LocaleController.getString(R.string.AccountHadBeenForzen));
        this.tvDesc.setText(LocaleController.getString(R.string.FreezeTips));
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void loadWalletInfo() {
        loadWalletInfo(false);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void loadWalletInfo(final boolean refresh) {
        TLRPCWallet.TL_getPaymentAccountInfo req = new TLRPCWallet.TL_getPaymentAccountInfo();
        getConnectionsManager().bindRequestToGuid(getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda6
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                WalletActivity.this.m3908lambda$loadWalletInfo$6$imuecahxyvjduiwalletWalletActivity(refresh, tLObject, tL_error);
            }
        }), this.classGuid);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadWalletInfo$6$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3908lambda$loadWalletInfo$6$imuecahxyvjduiwalletWalletActivity(final boolean refresh, final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda7
            @Override // java.lang.Runnable
            public final void run() {
                WalletActivity.this.m3907lambda$loadWalletInfo$5$imuecahxyvjduiwalletWalletActivity(refresh, error, response);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadWalletInfo$5$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3907lambda$loadWalletInfo$5$imuecahxyvjduiwalletWalletActivity(boolean refresh, TLRPC.TL_error error, TLObject response) {
        if (refresh) {
            this.refreshLayout.finishRefresh();
        }
        if (error != null) {
            SpinKitView spinKitView = this.loadView;
            if (spinKitView != null) {
                spinKitView.stop();
                this.loadView.setVisibility(8);
            }
            showError();
            ExceptionUtils.handleGetAccountInfoError(error.text);
        } else if (response instanceof TLRPCWallet.TL_paymentAccountInfoNotExist) {
            createWallet();
        } else {
            SpinKitView spinKitView2 = this.loadView;
            if (spinKitView2 != null) {
                spinKitView2.stop();
                this.loadView.setVisibility(8);
            }
            TLApiModel<WalletAccountInfo> model = TLJsonResolve.parse(response, WalletAccountInfo.class);
            if (model.isSuccess()) {
                WalletAccountInfo accountInfo = (WalletAccountInfo) model.model;
                getWalletController().setAccountInfo(accountInfo);
                WalletConfigBean.setWalletAccountInfo(accountInfo);
                WalletConfigBean.setConfigValue(((WalletAccountInfo) model.model).getRiskList());
                if (accountInfo.isLocked()) {
                    showLockTip();
                    return;
                } else {
                    showAccountInfo(refresh);
                    return;
                }
            }
            showError();
            ExceptionUtils.handleGetAccountInfoError(model.message);
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void createWallet() {
        TLRPCWallet.TL_createAccount req = new TLRPCWallet.TL_createAccount();
        getConnectionsManager().bindRequestToGuid(getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity$$ExternalSyntheticLambda5
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                WalletActivity.this.m3901lambda$createWallet$7$imuecahxyvjduiwalletWalletActivity(tLObject, tL_error);
            }
        }), this.classGuid);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createWallet$7$im-uecahxyvjd-ui-wallet-WalletActivity  reason: not valid java name */
    public /* synthetic */ void m3901lambda$createWallet$7$imuecahxyvjduiwalletWalletActivity(final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity.5
            @Override // java.lang.Runnable
            public void run() {
                if (WalletActivity.this.loadView != null) {
                    WalletActivity.this.loadView.stop();
                    WalletActivity.this.loadView.setVisibility(8);
                }
                if (error != null) {
                    WalletActivity.this.showError();
                    ToastUtils.show((CharSequence) LocaleController.getString(R.string.SystemIsBusyAndTryAgainLater));
                    return;
                }
                TLObject tLObject = response;
                if (tLObject instanceof TLRPCWallet.TL_paymentAccountInfo) {
                    TLApiModel<WalletAccountInfo> model = TLJsonResolve.parse(tLObject, WalletAccountInfo.class);
                    if (!model.isSuccess()) {
                        WalletActivity.this.showError();
                        ExceptionUtils.handleCreateAccountError(model.message);
                        return;
                    }
                    WalletAccountInfo accountInfo = (WalletAccountInfo) model.model;
                    WalletConfigBean.setWalletAccountInfo(accountInfo);
                    WalletConfigBean.setConfigValue(((WalletAccountInfo) model.model).getRiskList());
                    WalletActivity.this.getWalletController().setAccountInfo(accountInfo);
                    WalletActivity.this.showAccountInfo(false);
                }
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void showAccountInfo(boolean refresh) {
        this.menuItem.setVisibility(0);
        this.walletLayout.setVisibility(0);
        this.tipLayout.setVisibility(8);
        if (!refresh) {
            AnimationUtils.executeAlphaScaleDisplayAnimation(this.walletLayout);
        }
        hideCash(this.hide);
    }

    private void showPasswordTip() {
        this.menuItem.setVisibility(8);
        this.walletLayout.setVisibility(8);
        this.tipLayout.setVisibility(0);
        this.loadView.setVisibility(8);
        this.ivTip.setVisibility(0);
        this.tvDesc.setVisibility(0);
        this.btnSet.setVisibility(0);
        this.tvTips.setTextColor(ColorUtils.getColor(R.color.text_primary_color));
        this.tvTips.setText(LocaleController.getString(R.string.NoPaymentPassword));
        this.tvDesc.setText(LocaleController.getString(R.string.ThisFunNeedPayPassword));
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void showError() {
        this.menuItem.setVisibility(8);
        this.walletLayout.setVisibility(8);
        this.tipLayout.setVisibility(0);
        this.loadView.setVisibility(8);
        AnimationUtils.executeAlphaScaleDisplayAnimation(this.tipLayout);
        this.ivTip.setVisibility(0);
        this.ivTip.setImageResource(R.mipmap.ic_data_ex);
        this.tvDesc.setVisibility(0);
        this.btnSet.setVisibility(0);
        this.tvTips.setTextColor(ColorUtils.getColor(R.color.text_primary_color));
        this.tvTips.setText(LocaleController.getString(R.string.SystemIsBusyAndTryAgainLater));
        this.tvDesc.setText(LocaleController.getString(R.string.ClickTheButtonToTryAgain));
        this.btnSet.setText(LocaleController.getString(R.string.Refresh));
        this.btnSet.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletActivity.6
            @Override // android.view.View.OnClickListener
            public void onClick(View view) {
                if (WalletActivity.this.getWalletController().getAccountInfo() == null) {
                    WalletActivity.this.showLoading();
                    WalletActivity.this.loadWalletInfo();
                }
            }
        });
    }

    private void parseError(int errorCode, String errorMsg) {
        WalletDialogUtil.showConfirmBtnWalletDialog(this, WalletErrorUtil.getErrorDescription(errorMsg));
    }

    @Override // im.uecahxyvjd.messenger.NotificationCenter.NotificationCenterDelegate
    public void didReceivedNotification(int id, int account, Object... args) {
        if (id == NotificationCenter.walletInfoNeedReload) {
            showLoading();
            loadWalletInfo();
        } else if (id == NotificationCenter.paymentPasswordDidSet && getWalletController().getAccountInfo() != null) {
            getWalletController().getAccountInfo().setIsSetPayWord("1");
        }
    }
}
