package im.uecahxyvjd.ui.wallet;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import im.uecahxyvjd.messenger.AndroidUtilities;
import im.uecahxyvjd.messenger.LocaleController;
import im.uecahxyvjd.messenger.NotificationCenter;
import im.uecahxyvjd.messenger.R;
import im.uecahxyvjd.tgnet.RequestDelegate;
import im.uecahxyvjd.tgnet.TLApiModel;
import im.uecahxyvjd.tgnet.TLJsonResolve;
import im.uecahxyvjd.tgnet.TLObject;
import im.uecahxyvjd.tgnet.TLRPC;
import im.uecahxyvjd.tgnet.TLRPCWallet;
import im.uecahxyvjd.ui.actionbar.ActionBar;
import im.uecahxyvjd.ui.actionbar.BaseFragment;
import im.uecahxyvjd.ui.actionbar.Theme;
import im.uecahxyvjd.ui.components.TextCell;
import im.uecahxyvjd.ui.components.toast.ToastUtils;
import im.uecahxyvjd.ui.hui.wallet_public.bean.WalletAccountInfo;
import im.uecahxyvjd.ui.hui.wallet_public.bean.WalletConfigBean;
import im.uecahxyvjd.ui.wallet.utils.ExceptionUtils;
/* loaded from: classes18.dex */
public class WalletManagementActivity extends BaseFragment implements NotificationCenter.NotificationCenterDelegate {
    private TextCell passwordCell;

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public boolean onFragmentCreate() {
        super.onFragmentCreate();
        NotificationCenter.getInstance(this.currentAccount).addObserver(this, NotificationCenter.paymentPasswordDidSet);
        return true;
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public void onFragmentDestroy() {
        super.onFragmentDestroy();
        NotificationCenter.getInstance(this.currentAccount).removeObserver(this, NotificationCenter.paymentPasswordDidSet);
    }

    @Override // im.uecahxyvjd.ui.actionbar.BaseFragment
    public View createView(Context context) {
        this.fragmentView = LayoutInflater.from(context).inflate(R.layout.activity_wallet_actions_layout, (ViewGroup) null);
        initActionBar();
        initViews();
        if (getWalletController().getAccountInfo() == null) {
            loadWalletInfo();
        }
        return this.fragmentView;
    }

    private void initActionBar() {
        this.actionBar.setTitle(LocaleController.getString(R.string.WalletManagement));
        this.actionBar.setBackButtonImage(R.mipmap.ic_back);
        this.actionBar.setCastShadows(false);
        this.actionBar.setActionBarMenuOnItemClick(new ActionBar.ActionBarMenuOnItemClick() { // from class: im.uecahxyvjd.ui.wallet.WalletManagementActivity.1
            @Override // im.uecahxyvjd.ui.actionbar.ActionBar.ActionBarMenuOnItemClick
            public void onItemClick(int id) {
                if (id == -1) {
                    WalletManagementActivity.this.finishFragment();
                }
            }
        });
    }

    private void initViews() {
        this.passwordCell = (TextCell) this.fragmentView.findViewById(R.id.passwordCell);
        setPasswordCell();
        this.passwordCell.setBackground(Theme.getSelectorDrawable(false));
        this.passwordCell.setOnClickListener(new View.OnClickListener() { // from class: im.uecahxyvjd.ui.wallet.WalletManagementActivity.2
            @Override // android.view.View.OnClickListener
            public void onClick(View view) {
                if (WalletManagementActivity.this.getWalletController().getAccountInfo() == null) {
                    return;
                }
                if (WalletManagementActivity.this.getWalletController().getAccountInfo().hasPaypassword()) {
                    Bundle args = new Bundle();
                    args.putInt("step", 0);
                    args.putInt("type", 1);
                    WalletManagementActivity.this.presentFragment(new WalletPaymentPasswordActivity(args));
                    return;
                }
                Bundle args2 = new Bundle();
                args2.putInt("step", 0);
                args2.putInt("type", 0);
                WalletManagementActivity.this.presentFragment(new WalletPaymentPasswordActivity(args2));
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public void setPasswordCell() {
        if (getWalletController().getAccountInfo() != null) {
            if (getWalletController().getAccountInfo().hasPaypassword()) {
                this.passwordCell.setText(R.mipmap.profile_shareout, LocaleController.getString(R.string.ModifyPayPassword), true, false);
            } else {
                this.passwordCell.setText(R.mipmap.profile_shareout, LocaleController.getString(R.string.SetPayPassword), true, false);
            }
        }
    }

    private void loadWalletInfo() {
        TLRPCWallet.TL_getPaymentAccountInfo req = new TLRPCWallet.TL_getPaymentAccountInfo();
        getConnectionsManager().bindRequestToGuid(getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.wallet.WalletManagementActivity$$ExternalSyntheticLambda0
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                WalletManagementActivity.this.m3914xed685482(tLObject, tL_error);
            }
        }), this.classGuid);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadWalletInfo$1$im-uecahxyvjd-ui-wallet-WalletManagementActivity  reason: not valid java name */
    public /* synthetic */ void m3914xed685482(final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.wallet.WalletManagementActivity$$ExternalSyntheticLambda2
            @Override // java.lang.Runnable
            public final void run() {
                WalletManagementActivity.this.m3913xfbbeae63(error, response);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$loadWalletInfo$0$im-uecahxyvjd-ui-wallet-WalletManagementActivity  reason: not valid java name */
    public /* synthetic */ void m3913xfbbeae63(TLRPC.TL_error error, TLObject response) {
        if (error != null) {
            ExceptionUtils.handleGetAccountInfoError(error.text);
        } else if (response instanceof TLRPCWallet.TL_paymentAccountInfoNotExist) {
            createWallet();
        } else {
            TLApiModel<WalletAccountInfo> model = TLJsonResolve.parse(response, WalletAccountInfo.class);
            if (model.isSuccess()) {
                WalletAccountInfo accountInfo = (WalletAccountInfo) model.model;
                getWalletController().setAccountInfo(accountInfo);
                WalletConfigBean.setWalletAccountInfo(accountInfo);
                WalletConfigBean.setConfigValue(((WalletAccountInfo) model.model).getRiskList());
                setPasswordCell();
                return;
            }
            ExceptionUtils.handleGetAccountInfoError(model.message);
        }
    }

    private void createWallet() {
        TLRPCWallet.TL_createAccount req = new TLRPCWallet.TL_createAccount();
        getConnectionsManager().bindRequestToGuid(getConnectionsManager().sendRequest(req, new RequestDelegate() { // from class: im.uecahxyvjd.ui.wallet.WalletManagementActivity$$ExternalSyntheticLambda1
            @Override // im.uecahxyvjd.tgnet.RequestDelegate
            public final void run(TLObject tLObject, TLRPC.TL_error tL_error) {
                WalletManagementActivity.this.m3912xb902f019(tLObject, tL_error);
            }
        }), this.classGuid);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* renamed from: lambda$createWallet$2$im-uecahxyvjd-ui-wallet-WalletManagementActivity  reason: not valid java name */
    public /* synthetic */ void m3912xb902f019(final TLObject response, final TLRPC.TL_error error) {
        AndroidUtilities.runOnUIThread(new Runnable() { // from class: im.uecahxyvjd.ui.wallet.WalletManagementActivity.3
            @Override // java.lang.Runnable
            public void run() {
                if (error != null) {
                    ToastUtils.show((CharSequence) LocaleController.getString(R.string.SystemIsBusyAndTryAgainLater));
                    return;
                }
                TLObject tLObject = response;
                if (tLObject instanceof TLRPCWallet.TL_paymentAccountInfo) {
                    TLApiModel<WalletAccountInfo> model = TLJsonResolve.parse(tLObject, WalletAccountInfo.class);
                    if (model.isSuccess()) {
                        WalletAccountInfo accountInfo = (WalletAccountInfo) model.model;
                        WalletConfigBean.setWalletAccountInfo(accountInfo);
                        WalletConfigBean.setConfigValue(((WalletAccountInfo) model.model).getRiskList());
                        WalletManagementActivity.this.getWalletController().setAccountInfo(accountInfo);
                        WalletManagementActivity.this.setPasswordCell();
                        return;
                    }
                    ExceptionUtils.handleCreateAccountError(model.message);
                }
            }
        });
    }

    @Override // im.uecahxyvjd.messenger.NotificationCenter.NotificationCenterDelegate
    public void didReceivedNotification(int id, int account, Object... args) {
        if (id == NotificationCenter.paymentPasswordDidSet && getWalletController().getAccountInfo() != null) {
            getWalletController().getAccountInfo().setIsSetPayWord("1");
            setPasswordCell();
        }
    }
}
