package java9.util;

import com.google.android.exoplayer2.text.ttml.TtmlNode;
import java.util.ArrayDeque;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java9.util.Spliterator;
import java9.util.function.Consumer;
import sun.misc.Unsafe;
/* loaded from: classes19.dex */
public final class ArrayDequeSpliterator<E> implements Spliterator<E> {
    public static final long DATA_OFF;
    public static final long HEAD_OFF;
    public static final long TAIL_OFF;
    public static final Unsafe U;
    public final ArrayDeque<E> deq;
    public int fence;
    public int index;

    static {
        Unsafe unsafe = UnsafeAccess.unsafe;
        U = unsafe;
        try {
            TAIL_OFF = unsafe.objectFieldOffset(ArrayDeque.class.getDeclaredField("tail"));
            HEAD_OFF = unsafe.objectFieldOffset(ArrayDeque.class.getDeclaredField(TtmlNode.TAG_HEAD));
            DATA_OFF = unsafe.objectFieldOffset(ArrayDeque.class.getDeclaredField("elements"));
        } catch (Exception e) {
            throw new Error(e);
        }
    }

    public ArrayDequeSpliterator(ArrayDeque<E> arrayDeque, int i, int i2) {
        this.deq = arrayDeque;
        this.index = i;
        this.fence = i2;
    }

    public static <T> Object[] getData(ArrayDeque<T> arrayDeque) {
        return (Object[]) U.getObject(arrayDeque, DATA_OFF);
    }

    private int getFence() {
        int i = this.fence;
        if (i < 0) {
            int tail = getTail(this.deq);
            this.fence = tail;
            this.index = getHead(this.deq);
            return tail;
        }
        return i;
    }

    public static <T> int getHead(ArrayDeque<T> arrayDeque) {
        return U.getInt(arrayDeque, HEAD_OFF);
    }

    public static <T> int getTail(ArrayDeque<T> arrayDeque) {
        return U.getInt(arrayDeque, TAIL_OFF);
    }

    public static <T> Spliterator<T> spliterator(ArrayDeque<T> arrayDeque) {
        return new ArrayDequeSpliterator(arrayDeque, -1, -1);
    }

    @Override // java9.util.Spliterator
    public int characteristics() {
        return 16720;
    }

    @Override // java9.util.Spliterator
    public long estimateSize() {
        int fence = getFence() - this.index;
        if (fence < 0) {
            fence += getData(this.deq).length;
        }
        return fence;
    }

    @Override // java9.util.Spliterator
    public void forEachRemaining(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        Object[] data = getData(this.deq);
        int length = data.length - 1;
        int fence = getFence();
        int i = this.index;
        this.index = fence;
        while (i != fence) {
            Object obj = data[i];
            i = (i + 1) & length;
            if (obj == null) {
                throw new ConcurrentModificationException();
            }
            consumer.accept(obj);
        }
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ Comparator getComparator() {
        return Spliterator.CC.$default$getComparator(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ long getExactSizeIfKnown() {
        return Spliterator.CC.$default$getExactSizeIfKnown(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ boolean hasCharacteristics(int i) {
        return Spliterator.CC.$default$hasCharacteristics(this, i);
    }

    @Override // java9.util.Spliterator
    public boolean tryAdvance(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        Object[] data = getData(this.deq);
        int length = data.length - 1;
        getFence();
        int i = this.index;
        if (i != this.fence) {
            Object obj = data[i];
            this.index = length & (i + 1);
            if (obj != null) {
                consumer.accept(obj);
                return true;
            }
            throw new ConcurrentModificationException();
        }
        return false;
    }

    @Override // java9.util.Spliterator
    public ArrayDequeSpliterator<E> trySplit() {
        int fence = getFence();
        int i = this.index;
        int length = getData(this.deq).length;
        if (i != fence) {
            int i2 = length - 1;
            if (((i + 1) & i2) != fence) {
                if (i > fence) {
                    fence += length;
                }
                int i3 = ((fence + i) >>> 1) & i2;
                ArrayDeque<E> arrayDeque = this.deq;
                this.index = i3;
                return new ArrayDequeSpliterator<>(arrayDeque, i, i3);
            }
            return null;
        }
        return null;
    }
}
