package java9.util;

import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
import java.util.Arrays;
import java9.util.concurrent.CountedCompleter;
import java9.util.concurrent.RecursiveTask;
import kotlin.UByte;
/* loaded from: classes19.dex */
public final class DualPivotQuicksort {
    public static final int DELTA = 6;
    public static final int MAX_BYTE_INDEX = 384;
    public static final int MAX_INSERTION_SORT_SIZE = 44;
    public static final int MAX_MIXED_INSERTION_SORT_SIZE = 65;
    public static final int MAX_RECURSION_DEPTH = 384;
    public static final int MAX_RUN_CAPACITY = 5120;
    public static final int MAX_SHORT_INDEX = 98304;
    public static final int MIN_BYTE_COUNTING_SORT_SIZE = 64;
    public static final int MIN_FIRST_RUNS_FACTOR = 7;
    public static final int MIN_FIRST_RUN_SIZE = 16;
    public static final int MIN_PARALLEL_MERGE_PARTS_SIZE = 4096;
    public static final int MIN_PARALLEL_SORT_SIZE = 4096;
    public static final int MIN_RUN_COUNT = 4;
    public static final int MIN_SHORT_OR_CHAR_COUNTING_SORT_SIZE = 1750;
    public static final int MIN_TRY_MERGE_SIZE = 4096;
    public static final int NUM_BYTE_VALUES = 256;
    public static final int NUM_CHAR_VALUES = 65536;
    public static final int NUM_SHORT_VALUES = 65536;

    /* loaded from: classes19.dex */
    public static final class Merger extends CountedCompleter<Void> {
        public static final long serialVersionUID = 20180818;
        public final Object a1;
        public final Object a2;
        public final Object dst;
        public final int hi1;
        public final int hi2;
        public final int k;
        public final int lo1;
        public final int lo2;

        public Merger(CountedCompleter<?> countedCompleter, Object obj, int i, Object obj2, int i2, int i3, Object obj3, int i4, int i5) {
            super(countedCompleter);
            this.dst = obj;
            this.k = i;
            this.a1 = obj2;
            this.lo1 = i2;
            this.hi1 = i3;
            this.a2 = obj3;
            this.lo2 = i4;
            this.hi2 = i5;
        }

        @Override // java9.util.concurrent.CountedCompleter
        public final void compute() {
            Object obj = this.dst;
            if (obj instanceof int[]) {
                DualPivotQuicksort.mergeParts(this, (int[]) obj, this.k, (int[]) this.a1, this.lo1, this.hi1, (int[]) this.a2, this.lo2, this.hi2);
            } else if (obj instanceof long[]) {
                DualPivotQuicksort.mergeParts(this, (long[]) obj, this.k, (long[]) this.a1, this.lo1, this.hi1, (long[]) this.a2, this.lo2, this.hi2);
            } else if (obj instanceof float[]) {
                DualPivotQuicksort.mergeParts(this, (float[]) obj, this.k, (float[]) this.a1, this.lo1, this.hi1, (float[]) this.a2, this.lo2, this.hi2);
            } else if (!(obj instanceof double[])) {
                throw new IllegalArgumentException("Unknown type of array: " + this.dst.getClass().getName());
            } else {
                DualPivotQuicksort.mergeParts(this, (double[]) obj, this.k, (double[]) this.a1, this.lo1, this.hi1, (double[]) this.a2, this.lo2, this.hi2);
            }
            propagateCompletion();
        }

        public void forkMerger(Object obj, int i, Object obj2, int i2, int i3, Object obj3, int i4, int i5) {
            addToPendingCount(1);
            new Merger(this, obj, i, obj2, i2, i3, obj3, i4, i5).fork();
        }
    }

    /* loaded from: classes19.dex */
    public static final class RunMerger extends RecursiveTask<Object> {
        public static final long serialVersionUID = 20180818;

        /* renamed from: a  reason: collision with root package name */
        public final Object f690a;
        public final int aim;
        public final Object b;
        public final int hi;
        public final int lo;
        public final int offset;
        public final int[] run;

        public RunMerger(Object obj, Object obj2, int i, int i2, int[] iArr, int i3, int i4) {
            this.f690a = obj;
            this.b = obj2;
            this.offset = i;
            this.aim = i2;
            this.run = iArr;
            this.lo = i3;
            this.hi = i4;
        }

        @Override // java9.util.concurrent.RecursiveTask
        public final Object compute() {
            Object obj = this.f690a;
            if (obj instanceof int[]) {
                return DualPivotQuicksort.mergeRuns((int[]) obj, (int[]) this.b, this.offset, this.aim, true, this.run, this.lo, this.hi);
            }
            if (obj instanceof long[]) {
                return DualPivotQuicksort.mergeRuns((long[]) obj, (long[]) this.b, this.offset, this.aim, true, this.run, this.lo, this.hi);
            }
            if (obj instanceof float[]) {
                return DualPivotQuicksort.mergeRuns((float[]) obj, (float[]) this.b, this.offset, this.aim, true, this.run, this.lo, this.hi);
            }
            if (obj instanceof double[]) {
                return DualPivotQuicksort.mergeRuns((double[]) obj, (double[]) this.b, this.offset, this.aim, true, this.run, this.lo, this.hi);
            }
            throw new IllegalArgumentException("Unknown type of array: " + this.f690a.getClass().getName());
        }

        public RunMerger forkMe() {
            fork();
            return this;
        }

        public Object getDestination() {
            join();
            return getRawResult();
        }
    }

    /* loaded from: classes19.dex */
    public static final class Sorter extends CountedCompleter<Void> {
        public static final long serialVersionUID = 20180818;

        /* renamed from: a  reason: collision with root package name */
        public final Object f691a;
        public final Object b;
        public final int depth;
        public final int low;
        public final int offset;
        public final int size;

        public Sorter(CountedCompleter<?> countedCompleter, Object obj, Object obj2, int i, int i2, int i3, int i4) {
            super(countedCompleter);
            this.f691a = obj;
            this.b = obj2;
            this.low = i;
            this.size = i2;
            this.offset = i3;
            this.depth = i4;
        }

        @Override // java9.util.concurrent.CountedCompleter
        public final void compute() {
            int i = this.depth;
            if (i < 0) {
                setPendingCount(2);
                int i2 = this.size >> 1;
                new Sorter(this, this.b, this.f691a, this.low, i2, this.offset, this.depth + 1).fork();
                new Sorter(this, this.b, this.f691a, this.low + i2, this.size - i2, this.offset, this.depth + 1).compute();
            } else {
                Object obj = this.f691a;
                if (obj instanceof int[]) {
                    int i3 = this.low;
                    DualPivotQuicksort.sort(this, (int[]) obj, i, i3, this.size + i3);
                } else if (obj instanceof long[]) {
                    int i4 = this.low;
                    DualPivotQuicksort.sort(this, (long[]) obj, i, i4, this.size + i4);
                } else if (obj instanceof float[]) {
                    int i5 = this.low;
                    DualPivotQuicksort.sort(this, (float[]) obj, i, i5, this.size + i5);
                } else if (!(obj instanceof double[])) {
                    throw new IllegalArgumentException("Unknown type of array: " + this.f691a.getClass().getName());
                } else {
                    int i6 = this.low;
                    DualPivotQuicksort.sort(this, (double[]) obj, i, i6, this.size + i6);
                }
            }
            tryComplete();
        }

        public void forkSorter(int i, int i2, int i3) {
            addToPendingCount(1);
            new Sorter(this, this.f691a, this.b, i2, i3 - i2, this.offset, i).fork();
        }

        @Override // java9.util.concurrent.CountedCompleter
        public final void onCompletion(CountedCompleter<?> countedCompleter) {
            int i = this.depth;
            if (i < 0) {
                int i2 = this.low + (this.size >> 1);
                boolean z = (i & 1) == 0;
                Object obj = this.f691a;
                int i3 = this.low;
                if (!z) {
                    i3 -= this.offset;
                }
                int i4 = i3;
                Object obj2 = this.b;
                int i5 = this.low;
                if (z) {
                    i5 -= this.offset;
                }
                int i6 = i5;
                int i7 = z ? i2 - this.offset : i2;
                if (z) {
                    i2 -= this.offset;
                }
                int i8 = i2;
                int i9 = this.low + this.size;
                if (z) {
                    i9 -= this.offset;
                }
                new Merger(null, obj, i4, obj2, i6, i7, obj2, i8, i9).invoke();
            }
        }
    }

    public static void countingSort(byte[] bArr, int i, int i2) {
        int i3;
        int i4;
        int[] iArr = new int[256];
        int i5 = i2;
        while (i5 > i) {
            i5--;
            int i6 = bArr[i5] & UByte.MAX_VALUE;
            iArr[i6] = iArr[i6] + 1;
        }
        int i7 = 384;
        if (i2 - i <= 256) {
            while (i2 > i) {
                do {
                    i7--;
                    i3 = i7 & 255;
                    i4 = iArr[i3];
                } while (i4 == 0);
                do {
                    i2--;
                    bArr[i2] = (byte) i3;
                    i4--;
                } while (i4 > 0);
            }
            return;
        }
        while (true) {
            i7--;
            if (i7 <= 127) {
                return;
            }
            int i8 = i7 & 255;
            int i9 = i2 - iArr[i8];
            while (i2 > i9) {
                i2--;
                bArr[i2] = (byte) i8;
            }
        }
    }

    public static void countingSort(char[] cArr, int i, int i2) {
        int i3;
        int i4 = 65536;
        int[] iArr = new int[65536];
        int i5 = i2;
        while (i5 > i) {
            i5--;
            char c = cArr[i5];
            iArr[c] = iArr[c] + 1;
        }
        if (i2 - i <= 65536) {
            while (i2 > i) {
                do {
                    i4--;
                    i3 = iArr[i4];
                } while (i3 == 0);
                do {
                    i2--;
                    cArr[i2] = (char) i4;
                    i3--;
                } while (i3 > 0);
            }
            return;
        }
        while (i4 > 0) {
            i4--;
            int i6 = i2 - iArr[i4];
            while (i2 > i6) {
                i2--;
                cArr[i2] = (char) i4;
            }
        }
    }

    public static void countingSort(short[] sArr, int i, int i2) {
        int i3;
        int i4;
        int[] iArr = new int[65536];
        int i5 = i2;
        while (i5 > i) {
            i5--;
            int i6 = 65535 & sArr[i5];
            iArr[i6] = iArr[i6] + 1;
        }
        int i7 = i2 - i;
        int i8 = MAX_SHORT_INDEX;
        if (i7 <= 65536) {
            while (i2 > i) {
                do {
                    i8--;
                    i3 = i8 & 65535;
                    i4 = iArr[i3];
                } while (i4 == 0);
                do {
                    i2--;
                    sArr[i2] = (short) i3;
                    i4--;
                } while (i4 > 0);
            }
            return;
        }
        while (true) {
            i8--;
            if (i8 <= 32767) {
                return;
            }
            int i9 = i8 & 65535;
            int i10 = i2 - iArr[i9];
            while (i2 > i10) {
                i2--;
                sArr[i2] = (short) i9;
            }
        }
    }

    public static int getDepth(int i, int i2) {
        int i3 = 0;
        while (true) {
            i >>= 3;
            if (i <= 0 || (i2 = i2 >> 2) <= 0) {
                break;
            }
            i3 -= 2;
        }
        return i3;
    }

    public static void heapSort(double[] dArr, int i, int i2) {
        int i3 = (i + i2) >>> 1;
        while (i3 > i) {
            i3--;
            pushDown(dArr, i3, dArr[i3], i, i2);
        }
        while (true) {
            i2--;
            if (i2 <= i) {
                return;
            }
            double d = dArr[i];
            pushDown(dArr, i, dArr[i2], i, i2);
            dArr[i2] = d;
        }
    }

    public static void heapSort(float[] fArr, int i, int i2) {
        int i3 = (i + i2) >>> 1;
        while (i3 > i) {
            i3--;
            pushDown(fArr, i3, fArr[i3], i, i2);
        }
        while (true) {
            i2--;
            if (i2 <= i) {
                return;
            }
            float f = fArr[i];
            pushDown(fArr, i, fArr[i2], i, i2);
            fArr[i2] = f;
        }
    }

    public static void heapSort(int[] iArr, int i, int i2) {
        int i3 = (i + i2) >>> 1;
        while (i3 > i) {
            i3--;
            pushDown(iArr, i3, iArr[i3], i, i2);
        }
        while (true) {
            i2--;
            if (i2 <= i) {
                return;
            }
            int i4 = iArr[i];
            pushDown(iArr, i, iArr[i2], i, i2);
            iArr[i2] = i4;
        }
    }

    public static void heapSort(long[] jArr, int i, int i2) {
        int i3 = (i + i2) >>> 1;
        while (i3 > i) {
            i3--;
            pushDown(jArr, i3, jArr[i3], i, i2);
        }
        while (true) {
            i2--;
            if (i2 <= i) {
                return;
            }
            long j = jArr[i];
            pushDown(jArr, i, jArr[i2], i, i2);
            jArr[i2] = j;
        }
    }

    public static void insertionSort(byte[] bArr, int i, int i2) {
        byte b;
        int i3 = i;
        while (true) {
            i3++;
            if (i3 >= i2) {
                return;
            }
            byte b2 = bArr[i3];
            if (b2 < bArr[i3 - 1]) {
                int i4 = i3;
                while (true) {
                    i4--;
                    if (i4 < i || b2 >= (b = bArr[i4])) {
                        break;
                    }
                    bArr[i4 + 1] = b;
                }
                bArr[i4 + 1] = b2;
            }
        }
    }

    public static void insertionSort(char[] cArr, int i, int i2) {
        char c;
        int i3 = i;
        while (true) {
            i3++;
            if (i3 >= i2) {
                return;
            }
            char c2 = cArr[i3];
            if (c2 < cArr[i3 - 1]) {
                int i4 = i3;
                while (true) {
                    i4--;
                    if (i4 < i || c2 >= (c = cArr[i4])) {
                        break;
                    }
                    cArr[i4 + 1] = c;
                }
                cArr[i4 + 1] = c2;
            }
        }
    }

    public static void insertionSort(double[] dArr, int i, int i2) {
        int i3 = i;
        while (true) {
            i3++;
            if (i3 >= i2) {
                return;
            }
            double d = dArr[i3];
            if (d < dArr[i3 - 1]) {
                int i4 = i3;
                while (true) {
                    i4--;
                    if (i4 < i) {
                        break;
                    }
                    double d2 = dArr[i4];
                    if (d >= d2) {
                        break;
                    }
                    dArr[i4 + 1] = d2;
                }
                dArr[i4 + 1] = d;
            }
        }
    }

    public static void insertionSort(float[] fArr, int i, int i2) {
        int i3 = i;
        while (true) {
            i3++;
            if (i3 >= i2) {
                return;
            }
            float f = fArr[i3];
            if (f < fArr[i3 - 1]) {
                int i4 = i3;
                while (true) {
                    i4--;
                    if (i4 < i) {
                        break;
                    }
                    float f2 = fArr[i4];
                    if (f >= f2) {
                        break;
                    }
                    fArr[i4 + 1] = f2;
                }
                fArr[i4 + 1] = f;
            }
        }
    }

    public static void insertionSort(int[] iArr, int i, int i2) {
        int i3;
        int i4 = i;
        while (true) {
            i4++;
            if (i4 >= i2) {
                return;
            }
            int i5 = iArr[i4];
            if (i5 < iArr[i4 - 1]) {
                int i6 = i4;
                while (true) {
                    i6--;
                    if (i6 < i || i5 >= (i3 = iArr[i6])) {
                        break;
                    }
                    iArr[i6 + 1] = i3;
                }
                iArr[i6 + 1] = i5;
            }
        }
    }

    public static void insertionSort(long[] jArr, int i, int i2) {
        int i3 = i;
        while (true) {
            i3++;
            if (i3 >= i2) {
                return;
            }
            long j = jArr[i3];
            if (j < jArr[i3 - 1]) {
                int i4 = i3;
                while (true) {
                    i4--;
                    if (i4 < i) {
                        break;
                    }
                    long j2 = jArr[i4];
                    if (j >= j2) {
                        break;
                    }
                    jArr[i4 + 1] = j2;
                }
                jArr[i4 + 1] = j;
            }
        }
    }

    public static void insertionSort(short[] sArr, int i, int i2) {
        short s;
        int i3 = i;
        while (true) {
            i3++;
            if (i3 >= i2) {
                return;
            }
            short s2 = sArr[i3];
            if (s2 < sArr[i3 - 1]) {
                int i4 = i3;
                while (true) {
                    i4--;
                    if (i4 < i || s2 >= (s = sArr[i4])) {
                        break;
                    }
                    sArr[i4 + 1] = s;
                }
                sArr[i4 + 1] = s2;
            }
        }
    }

    public static void mergeParts(Merger merger, double[] dArr, int i, double[] dArr2, int i2, int i3, double[] dArr3, int i4, int i5) {
        int i6;
        int i7;
        int i8;
        int i9;
        int i10;
        if (merger == null || dArr2 != dArr3) {
            i6 = i;
            i7 = i2;
            i8 = i3;
            i9 = i4;
            i10 = i5;
        } else {
            int i11 = i2;
            int i12 = i3;
            int i13 = i4;
            int i14 = i5;
            while (true) {
                if (i12 - i11 < i14 - i13) {
                    i9 = i11;
                    i10 = i12;
                    i7 = i13;
                    i8 = i14;
                } else {
                    i7 = i11;
                    i8 = i12;
                    i9 = i13;
                    i10 = i14;
                }
                if (i8 - i7 < 4096) {
                    break;
                }
                int i15 = (i7 + i8) >>> 1;
                double d = dArr2[i15];
                int i16 = i10;
                int i17 = i9;
                while (i17 < i16) {
                    int i18 = (i17 + i16) >>> 1;
                    if (d > dArr3[i18]) {
                        i17 = i18 + 1;
                    } else {
                        i16 = i18;
                    }
                }
                merger.forkMerger(dArr, i + (((i16 - i9) + i15) - i7), dArr2, i15, i8, dArr3, i16, i10);
                i11 = i7;
                i13 = i9;
                i12 = i15;
                i14 = i16;
            }
            i6 = i;
        }
        while (i7 < i8 && i9 < i10) {
            int i19 = i6 + 1;
            double d2 = dArr2[i7];
            double d3 = dArr3[i9];
            if (d2 < d3) {
                i7++;
            } else {
                i9++;
                d2 = d3;
            }
            dArr[i6] = d2;
            i6 = i19;
        }
        if (dArr != dArr2 || i6 < i7) {
            while (i7 < i8) {
                dArr[i6] = dArr2[i7];
                i6++;
                i7++;
            }
        }
        if (dArr != dArr3 || i6 < i9) {
            while (i9 < i10) {
                dArr[i6] = dArr3[i9];
                i6++;
                i9++;
            }
        }
    }

    public static void mergeParts(Merger merger, float[] fArr, int i, float[] fArr2, int i2, int i3, float[] fArr3, int i4, int i5) {
        int i6;
        int i7;
        int i8;
        int i9;
        int i10;
        if (merger == null || fArr2 != fArr3) {
            i6 = i;
            i7 = i2;
            i8 = i3;
            i9 = i4;
            i10 = i5;
        } else {
            int i11 = i2;
            int i12 = i3;
            int i13 = i4;
            int i14 = i5;
            while (true) {
                if (i12 - i11 < i14 - i13) {
                    i9 = i11;
                    i10 = i12;
                    i7 = i13;
                    i8 = i14;
                } else {
                    i7 = i11;
                    i8 = i12;
                    i9 = i13;
                    i10 = i14;
                }
                if (i8 - i7 < 4096) {
                    break;
                }
                int i15 = (i7 + i8) >>> 1;
                float f = fArr2[i15];
                int i16 = i10;
                int i17 = i9;
                while (i17 < i16) {
                    int i18 = (i17 + i16) >>> 1;
                    if (f > fArr3[i18]) {
                        i17 = i18 + 1;
                    } else {
                        i16 = i18;
                    }
                }
                merger.forkMerger(fArr, i + (((i16 - i9) + i15) - i7), fArr2, i15, i8, fArr3, i16, i10);
                i11 = i7;
                i13 = i9;
                i12 = i15;
                i14 = i16;
            }
            i6 = i;
        }
        while (i7 < i8 && i9 < i10) {
            int i19 = i6 + 1;
            float f2 = fArr2[i7];
            float f3 = fArr3[i9];
            if (f2 < f3) {
                i7++;
            } else {
                i9++;
                f2 = f3;
            }
            fArr[i6] = f2;
            i6 = i19;
        }
        if (fArr != fArr2 || i6 < i7) {
            while (i7 < i8) {
                fArr[i6] = fArr2[i7];
                i6++;
                i7++;
            }
        }
        if (fArr != fArr3 || i6 < i9) {
            while (i9 < i10) {
                fArr[i6] = fArr3[i9];
                i6++;
                i9++;
            }
        }
    }

    public static void mergeParts(Merger merger, int[] iArr, int i, int[] iArr2, int i2, int i3, int[] iArr3, int i4, int i5) {
        int i6;
        int i7;
        int i8;
        int i9;
        int i10;
        if (merger == null || iArr2 != iArr3) {
            i6 = i;
            i7 = i2;
            i8 = i3;
            i9 = i4;
            i10 = i5;
        } else {
            int i11 = i2;
            int i12 = i3;
            int i13 = i4;
            int i14 = i5;
            while (true) {
                if (i12 - i11 < i14 - i13) {
                    i9 = i11;
                    i10 = i12;
                    i7 = i13;
                    i8 = i14;
                } else {
                    i7 = i11;
                    i8 = i12;
                    i9 = i13;
                    i10 = i14;
                }
                if (i8 - i7 < 4096) {
                    break;
                }
                int i15 = (i7 + i8) >>> 1;
                int i16 = iArr2[i15];
                int i17 = i10;
                int i18 = i9;
                while (i18 < i17) {
                    int i19 = (i18 + i17) >>> 1;
                    if (i16 > iArr3[i19]) {
                        i18 = i19 + 1;
                    } else {
                        i17 = i19;
                    }
                }
                merger.forkMerger(iArr, i + (((i17 - i9) + i15) - i7), iArr2, i15, i8, iArr3, i17, i10);
                i11 = i7;
                i13 = i9;
                i12 = i15;
                i14 = i17;
            }
            i6 = i;
        }
        while (i7 < i8 && i9 < i10) {
            int i20 = i6 + 1;
            int i21 = iArr2[i7];
            int i22 = iArr3[i9];
            if (i21 < i22) {
                i7++;
            } else {
                i9++;
                i21 = i22;
            }
            iArr[i6] = i21;
            i6 = i20;
        }
        if (iArr != iArr2 || i6 < i7) {
            while (i7 < i8) {
                iArr[i6] = iArr2[i7];
                i6++;
                i7++;
            }
        }
        if (iArr != iArr3 || i6 < i9) {
            while (i9 < i10) {
                iArr[i6] = iArr3[i9];
                i6++;
                i9++;
            }
        }
    }

    public static void mergeParts(Merger merger, long[] jArr, int i, long[] jArr2, int i2, int i3, long[] jArr3, int i4, int i5) {
        int i6;
        int i7;
        int i8;
        int i9;
        int i10;
        if (merger == null || jArr2 != jArr3) {
            i6 = i;
            i7 = i2;
            i8 = i3;
            i9 = i4;
            i10 = i5;
        } else {
            int i11 = i2;
            int i12 = i3;
            int i13 = i4;
            int i14 = i5;
            while (true) {
                if (i12 - i11 < i14 - i13) {
                    i9 = i11;
                    i10 = i12;
                    i7 = i13;
                    i8 = i14;
                } else {
                    i7 = i11;
                    i8 = i12;
                    i9 = i13;
                    i10 = i14;
                }
                if (i8 - i7 < 4096) {
                    break;
                }
                int i15 = (i7 + i8) >>> 1;
                long j = jArr2[i15];
                int i16 = i10;
                int i17 = i9;
                while (i17 < i16) {
                    int i18 = (i17 + i16) >>> 1;
                    if (j > jArr3[i18]) {
                        i17 = i18 + 1;
                    } else {
                        i16 = i18;
                    }
                }
                merger.forkMerger(jArr, i + (((i16 - i9) + i15) - i7), jArr2, i15, i8, jArr3, i16, i10);
                i11 = i7;
                i13 = i9;
                i12 = i15;
                i14 = i16;
            }
            i6 = i;
        }
        while (i7 < i8 && i9 < i10) {
            int i19 = i6 + 1;
            long j2 = jArr2[i7];
            long j3 = jArr3[i9];
            if (j2 < j3) {
                i7++;
            } else {
                i9++;
                j2 = j3;
            }
            jArr[i6] = j2;
            i6 = i19;
        }
        if (jArr != jArr2 || i6 < i7) {
            while (i7 < i8) {
                jArr[i6] = jArr2[i7];
                i6++;
                i7++;
            }
        }
        if (jArr != jArr3 || i6 < i9) {
            while (i9 < i10) {
                jArr[i6] = jArr3[i9];
                i6++;
                i9++;
            }
        }
    }

    public static double[] mergeRuns(double[] dArr, double[] dArr2, int i, int i2, boolean z, int[] iArr, int i3, int i4) {
        int i5;
        double[] mergeRuns;
        double[] mergeRuns2;
        int i6 = i4 - i3;
        if (i6 == 1) {
            if (i2 >= 0) {
                return dArr;
            }
            int i7 = iArr[i4];
            int i8 = i7 - i;
            int i9 = iArr[i3];
            while (i7 > i9) {
                i8--;
                i7--;
                dArr2[i8] = dArr[i7];
            }
            return dArr2;
        }
        int i10 = i3;
        while (true) {
            i5 = i10 + 1;
            if (iArr[i5 + 1] > ((iArr[i3] + iArr[i4]) >>> 1)) {
                break;
            }
            i10 = i5;
        }
        if (!z || i6 <= 4) {
            mergeRuns = mergeRuns(dArr, dArr2, i, -i2, false, iArr, i3, i5);
            mergeRuns2 = mergeRuns(dArr, dArr2, i, 0, false, iArr, i5, i4);
        } else {
            RunMerger forkMe = new RunMerger(dArr, dArr2, i, 0, iArr, i5, i4).forkMe();
            double[] mergeRuns3 = mergeRuns(dArr, dArr2, i, -i2, true, iArr, i3, i5);
            mergeRuns2 = (double[]) forkMe.getDestination();
            mergeRuns = mergeRuns3;
        }
        double[] dArr3 = mergeRuns == dArr ? dArr2 : dArr;
        int i11 = mergeRuns == dArr ? iArr[i3] - i : iArr[i3];
        int i12 = mergeRuns == dArr2 ? iArr[i3] - i : iArr[i3];
        int i13 = mergeRuns == dArr2 ? iArr[i5] - i : iArr[i5];
        int i14 = mergeRuns2 == dArr2 ? iArr[i5] - i : iArr[i5];
        int i15 = mergeRuns2 == dArr2 ? iArr[i4] - i : iArr[i4];
        if (z) {
            new Merger(null, dArr3, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15).invoke();
        } else {
            mergeParts((Merger) null, dArr3, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15);
        }
        return dArr3;
    }

    public static float[] mergeRuns(float[] fArr, float[] fArr2, int i, int i2, boolean z, int[] iArr, int i3, int i4) {
        int i5;
        float[] mergeRuns;
        float[] mergeRuns2;
        int i6 = i4 - i3;
        if (i6 == 1) {
            if (i2 >= 0) {
                return fArr;
            }
            int i7 = iArr[i4];
            int i8 = i7 - i;
            int i9 = iArr[i3];
            while (i7 > i9) {
                i8--;
                i7--;
                fArr2[i8] = fArr[i7];
            }
            return fArr2;
        }
        int i10 = i3;
        while (true) {
            i5 = i10 + 1;
            if (iArr[i5 + 1] > ((iArr[i3] + iArr[i4]) >>> 1)) {
                break;
            }
            i10 = i5;
        }
        if (!z || i6 <= 4) {
            mergeRuns = mergeRuns(fArr, fArr2, i, -i2, false, iArr, i3, i5);
            mergeRuns2 = mergeRuns(fArr, fArr2, i, 0, false, iArr, i5, i4);
        } else {
            RunMerger forkMe = new RunMerger(fArr, fArr2, i, 0, iArr, i5, i4).forkMe();
            float[] mergeRuns3 = mergeRuns(fArr, fArr2, i, -i2, true, iArr, i3, i5);
            mergeRuns2 = (float[]) forkMe.getDestination();
            mergeRuns = mergeRuns3;
        }
        float[] fArr3 = mergeRuns == fArr ? fArr2 : fArr;
        int i11 = mergeRuns == fArr ? iArr[i3] - i : iArr[i3];
        int i12 = mergeRuns == fArr2 ? iArr[i3] - i : iArr[i3];
        int i13 = mergeRuns == fArr2 ? iArr[i5] - i : iArr[i5];
        int i14 = mergeRuns2 == fArr2 ? iArr[i5] - i : iArr[i5];
        int i15 = mergeRuns2 == fArr2 ? iArr[i4] - i : iArr[i4];
        if (z) {
            new Merger(null, fArr3, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15).invoke();
        } else {
            mergeParts((Merger) null, fArr3, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15);
        }
        return fArr3;
    }

    public static int[] mergeRuns(int[] iArr, int[] iArr2, int i, int i2, boolean z, int[] iArr3, int i3, int i4) {
        int i5;
        int[] mergeRuns;
        int[] mergeRuns2;
        int i6 = i4 - i3;
        if (i6 == 1) {
            if (i2 >= 0) {
                return iArr;
            }
            int i7 = iArr3[i4];
            int i8 = i7 - i;
            int i9 = iArr3[i3];
            while (i7 > i9) {
                i8--;
                i7--;
                iArr2[i8] = iArr[i7];
            }
            return iArr2;
        }
        int i10 = i3;
        while (true) {
            i5 = i10 + 1;
            if (iArr3[i5 + 1] > ((iArr3[i3] + iArr3[i4]) >>> 1)) {
                break;
            }
            i10 = i5;
        }
        if (!z || i6 <= 4) {
            mergeRuns = mergeRuns(iArr, iArr2, i, -i2, false, iArr3, i3, i5);
            mergeRuns2 = mergeRuns(iArr, iArr2, i, 0, false, iArr3, i5, i4);
        } else {
            RunMerger forkMe = new RunMerger(iArr, iArr2, i, 0, iArr3, i5, i4).forkMe();
            int[] mergeRuns3 = mergeRuns(iArr, iArr2, i, -i2, true, iArr3, i3, i5);
            mergeRuns2 = (int[]) forkMe.getDestination();
            mergeRuns = mergeRuns3;
        }
        int[] iArr4 = mergeRuns == iArr ? iArr2 : iArr;
        int i11 = mergeRuns == iArr ? iArr3[i3] - i : iArr3[i3];
        int i12 = mergeRuns == iArr2 ? iArr3[i3] - i : iArr3[i3];
        int i13 = mergeRuns == iArr2 ? iArr3[i5] - i : iArr3[i5];
        int i14 = mergeRuns2 == iArr2 ? iArr3[i5] - i : iArr3[i5];
        int i15 = mergeRuns2 == iArr2 ? iArr3[i4] - i : iArr3[i4];
        if (z) {
            new Merger(null, iArr4, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15).invoke();
        } else {
            mergeParts((Merger) null, iArr4, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15);
        }
        return iArr4;
    }

    public static long[] mergeRuns(long[] jArr, long[] jArr2, int i, int i2, boolean z, int[] iArr, int i3, int i4) {
        int i5;
        long[] mergeRuns;
        long[] mergeRuns2;
        int i6 = i4 - i3;
        if (i6 == 1) {
            if (i2 >= 0) {
                return jArr;
            }
            int i7 = iArr[i4];
            int i8 = i7 - i;
            int i9 = iArr[i3];
            while (i7 > i9) {
                i8--;
                i7--;
                jArr2[i8] = jArr[i7];
            }
            return jArr2;
        }
        int i10 = i3;
        while (true) {
            i5 = i10 + 1;
            if (iArr[i5 + 1] > ((iArr[i3] + iArr[i4]) >>> 1)) {
                break;
            }
            i10 = i5;
        }
        if (!z || i6 <= 4) {
            mergeRuns = mergeRuns(jArr, jArr2, i, -i2, false, iArr, i3, i5);
            mergeRuns2 = mergeRuns(jArr, jArr2, i, 0, false, iArr, i5, i4);
        } else {
            RunMerger forkMe = new RunMerger(jArr, jArr2, i, 0, iArr, i5, i4).forkMe();
            long[] mergeRuns3 = mergeRuns(jArr, jArr2, i, -i2, true, iArr, i3, i5);
            mergeRuns2 = (long[]) forkMe.getDestination();
            mergeRuns = mergeRuns3;
        }
        long[] jArr3 = mergeRuns == jArr ? jArr2 : jArr;
        int i11 = mergeRuns == jArr ? iArr[i3] - i : iArr[i3];
        int i12 = mergeRuns == jArr2 ? iArr[i3] - i : iArr[i3];
        int i13 = mergeRuns == jArr2 ? iArr[i5] - i : iArr[i5];
        int i14 = mergeRuns2 == jArr2 ? iArr[i5] - i : iArr[i5];
        int i15 = mergeRuns2 == jArr2 ? iArr[i4] - i : iArr[i4];
        if (z) {
            new Merger(null, jArr3, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15).invoke();
        } else {
            mergeParts((Merger) null, jArr3, i11, mergeRuns, i12, i13, mergeRuns2, i14, i15);
        }
        return jArr3;
    }

    public static void mixedInsertionSort(double[] dArr, int i, int i2, int i3) {
        double d;
        if (i2 != i3) {
            double d2 = dArr[i2];
            int i4 = i3;
            while (true) {
                i++;
                if (i >= i2) {
                    break;
                }
                double d3 = dArr[i];
                if (d3 < dArr[i - 1]) {
                    int i5 = i - 1;
                    dArr[i] = dArr[i5];
                    while (true) {
                        i5--;
                        double d4 = dArr[i5];
                        if (d3 >= d4) {
                            break;
                        }
                        dArr[i5 + 1] = d4;
                    }
                    dArr[i5 + 1] = d3;
                } else if (i4 > i && d3 > d2) {
                    do {
                        i4--;
                        d = dArr[i4];
                    } while (d > d2);
                    if (i4 > i) {
                        dArr[i4] = dArr[i];
                        d3 = d;
                    }
                    int i6 = i;
                    while (true) {
                        i6--;
                        double d5 = dArr[i6];
                        if (d3 >= d5) {
                            break;
                        }
                        dArr[i6 + 1] = d5;
                    }
                    dArr[i6 + 1] = d3;
                }
            }
            while (i < i3) {
                double d6 = dArr[i];
                int i7 = i + 1;
                double d7 = dArr[i7];
                if (d6 > d7) {
                    while (true) {
                        i--;
                        double d8 = dArr[i];
                        if (d6 >= d8) {
                            break;
                        }
                        dArr[i + 2] = d8;
                    }
                    int i8 = i + 1;
                    dArr[i8 + 1] = d6;
                    while (true) {
                        i8--;
                        double d9 = dArr[i8];
                        if (d7 >= d9) {
                            break;
                        }
                        dArr[i8 + 1] = d9;
                    }
                    dArr[i8 + 1] = d7;
                } else if (d6 < dArr[i - 1]) {
                    while (true) {
                        i--;
                        double d10 = dArr[i];
                        if (d7 >= d10) {
                            break;
                        }
                        dArr[i + 2] = d10;
                    }
                    int i9 = i + 1;
                    dArr[i9 + 1] = d7;
                    while (true) {
                        i9--;
                        double d11 = dArr[i9];
                        if (d6 >= d11) {
                            break;
                        }
                        dArr[i9 + 1] = d11;
                    }
                    dArr[i9 + 1] = d6;
                }
                i = i7 + 1;
            }
            return;
        }
        while (true) {
            i++;
            if (i >= i2) {
                return;
            }
            double d12 = dArr[i];
            int i10 = i;
            while (true) {
                i10--;
                double d13 = dArr[i10];
                if (d12 < d13) {
                    dArr[i10 + 1] = d13;
                }
            }
            dArr[i10 + 1] = d12;
        }
    }

    public static void mixedInsertionSort(float[] fArr, int i, int i2, int i3) {
        float f;
        if (i2 != i3) {
            float f2 = fArr[i2];
            int i4 = i3;
            while (true) {
                i++;
                if (i >= i2) {
                    break;
                }
                float f3 = fArr[i];
                if (f3 < fArr[i - 1]) {
                    int i5 = i - 1;
                    fArr[i] = fArr[i5];
                    while (true) {
                        i5--;
                        float f4 = fArr[i5];
                        if (f3 >= f4) {
                            break;
                        }
                        fArr[i5 + 1] = f4;
                    }
                    fArr[i5 + 1] = f3;
                } else if (i4 > i && f3 > f2) {
                    do {
                        i4--;
                        f = fArr[i4];
                    } while (f > f2);
                    if (i4 > i) {
                        fArr[i4] = fArr[i];
                        f3 = f;
                    }
                    int i6 = i;
                    while (true) {
                        i6--;
                        float f5 = fArr[i6];
                        if (f3 >= f5) {
                            break;
                        }
                        fArr[i6 + 1] = f5;
                    }
                    fArr[i6 + 1] = f3;
                }
            }
            while (i < i3) {
                float f6 = fArr[i];
                int i7 = i + 1;
                float f7 = fArr[i7];
                if (f6 > f7) {
                    while (true) {
                        i--;
                        float f8 = fArr[i];
                        if (f6 >= f8) {
                            break;
                        }
                        fArr[i + 2] = f8;
                    }
                    int i8 = i + 1;
                    fArr[i8 + 1] = f6;
                    while (true) {
                        i8--;
                        float f9 = fArr[i8];
                        if (f7 >= f9) {
                            break;
                        }
                        fArr[i8 + 1] = f9;
                    }
                    fArr[i8 + 1] = f7;
                } else if (f6 < fArr[i - 1]) {
                    while (true) {
                        i--;
                        float f10 = fArr[i];
                        if (f7 >= f10) {
                            break;
                        }
                        fArr[i + 2] = f10;
                    }
                    int i9 = i + 1;
                    fArr[i9 + 1] = f7;
                    while (true) {
                        i9--;
                        float f11 = fArr[i9];
                        if (f6 >= f11) {
                            break;
                        }
                        fArr[i9 + 1] = f11;
                    }
                    fArr[i9 + 1] = f6;
                }
                i = i7 + 1;
            }
            return;
        }
        while (true) {
            i++;
            if (i >= i2) {
                return;
            }
            float f12 = fArr[i];
            int i10 = i;
            while (true) {
                i10--;
                float f13 = fArr[i10];
                if (f12 < f13) {
                    fArr[i10 + 1] = f13;
                }
            }
            fArr[i10 + 1] = f12;
        }
    }

    public static void mixedInsertionSort(int[] iArr, int i, int i2, int i3) {
        int i4;
        if (i2 != i3) {
            int i5 = iArr[i2];
            int i6 = i3;
            while (true) {
                i++;
                if (i >= i2) {
                    break;
                }
                int i7 = iArr[i];
                if (i7 < iArr[i - 1]) {
                    int i8 = i - 1;
                    iArr[i] = iArr[i8];
                    while (true) {
                        i8--;
                        int i9 = iArr[i8];
                        if (i7 >= i9) {
                            break;
                        }
                        iArr[i8 + 1] = i9;
                    }
                    iArr[i8 + 1] = i7;
                } else if (i6 > i && i7 > i5) {
                    do {
                        i6--;
                        i4 = iArr[i6];
                    } while (i4 > i5);
                    if (i6 > i) {
                        iArr[i6] = iArr[i];
                        i7 = i4;
                    }
                    int i10 = i;
                    while (true) {
                        i10--;
                        int i11 = iArr[i10];
                        if (i7 >= i11) {
                            break;
                        }
                        iArr[i10 + 1] = i11;
                    }
                    iArr[i10 + 1] = i7;
                }
            }
            while (i < i3) {
                int i12 = iArr[i];
                int i13 = i + 1;
                int i14 = iArr[i13];
                if (i12 > i14) {
                    while (true) {
                        i--;
                        int i15 = iArr[i];
                        if (i12 >= i15) {
                            break;
                        }
                        iArr[i + 2] = i15;
                    }
                    int i16 = i + 1;
                    iArr[i16 + 1] = i12;
                    while (true) {
                        i16--;
                        int i17 = iArr[i16];
                        if (i14 >= i17) {
                            break;
                        }
                        iArr[i16 + 1] = i17;
                    }
                    iArr[i16 + 1] = i14;
                } else if (i12 < iArr[i - 1]) {
                    while (true) {
                        i--;
                        int i18 = iArr[i];
                        if (i14 >= i18) {
                            break;
                        }
                        iArr[i + 2] = i18;
                    }
                    int i19 = i + 1;
                    iArr[i19 + 1] = i14;
                    while (true) {
                        i19--;
                        int i20 = iArr[i19];
                        if (i12 >= i20) {
                            break;
                        }
                        iArr[i19 + 1] = i20;
                    }
                    iArr[i19 + 1] = i12;
                }
                i = i13 + 1;
            }
            return;
        }
        while (true) {
            i++;
            if (i >= i2) {
                return;
            }
            int i21 = iArr[i];
            int i22 = i;
            while (true) {
                i22--;
                int i23 = iArr[i22];
                if (i21 < i23) {
                    iArr[i22 + 1] = i23;
                }
            }
            iArr[i22 + 1] = i21;
        }
    }

    public static void mixedInsertionSort(long[] jArr, int i, int i2, int i3) {
        long j;
        if (i2 != i3) {
            long j2 = jArr[i2];
            int i4 = i3;
            while (true) {
                i++;
                if (i >= i2) {
                    break;
                }
                long j3 = jArr[i];
                if (j3 < jArr[i - 1]) {
                    int i5 = i - 1;
                    jArr[i] = jArr[i5];
                    while (true) {
                        i5--;
                        long j4 = jArr[i5];
                        if (j3 >= j4) {
                            break;
                        }
                        jArr[i5 + 1] = j4;
                    }
                    jArr[i5 + 1] = j3;
                } else if (i4 > i && j3 > j2) {
                    do {
                        i4--;
                        j = jArr[i4];
                    } while (j > j2);
                    if (i4 > i) {
                        jArr[i4] = jArr[i];
                        j3 = j;
                    }
                    int i6 = i;
                    while (true) {
                        i6--;
                        long j5 = jArr[i6];
                        if (j3 >= j5) {
                            break;
                        }
                        jArr[i6 + 1] = j5;
                    }
                    jArr[i6 + 1] = j3;
                }
            }
            while (i < i3) {
                long j6 = jArr[i];
                int i7 = i + 1;
                long j7 = jArr[i7];
                if (j6 > j7) {
                    while (true) {
                        i--;
                        long j8 = jArr[i];
                        if (j6 >= j8) {
                            break;
                        }
                        jArr[i + 2] = j8;
                    }
                    int i8 = i + 1;
                    jArr[i8 + 1] = j6;
                    while (true) {
                        i8--;
                        long j9 = jArr[i8];
                        if (j7 >= j9) {
                            break;
                        }
                        jArr[i8 + 1] = j9;
                    }
                    jArr[i8 + 1] = j7;
                } else if (j6 < jArr[i - 1]) {
                    while (true) {
                        i--;
                        long j10 = jArr[i];
                        if (j7 >= j10) {
                            break;
                        }
                        jArr[i + 2] = j10;
                    }
                    int i9 = i + 1;
                    jArr[i9 + 1] = j7;
                    while (true) {
                        i9--;
                        long j11 = jArr[i9];
                        if (j6 >= j11) {
                            break;
                        }
                        jArr[i9 + 1] = j11;
                    }
                    jArr[i9 + 1] = j6;
                }
                i = i7 + 1;
            }
            return;
        }
        while (true) {
            i++;
            if (i >= i2) {
                return;
            }
            long j12 = jArr[i];
            int i10 = i;
            while (true) {
                i10--;
                long j13 = jArr[i10];
                if (j12 < j13) {
                    jArr[i10 + 1] = j13;
                }
            }
            jArr[i10 + 1] = j12;
        }
    }

    public static void pushDown(double[] dArr, int i, double d, int i2, int i3) {
        while (true) {
            int i4 = ((i << 1) - i2) + 2;
            if (i4 > i3) {
                break;
            }
            if (i4 == i3 || dArr[i4] < dArr[i4 - 1]) {
                i4--;
            }
            double d2 = dArr[i4];
            if (d2 <= d) {
                break;
            }
            dArr[i] = d2;
            i = i4;
        }
        dArr[i] = d;
    }

    public static void pushDown(float[] fArr, int i, float f, int i2, int i3) {
        while (true) {
            int i4 = ((i << 1) - i2) + 2;
            if (i4 > i3) {
                break;
            }
            if (i4 == i3 || fArr[i4] < fArr[i4 - 1]) {
                i4--;
            }
            float f2 = fArr[i4];
            if (f2 <= f) {
                break;
            }
            fArr[i] = f2;
            i = i4;
        }
        fArr[i] = f;
    }

    public static void pushDown(int[] iArr, int i, int i2, int i3, int i4) {
        while (true) {
            int i5 = ((i << 1) - i3) + 2;
            if (i5 > i4) {
                break;
            }
            if (i5 == i4 || iArr[i5] < iArr[i5 - 1]) {
                i5--;
            }
            int i6 = iArr[i5];
            if (i6 <= i2) {
                break;
            }
            iArr[i] = i6;
            i = i5;
        }
        iArr[i] = i2;
    }

    public static void pushDown(long[] jArr, int i, long j, int i2, int i3) {
        while (true) {
            int i4 = ((i << 1) - i2) + 2;
            if (i4 > i3) {
                break;
            }
            if (i4 == i3 || jArr[i4] < jArr[i4 - 1]) {
                i4--;
            }
            long j2 = jArr[i4];
            if (j2 <= j) {
                break;
            }
            jArr[i] = j2;
            i = i4;
        }
        jArr[i] = j;
    }

    public static void sort(Sorter sorter, double[] dArr, int i, int i2, int i3) {
        double d;
        int i4 = i;
        int i5 = i3;
        while (true) {
            int i6 = i5 - 1;
            int i7 = i5 - i2;
            if (i7 < i4 + 65 && (i4 & 1) > 0) {
                mixedInsertionSort(dArr, i2, i5 - (((i7 >> 5) << 3) * 3), i5);
                return;
            } else if (i7 < 44) {
                insertionSort(dArr, i2, i5);
                return;
            } else if ((i4 == 0 || (i7 > 4096 && (i4 & 1) > 0)) && tryMergeRuns(sorter, dArr, i2, i7)) {
                return;
            } else {
                i4 += 6;
                if (i4 > 384) {
                    heapSort(dArr, i2, i5);
                    return;
                }
                int i8 = ((i7 >> 3) * 3) + 3;
                int i9 = i2 + i8;
                int i10 = i6 - i8;
                int i11 = (i9 + i10) >>> 1;
                int i12 = (i9 + i11) >>> 1;
                int i13 = (i11 + i10) >>> 1;
                double d2 = dArr[i11];
                double d3 = dArr[i10];
                double d4 = dArr[i12];
                if (d3 < d4) {
                    dArr[i10] = d4;
                    dArr[i12] = d3;
                }
                double d5 = dArr[i13];
                double d6 = dArr[i9];
                if (d5 < d6) {
                    dArr[i13] = d6;
                    dArr[i9] = d5;
                }
                double d7 = dArr[i10];
                double d8 = dArr[i13];
                if (d7 < d8) {
                    dArr[i10] = d8;
                    dArr[i13] = d7;
                }
                double d9 = dArr[i12];
                double d10 = dArr[i9];
                if (d9 < d10) {
                    dArr[i12] = d10;
                    dArr[i9] = d9;
                }
                double d11 = dArr[i13];
                double d12 = dArr[i12];
                if (d11 < d12) {
                    dArr[i13] = d12;
                    dArr[i12] = d11;
                }
                double d13 = dArr[i12];
                if (d2 >= d13) {
                    double d14 = dArr[i13];
                    if (d2 > d14) {
                        if (d2 > dArr[i10]) {
                            dArr[i11] = d14;
                            dArr[i13] = dArr[i10];
                            dArr[i10] = d2;
                        } else {
                            dArr[i11] = d14;
                            dArr[i13] = d2;
                        }
                    }
                } else if (d2 < dArr[i9]) {
                    dArr[i11] = d13;
                    dArr[i12] = dArr[i9];
                    dArr[i9] = d2;
                } else {
                    dArr[i11] = d13;
                    dArr[i12] = d2;
                }
                double d15 = dArr[i9];
                double d16 = dArr[i12];
                if (d15 < d16) {
                    double d17 = dArr[i11];
                    if (d16 < d17) {
                        double d18 = dArr[i13];
                        if (d17 < d18) {
                            double d19 = dArr[i10];
                            if (d18 < d19) {
                                dArr[i9] = dArr[i2];
                                dArr[i10] = dArr[i6];
                                int i14 = i2;
                                do {
                                    i14++;
                                } while (dArr[i14] < d15);
                                int i15 = i6;
                                do {
                                    i15--;
                                } while (dArr[i15] > d19);
                                int i16 = i14 - 1;
                                int i17 = i15 + 1;
                                int i18 = i17;
                                while (true) {
                                    i17--;
                                    if (i17 <= i16) {
                                        break;
                                    }
                                    double d20 = dArr[i17];
                                    if (d20 < d15) {
                                        while (true) {
                                            if (i16 < i17) {
                                                i16++;
                                                double d21 = dArr[i16];
                                                if (d21 >= d15) {
                                                    if (d21 > d19) {
                                                        i18--;
                                                        dArr[i17] = dArr[i18];
                                                        dArr[i18] = dArr[i16];
                                                    } else {
                                                        dArr[i17] = d21;
                                                    }
                                                    dArr[i16] = d20;
                                                }
                                            }
                                        }
                                    } else if (d20 > d19) {
                                        i18--;
                                        dArr[i17] = dArr[i18];
                                        dArr[i18] = d20;
                                    }
                                }
                                dArr[i2] = dArr[i16];
                                dArr[i16] = d15;
                                dArr[i6] = dArr[i18];
                                dArr[i18] = d19;
                                if (i7 <= 4096 || sorter == null) {
                                    int i19 = i4 | 1;
                                    sort(sorter, dArr, i19, i16 + 1, i18);
                                    sort(sorter, dArr, i19, i18 + 1, i5);
                                } else {
                                    int i20 = i4 | 1;
                                    sorter.forkSorter(i20, i16 + 1, i18);
                                    sorter.forkSorter(i20, i18 + 1, i5);
                                }
                                i5 = i16;
                            }
                        }
                    }
                }
                double d22 = dArr[i11];
                dArr[i11] = dArr[i2];
                int i21 = i6 + 1;
                int i22 = i2;
                int i23 = i21;
                while (true) {
                    i21--;
                    if (i21 <= i22) {
                        break;
                    }
                    double d23 = dArr[i21];
                    if (d23 != d22) {
                        dArr[i21] = d22;
                        if (d23 < d22) {
                            do {
                                i22++;
                                d = dArr[i22];
                            } while (d < d22);
                            if (d > d22) {
                                i23--;
                                dArr[i23] = d;
                            }
                            dArr[i22] = d23;
                        } else {
                            i23--;
                            dArr[i23] = d23;
                        }
                    }
                }
                dArr[i2] = dArr[i22];
                dArr[i22] = d22;
                if (i7 <= 4096 || sorter == null) {
                    sort(sorter, dArr, i4 | 1, i23, i5);
                } else {
                    sorter.forkSorter(i4 | 1, i23, i5);
                }
                i5 = i22;
            }
        }
    }

    public static void sort(Sorter sorter, float[] fArr, int i, int i2, int i3) {
        float f;
        int i4 = i;
        int i5 = i3;
        while (true) {
            int i6 = i5 - 1;
            int i7 = i5 - i2;
            if (i7 < i4 + 65 && (i4 & 1) > 0) {
                mixedInsertionSort(fArr, i2, i5 - (((i7 >> 5) << 3) * 3), i5);
                return;
            } else if (i7 < 44) {
                insertionSort(fArr, i2, i5);
                return;
            } else if ((i4 == 0 || (i7 > 4096 && (i4 & 1) > 0)) && tryMergeRuns(sorter, fArr, i2, i7)) {
                return;
            } else {
                i4 += 6;
                if (i4 > 384) {
                    heapSort(fArr, i2, i5);
                    return;
                }
                int i8 = ((i7 >> 3) * 3) + 3;
                int i9 = i2 + i8;
                int i10 = i6 - i8;
                int i11 = (i9 + i10) >>> 1;
                int i12 = (i9 + i11) >>> 1;
                int i13 = (i11 + i10) >>> 1;
                float f2 = fArr[i11];
                float f3 = fArr[i10];
                float f4 = fArr[i12];
                if (f3 < f4) {
                    fArr[i10] = f4;
                    fArr[i12] = f3;
                }
                float f5 = fArr[i13];
                float f6 = fArr[i9];
                if (f5 < f6) {
                    fArr[i13] = f6;
                    fArr[i9] = f5;
                }
                float f7 = fArr[i10];
                float f8 = fArr[i13];
                if (f7 < f8) {
                    fArr[i10] = f8;
                    fArr[i13] = f7;
                }
                float f9 = fArr[i12];
                float f10 = fArr[i9];
                if (f9 < f10) {
                    fArr[i12] = f10;
                    fArr[i9] = f9;
                }
                float f11 = fArr[i13];
                float f12 = fArr[i12];
                if (f11 < f12) {
                    fArr[i13] = f12;
                    fArr[i12] = f11;
                }
                float f13 = fArr[i12];
                if (f2 >= f13) {
                    float f14 = fArr[i13];
                    if (f2 > f14) {
                        if (f2 > fArr[i10]) {
                            fArr[i11] = f14;
                            fArr[i13] = fArr[i10];
                            fArr[i10] = f2;
                        } else {
                            fArr[i11] = f14;
                            fArr[i13] = f2;
                        }
                    }
                } else if (f2 < fArr[i9]) {
                    fArr[i11] = f13;
                    fArr[i12] = fArr[i9];
                    fArr[i9] = f2;
                } else {
                    fArr[i11] = f13;
                    fArr[i12] = f2;
                }
                float f15 = fArr[i9];
                float f16 = fArr[i12];
                if (f15 < f16) {
                    float f17 = fArr[i11];
                    if (f16 < f17) {
                        float f18 = fArr[i13];
                        if (f17 < f18) {
                            float f19 = fArr[i10];
                            if (f18 < f19) {
                                fArr[i9] = fArr[i2];
                                fArr[i10] = fArr[i6];
                                int i14 = i2;
                                do {
                                    i14++;
                                } while (fArr[i14] < f15);
                                int i15 = i6;
                                do {
                                    i15--;
                                } while (fArr[i15] > f19);
                                int i16 = i14 - 1;
                                int i17 = i15 + 1;
                                int i18 = i17;
                                while (true) {
                                    i17--;
                                    if (i17 <= i16) {
                                        break;
                                    }
                                    float f20 = fArr[i17];
                                    if (f20 < f15) {
                                        while (true) {
                                            if (i16 < i17) {
                                                i16++;
                                                float f21 = fArr[i16];
                                                if (f21 >= f15) {
                                                    if (f21 > f19) {
                                                        i18--;
                                                        fArr[i17] = fArr[i18];
                                                        fArr[i18] = fArr[i16];
                                                    } else {
                                                        fArr[i17] = f21;
                                                    }
                                                    fArr[i16] = f20;
                                                }
                                            }
                                        }
                                    } else if (f20 > f19) {
                                        i18--;
                                        fArr[i17] = fArr[i18];
                                        fArr[i18] = f20;
                                    }
                                }
                                fArr[i2] = fArr[i16];
                                fArr[i16] = f15;
                                fArr[i6] = fArr[i18];
                                fArr[i18] = f19;
                                if (i7 <= 4096 || sorter == null) {
                                    int i19 = i4 | 1;
                                    sort(sorter, fArr, i19, i16 + 1, i18);
                                    sort(sorter, fArr, i19, i18 + 1, i5);
                                } else {
                                    int i20 = i4 | 1;
                                    sorter.forkSorter(i20, i16 + 1, i18);
                                    sorter.forkSorter(i20, i18 + 1, i5);
                                }
                                i5 = i16;
                            }
                        }
                    }
                }
                float f22 = fArr[i11];
                fArr[i11] = fArr[i2];
                int i21 = i6 + 1;
                int i22 = i2;
                int i23 = i21;
                while (true) {
                    i21--;
                    if (i21 <= i22) {
                        break;
                    }
                    float f23 = fArr[i21];
                    if (f23 != f22) {
                        fArr[i21] = f22;
                        if (f23 < f22) {
                            do {
                                i22++;
                                f = fArr[i22];
                            } while (f < f22);
                            if (f > f22) {
                                i23--;
                                fArr[i23] = f;
                            }
                            fArr[i22] = f23;
                        } else {
                            i23--;
                            fArr[i23] = f23;
                        }
                    }
                }
                fArr[i2] = fArr[i22];
                fArr[i22] = f22;
                if (i7 <= 4096 || sorter == null) {
                    sort(sorter, fArr, i4 | 1, i23, i5);
                } else {
                    sorter.forkSorter(i4 | 1, i23, i5);
                }
                i5 = i22;
            }
        }
    }

    public static void sort(Sorter sorter, int[] iArr, int i, int i2, int i3) {
        int i4;
        int i5;
        int i6;
        int i7;
        while (true) {
            int i8 = i3 - 1;
            int i9 = i3 - i2;
            if (i9 < i + 65 && (i & 1) > 0) {
                mixedInsertionSort(iArr, i2, i3 - (((i9 >> 5) << 3) * 3), i3);
                return;
            } else if (i9 < 44) {
                insertionSort(iArr, i2, i3);
                return;
            } else if ((i == 0 || (i9 > 4096 && (i & 1) > 0)) && tryMergeRuns(sorter, iArr, i2, i9)) {
                return;
            } else {
                i += 6;
                if (i > 384) {
                    heapSort(iArr, i2, i3);
                    return;
                }
                int i10 = ((i9 >> 3) * 3) + 3;
                int i11 = i2 + i10;
                int i12 = i8 - i10;
                int i13 = (i11 + i12) >>> 1;
                int i14 = (i11 + i13) >>> 1;
                int i15 = (i13 + i12) >>> 1;
                int i16 = iArr[i13];
                int i17 = iArr[i12];
                int i18 = iArr[i14];
                if (i17 < i18) {
                    iArr[i12] = i18;
                    iArr[i14] = i17;
                }
                int i19 = iArr[i15];
                int i20 = iArr[i11];
                if (i19 < i20) {
                    iArr[i15] = i20;
                    iArr[i11] = i19;
                }
                int i21 = iArr[i12];
                int i22 = iArr[i15];
                if (i21 < i22) {
                    iArr[i12] = i22;
                    iArr[i15] = i21;
                }
                int i23 = iArr[i14];
                int i24 = iArr[i11];
                if (i23 < i24) {
                    iArr[i14] = i24;
                    iArr[i11] = i23;
                }
                int i25 = iArr[i15];
                int i26 = iArr[i14];
                if (i25 < i26) {
                    iArr[i15] = i26;
                    iArr[i14] = i25;
                }
                int i27 = iArr[i14];
                if (i16 >= i27) {
                    int i28 = iArr[i15];
                    if (i16 > i28) {
                        if (i16 > iArr[i12]) {
                            iArr[i13] = i28;
                            iArr[i15] = iArr[i12];
                            iArr[i12] = i16;
                        } else {
                            iArr[i13] = i28;
                            iArr[i15] = i16;
                        }
                    }
                } else if (i16 < iArr[i11]) {
                    iArr[i13] = i27;
                    iArr[i14] = iArr[i11];
                    iArr[i11] = i16;
                } else {
                    iArr[i13] = i27;
                    iArr[i14] = i16;
                }
                int i29 = iArr[i11];
                int i30 = iArr[i14];
                if (i29 >= i30 || i30 >= (i5 = iArr[i13]) || i5 >= (i6 = iArr[i15]) || i6 >= (i7 = iArr[i12])) {
                    int i31 = iArr[i13];
                    iArr[i13] = iArr[i2];
                    int i32 = i8 + 1;
                    int i33 = i2;
                    int i34 = i32;
                    while (true) {
                        i32--;
                        if (i32 <= i33) {
                            break;
                        }
                        int i35 = iArr[i32];
                        if (i35 != i31) {
                            iArr[i32] = i31;
                            if (i35 < i31) {
                                do {
                                    i33++;
                                    i4 = iArr[i33];
                                } while (i4 < i31);
                                if (i4 > i31) {
                                    i34--;
                                    iArr[i34] = i4;
                                }
                                iArr[i33] = i35;
                            } else {
                                i34--;
                                iArr[i34] = i35;
                            }
                        }
                    }
                    iArr[i2] = iArr[i33];
                    iArr[i33] = i31;
                    if (i9 <= 4096 || sorter == null) {
                        sort(sorter, iArr, i | 1, i34, i3);
                    } else {
                        sorter.forkSorter(i | 1, i34, i3);
                    }
                    i3 = i33;
                } else {
                    iArr[i11] = iArr[i2];
                    iArr[i12] = iArr[i8];
                    int i36 = i2;
                    do {
                        i36++;
                    } while (iArr[i36] < i29);
                    int i37 = i8;
                    do {
                        i37--;
                    } while (iArr[i37] > i7);
                    int i38 = i36 - 1;
                    int i39 = i37 + 1;
                    int i40 = i39;
                    while (true) {
                        i39--;
                        if (i39 <= i38) {
                            break;
                        }
                        int i41 = iArr[i39];
                        if (i41 < i29) {
                            while (true) {
                                if (i38 >= i39) {
                                    break;
                                }
                                i38++;
                                int i42 = iArr[i38];
                                if (i42 >= i29) {
                                    if (i42 > i7) {
                                        i40--;
                                        iArr[i39] = iArr[i40];
                                        iArr[i40] = iArr[i38];
                                    } else {
                                        iArr[i39] = i42;
                                    }
                                    iArr[i38] = i41;
                                }
                            }
                        } else if (i41 > i7) {
                            i40--;
                            iArr[i39] = iArr[i40];
                            iArr[i40] = i41;
                        }
                    }
                    iArr[i2] = iArr[i38];
                    iArr[i38] = i29;
                    iArr[i8] = iArr[i40];
                    iArr[i40] = i7;
                    if (i9 <= 4096 || sorter == null) {
                        int i43 = i | 1;
                        sort(sorter, iArr, i43, i38 + 1, i40);
                        sort(sorter, iArr, i43, i40 + 1, i3);
                    } else {
                        int i44 = i | 1;
                        sorter.forkSorter(i44, i38 + 1, i40);
                        sorter.forkSorter(i44, i40 + 1, i3);
                    }
                    i3 = i38;
                }
            }
        }
    }

    public static void sort(Sorter sorter, long[] jArr, int i, int i2, int i3) {
        long j;
        int i4 = i;
        int i5 = i3;
        while (true) {
            int i6 = i5 - 1;
            int i7 = i5 - i2;
            if (i7 < i4 + 65 && (i4 & 1) > 0) {
                mixedInsertionSort(jArr, i2, i5 - (((i7 >> 5) << 3) * 3), i5);
                return;
            } else if (i7 < 44) {
                insertionSort(jArr, i2, i5);
                return;
            } else if ((i4 == 0 || (i7 > 4096 && (i4 & 1) > 0)) && tryMergeRuns(sorter, jArr, i2, i7)) {
                return;
            } else {
                i4 += 6;
                if (i4 > 384) {
                    heapSort(jArr, i2, i5);
                    return;
                }
                int i8 = ((i7 >> 3) * 3) + 3;
                int i9 = i2 + i8;
                int i10 = i6 - i8;
                int i11 = (i9 + i10) >>> 1;
                int i12 = (i9 + i11) >>> 1;
                int i13 = (i11 + i10) >>> 1;
                long j2 = jArr[i11];
                long j3 = jArr[i10];
                long j4 = jArr[i12];
                if (j3 < j4) {
                    jArr[i10] = j4;
                    jArr[i12] = j3;
                }
                long j5 = jArr[i13];
                long j6 = jArr[i9];
                if (j5 < j6) {
                    jArr[i13] = j6;
                    jArr[i9] = j5;
                }
                long j7 = jArr[i10];
                long j8 = jArr[i13];
                if (j7 < j8) {
                    jArr[i10] = j8;
                    jArr[i13] = j7;
                }
                long j9 = jArr[i12];
                long j10 = jArr[i9];
                if (j9 < j10) {
                    jArr[i12] = j10;
                    jArr[i9] = j9;
                }
                long j11 = jArr[i13];
                long j12 = jArr[i12];
                if (j11 < j12) {
                    jArr[i13] = j12;
                    jArr[i12] = j11;
                }
                long j13 = jArr[i12];
                if (j2 >= j13) {
                    long j14 = jArr[i13];
                    if (j2 > j14) {
                        if (j2 > jArr[i10]) {
                            jArr[i11] = j14;
                            jArr[i13] = jArr[i10];
                            jArr[i10] = j2;
                        } else {
                            jArr[i11] = j14;
                            jArr[i13] = j2;
                        }
                    }
                } else if (j2 < jArr[i9]) {
                    jArr[i11] = j13;
                    jArr[i12] = jArr[i9];
                    jArr[i9] = j2;
                } else {
                    jArr[i11] = j13;
                    jArr[i12] = j2;
                }
                long j15 = jArr[i9];
                long j16 = jArr[i12];
                if (j15 < j16) {
                    long j17 = jArr[i11];
                    if (j16 < j17) {
                        long j18 = jArr[i13];
                        if (j17 < j18) {
                            long j19 = jArr[i10];
                            if (j18 < j19) {
                                jArr[i9] = jArr[i2];
                                jArr[i10] = jArr[i6];
                                int i14 = i2;
                                do {
                                    i14++;
                                } while (jArr[i14] < j15);
                                int i15 = i6;
                                do {
                                    i15--;
                                } while (jArr[i15] > j19);
                                int i16 = i14 - 1;
                                int i17 = i15 + 1;
                                int i18 = i17;
                                while (true) {
                                    i17--;
                                    if (i17 <= i16) {
                                        break;
                                    }
                                    long j20 = jArr[i17];
                                    if (j20 < j15) {
                                        while (true) {
                                            if (i16 < i17) {
                                                i16++;
                                                long j21 = jArr[i16];
                                                if (j21 >= j15) {
                                                    if (j21 > j19) {
                                                        i18--;
                                                        jArr[i17] = jArr[i18];
                                                        jArr[i18] = jArr[i16];
                                                    } else {
                                                        jArr[i17] = j21;
                                                    }
                                                    jArr[i16] = j20;
                                                }
                                            }
                                        }
                                    } else if (j20 > j19) {
                                        i18--;
                                        jArr[i17] = jArr[i18];
                                        jArr[i18] = j20;
                                    }
                                }
                                jArr[i2] = jArr[i16];
                                jArr[i16] = j15;
                                jArr[i6] = jArr[i18];
                                jArr[i18] = j19;
                                if (i7 <= 4096 || sorter == null) {
                                    int i19 = i4 | 1;
                                    sort(sorter, jArr, i19, i16 + 1, i18);
                                    sort(sorter, jArr, i19, i18 + 1, i5);
                                } else {
                                    int i20 = i4 | 1;
                                    sorter.forkSorter(i20, i16 + 1, i18);
                                    sorter.forkSorter(i20, i18 + 1, i5);
                                }
                                i5 = i16;
                            }
                        }
                    }
                }
                long j22 = jArr[i11];
                jArr[i11] = jArr[i2];
                int i21 = i6 + 1;
                int i22 = i2;
                int i23 = i21;
                while (true) {
                    i21--;
                    if (i21 <= i22) {
                        break;
                    }
                    long j23 = jArr[i21];
                    int i24 = (j23 > j22 ? 1 : (j23 == j22 ? 0 : -1));
                    if (i24 != 0) {
                        jArr[i21] = j22;
                        if (i24 < 0) {
                            do {
                                i22++;
                                j = jArr[i22];
                            } while (j < j22);
                            if (j > j22) {
                                i23--;
                                jArr[i23] = j;
                            }
                            jArr[i22] = j23;
                        } else {
                            i23--;
                            jArr[i23] = j23;
                        }
                    }
                }
                jArr[i2] = jArr[i22];
                jArr[i22] = j22;
                if (i7 <= 4096 || sorter == null) {
                    sort(sorter, jArr, i4 | 1, i23, i5);
                } else {
                    sorter.forkSorter(i4 | 1, i23, i5);
                }
                i5 = i22;
            }
        }
    }

    public static void sort(byte[] bArr, int i, int i2) {
        if (i2 - i > 64) {
            countingSort(bArr, i, i2);
        } else {
            insertionSort(bArr, i, i2);
        }
    }

    public static void sort(char[] cArr, int i, int i2) {
        if (i2 - i > 1750) {
            countingSort(cArr, i, i2);
        } else {
            sort(cArr, 0, i, i2);
        }
    }

    public static void sort(char[] cArr, int i, int i2, int i3) {
        char c;
        char c2;
        char c3;
        char c4;
        while (true) {
            int i4 = i3 - 1;
            int i5 = i3 - i2;
            if (i5 < 44) {
                insertionSort(cArr, i2, i3);
                return;
            }
            i += 6;
            if (i > 384) {
                countingSort(cArr, i2, i3);
                return;
            }
            int i6 = ((i5 >> 3) * 3) + 3;
            int i7 = i2 + i6;
            int i8 = i4 - i6;
            int i9 = (i7 + i8) >>> 1;
            int i10 = (i7 + i9) >>> 1;
            int i11 = (i9 + i8) >>> 1;
            char c5 = cArr[i9];
            char c6 = cArr[i8];
            char c7 = cArr[i10];
            if (c6 < c7) {
                cArr[i8] = c7;
                cArr[i10] = c6;
            }
            char c8 = cArr[i11];
            char c9 = cArr[i7];
            if (c8 < c9) {
                cArr[i11] = c9;
                cArr[i7] = c8;
            }
            char c10 = cArr[i8];
            char c11 = cArr[i11];
            if (c10 < c11) {
                cArr[i8] = c11;
                cArr[i11] = c10;
            }
            char c12 = cArr[i10];
            char c13 = cArr[i7];
            if (c12 < c13) {
                cArr[i10] = c13;
                cArr[i7] = c12;
            }
            char c14 = cArr[i11];
            char c15 = cArr[i10];
            if (c14 < c15) {
                cArr[i11] = c15;
                cArr[i10] = c14;
            }
            char c16 = cArr[i10];
            if (c5 >= c16) {
                char c17 = cArr[i11];
                if (c5 > c17) {
                    if (c5 > cArr[i8]) {
                        cArr[i9] = c17;
                        cArr[i11] = cArr[i8];
                        cArr[i8] = c5;
                    } else {
                        cArr[i9] = c17;
                        cArr[i11] = c5;
                    }
                }
            } else if (c5 < cArr[i7]) {
                cArr[i9] = c16;
                cArr[i10] = cArr[i7];
                cArr[i7] = c5;
            } else {
                cArr[i9] = c16;
                cArr[i10] = c5;
            }
            char c18 = cArr[i7];
            char c19 = cArr[i10];
            if (c18 >= c19 || c19 >= (c2 = cArr[i9]) || c2 >= (c3 = cArr[i11]) || c3 >= (c4 = cArr[i8])) {
                char c20 = cArr[i9];
                cArr[i9] = cArr[i2];
                int i12 = i4 + 1;
                int i13 = i2;
                int i14 = i12;
                while (true) {
                    i12--;
                    if (i12 <= i13) {
                        break;
                    }
                    char c21 = cArr[i12];
                    if (c21 != c20) {
                        cArr[i12] = c20;
                        if (c21 < c20) {
                            do {
                                i13++;
                                c = cArr[i13];
                            } while (c < c20);
                            if (c > c20) {
                                i14--;
                                cArr[i14] = c;
                            }
                            cArr[i13] = c21;
                        } else {
                            i14--;
                            cArr[i14] = c21;
                        }
                    }
                }
                cArr[i2] = cArr[i13];
                cArr[i13] = c20;
                sort(cArr, i | 1, i14, i3);
                i3 = i13;
            } else {
                cArr[i7] = cArr[i2];
                cArr[i8] = cArr[i4];
                int i15 = i2;
                do {
                    i15++;
                } while (cArr[i15] < c18);
                int i16 = i4;
                do {
                    i16--;
                } while (cArr[i16] > c4);
                int i17 = i15 - 1;
                int i18 = i16 + 1;
                int i19 = i18;
                while (true) {
                    i18--;
                    if (i18 <= i17) {
                        break;
                    }
                    char c22 = cArr[i18];
                    if (c22 < c18) {
                        while (true) {
                            if (i17 >= i18) {
                                break;
                            }
                            i17++;
                            char c23 = cArr[i17];
                            if (c23 >= c18) {
                                if (c23 > c4) {
                                    i19--;
                                    cArr[i18] = cArr[i19];
                                    cArr[i19] = cArr[i17];
                                } else {
                                    cArr[i18] = c23;
                                }
                                cArr[i17] = c22;
                            }
                        }
                    } else if (c22 > c4) {
                        i19--;
                        cArr[i18] = cArr[i19];
                        cArr[i19] = c22;
                    }
                }
                cArr[i2] = cArr[i17];
                cArr[i17] = c18;
                cArr[i4] = cArr[i19];
                cArr[i19] = c4;
                int i20 = i | 1;
                sort(cArr, i20, i17 + 1, i19);
                sort(cArr, i20, i19 + 1, i3);
                i3 = i17;
            }
        }
    }

    public static void sort(double[] dArr, int i, int i2, int i3) {
        int i4 = i2;
        int i5 = i3;
        int i6 = i5;
        int i7 = 0;
        while (i5 > i4) {
            i5--;
            double d = dArr[i5];
            if (d == FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE && Double.doubleToRawLongBits(d) < 0) {
                i7++;
                dArr[i5] = 0.0d;
            } else if (Double.isNaN(d)) {
                i6--;
                dArr[i5] = dArr[i6];
                dArr[i6] = d;
            }
        }
        int i8 = i6 - i4;
        if (i <= 1 || i8 <= 4096) {
            sort((Sorter) null, dArr, 0, i4, i6);
        } else {
            int depth = getDepth(i, i8 >> 12);
            new Sorter(null, dArr, depth == 0 ? null : new double[i8], i2, i8, i2, depth).invoke();
        }
        int i9 = i7 + 1;
        if (i9 == 1) {
            return;
        }
        while (i4 <= i6) {
            int i10 = (i4 + i6) >>> 1;
            if (dArr[i10] < FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE) {
                i4 = i10 + 1;
            } else {
                i6 = i10 - 1;
            }
        }
        while (true) {
            i9--;
            if (i9 <= 0) {
                return;
            }
            i6++;
            dArr[i6] = -0.0d;
        }
    }

    public static void sort(float[] fArr, int i, int i2, int i3) {
        int i4 = i2;
        int i5 = i3;
        int i6 = i5;
        int i7 = 0;
        while (i5 > i4) {
            i5--;
            float f = fArr[i5];
            if (f == 0.0f && Float.floatToRawIntBits(f) < 0) {
                i7++;
                fArr[i5] = 0.0f;
            } else if (Float.isNaN(f)) {
                i6--;
                fArr[i5] = fArr[i6];
                fArr[i6] = f;
            }
        }
        int i8 = i6 - i4;
        if (i <= 1 || i8 <= 4096) {
            sort((Sorter) null, fArr, 0, i4, i6);
        } else {
            int depth = getDepth(i, i8 >> 12);
            new Sorter(null, fArr, depth == 0 ? null : new float[i8], i2, i8, i2, depth).invoke();
        }
        int i9 = i7 + 1;
        if (i9 == 1) {
            return;
        }
        while (i4 <= i6) {
            int i10 = (i4 + i6) >>> 1;
            if (fArr[i10] < 0.0f) {
                i4 = i10 + 1;
            } else {
                i6 = i10 - 1;
            }
        }
        while (true) {
            i9--;
            if (i9 <= 0) {
                return;
            }
            i6++;
            fArr[i6] = -0.0f;
        }
    }

    public static void sort(int[] iArr, int i, int i2, int i3) {
        int i4 = i3 - i2;
        if (i <= 1 || i4 <= 4096) {
            sort((Sorter) null, iArr, 0, i2, i3);
            return;
        }
        int depth = getDepth(i, i4 >> 12);
        new Sorter(null, iArr, depth == 0 ? null : new int[i4], i2, i4, i2, depth).invoke();
    }

    public static void sort(long[] jArr, int i, int i2, int i3) {
        int i4 = i3 - i2;
        if (i <= 1 || i4 <= 4096) {
            sort((Sorter) null, jArr, 0, i2, i3);
            return;
        }
        int depth = getDepth(i, i4 >> 12);
        new Sorter(null, jArr, depth == 0 ? null : new long[i4], i2, i4, i2, depth).invoke();
    }

    public static void sort(short[] sArr, int i, int i2) {
        if (i2 - i > 1750) {
            countingSort(sArr, i, i2);
        } else {
            sort(sArr, 0, i, i2);
        }
    }

    public static void sort(short[] sArr, int i, int i2, int i3) {
        short s;
        short s2;
        short s3;
        short s4;
        while (true) {
            int i4 = i3 - 1;
            int i5 = i3 - i2;
            if (i5 < 44) {
                insertionSort(sArr, i2, i3);
                return;
            }
            i += 6;
            if (i > 384) {
                countingSort(sArr, i2, i3);
                return;
            }
            int i6 = ((i5 >> 3) * 3) + 3;
            int i7 = i2 + i6;
            int i8 = i4 - i6;
            int i9 = (i7 + i8) >>> 1;
            int i10 = (i7 + i9) >>> 1;
            int i11 = (i9 + i8) >>> 1;
            short s5 = sArr[i9];
            short s6 = sArr[i8];
            short s7 = sArr[i10];
            if (s6 < s7) {
                sArr[i8] = s7;
                sArr[i10] = s6;
            }
            short s8 = sArr[i11];
            short s9 = sArr[i7];
            if (s8 < s9) {
                sArr[i11] = s9;
                sArr[i7] = s8;
            }
            short s10 = sArr[i8];
            short s11 = sArr[i11];
            if (s10 < s11) {
                sArr[i8] = s11;
                sArr[i11] = s10;
            }
            short s12 = sArr[i10];
            short s13 = sArr[i7];
            if (s12 < s13) {
                sArr[i10] = s13;
                sArr[i7] = s12;
            }
            short s14 = sArr[i11];
            short s15 = sArr[i10];
            if (s14 < s15) {
                sArr[i11] = s15;
                sArr[i10] = s14;
            }
            short s16 = sArr[i10];
            if (s5 >= s16) {
                short s17 = sArr[i11];
                if (s5 > s17) {
                    if (s5 > sArr[i8]) {
                        sArr[i9] = s17;
                        sArr[i11] = sArr[i8];
                        sArr[i8] = s5;
                    } else {
                        sArr[i9] = s17;
                        sArr[i11] = s5;
                    }
                }
            } else if (s5 < sArr[i7]) {
                sArr[i9] = s16;
                sArr[i10] = sArr[i7];
                sArr[i7] = s5;
            } else {
                sArr[i9] = s16;
                sArr[i10] = s5;
            }
            short s18 = sArr[i7];
            short s19 = sArr[i10];
            if (s18 >= s19 || s19 >= (s2 = sArr[i9]) || s2 >= (s3 = sArr[i11]) || s3 >= (s4 = sArr[i8])) {
                short s20 = sArr[i9];
                sArr[i9] = sArr[i2];
                int i12 = i4 + 1;
                int i13 = i2;
                int i14 = i12;
                while (true) {
                    i12--;
                    if (i12 <= i13) {
                        break;
                    }
                    short s21 = sArr[i12];
                    if (s21 != s20) {
                        sArr[i12] = s20;
                        if (s21 < s20) {
                            do {
                                i13++;
                                s = sArr[i13];
                            } while (s < s20);
                            if (s > s20) {
                                i14--;
                                sArr[i14] = s;
                            }
                            sArr[i13] = s21;
                        } else {
                            i14--;
                            sArr[i14] = s21;
                        }
                    }
                }
                sArr[i2] = sArr[i13];
                sArr[i13] = s20;
                sort(sArr, i | 1, i14, i3);
                i3 = i13;
            } else {
                sArr[i7] = sArr[i2];
                sArr[i8] = sArr[i4];
                int i15 = i2;
                do {
                    i15++;
                } while (sArr[i15] < s18);
                int i16 = i4;
                do {
                    i16--;
                } while (sArr[i16] > s4);
                int i17 = i15 - 1;
                int i18 = i16 + 1;
                int i19 = i18;
                while (true) {
                    i18--;
                    if (i18 <= i17) {
                        break;
                    }
                    short s22 = sArr[i18];
                    if (s22 < s18) {
                        while (true) {
                            if (i17 >= i18) {
                                break;
                            }
                            i17++;
                            short s23 = sArr[i17];
                            if (s23 >= s18) {
                                if (s23 > s4) {
                                    i19--;
                                    sArr[i18] = sArr[i19];
                                    sArr[i19] = sArr[i17];
                                } else {
                                    sArr[i18] = s23;
                                }
                                sArr[i17] = s22;
                            }
                        }
                    } else if (s22 > s4) {
                        i19--;
                        sArr[i18] = sArr[i19];
                        sArr[i19] = s22;
                    }
                }
                sArr[i2] = sArr[i17];
                sArr[i17] = s18;
                sArr[i4] = sArr[i19];
                sArr[i19] = s4;
                int i20 = i | 1;
                sort(sArr, i20, i17 + 1, i19);
                sort(sArr, i20, i19 + 1, i3);
                i3 = i17;
            }
        }
    }

    public static boolean tryMergeRuns(Sorter sorter, double[] dArr, int i, int i2) {
        double[] dArr2;
        int i3;
        double[] dArr3;
        int i4 = i + i2;
        int i5 = i + 1;
        int[] iArr = null;
        int i6 = 1;
        int i7 = i;
        while (i5 < i4) {
            double d = dArr[i5 - 1];
            double d2 = dArr[i5];
            if (d < d2) {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (dArr[i5 - 1] <= dArr[i5]);
            } else if (d > d2) {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (dArr[i5 - 1] >= dArr[i5]);
                int i8 = i7 - 1;
                int i9 = i5;
                while (true) {
                    i8++;
                    i9--;
                    if (i8 >= i9) {
                        break;
                    }
                    double d3 = dArr[i8];
                    double d4 = dArr[i9];
                    if (d3 <= d4) {
                        break;
                    }
                    dArr[i8] = d4;
                    dArr[i9] = d3;
                }
            } else {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (d2 == dArr[i5]);
                if (i5 < i4) {
                    continue;
                }
            }
            if (iArr == null) {
                if (i5 == i4) {
                    return true;
                }
                if (i5 - i < 16) {
                    return false;
                }
                iArr = new int[((i2 >> 10) | 127) & 1023];
                iArr[0] = i;
            } else if (dArr[i7 - 1] > dArr[i7]) {
                if (i6 > ((i5 - i) >> 7) || (i6 = i6 + 1) == 5120) {
                    return false;
                }
                if (i6 == iArr.length) {
                    iArr = Arrays.copyOf(iArr, i6 << 1);
                }
            }
            iArr[i6] = i5;
            i7 = i5;
        }
        if (i6 > 1) {
            if (sorter == null || (dArr3 = (double[]) sorter.b) == null) {
                dArr2 = new double[i2];
                i3 = i;
            } else {
                i3 = sorter.offset;
                dArr2 = dArr3;
            }
            mergeRuns(dArr, dArr2, i3, 1, sorter != null, iArr, 0, i6);
        }
        return true;
    }

    public static boolean tryMergeRuns(Sorter sorter, float[] fArr, int i, int i2) {
        int i3;
        float[] fArr2;
        float[] fArr3;
        int[] copyOf;
        int i4 = i + i2;
        int i5 = i + 1;
        int[] iArr = null;
        int i6 = 1;
        int i7 = i;
        while (i5 < i4) {
            float f = fArr[i5 - 1];
            float f2 = fArr[i5];
            if (f < f2) {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (fArr[i5 - 1] <= fArr[i5]);
            } else if (f > f2) {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (fArr[i5 - 1] >= fArr[i5]);
                int i8 = i7 - 1;
                int i9 = i5;
                while (true) {
                    i8++;
                    i9--;
                    if (i8 >= i9) {
                        break;
                    }
                    float f3 = fArr[i8];
                    float f4 = fArr[i9];
                    if (f3 <= f4) {
                        break;
                    }
                    fArr[i8] = f4;
                    fArr[i9] = f3;
                }
            } else {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (f2 == fArr[i5]);
                if (i5 < i4) {
                    continue;
                }
            }
            if (iArr != null) {
                if (fArr[i7 - 1] > fArr[i7]) {
                    if (i6 > ((i5 - i) >> 7) || (i6 = i6 + 1) == 5120) {
                        return false;
                    }
                    if (i6 == iArr.length) {
                        copyOf = Arrays.copyOf(iArr, i6 << 1);
                    }
                }
                iArr[i6] = i5;
                i7 = i5;
            } else if (i5 == i4) {
                return true;
            } else {
                if (i5 - i < 16) {
                    return false;
                }
                copyOf = new int[((i2 >> 10) | 127) & 1023];
                copyOf[0] = i;
            }
            iArr = copyOf;
            iArr[i6] = i5;
            i7 = i5;
        }
        if (i6 > 1) {
            if (sorter == null || (fArr3 = (float[]) sorter.b) == null) {
                i3 = i;
                fArr2 = new float[i2];
            } else {
                i3 = sorter.offset;
                fArr2 = fArr3;
            }
            mergeRuns(fArr, fArr2, i3, 1, sorter != null, iArr, 0, i6);
        }
        return true;
    }

    public static boolean tryMergeRuns(Sorter sorter, int[] iArr, int i, int i2) {
        int i3;
        int[] iArr2;
        int[] iArr3;
        int[] copyOf;
        int i4;
        int i5;
        int i6 = i + i2;
        int i7 = i + 1;
        int[] iArr4 = null;
        int i8 = 1;
        int i9 = i;
        while (i7 < i6) {
            int i10 = iArr[i7 - 1];
            int i11 = iArr[i7];
            if (i10 < i11) {
                do {
                    i7++;
                    if (i7 >= i6) {
                        break;
                    }
                } while (iArr[i7 - 1] <= iArr[i7]);
            } else if (i10 > i11) {
                do {
                    i7++;
                    if (i7 >= i6) {
                        break;
                    }
                } while (iArr[i7 - 1] >= iArr[i7]);
                int i12 = i9 - 1;
                int i13 = i7;
                while (true) {
                    i12++;
                    i13--;
                    if (i12 >= i13 || (i4 = iArr[i12]) <= (i5 = iArr[i13])) {
                        break;
                    }
                    iArr[i12] = i5;
                    iArr[i13] = i4;
                }
            } else {
                do {
                    i7++;
                    if (i7 >= i6) {
                        break;
                    }
                } while (i11 == iArr[i7]);
                if (i7 < i6) {
                    continue;
                }
            }
            if (iArr4 != null) {
                if (iArr[i9 - 1] > iArr[i9]) {
                    if (i8 > ((i7 - i) >> 7) || (i8 = i8 + 1) == 5120) {
                        return false;
                    }
                    if (i8 == iArr4.length) {
                        copyOf = Arrays.copyOf(iArr4, i8 << 1);
                    }
                }
                iArr4[i8] = i7;
                i9 = i7;
            } else if (i7 == i6) {
                return true;
            } else {
                if (i7 - i < 16) {
                    return false;
                }
                copyOf = new int[((i2 >> 10) | 127) & 1023];
                copyOf[0] = i;
            }
            iArr4 = copyOf;
            iArr4[i8] = i7;
            i9 = i7;
        }
        if (i8 > 1) {
            if (sorter == null || (iArr3 = (int[]) sorter.b) == null) {
                i3 = i;
                iArr2 = new int[i2];
            } else {
                i3 = sorter.offset;
                iArr2 = iArr3;
            }
            mergeRuns(iArr, iArr2, i3, 1, sorter != null, iArr4, 0, i8);
        }
        return true;
    }

    public static boolean tryMergeRuns(Sorter sorter, long[] jArr, int i, int i2) {
        long[] jArr2;
        int i3;
        long[] jArr3;
        int i4 = i + i2;
        int i5 = i + 1;
        int[] iArr = null;
        int i6 = 1;
        int i7 = i;
        while (i5 < i4) {
            long j = jArr[i5 - 1];
            long j2 = jArr[i5];
            if (j < j2) {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (jArr[i5 - 1] <= jArr[i5]);
            } else if (j > j2) {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (jArr[i5 - 1] >= jArr[i5]);
                int i8 = i7 - 1;
                int i9 = i5;
                while (true) {
                    i8++;
                    i9--;
                    if (i8 >= i9) {
                        break;
                    }
                    long j3 = jArr[i8];
                    long j4 = jArr[i9];
                    if (j3 <= j4) {
                        break;
                    }
                    jArr[i8] = j4;
                    jArr[i9] = j3;
                }
            } else {
                do {
                    i5++;
                    if (i5 >= i4) {
                        break;
                    }
                } while (j2 == jArr[i5]);
                if (i5 < i4) {
                    continue;
                }
            }
            if (iArr == null) {
                if (i5 == i4) {
                    return true;
                }
                if (i5 - i < 16) {
                    return false;
                }
                iArr = new int[((i2 >> 10) | 127) & 1023];
                iArr[0] = i;
            } else if (jArr[i7 - 1] > jArr[i7]) {
                if (i6 > ((i5 - i) >> 7) || (i6 = i6 + 1) == 5120) {
                    return false;
                }
                if (i6 == iArr.length) {
                    iArr = Arrays.copyOf(iArr, i6 << 1);
                }
            }
            iArr[i6] = i5;
            i7 = i5;
        }
        if (i6 > 1) {
            if (sorter == null || (jArr3 = (long[]) sorter.b) == null) {
                jArr2 = new long[i2];
                i3 = i;
            } else {
                i3 = sorter.offset;
                jArr2 = jArr3;
            }
            mergeRuns(jArr, jArr2, i3, 1, sorter != null, iArr, 0, i6);
        }
        return true;
    }
}
