package java9.util;

import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
import java9.util.function.IntConsumer;
/* loaded from: classes19.dex */
public class IntSummaryStatistics implements IntConsumer {
    public long count;
    public int max;
    public int min;
    public long sum;

    public IntSummaryStatistics() {
        this.min = Integer.MAX_VALUE;
        this.max = Integer.MIN_VALUE;
    }

    public IntSummaryStatistics(long j, int i, int i2, long j2) throws IllegalArgumentException {
        this.min = Integer.MAX_VALUE;
        this.max = Integer.MIN_VALUE;
        int i3 = (j > 0L ? 1 : (j == 0L ? 0 : -1));
        if (i3 < 0) {
            throw new IllegalArgumentException("Negative count value");
        }
        if (i3 > 0) {
            if (i > i2) {
                throw new IllegalArgumentException("Minimum greater than maximum");
            }
            this.count = j;
            this.sum = j2;
            this.min = i;
            this.max = i2;
        }
    }

    @Override // java9.util.function.IntConsumer
    public void accept(int i) {
        this.count++;
        this.sum += i;
        this.min = Math.min(this.min, i);
        this.max = Math.max(this.max, i);
    }

    @Override // java9.util.function.IntConsumer
    public /* synthetic */ IntConsumer andThen(IntConsumer intConsumer) {
        return Objects.requireNonNull(intConsumer);
    }

    public void combine(IntSummaryStatistics intSummaryStatistics) {
        this.count += intSummaryStatistics.count;
        this.sum += intSummaryStatistics.sum;
        this.min = Math.min(this.min, intSummaryStatistics.min);
        this.max = Math.max(this.max, intSummaryStatistics.max);
    }

    public final double getAverage() {
        return getCount() > 0 ? getSum() / getCount() : FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE;
    }

    public final long getCount() {
        return this.count;
    }

    public final int getMax() {
        return this.max;
    }

    public final int getMin() {
        return this.min;
    }

    public final long getSum() {
        return this.sum;
    }

    public String toString() {
        return String.format("%s{count=%d, sum=%d, min=%d, average=%f, max=%d}", IntSummaryStatistics.class.getSimpleName(), Long.valueOf(getCount()), Long.valueOf(getSum()), Integer.valueOf(getMin()), Double.valueOf(getAverage()), Integer.valueOf(getMax()));
    }
}
