package java9.util;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java9.util.function.Consumer;
/* loaded from: classes19.dex */
public final class Iterators {

    /* loaded from: classes19.dex */
    public static final class EmptyIt<E> extends ImmutableIt<E> {
        public static final EmptyIt<Object> EMPTY_ITERATOR = new EmptyIt<>();

        @Override // java.util.Iterator
        public boolean hasNext() {
            return false;
        }

        @Override // java.util.Iterator
        public E next() {
            throw new NoSuchElementException();
        }
    }

    /* loaded from: classes19.dex */
    public static abstract class ImmutableIt<T> implements Iterator<T> {
        @Override // java.util.Iterator
        public void remove() {
            throw new UnsupportedOperationException("remove");
        }
    }

    public static <E> Iterator<E> asIterator(final Enumeration<E> enumeration) {
        Objects.requireNonNull(enumeration);
        return new ImmutableIt<E>() { // from class: java9.util.Iterators.1
            @Override // java.util.Iterator
            public boolean hasNext() {
                return enumeration.hasMoreElements();
            }

            /* JADX WARN: Type inference failed for: r0v1, types: [E, java.lang.Object] */
            @Override // java.util.Iterator
            public E next() {
                return enumeration.nextElement();
            }
        };
    }

    public static <T> Iterator<T> emptyIterator() {
        return EmptyIt.EMPTY_ITERATOR;
    }

    public static <E> void forEachRemaining(Iterator<E> it, Consumer<? super E> consumer) {
        Objects.requireNonNull(it);
        Objects.requireNonNull(consumer);
        while (it.hasNext()) {
            consumer.accept(it.next());
        }
    }

    public static <E> Iterator<E> singletonIterator(final E e) {
        return new ImmutableIt<E>() { // from class: java9.util.Iterators.2
            public boolean hasNext = true;

            @Override // java.util.Iterator
            public boolean hasNext() {
                return this.hasNext;
            }

            /* JADX WARN: Type inference failed for: r0v3, types: [E, java.lang.Object] */
            @Override // java.util.Iterator
            public E next() {
                if (this.hasNext) {
                    this.hasNext = false;
                    return e;
                }
                throw new NoSuchElementException();
            }
        };
    }
}
