package java9.util;

import java.util.Comparator;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.locks.ReentrantLock;
import java9.util.Spliterator;
import java9.util.function.Consumer;
import org.aspectj.lang.JoinPoint;
import sun.misc.Unsafe;
/* loaded from: classes19.dex */
public final class LBDSpliterator<E> implements Spliterator<E> {
    public static final long FIRST_OFF;
    public static final long LOCK_OFF;
    public static final int MAX_BATCH = 33554432;
    public static final long NODE_ITEM_OFF;
    public static final long NODE_NEXT_OFF;
    public static final Unsafe U;
    public int batch;
    public Object current;
    public long est;
    public boolean exhausted;
    public final LinkedBlockingDeque<E> queue;
    public final ReentrantLock queueLock;

    static {
        Unsafe unsafe = UnsafeAccess.unsafe;
        U = unsafe;
        try {
            Class<?> cls = Class.forName("java.util.concurrent.LinkedBlockingDeque$Node");
            FIRST_OFF = unsafe.objectFieldOffset(LinkedBlockingDeque.class.getDeclaredField("first"));
            LOCK_OFF = unsafe.objectFieldOffset(LinkedBlockingDeque.class.getDeclaredField(JoinPoint.SYNCHRONIZATION_LOCK));
            NODE_ITEM_OFF = unsafe.objectFieldOffset(cls.getDeclaredField("item"));
            NODE_NEXT_OFF = unsafe.objectFieldOffset(cls.getDeclaredField("next"));
        } catch (Exception e) {
            throw new Error(e);
        }
    }

    public LBDSpliterator(LinkedBlockingDeque<E> linkedBlockingDeque) {
        this.queue = linkedBlockingDeque;
        this.est = linkedBlockingDeque.size();
        this.queueLock = getQueueLock(linkedBlockingDeque);
    }

    public static Object getNextNode(Object obj) {
        return U.getObject(obj, NODE_NEXT_OFF);
    }

    public static <T> T getNodeItem(Object obj) {
        return (T) U.getObject(obj, NODE_ITEM_OFF);
    }

    public static Object getQueueFirst(LinkedBlockingDeque<?> linkedBlockingDeque) {
        return U.getObject(linkedBlockingDeque, FIRST_OFF);
    }

    public static ReentrantLock getQueueLock(LinkedBlockingDeque<?> linkedBlockingDeque) {
        return (ReentrantLock) U.getObject(linkedBlockingDeque, LOCK_OFF);
    }

    public static <T> Spliterator<T> spliterator(LinkedBlockingDeque<T> linkedBlockingDeque) {
        return new LBDSpliterator(linkedBlockingDeque);
    }

    @Override // java9.util.Spliterator
    public int characteristics() {
        return 4368;
    }

    @Override // java9.util.Spliterator
    public long estimateSize() {
        return this.est;
    }

    public void forEachFrom(Consumer<? super E> consumer, Object obj) {
        ReentrantLock reentrantLock = this.queueLock;
        Object[] objArr = null;
        int i = 0;
        do {
            reentrantLock.lock();
            if (objArr == null) {
                if (obj == null) {
                    try {
                        obj = getQueueFirst(this.queue);
                    } catch (Throwable th) {
                        reentrantLock.unlock();
                        throw th;
                    }
                }
                Object obj2 = obj;
                while (obj2 != null && (getNodeItem(obj2) == null || (i = i + 1) != 64)) {
                    obj2 = succ(obj2);
                }
                objArr = new Object[i];
            }
            int i2 = 0;
            while (obj != null && i2 < i) {
                Object nodeItem = getNodeItem(obj);
                objArr[i2] = nodeItem;
                if (nodeItem != null) {
                    i2++;
                }
                obj = succ(obj);
            }
            reentrantLock.unlock();
            for (int i3 = 0; i3 < i2; i3++) {
                consumer.accept(objArr[i3]);
            }
            if (i2 <= 0) {
                return;
            }
        } while (obj != null);
    }

    @Override // java9.util.Spliterator
    public void forEachRemaining(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        if (this.exhausted) {
            return;
        }
        this.exhausted = true;
        Object obj = this.current;
        this.current = null;
        forEachFrom(consumer, obj);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ Comparator getComparator() {
        return Spliterator.CC.$default$getComparator(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ long getExactSizeIfKnown() {
        return Spliterator.CC.$default$getExactSizeIfKnown(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ boolean hasCharacteristics(int i) {
        return Spliterator.CC.$default$hasCharacteristics(this, i);
    }

    public Object succ(Object obj) {
        Object nextNode = getNextNode(obj);
        return obj == nextNode ? getQueueFirst(this.queue) : nextNode;
    }

    /* JADX WARN: Removed duplicated region for block: B:16:0x002c A[Catch: all -> 0x0037, TRY_LEAVE, TryCatch #0 {all -> 0x0037, blocks: (B:5:0x000c, B:7:0x0010, B:14:0x0027, B:16:0x002c, B:11:0x001b), top: B:26:0x000c }] */
    /* JADX WARN: Removed duplicated region for block: B:19:0x0033  */
    /* JADX WARN: Removed duplicated region for block: B:31:? A[RETURN, SYNTHETIC] */
    @Override // java9.util.Spliterator
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean tryAdvance(Consumer<? super E> consumer) {
        Object obj;
        Objects.requireNonNull(consumer);
        if (this.exhausted) {
            return false;
        }
        ReentrantLock reentrantLock = this.queueLock;
        reentrantLock.lock();
        try {
            Object obj2 = this.current;
            if (obj2 == null && (obj2 = getQueueFirst(this.queue)) == null) {
                obj = (Object) null;
                this.current = obj2;
                if (obj2 == null) {
                    this.exhausted = true;
                }
                if (obj == null) {
                    consumer.accept(obj);
                    return true;
                }
                return false;
            }
            do {
                obj = (Object) getNodeItem(obj2);
                obj2 = succ(obj2);
                if (obj != null) {
                    break;
                }
            } while (obj2 != null);
            this.current = obj2;
            if (obj2 == null) {
            }
            if (obj == null) {
            }
        } finally {
            reentrantLock.unlock();
        }
    }

    /* JADX WARN: Finally extract failed */
    /* JADX WARN: Removed duplicated region for block: B:30:0x005a  */
    /* JADX WARN: Removed duplicated region for block: B:31:0x005f  */
    /* JADX WARN: Removed duplicated region for block: B:35:0x006d  */
    /* JADX WARN: Removed duplicated region for block: B:47:? A[RETURN, SYNTHETIC] */
    @Override // java9.util.Spliterator
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public Spliterator<E> trySplit() {
        int i;
        LinkedBlockingDeque<E> linkedBlockingDeque = this.queue;
        if (this.exhausted) {
            return null;
        }
        Object obj = this.current;
        if ((obj == null && (obj = getQueueFirst(linkedBlockingDeque)) == null) || getNextNode(obj) == null) {
            return null;
        }
        int min = Math.min(this.batch + 1, 33554432);
        this.batch = min;
        Object[] objArr = new Object[min];
        ReentrantLock reentrantLock = this.queueLock;
        Object obj2 = this.current;
        reentrantLock.lock();
        if (obj2 == null) {
            try {
                obj2 = getQueueFirst(linkedBlockingDeque);
                if (obj2 == null) {
                    i = 0;
                    reentrantLock.unlock();
                    this.current = obj2;
                    if (obj2 != null) {
                        this.est = 0L;
                        this.exhausted = true;
                    } else {
                        long j = this.est - i;
                        this.est = j;
                        if (j < 0) {
                            this.est = 0L;
                        }
                    }
                    if (i <= 0) {
                        return Spliterators.spliterator(objArr, 0, i, 4368);
                    }
                    return null;
                }
            } catch (Throwable th) {
                reentrantLock.unlock();
                throw th;
            }
        }
        i = 0;
        while (obj2 != null && i < min) {
            Object nodeItem = getNodeItem(obj2);
            objArr[i] = nodeItem;
            if (nodeItem != null) {
                i++;
            }
            obj2 = succ(obj2);
        }
        reentrantLock.unlock();
        this.current = obj2;
        if (obj2 != null) {
        }
        if (i <= 0) {
        }
    }
}
