package java9.util;

import com.google.android.exoplayer2.text.ttml.TtmlNode;
import java.util.Comparator;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.locks.ReentrantLock;
import java9.util.Spliterator;
import java9.util.function.Consumer;
import sun.misc.Unsafe;
/* loaded from: classes19.dex */
public final class LBQSpliterator<E> implements Spliterator<E> {
    public static final long HEAD_OFF;
    public static final int MAX_BATCH = 33554432;
    public static final long NODE_ITEM_OFF;
    public static final long NODE_NEXT_OFF;
    public static final long PUT_LOCK_OFF;
    public static final long TAKE_LOCK_OFF;
    public static final Unsafe U;
    public int batch;
    public Object current;
    public long est;
    public boolean exhausted;
    public final ReentrantLock putLock;
    public final LinkedBlockingQueue<E> queue;
    public final ReentrantLock takeLock;

    static {
        Unsafe unsafe = UnsafeAccess.unsafe;
        U = unsafe;
        try {
            Class<?> cls = Class.forName("java.util.concurrent.LinkedBlockingQueue$Node");
            HEAD_OFF = unsafe.objectFieldOffset(LinkedBlockingQueue.class.getDeclaredField(TtmlNode.TAG_HEAD));
            NODE_ITEM_OFF = unsafe.objectFieldOffset(cls.getDeclaredField("item"));
            NODE_NEXT_OFF = unsafe.objectFieldOffset(cls.getDeclaredField("next"));
            PUT_LOCK_OFF = unsafe.objectFieldOffset(LinkedBlockingQueue.class.getDeclaredField("putLock"));
            TAKE_LOCK_OFF = unsafe.objectFieldOffset(LinkedBlockingQueue.class.getDeclaredField("takeLock"));
        } catch (Exception e) {
            throw new Error(e);
        }
    }

    public LBQSpliterator(LinkedBlockingQueue<E> linkedBlockingQueue) {
        this.queue = linkedBlockingQueue;
        this.est = linkedBlockingQueue.size();
        this.putLock = getPutLock(linkedBlockingQueue);
        this.takeLock = getTakeLock(linkedBlockingQueue);
    }

    private void fullyLock() {
        this.putLock.lock();
        this.takeLock.lock();
    }

    private void fullyUnlock() {
        this.takeLock.unlock();
        this.putLock.unlock();
    }

    public static <T> Object getHeadNext(LinkedBlockingQueue<T> linkedBlockingQueue) {
        return getNextNode(U.getObject(linkedBlockingQueue, HEAD_OFF));
    }

    public static Object getNextNode(Object obj) {
        return U.getObject(obj, NODE_NEXT_OFF);
    }

    public static <T> T getNodeItem(Object obj) {
        return (T) U.getObject(obj, NODE_ITEM_OFF);
    }

    public static ReentrantLock getPutLock(LinkedBlockingQueue<?> linkedBlockingQueue) {
        return (ReentrantLock) U.getObject(linkedBlockingQueue, PUT_LOCK_OFF);
    }

    public static ReentrantLock getTakeLock(LinkedBlockingQueue<?> linkedBlockingQueue) {
        return (ReentrantLock) U.getObject(linkedBlockingQueue, TAKE_LOCK_OFF);
    }

    public static <T> Spliterator<T> spliterator(LinkedBlockingQueue<T> linkedBlockingQueue) {
        return new LBQSpliterator(linkedBlockingQueue);
    }

    @Override // java9.util.Spliterator
    public int characteristics() {
        return 4368;
    }

    @Override // java9.util.Spliterator
    public long estimateSize() {
        return this.est;
    }

    public void forEachFrom(Consumer<? super E> consumer, Object obj) {
        Object[] objArr = null;
        int i = 0;
        do {
            fullyLock();
            if (objArr == null) {
                if (obj == null) {
                    try {
                        obj = getHeadNext(this.queue);
                    } catch (Throwable th) {
                        fullyUnlock();
                        throw th;
                    }
                }
                Object obj2 = obj;
                while (obj2 != null && (getNodeItem(obj2) == null || (i = i + 1) != 64)) {
                    obj2 = succ(obj2);
                }
                objArr = new Object[i];
            }
            int i2 = 0;
            while (obj != null && i2 < i) {
                Object nodeItem = getNodeItem(obj);
                objArr[i2] = nodeItem;
                if (nodeItem != null) {
                    i2++;
                }
                obj = succ(obj);
            }
            fullyUnlock();
            for (int i3 = 0; i3 < i2; i3++) {
                consumer.accept(objArr[i3]);
            }
            if (i2 <= 0) {
                return;
            }
        } while (obj != null);
    }

    @Override // java9.util.Spliterator
    public void forEachRemaining(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        if (this.exhausted) {
            return;
        }
        this.exhausted = true;
        Object obj = this.current;
        this.current = null;
        forEachFrom(consumer, obj);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ Comparator getComparator() {
        return Spliterator.CC.$default$getComparator(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ long getExactSizeIfKnown() {
        return Spliterator.CC.$default$getExactSizeIfKnown(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ boolean hasCharacteristics(int i) {
        return Spliterator.CC.$default$hasCharacteristics(this, i);
    }

    public Object succ(Object obj) {
        Object nextNode = getNextNode(obj);
        return obj == nextNode ? getHeadNext(this.queue) : nextNode;
    }

    /* JADX WARN: Removed duplicated region for block: B:16:0x002a A[Catch: all -> 0x0035, TRY_LEAVE, TryCatch #0 {all -> 0x0035, blocks: (B:5:0x000a, B:7:0x000e, B:14:0x0025, B:16:0x002a, B:11:0x0019), top: B:26:0x000a }] */
    /* JADX WARN: Removed duplicated region for block: B:19:0x0031  */
    /* JADX WARN: Removed duplicated region for block: B:31:? A[RETURN, SYNTHETIC] */
    @Override // java9.util.Spliterator
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public boolean tryAdvance(Consumer<? super E> consumer) {
        Object obj;
        Objects.requireNonNull(consumer);
        if (this.exhausted) {
            return false;
        }
        fullyLock();
        try {
            Object obj2 = this.current;
            if (obj2 == null && (obj2 = getHeadNext(this.queue)) == null) {
                obj = (Object) null;
                this.current = obj2;
                if (obj2 == null) {
                    this.exhausted = true;
                }
                if (obj == null) {
                    consumer.accept(obj);
                    return true;
                }
                return false;
            }
            do {
                obj = (Object) getNodeItem(obj2);
                obj2 = succ(obj2);
                if (obj != null) {
                    break;
                }
            } while (obj2 != null);
            this.current = obj2;
            if (obj2 == null) {
            }
            if (obj == null) {
            }
        } finally {
            fullyUnlock();
        }
    }

    /* JADX WARN: Finally extract failed */
    /* JADX WARN: Removed duplicated region for block: B:30:0x0058  */
    /* JADX WARN: Removed duplicated region for block: B:31:0x005d  */
    /* JADX WARN: Removed duplicated region for block: B:35:0x006b  */
    /* JADX WARN: Removed duplicated region for block: B:47:? A[RETURN, SYNTHETIC] */
    @Override // java9.util.Spliterator
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public Spliterator<E> trySplit() {
        int i;
        LinkedBlockingQueue<E> linkedBlockingQueue = this.queue;
        if (this.exhausted) {
            return null;
        }
        Object obj = this.current;
        if ((obj == null && (obj = getHeadNext(linkedBlockingQueue)) == null) || getNextNode(obj) == null) {
            return null;
        }
        int min = Math.min(this.batch + 1, 33554432);
        this.batch = min;
        Object[] objArr = new Object[min];
        Object obj2 = this.current;
        fullyLock();
        if (obj2 == null) {
            try {
                obj2 = getHeadNext(linkedBlockingQueue);
                if (obj2 == null) {
                    i = 0;
                    fullyUnlock();
                    this.current = obj2;
                    if (obj2 != null) {
                        this.est = 0L;
                        this.exhausted = true;
                    } else {
                        long j = this.est - i;
                        this.est = j;
                        if (j < 0) {
                            this.est = 0L;
                        }
                    }
                    if (i <= 0) {
                        return Spliterators.spliterator(objArr, 0, i, 4368);
                    }
                    return null;
                }
            } catch (Throwable th) {
                fullyUnlock();
                throw th;
            }
        }
        i = 0;
        while (obj2 != null && i < min) {
            Object nodeItem = getNodeItem(obj2);
            objArr[i] = nodeItem;
            if (nodeItem != null) {
                i++;
            }
            obj2 = succ(obj2);
        }
        fullyUnlock();
        this.current = obj2;
        if (obj2 != null) {
        }
        if (i <= 0) {
        }
    }
}
