package java9.util;

import java.util.NoSuchElementException;
import java9.util.function.LongConsumer;
import java9.util.function.LongSupplier;
import java9.util.function.Supplier;
import java9.util.stream.LongStream;
/* loaded from: classes19.dex */
public final class OptionalLong {
    public static final OptionalLong EMPTY = new OptionalLong();
    public final boolean isPresent;
    public final long value;

    /* loaded from: classes19.dex */
    public static final class OLCache {
        public static final OptionalLong[] cache = new OptionalLong[256];

        static {
            int i = 0;
            while (true) {
                OptionalLong[] optionalLongArr = cache;
                if (i >= optionalLongArr.length) {
                    return;
                }
                optionalLongArr[i] = new OptionalLong(i - 128);
                i++;
            }
        }
    }

    public OptionalLong() {
        this.isPresent = false;
        this.value = 0L;
    }

    public OptionalLong(long j) {
        this.isPresent = true;
        this.value = j;
    }

    public static OptionalLong empty() {
        return EMPTY;
    }

    public static OptionalLong of(long j) {
        return (j < -128 || j > 127) ? new OptionalLong(j) : OLCache.cache[((int) j) + 128];
    }

    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof OptionalLong) {
            OptionalLong optionalLong = (OptionalLong) obj;
            boolean z = this.isPresent;
            if (z && optionalLong.isPresent) {
                if (this.value == optionalLong.value) {
                    return true;
                }
            } else if (z == optionalLong.isPresent) {
                return true;
            }
            return false;
        }
        return false;
    }

    public long getAsLong() {
        return orElseThrow();
    }

    public int hashCode() {
        if (this.isPresent) {
            long j = this.value;
            return (int) (j ^ (j >>> 32));
        }
        return 0;
    }

    public void ifPresent(LongConsumer longConsumer) {
        if (this.isPresent) {
            longConsumer.accept(this.value);
        }
    }

    public void ifPresentOrElse(LongConsumer longConsumer, Runnable runnable) {
        if (this.isPresent) {
            longConsumer.accept(this.value);
        } else {
            runnable.run();
        }
    }

    public boolean isEmpty() {
        return !this.isPresent;
    }

    public boolean isPresent() {
        return this.isPresent;
    }

    public long orElse(long j) {
        return this.isPresent ? this.value : j;
    }

    public long orElseGet(LongSupplier longSupplier) {
        return this.isPresent ? this.value : longSupplier.getAsLong();
    }

    public long orElseThrow() {
        if (this.isPresent) {
            return this.value;
        }
        throw new NoSuchElementException("No value present");
    }

    public <X extends Throwable> long orElseThrow(Supplier<? extends X> supplier) throws Throwable {
        if (this.isPresent) {
            return this.value;
        }
        throw supplier.get();
    }

    public LongStream stream() {
        return this.isPresent ? LongStream.CC.of(this.value) : LongStream.CC.empty();
    }

    public String toString() {
        return this.isPresent ? String.format("OptionalLong[%s]", Long.valueOf(this.value)) : "OptionalLong.empty";
    }
}
