package java9.util;

import java.util.AbstractList;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.List;
import java9.util.Spliterator;
import java9.util.function.Consumer;
import java9.util.stream.SpinedBuffer;
import sun.misc.Unsafe;
/* loaded from: classes19.dex */
public final class RASpliterator<E> implements Spliterator<E> {
    public static final long MODCOUNT_OFF;
    public static final Unsafe U;
    public final AbstractList<E> alist;
    public int expectedModCount;
    public int fence;
    public int index;
    public final List<E> list;

    static {
        Unsafe unsafe = UnsafeAccess.unsafe;
        U = unsafe;
        try {
            MODCOUNT_OFF = unsafe.objectFieldOffset(AbstractList.class.getDeclaredField("modCount"));
        } catch (Exception e) {
            throw new Error(e);
        }
    }

    public RASpliterator(List<E> list, int i, int i2, int i3) {
        this.list = list;
        this.index = i;
        this.fence = i2;
        this.alist = list instanceof AbstractList ? (AbstractList) list : null;
        this.expectedModCount = i3;
    }

    public static void checkAbsListModCount(AbstractList<?> abstractList, int i) {
        if (abstractList != null && getModCount(abstractList) != i) {
            throw new ConcurrentModificationException();
        }
    }

    private int getFence() {
        List<E> list = this.list;
        int i = this.fence;
        if (i < 0) {
            AbstractList<E> abstractList = this.alist;
            if (abstractList != null) {
                this.expectedModCount = getModCount(abstractList);
            }
            int size = list.size();
            this.fence = size;
            return size;
        }
        return i;
    }

    public static <T> int getModCount(List<T> list) {
        return U.getInt(list, MODCOUNT_OFF);
    }

    public static <T> Spliterator<T> spliterator(List<T> list) {
        return new RASpliterator(list, 0, -1, 0);
    }

    @Override // java9.util.Spliterator
    public int characteristics() {
        return SpinedBuffer.SPLITERATOR_CHARACTERISTICS;
    }

    @Override // java9.util.Spliterator
    public long estimateSize() {
        return getFence() - this.index;
    }

    @Override // java9.util.Spliterator
    public void forEachRemaining(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        List<E> list = this.list;
        int fence = getFence();
        this.index = fence;
        for (int i = this.index; i < fence; i++) {
            try {
                consumer.accept((E) list.get(i));
            } catch (IndexOutOfBoundsException e) {
                throw new ConcurrentModificationException();
            }
        }
        checkAbsListModCount(this.alist, this.expectedModCount);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ Comparator getComparator() {
        return Spliterator.CC.$default$getComparator(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ long getExactSizeIfKnown() {
        return Spliterator.CC.$default$getExactSizeIfKnown(this);
    }

    @Override // java9.util.Spliterator
    public /* synthetic */ boolean hasCharacteristics(int i) {
        return Spliterator.CC.$default$hasCharacteristics(this, i);
    }

    @Override // java9.util.Spliterator
    public boolean tryAdvance(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        int fence = getFence();
        int i = this.index;
        if (i < fence) {
            this.index = i + 1;
            consumer.accept((E) this.list.get(i));
            checkAbsListModCount(this.alist, this.expectedModCount);
            return true;
        }
        return false;
    }

    @Override // java9.util.Spliterator
    public Spliterator<E> trySplit() {
        int fence = getFence();
        int i = this.index;
        int i2 = (fence + i) >>> 1;
        if (i >= i2) {
            return null;
        }
        List<E> list = this.list;
        this.index = i2;
        return new RASpliterator(list, i, i2, this.expectedModCount);
    }
}
