package java9.util.concurrent;

import java.util.Map;
import java.util.concurrent.ConcurrentMap;
import java9.util.Objects;
import java9.util.function.BiConsumer;
import java9.util.function.BiFunction;
import java9.util.function.Function;
/* loaded from: classes19.dex */
public final class ConcurrentMaps {
    public static /* synthetic */ void a(ConcurrentMap concurrentMap, BiFunction biFunction, Object obj, Object obj2) {
        while (!concurrentMap.replace(obj, obj2, biFunction.apply(obj, obj2)) && (obj2 = concurrentMap.get(obj)) != null) {
        }
    }

    /* JADX WARN: Multi-variable type inference failed */
    public static <K, V> V compute(ConcurrentMap<K, V> concurrentMap, K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(concurrentMap);
        while (true) {
            Object obj = (Object) concurrentMap.get(k);
            while (true) {
                V apply = biFunction.apply(k, obj);
                if (apply != null) {
                    if (obj == null) {
                        obj = (Object) concurrentMap.putIfAbsent(k, apply);
                        if (obj == null) {
                            return apply;
                        }
                    } else if (concurrentMap.replace(k, obj, apply)) {
                        return apply;
                    }
                } else if (obj == null || concurrentMap.remove(k, obj)) {
                    return null;
                }
            }
        }
    }

    public static <K, V> V computeIfAbsent(ConcurrentMap<K, V> concurrentMap, K k, Function<? super K, ? extends V> function) {
        V apply;
        Objects.requireNonNull(concurrentMap);
        Objects.requireNonNull(function);
        V v = concurrentMap.get(k);
        if (v != null || (apply = function.apply(k)) == null) {
            return v;
        }
        V putIfAbsent = concurrentMap.putIfAbsent(k, apply);
        return putIfAbsent == null ? apply : putIfAbsent;
    }

    /* JADX WARN: Multi-variable type inference failed */
    public static <K, V> V computeIfPresent(ConcurrentMap<K, V> concurrentMap, K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        V apply;
        Objects.requireNonNull(concurrentMap);
        Objects.requireNonNull(biFunction);
        while (true) {
            Object obj = (Object) concurrentMap.get(k);
            if (obj == 0) {
                return null;
            }
            apply = biFunction.apply(k, obj);
            if (apply == null) {
                if (concurrentMap.remove(k, obj)) {
                    break;
                }
            } else if (concurrentMap.replace(k, obj, apply)) {
                break;
            }
        }
        return apply;
    }

    public static <K, V> void forEach(ConcurrentMap<K, V> concurrentMap, BiConsumer<? super K, ? super V> biConsumer) {
        Objects.requireNonNull(concurrentMap);
        Objects.requireNonNull(biConsumer);
        for (Map.Entry<K, V> entry : concurrentMap.entrySet()) {
            try {
                biConsumer.accept(entry.getKey(), entry.getValue());
            } catch (IllegalStateException e) {
            }
        }
    }

    public static <K, V> V getOrDefault(ConcurrentMap<K, V> concurrentMap, Object obj, V v) {
        Objects.requireNonNull(concurrentMap);
        V v2 = concurrentMap.get(obj);
        return v2 != null ? v2 : v;
    }

    /* JADX WARN: Multi-variable type inference failed */
    public static <K, V> V merge(ConcurrentMap<K, V> concurrentMap, K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(concurrentMap);
        Objects.requireNonNull(biFunction);
        Objects.requireNonNull(v);
        while (true) {
            Object obj = (Object) concurrentMap.get(k);
            while (obj == null) {
                obj = (Object) concurrentMap.putIfAbsent(k, v);
                if (obj == null) {
                    return v;
                }
            }
            V apply = biFunction.apply(obj, v);
            if (apply != null) {
                if (concurrentMap.replace(k, obj, apply)) {
                    return apply;
                }
            } else if (concurrentMap.remove(k, obj)) {
                return null;
            }
        }
    }

    public static <K, V> void replaceAll(final ConcurrentMap<K, V> concurrentMap, final BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(concurrentMap);
        Objects.requireNonNull(biFunction);
        forEach(concurrentMap, new BiConsumer() { // from class: java9.util.concurrent.ConcurrentMaps$$ExternalSyntheticLambda0
            @Override // java9.util.function.BiConsumer
            public final void accept(Object obj, Object obj2) {
                ConcurrentMaps.a(concurrentMap, biFunction, obj, obj2);
            }

            @Override // java9.util.function.BiConsumer
            public /* synthetic */ BiConsumer andThen(BiConsumer biConsumer) {
                return Objects.requireNonNull(biConsumer);
            }
        });
    }
}
