package java9.util.concurrent;

import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.SecureRandom;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;
import sun.misc.Unsafe;
/* loaded from: classes19.dex */
public final class TLRandom {
    public static final long CCL;
    public static final long GAMMA = -7046029254386353131L;
    public static final long INHERITABLETHREADLOCALS;
    public static final long INHERITEDACCESSCONTROLCONTEXT;
    public static final boolean IS_ANDROID;
    public static final boolean IS_PRE8_IBM;
    public static final int PROBE_INCREMENT = -1640531527;
    public static final long SEEDER_INCREMENT = -4942790177534073029L;
    public static final long THREADLOCALS;
    public static final Unsafe U;
    public static final ThreadLocal<SeedsHolder> localSeeds;
    public static final AtomicInteger probeGenerator;
    public static final AtomicLong seeder;

    /* loaded from: classes19.dex */
    public static final class SeedsHolder {
        public int threadProbe;
        public int threadSecondarySeed;
        public long threadSeed;
    }

    static {
        long j;
        Unsafe unsafe = UnsafeAccess.unsafe;
        U = unsafe;
        try {
            boolean isIBMPre8 = isIBMPre8();
            IS_PRE8_IBM = isIBMPre8;
            boolean isAndroid = isAndroid();
            IS_ANDROID = isAndroid;
            if (isAndroid) {
                j = 0;
                THREADLOCALS = 0L;
                INHERITABLETHREADLOCALS = 0L;
                INHERITEDACCESSCONTROLCONTEXT = 0L;
            } else {
                THREADLOCALS = unsafe.objectFieldOffset(Thread.class.getDeclaredField("threadLocals"));
                INHERITABLETHREADLOCALS = unsafe.objectFieldOffset(Thread.class.getDeclaredField("inheritableThreadLocals"));
                INHERITEDACCESSCONTROLCONTEXT = unsafe.objectFieldOffset(Thread.class.getDeclaredField(isIBMPre8 ? "accessControlContext" : "inheritedAccessControlContext"));
                j = unsafe.objectFieldOffset(Thread.class.getDeclaredField("contextClassLoader"));
            }
            CCL = j;
            localSeeds = new ThreadLocal<SeedsHolder>() { // from class: java9.util.concurrent.TLRandom.1
                @Override // java.lang.ThreadLocal
                public SeedsHolder initialValue() {
                    return new SeedsHolder();
                }
            };
            probeGenerator = new AtomicInteger();
            seeder = new AtomicLong(mix64(System.currentTimeMillis()) ^ mix64(System.nanoTime()));
            if (((Boolean) AccessController.doPrivileged(new PrivilegedAction<Boolean>() { // from class: java9.util.concurrent.TLRandom.2
                /* JADX WARN: Can't rename method to resolve collision */
                @Override // java.security.PrivilegedAction
                public Boolean run() {
                    return Boolean.valueOf(Boolean.getBoolean("java.util.secureRandomSeed"));
                }
            })).booleanValue()) {
                byte[] seed = SecureRandom.getSeed(8);
                long j2 = seed[0] & 255;
                for (int i = 1; i < 8; i++) {
                    j2 = (j2 << 8) | (seed[i] & 255);
                }
                seeder.set(j2);
            }
        } catch (Exception e) {
            throw new ExceptionInInitializerError(e);
        }
    }

    public static final int advanceProbe(int i) {
        int i2 = i ^ (i << 13);
        int i3 = i2 ^ (i2 >>> 17);
        int i4 = i3 ^ (i3 << 5);
        setThreadLocalRandomProbe(i4);
        return i4;
    }

    public static final void eraseThreadLocals(Thread thread) {
        if (IS_ANDROID) {
            return;
        }
        Unsafe unsafe = U;
        unsafe.putObject(thread, THREADLOCALS, (Object) null);
        unsafe.putObject(thread, INHERITABLETHREADLOCALS, (Object) null);
    }

    public static final int getProbe() {
        return getThreadLocalRandomProbe();
    }

    public static int getThreadLocalRandomProbe() {
        return localSeeds.get().threadProbe;
    }

    public static int getThreadLocalRandomSecondarySeed() {
        return localSeeds.get().threadSecondarySeed;
    }

    public static long getThreadLocalRandomSeed() {
        return localSeeds.get().threadSeed;
    }

    public static boolean isAndroid() {
        if (isClassPresent("android.util.DisplayMetrics")) {
            return true;
        }
        return isClassPresent("org.robovm.rt.bro.Bro");
    }

    public static boolean isClassPresent(String str) {
        Class<?> cls;
        try {
            cls = Class.forName(str, false, TLRandom.class.getClassLoader());
        } catch (Throwable th) {
            cls = null;
        }
        return cls != null;
    }

    public static boolean isIBMPre8() {
        String property;
        return isClassPresent("com.ibm.misc.JarVersion") && (property = System.getProperty("java.class.version", "45")) != null && property.length() >= 2 && "52".compareTo(property.substring(0, 2)) > 0;
    }

    public static final void localInit() {
        int addAndGet = probeGenerator.addAndGet(-1640531527);
        if (addAndGet == 0) {
            addAndGet = 1;
        }
        setThreadLocalRandomSeed(mix64(seeder.getAndAdd(SEEDER_INCREMENT)));
        setThreadLocalRandomProbe(addAndGet);
    }

    public static int mix32(long j) {
        long j2 = (j ^ (j >>> 33)) * (-49064778989728563L);
        return (int) (((j2 ^ (j2 >>> 33)) * (-4265267296055464877L)) >>> 32);
    }

    public static long mix64(long j) {
        long j2 = (j ^ (j >>> 33)) * (-49064778989728563L);
        long j3 = (j2 ^ (j2 >>> 33)) * (-4265267296055464877L);
        return j3 ^ (j3 >>> 33);
    }

    public static final int nextSecondarySeed() {
        int mix32;
        int threadLocalRandomSecondarySeed = getThreadLocalRandomSecondarySeed();
        if (threadLocalRandomSecondarySeed != 0) {
            int i = threadLocalRandomSecondarySeed ^ (threadLocalRandomSecondarySeed << 13);
            int i2 = i ^ (i >>> 17);
            mix32 = i2 ^ (i2 << 5);
        } else {
            mix32 = mix32(seeder.getAndAdd(SEEDER_INCREMENT));
            if (mix32 == 0) {
                mix32 = 1;
            }
        }
        setThreadLocalRandomSecondarySeed(mix32);
        return mix32;
    }

    public static final long nextSeed() {
        long threadLocalRandomSeed = getThreadLocalRandomSeed() - 7046029254386353131L;
        setThreadLocalRandomSeed(threadLocalRandomSeed);
        return threadLocalRandomSeed;
    }

    public static final void setContextClassLoader(Thread thread, ClassLoader classLoader) {
        if (IS_ANDROID) {
            return;
        }
        U.putObject(thread, CCL, classLoader);
    }

    public static final void setInheritedAccessControlContext(Thread thread, AccessControlContext accessControlContext) {
        if (IS_ANDROID) {
            return;
        }
        U.putOrderedObject(thread, INHERITEDACCESSCONTROLCONTEXT, accessControlContext);
    }

    public static void setThreadLocalRandomProbe(int i) {
        localSeeds.get().threadProbe = i;
    }

    public static void setThreadLocalRandomSecondarySeed(int i) {
        localSeeds.get().threadSecondarySeed = i;
    }

    public static void setThreadLocalRandomSeed(long j) {
        localSeeds.get().threadSeed = j;
    }
}
