package java9.util.stream;

import java9.util.Objects;
import java9.util.Spliterator;
import java9.util.function.Consumer;
import java9.util.function.IntFunction;
import java9.util.function.Supplier;
import java9.util.stream.BaseStream;
import java9.util.stream.Node;
/* loaded from: classes19.dex */
public abstract class AbstractPipeline<E_IN, E_OUT, S extends BaseStream<E_OUT, S>> extends PipelineHelper<E_OUT> implements BaseStream<E_OUT, S> {
    public static final String MSG_CONSUMED = "source already consumed or closed";
    public static final String MSG_STREAM_LINKED = "stream has already been operated upon or closed";
    public int combinedFlags;
    public int depth;
    public boolean linkedOrConsumed;
    public AbstractPipeline nextStage;
    public boolean parallel;
    public final AbstractPipeline previousStage;
    public boolean sourceAnyStateful;
    public Runnable sourceCloseAction;
    public final int sourceOrOpFlags;
    public Spliterator<?> sourceSpliterator;
    public final AbstractPipeline sourceStage;
    public Supplier<? extends Spliterator<?>> sourceSupplier;

    public AbstractPipeline(Spliterator<?> spliterator, int i, boolean z) {
        this.previousStage = null;
        this.sourceSpliterator = spliterator;
        this.sourceStage = this;
        int i2 = StreamOpFlag.STREAM_MASK & i;
        this.sourceOrOpFlags = i2;
        this.combinedFlags = (~(i2 << 1)) & StreamOpFlag.INITIAL_OPS_VALUE;
        this.depth = 0;
        this.parallel = z;
    }

    public AbstractPipeline(Supplier<? extends Spliterator<?>> supplier, int i, boolean z) {
        this.previousStage = null;
        this.sourceSupplier = supplier;
        this.sourceStage = this;
        int i2 = StreamOpFlag.STREAM_MASK & i;
        this.sourceOrOpFlags = i2;
        this.combinedFlags = (~(i2 << 1)) & StreamOpFlag.INITIAL_OPS_VALUE;
        this.depth = 0;
        this.parallel = z;
    }

    public AbstractPipeline(AbstractPipeline<?, E_IN, ?> abstractPipeline, int i) {
        if (abstractPipeline.linkedOrConsumed) {
            throw new IllegalStateException(MSG_STREAM_LINKED);
        }
        abstractPipeline.linkedOrConsumed = true;
        abstractPipeline.nextStage = this;
        this.previousStage = abstractPipeline;
        this.sourceOrOpFlags = StreamOpFlag.OP_MASK & i;
        this.combinedFlags = StreamOpFlag.combineOpFlags(i, abstractPipeline.combinedFlags);
        AbstractPipeline abstractPipeline2 = abstractPipeline.sourceStage;
        this.sourceStage = abstractPipeline2;
        if (opIsStateful()) {
            abstractPipeline2.sourceAnyStateful = true;
        }
        this.depth = abstractPipeline.depth + 1;
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ Spliterator a() {
        return sourceSpliterator(0);
    }

    public static /* synthetic */ Spliterator a(Spliterator spliterator) {
        return spliterator;
    }

    public static /* synthetic */ Object[] a(int i) {
        return new Object[i];
    }

    private Spliterator<?> sourceSpliterator(int i) {
        Spliterator spliterator;
        int i2;
        int i3;
        AbstractPipeline abstractPipeline = this.sourceStage;
        Spliterator spliterator2 = abstractPipeline.sourceSpliterator;
        if (spliterator2 != null) {
            abstractPipeline.sourceSpliterator = null;
            spliterator = spliterator2;
        } else {
            Supplier<? extends Spliterator<?>> supplier = abstractPipeline.sourceSupplier;
            if (supplier == null) {
                throw new IllegalStateException(MSG_CONSUMED);
            }
            Spliterator spliterator3 = supplier.get();
            this.sourceStage.sourceSupplier = null;
            spliterator = spliterator3;
        }
        if (isParallel()) {
            AbstractPipeline<E_IN, E_OUT, S> abstractPipeline2 = this.sourceStage;
            if (abstractPipeline2.sourceAnyStateful) {
                int i4 = 1;
                spliterator = spliterator;
                AbstractPipeline abstractPipeline3 = abstractPipeline2.nextStage;
                while (abstractPipeline2 != this) {
                    int i5 = abstractPipeline3.sourceOrOpFlags;
                    if (abstractPipeline3.opIsStateful()) {
                        if (StreamOpFlag.SHORT_CIRCUIT.isKnown(i5)) {
                            i5 &= ~StreamOpFlag.IS_SHORT_CIRCUIT;
                        }
                        spliterator = (Spliterator<E_OUT>) abstractPipeline3.opEvaluateParallelLazy(abstractPipeline2, spliterator);
                        if (spliterator.hasCharacteristics(64)) {
                            i2 = (~StreamOpFlag.NOT_SIZED) & i5;
                            i3 = StreamOpFlag.IS_SIZED;
                        } else {
                            i2 = (~StreamOpFlag.IS_SIZED) & i5;
                            i3 = StreamOpFlag.NOT_SIZED;
                        }
                        i5 = i3 | i2;
                        i4 = 0;
                    }
                    abstractPipeline3.depth = i4;
                    abstractPipeline3.combinedFlags = StreamOpFlag.combineOpFlags(i5, abstractPipeline2.combinedFlags);
                    i4++;
                    AbstractPipeline<E_IN, E_OUT, S> abstractPipeline4 = abstractPipeline3;
                    abstractPipeline3 = abstractPipeline3.nextStage;
                    abstractPipeline2 = abstractPipeline4;
                    spliterator = spliterator;
                }
            }
        }
        if (i != 0) {
            this.combinedFlags = StreamOpFlag.combineOpFlags(i, this.combinedFlags);
        }
        return spliterator;
    }

    @Override // java9.util.stream.BaseStream
    public void close() {
        this.linkedOrConsumed = true;
        this.sourceSupplier = null;
        this.sourceSpliterator = null;
        AbstractPipeline abstractPipeline = this.sourceStage;
        Runnable runnable = abstractPipeline.sourceCloseAction;
        if (runnable != null) {
            abstractPipeline.sourceCloseAction = null;
            runnable.run();
        }
    }

    @Override // java9.util.stream.PipelineHelper
    public final <P_IN> void copyInto(Sink<P_IN> sink, Spliterator<P_IN> spliterator) {
        Objects.requireNonNull(sink);
        if (StreamOpFlag.SHORT_CIRCUIT.isKnown(getStreamAndOpFlags())) {
            copyIntoWithCancel(sink, spliterator);
            return;
        }
        sink.begin(spliterator.getExactSizeIfKnown());
        spliterator.forEachRemaining(sink);
        sink.end();
    }

    /* JADX WARN: Multi-variable type inference failed */
    @Override // java9.util.stream.PipelineHelper
    public final <P_IN> boolean copyIntoWithCancel(Sink<P_IN> sink, Spliterator<P_IN> spliterator) {
        AbstractPipeline<E_IN, E_OUT, S> abstractPipeline = this;
        while (abstractPipeline.depth > 0) {
            abstractPipeline = abstractPipeline.previousStage;
        }
        sink.begin(spliterator.getExactSizeIfKnown());
        boolean forEachWithCancel = abstractPipeline.forEachWithCancel(spliterator, sink);
        sink.end();
        return forEachWithCancel;
    }

    public final <R> R evaluate(TerminalOp<E_OUT, R> terminalOp) {
        if (this.linkedOrConsumed) {
            throw new IllegalStateException(MSG_STREAM_LINKED);
        }
        this.linkedOrConsumed = true;
        return isParallel() ? terminalOp.evaluateParallel(this, sourceSpliterator(terminalOp.getOpFlags())) : terminalOp.evaluateSequential(this, sourceSpliterator(terminalOp.getOpFlags()));
    }

    @Override // java9.util.stream.PipelineHelper
    public final <P_IN> Node<E_OUT> evaluate(Spliterator<P_IN> spliterator, boolean z, IntFunction<E_OUT[]> intFunction) {
        return isParallel() ? evaluateToNode(this, spliterator, z, intFunction) : ((Node.Builder) wrapAndCopyInto(makeNodeBuilder(exactOutputSizeIfKnown(spliterator), intFunction), spliterator)).build();
    }

    public final Node<E_OUT> evaluateToArrayNode(IntFunction<E_OUT[]> intFunction) {
        if (this.linkedOrConsumed) {
            throw new IllegalStateException(MSG_STREAM_LINKED);
        }
        this.linkedOrConsumed = true;
        if (isParallel() && this.previousStage != null && opIsStateful()) {
            this.depth = 0;
            AbstractPipeline abstractPipeline = this.previousStage;
            return opEvaluateParallel(abstractPipeline, abstractPipeline.sourceSpliterator(0), intFunction);
        }
        return evaluate(sourceSpliterator(0), true, intFunction);
    }

    public abstract <P_IN> Node<E_OUT> evaluateToNode(PipelineHelper<E_OUT> pipelineHelper, Spliterator<P_IN> spliterator, boolean z, IntFunction<E_OUT[]> intFunction);

    public long exactOutputSize(long j) {
        return j;
    }

    @Override // java9.util.stream.PipelineHelper
    public final <P_IN> long exactOutputSizeIfKnown(Spliterator<P_IN> spliterator) {
        int streamAndOpFlags = getStreamAndOpFlags();
        long exactSizeIfKnown = StreamOpFlag.SIZED.isKnown(streamAndOpFlags) ? spliterator.getExactSizeIfKnown() : -1L;
        if (exactSizeIfKnown != -1 && StreamOpFlag.SIZE_ADJUSTING.isKnown(streamAndOpFlags) && !isParallel()) {
            AbstractPipeline abstractPipeline = this.sourceStage;
            while (true) {
                abstractPipeline = abstractPipeline.nextStage;
                if (abstractPipeline == null) {
                    break;
                }
                exactSizeIfKnown = abstractPipeline.exactOutputSize(exactSizeIfKnown);
            }
        }
        return exactSizeIfKnown;
    }

    public abstract boolean forEachWithCancel(Spliterator<E_OUT> spliterator, Sink<E_OUT> sink);

    public abstract StreamShape getOutputShape();

    @Override // java9.util.stream.PipelineHelper
    public final StreamShape getSourceShape() {
        AbstractPipeline<E_IN, E_OUT, S> abstractPipeline = this;
        while (abstractPipeline.depth > 0) {
            abstractPipeline = abstractPipeline.previousStage;
        }
        return abstractPipeline.getOutputShape();
    }

    @Override // java9.util.stream.PipelineHelper
    public final int getStreamAndOpFlags() {
        return this.combinedFlags;
    }

    public final int getStreamFlags() {
        return StreamOpFlag.toStreamFlags(this.combinedFlags);
    }

    public final boolean isOrdered() {
        return StreamOpFlag.ORDERED.isKnown(this.combinedFlags);
    }

    @Override // java9.util.stream.BaseStream
    public final boolean isParallel() {
        return this.sourceStage.parallel;
    }

    public abstract Spliterator<E_OUT> lazySpliterator(Supplier<? extends Spliterator<E_OUT>> supplier);

    @Override // java9.util.stream.PipelineHelper
    public abstract Node.Builder<E_OUT> makeNodeBuilder(long j, IntFunction<E_OUT[]> intFunction);

    @Override // java9.util.stream.BaseStream
    public S onClose(Runnable runnable) {
        if (this.linkedOrConsumed) {
            throw new IllegalStateException(MSG_STREAM_LINKED);
        }
        Objects.requireNonNull(runnable);
        AbstractPipeline abstractPipeline = this.sourceStage;
        Runnable runnable2 = abstractPipeline.sourceCloseAction;
        if (runnable2 != null) {
            runnable = Streams.composeWithExceptions(runnable2, runnable);
        }
        abstractPipeline.sourceCloseAction = runnable;
        return this;
    }

    public <P_IN> Node<E_OUT> opEvaluateParallel(PipelineHelper<E_OUT> pipelineHelper, Spliterator<P_IN> spliterator, IntFunction<E_OUT[]> intFunction) {
        throw new UnsupportedOperationException("Parallel evaluation is not supported");
    }

    public <P_IN> Spliterator<E_OUT> opEvaluateParallelLazy(PipelineHelper<E_OUT> pipelineHelper, Spliterator<P_IN> spliterator) {
        return opEvaluateParallel(pipelineHelper, spliterator, new IntFunction() { // from class: java9.util.stream.AbstractPipeline$$ExternalSyntheticLambda1
            @Override // java9.util.function.IntFunction
            public final Object apply(int i) {
                return AbstractPipeline.a(i);
            }
        }).spliterator();
    }

    public abstract boolean opIsStateful();

    public abstract Sink<E_IN> opWrapSink(int i, Sink<E_OUT> sink);

    @Override // java9.util.stream.BaseStream
    public final S parallel() {
        this.sourceStage.parallel = true;
        return this;
    }

    @Override // java9.util.stream.BaseStream
    public final S sequential() {
        this.sourceStage.parallel = false;
        return this;
    }

    public final Spliterator<E_OUT> sourceStageSpliterator() {
        AbstractPipeline<E_IN, E_OUT, S> abstractPipeline = this.sourceStage;
        if (this == abstractPipeline) {
            if (this.linkedOrConsumed) {
                throw new IllegalStateException(MSG_STREAM_LINKED);
            }
            this.linkedOrConsumed = true;
            Spliterator<E_OUT> spliterator = (Spliterator<E_OUT>) abstractPipeline.sourceSpliterator;
            if (spliterator != null) {
                abstractPipeline.sourceSpliterator = null;
                return spliterator;
            }
            Supplier<? extends Spliterator<?>> supplier = abstractPipeline.sourceSupplier;
            if (supplier != null) {
                Spliterator<E_OUT> spliterator2 = (Spliterator) supplier.get();
                this.sourceStage.sourceSupplier = null;
                return spliterator2;
            }
            throw new IllegalStateException(MSG_CONSUMED);
        }
        throw new IllegalStateException();
    }

    @Override // java9.util.stream.BaseStream
    public Spliterator<E_OUT> spliterator() {
        if (this.linkedOrConsumed) {
            throw new IllegalStateException(MSG_STREAM_LINKED);
        }
        this.linkedOrConsumed = true;
        AbstractPipeline<E_IN, E_OUT, S> abstractPipeline = this.sourceStage;
        if (this == abstractPipeline) {
            Spliterator<E_OUT> spliterator = (Spliterator<E_OUT>) abstractPipeline.sourceSpliterator;
            if (spliterator != null) {
                abstractPipeline.sourceSpliterator = null;
                return spliterator;
            }
            Supplier<? extends Spliterator<?>> supplier = abstractPipeline.sourceSupplier;
            if (supplier != null) {
                abstractPipeline.sourceSupplier = null;
                return lazySpliterator(supplier);
            }
            throw new IllegalStateException(MSG_CONSUMED);
        }
        return wrap(this, new Supplier() { // from class: java9.util.stream.AbstractPipeline$$ExternalSyntheticLambda2
            @Override // java9.util.function.Supplier
            public final Object get() {
                Spliterator a2;
                a2 = AbstractPipeline.this.a();
                return a2;
            }
        }, isParallel());
    }

    public abstract <P_IN> Spliterator<E_OUT> wrap(PipelineHelper<E_OUT> pipelineHelper, Supplier<Spliterator<P_IN>> supplier, boolean z);

    @Override // java9.util.stream.PipelineHelper
    public final <P_IN, S_ extends Sink<E_OUT>> S_ wrapAndCopyInto(S_ s_, Spliterator<P_IN> spliterator) {
        copyInto(wrapSink((Sink) Objects.requireNonNull(s_)), spliterator);
        return s_;
    }

    @Override // java9.util.stream.PipelineHelper
    public final <P_IN> Sink<P_IN> wrapSink(Sink<E_OUT> sink) {
        Objects.requireNonNull(sink);
        AbstractPipeline<E_IN, E_OUT, S> abstractPipeline = this;
        Consumer consumer = sink;
        while (abstractPipeline.depth > 0) {
            abstractPipeline = abstractPipeline.previousStage;
            consumer = (Sink<E_IN>) abstractPipeline.opWrapSink(abstractPipeline.previousStage.combinedFlags, (Sink) consumer);
        }
        return (Sink<P_IN>) consumer;
    }

    /* JADX WARN: Multi-variable type inference failed */
    @Override // java9.util.stream.PipelineHelper
    public final <P_IN> Spliterator<E_OUT> wrapSpliterator(final Spliterator<P_IN> spliterator) {
        return this.depth == 0 ? spliterator : wrap(this, new Supplier() { // from class: java9.util.stream.AbstractPipeline$$ExternalSyntheticLambda0
            @Override // java9.util.function.Supplier
            public final Object get() {
                return AbstractPipeline.a(Spliterator.this);
            }
        }, isParallel());
    }
}
