package java9.util.stream;

import java.util.EnumMap;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;
import java9.util.Maps;
import java9.util.Spliterator;
/* loaded from: classes19.dex */
public enum StreamOpFlag {
    DISTINCT(0, set(Type.SPLITERATOR).set(Type.STREAM).setAndClear(Type.OP)),
    SORTED(1, set(Type.SPLITERATOR).set(Type.STREAM).setAndClear(Type.OP)),
    ORDERED(2, set(Type.SPLITERATOR).set(Type.STREAM).setAndClear(Type.OP).clear(Type.TERMINAL_OP).clear(Type.UPSTREAM_TERMINAL_OP)),
    SIZED(3, set(Type.SPLITERATOR).set(Type.STREAM).clear(Type.OP)),
    SHORT_CIRCUIT(12, set(Type.OP).set(Type.TERMINAL_OP)),
    SIZE_ADJUSTING(13, set(Type.OP));
    
    public static final int CLEAR_BITS = 2;
    public static final int FLAG_MASK;
    public static final int FLAG_MASK_IS;
    public static final int FLAG_MASK_NOT;
    public static final int INITIAL_OPS_VALUE;
    public static final int IS_DISTINCT;
    public static final int IS_ORDERED;
    public static final int IS_SHORT_CIRCUIT;
    public static final int IS_SIZED;
    public static final int IS_SIZE_ADJUSTING;
    public static final int IS_SORTED;
    public static final int NOT_DISTINCT;
    public static final int NOT_ORDERED;
    public static final int NOT_SIZED;
    public static final int NOT_SORTED;
    public static final int OP_MASK;
    public static final int PRESERVE_BITS = 3;
    public static final int SET_BITS = 1;
    public static final int SPLITERATOR_CHARACTERISTICS_MASK;
    public static final int STREAM_MASK;
    public static final int TERMINAL_OP_MASK;
    public static final int UPSTREAM_TERMINAL_OP_MASK;
    public final int bitPosition;
    public final int clear;
    public final Map<Type, Integer> maskTable;
    public final int preserve;
    public final int set;

    /* loaded from: classes19.dex */
    public static class MaskBuilder {
        public final Map<Type, Integer> map;

        public MaskBuilder(Map<Type, Integer> map) {
            this.map = map;
        }

        public Map<Type, Integer> build() {
            Map<Type, Integer> map = this.map;
            int i = 0;
            if (map instanceof ConcurrentMap) {
                ConcurrentMap concurrentMap = (ConcurrentMap) map;
                Type[] values = Type.values();
                int length = values.length;
                while (i < length) {
                    concurrentMap.putIfAbsent(values[i], 0);
                    i++;
                }
                return concurrentMap;
            }
            Type[] values2 = Type.values();
            int length2 = values2.length;
            while (i < length2) {
                Maps.putIfAbsent(this.map, values2[i], 0);
                i++;
            }
            return this.map;
        }

        public MaskBuilder clear(Type type) {
            return mask(type, 2);
        }

        public MaskBuilder mask(Type type, Integer num) {
            this.map.put(type, num);
            return this;
        }

        public MaskBuilder set(Type type) {
            return mask(type, 1);
        }

        public MaskBuilder setAndClear(Type type) {
            return mask(type, 3);
        }
    }

    /* loaded from: classes19.dex */
    public enum Type {
        SPLITERATOR,
        STREAM,
        OP,
        TERMINAL_OP,
        UPSTREAM_TERMINAL_OP
    }

    static {
        StreamOpFlag streamOpFlag = DISTINCT;
        StreamOpFlag streamOpFlag2 = SORTED;
        StreamOpFlag streamOpFlag3 = ORDERED;
        StreamOpFlag streamOpFlag4 = SIZED;
        StreamOpFlag streamOpFlag5 = SHORT_CIRCUIT;
        StreamOpFlag streamOpFlag6 = SIZE_ADJUSTING;
        SPLITERATOR_CHARACTERISTICS_MASK = createMask(Type.SPLITERATOR);
        int createMask = createMask(Type.STREAM);
        STREAM_MASK = createMask;
        OP_MASK = createMask(Type.OP);
        TERMINAL_OP_MASK = createMask(Type.TERMINAL_OP);
        UPSTREAM_TERMINAL_OP_MASK = createMask(Type.UPSTREAM_TERMINAL_OP);
        FLAG_MASK = createFlagMask();
        FLAG_MASK_IS = createMask;
        int i = createMask << 1;
        FLAG_MASK_NOT = i;
        INITIAL_OPS_VALUE = createMask | i;
        IS_DISTINCT = streamOpFlag.set;
        NOT_DISTINCT = streamOpFlag.clear;
        IS_SORTED = streamOpFlag2.set;
        NOT_SORTED = streamOpFlag2.clear;
        IS_ORDERED = streamOpFlag3.set;
        NOT_ORDERED = streamOpFlag3.clear;
        IS_SIZED = streamOpFlag4.set;
        NOT_SIZED = streamOpFlag4.clear;
        IS_SHORT_CIRCUIT = streamOpFlag5.set;
        IS_SIZE_ADJUSTING = streamOpFlag6.set;
    }

    StreamOpFlag(int i, MaskBuilder maskBuilder) {
        this.maskTable = maskBuilder.build();
        int i2 = i * 2;
        this.bitPosition = i2;
        this.set = 1 << i2;
        this.clear = 2 << i2;
        this.preserve = 3 << i2;
    }

    public static int combineOpFlags(int i, int i2) {
        return i | (i2 & getMask(i));
    }

    public static int createFlagMask() {
        int i = 0;
        for (StreamOpFlag streamOpFlag : values()) {
            i |= streamOpFlag.preserve;
        }
        return i;
    }

    public static int createMask(Type type) {
        StreamOpFlag[] values;
        int i = 0;
        for (StreamOpFlag streamOpFlag : values()) {
            i |= streamOpFlag.maskTable.get(type).intValue() << streamOpFlag.bitPosition;
        }
        return i;
    }

    public static int fromCharacteristics(int i) {
        return i & SPLITERATOR_CHARACTERISTICS_MASK;
    }

    public static int fromCharacteristics(Spliterator<?> spliterator) {
        int characteristics = spliterator.characteristics();
        return ((characteristics & 4) == 0 || spliterator.getComparator() == null) ? SPLITERATOR_CHARACTERISTICS_MASK & characteristics : SPLITERATOR_CHARACTERISTICS_MASK & characteristics & (-5);
    }

    public static int getMask(int i) {
        if (i == 0) {
            return FLAG_MASK;
        }
        return ~(((i & FLAG_MASK_NOT) >> 1) | ((FLAG_MASK_IS & i) << 1) | i);
    }

    public static MaskBuilder set(Type type) {
        return new MaskBuilder(new EnumMap(Type.class)).set(type);
    }

    public static int toCharacteristics(int i) {
        return i & SPLITERATOR_CHARACTERISTICS_MASK;
    }

    public static int toStreamFlags(int i) {
        return i & ((~i) >> 1) & FLAG_MASK_IS;
    }

    public boolean canSet(Type type) {
        return (this.maskTable.get(type).intValue() & 1) > 0;
    }

    public int clear() {
        return this.clear;
    }

    public boolean isCleared(int i) {
        return (i & this.preserve) == this.clear;
    }

    public boolean isKnown(int i) {
        return (i & this.preserve) == this.set;
    }

    public boolean isPreserved(int i) {
        int i2 = this.preserve;
        return (i & i2) == i2;
    }

    public boolean isStreamFlag() {
        return this.maskTable.get(Type.STREAM).intValue() > 0;
    }

    public int set() {
        return this.set;
    }
}
