package java9.util.stream;

import java.util.Comparator;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicLong;
import java9.util.Objects;
import java9.util.RefConsumer;
import java9.util.Spliterator;
import java9.util.concurrent.ForkJoinPool;
import java9.util.function.BooleanSupplier;
import java9.util.function.Consumer;
import java9.util.function.DoubleConsumer;
import java9.util.function.DoubleSupplier;
import java9.util.function.IntConsumer;
import java9.util.function.IntSupplier;
import java9.util.function.LongConsumer;
import java9.util.function.LongSupplier;
import java9.util.function.Supplier;
import java9.util.stream.Sink;
import java9.util.stream.SpinedBuffer;
import java9.util.stream.StreamSpliterators;
/* loaded from: classes19.dex */
public class StreamSpliterators {

    /* loaded from: classes19.dex */
    public static abstract class AbstractWrappingSpliterator<P_IN, P_OUT, T_BUFFER extends AbstractSpinedBuffer> implements Spliterator<P_OUT> {
        public T_BUFFER buffer;
        public Sink<P_IN> bufferSink;
        public boolean finished;
        public final boolean isParallel;
        public long nextToConsume;
        public final PipelineHelper<P_OUT> ph;
        public BooleanSupplier pusher;
        public Spliterator<P_IN> spliterator;
        public Supplier<Spliterator<P_IN>> spliteratorSupplier;

        public AbstractWrappingSpliterator(PipelineHelper<P_OUT> pipelineHelper, Spliterator<P_IN> spliterator, boolean z) {
            this.ph = pipelineHelper;
            this.spliteratorSupplier = null;
            this.spliterator = spliterator;
            this.isParallel = z;
        }

        public AbstractWrappingSpliterator(PipelineHelper<P_OUT> pipelineHelper, Supplier<Spliterator<P_IN>> supplier, boolean z) {
            this.ph = pipelineHelper;
            this.spliteratorSupplier = supplier;
            this.spliterator = null;
            this.isParallel = z;
        }

        private boolean fillBuffer() {
            while (this.buffer.count() == 0) {
                if (this.bufferSink.cancellationRequested() || !this.pusher.getAsBoolean()) {
                    if (this.finished) {
                        return false;
                    }
                    this.bufferSink.end();
                    this.finished = true;
                }
            }
            return true;
        }

        @Override // java9.util.Spliterator
        public final int characteristics() {
            init();
            int characteristics = StreamOpFlag.toCharacteristics(StreamOpFlag.toStreamFlags(this.ph.getStreamAndOpFlags()));
            return (characteristics & 64) != 0 ? (characteristics & (-16449)) | (this.spliterator.characteristics() & 16448) : characteristics;
        }

        public final boolean doAdvance() {
            T_BUFFER t_buffer = this.buffer;
            if (t_buffer == null) {
                if (this.finished) {
                    return false;
                }
                init();
                initPartialTraversalState();
                this.nextToConsume = 0L;
                this.bufferSink.begin(this.spliterator.getExactSizeIfKnown());
                return fillBuffer();
            }
            long j = this.nextToConsume + 1;
            this.nextToConsume = j;
            boolean z = j < t_buffer.count();
            if (z) {
                return z;
            }
            this.nextToConsume = 0L;
            this.buffer.clear();
            return fillBuffer();
        }

        @Override // java9.util.Spliterator
        public final long estimateSize() {
            long exactSizeIfKnown = getExactSizeIfKnown();
            return exactSizeIfKnown == -1 ? this.spliterator.estimateSize() : exactSizeIfKnown;
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ void forEachRemaining(Consumer consumer) {
            Spliterator.CC.$default$forEachRemaining(this, consumer);
        }

        @Override // java9.util.Spliterator
        public Comparator<? super P_OUT> getComparator() {
            if (hasCharacteristics(4)) {
                return null;
            }
            throw new IllegalStateException();
        }

        @Override // java9.util.Spliterator
        public final long getExactSizeIfKnown() {
            init();
            return this.ph.exactOutputSizeIfKnown(this.spliterator);
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean hasCharacteristics(int i) {
            return Spliterator.CC.$default$hasCharacteristics(this, i);
        }

        public final void init() {
            if (this.spliterator == null) {
                this.spliterator = this.spliteratorSupplier.get();
                this.spliteratorSupplier = null;
            }
        }

        public abstract void initPartialTraversalState();

        public final String toString() {
            return String.format("%s[%s]", getClass().getName(), this.spliterator);
        }

        @Override // java9.util.Spliterator
        public Spliterator<P_OUT> trySplit() {
            if (this.isParallel && this.buffer == null && !this.finished) {
                init();
                Spliterator<P_IN> trySplit = this.spliterator.trySplit();
                if (trySplit == null) {
                    return null;
                }
                return wrap(trySplit);
            }
            return null;
        }

        public abstract AbstractWrappingSpliterator<P_IN, P_OUT, ?> wrap(Spliterator<P_IN> spliterator);
    }

    /* loaded from: classes19.dex */
    public static abstract class ArrayBuffer {
        public int index;

        /* loaded from: classes19.dex */
        public static final class OfDouble extends OfPrimitive<DoubleConsumer> implements DoubleConsumer {
            public final double[] array;

            public OfDouble(int i) {
                this.array = new double[i];
            }

            @Override // java9.util.function.DoubleConsumer
            public void accept(double d) {
                double[] dArr = this.array;
                int i = ((OfPrimitive) this).index;
                ((OfPrimitive) this).index = i + 1;
                dArr[i] = d;
            }

            @Override // java9.util.function.DoubleConsumer
            public /* synthetic */ DoubleConsumer andThen(DoubleConsumer doubleConsumer) {
                return Objects.requireNonNull(doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.ArrayBuffer.OfPrimitive
            public void forEach(DoubleConsumer doubleConsumer, long j) {
                for (int i = 0; i < j; i++) {
                    doubleConsumer.accept(this.array[i]);
                }
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfInt extends OfPrimitive<IntConsumer> implements IntConsumer {
            public final int[] array;

            public OfInt(int i) {
                this.array = new int[i];
            }

            @Override // java9.util.function.IntConsumer
            public void accept(int i) {
                int[] iArr = this.array;
                int i2 = ((OfPrimitive) this).index;
                ((OfPrimitive) this).index = i2 + 1;
                iArr[i2] = i;
            }

            @Override // java9.util.function.IntConsumer
            public /* synthetic */ IntConsumer andThen(IntConsumer intConsumer) {
                return Objects.requireNonNull(intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.ArrayBuffer.OfPrimitive
            public void forEach(IntConsumer intConsumer, long j) {
                for (int i = 0; i < j; i++) {
                    intConsumer.accept(this.array[i]);
                }
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfLong extends OfPrimitive<LongConsumer> implements LongConsumer {
            public final long[] array;

            public OfLong(int i) {
                this.array = new long[i];
            }

            @Override // java9.util.function.LongConsumer
            public void accept(long j) {
                long[] jArr = this.array;
                int i = ((OfPrimitive) this).index;
                ((OfPrimitive) this).index = i + 1;
                jArr[i] = j;
            }

            @Override // java9.util.function.LongConsumer
            public /* synthetic */ LongConsumer andThen(LongConsumer longConsumer) {
                return Objects.requireNonNull(longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.ArrayBuffer.OfPrimitive
            public void forEach(LongConsumer longConsumer, long j) {
                for (int i = 0; i < j; i++) {
                    longConsumer.accept(this.array[i]);
                }
            }
        }

        /* loaded from: classes19.dex */
        public static abstract class OfPrimitive<T_CONS> extends ArrayBuffer {
            public int index;

            public abstract void forEach(T_CONS t_cons, long j);

            @Override // java9.util.stream.StreamSpliterators.ArrayBuffer
            public void reset() {
                this.index = 0;
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfRef<T> extends ArrayBuffer implements Consumer<T> {
            public final Object[] array;

            public OfRef(int i) {
                this.array = new Object[i];
            }

            @Override // java9.util.function.Consumer
            public void accept(T t) {
                Object[] objArr = this.array;
                int i = this.index;
                this.index = i + 1;
                objArr[i] = t;
            }

            @Override // java9.util.function.Consumer
            public /* synthetic */ Consumer andThen(Consumer consumer) {
                return Objects.requireNonNull(consumer);
            }

            public void forEach(Consumer<? super T> consumer, long j) {
                for (int i = 0; i < j; i++) {
                    consumer.accept(this.array[i]);
                }
            }
        }

        public void reset() {
            this.index = 0;
        }
    }

    /* loaded from: classes19.dex */
    public static class DelegatingSpliterator<T, T_SPLITR extends Spliterator<T>> implements Spliterator<T> {
        public T_SPLITR s;
        public final Supplier<? extends T_SPLITR> supplier;

        /* loaded from: classes19.dex */
        public static final class OfDouble extends OfPrimitive<Double, DoubleConsumer, Spliterator.OfDouble> implements Spliterator.OfDouble {
            public OfDouble(Supplier<Spliterator.OfDouble> supplier) {
                super(supplier);
            }

            @Override // java9.util.Spliterator.OfDouble
            public /* bridge */ /* synthetic */ void forEachRemaining(DoubleConsumer doubleConsumer) {
                super.forEachRemaining((OfDouble) doubleConsumer);
            }

            @Override // java9.util.Spliterator.OfDouble
            public /* bridge */ /* synthetic */ boolean tryAdvance(DoubleConsumer doubleConsumer) {
                return super.tryAdvance((OfDouble) doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.DelegatingSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.DelegatingSpliterator, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfDouble trySplit() {
                return (Spliterator.OfDouble) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfInt extends OfPrimitive<Integer, IntConsumer, Spliterator.OfInt> implements Spliterator.OfInt {
            public OfInt(Supplier<Spliterator.OfInt> supplier) {
                super(supplier);
            }

            @Override // java9.util.Spliterator.OfInt
            public /* bridge */ /* synthetic */ void forEachRemaining(IntConsumer intConsumer) {
                super.forEachRemaining((OfInt) intConsumer);
            }

            @Override // java9.util.Spliterator.OfInt
            public /* bridge */ /* synthetic */ boolean tryAdvance(IntConsumer intConsumer) {
                return super.tryAdvance((OfInt) intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.DelegatingSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.DelegatingSpliterator, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfInt trySplit() {
                return (Spliterator.OfInt) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfLong extends OfPrimitive<Long, LongConsumer, Spliterator.OfLong> implements Spliterator.OfLong {
            public OfLong(Supplier<Spliterator.OfLong> supplier) {
                super(supplier);
            }

            @Override // java9.util.Spliterator.OfLong
            public /* bridge */ /* synthetic */ void forEachRemaining(LongConsumer longConsumer) {
                super.forEachRemaining((OfLong) longConsumer);
            }

            @Override // java9.util.Spliterator.OfLong
            public /* bridge */ /* synthetic */ boolean tryAdvance(LongConsumer longConsumer) {
                return super.tryAdvance((OfLong) longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.DelegatingSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.DelegatingSpliterator, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfLong trySplit() {
                return (Spliterator.OfLong) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static class OfPrimitive<T, T_CONS, T_SPLITR extends Spliterator.OfPrimitive<T, T_CONS, T_SPLITR>> extends DelegatingSpliterator<T, T_SPLITR> implements Spliterator.OfPrimitive<T, T_CONS, T_SPLITR> {
            public OfPrimitive(Supplier<? extends T_SPLITR> supplier) {
                super(supplier);
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public void forEachRemaining(T_CONS t_cons) {
                ((Spliterator.OfPrimitive) get()).forEachRemaining((Spliterator.OfPrimitive) t_cons);
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public boolean tryAdvance(T_CONS t_cons) {
                return ((Spliterator.OfPrimitive) get()).tryAdvance((Spliterator.OfPrimitive) t_cons);
            }

            @Override // java9.util.stream.StreamSpliterators.DelegatingSpliterator, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfPrimitive trySplit() {
                return (Spliterator.OfPrimitive) super.trySplit();
            }
        }

        public DelegatingSpliterator(Supplier<? extends T_SPLITR> supplier) {
            this.supplier = supplier;
        }

        @Override // java9.util.Spliterator
        public int characteristics() {
            return get().characteristics();
        }

        @Override // java9.util.Spliterator
        public long estimateSize() {
            return get().estimateSize();
        }

        @Override // java9.util.Spliterator
        public void forEachRemaining(Consumer<? super T> consumer) {
            get().forEachRemaining(consumer);
        }

        public T_SPLITR get() {
            if (this.s == null) {
                this.s = this.supplier.get();
            }
            return this.s;
        }

        @Override // java9.util.Spliterator
        public Comparator<? super T> getComparator() {
            return get().getComparator();
        }

        @Override // java9.util.Spliterator
        public long getExactSizeIfKnown() {
            return get().getExactSizeIfKnown();
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean hasCharacteristics(int i) {
            return Spliterator.CC.$default$hasCharacteristics(this, i);
        }

        public String toString() {
            return getClass().getName() + "[" + get() + "]";
        }

        @Override // java9.util.Spliterator
        public boolean tryAdvance(Consumer<? super T> consumer) {
            return get().tryAdvance(consumer);
        }

        @Override // java9.util.Spliterator
        public T_SPLITR trySplit() {
            return (T_SPLITR) get().trySplit();
        }
    }

    /* loaded from: classes19.dex */
    public static final class DistinctSpliterator<T> implements Spliterator<T>, Consumer<T> {
        public static final Object NULL_VALUE = new Object();
        public final Spliterator<T> s;
        public final ConcurrentMap<T, Boolean> seen;
        public T tmpSlot;

        public DistinctSpliterator(Spliterator<T> spliterator) {
            this(spliterator, new ConcurrentHashMap(512, 0.75f, ForkJoinPool.getCommonPoolParallelism() + 1));
        }

        public DistinctSpliterator(Spliterator<T> spliterator, ConcurrentMap<T, Boolean> concurrentMap) {
            this.s = spliterator;
            this.seen = concurrentMap;
        }

        /* JADX INFO: Access modifiers changed from: private */
        /* JADX WARN: Multi-variable type inference failed */
        public /* synthetic */ void b(Consumer consumer, Object obj) {
            if (this.seen.putIfAbsent(mapNull(obj), Boolean.TRUE) == null) {
                consumer.accept(obj);
            }
        }

        private T mapNull(T t) {
            return t != null ? t : (T) NULL_VALUE;
        }

        @Override // java9.util.function.Consumer
        public void accept(T t) {
            this.tmpSlot = t;
        }

        @Override // java9.util.function.Consumer
        public /* synthetic */ Consumer andThen(Consumer consumer) {
            return Objects.requireNonNull(consumer);
        }

        @Override // java9.util.Spliterator
        public int characteristics() {
            return (this.s.characteristics() & (-16469)) | 1;
        }

        @Override // java9.util.Spliterator
        public long estimateSize() {
            return this.s.estimateSize();
        }

        @Override // java9.util.Spliterator
        public void forEachRemaining(final Consumer<? super T> consumer) {
            this.s.forEachRemaining(new Consumer() { // from class: java9.util.stream.StreamSpliterators$DistinctSpliterator$$ExternalSyntheticLambda0
                @Override // java9.util.function.Consumer
                public final void accept(Object obj) {
                    StreamSpliterators.DistinctSpliterator.this.b(consumer, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer2) {
                    return Objects.requireNonNull(consumer2);
                }
            });
        }

        @Override // java9.util.Spliterator
        public Comparator<? super T> getComparator() {
            return this.s.getComparator();
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ long getExactSizeIfKnown() {
            return Spliterator.CC.$default$getExactSizeIfKnown(this);
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean hasCharacteristics(int i) {
            return Spliterator.CC.$default$hasCharacteristics(this, i);
        }

        @Override // java9.util.Spliterator
        public boolean tryAdvance(Consumer<? super T> consumer) {
            while (this.s.tryAdvance(this)) {
                if (this.seen.putIfAbsent(mapNull(this.tmpSlot), Boolean.TRUE) == null) {
                    consumer.accept((T) this.tmpSlot);
                    this.tmpSlot = null;
                    return true;
                }
            }
            return false;
        }

        @Override // java9.util.Spliterator
        public Spliterator<T> trySplit() {
            Spliterator<T> trySplit = this.s.trySplit();
            if (trySplit != null) {
                return new DistinctSpliterator(trySplit, this.seen);
            }
            return null;
        }
    }

    /* loaded from: classes19.dex */
    public static final class DoubleWrappingSpliterator<P_IN> extends AbstractWrappingSpliterator<P_IN, Double, SpinedBuffer.OfDouble> implements Spliterator.OfDouble {
        public DoubleWrappingSpliterator(PipelineHelper<Double> pipelineHelper, Spliterator<P_IN> spliterator, boolean z) {
            super(pipelineHelper, spliterator, z);
        }

        public DoubleWrappingSpliterator(PipelineHelper<Double> pipelineHelper, Supplier<Spliterator<P_IN>> supplier, boolean z) {
            super(pipelineHelper, supplier, z);
        }

        /* JADX INFO: Access modifiers changed from: private */
        public /* synthetic */ boolean a() {
            return this.spliterator.tryAdvance(this.bufferSink);
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public /* synthetic */ void forEachRemaining(Consumer consumer) {
            forEachRemaining(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
        }

        @Override // java9.util.Spliterator.OfPrimitive
        public void forEachRemaining(final DoubleConsumer doubleConsumer) {
            if (this.buffer != 0 || this.finished) {
                do {
                } while (tryAdvance(doubleConsumer));
                return;
            }
            Objects.requireNonNull(doubleConsumer);
            init();
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            doubleConsumer.getClass();
            pipelineHelper.wrapAndCopyInto(new Sink.OfDouble() { // from class: java9.util.stream.StreamSpliterators$DoubleWrappingSpliterator$$ExternalSyntheticLambda2
                @Override // java9.util.stream.Sink.OfDouble, java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public final void accept(double d) {
                    DoubleConsumer.this.accept(d);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(int i) {
                    Sink.CC.$default$accept((Sink) this, i);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(long j) {
                    Sink.CC.$default$accept((Sink) this, j);
                }

                @Override // java9.util.stream.Sink.OfDouble
                public /* synthetic */ void accept(Double d) {
                    Sink.OfDouble.CC.$default$accept((Sink.OfDouble) this, d);
                }

                @Override // java9.util.function.Consumer
                public /* bridge */ /* synthetic */ void accept(Object obj) {
                    Sink.OfDouble.CC.$default$accept(this, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.function.DoubleConsumer
                public /* synthetic */ DoubleConsumer andThen(DoubleConsumer doubleConsumer2) {
                    return Objects.requireNonNull(doubleConsumer2);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            }, this.spliterator);
            this.finished = true;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public void initPartialTraversalState() {
            final SpinedBuffer.OfDouble ofDouble = new SpinedBuffer.OfDouble();
            this.buffer = ofDouble;
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            ofDouble.getClass();
            this.bufferSink = pipelineHelper.wrapSink(new Sink.OfDouble() { // from class: java9.util.stream.StreamSpliterators$DoubleWrappingSpliterator$$ExternalSyntheticLambda0
                @Override // java9.util.stream.Sink.OfDouble, java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public final void accept(double d) {
                    SpinedBuffer.OfDouble.this.accept(d);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(int i) {
                    Sink.CC.$default$accept((Sink) this, i);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(long j) {
                    Sink.CC.$default$accept((Sink) this, j);
                }

                @Override // java9.util.stream.Sink.OfDouble
                public /* synthetic */ void accept(Double d) {
                    Sink.OfDouble.CC.$default$accept((Sink.OfDouble) this, d);
                }

                @Override // java9.util.function.Consumer
                public /* bridge */ /* synthetic */ void accept(Object obj) {
                    Sink.OfDouble.CC.$default$accept(this, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.function.DoubleConsumer
                public /* synthetic */ DoubleConsumer andThen(DoubleConsumer doubleConsumer) {
                    return Objects.requireNonNull(doubleConsumer);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            });
            this.pusher = new BooleanSupplier() { // from class: java9.util.stream.StreamSpliterators$DoubleWrappingSpliterator$$ExternalSyntheticLambda1
                @Override // java9.util.function.BooleanSupplier
                public final boolean getAsBoolean() {
                    boolean a2;
                    a2 = StreamSpliterators.DoubleWrappingSpliterator.this.a();
                    return a2;
                }
            };
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean tryAdvance(Consumer consumer) {
            boolean tryAdvance;
            tryAdvance = tryAdvance(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
            return tryAdvance;
        }

        @Override // java9.util.Spliterator.OfPrimitive
        public boolean tryAdvance(DoubleConsumer doubleConsumer) {
            Objects.requireNonNull(doubleConsumer);
            boolean doAdvance = doAdvance();
            if (doAdvance) {
                doubleConsumer.accept(((SpinedBuffer.OfDouble) this.buffer).get(this.nextToConsume));
            }
            return doAdvance;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public Spliterator.OfDouble trySplit() {
            return (Spliterator.OfDouble) super.trySplit();
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public AbstractWrappingSpliterator<P_IN, Double, ?> wrap(Spliterator<P_IN> spliterator) {
            return new DoubleWrappingSpliterator((PipelineHelper<Double>) this.ph, (Spliterator) spliterator, this.isParallel);
        }
    }

    /* loaded from: classes19.dex */
    public static abstract class InfiniteSupplyingSpliterator<T> implements Spliterator<T> {
        public long estimate;

        /* loaded from: classes19.dex */
        public static final class OfDouble extends InfiniteSupplyingSpliterator<Double> implements Spliterator.OfDouble {
            public final DoubleSupplier s;

            public OfDouble(long j, DoubleSupplier doubleSupplier) {
                super(j);
                this.s = doubleSupplier;
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public /* bridge */ /* synthetic */ void forEachRemaining(DoubleConsumer doubleConsumer) {
                forEachRemaining((DoubleConsumer) doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.InfiniteSupplyingSpliterator, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfDouble
            /* renamed from: forEachRemaining  reason: avoid collision after fix types in other method */
            public /* synthetic */ void forEachRemaining2(DoubleConsumer doubleConsumer) {
                Spliterator.OfDouble.CC.$default$forEachRemaining((Spliterator.OfDouble) this, doubleConsumer);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public boolean tryAdvance(DoubleConsumer doubleConsumer) {
                Objects.requireNonNull(doubleConsumer);
                doubleConsumer.accept(this.s.getAsDouble());
                return true;
            }

            @Override // java9.util.Spliterator
            public Spliterator.OfDouble trySplit() {
                long j = this.estimate;
                if (j == 0) {
                    return null;
                }
                long j2 = j >>> 1;
                this.estimate = j2;
                return new OfDouble(j2, this.s);
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfInt extends InfiniteSupplyingSpliterator<Integer> implements Spliterator.OfInt {
            public final IntSupplier s;

            public OfInt(long j, IntSupplier intSupplier) {
                super(j);
                this.s = intSupplier;
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public /* bridge */ /* synthetic */ void forEachRemaining(IntConsumer intConsumer) {
                forEachRemaining((IntConsumer) intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.InfiniteSupplyingSpliterator, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfInt
            /* renamed from: forEachRemaining  reason: avoid collision after fix types in other method */
            public /* synthetic */ void forEachRemaining2(IntConsumer intConsumer) {
                Spliterator.OfInt.CC.$default$forEachRemaining((Spliterator.OfInt) this, intConsumer);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public boolean tryAdvance(IntConsumer intConsumer) {
                Objects.requireNonNull(intConsumer);
                intConsumer.accept(this.s.getAsInt());
                return true;
            }

            @Override // java9.util.Spliterator
            public Spliterator.OfInt trySplit() {
                long j = this.estimate;
                if (j == 0) {
                    return null;
                }
                long j2 = j >>> 1;
                this.estimate = j2;
                return new OfInt(j2, this.s);
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfLong extends InfiniteSupplyingSpliterator<Long> implements Spliterator.OfLong {
            public final LongSupplier s;

            public OfLong(long j, LongSupplier longSupplier) {
                super(j);
                this.s = longSupplier;
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public /* bridge */ /* synthetic */ void forEachRemaining(LongConsumer longConsumer) {
                forEachRemaining((LongConsumer) longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.InfiniteSupplyingSpliterator, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfLong
            /* renamed from: forEachRemaining  reason: avoid collision after fix types in other method */
            public /* synthetic */ void forEachRemaining2(LongConsumer longConsumer) {
                Spliterator.OfLong.CC.$default$forEachRemaining((Spliterator.OfLong) this, longConsumer);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public boolean tryAdvance(LongConsumer longConsumer) {
                Objects.requireNonNull(longConsumer);
                longConsumer.accept(this.s.getAsLong());
                return true;
            }

            @Override // java9.util.Spliterator
            public Spliterator.OfLong trySplit() {
                long j = this.estimate;
                if (j == 0) {
                    return null;
                }
                long j2 = j >>> 1;
                this.estimate = j2;
                return new OfLong(j2, this.s);
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfRef<T> extends InfiniteSupplyingSpliterator<T> {
            public final Supplier<? extends T> s;

            public OfRef(long j, Supplier<? extends T> supplier) {
                super(j);
                this.s = supplier;
            }

            @Override // java9.util.Spliterator
            public boolean tryAdvance(Consumer<? super T> consumer) {
                Objects.requireNonNull(consumer);
                consumer.accept((T) this.s.get());
                return true;
            }

            @Override // java9.util.Spliterator
            public Spliterator<T> trySplit() {
                long j = this.estimate;
                if (j == 0) {
                    return null;
                }
                long j2 = j >>> 1;
                this.estimate = j2;
                return new OfRef(j2, this.s);
            }
        }

        public InfiniteSupplyingSpliterator(long j) {
            this.estimate = j;
        }

        @Override // java9.util.Spliterator
        public int characteristics() {
            return 1024;
        }

        @Override // java9.util.Spliterator
        public long estimateSize() {
            return this.estimate;
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ void forEachRemaining(Consumer consumer) {
            Spliterator.CC.$default$forEachRemaining(this, consumer);
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ Comparator getComparator() {
            return Spliterator.CC.$default$getComparator(this);
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ long getExactSizeIfKnown() {
            return Spliterator.CC.$default$getExactSizeIfKnown(this);
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean hasCharacteristics(int i) {
            return Spliterator.CC.$default$hasCharacteristics(this, i);
        }
    }

    /* loaded from: classes19.dex */
    public static final class IntWrappingSpliterator<P_IN> extends AbstractWrappingSpliterator<P_IN, Integer, SpinedBuffer.OfInt> implements Spliterator.OfInt {
        public IntWrappingSpliterator(PipelineHelper<Integer> pipelineHelper, Spliterator<P_IN> spliterator, boolean z) {
            super(pipelineHelper, spliterator, z);
        }

        public IntWrappingSpliterator(PipelineHelper<Integer> pipelineHelper, Supplier<Spliterator<P_IN>> supplier, boolean z) {
            super(pipelineHelper, supplier, z);
        }

        /* JADX INFO: Access modifiers changed from: private */
        public /* synthetic */ boolean a() {
            return this.spliterator.tryAdvance(this.bufferSink);
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public /* synthetic */ void forEachRemaining(Consumer consumer) {
            forEachRemaining(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
        }

        @Override // java9.util.Spliterator.OfPrimitive
        public void forEachRemaining(final IntConsumer intConsumer) {
            if (this.buffer != 0 || this.finished) {
                do {
                } while (tryAdvance(intConsumer));
                return;
            }
            Objects.requireNonNull(intConsumer);
            init();
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            intConsumer.getClass();
            pipelineHelper.wrapAndCopyInto(new Sink.OfInt() { // from class: java9.util.stream.StreamSpliterators$IntWrappingSpliterator$$ExternalSyntheticLambda0
                @Override // java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public /* synthetic */ void accept(double d) {
                    Sink.CC.$default$accept(this, d);
                }

                @Override // java9.util.stream.Sink.OfInt, java9.util.stream.Sink
                public final void accept(int i) {
                    IntConsumer.this.accept(i);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(long j) {
                    Sink.CC.$default$accept((Sink) this, j);
                }

                @Override // java9.util.stream.Sink.OfInt
                public /* synthetic */ void accept(Integer num) {
                    Sink.OfInt.CC.$default$accept((Sink.OfInt) this, num);
                }

                @Override // java9.util.function.Consumer
                public /* bridge */ /* synthetic */ void accept(Object obj) {
                    Sink.OfInt.CC.$default$accept(this, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.function.IntConsumer
                public /* synthetic */ IntConsumer andThen(IntConsumer intConsumer2) {
                    return Objects.requireNonNull(intConsumer2);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            }, this.spliterator);
            this.finished = true;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public void initPartialTraversalState() {
            final SpinedBuffer.OfInt ofInt = new SpinedBuffer.OfInt();
            this.buffer = ofInt;
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            ofInt.getClass();
            this.bufferSink = pipelineHelper.wrapSink(new Sink.OfInt() { // from class: java9.util.stream.StreamSpliterators$IntWrappingSpliterator$$ExternalSyntheticLambda1
                @Override // java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public /* synthetic */ void accept(double d) {
                    Sink.CC.$default$accept(this, d);
                }

                @Override // java9.util.stream.Sink.OfInt, java9.util.stream.Sink
                public final void accept(int i) {
                    SpinedBuffer.OfInt.this.accept(i);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(long j) {
                    Sink.CC.$default$accept((Sink) this, j);
                }

                @Override // java9.util.stream.Sink.OfInt
                public /* synthetic */ void accept(Integer num) {
                    Sink.OfInt.CC.$default$accept((Sink.OfInt) this, num);
                }

                @Override // java9.util.function.Consumer
                public /* bridge */ /* synthetic */ void accept(Object obj) {
                    Sink.OfInt.CC.$default$accept(this, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.function.IntConsumer
                public /* synthetic */ IntConsumer andThen(IntConsumer intConsumer) {
                    return Objects.requireNonNull(intConsumer);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            });
            this.pusher = new BooleanSupplier() { // from class: java9.util.stream.StreamSpliterators$IntWrappingSpliterator$$ExternalSyntheticLambda2
                @Override // java9.util.function.BooleanSupplier
                public final boolean getAsBoolean() {
                    boolean a2;
                    a2 = StreamSpliterators.IntWrappingSpliterator.this.a();
                    return a2;
                }
            };
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean tryAdvance(Consumer consumer) {
            boolean tryAdvance;
            tryAdvance = tryAdvance(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
            return tryAdvance;
        }

        @Override // java9.util.Spliterator.OfPrimitive
        public boolean tryAdvance(IntConsumer intConsumer) {
            Objects.requireNonNull(intConsumer);
            boolean doAdvance = doAdvance();
            if (doAdvance) {
                intConsumer.accept(((SpinedBuffer.OfInt) this.buffer).get(this.nextToConsume));
            }
            return doAdvance;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public Spliterator.OfInt trySplit() {
            return (Spliterator.OfInt) super.trySplit();
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public AbstractWrappingSpliterator<P_IN, Integer, ?> wrap(Spliterator<P_IN> spliterator) {
            return new IntWrappingSpliterator((PipelineHelper<Integer>) this.ph, (Spliterator) spliterator, this.isParallel);
        }
    }

    /* loaded from: classes19.dex */
    public static final class LongWrappingSpliterator<P_IN> extends AbstractWrappingSpliterator<P_IN, Long, SpinedBuffer.OfLong> implements Spliterator.OfLong {
        public LongWrappingSpliterator(PipelineHelper<Long> pipelineHelper, Spliterator<P_IN> spliterator, boolean z) {
            super(pipelineHelper, spliterator, z);
        }

        public LongWrappingSpliterator(PipelineHelper<Long> pipelineHelper, Supplier<Spliterator<P_IN>> supplier, boolean z) {
            super(pipelineHelper, supplier, z);
        }

        /* JADX INFO: Access modifiers changed from: private */
        public /* synthetic */ boolean a() {
            return this.spliterator.tryAdvance(this.bufferSink);
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public /* synthetic */ void forEachRemaining(Consumer consumer) {
            forEachRemaining(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
        }

        @Override // java9.util.Spliterator.OfPrimitive
        public void forEachRemaining(final LongConsumer longConsumer) {
            if (this.buffer != 0 || this.finished) {
                do {
                } while (tryAdvance(longConsumer));
                return;
            }
            Objects.requireNonNull(longConsumer);
            init();
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            longConsumer.getClass();
            pipelineHelper.wrapAndCopyInto(new Sink.OfLong() { // from class: java9.util.stream.StreamSpliterators$LongWrappingSpliterator$$ExternalSyntheticLambda2
                @Override // java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public /* synthetic */ void accept(double d) {
                    Sink.CC.$default$accept(this, d);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(int i) {
                    Sink.CC.$default$accept((Sink) this, i);
                }

                @Override // java9.util.stream.Sink.OfLong, java9.util.stream.Sink
                public final void accept(long j) {
                    LongConsumer.this.accept(j);
                }

                @Override // java9.util.stream.Sink.OfLong
                public /* synthetic */ void accept(Long l) {
                    Sink.OfLong.CC.$default$accept((Sink.OfLong) this, l);
                }

                @Override // java9.util.function.Consumer
                public /* bridge */ /* synthetic */ void accept(Object obj) {
                    Sink.OfLong.CC.$default$accept(this, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.function.LongConsumer
                public /* synthetic */ LongConsumer andThen(LongConsumer longConsumer2) {
                    return Objects.requireNonNull(longConsumer2);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            }, this.spliterator);
            this.finished = true;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public void initPartialTraversalState() {
            final SpinedBuffer.OfLong ofLong = new SpinedBuffer.OfLong();
            this.buffer = ofLong;
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            ofLong.getClass();
            this.bufferSink = pipelineHelper.wrapSink(new Sink.OfLong() { // from class: java9.util.stream.StreamSpliterators$LongWrappingSpliterator$$ExternalSyntheticLambda0
                @Override // java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public /* synthetic */ void accept(double d) {
                    Sink.CC.$default$accept(this, d);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(int i) {
                    Sink.CC.$default$accept((Sink) this, i);
                }

                @Override // java9.util.stream.Sink.OfLong, java9.util.stream.Sink
                public final void accept(long j) {
                    SpinedBuffer.OfLong.this.accept(j);
                }

                @Override // java9.util.stream.Sink.OfLong
                public /* synthetic */ void accept(Long l) {
                    Sink.OfLong.CC.$default$accept((Sink.OfLong) this, l);
                }

                @Override // java9.util.function.Consumer
                public /* bridge */ /* synthetic */ void accept(Object obj) {
                    Sink.OfLong.CC.$default$accept(this, obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.function.LongConsumer
                public /* synthetic */ LongConsumer andThen(LongConsumer longConsumer) {
                    return Objects.requireNonNull(longConsumer);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            });
            this.pusher = new BooleanSupplier() { // from class: java9.util.stream.StreamSpliterators$LongWrappingSpliterator$$ExternalSyntheticLambda1
                @Override // java9.util.function.BooleanSupplier
                public final boolean getAsBoolean() {
                    boolean a2;
                    a2 = StreamSpliterators.LongWrappingSpliterator.this.a();
                    return a2;
                }
            };
        }

        @Override // java9.util.Spliterator
        public /* synthetic */ boolean tryAdvance(Consumer consumer) {
            boolean tryAdvance;
            tryAdvance = tryAdvance(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
            return tryAdvance;
        }

        @Override // java9.util.Spliterator.OfPrimitive
        public boolean tryAdvance(LongConsumer longConsumer) {
            Objects.requireNonNull(longConsumer);
            boolean doAdvance = doAdvance();
            if (doAdvance) {
                longConsumer.accept(((SpinedBuffer.OfLong) this.buffer).get(this.nextToConsume));
            }
            return doAdvance;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public Spliterator.OfLong trySplit() {
            return (Spliterator.OfLong) super.trySplit();
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public AbstractWrappingSpliterator<P_IN, Long, ?> wrap(Spliterator<P_IN> spliterator) {
            return new LongWrappingSpliterator((PipelineHelper<Long>) this.ph, (Spliterator) spliterator, this.isParallel);
        }
    }

    /* loaded from: classes19.dex */
    public static abstract class SliceSpliterator<T, T_SPLITR extends Spliterator<T>> {
        public long fence;
        public long index;
        public T_SPLITR s;
        public final long sliceFence;
        public final long sliceOrigin;

        /* loaded from: classes19.dex */
        public static final class OfDouble extends OfPrimitive<Double, Spliterator.OfDouble, DoubleConsumer> implements Spliterator.OfDouble {
            public OfDouble(Spliterator.OfDouble ofDouble, long j, long j2) {
                super(ofDouble, j, j2);
            }

            public OfDouble(Spliterator.OfDouble ofDouble, long j, long j2, long j3, long j4) {
                super(ofDouble, j, j2, j3, j4);
            }

            public static /* synthetic */ void a(double d) {
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive
            public DoubleConsumer emptyConsumer() {
                return new DoubleConsumer() { // from class: java9.util.stream.StreamSpliterators$SliceSpliterator$OfDouble$$ExternalSyntheticLambda0
                    @Override // java9.util.function.DoubleConsumer
                    public final void accept(double d) {
                        StreamSpliterators.SliceSpliterator.OfDouble.a(d);
                    }

                    @Override // java9.util.function.DoubleConsumer
                    public /* synthetic */ DoubleConsumer andThen(DoubleConsumer doubleConsumer) {
                        return Objects.requireNonNull(doubleConsumer);
                    }
                };
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfDouble
            public /* bridge */ /* synthetic */ void forEachRemaining(DoubleConsumer doubleConsumer) {
                super.forEachRemaining((OfDouble) doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator
            public Spliterator.OfDouble makeSpliterator(Spliterator.OfDouble ofDouble, long j, long j2, long j3, long j4) {
                return new OfDouble(ofDouble, j, j2, j3, j4);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfDouble
            public /* bridge */ /* synthetic */ boolean tryAdvance(DoubleConsumer doubleConsumer) {
                return super.tryAdvance((OfDouble) doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.SliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfDouble trySplit() {
                return (Spliterator.OfDouble) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfInt extends OfPrimitive<Integer, Spliterator.OfInt, IntConsumer> implements Spliterator.OfInt {
            public OfInt(Spliterator.OfInt ofInt, long j, long j2) {
                super(ofInt, j, j2);
            }

            public OfInt(Spliterator.OfInt ofInt, long j, long j2, long j3, long j4) {
                super(ofInt, j, j2, j3, j4);
            }

            public static /* synthetic */ void a(int i) {
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive
            public IntConsumer emptyConsumer() {
                return new IntConsumer() { // from class: java9.util.stream.StreamSpliterators$SliceSpliterator$OfInt$$ExternalSyntheticLambda0
                    @Override // java9.util.function.IntConsumer
                    public final void accept(int i) {
                        StreamSpliterators.SliceSpliterator.OfInt.a(i);
                    }

                    @Override // java9.util.function.IntConsumer
                    public /* synthetic */ IntConsumer andThen(IntConsumer intConsumer) {
                        return Objects.requireNonNull(intConsumer);
                    }
                };
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfInt
            public /* bridge */ /* synthetic */ void forEachRemaining(IntConsumer intConsumer) {
                super.forEachRemaining((OfInt) intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator
            public Spliterator.OfInt makeSpliterator(Spliterator.OfInt ofInt, long j, long j2, long j3, long j4) {
                return new OfInt(ofInt, j, j2, j3, j4);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfInt
            public /* bridge */ /* synthetic */ boolean tryAdvance(IntConsumer intConsumer) {
                return super.tryAdvance((OfInt) intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.SliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfInt trySplit() {
                return (Spliterator.OfInt) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfLong extends OfPrimitive<Long, Spliterator.OfLong, LongConsumer> implements Spliterator.OfLong {
            public OfLong(Spliterator.OfLong ofLong, long j, long j2) {
                super(ofLong, j, j2);
            }

            public OfLong(Spliterator.OfLong ofLong, long j, long j2, long j3, long j4) {
                super(ofLong, j, j2, j3, j4);
            }

            public static /* synthetic */ void a(long j) {
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive
            public LongConsumer emptyConsumer() {
                return new LongConsumer() { // from class: java9.util.stream.StreamSpliterators$SliceSpliterator$OfLong$$ExternalSyntheticLambda0
                    @Override // java9.util.function.LongConsumer
                    public final void accept(long j) {
                        StreamSpliterators.SliceSpliterator.OfLong.a(j);
                    }

                    @Override // java9.util.function.LongConsumer
                    public /* synthetic */ LongConsumer andThen(LongConsumer longConsumer) {
                        return Objects.requireNonNull(longConsumer);
                    }
                };
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfLong
            public /* bridge */ /* synthetic */ void forEachRemaining(LongConsumer longConsumer) {
                super.forEachRemaining((OfLong) longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator
            public Spliterator.OfLong makeSpliterator(Spliterator.OfLong ofLong, long j, long j2, long j3, long j4) {
                return new OfLong(ofLong, j, j2, j3, j4);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfLong
            public /* bridge */ /* synthetic */ boolean tryAdvance(LongConsumer longConsumer) {
                return super.tryAdvance((OfLong) longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.SliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfLong trySplit() {
                return (Spliterator.OfLong) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static abstract class OfPrimitive<T, T_SPLITR extends Spliterator.OfPrimitive<T, T_CONS, T_SPLITR>, T_CONS> extends SliceSpliterator<T, T_SPLITR> implements Spliterator.OfPrimitive<T, T_CONS, T_SPLITR> {
            public OfPrimitive(T_SPLITR t_splitr, long j, long j2) {
                this(t_splitr, j, j2, 0L, Math.min(t_splitr.estimateSize(), j2));
            }

            public OfPrimitive(T_SPLITR t_splitr, long j, long j2, long j3, long j4) {
                super(t_splitr, j, j2, j3, j4);
            }

            public abstract T_CONS emptyConsumer();

            @Override // java9.util.Spliterator.OfPrimitive
            public void forEachRemaining(T_CONS t_cons) {
                Objects.requireNonNull(t_cons);
                long j = this.sliceOrigin;
                long j2 = this.fence;
                if (j >= j2) {
                    return;
                }
                long j3 = this.index;
                if (j3 >= j2) {
                    return;
                }
                if (j3 >= j && j3 + ((Spliterator.OfPrimitive) this.s).estimateSize() <= this.sliceFence) {
                    ((Spliterator.OfPrimitive) this.s).forEachRemaining((Spliterator.OfPrimitive) t_cons);
                    this.index = this.fence;
                    return;
                }
                while (this.sliceOrigin > this.index) {
                    ((Spliterator.OfPrimitive) this.s).tryAdvance((Spliterator.OfPrimitive) emptyConsumer());
                    this.index++;
                }
                while (this.index < this.fence) {
                    ((Spliterator.OfPrimitive) this.s).tryAdvance((Spliterator.OfPrimitive) t_cons);
                    this.index++;
                }
            }

            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                Spliterator.CC.$default$forEachRemaining(this, consumer);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ Comparator getComparator() {
                return Spliterator.CC.$default$getComparator(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ long getExactSizeIfKnown() {
                return Spliterator.CC.$default$getExactSizeIfKnown(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean hasCharacteristics(int i) {
                return Spliterator.CC.$default$hasCharacteristics(this, i);
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public boolean tryAdvance(T_CONS t_cons) {
                long j;
                Objects.requireNonNull(t_cons);
                if (this.sliceOrigin >= this.fence) {
                    return false;
                }
                while (true) {
                    long j2 = this.sliceOrigin;
                    j = this.index;
                    if (j2 <= j) {
                        break;
                    }
                    ((Spliterator.OfPrimitive) this.s).tryAdvance((Spliterator.OfPrimitive) emptyConsumer());
                    this.index++;
                }
                if (j >= this.fence) {
                    return false;
                }
                this.index = j + 1;
                return ((Spliterator.OfPrimitive) this.s).tryAdvance((Spliterator.OfPrimitive) t_cons);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfPrimitive trySplit() {
                return (Spliterator.OfPrimitive) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfRef<T> extends SliceSpliterator<T, Spliterator<T>> implements Spliterator<T> {
            public OfRef(Spliterator<T> spliterator, long j, long j2) {
                this(spliterator, j, j2, 0L, Math.min(spliterator.estimateSize(), j2));
            }

            public OfRef(Spliterator<T> spliterator, long j, long j2, long j3, long j4) {
                super(spliterator, j, j2, j3, j4);
            }

            public static /* synthetic */ void a(Object obj) {
            }

            public static /* synthetic */ void b(Object obj) {
            }

            @Override // java9.util.Spliterator
            public void forEachRemaining(Consumer<? super T> consumer) {
                Objects.requireNonNull(consumer);
                long j = this.sliceOrigin;
                long j2 = this.fence;
                if (j >= j2) {
                    return;
                }
                long j3 = this.index;
                if (j3 >= j2) {
                    return;
                }
                if (j3 >= j && j3 + this.s.estimateSize() <= this.sliceFence) {
                    this.s.forEachRemaining(consumer);
                    this.index = this.fence;
                    return;
                }
                while (this.sliceOrigin > this.index) {
                    this.s.tryAdvance(new Consumer() { // from class: java9.util.stream.StreamSpliterators$SliceSpliterator$OfRef$$ExternalSyntheticLambda1
                        @Override // java9.util.function.Consumer
                        public final void accept(Object obj) {
                            StreamSpliterators.SliceSpliterator.OfRef.a(obj);
                        }

                        @Override // java9.util.function.Consumer
                        public /* synthetic */ Consumer andThen(Consumer consumer2) {
                            return Objects.requireNonNull(consumer2);
                        }
                    });
                    this.index++;
                }
                while (this.index < this.fence) {
                    this.s.tryAdvance(consumer);
                    this.index++;
                }
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ Comparator getComparator() {
                return Spliterator.CC.$default$getComparator(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ long getExactSizeIfKnown() {
                return Spliterator.CC.$default$getExactSizeIfKnown(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean hasCharacteristics(int i) {
                return Spliterator.CC.$default$hasCharacteristics(this, i);
            }

            @Override // java9.util.stream.StreamSpliterators.SliceSpliterator
            public Spliterator<T> makeSpliterator(Spliterator<T> spliterator, long j, long j2, long j3, long j4) {
                return new OfRef(spliterator, j, j2, j3, j4);
            }

            @Override // java9.util.Spliterator
            public boolean tryAdvance(Consumer<? super T> consumer) {
                long j;
                Objects.requireNonNull(consumer);
                if (this.sliceOrigin >= this.fence) {
                    return false;
                }
                while (true) {
                    long j2 = this.sliceOrigin;
                    j = this.index;
                    if (j2 <= j) {
                        break;
                    }
                    this.s.tryAdvance(new Consumer() { // from class: java9.util.stream.StreamSpliterators$SliceSpliterator$OfRef$$ExternalSyntheticLambda0
                        @Override // java9.util.function.Consumer
                        public final void accept(Object obj) {
                            StreamSpliterators.SliceSpliterator.OfRef.b(obj);
                        }

                        @Override // java9.util.function.Consumer
                        public /* synthetic */ Consumer andThen(Consumer consumer2) {
                            return Objects.requireNonNull(consumer2);
                        }
                    });
                    this.index++;
                }
                if (j >= this.fence) {
                    return false;
                }
                this.index = j + 1;
                return this.s.tryAdvance(consumer);
            }
        }

        public SliceSpliterator(T_SPLITR t_splitr, long j, long j2, long j3, long j4) {
            this.s = t_splitr;
            this.sliceOrigin = j;
            this.sliceFence = j2;
            this.index = j3;
            this.fence = j4;
        }

        public int characteristics() {
            return this.s.characteristics();
        }

        public long estimateSize() {
            long j = this.sliceOrigin;
            long j2 = this.fence;
            if (j < j2) {
                return j2 - Math.max(j, this.index);
            }
            return 0L;
        }

        public abstract T_SPLITR makeSpliterator(T_SPLITR t_splitr, long j, long j2, long j3, long j4);

        public T_SPLITR trySplit() {
            long j = this.sliceOrigin;
            long j2 = this.fence;
            if (j >= j2 || this.index >= j2) {
                return null;
            }
            while (true) {
                T_SPLITR t_splitr = (T_SPLITR) this.s.trySplit();
                if (t_splitr == null) {
                    return null;
                }
                long estimateSize = this.index + t_splitr.estimateSize();
                long min = Math.min(estimateSize, this.sliceFence);
                long j3 = this.sliceOrigin;
                if (j3 >= min) {
                    this.index = min;
                } else {
                    long j4 = this.sliceFence;
                    if (min < j4) {
                        long j5 = this.index;
                        if (j5 < j3 || estimateSize > j4) {
                            this.index = min;
                            return makeSpliterator(t_splitr, j3, j4, j5, min);
                        }
                        this.index = min;
                        return t_splitr;
                    }
                    this.s = t_splitr;
                    this.fence = min;
                }
            }
        }
    }

    /* loaded from: classes19.dex */
    public static abstract class UnorderedSliceSpliterator<T, T_SPLITR extends Spliterator<T>> {
        public static final int CHUNK_SIZE = 128;
        public final int chunkSize;
        public final AtomicLong permits;
        public final T_SPLITR s;
        public final long skipThreshold;
        public final boolean unlimited;

        /* loaded from: classes19.dex */
        public static final class OfDouble extends OfPrimitive<Double, DoubleConsumer, ArrayBuffer.OfDouble, Spliterator.OfDouble> implements Spliterator.OfDouble, DoubleConsumer {
            public double tmpValue;

            public OfDouble(Spliterator.OfDouble ofDouble, long j, long j2) {
                super(ofDouble, j, j2);
            }

            public OfDouble(Spliterator.OfDouble ofDouble, OfDouble ofDouble2) {
                super(ofDouble, ofDouble2);
            }

            @Override // java9.util.function.DoubleConsumer
            public void accept(double d) {
                this.tmpValue = d;
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive
            public void acceptConsumed(DoubleConsumer doubleConsumer) {
                doubleConsumer.accept(this.tmpValue);
            }

            @Override // java9.util.function.DoubleConsumer
            public /* synthetic */ DoubleConsumer andThen(DoubleConsumer doubleConsumer) {
                return Objects.requireNonNull(doubleConsumer);
            }

            /* JADX WARN: Can't rename method to resolve collision */
            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive
            public ArrayBuffer.OfDouble bufferCreate(int i) {
                return new ArrayBuffer.OfDouble(i);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfDouble
            public /* bridge */ /* synthetic */ void forEachRemaining(DoubleConsumer doubleConsumer) {
                super.forEachRemaining((OfDouble) doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator
            public Spliterator.OfDouble makeSpliterator(Spliterator.OfDouble ofDouble) {
                return new OfDouble(ofDouble, this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof DoubleConsumer ? (DoubleConsumer) consumer : RefConsumer.toDoubleConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfDouble
            public /* bridge */ /* synthetic */ boolean tryAdvance(DoubleConsumer doubleConsumer) {
                return super.tryAdvance((OfDouble) doubleConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfDouble trySplit() {
                return (Spliterator.OfDouble) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfInt extends OfPrimitive<Integer, IntConsumer, ArrayBuffer.OfInt, Spliterator.OfInt> implements Spliterator.OfInt, IntConsumer {
            public int tmpValue;

            public OfInt(Spliterator.OfInt ofInt, long j, long j2) {
                super(ofInt, j, j2);
            }

            public OfInt(Spliterator.OfInt ofInt, OfInt ofInt2) {
                super(ofInt, ofInt2);
            }

            @Override // java9.util.function.IntConsumer
            public void accept(int i) {
                this.tmpValue = i;
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive
            public void acceptConsumed(IntConsumer intConsumer) {
                intConsumer.accept(this.tmpValue);
            }

            @Override // java9.util.function.IntConsumer
            public /* synthetic */ IntConsumer andThen(IntConsumer intConsumer) {
                return Objects.requireNonNull(intConsumer);
            }

            /* JADX WARN: Can't rename method to resolve collision */
            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive
            public ArrayBuffer.OfInt bufferCreate(int i) {
                return new ArrayBuffer.OfInt(i);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfInt
            public /* bridge */ /* synthetic */ void forEachRemaining(IntConsumer intConsumer) {
                super.forEachRemaining((OfInt) intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator
            public Spliterator.OfInt makeSpliterator(Spliterator.OfInt ofInt) {
                return new OfInt(ofInt, this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof IntConsumer ? (IntConsumer) consumer : RefConsumer.toIntConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfInt
            public /* bridge */ /* synthetic */ boolean tryAdvance(IntConsumer intConsumer) {
                return super.tryAdvance((OfInt) intConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfInt trySplit() {
                return (Spliterator.OfInt) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfLong extends OfPrimitive<Long, LongConsumer, ArrayBuffer.OfLong, Spliterator.OfLong> implements Spliterator.OfLong, LongConsumer {
            public long tmpValue;

            public OfLong(Spliterator.OfLong ofLong, long j, long j2) {
                super(ofLong, j, j2);
            }

            public OfLong(Spliterator.OfLong ofLong, OfLong ofLong2) {
                super(ofLong, ofLong2);
            }

            @Override // java9.util.function.LongConsumer
            public void accept(long j) {
                this.tmpValue = j;
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive
            public void acceptConsumed(LongConsumer longConsumer) {
                longConsumer.accept(this.tmpValue);
            }

            @Override // java9.util.function.LongConsumer
            public /* synthetic */ LongConsumer andThen(LongConsumer longConsumer) {
                return Objects.requireNonNull(longConsumer);
            }

            /* JADX WARN: Can't rename method to resolve collision */
            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive
            public ArrayBuffer.OfLong bufferCreate(int i) {
                return new ArrayBuffer.OfLong(i);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive, java9.util.Spliterator
            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                forEachRemaining(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
            }

            @Override // java9.util.Spliterator.OfLong
            public /* bridge */ /* synthetic */ void forEachRemaining(LongConsumer longConsumer) {
                super.forEachRemaining((OfLong) longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator
            public Spliterator.OfLong makeSpliterator(Spliterator.OfLong ofLong) {
                return new OfLong(ofLong, this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean tryAdvance(Consumer consumer) {
                boolean tryAdvance;
                tryAdvance = tryAdvance(r2 instanceof LongConsumer ? (LongConsumer) consumer : RefConsumer.toLongConsumer(consumer));
                return tryAdvance;
            }

            @Override // java9.util.Spliterator.OfLong
            public /* bridge */ /* synthetic */ boolean tryAdvance(LongConsumer longConsumer) {
                return super.tryAdvance((OfLong) longConsumer);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator.OfPrimitive, java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfLong trySplit() {
                return (Spliterator.OfLong) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static abstract class OfPrimitive<T, T_CONS, T_BUFF extends ArrayBuffer.OfPrimitive<T_CONS>, T_SPLITR extends Spliterator.OfPrimitive<T, T_CONS, T_SPLITR>> extends UnorderedSliceSpliterator<T, T_SPLITR> implements Spliterator.OfPrimitive<T, T_CONS, T_SPLITR> {
            public OfPrimitive(T_SPLITR t_splitr, long j, long j2) {
                super(t_splitr, j, j2);
            }

            public OfPrimitive(T_SPLITR t_splitr, OfPrimitive<T, T_CONS, T_BUFF, T_SPLITR> ofPrimitive) {
                super(t_splitr, ofPrimitive);
            }

            public abstract void acceptConsumed(T_CONS t_cons);

            public abstract T_BUFF bufferCreate(int i);

            @Override // java9.util.Spliterator.OfPrimitive
            public void forEachRemaining(T_CONS t_cons) {
                Objects.requireNonNull(t_cons);
                T_BUFF t_buff = null;
                while (true) {
                    PermitStatus permitStatus = permitStatus();
                    if (permitStatus == PermitStatus.NO_MORE) {
                        return;
                    }
                    if (permitStatus != PermitStatus.MAYBE_MORE) {
                        ((Spliterator.OfPrimitive) this.s).forEachRemaining((Spliterator.OfPrimitive) t_cons);
                        return;
                    }
                    if (t_buff == null) {
                        t_buff = bufferCreate(this.chunkSize);
                    } else {
                        t_buff.reset();
                    }
                    long j = 0;
                    while (((Spliterator.OfPrimitive) this.s).tryAdvance((Spliterator.OfPrimitive) t_buff)) {
                        j++;
                        if (j >= this.chunkSize) {
                            break;
                        }
                    }
                    if (j == 0) {
                        return;
                    }
                    t_buff.forEach(t_cons, acquirePermits(j));
                }
            }

            public /* synthetic */ void forEachRemaining(Consumer consumer) {
                Spliterator.CC.$default$forEachRemaining(this, consumer);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ Comparator getComparator() {
                return Spliterator.CC.$default$getComparator(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ long getExactSizeIfKnown() {
                return Spliterator.CC.$default$getExactSizeIfKnown(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean hasCharacteristics(int i) {
                return Spliterator.CC.$default$hasCharacteristics(this, i);
            }

            @Override // java9.util.Spliterator.OfPrimitive
            public boolean tryAdvance(T_CONS t_cons) {
                Objects.requireNonNull(t_cons);
                while (permitStatus() != PermitStatus.NO_MORE && ((Spliterator.OfPrimitive) this.s).tryAdvance(this)) {
                    if (acquirePermits(1L) == 1) {
                        acceptConsumed(t_cons);
                        return true;
                    }
                }
                return false;
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator, java9.util.Spliterator.OfPrimitive, java9.util.Spliterator
            public /* bridge */ /* synthetic */ Spliterator.OfPrimitive trySplit() {
                return (Spliterator.OfPrimitive) super.trySplit();
            }
        }

        /* loaded from: classes19.dex */
        public static final class OfRef<T> extends UnorderedSliceSpliterator<T, Spliterator<T>> implements Spliterator<T>, Consumer<T> {
            public T tmpSlot;

            public OfRef(Spliterator<T> spliterator, long j, long j2) {
                super(spliterator, j, j2);
            }

            public OfRef(Spliterator<T> spliterator, OfRef<T> ofRef) {
                super(spliterator, ofRef);
            }

            @Override // java9.util.function.Consumer
            public final void accept(T t) {
                this.tmpSlot = t;
            }

            @Override // java9.util.function.Consumer
            public /* synthetic */ Consumer andThen(Consumer consumer) {
                return Objects.requireNonNull(consumer);
            }

            @Override // java9.util.Spliterator
            public void forEachRemaining(Consumer<? super T> consumer) {
                Objects.requireNonNull(consumer);
                ArrayBuffer.OfRef ofRef = null;
                while (true) {
                    PermitStatus permitStatus = permitStatus();
                    if (permitStatus == PermitStatus.NO_MORE) {
                        return;
                    }
                    if (permitStatus != PermitStatus.MAYBE_MORE) {
                        this.s.forEachRemaining(consumer);
                        return;
                    }
                    if (ofRef == null) {
                        ofRef = new ArrayBuffer.OfRef(this.chunkSize);
                    } else {
                        ofRef.reset();
                    }
                    long j = 0;
                    while (this.s.tryAdvance(ofRef)) {
                        j++;
                        if (j >= this.chunkSize) {
                            break;
                        }
                    }
                    if (j == 0) {
                        return;
                    }
                    ofRef.forEach(consumer, acquirePermits(j));
                }
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ Comparator getComparator() {
                return Spliterator.CC.$default$getComparator(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ long getExactSizeIfKnown() {
                return Spliterator.CC.$default$getExactSizeIfKnown(this);
            }

            @Override // java9.util.Spliterator
            public /* synthetic */ boolean hasCharacteristics(int i) {
                return Spliterator.CC.$default$hasCharacteristics(this, i);
            }

            @Override // java9.util.stream.StreamSpliterators.UnorderedSliceSpliterator
            public Spliterator<T> makeSpliterator(Spliterator<T> spliterator) {
                return new OfRef(spliterator, this);
            }

            @Override // java9.util.Spliterator
            public boolean tryAdvance(Consumer<? super T> consumer) {
                Objects.requireNonNull(consumer);
                while (permitStatus() != PermitStatus.NO_MORE && this.s.tryAdvance(this)) {
                    if (acquirePermits(1L) == 1) {
                        consumer.accept((T) this.tmpSlot);
                        this.tmpSlot = null;
                        return true;
                    }
                }
                return false;
            }
        }

        /* loaded from: classes19.dex */
        public enum PermitStatus {
            NO_MORE,
            MAYBE_MORE,
            UNLIMITED
        }

        public UnorderedSliceSpliterator(T_SPLITR t_splitr, long j, long j2) {
            this.s = t_splitr;
            int i = (j2 > 0L ? 1 : (j2 == 0L ? 0 : -1));
            this.unlimited = i < 0;
            this.skipThreshold = i >= 0 ? j2 : 0L;
            this.chunkSize = i >= 0 ? (int) Math.min(128L, ((j + j2) / AbstractTask.getLeafTarget()) + 1) : 128;
            this.permits = new AtomicLong(i >= 0 ? j + j2 : j);
        }

        public UnorderedSliceSpliterator(T_SPLITR t_splitr, UnorderedSliceSpliterator<T, T_SPLITR> unorderedSliceSpliterator) {
            this.s = t_splitr;
            this.unlimited = unorderedSliceSpliterator.unlimited;
            this.permits = unorderedSliceSpliterator.permits;
            this.skipThreshold = unorderedSliceSpliterator.skipThreshold;
            this.chunkSize = unorderedSliceSpliterator.chunkSize;
        }

        public final long acquirePermits(long j) {
            long j2;
            long min;
            do {
                j2 = this.permits.get();
                if (j2 != 0) {
                    min = Math.min(j2, j);
                    if (min <= 0) {
                        break;
                    }
                } else if (this.unlimited) {
                    return j;
                } else {
                    return 0L;
                }
            } while (!this.permits.compareAndSet(j2, j2 - min));
            if (this.unlimited) {
                return Math.max(j - min, 0L);
            }
            long j3 = this.skipThreshold;
            return j2 > j3 ? Math.max(min - (j2 - j3), 0L) : min;
        }

        public final int characteristics() {
            return this.s.characteristics() & (-16465);
        }

        public final long estimateSize() {
            return this.s.estimateSize();
        }

        public abstract T_SPLITR makeSpliterator(T_SPLITR t_splitr);

        public final PermitStatus permitStatus() {
            return this.permits.get() > 0 ? PermitStatus.MAYBE_MORE : this.unlimited ? PermitStatus.UNLIMITED : PermitStatus.NO_MORE;
        }

        /* JADX WARN: Multi-variable type inference failed */
        public final T_SPLITR trySplit() {
            Spliterator<T> trySplit;
            if (this.permits.get() == 0 || (trySplit = this.s.trySplit()) == null) {
                return null;
            }
            return (T_SPLITR) makeSpliterator(trySplit);
        }
    }

    /* loaded from: classes19.dex */
    public static final class WrappingSpliterator<P_IN, P_OUT> extends AbstractWrappingSpliterator<P_IN, P_OUT, SpinedBuffer<P_OUT>> {
        public WrappingSpliterator(PipelineHelper<P_OUT> pipelineHelper, Spliterator<P_IN> spliterator, boolean z) {
            super(pipelineHelper, spliterator, z);
        }

        public WrappingSpliterator(PipelineHelper<P_OUT> pipelineHelper, Supplier<Spliterator<P_IN>> supplier, boolean z) {
            super(pipelineHelper, supplier, z);
        }

        /* JADX INFO: Access modifiers changed from: private */
        public /* synthetic */ boolean a() {
            return this.spliterator.tryAdvance(this.bufferSink);
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator, java9.util.Spliterator
        public void forEachRemaining(final Consumer<? super P_OUT> consumer) {
            if (this.buffer != 0 || this.finished) {
                do {
                } while (tryAdvance(consumer));
                return;
            }
            Objects.requireNonNull(consumer);
            init();
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            consumer.getClass();
            pipelineHelper.wrapAndCopyInto(new Sink() { // from class: java9.util.stream.StreamSpliterators$WrappingSpliterator$$ExternalSyntheticLambda0
                @Override // java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public /* synthetic */ void accept(double d) {
                    Sink.CC.$default$accept(this, d);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(int i) {
                    Sink.CC.$default$accept((Sink) this, i);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(long j) {
                    Sink.CC.$default$accept((Sink) this, j);
                }

                @Override // java9.util.function.Consumer
                public final void accept(Object obj) {
                    Consumer.this.accept(obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer2) {
                    return Objects.requireNonNull(consumer2);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            }, this.spliterator);
            this.finished = true;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public void initPartialTraversalState() {
            final SpinedBuffer spinedBuffer = new SpinedBuffer();
            this.buffer = spinedBuffer;
            PipelineHelper<P_OUT> pipelineHelper = this.ph;
            spinedBuffer.getClass();
            this.bufferSink = pipelineHelper.wrapSink(new Sink() { // from class: java9.util.stream.StreamSpliterators$WrappingSpliterator$$ExternalSyntheticLambda1
                @Override // java9.util.stream.Sink, java9.util.function.DoubleConsumer
                public /* synthetic */ void accept(double d) {
                    Sink.CC.$default$accept(this, d);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(int i) {
                    Sink.CC.$default$accept((Sink) this, i);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void accept(long j) {
                    Sink.CC.$default$accept((Sink) this, j);
                }

                @Override // java9.util.function.Consumer
                public final void accept(Object obj) {
                    SpinedBuffer.this.accept(obj);
                }

                @Override // java9.util.function.Consumer
                public /* synthetic */ Consumer andThen(Consumer consumer) {
                    return Objects.requireNonNull(consumer);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void begin(long j) {
                    Sink.CC.$default$begin(this, j);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ boolean cancellationRequested() {
                    return Sink.CC.$default$cancellationRequested(this);
                }

                @Override // java9.util.stream.Sink
                public /* synthetic */ void end() {
                    Sink.CC.$default$end(this);
                }
            });
            this.pusher = new BooleanSupplier() { // from class: java9.util.stream.StreamSpliterators$WrappingSpliterator$$ExternalSyntheticLambda2
                @Override // java9.util.function.BooleanSupplier
                public final boolean getAsBoolean() {
                    boolean a2;
                    a2 = StreamSpliterators.WrappingSpliterator.this.a();
                    return a2;
                }
            };
        }

        @Override // java9.util.Spliterator
        public boolean tryAdvance(Consumer<? super P_OUT> consumer) {
            Objects.requireNonNull(consumer);
            boolean doAdvance = doAdvance();
            if (doAdvance) {
                consumer.accept((Object) ((SpinedBuffer) this.buffer).get(this.nextToConsume));
            }
            return doAdvance;
        }

        @Override // java9.util.stream.StreamSpliterators.AbstractWrappingSpliterator
        public WrappingSpliterator<P_IN, P_OUT> wrap(Spliterator<P_IN> spliterator) {
            return new WrappingSpliterator<>(this.ph, spliterator, this.isParallel);
        }
    }
}
