package org.webrtc.ali;

import android.content.Context;
import android.content.res.AssetFileDescriptor;
import android.media.MediaCodec;
import android.media.MediaCodecInfo;
import android.media.MediaCodecList;
import android.media.MediaCrypto;
import android.media.MediaExtractor;
import android.media.MediaFormat;
import android.os.Build;
import android.view.Surface;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.nio.ByteBuffer;
import java9.util.DualPivotQuicksort;
/* loaded from: classes19.dex */
public class AliHardwareAudioDecoder {
    private Context context_;
    private long fileLength_;
    private ByteBuffer[] inputBuffers_;
    private boolean isMeizu_;
    private boolean isVivo_;
    private ByteBuffer[] outputBuffers_;
    private static String TAG = "MediaCodecAudioDecoder";
    private static int MAX_DECODER_RETRY_COUNT = 64;
    private static int HTTP_REQUEST_TIMEOUT = 6000;
    private static int DECODE_READ_BUFFER_SIZE = DualPivotQuicksort.MAX_SHORT_INDEX;
    private boolean inputStreamEnd_ = false;
    private boolean outputStreamEnd_ = false;
    private ByteBuffer decodeReadBuffer_ = ByteBuffer.allocateDirect(DECODE_READ_BUFFER_SIZE);
    private MediaCodec mediaCodec_ = null;
    private MediaExtractor extractor_ = null;
    private MediaFormat trackFormat_ = null;
    private boolean decodeIsReady_ = false;
    private int sampleRate_ = 48000;
    private int channels_ = 2;
    private int retryCount_ = 0;
    private long nativeAudioCodec_ = 0;
    MediaCodec.BufferInfo info = new MediaCodec.BufferInfo();

    private native void nativeCacheDirectBufferAddress(ByteBuffer byteBuffer, long j);

    private native void nativeCodecInit(int i, int i2, long j, long j2);

    private native void nativeDataBufferIsReady(int i, int i2, int i3, long j);

    public AliHardwareAudioDecoder(Context context) {
        this.context_ = null;
        boolean z = false;
        this.isMeizu_ = false;
        this.isVivo_ = false;
        this.context_ = context;
        this.isMeizu_ = Build.BRAND.toLowerCase().contains("meizu") || Build.MANUFACTURER.toLowerCase().contains("meizu");
        this.isVivo_ = (Build.BRAND.toLowerCase().contains("vivo") || Build.MANUFACTURER.toLowerCase().contains("vivo")) ? true : true;
    }

    /* JADX WARN: Removed duplicated region for block: B:32:0x0101 A[Catch: Exception -> 0x018d, LOOP:0: B:31:0x00ff->B:32:0x0101, LOOP_END, TryCatch #1 {Exception -> 0x018d, blocks: (B:3:0x0008, B:5:0x0041, B:9:0x0047, B:30:0x00f7, B:32:0x0101, B:35:0x010c, B:37:0x0122, B:38:0x0134, B:39:0x0137, B:43:0x013d, B:12:0x006b, B:17:0x0088, B:19:0x00ab, B:24:0x00b4, B:27:0x00d6, B:29:0x00f4), top: B:50:0x0008, inners: #0, #2, #3 }] */
    /* JADX WARN: Removed duplicated region for block: B:35:0x010c A[Catch: Exception -> 0x018d, TryCatch #1 {Exception -> 0x018d, blocks: (B:3:0x0008, B:5:0x0041, B:9:0x0047, B:30:0x00f7, B:32:0x0101, B:35:0x010c, B:37:0x0122, B:38:0x0134, B:39:0x0137, B:43:0x013d, B:12:0x006b, B:17:0x0088, B:19:0x00ab, B:24:0x00b4, B:27:0x00d6, B:29:0x00f4), top: B:50:0x0008, inners: #0, #2, #3 }] */
    /* JADX WARN: Removed duplicated region for block: B:41:0x013b A[RETURN] */
    /* JADX WARN: Removed duplicated region for block: B:43:0x013d A[Catch: Exception -> 0x018d, TRY_LEAVE, TryCatch #1 {Exception -> 0x018d, blocks: (B:3:0x0008, B:5:0x0041, B:9:0x0047, B:30:0x00f7, B:32:0x0101, B:35:0x010c, B:37:0x0122, B:38:0x0134, B:39:0x0137, B:43:0x013d, B:12:0x006b, B:17:0x0088, B:19:0x00ab, B:24:0x00b4, B:27:0x00d6, B:29:0x00f4), top: B:50:0x0008, inners: #0, #2, #3 }] */
    /* JADX WARN: Removed duplicated region for block: B:55:0x0137 A[EDGE_INSN: B:55:0x0137->B:39:0x0137 ?: BREAK  , SYNTHETIC] */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    public int initDecoder(String filename) {
        int nTrackCount;
        int i;
        int i2;
        MediaCodec mediaCodec;
        try {
            this.retryCount_ = 0;
            this.decodeReadBuffer_.clear();
            Logging.d(TAG, "open Audio File name:" + filename);
            boolean isHttpFile = filename.toLowerCase().startsWith("http");
            boolean isAssetsFile = filename.startsWith("/assets/");
            MediaExtractor mediaExtractor = new MediaExtractor();
            this.extractor_ = mediaExtractor;
            if (isAssetsFile) {
                Context context = this.context_;
                if (context != null) {
                    try {
                        AssetFileDescriptor assetfd = context.getAssets().openFd(filename.substring("/assets/".length()));
                        this.extractor_.setDataSource(assetfd.getFileDescriptor(), assetfd.getStartOffset(), assetfd.getLength());
                        nTrackCount = this.extractor_.getTrackCount();
                        for (i = 0; i < nTrackCount; i++) {
                            this.extractor_.unselectTrack(i);
                        }
                        i2 = 0;
                        while (true) {
                            if (i2 < nTrackCount) {
                                break;
                            }
                            MediaFormat trackFormat = this.extractor_.getTrackFormat(i2);
                            this.trackFormat_ = trackFormat;
                            String mimeType = trackFormat.getString("mime");
                            if (!mimeType.contains("audio/")) {
                                i2++;
                            } else {
                                this.extractor_.selectTrack(i2);
                                MediaCodec createDecoderByType = MediaCodec.createDecoderByType(mimeType);
                                this.mediaCodec_ = createDecoderByType;
                                createDecoderByType.configure(this.trackFormat_, (Surface) null, (MediaCrypto) null, 0);
                                break;
                            }
                        }
                        mediaCodec = this.mediaCodec_;
                        if (mediaCodec != null) {
                            return -6;
                        }
                        mediaCodec.start();
                        this.sampleRate_ = this.trackFormat_.getInteger("sample-rate");
                        this.channels_ = this.trackFormat_.getInteger("channel-count");
                        long j = this.trackFormat_.getLong("durationUs");
                        this.fileLength_ = j;
                        nativeCodecInit(this.sampleRate_, this.channels_, j, this.nativeAudioCodec_);
                        nativeCacheDirectBufferAddress(this.decodeReadBuffer_, this.nativeAudioCodec_);
                        Logging.d(TAG, "open Audio File name:" + filename + " succ!");
                        return 0;
                    } catch (IOException e) {
                        Logging.e(TAG, "Failed to open assert File:" + filename);
                        return -2;
                    }
                }
                return -1;
            }
            if (isHttpFile) {
                try {
                    try {
                        HttpURLConnection.setFollowRedirects(true);
                        HttpURLConnection con = (HttpURLConnection) new URL(filename).openConnection();
                        con.setConnectTimeout(HTTP_REQUEST_TIMEOUT);
                        con.setReadTimeout(HTTP_REQUEST_TIMEOUT);
                        con.connect();
                        if (con.getResponseCode() == 200) {
                            this.extractor_.setDataSource(filename);
                        } else {
                            return -3;
                        }
                    } catch (IOException ex) {
                        Logging.e(TAG, "Failed to Connect URL:" + filename + " exception!");
                        ex.printStackTrace();
                        return -5;
                    }
                } catch (SocketTimeoutException e2) {
                    Logging.e(TAG, "Failed to Connect URL:" + filename + " timeout!");
                    return -4;
                }
            } else {
                mediaExtractor.setDataSource(filename);
            }
            nTrackCount = this.extractor_.getTrackCount();
            while (i < nTrackCount) {
            }
            i2 = 0;
            while (true) {
                if (i2 < nTrackCount) {
                }
                i2++;
            }
            mediaCodec = this.mediaCodec_;
            if (mediaCodec != null) {
            }
        } catch (Exception ex2) {
            Logging.e(TAG, "Failed to open file:" + filename);
            ex2.printStackTrace();
            return -7;
        }
    }

    public int getAudioChannels() {
        return this.channels_;
    }

    public int getAudioSampleRate() {
        return this.sampleRate_;
    }

    public long getFileLength() {
        return this.fileLength_;
    }

    public void rewindDecoder() {
        try {
            this.extractor_.seekTo(0L, 1);
            this.mediaCodec_.flush();
        } catch (Exception e) {
            Logging.e(TAG, "Failed to rewind file!");
        }
        this.inputStreamEnd_ = false;
        this.outputStreamEnd_ = false;
        this.decodeIsReady_ = false;
    }

    public void releaseDecoder() {
        try {
            MediaCodec mediaCodec = this.mediaCodec_;
            if (mediaCodec != null) {
                mediaCodec.stop();
                this.mediaCodec_.release();
                this.mediaCodec_ = null;
            }
            MediaExtractor mediaExtractor = this.extractor_;
            if (mediaExtractor != null) {
                mediaExtractor.release();
                this.extractor_ = null;
            }
        } catch (Exception e) {
            Logging.e(TAG, "Failed to releaseDecoder file!");
            e.printStackTrace();
        }
        Logging.d(TAG, "releaseDecoder!");
        this.outputStreamEnd_ = false;
        this.inputStreamEnd_ = false;
    }

    private void dequeueInputBuffer() {
        int sampleSize;
        int index = this.mediaCodec_.dequeueInputBuffer(0L);
        if (index >= 0) {
            ByteBuffer inBuf = this.mediaCodec_.getInputBuffer(index);
            long sampleTime = this.extractor_.getSampleTime();
            int flags = this.extractor_.getSampleFlags();
            int sampleSize2 = this.extractor_.readSampleData(inBuf, 0);
            if (sampleSize2 > 0) {
                sampleSize = sampleSize2;
            } else {
                this.inputStreamEnd_ = true;
                flags |= 4;
                sampleSize = 0;
            }
            this.mediaCodec_.queueInputBuffer(index, 0, sampleSize, sampleTime, flags);
            this.extractor_.advance();
        }
    }

    private void dequeueOutputBuffer() {
        int index = this.mediaCodec_.dequeueOutputBuffer(this.info, 0L);
        this.decodeIsReady_ = false;
        switch (index) {
            case -3:
            case -2:
                return;
            case -1:
                int i = this.retryCount_ + 1;
                this.retryCount_ = i;
                if (i >= MAX_DECODER_RETRY_COUNT) {
                    if (this.isMeizu_ || this.isVivo_) {
                        Logging.e(TAG, "Failed to dequeueBuffer trycount=" + this.retryCount_ + " presentationTime=" + this.info.presentationTimeUs + " total=" + this.fileLength_);
                        this.retryCount_ = 0;
                        this.outputStreamEnd_ = true;
                        return;
                    }
                    return;
                }
                return;
            default:
                if (index >= 0) {
                    this.retryCount_ = 0;
                    if ((this.info.flags & 4) != 0) {
                        this.outputStreamEnd_ = true;
                    }
                    ByteBuffer outbuf = this.mediaCodec_.getOutputBuffer(index);
                    copyToReadBuffer(outbuf, outbuf.limit());
                    this.mediaCodec_.releaseOutputBuffer(index, false);
                    this.decodeIsReady_ = true;
                    return;
                }
                return;
        }
    }

    public boolean decodeData() {
        try {
            if (!this.inputStreamEnd_) {
                dequeueInputBuffer();
            }
            if (!this.outputStreamEnd_) {
                dequeueOutputBuffer();
            }
            return this.outputStreamEnd_;
        } catch (Exception ex) {
            Logging.e(TAG, "Failed to decode data!");
            ex.printStackTrace();
            return false;
        }
    }

    private boolean checkAudioForamtChange() {
        boolean change = false;
        try {
            MediaFormat newformat = this.mediaCodec_.getOutputFormat();
            int new_channels = newformat.getInteger("channel-count");
            int new_rate = newformat.getInteger("sample-rate");
            change = (this.sampleRate_ == new_rate && this.channels_ == new_channels) ? true : true;
            this.sampleRate_ = new_rate;
            this.channels_ = new_channels;
            return change;
        } catch (Exception e) {
            Logging.e(TAG, "Failed to get new audio format!");
            return false;
        }
    }

    private void copyToReadBuffer(ByteBuffer inputBuffer, int length) {
        int readBytes;
        int pos = 0;
        boolean bResetPos = false;
        try {
            checkAudioForamtChange();
            while (pos < length) {
                int readBytes2 = length - pos;
                int readBytes3 = DECODE_READ_BUFFER_SIZE;
                if (readBytes2 <= readBytes3) {
                    readBytes = readBytes2;
                } else {
                    bResetPos = true;
                    readBytes = readBytes3;
                }
                if (bResetPos) {
                    inputBuffer.position(pos);
                    inputBuffer.limit(pos + readBytes);
                }
                this.decodeReadBuffer_.position(0);
                this.decodeReadBuffer_.put(inputBuffer);
                nativeDataBufferIsReady(readBytes, this.sampleRate_, this.channels_, this.nativeAudioCodec_);
                pos += readBytes;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static boolean isAACSupported() {
        try {
            MediaCodecList codecList = new MediaCodecList(1);
            MediaCodecInfo[] codecInfos = codecList.getCodecInfos();
            for (MediaCodecInfo codecInfo : codecInfos) {
                if (!codecInfo.isEncoder()) {
                    String name = codecInfo.getName().toLowerCase();
                    if (name.contains("nvidia")) {
                        return false;
                    }
                }
            }
            return true;
        } catch (Exception ex) {
            Logging.e(TAG, "Failed to get aac decoder");
            ex.printStackTrace();
            return false;
        }
    }

    public long getFilePosition() {
        return this.extractor_.getSampleTime();
    }

    public void setFilePosition(long pos) {
        this.extractor_.seekTo(pos, 2);
    }

    public boolean decodeIsReady() {
        return this.decodeIsReady_;
    }
}
