package org.webrtc.ali;

import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
/* loaded from: classes19.dex */
class DynamicBitrateAdjuster extends BaseBitrateAdjuster {
    private static final double BITRATE_ADJUSTMENT_MAX_SCALE = 4.0d;
    private static final double BITRATE_ADJUSTMENT_SEC = 3.0d;
    private static final int BITRATE_ADJUSTMENT_STEPS = 20;
    private static final double BITS_PER_BYTE = 8.0d;
    private double deviationBytes = FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE;
    private double timeSinceLastAdjustmentMs = FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE;
    private int bitrateAdjustmentScaleExp = 0;

    @Override // org.webrtc.ali.BaseBitrateAdjuster, org.webrtc.ali.BitrateAdjuster
    public void setTargets(int targetBitrateBps, int targetFps) {
        if (this.targetBitrateBps > 0 && targetBitrateBps < this.targetBitrateBps) {
            this.deviationBytes = (this.deviationBytes * targetBitrateBps) / this.targetBitrateBps;
        }
        super.setTargets(targetBitrateBps, targetFps);
    }

    @Override // org.webrtc.ali.BaseBitrateAdjuster, org.webrtc.ali.BitrateAdjuster
    public void reportEncodedFrame(int size) {
        if (this.targetFps == 0) {
            return;
        }
        double expectedBytesPerFrame = (this.targetBitrateBps / BITS_PER_BYTE) / this.targetFps;
        this.deviationBytes += size - expectedBytesPerFrame;
        this.timeSinceLastAdjustmentMs += 1000.0d / this.targetFps;
        double deviationThresholdBytes = this.targetBitrateBps / BITS_PER_BYTE;
        double deviationCap = BITRATE_ADJUSTMENT_SEC * deviationThresholdBytes;
        double min = Math.min(this.deviationBytes, deviationCap);
        this.deviationBytes = min;
        double max = Math.max(min, -deviationCap);
        this.deviationBytes = max;
        if (this.timeSinceLastAdjustmentMs <= 3000.0d) {
            return;
        }
        if (max > deviationThresholdBytes) {
            int bitrateAdjustmentInc = (int) ((max / deviationThresholdBytes) + 0.5d);
            int i = this.bitrateAdjustmentScaleExp - bitrateAdjustmentInc;
            this.bitrateAdjustmentScaleExp = i;
            this.bitrateAdjustmentScaleExp = Math.max(i, -20);
            this.deviationBytes = deviationThresholdBytes;
        } else if (max < (-deviationThresholdBytes)) {
            int bitrateAdjustmentInc2 = (int) (((-max) / deviationThresholdBytes) + 0.5d);
            int i2 = this.bitrateAdjustmentScaleExp + bitrateAdjustmentInc2;
            this.bitrateAdjustmentScaleExp = i2;
            this.bitrateAdjustmentScaleExp = Math.min(i2, 20);
            this.deviationBytes = -deviationThresholdBytes;
        }
        this.timeSinceLastAdjustmentMs = FirebaseRemoteConfig.DEFAULT_VALUE_FOR_DOUBLE;
    }

    @Override // org.webrtc.ali.BaseBitrateAdjuster, org.webrtc.ali.BitrateAdjuster
    public int getAdjustedBitrateBps() {
        return (int) (this.targetBitrateBps * Math.pow(BITRATE_ADJUSTMENT_MAX_SCALE, this.bitrateAdjustmentScaleExp / 20.0d));
    }
}
