package org.webrtc.ali;

import java.nio.ByteBuffer;
/* loaded from: classes19.dex */
public class EncodedImage {
    public final ByteBuffer buffer;
    public final long captureTimeMs;
    public final boolean completeFrame;
    public final int encodedHeight;
    public final int encodedWidth;
    public final FrameType frameType;
    public final Integer qp;
    public final int rotation;

    /* loaded from: classes19.dex */
    public enum FrameType {
        EmptyFrame,
        VideoFrameKey,
        VideoFrameDelta
    }

    private EncodedImage(ByteBuffer buffer, int encodedWidth, int encodedHeight, long captureTimeMs, FrameType frameType, int rotation, boolean completeFrame, Integer qp) {
        this.buffer = buffer;
        this.encodedWidth = encodedWidth;
        this.encodedHeight = encodedHeight;
        this.captureTimeMs = captureTimeMs;
        this.frameType = frameType;
        this.rotation = rotation;
        this.completeFrame = completeFrame;
        this.qp = qp;
    }

    public static Builder builder() {
        return new Builder();
    }

    /* loaded from: classes19.dex */
    public static class Builder {
        private ByteBuffer buffer;
        private long captureTimeMs;
        private boolean completeFrame;
        private int encodedHeight;
        private int encodedWidth;
        private FrameType frameType;
        private Integer qp;
        private int rotation;

        private Builder() {
        }

        public Builder setBuffer(ByteBuffer buffer) {
            this.buffer = buffer;
            return this;
        }

        public Builder setEncodedWidth(int encodedWidth) {
            this.encodedWidth = encodedWidth;
            return this;
        }

        public Builder setEncodedHeight(int encodedHeight) {
            this.encodedHeight = encodedHeight;
            return this;
        }

        public Builder setCaptureTimeMs(long captureTimeMs) {
            this.captureTimeMs = captureTimeMs;
            return this;
        }

        public Builder setFrameType(FrameType frameType) {
            this.frameType = frameType;
            return this;
        }

        public Builder setRotation(int rotation) {
            this.rotation = rotation;
            return this;
        }

        public Builder setCompleteFrame(boolean completeFrame) {
            this.completeFrame = completeFrame;
            return this;
        }

        public Builder setQp(Integer qp) {
            this.qp = qp;
            return this;
        }

        public EncodedImage createEncodedImage() {
            return new EncodedImage(this.buffer, this.encodedWidth, this.encodedHeight, this.captureTimeMs, this.frameType, this.rotation, this.completeFrame, this.qp);
        }
    }
}
