package org.webrtc.ali;

import android.text.TextUtils;
import android.util.Log;
import com.serenegiant.usb.USBMonitor;
/* loaded from: classes19.dex */
public class USBAudioDevice {
    private static final String DEFAULT_USBFS = "/dev/bus/usb";
    static final int LOG_DEBUG = 2;
    static final int LOG_ERROR = 5;
    static final int LOG_FATAL = 6;
    static final int LOG_INFO = 3;
    static final int LOG_SILENT = 7;
    static final int LOG_VERBOSE = 1;
    static final int LOG_WARN = 4;
    private long audioDevicePlayerHandle_;
    private long audioDeviceRecordHandle_;
    private USBMonitor.UsbControlBlock usbControlPlayBlock_;
    private USBMonitor.UsbControlBlock usbControlRecordBlock_;
    private final String TAG = "USBAudioDevice";
    private int audioRecordDeviceCount_ = 0;
    private int audioPlayerDeviceCount_ = 0;

    private static final native int nativeCloseUSBPlayer(long j);

    private static final native int nativeCloseUSBRecord(long j);

    private final native int nativeConnectUSBPlayer(long j, int i, int i2, int i3, int i4, int i5, String str);

    private final native int nativeConnectUSBRecord(long j, int i, int i2, int i3, int i4, int i5, String str);

    private static final native int nativeCreateUSBLog(String str, int i);

    private final native long nativeCreateUSBPlayer();

    private final native long nativeCreateUSBRecord();

    private static final native int nativeDestroyUSBLog(int i);

    private final native void nativeDestroyUSBPlayer(long j);

    private final native void nativeDestroyUSBRecord(long j);

    private static final native int nativeStopUSBPlayer(long j);

    private static final native int nativeStopUSBRecord(long j);

    private static final native int nativeWriteUSBLog(int i, String str);

    static {
        System.loadLibrary("usb100");
        System.loadLibrary("USBAudioDevice");
    }

    private final String getUSBFSName(String deviceName) {
        String result = null;
        String[] v = !TextUtils.isEmpty(deviceName) ? deviceName.split("/") : null;
        if (v != null && v.length > 2) {
            StringBuilder sb = new StringBuilder(v[0]);
            for (int i = 1; i < v.length - 2; i++) {
                sb.append("/").append(v[i]);
            }
            result = sb.toString();
        }
        if (TextUtils.isEmpty(result)) {
            Log.w("USBAudioDevice", "failed to get USBFS path, try to use default path:" + deviceName);
            return DEFAULT_USBFS;
        }
        return result;
    }

    public USBAudioDevice() {
        this.audioDeviceRecordHandle_ = 0L;
        this.audioDevicePlayerHandle_ = 0L;
        this.audioDeviceRecordHandle_ = nativeCreateUSBRecord();
        this.audioDevicePlayerHandle_ = nativeCreateUSBPlayer();
    }

    public synchronized void release() {
        releaseUSBRecord();
        releaseUSBPlayer();
        releaseLogFile(0);
    }

    public synchronized long getAudioRecordHandle() {
        return this.audioDeviceRecordHandle_;
    }

    public int getAudioRecordDeviceCount() {
        return this.audioRecordDeviceCount_;
    }

    public synchronized int connectUSBRecord(USBMonitor.UsbControlBlock usbControlBlock) {
        if (this.audioDeviceRecordHandle_ == 0) {
            return -1;
        }
        try {
            USBMonitor.UsbControlBlock m92clone = usbControlBlock.m92clone();
            this.usbControlRecordBlock_ = m92clone;
            int rc = nativeConnectUSBRecord(this.audioDeviceRecordHandle_, m92clone.getVenderId(), this.usbControlRecordBlock_.getProductId(), this.usbControlRecordBlock_.getFileDescriptor(), this.usbControlRecordBlock_.getBusNum(), this.usbControlRecordBlock_.getDevNum(), getUSBFSName(this.usbControlRecordBlock_.getDeviceName()));
            if (rc <= 0) {
                return -2;
            }
            this.audioRecordDeviceCount_ = rc;
            return 0;
        } catch (Exception e) {
            writeLog(5, "info:" + e.getMessage());
            return -3;
        }
    }

    public synchronized int closeUSBRecord() {
        long j = this.audioDeviceRecordHandle_;
        if (j == 0) {
            return -1;
        }
        this.audioRecordDeviceCount_ = 0;
        return nativeCloseUSBRecord(j);
    }

    public synchronized int stopUSBRecord() {
        long j = this.audioDeviceRecordHandle_;
        if (j == 0) {
            return -1;
        }
        return nativeStopUSBRecord(j);
    }

    public synchronized void releaseUSBRecord() {
        long j = this.audioDeviceRecordHandle_;
        if (j == 0) {
            return;
        }
        nativeDestroyUSBRecord(j);
        this.audioDeviceRecordHandle_ = 0L;
        this.audioRecordDeviceCount_ = 0;
    }

    public synchronized long getAudioPlayerHandle() {
        return this.audioDevicePlayerHandle_;
    }

    public synchronized int getAudioPlayerDeviceCount() {
        return this.audioPlayerDeviceCount_;
    }

    public synchronized int connectUSBPlayer(USBMonitor.UsbControlBlock usbControlBlock) {
        if (this.audioDevicePlayerHandle_ == 0) {
            return -1;
        }
        try {
            USBMonitor.UsbControlBlock m92clone = usbControlBlock.m92clone();
            this.usbControlPlayBlock_ = m92clone;
            int rc = nativeConnectUSBPlayer(this.audioDevicePlayerHandle_, m92clone.getVenderId(), this.usbControlPlayBlock_.getProductId(), this.usbControlPlayBlock_.getFileDescriptor(), this.usbControlPlayBlock_.getBusNum(), this.usbControlPlayBlock_.getDevNum(), getUSBFSName(this.usbControlPlayBlock_.getDeviceName()));
            if (rc <= 0) {
                return -2;
            }
            this.audioPlayerDeviceCount_ = rc;
            return 0;
        } catch (Exception e) {
            writeLog(5, "info:" + e.getMessage());
            return -3;
        }
    }

    public synchronized int closeUSBPlayer() {
        long j = this.audioDevicePlayerHandle_;
        if (j == 0) {
            return -1;
        }
        this.audioPlayerDeviceCount_ = 0;
        return nativeCloseUSBPlayer(j);
    }

    public synchronized int stopUSBPlayer() {
        long j = this.audioDevicePlayerHandle_;
        if (j == 0) {
            return -1;
        }
        return nativeStopUSBPlayer(j);
    }

    public synchronized void releaseUSBPlayer() {
        long j = this.audioDevicePlayerHandle_;
        if (j == 0) {
            return;
        }
        nativeDestroyUSBPlayer(j);
        this.audioDevicePlayerHandle_ = 0L;
        this.audioPlayerDeviceCount_ = 0;
    }

    public static synchronized int createLogFile(String strPath, int level) {
        int nativeCreateUSBLog;
        synchronized (USBAudioDevice.class) {
            nativeCreateUSBLog = nativeCreateUSBLog(strPath, level);
        }
        return nativeCreateUSBLog;
    }

    public static int writeLog(int tag, String strLog) {
        return nativeWriteUSBLog(tag, strLog);
    }

    public static int writeLog(String strLog) {
        return nativeWriteUSBLog(3, strLog);
    }

    public static synchronized int releaseLogFile(int param) {
        int nativeDestroyUSBLog;
        synchronized (USBAudioDevice.class) {
            nativeDestroyUSBLog = nativeDestroyUSBLog(param);
        }
        return nativeDestroyUSBLog;
    }
}
