package org.webrtc.ali;

import android.graphics.Matrix;
import java.nio.ByteBuffer;
/* loaded from: classes19.dex */
public class VideoFrame {
    private final Buffer buffer;
    private final int rotation;
    private final long timestampNs;
    private final Matrix transformMatrix;

    /* loaded from: classes19.dex */
    public interface Buffer {
        int getHeight();

        int getWidth();

        void release();

        void retain();

        I420Buffer toI420();
    }

    /* loaded from: classes19.dex */
    public interface I420Buffer extends Buffer {
        ByteBuffer getDataU();

        ByteBuffer getDataV();

        ByteBuffer getDataY();

        int getStrideU();

        int getStrideV();

        int getStrideY();
    }

    /* loaded from: classes19.dex */
    public interface TextureBuffer extends Buffer {

        /* loaded from: classes19.dex */
        public enum Type {
            OES,
            RGB
        }

        int getTextureId();

        Type getType();
    }

    public VideoFrame(Buffer buffer, int rotation, long timestampNs, Matrix transformMatrix) {
        if (buffer == null) {
            throw new IllegalArgumentException("buffer not allowed to be null");
        }
        if (transformMatrix == null) {
            throw new IllegalArgumentException("transformMatrix not allowed to be null");
        }
        this.buffer = buffer;
        this.rotation = rotation;
        this.timestampNs = timestampNs;
        this.transformMatrix = transformMatrix;
    }

    public Buffer getBuffer() {
        return this.buffer;
    }

    public int getRotation() {
        return this.rotation;
    }

    public long getTimestampNs() {
        return this.timestampNs;
    }

    public Matrix getTransformMatrix() {
        return this.transformMatrix;
    }

    public int getWidth() {
        return this.buffer.getWidth();
    }

    public int getHeight() {
        return this.buffer.getHeight();
    }

    public void retain() {
        this.buffer.retain();
    }

    public void release() {
        this.buffer.release();
    }
}
