package org.webrtc.ali;

import com.king.zxing.util.LogUtils;
import java.nio.ByteBuffer;
import org.webrtc.ali.VideoFrame;
/* loaded from: classes19.dex */
public class VideoRenderer {
    long nativeVideoRenderer;

    /* loaded from: classes19.dex */
    public interface Callbacks {
        void renderFrame(I420Frame i420Frame);
    }

    private static native void freeWrappedVideoRenderer(long j);

    public static native void nativeCopyPlane(ByteBuffer byteBuffer, int i, int i2, int i3, ByteBuffer byteBuffer2, int i4);

    private static native long nativeWrapVideoRenderer(Callbacks callbacks);

    private static native void releaseNativeFrame(long j);

    /* loaded from: classes19.dex */
    public static class I420Frame {
        public final int height;
        private long nativeFramePointer;
        public int rotationDegree;
        public final float[] samplingMatrix;
        public int textureId;
        public final int width;
        public final boolean yuvFrame;
        public ByteBuffer[] yuvPlanes;
        public final int[] yuvStrides;

        public I420Frame(int width, int height, int rotationDegree, int[] yuvStrides, ByteBuffer[] yuvPlanes, long nativeFramePointer) {
            this.width = width;
            this.height = height;
            this.yuvStrides = yuvStrides;
            this.yuvPlanes = yuvPlanes;
            this.yuvFrame = true;
            this.rotationDegree = rotationDegree;
            this.nativeFramePointer = nativeFramePointer;
            if (rotationDegree % 90 != 0) {
                throw new IllegalArgumentException("Rotation degree not multiple of 90: " + rotationDegree);
            }
            this.samplingMatrix = RendererCommon.verticalFlipMatrix();
        }

        public I420Frame(int width, int height, int rotationDegree, int textureId, float[] samplingMatrix, long nativeFramePointer) {
            this.width = width;
            this.height = height;
            this.yuvStrides = null;
            this.yuvPlanes = null;
            this.samplingMatrix = samplingMatrix;
            this.textureId = textureId;
            this.yuvFrame = false;
            this.rotationDegree = rotationDegree;
            this.nativeFramePointer = nativeFramePointer;
            if (rotationDegree % 90 != 0) {
                throw new IllegalArgumentException("Rotation degree not multiple of 90: " + rotationDegree);
            }
        }

        public I420Frame(int width, int height, int rotationDegree, float[] samplingMatrix, VideoFrame.Buffer buffer, long nativeFramePointer) {
            this.width = width;
            this.height = height;
            this.rotationDegree = rotationDegree;
            if (rotationDegree % 90 != 0) {
                throw new IllegalArgumentException("Rotation degree not multiple of 90: " + rotationDegree);
            }
            if (buffer instanceof VideoFrame.TextureBuffer) {
                VideoFrame.TextureBuffer textureBuffer = (VideoFrame.TextureBuffer) buffer;
                this.yuvFrame = false;
                this.textureId = textureBuffer.getTextureId();
                this.samplingMatrix = samplingMatrix;
                this.yuvStrides = null;
                this.yuvPlanes = null;
            } else {
                VideoFrame.I420Buffer i420Buffer = buffer.toI420();
                this.yuvFrame = true;
                this.yuvStrides = new int[]{i420Buffer.getStrideY(), i420Buffer.getStrideU(), i420Buffer.getStrideV()};
                this.yuvPlanes = new ByteBuffer[]{i420Buffer.getDataY(), i420Buffer.getDataU(), i420Buffer.getDataV()};
                this.samplingMatrix = RendererCommon.multiplyMatrices(samplingMatrix, RendererCommon.verticalFlipMatrix());
                this.textureId = 0;
            }
            this.nativeFramePointer = nativeFramePointer;
        }

        public int rotatedWidth() {
            return this.rotationDegree % 180 == 0 ? this.width : this.height;
        }

        public int rotatedHeight() {
            return this.rotationDegree % 180 == 0 ? this.height : this.width;
        }

        public String toString() {
            return this.width + "x" + this.height + LogUtils.COLON + this.yuvStrides[0] + LogUtils.COLON + this.yuvStrides[1] + LogUtils.COLON + this.yuvStrides[2];
        }
    }

    public static void renderFrameDone(I420Frame frame) {
        frame.yuvPlanes = null;
        frame.textureId = 0;
        if (frame.nativeFramePointer != 0) {
            releaseNativeFrame(frame.nativeFramePointer);
            frame.nativeFramePointer = 0L;
        }
    }

    public VideoRenderer(Callbacks callbacks) {
        this.nativeVideoRenderer = nativeWrapVideoRenderer(callbacks);
    }

    public void dispose() {
        long j = this.nativeVideoRenderer;
        if (j == 0) {
            return;
        }
        freeWrappedVideoRenderer(j);
        this.nativeVideoRenderer = 0L;
    }
}
