package tlschannel;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ByteChannel;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Supplier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLSession;
import tlschannel.ClientTlsChannel;
import tlschannel.impl.ByteBufferSet;
import tlschannel.impl.TlsChannelImpl;
/* loaded from: classes19.dex */
public class ClientTlsChannel implements TlsChannel {
    public final TlsChannelImpl impl;
    public final ByteChannel underlying;

    /* loaded from: classes19.dex */
    public static class Builder extends TlsChannelBuilder<Builder> {
        public final Supplier<SSLEngine> sslEngineFactory;

        public Builder(ByteChannel byteChannel, final SSLContext sSLContext) {
            super(byteChannel);
            this.sslEngineFactory = new Supplier() { // from class: tlschannel.ClientTlsChannel$Builder$$ExternalSyntheticLambda0
                @Override // java.util.function.Supplier
                public final Object get() {
                    SSLEngine defaultSSLEngineFactory;
                    defaultSSLEngineFactory = ClientTlsChannel.defaultSSLEngineFactory(sSLContext);
                    return defaultSSLEngineFactory;
                }
            };
        }

        public Builder(ByteChannel byteChannel, final SSLEngine sSLEngine) {
            super(byteChannel);
            this.sslEngineFactory = new Supplier() { // from class: tlschannel.ClientTlsChannel$Builder$$ExternalSyntheticLambda1
                @Override // java.util.function.Supplier
                public final Object get() {
                    return ClientTlsChannel.Builder.a(sSLEngine);
                }
            };
        }

        public static /* synthetic */ SSLEngine a(SSLEngine sSLEngine) {
            return sSLEngine;
        }

        public ClientTlsChannel build() {
            return new ClientTlsChannel(this.underlying, this.sslEngineFactory.get(), this.sessionInitCallback, this.runTasks, this.plainBufferAllocator, this.encryptedBufferAllocator, this.releaseBuffers, this.waitForCloseConfirmation);
        }

        /* JADX WARN: Can't rename method to resolve collision */
        @Override // tlschannel.TlsChannelBuilder
        public Builder getThis() {
            return this;
        }
    }

    public ClientTlsChannel(ByteChannel byteChannel, SSLEngine sSLEngine, Consumer<SSLSession> consumer, boolean z, BufferAllocator bufferAllocator, BufferAllocator bufferAllocator2, boolean z2, boolean z3) {
        if (!sSLEngine.getUseClientMode()) {
            throw new IllegalArgumentException("SSLEngine must be in client mode");
        }
        this.underlying = byteChannel;
        this.impl = new TlsChannelImpl(byteChannel, byteChannel, sSLEngine, Optional.empty(), consumer, z, new TrackingAllocator(bufferAllocator), new TrackingAllocator(bufferAllocator2), z2, z3);
    }

    public static SSLEngine defaultSSLEngineFactory(SSLContext sSLContext) {
        SSLEngine createSSLEngine = sSLContext.createSSLEngine();
        createSSLEngine.setUseClientMode(true);
        return createSSLEngine;
    }

    public static Builder newBuilder(ByteChannel byteChannel, SSLContext sSLContext) {
        return new Builder(byteChannel, sSLContext);
    }

    public static Builder newBuilder(ByteChannel byteChannel, SSLEngine sSLEngine) {
        return new Builder(byteChannel, sSLEngine);
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.Channel, java.io.Closeable, java.lang.AutoCloseable
    public void close() throws IOException {
        this.impl.close();
    }

    @Override // tlschannel.TlsChannel
    public TrackingAllocator getEncryptedBufferAllocator() {
        return this.impl.getEncryptedBufferAllocator();
    }

    @Override // tlschannel.TlsChannel
    public TrackingAllocator getPlainBufferAllocator() {
        return this.impl.getPlainBufferAllocator();
    }

    @Override // tlschannel.TlsChannel
    public boolean getRunTasks() {
        return this.impl.getRunTasks();
    }

    @Override // tlschannel.TlsChannel
    public Consumer<SSLSession> getSessionInitCallback() {
        return this.impl.getSessionInitCallback();
    }

    @Override // tlschannel.TlsChannel
    public SSLEngine getSslEngine() {
        return this.impl.engine();
    }

    @Override // tlschannel.TlsChannel
    public ByteChannel getUnderlying() {
        return this.underlying;
    }

    @Override // tlschannel.TlsChannel
    public void handshake() throws IOException {
        this.impl.handshake();
    }

    @Override // java.nio.channels.Channel
    public boolean isOpen() {
        return this.impl.isOpen();
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.ReadableByteChannel
    public int read(ByteBuffer byteBuffer) throws IOException {
        return (int) read(new ByteBuffer[]{byteBuffer});
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.ScatteringByteChannel
    public long read(ByteBuffer[] byteBufferArr) throws IOException {
        return read(byteBufferArr, 0, byteBufferArr.length);
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.ScatteringByteChannel
    public long read(ByteBuffer[] byteBufferArr, int i, int i2) throws IOException {
        ByteBufferSet byteBufferSet = new ByteBufferSet(byteBufferArr, i, i2);
        TlsChannelImpl.checkReadBuffer(byteBufferSet);
        return this.impl.read(byteBufferSet);
    }

    @Override // tlschannel.TlsChannel
    public void renegotiate() throws IOException {
        this.impl.renegotiate();
    }

    @Override // tlschannel.TlsChannel
    public boolean shutdown() throws IOException {
        return this.impl.shutdown();
    }

    @Override // tlschannel.TlsChannel
    public boolean shutdownReceived() {
        return this.impl.shutdownReceived();
    }

    @Override // tlschannel.TlsChannel
    public boolean shutdownSent() {
        return this.impl.shutdownSent();
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.WritableByteChannel
    public int write(ByteBuffer byteBuffer) throws IOException {
        return (int) write(new ByteBuffer[]{byteBuffer});
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.GatheringByteChannel
    public long write(ByteBuffer[] byteBufferArr) throws IOException {
        return write(byteBufferArr, 0, byteBufferArr.length);
    }

    @Override // tlschannel.TlsChannel, java.nio.channels.GatheringByteChannel
    public long write(ByteBuffer[] byteBufferArr, int i, int i2) throws IOException {
        return this.impl.write(new ByteBufferSet(byteBufferArr, i, i2));
    }
}
