package tlschannel;

import java.nio.ByteBuffer;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.LongAccumulator;
import java.util.concurrent.atomic.LongAdder;
import java.util.function.LongBinaryOperator;
/* loaded from: classes19.dex */
public class TrackingAllocator implements BufferAllocator {
    public final BufferAllocator impl;
    public final LongAdder bytesAllocatedAdder = new LongAdder();
    public final LongAdder bytesDeallocatedAdder = new LongAdder();
    public final AtomicLong currentAllocationSize = new AtomicLong();
    public final LongAccumulator maxAllocationSizeAcc = new LongAccumulator(new LongBinaryOperator() { // from class: tlschannel.TrackingAllocator$$ExternalSyntheticLambda0
        @Override // java.util.function.LongBinaryOperator
        public final long applyAsLong(long j, long j2) {
            return Math.max(j, j2);
        }
    }, 0);
    public final LongAdder buffersAllocatedAdder = new LongAdder();
    public final LongAdder buffersDeallocatedAdder = new LongAdder();

    public TrackingAllocator(BufferAllocator bufferAllocator) {
        this.impl = bufferAllocator;
    }

    @Override // tlschannel.BufferAllocator
    public ByteBuffer allocate(int i) {
        long j = i;
        this.bytesAllocatedAdder.add(j);
        this.currentAllocationSize.addAndGet(j);
        this.buffersAllocatedAdder.increment();
        return this.impl.allocate(i);
    }

    public long buffersAllocated() {
        return this.buffersAllocatedAdder.longValue();
    }

    public long buffersDeallocated() {
        return this.buffersDeallocatedAdder.longValue();
    }

    public long bytesAllocated() {
        return this.bytesAllocatedAdder.longValue();
    }

    public long bytesDeallocated() {
        return this.bytesDeallocatedAdder.longValue();
    }

    public long currentAllocation() {
        return this.currentAllocationSize.longValue();
    }

    @Override // tlschannel.BufferAllocator
    public void free(ByteBuffer byteBuffer) {
        int capacity = byteBuffer.capacity();
        this.bytesDeallocatedAdder.add(capacity);
        this.maxAllocationSizeAcc.accumulate(this.currentAllocationSize.longValue());
        this.currentAllocationSize.addAndGet(-capacity);
        this.buffersDeallocatedAdder.increment();
        this.impl.free(byteBuffer);
    }

    public long maxAllocation() {
        return this.maxAllocationSizeAcc.longValue();
    }
}
