package tlschannel.async;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.CompletionHandler;
import java.nio.channels.SocketChannel;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;
import java.util.function.LongConsumer;
import tlschannel.TlsChannel;
import tlschannel.async.AsynchronousTlsChannel;
import tlschannel.async.AsynchronousTlsChannelGroup;
import tlschannel.impl.ByteBufferSet;
/* loaded from: classes19.dex */
public class AsynchronousTlsChannel implements ExtendedAsynchronousByteChannel {
    public final AsynchronousTlsChannelGroup group;
    public final AsynchronousTlsChannelGroup.RegisteredSocket registeredSocket;
    public final TlsChannel tlsChannel;

    /* loaded from: classes19.dex */
    public class FutureReadResult extends CompletableFuture<Integer> {
        public AsynchronousTlsChannelGroup.ReadOperation op;

        public FutureReadResult() {
        }

        @Override // java.util.concurrent.CompletableFuture, java.util.concurrent.Future
        public boolean cancel(boolean z) {
            super.cancel(z);
            return AsynchronousTlsChannel.this.group.doCancelRead(AsynchronousTlsChannel.this.registeredSocket, this.op);
        }
    }

    /* loaded from: classes19.dex */
    public class FutureWriteResult extends CompletableFuture<Integer> {
        public AsynchronousTlsChannelGroup.WriteOperation op;

        public FutureWriteResult() {
        }

        @Override // java.util.concurrent.CompletableFuture, java.util.concurrent.Future
        public boolean cancel(boolean z) {
            super.cancel(z);
            return AsynchronousTlsChannel.this.group.doCancelWrite(AsynchronousTlsChannel.this.registeredSocket, this.op);
        }
    }

    public AsynchronousTlsChannel(AsynchronousTlsChannelGroup asynchronousTlsChannelGroup, TlsChannel tlsChannel, SocketChannel socketChannel) throws ClosedChannelException, IllegalArgumentException {
        if (!tlsChannel.isOpen() || !socketChannel.isOpen()) {
            throw new ClosedChannelException();
        }
        if (socketChannel.isBlocking()) {
            throw new IllegalArgumentException("socket channel must be in non-blocking mode");
        }
        this.group = asynchronousTlsChannelGroup;
        this.tlsChannel = tlsChannel;
        this.registeredSocket = asynchronousTlsChannelGroup.registerSocket(tlsChannel, socketChannel);
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void a(final CompletionHandler completionHandler, final Object obj, final long j) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda26
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(Integer.valueOf((int) j), obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void a(final CompletionHandler completionHandler, final Object obj, final Throwable th) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda5
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.failed(th, obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void b(final CompletionHandler completionHandler, final Object obj, final long j) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda17
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(Integer.valueOf((int) j), obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void b(final CompletionHandler completionHandler, final Object obj, final Throwable th) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda1
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.failed(th, obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void c(final CompletionHandler completionHandler, final Object obj, final long j) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda15
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(Long.valueOf(j), obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void c(final CompletionHandler completionHandler, final Object obj, final Throwable th) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda14
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.failed(th, obj);
            }
        });
    }

    private void checkReadOnly(ByteBuffer byteBuffer) {
        if (byteBuffer.isReadOnly()) {
            throw new IllegalArgumentException("buffer is read-only");
        }
    }

    private <A> void completeWithZeroInt(final A a2, final CompletionHandler<Integer, ? super A> completionHandler) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda27
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(0, a2);
            }
        });
    }

    private <A> void completeWithZeroLong(final A a2, final CompletionHandler<Long, ? super A> completionHandler) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda4
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(0L, a2);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void d(final CompletionHandler completionHandler, final Object obj, final long j) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda23
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(Integer.valueOf((int) j), obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void d(final CompletionHandler completionHandler, final Object obj, final Throwable th) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda21
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.failed(th, obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void e(final CompletionHandler completionHandler, final Object obj, final long j) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda22
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(Integer.valueOf((int) j), obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void e(final CompletionHandler completionHandler, final Object obj, final Throwable th) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda16
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.failed(th, obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void f(final CompletionHandler completionHandler, final Object obj, final long j) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda8
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.completed(Long.valueOf(j), obj);
            }
        });
    }

    /* JADX INFO: Access modifiers changed from: private */
    public /* synthetic */ void f(final CompletionHandler completionHandler, final Object obj, final Throwable th) {
        this.group.executor.submit(new Runnable() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda18
            @Override // java.lang.Runnable
            public final void run() {
                completionHandler.failed(th, obj);
            }
        });
    }

    @Override // java.nio.channels.AsynchronousChannel, java.nio.channels.Channel, java.io.Closeable, java.lang.AutoCloseable
    public void close() throws IOException {
        this.tlsChannel.close();
        this.registeredSocket.close();
    }

    @Override // java.nio.channels.Channel
    public boolean isOpen() {
        return this.tlsChannel.isOpen();
    }

    @Override // java.nio.channels.AsynchronousByteChannel
    public Future<Integer> read(ByteBuffer byteBuffer) {
        checkReadOnly(byteBuffer);
        if (byteBuffer.hasRemaining()) {
            final FutureReadResult futureReadResult = new FutureReadResult();
            AsynchronousTlsChannelGroup asynchronousTlsChannelGroup = this.group;
            AsynchronousTlsChannelGroup.RegisteredSocket registeredSocket = this.registeredSocket;
            ByteBufferSet byteBufferSet = new ByteBufferSet(byteBuffer);
            TimeUnit timeUnit = TimeUnit.MILLISECONDS;
            LongConsumer longConsumer = new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda24
                @Override // java.util.function.LongConsumer
                public final void accept(long j) {
                    AsynchronousTlsChannel.FutureReadResult.this.complete(Integer.valueOf((int) j));
                }
            };
            Objects.requireNonNull(futureReadResult);
            futureReadResult.op = asynchronousTlsChannelGroup.startRead(registeredSocket, byteBufferSet, 0L, timeUnit, longConsumer, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda25
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.FutureReadResult.this.completeExceptionally((Throwable) obj);
                }
            });
            return futureReadResult;
        }
        return CompletableFuture.completedFuture(0);
    }

    @Override // tlschannel.async.ExtendedAsynchronousByteChannel
    public <A> void read(ByteBuffer byteBuffer, long j, TimeUnit timeUnit, final A a2, final CompletionHandler<Integer, ? super A> completionHandler) {
        checkReadOnly(byteBuffer);
        if (byteBuffer.hasRemaining()) {
            this.group.startRead(this.registeredSocket, new ByteBufferSet(byteBuffer), j, timeUnit, new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda6
                @Override // java.util.function.LongConsumer
                public final void accept(long j2) {
                    AsynchronousTlsChannel.this.b(completionHandler, a2, j2);
                }
            }, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda7
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.this.c(completionHandler, a2, (Throwable) obj);
                }
            });
        } else {
            completeWithZeroInt(a2, completionHandler);
        }
    }

    @Override // java.nio.channels.AsynchronousByteChannel
    public <A> void read(ByteBuffer byteBuffer, final A a2, final CompletionHandler<Integer, ? super A> completionHandler) {
        checkReadOnly(byteBuffer);
        if (byteBuffer.hasRemaining()) {
            this.group.startRead(this.registeredSocket, new ByteBufferSet(byteBuffer), 0L, TimeUnit.MILLISECONDS, new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda2
                @Override // java.util.function.LongConsumer
                public final void accept(long j) {
                    AsynchronousTlsChannel.this.a(completionHandler, a2, j);
                }
            }, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda3
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.this.b(completionHandler, a2, (Throwable) obj);
                }
            });
        } else {
            completeWithZeroInt(a2, completionHandler);
        }
    }

    @Override // tlschannel.async.ExtendedAsynchronousByteChannel
    public <A> void read(ByteBuffer[] byteBufferArr, int i, int i2, long j, TimeUnit timeUnit, final A a2, final CompletionHandler<Long, ? super A> completionHandler) {
        ByteBufferSet byteBufferSet = new ByteBufferSet(byteBufferArr, i, i2);
        if (byteBufferSet.isReadOnly()) {
            throw new IllegalArgumentException("buffer is read-only");
        }
        if (byteBufferSet.hasRemaining()) {
            this.group.startRead(this.registeredSocket, byteBufferSet, j, timeUnit, new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda19
                @Override // java.util.function.LongConsumer
                public final void accept(long j2) {
                    AsynchronousTlsChannel.this.c(completionHandler, a2, j2);
                }
            }, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda20
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.this.a(completionHandler, a2, (Throwable) obj);
                }
            });
        } else {
            completeWithZeroLong(a2, completionHandler);
        }
    }

    @Override // java.nio.channels.AsynchronousByteChannel
    public Future<Integer> write(ByteBuffer byteBuffer) {
        if (byteBuffer.hasRemaining()) {
            final FutureWriteResult futureWriteResult = new FutureWriteResult();
            AsynchronousTlsChannelGroup asynchronousTlsChannelGroup = this.group;
            AsynchronousTlsChannelGroup.RegisteredSocket registeredSocket = this.registeredSocket;
            ByteBufferSet byteBufferSet = new ByteBufferSet(byteBuffer);
            TimeUnit timeUnit = TimeUnit.MILLISECONDS;
            LongConsumer longConsumer = new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda9
                @Override // java.util.function.LongConsumer
                public final void accept(long j) {
                    AsynchronousTlsChannel.FutureWriteResult.this.complete(Integer.valueOf((int) j));
                }
            };
            Objects.requireNonNull(futureWriteResult);
            futureWriteResult.op = asynchronousTlsChannelGroup.startWrite(registeredSocket, byteBufferSet, 0L, timeUnit, longConsumer, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda10
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.FutureWriteResult.this.completeExceptionally((Throwable) obj);
                }
            });
            return futureWriteResult;
        }
        return CompletableFuture.completedFuture(0);
    }

    @Override // tlschannel.async.ExtendedAsynchronousByteChannel
    public <A> void write(ByteBuffer byteBuffer, long j, TimeUnit timeUnit, final A a2, final CompletionHandler<Integer, ? super A> completionHandler) {
        if (byteBuffer.hasRemaining()) {
            this.group.startWrite(this.registeredSocket, new ByteBufferSet(byteBuffer), j, timeUnit, new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda0
                @Override // java.util.function.LongConsumer
                public final void accept(long j2) {
                    AsynchronousTlsChannel.this.e(completionHandler, a2, j2);
                }
            }, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda11
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.this.e(completionHandler, a2, (Throwable) obj);
                }
            });
        } else {
            completeWithZeroInt(a2, completionHandler);
        }
    }

    @Override // java.nio.channels.AsynchronousByteChannel
    public <A> void write(ByteBuffer byteBuffer, final A a2, final CompletionHandler<Integer, ? super A> completionHandler) {
        if (byteBuffer.hasRemaining()) {
            this.group.startWrite(this.registeredSocket, new ByteBufferSet(byteBuffer), 0L, TimeUnit.MILLISECONDS, new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda28
                @Override // java.util.function.LongConsumer
                public final void accept(long j) {
                    AsynchronousTlsChannel.this.d(completionHandler, a2, j);
                }
            }, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda29
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.this.d(completionHandler, a2, (Throwable) obj);
                }
            });
        } else {
            completeWithZeroInt(a2, completionHandler);
        }
    }

    @Override // tlschannel.async.ExtendedAsynchronousByteChannel
    public <A> void write(ByteBuffer[] byteBufferArr, int i, int i2, long j, TimeUnit timeUnit, final A a2, final CompletionHandler<Long, ? super A> completionHandler) {
        ByteBufferSet byteBufferSet = new ByteBufferSet(byteBufferArr, i, i2);
        if (byteBufferSet.hasRemaining()) {
            this.group.startWrite(this.registeredSocket, byteBufferSet, j, timeUnit, new LongConsumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda12
                @Override // java.util.function.LongConsumer
                public final void accept(long j2) {
                    AsynchronousTlsChannel.this.f(completionHandler, a2, j2);
                }
            }, new Consumer() { // from class: tlschannel.async.AsynchronousTlsChannel$$ExternalSyntheticLambda13
                @Override // java.util.function.Consumer
                public final void accept(Object obj) {
                    AsynchronousTlsChannel.this.f(completionHandler, a2, (Throwable) obj);
                }
            });
        } else {
            completeWithZeroLong(a2, completionHandler);
        }
    }
}
