package tlschannel.impl;

import java.nio.ByteBuffer;
import java.util.Optional;
import java.util.function.Function;
import java.util.logging.Level;
import java.util.logging.Logger;
import tlschannel.BufferAllocator;
/* loaded from: classes19.dex */
public class BufferHolder {
    public static final Logger logger = Logger.getLogger(BufferHolder.class.getName());
    public static final byte[] zeros = new byte[TlsChannelImpl.maxTlsPacketSize];
    public final BufferAllocator allocator;
    public ByteBuffer buffer;
    public int lastSize;
    public final int maxSize;
    public final String name;
    public final boolean opportunisticDispose;
    public final boolean plainData;

    public BufferHolder(String str, Optional<ByteBuffer> optional, BufferAllocator bufferAllocator, int i, int i2, boolean z, boolean z2) {
        this.name = str;
        this.allocator = bufferAllocator;
        this.buffer = optional.orElse(null);
        this.maxSize = i2;
        this.plainData = z;
        this.opportunisticDispose = z2;
        this.lastSize = ((Integer) optional.map(new Function() { // from class: tlschannel.impl.BufferHolder$$ExternalSyntheticLambda0
            @Override // java.util.function.Function
            public final Object apply(Object obj) {
                Integer valueOf;
                valueOf = Integer.valueOf(((ByteBuffer) obj).capacity());
                return valueOf;
            }
        }).orElse(Integer.valueOf(i))).intValue();
    }

    private void resize(int i) {
        ByteBuffer allocate = this.allocator.allocate(i);
        this.buffer.flip();
        allocate.put(this.buffer);
        if (this.plainData) {
            zero();
        }
        this.allocator.free(this.buffer);
        this.buffer = allocate;
        this.lastSize = i;
    }

    private void zero(int i) {
        this.buffer.mark();
        this.buffer.position(i);
        int remaining = this.buffer.remaining();
        int min = Math.min(remaining, zeros.length);
        int i2 = 0;
        while (min > 0) {
            ByteBuffer byteBuffer = this.buffer;
            byte[] bArr = zeros;
            byteBuffer.put(bArr, 0, min);
            i2 += min;
            min = Math.min(remaining - i2, bArr.length);
        }
        this.buffer.reset();
    }

    public boolean dispose() {
        ByteBuffer byteBuffer = this.buffer;
        if (byteBuffer != null) {
            this.allocator.free(byteBuffer);
            this.buffer = null;
            return true;
        }
        return false;
    }

    public void enlarge() {
        if (this.buffer.capacity() >= this.maxSize) {
            throw new IllegalStateException(String.format("%s buffer insufficient despite having capacity of %d", this.name, Integer.valueOf(this.buffer.capacity())));
        }
        int min = Math.min(this.buffer.capacity() * 2, this.maxSize);
        Logger logger2 = logger;
        if (logger2.isLoggable(Level.FINEST)) {
            logger2.log(Level.FINEST, "enlarging buffer {0}, increasing from {1} to {2}", new Object[]{this.name, Integer.valueOf(this.buffer.capacity()), Integer.valueOf(min)});
        }
        resize(min);
    }

    public boolean nullOrEmpty() {
        ByteBuffer byteBuffer = this.buffer;
        return byteBuffer == null || byteBuffer.position() == 0;
    }

    public void prepare() {
        if (this.buffer == null) {
            this.buffer = this.allocator.allocate(this.lastSize);
        }
    }

    public boolean release() {
        if (this.opportunisticDispose && this.buffer.position() == 0) {
            return dispose();
        }
        return false;
    }

    public String toString() {
        return "BufferHolder{name='" + this.name + "', allocator=" + this.allocator + ", plainData=" + this.plainData + ", maxSize=" + this.maxSize + ", opportunisticDispose=" + this.opportunisticDispose + ", buffer=" + this.buffer + ", lastSize=" + this.lastSize + '}';
    }

    public void zero() {
        zero(0);
    }

    public void zeroRemaining() {
        zero(this.buffer.position());
    }
}
