package tlschannel.impl;

import com.king.zxing.util.LogUtils;
import java.nio.ByteBuffer;
import java.util.Arrays;
/* loaded from: classes19.dex */
public class ByteBufferSet {
    public final ByteBuffer[] array;
    public final int length;
    public final int offset;

    public ByteBufferSet(ByteBuffer byteBuffer) {
        this(new ByteBuffer[]{byteBuffer});
    }

    public ByteBufferSet(ByteBuffer[] byteBufferArr) {
        this(byteBufferArr, 0, byteBufferArr.length);
    }

    public ByteBufferSet(ByteBuffer[] byteBufferArr, int i, int i2) {
        if (byteBufferArr == null) {
            throw null;
        }
        if (byteBufferArr.length < i) {
            throw new IndexOutOfBoundsException();
        }
        int i3 = i + i2;
        if (byteBufferArr.length < i3) {
            throw new IndexOutOfBoundsException();
        }
        for (int i4 = i; i4 < i3; i4++) {
            if (byteBufferArr[i4] == null) {
                throw null;
            }
        }
        this.array = byteBufferArr;
        this.offset = i;
        this.length = i2;
    }

    public ByteBufferSet get(ByteBuffer byteBuffer, int i) {
        int i2;
        if (remaining() >= i) {
            if (byteBuffer.remaining() >= i) {
                int i3 = 0;
                for (int i4 = this.offset; i4 < this.offset + this.length && (i2 = i - i3) != 0; i4++) {
                    ByteBuffer byteBuffer2 = this.array[i4];
                    int min = Math.min(i2, byteBuffer2.remaining());
                    ByteBufferUtil.copy(byteBuffer2, byteBuffer, min);
                    i3 += min;
                }
                return this;
            }
            throw new IllegalArgumentException();
        }
        throw new IllegalArgumentException();
    }

    public int getRemaining(ByteBuffer byteBuffer) {
        int i = 0;
        for (int i2 = this.offset; i2 < this.offset + this.length && byteBuffer.hasRemaining(); i2++) {
            ByteBuffer byteBuffer2 = this.array[i2];
            int min = Math.min(byteBuffer.remaining(), byteBuffer2.remaining());
            ByteBufferUtil.copy(byteBuffer2, byteBuffer, min);
            i += min;
        }
        return i;
    }

    public boolean hasRemaining() {
        return remaining() > 0;
    }

    public boolean isReadOnly() {
        for (int i = this.offset; i < this.offset + this.length; i++) {
            if (this.array[i].isReadOnly()) {
                return true;
            }
        }
        return false;
    }

    public long position() {
        long j = 0;
        for (int i = this.offset; i < this.offset + this.length; i++) {
            j += this.array[i].position();
        }
        return j;
    }

    public ByteBufferSet put(ByteBuffer byteBuffer, int i) {
        int i2;
        if (byteBuffer.remaining() >= i) {
            if (remaining() >= i) {
                int i3 = 0;
                for (int i4 = this.offset; i4 < this.offset + this.length && (i2 = i - i3) != 0; i4++) {
                    int min = Math.min(i2, (int) remaining());
                    ByteBufferUtil.copy(byteBuffer, this.array[i4], min);
                    i3 += min;
                }
                return this;
            }
            throw new IllegalArgumentException();
        }
        throw new IllegalArgumentException();
    }

    public int putRemaining(ByteBuffer byteBuffer) {
        int i = 0;
        for (int i2 = this.offset; i2 < this.offset + this.length && byteBuffer.hasRemaining(); i2++) {
            ByteBuffer byteBuffer2 = this.array[i2];
            int min = Math.min(byteBuffer.remaining(), byteBuffer2.remaining());
            ByteBufferUtil.copy(byteBuffer, byteBuffer2, min);
            i += min;
        }
        return i;
    }

    public long remaining() {
        long j = 0;
        for (int i = this.offset; i < this.offset + this.length; i++) {
            j += this.array[i].remaining();
        }
        return j;
    }

    public String toString() {
        return "ByteBufferSet[" + Arrays.toString(this.array) + LogUtils.COLON + this.offset + LogUtils.COLON + this.length + "]";
    }
}
