package tlschannel.impl;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ByteChannel;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.util.Optional;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Consumer;
import java.util.function.Supplier;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLEngineResult;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSession;
import tlschannel.NeedsReadException;
import tlschannel.NeedsTaskException;
import tlschannel.NeedsWriteException;
import tlschannel.TlsChannelCallbackException;
import tlschannel.TrackingAllocator;
import tlschannel.WouldBlockException;
import tlschannel.util.Util;
/* loaded from: classes19.dex */
public class TlsChannelImpl implements ByteChannel {
    public static final int buffersInitialSize = 4096;
    public static final Logger logger = Logger.getLogger(TlsChannelImpl.class.getName());
    public static final int maxTlsPacketSize = 17408;
    public int bytesToReturn;
    public final TrackingAllocator encryptedBufAllocator;
    public final SSLEngine engine;
    public final BufferHolder inEncrypted;
    public final BufferHolder inPlain;
    public final Consumer<SSLSession> initSessionCallback;
    public final BufferHolder outEncrypted;
    public final TrackingAllocator plainBufAllocator;
    public final ReadableByteChannel readChannel;
    public final boolean runTasks;
    public ByteBufferSet suppliedInPlain;
    public final boolean waitForCloseConfirmation;
    public final WritableByteChannel writeChannel;
    public final Lock initLock = new ReentrantLock();
    public final Lock readLock = new ReentrantLock();
    public final Lock writeLock = new ReentrantLock();
    public volatile boolean negotiated = false;
    public volatile boolean invalid = false;
    public volatile boolean shutdownSent = false;
    public volatile boolean shutdownReceived = false;
    public final ByteBufferSet dummyOut = new ByteBufferSet(new ByteBuffer[]{ByteBuffer.allocate(0)});

    /* renamed from: tlschannel.impl.TlsChannelImpl$1  reason: invalid class name */
    /* loaded from: classes19.dex */
    public static /* synthetic */ class AnonymousClass1 {
        public static final /* synthetic */ int[] $SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus;
        public static final /* synthetic */ int[] $SwitchMap$javax$net$ssl$SSLEngineResult$Status;

        static {
            int[] iArr = new int[SSLEngineResult.Status.values().length];
            $SwitchMap$javax$net$ssl$SSLEngineResult$Status = iArr;
            try {
                iArr[SSLEngineResult.Status.OK.ordinal()] = 1;
            } catch (NoSuchFieldError e) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$Status[SSLEngineResult.Status.CLOSED.ordinal()] = 2;
            } catch (NoSuchFieldError e2) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$Status[SSLEngineResult.Status.BUFFER_OVERFLOW.ordinal()] = 3;
            } catch (NoSuchFieldError e3) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$Status[SSLEngineResult.Status.BUFFER_UNDERFLOW.ordinal()] = 4;
            } catch (NoSuchFieldError e4) {
            }
            int[] iArr2 = new int[SSLEngineResult.HandshakeStatus.values().length];
            $SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus = iArr2;
            try {
                iArr2[SSLEngineResult.HandshakeStatus.NEED_UNWRAP.ordinal()] = 1;
            } catch (NoSuchFieldError e5) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus[SSLEngineResult.HandshakeStatus.NEED_WRAP.ordinal()] = 2;
            } catch (NoSuchFieldError e6) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus[SSLEngineResult.HandshakeStatus.NOT_HANDSHAKING.ordinal()] = 3;
            } catch (NoSuchFieldError e7) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus[SSLEngineResult.HandshakeStatus.FINISHED.ordinal()] = 4;
            } catch (NoSuchFieldError e8) {
            }
            try {
                $SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus[SSLEngineResult.HandshakeStatus.NEED_TASK.ordinal()] = 5;
            } catch (NoSuchFieldError e9) {
            }
        }
    }

    /* loaded from: classes19.dex */
    public static class EofException extends Exception {
        public static final long serialVersionUID = -3859156713994602991L;

        @Override // java.lang.Throwable
        public Throwable fillInStackTrace() {
            return this;
        }
    }

    public TlsChannelImpl(ReadableByteChannel readableByteChannel, WritableByteChannel writableByteChannel, SSLEngine sSLEngine, Optional<BufferHolder> optional, Consumer<SSLSession> consumer, boolean z, TrackingAllocator trackingAllocator, final TrackingAllocator trackingAllocator2, final boolean z2, boolean z3) {
        this.readChannel = readableByteChannel;
        this.writeChannel = writableByteChannel;
        this.engine = sSLEngine;
        this.inEncrypted = optional.orElseGet(new Supplier() { // from class: tlschannel.impl.TlsChannelImpl$$ExternalSyntheticLambda0
            @Override // java.util.function.Supplier
            public final Object get() {
                return TlsChannelImpl.a(TrackingAllocator.this, z2);
            }
        });
        this.initSessionCallback = consumer;
        this.runTasks = z;
        this.plainBufAllocator = trackingAllocator;
        this.encryptedBufAllocator = trackingAllocator2;
        this.waitForCloseConfirmation = z3;
        this.inPlain = new BufferHolder("inPlain", Optional.empty(), trackingAllocator, 4096, maxTlsPacketSize, true, z2);
        this.outEncrypted = new BufferHolder("outEncrypted", Optional.empty(), trackingAllocator2, 4096, maxTlsPacketSize, false, z2);
    }

    public static /* synthetic */ BufferHolder a(TrackingAllocator trackingAllocator, boolean z) {
        return new BufferHolder("inEncrypted", Optional.empty(), trackingAllocator, 4096, maxTlsPacketSize, false, z);
    }

    public static void callChannelRead(ReadableByteChannel readableByteChannel, ByteBuffer byteBuffer) throws IOException, EofException {
        Util.assertTrue(byteBuffer.hasRemaining());
        Logger logger2 = logger;
        logger2.log(Level.FINEST, "Reading from channel");
        int read = readableByteChannel.read(byteBuffer);
        if (logger2.isLoggable(Level.FINEST)) {
            logger2.log(Level.FINEST, "Read from channel; response: {}, buffer: {}", new Object[]{Integer.valueOf(read), byteBuffer});
        }
        if (read == -1) {
            throw new EofException();
        }
        if (read == 0) {
            throw new NeedsReadException();
        }
    }

    public static void callChannelWrite(WritableByteChannel writableByteChannel, ByteBuffer byteBuffer) throws IOException {
        while (byteBuffer.hasRemaining()) {
            logger.log(Level.FINEST, "calling channel.write({0})", byteBuffer);
            if (writableByteChannel.write(byteBuffer) == 0) {
                throw new NeedsWriteException();
            }
        }
    }

    private SSLEngineResult callEngineUnwrap(ByteBufferSet byteBufferSet) throws SSLException {
        this.inEncrypted.buffer.flip();
        try {
            try {
                SSLEngineResult unwrap = this.engine.unwrap(this.inEncrypted.buffer, byteBufferSet.array, byteBufferSet.offset, byteBufferSet.length);
                Logger logger2 = logger;
                if (logger2.isLoggable(Level.FINEST)) {
                    logger2.log(Level.FINEST, "engine.unwrap() result [{0}]. engine status: {1}; inEncrypted {2}; inPlain: {3}", new Object[]{Util.resultToString(unwrap), this.engine.getHandshakeStatus(), this.inEncrypted, byteBufferSet});
                }
                return unwrap;
            } catch (SSLException e) {
                this.invalid = true;
                throw e;
            }
        } finally {
            this.inEncrypted.buffer.compact();
        }
    }

    private SSLEngineResult callEngineWrap(ByteBufferSet byteBufferSet) throws SSLException {
        try {
            SSLEngineResult wrap = this.engine.wrap(byteBufferSet.array, byteBufferSet.offset, byteBufferSet.length, this.outEncrypted.buffer);
            Logger logger2 = logger;
            if (logger2.isLoggable(Level.FINEST)) {
                logger2.log(Level.FINEST, "engine.wrap() result: [{0}]; engine status: {1}; srcBuffer: {2}, outEncrypted: {3}", new Object[]{Util.resultToString(wrap), wrap.getHandshakeStatus(), byteBufferSet, this.outEncrypted});
            }
            return wrap;
        } catch (SSLException e) {
            this.invalid = true;
            throw e;
        }
    }

    public static void checkReadBuffer(ByteBufferSet byteBufferSet) {
        if (byteBufferSet.isReadOnly()) {
            throw new IllegalArgumentException();
        }
    }

    private void doHandshake(boolean z) throws IOException, EofException {
        if (z || !this.negotiated) {
            this.initLock.lock();
            try {
                if (this.invalid || this.shutdownSent) {
                    throw new ClosedChannelException();
                }
                if (z || !this.negotiated) {
                    logger.log(Level.FINEST, "Calling SSLEngine.beginHandshake()");
                    this.engine.beginHandshake();
                    writeAndHandshake();
                    if (this.engine.getSession().getProtocol().startsWith("DTLS")) {
                        throw new IllegalArgumentException("DTLS not supported");
                    }
                    try {
                        this.initSessionCallback.accept(this.engine.getSession());
                        this.negotiated = true;
                    } catch (Exception e) {
                        logger.log(Level.FINEST, "client code threw exception in session initialization callback", (Throwable) e);
                        throw new TlsChannelCallbackException("session initialization callback failed", e);
                    }
                }
            } finally {
                this.initLock.unlock();
            }
        }
    }

    private void freeBuffers() {
        this.inEncrypted.dispose();
        this.inPlain.dispose();
        this.outEncrypted.dispose();
    }

    private void handleTask() throws NeedsTaskException {
        Runnable delegatedTask = this.engine.getDelegatedTask();
        if (!this.runTasks) {
            logger.log(Level.FINEST, "task needed, throwing exception: {0}", delegatedTask);
            throw new NeedsTaskException(delegatedTask);
        }
        logger.log(Level.FINEST, "delegating in task: {0}", delegatedTask);
        delegatedTask.run();
    }

    private void handshakeLoop() throws IOException, EofException {
        Util.assertTrue(this.inPlain.nullOrEmpty());
        while (true) {
            switch (AnonymousClass1.$SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus[this.engine.getHandshakeStatus().ordinal()]) {
                case 1:
                    readAndUnwrap();
                    if (this.bytesToReturn <= 0) {
                        break;
                    } else {
                        return;
                    }
                case 2:
                    Util.assertTrue(this.outEncrypted.nullOrEmpty());
                    wrapLoop(this.dummyOut);
                    writeToChannel();
                    break;
                case 3:
                    return;
                case 4:
                    throw new IllegalStateException();
                case 5:
                    handleTask();
                    break;
                default:
                    throw new IllegalStateException();
            }
        }
    }

    private void readAndUnwrap() throws IOException, EofException {
        this.inEncrypted.prepare();
        while (true) {
            try {
                Util.assertTrue(this.inPlain.nullOrEmpty());
                SSLEngineResult unwrapLoop = unwrapLoop();
                SSLEngineResult.HandshakeStatus handshakeStatus = this.engine.getHandshakeStatus();
                if (unwrapLoop.bytesProduced() <= 0) {
                    if (unwrapLoop.getStatus() != SSLEngineResult.Status.CLOSED) {
                        if (unwrapLoop.getHandshakeStatus() == SSLEngineResult.HandshakeStatus.FINISHED || handshakeStatus == SSLEngineResult.HandshakeStatus.NEED_TASK || handshakeStatus == SSLEngineResult.HandshakeStatus.NEED_WRAP) {
                            break;
                        }
                        if (!this.inEncrypted.buffer.hasRemaining()) {
                            this.inEncrypted.enlarge();
                        }
                        readFromChannel();
                    } else {
                        this.shutdownReceived = true;
                        break;
                    }
                } else {
                    this.bytesToReturn = unwrapLoop.bytesProduced();
                    break;
                }
            } finally {
                this.inEncrypted.release();
            }
        }
    }

    private void readFromChannel() throws IOException, EofException {
        try {
            callChannelRead(this.readChannel, this.inEncrypted.buffer);
        } catch (WouldBlockException e) {
            throw e;
        } catch (IOException e2) {
            this.invalid = true;
            throw e2;
        }
    }

    private int transferPendingPlain(ByteBufferSet byteBufferSet) {
        this.inPlain.buffer.flip();
        int putRemaining = byteBufferSet.putRemaining(this.inPlain.buffer);
        this.inPlain.buffer.compact();
        if (!this.inPlain.release()) {
            this.inPlain.zeroRemaining();
        }
        return putRemaining;
    }

    private void tryShutdown() {
        if (this.readLock.tryLock()) {
            try {
                if (this.writeLock.tryLock()) {
                    if (!this.shutdownSent) {
                        try {
                            if (!shutdown() && this.waitForCloseConfirmation) {
                                shutdown();
                            }
                        } catch (Throwable th) {
                            logger.log(Level.FINEST, "error doing TLS shutdown on close(), continuing: {0}", th.getMessage());
                        }
                    }
                    this.writeLock.unlock();
                }
            } finally {
                this.readLock.unlock();
            }
        }
    }

    /* JADX WARN: Code restructure failed: missing block: B:27:0x0072, code lost:
        if (r5.inPlain.buffer.capacity() <= r5.suppliedInPlain.remaining()) goto L27;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private SSLEngineResult unwrapLoop() throws SSLException {
        ByteBufferSet byteBufferSet = this.suppliedInPlain;
        if (byteBufferSet == null) {
            this.inPlain.prepare();
            byteBufferSet = new ByteBufferSet(this.inPlain.buffer);
        }
        while (true) {
            Util.assertTrue(this.inPlain.nullOrEmpty());
            SSLEngineResult callEngineUnwrap = callEngineUnwrap(byteBufferSet);
            SSLEngineResult.HandshakeStatus handshakeStatus = this.engine.getHandshakeStatus();
            if (callEngineUnwrap.bytesProduced() <= 0 && callEngineUnwrap.getStatus() != SSLEngineResult.Status.CLOSED && callEngineUnwrap.getStatus() != SSLEngineResult.Status.BUFFER_UNDERFLOW && callEngineUnwrap.getHandshakeStatus() != SSLEngineResult.HandshakeStatus.FINISHED && handshakeStatus != SSLEngineResult.HandshakeStatus.NEED_TASK && handshakeStatus != SSLEngineResult.HandshakeStatus.NEED_WRAP) {
                if (callEngineUnwrap.getStatus() == SSLEngineResult.Status.BUFFER_OVERFLOW) {
                    if (byteBufferSet == this.suppliedInPlain) {
                        this.inPlain.prepare();
                    }
                    this.inPlain.enlarge();
                    byteBufferSet = new ByteBufferSet(this.inPlain.buffer);
                }
            }
        }
    }

    /* JADX WARN: Code restructure failed: missing block: B:11:0x002c, code lost:
        r0 = r0 - r8.remaining();
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
    */
    private long wrapAndWrite(ByteBufferSet byteBufferSet) throws IOException {
        long remaining = byteBufferSet.remaining();
        this.outEncrypted.prepare();
        while (true) {
            try {
                writeToChannel();
                if (byteBufferSet.remaining() == 0) {
                    break;
                } else if (wrapLoop(byteBufferSet).getStatus() == SSLEngineResult.Status.CLOSED) {
                    break;
                }
            } finally {
                this.outEncrypted.release();
            }
        }
        return remaining;
    }

    private SSLEngineResult wrapLoop(ByteBufferSet byteBufferSet) throws SSLException {
        while (true) {
            SSLEngineResult callEngineWrap = callEngineWrap(byteBufferSet);
            switch (AnonymousClass1.$SwitchMap$javax$net$ssl$SSLEngineResult$Status[callEngineWrap.getStatus().ordinal()]) {
                case 1:
                case 2:
                    return callEngineWrap;
                case 3:
                    Util.assertTrue(callEngineWrap.bytesConsumed() == 0);
                    this.outEncrypted.enlarge();
                    break;
                case 4:
                    throw new IllegalStateException();
            }
        }
    }

    private void writeAndHandshake() throws IOException, EofException {
        this.readLock.lock();
        try {
            this.writeLock.lock();
            Util.assertTrue(this.inPlain.nullOrEmpty());
            this.outEncrypted.prepare();
            writeToChannel();
            handshakeLoop();
            this.outEncrypted.release();
            this.writeLock.unlock();
        } finally {
            this.readLock.unlock();
        }
    }

    private void writeToChannel() throws IOException {
        if (this.outEncrypted.buffer.position() == 0) {
            return;
        }
        this.outEncrypted.buffer.flip();
        try {
            try {
                try {
                    callChannelWrite(this.writeChannel, this.outEncrypted.buffer);
                } catch (IOException e) {
                    this.invalid = true;
                    throw e;
                }
            } catch (WouldBlockException e2) {
                throw e2;
            }
        } finally {
            this.outEncrypted.buffer.compact();
        }
    }

    @Override // java.nio.channels.Channel, java.io.Closeable, java.lang.AutoCloseable
    public void close() throws IOException {
        tryShutdown();
        this.writeChannel.close();
        this.readChannel.close();
        this.readLock.lock();
        try {
            this.writeLock.lock();
            freeBuffers();
            this.writeLock.unlock();
        } finally {
            this.readLock.unlock();
        }
    }

    public SSLEngine engine() {
        return this.engine;
    }

    public TrackingAllocator getEncryptedBufferAllocator() {
        return this.encryptedBufAllocator;
    }

    public TrackingAllocator getPlainBufferAllocator() {
        return this.plainBufAllocator;
    }

    public boolean getRunTasks() {
        return this.runTasks;
    }

    public Consumer<SSLSession> getSessionInitCallback() {
        return this.initSessionCallback;
    }

    public void handshake() throws IOException {
        try {
            doHandshake(false);
        } catch (EofException e) {
            throw new ClosedChannelException();
        }
    }

    @Override // java.nio.channels.Channel
    public boolean isOpen() {
        return !this.invalid && this.writeChannel.isOpen() && this.readChannel.isOpen();
    }

    public ReadableByteChannel plainReadableChannel() {
        return this.readChannel;
    }

    public WritableByteChannel plainWritableChannel() {
        return this.writeChannel;
    }

    @Override // java.nio.channels.ReadableByteChannel
    public int read(ByteBuffer byteBuffer) throws IOException {
        return (int) read(new ByteBufferSet(byteBuffer));
    }

    public long read(ByteBufferSet byteBufferSet) throws IOException {
        int transferPendingPlain;
        checkReadBuffer(byteBufferSet);
        if (byteBufferSet.hasRemaining()) {
            handshake();
            this.readLock.lock();
            long j = -1;
            try {
            } catch (EofException e) {
            } catch (Throwable th) {
                this.bytesToReturn = 0;
                this.suppliedInPlain = null;
                this.readLock.unlock();
                throw th;
            }
            if (this.invalid || this.shutdownSent) {
                throw new ClosedChannelException();
            }
            long position = byteBufferSet.position();
            this.suppliedInPlain = byteBufferSet;
            this.bytesToReturn = this.inPlain.nullOrEmpty() ? 0 : this.inPlain.buffer.position();
            while (true) {
                if (this.bytesToReturn <= 0) {
                    if (!this.shutdownReceived) {
                        Util.assertTrue(this.inPlain.nullOrEmpty());
                        switch (AnonymousClass1.$SwitchMap$javax$net$ssl$SSLEngineResult$HandshakeStatus[this.engine.getHandshakeStatus().ordinal()]) {
                            case 1:
                            case 2:
                                writeAndHandshake();
                                break;
                            case 3:
                            case 4:
                                readAndUnwrap();
                                if (!this.shutdownReceived) {
                                    break;
                                } else {
                                    break;
                                }
                            case 5:
                                handleTask();
                                break;
                        }
                    }
                } else {
                    boolean z = true;
                    if (this.inPlain.nullOrEmpty()) {
                        if (byteBufferSet.position() != position + this.bytesToReturn) {
                            z = false;
                        }
                        Util.assertTrue(z);
                        transferPendingPlain = this.bytesToReturn;
                    } else {
                        if (this.inPlain.buffer.position() != this.bytesToReturn) {
                            z = false;
                        }
                        Util.assertTrue(z);
                        transferPendingPlain = transferPendingPlain(byteBufferSet);
                    }
                    j = transferPendingPlain;
                }
            }
            this.bytesToReturn = 0;
            this.suppliedInPlain = null;
            this.readLock.unlock();
            return j;
        }
        return 0L;
    }

    public void renegotiate() throws IOException {
        if (this.engine.getSession().getProtocol().compareTo("TLSv1.3") >= 0) {
            throw new SSLException("renegotiation not supported in TLS 1.3 or latter");
        }
        try {
            doHandshake(true);
        } catch (EofException e) {
            throw new ClosedChannelException();
        }
    }

    public boolean shutdown() throws IOException {
        this.readLock.lock();
        try {
            this.writeLock.lock();
            if (this.invalid) {
                throw new ClosedChannelException();
            }
            if (this.shutdownSent) {
                if (!this.shutdownReceived) {
                    try {
                        readAndUnwrap();
                        Util.assertTrue(this.shutdownReceived);
                    } catch (EofException e) {
                        throw new ClosedChannelException();
                    }
                }
                freeBuffers();
                this.writeLock.unlock();
                return true;
            }
            this.shutdownSent = true;
            this.outEncrypted.prepare();
            writeToChannel();
            this.engine.closeOutbound();
            wrapLoop(this.dummyOut);
            writeToChannel();
            this.outEncrypted.release();
            if (this.shutdownReceived) {
                freeBuffers();
            }
            boolean z = this.shutdownReceived;
            this.writeLock.unlock();
            return z;
        } finally {
            this.readLock.unlock();
        }
    }

    public boolean shutdownReceived() {
        return this.shutdownReceived;
    }

    public boolean shutdownSent() {
        return this.shutdownSent;
    }

    @Override // java.nio.channels.WritableByteChannel
    public int write(ByteBuffer byteBuffer) throws IOException {
        return (int) write(new ByteBufferSet(byteBuffer));
    }

    public long write(ByteBufferSet byteBufferSet) throws IOException {
        handshake();
        this.writeLock.lock();
        try {
            if (this.invalid || this.shutdownSent) {
                throw new ClosedChannelException();
            }
            return wrapAndWrite(byteBufferSet);
        } finally {
            this.writeLock.unlock();
        }
    }
}
